use phf::{phf_map, Map};
#[cfg(target_arch = "wasm32")]
use wasm_bindgen::prelude::*;
#[cfg(target_arch = "wasm32")]
use js_sys::Array;

#[cfg(test)]
mod tests {

    #[test]
    fn test_from_code_1() {
        let l = crate::from_code_1("zh");
        print!("test_from_code result {:?}", l)
    }

    #[test]
    fn test_from_code_2t() {
        let l = crate::from_code_2t("zho");
        print!("test_from_code_2t result {:?}", l)
    }

    #[test]
    fn test_from_code_2b() {
        let l = crate::from_code_2b("chi");
        print!("test_from_code_2b result {:?}", l)
    }

    #[test]
    fn test_from_code_3() {
        let l = crate::from_code_3("zho");
        print!("test_from_code_3 result {:?}", l)
    }

    #[test]
    fn test_all() {
        println!("{:?}", crate::ALL);
        println!("{:?}", crate::ALL_1);
        println!("{:?}", crate::ALL_2B);
        println!("{:?}", crate::ALL_2T);
        println!("{:?}", crate::ALL_3);


        println!("{:?}", crate::ALL_MAP);
        println!("{:?}", crate::ALL_1_MAP);
        println!("{:?}", crate::ALL_2B_MAP);
        println!("{:?}", crate::ALL_2T_MAP);
        println!("{:?}", crate::ALL_3_MAP);

    }
}

#[cfg(target_arch = "wasm32")]
#[wasm_bindgen]
#[derive(Debug, Copy, Clone, PartialEq, Eq, Hash)]
pub struct LanguageCode {
    ///ISO Language Name
    name: &'static str,
    ///639-1
    code: &'static str,
    ///639-2/T
    code_2t: &'static str,
    ///639-2/B
    code_2b: &'static str,
    //639-3 Macrolanguage
    code_3: &'static str,
    individual_languages: &'static[IndividualLanguages],
}

#[cfg(not(target_arch = "wasm32"))]
#[derive(Debug, Copy, Clone, PartialEq, Eq, Hash)]
pub struct LanguageCode<'a> {
    ///ISO Language Name
    pub name: &'static str,
    ///639-1
    pub code: &'static str,
    ///639-2/T
    pub code_2t: &'static str,
    ///639-2/B
    pub code_2b: &'static str,
    //639-3 Macrolanguage
    pub code_3: &'static str,

    pub individual_languages: &'a [IndividualLanguages],
}

#[cfg(target_arch = "wasm32")]
#[cfg_attr(target_arch = "wasm32", wasm_bindgen)]
impl LanguageCode {
     #[wasm_bindgen(getter)]
    pub fn name(&self) -> String {
        self.name.into()
    }
     #[wasm_bindgen(getter)]
    pub fn code(&self) -> String {
        self.code.into()
    }

    #[wasm_bindgen(getter)]
    pub fn code_2t(&self) -> String {
        self.code_2t.into()
    }

    #[wasm_bindgen(getter)]
    pub fn code_2b(&self) -> String {
        self.code_2b.into()
    }

    #[wasm_bindgen(getter)]
    pub fn code_3(&self) -> String {
        self.code_3.into()
    }
	
    #[wasm_bindgen(getter)]
    pub fn individual_languages(&self) -> Array {
		let mut vector: Vec<IndividualLanguages> = Vec::new(); 
        // self.individual_languages.into_serde().unwrap();
		for i in 0..self.individual_languages.len() {
			vector.push(self.individual_languages[i])
		}
		vector.into_iter().map(JsValue::from).collect()
    }
}


#[cfg(target_arch = "wasm32")]
#[wasm_bindgen]
#[derive(Debug, Copy, Clone, PartialEq, Eq, Hash)]
pub struct IndividualLanguages {
    ///Name
    name: &'static str,
    ///Code
    code: &'static str,
}

#[cfg(not(target_arch = "wasm32"))]
#[derive(Debug, Copy, Clone, PartialEq, Eq, Hash)]
pub struct IndividualLanguages {
    ///Name
    pub name: &'static str,
    ///Code
    pub code: &'static str,
}

#[cfg(target_arch = "wasm32")]
#[cfg_attr(target_arch = "wasm32", wasm_bindgen)]
impl IndividualLanguages {
     #[wasm_bindgen(getter)]
    pub fn name(&self) -> String {
        self.name.into()
    }
     #[wasm_bindgen(getter)]
    pub fn code(&self) -> String {
        self.code.into()
    }
}



#[cfg_attr(target_arch = "wasm32", wasm_bindgen)]
pub fn from_code_1(code: &str) -> Option<LanguageCode> {
    let up = code.to_lowercase();
    ALL_1_MAP.get(up.as_str()).cloned()
}

#[cfg_attr(target_arch = "wasm32", wasm_bindgen)]
pub fn from_code_2t(code: &str) -> Option<LanguageCode> {
    let up = code.to_lowercase();
    ALL_2T_MAP.get(up.as_str()).cloned()
}

#[cfg_attr(target_arch = "wasm32", wasm_bindgen)]
pub fn from_code_2b(code: &str) -> Option<LanguageCode> {
    let up = code.to_lowercase();
    ALL_2B_MAP.get(up.as_str()).cloned()
}
#[cfg_attr(target_arch = "wasm32", wasm_bindgen)]
pub fn from_code_3(code: &str) -> Option<LanguageCode> {
    let up = code.to_lowercase();
    ALL_3_MAP.get(up.as_str()).cloned()
}


pub const AB: LanguageCode = LanguageCode {
    name: "Abkhazian",
    code: "ab",
    code_2t: "abk",
    code_2b: "abk",
    code_3: "abk",
    individual_languages: &[
    ],
};


pub const AA: LanguageCode = LanguageCode {
    name: "Afar",
    code: "aa",
    code_2t: "aar",
    code_2b: "aar",
    code_3: "aar",
    individual_languages: &[
    ],
};


pub const AF: LanguageCode = LanguageCode {
    name: "Afrikaans",
    code: "af",
    code_2t: "afr",
    code_2b: "afr",
    code_3: "afr",
    individual_languages: &[
    ],
};


pub const AK: LanguageCode = LanguageCode {
    name: "Akan",
    code: "ak",
    code_2t: "aka",
    code_2b: "aka",
    code_3: "aka",
    individual_languages: &[
        IndividualLanguages {
            name: "Fanti",
            code: "fat",
        },
        IndividualLanguages {
            name: "Twi",
            code: "twi",
        },
    ],
};


pub const SQ: LanguageCode = LanguageCode {
    name: "Albanian",
    code: "sq",
    code_2t: "sqi",
    code_2b: "alb",
    code_3: "sqi",
    individual_languages: &[
        IndividualLanguages {
            name: "Arbëreshë Albanian",
            code: "aae",
        },
        IndividualLanguages {
            name: "Arvanitika Albanian",
            code: "aat",
        },
        IndividualLanguages {
            name: "Gheg Albanian",
            code: "aln",
        },
        IndividualLanguages {
            name: "Tosk Albanian",
            code: "als",
        },
    ],
};


pub const AM: LanguageCode = LanguageCode {
    name: "Amharic",
    code: "am",
    code_2t: "amh",
    code_2b: "amh",
    code_3: "amh",
    individual_languages: &[
    ],
};


pub const AR: LanguageCode = LanguageCode {
    name: "Arabic",
    code: "ar",
    code_2t: "ara",
    code_2b: "ara",
    code_3: "ara",
    individual_languages: &[
        IndividualLanguages {
            name: "Algerian Saharan Arabic",
            code: "aao",
        },
        IndividualLanguages {
            name: "Tajiki Arabic",
            code: "abh",
        },
        IndividualLanguages {
            name: "Baharna Arabic",
            code: "abv",
        },
        IndividualLanguages {
            name: "Mesopotamian Arabic",
            code: "acm",
        },
        IndividualLanguages {
            name: "Ta'izzi-Adeni Arabic",
            code: "acq",
        },
        IndividualLanguages {
            name: "Hijazi Arabic",
            code: "acw",
        },
        IndividualLanguages {
            name: "Omani Arabic",
            code: "acx",
        },
        IndividualLanguages {
            name: "Cypriot Arabic",
            code: "acy",
        },
        IndividualLanguages {
            name: "Dhofari Arabic",
            code: "adf",
        },
        IndividualLanguages {
            name: "Tunisian Arabic",
            code: "aeb",
        },
        IndividualLanguages {
            name: "Saidi Arabic",
            code: "aec",
        },
        IndividualLanguages {
            name: "Gulf Arabic",
            code: "afb",
        },
        IndividualLanguages {
            name: "South Levantine Arabic",
            code: "ajp",
        },
        IndividualLanguages {
            name: "North Levantine Arabic",
            code: "apc",
        },
        IndividualLanguages {
            name: "Sudanese Arabic",
            code: "apd",
        },
        IndividualLanguages {
            name: "Standard Arabic",
            code: "arb",
        },
        IndividualLanguages {
            name: "Algerian Arabic",
            code: "arq",
        },
        IndividualLanguages {
            name: "Najdi Arabic",
            code: "ars",
        },
        IndividualLanguages {
            name: "Moroccan Arabic",
            code: "ary",
        },
        IndividualLanguages {
            name: "Egyptian Arabic",
            code: "arz",
        },
        IndividualLanguages {
            name: "Uzbeki Arabic",
            code: "auz",
        },
        IndividualLanguages {
            name: "Eastern Egyptian Bedawi Arabic",
            code: "avl",
        },
        IndividualLanguages {
            name: "Hadrami Arabic",
            code: "ayh",
        },
        IndividualLanguages {
            name: "Libyan Arabic",
            code: "ayl",
        },
        IndividualLanguages {
            name: "Sanaani Arabic",
            code: "ayn",
        },
        IndividualLanguages {
            name: "North Mesopotamian Arabic",
            code: "ayp",
        },
        IndividualLanguages {
            name: "",
            code: "bbz",
        },
        IndividualLanguages {
            name: "Sudanese Creole Arabic",
            code: "pga",
        },
        IndividualLanguages {
            name: "Chadian Arabic",
            code: "shu",
        },
        IndividualLanguages {
            name: "Shihhi Arabic",
            code: "ssh",
        },
    ],
};


pub const AN: LanguageCode = LanguageCode {
    name: "Aragonese",
    code: "an",
    code_2t: "arg",
    code_2b: "arg",
    code_3: "arg",
    individual_languages: &[
    ],
};


pub const HY: LanguageCode = LanguageCode {
    name: "Armenian",
    code: "hy",
    code_2t: "hye",
    code_2b: "arm",
    code_3: "hye",
    individual_languages: &[
    ],
};


pub const AS: LanguageCode = LanguageCode {
    name: "Assamese",
    code: "as",
    code_2t: "asm",
    code_2b: "asm",
    code_3: "asm",
    individual_languages: &[
    ],
};


pub const AV: LanguageCode = LanguageCode {
    name: "Avaric",
    code: "av",
    code_2t: "ava",
    code_2b: "ava",
    code_3: "ava",
    individual_languages: &[
    ],
};


pub const AE: LanguageCode = LanguageCode {
    name: "Avestan",
    code: "ae",
    code_2t: "ave",
    code_2b: "ave",
    code_3: "ave",
    individual_languages: &[
    ],
};


pub const AY: LanguageCode = LanguageCode {
    name: "Aymara",
    code: "ay",
    code_2t: "aym",
    code_2b: "aym",
    code_3: "aym",
    individual_languages: &[
        IndividualLanguages {
            name: "Southern Aymara",
            code: "ayc",
        },
        IndividualLanguages {
            name: "Central Aymara",
            code: "ayr",
        },
    ],
};


pub const AZ: LanguageCode = LanguageCode {
    name: "Azerbaijani",
    code: "az",
    code_2t: "aze",
    code_2b: "aze",
    code_3: "aze",
    individual_languages: &[
        IndividualLanguages {
            name: "South Azerbaijani",
            code: "azb",
        },
        IndividualLanguages {
            name: "North Azerbaijani",
            code: "azj",
        },
    ],
};


pub const BM: LanguageCode = LanguageCode {
    name: "Bambara",
    code: "bm",
    code_2t: "bam",
    code_2b: "bam",
    code_3: "bam",
    individual_languages: &[
    ],
};


pub const BA: LanguageCode = LanguageCode {
    name: "Bashkir",
    code: "ba",
    code_2t: "bak",
    code_2b: "bak",
    code_3: "bak",
    individual_languages: &[
    ],
};


pub const EU: LanguageCode = LanguageCode {
    name: "Basque",
    code: "eu",
    code_2t: "eus",
    code_2b: "baq",
    code_3: "eus",
    individual_languages: &[
    ],
};


pub const BE: LanguageCode = LanguageCode {
    name: "Belarusian",
    code: "be",
    code_2t: "bel",
    code_2b: "bel",
    code_3: "bel",
    individual_languages: &[
    ],
};


pub const BN: LanguageCode = LanguageCode {
    name: "Bengali",
    code: "bn",
    code_2t: "ben",
    code_2b: "ben",
    code_3: "ben",
    individual_languages: &[
    ],
};


pub const BI: LanguageCode = LanguageCode {
    name: "Bislama",
    code: "bi",
    code_2t: "bis",
    code_2b: "bis",
    code_3: "bis",
    individual_languages: &[
    ],
};


pub const BS: LanguageCode = LanguageCode {
    name: "Bosnian",
    code: "bs",
    code_2t: "bos",
    code_2b: "bos",
    code_3: "bos",
    individual_languages: &[
    ],
};


pub const BR: LanguageCode = LanguageCode {
    name: "Breton",
    code: "br",
    code_2t: "bre",
    code_2b: "bre",
    code_3: "bre",
    individual_languages: &[
    ],
};


pub const BG: LanguageCode = LanguageCode {
    name: "Bulgarian",
    code: "bg",
    code_2t: "bul",
    code_2b: "bul",
    code_3: "bul",
    individual_languages: &[
    ],
};


pub const MY: LanguageCode = LanguageCode {
    name: "Burmese",
    code: "my",
    code_2t: "mya",
    code_2b: "bur",
    code_3: "mya",
    individual_languages: &[
    ],
};


pub const CA: LanguageCode = LanguageCode {
    name: "Catalan, Valencian",
    code: "ca",
    code_2t: "cat",
    code_2b: "cat",
    code_3: "cat",
    individual_languages: &[
    ],
};


pub const CH: LanguageCode = LanguageCode {
    name: "Chamorro",
    code: "ch",
    code_2t: "cha",
    code_2b: "cha",
    code_3: "cha",
    individual_languages: &[
    ],
};


pub const CE: LanguageCode = LanguageCode {
    name: "Chechen",
    code: "ce",
    code_2t: "che",
    code_2b: "che",
    code_3: "che",
    individual_languages: &[
    ],
};


pub const NY: LanguageCode = LanguageCode {
    name: "Chichewa, Chewa, Nyanja",
    code: "ny",
    code_2t: "nya",
    code_2b: "nya",
    code_3: "nya",
    individual_languages: &[
    ],
};


pub const ZH: LanguageCode = LanguageCode {
    name: "Chinese",
    code: "zh",
    code_2t: "zho",
    code_2b: "chi",
    code_3: "zho",
    individual_languages: &[
        IndividualLanguages {
            name: "Min Dong Chinese",
            code: "cdo",
        },
        IndividualLanguages {
            name: "Jinyu Chinese",
            code: "cjy",
        },
        IndividualLanguages {
            name: "Mandarin Chinese",
            code: "cmn",
        },
        IndividualLanguages {
            name: "Northern Pinghua",
            code: "cnp",
        },
        IndividualLanguages {
            name: "Pu-Xian Chinese",
            code: "cpx",
        },
        IndividualLanguages {
            name: "Southern Pinghua",
            code: "csp",
        },
        IndividualLanguages {
            name: "Huizhou Chinese",
            code: "czh",
        },
        IndividualLanguages {
            name: "Min Zhong Chinese",
            code: "czo",
        },
        IndividualLanguages {
            name: "Gan Chinese",
            code: "gan",
        },
        IndividualLanguages {
            name: "Hakka Chinese",
            code: "hak",
        },
        IndividualLanguages {
            name: "Xiang Chinese",
            code: "hsn",
        },
        IndividualLanguages {
            name: "Literary Chinese",
            code: "lzh",
        },
        IndividualLanguages {
            name: "Min Bei Chinese",
            code: "mnp",
        },
        IndividualLanguages {
            name: "Min Nan Chinese",
            code: "nan",
        },
        IndividualLanguages {
            name: "Wu Chinese",
            code: "wuu",
        },
        IndividualLanguages {
            name: "Yue Chinese",
            code: "yue",
        },
    ],
};


pub const CU: LanguageCode = LanguageCode {
    name: "Old&nbsp;Church&nbsp;Slavonic",
    code: "cu",
    code_2t: "chu",
    code_2b: "chu",
    code_3: "chu",
    individual_languages: &[
    ],
};


pub const CV: LanguageCode = LanguageCode {
    name: "Chuvash",
    code: "cv",
    code_2t: "chv",
    code_2b: "chv",
    code_3: "chv",
    individual_languages: &[
    ],
};


pub const KW: LanguageCode = LanguageCode {
    name: "Cornish",
    code: "kw",
    code_2t: "cor",
    code_2b: "cor",
    code_3: "cor",
    individual_languages: &[
    ],
};


pub const CO: LanguageCode = LanguageCode {
    name: "Corsican",
    code: "co",
    code_2t: "cos",
    code_2b: "cos",
    code_3: "cos",
    individual_languages: &[
    ],
};


pub const CR: LanguageCode = LanguageCode {
    name: "Cree",
    code: "cr",
    code_2t: "cre",
    code_2b: "cre",
    code_3: "cre",
    individual_languages: &[
        IndividualLanguages {
            name: "Southern East Cree",
            code: "crj",
        },
        IndividualLanguages {
            name: "Plains Cree",
            code: "crk",
        },
        IndividualLanguages {
            name: "Northern East Cree",
            code: "crl",
        },
        IndividualLanguages {
            name: "Moose Cree",
            code: "crm",
        },
        IndividualLanguages {
            name: "Swampy Cree",
            code: "csw",
        },
        IndividualLanguages {
            name: "Woods Cree",
            code: "cwd",
        },
    ],
};


pub const HR: LanguageCode = LanguageCode {
    name: "Croatian",
    code: "hr",
    code_2t: "hrv",
    code_2b: "hrv",
    code_3: "hrv",
    individual_languages: &[
    ],
};


pub const CS: LanguageCode = LanguageCode {
    name: "Czech",
    code: "cs",
    code_2t: "ces",
    code_2b: "cze",
    code_3: "ces",
    individual_languages: &[
    ],
};


pub const DA: LanguageCode = LanguageCode {
    name: "Danish",
    code: "da",
    code_2t: "dan",
    code_2b: "dan",
    code_3: "dan",
    individual_languages: &[
    ],
};


pub const DV: LanguageCode = LanguageCode {
    name: "Divehi, Dhivehi, Maldivian",
    code: "dv",
    code_2t: "div",
    code_2b: "div",
    code_3: "div",
    individual_languages: &[
    ],
};


pub const NL: LanguageCode = LanguageCode {
    name: "Dutch, Flemish",
    code: "nl",
    code_2t: "nld",
    code_2b: "dut",
    code_3: "nld",
    individual_languages: &[
    ],
};


pub const DZ: LanguageCode = LanguageCode {
    name: "Dzongkha",
    code: "dz",
    code_2t: "dzo",
    code_2b: "dzo",
    code_3: "dzo",
    individual_languages: &[
    ],
};


pub const EN: LanguageCode = LanguageCode {
    name: "English",
    code: "en",
    code_2t: "eng",
    code_2b: "eng",
    code_3: "eng",
    individual_languages: &[
    ],
};


pub const EO: LanguageCode = LanguageCode {
    name: "Esperanto",
    code: "eo",
    code_2t: "epo",
    code_2b: "epo",
    code_3: "epo",
    individual_languages: &[
    ],
};


pub const ET: LanguageCode = LanguageCode {
    name: "Estonian",
    code: "et",
    code_2t: "est",
    code_2b: "est",
    code_3: "est",
    individual_languages: &[
        IndividualLanguages {
            name: "Standard Estonian",
            code: "ekk",
        },
        IndividualLanguages {
            name: "Võro",
            code: "vro",
        },
    ],
};


pub const EE: LanguageCode = LanguageCode {
    name: "Ewe",
    code: "ee",
    code_2t: "ewe",
    code_2b: "ewe",
    code_3: "ewe",
    individual_languages: &[
    ],
};


pub const FO: LanguageCode = LanguageCode {
    name: "Faroese",
    code: "fo",
    code_2t: "fao",
    code_2b: "fao",
    code_3: "fao",
    individual_languages: &[
    ],
};


pub const FJ: LanguageCode = LanguageCode {
    name: "Fijian",
    code: "fj",
    code_2t: "fij",
    code_2b: "fij",
    code_3: "fij",
    individual_languages: &[
    ],
};


pub const FI: LanguageCode = LanguageCode {
    name: "Finnish",
    code: "fi",
    code_2t: "fin",
    code_2b: "fin",
    code_3: "fin",
    individual_languages: &[
    ],
};


pub const FR: LanguageCode = LanguageCode {
    name: "French",
    code: "fr",
    code_2t: "fra",
    code_2b: "fre",
    code_3: "fra",
    individual_languages: &[
    ],
};


pub const FY: LanguageCode = LanguageCode {
    name: "Western Frisian",
    code: "fy",
    code_2t: "fry",
    code_2b: "fry",
    code_3: "fry",
    individual_languages: &[
    ],
};


pub const FF: LanguageCode = LanguageCode {
    name: "Fulah",
    code: "ff",
    code_2t: "ful",
    code_2b: "ful",
    code_3: "ful",
    individual_languages: &[
        IndividualLanguages {
            name: "Maasina Fulfulde",
            code: "ffm",
        },
        IndividualLanguages {
            name: "Adamawa Fulfulde",
            code: "fub",
        },
        IndividualLanguages {
            name: "Pulaar",
            code: "fuc",
        },
        IndividualLanguages {
            name: "Borgu Fulfulde",
            code: "fue",
        },
        IndividualLanguages {
            name: "Pular",
            code: "fuf",
        },
        IndividualLanguages {
            name: "Western Niger Fulfulde",
            code: "fuh",
        },
        IndividualLanguages {
            name: "Bagirmi Fulfulde",
            code: "fui",
        },
        IndividualLanguages {
            name: "Central-Eastern Niger Fulfulde",
            code: "fuq",
        },
        IndividualLanguages {
            name: "Nigerian Fulfulde",
            code: "fuv",
        },
    ],
};


pub const GD: LanguageCode = LanguageCode {
    name: "Gaelic, Scottish Gaelic",
    code: "gd",
    code_2t: "gla",
    code_2b: "gla",
    code_3: "gla",
    individual_languages: &[
    ],
};


pub const GL: LanguageCode = LanguageCode {
    name: "Galician",
    code: "gl",
    code_2t: "glg",
    code_2b: "glg",
    code_3: "glg",
    individual_languages: &[
    ],
};


pub const LG: LanguageCode = LanguageCode {
    name: "Ganda",
    code: "lg",
    code_2t: "lug",
    code_2b: "lug",
    code_3: "lug",
    individual_languages: &[
    ],
};


pub const KA: LanguageCode = LanguageCode {
    name: "Georgian",
    code: "ka",
    code_2t: "kat",
    code_2b: "geo",
    code_3: "kat",
    individual_languages: &[
    ],
};


pub const DE: LanguageCode = LanguageCode {
    name: "German",
    code: "de",
    code_2t: "deu",
    code_2b: "ger",
    code_3: "deu",
    individual_languages: &[
    ],
};


pub const EL: LanguageCode = LanguageCode {
    name: "Greek, Modern (1453–)",
    code: "el",
    code_2t: "ell",
    code_2b: "gre",
    code_3: "ell",
    individual_languages: &[
    ],
};


pub const KL: LanguageCode = LanguageCode {
    name: "Kalaallisut, Greenlandic",
    code: "kl",
    code_2t: "kal",
    code_2b: "kal",
    code_3: "kal",
    individual_languages: &[
    ],
};


pub const GN: LanguageCode = LanguageCode {
    name: "Guarani",
    code: "gn",
    code_2t: "grn",
    code_2b: "grn",
    code_3: "grn",
    individual_languages: &[
        IndividualLanguages {
            name: "Western Bolivian Guaraní",
            code: "gnw",
        },
        IndividualLanguages {
            name: "Paraguayan Guaraní",
            code: "gug",
        },
        IndividualLanguages {
            name: "Eastern Bolivian Guaraní",
            code: "gui",
        },
        IndividualLanguages {
            name: "Mbyá Guaraní",
            code: "gun",
        },
        IndividualLanguages {
            name: "Chiripá",
            code: "nhd",
        },
    ],
};


pub const GU: LanguageCode = LanguageCode {
    name: "Gujarati",
    code: "gu",
    code_2t: "guj",
    code_2b: "guj",
    code_3: "guj",
    individual_languages: &[
    ],
};


pub const HT: LanguageCode = LanguageCode {
    name: "Haitian, Haitian Creole",
    code: "ht",
    code_2t: "hat",
    code_2b: "hat",
    code_3: "hat",
    individual_languages: &[
    ],
};


pub const HA: LanguageCode = LanguageCode {
    name: "Hausa",
    code: "ha",
    code_2t: "hau",
    code_2b: "hau",
    code_3: "hau",
    individual_languages: &[
    ],
};


pub const HE: LanguageCode = LanguageCode {
    name: "Hebrew",
    code: "he",
    code_2t: "heb",
    code_2b: "heb",
    code_3: "heb",
    individual_languages: &[
    ],
};


pub const HZ: LanguageCode = LanguageCode {
    name: "Herero",
    code: "hz",
    code_2t: "her",
    code_2b: "her",
    code_3: "her",
    individual_languages: &[
    ],
};


pub const HI: LanguageCode = LanguageCode {
    name: "Hindi",
    code: "hi",
    code_2t: "hin",
    code_2b: "hin",
    code_3: "hin",
    individual_languages: &[
    ],
};


pub const HO: LanguageCode = LanguageCode {
    name: "Hiri Motu",
    code: "ho",
    code_2t: "hmo",
    code_2b: "hmo",
    code_3: "hmo",
    individual_languages: &[
    ],
};


pub const HU: LanguageCode = LanguageCode {
    name: "Hungarian",
    code: "hu",
    code_2t: "hun",
    code_2b: "hun",
    code_3: "hun",
    individual_languages: &[
    ],
};


pub const IS: LanguageCode = LanguageCode {
    name: "Icelandic",
    code: "is",
    code_2t: "isl",
    code_2b: "ice",
    code_3: "isl",
    individual_languages: &[
    ],
};


pub const IO: LanguageCode = LanguageCode {
    name: "Ido",
    code: "io",
    code_2t: "ido",
    code_2b: "ido",
    code_3: "ido",
    individual_languages: &[
    ],
};


pub const IG: LanguageCode = LanguageCode {
    name: "Igbo",
    code: "ig",
    code_2t: "ibo",
    code_2b: "ibo",
    code_3: "ibo",
    individual_languages: &[
    ],
};


pub const ID: LanguageCode = LanguageCode {
    name: "Indonesian",
    code: "id",
    code_2t: "ind",
    code_2b: "ind",
    code_3: "ind",
    individual_languages: &[
    ],
};


pub const IA: LanguageCode = LanguageCode {
    name: "Interlingua (International Auxiliary Language Association)",
    code: "ia",
    code_2t: "ina",
    code_2b: "ina",
    code_3: "ina",
    individual_languages: &[
    ],
};


pub const IE: LanguageCode = LanguageCode {
    name: "Interlingue, Occidental",
    code: "ie",
    code_2t: "ile",
    code_2b: "ile",
    code_3: "ile",
    individual_languages: &[
    ],
};


pub const IU: LanguageCode = LanguageCode {
    name: "Inuktitut",
    code: "iu",
    code_2t: "iku",
    code_2b: "iku",
    code_3: "iku",
    individual_languages: &[
        IndividualLanguages {
            name: "Eastern Canadian Inuktitut",
            code: "ike",
        },
        IndividualLanguages {
            name: "Western Canadian Inuktitut",
            code: "ikt",
        },
    ],
};


pub const IK: LanguageCode = LanguageCode {
    name: "Inupiaq",
    code: "ik",
    code_2t: "ipk",
    code_2b: "ipk",
    code_3: "ipk",
    individual_languages: &[
        IndividualLanguages {
            name: "North Alaskan Inupiatun",
            code: "esi",
        },
        IndividualLanguages {
            name: "Northwest Alaska Inupiatun",
            code: "esk",
        },
    ],
};


pub const GA: LanguageCode = LanguageCode {
    name: "Irish",
    code: "ga",
    code_2t: "gle",
    code_2b: "gle",
    code_3: "gle",
    individual_languages: &[
    ],
};


pub const IT: LanguageCode = LanguageCode {
    name: "Italian",
    code: "it",
    code_2t: "ita",
    code_2b: "ita",
    code_3: "ita",
    individual_languages: &[
    ],
};


pub const JA: LanguageCode = LanguageCode {
    name: "Japanese",
    code: "ja",
    code_2t: "jpn",
    code_2b: "jpn",
    code_3: "jpn",
    individual_languages: &[
    ],
};


pub const JV: LanguageCode = LanguageCode {
    name: "Javanese",
    code: "jv",
    code_2t: "jav",
    code_2b: "jav",
    code_3: "jav",
    individual_languages: &[
    ],
};


pub const KN: LanguageCode = LanguageCode {
    name: "Kannada",
    code: "kn",
    code_2t: "kan",
    code_2b: "kan",
    code_3: "kan",
    individual_languages: &[
    ],
};


pub const KR: LanguageCode = LanguageCode {
    name: "Kanuri",
    code: "kr",
    code_2t: "kau",
    code_2b: "kau",
    code_3: "kau",
    individual_languages: &[
        IndividualLanguages {
            name: "Manga Kanuri",
            code: "kby",
        },
        IndividualLanguages {
            name: "Central Kanuri",
            code: "knc",
        },
        IndividualLanguages {
            name: "Tumari Kanuri",
            code: "krt",
        },
    ],
};


pub const KS: LanguageCode = LanguageCode {
    name: "Kashmiri",
    code: "ks",
    code_2t: "kas",
    code_2b: "kas",
    code_3: "kas",
    individual_languages: &[
    ],
};


pub const KK: LanguageCode = LanguageCode {
    name: "Kazakh",
    code: "kk",
    code_2t: "kaz",
    code_2b: "kaz",
    code_3: "kaz",
    individual_languages: &[
    ],
};


pub const KM: LanguageCode = LanguageCode {
    name: "Central Khmer",
    code: "km",
    code_2t: "khm",
    code_2b: "khm",
    code_3: "khm",
    individual_languages: &[
    ],
};


pub const KI: LanguageCode = LanguageCode {
    name: "Kikuyu, Gikuyu",
    code: "ki",
    code_2t: "kik",
    code_2b: "kik",
    code_3: "kik",
    individual_languages: &[
    ],
};


pub const RW: LanguageCode = LanguageCode {
    name: "Kinyarwanda",
    code: "rw",
    code_2t: "kin",
    code_2b: "kin",
    code_3: "kin",
    individual_languages: &[
    ],
};


pub const KY: LanguageCode = LanguageCode {
    name: "Kirghiz, Kyrgyz",
    code: "ky",
    code_2t: "kir",
    code_2b: "kir",
    code_3: "kir",
    individual_languages: &[
    ],
};


pub const KV: LanguageCode = LanguageCode {
    name: "Komi",
    code: "kv",
    code_2t: "kom",
    code_2b: "kom",
    code_3: "kom",
    individual_languages: &[
        IndividualLanguages {
            name: "Komi-Permyak",
            code: "koi",
        },
        IndividualLanguages {
            name: "Komi-Zyrian",
            code: "kpv",
        },
    ],
};


pub const KG: LanguageCode = LanguageCode {
    name: "Kongo",
    code: "kg",
    code_2t: "kon",
    code_2b: "kon",
    code_3: "kon",
    individual_languages: &[
        IndividualLanguages {
            name: "Koongo",
            code: "kng",
        },
        IndividualLanguages {
            name: "San Salvador Kongo",
            code: "kwy",
        },
        IndividualLanguages {
            name: "Laari",
            code: "ldi",
        },
    ],
};


pub const KO: LanguageCode = LanguageCode {
    name: "Korean",
    code: "ko",
    code_2t: "kor",
    code_2b: "kor",
    code_3: "kor",
    individual_languages: &[
    ],
};


pub const KJ: LanguageCode = LanguageCode {
    name: "Kuanyama, Kwanyama",
    code: "kj",
    code_2t: "kua",
    code_2b: "kua",
    code_3: "kua",
    individual_languages: &[
    ],
};


pub const KU: LanguageCode = LanguageCode {
    name: "Kurdish",
    code: "ku",
    code_2t: "kur",
    code_2b: "kur",
    code_3: "kur",
    individual_languages: &[
        IndividualLanguages {
            name: "Central Kurdish",
            code: "ckb",
        },
        IndividualLanguages {
            name: "Northern Kurdish",
            code: "kmr",
        },
        IndividualLanguages {
            name: "Southern Kurdish",
            code: "sdh",
        },
    ],
};


pub const LO: LanguageCode = LanguageCode {
    name: "Lao",
    code: "lo",
    code_2t: "lao",
    code_2b: "lao",
    code_3: "lao",
    individual_languages: &[
    ],
};


pub const LA: LanguageCode = LanguageCode {
    name: "Latin",
    code: "la",
    code_2t: "lat",
    code_2b: "lat",
    code_3: "lat",
    individual_languages: &[
    ],
};


pub const LV: LanguageCode = LanguageCode {
    name: "Latvian",
    code: "lv",
    code_2t: "lav",
    code_2b: "lav",
    code_3: "lav",
    individual_languages: &[
        IndividualLanguages {
            name: "Latgalian",
            code: "ltg",
        },
        IndividualLanguages {
            name: "Standard Latvian",
            code: "lvs",
        },
    ],
};


pub const LI: LanguageCode = LanguageCode {
    name: "Limburgan, Limburger, Limburgish",
    code: "li",
    code_2t: "lim",
    code_2b: "lim",
    code_3: "lim",
    individual_languages: &[
    ],
};


pub const LN: LanguageCode = LanguageCode {
    name: "Lingala",
    code: "ln",
    code_2t: "lin",
    code_2b: "lin",
    code_3: "lin",
    individual_languages: &[
    ],
};


pub const LT: LanguageCode = LanguageCode {
    name: "Lithuanian",
    code: "lt",
    code_2t: "lit",
    code_2b: "lit",
    code_3: "lit",
    individual_languages: &[
    ],
};


pub const LU: LanguageCode = LanguageCode {
    name: "Luba-Katanga",
    code: "lu",
    code_2t: "lub",
    code_2b: "lub",
    code_3: "lub",
    individual_languages: &[
    ],
};


pub const LB: LanguageCode = LanguageCode {
    name: "Luxembourgish, Letzeburgesch",
    code: "lb",
    code_2t: "ltz",
    code_2b: "ltz",
    code_3: "ltz",
    individual_languages: &[
    ],
};


pub const MK: LanguageCode = LanguageCode {
    name: "Macedonian",
    code: "mk",
    code_2t: "mkd",
    code_2b: "mac",
    code_3: "mkd",
    individual_languages: &[
    ],
};


pub const MG: LanguageCode = LanguageCode {
    name: "Malagasy",
    code: "mg",
    code_2t: "mlg",
    code_2b: "mlg",
    code_3: "mlg",
    individual_languages: &[
        IndividualLanguages {
            name: "Bara Malagasy",
            code: "bhr",
        },
        IndividualLanguages {
            name: "",
            code: "bjq",
        },
        IndividualLanguages {
            name: "Northern Betsimisaraka Malagasy",
            code: "bmm",
        },
        IndividualLanguages {
            name: "Southern Betsimisaraka Malagasy",
            code: "bzc",
        },
        IndividualLanguages {
            name: "Masikoro Malagasy",
            code: "msh",
        },
        IndividualLanguages {
            name: "Plateau Malagasy",
            code: "plt",
        },
        IndividualLanguages {
            name: "Sakalava Malagasy",
            code: "skg",
        },
        IndividualLanguages {
            name: "Tandroy-Mahafaly Malagasy",
            code: "tdx",
        },
        IndividualLanguages {
            name: "Tesaka Malagasy",
            code: "tkg",
        },
        IndividualLanguages {
            name: "Tanosy Malagasy",
            code: "txy",
        },
        IndividualLanguages {
            name: "Tankarana Malagasy",
            code: "xmv",
        },
        IndividualLanguages {
            name: "Tsimihety Malagasy",
            code: "xmw",
        },
    ],
};


pub const MS: LanguageCode = LanguageCode {
    name: "Malay",
    code: "ms",
    code_2t: "msa",
    code_2b: "may",
    code_3: "msa",
    individual_languages: &[
        IndividualLanguages {
            name: "Banjar",
            code: "bjn",
        },
        IndividualLanguages {
            name: "Bacanese Malay",
            code: "btj",
        },
        IndividualLanguages {
            name: "Berau Malay",
            code: "bve",
        },
        IndividualLanguages {
            name: "Bukit Malay",
            code: "bvu",
        },
        IndividualLanguages {
            name: "Cocos Islands Malay",
            code: "coa",
        },
        IndividualLanguages {
            name: "Duano",
            code: "dup",
        },
        IndividualLanguages {
            name: "Haji",
            code: "hji",
        },
        IndividualLanguages {
            name: "Indonesian",
            code: "ind",
        },
        IndividualLanguages {
            name: "Jakun",
            code: "jak",
        },
        IndividualLanguages {
            name: "Jambi Malay",
            code: "jax",
        },
        IndividualLanguages {
            name: "Kubu",
            code: "kvb",
        },
        IndividualLanguages {
            name: "Kerinci",
            code: "kvr",
        },
        IndividualLanguages {
            name: "Brunei",
            code: "kxd",
        },
        IndividualLanguages {
            name: "Sekak",
            code: "lce",
        },
        IndividualLanguages {
            name: "Lubu",
            code: "lcf",
        },
        IndividualLanguages {
            name: "Col",
            code: "liw",
        },
        IndividualLanguages {
            name: "North Moluccan Malay",
            code: "max",
        },
        IndividualLanguages {
            name: "Kedah Malay",
            code: "meo",
        },
        IndividualLanguages {
            name: "Pattani Malay",
            code: "mfa",
        },
        IndividualLanguages {
            name: "Bangka",
            code: "mfb",
        },
        IndividualLanguages {
            name: "Minangkabau",
            code: "min",
        },
        IndividualLanguages {
            name: "",
            code: "mly",
        },
        IndividualLanguages {
            name: "Kota Bangun Kutai Malay",
            code: "mqg",
        },
        IndividualLanguages {
            name: "Sabah Malay",
            code: "msi",
        },
        IndividualLanguages {
            name: "Musi",
            code: "mui",
        },
        IndividualLanguages {
            name: "Orang Kanaq",
            code: "orn",
        },
        IndividualLanguages {
            name: "Orang Seletar",
            code: "ors",
        },
        IndividualLanguages {
            name: "Pekal",
            code: "pel",
        },
        IndividualLanguages {
            name: "Central Malay",
            code: "pse",
        },
        IndividualLanguages {
            name: "Temuan",
            code: "tmw",
        },
        IndividualLanguages {
            name: "Urak Lawoi'",
            code: "urk",
        },
        IndividualLanguages {
            name: "Kaur",
            code: "vkk",
        },
        IndividualLanguages {
            name: "Tenggarong Kutai Malay",
            code: "vkt",
        },
        IndividualLanguages {
            name: "Manado Malay",
            code: "xmm",
        },
        IndividualLanguages {
            name: "Malay (individual language)",
            code: "zlm",
        },
        IndividualLanguages {
            name: "Negeri Sembilan Malay",
            code: "zmi",
        },
        IndividualLanguages {
            name: "Standard Malay",
            code: "zsm",
        },
    ],
};


pub const ML: LanguageCode = LanguageCode {
    name: "Malayalam",
    code: "ml",
    code_2t: "mal",
    code_2b: "mal",
    code_3: "mal",
    individual_languages: &[
    ],
};


pub const MT: LanguageCode = LanguageCode {
    name: "Maltese",
    code: "mt",
    code_2t: "mlt",
    code_2b: "mlt",
    code_3: "mlt",
    individual_languages: &[
    ],
};


pub const GV: LanguageCode = LanguageCode {
    name: "Manx",
    code: "gv",
    code_2t: "glv",
    code_2b: "glv",
    code_3: "glv",
    individual_languages: &[
    ],
};


pub const MI: LanguageCode = LanguageCode {
    name: "Maori",
    code: "mi",
    code_2t: "mri",
    code_2b: "mao",
    code_3: "mri",
    individual_languages: &[
    ],
};


pub const MR: LanguageCode = LanguageCode {
    name: "Marathi",
    code: "mr",
    code_2t: "mar",
    code_2b: "mar",
    code_3: "mar",
    individual_languages: &[
    ],
};


pub const MH: LanguageCode = LanguageCode {
    name: "Marshallese",
    code: "mh",
    code_2t: "mah",
    code_2b: "mah",
    code_3: "mah",
    individual_languages: &[
    ],
};


pub const MN: LanguageCode = LanguageCode {
    name: "Mongolian",
    code: "mn",
    code_2t: "mon",
    code_2b: "mon",
    code_3: "mon",
    individual_languages: &[
        IndividualLanguages {
            name: "Halh Mongolian",
            code: "khk",
        },
        IndividualLanguages {
            name: "Peripheral Mongolian",
            code: "mvf",
        },
    ],
};


pub const NA: LanguageCode = LanguageCode {
    name: "Nauru",
    code: "na",
    code_2t: "nau",
    code_2b: "nau",
    code_3: "nau",
    individual_languages: &[
    ],
};


pub const NV: LanguageCode = LanguageCode {
    name: "Navajo, Navaho",
    code: "nv",
    code_2t: "nav",
    code_2b: "nav",
    code_3: "nav",
    individual_languages: &[
    ],
};


pub const ND: LanguageCode = LanguageCode {
    name: "North Ndebele",
    code: "nd",
    code_2t: "nde",
    code_2b: "nde",
    code_3: "nde",
    individual_languages: &[
    ],
};


pub const NR: LanguageCode = LanguageCode {
    name: "South Ndebele",
    code: "nr",
    code_2t: "nbl",
    code_2b: "nbl",
    code_3: "nbl",
    individual_languages: &[
    ],
};


pub const NG: LanguageCode = LanguageCode {
    name: "Ndonga",
    code: "ng",
    code_2t: "ndo",
    code_2b: "ndo",
    code_3: "ndo",
    individual_languages: &[
    ],
};


pub const NE: LanguageCode = LanguageCode {
    name: "Nepali",
    code: "ne",
    code_2t: "nep",
    code_2b: "nep",
    code_3: "nep",
    individual_languages: &[
        IndividualLanguages {
            name: "Dotyali",
            code: "dty",
        },
        IndividualLanguages {
            name: "Nepali (individual language)",
            code: "npi",
        },
    ],
};


pub const NO: LanguageCode = LanguageCode {
    name: "Norwegian",
    code: "no",
    code_2t: "nor",
    code_2b: "nor",
    code_3: "nor",
    individual_languages: &[
        IndividualLanguages {
            name: "Norwegian Nynorsk",
            code: "nno",
        },
        IndividualLanguages {
            name: "Norwegian Bokmål",
            code: "nob",
        },
    ],
};


pub const NB: LanguageCode = LanguageCode {
    name: "Norwegian Bokmål",
    code: "nb",
    code_2t: "nob",
    code_2b: "nob",
    code_3: "nob",
    individual_languages: &[
    ],
};


pub const NN: LanguageCode = LanguageCode {
    name: "Norwegian Nynorsk",
    code: "nn",
    code_2t: "nno",
    code_2b: "nno",
    code_3: "nno",
    individual_languages: &[
    ],
};


pub const II: LanguageCode = LanguageCode {
    name: "Sichuan Yi, Nuosu",
    code: "ii",
    code_2t: "iii",
    code_2b: "iii",
    code_3: "iii",
    individual_languages: &[
    ],
};


pub const OC: LanguageCode = LanguageCode {
    name: "Occitan",
    code: "oc",
    code_2t: "oci",
    code_2b: "oci",
    code_3: "oci",
    individual_languages: &[
    ],
};


pub const OJ: LanguageCode = LanguageCode {
    name: "Ojibwa",
    code: "oj",
    code_2t: "oji",
    code_2b: "oji",
    code_3: "oji",
    individual_languages: &[
        IndividualLanguages {
            name: "Chippewa",
            code: "ciw",
        },
        IndividualLanguages {
            name: "Northwestern Ojibwa",
            code: "ojb",
        },
        IndividualLanguages {
            name: "Central Ojibwa",
            code: "ojc",
        },
        IndividualLanguages {
            name: "Eastern Ojibwa",
            code: "ojg",
        },
        IndividualLanguages {
            name: "Severn Ojibwa",
            code: "ojs",
        },
        IndividualLanguages {
            name: "Western Ojibwa",
            code: "ojw",
        },
        IndividualLanguages {
            name: "Ottawa",
            code: "otw",
        },
    ],
};


pub const OR: LanguageCode = LanguageCode {
    name: "Oriya",
    code: "or",
    code_2t: "ori",
    code_2b: "ori",
    code_3: "ori",
    individual_languages: &[
        IndividualLanguages {
            name: "Oriya (individual language)",
            code: "ory",
        },
        IndividualLanguages {
            name: "Sambalpuri",
            code: "spv",
        },
    ],
};


pub const OM: LanguageCode = LanguageCode {
    name: "Oromo",
    code: "om",
    code_2t: "orm",
    code_2b: "orm",
    code_3: "orm",
    individual_languages: &[
        IndividualLanguages {
            name: "Borana-Arsi-Guji Oromo",
            code: "gax",
        },
        IndividualLanguages {
            name: "West Central Oromo",
            code: "gaz",
        },
        IndividualLanguages {
            name: "Eastern Oromo",
            code: "hae",
        },
        IndividualLanguages {
            name: "Orma",
            code: "orc",
        },
    ],
};


pub const OS: LanguageCode = LanguageCode {
    name: "Ossetian, Ossetic",
    code: "os",
    code_2t: "oss",
    code_2b: "oss",
    code_3: "oss",
    individual_languages: &[
    ],
};


pub const PI: LanguageCode = LanguageCode {
    name: "Pali",
    code: "pi",
    code_2t: "pli",
    code_2b: "pli",
    code_3: "pli",
    individual_languages: &[
    ],
};


pub const PS: LanguageCode = LanguageCode {
    name: "Pashto, Pushto",
    code: "ps",
    code_2t: "pus",
    code_2b: "pus",
    code_3: "pus",
    individual_languages: &[
        IndividualLanguages {
            name: "Southern Pashto",
            code: "pbt",
        },
        IndividualLanguages {
            name: "Northern Pashto",
            code: "pbu",
        },
        IndividualLanguages {
            name: "Central Pashto",
            code: "pst",
        },
    ],
};


pub const FA: LanguageCode = LanguageCode {
    name: "Persian",
    code: "fa",
    code_2t: "fas",
    code_2b: "per",
    code_3: "fas",
    individual_languages: &[
        IndividualLanguages {
            name: "Iranian Persian",
            code: "pes",
        },
        IndividualLanguages {
            name: "Dari",
            code: "prs",
        },
    ],
};


pub const PL: LanguageCode = LanguageCode {
    name: "Polish",
    code: "pl",
    code_2t: "pol",
    code_2b: "pol",
    code_3: "pol",
    individual_languages: &[
    ],
};


pub const PT: LanguageCode = LanguageCode {
    name: "Portuguese",
    code: "pt",
    code_2t: "por",
    code_2b: "por",
    code_3: "por",
    individual_languages: &[
    ],
};


pub const PA: LanguageCode = LanguageCode {
    name: "Punjabi, Panjabi",
    code: "pa",
    code_2t: "pan",
    code_2b: "pan",
    code_3: "pan",
    individual_languages: &[
    ],
};


pub const QU: LanguageCode = LanguageCode {
    name: "Quechua",
    code: "qu",
    code_2t: "que",
    code_2b: "que",
    code_3: "que",
    individual_languages: &[
        IndividualLanguages {
            name: "",
            code: "cqu",
        },
        IndividualLanguages {
            name: "Huallaga Huánuco Quechua",
            code: "qub",
        },
        IndividualLanguages {
            name: "Calderón Highland Quichua",
            code: "qud",
        },
        IndividualLanguages {
            name: "Lambayeque Quechua",
            code: "quf",
        },
        IndividualLanguages {
            name: "Chimborazo Highland Quichua",
            code: "qug",
        },
        IndividualLanguages {
            name: "South Bolivian Quechua",
            code: "quh",
        },
        IndividualLanguages {
            name: "Chachapoyas Quechua",
            code: "quk",
        },
        IndividualLanguages {
            name: "North Bolivian Quechua",
            code: "qul",
        },
        IndividualLanguages {
            name: "Southern Pastaza Quechua",
            code: "qup",
        },
        IndividualLanguages {
            name: "Yanahuanca Pasco Quechua",
            code: "qur",
        },
        IndividualLanguages {
            name: "Santiago del Estero Quichua",
            code: "qus",
        },
        IndividualLanguages {
            name: "Tena Lowland Quichua",
            code: "quw",
        },
        IndividualLanguages {
            name: "Yauyos Quechua",
            code: "qux",
        },
        IndividualLanguages {
            name: "Ayacucho Quechua",
            code: "quy",
        },
        IndividualLanguages {
            name: "Cusco Quechua",
            code: "quz",
        },
        IndividualLanguages {
            name: "Ambo-Pasco Quechua",
            code: "qva",
        },
        IndividualLanguages {
            name: "Cajamarca Quechua",
            code: "qvc",
        },
        IndividualLanguages {
            name: "Eastern Apurímac Quechua",
            code: "qve",
        },
        IndividualLanguages {
            name: "Huamalíes-Dos de Mayo Huánuco Quechua",
            code: "qvh",
        },
        IndividualLanguages {
            name: "Imbabura Highland Quichua",
            code: "qvi",
        },
        IndividualLanguages {
            name: "Loja Highland Quichua",
            code: "qvj",
        },
        IndividualLanguages {
            name: "Cajatambo North Lima Quechua",
            code: "qvl",
        },
        IndividualLanguages {
            name: "Margos-Yarowilca-Lauricocha Quechua",
            code: "qvm",
        },
        IndividualLanguages {
            name: "North Junín Quechua",
            code: "qvn",
        },
        IndividualLanguages {
            name: "Napo Lowland Quechua",
            code: "qvo",
        },
        IndividualLanguages {
            name: "Pacaraos Quechua",
            code: "qvp",
        },
        IndividualLanguages {
            name: "San Martín Quechua",
            code: "qvs",
        },
        IndividualLanguages {
            name: "Huaylla Wanca Quechua",
            code: "qvw",
        },
        IndividualLanguages {
            name: "Northern Pastaza Quichua",
            code: "qvz",
        },
        IndividualLanguages {
            name: "Corongo Ancash Quechua",
            code: "qwa",
        },
        IndividualLanguages {
            name: "Classical Quechua",
            code: "qwc",
        },
        IndividualLanguages {
            name: "Huaylas Ancash Quechua",
            code: "qwh",
        },
        IndividualLanguages {
            name: "Sihuas Ancash Quechua",
            code: "qws",
        },
        IndividualLanguages {
            name: "Chiquián Ancash Quechua",
            code: "qxa",
        },
        IndividualLanguages {
            name: "Chincha Quechua",
            code: "qxc",
        },
        IndividualLanguages {
            name: "Panao Huánuco Quechua",
            code: "qxh",
        },
        IndividualLanguages {
            name: "Salasaca Highland Quichua",
            code: "qxl",
        },
        IndividualLanguages {
            name: "Northern Conchucos Ancash Quechua",
            code: "qxn",
        },
        IndividualLanguages {
            name: "Southern Conchucos Ancash Quechua",
            code: "qxo",
        },
        IndividualLanguages {
            name: "Puno Quechua",
            code: "qxp",
        },
        IndividualLanguages {
            name: "Cañar Highland Quichua",
            code: "qxr",
        },
        IndividualLanguages {
            name: "Santa Ana de Tusi Pasco Quechua",
            code: "qxt",
        },
        IndividualLanguages {
            name: "Arequipa-La Unión Quechua",
            code: "qxu",
        },
        IndividualLanguages {
            name: "Jauja Wanca Quechua",
            code: "qxw",
        },
    ],
};


pub const RO: LanguageCode = LanguageCode {
    name: "Moldavian, Moldovan",
    code: "ro",
    code_2t: "ron",
    code_2b: "rum",
    code_3: "ron",
    individual_languages: &[
    ],
};


pub const RM: LanguageCode = LanguageCode {
    name: "Romansh",
    code: "rm",
    code_2t: "roh",
    code_2b: "roh",
    code_3: "roh",
    individual_languages: &[
    ],
};


pub const RN: LanguageCode = LanguageCode {
    name: "Rundi",
    code: "rn",
    code_2t: "run",
    code_2b: "run",
    code_3: "run",
    individual_languages: &[
    ],
};


pub const RU: LanguageCode = LanguageCode {
    name: "Russian",
    code: "ru",
    code_2t: "rus",
    code_2b: "rus",
    code_3: "rus",
    individual_languages: &[
    ],
};


pub const SE: LanguageCode = LanguageCode {
    name: "Northern Sami",
    code: "se",
    code_2t: "sme",
    code_2b: "sme",
    code_3: "sme",
    individual_languages: &[
    ],
};


pub const SM: LanguageCode = LanguageCode {
    name: "Samoan",
    code: "sm",
    code_2t: "smo",
    code_2b: "smo",
    code_3: "smo",
    individual_languages: &[
    ],
};


pub const SG: LanguageCode = LanguageCode {
    name: "Sango",
    code: "sg",
    code_2t: "sag",
    code_2b: "sag",
    code_3: "sag",
    individual_languages: &[
    ],
};


pub const SA: LanguageCode = LanguageCode {
    name: "Sanskrit",
    code: "sa",
    code_2t: "san",
    code_2b: "san",
    code_3: "san",
    individual_languages: &[
    ],
};


pub const SC: LanguageCode = LanguageCode {
    name: "Sardinian",
    code: "sc",
    code_2t: "srd",
    code_2b: "srd",
    code_3: "srd",
    individual_languages: &[
        IndividualLanguages {
            name: "Sassarese Sardinian",
            code: "sdc",
        },
        IndividualLanguages {
            name: "Gallurese Sardinian",
            code: "sdn",
        },
        IndividualLanguages {
            name: "Logudorese Sardinian",
            code: "src",
        },
        IndividualLanguages {
            name: "Campidanese Sardinian",
            code: "sro",
        },
    ],
};


pub const SR: LanguageCode = LanguageCode {
    name: "Serbian",
    code: "sr",
    code_2t: "srp",
    code_2b: "srp",
    code_3: "srp",
    individual_languages: &[
    ],
};


pub const SN: LanguageCode = LanguageCode {
    name: "Shona",
    code: "sn",
    code_2t: "sna",
    code_2b: "sna",
    code_3: "sna",
    individual_languages: &[
    ],
};


pub const SD: LanguageCode = LanguageCode {
    name: "Sindhi",
    code: "sd",
    code_2t: "snd",
    code_2b: "snd",
    code_3: "snd",
    individual_languages: &[
    ],
};


pub const SI: LanguageCode = LanguageCode {
    name: "Sinhala, Sinhalese",
    code: "si",
    code_2t: "sin",
    code_2b: "sin",
    code_3: "sin",
    individual_languages: &[
    ],
};


pub const SK: LanguageCode = LanguageCode {
    name: "Slovak",
    code: "sk",
    code_2t: "slk",
    code_2b: "slo",
    code_3: "slk",
    individual_languages: &[
    ],
};


pub const SL: LanguageCode = LanguageCode {
    name: "Slovenian",
    code: "sl",
    code_2t: "slv",
    code_2b: "slv",
    code_3: "slv",
    individual_languages: &[
    ],
};


pub const SO: LanguageCode = LanguageCode {
    name: "Somali",
    code: "so",
    code_2t: "som",
    code_2b: "som",
    code_3: "som",
    individual_languages: &[
    ],
};


pub const ST: LanguageCode = LanguageCode {
    name: "Southern Sotho",
    code: "st",
    code_2t: "sot",
    code_2b: "sot",
    code_3: "sot",
    individual_languages: &[
    ],
};


pub const ES: LanguageCode = LanguageCode {
    name: "Spanish, Castilian",
    code: "es",
    code_2t: "spa",
    code_2b: "spa",
    code_3: "spa",
    individual_languages: &[
    ],
};


pub const SU: LanguageCode = LanguageCode {
    name: "Sundanese",
    code: "su",
    code_2t: "sun",
    code_2b: "sun",
    code_3: "sun",
    individual_languages: &[
    ],
};


pub const SW: LanguageCode = LanguageCode {
    name: "Swahili",
    code: "sw",
    code_2t: "swa",
    code_2b: "swa",
    code_3: "swa",
    individual_languages: &[
        IndividualLanguages {
            name: "Congo Swahili",
            code: "swc",
        },
        IndividualLanguages {
            name: "Swahili (individual language)",
            code: "swh",
        },
    ],
};


pub const SS: LanguageCode = LanguageCode {
    name: "Swati",
    code: "ss",
    code_2t: "ssw",
    code_2b: "ssw",
    code_3: "ssw",
    individual_languages: &[
    ],
};


pub const SV: LanguageCode = LanguageCode {
    name: "Swedish",
    code: "sv",
    code_2t: "swe",
    code_2b: "swe",
    code_3: "swe",
    individual_languages: &[
    ],
};


pub const TL: LanguageCode = LanguageCode {
    name: "Tagalog",
    code: "tl",
    code_2t: "tgl",
    code_2b: "tgl",
    code_3: "tgl",
    individual_languages: &[
    ],
};


pub const TY: LanguageCode = LanguageCode {
    name: "Tahitian",
    code: "ty",
    code_2t: "tah",
    code_2b: "tah",
    code_3: "tah",
    individual_languages: &[
    ],
};


pub const TG: LanguageCode = LanguageCode {
    name: "Tajik",
    code: "tg",
    code_2t: "tgk",
    code_2b: "tgk",
    code_3: "tgk",
    individual_languages: &[
    ],
};


pub const TA: LanguageCode = LanguageCode {
    name: "Tamil",
    code: "ta",
    code_2t: "tam",
    code_2b: "tam",
    code_3: "tam",
    individual_languages: &[
    ],
};


pub const TT: LanguageCode = LanguageCode {
    name: "Tatar",
    code: "tt",
    code_2t: "tat",
    code_2b: "tat",
    code_3: "tat",
    individual_languages: &[
    ],
};


pub const TE: LanguageCode = LanguageCode {
    name: "Telugu",
    code: "te",
    code_2t: "tel",
    code_2b: "tel",
    code_3: "tel",
    individual_languages: &[
    ],
};


pub const TH: LanguageCode = LanguageCode {
    name: "Thai",
    code: "th",
    code_2t: "tha",
    code_2b: "tha",
    code_3: "tha",
    individual_languages: &[
    ],
};


pub const BO: LanguageCode = LanguageCode {
    name: "Tibetan",
    code: "bo",
    code_2t: "bod",
    code_2b: "tib",
    code_3: "bod",
    individual_languages: &[
    ],
};


pub const TI: LanguageCode = LanguageCode {
    name: "Tigrinya",
    code: "ti",
    code_2t: "tir",
    code_2b: "tir",
    code_3: "tir",
    individual_languages: &[
    ],
};


pub const TO: LanguageCode = LanguageCode {
    name: "Tonga (Tonga Islands)",
    code: "to",
    code_2t: "ton",
    code_2b: "ton",
    code_3: "ton",
    individual_languages: &[
    ],
};


pub const TS: LanguageCode = LanguageCode {
    name: "Tsonga",
    code: "ts",
    code_2t: "tso",
    code_2b: "tso",
    code_3: "tso",
    individual_languages: &[
    ],
};


pub const TN: LanguageCode = LanguageCode {
    name: "Tswana",
    code: "tn",
    code_2t: "tsn",
    code_2b: "tsn",
    code_3: "tsn",
    individual_languages: &[
    ],
};


pub const TR: LanguageCode = LanguageCode {
    name: "Turkish",
    code: "tr",
    code_2t: "tur",
    code_2b: "tur",
    code_3: "tur",
    individual_languages: &[
    ],
};


pub const TK: LanguageCode = LanguageCode {
    name: "Turkmen",
    code: "tk",
    code_2t: "tuk",
    code_2b: "tuk",
    code_3: "tuk",
    individual_languages: &[
    ],
};


pub const TW: LanguageCode = LanguageCode {
    name: "Twi",
    code: "tw",
    code_2t: "twi",
    code_2b: "twi",
    code_3: "twi",
    individual_languages: &[
    ],
};


pub const UG: LanguageCode = LanguageCode {
    name: "Uighur, Uyghur",
    code: "ug",
    code_2t: "uig",
    code_2b: "uig",
    code_3: "uig",
    individual_languages: &[
    ],
};


pub const UK: LanguageCode = LanguageCode {
    name: "Ukrainian",
    code: "uk",
    code_2t: "ukr",
    code_2b: "ukr",
    code_3: "ukr",
    individual_languages: &[
    ],
};


pub const UR: LanguageCode = LanguageCode {
    name: "Urdu",
    code: "ur",
    code_2t: "urd",
    code_2b: "urd",
    code_3: "urd",
    individual_languages: &[
    ],
};


pub const UZ: LanguageCode = LanguageCode {
    name: "Uzbek",
    code: "uz",
    code_2t: "uzb",
    code_2b: "uzb",
    code_3: "uzb",
    individual_languages: &[
        IndividualLanguages {
            name: "Northern Uzbek",
            code: "uzn",
        },
        IndividualLanguages {
            name: "Southern Uzbek",
            code: "uzs",
        },
    ],
};


pub const VE: LanguageCode = LanguageCode {
    name: "Venda",
    code: "ve",
    code_2t: "ven",
    code_2b: "ven",
    code_3: "ven",
    individual_languages: &[
    ],
};


pub const VI: LanguageCode = LanguageCode {
    name: "Vietnamese",
    code: "vi",
    code_2t: "vie",
    code_2b: "vie",
    code_3: "vie",
    individual_languages: &[
    ],
};


pub const VO: LanguageCode = LanguageCode {
    name: "Volapük",
    code: "vo",
    code_2t: "vol",
    code_2b: "vol",
    code_3: "vol",
    individual_languages: &[
    ],
};


pub const WA: LanguageCode = LanguageCode {
    name: "Walloon",
    code: "wa",
    code_2t: "wln",
    code_2b: "wln",
    code_3: "wln",
    individual_languages: &[
    ],
};


pub const CY: LanguageCode = LanguageCode {
    name: "Welsh",
    code: "cy",
    code_2t: "cym",
    code_2b: "wel",
    code_3: "cym",
    individual_languages: &[
    ],
};


pub const WO: LanguageCode = LanguageCode {
    name: "Wolof",
    code: "wo",
    code_2t: "wol",
    code_2b: "wol",
    code_3: "wol",
    individual_languages: &[
    ],
};


pub const XH: LanguageCode = LanguageCode {
    name: "Xhosa",
    code: "xh",
    code_2t: "xho",
    code_2b: "xho",
    code_3: "xho",
    individual_languages: &[
    ],
};


pub const YI: LanguageCode = LanguageCode {
    name: "Yiddish",
    code: "yi",
    code_2t: "yid",
    code_2b: "yid",
    code_3: "yid",
    individual_languages: &[
        IndividualLanguages {
            name: "Eastern Yiddish",
            code: "ydd",
        },
        IndividualLanguages {
            name: "Western Yiddish",
            code: "yih",
        },
    ],
};


pub const YO: LanguageCode = LanguageCode {
    name: "Yoruba",
    code: "yo",
    code_2t: "yor",
    code_2b: "yor",
    code_3: "yor",
    individual_languages: &[
    ],
};


pub const ZA: LanguageCode = LanguageCode {
    name: "Zhuang, Chuang",
    code: "za",
    code_2t: "zha",
    code_2b: "zha",
    code_3: "zha",
    individual_languages: &[
        IndividualLanguages {
            name: "",
            code: "ccx",
        },
        IndividualLanguages {
            name: "",
            code: "ccy",
        },
        IndividualLanguages {
            name: "Central Hongshuihe Zhuang",
            code: "zch",
        },
        IndividualLanguages {
            name: "Eastern Hongshuihe Zhuang",
            code: "zeh",
        },
        IndividualLanguages {
            name: "Guibei Zhuang",
            code: "zgb",
        },
        IndividualLanguages {
            name: "Minz Zhuang",
            code: "zgm",
        },
        IndividualLanguages {
            name: "Guibian Zhuang",
            code: "zgn",
        },
        IndividualLanguages {
            name: "Dai Zhuang",
            code: "zhd",
        },
        IndividualLanguages {
            name: "Nong Zhuang",
            code: "zhn",
        },
        IndividualLanguages {
            name: "Liujiang Zhuang",
            code: "zlj",
        },
        IndividualLanguages {
            name: "Lianshan Zhuang",
            code: "zln",
        },
        IndividualLanguages {
            name: "Liuqian Zhuang",
            code: "zlq",
        },
        IndividualLanguages {
            name: "Qiubei Zhuang",
            code: "zqe",
        },
        IndividualLanguages {
            name: "Yongbei Zhuang",
            code: "zyb",
        },
        IndividualLanguages {
            name: "Yang Zhuang",
            code: "zyg",
        },
        IndividualLanguages {
            name: "Youjiang Zhuang",
            code: "zyj",
        },
        IndividualLanguages {
            name: "Yongnan Zhuang",
            code: "zyn",
        },
        IndividualLanguages {
            name: "Zuojiang Zhuang",
            code: "zzj",
        },
    ],
};


pub const ZU: LanguageCode = LanguageCode {
    name: "Zulu",
    code: "zu",
    code_2t: "zul",
    code_2b: "zul",
    code_3: "zul",
    individual_languages: &[
    ],
};


pub const ACE: LanguageCode = LanguageCode {
    name: "Achinese",
    code: "",
    code_2t: "ace",
    code_2b: "ace",
    code_3: "ace",
    individual_languages: &[
    ],
};


pub const ACH: LanguageCode = LanguageCode {
    name: "Acoli",
    code: "",
    code_2t: "ach",
    code_2b: "ach",
    code_3: "ach",
    individual_languages: &[
    ],
};


pub const ADA: LanguageCode = LanguageCode {
    name: "Adangme",
    code: "",
    code_2t: "ada",
    code_2b: "ada",
    code_3: "ada",
    individual_languages: &[
    ],
};


pub const ADY: LanguageCode = LanguageCode {
    name: "Adyghe; Adygei",
    code: "",
    code_2t: "ady",
    code_2b: "ady",
    code_3: "ady",
    individual_languages: &[
    ],
};


pub const AFA: LanguageCode = LanguageCode {
    name: "Afroasiatic languages|Afro-Asiatic languages",
    code: "",
    code_2t: "afa",
    code_2b: "afa",
    code_3: "",
    individual_languages: &[
    ],
};


pub const AFH: LanguageCode = LanguageCode {
    name: "Afrihili",
    code: "",
    code_2t: "afh",
    code_2b: "afh",
    code_3: "afh",
    individual_languages: &[
    ],
};


pub const AIN: LanguageCode = LanguageCode {
    name: "Ainu",
    code: "",
    code_2t: "ain",
    code_2b: "ain",
    code_3: "ain",
    individual_languages: &[
    ],
};


pub const AKK: LanguageCode = LanguageCode {
    name: "Akkadian",
    code: "",
    code_2t: "akk",
    code_2b: "akk",
    code_3: "akk",
    individual_languages: &[
    ],
};


pub const ALE: LanguageCode = LanguageCode {
    name: "Aleut",
    code: "",
    code_2t: "ale",
    code_2b: "ale",
    code_3: "ale",
    individual_languages: &[
    ],
};


pub const ALG: LanguageCode = LanguageCode {
    name: "Algonquian languages",
    code: "",
    code_2t: "alg",
    code_2b: "alg",
    code_3: "",
    individual_languages: &[
    ],
};


pub const ALT: LanguageCode = LanguageCode {
    name: "Southern Southern Altai",
    code: "",
    code_2t: "alt",
    code_2b: "alt",
    code_3: "alt",
    individual_languages: &[
    ],
};


pub const ANG: LanguageCode = LanguageCode {
    name: "Old English|English, Old (ca.450–1100)",
    code: "",
    code_2t: "ang",
    code_2b: "ang",
    code_3: "ang",
    individual_languages: &[
    ],
};


pub const ANP: LanguageCode = LanguageCode {
    name: "Angika",
    code: "",
    code_2t: "anp",
    code_2b: "anp",
    code_3: "anp",
    individual_languages: &[
    ],
};


pub const APA: LanguageCode = LanguageCode {
    name: "Southern Athabaskan languages|Apache languages",
    code: "",
    code_2t: "apa",
    code_2b: "apa",
    code_3: "",
    individual_languages: &[
    ],
};


pub const ARC: LanguageCode = LanguageCode {
    name: "Old Official Aramaic (700–300 BCE); {{nowrap|Imperial Imperial Aramaic (700–300 BCE)}}",
    code: "",
    code_2t: "arc",
    code_2b: "arc",
    code_3: "arc",
    individual_languages: &[
    ],
};


pub const ARN: LanguageCode = LanguageCode {
    name: "Mapudungun; Mapuche",
    code: "",
    code_2t: "arn",
    code_2b: "arn",
    code_3: "arn",
    individual_languages: &[
    ],
};


pub const ARP: LanguageCode = LanguageCode {
    name: "Arapaho",
    code: "",
    code_2t: "arp",
    code_2b: "arp",
    code_3: "arp",
    individual_languages: &[
    ],
};


pub const ART: LanguageCode = LanguageCode {
    name: "Artificial languages",
    code: "",
    code_2t: "art",
    code_2b: "art",
    code_3: "",
    individual_languages: &[
    ],
};


pub const ARW: LanguageCode = LanguageCode {
    name: "Arawak",
    code: "",
    code_2t: "arw",
    code_2b: "arw",
    code_3: "arw",
    individual_languages: &[
    ],
};


pub const AST: LanguageCode = LanguageCode {
    name: "Asturian; Bable; Leonese; Asturleonese",
    code: "",
    code_2t: "ast",
    code_2b: "ast",
    code_3: "ast",
    individual_languages: &[
    ],
};


pub const ATH: LanguageCode = LanguageCode {
    name: "Athapascan languages",
    code: "",
    code_2t: "ath",
    code_2b: "ath",
    code_3: "",
    individual_languages: &[
    ],
};


pub const AUS: LanguageCode = LanguageCode {
    name: "Australian languages",
    code: "",
    code_2t: "aus",
    code_2b: "aus",
    code_3: "",
    individual_languages: &[
    ],
};


pub const AWA: LanguageCode = LanguageCode {
    name: "Awadhi",
    code: "",
    code_2t: "awa",
    code_2b: "awa",
    code_3: "awa",
    individual_languages: &[
    ],
};


pub const BAD: LanguageCode = LanguageCode {
    name: "Banda languages",
    code: "",
    code_2t: "bad",
    code_2b: "bad",
    code_3: "",
    individual_languages: &[
    ],
};


pub const BAI: LanguageCode = LanguageCode {
    name: "Bamileke languages",
    code: "",
    code_2t: "bai",
    code_2b: "bai",
    code_3: "",
    individual_languages: &[
    ],
};


pub const BAL: LanguageCode = LanguageCode {
    name: "Baluchi",
    code: "",
    code_2t: "bal",
    code_2b: "bal",
    code_3: "bal",
    individual_languages: &[
        IndividualLanguages {
            name: "Southern Balochi",
            code: "bcc",
        },
        IndividualLanguages {
            name: "Western Balochi",
            code: "bgn",
        },
        IndividualLanguages {
            name: "Eastern Balochi",
            code: "bgp",
        },
    ],
};


pub const BAN: LanguageCode = LanguageCode {
    name: "Balinese",
    code: "",
    code_2t: "ban",
    code_2b: "ban",
    code_3: "ban",
    individual_languages: &[
    ],
};


pub const BAS: LanguageCode = LanguageCode {
    name: "Basa",
    code: "",
    code_2t: "bas",
    code_2b: "bas",
    code_3: "bas",
    individual_languages: &[
    ],
};


pub const BAT: LanguageCode = LanguageCode {
    name: "Baltic languages",
    code: "",
    code_2t: "bat",
    code_2b: "bat",
    code_3: "",
    individual_languages: &[
    ],
};


pub const BEJ: LanguageCode = LanguageCode {
    name: "Beja; Bedawiyet",
    code: "",
    code_2t: "bej",
    code_2b: "bej",
    code_3: "bej",
    individual_languages: &[
    ],
};


pub const BEM: LanguageCode = LanguageCode {
    name: "Bemba",
    code: "",
    code_2t: "bem",
    code_2b: "bem",
    code_3: "bem",
    individual_languages: &[
    ],
};


pub const BER: LanguageCode = LanguageCode {
    name: "Berber languages",
    code: "",
    code_2t: "ber",
    code_2b: "ber",
    code_3: "",
    individual_languages: &[
    ],
};


pub const BHO: LanguageCode = LanguageCode {
    name: "Bhojpuri",
    code: "",
    code_2t: "bho",
    code_2b: "bho",
    code_3: "bho",
    individual_languages: &[
    ],
};


pub const BIH: LanguageCode = LanguageCode {
    name: "Bihari languages",
    code: "",
    code_2t: "bih",
    code_2b: "bih",
    code_3: "",
    individual_languages: &[
    ],
};


pub const BIK: LanguageCode = LanguageCode {
    name: "Bikol",
    code: "",
    code_2t: "bik",
    code_2b: "bik",
    code_3: "bik",
    individual_languages: &[
        IndividualLanguages {
            name: "Central Bikol",
            code: "bcl",
        },
        IndividualLanguages {
            name: "",
            code: "bhk",
        },
        IndividualLanguages {
            name: "Southern Catanduanes Bikol",
            code: "bln",
        },
        IndividualLanguages {
            name: "Rinconada Bikol",
            code: "bto",
        },
        IndividualLanguages {
            name: "Northern Catanduanes Bikol",
            code: "cts",
        },
        IndividualLanguages {
            name: "West Albay Bikol",
            code: "fbl",
        },
        IndividualLanguages {
            name: "Libon Bikol",
            code: "lbl",
        },
        IndividualLanguages {
            name: "Miraya Bikol",
            code: "rbl",
        },
        IndividualLanguages {
            name: "Buhi'non Bikol",
            code: "ubl",
        },
    ],
};


pub const BIN: LanguageCode = LanguageCode {
    name: "Bini; Edo",
    code: "",
    code_2t: "bin",
    code_2b: "bin",
    code_3: "bin",
    individual_languages: &[
    ],
};


pub const BLA: LanguageCode = LanguageCode {
    name: "Siksika",
    code: "",
    code_2t: "bla",
    code_2b: "bla",
    code_3: "bla",
    individual_languages: &[
    ],
};


pub const BNT: LanguageCode = LanguageCode {
    name: "Bantu languages",
    code: "",
    code_2t: "bnt",
    code_2b: "bnt",
    code_3: "",
    individual_languages: &[
    ],
};


pub const BRA: LanguageCode = LanguageCode {
    name: "Braj",
    code: "",
    code_2t: "bra",
    code_2b: "bra",
    code_3: "bra",
    individual_languages: &[
    ],
};


pub const BTK: LanguageCode = LanguageCode {
    name: "Batak languages",
    code: "",
    code_2t: "btk",
    code_2b: "btk",
    code_3: "",
    individual_languages: &[
    ],
};


pub const BUA: LanguageCode = LanguageCode {
    name: "Buriat",
    code: "",
    code_2t: "bua",
    code_2b: "bua",
    code_3: "bua",
    individual_languages: &[
        IndividualLanguages {
            name: "Mongolia Buriat",
            code: "bxm",
        },
        IndividualLanguages {
            name: "Russia Buriat",
            code: "bxr",
        },
        IndividualLanguages {
            name: "China Buriat",
            code: "bxu",
        },
    ],
};


pub const BUG: LanguageCode = LanguageCode {
    name: "Buginese",
    code: "",
    code_2t: "bug",
    code_2b: "bug",
    code_3: "bug",
    individual_languages: &[
    ],
};


pub const BYN: LanguageCode = LanguageCode {
    name: "Blin; Bilin",
    code: "",
    code_2t: "byn",
    code_2b: "byn",
    code_3: "byn",
    individual_languages: &[
    ],
};


pub const CAD: LanguageCode = LanguageCode {
    name: "Caddo",
    code: "",
    code_2t: "cad",
    code_2b: "cad",
    code_3: "cad",
    individual_languages: &[
    ],
};


pub const CAI: LanguageCode = LanguageCode {
    name: "Central American Indian languages",
    code: "",
    code_2t: "cai",
    code_2b: "cai",
    code_3: "",
    individual_languages: &[
    ],
};


pub const CAR: LanguageCode = LanguageCode {
    name: "Galibi Galibi Carib",
    code: "",
    code_2t: "car",
    code_2b: "car",
    code_3: "car",
    individual_languages: &[
    ],
};


pub const CAU: LanguageCode = LanguageCode {
    name: "Caucasian languages",
    code: "",
    code_2t: "cau",
    code_2b: "cau",
    code_3: "",
    individual_languages: &[
    ],
};


pub const CEB: LanguageCode = LanguageCode {
    name: "Cebuano",
    code: "",
    code_2t: "ceb",
    code_2b: "ceb",
    code_3: "ceb",
    individual_languages: &[
    ],
};


pub const CEL: LanguageCode = LanguageCode {
    name: "Celtic languages",
    code: "",
    code_2t: "cel",
    code_2b: "cel",
    code_3: "",
    individual_languages: &[
    ],
};


pub const CHB: LanguageCode = LanguageCode {
    name: "Chibcha",
    code: "",
    code_2t: "chb",
    code_2b: "chb",
    code_3: "chb",
    individual_languages: &[
    ],
};


pub const CHG: LanguageCode = LanguageCode {
    name: "Chagatai",
    code: "",
    code_2t: "chg",
    code_2b: "chg",
    code_3: "chg",
    individual_languages: &[
    ],
};


pub const CHK: LanguageCode = LanguageCode {
    name: "Chuukese",
    code: "",
    code_2t: "chk",
    code_2b: "chk",
    code_3: "chk",
    individual_languages: &[
    ],
};


pub const CHM: LanguageCode = LanguageCode {
    name: "Mari",
    code: "",
    code_2t: "chm",
    code_2b: "chm",
    code_3: "chm",
    individual_languages: &[
        IndividualLanguages {
            name: "Eastern Mari",
            code: "mhr",
        },
        IndividualLanguages {
            name: "Western Mari",
            code: "mrj",
        },
    ],
};


pub const CHN: LanguageCode = LanguageCode {
    name: "Chinook Chinook jargon",
    code: "",
    code_2t: "chn",
    code_2b: "chn",
    code_3: "chn",
    individual_languages: &[
    ],
};


pub const CHO: LanguageCode = LanguageCode {
    name: "Choctaw",
    code: "",
    code_2t: "cho",
    code_2b: "cho",
    code_3: "cho",
    individual_languages: &[
    ],
};


pub const CHP: LanguageCode = LanguageCode {
    name: "Chipewyan; Dene Dene Suline",
    code: "",
    code_2t: "chp",
    code_2b: "chp",
    code_3: "chp",
    individual_languages: &[
    ],
};


pub const CHR: LanguageCode = LanguageCode {
    name: "Cherokee",
    code: "",
    code_2t: "chr",
    code_2b: "chr",
    code_3: "chr",
    individual_languages: &[
    ],
};


pub const CHY: LanguageCode = LanguageCode {
    name: "Cheyenne",
    code: "",
    code_2t: "chy",
    code_2b: "chy",
    code_3: "chy",
    individual_languages: &[
    ],
};


pub const CMC: LanguageCode = LanguageCode {
    name: "Chamic languages",
    code: "",
    code_2t: "cmc",
    code_2b: "cmc",
    code_3: "",
    individual_languages: &[
    ],
};


pub const CNR: LanguageCode = LanguageCode {
    name: "Montenegrin",
    code: "",
    code_2t: "cnr",
    code_2b: "cnr",
    code_3: "cnr",
    individual_languages: &[
    ],
};


pub const COP: LanguageCode = LanguageCode {
    name: "Coptic",
    code: "",
    code_2t: "cop",
    code_2b: "cop",
    code_3: "cop",
    individual_languages: &[
    ],
};


pub const CPE: LanguageCode = LanguageCode {
    name: "English-based Creoles and List of English-based pidgins|pidgins, English based",
    code: "",
    code_2t: "cpe",
    code_2b: "cpe",
    code_3: "",
    individual_languages: &[
    ],
};


pub const CPF: LanguageCode = LanguageCode {
    name: "French-based creole languages|Creoles and pidgins, French-based",
    code: "",
    code_2t: "cpf",
    code_2b: "cpf",
    code_3: "",
    individual_languages: &[
    ],
};


pub const CPP: LanguageCode = LanguageCode {
    name: "Portuguese-based creole languages|Creoles and pidgins, Portuguese-based",
    code: "",
    code_2t: "cpp",
    code_2b: "cpp",
    code_3: "",
    individual_languages: &[
    ],
};


pub const CRH: LanguageCode = LanguageCode {
    name: "Crimean Crimean Tatar; Crimean Crimean Turkish",
    code: "",
    code_2t: "crh",
    code_2b: "crh",
    code_3: "crh",
    individual_languages: &[
    ],
};


pub const CRP: LanguageCode = LanguageCode {
    name: "Creoles and pidgins",
    code: "",
    code_2t: "crp",
    code_2b: "crp",
    code_3: "",
    individual_languages: &[
    ],
};


pub const CSB: LanguageCode = LanguageCode {
    name: "Kashubian",
    code: "",
    code_2t: "csb",
    code_2b: "csb",
    code_3: "csb",
    individual_languages: &[
    ],
};


pub const CUS: LanguageCode = LanguageCode {
    name: "Cushitic languages",
    code: "",
    code_2t: "cus",
    code_2b: "cus",
    code_3: "",
    individual_languages: &[
    ],
};


pub const DAK: LanguageCode = LanguageCode {
    name: "Dakota",
    code: "",
    code_2t: "dak",
    code_2b: "dak",
    code_3: "dak",
    individual_languages: &[
    ],
};


pub const DAR: LanguageCode = LanguageCode {
    name: "Dargwa",
    code: "",
    code_2t: "dar",
    code_2b: "dar",
    code_3: "dar",
    individual_languages: &[
    ],
};


pub const DAY: LanguageCode = LanguageCode {
    name: "Land Dayak languages",
    code: "",
    code_2t: "day",
    code_2b: "day",
    code_3: "",
    individual_languages: &[
    ],
};


pub const DEL: LanguageCode = LanguageCode {
    name: "Delaware",
    code: "",
    code_2t: "del",
    code_2b: "del",
    code_3: "del",
    individual_languages: &[
        IndividualLanguages {
            name: "Munsee",
            code: "umu",
        },
        IndividualLanguages {
            name: "Unami",
            code: "unm",
        },
    ],
};


pub const DEN: LanguageCode = LanguageCode {
    name: "Slave language (Athapascan)|Slave (Athapascan)",
    code: "",
    code_2t: "den",
    code_2b: "den",
    code_3: "den",
    individual_languages: &[
        IndividualLanguages {
            name: "North Slavey",
            code: "scs",
        },
        IndividualLanguages {
            name: "South Slavey",
            code: "xsl",
        },
    ],
};


pub const DGR: LanguageCode = LanguageCode {
    name: "Dogrib",
    code: "",
    code_2t: "dgr",
    code_2b: "dgr",
    code_3: "dgr",
    individual_languages: &[
    ],
};


pub const DIN: LanguageCode = LanguageCode {
    name: "Dinka",
    code: "",
    code_2t: "din",
    code_2b: "din",
    code_3: "din",
    individual_languages: &[
        IndividualLanguages {
            name: "South Central Dinka",
            code: "dib",
        },
        IndividualLanguages {
            name: "Southwestern Dinka",
            code: "dik",
        },
        IndividualLanguages {
            name: "Northeastern Dinka",
            code: "dip",
        },
        IndividualLanguages {
            name: "Northwestern Dinka",
            code: "diw",
        },
        IndividualLanguages {
            name: "Southeastern Dinka",
            code: "dks",
        },
    ],
};


pub const DOI: LanguageCode = LanguageCode {
    name: "Dogri",
    code: "",
    code_2t: "doi",
    code_2b: "doi",
    code_3: "doi",
    individual_languages: &[
        IndividualLanguages {
            name: "Dogri (individual language)",
            code: "dgo",
        },
        IndividualLanguages {
            name: "Kangri",
            code: "xnr",
        },
    ],
};


pub const DRA: LanguageCode = LanguageCode {
    name: "Dravidian languages",
    code: "",
    code_2t: "dra",
    code_2b: "dra",
    code_3: "",
    individual_languages: &[
    ],
};


pub const DSB: LanguageCode = LanguageCode {
    name: "Lower Lower Sorbian",
    code: "",
    code_2t: "dsb",
    code_2b: "dsb",
    code_3: "dsb",
    individual_languages: &[
    ],
};


pub const DUA: LanguageCode = LanguageCode {
    name: "Duala",
    code: "",
    code_2t: "dua",
    code_2b: "dua",
    code_3: "dua",
    individual_languages: &[
    ],
};


pub const DUM: LanguageCode = LanguageCode {
    name: "Middle Dutch|Dutch, Middle (ca. 1050–1350)",
    code: "",
    code_2t: "dum",
    code_2b: "dum",
    code_3: "dum",
    individual_languages: &[
    ],
};


pub const DYU: LanguageCode = LanguageCode {
    name: "Dyula",
    code: "",
    code_2t: "dyu",
    code_2b: "dyu",
    code_3: "dyu",
    individual_languages: &[
    ],
};


pub const EFI: LanguageCode = LanguageCode {
    name: "Efik",
    code: "",
    code_2t: "efi",
    code_2b: "efi",
    code_3: "efi",
    individual_languages: &[
    ],
};


pub const EGY: LanguageCode = LanguageCode {
    name: "Egyptian language (Ancient)|Egyptian (Ancient)",
    code: "",
    code_2t: "egy",
    code_2b: "egy",
    code_3: "egy",
    individual_languages: &[
    ],
};


pub const EKA: LanguageCode = LanguageCode {
    name: "Ekajuk",
    code: "",
    code_2t: "eka",
    code_2b: "eka",
    code_3: "eka",
    individual_languages: &[
    ],
};


pub const ELX: LanguageCode = LanguageCode {
    name: "Elamite",
    code: "",
    code_2t: "elx",
    code_2b: "elx",
    code_3: "elx",
    individual_languages: &[
    ],
};


pub const ENM: LanguageCode = LanguageCode {
    name: "Middle English|English, Middle (1100–1500)",
    code: "",
    code_2t: "enm",
    code_2b: "enm",
    code_3: "enm",
    individual_languages: &[
    ],
};


pub const EWO: LanguageCode = LanguageCode {
    name: "Ewondo",
    code: "",
    code_2t: "ewo",
    code_2b: "ewo",
    code_3: "ewo",
    individual_languages: &[
    ],
};


pub const FAN: LanguageCode = LanguageCode {
    name: "Fang",
    code: "",
    code_2t: "fan",
    code_2b: "fan",
    code_3: "fan",
    individual_languages: &[
    ],
};


pub const FAT: LanguageCode = LanguageCode {
    name: "Fanti",
    code: "",
    code_2t: "fat",
    code_2b: "fat",
    code_3: "fat",
    individual_languages: &[
    ],
};


pub const FIL: LanguageCode = LanguageCode {
    name: "Filipino; Pilipino",
    code: "",
    code_2t: "fil",
    code_2b: "fil",
    code_3: "fil",
    individual_languages: &[
    ],
};


pub const FIU: LanguageCode = LanguageCode {
    name: "Finno-Ugrian languages",
    code: "",
    code_2t: "fiu",
    code_2b: "fiu",
    code_3: "",
    individual_languages: &[
    ],
};


pub const FON: LanguageCode = LanguageCode {
    name: "Fon",
    code: "",
    code_2t: "fon",
    code_2b: "fon",
    code_3: "fon",
    individual_languages: &[
    ],
};


pub const FRM: LanguageCode = LanguageCode {
    name: "Middle French|French, Middle (ca. 1400–1600)",
    code: "",
    code_2t: "frm",
    code_2b: "frm",
    code_3: "frm",
    individual_languages: &[
    ],
};


pub const FRO: LanguageCode = LanguageCode {
    name: "Old French|French, Old (842–ca. 1400)",
    code: "",
    code_2t: "fro",
    code_2b: "fro",
    code_3: "fro",
    individual_languages: &[
    ],
};


pub const FRR: LanguageCode = LanguageCode {
    name: "Northern Northern Frisian",
    code: "",
    code_2t: "frr",
    code_2b: "frr",
    code_3: "frr",
    individual_languages: &[
    ],
};


pub const FRS: LanguageCode = LanguageCode {
    name: "East Frisian Low Saxon",
    code: "",
    code_2t: "frs",
    code_2b: "frs",
    code_3: "frs",
    individual_languages: &[
    ],
};


pub const FUR: LanguageCode = LanguageCode {
    name: "Friulian",
    code: "",
    code_2t: "fur",
    code_2b: "fur",
    code_3: "fur",
    individual_languages: &[
    ],
};


pub const GAA: LanguageCode = LanguageCode {
    name: "Ga",
    code: "",
    code_2t: "gaa",
    code_2b: "gaa",
    code_3: "gaa",
    individual_languages: &[
    ],
};


pub const GAY: LanguageCode = LanguageCode {
    name: "Gayo",
    code: "",
    code_2t: "gay",
    code_2b: "gay",
    code_3: "gay",
    individual_languages: &[
    ],
};


pub const GBA: LanguageCode = LanguageCode {
    name: "Gbaya",
    code: "",
    code_2t: "gba",
    code_2b: "gba",
    code_3: "gba",
    individual_languages: &[
        IndividualLanguages {
            name: "Bokoto",
            code: "bdt",
        },
        IndividualLanguages {
            name: "Gbaya-Bossangoa",
            code: "gbp",
        },
        IndividualLanguages {
            name: "Gbaya-Bozoum",
            code: "gbq",
        },
        IndividualLanguages {
            name: "Gbaya-Mbodomo",
            code: "gmm",
        },
        IndividualLanguages {
            name: "Southwest Gbaya",
            code: "gso",
        },
        IndividualLanguages {
            name: "Northwest Gbaya",
            code: "gya",
        },
        IndividualLanguages {
            name: "",
            code: "mdo",
        },
    ],
};


pub const GEM: LanguageCode = LanguageCode {
    name: "Germanic languages",
    code: "",
    code_2t: "gem",
    code_2b: "gem",
    code_3: "",
    individual_languages: &[
    ],
};


pub const GEZ: LanguageCode = LanguageCode {
    name: "Geez",
    code: "",
    code_2t: "gez",
    code_2b: "gez",
    code_3: "gez",
    individual_languages: &[
    ],
};


pub const GIL: LanguageCode = LanguageCode {
    name: "Gilbertese",
    code: "",
    code_2t: "gil",
    code_2b: "gil",
    code_3: "gil",
    individual_languages: &[
    ],
};


pub const GMH: LanguageCode = LanguageCode {
    name: "Middle High German|German, Middle High (ca. 1050–1500)",
    code: "",
    code_2t: "gmh",
    code_2b: "gmh",
    code_3: "gmh",
    individual_languages: &[
    ],
};


pub const GOH: LanguageCode = LanguageCode {
    name: "Old High German|German, Old High (ca. 750–1050)",
    code: "",
    code_2t: "goh",
    code_2b: "goh",
    code_3: "goh",
    individual_languages: &[
    ],
};


pub const GON: LanguageCode = LanguageCode {
    name: "Gondi",
    code: "",
    code_2t: "gon",
    code_2b: "gon",
    code_3: "gon",
    individual_languages: &[
        IndividualLanguages {
            name: "Aheri Gondi",
            code: "esg",
        },
        IndividualLanguages {
            name: "",
            code: "ggo",
        },
        IndividualLanguages {
            name: "Northern Gondi",
            code: "gno",
        },
        IndividualLanguages {
            name: "Adilabad Gondi",
            code: "wsg",
        },
    ],
};


pub const GOR: LanguageCode = LanguageCode {
    name: "Gorontalo",
    code: "",
    code_2t: "gor",
    code_2b: "gor",
    code_3: "gor",
    individual_languages: &[
    ],
};


pub const GOT: LanguageCode = LanguageCode {
    name: "Gothic",
    code: "",
    code_2t: "got",
    code_2b: "got",
    code_3: "got",
    individual_languages: &[
    ],
};


pub const GRB: LanguageCode = LanguageCode {
    name: "Grebo",
    code: "",
    code_2t: "grb",
    code_2b: "grb",
    code_3: "grb",
    individual_languages: &[
        IndividualLanguages {
            name: "Northern Grebo",
            code: "gbo",
        },
        IndividualLanguages {
            name: "Gboloo Grebo",
            code: "gec",
        },
        IndividualLanguages {
            name: "Southern Grebo",
            code: "grj",
        },
        IndividualLanguages {
            name: "Central Grebo",
            code: "grv",
        },
        IndividualLanguages {
            name: "Barclayville Grebo",
            code: "gry",
        },
    ],
};


pub const GRC: LanguageCode = LanguageCode {
    name: "Ancient Greek|Greek, Ancient (to 1453)",
    code: "",
    code_2t: "grc",
    code_2b: "grc",
    code_3: "grc",
    individual_languages: &[
    ],
};


pub const GSW: LanguageCode = LanguageCode {
    name: "Swiss Swiss German; Alemannic; Alsatian",
    code: "",
    code_2t: "gsw",
    code_2b: "gsw",
    code_3: "gsw",
    individual_languages: &[
    ],
};


pub const GWI: LanguageCode = LanguageCode {
    name: "Gwich’Gwich'in",
    code: "",
    code_2t: "gwi",
    code_2b: "gwi",
    code_3: "gwi",
    individual_languages: &[
    ],
};


pub const HAI: LanguageCode = LanguageCode {
    name: "Haida",
    code: "",
    code_2t: "hai",
    code_2b: "hai",
    code_3: "hai",
    individual_languages: &[
        IndividualLanguages {
            name: "Southern Haida",
            code: "hax",
        },
        IndividualLanguages {
            name: "Northern Haida",
            code: "hdn",
        },
    ],
};


pub const HAW: LanguageCode = LanguageCode {
    name: "Hawaiian",
    code: "",
    code_2t: "haw",
    code_2b: "haw",
    code_3: "haw",
    individual_languages: &[
    ],
};


pub const HIL: LanguageCode = LanguageCode {
    name: "Hiligaynon",
    code: "",
    code_2t: "hil",
    code_2b: "hil",
    code_3: "hil",
    individual_languages: &[
    ],
};


pub const HIM: LanguageCode = LanguageCode {
    name: "Western Pahari|Himachali&nbsp;languages; Pahari&nbsp;languages",
    code: "",
    code_2t: "him",
    code_2b: "him",
    code_3: "",
    individual_languages: &[
    ],
};


pub const HIT: LanguageCode = LanguageCode {
    name: "Hittite",
    code: "",
    code_2t: "hit",
    code_2b: "hit",
    code_3: "hit",
    individual_languages: &[
    ],
};


pub const HMN: LanguageCode = LanguageCode {
    name: "Hmong; Mong",
    code: "",
    code_2t: "hmn",
    code_2b: "hmn",
    code_3: "hmn",
    individual_languages: &[
        IndividualLanguages {
            name: "",
            code: "blu",
        },
        IndividualLanguages {
            name: "Chuanqiandian Cluster Miao",
            code: "cqd",
        },
        IndividualLanguages {
            name: "Northern Qiandong Miao",
            code: "hea",
        },
        IndividualLanguages {
            name: "Southern Mashan Miao",
            code: "hma",
        },
        IndividualLanguages {
            name: "Central Huishui Miao",
            code: "hmc",
        },
        IndividualLanguages {
            name: "Large Flowery Miao",
            code: "hmd",
        },
        IndividualLanguages {
            name: "Eastern Huishui Miao",
            code: "hme",
        },
        IndividualLanguages {
            name: "Southwestern Guiyang Hmong",
            code: "hmg",
        },
        IndividualLanguages {
            name: "Southwestern Huishui Miao",
            code: "hmh",
        },
        IndividualLanguages {
            name: "Northern Huishui Miao",
            code: "hmi",
        },
        IndividualLanguages {
            name: "Gejia",
            code: "hmj",
        },
        IndividualLanguages {
            name: "Luopohe Miao",
            code: "hml",
        },
        IndividualLanguages {
            name: "Central Mashan Miao",
            code: "hmm",
        },
        IndividualLanguages {
            name: "Northern Mashan Miao",
            code: "hmp",
        },
        IndividualLanguages {
            name: "Eastern Qiandong Miao",
            code: "hmq",
        },
        IndividualLanguages {
            name: "Southern Qiandong Miao",
            code: "hms",
        },
        IndividualLanguages {
            name: "Western Mashan Miao",
            code: "hmw",
        },
        IndividualLanguages {
            name: "Southern Guiyang Miao",
            code: "hmy",
        },
        IndividualLanguages {
            name: "Sinicized Miao",
            code: "hmz",
        },
        IndividualLanguages {
            name: "Mong Njua",
            code: "hnj",
        },
        IndividualLanguages {
            name: "Horned Miao",
            code: "hrm",
        },
        IndividualLanguages {
            name: "Northern Guiyang Miao",
            code: "huj",
        },
        IndividualLanguages {
            name: "Western Xiangxi Miao",
            code: "mmr",
        },
        IndividualLanguages {
            name: "Eastern Xiangxi Miao",
            code: "muq",
        },
        IndividualLanguages {
            name: "Hmong Daw",
            code: "mww",
        },
        IndividualLanguages {
            name: "Small Flowery Miao",
            code: "sfm",
        },
    ],
};


pub const HSB: LanguageCode = LanguageCode {
    name: "Upper Upper Sorbian",
    code: "",
    code_2t: "hsb",
    code_2b: "hsb",
    code_3: "hsb",
    individual_languages: &[
    ],
};


pub const HUP: LanguageCode = LanguageCode {
    name: "Hupa",
    code: "",
    code_2t: "hup",
    code_2b: "hup",
    code_3: "hup",
    individual_languages: &[
    ],
};


pub const IBA: LanguageCode = LanguageCode {
    name: "Iban",
    code: "",
    code_2t: "iba",
    code_2b: "iba",
    code_3: "iba",
    individual_languages: &[
    ],
};


pub const IJO: LanguageCode = LanguageCode {
    name: "Ijo languages",
    code: "",
    code_2t: "ijo",
    code_2b: "ijo",
    code_3: "",
    individual_languages: &[
    ],
};


pub const ILO: LanguageCode = LanguageCode {
    name: "Iloko",
    code: "",
    code_2t: "ilo",
    code_2b: "ilo",
    code_3: "ilo",
    individual_languages: &[
    ],
};


pub const INC: LanguageCode = LanguageCode {
    name: "Indo-Aryan languages",
    code: "",
    code_2t: "inc",
    code_2b: "inc",
    code_3: "",
    individual_languages: &[
    ],
};


pub const INE: LanguageCode = LanguageCode {
    name: "Indo-European languages",
    code: "",
    code_2t: "ine",
    code_2b: "ine",
    code_3: "",
    individual_languages: &[
    ],
};


pub const INH: LanguageCode = LanguageCode {
    name: "Ingush",
    code: "",
    code_2t: "inh",
    code_2b: "inh",
    code_3: "inh",
    individual_languages: &[
    ],
};


pub const IRA: LanguageCode = LanguageCode {
    name: "Iranian languages",
    code: "",
    code_2t: "ira",
    code_2b: "ira",
    code_3: "",
    individual_languages: &[
    ],
};


pub const IRO: LanguageCode = LanguageCode {
    name: "Iroquoian languages",
    code: "",
    code_2t: "iro",
    code_2b: "iro",
    code_3: "",
    individual_languages: &[
    ],
};


pub const JBO: LanguageCode = LanguageCode {
    name: "Lojban",
    code: "",
    code_2t: "jbo",
    code_2b: "jbo",
    code_3: "jbo",
    individual_languages: &[
    ],
};


pub const JPR: LanguageCode = LanguageCode {
    name: "Judeo-Judeo-Persian",
    code: "",
    code_2t: "jpr",
    code_2b: "jpr",
    code_3: "jpr",
    individual_languages: &[
    ],
};


pub const JRB: LanguageCode = LanguageCode {
    name: "Judeo-Judeo-Arabic",
    code: "",
    code_2t: "jrb",
    code_2b: "jrb",
    code_3: "jrb",
    individual_languages: &[
        IndividualLanguages {
            name: "",
            code: "ajt",
        },
        IndividualLanguages {
            name: "Judeo-Moroccan Arabic",
            code: "aju",
        },
        IndividualLanguages {
            name: "Judeo-Yemeni Arabic",
            code: "jye",
        },
        IndividualLanguages {
            name: "Judeo-Iraqi Arabic",
            code: "yhd",
        },
        IndividualLanguages {
            name: "Judeo-Tripolitanian Arabic",
            code: "yud",
        },
    ],
};


pub const KAA: LanguageCode = LanguageCode {
    name: "Kara-Kara-Kalpak",
    code: "",
    code_2t: "kaa",
    code_2b: "kaa",
    code_3: "kaa",
    individual_languages: &[
    ],
};


pub const KAB: LanguageCode = LanguageCode {
    name: "Kabyle",
    code: "",
    code_2t: "kab",
    code_2b: "kab",
    code_3: "kab",
    individual_languages: &[
    ],
};


pub const KAC: LanguageCode = LanguageCode {
    name: "Kachin; Jingpho",
    code: "",
    code_2t: "kac",
    code_2b: "kac",
    code_3: "kac",
    individual_languages: &[
    ],
};


pub const KAM: LanguageCode = LanguageCode {
    name: "Kamba",
    code: "",
    code_2t: "kam",
    code_2b: "kam",
    code_3: "kam",
    individual_languages: &[
    ],
};


pub const KAR: LanguageCode = LanguageCode {
    name: "Karen languages",
    code: "",
    code_2t: "kar",
    code_2b: "kar",
    code_3: "",
    individual_languages: &[
    ],
};


pub const KAW: LanguageCode = LanguageCode {
    name: "Kawi",
    code: "",
    code_2t: "kaw",
    code_2b: "kaw",
    code_3: "kaw",
    individual_languages: &[
    ],
};


pub const KBD: LanguageCode = LanguageCode {
    name: "Kabardian",
    code: "",
    code_2t: "kbd",
    code_2b: "kbd",
    code_3: "kbd",
    individual_languages: &[
    ],
};


pub const KHA: LanguageCode = LanguageCode {
    name: "Khasi",
    code: "",
    code_2t: "kha",
    code_2b: "kha",
    code_3: "kha",
    individual_languages: &[
    ],
};


pub const KHI: LanguageCode = LanguageCode {
    name: "Khoisan languages",
    code: "",
    code_2t: "khi",
    code_2b: "khi",
    code_3: "",
    individual_languages: &[
    ],
};


pub const KHO: LanguageCode = LanguageCode {
    name: "Khotanese; Sakan",
    code: "",
    code_2t: "kho",
    code_2b: "kho",
    code_3: "kho",
    individual_languages: &[
    ],
};


pub const KMB: LanguageCode = LanguageCode {
    name: "Kimbundu",
    code: "",
    code_2t: "kmb",
    code_2b: "kmb",
    code_3: "kmb",
    individual_languages: &[
    ],
};


pub const KOK: LanguageCode = LanguageCode {
    name: "Konkani",
    code: "",
    code_2t: "kok",
    code_2b: "kok",
    code_3: "kok",
    individual_languages: &[
        IndividualLanguages {
            name: "Goan Konkani",
            code: "gom",
        },
        IndividualLanguages {
            name: "Konkani (individual language)",
            code: "knn",
        },
    ],
};


pub const KOS: LanguageCode = LanguageCode {
    name: "Kosraean",
    code: "",
    code_2t: "kos",
    code_2b: "kos",
    code_3: "kos",
    individual_languages: &[
    ],
};


pub const KPE: LanguageCode = LanguageCode {
    name: "Kpelle",
    code: "",
    code_2t: "kpe",
    code_2b: "kpe",
    code_3: "kpe",
    individual_languages: &[
        IndividualLanguages {
            name: "Guinea Kpelle",
            code: "gkp",
        },
        IndividualLanguages {
            name: "Liberia Kpelle",
            code: "xpe",
        },
    ],
};


pub const KRC: LanguageCode = LanguageCode {
    name: "Karachay-Karachay-Balkar",
    code: "",
    code_2t: "krc",
    code_2b: "krc",
    code_3: "krc",
    individual_languages: &[
    ],
};


pub const KRL: LanguageCode = LanguageCode {
    name: "Karelian",
    code: "",
    code_2t: "krl",
    code_2b: "krl",
    code_3: "krl",
    individual_languages: &[
    ],
};


pub const KRO: LanguageCode = LanguageCode {
    name: "Kru languages",
    code: "",
    code_2t: "kro",
    code_2b: "kro",
    code_3: "",
    individual_languages: &[
    ],
};


pub const KRU: LanguageCode = LanguageCode {
    name: "Kurukh",
    code: "",
    code_2t: "kru",
    code_2b: "kru",
    code_3: "kru",
    individual_languages: &[
    ],
};


pub const KUM: LanguageCode = LanguageCode {
    name: "Kumyk",
    code: "",
    code_2t: "kum",
    code_2b: "kum",
    code_3: "kum",
    individual_languages: &[
    ],
};


pub const KUT: LanguageCode = LanguageCode {
    name: "Kutenai",
    code: "",
    code_2t: "kut",
    code_2b: "kut",
    code_3: "kut",
    individual_languages: &[
    ],
};


pub const LAD: LanguageCode = LanguageCode {
    name: "Ladino",
    code: "",
    code_2t: "lad",
    code_2b: "lad",
    code_3: "lad",
    individual_languages: &[
    ],
};


pub const LAH: LanguageCode = LanguageCode {
    name: "Lahnda",
    code: "",
    code_2t: "lah",
    code_2b: "lah",
    code_3: "lah",
    individual_languages: &[
        IndividualLanguages {
            name: "Southern Hindko",
            code: "hnd",
        },
        IndividualLanguages {
            name: "Northern Hindko",
            code: "hno",
        },
        IndividualLanguages {
            name: "Jakati",
            code: "jat",
        },
        IndividualLanguages {
            name: "Pahari-Potwari",
            code: "phr",
        },
        IndividualLanguages {
            name: "",
            code: "pmu",
        },
        IndividualLanguages {
            name: "Western Panjabi",
            code: "pnb",
        },
        IndividualLanguages {
            name: "Seraiki",
            code: "skr",
        },
        IndividualLanguages {
            name: "Khetrani",
            code: "xhe",
        },
    ],
};


pub const LAM: LanguageCode = LanguageCode {
    name: "Lamba",
    code: "",
    code_2t: "lam",
    code_2b: "lam",
    code_3: "lam",
    individual_languages: &[
    ],
};


pub const LEZ: LanguageCode = LanguageCode {
    name: "Lezghian",
    code: "",
    code_2t: "lez",
    code_2b: "lez",
    code_3: "lez",
    individual_languages: &[
    ],
};


pub const LOL: LanguageCode = LanguageCode {
    name: "Mongo",
    code: "",
    code_2t: "lol",
    code_2b: "lol",
    code_3: "lol",
    individual_languages: &[
    ],
};


pub const LOZ: LanguageCode = LanguageCode {
    name: "Lozi",
    code: "",
    code_2t: "loz",
    code_2b: "loz",
    code_3: "loz",
    individual_languages: &[
    ],
};


pub const LUA: LanguageCode = LanguageCode {
    name: "Luba-Luba-Lulua",
    code: "",
    code_2t: "lua",
    code_2b: "lua",
    code_3: "lua",
    individual_languages: &[
    ],
};


pub const LUI: LanguageCode = LanguageCode {
    name: "Luiseno",
    code: "",
    code_2t: "lui",
    code_2b: "lui",
    code_3: "lui",
    individual_languages: &[
    ],
};


pub const LUN: LanguageCode = LanguageCode {
    name: "Lunda",
    code: "",
    code_2t: "lun",
    code_2b: "lun",
    code_3: "lun",
    individual_languages: &[
    ],
};


pub const LUO: LanguageCode = LanguageCode {
    name: "Luo language (Kenya and Tanzania)|Luo (Kenya and Tanzania)",
    code: "",
    code_2t: "luo",
    code_2b: "luo",
    code_3: "luo",
    individual_languages: &[
    ],
};


pub const LUS: LanguageCode = LanguageCode {
    name: "Lushai",
    code: "",
    code_2t: "lus",
    code_2b: "lus",
    code_3: "lus",
    individual_languages: &[
    ],
};


pub const MAD: LanguageCode = LanguageCode {
    name: "Madurese",
    code: "",
    code_2t: "mad",
    code_2b: "mad",
    code_3: "mad",
    individual_languages: &[
    ],
};


pub const MAG: LanguageCode = LanguageCode {
    name: "Magahi",
    code: "",
    code_2t: "mag",
    code_2b: "mag",
    code_3: "mag",
    individual_languages: &[
    ],
};


pub const MAI: LanguageCode = LanguageCode {
    name: "Maithili",
    code: "",
    code_2t: "mai",
    code_2b: "mai",
    code_3: "mai",
    individual_languages: &[
    ],
};


pub const MAK: LanguageCode = LanguageCode {
    name: "Makasar",
    code: "",
    code_2t: "mak",
    code_2b: "mak",
    code_3: "mak",
    individual_languages: &[
    ],
};


pub const MAN: LanguageCode = LanguageCode {
    name: "Mandingo",
    code: "",
    code_2t: "man",
    code_2b: "man",
    code_3: "man",
    individual_languages: &[
        IndividualLanguages {
            name: "Eastern Maninkakan",
            code: "emk",
        },
        IndividualLanguages {
            name: "Konyanka Maninka",
            code: "mku",
        },
        IndividualLanguages {
            name: "Western Maninkakan",
            code: "mlq",
        },
        IndividualLanguages {
            name: "Mandinka",
            code: "mnk",
        },
        IndividualLanguages {
            name: "Sankaran Maninka",
            code: "msc",
        },
        IndividualLanguages {
            name: "Kita Maninkakan",
            code: "mwk",
        },
        IndividualLanguages {
            name: "",
            code: "myq",
        },
    ],
};


pub const MAP: LanguageCode = LanguageCode {
    name: "Austronesian languages",
    code: "",
    code_2t: "map",
    code_2b: "map",
    code_3: "",
    individual_languages: &[
    ],
};


pub const MAS: LanguageCode = LanguageCode {
    name: "Masai",
    code: "",
    code_2t: "mas",
    code_2b: "mas",
    code_3: "mas",
    individual_languages: &[
    ],
};


pub const MDF: LanguageCode = LanguageCode {
    name: "Moksha",
    code: "",
    code_2t: "mdf",
    code_2b: "mdf",
    code_3: "mdf",
    individual_languages: &[
    ],
};


pub const MDR: LanguageCode = LanguageCode {
    name: "Mandar",
    code: "",
    code_2t: "mdr",
    code_2b: "mdr",
    code_3: "mdr",
    individual_languages: &[
    ],
};


pub const MEN: LanguageCode = LanguageCode {
    name: "Mende",
    code: "",
    code_2t: "men",
    code_2b: "men",
    code_3: "men",
    individual_languages: &[
    ],
};


pub const MGA: LanguageCode = LanguageCode {
    name: "Middle Irish|Irish, Middle (900–1200)",
    code: "",
    code_2t: "mga",
    code_2b: "mga",
    code_3: "mga",
    individual_languages: &[
    ],
};


pub const MIC: LanguageCode = LanguageCode {
    name: "Mi'Mi'kmaq; Micmac",
    code: "",
    code_2t: "mic",
    code_2b: "mic",
    code_3: "mic",
    individual_languages: &[
    ],
};


pub const MIN: LanguageCode = LanguageCode {
    name: "Minangkabau",
    code: "",
    code_2t: "min",
    code_2b: "min",
    code_3: "min",
    individual_languages: &[
    ],
};


pub const MIS: LanguageCode = LanguageCode {
    name: "Uncoded languages",
    code: "",
    code_2t: "mis",
    code_2b: "mis",
    code_3: "mis",
    individual_languages: &[
    ],
};


pub const MKH: LanguageCode = LanguageCode {
    name: "Mon-Khmer languages",
    code: "",
    code_2t: "mkh",
    code_2b: "mkh",
    code_3: "",
    individual_languages: &[
    ],
};


pub const MNC: LanguageCode = LanguageCode {
    name: "Manchu",
    code: "",
    code_2t: "mnc",
    code_2b: "mnc",
    code_3: "mnc",
    individual_languages: &[
    ],
};


pub const MNI: LanguageCode = LanguageCode {
    name: "Manipuri",
    code: "",
    code_2t: "mni",
    code_2b: "mni",
    code_3: "mni",
    individual_languages: &[
    ],
};


pub const MNO: LanguageCode = LanguageCode {
    name: "Manobo languages",
    code: "",
    code_2t: "mno",
    code_2b: "mno",
    code_3: "",
    individual_languages: &[
    ],
};


pub const MOH: LanguageCode = LanguageCode {
    name: "Mohawk",
    code: "",
    code_2t: "moh",
    code_2b: "moh",
    code_3: "moh",
    individual_languages: &[
    ],
};


pub const MOS: LanguageCode = LanguageCode {
    name: "Mossi",
    code: "",
    code_2t: "mos",
    code_2b: "mos",
    code_3: "mos",
    individual_languages: &[
    ],
};


pub const MUL: LanguageCode = LanguageCode {
    name: "Multiple languages",
    code: "",
    code_2t: "mul",
    code_2b: "mul",
    code_3: "mul",
    individual_languages: &[
    ],
};


pub const MUN: LanguageCode = LanguageCode {
    name: "Munda languages",
    code: "",
    code_2t: "mun",
    code_2b: "mun",
    code_3: "",
    individual_languages: &[
    ],
};


pub const MUS: LanguageCode = LanguageCode {
    name: "Creek",
    code: "",
    code_2t: "mus",
    code_2b: "mus",
    code_3: "mus",
    individual_languages: &[
    ],
};


pub const MWL: LanguageCode = LanguageCode {
    name: "Mirandese",
    code: "",
    code_2t: "mwl",
    code_2b: "mwl",
    code_3: "mwl",
    individual_languages: &[
    ],
};


pub const MWR: LanguageCode = LanguageCode {
    name: "Marwari",
    code: "",
    code_2t: "mwr",
    code_2b: "mwr",
    code_3: "mwr",
    individual_languages: &[
        IndividualLanguages {
            name: "Dhundari",
            code: "dhd",
        },
        IndividualLanguages {
            name: "Mewari",
            code: "mtr",
        },
        IndividualLanguages {
            name: "Marwari (Pakistan)",
            code: "mve",
        },
        IndividualLanguages {
            name: "Marwari (India)",
            code: "rwr",
        },
        IndividualLanguages {
            name: "Shekhawati",
            code: "swv",
        },
        IndividualLanguages {
            name: "Merwari",
            code: "wry",
        },
    ],
};


pub const MYN: LanguageCode = LanguageCode {
    name: "Mayan languages",
    code: "",
    code_2t: "myn",
    code_2b: "myn",
    code_3: "",
    individual_languages: &[
    ],
};


pub const MYV: LanguageCode = LanguageCode {
    name: "Erzya",
    code: "",
    code_2t: "myv",
    code_2b: "myv",
    code_3: "myv",
    individual_languages: &[
    ],
};


pub const NAH: LanguageCode = LanguageCode {
    name: "Nahuatl languages",
    code: "",
    code_2t: "nah",
    code_2b: "nah",
    code_3: "",
    individual_languages: &[
    ],
};


pub const NAI: LanguageCode = LanguageCode {
    name: "North American Indian languages",
    code: "",
    code_2t: "nai",
    code_2b: "nai",
    code_3: "",
    individual_languages: &[
    ],
};


pub const NAP: LanguageCode = LanguageCode {
    name: "Neapolitan",
    code: "",
    code_2t: "nap",
    code_2b: "nap",
    code_3: "nap",
    individual_languages: &[
    ],
};


pub const NDS: LanguageCode = LanguageCode {
    name: "Low German; Low Saxon; German, Low; Saxon, Low",
    code: "",
    code_2t: "nds",
    code_2b: "nds",
    code_3: "nds",
    individual_languages: &[
    ],
};


pub const NEW: LanguageCode = LanguageCode {
    name: "Nepal Nepal Bhasa; Newari",
    code: "",
    code_2t: "new",
    code_2b: "new",
    code_3: "new",
    individual_languages: &[
    ],
};


pub const NIA: LanguageCode = LanguageCode {
    name: "Nias",
    code: "",
    code_2t: "nia",
    code_2b: "nia",
    code_3: "nia",
    individual_languages: &[
    ],
};


pub const NIC: LanguageCode = LanguageCode {
    name: "Niger-Kordofanian languages",
    code: "",
    code_2t: "nic",
    code_2b: "nic",
    code_3: "",
    individual_languages: &[
    ],
};


pub const NIU: LanguageCode = LanguageCode {
    name: "Niuean",
    code: "",
    code_2t: "niu",
    code_2b: "niu",
    code_3: "niu",
    individual_languages: &[
    ],
};


pub const NOG: LanguageCode = LanguageCode {
    name: "Nogai",
    code: "",
    code_2t: "nog",
    code_2b: "nog",
    code_3: "nog",
    individual_languages: &[
    ],
};


pub const NON: LanguageCode = LanguageCode {
    name: "Old Norse, Old",
    code: "",
    code_2t: "non",
    code_2b: "non",
    code_3: "non",
    individual_languages: &[
    ],
};


pub const NQO: LanguageCode = LanguageCode {
    name: "N'N'Ko",
    code: "",
    code_2t: "nqo",
    code_2b: "nqo",
    code_3: "nqo",
    individual_languages: &[
    ],
};


pub const NSO: LanguageCode = LanguageCode {
    name: "Pedi; Sepedi; Northern Northern Sotho",
    code: "",
    code_2t: "nso",
    code_2b: "nso",
    code_3: "nso",
    individual_languages: &[
    ],
};


pub const NUB: LanguageCode = LanguageCode {
    name: "Nubian languages",
    code: "",
    code_2t: "nub",
    code_2b: "nub",
    code_3: "",
    individual_languages: &[
    ],
};


pub const NWC: LanguageCode = LanguageCode {
    name: "Classical Classical&nbsp;Newari; Old Old&nbsp;Newari; Classical Nepal Classical&nbsp;Nepal&nbsp;Bhasa",
    code: "",
    code_2t: "nwc",
    code_2b: "nwc",
    code_3: "nwc",
    individual_languages: &[
    ],
};


pub const NYM: LanguageCode = LanguageCode {
    name: "Nyamwezi",
    code: "",
    code_2t: "nym",
    code_2b: "nym",
    code_3: "nym",
    individual_languages: &[
    ],
};


pub const NYN: LanguageCode = LanguageCode {
    name: "Nyankole",
    code: "",
    code_2t: "nyn",
    code_2b: "nyn",
    code_3: "nyn",
    individual_languages: &[
    ],
};


pub const NYO: LanguageCode = LanguageCode {
    name: "Nyoro",
    code: "",
    code_2t: "nyo",
    code_2b: "nyo",
    code_3: "nyo",
    individual_languages: &[
    ],
};


pub const NZI: LanguageCode = LanguageCode {
    name: "Nzima",
    code: "",
    code_2t: "nzi",
    code_2b: "nzi",
    code_3: "nzi",
    individual_languages: &[
    ],
};


pub const OSA: LanguageCode = LanguageCode {
    name: "Osage",
    code: "",
    code_2t: "osa",
    code_2b: "osa",
    code_3: "osa",
    individual_languages: &[
    ],
};


pub const OTA: LanguageCode = LanguageCode {
    name: "Ottoman Turkish, Ottoman (1500–1928)",
    code: "",
    code_2t: "ota",
    code_2b: "ota",
    code_3: "ota",
    individual_languages: &[
    ],
};


pub const OTO: LanguageCode = LanguageCode {
    name: "Otomian languages",
    code: "",
    code_2t: "oto",
    code_2b: "oto",
    code_3: "",
    individual_languages: &[
    ],
};


pub const PAA: LanguageCode = LanguageCode {
    name: "Papuan languages",
    code: "",
    code_2t: "paa",
    code_2b: "paa",
    code_3: "",
    individual_languages: &[
    ],
};


pub const PAG: LanguageCode = LanguageCode {
    name: "Pangasinan",
    code: "",
    code_2t: "pag",
    code_2b: "pag",
    code_3: "pag",
    individual_languages: &[
    ],
};


pub const PAL: LanguageCode = LanguageCode {
    name: "Pahlavi",
    code: "",
    code_2t: "pal",
    code_2b: "pal",
    code_3: "pal",
    individual_languages: &[
    ],
};


pub const PAM: LanguageCode = LanguageCode {
    name: "Pampanga; Kapampangan",
    code: "",
    code_2t: "pam",
    code_2b: "pam",
    code_3: "pam",
    individual_languages: &[
    ],
};


pub const PAP: LanguageCode = LanguageCode {
    name: "Papiamento",
    code: "",
    code_2t: "pap",
    code_2b: "pap",
    code_3: "pap",
    individual_languages: &[
    ],
};


pub const PAU: LanguageCode = LanguageCode {
    name: "Palauan",
    code: "",
    code_2t: "pau",
    code_2b: "pau",
    code_3: "pau",
    individual_languages: &[
    ],
};


pub const PEO: LanguageCode = LanguageCode {
    name: "Old Persian, Old (ca. 600–400 B.C.)",
    code: "",
    code_2t: "peo",
    code_2b: "peo",
    code_3: "peo",
    individual_languages: &[
    ],
};


pub const PHI: LanguageCode = LanguageCode {
    name: "Philippine languages",
    code: "",
    code_2t: "phi",
    code_2b: "phi",
    code_3: "",
    individual_languages: &[
    ],
};


pub const PHN: LanguageCode = LanguageCode {
    name: "Phoenician",
    code: "",
    code_2t: "phn",
    code_2b: "phn",
    code_3: "phn",
    individual_languages: &[
    ],
};


pub const PON: LanguageCode = LanguageCode {
    name: "Pohnpeian",
    code: "",
    code_2t: "pon",
    code_2b: "pon",
    code_3: "pon",
    individual_languages: &[
    ],
};


pub const PRA: LanguageCode = LanguageCode {
    name: "Prakrit languages",
    code: "",
    code_2t: "pra",
    code_2b: "pra",
    code_3: "",
    individual_languages: &[
    ],
};


pub const PRO: LanguageCode = LanguageCode {
    name: "Old Provençal, Old (to 1500); Old Old&nbsp;Occitan&nbsp;(to&nbsp;1500)",
    code: "",
    code_2t: "pro",
    code_2b: "pro",
    code_3: "pro",
    individual_languages: &[
    ],
};


pub const QAA: LanguageCode = LanguageCode {
    name: "Reserved for local use",
    code: "",
    code_2t: "qaa-qtz",
    code_2b: "qaa-qtz",
    code_3: "qaa-qtz",
    individual_languages: &[
    ],
};


pub const RAJ: LanguageCode = LanguageCode {
    name: "Rajasthani",
    code: "",
    code_2t: "raj",
    code_2b: "raj",
    code_3: "raj",
    individual_languages: &[
        IndividualLanguages {
            name: "Bagri",
            code: "bgq",
        },
        IndividualLanguages {
            name: "Gade Lohar",
            code: "gda",
        },
        IndividualLanguages {
            name: "Gujari",
            code: "gju",
        },
        IndividualLanguages {
            name: "Haroti",
            code: "hoj",
        },
        IndividualLanguages {
            name: "Malvi",
            code: "mup",
        },
        IndividualLanguages {
            name: "Wagdi",
            code: "wbr",
        },
    ],
};


pub const RAP: LanguageCode = LanguageCode {
    name: "Rapanui",
    code: "",
    code_2t: "rap",
    code_2b: "rap",
    code_3: "rap",
    individual_languages: &[
    ],
};


pub const RAR: LanguageCode = LanguageCode {
    name: "Rarotongan; Cook Islands Cook Islands Māori",
    code: "",
    code_2t: "rar",
    code_2b: "rar",
    code_3: "rar",
    individual_languages: &[
    ],
};


pub const ROA: LanguageCode = LanguageCode {
    name: "Romance languages",
    code: "",
    code_2t: "roa",
    code_2b: "roa",
    code_3: "",
    individual_languages: &[
    ],
};


pub const ROM: LanguageCode = LanguageCode {
    name: "Romany",
    code: "",
    code_2t: "rom",
    code_2b: "rom",
    code_3: "rom",
    individual_languages: &[
        IndividualLanguages {
            name: "Carpathian Romani",
            code: "rmc",
        },
        IndividualLanguages {
            name: "Kalo Finnish Romani",
            code: "rmf",
        },
        IndividualLanguages {
            name: "Baltic Romani",
            code: "rml",
        },
        IndividualLanguages {
            name: "Balkan Romani",
            code: "rmn",
        },
        IndividualLanguages {
            name: "Sinte Romani",
            code: "rmo",
        },
        IndividualLanguages {
            name: "Welsh Romani",
            code: "rmw",
        },
        IndividualLanguages {
            name: "Vlax Romani",
            code: "rmy",
        },
    ],
};


pub const RUP: LanguageCode = LanguageCode {
    name: "Aromanian; Arumanian; Macedo-Romanian{{efn|The term \"Macedo-Romanian\" might also be used for the Megleno-Romanians and Megleno-their language.<ref>{{cite journal|url=https://www.ceeol.com/search/article-detail?id=87227|title=Some topics of the traditional wedding customs of the Macedo–Romanians (Aromanians and Megleno–Romanians)|first=Emil|last=Țîrcomnicu|journal=Romanian Journal of Population Studies|issue=3|pages=141–152|year=2009|volume=3 }}</ref>}}",
    code: "",
    code_2t: "rup",
    code_2b: "rup",
    code_3: "rup",
    individual_languages: &[
    ],
};


pub const SAD: LanguageCode = LanguageCode {
    name: "Sandawe",
    code: "",
    code_2t: "sad",
    code_2b: "sad",
    code_3: "sad",
    individual_languages: &[
    ],
};


pub const SAH: LanguageCode = LanguageCode {
    name: "Yakut",
    code: "",
    code_2t: "sah",
    code_2b: "sah",
    code_3: "sah",
    individual_languages: &[
    ],
};


pub const SAI: LanguageCode = LanguageCode {
    name: "South American Indian languages",
    code: "",
    code_2t: "sai",
    code_2b: "sai",
    code_3: "",
    individual_languages: &[
    ],
};


pub const SAL: LanguageCode = LanguageCode {
    name: "Salishan languages",
    code: "",
    code_2t: "sal",
    code_2b: "sal",
    code_3: "",
    individual_languages: &[
    ],
};


pub const SAM: LanguageCode = LanguageCode {
    name: "Samaritan Samaritan Aramaic",
    code: "",
    code_2t: "sam",
    code_2b: "sam",
    code_3: "sam",
    individual_languages: &[
    ],
};


pub const SAS: LanguageCode = LanguageCode {
    name: "Sasak",
    code: "",
    code_2t: "sas",
    code_2b: "sas",
    code_3: "sas",
    individual_languages: &[
    ],
};


pub const SAT: LanguageCode = LanguageCode {
    name: "Santali",
    code: "",
    code_2t: "sat",
    code_2b: "sat",
    code_3: "sat",
    individual_languages: &[
    ],
};


pub const SCN: LanguageCode = LanguageCode {
    name: "Sicilian",
    code: "",
    code_2t: "scn",
    code_2b: "scn",
    code_3: "scn",
    individual_languages: &[
    ],
};


pub const SCO: LanguageCode = LanguageCode {
    name: "Scots",
    code: "",
    code_2t: "sco",
    code_2b: "sco",
    code_3: "sco",
    individual_languages: &[
    ],
};


pub const SEL: LanguageCode = LanguageCode {
    name: "Selkup",
    code: "",
    code_2t: "sel",
    code_2b: "sel",
    code_3: "sel",
    individual_languages: &[
    ],
};


pub const SEM: LanguageCode = LanguageCode {
    name: "Semitic languages",
    code: "",
    code_2t: "sem",
    code_2b: "sem",
    code_3: "",
    individual_languages: &[
    ],
};


pub const SGA: LanguageCode = LanguageCode {
    name: "Old Irish, Old (to 900)",
    code: "",
    code_2t: "sga",
    code_2b: "sga",
    code_3: "sga",
    individual_languages: &[
    ],
};


pub const SGN: LanguageCode = LanguageCode {
    name: "Sign Languages",
    code: "",
    code_2t: "sgn",
    code_2b: "sgn",
    code_3: "",
    individual_languages: &[
    ],
};


pub const SHN: LanguageCode = LanguageCode {
    name: "Shan",
    code: "",
    code_2t: "shn",
    code_2b: "shn",
    code_3: "shn",
    individual_languages: &[
    ],
};


pub const SID: LanguageCode = LanguageCode {
    name: "Sidamo",
    code: "",
    code_2t: "sid",
    code_2b: "sid",
    code_3: "sid",
    individual_languages: &[
    ],
};


pub const SIO: LanguageCode = LanguageCode {
    name: "Siouan languages",
    code: "",
    code_2t: "sio",
    code_2b: "sio",
    code_3: "",
    individual_languages: &[
    ],
};


pub const SIT: LanguageCode = LanguageCode {
    name: "Sino-Tibetan languages",
    code: "",
    code_2t: "sit",
    code_2b: "sit",
    code_3: "",
    individual_languages: &[
    ],
};


pub const SLA: LanguageCode = LanguageCode {
    name: "Slavic languages",
    code: "",
    code_2t: "sla",
    code_2b: "sla",
    code_3: "",
    individual_languages: &[
    ],
};


pub const SMA: LanguageCode = LanguageCode {
    name: "Southern Southern Sami",
    code: "",
    code_2t: "sma",
    code_2b: "sma",
    code_3: "sma",
    individual_languages: &[
    ],
};


pub const SMI: LanguageCode = LanguageCode {
    name: "Sami languages",
    code: "",
    code_2t: "smi",
    code_2b: "smi",
    code_3: "",
    individual_languages: &[
    ],
};


pub const SMJ: LanguageCode = LanguageCode {
    name: "Lule Lule Sami",
    code: "",
    code_2t: "smj",
    code_2b: "smj",
    code_3: "smj",
    individual_languages: &[
    ],
};


pub const SMN: LanguageCode = LanguageCode {
    name: "Inari Inari Sami",
    code: "",
    code_2t: "smn",
    code_2b: "smn",
    code_3: "smn",
    individual_languages: &[
    ],
};


pub const SMS: LanguageCode = LanguageCode {
    name: "Skolt Skolt Sami",
    code: "",
    code_2t: "sms",
    code_2b: "sms",
    code_3: "sms",
    individual_languages: &[
    ],
};


pub const SNK: LanguageCode = LanguageCode {
    name: "Soninke",
    code: "",
    code_2t: "snk",
    code_2b: "snk",
    code_3: "snk",
    individual_languages: &[
    ],
};


pub const SOG: LanguageCode = LanguageCode {
    name: "Sogdian",
    code: "",
    code_2t: "sog",
    code_2b: "sog",
    code_3: "sog",
    individual_languages: &[
    ],
};


pub const SON: LanguageCode = LanguageCode {
    name: "Songhai languages",
    code: "",
    code_2t: "son",
    code_2b: "son",
    code_3: "",
    individual_languages: &[
    ],
};


pub const SRN: LanguageCode = LanguageCode {
    name: "Sranan Sranan Tongo",
    code: "",
    code_2t: "srn",
    code_2b: "srn",
    code_3: "srn",
    individual_languages: &[
    ],
};


pub const SRR: LanguageCode = LanguageCode {
    name: "Serer",
    code: "",
    code_2t: "srr",
    code_2b: "srr",
    code_3: "srr",
    individual_languages: &[
    ],
};


pub const SSA: LanguageCode = LanguageCode {
    name: "Nilo-Saharan languages",
    code: "",
    code_2t: "ssa",
    code_2b: "ssa",
    code_3: "",
    individual_languages: &[
    ],
};


pub const SUK: LanguageCode = LanguageCode {
    name: "Sukuma",
    code: "",
    code_2t: "suk",
    code_2b: "suk",
    code_3: "suk",
    individual_languages: &[
    ],
};


pub const SUS: LanguageCode = LanguageCode {
    name: "Susu",
    code: "",
    code_2t: "sus",
    code_2b: "sus",
    code_3: "sus",
    individual_languages: &[
    ],
};


pub const SUX: LanguageCode = LanguageCode {
    name: "Sumerian",
    code: "",
    code_2t: "sux",
    code_2b: "sux",
    code_3: "sux",
    individual_languages: &[
    ],
};


pub const SYC: LanguageCode = LanguageCode {
    name: "Classical Classical Syriac",
    code: "",
    code_2t: "syc",
    code_2b: "syc",
    code_3: "syc",
    individual_languages: &[
    ],
};


pub const SYR: LanguageCode = LanguageCode {
    name: "Syriac",
    code: "",
    code_2t: "syr",
    code_2b: "syr",
    code_3: "syr",
    individual_languages: &[
        IndividualLanguages {
            name: "Assyrian Neo-Aramaic",
            code: "aii",
        },
        IndividualLanguages {
            name: "Chaldean Neo-Aramaic",
            code: "cld",
        },
    ],
};


pub const TAI: LanguageCode = LanguageCode {
    name: "Tai languages",
    code: "",
    code_2t: "tai",
    code_2b: "tai",
    code_3: "",
    individual_languages: &[
    ],
};


pub const TEM: LanguageCode = LanguageCode {
    name: "Timne",
    code: "",
    code_2t: "tem",
    code_2b: "tem",
    code_3: "tem",
    individual_languages: &[
    ],
};


pub const TER: LanguageCode = LanguageCode {
    name: "Tereno",
    code: "",
    code_2t: "ter",
    code_2b: "ter",
    code_3: "ter",
    individual_languages: &[
    ],
};


pub const TET: LanguageCode = LanguageCode {
    name: "Tetum",
    code: "",
    code_2t: "tet",
    code_2b: "tet",
    code_3: "tet",
    individual_languages: &[
    ],
};


pub const TIG: LanguageCode = LanguageCode {
    name: "Tigre",
    code: "",
    code_2t: "tig",
    code_2b: "tig",
    code_3: "tig",
    individual_languages: &[
    ],
};


pub const TIV: LanguageCode = LanguageCode {
    name: "Tiv",
    code: "",
    code_2t: "tiv",
    code_2b: "tiv",
    code_3: "tiv",
    individual_languages: &[
    ],
};


pub const TKL: LanguageCode = LanguageCode {
    name: "Tokelau",
    code: "",
    code_2t: "tkl",
    code_2b: "tkl",
    code_3: "tkl",
    individual_languages: &[
    ],
};


pub const TLH: LanguageCode = LanguageCode {
    name: "Klingon; tlhIngan-tlhIngan-Hol<!--starts with lowercase-->",
    code: "",
    code_2t: "tlh",
    code_2b: "tlh",
    code_3: "tlh",
    individual_languages: &[
    ],
};


pub const TLI: LanguageCode = LanguageCode {
    name: "Tlingit",
    code: "",
    code_2t: "tli",
    code_2b: "tli",
    code_3: "tli",
    individual_languages: &[
    ],
};


pub const TMH: LanguageCode = LanguageCode {
    name: "Tamashek",
    code: "",
    code_2t: "tmh",
    code_2b: "tmh",
    code_3: "tmh",
    individual_languages: &[
        IndividualLanguages {
            name: "Tamasheq",
            code: "taq",
        },
        IndividualLanguages {
            name: "Tahaggart Tamahaq",
            code: "thv",
        },
        IndividualLanguages {
            name: "Tayart Tamajeq",
            code: "thz",
        },
        IndividualLanguages {
            name: "Tawallammat Tamajaq",
            code: "ttq",
        },
    ],
};


pub const TOG: LanguageCode = LanguageCode {
    name: "Tonga language (Nyasa)|Tonga (Nyasa)",
    code: "",
    code_2t: "tog",
    code_2b: "tog",
    code_3: "tog",
    individual_languages: &[
    ],
};


pub const TPI: LanguageCode = LanguageCode {
    name: "Tok Tok Pisin",
    code: "",
    code_2t: "tpi",
    code_2b: "tpi",
    code_3: "tpi",
    individual_languages: &[
    ],
};


pub const TSI: LanguageCode = LanguageCode {
    name: "Tsimshian",
    code: "",
    code_2t: "tsi",
    code_2b: "tsi",
    code_3: "tsi",
    individual_languages: &[
    ],
};


pub const TUM: LanguageCode = LanguageCode {
    name: "Tumbuka",
    code: "",
    code_2t: "tum",
    code_2b: "tum",
    code_3: "tum",
    individual_languages: &[
    ],
};


pub const TUP: LanguageCode = LanguageCode {
    name: "Tupi languages",
    code: "",
    code_2t: "tup",
    code_2b: "tup",
    code_3: "",
    individual_languages: &[
    ],
};


pub const TUT: LanguageCode = LanguageCode {
    name: "Altaic languages",
    code: "",
    code_2t: "tut",
    code_2b: "tut",
    code_3: "",
    individual_languages: &[
    ],
};


pub const TVL: LanguageCode = LanguageCode {
    name: "Tuvalu",
    code: "",
    code_2t: "tvl",
    code_2b: "tvl",
    code_3: "tvl",
    individual_languages: &[
    ],
};


pub const TYV: LanguageCode = LanguageCode {
    name: "Tuvinian",
    code: "",
    code_2t: "tyv",
    code_2b: "tyv",
    code_3: "tyv",
    individual_languages: &[
    ],
};


pub const UDM: LanguageCode = LanguageCode {
    name: "Udmurt",
    code: "",
    code_2t: "udm",
    code_2b: "udm",
    code_3: "udm",
    individual_languages: &[
    ],
};


pub const UGA: LanguageCode = LanguageCode {
    name: "Ugaritic",
    code: "",
    code_2t: "uga",
    code_2b: "uga",
    code_3: "uga",
    individual_languages: &[
    ],
};


pub const UMB: LanguageCode = LanguageCode {
    name: "Umbundu",
    code: "",
    code_2t: "umb",
    code_2b: "umb",
    code_3: "umb",
    individual_languages: &[
    ],
};


pub const UND: LanguageCode = LanguageCode {
    name: "Undetermined",
    code: "",
    code_2t: "und",
    code_2b: "und",
    code_3: "und",
    individual_languages: &[
    ],
};


pub const VAI: LanguageCode = LanguageCode {
    name: "Vai",
    code: "",
    code_2t: "vai",
    code_2b: "vai",
    code_3: "vai",
    individual_languages: &[
    ],
};


pub const VOT: LanguageCode = LanguageCode {
    name: "Votic",
    code: "",
    code_2t: "vot",
    code_2b: "vot",
    code_3: "vot",
    individual_languages: &[
    ],
};


pub const WAK: LanguageCode = LanguageCode {
    name: "Wakashan languages",
    code: "",
    code_2t: "wak",
    code_2b: "wak",
    code_3: "",
    individual_languages: &[
    ],
};


pub const WAL: LanguageCode = LanguageCode {
    name: "Wolaitta; Wolaytta",
    code: "",
    code_2t: "wal",
    code_2b: "wal",
    code_3: "wal",
    individual_languages: &[
    ],
};


pub const WAR: LanguageCode = LanguageCode {
    name: "Waray",
    code: "",
    code_2t: "war",
    code_2b: "war",
    code_3: "war",
    individual_languages: &[
    ],
};


pub const WAS: LanguageCode = LanguageCode {
    name: "Washo",
    code: "",
    code_2t: "was",
    code_2b: "was",
    code_3: "was",
    individual_languages: &[
    ],
};


pub const WEN: LanguageCode = LanguageCode {
    name: "Sorbian languages",
    code: "",
    code_2t: "wen",
    code_2b: "wen",
    code_3: "",
    individual_languages: &[
    ],
};


pub const XAL: LanguageCode = LanguageCode {
    name: "Kalmyk; Oirat",
    code: "",
    code_2t: "xal",
    code_2b: "xal",
    code_3: "xal",
    individual_languages: &[
    ],
};


pub const YAO: LanguageCode = LanguageCode {
    name: "Yao",
    code: "",
    code_2t: "yao",
    code_2b: "yao",
    code_3: "yao",
    individual_languages: &[
    ],
};


pub const YAP: LanguageCode = LanguageCode {
    name: "Yapese",
    code: "",
    code_2t: "yap",
    code_2b: "yap",
    code_3: "yap",
    individual_languages: &[
    ],
};


pub const YPK: LanguageCode = LanguageCode {
    name: "Yupik languages",
    code: "",
    code_2t: "ypk",
    code_2b: "ypk",
    code_3: "",
    individual_languages: &[
    ],
};


pub const ZAP: LanguageCode = LanguageCode {
    name: "Zapotec",
    code: "",
    code_2t: "zap",
    code_2b: "zap",
    code_3: "zap",
    individual_languages: &[
        IndividualLanguages {
            name: "Sierra de Juárez Zapotec",
            code: "zaa",
        },
        IndividualLanguages {
            name: "Western Tlacolula Valley Zapotec",
            code: "zab",
        },
        IndividualLanguages {
            name: "Ocotlán Zapotec",
            code: "zac",
        },
        IndividualLanguages {
            name: "Cajonos Zapotec",
            code: "zad",
        },
        IndividualLanguages {
            name: "Yareni Zapotec",
            code: "zae",
        },
        IndividualLanguages {
            name: "Ayoquesco Zapotec",
            code: "zaf",
        },
        IndividualLanguages {
            name: "Isthmus Zapotec",
            code: "zai",
        },
        IndividualLanguages {
            name: "Miahuatlán Zapotec",
            code: "zam",
        },
        IndividualLanguages {
            name: "Ozolotepec Zapotec",
            code: "zao",
        },
        IndividualLanguages {
            name: "Aloápam Zapotec",
            code: "zaq",
        },
        IndividualLanguages {
            name: "Rincón Zapotec",
            code: "zar",
        },
        IndividualLanguages {
            name: "Santo Domingo Albarradas Zapotec",
            code: "zas",
        },
        IndividualLanguages {
            name: "Tabaa Zapotec",
            code: "zat",
        },
        IndividualLanguages {
            name: "Yatzachi Zapotec",
            code: "zav",
        },
        IndividualLanguages {
            name: "Mitla Zapotec",
            code: "zaw",
        },
        IndividualLanguages {
            name: "Xadani Zapotec",
            code: "zax",
        },
        IndividualLanguages {
            name: "Coatecas Altas Zapotec",
            code: "zca",
        },
        IndividualLanguages {
            name: "Las Delicias Zapotec",
            code: "zcd",
        },
        IndividualLanguages {
            name: "Asunción Mixtepec Zapotec",
            code: "zoo",
        },
        IndividualLanguages {
            name: "Lachiguiri Zapotec",
            code: "zpa",
        },
        IndividualLanguages {
            name: "Yautepec Zapotec",
            code: "zpb",
        },
        IndividualLanguages {
            name: "Choapan Zapotec",
            code: "zpc",
        },
        IndividualLanguages {
            name: "Southeastern Ixtlán Zapotec",
            code: "zpd",
        },
        IndividualLanguages {
            name: "Petapa Zapotec",
            code: "zpe",
        },
        IndividualLanguages {
            name: "San Pedro Quiatoni Zapotec",
            code: "zpf",
        },
        IndividualLanguages {
            name: "Guevea De Humboldt Zapotec",
            code: "zpg",
        },
        IndividualLanguages {
            name: "Totomachapan Zapotec",
            code: "zph",
        },
        IndividualLanguages {
            name: "Santa María Quiegolani Zapotec",
            code: "zpi",
        },
        IndividualLanguages {
            name: "Quiavicuzas Zapotec",
            code: "zpj",
        },
        IndividualLanguages {
            name: "Tlacolulita Zapotec",
            code: "zpk",
        },
        IndividualLanguages {
            name: "Lachixío Zapotec",
            code: "zpl",
        },
        IndividualLanguages {
            name: "Mixtepec Zapotec",
            code: "zpm",
        },
        IndividualLanguages {
            name: "Santa Inés Yatzechi Zapotec",
            code: "zpn",
        },
        IndividualLanguages {
            name: "Amatlán Zapotec",
            code: "zpo",
        },
        IndividualLanguages {
            name: "El Alto Zapotec",
            code: "zpp",
        },
        IndividualLanguages {
            name: "Zoogocho Zapotec",
            code: "zpq",
        },
        IndividualLanguages {
            name: "Santiago Xanica Zapotec",
            code: "zpr",
        },
        IndividualLanguages {
            name: "Coatlán Zapotec",
            code: "zps",
        },
        IndividualLanguages {
            name: "San Vicente Coatlán Zapotec",
            code: "zpt",
        },
        IndividualLanguages {
            name: "Yalálag Zapotec",
            code: "zpu",
        },
        IndividualLanguages {
            name: "Chichicapan Zapotec",
            code: "zpv",
        },
        IndividualLanguages {
            name: "Zaniza Zapotec",
            code: "zpw",
        },
        IndividualLanguages {
            name: "San Baltazar Loxicha Zapotec",
            code: "zpx",
        },
        IndividualLanguages {
            name: "Mazaltepec Zapotec",
            code: "zpy",
        },
        IndividualLanguages {
            name: "Texmelucan Zapotec",
            code: "zpz",
        },
        IndividualLanguages {
            name: "Southern Rincon Zapotec",
            code: "zsr",
        },
        IndividualLanguages {
            name: "",
            code: "ztc",
        },
        IndividualLanguages {
            name: "Elotepec Zapotec",
            code: "zte",
        },
        IndividualLanguages {
            name: "Xanaguía Zapotec",
            code: "ztg",
        },
        IndividualLanguages {
            name: "Lapaguía-Guivini Zapotec",
            code: "ztl",
        },
        IndividualLanguages {
            name: "San Agustín Mixtepec Zapotec",
            code: "ztm",
        },
        IndividualLanguages {
            name: "Santa Catarina Albarradas Zapotec",
            code: "ztn",
        },
        IndividualLanguages {
            name: "Loxicha Zapotec",
            code: "ztp",
        },
        IndividualLanguages {
            name: "Quioquitani-Quierí Zapotec",
            code: "ztq",
        },
        IndividualLanguages {
            name: "Tilquiapan Zapotec",
            code: "zts",
        },
        IndividualLanguages {
            name: "Tejalapan Zapotec",
            code: "ztt",
        },
        IndividualLanguages {
            name: "Güilá Zapotec",
            code: "ztu",
        },
        IndividualLanguages {
            name: "Zaachila Zapotec",
            code: "ztx",
        },
        IndividualLanguages {
            name: "Yatee Zapotec",
            code: "zty",
        },
    ],
};


pub const ZBL: LanguageCode = LanguageCode {
    name: "Blissymbols; Blissymbolics; Bliss",
    code: "",
    code_2t: "zbl",
    code_2b: "zbl",
    code_3: "zbl",
    individual_languages: &[
    ],
};


pub const ZEN: LanguageCode = LanguageCode {
    name: "Zenaga",
    code: "",
    code_2t: "zen",
    code_2b: "zen",
    code_3: "zen",
    individual_languages: &[
    ],
};


pub const ZGH: LanguageCode = LanguageCode {
    name: "Standard Moroccan Standard Moroccan Tamazight",
    code: "",
    code_2t: "zgh",
    code_2b: "zgh",
    code_3: "zgh",
    individual_languages: &[
    ],
};


pub const ZND: LanguageCode = LanguageCode {
    name: "Zande languages",
    code: "",
    code_2t: "znd",
    code_2b: "znd",
    code_3: "",
    individual_languages: &[
    ],
};


pub const ZUN: LanguageCode = LanguageCode {
    name: "Zuni",
    code: "",
    code_2t: "zun",
    code_2b: "zun",
    code_3: "zun",
    individual_languages: &[
    ],
};


pub const ZXX: LanguageCode = LanguageCode {
    name: "No linguistic content; Not applicable",
    code: "",
    code_2t: "zxx",
    code_2b: "zxx",
    code_3: "zxx",
    individual_languages: &[
    ],
};


pub const ZZA: LanguageCode = LanguageCode {
    name: "Zaza; Dimili; Dimli; Kirdki; Kirmanjki; Zazaki",
    code: "",
    code_2t: "zza",
    code_2b: "zza",
    code_3: "zza",
    individual_languages: &[
        IndividualLanguages {
            name: "Dimli (individual language)",
            code: "diq",
        },
        IndividualLanguages {
            name: "Kirmanjki (individual language)",
            code: "kiu",
        },
    ],
};


pub const AAA: LanguageCode = LanguageCode {
    name: "Ghotuo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aaa",
    individual_languages: &[
    ],
};


pub const AAB: LanguageCode = LanguageCode {
    name: "Alumu-Tesu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aab",
    individual_languages: &[
    ],
};


pub const AAC: LanguageCode = LanguageCode {
    name: "Ari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aac",
    individual_languages: &[
    ],
};


pub const AAD: LanguageCode = LanguageCode {
    name: "Amal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aad",
    individual_languages: &[
    ],
};


pub const AAE: LanguageCode = LanguageCode {
    name: "Arbëreshë Albanian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aae",
    individual_languages: &[
    ],
};


pub const AAF: LanguageCode = LanguageCode {
    name: "Aranadan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aaf",
    individual_languages: &[
    ],
};


pub const AAG: LanguageCode = LanguageCode {
    name: "Ambrak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aag",
    individual_languages: &[
    ],
};


pub const AAH: LanguageCode = LanguageCode {
    name: "Abu' Arapesh",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aah",
    individual_languages: &[
    ],
};


pub const AAI: LanguageCode = LanguageCode {
    name: "Arifama-Miniafia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aai",
    individual_languages: &[
    ],
};


pub const AAK: LanguageCode = LanguageCode {
    name: "Ankave",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aak",
    individual_languages: &[
    ],
};


pub const AAL: LanguageCode = LanguageCode {
    name: "Afade",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aal",
    individual_languages: &[
    ],
};


pub const AAN: LanguageCode = LanguageCode {
    name: "Anambé",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aan",
    individual_languages: &[
    ],
};


pub const AAO: LanguageCode = LanguageCode {
    name: "Algerian Saharan Arabic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aao",
    individual_languages: &[
    ],
};


pub const AAP: LanguageCode = LanguageCode {
    name: "Pará Arára",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aap",
    individual_languages: &[
    ],
};


pub const AAQ: LanguageCode = LanguageCode {
    name: "Eastern Abnaki",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aaq",
    individual_languages: &[
    ],
};


pub const AAS: LanguageCode = LanguageCode {
    name: "Aasáx",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aas",
    individual_languages: &[
    ],
};


pub const AAT: LanguageCode = LanguageCode {
    name: "Arvanitika Albanian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aat",
    individual_languages: &[
    ],
};


pub const AAU: LanguageCode = LanguageCode {
    name: "Abau",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aau",
    individual_languages: &[
    ],
};


pub const AAW: LanguageCode = LanguageCode {
    name: "Solong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aaw",
    individual_languages: &[
    ],
};


pub const AAX: LanguageCode = LanguageCode {
    name: "Mandobo Atas",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aax",
    individual_languages: &[
    ],
};


pub const AAZ: LanguageCode = LanguageCode {
    name: "Amarasi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aaz",
    individual_languages: &[
    ],
};


pub const ABA: LanguageCode = LanguageCode {
    name: "Abé",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aba",
    individual_languages: &[
    ],
};


pub const ABB: LanguageCode = LanguageCode {
    name: "Bankon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "abb",
    individual_languages: &[
    ],
};


pub const ABC: LanguageCode = LanguageCode {
    name: "Ambala Ayta",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "abc",
    individual_languages: &[
    ],
};


pub const ABD: LanguageCode = LanguageCode {
    name: "Manide",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "abd",
    individual_languages: &[
    ],
};


pub const ABE: LanguageCode = LanguageCode {
    name: "Western Abnaki",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "abe",
    individual_languages: &[
    ],
};


pub const ABF: LanguageCode = LanguageCode {
    name: "Abai Sungai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "abf",
    individual_languages: &[
    ],
};


pub const ABG: LanguageCode = LanguageCode {
    name: "Abaga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "abg",
    individual_languages: &[
    ],
};


pub const ABH: LanguageCode = LanguageCode {
    name: "Tajiki Arabic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "abh",
    individual_languages: &[
    ],
};


pub const ABI: LanguageCode = LanguageCode {
    name: "Abidji",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "abi",
    individual_languages: &[
    ],
};


pub const ABJ: LanguageCode = LanguageCode {
    name: "Aka-Bea",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "abj",
    individual_languages: &[
    ],
};


pub const ABL: LanguageCode = LanguageCode {
    name: "Lampung Nyo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "abl",
    individual_languages: &[
    ],
};


pub const ABM: LanguageCode = LanguageCode {
    name: "Abanyom",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "abm",
    individual_languages: &[
    ],
};


pub const ABN: LanguageCode = LanguageCode {
    name: "Abua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "abn",
    individual_languages: &[
    ],
};


pub const ABO: LanguageCode = LanguageCode {
    name: "Abon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "abo",
    individual_languages: &[
    ],
};


pub const ABP: LanguageCode = LanguageCode {
    name: "Abellen Ayta",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "abp",
    individual_languages: &[
    ],
};


pub const ABQ: LanguageCode = LanguageCode {
    name: "Abaza",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "abq",
    individual_languages: &[
    ],
};


pub const ABR: LanguageCode = LanguageCode {
    name: "Abron",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "abr",
    individual_languages: &[
    ],
};


pub const ABS: LanguageCode = LanguageCode {
    name: "Ambonese Malay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "abs",
    individual_languages: &[
    ],
};


pub const ABT: LanguageCode = LanguageCode {
    name: "Ambulas",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "abt",
    individual_languages: &[
    ],
};


pub const ABU: LanguageCode = LanguageCode {
    name: "Abure",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "abu",
    individual_languages: &[
    ],
};


pub const ABV: LanguageCode = LanguageCode {
    name: "Baharna Arabic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "abv",
    individual_languages: &[
    ],
};


pub const ABW: LanguageCode = LanguageCode {
    name: "Pal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "abw",
    individual_languages: &[
    ],
};


pub const ABX: LanguageCode = LanguageCode {
    name: "Inabaknon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "abx",
    individual_languages: &[
    ],
};


pub const ABY: LanguageCode = LanguageCode {
    name: "Aneme Wake",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aby",
    individual_languages: &[
    ],
};


pub const ABZ: LanguageCode = LanguageCode {
    name: "Abui",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "abz",
    individual_languages: &[
    ],
};


pub const ACA: LanguageCode = LanguageCode {
    name: "Achagua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aca",
    individual_languages: &[
    ],
};


pub const ACB: LanguageCode = LanguageCode {
    name: "Áncá",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "acb",
    individual_languages: &[
    ],
};


pub const ACD: LanguageCode = LanguageCode {
    name: "Gikyode",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "acd",
    individual_languages: &[
    ],
};


pub const ACF: LanguageCode = LanguageCode {
    name: "Saint Lucian Creole French",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "acf",
    individual_languages: &[
    ],
};


pub const ACI: LanguageCode = LanguageCode {
    name: "Aka-Cari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aci",
    individual_languages: &[
    ],
};


pub const ACK: LanguageCode = LanguageCode {
    name: "Aka-Kora",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ack",
    individual_languages: &[
    ],
};


pub const ACL: LanguageCode = LanguageCode {
    name: "Akar-Bale",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "acl",
    individual_languages: &[
    ],
};


pub const ACM: LanguageCode = LanguageCode {
    name: "Mesopotamian Arabic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "acm",
    individual_languages: &[
    ],
};


pub const ACN: LanguageCode = LanguageCode {
    name: "Achang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "acn",
    individual_languages: &[
    ],
};


pub const ACP: LanguageCode = LanguageCode {
    name: "Eastern Acipa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "acp",
    individual_languages: &[
    ],
};


pub const ACQ: LanguageCode = LanguageCode {
    name: "Ta'izzi-Adeni Arabic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "acq",
    individual_languages: &[
    ],
};


pub const ACR: LanguageCode = LanguageCode {
    name: "Achi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "acr",
    individual_languages: &[
    ],
};


pub const ACS: LanguageCode = LanguageCode {
    name: "Acroá",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "acs",
    individual_languages: &[
    ],
};


pub const ACT: LanguageCode = LanguageCode {
    name: "Achterhoeks",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "act",
    individual_languages: &[
    ],
};


pub const ACU: LanguageCode = LanguageCode {
    name: "Achuar-Shiwiar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "acu",
    individual_languages: &[
    ],
};


pub const ACV: LanguageCode = LanguageCode {
    name: "Achumawi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "acv",
    individual_languages: &[
    ],
};


pub const ACW: LanguageCode = LanguageCode {
    name: "Hijazi Arabic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "acw",
    individual_languages: &[
    ],
};


pub const ACX: LanguageCode = LanguageCode {
    name: "Omani Arabic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "acx",
    individual_languages: &[
    ],
};


pub const ACY: LanguageCode = LanguageCode {
    name: "Cypriot Arabic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "acy",
    individual_languages: &[
    ],
};


pub const ACZ: LanguageCode = LanguageCode {
    name: "Acheron",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "acz",
    individual_languages: &[
    ],
};


pub const ADB: LanguageCode = LanguageCode {
    name: "Atauran",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "adb",
    individual_languages: &[
    ],
};


pub const ADD: LanguageCode = LanguageCode {
    name: "Lidzonka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "add",
    individual_languages: &[
    ],
};


pub const ADE: LanguageCode = LanguageCode {
    name: "Adele",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ade",
    individual_languages: &[
    ],
};


pub const ADF: LanguageCode = LanguageCode {
    name: "Dhofari Arabic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "adf",
    individual_languages: &[
    ],
};


pub const ADG: LanguageCode = LanguageCode {
    name: "Andegerebinha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "adg",
    individual_languages: &[
    ],
};


pub const ADH: LanguageCode = LanguageCode {
    name: "Adhola",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "adh",
    individual_languages: &[
    ],
};


pub const ADI: LanguageCode = LanguageCode {
    name: "Adi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "adi",
    individual_languages: &[
    ],
};


pub const ADJ: LanguageCode = LanguageCode {
    name: "Adioukrou",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "adj",
    individual_languages: &[
    ],
};


pub const ADL: LanguageCode = LanguageCode {
    name: "Galo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "adl",
    individual_languages: &[
    ],
};


pub const ADN: LanguageCode = LanguageCode {
    name: "Adang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "adn",
    individual_languages: &[
    ],
};


pub const ADO: LanguageCode = LanguageCode {
    name: "Abu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ado",
    individual_languages: &[
    ],
};


pub const ADQ: LanguageCode = LanguageCode {
    name: "Adangbe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "adq",
    individual_languages: &[
    ],
};


pub const ADR: LanguageCode = LanguageCode {
    name: "Adonara",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "adr",
    individual_languages: &[
    ],
};


pub const ADS: LanguageCode = LanguageCode {
    name: "Adamorobe Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ads",
    individual_languages: &[
    ],
};


pub const ADT: LanguageCode = LanguageCode {
    name: "Adnyamathanha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "adt",
    individual_languages: &[
    ],
};


pub const ADU: LanguageCode = LanguageCode {
    name: "Aduge",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "adu",
    individual_languages: &[
    ],
};


pub const ADW: LanguageCode = LanguageCode {
    name: "Amundava",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "adw",
    individual_languages: &[
    ],
};


pub const ADX: LanguageCode = LanguageCode {
    name: "Amdo Tibetan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "adx",
    individual_languages: &[
    ],
};


pub const ADZ: LanguageCode = LanguageCode {
    name: "Adzera",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "adz",
    individual_languages: &[
    ],
};


pub const AEA: LanguageCode = LanguageCode {
    name: "Areba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aea",
    individual_languages: &[
    ],
};


pub const AEB: LanguageCode = LanguageCode {
    name: "Tunisian Arabic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aeb",
    individual_languages: &[
    ],
};


pub const AEC: LanguageCode = LanguageCode {
    name: "Saidi Arabic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aec",
    individual_languages: &[
    ],
};


pub const AED: LanguageCode = LanguageCode {
    name: "Argentine Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aed",
    individual_languages: &[
    ],
};


pub const AEE: LanguageCode = LanguageCode {
    name: "Northeast Pashai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aee",
    individual_languages: &[
    ],
};


pub const AEK: LanguageCode = LanguageCode {
    name: "Haeke",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aek",
    individual_languages: &[
    ],
};


pub const AEL: LanguageCode = LanguageCode {
    name: "Ambele",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ael",
    individual_languages: &[
    ],
};


pub const AEM: LanguageCode = LanguageCode {
    name: "Arem",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aem",
    individual_languages: &[
    ],
};


pub const AEN: LanguageCode = LanguageCode {
    name: "Armenian Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aen",
    individual_languages: &[
    ],
};


pub const AEQ: LanguageCode = LanguageCode {
    name: "Aer",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aeq",
    individual_languages: &[
    ],
};


pub const AER: LanguageCode = LanguageCode {
    name: "Eastern Arrernte",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aer",
    individual_languages: &[
    ],
};


pub const AES: LanguageCode = LanguageCode {
    name: "Alsea",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aes",
    individual_languages: &[
    ],
};


pub const AEU: LanguageCode = LanguageCode {
    name: "Akeu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aeu",
    individual_languages: &[
    ],
};


pub const AEW: LanguageCode = LanguageCode {
    name: "Ambakich",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aew",
    individual_languages: &[
    ],
};


pub const AEY: LanguageCode = LanguageCode {
    name: "Amele",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aey",
    individual_languages: &[
    ],
};


pub const AEZ: LanguageCode = LanguageCode {
    name: "Aeka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aez",
    individual_languages: &[
    ],
};


pub const AFB: LanguageCode = LanguageCode {
    name: "Gulf Arabic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "afb",
    individual_languages: &[
    ],
};


pub const AFD: LanguageCode = LanguageCode {
    name: "Andai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "afd",
    individual_languages: &[
    ],
};


pub const AFE: LanguageCode = LanguageCode {
    name: "Putukwam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "afe",
    individual_languages: &[
    ],
};


pub const AFG: LanguageCode = LanguageCode {
    name: "Afghan Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "afg",
    individual_languages: &[
    ],
};


pub const AFI: LanguageCode = LanguageCode {
    name: "Akrukay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "afi",
    individual_languages: &[
    ],
};


pub const AFK: LanguageCode = LanguageCode {
    name: "Nanubae",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "afk",
    individual_languages: &[
    ],
};


pub const AFN: LanguageCode = LanguageCode {
    name: "Defaka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "afn",
    individual_languages: &[
    ],
};


pub const AFO: LanguageCode = LanguageCode {
    name: "Eloyi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "afo",
    individual_languages: &[
    ],
};


pub const AFP: LanguageCode = LanguageCode {
    name: "Tapei",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "afp",
    individual_languages: &[
    ],
};


pub const AFS: LanguageCode = LanguageCode {
    name: "Afro-Seminole Creole",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "afs",
    individual_languages: &[
    ],
};


pub const AFT: LanguageCode = LanguageCode {
    name: "Afitti",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aft",
    individual_languages: &[
    ],
};


pub const AFU: LanguageCode = LanguageCode {
    name: "Awutu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "afu",
    individual_languages: &[
    ],
};


pub const AFZ: LanguageCode = LanguageCode {
    name: "Obokuitai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "afz",
    individual_languages: &[
    ],
};


pub const AGA: LanguageCode = LanguageCode {
    name: "Aguano",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aga",
    individual_languages: &[
    ],
};


pub const AGB: LanguageCode = LanguageCode {
    name: "Legbo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "agb",
    individual_languages: &[
    ],
};


pub const AGC: LanguageCode = LanguageCode {
    name: "Agatu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "agc",
    individual_languages: &[
    ],
};


pub const AGD: LanguageCode = LanguageCode {
    name: "Agarabi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "agd",
    individual_languages: &[
    ],
};


pub const AGE: LanguageCode = LanguageCode {
    name: "Angal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "age",
    individual_languages: &[
    ],
};


pub const AGF: LanguageCode = LanguageCode {
    name: "Arguni",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "agf",
    individual_languages: &[
    ],
};


pub const AGG: LanguageCode = LanguageCode {
    name: "Angor",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "agg",
    individual_languages: &[
    ],
};


pub const AGH: LanguageCode = LanguageCode {
    name: "Ngelima",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "agh",
    individual_languages: &[
    ],
};


pub const AGI: LanguageCode = LanguageCode {
    name: "Agariya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "agi",
    individual_languages: &[
    ],
};


pub const AGJ: LanguageCode = LanguageCode {
    name: "Argobba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "agj",
    individual_languages: &[
    ],
};


pub const AGK: LanguageCode = LanguageCode {
    name: "Isarog Agta",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "agk",
    individual_languages: &[
    ],
};


pub const AGL: LanguageCode = LanguageCode {
    name: "Fembe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "agl",
    individual_languages: &[
    ],
};


pub const AGM: LanguageCode = LanguageCode {
    name: "Angaataha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "agm",
    individual_languages: &[
    ],
};


pub const AGN: LanguageCode = LanguageCode {
    name: "Agutaynen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "agn",
    individual_languages: &[
    ],
};


pub const AGO: LanguageCode = LanguageCode {
    name: "Tainae",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ago",
    individual_languages: &[
    ],
};


pub const AGQ: LanguageCode = LanguageCode {
    name: "Aghem",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "agq",
    individual_languages: &[
    ],
};


pub const AGR: LanguageCode = LanguageCode {
    name: "Aguaruna",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "agr",
    individual_languages: &[
    ],
};


pub const AGS: LanguageCode = LanguageCode {
    name: "Esimbi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ags",
    individual_languages: &[
    ],
};


pub const AGT: LanguageCode = LanguageCode {
    name: "Central Cagayan Agta",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "agt",
    individual_languages: &[
    ],
};


pub const AGU: LanguageCode = LanguageCode {
    name: "Aguacateco",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "agu",
    individual_languages: &[
    ],
};


pub const AGV: LanguageCode = LanguageCode {
    name: "Remontado Dumagat",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "agv",
    individual_languages: &[
    ],
};


pub const AGW: LanguageCode = LanguageCode {
    name: "Kahua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "agw",
    individual_languages: &[
    ],
};


pub const AGX: LanguageCode = LanguageCode {
    name: "Aghul",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "agx",
    individual_languages: &[
    ],
};


pub const AGY: LanguageCode = LanguageCode {
    name: "Southern Alta",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "agy",
    individual_languages: &[
    ],
};


pub const AGZ: LanguageCode = LanguageCode {
    name: "Mt. Iriga Agta",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "agz",
    individual_languages: &[
    ],
};


pub const AHA: LanguageCode = LanguageCode {
    name: "Ahanta",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aha",
    individual_languages: &[
    ],
};


pub const AHB: LanguageCode = LanguageCode {
    name: "Axamb",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ahb",
    individual_languages: &[
    ],
};


pub const AHG: LanguageCode = LanguageCode {
    name: "Qimant",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ahg",
    individual_languages: &[
    ],
};


pub const AHH: LanguageCode = LanguageCode {
    name: "Aghu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ahh",
    individual_languages: &[
    ],
};


pub const AHI: LanguageCode = LanguageCode {
    name: "Tiagbamrin Aizi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ahi",
    individual_languages: &[
    ],
};


pub const AHK: LanguageCode = LanguageCode {
    name: "Akha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ahk",
    individual_languages: &[
    ],
};


pub const AHL: LanguageCode = LanguageCode {
    name: "Igo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ahl",
    individual_languages: &[
    ],
};


pub const AHM: LanguageCode = LanguageCode {
    name: "Mobumrin Aizi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ahm",
    individual_languages: &[
    ],
};


pub const AHN: LanguageCode = LanguageCode {
    name: "Àhàn",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ahn",
    individual_languages: &[
    ],
};


pub const AHO: LanguageCode = LanguageCode {
    name: "Ahom",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aho",
    individual_languages: &[
    ],
};


pub const AHP: LanguageCode = LanguageCode {
    name: "Aproumu Aizi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ahp",
    individual_languages: &[
    ],
};


pub const AHR: LanguageCode = LanguageCode {
    name: "Ahirani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ahr",
    individual_languages: &[
    ],
};


pub const AHS: LanguageCode = LanguageCode {
    name: "Ashe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ahs",
    individual_languages: &[
    ],
};


pub const AHT: LanguageCode = LanguageCode {
    name: "Ahtena",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aht",
    individual_languages: &[
    ],
};


pub const AIA: LanguageCode = LanguageCode {
    name: "Arosi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aia",
    individual_languages: &[
    ],
};


pub const AIB: LanguageCode = LanguageCode {
    name: "Ainu (China)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aib",
    individual_languages: &[
    ],
};


pub const AIC: LanguageCode = LanguageCode {
    name: "Ainbai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aic",
    individual_languages: &[
    ],
};


pub const AID: LanguageCode = LanguageCode {
    name: "Alngith",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aid",
    individual_languages: &[
    ],
};


pub const AIE: LanguageCode = LanguageCode {
    name: "Amara",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aie",
    individual_languages: &[
    ],
};


pub const AIF: LanguageCode = LanguageCode {
    name: "Agi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aif",
    individual_languages: &[
    ],
};


pub const AIG: LanguageCode = LanguageCode {
    name: "Antigua and Barbuda Creole English",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aig",
    individual_languages: &[
    ],
};


pub const AIH: LanguageCode = LanguageCode {
    name: "Ai-Cham",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aih",
    individual_languages: &[
    ],
};


pub const AII: LanguageCode = LanguageCode {
    name: "Assyrian Neo-Aramaic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aii",
    individual_languages: &[
    ],
};


pub const AIJ: LanguageCode = LanguageCode {
    name: "Lishanid Noshan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aij",
    individual_languages: &[
    ],
};


pub const AIK: LanguageCode = LanguageCode {
    name: "Ake",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aik",
    individual_languages: &[
    ],
};


pub const AIL: LanguageCode = LanguageCode {
    name: "Aimele",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ail",
    individual_languages: &[
    ],
};


pub const AIM: LanguageCode = LanguageCode {
    name: "Aimol",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aim",
    individual_languages: &[
    ],
};


pub const AIO: LanguageCode = LanguageCode {
    name: "Aiton",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aio",
    individual_languages: &[
    ],
};


pub const AIP: LanguageCode = LanguageCode {
    name: "Burumakok",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aip",
    individual_languages: &[
    ],
};


pub const AIQ: LanguageCode = LanguageCode {
    name: "Aimaq",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aiq",
    individual_languages: &[
    ],
};


pub const AIR: LanguageCode = LanguageCode {
    name: "Airoran",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "air",
    individual_languages: &[
    ],
};


pub const AIT: LanguageCode = LanguageCode {
    name: "Arikem",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ait",
    individual_languages: &[
    ],
};


pub const AIW: LanguageCode = LanguageCode {
    name: "Aari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aiw",
    individual_languages: &[
    ],
};


pub const AIX: LanguageCode = LanguageCode {
    name: "Aighon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aix",
    individual_languages: &[
    ],
};


pub const AIY: LanguageCode = LanguageCode {
    name: "Ali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aiy",
    individual_languages: &[
    ],
};


pub const AJA: LanguageCode = LanguageCode {
    name: "Aja (South Sudan)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aja",
    individual_languages: &[
    ],
};


pub const AJG: LanguageCode = LanguageCode {
    name: "Aja (Benin)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ajg",
    individual_languages: &[
    ],
};


pub const AJI: LanguageCode = LanguageCode {
    name: "Ajië",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aji",
    individual_languages: &[
    ],
};


pub const AJN: LanguageCode = LanguageCode {
    name: "Andajin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ajn",
    individual_languages: &[
    ],
};


pub const AJP: LanguageCode = LanguageCode {
    name: "South Levantine Arabic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ajp",
    individual_languages: &[
    ],
};


pub const AJS: LanguageCode = LanguageCode {
    name: "Algerian Jewish Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ajs",
    individual_languages: &[
    ],
};


pub const AJU: LanguageCode = LanguageCode {
    name: "Judeo-Moroccan Arabic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aju",
    individual_languages: &[
    ],
};


pub const AJW: LanguageCode = LanguageCode {
    name: "Ajawa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ajw",
    individual_languages: &[
    ],
};


pub const AJZ: LanguageCode = LanguageCode {
    name: "Amri Karbi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ajz",
    individual_languages: &[
    ],
};


pub const AKB: LanguageCode = LanguageCode {
    name: "Batak Angkola",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "akb",
    individual_languages: &[
    ],
};


pub const AKC: LanguageCode = LanguageCode {
    name: "Mpur",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "akc",
    individual_languages: &[
    ],
};


pub const AKD: LanguageCode = LanguageCode {
    name: "Ukpet-Ehom",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "akd",
    individual_languages: &[
    ],
};


pub const AKE: LanguageCode = LanguageCode {
    name: "Akawaio",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ake",
    individual_languages: &[
    ],
};


pub const AKF: LanguageCode = LanguageCode {
    name: "Akpa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "akf",
    individual_languages: &[
    ],
};


pub const AKG: LanguageCode = LanguageCode {
    name: "Anakalangu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "akg",
    individual_languages: &[
    ],
};


pub const AKH: LanguageCode = LanguageCode {
    name: "Angal Heneng",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "akh",
    individual_languages: &[
    ],
};


pub const AKI: LanguageCode = LanguageCode {
    name: "Aiome",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aki",
    individual_languages: &[
    ],
};


pub const AKJ: LanguageCode = LanguageCode {
    name: "Aka-Jeru",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "akj",
    individual_languages: &[
    ],
};


pub const AKL: LanguageCode = LanguageCode {
    name: "Aklanon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "akl",
    individual_languages: &[
    ],
};


pub const AKM: LanguageCode = LanguageCode {
    name: "Aka-Bo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "akm",
    individual_languages: &[
    ],
};


pub const AKO: LanguageCode = LanguageCode {
    name: "Akurio",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ako",
    individual_languages: &[
    ],
};


pub const AKP: LanguageCode = LanguageCode {
    name: "Siwu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "akp",
    individual_languages: &[
    ],
};


pub const AKQ: LanguageCode = LanguageCode {
    name: "Ak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "akq",
    individual_languages: &[
    ],
};


pub const AKR: LanguageCode = LanguageCode {
    name: "Araki",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "akr",
    individual_languages: &[
    ],
};


pub const AKS: LanguageCode = LanguageCode {
    name: "Akaselem",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aks",
    individual_languages: &[
    ],
};


pub const AKT: LanguageCode = LanguageCode {
    name: "Akolet",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "akt",
    individual_languages: &[
    ],
};


pub const AKU: LanguageCode = LanguageCode {
    name: "Akum",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aku",
    individual_languages: &[
    ],
};


pub const AKV: LanguageCode = LanguageCode {
    name: "Akhvakh",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "akv",
    individual_languages: &[
    ],
};


pub const AKW: LanguageCode = LanguageCode {
    name: "Akwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "akw",
    individual_languages: &[
    ],
};


pub const AKX: LanguageCode = LanguageCode {
    name: "Aka-Kede",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "akx",
    individual_languages: &[
    ],
};


pub const AKY: LanguageCode = LanguageCode {
    name: "Aka-Kol",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aky",
    individual_languages: &[
    ],
};


pub const AKZ: LanguageCode = LanguageCode {
    name: "Alabama",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "akz",
    individual_languages: &[
    ],
};


pub const ALA: LanguageCode = LanguageCode {
    name: "Alago",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ala",
    individual_languages: &[
    ],
};


pub const ALC: LanguageCode = LanguageCode {
    name: "Qawasqar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "alc",
    individual_languages: &[
    ],
};


pub const ALD: LanguageCode = LanguageCode {
    name: "Alladian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ald",
    individual_languages: &[
    ],
};


pub const ALF: LanguageCode = LanguageCode {
    name: "Alege",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "alf",
    individual_languages: &[
    ],
};


pub const ALH: LanguageCode = LanguageCode {
    name: "Alawa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "alh",
    individual_languages: &[
    ],
};


pub const ALI: LanguageCode = LanguageCode {
    name: "Amaimon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ali",
    individual_languages: &[
    ],
};


pub const ALJ: LanguageCode = LanguageCode {
    name: "Alangan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "alj",
    individual_languages: &[
    ],
};


pub const ALK: LanguageCode = LanguageCode {
    name: "Alak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "alk",
    individual_languages: &[
    ],
};


pub const ALL: LanguageCode = LanguageCode {
    name: "Allar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "all",
    individual_languages: &[
    ],
};


pub const ALM: LanguageCode = LanguageCode {
    name: "Amblong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "alm",
    individual_languages: &[
    ],
};


pub const ALN: LanguageCode = LanguageCode {
    name: "Gheg Albanian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aln",
    individual_languages: &[
    ],
};


pub const ALO: LanguageCode = LanguageCode {
    name: "Larike-Wakasihu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "alo",
    individual_languages: &[
    ],
};


pub const ALP: LanguageCode = LanguageCode {
    name: "Alune",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "alp",
    individual_languages: &[
    ],
};


pub const ALQ: LanguageCode = LanguageCode {
    name: "Algonquin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "alq",
    individual_languages: &[
    ],
};


pub const ALR: LanguageCode = LanguageCode {
    name: "Alutor",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "alr",
    individual_languages: &[
    ],
};


pub const ALS: LanguageCode = LanguageCode {
    name: "Tosk Albanian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "als",
    individual_languages: &[
    ],
};


pub const ALU: LanguageCode = LanguageCode {
    name: "'Are'are",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "alu",
    individual_languages: &[
    ],
};


pub const ALW: LanguageCode = LanguageCode {
    name: "Alaba-K’abeena",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "alw",
    individual_languages: &[
    ],
};


pub const ALX: LanguageCode = LanguageCode {
    name: "Amol",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "alx",
    individual_languages: &[
    ],
};


pub const ALY: LanguageCode = LanguageCode {
    name: "Alyawarr",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aly",
    individual_languages: &[
    ],
};


pub const ALZ: LanguageCode = LanguageCode {
    name: "Alur",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "alz",
    individual_languages: &[
    ],
};


pub const AMA: LanguageCode = LanguageCode {
    name: "Amanayé",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ama",
    individual_languages: &[
    ],
};


pub const AMB: LanguageCode = LanguageCode {
    name: "Ambo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "amb",
    individual_languages: &[
    ],
};


pub const AMC: LanguageCode = LanguageCode {
    name: "Amahuaca",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "amc",
    individual_languages: &[
    ],
};


pub const AME: LanguageCode = LanguageCode {
    name: "Yanesha'",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ame",
    individual_languages: &[
    ],
};


pub const AMF: LanguageCode = LanguageCode {
    name: "Hamer-Banna",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "amf",
    individual_languages: &[
    ],
};


pub const AMG: LanguageCode = LanguageCode {
    name: "Amurdak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "amg",
    individual_languages: &[
    ],
};


pub const AMI: LanguageCode = LanguageCode {
    name: "Amis",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ami",
    individual_languages: &[
    ],
};


pub const AMJ: LanguageCode = LanguageCode {
    name: "Amdang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "amj",
    individual_languages: &[
    ],
};


pub const AMK: LanguageCode = LanguageCode {
    name: "Ambai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "amk",
    individual_languages: &[
    ],
};


pub const AML: LanguageCode = LanguageCode {
    name: "War-Jaintia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aml",
    individual_languages: &[
    ],
};


pub const AMM: LanguageCode = LanguageCode {
    name: "Ama (Papua New Guinea)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "amm",
    individual_languages: &[
    ],
};


pub const AMN: LanguageCode = LanguageCode {
    name: "Amanab",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "amn",
    individual_languages: &[
    ],
};


pub const AMO: LanguageCode = LanguageCode {
    name: "Amo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "amo",
    individual_languages: &[
    ],
};


pub const AMP: LanguageCode = LanguageCode {
    name: "Alamblak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "amp",
    individual_languages: &[
    ],
};


pub const AMQ: LanguageCode = LanguageCode {
    name: "Amahai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "amq",
    individual_languages: &[
    ],
};


pub const AMR: LanguageCode = LanguageCode {
    name: "Amarakaeri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "amr",
    individual_languages: &[
    ],
};


pub const AMS: LanguageCode = LanguageCode {
    name: "Southern Amami-Oshima",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ams",
    individual_languages: &[
    ],
};


pub const AMT: LanguageCode = LanguageCode {
    name: "Amto",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "amt",
    individual_languages: &[
    ],
};


pub const AMU: LanguageCode = LanguageCode {
    name: "Guerrero Amuzgo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "amu",
    individual_languages: &[
    ],
};


pub const AMV: LanguageCode = LanguageCode {
    name: "Ambelau",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "amv",
    individual_languages: &[
    ],
};


pub const AMW: LanguageCode = LanguageCode {
    name: "Western Neo-Aramaic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "amw",
    individual_languages: &[
    ],
};


pub const AMX: LanguageCode = LanguageCode {
    name: "Anmatyerre",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "amx",
    individual_languages: &[
    ],
};


pub const AMY: LanguageCode = LanguageCode {
    name: "Ami",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "amy",
    individual_languages: &[
    ],
};


pub const AMZ: LanguageCode = LanguageCode {
    name: "Atampaya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "amz",
    individual_languages: &[
    ],
};


pub const ANA: LanguageCode = LanguageCode {
    name: "Andaqui",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ana",
    individual_languages: &[
    ],
};


pub const ANB: LanguageCode = LanguageCode {
    name: "Andoa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "anb",
    individual_languages: &[
    ],
};


pub const ANC: LanguageCode = LanguageCode {
    name: "Ngas",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "anc",
    individual_languages: &[
    ],
};


pub const AND: LanguageCode = LanguageCode {
    name: "Ansus",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "and",
    individual_languages: &[
    ],
};


pub const ANE: LanguageCode = LanguageCode {
    name: "Xârâcùù",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ane",
    individual_languages: &[
    ],
};


pub const ANF: LanguageCode = LanguageCode {
    name: "Animere",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "anf",
    individual_languages: &[
    ],
};


pub const ANH: LanguageCode = LanguageCode {
    name: "Nend",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "anh",
    individual_languages: &[
    ],
};


pub const ANI: LanguageCode = LanguageCode {
    name: "Andi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ani",
    individual_languages: &[
    ],
};


pub const ANJ: LanguageCode = LanguageCode {
    name: "Anor",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "anj",
    individual_languages: &[
    ],
};


pub const ANK: LanguageCode = LanguageCode {
    name: "Goemai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ank",
    individual_languages: &[
    ],
};


pub const ANL: LanguageCode = LanguageCode {
    name: "Anu-Hkongso Chin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "anl",
    individual_languages: &[
    ],
};


pub const ANM: LanguageCode = LanguageCode {
    name: "Anal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "anm",
    individual_languages: &[
    ],
};


pub const ANN: LanguageCode = LanguageCode {
    name: "Obolo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ann",
    individual_languages: &[
    ],
};


pub const ANO: LanguageCode = LanguageCode {
    name: "Andoque",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ano",
    individual_languages: &[
    ],
};


pub const ANQ: LanguageCode = LanguageCode {
    name: "Jarawa (India)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "anq",
    individual_languages: &[
    ],
};


pub const ANR: LanguageCode = LanguageCode {
    name: "Andh",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "anr",
    individual_languages: &[
    ],
};


pub const ANS: LanguageCode = LanguageCode {
    name: "Anserma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ans",
    individual_languages: &[
    ],
};


pub const ANT: LanguageCode = LanguageCode {
    name: "Antakarinya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ant",
    individual_languages: &[
    ],
};


pub const ANU: LanguageCode = LanguageCode {
    name: "Anuak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "anu",
    individual_languages: &[
    ],
};


pub const ANV: LanguageCode = LanguageCode {
    name: "Denya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "anv",
    individual_languages: &[
    ],
};


pub const ANW: LanguageCode = LanguageCode {
    name: "Anaang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "anw",
    individual_languages: &[
    ],
};


pub const ANX: LanguageCode = LanguageCode {
    name: "Andra-Hus",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "anx",
    individual_languages: &[
    ],
};


pub const ANY: LanguageCode = LanguageCode {
    name: "Anyin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "any",
    individual_languages: &[
    ],
};


pub const ANZ: LanguageCode = LanguageCode {
    name: "Anem",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "anz",
    individual_languages: &[
    ],
};


pub const AOA: LanguageCode = LanguageCode {
    name: "Angolar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aoa",
    individual_languages: &[
    ],
};


pub const AOB: LanguageCode = LanguageCode {
    name: "Abom",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aob",
    individual_languages: &[
    ],
};


pub const AOC: LanguageCode = LanguageCode {
    name: "Pemon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aoc",
    individual_languages: &[
    ],
};


pub const AOD: LanguageCode = LanguageCode {
    name: "Andarum",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aod",
    individual_languages: &[
    ],
};


pub const AOE: LanguageCode = LanguageCode {
    name: "Angal Enen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aoe",
    individual_languages: &[
    ],
};


pub const AOF: LanguageCode = LanguageCode {
    name: "Bragat",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aof",
    individual_languages: &[
    ],
};


pub const AOG: LanguageCode = LanguageCode {
    name: "Angoram",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aog",
    individual_languages: &[
    ],
};


pub const AOI: LanguageCode = LanguageCode {
    name: "Anindilyakwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aoi",
    individual_languages: &[
    ],
};


pub const AOJ: LanguageCode = LanguageCode {
    name: "Mufian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aoj",
    individual_languages: &[
    ],
};


pub const AOK: LanguageCode = LanguageCode {
    name: "Arhö",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aok",
    individual_languages: &[
    ],
};


pub const AOL: LanguageCode = LanguageCode {
    name: "Alor",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aol",
    individual_languages: &[
    ],
};


pub const AOM: LanguageCode = LanguageCode {
    name: "Ömie",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aom",
    individual_languages: &[
    ],
};


pub const AON: LanguageCode = LanguageCode {
    name: "Bumbita Arapesh",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aon",
    individual_languages: &[
    ],
};


pub const AOR: LanguageCode = LanguageCode {
    name: "Aore",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aor",
    individual_languages: &[
    ],
};


pub const AOS: LanguageCode = LanguageCode {
    name: "Taikat",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aos",
    individual_languages: &[
    ],
};


pub const AOT: LanguageCode = LanguageCode {
    name: "Atong (India)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aot",
    individual_languages: &[
    ],
};


pub const AOU: LanguageCode = LanguageCode {
    name: "A'ou",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aou",
    individual_languages: &[
    ],
};


pub const AOX: LanguageCode = LanguageCode {
    name: "Atorada",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aox",
    individual_languages: &[
    ],
};


pub const AOZ: LanguageCode = LanguageCode {
    name: "Uab Meto",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aoz",
    individual_languages: &[
    ],
};


pub const APB: LanguageCode = LanguageCode {
    name: "Sa'a",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "apb",
    individual_languages: &[
    ],
};


pub const APC: LanguageCode = LanguageCode {
    name: "North Levantine Arabic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "apc",
    individual_languages: &[
    ],
};


pub const APD: LanguageCode = LanguageCode {
    name: "Sudanese Arabic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "apd",
    individual_languages: &[
    ],
};


pub const APE: LanguageCode = LanguageCode {
    name: "Bukiyip",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ape",
    individual_languages: &[
    ],
};


pub const APF: LanguageCode = LanguageCode {
    name: "Pahanan Agta",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "apf",
    individual_languages: &[
    ],
};


pub const APG: LanguageCode = LanguageCode {
    name: "Ampanang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "apg",
    individual_languages: &[
    ],
};


pub const APH: LanguageCode = LanguageCode {
    name: "Athpariya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aph",
    individual_languages: &[
    ],
};


pub const API: LanguageCode = LanguageCode {
    name: "Apiaká",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "api",
    individual_languages: &[
    ],
};


pub const APJ: LanguageCode = LanguageCode {
    name: "Jicarilla Apache",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "apj",
    individual_languages: &[
    ],
};


pub const APK: LanguageCode = LanguageCode {
    name: "Kiowa Apache",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "apk",
    individual_languages: &[
    ],
};


pub const APL: LanguageCode = LanguageCode {
    name: "Lipan Apache",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "apl",
    individual_languages: &[
    ],
};


pub const APM: LanguageCode = LanguageCode {
    name: "Mescalero-Chiricahua Apache",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "apm",
    individual_languages: &[
    ],
};


pub const APN: LanguageCode = LanguageCode {
    name: "Apinayé",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "apn",
    individual_languages: &[
    ],
};


pub const APO: LanguageCode = LanguageCode {
    name: "Ambul",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "apo",
    individual_languages: &[
    ],
};


pub const APP: LanguageCode = LanguageCode {
    name: "Apma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "app",
    individual_languages: &[
    ],
};


pub const APQ: LanguageCode = LanguageCode {
    name: "A-Pucikwar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "apq",
    individual_languages: &[
    ],
};


pub const APR: LanguageCode = LanguageCode {
    name: "Arop-Lokep",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "apr",
    individual_languages: &[
    ],
};


pub const APS: LanguageCode = LanguageCode {
    name: "Arop-Sissano",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aps",
    individual_languages: &[
    ],
};


pub const APT: LanguageCode = LanguageCode {
    name: "Apatani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "apt",
    individual_languages: &[
    ],
};


pub const APU: LanguageCode = LanguageCode {
    name: "Apurinã",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "apu",
    individual_languages: &[
    ],
};


pub const APV: LanguageCode = LanguageCode {
    name: "Alapmunte",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "apv",
    individual_languages: &[
    ],
};


pub const APW: LanguageCode = LanguageCode {
    name: "Western Apache",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "apw",
    individual_languages: &[
    ],
};


pub const APX: LanguageCode = LanguageCode {
    name: "Aputai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "apx",
    individual_languages: &[
    ],
};


pub const APY: LanguageCode = LanguageCode {
    name: "Apalaí",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "apy",
    individual_languages: &[
    ],
};


pub const APZ: LanguageCode = LanguageCode {
    name: "Safeyoka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "apz",
    individual_languages: &[
    ],
};


pub const AQC: LanguageCode = LanguageCode {
    name: "Archi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aqc",
    individual_languages: &[
    ],
};


pub const AQD: LanguageCode = LanguageCode {
    name: "Ampari Dogon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aqd",
    individual_languages: &[
    ],
};


pub const AQG: LanguageCode = LanguageCode {
    name: "Arigidi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aqg",
    individual_languages: &[
    ],
};


pub const AQK: LanguageCode = LanguageCode {
    name: "Aninka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aqk",
    individual_languages: &[
    ],
};


pub const AQM: LanguageCode = LanguageCode {
    name: "Atohwaim",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aqm",
    individual_languages: &[
    ],
};


pub const AQN: LanguageCode = LanguageCode {
    name: "Northern Alta",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aqn",
    individual_languages: &[
    ],
};


pub const AQP: LanguageCode = LanguageCode {
    name: "Atakapa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aqp",
    individual_languages: &[
    ],
};


pub const AQR: LanguageCode = LanguageCode {
    name: "Arhâ",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aqr",
    individual_languages: &[
    ],
};


pub const AQT: LanguageCode = LanguageCode {
    name: "Angaité",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aqt",
    individual_languages: &[
    ],
};


pub const AQZ: LanguageCode = LanguageCode {
    name: "Akuntsu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aqz",
    individual_languages: &[
    ],
};


pub const ARB: LanguageCode = LanguageCode {
    name: "Standard Arabic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "arb",
    individual_languages: &[
    ],
};


pub const ARD: LanguageCode = LanguageCode {
    name: "Arabana",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ard",
    individual_languages: &[
    ],
};


pub const ARE: LanguageCode = LanguageCode {
    name: "Western Arrarnta",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "are",
    individual_languages: &[
    ],
};


pub const ARH: LanguageCode = LanguageCode {
    name: "Arhuaco",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "arh",
    individual_languages: &[
    ],
};


pub const ARI: LanguageCode = LanguageCode {
    name: "Arikara",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ari",
    individual_languages: &[
    ],
};


pub const ARJ: LanguageCode = LanguageCode {
    name: "Arapaso",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "arj",
    individual_languages: &[
    ],
};


pub const ARK: LanguageCode = LanguageCode {
    name: "Arikapú",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ark",
    individual_languages: &[
    ],
};


pub const ARL: LanguageCode = LanguageCode {
    name: "Arabela",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "arl",
    individual_languages: &[
    ],
};


pub const ARO: LanguageCode = LanguageCode {
    name: "Araona",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aro",
    individual_languages: &[
    ],
};


pub const ARQ: LanguageCode = LanguageCode {
    name: "Algerian Arabic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "arq",
    individual_languages: &[
    ],
};


pub const ARR: LanguageCode = LanguageCode {
    name: "Karo (Brazil)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "arr",
    individual_languages: &[
    ],
};


pub const ARS: LanguageCode = LanguageCode {
    name: "Najdi Arabic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ars",
    individual_languages: &[
    ],
};


pub const ARU: LanguageCode = LanguageCode {
    name: "Aruá (Amazonas State)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aru",
    individual_languages: &[
    ],
};


pub const ARV: LanguageCode = LanguageCode {
    name: "Arbore",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "arv",
    individual_languages: &[
    ],
};


pub const ARX: LanguageCode = LanguageCode {
    name: "Aruá (Rodonia State)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "arx",
    individual_languages: &[
    ],
};


pub const ARY: LanguageCode = LanguageCode {
    name: "Moroccan Arabic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ary",
    individual_languages: &[
    ],
};


pub const ARZ: LanguageCode = LanguageCode {
    name: "Egyptian Arabic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "arz",
    individual_languages: &[
    ],
};


pub const ASA: LanguageCode = LanguageCode {
    name: "Asu (Tanzania)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "asa",
    individual_languages: &[
    ],
};


pub const ASB: LanguageCode = LanguageCode {
    name: "Assiniboine",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "asb",
    individual_languages: &[
    ],
};


pub const ASC: LanguageCode = LanguageCode {
    name: "Casuarina Coast Asmat",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "asc",
    individual_languages: &[
    ],
};


pub const ASE: LanguageCode = LanguageCode {
    name: "American Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ase",
    individual_languages: &[
    ],
};


pub const ASF: LanguageCode = LanguageCode {
    name: "Auslan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "asf",
    individual_languages: &[
    ],
};


pub const ASG: LanguageCode = LanguageCode {
    name: "Cishingini",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "asg",
    individual_languages: &[
    ],
};


pub const ASH: LanguageCode = LanguageCode {
    name: "Abishira",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ash",
    individual_languages: &[
    ],
};


pub const ASI: LanguageCode = LanguageCode {
    name: "Buruwai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "asi",
    individual_languages: &[
    ],
};


pub const ASJ: LanguageCode = LanguageCode {
    name: "Sari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "asj",
    individual_languages: &[
    ],
};


pub const ASK: LanguageCode = LanguageCode {
    name: "Ashkun",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ask",
    individual_languages: &[
    ],
};


pub const ASL: LanguageCode = LanguageCode {
    name: "Asilulu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "asl",
    individual_languages: &[
    ],
};


pub const ASN: LanguageCode = LanguageCode {
    name: "Xingú Asuriní",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "asn",
    individual_languages: &[
    ],
};


pub const ASO: LanguageCode = LanguageCode {
    name: "Dano",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aso",
    individual_languages: &[
    ],
};


pub const ASP: LanguageCode = LanguageCode {
    name: "Algerian Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "asp",
    individual_languages: &[
    ],
};


pub const ASQ: LanguageCode = LanguageCode {
    name: "Austrian Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "asq",
    individual_languages: &[
    ],
};


pub const ASR: LanguageCode = LanguageCode {
    name: "Asuri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "asr",
    individual_languages: &[
    ],
};


pub const ASS: LanguageCode = LanguageCode {
    name: "Ipulo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ass",
    individual_languages: &[
    ],
};


pub const ASU: LanguageCode = LanguageCode {
    name: "Tocantins Asurini",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "asu",
    individual_languages: &[
    ],
};


pub const ASV: LanguageCode = LanguageCode {
    name: "Asoa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "asv",
    individual_languages: &[
    ],
};


pub const ASW: LanguageCode = LanguageCode {
    name: "Australian Aborigines Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "asw",
    individual_languages: &[
    ],
};


pub const ASX: LanguageCode = LanguageCode {
    name: "Muratayak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "asx",
    individual_languages: &[
    ],
};


pub const ASY: LanguageCode = LanguageCode {
    name: "Yaosakor Asmat",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "asy",
    individual_languages: &[
    ],
};


pub const ASZ: LanguageCode = LanguageCode {
    name: "As",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "asz",
    individual_languages: &[
    ],
};


pub const ATA: LanguageCode = LanguageCode {
    name: "Pele-Ata",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ata",
    individual_languages: &[
    ],
};


pub const ATB: LanguageCode = LanguageCode {
    name: "Zaiwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "atb",
    individual_languages: &[
    ],
};


pub const ATC: LanguageCode = LanguageCode {
    name: "Atsahuaca",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "atc",
    individual_languages: &[
    ],
};


pub const ATD: LanguageCode = LanguageCode {
    name: "Ata Manobo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "atd",
    individual_languages: &[
    ],
};


pub const ATE: LanguageCode = LanguageCode {
    name: "Atemble",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ate",
    individual_languages: &[
    ],
};


pub const ATG: LanguageCode = LanguageCode {
    name: "Ivbie North-Okpela-Arhe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "atg",
    individual_languages: &[
    ],
};


pub const ATI: LanguageCode = LanguageCode {
    name: "Attié",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ati",
    individual_languages: &[
    ],
};


pub const ATJ: LanguageCode = LanguageCode {
    name: "Atikamekw",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "atj",
    individual_languages: &[
    ],
};


pub const ATK: LanguageCode = LanguageCode {
    name: "Ati",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "atk",
    individual_languages: &[
    ],
};


pub const ATL: LanguageCode = LanguageCode {
    name: "Mt. Iraya Agta",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "atl",
    individual_languages: &[
    ],
};


pub const ATM: LanguageCode = LanguageCode {
    name: "Ata",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "atm",
    individual_languages: &[
    ],
};


pub const ATN: LanguageCode = LanguageCode {
    name: "Ashtiani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "atn",
    individual_languages: &[
    ],
};


pub const ATO: LanguageCode = LanguageCode {
    name: "Atong (Cameroon)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ato",
    individual_languages: &[
    ],
};


pub const ATP: LanguageCode = LanguageCode {
    name: "Pudtol Atta",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "atp",
    individual_languages: &[
    ],
};


pub const ATQ: LanguageCode = LanguageCode {
    name: "Aralle-Tabulahan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "atq",
    individual_languages: &[
    ],
};


pub const ATR: LanguageCode = LanguageCode {
    name: "Waimiri-Atroari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "atr",
    individual_languages: &[
    ],
};


pub const ATS: LanguageCode = LanguageCode {
    name: "Gros Ventre",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ats",
    individual_languages: &[
    ],
};


pub const ATT: LanguageCode = LanguageCode {
    name: "Pamplona Atta",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "att",
    individual_languages: &[
    ],
};


pub const ATU: LanguageCode = LanguageCode {
    name: "Reel",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "atu",
    individual_languages: &[
    ],
};


pub const ATV: LanguageCode = LanguageCode {
    name: "Northern Altai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "atv",
    individual_languages: &[
    ],
};


pub const ATW: LanguageCode = LanguageCode {
    name: "Atsugewi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "atw",
    individual_languages: &[
    ],
};


pub const ATX: LanguageCode = LanguageCode {
    name: "Arutani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "atx",
    individual_languages: &[
    ],
};


pub const ATY: LanguageCode = LanguageCode {
    name: "Aneityum",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aty",
    individual_languages: &[
    ],
};


pub const ATZ: LanguageCode = LanguageCode {
    name: "Arta",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "atz",
    individual_languages: &[
    ],
};


pub const AUA: LanguageCode = LanguageCode {
    name: "Asumboa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aua",
    individual_languages: &[
    ],
};


pub const AUB: LanguageCode = LanguageCode {
    name: "Alugu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aub",
    individual_languages: &[
    ],
};


pub const AUC: LanguageCode = LanguageCode {
    name: "Waorani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "auc",
    individual_languages: &[
    ],
};


pub const AUD: LanguageCode = LanguageCode {
    name: "Anuta",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aud",
    individual_languages: &[
    ],
};


pub const AUG: LanguageCode = LanguageCode {
    name: "Aguna",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aug",
    individual_languages: &[
    ],
};


pub const AUH: LanguageCode = LanguageCode {
    name: "Aushi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "auh",
    individual_languages: &[
    ],
};


pub const AUI: LanguageCode = LanguageCode {
    name: "Anuki",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aui",
    individual_languages: &[
    ],
};


pub const AUJ: LanguageCode = LanguageCode {
    name: "Awjilah",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "auj",
    individual_languages: &[
    ],
};


pub const AUK: LanguageCode = LanguageCode {
    name: "Heyo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "auk",
    individual_languages: &[
    ],
};


pub const AUL: LanguageCode = LanguageCode {
    name: "Aulua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aul",
    individual_languages: &[
    ],
};


pub const AUM: LanguageCode = LanguageCode {
    name: "Asu (Nigeria)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aum",
    individual_languages: &[
    ],
};


pub const AUN: LanguageCode = LanguageCode {
    name: "Molmo One",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aun",
    individual_languages: &[
    ],
};


pub const AUO: LanguageCode = LanguageCode {
    name: "Auyokawa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "auo",
    individual_languages: &[
    ],
};


pub const AUP: LanguageCode = LanguageCode {
    name: "Makayam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aup",
    individual_languages: &[
    ],
};


pub const AUQ: LanguageCode = LanguageCode {
    name: "Anus",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "auq",
    individual_languages: &[
    ],
};


pub const AUR: LanguageCode = LanguageCode {
    name: "Aruek",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aur",
    individual_languages: &[
    ],
};


pub const AUT: LanguageCode = LanguageCode {
    name: "Austral",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aut",
    individual_languages: &[
    ],
};


pub const AUU: LanguageCode = LanguageCode {
    name: "Auye",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "auu",
    individual_languages: &[
    ],
};


pub const AUW: LanguageCode = LanguageCode {
    name: "Awyi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "auw",
    individual_languages: &[
    ],
};


pub const AUX: LanguageCode = LanguageCode {
    name: "Aurá",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aux",
    individual_languages: &[
    ],
};


pub const AUY: LanguageCode = LanguageCode {
    name: "Awiyaana",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "auy",
    individual_languages: &[
    ],
};


pub const AUZ: LanguageCode = LanguageCode {
    name: "Uzbeki Arabic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "auz",
    individual_languages: &[
    ],
};


pub const AVB: LanguageCode = LanguageCode {
    name: "Avau",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "avb",
    individual_languages: &[
    ],
};


pub const AVD: LanguageCode = LanguageCode {
    name: "Alviri-Vidari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "avd",
    individual_languages: &[
    ],
};


pub const AVI: LanguageCode = LanguageCode {
    name: "Avikam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "avi",
    individual_languages: &[
    ],
};


pub const AVK: LanguageCode = LanguageCode {
    name: "Kotava",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "avk",
    individual_languages: &[
    ],
};


pub const AVL: LanguageCode = LanguageCode {
    name: "Eastern Egyptian Bedawi Arabic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "avl",
    individual_languages: &[
    ],
};


pub const AVM: LanguageCode = LanguageCode {
    name: "Angkamuthi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "avm",
    individual_languages: &[
    ],
};


pub const AVN: LanguageCode = LanguageCode {
    name: "Avatime",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "avn",
    individual_languages: &[
    ],
};


pub const AVO: LanguageCode = LanguageCode {
    name: "Agavotaguerra",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "avo",
    individual_languages: &[
    ],
};


pub const AVS: LanguageCode = LanguageCode {
    name: "Aushiri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "avs",
    individual_languages: &[
    ],
};


pub const AVT: LanguageCode = LanguageCode {
    name: "Au",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "avt",
    individual_languages: &[
    ],
};


pub const AVU: LanguageCode = LanguageCode {
    name: "Avokaya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "avu",
    individual_languages: &[
    ],
};


pub const AVV: LanguageCode = LanguageCode {
    name: "Avá-Canoeiro",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "avv",
    individual_languages: &[
    ],
};


pub const AWB: LanguageCode = LanguageCode {
    name: "Awa (Papua New Guinea)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "awb",
    individual_languages: &[
    ],
};


pub const AWC: LanguageCode = LanguageCode {
    name: "Cicipu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "awc",
    individual_languages: &[
    ],
};


pub const AWE: LanguageCode = LanguageCode {
    name: "Awetí",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "awe",
    individual_languages: &[
    ],
};


pub const AWG: LanguageCode = LanguageCode {
    name: "Anguthimri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "awg",
    individual_languages: &[
    ],
};


pub const AWH: LanguageCode = LanguageCode {
    name: "Awbono",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "awh",
    individual_languages: &[
    ],
};


pub const AWI: LanguageCode = LanguageCode {
    name: "Aekyom",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "awi",
    individual_languages: &[
    ],
};


pub const AWK: LanguageCode = LanguageCode {
    name: "Awabakal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "awk",
    individual_languages: &[
    ],
};


pub const AWM: LanguageCode = LanguageCode {
    name: "Arawum",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "awm",
    individual_languages: &[
    ],
};


pub const AWN: LanguageCode = LanguageCode {
    name: "Awngi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "awn",
    individual_languages: &[
    ],
};


pub const AWO: LanguageCode = LanguageCode {
    name: "Awak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "awo",
    individual_languages: &[
    ],
};


pub const AWR: LanguageCode = LanguageCode {
    name: "Awera",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "awr",
    individual_languages: &[
    ],
};


pub const AWS: LanguageCode = LanguageCode {
    name: "South Awyu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aws",
    individual_languages: &[
    ],
};


pub const AWT: LanguageCode = LanguageCode {
    name: "Araweté",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "awt",
    individual_languages: &[
    ],
};


pub const AWU: LanguageCode = LanguageCode {
    name: "Central Awyu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "awu",
    individual_languages: &[
    ],
};


pub const AWV: LanguageCode = LanguageCode {
    name: "Jair Awyu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "awv",
    individual_languages: &[
    ],
};


pub const AWW: LanguageCode = LanguageCode {
    name: "Awun",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aww",
    individual_languages: &[
    ],
};


pub const AWX: LanguageCode = LanguageCode {
    name: "Awara",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "awx",
    individual_languages: &[
    ],
};


pub const AWY: LanguageCode = LanguageCode {
    name: "Edera Awyu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "awy",
    individual_languages: &[
    ],
};


pub const AXB: LanguageCode = LanguageCode {
    name: "Abipon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "axb",
    individual_languages: &[
    ],
};


pub const AXE: LanguageCode = LanguageCode {
    name: "Ayerrerenge",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "axe",
    individual_languages: &[
    ],
};


pub const AXG: LanguageCode = LanguageCode {
    name: "Mato Grosso Arára",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "axg",
    individual_languages: &[
    ],
};


pub const AXK: LanguageCode = LanguageCode {
    name: "Yaka (Central African Republic)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "axk",
    individual_languages: &[
    ],
};


pub const AXL: LanguageCode = LanguageCode {
    name: "Lower Southern Aranda",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "axl",
    individual_languages: &[
    ],
};


pub const AXM: LanguageCode = LanguageCode {
    name: "Middle Armenian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "axm",
    individual_languages: &[
    ],
};


pub const AXX: LanguageCode = LanguageCode {
    name: "Xârâgurè",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "axx",
    individual_languages: &[
    ],
};


pub const AYA: LanguageCode = LanguageCode {
    name: "Awar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aya",
    individual_languages: &[
    ],
};


pub const AYB: LanguageCode = LanguageCode {
    name: "Ayizo Gbe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ayb",
    individual_languages: &[
    ],
};


pub const AYC: LanguageCode = LanguageCode {
    name: "Southern Aymara",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ayc",
    individual_languages: &[
    ],
};


pub const AYD: LanguageCode = LanguageCode {
    name: "Ayabadhu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ayd",
    individual_languages: &[
    ],
};


pub const AYE: LanguageCode = LanguageCode {
    name: "Ayere",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aye",
    individual_languages: &[
    ],
};


pub const AYG: LanguageCode = LanguageCode {
    name: "Ginyanga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ayg",
    individual_languages: &[
    ],
};


pub const AYH: LanguageCode = LanguageCode {
    name: "Hadrami Arabic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ayh",
    individual_languages: &[
    ],
};


pub const AYI: LanguageCode = LanguageCode {
    name: "Leyigha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ayi",
    individual_languages: &[
    ],
};


pub const AYK: LanguageCode = LanguageCode {
    name: "Akuku",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ayk",
    individual_languages: &[
    ],
};


pub const AYL: LanguageCode = LanguageCode {
    name: "Libyan Arabic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ayl",
    individual_languages: &[
    ],
};


pub const AYN: LanguageCode = LanguageCode {
    name: "Sanaani Arabic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ayn",
    individual_languages: &[
    ],
};


pub const AYO: LanguageCode = LanguageCode {
    name: "Ayoreo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ayo",
    individual_languages: &[
    ],
};


pub const AYP: LanguageCode = LanguageCode {
    name: "North Mesopotamian Arabic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ayp",
    individual_languages: &[
    ],
};


pub const AYQ: LanguageCode = LanguageCode {
    name: "Ayi (Papua New Guinea)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ayq",
    individual_languages: &[
    ],
};


pub const AYR: LanguageCode = LanguageCode {
    name: "Central Aymara",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ayr",
    individual_languages: &[
    ],
};


pub const AYS: LanguageCode = LanguageCode {
    name: "Sorsogon Ayta",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ays",
    individual_languages: &[
    ],
};


pub const AYT: LanguageCode = LanguageCode {
    name: "Magbukun Ayta",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ayt",
    individual_languages: &[
    ],
};


pub const AYU: LanguageCode = LanguageCode {
    name: "Ayu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ayu",
    individual_languages: &[
    ],
};


pub const AYZ: LanguageCode = LanguageCode {
    name: "Mai Brat",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ayz",
    individual_languages: &[
    ],
};


pub const AZA: LanguageCode = LanguageCode {
    name: "Azha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "aza",
    individual_languages: &[
    ],
};


pub const AZB: LanguageCode = LanguageCode {
    name: "South Azerbaijani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "azb",
    individual_languages: &[
    ],
};


pub const AZD: LanguageCode = LanguageCode {
    name: "Eastern Durango Nahuatl",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "azd",
    individual_languages: &[
    ],
};


pub const AZG: LanguageCode = LanguageCode {
    name: "San Pedro Amuzgos Amuzgo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "azg",
    individual_languages: &[
    ],
};


pub const AZJ: LanguageCode = LanguageCode {
    name: "North Azerbaijani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "azj",
    individual_languages: &[
    ],
};


pub const AZM: LanguageCode = LanguageCode {
    name: "Ipalapa Amuzgo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "azm",
    individual_languages: &[
    ],
};


pub const AZN: LanguageCode = LanguageCode {
    name: "Western Durango Nahuatl",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "azn",
    individual_languages: &[
    ],
};


pub const AZO: LanguageCode = LanguageCode {
    name: "Awing",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "azo",
    individual_languages: &[
    ],
};


pub const AZT: LanguageCode = LanguageCode {
    name: "Faire Atta",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "azt",
    individual_languages: &[
    ],
};


pub const AZZ: LanguageCode = LanguageCode {
    name: "Highland Puebla Nahuatl",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "azz",
    individual_languages: &[
    ],
};


pub const BAA: LanguageCode = LanguageCode {
    name: "Babatana",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "baa",
    individual_languages: &[
    ],
};


pub const BAB: LanguageCode = LanguageCode {
    name: "Bainouk-Gunyuño",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bab",
    individual_languages: &[
    ],
};


pub const BAC: LanguageCode = LanguageCode {
    name: "Badui",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bac",
    individual_languages: &[
    ],
};


pub const BAE: LanguageCode = LanguageCode {
    name: "Baré",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bae",
    individual_languages: &[
    ],
};


pub const BAF: LanguageCode = LanguageCode {
    name: "Nubaca",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "baf",
    individual_languages: &[
    ],
};


pub const BAG: LanguageCode = LanguageCode {
    name: "Tuki",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bag",
    individual_languages: &[
    ],
};


pub const BAH: LanguageCode = LanguageCode {
    name: "Bahamas Creole English",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bah",
    individual_languages: &[
    ],
};


pub const BAJ: LanguageCode = LanguageCode {
    name: "Barakai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "baj",
    individual_languages: &[
    ],
};


pub const BAO: LanguageCode = LanguageCode {
    name: "Waimaha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bao",
    individual_languages: &[
    ],
};


pub const BAP: LanguageCode = LanguageCode {
    name: "Bantawa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bap",
    individual_languages: &[
    ],
};


pub const BAR: LanguageCode = LanguageCode {
    name: "Bavarian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bar",
    individual_languages: &[
    ],
};


pub const BAU: LanguageCode = LanguageCode {
    name: "Bada (Nigeria)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bau",
    individual_languages: &[
    ],
};


pub const BAV: LanguageCode = LanguageCode {
    name: "Vengo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bav",
    individual_languages: &[
    ],
};


pub const BAW: LanguageCode = LanguageCode {
    name: "Bambili-Bambui",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "baw",
    individual_languages: &[
    ],
};


pub const BAX: LanguageCode = LanguageCode {
    name: "Bamun",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bax",
    individual_languages: &[
    ],
};


pub const BAY: LanguageCode = LanguageCode {
    name: "Batuley",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bay",
    individual_languages: &[
    ],
};


pub const BBA: LanguageCode = LanguageCode {
    name: "Baatonum",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bba",
    individual_languages: &[
    ],
};


pub const BBB: LanguageCode = LanguageCode {
    name: "Barai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bbb",
    individual_languages: &[
    ],
};


pub const BBC: LanguageCode = LanguageCode {
    name: "Batak Toba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bbc",
    individual_languages: &[
    ],
};


pub const BBD: LanguageCode = LanguageCode {
    name: "Bau",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bbd",
    individual_languages: &[
    ],
};


pub const BBE: LanguageCode = LanguageCode {
    name: "Bangba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bbe",
    individual_languages: &[
    ],
};


pub const BBF: LanguageCode = LanguageCode {
    name: "Baibai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bbf",
    individual_languages: &[
    ],
};


pub const BBG: LanguageCode = LanguageCode {
    name: "Barama",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bbg",
    individual_languages: &[
    ],
};


pub const BBH: LanguageCode = LanguageCode {
    name: "Bugan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bbh",
    individual_languages: &[
    ],
};


pub const BBI: LanguageCode = LanguageCode {
    name: "Barombi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bbi",
    individual_languages: &[
    ],
};


pub const BBJ: LanguageCode = LanguageCode {
    name: "Ghomálá'",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bbj",
    individual_languages: &[
    ],
};


pub const BBK: LanguageCode = LanguageCode {
    name: "Babanki",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bbk",
    individual_languages: &[
    ],
};


pub const BBL: LanguageCode = LanguageCode {
    name: "Bats",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bbl",
    individual_languages: &[
    ],
};


pub const BBM: LanguageCode = LanguageCode {
    name: "Babango",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bbm",
    individual_languages: &[
    ],
};


pub const BBN: LanguageCode = LanguageCode {
    name: "Uneapa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bbn",
    individual_languages: &[
    ],
};


pub const BBO: LanguageCode = LanguageCode {
    name: "Northern Bobo Madaré",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bbo",
    individual_languages: &[
    ],
};


pub const BBP: LanguageCode = LanguageCode {
    name: "West Central Banda",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bbp",
    individual_languages: &[
    ],
};


pub const BBQ: LanguageCode = LanguageCode {
    name: "Bamali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bbq",
    individual_languages: &[
    ],
};


pub const BBR: LanguageCode = LanguageCode {
    name: "Girawa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bbr",
    individual_languages: &[
    ],
};


pub const BBS: LanguageCode = LanguageCode {
    name: "Bakpinka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bbs",
    individual_languages: &[
    ],
};


pub const BBT: LanguageCode = LanguageCode {
    name: "Mburku",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bbt",
    individual_languages: &[
    ],
};


pub const BBU: LanguageCode = LanguageCode {
    name: "Kulung (Nigeria)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bbu",
    individual_languages: &[
    ],
};


pub const BBV: LanguageCode = LanguageCode {
    name: "Karnai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bbv",
    individual_languages: &[
    ],
};


pub const BBW: LanguageCode = LanguageCode {
    name: "Baba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bbw",
    individual_languages: &[
    ],
};


pub const BBX: LanguageCode = LanguageCode {
    name: "Bubia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bbx",
    individual_languages: &[
    ],
};


pub const BBY: LanguageCode = LanguageCode {
    name: "Befang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bby",
    individual_languages: &[
    ],
};


pub const BCA: LanguageCode = LanguageCode {
    name: "Central Bai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bca",
    individual_languages: &[
    ],
};


pub const BCB: LanguageCode = LanguageCode {
    name: "Bainouk-Samik",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bcb",
    individual_languages: &[
    ],
};


pub const BCC: LanguageCode = LanguageCode {
    name: "Southern Balochi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bcc",
    individual_languages: &[
    ],
};


pub const BCD: LanguageCode = LanguageCode {
    name: "North Babar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bcd",
    individual_languages: &[
    ],
};


pub const BCE: LanguageCode = LanguageCode {
    name: "Bamenyam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bce",
    individual_languages: &[
    ],
};


pub const BCF: LanguageCode = LanguageCode {
    name: "Bamu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bcf",
    individual_languages: &[
    ],
};


pub const BCG: LanguageCode = LanguageCode {
    name: "Baga Pokur",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bcg",
    individual_languages: &[
    ],
};


pub const BCH: LanguageCode = LanguageCode {
    name: "Bariai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bch",
    individual_languages: &[
    ],
};


pub const BCI: LanguageCode = LanguageCode {
    name: "Baoulé",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bci",
    individual_languages: &[
    ],
};


pub const BCJ: LanguageCode = LanguageCode {
    name: "Bardi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bcj",
    individual_languages: &[
    ],
};


pub const BCK: LanguageCode = LanguageCode {
    name: "Bunuba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bck",
    individual_languages: &[
    ],
};


pub const BCL: LanguageCode = LanguageCode {
    name: "Central Bikol",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bcl",
    individual_languages: &[
    ],
};


pub const BCM: LanguageCode = LanguageCode {
    name: "Bannoni",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bcm",
    individual_languages: &[
    ],
};


pub const BCN: LanguageCode = LanguageCode {
    name: "Bali (Nigeria)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bcn",
    individual_languages: &[
    ],
};


pub const BCO: LanguageCode = LanguageCode {
    name: "Kaluli",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bco",
    individual_languages: &[
    ],
};


pub const BCP: LanguageCode = LanguageCode {
    name: "Bali (Democratic Republic of Congo)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bcp",
    individual_languages: &[
    ],
};


pub const BCQ: LanguageCode = LanguageCode {
    name: "Bench",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bcq",
    individual_languages: &[
    ],
};


pub const BCR: LanguageCode = LanguageCode {
    name: "Babine",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bcr",
    individual_languages: &[
    ],
};


pub const BCS: LanguageCode = LanguageCode {
    name: "Kohumono",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bcs",
    individual_languages: &[
    ],
};


pub const BCT: LanguageCode = LanguageCode {
    name: "Bendi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bct",
    individual_languages: &[
    ],
};


pub const BCU: LanguageCode = LanguageCode {
    name: "Awad Bing",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bcu",
    individual_languages: &[
    ],
};


pub const BCV: LanguageCode = LanguageCode {
    name: "Shoo-Minda-Nye",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bcv",
    individual_languages: &[
    ],
};


pub const BCW: LanguageCode = LanguageCode {
    name: "Bana",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bcw",
    individual_languages: &[
    ],
};


pub const BCY: LanguageCode = LanguageCode {
    name: "Bacama",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bcy",
    individual_languages: &[
    ],
};


pub const BCZ: LanguageCode = LanguageCode {
    name: "Bainouk-Gunyaamolo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bcz",
    individual_languages: &[
    ],
};


pub const BDA: LanguageCode = LanguageCode {
    name: "Bayot",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bda",
    individual_languages: &[
    ],
};


pub const BDB: LanguageCode = LanguageCode {
    name: "Basap",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bdb",
    individual_languages: &[
    ],
};


pub const BDC: LanguageCode = LanguageCode {
    name: "Emberá-Baudó",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bdc",
    individual_languages: &[
    ],
};


pub const BDD: LanguageCode = LanguageCode {
    name: "Bunama",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bdd",
    individual_languages: &[
    ],
};


pub const BDE: LanguageCode = LanguageCode {
    name: "Bade",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bde",
    individual_languages: &[
    ],
};


pub const BDF: LanguageCode = LanguageCode {
    name: "Biage",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bdf",
    individual_languages: &[
    ],
};


pub const BDG: LanguageCode = LanguageCode {
    name: "Bonggi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bdg",
    individual_languages: &[
    ],
};


pub const BDH: LanguageCode = LanguageCode {
    name: "Baka (South Sudan)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bdh",
    individual_languages: &[
    ],
};


pub const BDI: LanguageCode = LanguageCode {
    name: "Burun",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bdi",
    individual_languages: &[
    ],
};


pub const BDJ: LanguageCode = LanguageCode {
    name: "Bai (South Sudan)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bdj",
    individual_languages: &[
    ],
};


pub const BDK: LanguageCode = LanguageCode {
    name: "Budukh",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bdk",
    individual_languages: &[
    ],
};


pub const BDL: LanguageCode = LanguageCode {
    name: "Indonesian Bajau",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bdl",
    individual_languages: &[
    ],
};


pub const BDM: LanguageCode = LanguageCode {
    name: "Buduma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bdm",
    individual_languages: &[
    ],
};


pub const BDN: LanguageCode = LanguageCode {
    name: "Baldemu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bdn",
    individual_languages: &[
    ],
};


pub const BDO: LanguageCode = LanguageCode {
    name: "Morom",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bdo",
    individual_languages: &[
    ],
};


pub const BDP: LanguageCode = LanguageCode {
    name: "Bende",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bdp",
    individual_languages: &[
    ],
};


pub const BDQ: LanguageCode = LanguageCode {
    name: "Bahnar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bdq",
    individual_languages: &[
    ],
};


pub const BDR: LanguageCode = LanguageCode {
    name: "West Coast Bajau",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bdr",
    individual_languages: &[
    ],
};


pub const BDS: LanguageCode = LanguageCode {
    name: "Burunge",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bds",
    individual_languages: &[
    ],
};


pub const BDT: LanguageCode = LanguageCode {
    name: "Bokoto",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bdt",
    individual_languages: &[
    ],
};


pub const BDU: LanguageCode = LanguageCode {
    name: "Oroko",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bdu",
    individual_languages: &[
    ],
};


pub const BDV: LanguageCode = LanguageCode {
    name: "Bodo Parja",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bdv",
    individual_languages: &[
    ],
};


pub const BDW: LanguageCode = LanguageCode {
    name: "Baham",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bdw",
    individual_languages: &[
    ],
};


pub const BDX: LanguageCode = LanguageCode {
    name: "Budong-Budong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bdx",
    individual_languages: &[
    ],
};


pub const BDY: LanguageCode = LanguageCode {
    name: "Bandjalang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bdy",
    individual_languages: &[
    ],
};


pub const BDZ: LanguageCode = LanguageCode {
    name: "Badeshi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bdz",
    individual_languages: &[
    ],
};


pub const BEA: LanguageCode = LanguageCode {
    name: "Beaver",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bea",
    individual_languages: &[
    ],
};


pub const BEB: LanguageCode = LanguageCode {
    name: "Bebele",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "beb",
    individual_languages: &[
    ],
};


pub const BEC: LanguageCode = LanguageCode {
    name: "Iceve-Maci",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bec",
    individual_languages: &[
    ],
};


pub const BED: LanguageCode = LanguageCode {
    name: "Bedoanas",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bed",
    individual_languages: &[
    ],
};


pub const BEE: LanguageCode = LanguageCode {
    name: "Byangsi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bee",
    individual_languages: &[
    ],
};


pub const BEF: LanguageCode = LanguageCode {
    name: "Benabena",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bef",
    individual_languages: &[
    ],
};


pub const BEG: LanguageCode = LanguageCode {
    name: "Belait",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "beg",
    individual_languages: &[
    ],
};


pub const BEH: LanguageCode = LanguageCode {
    name: "Biali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "beh",
    individual_languages: &[
    ],
};


pub const BEI: LanguageCode = LanguageCode {
    name: "Bekati'",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bei",
    individual_languages: &[
    ],
};


pub const BEK: LanguageCode = LanguageCode {
    name: "Bebeli",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bek",
    individual_languages: &[
    ],
};


pub const BEO: LanguageCode = LanguageCode {
    name: "Beami",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "beo",
    individual_languages: &[
    ],
};


pub const BEP: LanguageCode = LanguageCode {
    name: "Besoa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bep",
    individual_languages: &[
    ],
};


pub const BEQ: LanguageCode = LanguageCode {
    name: "Beembe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "beq",
    individual_languages: &[
    ],
};


pub const BES: LanguageCode = LanguageCode {
    name: "Besme",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bes",
    individual_languages: &[
    ],
};


pub const BET: LanguageCode = LanguageCode {
    name: "Guiberoua Béte",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bet",
    individual_languages: &[
    ],
};


pub const BEU: LanguageCode = LanguageCode {
    name: "Blagar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "beu",
    individual_languages: &[
    ],
};


pub const BEV: LanguageCode = LanguageCode {
    name: "Daloa Bété",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bev",
    individual_languages: &[
    ],
};


pub const BEW: LanguageCode = LanguageCode {
    name: "Betawi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bew",
    individual_languages: &[
    ],
};


pub const BEX: LanguageCode = LanguageCode {
    name: "Jur Modo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bex",
    individual_languages: &[
    ],
};


pub const BEY: LanguageCode = LanguageCode {
    name: "Beli (Papua New Guinea)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bey",
    individual_languages: &[
    ],
};


pub const BEZ: LanguageCode = LanguageCode {
    name: "Bena (Tanzania)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bez",
    individual_languages: &[
    ],
};


pub const BFA: LanguageCode = LanguageCode {
    name: "Bari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bfa",
    individual_languages: &[
    ],
};


pub const BFB: LanguageCode = LanguageCode {
    name: "Pauri Bareli",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bfb",
    individual_languages: &[
    ],
};


pub const BFC: LanguageCode = LanguageCode {
    name: "Panyi Bai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bfc",
    individual_languages: &[
    ],
};


pub const BFD: LanguageCode = LanguageCode {
    name: "Bafut",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bfd",
    individual_languages: &[
    ],
};


pub const BFE: LanguageCode = LanguageCode {
    name: "Betaf",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bfe",
    individual_languages: &[
    ],
};


pub const BFF: LanguageCode = LanguageCode {
    name: "Bofi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bff",
    individual_languages: &[
    ],
};


pub const BFG: LanguageCode = LanguageCode {
    name: "Busang Kayan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bfg",
    individual_languages: &[
    ],
};


pub const BFH: LanguageCode = LanguageCode {
    name: "Blafe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bfh",
    individual_languages: &[
    ],
};


pub const BFI: LanguageCode = LanguageCode {
    name: "British Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bfi",
    individual_languages: &[
    ],
};


pub const BFJ: LanguageCode = LanguageCode {
    name: "Bafanji",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bfj",
    individual_languages: &[
    ],
};


pub const BFK: LanguageCode = LanguageCode {
    name: "Ban Khor Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bfk",
    individual_languages: &[
    ],
};


pub const BFL: LanguageCode = LanguageCode {
    name: "Banda-Ndélé",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bfl",
    individual_languages: &[
    ],
};


pub const BFM: LanguageCode = LanguageCode {
    name: "Mmen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bfm",
    individual_languages: &[
    ],
};


pub const BFN: LanguageCode = LanguageCode {
    name: "Bunak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bfn",
    individual_languages: &[
    ],
};


pub const BFO: LanguageCode = LanguageCode {
    name: "Malba Birifor",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bfo",
    individual_languages: &[
    ],
};


pub const BFP: LanguageCode = LanguageCode {
    name: "Beba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bfp",
    individual_languages: &[
    ],
};


pub const BFQ: LanguageCode = LanguageCode {
    name: "Badaga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bfq",
    individual_languages: &[
    ],
};


pub const BFR: LanguageCode = LanguageCode {
    name: "Bazigar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bfr",
    individual_languages: &[
    ],
};


pub const BFS: LanguageCode = LanguageCode {
    name: "Southern Bai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bfs",
    individual_languages: &[
    ],
};


pub const BFT: LanguageCode = LanguageCode {
    name: "Balti",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bft",
    individual_languages: &[
    ],
};


pub const BFU: LanguageCode = LanguageCode {
    name: "Gahri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bfu",
    individual_languages: &[
    ],
};


pub const BFW: LanguageCode = LanguageCode {
    name: "Bondo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bfw",
    individual_languages: &[
    ],
};


pub const BFX: LanguageCode = LanguageCode {
    name: "Bantayanon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bfx",
    individual_languages: &[
    ],
};


pub const BFY: LanguageCode = LanguageCode {
    name: "Bagheli",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bfy",
    individual_languages: &[
    ],
};


pub const BFZ: LanguageCode = LanguageCode {
    name: "Mahasu Pahari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bfz",
    individual_languages: &[
    ],
};


pub const BGA: LanguageCode = LanguageCode {
    name: "Gwamhi-Wuri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bga",
    individual_languages: &[
    ],
};


pub const BGB: LanguageCode = LanguageCode {
    name: "Bobongko",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bgb",
    individual_languages: &[
    ],
};


pub const BGC: LanguageCode = LanguageCode {
    name: "Haryanvi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bgc",
    individual_languages: &[
    ],
};


pub const BGD: LanguageCode = LanguageCode {
    name: "Rathwi Bareli",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bgd",
    individual_languages: &[
    ],
};


pub const BGE: LanguageCode = LanguageCode {
    name: "Bauria",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bge",
    individual_languages: &[
    ],
};


pub const BGF: LanguageCode = LanguageCode {
    name: "Bangandu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bgf",
    individual_languages: &[
    ],
};


pub const BGG: LanguageCode = LanguageCode {
    name: "Bugun",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bgg",
    individual_languages: &[
    ],
};


pub const BGI: LanguageCode = LanguageCode {
    name: "Giangan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bgi",
    individual_languages: &[
    ],
};


pub const BGJ: LanguageCode = LanguageCode {
    name: "Bangolan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bgj",
    individual_languages: &[
    ],
};


pub const BGK: LanguageCode = LanguageCode {
    name: "Bit",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bgk",
    individual_languages: &[
    ],
};


pub const BGL: LanguageCode = LanguageCode {
    name: "Bo (Laos)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bgl",
    individual_languages: &[
    ],
};


pub const BGN: LanguageCode = LanguageCode {
    name: "Western Balochi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bgn",
    individual_languages: &[
    ],
};


pub const BGO: LanguageCode = LanguageCode {
    name: "Baga Koga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bgo",
    individual_languages: &[
    ],
};


pub const BGP: LanguageCode = LanguageCode {
    name: "Eastern Balochi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bgp",
    individual_languages: &[
    ],
};


pub const BGQ: LanguageCode = LanguageCode {
    name: "Bagri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bgq",
    individual_languages: &[
    ],
};


pub const BGR: LanguageCode = LanguageCode {
    name: "Bawm Chin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bgr",
    individual_languages: &[
    ],
};


pub const BGS: LanguageCode = LanguageCode {
    name: "Tagabawa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bgs",
    individual_languages: &[
    ],
};


pub const BGT: LanguageCode = LanguageCode {
    name: "Bughotu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bgt",
    individual_languages: &[
    ],
};


pub const BGU: LanguageCode = LanguageCode {
    name: "Mbongno",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bgu",
    individual_languages: &[
    ],
};


pub const BGV: LanguageCode = LanguageCode {
    name: "Warkay-Bipim",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bgv",
    individual_languages: &[
    ],
};


pub const BGW: LanguageCode = LanguageCode {
    name: "Bhatri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bgw",
    individual_languages: &[
    ],
};


pub const BGX: LanguageCode = LanguageCode {
    name: "Balkan Gagauz Turkish",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bgx",
    individual_languages: &[
    ],
};


pub const BGY: LanguageCode = LanguageCode {
    name: "Benggoi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bgy",
    individual_languages: &[
    ],
};


pub const BGZ: LanguageCode = LanguageCode {
    name: "Banggai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bgz",
    individual_languages: &[
    ],
};


pub const BHA: LanguageCode = LanguageCode {
    name: "Bharia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bha",
    individual_languages: &[
    ],
};


pub const BHB: LanguageCode = LanguageCode {
    name: "Bhili",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bhb",
    individual_languages: &[
    ],
};


pub const BHC: LanguageCode = LanguageCode {
    name: "Biga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bhc",
    individual_languages: &[
    ],
};


pub const BHD: LanguageCode = LanguageCode {
    name: "Bhadrawahi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bhd",
    individual_languages: &[
    ],
};


pub const BHE: LanguageCode = LanguageCode {
    name: "Bhaya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bhe",
    individual_languages: &[
    ],
};


pub const BHF: LanguageCode = LanguageCode {
    name: "Odiai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bhf",
    individual_languages: &[
    ],
};


pub const BHG: LanguageCode = LanguageCode {
    name: "Binandere",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bhg",
    individual_languages: &[
    ],
};


pub const BHH: LanguageCode = LanguageCode {
    name: "Bukharic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bhh",
    individual_languages: &[
    ],
};


pub const BHI: LanguageCode = LanguageCode {
    name: "Bhilali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bhi",
    individual_languages: &[
    ],
};


pub const BHJ: LanguageCode = LanguageCode {
    name: "Bahing",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bhj",
    individual_languages: &[
    ],
};


pub const BHL: LanguageCode = LanguageCode {
    name: "Bimin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bhl",
    individual_languages: &[
    ],
};


pub const BHM: LanguageCode = LanguageCode {
    name: "Bathari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bhm",
    individual_languages: &[
    ],
};


pub const BHN: LanguageCode = LanguageCode {
    name: "Bohtan Neo-Aramaic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bhn",
    individual_languages: &[
    ],
};


pub const BHP: LanguageCode = LanguageCode {
    name: "Bima",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bhp",
    individual_languages: &[
    ],
};


pub const BHQ: LanguageCode = LanguageCode {
    name: "Tukang Besi South",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bhq",
    individual_languages: &[
    ],
};


pub const BHR: LanguageCode = LanguageCode {
    name: "Bara Malagasy",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bhr",
    individual_languages: &[
    ],
};


pub const BHS: LanguageCode = LanguageCode {
    name: "Buwal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bhs",
    individual_languages: &[
    ],
};


pub const BHT: LanguageCode = LanguageCode {
    name: "Bhattiyali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bht",
    individual_languages: &[
    ],
};


pub const BHU: LanguageCode = LanguageCode {
    name: "Bhunjia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bhu",
    individual_languages: &[
    ],
};


pub const BHV: LanguageCode = LanguageCode {
    name: "Bahau",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bhv",
    individual_languages: &[
    ],
};


pub const BHW: LanguageCode = LanguageCode {
    name: "Biak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bhw",
    individual_languages: &[
    ],
};


pub const BHX: LanguageCode = LanguageCode {
    name: "Bhalay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bhx",
    individual_languages: &[
    ],
};


pub const BHY: LanguageCode = LanguageCode {
    name: "Bhele",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bhy",
    individual_languages: &[
    ],
};


pub const BHZ: LanguageCode = LanguageCode {
    name: "Bada (Indonesia)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bhz",
    individual_languages: &[
    ],
};


pub const BIA: LanguageCode = LanguageCode {
    name: "Badimaya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bia",
    individual_languages: &[
    ],
};


pub const BIB: LanguageCode = LanguageCode {
    name: "Bissa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bib",
    individual_languages: &[
    ],
};


pub const BID: LanguageCode = LanguageCode {
    name: "Bidiyo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bid",
    individual_languages: &[
    ],
};


pub const BIE: LanguageCode = LanguageCode {
    name: "Bepour",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bie",
    individual_languages: &[
    ],
};


pub const BIF: LanguageCode = LanguageCode {
    name: "Biafada",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bif",
    individual_languages: &[
    ],
};


pub const BIG: LanguageCode = LanguageCode {
    name: "Biangai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "big",
    individual_languages: &[
    ],
};


pub const BIL: LanguageCode = LanguageCode {
    name: "Bile",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bil",
    individual_languages: &[
    ],
};


pub const BIM: LanguageCode = LanguageCode {
    name: "Bimoba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bim",
    individual_languages: &[
    ],
};


pub const BIO: LanguageCode = LanguageCode {
    name: "Nai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bio",
    individual_languages: &[
    ],
};


pub const BIP: LanguageCode = LanguageCode {
    name: "Bila",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bip",
    individual_languages: &[
    ],
};


pub const BIQ: LanguageCode = LanguageCode {
    name: "Bipi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "biq",
    individual_languages: &[
    ],
};


pub const BIR: LanguageCode = LanguageCode {
    name: "Bisorio",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bir",
    individual_languages: &[
    ],
};


pub const BIT: LanguageCode = LanguageCode {
    name: "Berinomo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bit",
    individual_languages: &[
    ],
};


pub const BIU: LanguageCode = LanguageCode {
    name: "Biete",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "biu",
    individual_languages: &[
    ],
};


pub const BIV: LanguageCode = LanguageCode {
    name: "Southern Birifor",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "biv",
    individual_languages: &[
    ],
};


pub const BIW: LanguageCode = LanguageCode {
    name: "Kol (Cameroon)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "biw",
    individual_languages: &[
    ],
};


pub const BIX: LanguageCode = LanguageCode {
    name: "Bijori",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bix",
    individual_languages: &[
    ],
};


pub const BIY: LanguageCode = LanguageCode {
    name: "Birhor",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "biy",
    individual_languages: &[
    ],
};


pub const BIZ: LanguageCode = LanguageCode {
    name: "Baloi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "biz",
    individual_languages: &[
    ],
};


pub const BJA: LanguageCode = LanguageCode {
    name: "Budza",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bja",
    individual_languages: &[
    ],
};


pub const BJB: LanguageCode = LanguageCode {
    name: "Banggarla",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bjb",
    individual_languages: &[
    ],
};


pub const BJC: LanguageCode = LanguageCode {
    name: "Bariji",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bjc",
    individual_languages: &[
    ],
};


pub const BJE: LanguageCode = LanguageCode {
    name: "Biao-Jiao Mien",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bje",
    individual_languages: &[
    ],
};


pub const BJF: LanguageCode = LanguageCode {
    name: "Barzani Jewish Neo-Aramaic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bjf",
    individual_languages: &[
    ],
};


pub const BJG: LanguageCode = LanguageCode {
    name: "Bidyogo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bjg",
    individual_languages: &[
    ],
};


pub const BJH: LanguageCode = LanguageCode {
    name: "Bahinemo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bjh",
    individual_languages: &[
    ],
};


pub const BJI: LanguageCode = LanguageCode {
    name: "Burji",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bji",
    individual_languages: &[
    ],
};


pub const BJJ: LanguageCode = LanguageCode {
    name: "Kanauji",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bjj",
    individual_languages: &[
    ],
};


pub const BJK: LanguageCode = LanguageCode {
    name: "Barok",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bjk",
    individual_languages: &[
    ],
};


pub const BJL: LanguageCode = LanguageCode {
    name: "Bulu (Papua New Guinea)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bjl",
    individual_languages: &[
    ],
};


pub const BJM: LanguageCode = LanguageCode {
    name: "Bajelani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bjm",
    individual_languages: &[
    ],
};


pub const BJN: LanguageCode = LanguageCode {
    name: "Banjar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bjn",
    individual_languages: &[
    ],
};


pub const BJO: LanguageCode = LanguageCode {
    name: "Mid-Southern Banda",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bjo",
    individual_languages: &[
    ],
};


pub const BJP: LanguageCode = LanguageCode {
    name: "Fanamaket",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bjp",
    individual_languages: &[
    ],
};


pub const BJR: LanguageCode = LanguageCode {
    name: "Binumarien",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bjr",
    individual_languages: &[
    ],
};


pub const BJS: LanguageCode = LanguageCode {
    name: "Bajan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bjs",
    individual_languages: &[
    ],
};


pub const BJT: LanguageCode = LanguageCode {
    name: "Balanta-Ganja",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bjt",
    individual_languages: &[
    ],
};


pub const BJU: LanguageCode = LanguageCode {
    name: "Busuu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bju",
    individual_languages: &[
    ],
};


pub const BJV: LanguageCode = LanguageCode {
    name: "Bedjond",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bjv",
    individual_languages: &[
    ],
};


pub const BJW: LanguageCode = LanguageCode {
    name: "Bakwé",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bjw",
    individual_languages: &[
    ],
};


pub const BJX: LanguageCode = LanguageCode {
    name: "Banao Itneg",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bjx",
    individual_languages: &[
    ],
};


pub const BJY: LanguageCode = LanguageCode {
    name: "Bayali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bjy",
    individual_languages: &[
    ],
};


pub const BJZ: LanguageCode = LanguageCode {
    name: "Baruga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bjz",
    individual_languages: &[
    ],
};


pub const BKA: LanguageCode = LanguageCode {
    name: "Kyak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bka",
    individual_languages: &[
    ],
};


pub const BKC: LanguageCode = LanguageCode {
    name: "Baka (Cameroon)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bkc",
    individual_languages: &[
    ],
};


pub const BKD: LanguageCode = LanguageCode {
    name: "Binukid",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bkd",
    individual_languages: &[
    ],
};


pub const BKF: LanguageCode = LanguageCode {
    name: "Beeke",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bkf",
    individual_languages: &[
    ],
};


pub const BKG: LanguageCode = LanguageCode {
    name: "Buraka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bkg",
    individual_languages: &[
    ],
};


pub const BKH: LanguageCode = LanguageCode {
    name: "Bakoko",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bkh",
    individual_languages: &[
    ],
};


pub const BKI: LanguageCode = LanguageCode {
    name: "Baki",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bki",
    individual_languages: &[
    ],
};


pub const BKJ: LanguageCode = LanguageCode {
    name: "Pande",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bkj",
    individual_languages: &[
    ],
};


pub const BKK: LanguageCode = LanguageCode {
    name: "Brokskat",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bkk",
    individual_languages: &[
    ],
};


pub const BKL: LanguageCode = LanguageCode {
    name: "Berik",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bkl",
    individual_languages: &[
    ],
};


pub const BKM: LanguageCode = LanguageCode {
    name: "Kom (Cameroon)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bkm",
    individual_languages: &[
    ],
};


pub const BKN: LanguageCode = LanguageCode {
    name: "Bukitan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bkn",
    individual_languages: &[
    ],
};


pub const BKO: LanguageCode = LanguageCode {
    name: "Kwa'",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bko",
    individual_languages: &[
    ],
};


pub const BKP: LanguageCode = LanguageCode {
    name: "Boko (Democratic Republic of Congo)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bkp",
    individual_languages: &[
    ],
};


pub const BKQ: LanguageCode = LanguageCode {
    name: "Bakairí",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bkq",
    individual_languages: &[
    ],
};


pub const BKR: LanguageCode = LanguageCode {
    name: "Bakumpai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bkr",
    individual_languages: &[
    ],
};


pub const BKS: LanguageCode = LanguageCode {
    name: "Northern Sorsoganon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bks",
    individual_languages: &[
    ],
};


pub const BKT: LanguageCode = LanguageCode {
    name: "Boloki",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bkt",
    individual_languages: &[
    ],
};


pub const BKU: LanguageCode = LanguageCode {
    name: "Buhid",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bku",
    individual_languages: &[
    ],
};


pub const BKV: LanguageCode = LanguageCode {
    name: "Bekwarra",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bkv",
    individual_languages: &[
    ],
};


pub const BKW: LanguageCode = LanguageCode {
    name: "Bekwel",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bkw",
    individual_languages: &[
    ],
};


pub const BKX: LanguageCode = LanguageCode {
    name: "Baikeno",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bkx",
    individual_languages: &[
    ],
};


pub const BKY: LanguageCode = LanguageCode {
    name: "Bokyi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bky",
    individual_languages: &[
    ],
};


pub const BKZ: LanguageCode = LanguageCode {
    name: "Bungku",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bkz",
    individual_languages: &[
    ],
};


pub const BLB: LanguageCode = LanguageCode {
    name: "Bilua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "blb",
    individual_languages: &[
    ],
};


pub const BLC: LanguageCode = LanguageCode {
    name: "Bella Coola",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "blc",
    individual_languages: &[
    ],
};


pub const BLD: LanguageCode = LanguageCode {
    name: "Bolango",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bld",
    individual_languages: &[
    ],
};


pub const BLE: LanguageCode = LanguageCode {
    name: "Balanta-Kentohe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ble",
    individual_languages: &[
    ],
};


pub const BLF: LanguageCode = LanguageCode {
    name: "Buol",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "blf",
    individual_languages: &[
    ],
};


pub const BLH: LanguageCode = LanguageCode {
    name: "Kuwaa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "blh",
    individual_languages: &[
    ],
};


pub const BLI: LanguageCode = LanguageCode {
    name: "Bolia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bli",
    individual_languages: &[
    ],
};


pub const BLJ: LanguageCode = LanguageCode {
    name: "Bolongan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "blj",
    individual_languages: &[
    ],
};


pub const BLK: LanguageCode = LanguageCode {
    name: "Pa'o Karen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "blk",
    individual_languages: &[
    ],
};


pub const BLL: LanguageCode = LanguageCode {
    name: "Biloxi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bll",
    individual_languages: &[
    ],
};


pub const BLM: LanguageCode = LanguageCode {
    name: "Beli (South Sudan)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "blm",
    individual_languages: &[
    ],
};


pub const BLN: LanguageCode = LanguageCode {
    name: "Southern Catanduanes Bikol",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bln",
    individual_languages: &[
    ],
};


pub const BLO: LanguageCode = LanguageCode {
    name: "Anii",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "blo",
    individual_languages: &[
    ],
};


pub const BLP: LanguageCode = LanguageCode {
    name: "Blablanga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "blp",
    individual_languages: &[
    ],
};


pub const BLQ: LanguageCode = LanguageCode {
    name: "Baluan-Pam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "blq",
    individual_languages: &[
    ],
};


pub const BLR: LanguageCode = LanguageCode {
    name: "Blang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "blr",
    individual_languages: &[
    ],
};


pub const BLS: LanguageCode = LanguageCode {
    name: "Balaesang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bls",
    individual_languages: &[
    ],
};


pub const BLT: LanguageCode = LanguageCode {
    name: "Tai Dam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "blt",
    individual_languages: &[
    ],
};


pub const BLV: LanguageCode = LanguageCode {
    name: "Kibala",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "blv",
    individual_languages: &[
    ],
};


pub const BLW: LanguageCode = LanguageCode {
    name: "Balangao",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "blw",
    individual_languages: &[
    ],
};


pub const BLX: LanguageCode = LanguageCode {
    name: "Mag-Indi Ayta",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "blx",
    individual_languages: &[
    ],
};


pub const BLY: LanguageCode = LanguageCode {
    name: "Notre",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bly",
    individual_languages: &[
    ],
};


pub const BLZ: LanguageCode = LanguageCode {
    name: "Balantak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "blz",
    individual_languages: &[
    ],
};


pub const BMA: LanguageCode = LanguageCode {
    name: "Lame",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bma",
    individual_languages: &[
    ],
};


pub const BMB: LanguageCode = LanguageCode {
    name: "Bembe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bmb",
    individual_languages: &[
    ],
};


pub const BMC: LanguageCode = LanguageCode {
    name: "Biem",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bmc",
    individual_languages: &[
    ],
};


pub const BMD: LanguageCode = LanguageCode {
    name: "Baga Manduri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bmd",
    individual_languages: &[
    ],
};


pub const BME: LanguageCode = LanguageCode {
    name: "Limassa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bme",
    individual_languages: &[
    ],
};


pub const BMF: LanguageCode = LanguageCode {
    name: "Bom-Kim",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bmf",
    individual_languages: &[
    ],
};


pub const BMG: LanguageCode = LanguageCode {
    name: "Bamwe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bmg",
    individual_languages: &[
    ],
};


pub const BMH: LanguageCode = LanguageCode {
    name: "Kein",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bmh",
    individual_languages: &[
    ],
};


pub const BMI: LanguageCode = LanguageCode {
    name: "Bagirmi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bmi",
    individual_languages: &[
    ],
};


pub const BMJ: LanguageCode = LanguageCode {
    name: "Bote-Majhi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bmj",
    individual_languages: &[
    ],
};


pub const BMK: LanguageCode = LanguageCode {
    name: "Ghayavi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bmk",
    individual_languages: &[
    ],
};


pub const BML: LanguageCode = LanguageCode {
    name: "Bomboli",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bml",
    individual_languages: &[
    ],
};


pub const BMM: LanguageCode = LanguageCode {
    name: "Northern Betsimisaraka Malagasy",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bmm",
    individual_languages: &[
    ],
};


pub const BMN: LanguageCode = LanguageCode {
    name: "Bina (Papua New Guinea)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bmn",
    individual_languages: &[
    ],
};


pub const BMO: LanguageCode = LanguageCode {
    name: "Bambalang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bmo",
    individual_languages: &[
    ],
};


pub const BMP: LanguageCode = LanguageCode {
    name: "Bulgebi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bmp",
    individual_languages: &[
    ],
};


pub const BMQ: LanguageCode = LanguageCode {
    name: "Bomu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bmq",
    individual_languages: &[
    ],
};


pub const BMR: LanguageCode = LanguageCode {
    name: "Muinane",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bmr",
    individual_languages: &[
    ],
};


pub const BMS: LanguageCode = LanguageCode {
    name: "Bilma Kanuri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bms",
    individual_languages: &[
    ],
};


pub const BMT: LanguageCode = LanguageCode {
    name: "Biao Mon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bmt",
    individual_languages: &[
    ],
};


pub const BMU: LanguageCode = LanguageCode {
    name: "Somba-Siawari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bmu",
    individual_languages: &[
    ],
};


pub const BMV: LanguageCode = LanguageCode {
    name: "Bum",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bmv",
    individual_languages: &[
    ],
};


pub const BMW: LanguageCode = LanguageCode {
    name: "Bomwali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bmw",
    individual_languages: &[
    ],
};


pub const BMX: LanguageCode = LanguageCode {
    name: "Baimak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bmx",
    individual_languages: &[
    ],
};


pub const BMZ: LanguageCode = LanguageCode {
    name: "Baramu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bmz",
    individual_languages: &[
    ],
};


pub const BNA: LanguageCode = LanguageCode {
    name: "Bonerate",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bna",
    individual_languages: &[
    ],
};


pub const BNB: LanguageCode = LanguageCode {
    name: "Bookan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bnb",
    individual_languages: &[
    ],
};


pub const BNC: LanguageCode = LanguageCode {
    name: "Bontok",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bnc",
    individual_languages: &[
        IndividualLanguages {
            name: "Eastern Bontok",
            code: "ebk",
        },
        IndividualLanguages {
            name: "Central Bontok",
            code: "lbk",
        },
        IndividualLanguages {
            name: "Southern Bontok",
            code: "obk",
        },
        IndividualLanguages {
            name: "Northern Bontok",
            code: "rbk",
        },
        IndividualLanguages {
            name: "Southwestern Bontok",
            code: "vbk",
        },
    ],
};


pub const BND: LanguageCode = LanguageCode {
    name: "Banda (Indonesia)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bnd",
    individual_languages: &[
    ],
};


pub const BNE: LanguageCode = LanguageCode {
    name: "Bintauna",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bne",
    individual_languages: &[
    ],
};


pub const BNF: LanguageCode = LanguageCode {
    name: "Masiwang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bnf",
    individual_languages: &[
    ],
};


pub const BNG: LanguageCode = LanguageCode {
    name: "Benga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bng",
    individual_languages: &[
    ],
};


pub const BNI: LanguageCode = LanguageCode {
    name: "Bangi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bni",
    individual_languages: &[
    ],
};


pub const BNJ: LanguageCode = LanguageCode {
    name: "Eastern Tawbuid",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bnj",
    individual_languages: &[
    ],
};


pub const BNK: LanguageCode = LanguageCode {
    name: "Bierebo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bnk",
    individual_languages: &[
    ],
};


pub const BNL: LanguageCode = LanguageCode {
    name: "Boon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bnl",
    individual_languages: &[
    ],
};


pub const BNM: LanguageCode = LanguageCode {
    name: "Batanga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bnm",
    individual_languages: &[
    ],
};


pub const BNN: LanguageCode = LanguageCode {
    name: "Bunun",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bnn",
    individual_languages: &[
    ],
};


pub const BNO: LanguageCode = LanguageCode {
    name: "Bantoanon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bno",
    individual_languages: &[
    ],
};


pub const BNP: LanguageCode = LanguageCode {
    name: "Bola",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bnp",
    individual_languages: &[
    ],
};


pub const BNQ: LanguageCode = LanguageCode {
    name: "Bantik",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bnq",
    individual_languages: &[
    ],
};


pub const BNR: LanguageCode = LanguageCode {
    name: "Butmas-Tur",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bnr",
    individual_languages: &[
    ],
};


pub const BNS: LanguageCode = LanguageCode {
    name: "Bundeli",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bns",
    individual_languages: &[
    ],
};


pub const BNU: LanguageCode = LanguageCode {
    name: "Bentong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bnu",
    individual_languages: &[
    ],
};


pub const BNV: LanguageCode = LanguageCode {
    name: "Bonerif",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bnv",
    individual_languages: &[
    ],
};


pub const BNW: LanguageCode = LanguageCode {
    name: "Bisis",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bnw",
    individual_languages: &[
    ],
};


pub const BNX: LanguageCode = LanguageCode {
    name: "Bangubangu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bnx",
    individual_languages: &[
    ],
};


pub const BNY: LanguageCode = LanguageCode {
    name: "Bintulu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bny",
    individual_languages: &[
    ],
};


pub const BNZ: LanguageCode = LanguageCode {
    name: "Beezen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bnz",
    individual_languages: &[
    ],
};


pub const BOA: LanguageCode = LanguageCode {
    name: "Bora",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "boa",
    individual_languages: &[
    ],
};


pub const BOB: LanguageCode = LanguageCode {
    name: "Aweer",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bob",
    individual_languages: &[
    ],
};


pub const BOE: LanguageCode = LanguageCode {
    name: "Mundabli",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "boe",
    individual_languages: &[
    ],
};


pub const BOF: LanguageCode = LanguageCode {
    name: "Bolon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bof",
    individual_languages: &[
    ],
};


pub const BOG: LanguageCode = LanguageCode {
    name: "Bamako Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bog",
    individual_languages: &[
    ],
};


pub const BOH: LanguageCode = LanguageCode {
    name: "Boma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "boh",
    individual_languages: &[
    ],
};


pub const BOI: LanguageCode = LanguageCode {
    name: "Barbareño",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "boi",
    individual_languages: &[
    ],
};


pub const BOJ: LanguageCode = LanguageCode {
    name: "Anjam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "boj",
    individual_languages: &[
    ],
};


pub const BOK: LanguageCode = LanguageCode {
    name: "Bonjo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bok",
    individual_languages: &[
    ],
};


pub const BOL: LanguageCode = LanguageCode {
    name: "Bole",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bol",
    individual_languages: &[
    ],
};


pub const BOM: LanguageCode = LanguageCode {
    name: "Berom",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bom",
    individual_languages: &[
    ],
};


pub const BON: LanguageCode = LanguageCode {
    name: "Bine",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bon",
    individual_languages: &[
    ],
};


pub const BOO: LanguageCode = LanguageCode {
    name: "Tiemacèwè Bozo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "boo",
    individual_languages: &[
    ],
};


pub const BOP: LanguageCode = LanguageCode {
    name: "Bonkiman",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bop",
    individual_languages: &[
    ],
};


pub const BOQ: LanguageCode = LanguageCode {
    name: "Bogaya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "boq",
    individual_languages: &[
    ],
};


pub const BOR: LanguageCode = LanguageCode {
    name: "Borôro",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bor",
    individual_languages: &[
    ],
};


pub const BOT: LanguageCode = LanguageCode {
    name: "Bongo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bot",
    individual_languages: &[
    ],
};


pub const BOU: LanguageCode = LanguageCode {
    name: "Bondei",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bou",
    individual_languages: &[
    ],
};


pub const BOV: LanguageCode = LanguageCode {
    name: "Tuwuli",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bov",
    individual_languages: &[
    ],
};


pub const BOW: LanguageCode = LanguageCode {
    name: "Rema",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bow",
    individual_languages: &[
    ],
};


pub const BOX: LanguageCode = LanguageCode {
    name: "Buamu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "box",
    individual_languages: &[
    ],
};


pub const BOY: LanguageCode = LanguageCode {
    name: "Bodo (Central African Republic)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "boy",
    individual_languages: &[
    ],
};


pub const BOZ: LanguageCode = LanguageCode {
    name: "Tiéyaxo Bozo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "boz",
    individual_languages: &[
    ],
};


pub const BPA: LanguageCode = LanguageCode {
    name: "Daakaka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bpa",
    individual_languages: &[
    ],
};


pub const BPC: LanguageCode = LanguageCode {
    name: "Mbuk",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bpc",
    individual_languages: &[
    ],
};


pub const BPD: LanguageCode = LanguageCode {
    name: "Banda-Banda",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bpd",
    individual_languages: &[
    ],
};


pub const BPE: LanguageCode = LanguageCode {
    name: "Bauni",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bpe",
    individual_languages: &[
    ],
};


pub const BPG: LanguageCode = LanguageCode {
    name: "Bonggo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bpg",
    individual_languages: &[
    ],
};


pub const BPH: LanguageCode = LanguageCode {
    name: "Botlikh",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bph",
    individual_languages: &[
    ],
};


pub const BPI: LanguageCode = LanguageCode {
    name: "Bagupi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bpi",
    individual_languages: &[
    ],
};


pub const BPJ: LanguageCode = LanguageCode {
    name: "Binji",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bpj",
    individual_languages: &[
    ],
};


pub const BPK: LanguageCode = LanguageCode {
    name: "Orowe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bpk",
    individual_languages: &[
    ],
};


pub const BPL: LanguageCode = LanguageCode {
    name: "Broome Pearling Lugger Pidgin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bpl",
    individual_languages: &[
    ],
};


pub const BPM: LanguageCode = LanguageCode {
    name: "Biyom",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bpm",
    individual_languages: &[
    ],
};


pub const BPN: LanguageCode = LanguageCode {
    name: "Dzao Min",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bpn",
    individual_languages: &[
    ],
};


pub const BPO: LanguageCode = LanguageCode {
    name: "Anasi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bpo",
    individual_languages: &[
    ],
};


pub const BPP: LanguageCode = LanguageCode {
    name: "Kaure",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bpp",
    individual_languages: &[
    ],
};


pub const BPQ: LanguageCode = LanguageCode {
    name: "Banda Malay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bpq",
    individual_languages: &[
    ],
};


pub const BPR: LanguageCode = LanguageCode {
    name: "Koronadal Blaan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bpr",
    individual_languages: &[
    ],
};


pub const BPS: LanguageCode = LanguageCode {
    name: "Sarangani Blaan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bps",
    individual_languages: &[
    ],
};


pub const BPT: LanguageCode = LanguageCode {
    name: "Barrow Point",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bpt",
    individual_languages: &[
    ],
};


pub const BPU: LanguageCode = LanguageCode {
    name: "Bongu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bpu",
    individual_languages: &[
    ],
};


pub const BPV: LanguageCode = LanguageCode {
    name: "Bian Marind",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bpv",
    individual_languages: &[
    ],
};


pub const BPW: LanguageCode = LanguageCode {
    name: "Bo (Papua New Guinea)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bpw",
    individual_languages: &[
    ],
};


pub const BPX: LanguageCode = LanguageCode {
    name: "Palya Bareli",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bpx",
    individual_languages: &[
    ],
};


pub const BPY: LanguageCode = LanguageCode {
    name: "Bishnupriya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bpy",
    individual_languages: &[
    ],
};


pub const BPZ: LanguageCode = LanguageCode {
    name: "Bilba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bpz",
    individual_languages: &[
    ],
};


pub const BQA: LanguageCode = LanguageCode {
    name: "Tchumbuli",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bqa",
    individual_languages: &[
    ],
};


pub const BQB: LanguageCode = LanguageCode {
    name: "Bagusa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bqb",
    individual_languages: &[
    ],
};


pub const BQC: LanguageCode = LanguageCode {
    name: "Boko (Benin)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bqc",
    individual_languages: &[
    ],
};


pub const BQD: LanguageCode = LanguageCode {
    name: "Bung",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bqd",
    individual_languages: &[
    ],
};


pub const BQF: LanguageCode = LanguageCode {
    name: "Baga Kaloum",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bqf",
    individual_languages: &[
    ],
};


pub const BQG: LanguageCode = LanguageCode {
    name: "Bago-Kusuntu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bqg",
    individual_languages: &[
    ],
};


pub const BQH: LanguageCode = LanguageCode {
    name: "Baima",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bqh",
    individual_languages: &[
    ],
};


pub const BQI: LanguageCode = LanguageCode {
    name: "Bakhtiari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bqi",
    individual_languages: &[
    ],
};


pub const BQJ: LanguageCode = LanguageCode {
    name: "Bandial",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bqj",
    individual_languages: &[
    ],
};


pub const BQK: LanguageCode = LanguageCode {
    name: "Banda-Mbrès",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bqk",
    individual_languages: &[
    ],
};


pub const BQL: LanguageCode = LanguageCode {
    name: "Bilakura",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bql",
    individual_languages: &[
    ],
};


pub const BQM: LanguageCode = LanguageCode {
    name: "Wumboko",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bqm",
    individual_languages: &[
    ],
};


pub const BQN: LanguageCode = LanguageCode {
    name: "Bulgarian Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bqn",
    individual_languages: &[
    ],
};


pub const BQO: LanguageCode = LanguageCode {
    name: "Balo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bqo",
    individual_languages: &[
    ],
};


pub const BQP: LanguageCode = LanguageCode {
    name: "Busa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bqp",
    individual_languages: &[
    ],
};


pub const BQQ: LanguageCode = LanguageCode {
    name: "Biritai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bqq",
    individual_languages: &[
    ],
};


pub const BQR: LanguageCode = LanguageCode {
    name: "Burusu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bqr",
    individual_languages: &[
    ],
};


pub const BQS: LanguageCode = LanguageCode {
    name: "Bosngun",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bqs",
    individual_languages: &[
    ],
};


pub const BQT: LanguageCode = LanguageCode {
    name: "Bamukumbit",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bqt",
    individual_languages: &[
    ],
};


pub const BQU: LanguageCode = LanguageCode {
    name: "Boguru",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bqu",
    individual_languages: &[
    ],
};


pub const BQV: LanguageCode = LanguageCode {
    name: "Koro Wachi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bqv",
    individual_languages: &[
    ],
};


pub const BQW: LanguageCode = LanguageCode {
    name: "Buru (Nigeria)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bqw",
    individual_languages: &[
    ],
};


pub const BQX: LanguageCode = LanguageCode {
    name: "Baangi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bqx",
    individual_languages: &[
    ],
};


pub const BQY: LanguageCode = LanguageCode {
    name: "Bengkala Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bqy",
    individual_languages: &[
    ],
};


pub const BQZ: LanguageCode = LanguageCode {
    name: "Bakaka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bqz",
    individual_languages: &[
    ],
};


pub const BRB: LanguageCode = LanguageCode {
    name: "Brao",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "brb",
    individual_languages: &[
    ],
};


pub const BRC: LanguageCode = LanguageCode {
    name: "Berbice Creole Dutch",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "brc",
    individual_languages: &[
    ],
};


pub const BRD: LanguageCode = LanguageCode {
    name: "Baraamu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "brd",
    individual_languages: &[
    ],
};


pub const BRF: LanguageCode = LanguageCode {
    name: "Bira",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "brf",
    individual_languages: &[
    ],
};


pub const BRG: LanguageCode = LanguageCode {
    name: "Baure",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "brg",
    individual_languages: &[
    ],
};


pub const BRH: LanguageCode = LanguageCode {
    name: "Brahui",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "brh",
    individual_languages: &[
    ],
};


pub const BRI: LanguageCode = LanguageCode {
    name: "Mokpwe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bri",
    individual_languages: &[
    ],
};


pub const BRJ: LanguageCode = LanguageCode {
    name: "Bieria",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "brj",
    individual_languages: &[
    ],
};


pub const BRK: LanguageCode = LanguageCode {
    name: "Birked",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "brk",
    individual_languages: &[
    ],
};


pub const BRL: LanguageCode = LanguageCode {
    name: "Birwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "brl",
    individual_languages: &[
    ],
};


pub const BRM: LanguageCode = LanguageCode {
    name: "Barambu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "brm",
    individual_languages: &[
    ],
};


pub const BRN: LanguageCode = LanguageCode {
    name: "Boruca",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "brn",
    individual_languages: &[
    ],
};


pub const BRO: LanguageCode = LanguageCode {
    name: "Brokkat",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bro",
    individual_languages: &[
    ],
};


pub const BRP: LanguageCode = LanguageCode {
    name: "Barapasi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "brp",
    individual_languages: &[
    ],
};


pub const BRQ: LanguageCode = LanguageCode {
    name: "Breri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "brq",
    individual_languages: &[
    ],
};


pub const BRR: LanguageCode = LanguageCode {
    name: "Birao",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "brr",
    individual_languages: &[
    ],
};


pub const BRS: LanguageCode = LanguageCode {
    name: "Baras",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "brs",
    individual_languages: &[
    ],
};


pub const BRT: LanguageCode = LanguageCode {
    name: "Bitare",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "brt",
    individual_languages: &[
    ],
};


pub const BRU: LanguageCode = LanguageCode {
    name: "Eastern Bru",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bru",
    individual_languages: &[
    ],
};


pub const BRV: LanguageCode = LanguageCode {
    name: "Western Bru",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "brv",
    individual_languages: &[
    ],
};


pub const BRW: LanguageCode = LanguageCode {
    name: "Bellari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "brw",
    individual_languages: &[
    ],
};


pub const BRX: LanguageCode = LanguageCode {
    name: "Bodo (India)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "brx",
    individual_languages: &[
    ],
};


pub const BRY: LanguageCode = LanguageCode {
    name: "Burui",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bry",
    individual_languages: &[
    ],
};


pub const BRZ: LanguageCode = LanguageCode {
    name: "Bilbil",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "brz",
    individual_languages: &[
    ],
};


pub const BSA: LanguageCode = LanguageCode {
    name: "Abinomn",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bsa",
    individual_languages: &[
    ],
};


pub const BSB: LanguageCode = LanguageCode {
    name: "Brunei Bisaya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bsb",
    individual_languages: &[
    ],
};


pub const BSC: LanguageCode = LanguageCode {
    name: "Bassari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bsc",
    individual_languages: &[
    ],
};


pub const BSE: LanguageCode = LanguageCode {
    name: "Wushi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bse",
    individual_languages: &[
    ],
};


pub const BSF: LanguageCode = LanguageCode {
    name: "Bauchi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bsf",
    individual_languages: &[
    ],
};


pub const BSG: LanguageCode = LanguageCode {
    name: "Bashkardi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bsg",
    individual_languages: &[
    ],
};


pub const BSH: LanguageCode = LanguageCode {
    name: "Kati",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bsh",
    individual_languages: &[
    ],
};


pub const BSI: LanguageCode = LanguageCode {
    name: "Bassossi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bsi",
    individual_languages: &[
    ],
};


pub const BSJ: LanguageCode = LanguageCode {
    name: "Bangwinji",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bsj",
    individual_languages: &[
    ],
};


pub const BSK: LanguageCode = LanguageCode {
    name: "Burushaski",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bsk",
    individual_languages: &[
    ],
};


pub const BSL: LanguageCode = LanguageCode {
    name: "Basa-Gumna",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bsl",
    individual_languages: &[
    ],
};


pub const BSM: LanguageCode = LanguageCode {
    name: "Busami",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bsm",
    individual_languages: &[
    ],
};


pub const BSN: LanguageCode = LanguageCode {
    name: "Barasana-Eduria",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bsn",
    individual_languages: &[
    ],
};


pub const BSO: LanguageCode = LanguageCode {
    name: "Buso",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bso",
    individual_languages: &[
    ],
};


pub const BSP: LanguageCode = LanguageCode {
    name: "Baga Sitemu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bsp",
    individual_languages: &[
    ],
};


pub const BSQ: LanguageCode = LanguageCode {
    name: "Bassa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bsq",
    individual_languages: &[
    ],
};


pub const BSR: LanguageCode = LanguageCode {
    name: "Bassa-Kontagora",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bsr",
    individual_languages: &[
    ],
};


pub const BSS: LanguageCode = LanguageCode {
    name: "Akoose",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bss",
    individual_languages: &[
    ],
};


pub const BST: LanguageCode = LanguageCode {
    name: "Basketo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bst",
    individual_languages: &[
    ],
};


pub const BSU: LanguageCode = LanguageCode {
    name: "Bahonsuai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bsu",
    individual_languages: &[
    ],
};


pub const BSV: LanguageCode = LanguageCode {
    name: "Baga Sobané",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bsv",
    individual_languages: &[
    ],
};


pub const BSW: LanguageCode = LanguageCode {
    name: "Baiso",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bsw",
    individual_languages: &[
    ],
};


pub const BSX: LanguageCode = LanguageCode {
    name: "Yangkam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bsx",
    individual_languages: &[
    ],
};


pub const BSY: LanguageCode = LanguageCode {
    name: "Sabah Bisaya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bsy",
    individual_languages: &[
    ],
};


pub const BTA: LanguageCode = LanguageCode {
    name: "Bata",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bta",
    individual_languages: &[
    ],
};


pub const BTC: LanguageCode = LanguageCode {
    name: "Bati (Cameroon)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "btc",
    individual_languages: &[
    ],
};


pub const BTD: LanguageCode = LanguageCode {
    name: "Batak Dairi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "btd",
    individual_languages: &[
    ],
};


pub const BTE: LanguageCode = LanguageCode {
    name: "Gamo-Ningi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bte",
    individual_languages: &[
    ],
};


pub const BTF: LanguageCode = LanguageCode {
    name: "Birgit",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "btf",
    individual_languages: &[
    ],
};


pub const BTG: LanguageCode = LanguageCode {
    name: "Gagnoa Bété",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "btg",
    individual_languages: &[
    ],
};


pub const BTH: LanguageCode = LanguageCode {
    name: "Biatah Bidayuh",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bth",
    individual_languages: &[
    ],
};


pub const BTI: LanguageCode = LanguageCode {
    name: "Burate",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bti",
    individual_languages: &[
    ],
};


pub const BTJ: LanguageCode = LanguageCode {
    name: "Bacanese Malay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "btj",
    individual_languages: &[
    ],
};


pub const BTM: LanguageCode = LanguageCode {
    name: "Batak Mandailing",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "btm",
    individual_languages: &[
    ],
};


pub const BTN: LanguageCode = LanguageCode {
    name: "Ratagnon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "btn",
    individual_languages: &[
    ],
};


pub const BTO: LanguageCode = LanguageCode {
    name: "Rinconada Bikol",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bto",
    individual_languages: &[
    ],
};


pub const BTP: LanguageCode = LanguageCode {
    name: "Budibud",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "btp",
    individual_languages: &[
    ],
};


pub const BTQ: LanguageCode = LanguageCode {
    name: "Batek",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "btq",
    individual_languages: &[
    ],
};


pub const BTR: LanguageCode = LanguageCode {
    name: "Baetora",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "btr",
    individual_languages: &[
    ],
};


pub const BTS: LanguageCode = LanguageCode {
    name: "Batak Simalungun",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bts",
    individual_languages: &[
    ],
};


pub const BTT: LanguageCode = LanguageCode {
    name: "Bete-Bendi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "btt",
    individual_languages: &[
    ],
};


pub const BTU: LanguageCode = LanguageCode {
    name: "Batu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "btu",
    individual_languages: &[
    ],
};


pub const BTV: LanguageCode = LanguageCode {
    name: "Bateri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "btv",
    individual_languages: &[
    ],
};


pub const BTW: LanguageCode = LanguageCode {
    name: "Butuanon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "btw",
    individual_languages: &[
    ],
};


pub const BTX: LanguageCode = LanguageCode {
    name: "Batak Karo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "btx",
    individual_languages: &[
    ],
};


pub const BTY: LanguageCode = LanguageCode {
    name: "Bobot",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bty",
    individual_languages: &[
    ],
};


pub const BTZ: LanguageCode = LanguageCode {
    name: "Batak Alas-Kluet",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "btz",
    individual_languages: &[
    ],
};


pub const BUB: LanguageCode = LanguageCode {
    name: "Bua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bub",
    individual_languages: &[
    ],
};


pub const BUC: LanguageCode = LanguageCode {
    name: "Bushi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "buc",
    individual_languages: &[
    ],
};


pub const BUD: LanguageCode = LanguageCode {
    name: "Ntcham",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bud",
    individual_languages: &[
    ],
};


pub const BUE: LanguageCode = LanguageCode {
    name: "Beothuk",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bue",
    individual_languages: &[
    ],
};


pub const BUF: LanguageCode = LanguageCode {
    name: "Bushoong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "buf",
    individual_languages: &[
    ],
};


pub const BUH: LanguageCode = LanguageCode {
    name: "Younuo Bunu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "buh",
    individual_languages: &[
    ],
};


pub const BUI: LanguageCode = LanguageCode {
    name: "Bongili",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bui",
    individual_languages: &[
    ],
};


pub const BUJ: LanguageCode = LanguageCode {
    name: "Basa-Gurmana",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "buj",
    individual_languages: &[
    ],
};


pub const BUK: LanguageCode = LanguageCode {
    name: "Bugawac",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "buk",
    individual_languages: &[
    ],
};


pub const BUM: LanguageCode = LanguageCode {
    name: "Bulu (Cameroon)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bum",
    individual_languages: &[
    ],
};


pub const BUN: LanguageCode = LanguageCode {
    name: "Sherbro",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bun",
    individual_languages: &[
    ],
};


pub const BUO: LanguageCode = LanguageCode {
    name: "Terei",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "buo",
    individual_languages: &[
    ],
};


pub const BUP: LanguageCode = LanguageCode {
    name: "Busoa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bup",
    individual_languages: &[
    ],
};


pub const BUQ: LanguageCode = LanguageCode {
    name: "Brem",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "buq",
    individual_languages: &[
    ],
};


pub const BUS: LanguageCode = LanguageCode {
    name: "Bokobaru",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bus",
    individual_languages: &[
    ],
};


pub const BUT: LanguageCode = LanguageCode {
    name: "Bungain",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "but",
    individual_languages: &[
    ],
};


pub const BUU: LanguageCode = LanguageCode {
    name: "Budu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "buu",
    individual_languages: &[
    ],
};


pub const BUV: LanguageCode = LanguageCode {
    name: "Bun",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "buv",
    individual_languages: &[
    ],
};


pub const BUW: LanguageCode = LanguageCode {
    name: "Bubi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "buw",
    individual_languages: &[
    ],
};


pub const BUX: LanguageCode = LanguageCode {
    name: "Boghom",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bux",
    individual_languages: &[
    ],
};


pub const BUY: LanguageCode = LanguageCode {
    name: "Bullom So",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "buy",
    individual_languages: &[
    ],
};


pub const BUZ: LanguageCode = LanguageCode {
    name: "Bukwen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "buz",
    individual_languages: &[
    ],
};


pub const BVA: LanguageCode = LanguageCode {
    name: "Barein",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bva",
    individual_languages: &[
    ],
};


pub const BVB: LanguageCode = LanguageCode {
    name: "Bube",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bvb",
    individual_languages: &[
    ],
};


pub const BVC: LanguageCode = LanguageCode {
    name: "Baelelea",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bvc",
    individual_languages: &[
    ],
};


pub const BVD: LanguageCode = LanguageCode {
    name: "Baeggu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bvd",
    individual_languages: &[
    ],
};


pub const BVE: LanguageCode = LanguageCode {
    name: "Berau Malay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bve",
    individual_languages: &[
    ],
};


pub const BVF: LanguageCode = LanguageCode {
    name: "Boor",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bvf",
    individual_languages: &[
    ],
};


pub const BVG: LanguageCode = LanguageCode {
    name: "Bonkeng",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bvg",
    individual_languages: &[
    ],
};


pub const BVH: LanguageCode = LanguageCode {
    name: "Bure",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bvh",
    individual_languages: &[
    ],
};


pub const BVI: LanguageCode = LanguageCode {
    name: "Belanda Viri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bvi",
    individual_languages: &[
    ],
};


pub const BVJ: LanguageCode = LanguageCode {
    name: "Baan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bvj",
    individual_languages: &[
    ],
};


pub const BVK: LanguageCode = LanguageCode {
    name: "Bukat",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bvk",
    individual_languages: &[
    ],
};


pub const BVL: LanguageCode = LanguageCode {
    name: "Bolivian Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bvl",
    individual_languages: &[
    ],
};


pub const BVM: LanguageCode = LanguageCode {
    name: "Bamunka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bvm",
    individual_languages: &[
    ],
};


pub const BVN: LanguageCode = LanguageCode {
    name: "Buna",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bvn",
    individual_languages: &[
    ],
};


pub const BVO: LanguageCode = LanguageCode {
    name: "Bolgo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bvo",
    individual_languages: &[
    ],
};


pub const BVP: LanguageCode = LanguageCode {
    name: "Bumang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bvp",
    individual_languages: &[
    ],
};


pub const BVQ: LanguageCode = LanguageCode {
    name: "Birri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bvq",
    individual_languages: &[
    ],
};


pub const BVR: LanguageCode = LanguageCode {
    name: "Burarra",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bvr",
    individual_languages: &[
    ],
};


pub const BVT: LanguageCode = LanguageCode {
    name: "Bati (Indonesia)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bvt",
    individual_languages: &[
    ],
};


pub const BVU: LanguageCode = LanguageCode {
    name: "Bukit Malay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bvu",
    individual_languages: &[
    ],
};


pub const BVV: LanguageCode = LanguageCode {
    name: "Baniva",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bvv",
    individual_languages: &[
    ],
};


pub const BVW: LanguageCode = LanguageCode {
    name: "Boga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bvw",
    individual_languages: &[
    ],
};


pub const BVX: LanguageCode = LanguageCode {
    name: "Dibole",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bvx",
    individual_languages: &[
    ],
};


pub const BVY: LanguageCode = LanguageCode {
    name: "Baybayanon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bvy",
    individual_languages: &[
    ],
};


pub const BVZ: LanguageCode = LanguageCode {
    name: "Bauzi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bvz",
    individual_languages: &[
    ],
};


pub const BWA: LanguageCode = LanguageCode {
    name: "Bwatoo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bwa",
    individual_languages: &[
    ],
};


pub const BWB: LanguageCode = LanguageCode {
    name: "Namosi-Naitasiri-Serua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bwb",
    individual_languages: &[
    ],
};


pub const BWC: LanguageCode = LanguageCode {
    name: "Bwile",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bwc",
    individual_languages: &[
    ],
};


pub const BWD: LanguageCode = LanguageCode {
    name: "Bwaidoka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bwd",
    individual_languages: &[
    ],
};


pub const BWE: LanguageCode = LanguageCode {
    name: "Bwe Karen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bwe",
    individual_languages: &[
    ],
};


pub const BWF: LanguageCode = LanguageCode {
    name: "Boselewa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bwf",
    individual_languages: &[
    ],
};


pub const BWG: LanguageCode = LanguageCode {
    name: "Barwe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bwg",
    individual_languages: &[
    ],
};


pub const BWH: LanguageCode = LanguageCode {
    name: "Bishuo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bwh",
    individual_languages: &[
    ],
};


pub const BWI: LanguageCode = LanguageCode {
    name: "Baniwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bwi",
    individual_languages: &[
    ],
};


pub const BWJ: LanguageCode = LanguageCode {
    name: "Láá Láá Bwamu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bwj",
    individual_languages: &[
    ],
};


pub const BWK: LanguageCode = LanguageCode {
    name: "Bauwaki",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bwk",
    individual_languages: &[
    ],
};


pub const BWL: LanguageCode = LanguageCode {
    name: "Bwela",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bwl",
    individual_languages: &[
    ],
};


pub const BWM: LanguageCode = LanguageCode {
    name: "Biwat",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bwm",
    individual_languages: &[
    ],
};


pub const BWN: LanguageCode = LanguageCode {
    name: "Wunai Bunu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bwn",
    individual_languages: &[
    ],
};


pub const BWO: LanguageCode = LanguageCode {
    name: "Boro (Ethiopia)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bwo",
    individual_languages: &[
    ],
};


pub const BWP: LanguageCode = LanguageCode {
    name: "Mandobo Bawah",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bwp",
    individual_languages: &[
    ],
};


pub const BWQ: LanguageCode = LanguageCode {
    name: "Southern Bobo Madaré",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bwq",
    individual_languages: &[
    ],
};


pub const BWR: LanguageCode = LanguageCode {
    name: "Bura-Pabir",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bwr",
    individual_languages: &[
    ],
};


pub const BWS: LanguageCode = LanguageCode {
    name: "Bomboma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bws",
    individual_languages: &[
    ],
};


pub const BWT: LanguageCode = LanguageCode {
    name: "Bafaw-Balong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bwt",
    individual_languages: &[
    ],
};


pub const BWU: LanguageCode = LanguageCode {
    name: "Buli (Ghana)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bwu",
    individual_languages: &[
    ],
};


pub const BWW: LanguageCode = LanguageCode {
    name: "Bwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bww",
    individual_languages: &[
    ],
};


pub const BWX: LanguageCode = LanguageCode {
    name: "Bu-Nao Bunu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bwx",
    individual_languages: &[
    ],
};


pub const BWY: LanguageCode = LanguageCode {
    name: "Cwi Bwamu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bwy",
    individual_languages: &[
    ],
};


pub const BWZ: LanguageCode = LanguageCode {
    name: "Bwisi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bwz",
    individual_languages: &[
    ],
};


pub const BXA: LanguageCode = LanguageCode {
    name: "Tairaha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bxa",
    individual_languages: &[
    ],
};


pub const BXB: LanguageCode = LanguageCode {
    name: "Belanda Bor",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bxb",
    individual_languages: &[
    ],
};


pub const BXC: LanguageCode = LanguageCode {
    name: "Molengue",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bxc",
    individual_languages: &[
    ],
};


pub const BXD: LanguageCode = LanguageCode {
    name: "Pela",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bxd",
    individual_languages: &[
    ],
};


pub const BXE: LanguageCode = LanguageCode {
    name: "Birale",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bxe",
    individual_languages: &[
    ],
};


pub const BXF: LanguageCode = LanguageCode {
    name: "Bilur",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bxf",
    individual_languages: &[
    ],
};


pub const BXG: LanguageCode = LanguageCode {
    name: "Bangala",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bxg",
    individual_languages: &[
    ],
};


pub const BXH: LanguageCode = LanguageCode {
    name: "Buhutu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bxh",
    individual_languages: &[
    ],
};


pub const BXI: LanguageCode = LanguageCode {
    name: "Pirlatapa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bxi",
    individual_languages: &[
    ],
};


pub const BXJ: LanguageCode = LanguageCode {
    name: "Bayungu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bxj",
    individual_languages: &[
    ],
};


pub const BXK: LanguageCode = LanguageCode {
    name: "Bukusu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bxk",
    individual_languages: &[
    ],
};


pub const BXL: LanguageCode = LanguageCode {
    name: "Jalkunan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bxl",
    individual_languages: &[
    ],
};


pub const BXM: LanguageCode = LanguageCode {
    name: "Mongolia Buriat",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bxm",
    individual_languages: &[
    ],
};


pub const BXN: LanguageCode = LanguageCode {
    name: "Burduna",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bxn",
    individual_languages: &[
    ],
};


pub const BXO: LanguageCode = LanguageCode {
    name: "Barikanchi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bxo",
    individual_languages: &[
    ],
};


pub const BXP: LanguageCode = LanguageCode {
    name: "Bebil",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bxp",
    individual_languages: &[
    ],
};


pub const BXQ: LanguageCode = LanguageCode {
    name: "Beele",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bxq",
    individual_languages: &[
    ],
};


pub const BXR: LanguageCode = LanguageCode {
    name: "Russia Buriat",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bxr",
    individual_languages: &[
    ],
};


pub const BXS: LanguageCode = LanguageCode {
    name: "Busam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bxs",
    individual_languages: &[
    ],
};


pub const BXU: LanguageCode = LanguageCode {
    name: "China Buriat",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bxu",
    individual_languages: &[
    ],
};


pub const BXV: LanguageCode = LanguageCode {
    name: "Berakou",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bxv",
    individual_languages: &[
    ],
};


pub const BXW: LanguageCode = LanguageCode {
    name: "Bankagooma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bxw",
    individual_languages: &[
    ],
};


pub const BXZ: LanguageCode = LanguageCode {
    name: "Binahari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bxz",
    individual_languages: &[
    ],
};


pub const BYA: LanguageCode = LanguageCode {
    name: "Batak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bya",
    individual_languages: &[
    ],
};


pub const BYB: LanguageCode = LanguageCode {
    name: "Bikya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "byb",
    individual_languages: &[
    ],
};


pub const BYC: LanguageCode = LanguageCode {
    name: "Ubaghara",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "byc",
    individual_languages: &[
    ],
};


pub const BYD: LanguageCode = LanguageCode {
    name: "Benyadu'",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "byd",
    individual_languages: &[
    ],
};


pub const BYE: LanguageCode = LanguageCode {
    name: "Pouye",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bye",
    individual_languages: &[
    ],
};


pub const BYF: LanguageCode = LanguageCode {
    name: "Bete",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "byf",
    individual_languages: &[
    ],
};


pub const BYG: LanguageCode = LanguageCode {
    name: "Baygo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "byg",
    individual_languages: &[
    ],
};


pub const BYH: LanguageCode = LanguageCode {
    name: "Bhujel",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "byh",
    individual_languages: &[
    ],
};


pub const BYI: LanguageCode = LanguageCode {
    name: "Buyu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "byi",
    individual_languages: &[
    ],
};


pub const BYJ: LanguageCode = LanguageCode {
    name: "Bina (Nigeria)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "byj",
    individual_languages: &[
    ],
};


pub const BYK: LanguageCode = LanguageCode {
    name: "Biao",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "byk",
    individual_languages: &[
    ],
};


pub const BYL: LanguageCode = LanguageCode {
    name: "Bayono",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "byl",
    individual_languages: &[
    ],
};


pub const BYM: LanguageCode = LanguageCode {
    name: "Bidjara",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bym",
    individual_languages: &[
    ],
};


pub const BYO: LanguageCode = LanguageCode {
    name: "Biyo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "byo",
    individual_languages: &[
    ],
};


pub const BYP: LanguageCode = LanguageCode {
    name: "Bumaji",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "byp",
    individual_languages: &[
    ],
};


pub const BYQ: LanguageCode = LanguageCode {
    name: "Basay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "byq",
    individual_languages: &[
    ],
};


pub const BYR: LanguageCode = LanguageCode {
    name: "Baruya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "byr",
    individual_languages: &[
    ],
};


pub const BYS: LanguageCode = LanguageCode {
    name: "Burak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bys",
    individual_languages: &[
    ],
};


pub const BYT: LanguageCode = LanguageCode {
    name: "Berti",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "byt",
    individual_languages: &[
    ],
};


pub const BYV: LanguageCode = LanguageCode {
    name: "Medumba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "byv",
    individual_languages: &[
    ],
};


pub const BYW: LanguageCode = LanguageCode {
    name: "Belhariya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "byw",
    individual_languages: &[
    ],
};


pub const BYX: LanguageCode = LanguageCode {
    name: "Qaqet",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "byx",
    individual_languages: &[
    ],
};


pub const BYZ: LanguageCode = LanguageCode {
    name: "Banaro",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "byz",
    individual_languages: &[
    ],
};


pub const BZA: LanguageCode = LanguageCode {
    name: "Bandi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bza",
    individual_languages: &[
    ],
};


pub const BZB: LanguageCode = LanguageCode {
    name: "Andio",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bzb",
    individual_languages: &[
    ],
};


pub const BZC: LanguageCode = LanguageCode {
    name: "Southern Betsimisaraka Malagasy",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bzc",
    individual_languages: &[
    ],
};


pub const BZD: LanguageCode = LanguageCode {
    name: "Bribri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bzd",
    individual_languages: &[
    ],
};


pub const BZE: LanguageCode = LanguageCode {
    name: "Jenaama Bozo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bze",
    individual_languages: &[
    ],
};


pub const BZF: LanguageCode = LanguageCode {
    name: "Boikin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bzf",
    individual_languages: &[
    ],
};


pub const BZG: LanguageCode = LanguageCode {
    name: "Babuza",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bzg",
    individual_languages: &[
    ],
};


pub const BZH: LanguageCode = LanguageCode {
    name: "Mapos Buang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bzh",
    individual_languages: &[
    ],
};


pub const BZI: LanguageCode = LanguageCode {
    name: "Bisu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bzi",
    individual_languages: &[
    ],
};


pub const BZJ: LanguageCode = LanguageCode {
    name: "Belize Kriol English",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bzj",
    individual_languages: &[
    ],
};


pub const BZK: LanguageCode = LanguageCode {
    name: "Nicaragua Creole English",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bzk",
    individual_languages: &[
    ],
};


pub const BZL: LanguageCode = LanguageCode {
    name: "Boano (Sulawesi)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bzl",
    individual_languages: &[
    ],
};


pub const BZM: LanguageCode = LanguageCode {
    name: "Bolondo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bzm",
    individual_languages: &[
    ],
};


pub const BZN: LanguageCode = LanguageCode {
    name: "Boano (Maluku)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bzn",
    individual_languages: &[
    ],
};


pub const BZO: LanguageCode = LanguageCode {
    name: "Bozaba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bzo",
    individual_languages: &[
    ],
};


pub const BZP: LanguageCode = LanguageCode {
    name: "Kemberano",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bzp",
    individual_languages: &[
    ],
};


pub const BZQ: LanguageCode = LanguageCode {
    name: "Buli (Indonesia)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bzq",
    individual_languages: &[
    ],
};


pub const BZR: LanguageCode = LanguageCode {
    name: "Biri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bzr",
    individual_languages: &[
    ],
};


pub const BZS: LanguageCode = LanguageCode {
    name: "Brazilian Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bzs",
    individual_languages: &[
    ],
};


pub const BZT: LanguageCode = LanguageCode {
    name: "Brithenig",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bzt",
    individual_languages: &[
    ],
};


pub const BZU: LanguageCode = LanguageCode {
    name: "Burmeso",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bzu",
    individual_languages: &[
    ],
};


pub const BZV: LanguageCode = LanguageCode {
    name: "Naami",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bzv",
    individual_languages: &[
    ],
};


pub const BZW: LanguageCode = LanguageCode {
    name: "Basa (Nigeria)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bzw",
    individual_languages: &[
    ],
};


pub const BZX: LanguageCode = LanguageCode {
    name: "Kɛlɛngaxo Bozo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bzx",
    individual_languages: &[
    ],
};


pub const BZY: LanguageCode = LanguageCode {
    name: "Obanliku",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bzy",
    individual_languages: &[
    ],
};


pub const BZZ: LanguageCode = LanguageCode {
    name: "Evant",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "bzz",
    individual_languages: &[
    ],
};


pub const CAA: LanguageCode = LanguageCode {
    name: "Chortí",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "caa",
    individual_languages: &[
    ],
};


pub const CAB: LanguageCode = LanguageCode {
    name: "Garifuna",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cab",
    individual_languages: &[
    ],
};


pub const CAC: LanguageCode = LanguageCode {
    name: "Chuj",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cac",
    individual_languages: &[
    ],
};


pub const CAE: LanguageCode = LanguageCode {
    name: "Lehar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cae",
    individual_languages: &[
    ],
};


pub const CAF: LanguageCode = LanguageCode {
    name: "Southern Carrier",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "caf",
    individual_languages: &[
    ],
};


pub const CAG: LanguageCode = LanguageCode {
    name: "Nivaclé",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cag",
    individual_languages: &[
    ],
};


pub const CAH: LanguageCode = LanguageCode {
    name: "Cahuarano",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cah",
    individual_languages: &[
    ],
};


pub const CAJ: LanguageCode = LanguageCode {
    name: "Chané",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "caj",
    individual_languages: &[
    ],
};


pub const CAK: LanguageCode = LanguageCode {
    name: "Kaqchikel",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cak",
    individual_languages: &[
    ],
};


pub const CAL: LanguageCode = LanguageCode {
    name: "Carolinian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cal",
    individual_languages: &[
    ],
};


pub const CAM: LanguageCode = LanguageCode {
    name: "Cemuhî",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cam",
    individual_languages: &[
    ],
};


pub const CAN: LanguageCode = LanguageCode {
    name: "Chambri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "can",
    individual_languages: &[
    ],
};


pub const CAO: LanguageCode = LanguageCode {
    name: "Chácobo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cao",
    individual_languages: &[
    ],
};


pub const CAP: LanguageCode = LanguageCode {
    name: "Chipaya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cap",
    individual_languages: &[
    ],
};


pub const CAQ: LanguageCode = LanguageCode {
    name: "Car Nicobarese",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "caq",
    individual_languages: &[
    ],
};


pub const CAS: LanguageCode = LanguageCode {
    name: "Tsimané",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cas",
    individual_languages: &[
    ],
};


pub const CAV: LanguageCode = LanguageCode {
    name: "Cavineña",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cav",
    individual_languages: &[
    ],
};


pub const CAW: LanguageCode = LanguageCode {
    name: "Callawalla",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "caw",
    individual_languages: &[
    ],
};


pub const CAX: LanguageCode = LanguageCode {
    name: "Chiquitano",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cax",
    individual_languages: &[
    ],
};


pub const CAY: LanguageCode = LanguageCode {
    name: "Cayuga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cay",
    individual_languages: &[
    ],
};


pub const CAZ: LanguageCode = LanguageCode {
    name: "Canichana",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "caz",
    individual_languages: &[
    ],
};


pub const CBB: LanguageCode = LanguageCode {
    name: "Cabiyarí",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cbb",
    individual_languages: &[
    ],
};


pub const CBC: LanguageCode = LanguageCode {
    name: "Carapana",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cbc",
    individual_languages: &[
    ],
};


pub const CBD: LanguageCode = LanguageCode {
    name: "Carijona",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cbd",
    individual_languages: &[
    ],
};


pub const CBG: LanguageCode = LanguageCode {
    name: "Chimila",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cbg",
    individual_languages: &[
    ],
};


pub const CBI: LanguageCode = LanguageCode {
    name: "Chachi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cbi",
    individual_languages: &[
    ],
};


pub const CBJ: LanguageCode = LanguageCode {
    name: "Ede Cabe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cbj",
    individual_languages: &[
    ],
};


pub const CBK: LanguageCode = LanguageCode {
    name: "Chavacano",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cbk",
    individual_languages: &[
    ],
};


pub const CBL: LanguageCode = LanguageCode {
    name: "Bualkhaw Chin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cbl",
    individual_languages: &[
    ],
};


pub const CBN: LanguageCode = LanguageCode {
    name: "Nyahkur",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cbn",
    individual_languages: &[
    ],
};


pub const CBO: LanguageCode = LanguageCode {
    name: "Izora",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cbo",
    individual_languages: &[
    ],
};


pub const CBQ: LanguageCode = LanguageCode {
    name: "Tsucuba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cbq",
    individual_languages: &[
    ],
};


pub const CBR: LanguageCode = LanguageCode {
    name: "Cashibo-Cacataibo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cbr",
    individual_languages: &[
    ],
};


pub const CBS: LanguageCode = LanguageCode {
    name: "Cashinahua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cbs",
    individual_languages: &[
    ],
};


pub const CBT: LanguageCode = LanguageCode {
    name: "Chayahuita",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cbt",
    individual_languages: &[
    ],
};


pub const CBU: LanguageCode = LanguageCode {
    name: "Candoshi-Shapra",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cbu",
    individual_languages: &[
    ],
};


pub const CBV: LanguageCode = LanguageCode {
    name: "Cacua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cbv",
    individual_languages: &[
    ],
};


pub const CBW: LanguageCode = LanguageCode {
    name: "Kinabalian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cbw",
    individual_languages: &[
    ],
};


pub const CBY: LanguageCode = LanguageCode {
    name: "Carabayo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cby",
    individual_languages: &[
    ],
};


pub const CCC: LanguageCode = LanguageCode {
    name: "Chamicuro",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ccc",
    individual_languages: &[
    ],
};


pub const CCD: LanguageCode = LanguageCode {
    name: "Cafundo Creole",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ccd",
    individual_languages: &[
    ],
};


pub const CCE: LanguageCode = LanguageCode {
    name: "Chopi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cce",
    individual_languages: &[
    ],
};


pub const CCG: LanguageCode = LanguageCode {
    name: "Samba Daka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ccg",
    individual_languages: &[
    ],
};


pub const CCH: LanguageCode = LanguageCode {
    name: "Atsam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cch",
    individual_languages: &[
    ],
};


pub const CCJ: LanguageCode = LanguageCode {
    name: "Kasanga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ccj",
    individual_languages: &[
    ],
};


pub const CCL: LanguageCode = LanguageCode {
    name: "Cutchi-Swahili",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ccl",
    individual_languages: &[
    ],
};


pub const CCM: LanguageCode = LanguageCode {
    name: "Malaccan Creole Malay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ccm",
    individual_languages: &[
    ],
};


pub const CCO: LanguageCode = LanguageCode {
    name: "Comaltepec Chinantec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cco",
    individual_languages: &[
    ],
};


pub const CCP: LanguageCode = LanguageCode {
    name: "Chakma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ccp",
    individual_languages: &[
    ],
};


pub const CCR: LanguageCode = LanguageCode {
    name: "Cacaopera",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ccr",
    individual_languages: &[
    ],
};


pub const CDA: LanguageCode = LanguageCode {
    name: "Choni",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cda",
    individual_languages: &[
    ],
};


pub const CDE: LanguageCode = LanguageCode {
    name: "Chenchu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cde",
    individual_languages: &[
    ],
};


pub const CDF: LanguageCode = LanguageCode {
    name: "Chiru",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cdf",
    individual_languages: &[
    ],
};


pub const CDH: LanguageCode = LanguageCode {
    name: "Chambeali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cdh",
    individual_languages: &[
    ],
};


pub const CDI: LanguageCode = LanguageCode {
    name: "Chodri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cdi",
    individual_languages: &[
    ],
};


pub const CDJ: LanguageCode = LanguageCode {
    name: "Churahi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cdj",
    individual_languages: &[
    ],
};


pub const CDM: LanguageCode = LanguageCode {
    name: "Chepang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cdm",
    individual_languages: &[
    ],
};


pub const CDN: LanguageCode = LanguageCode {
    name: "Chaudangsi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cdn",
    individual_languages: &[
    ],
};


pub const CDO: LanguageCode = LanguageCode {
    name: "Min Dong Chinese",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cdo",
    individual_languages: &[
    ],
};


pub const CDR: LanguageCode = LanguageCode {
    name: "Cinda-Regi-Tiyal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cdr",
    individual_languages: &[
    ],
};


pub const CDS: LanguageCode = LanguageCode {
    name: "Chadian Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cds",
    individual_languages: &[
    ],
};


pub const CDY: LanguageCode = LanguageCode {
    name: "Chadong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cdy",
    individual_languages: &[
    ],
};


pub const CDZ: LanguageCode = LanguageCode {
    name: "Koda",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cdz",
    individual_languages: &[
    ],
};


pub const CEA: LanguageCode = LanguageCode {
    name: "Lower Chehalis",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cea",
    individual_languages: &[
    ],
};


pub const CEG: LanguageCode = LanguageCode {
    name: "Chamacoco",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ceg",
    individual_languages: &[
    ],
};


pub const CEK: LanguageCode = LanguageCode {
    name: "Eastern Khumi Chin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cek",
    individual_languages: &[
    ],
};


pub const CEN: LanguageCode = LanguageCode {
    name: "Cen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cen",
    individual_languages: &[
    ],
};


pub const CET: LanguageCode = LanguageCode {
    name: "Centúúm",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cet",
    individual_languages: &[
    ],
};


pub const CEY: LanguageCode = LanguageCode {
    name: "Ekai Chin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cey",
    individual_languages: &[
    ],
};


pub const CFA: LanguageCode = LanguageCode {
    name: "Dijim-Bwilim",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cfa",
    individual_languages: &[
    ],
};


pub const CFD: LanguageCode = LanguageCode {
    name: "Cara",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cfd",
    individual_languages: &[
    ],
};


pub const CFG: LanguageCode = LanguageCode {
    name: "Como Karim",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cfg",
    individual_languages: &[
    ],
};


pub const CFM: LanguageCode = LanguageCode {
    name: "Falam Chin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cfm",
    individual_languages: &[
    ],
};


pub const CGA: LanguageCode = LanguageCode {
    name: "Changriwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cga",
    individual_languages: &[
    ],
};


pub const CGC: LanguageCode = LanguageCode {
    name: "Kagayanen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cgc",
    individual_languages: &[
    ],
};


pub const CGG: LanguageCode = LanguageCode {
    name: "Chiga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cgg",
    individual_languages: &[
    ],
};


pub const CGK: LanguageCode = LanguageCode {
    name: "Chocangacakha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cgk",
    individual_languages: &[
    ],
};


pub const CHC: LanguageCode = LanguageCode {
    name: "Catawba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "chc",
    individual_languages: &[
    ],
};


pub const CHD: LanguageCode = LanguageCode {
    name: "Highland Oaxaca Chontal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "chd",
    individual_languages: &[
    ],
};


pub const CHF: LanguageCode = LanguageCode {
    name: "Tabasco Chontal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "chf",
    individual_languages: &[
    ],
};


pub const CHH: LanguageCode = LanguageCode {
    name: "Chinook",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "chh",
    individual_languages: &[
    ],
};


pub const CHJ: LanguageCode = LanguageCode {
    name: "Ojitlán Chinantec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "chj",
    individual_languages: &[
    ],
};


pub const CHL: LanguageCode = LanguageCode {
    name: "Cahuilla",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "chl",
    individual_languages: &[
    ],
};


pub const CHQ: LanguageCode = LanguageCode {
    name: "Quiotepec Chinantec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "chq",
    individual_languages: &[
    ],
};


pub const CHT: LanguageCode = LanguageCode {
    name: "Cholón",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cht",
    individual_languages: &[
    ],
};


pub const CHW: LanguageCode = LanguageCode {
    name: "Chuwabu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "chw",
    individual_languages: &[
    ],
};


pub const CHX: LanguageCode = LanguageCode {
    name: "Chantyal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "chx",
    individual_languages: &[
    ],
};


pub const CHZ: LanguageCode = LanguageCode {
    name: "Ozumacín Chinantec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "chz",
    individual_languages: &[
    ],
};


pub const CIA: LanguageCode = LanguageCode {
    name: "Cia-Cia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cia",
    individual_languages: &[
    ],
};


pub const CIB: LanguageCode = LanguageCode {
    name: "Ci Gbe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cib",
    individual_languages: &[
    ],
};


pub const CIC: LanguageCode = LanguageCode {
    name: "Chickasaw",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cic",
    individual_languages: &[
    ],
};


pub const CID: LanguageCode = LanguageCode {
    name: "Chimariko",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cid",
    individual_languages: &[
    ],
};


pub const CIE: LanguageCode = LanguageCode {
    name: "Cineni",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cie",
    individual_languages: &[
    ],
};


pub const CIH: LanguageCode = LanguageCode {
    name: "Chinali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cih",
    individual_languages: &[
    ],
};


pub const CIK: LanguageCode = LanguageCode {
    name: "Chitkuli Kinnauri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cik",
    individual_languages: &[
    ],
};


pub const CIM: LanguageCode = LanguageCode {
    name: "Cimbrian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cim",
    individual_languages: &[
    ],
};


pub const CIN: LanguageCode = LanguageCode {
    name: "Cinta Larga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cin",
    individual_languages: &[
    ],
};


pub const CIP: LanguageCode = LanguageCode {
    name: "Chiapanec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cip",
    individual_languages: &[
    ],
};


pub const CIR: LanguageCode = LanguageCode {
    name: "Tiri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cir",
    individual_languages: &[
    ],
};


pub const CIW: LanguageCode = LanguageCode {
    name: "Chippewa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ciw",
    individual_languages: &[
    ],
};


pub const CIY: LanguageCode = LanguageCode {
    name: "Chaima",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ciy",
    individual_languages: &[
    ],
};


pub const CJA: LanguageCode = LanguageCode {
    name: "Western Cham",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cja",
    individual_languages: &[
    ],
};


pub const CJE: LanguageCode = LanguageCode {
    name: "Chru",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cje",
    individual_languages: &[
    ],
};


pub const CJH: LanguageCode = LanguageCode {
    name: "Upper Chehalis",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cjh",
    individual_languages: &[
    ],
};


pub const CJI: LanguageCode = LanguageCode {
    name: "Chamalal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cji",
    individual_languages: &[
    ],
};


pub const CJK: LanguageCode = LanguageCode {
    name: "Chokwe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cjk",
    individual_languages: &[
    ],
};


pub const CJM: LanguageCode = LanguageCode {
    name: "Eastern Cham",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cjm",
    individual_languages: &[
    ],
};


pub const CJN: LanguageCode = LanguageCode {
    name: "Chenapian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cjn",
    individual_languages: &[
    ],
};


pub const CJO: LanguageCode = LanguageCode {
    name: "Ashéninka Pajonal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cjo",
    individual_languages: &[
    ],
};


pub const CJP: LanguageCode = LanguageCode {
    name: "Cabécar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cjp",
    individual_languages: &[
    ],
};


pub const CJS: LanguageCode = LanguageCode {
    name: "Shor",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cjs",
    individual_languages: &[
    ],
};


pub const CJV: LanguageCode = LanguageCode {
    name: "Chuave",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cjv",
    individual_languages: &[
    ],
};


pub const CJY: LanguageCode = LanguageCode {
    name: "Jinyu Chinese",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cjy",
    individual_languages: &[
    ],
};


pub const CKB: LanguageCode = LanguageCode {
    name: "Central Kurdish",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ckb",
    individual_languages: &[
    ],
};


pub const CKH: LanguageCode = LanguageCode {
    name: "Chak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ckh",
    individual_languages: &[
    ],
};


pub const CKL: LanguageCode = LanguageCode {
    name: "Cibak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ckl",
    individual_languages: &[
    ],
};


pub const CKM: LanguageCode = LanguageCode {
    name: "Chakavian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ckm",
    individual_languages: &[
    ],
};


pub const CKN: LanguageCode = LanguageCode {
    name: "Kaang Chin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ckn",
    individual_languages: &[
    ],
};


pub const CKO: LanguageCode = LanguageCode {
    name: "Anufo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cko",
    individual_languages: &[
    ],
};


pub const CKQ: LanguageCode = LanguageCode {
    name: "Kajakse",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ckq",
    individual_languages: &[
    ],
};


pub const CKR: LanguageCode = LanguageCode {
    name: "Kairak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ckr",
    individual_languages: &[
    ],
};


pub const CKS: LanguageCode = LanguageCode {
    name: "Tayo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cks",
    individual_languages: &[
    ],
};


pub const CKT: LanguageCode = LanguageCode {
    name: "Chukot",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ckt",
    individual_languages: &[
    ],
};


pub const CKU: LanguageCode = LanguageCode {
    name: "Koasati",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cku",
    individual_languages: &[
    ],
};


pub const CKV: LanguageCode = LanguageCode {
    name: "Kavalan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ckv",
    individual_languages: &[
    ],
};


pub const CKX: LanguageCode = LanguageCode {
    name: "Caka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ckx",
    individual_languages: &[
    ],
};


pub const CKY: LanguageCode = LanguageCode {
    name: "Cakfem-Mushere",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cky",
    individual_languages: &[
    ],
};


pub const CKZ: LanguageCode = LanguageCode {
    name: "Cakchiquel-Quiché Mixed Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ckz",
    individual_languages: &[
    ],
};


pub const CLA: LanguageCode = LanguageCode {
    name: "Ron",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cla",
    individual_languages: &[
    ],
};


pub const CLC: LanguageCode = LanguageCode {
    name: "Chilcotin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "clc",
    individual_languages: &[
    ],
};


pub const CLD: LanguageCode = LanguageCode {
    name: "Chaldean Neo-Aramaic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cld",
    individual_languages: &[
    ],
};


pub const CLE: LanguageCode = LanguageCode {
    name: "Lealao Chinantec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cle",
    individual_languages: &[
    ],
};


pub const CLH: LanguageCode = LanguageCode {
    name: "Chilisso",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "clh",
    individual_languages: &[
    ],
};


pub const CLI: LanguageCode = LanguageCode {
    name: "Chakali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cli",
    individual_languages: &[
    ],
};


pub const CLJ: LanguageCode = LanguageCode {
    name: "Laitu Chin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "clj",
    individual_languages: &[
    ],
};


pub const CLK: LanguageCode = LanguageCode {
    name: "Idu-Mishmi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "clk",
    individual_languages: &[
    ],
};


pub const CLL: LanguageCode = LanguageCode {
    name: "Chala",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cll",
    individual_languages: &[
    ],
};


pub const CLM: LanguageCode = LanguageCode {
    name: "Clallam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "clm",
    individual_languages: &[
    ],
};


pub const CLO: LanguageCode = LanguageCode {
    name: "Lowland Oaxaca Chontal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "clo",
    individual_languages: &[
    ],
};


pub const CLT: LanguageCode = LanguageCode {
    name: "Lautu Chin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "clt",
    individual_languages: &[
    ],
};


pub const CLU: LanguageCode = LanguageCode {
    name: "Caluyanun",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "clu",
    individual_languages: &[
    ],
};


pub const CLW: LanguageCode = LanguageCode {
    name: "Chulym",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "clw",
    individual_languages: &[
    ],
};


pub const CLY: LanguageCode = LanguageCode {
    name: "Eastern Highland Chatino",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cly",
    individual_languages: &[
    ],
};


pub const CMA: LanguageCode = LanguageCode {
    name: "Maa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cma",
    individual_languages: &[
    ],
};


pub const CME: LanguageCode = LanguageCode {
    name: "Cerma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cme",
    individual_languages: &[
    ],
};


pub const CMG: LanguageCode = LanguageCode {
    name: "Classical Mongolian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cmg",
    individual_languages: &[
    ],
};


pub const CMI: LanguageCode = LanguageCode {
    name: "Emberá-Chamí",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cmi",
    individual_languages: &[
    ],
};


pub const CML: LanguageCode = LanguageCode {
    name: "Campalagian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cml",
    individual_languages: &[
    ],
};


pub const CMM: LanguageCode = LanguageCode {
    name: "Michigamea",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cmm",
    individual_languages: &[
    ],
};


pub const CMN: LanguageCode = LanguageCode {
    name: "Mandarin Chinese",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cmn",
    individual_languages: &[
    ],
};


pub const CMO: LanguageCode = LanguageCode {
    name: "Central Mnong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cmo",
    individual_languages: &[
    ],
};


pub const CMR: LanguageCode = LanguageCode {
    name: "Mro-Khimi Chin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cmr",
    individual_languages: &[
    ],
};


pub const CMS: LanguageCode = LanguageCode {
    name: "Messapic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cms",
    individual_languages: &[
    ],
};


pub const CMT: LanguageCode = LanguageCode {
    name: "Camtho",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cmt",
    individual_languages: &[
    ],
};


pub const CNA: LanguageCode = LanguageCode {
    name: "Changthang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cna",
    individual_languages: &[
    ],
};


pub const CNB: LanguageCode = LanguageCode {
    name: "Chinbon Chin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cnb",
    individual_languages: &[
    ],
};


pub const CNC: LanguageCode = LanguageCode {
    name: "Côông",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cnc",
    individual_languages: &[
    ],
};


pub const CNG: LanguageCode = LanguageCode {
    name: "Northern Qiang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cng",
    individual_languages: &[
    ],
};


pub const CNH: LanguageCode = LanguageCode {
    name: "Hakha Chin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cnh",
    individual_languages: &[
    ],
};


pub const CNI: LanguageCode = LanguageCode {
    name: "Asháninka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cni",
    individual_languages: &[
    ],
};


pub const CNK: LanguageCode = LanguageCode {
    name: "Khumi Chin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cnk",
    individual_languages: &[
    ],
};


pub const CNL: LanguageCode = LanguageCode {
    name: "Lalana Chinantec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cnl",
    individual_languages: &[
    ],
};


pub const CNO: LanguageCode = LanguageCode {
    name: "Con",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cno",
    individual_languages: &[
    ],
};


pub const CNP: LanguageCode = LanguageCode {
    name: "Northern Ping Chinese",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cnp",
    individual_languages: &[
    ],
};


pub const CNQ: LanguageCode = LanguageCode {
    name: "Chung",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cnq",
    individual_languages: &[
    ],
};


pub const CNS: LanguageCode = LanguageCode {
    name: "Central Asmat",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cns",
    individual_languages: &[
    ],
};


pub const CNT: LanguageCode = LanguageCode {
    name: "Tepetotutla Chinantec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cnt",
    individual_languages: &[
    ],
};


pub const CNU: LanguageCode = LanguageCode {
    name: "Chenoua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cnu",
    individual_languages: &[
    ],
};


pub const CNW: LanguageCode = LanguageCode {
    name: "Ngawn Chin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cnw",
    individual_languages: &[
    ],
};


pub const CNX: LanguageCode = LanguageCode {
    name: "Middle Cornish",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cnx",
    individual_languages: &[
    ],
};


pub const COA: LanguageCode = LanguageCode {
    name: "Cocos Islands Malay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "coa",
    individual_languages: &[
    ],
};


pub const COB: LanguageCode = LanguageCode {
    name: "Chicomuceltec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cob",
    individual_languages: &[
    ],
};


pub const COC: LanguageCode = LanguageCode {
    name: "Cocopa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "coc",
    individual_languages: &[
    ],
};


pub const COD: LanguageCode = LanguageCode {
    name: "Cocama-Cocamilla",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cod",
    individual_languages: &[
    ],
};


pub const COE: LanguageCode = LanguageCode {
    name: "Koreguaje",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "coe",
    individual_languages: &[
    ],
};


pub const COF: LanguageCode = LanguageCode {
    name: "Colorado",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cof",
    individual_languages: &[
    ],
};


pub const COG: LanguageCode = LanguageCode {
    name: "Chong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cog",
    individual_languages: &[
    ],
};


pub const COH: LanguageCode = LanguageCode {
    name: "Chonyi-Dzihana-Kauma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "coh",
    individual_languages: &[
    ],
};


pub const COJ: LanguageCode = LanguageCode {
    name: "Cochimi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "coj",
    individual_languages: &[
    ],
};


pub const COK: LanguageCode = LanguageCode {
    name: "Santa Teresa Cora",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cok",
    individual_languages: &[
    ],
};


pub const COL: LanguageCode = LanguageCode {
    name: "Columbia-Wenatchi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "col",
    individual_languages: &[
    ],
};


pub const COM: LanguageCode = LanguageCode {
    name: "Comanche",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "com",
    individual_languages: &[
    ],
};


pub const CON: LanguageCode = LanguageCode {
    name: "Cofán",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "con",
    individual_languages: &[
    ],
};


pub const COO: LanguageCode = LanguageCode {
    name: "Comox",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "coo",
    individual_languages: &[
    ],
};


pub const COQ: LanguageCode = LanguageCode {
    name: "Coquille",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "coq",
    individual_languages: &[
    ],
};


pub const COT: LanguageCode = LanguageCode {
    name: "Caquinte",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cot",
    individual_languages: &[
    ],
};


pub const COU: LanguageCode = LanguageCode {
    name: "Wamey",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cou",
    individual_languages: &[
    ],
};


pub const COV: LanguageCode = LanguageCode {
    name: "Cao Miao",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cov",
    individual_languages: &[
    ],
};


pub const COW: LanguageCode = LanguageCode {
    name: "Cowlitz",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cow",
    individual_languages: &[
    ],
};


pub const COX: LanguageCode = LanguageCode {
    name: "Nanti",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cox",
    individual_languages: &[
    ],
};


pub const COZ: LanguageCode = LanguageCode {
    name: "Chochotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "coz",
    individual_languages: &[
    ],
};


pub const CPA: LanguageCode = LanguageCode {
    name: "Palantla Chinantec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cpa",
    individual_languages: &[
    ],
};


pub const CPB: LanguageCode = LanguageCode {
    name: "Ucayali-Yurúa Ashéninka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cpb",
    individual_languages: &[
    ],
};


pub const CPC: LanguageCode = LanguageCode {
    name: "Ajyíninka Apurucayali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cpc",
    individual_languages: &[
    ],
};


pub const CPG: LanguageCode = LanguageCode {
    name: "Cappadocian Greek",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cpg",
    individual_languages: &[
    ],
};


pub const CPI: LanguageCode = LanguageCode {
    name: "Chinese Pidgin English",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cpi",
    individual_languages: &[
    ],
};


pub const CPN: LanguageCode = LanguageCode {
    name: "Cherepon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cpn",
    individual_languages: &[
    ],
};


pub const CPO: LanguageCode = LanguageCode {
    name: "Kpeego",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cpo",
    individual_languages: &[
    ],
};


pub const CPS: LanguageCode = LanguageCode {
    name: "Capiznon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cps",
    individual_languages: &[
    ],
};


pub const CPU: LanguageCode = LanguageCode {
    name: "Pichis Ashéninka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cpu",
    individual_languages: &[
    ],
};


pub const CPX: LanguageCode = LanguageCode {
    name: "Pu-Xian Chinese",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cpx",
    individual_languages: &[
    ],
};


pub const CPY: LanguageCode = LanguageCode {
    name: "South Ucayali Ashéninka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cpy",
    individual_languages: &[
    ],
};


pub const CQD: LanguageCode = LanguageCode {
    name: "Chuanqiandian Cluster Miao",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cqd",
    individual_languages: &[
    ],
};


pub const CRA: LanguageCode = LanguageCode {
    name: "Chara",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cra",
    individual_languages: &[
    ],
};


pub const CRB: LanguageCode = LanguageCode {
    name: "Island Carib",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "crb",
    individual_languages: &[
    ],
};


pub const CRC: LanguageCode = LanguageCode {
    name: "Lonwolwol",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "crc",
    individual_languages: &[
    ],
};


pub const CRD: LanguageCode = LanguageCode {
    name: "Coeur d'Alene",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "crd",
    individual_languages: &[
    ],
};


pub const CRF: LanguageCode = LanguageCode {
    name: "Caramanta",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "crf",
    individual_languages: &[
    ],
};


pub const CRG: LanguageCode = LanguageCode {
    name: "Michif",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "crg",
    individual_languages: &[
    ],
};


pub const CRI: LanguageCode = LanguageCode {
    name: "Sãotomense",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cri",
    individual_languages: &[
    ],
};


pub const CRJ: LanguageCode = LanguageCode {
    name: "Southern East Cree",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "crj",
    individual_languages: &[
    ],
};


pub const CRK: LanguageCode = LanguageCode {
    name: "Plains Cree",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "crk",
    individual_languages: &[
    ],
};


pub const CRL: LanguageCode = LanguageCode {
    name: "Northern East Cree",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "crl",
    individual_languages: &[
    ],
};


pub const CRM: LanguageCode = LanguageCode {
    name: "Moose Cree",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "crm",
    individual_languages: &[
    ],
};


pub const CRN: LanguageCode = LanguageCode {
    name: "El Nayar Cora",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "crn",
    individual_languages: &[
    ],
};


pub const CRO: LanguageCode = LanguageCode {
    name: "Crow",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cro",
    individual_languages: &[
    ],
};


pub const CRQ: LanguageCode = LanguageCode {
    name: "Iyo'wujwa Chorote",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "crq",
    individual_languages: &[
    ],
};


pub const CRR: LanguageCode = LanguageCode {
    name: "Carolina Algonquian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "crr",
    individual_languages: &[
    ],
};


pub const CRS: LanguageCode = LanguageCode {
    name: "Seselwa Creole French",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "crs",
    individual_languages: &[
    ],
};


pub const CRT: LanguageCode = LanguageCode {
    name: "Iyojwa'ja Chorote",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "crt",
    individual_languages: &[
    ],
};


pub const CRV: LanguageCode = LanguageCode {
    name: "Chaura",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "crv",
    individual_languages: &[
    ],
};


pub const CRW: LanguageCode = LanguageCode {
    name: "Chrau",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "crw",
    individual_languages: &[
    ],
};


pub const CRX: LanguageCode = LanguageCode {
    name: "Carrier",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "crx",
    individual_languages: &[
    ],
};


pub const CRY: LanguageCode = LanguageCode {
    name: "Cori",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cry",
    individual_languages: &[
    ],
};


pub const CRZ: LanguageCode = LanguageCode {
    name: "Cruzeño",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "crz",
    individual_languages: &[
    ],
};


pub const CSA: LanguageCode = LanguageCode {
    name: "Chiltepec Chinantec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "csa",
    individual_languages: &[
    ],
};


pub const CSC: LanguageCode = LanguageCode {
    name: "Catalan Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "csc",
    individual_languages: &[
    ],
};


pub const CSD: LanguageCode = LanguageCode {
    name: "Chiangmai Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "csd",
    individual_languages: &[
    ],
};


pub const CSE: LanguageCode = LanguageCode {
    name: "Czech Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cse",
    individual_languages: &[
    ],
};


pub const CSF: LanguageCode = LanguageCode {
    name: "Cuba Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "csf",
    individual_languages: &[
    ],
};


pub const CSG: LanguageCode = LanguageCode {
    name: "Chilean Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "csg",
    individual_languages: &[
    ],
};


pub const CSH: LanguageCode = LanguageCode {
    name: "Asho Chin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "csh",
    individual_languages: &[
    ],
};


pub const CSI: LanguageCode = LanguageCode {
    name: "Coast Miwok",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "csi",
    individual_languages: &[
    ],
};


pub const CSJ: LanguageCode = LanguageCode {
    name: "Songlai Chin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "csj",
    individual_languages: &[
    ],
};


pub const CSK: LanguageCode = LanguageCode {
    name: "Jola-Kasa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "csk",
    individual_languages: &[
    ],
};


pub const CSL: LanguageCode = LanguageCode {
    name: "Chinese Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "csl",
    individual_languages: &[
    ],
};


pub const CSM: LanguageCode = LanguageCode {
    name: "Central Sierra Miwok",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "csm",
    individual_languages: &[
    ],
};


pub const CSN: LanguageCode = LanguageCode {
    name: "Colombian Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "csn",
    individual_languages: &[
    ],
};


pub const CSO: LanguageCode = LanguageCode {
    name: "Sochiapam Chinantec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cso",
    individual_languages: &[
    ],
};


pub const CSP: LanguageCode = LanguageCode {
    name: "Southern Ping Chinese",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "csp",
    individual_languages: &[
    ],
};


pub const CSQ: LanguageCode = LanguageCode {
    name: "Croatia Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "csq",
    individual_languages: &[
    ],
};


pub const CSR: LanguageCode = LanguageCode {
    name: "Costa Rican Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "csr",
    individual_languages: &[
    ],
};


pub const CSS: LanguageCode = LanguageCode {
    name: "Southern Ohlone",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "css",
    individual_languages: &[
    ],
};


pub const CST: LanguageCode = LanguageCode {
    name: "Northern Ohlone",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cst",
    individual_languages: &[
    ],
};


pub const CSV: LanguageCode = LanguageCode {
    name: "Sumtu Chin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "csv",
    individual_languages: &[
    ],
};


pub const CSW: LanguageCode = LanguageCode {
    name: "Swampy Cree",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "csw",
    individual_languages: &[
    ],
};


pub const CSX: LanguageCode = LanguageCode {
    name: "Cambodian Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "csx",
    individual_languages: &[
    ],
};


pub const CSY: LanguageCode = LanguageCode {
    name: "Siyin Chin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "csy",
    individual_languages: &[
    ],
};


pub const CSZ: LanguageCode = LanguageCode {
    name: "Coos",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "csz",
    individual_languages: &[
    ],
};


pub const CTA: LanguageCode = LanguageCode {
    name: "Tataltepec Chatino",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cta",
    individual_languages: &[
    ],
};


pub const CTC: LanguageCode = LanguageCode {
    name: "Chetco",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ctc",
    individual_languages: &[
    ],
};


pub const CTD: LanguageCode = LanguageCode {
    name: "Tedim Chin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ctd",
    individual_languages: &[
    ],
};


pub const CTE: LanguageCode = LanguageCode {
    name: "Tepinapa Chinantec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cte",
    individual_languages: &[
    ],
};


pub const CTG: LanguageCode = LanguageCode {
    name: "Chittagonian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ctg",
    individual_languages: &[
    ],
};


pub const CTH: LanguageCode = LanguageCode {
    name: "Thaiphum Chin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cth",
    individual_languages: &[
    ],
};


pub const CTL: LanguageCode = LanguageCode {
    name: "Tlacoatzintepec Chinantec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ctl",
    individual_languages: &[
    ],
};


pub const CTM: LanguageCode = LanguageCode {
    name: "Chitimacha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ctm",
    individual_languages: &[
    ],
};


pub const CTN: LanguageCode = LanguageCode {
    name: "Chhintange",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ctn",
    individual_languages: &[
    ],
};


pub const CTO: LanguageCode = LanguageCode {
    name: "Emberá-Catío",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cto",
    individual_languages: &[
    ],
};


pub const CTP: LanguageCode = LanguageCode {
    name: "Western Highland Chatino",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ctp",
    individual_languages: &[
    ],
};


pub const CTS: LanguageCode = LanguageCode {
    name: "Northern Catanduanes Bikol",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cts",
    individual_languages: &[
    ],
};


pub const CTT: LanguageCode = LanguageCode {
    name: "Wayanad Chetti",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ctt",
    individual_languages: &[
    ],
};


pub const CTU: LanguageCode = LanguageCode {
    name: "Chol",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ctu",
    individual_languages: &[
    ],
};


pub const CTY: LanguageCode = LanguageCode {
    name: "Moundadan Chetty",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cty",
    individual_languages: &[
    ],
};


pub const CTZ: LanguageCode = LanguageCode {
    name: "Zacatepec Chatino",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ctz",
    individual_languages: &[
    ],
};


pub const CUA: LanguageCode = LanguageCode {
    name: "Cua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cua",
    individual_languages: &[
    ],
};


pub const CUB: LanguageCode = LanguageCode {
    name: "Cubeo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cub",
    individual_languages: &[
    ],
};


pub const CUC: LanguageCode = LanguageCode {
    name: "Usila Chinantec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cuc",
    individual_languages: &[
    ],
};


pub const CUH: LanguageCode = LanguageCode {
    name: "Chuka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cuh",
    individual_languages: &[
    ],
};


pub const CUI: LanguageCode = LanguageCode {
    name: "Cuiba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cui",
    individual_languages: &[
    ],
};


pub const CUJ: LanguageCode = LanguageCode {
    name: "Mashco Piro",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cuj",
    individual_languages: &[
    ],
};


pub const CUK: LanguageCode = LanguageCode {
    name: "San Blas Kuna",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cuk",
    individual_languages: &[
    ],
};


pub const CUL: LanguageCode = LanguageCode {
    name: "Culina",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cul",
    individual_languages: &[
    ],
};


pub const CUO: LanguageCode = LanguageCode {
    name: "Cumanagoto",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cuo",
    individual_languages: &[
    ],
};


pub const CUP: LanguageCode = LanguageCode {
    name: "Cupeño",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cup",
    individual_languages: &[
    ],
};


pub const CUQ: LanguageCode = LanguageCode {
    name: "Cun",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cuq",
    individual_languages: &[
    ],
};


pub const CUR: LanguageCode = LanguageCode {
    name: "Chhulung",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cur",
    individual_languages: &[
    ],
};


pub const CUT: LanguageCode = LanguageCode {
    name: "Teutila Cuicatec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cut",
    individual_languages: &[
    ],
};


pub const CUU: LanguageCode = LanguageCode {
    name: "Tai Ya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cuu",
    individual_languages: &[
    ],
};


pub const CUV: LanguageCode = LanguageCode {
    name: "Cuvok",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cuv",
    individual_languages: &[
    ],
};


pub const CUW: LanguageCode = LanguageCode {
    name: "Chukwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cuw",
    individual_languages: &[
    ],
};


pub const CUX: LanguageCode = LanguageCode {
    name: "Tepeuxila Cuicatec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cux",
    individual_languages: &[
    ],
};


pub const CUY: LanguageCode = LanguageCode {
    name: "Cuitlatec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cuy",
    individual_languages: &[
    ],
};


pub const CVG: LanguageCode = LanguageCode {
    name: "Chug",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cvg",
    individual_languages: &[
    ],
};


pub const CVN: LanguageCode = LanguageCode {
    name: "Valle Nacional Chinantec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cvn",
    individual_languages: &[
    ],
};


pub const CWA: LanguageCode = LanguageCode {
    name: "Kabwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cwa",
    individual_languages: &[
    ],
};


pub const CWB: LanguageCode = LanguageCode {
    name: "Maindo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cwb",
    individual_languages: &[
    ],
};


pub const CWD: LanguageCode = LanguageCode {
    name: "Woods Cree",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cwd",
    individual_languages: &[
    ],
};


pub const CWE: LanguageCode = LanguageCode {
    name: "Kwere",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cwe",
    individual_languages: &[
    ],
};


pub const CWG: LanguageCode = LanguageCode {
    name: "Chewong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cwg",
    individual_languages: &[
    ],
};


pub const CWT: LanguageCode = LanguageCode {
    name: "Kuwaataay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cwt",
    individual_languages: &[
    ],
};


pub const CYA: LanguageCode = LanguageCode {
    name: "Nopala Chatino",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cya",
    individual_languages: &[
    ],
};


pub const CYB: LanguageCode = LanguageCode {
    name: "Cayubaba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cyb",
    individual_languages: &[
    ],
};


pub const CYO: LanguageCode = LanguageCode {
    name: "Cuyonon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "cyo",
    individual_languages: &[
    ],
};


pub const CZH: LanguageCode = LanguageCode {
    name: "Huizhou Chinese",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "czh",
    individual_languages: &[
    ],
};


pub const CZK: LanguageCode = LanguageCode {
    name: "Knaanic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "czk",
    individual_languages: &[
    ],
};


pub const CZN: LanguageCode = LanguageCode {
    name: "Zenzontepec Chatino",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "czn",
    individual_languages: &[
    ],
};


pub const CZO: LanguageCode = LanguageCode {
    name: "Min Zhong Chinese",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "czo",
    individual_languages: &[
    ],
};


pub const CZT: LanguageCode = LanguageCode {
    name: "Zotung Chin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "czt",
    individual_languages: &[
    ],
};


pub const DAA: LanguageCode = LanguageCode {
    name: "Dangaléat",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "daa",
    individual_languages: &[
    ],
};


pub const DAC: LanguageCode = LanguageCode {
    name: "Dambi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dac",
    individual_languages: &[
    ],
};


pub const DAD: LanguageCode = LanguageCode {
    name: "Marik",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dad",
    individual_languages: &[
    ],
};


pub const DAE: LanguageCode = LanguageCode {
    name: "Duupa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dae",
    individual_languages: &[
    ],
};


pub const DAG: LanguageCode = LanguageCode {
    name: "Dagbani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dag",
    individual_languages: &[
    ],
};


pub const DAH: LanguageCode = LanguageCode {
    name: "Gwahatike",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dah",
    individual_languages: &[
    ],
};


pub const DAI: LanguageCode = LanguageCode {
    name: "Day",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dai",
    individual_languages: &[
    ],
};


pub const DAJ: LanguageCode = LanguageCode {
    name: "Dar Fur Daju",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "daj",
    individual_languages: &[
    ],
};


pub const DAL: LanguageCode = LanguageCode {
    name: "Dahalo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dal",
    individual_languages: &[
    ],
};


pub const DAM: LanguageCode = LanguageCode {
    name: "Damakawa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dam",
    individual_languages: &[
    ],
};


pub const DAO: LanguageCode = LanguageCode {
    name: "Daai Chin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dao",
    individual_languages: &[
    ],
};


pub const DAQ: LanguageCode = LanguageCode {
    name: "Dandami Maria",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "daq",
    individual_languages: &[
    ],
};


pub const DAS: LanguageCode = LanguageCode {
    name: "Daho-Doo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "das",
    individual_languages: &[
    ],
};


pub const DAU: LanguageCode = LanguageCode {
    name: "Dar Sila Daju",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dau",
    individual_languages: &[
    ],
};


pub const DAV: LanguageCode = LanguageCode {
    name: "Taita",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dav",
    individual_languages: &[
    ],
};


pub const DAW: LanguageCode = LanguageCode {
    name: "Davawenyo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "daw",
    individual_languages: &[
    ],
};


pub const DAX: LanguageCode = LanguageCode {
    name: "Dayi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dax",
    individual_languages: &[
    ],
};


pub const DAZ: LanguageCode = LanguageCode {
    name: "Dao",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "daz",
    individual_languages: &[
    ],
};


pub const DBA: LanguageCode = LanguageCode {
    name: "Bangime",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dba",
    individual_languages: &[
    ],
};


pub const DBB: LanguageCode = LanguageCode {
    name: "Deno",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dbb",
    individual_languages: &[
    ],
};


pub const DBD: LanguageCode = LanguageCode {
    name: "Dadiya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dbd",
    individual_languages: &[
    ],
};


pub const DBE: LanguageCode = LanguageCode {
    name: "Dabe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dbe",
    individual_languages: &[
    ],
};


pub const DBF: LanguageCode = LanguageCode {
    name: "Edopi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dbf",
    individual_languages: &[
    ],
};


pub const DBG: LanguageCode = LanguageCode {
    name: "Dogul Dom Dogon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dbg",
    individual_languages: &[
    ],
};


pub const DBI: LanguageCode = LanguageCode {
    name: "Doka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dbi",
    individual_languages: &[
    ],
};


pub const DBJ: LanguageCode = LanguageCode {
    name: "Ida'an",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dbj",
    individual_languages: &[
    ],
};


pub const DBL: LanguageCode = LanguageCode {
    name: "Dyirbal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dbl",
    individual_languages: &[
    ],
};


pub const DBM: LanguageCode = LanguageCode {
    name: "Duguri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dbm",
    individual_languages: &[
    ],
};


pub const DBN: LanguageCode = LanguageCode {
    name: "Duriankere",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dbn",
    individual_languages: &[
    ],
};


pub const DBO: LanguageCode = LanguageCode {
    name: "Dulbu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dbo",
    individual_languages: &[
    ],
};


pub const DBP: LanguageCode = LanguageCode {
    name: "Duwai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dbp",
    individual_languages: &[
    ],
};


pub const DBQ: LanguageCode = LanguageCode {
    name: "Daba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dbq",
    individual_languages: &[
    ],
};


pub const DBR: LanguageCode = LanguageCode {
    name: "Dabarre",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dbr",
    individual_languages: &[
    ],
};


pub const DBT: LanguageCode = LanguageCode {
    name: "Ben Tey Dogon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dbt",
    individual_languages: &[
    ],
};


pub const DBU: LanguageCode = LanguageCode {
    name: "Bondum Dom Dogon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dbu",
    individual_languages: &[
    ],
};


pub const DBV: LanguageCode = LanguageCode {
    name: "Dungu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dbv",
    individual_languages: &[
    ],
};


pub const DBW: LanguageCode = LanguageCode {
    name: "Bankan Tey Dogon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dbw",
    individual_languages: &[
    ],
};


pub const DBY: LanguageCode = LanguageCode {
    name: "Dibiyaso",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dby",
    individual_languages: &[
    ],
};


pub const DCC: LanguageCode = LanguageCode {
    name: "Deccan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dcc",
    individual_languages: &[
    ],
};


pub const DCR: LanguageCode = LanguageCode {
    name: "Negerhollands",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dcr",
    individual_languages: &[
    ],
};


pub const DDA: LanguageCode = LanguageCode {
    name: "Dadi Dadi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dda",
    individual_languages: &[
    ],
};


pub const DDD: LanguageCode = LanguageCode {
    name: "Dongotono",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ddd",
    individual_languages: &[
    ],
};


pub const DDE: LanguageCode = LanguageCode {
    name: "Doondo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dde",
    individual_languages: &[
    ],
};


pub const DDG: LanguageCode = LanguageCode {
    name: "Fataluku",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ddg",
    individual_languages: &[
    ],
};


pub const DDI: LanguageCode = LanguageCode {
    name: "West Goodenough",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ddi",
    individual_languages: &[
    ],
};


pub const DDJ: LanguageCode = LanguageCode {
    name: "Jaru",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ddj",
    individual_languages: &[
    ],
};


pub const DDN: LanguageCode = LanguageCode {
    name: "Dendi (Benin)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ddn",
    individual_languages: &[
    ],
};


pub const DDO: LanguageCode = LanguageCode {
    name: "Dido",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ddo",
    individual_languages: &[
    ],
};


pub const DDR: LanguageCode = LanguageCode {
    name: "Dhudhuroa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ddr",
    individual_languages: &[
    ],
};


pub const DDS: LanguageCode = LanguageCode {
    name: "Donno So Dogon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dds",
    individual_languages: &[
    ],
};


pub const DDW: LanguageCode = LanguageCode {
    name: "Dawera-Daweloor",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ddw",
    individual_languages: &[
    ],
};


pub const DEC: LanguageCode = LanguageCode {
    name: "Dagik",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dec",
    individual_languages: &[
    ],
};


pub const DED: LanguageCode = LanguageCode {
    name: "Dedua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ded",
    individual_languages: &[
    ],
};


pub const DEE: LanguageCode = LanguageCode {
    name: "Dewoin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dee",
    individual_languages: &[
    ],
};


pub const DEF: LanguageCode = LanguageCode {
    name: "Dezfuli",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "def",
    individual_languages: &[
    ],
};


pub const DEG: LanguageCode = LanguageCode {
    name: "Degema",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "deg",
    individual_languages: &[
    ],
};


pub const DEH: LanguageCode = LanguageCode {
    name: "Dehwari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "deh",
    individual_languages: &[
    ],
};


pub const DEI: LanguageCode = LanguageCode {
    name: "Demisa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dei",
    individual_languages: &[
    ],
};


pub const DEK: LanguageCode = LanguageCode {
    name: "Dek",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dek",
    individual_languages: &[
    ],
};


pub const DEM: LanguageCode = LanguageCode {
    name: "Dem",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dem",
    individual_languages: &[
    ],
};


pub const DEP: LanguageCode = LanguageCode {
    name: "Pidgin Delaware",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dep",
    individual_languages: &[
    ],
};


pub const DEQ: LanguageCode = LanguageCode {
    name: "Dendi (Central African Republic)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "deq",
    individual_languages: &[
    ],
};


pub const DER: LanguageCode = LanguageCode {
    name: "Deori",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "der",
    individual_languages: &[
    ],
};


pub const DES: LanguageCode = LanguageCode {
    name: "Desano",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "des",
    individual_languages: &[
    ],
};


pub const DEV: LanguageCode = LanguageCode {
    name: "Domung",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dev",
    individual_languages: &[
    ],
};


pub const DEZ: LanguageCode = LanguageCode {
    name: "Dengese",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dez",
    individual_languages: &[
    ],
};


pub const DGA: LanguageCode = LanguageCode {
    name: "Southern Dagaare",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dga",
    individual_languages: &[
    ],
};


pub const DGB: LanguageCode = LanguageCode {
    name: "Bunoge Dogon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dgb",
    individual_languages: &[
    ],
};


pub const DGC: LanguageCode = LanguageCode {
    name: "Casiguran Dumagat Agta",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dgc",
    individual_languages: &[
    ],
};


pub const DGD: LanguageCode = LanguageCode {
    name: "Dagaari Dioula",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dgd",
    individual_languages: &[
    ],
};


pub const DGE: LanguageCode = LanguageCode {
    name: "Degenan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dge",
    individual_languages: &[
    ],
};


pub const DGG: LanguageCode = LanguageCode {
    name: "Doga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dgg",
    individual_languages: &[
    ],
};


pub const DGH: LanguageCode = LanguageCode {
    name: "Dghwede",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dgh",
    individual_languages: &[
    ],
};


pub const DGI: LanguageCode = LanguageCode {
    name: "Northern Dagara",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dgi",
    individual_languages: &[
    ],
};


pub const DGK: LanguageCode = LanguageCode {
    name: "Dagba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dgk",
    individual_languages: &[
    ],
};


pub const DGL: LanguageCode = LanguageCode {
    name: "Andaandi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dgl",
    individual_languages: &[
    ],
};


pub const DGN: LanguageCode = LanguageCode {
    name: "Dagoman",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dgn",
    individual_languages: &[
    ],
};


pub const DGO: LanguageCode = LanguageCode {
    name: "Dogri (individual language)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dgo",
    individual_languages: &[
    ],
};


pub const DGS: LanguageCode = LanguageCode {
    name: "Dogoso",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dgs",
    individual_languages: &[
    ],
};


pub const DGT: LanguageCode = LanguageCode {
    name: "Ndra'ngith",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dgt",
    individual_languages: &[
    ],
};


pub const DGW: LanguageCode = LanguageCode {
    name: "Daungwurrung",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dgw",
    individual_languages: &[
    ],
};


pub const DGX: LanguageCode = LanguageCode {
    name: "Doghoro",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dgx",
    individual_languages: &[
    ],
};


pub const DGZ: LanguageCode = LanguageCode {
    name: "Daga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dgz",
    individual_languages: &[
    ],
};


pub const DHD: LanguageCode = LanguageCode {
    name: "Dhundari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dhd",
    individual_languages: &[
    ],
};


pub const DHG: LanguageCode = LanguageCode {
    name: "Dhangu-Djangu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dhg",
    individual_languages: &[
    ],
};


pub const DHI: LanguageCode = LanguageCode {
    name: "Dhimal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dhi",
    individual_languages: &[
    ],
};


pub const DHL: LanguageCode = LanguageCode {
    name: "Dhalandji",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dhl",
    individual_languages: &[
    ],
};


pub const DHM: LanguageCode = LanguageCode {
    name: "Zemba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dhm",
    individual_languages: &[
    ],
};


pub const DHN: LanguageCode = LanguageCode {
    name: "Dhanki",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dhn",
    individual_languages: &[
    ],
};


pub const DHO: LanguageCode = LanguageCode {
    name: "Dhodia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dho",
    individual_languages: &[
    ],
};


pub const DHR: LanguageCode = LanguageCode {
    name: "Dhargari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dhr",
    individual_languages: &[
    ],
};


pub const DHS: LanguageCode = LanguageCode {
    name: "Dhaiso",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dhs",
    individual_languages: &[
    ],
};


pub const DHU: LanguageCode = LanguageCode {
    name: "Dhurga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dhu",
    individual_languages: &[
    ],
};


pub const DHV: LanguageCode = LanguageCode {
    name: "Dehu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dhv",
    individual_languages: &[
    ],
};


pub const DHW: LanguageCode = LanguageCode {
    name: "Dhanwar (Nepal)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dhw",
    individual_languages: &[
    ],
};


pub const DHX: LanguageCode = LanguageCode {
    name: "Dhungaloo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dhx",
    individual_languages: &[
    ],
};


pub const DIA: LanguageCode = LanguageCode {
    name: "Dia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dia",
    individual_languages: &[
    ],
};


pub const DIB: LanguageCode = LanguageCode {
    name: "South Central Dinka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dib",
    individual_languages: &[
    ],
};


pub const DIC: LanguageCode = LanguageCode {
    name: "Lakota Dida",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dic",
    individual_languages: &[
    ],
};


pub const DID: LanguageCode = LanguageCode {
    name: "Didinga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "did",
    individual_languages: &[
    ],
};


pub const DIF: LanguageCode = LanguageCode {
    name: "Dieri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dif",
    individual_languages: &[
    ],
};


pub const DIG: LanguageCode = LanguageCode {
    name: "Digo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dig",
    individual_languages: &[
    ],
};


pub const DIH: LanguageCode = LanguageCode {
    name: "Kumiai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dih",
    individual_languages: &[
    ],
};


pub const DII: LanguageCode = LanguageCode {
    name: "Dimbong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dii",
    individual_languages: &[
    ],
};


pub const DIJ: LanguageCode = LanguageCode {
    name: "Dai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dij",
    individual_languages: &[
    ],
};


pub const DIK: LanguageCode = LanguageCode {
    name: "Southwestern Dinka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dik",
    individual_languages: &[
    ],
};


pub const DIL: LanguageCode = LanguageCode {
    name: "Dilling",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dil",
    individual_languages: &[
    ],
};


pub const DIM: LanguageCode = LanguageCode {
    name: "Dime",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dim",
    individual_languages: &[
    ],
};


pub const DIO: LanguageCode = LanguageCode {
    name: "Dibo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dio",
    individual_languages: &[
    ],
};


pub const DIP: LanguageCode = LanguageCode {
    name: "Northeastern Dinka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dip",
    individual_languages: &[
    ],
};


pub const DIQ: LanguageCode = LanguageCode {
    name: "Dimli (individual language)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "diq",
    individual_languages: &[
    ],
};


pub const DIR: LanguageCode = LanguageCode {
    name: "Dirim",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dir",
    individual_languages: &[
    ],
};


pub const DIS: LanguageCode = LanguageCode {
    name: "Dimasa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dis",
    individual_languages: &[
    ],
};


pub const DIU: LanguageCode = LanguageCode {
    name: "Diriku",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "diu",
    individual_languages: &[
    ],
};


pub const DIW: LanguageCode = LanguageCode {
    name: "Northwestern Dinka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "diw",
    individual_languages: &[
    ],
};


pub const DIX: LanguageCode = LanguageCode {
    name: "Dixon Reef",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dix",
    individual_languages: &[
    ],
};


pub const DIY: LanguageCode = LanguageCode {
    name: "Diuwe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "diy",
    individual_languages: &[
    ],
};


pub const DIZ: LanguageCode = LanguageCode {
    name: "Ding",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "diz",
    individual_languages: &[
    ],
};


pub const DJA: LanguageCode = LanguageCode {
    name: "Djadjawurrung",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dja",
    individual_languages: &[
    ],
};


pub const DJB: LanguageCode = LanguageCode {
    name: "Djinba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "djb",
    individual_languages: &[
    ],
};


pub const DJC: LanguageCode = LanguageCode {
    name: "Dar Daju Daju",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "djc",
    individual_languages: &[
    ],
};


pub const DJD: LanguageCode = LanguageCode {
    name: "Djamindjung",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "djd",
    individual_languages: &[
    ],
};


pub const DJE: LanguageCode = LanguageCode {
    name: "Zarma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dje",
    individual_languages: &[
    ],
};


pub const DJF: LanguageCode = LanguageCode {
    name: "Djangun",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "djf",
    individual_languages: &[
    ],
};


pub const DJI: LanguageCode = LanguageCode {
    name: "Djinang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dji",
    individual_languages: &[
    ],
};


pub const DJJ: LanguageCode = LanguageCode {
    name: "Djeebbana",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "djj",
    individual_languages: &[
    ],
};


pub const DJK: LanguageCode = LanguageCode {
    name: "Eastern Maroon Creole",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "djk",
    individual_languages: &[
    ],
};


pub const DJM: LanguageCode = LanguageCode {
    name: "Jamsay Dogon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "djm",
    individual_languages: &[
    ],
};


pub const DJN: LanguageCode = LanguageCode {
    name: "Jawoyn",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "djn",
    individual_languages: &[
    ],
};


pub const DJO: LanguageCode = LanguageCode {
    name: "Jangkang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "djo",
    individual_languages: &[
    ],
};


pub const DJR: LanguageCode = LanguageCode {
    name: "Djambarrpuyngu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "djr",
    individual_languages: &[
    ],
};


pub const DJU: LanguageCode = LanguageCode {
    name: "Kapriman",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dju",
    individual_languages: &[
    ],
};


pub const DJW: LanguageCode = LanguageCode {
    name: "Djawi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "djw",
    individual_languages: &[
    ],
};


pub const DKA: LanguageCode = LanguageCode {
    name: "Dakpakha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dka",
    individual_languages: &[
    ],
};


pub const DKG: LanguageCode = LanguageCode {
    name: "Kadung",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dkg",
    individual_languages: &[
    ],
};


pub const DKK: LanguageCode = LanguageCode {
    name: "Dakka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dkk",
    individual_languages: &[
    ],
};


pub const DKR: LanguageCode = LanguageCode {
    name: "Kuijau",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dkr",
    individual_languages: &[
    ],
};


pub const DKS: LanguageCode = LanguageCode {
    name: "Southeastern Dinka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dks",
    individual_languages: &[
    ],
};


pub const DKX: LanguageCode = LanguageCode {
    name: "Mazagway",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dkx",
    individual_languages: &[
    ],
};


pub const DLG: LanguageCode = LanguageCode {
    name: "Dolgan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dlg",
    individual_languages: &[
    ],
};


pub const DLK: LanguageCode = LanguageCode {
    name: "Dahalik",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dlk",
    individual_languages: &[
    ],
};


pub const DLM: LanguageCode = LanguageCode {
    name: "Dalmatian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dlm",
    individual_languages: &[
    ],
};


pub const DLN: LanguageCode = LanguageCode {
    name: "Darlong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dln",
    individual_languages: &[
    ],
};


pub const DMA: LanguageCode = LanguageCode {
    name: "Duma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dma",
    individual_languages: &[
    ],
};


pub const DMB: LanguageCode = LanguageCode {
    name: "Mombo Dogon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dmb",
    individual_languages: &[
    ],
};


pub const DMC: LanguageCode = LanguageCode {
    name: "Gavak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dmc",
    individual_languages: &[
    ],
};


pub const DMD: LanguageCode = LanguageCode {
    name: "Madhi Madhi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dmd",
    individual_languages: &[
    ],
};


pub const DME: LanguageCode = LanguageCode {
    name: "Dugwor",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dme",
    individual_languages: &[
    ],
};


pub const DMF: LanguageCode = LanguageCode {
    name: "Medefaidrin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dmf",
    individual_languages: &[
    ],
};


pub const DMG: LanguageCode = LanguageCode {
    name: "Upper Kinabatangan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dmg",
    individual_languages: &[
    ],
};


pub const DMK: LanguageCode = LanguageCode {
    name: "Domaaki",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dmk",
    individual_languages: &[
    ],
};


pub const DML: LanguageCode = LanguageCode {
    name: "Dameli",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dml",
    individual_languages: &[
    ],
};


pub const DMM: LanguageCode = LanguageCode {
    name: "Dama",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dmm",
    individual_languages: &[
    ],
};


pub const DMO: LanguageCode = LanguageCode {
    name: "Kemedzung",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dmo",
    individual_languages: &[
    ],
};


pub const DMR: LanguageCode = LanguageCode {
    name: "East Damar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dmr",
    individual_languages: &[
    ],
};


pub const DMS: LanguageCode = LanguageCode {
    name: "Dampelas",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dms",
    individual_languages: &[
    ],
};


pub const DMU: LanguageCode = LanguageCode {
    name: "Dubu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dmu",
    individual_languages: &[
    ],
};


pub const DMV: LanguageCode = LanguageCode {
    name: "Dumpas",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dmv",
    individual_languages: &[
    ],
};


pub const DMW: LanguageCode = LanguageCode {
    name: "Mudburra",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dmw",
    individual_languages: &[
    ],
};


pub const DMX: LanguageCode = LanguageCode {
    name: "Dema",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dmx",
    individual_languages: &[
    ],
};


pub const DMY: LanguageCode = LanguageCode {
    name: "Demta",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dmy",
    individual_languages: &[
    ],
};


pub const DNA: LanguageCode = LanguageCode {
    name: "Upper Grand Valley Dani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dna",
    individual_languages: &[
    ],
};


pub const DND: LanguageCode = LanguageCode {
    name: "Daonda",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dnd",
    individual_languages: &[
    ],
};


pub const DNE: LanguageCode = LanguageCode {
    name: "Ndendeule",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dne",
    individual_languages: &[
    ],
};


pub const DNG: LanguageCode = LanguageCode {
    name: "Dungan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dng",
    individual_languages: &[
    ],
};


pub const DNI: LanguageCode = LanguageCode {
    name: "Lower Grand Valley Dani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dni",
    individual_languages: &[
    ],
};


pub const DNJ: LanguageCode = LanguageCode {
    name: "Dan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dnj",
    individual_languages: &[
    ],
};


pub const DNK: LanguageCode = LanguageCode {
    name: "Dengka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dnk",
    individual_languages: &[
    ],
};


pub const DNN: LanguageCode = LanguageCode {
    name: "Dzùùngoo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dnn",
    individual_languages: &[
    ],
};


pub const DNO: LanguageCode = LanguageCode {
    name: "Ndrulo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dno",
    individual_languages: &[
    ],
};


pub const DNR: LanguageCode = LanguageCode {
    name: "Danaru",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dnr",
    individual_languages: &[
    ],
};


pub const DNT: LanguageCode = LanguageCode {
    name: "Mid Grand Valley Dani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dnt",
    individual_languages: &[
    ],
};


pub const DNU: LanguageCode = LanguageCode {
    name: "Danau",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dnu",
    individual_languages: &[
    ],
};


pub const DNV: LanguageCode = LanguageCode {
    name: "Danu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dnv",
    individual_languages: &[
    ],
};


pub const DNW: LanguageCode = LanguageCode {
    name: "Western Dani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dnw",
    individual_languages: &[
    ],
};


pub const DNY: LanguageCode = LanguageCode {
    name: "Dení",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dny",
    individual_languages: &[
    ],
};


pub const DOA: LanguageCode = LanguageCode {
    name: "Dom",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "doa",
    individual_languages: &[
    ],
};


pub const DOB: LanguageCode = LanguageCode {
    name: "Dobu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dob",
    individual_languages: &[
    ],
};


pub const DOC: LanguageCode = LanguageCode {
    name: "Northern Dong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "doc",
    individual_languages: &[
    ],
};


pub const DOE: LanguageCode = LanguageCode {
    name: "Doe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "doe",
    individual_languages: &[
    ],
};


pub const DOF: LanguageCode = LanguageCode {
    name: "Domu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dof",
    individual_languages: &[
    ],
};


pub const DOH: LanguageCode = LanguageCode {
    name: "Dong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "doh",
    individual_languages: &[
    ],
};


pub const DOK: LanguageCode = LanguageCode {
    name: "Dondo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dok",
    individual_languages: &[
    ],
};


pub const DOL: LanguageCode = LanguageCode {
    name: "Doso",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dol",
    individual_languages: &[
    ],
};


pub const DON: LanguageCode = LanguageCode {
    name: "Toura (Papua New Guinea)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "don",
    individual_languages: &[
    ],
};


pub const DOO: LanguageCode = LanguageCode {
    name: "Dongo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "doo",
    individual_languages: &[
    ],
};


pub const DOP: LanguageCode = LanguageCode {
    name: "Lukpa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dop",
    individual_languages: &[
    ],
};


pub const DOQ: LanguageCode = LanguageCode {
    name: "Dominican Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "doq",
    individual_languages: &[
    ],
};


pub const DOR: LanguageCode = LanguageCode {
    name: "Dori'o",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dor",
    individual_languages: &[
    ],
};


pub const DOS: LanguageCode = LanguageCode {
    name: "Dogosé",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dos",
    individual_languages: &[
    ],
};


pub const DOT: LanguageCode = LanguageCode {
    name: "Dass",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dot",
    individual_languages: &[
    ],
};


pub const DOV: LanguageCode = LanguageCode {
    name: "Dombe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dov",
    individual_languages: &[
    ],
};


pub const DOW: LanguageCode = LanguageCode {
    name: "Doyayo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dow",
    individual_languages: &[
    ],
};


pub const DOX: LanguageCode = LanguageCode {
    name: "Bussa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dox",
    individual_languages: &[
    ],
};


pub const DOY: LanguageCode = LanguageCode {
    name: "Dompo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "doy",
    individual_languages: &[
    ],
};


pub const DOZ: LanguageCode = LanguageCode {
    name: "Dorze",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "doz",
    individual_languages: &[
    ],
};


pub const DPP: LanguageCode = LanguageCode {
    name: "Papar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dpp",
    individual_languages: &[
    ],
};


pub const DRB: LanguageCode = LanguageCode {
    name: "Dair",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "drb",
    individual_languages: &[
    ],
};


pub const DRC: LanguageCode = LanguageCode {
    name: "Minderico",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "drc",
    individual_languages: &[
    ],
};


pub const DRD: LanguageCode = LanguageCode {
    name: "Darmiya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "drd",
    individual_languages: &[
    ],
};


pub const DRE: LanguageCode = LanguageCode {
    name: "Dolpo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dre",
    individual_languages: &[
    ],
};


pub const DRG: LanguageCode = LanguageCode {
    name: "Rungus",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "drg",
    individual_languages: &[
    ],
};


pub const DRI: LanguageCode = LanguageCode {
    name: "C'Lela",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dri",
    individual_languages: &[
    ],
};


pub const DRL: LanguageCode = LanguageCode {
    name: "Paakantyi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "drl",
    individual_languages: &[
    ],
};


pub const DRN: LanguageCode = LanguageCode {
    name: "West Damar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "drn",
    individual_languages: &[
    ],
};


pub const DRO: LanguageCode = LanguageCode {
    name: "Daro-Matu Melanau",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dro",
    individual_languages: &[
    ],
};


pub const DRQ: LanguageCode = LanguageCode {
    name: "Dura",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "drq",
    individual_languages: &[
    ],
};


pub const DRS: LanguageCode = LanguageCode {
    name: "Gedeo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "drs",
    individual_languages: &[
    ],
};


pub const DRT: LanguageCode = LanguageCode {
    name: "Drents",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "drt",
    individual_languages: &[
    ],
};


pub const DRU: LanguageCode = LanguageCode {
    name: "Rukai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dru",
    individual_languages: &[
    ],
};


pub const DRY: LanguageCode = LanguageCode {
    name: "Darai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dry",
    individual_languages: &[
    ],
};


pub const DSE: LanguageCode = LanguageCode {
    name: "Dutch Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dse",
    individual_languages: &[
    ],
};


pub const DSH: LanguageCode = LanguageCode {
    name: "Daasanach",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dsh",
    individual_languages: &[
    ],
};


pub const DSI: LanguageCode = LanguageCode {
    name: "Disa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dsi",
    individual_languages: &[
    ],
};


pub const DSL: LanguageCode = LanguageCode {
    name: "Danish Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dsl",
    individual_languages: &[
    ],
};


pub const DSN: LanguageCode = LanguageCode {
    name: "Dusner",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dsn",
    individual_languages: &[
    ],
};


pub const DSO: LanguageCode = LanguageCode {
    name: "Desiya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dso",
    individual_languages: &[
    ],
};


pub const DSQ: LanguageCode = LanguageCode {
    name: "Tadaksahak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dsq",
    individual_languages: &[
    ],
};


pub const DSZ: LanguageCode = LanguageCode {
    name: "Mardin Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dsz",
    individual_languages: &[
    ],
};


pub const DTA: LanguageCode = LanguageCode {
    name: "Daur",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dta",
    individual_languages: &[
    ],
};


pub const DTB: LanguageCode = LanguageCode {
    name: "Labuk-Kinabatangan Kadazan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dtb",
    individual_languages: &[
    ],
};


pub const DTD: LanguageCode = LanguageCode {
    name: "Ditidaht",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dtd",
    individual_languages: &[
    ],
};


pub const DTH: LanguageCode = LanguageCode {
    name: "Adithinngithigh",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dth",
    individual_languages: &[
    ],
};


pub const DTI: LanguageCode = LanguageCode {
    name: "Ana Tinga Dogon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dti",
    individual_languages: &[
    ],
};


pub const DTK: LanguageCode = LanguageCode {
    name: "Tene Kan Dogon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dtk",
    individual_languages: &[
    ],
};


pub const DTM: LanguageCode = LanguageCode {
    name: "Tomo Kan Dogon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dtm",
    individual_languages: &[
    ],
};


pub const DTN: LanguageCode = LanguageCode {
    name: "Daatsʼíin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dtn",
    individual_languages: &[
    ],
};


pub const DTO: LanguageCode = LanguageCode {
    name: "Tommo So Dogon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dto",
    individual_languages: &[
    ],
};


pub const DTP: LanguageCode = LanguageCode {
    name: "Kadazan Dusun",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dtp",
    individual_languages: &[
    ],
};


pub const DTR: LanguageCode = LanguageCode {
    name: "Lotud",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dtr",
    individual_languages: &[
    ],
};


pub const DTS: LanguageCode = LanguageCode {
    name: "Toro So Dogon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dts",
    individual_languages: &[
    ],
};


pub const DTT: LanguageCode = LanguageCode {
    name: "Toro Tegu Dogon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dtt",
    individual_languages: &[
    ],
};


pub const DTU: LanguageCode = LanguageCode {
    name: "Tebul Ure Dogon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dtu",
    individual_languages: &[
    ],
};


pub const DTY: LanguageCode = LanguageCode {
    name: "Dotyali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dty",
    individual_languages: &[
    ],
};


pub const DUB: LanguageCode = LanguageCode {
    name: "Dubli",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dub",
    individual_languages: &[
    ],
};


pub const DUC: LanguageCode = LanguageCode {
    name: "Duna",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "duc",
    individual_languages: &[
    ],
};


pub const DUE: LanguageCode = LanguageCode {
    name: "Umiray Dumaget Agta",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "due",
    individual_languages: &[
    ],
};


pub const DUF: LanguageCode = LanguageCode {
    name: "Dumbea",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "duf",
    individual_languages: &[
    ],
};


pub const DUG: LanguageCode = LanguageCode {
    name: "Duruma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dug",
    individual_languages: &[
    ],
};


pub const DUH: LanguageCode = LanguageCode {
    name: "Dungra Bhil",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "duh",
    individual_languages: &[
    ],
};


pub const DUI: LanguageCode = LanguageCode {
    name: "Dumun",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dui",
    individual_languages: &[
    ],
};


pub const DUK: LanguageCode = LanguageCode {
    name: "Uyajitaya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "duk",
    individual_languages: &[
    ],
};


pub const DUL: LanguageCode = LanguageCode {
    name: "Alabat Island Agta",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dul",
    individual_languages: &[
    ],
};


pub const DUN: LanguageCode = LanguageCode {
    name: "Dusun Deyah",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dun",
    individual_languages: &[
    ],
};


pub const DUO: LanguageCode = LanguageCode {
    name: "Dupaninan Agta",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "duo",
    individual_languages: &[
    ],
};


pub const DUP: LanguageCode = LanguageCode {
    name: "Duano",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dup",
    individual_languages: &[
    ],
};


pub const DUQ: LanguageCode = LanguageCode {
    name: "Dusun Malang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "duq",
    individual_languages: &[
    ],
};


pub const DUR: LanguageCode = LanguageCode {
    name: "Dii",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dur",
    individual_languages: &[
    ],
};


pub const DUS: LanguageCode = LanguageCode {
    name: "Dumi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dus",
    individual_languages: &[
    ],
};


pub const DUU: LanguageCode = LanguageCode {
    name: "Drung",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "duu",
    individual_languages: &[
    ],
};


pub const DUV: LanguageCode = LanguageCode {
    name: "Duvle",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "duv",
    individual_languages: &[
    ],
};


pub const DUW: LanguageCode = LanguageCode {
    name: "Dusun Witu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "duw",
    individual_languages: &[
    ],
};


pub const DUX: LanguageCode = LanguageCode {
    name: "Duungooma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dux",
    individual_languages: &[
    ],
};


pub const DUY: LanguageCode = LanguageCode {
    name: "Dicamay Agta",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "duy",
    individual_languages: &[
    ],
};


pub const DUZ: LanguageCode = LanguageCode {
    name: "Duli-Gey",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "duz",
    individual_languages: &[
    ],
};


pub const DVA: LanguageCode = LanguageCode {
    name: "Duau",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dva",
    individual_languages: &[
    ],
};


pub const DWA: LanguageCode = LanguageCode {
    name: "Diri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dwa",
    individual_languages: &[
    ],
};


pub const DWK: LanguageCode = LanguageCode {
    name: "Dawik Kui",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dwk",
    individual_languages: &[
    ],
};


pub const DWR: LanguageCode = LanguageCode {
    name: "Dawro",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dwr",
    individual_languages: &[
    ],
};


pub const DWS: LanguageCode = LanguageCode {
    name: "Dutton World Speedwords",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dws",
    individual_languages: &[
    ],
};


pub const DWU: LanguageCode = LanguageCode {
    name: "Dhuwal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dwu",
    individual_languages: &[
    ],
};


pub const DWW: LanguageCode = LanguageCode {
    name: "Dawawa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dww",
    individual_languages: &[
    ],
};


pub const DWY: LanguageCode = LanguageCode {
    name: "Dhuwaya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dwy",
    individual_languages: &[
    ],
};


pub const DWZ: LanguageCode = LanguageCode {
    name: "Dewas Rai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dwz",
    individual_languages: &[
    ],
};


pub const DYA: LanguageCode = LanguageCode {
    name: "Dyan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dya",
    individual_languages: &[
    ],
};


pub const DYB: LanguageCode = LanguageCode {
    name: "Dyaberdyaber",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dyb",
    individual_languages: &[
    ],
};


pub const DYD: LanguageCode = LanguageCode {
    name: "Dyugun",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dyd",
    individual_languages: &[
    ],
};


pub const DYG: LanguageCode = LanguageCode {
    name: "Villa Viciosa Agta",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dyg",
    individual_languages: &[
    ],
};


pub const DYI: LanguageCode = LanguageCode {
    name: "Djimini Senoufo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dyi",
    individual_languages: &[
    ],
};


pub const DYM: LanguageCode = LanguageCode {
    name: "Yanda Dom Dogon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dym",
    individual_languages: &[
    ],
};


pub const DYN: LanguageCode = LanguageCode {
    name: "Dyangadi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dyn",
    individual_languages: &[
    ],
};


pub const DYO: LanguageCode = LanguageCode {
    name: "Jola-Fonyi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dyo",
    individual_languages: &[
    ],
};


pub const DYY: LanguageCode = LanguageCode {
    name: "Djabugay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dyy",
    individual_languages: &[
    ],
};


pub const DZA: LanguageCode = LanguageCode {
    name: "Tunzu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dza",
    individual_languages: &[
    ],
};


pub const DZE: LanguageCode = LanguageCode {
    name: "Djiwarli",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dze",
    individual_languages: &[
    ],
};


pub const DZG: LanguageCode = LanguageCode {
    name: "Dazaga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dzg",
    individual_languages: &[
    ],
};


pub const DZL: LanguageCode = LanguageCode {
    name: "Dzalakha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dzl",
    individual_languages: &[
    ],
};


pub const DZN: LanguageCode = LanguageCode {
    name: "Dzando",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "dzn",
    individual_languages: &[
    ],
};


pub const EAA: LanguageCode = LanguageCode {
    name: "Karenggapa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "eaa",
    individual_languages: &[
    ],
};


pub const EBC: LanguageCode = LanguageCode {
    name: "Beginci",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ebc",
    individual_languages: &[
    ],
};


pub const EBG: LanguageCode = LanguageCode {
    name: "Ebughu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ebg",
    individual_languages: &[
    ],
};


pub const EBK: LanguageCode = LanguageCode {
    name: "Eastern Bontok",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ebk",
    individual_languages: &[
    ],
};


pub const EBO: LanguageCode = LanguageCode {
    name: "Teke-Ebo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ebo",
    individual_languages: &[
    ],
};


pub const EBR: LanguageCode = LanguageCode {
    name: "Ebrié",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ebr",
    individual_languages: &[
    ],
};


pub const EBU: LanguageCode = LanguageCode {
    name: "Embu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ebu",
    individual_languages: &[
    ],
};


pub const ECR: LanguageCode = LanguageCode {
    name: "Eteocretan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ecr",
    individual_languages: &[
    ],
};


pub const ECS: LanguageCode = LanguageCode {
    name: "Ecuadorian Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ecs",
    individual_languages: &[
    ],
};


pub const ECY: LanguageCode = LanguageCode {
    name: "Eteocypriot",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ecy",
    individual_languages: &[
    ],
};


pub const EEE: LanguageCode = LanguageCode {
    name: "E",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "eee",
    individual_languages: &[
    ],
};


pub const EFA: LanguageCode = LanguageCode {
    name: "Efai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "efa",
    individual_languages: &[
    ],
};


pub const EFE: LanguageCode = LanguageCode {
    name: "Efe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "efe",
    individual_languages: &[
    ],
};


pub const EGA: LanguageCode = LanguageCode {
    name: "Ega",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ega",
    individual_languages: &[
    ],
};


pub const EGL: LanguageCode = LanguageCode {
    name: "Emilian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "egl",
    individual_languages: &[
    ],
};


pub const EGM: LanguageCode = LanguageCode {
    name: "Benamanga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "egm",
    individual_languages: &[
    ],
};


pub const EGO: LanguageCode = LanguageCode {
    name: "Eggon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ego",
    individual_languages: &[
    ],
};


pub const EHS: LanguageCode = LanguageCode {
    name: "Miyakubo Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ehs",
    individual_languages: &[
    ],
};


pub const EHU: LanguageCode = LanguageCode {
    name: "Ehueun",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ehu",
    individual_languages: &[
    ],
};


pub const EIP: LanguageCode = LanguageCode {
    name: "Eipomek",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "eip",
    individual_languages: &[
    ],
};


pub const EIT: LanguageCode = LanguageCode {
    name: "Eitiep",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "eit",
    individual_languages: &[
    ],
};


pub const EIV: LanguageCode = LanguageCode {
    name: "Askopan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "eiv",
    individual_languages: &[
    ],
};


pub const EJA: LanguageCode = LanguageCode {
    name: "Ejamat",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "eja",
    individual_languages: &[
    ],
};


pub const EKE: LanguageCode = LanguageCode {
    name: "Ekit",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "eke",
    individual_languages: &[
    ],
};


pub const EKG: LanguageCode = LanguageCode {
    name: "Ekari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ekg",
    individual_languages: &[
    ],
};


pub const EKI: LanguageCode = LanguageCode {
    name: "Eki",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "eki",
    individual_languages: &[
    ],
};


pub const EKK: LanguageCode = LanguageCode {
    name: "Standard Estonian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ekk",
    individual_languages: &[
    ],
};


pub const EKL: LanguageCode = LanguageCode {
    name: "Kol (Bangladesh)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ekl",
    individual_languages: &[
    ],
};


pub const EKM: LanguageCode = LanguageCode {
    name: "Elip",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ekm",
    individual_languages: &[
    ],
};


pub const EKO: LanguageCode = LanguageCode {
    name: "Koti",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "eko",
    individual_languages: &[
    ],
};


pub const EKP: LanguageCode = LanguageCode {
    name: "Ekpeye",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ekp",
    individual_languages: &[
    ],
};


pub const EKR: LanguageCode = LanguageCode {
    name: "Yace",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ekr",
    individual_languages: &[
    ],
};


pub const EKY: LanguageCode = LanguageCode {
    name: "Eastern Kayah",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "eky",
    individual_languages: &[
    ],
};


pub const ELE: LanguageCode = LanguageCode {
    name: "Elepi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ele",
    individual_languages: &[
    ],
};


pub const ELH: LanguageCode = LanguageCode {
    name: "El Hugeirat",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "elh",
    individual_languages: &[
    ],
};


pub const ELI: LanguageCode = LanguageCode {
    name: "Nding",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "eli",
    individual_languages: &[
    ],
};


pub const ELK: LanguageCode = LanguageCode {
    name: "Elkei",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "elk",
    individual_languages: &[
    ],
};


pub const ELM: LanguageCode = LanguageCode {
    name: "Eleme",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "elm",
    individual_languages: &[
    ],
};


pub const ELO: LanguageCode = LanguageCode {
    name: "El Molo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "elo",
    individual_languages: &[
    ],
};


pub const ELU: LanguageCode = LanguageCode {
    name: "Elu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "elu",
    individual_languages: &[
    ],
};


pub const EMA: LanguageCode = LanguageCode {
    name: "Emai-Iuleha-Ora",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ema",
    individual_languages: &[
    ],
};


pub const EMB: LanguageCode = LanguageCode {
    name: "Embaloh",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "emb",
    individual_languages: &[
    ],
};


pub const EME: LanguageCode = LanguageCode {
    name: "Emerillon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "eme",
    individual_languages: &[
    ],
};


pub const EMG: LanguageCode = LanguageCode {
    name: "Eastern Meohang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "emg",
    individual_languages: &[
    ],
};


pub const EMI: LanguageCode = LanguageCode {
    name: "Mussau-Emira",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "emi",
    individual_languages: &[
    ],
};


pub const EMK: LanguageCode = LanguageCode {
    name: "Eastern Maninkakan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "emk",
    individual_languages: &[
    ],
};


pub const EMM: LanguageCode = LanguageCode {
    name: "Mamulique",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "emm",
    individual_languages: &[
    ],
};


pub const EMN: LanguageCode = LanguageCode {
    name: "Eman",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "emn",
    individual_languages: &[
    ],
};


pub const EMP: LanguageCode = LanguageCode {
    name: "Northern Emberá",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "emp",
    individual_languages: &[
    ],
};


pub const EMQ: LanguageCode = LanguageCode {
    name: "Eastern Minyag",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "emq",
    individual_languages: &[
    ],
};


pub const EMS: LanguageCode = LanguageCode {
    name: "Pacific Gulf Yupik",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ems",
    individual_languages: &[
    ],
};


pub const EMU: LanguageCode = LanguageCode {
    name: "Eastern Muria",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "emu",
    individual_languages: &[
    ],
};


pub const EMW: LanguageCode = LanguageCode {
    name: "Emplawas",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "emw",
    individual_languages: &[
    ],
};


pub const EMX: LanguageCode = LanguageCode {
    name: "Erromintxela",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "emx",
    individual_languages: &[
    ],
};


pub const EMY: LanguageCode = LanguageCode {
    name: "Epigraphic Mayan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "emy",
    individual_languages: &[
    ],
};


pub const EMZ: LanguageCode = LanguageCode {
    name: "Mbessa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "emz",
    individual_languages: &[
    ],
};


pub const ENA: LanguageCode = LanguageCode {
    name: "Apali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ena",
    individual_languages: &[
    ],
};


pub const ENB: LanguageCode = LanguageCode {
    name: "Markweeta",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "enb",
    individual_languages: &[
    ],
};


pub const ENC: LanguageCode = LanguageCode {
    name: "En",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "enc",
    individual_languages: &[
    ],
};


pub const END: LanguageCode = LanguageCode {
    name: "Ende",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "end",
    individual_languages: &[
    ],
};


pub const ENF: LanguageCode = LanguageCode {
    name: "Forest Enets",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "enf",
    individual_languages: &[
    ],
};


pub const ENH: LanguageCode = LanguageCode {
    name: "Tundra Enets",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "enh",
    individual_languages: &[
    ],
};


pub const ENL: LanguageCode = LanguageCode {
    name: "Enlhet",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "enl",
    individual_languages: &[
    ],
};


pub const ENN: LanguageCode = LanguageCode {
    name: "Engenni",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "enn",
    individual_languages: &[
    ],
};


pub const ENO: LanguageCode = LanguageCode {
    name: "Enggano",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "eno",
    individual_languages: &[
    ],
};


pub const ENQ: LanguageCode = LanguageCode {
    name: "Enga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "enq",
    individual_languages: &[
    ],
};


pub const ENR: LanguageCode = LanguageCode {
    name: "Emumu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "enr",
    individual_languages: &[
    ],
};


pub const ENU: LanguageCode = LanguageCode {
    name: "Enu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "enu",
    individual_languages: &[
    ],
};


pub const ENV: LanguageCode = LanguageCode {
    name: "Enwan (Edo State)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "env",
    individual_languages: &[
    ],
};


pub const ENW: LanguageCode = LanguageCode {
    name: "Enwan (Akwa Ibom State)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "enw",
    individual_languages: &[
    ],
};


pub const ENX: LanguageCode = LanguageCode {
    name: "Enxet",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "enx",
    individual_languages: &[
    ],
};


pub const EOT: LanguageCode = LanguageCode {
    name: "Beti (Côte d'Ivoire)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "eot",
    individual_languages: &[
    ],
};


pub const EPI: LanguageCode = LanguageCode {
    name: "Epie",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "epi",
    individual_languages: &[
    ],
};


pub const ERA: LanguageCode = LanguageCode {
    name: "Eravallan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "era",
    individual_languages: &[
    ],
};


pub const ERG: LanguageCode = LanguageCode {
    name: "Sie",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "erg",
    individual_languages: &[
    ],
};


pub const ERH: LanguageCode = LanguageCode {
    name: "Eruwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "erh",
    individual_languages: &[
    ],
};


pub const ERI: LanguageCode = LanguageCode {
    name: "Ogea",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "eri",
    individual_languages: &[
    ],
};


pub const ERK: LanguageCode = LanguageCode {
    name: "South Efate",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "erk",
    individual_languages: &[
    ],
};


pub const ERO: LanguageCode = LanguageCode {
    name: "Horpa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ero",
    individual_languages: &[
    ],
};


pub const ERR: LanguageCode = LanguageCode {
    name: "Erre",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "err",
    individual_languages: &[
    ],
};


pub const ERS: LanguageCode = LanguageCode {
    name: "Ersu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ers",
    individual_languages: &[
    ],
};


pub const ERT: LanguageCode = LanguageCode {
    name: "Eritai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ert",
    individual_languages: &[
    ],
};


pub const ERW: LanguageCode = LanguageCode {
    name: "Erokwanas",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "erw",
    individual_languages: &[
    ],
};


pub const ESE: LanguageCode = LanguageCode {
    name: "Ese Ejja",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ese",
    individual_languages: &[
    ],
};


pub const ESG: LanguageCode = LanguageCode {
    name: "Aheri Gondi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "esg",
    individual_languages: &[
    ],
};


pub const ESH: LanguageCode = LanguageCode {
    name: "Eshtehardi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "esh",
    individual_languages: &[
    ],
};


pub const ESI: LanguageCode = LanguageCode {
    name: "North Alaskan Inupiatun",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "esi",
    individual_languages: &[
    ],
};


pub const ESK: LanguageCode = LanguageCode {
    name: "Northwest Alaska Inupiatun",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "esk",
    individual_languages: &[
    ],
};


pub const ESL: LanguageCode = LanguageCode {
    name: "Egypt Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "esl",
    individual_languages: &[
    ],
};


pub const ESM: LanguageCode = LanguageCode {
    name: "Esuma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "esm",
    individual_languages: &[
    ],
};


pub const ESN: LanguageCode = LanguageCode {
    name: "Salvadoran Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "esn",
    individual_languages: &[
    ],
};


pub const ESO: LanguageCode = LanguageCode {
    name: "Estonian Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "eso",
    individual_languages: &[
    ],
};


pub const ESQ: LanguageCode = LanguageCode {
    name: "Esselen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "esq",
    individual_languages: &[
    ],
};


pub const ESS: LanguageCode = LanguageCode {
    name: "Central Siberian Yupik",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ess",
    individual_languages: &[
    ],
};


pub const ESU: LanguageCode = LanguageCode {
    name: "Central Yupik",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "esu",
    individual_languages: &[
    ],
};


pub const ESY: LanguageCode = LanguageCode {
    name: "Eskayan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "esy",
    individual_languages: &[
    ],
};


pub const ETB: LanguageCode = LanguageCode {
    name: "Etebi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "etb",
    individual_languages: &[
    ],
};


pub const ETC: LanguageCode = LanguageCode {
    name: "Etchemin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "etc",
    individual_languages: &[
    ],
};


pub const ETH: LanguageCode = LanguageCode {
    name: "Ethiopian Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "eth",
    individual_languages: &[
    ],
};


pub const ETN: LanguageCode = LanguageCode {
    name: "Eton (Vanuatu)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "etn",
    individual_languages: &[
    ],
};


pub const ETO: LanguageCode = LanguageCode {
    name: "Eton (Cameroon)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "eto",
    individual_languages: &[
    ],
};


pub const ETR: LanguageCode = LanguageCode {
    name: "Edolo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "etr",
    individual_languages: &[
    ],
};


pub const ETS: LanguageCode = LanguageCode {
    name: "Yekhee",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ets",
    individual_languages: &[
    ],
};


pub const ETT: LanguageCode = LanguageCode {
    name: "Etruscan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ett",
    individual_languages: &[
    ],
};


pub const ETU: LanguageCode = LanguageCode {
    name: "Ejagham",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "etu",
    individual_languages: &[
    ],
};


pub const ETX: LanguageCode = LanguageCode {
    name: "Eten",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "etx",
    individual_languages: &[
    ],
};


pub const ETZ: LanguageCode = LanguageCode {
    name: "Semimi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "etz",
    individual_languages: &[
    ],
};


pub const EVE: LanguageCode = LanguageCode {
    name: "Even",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "eve",
    individual_languages: &[
    ],
};


pub const EVH: LanguageCode = LanguageCode {
    name: "Uvbie",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "evh",
    individual_languages: &[
    ],
};


pub const EVN: LanguageCode = LanguageCode {
    name: "Evenki",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "evn",
    individual_languages: &[
    ],
};


pub const EXT: LanguageCode = LanguageCode {
    name: "Extremaduran",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ext",
    individual_languages: &[
    ],
};


pub const EYA: LanguageCode = LanguageCode {
    name: "Eyak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "eya",
    individual_languages: &[
    ],
};


pub const EYO: LanguageCode = LanguageCode {
    name: "Keiyo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "eyo",
    individual_languages: &[
    ],
};


pub const EZA: LanguageCode = LanguageCode {
    name: "Ezaa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "eza",
    individual_languages: &[
    ],
};


pub const EZE: LanguageCode = LanguageCode {
    name: "Uzekwe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "eze",
    individual_languages: &[
    ],
};


pub const FAA: LanguageCode = LanguageCode {
    name: "Fasu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "faa",
    individual_languages: &[
    ],
};


pub const FAB: LanguageCode = LanguageCode {
    name: "Fa d'Ambu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fab",
    individual_languages: &[
    ],
};


pub const FAD: LanguageCode = LanguageCode {
    name: "Wagi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fad",
    individual_languages: &[
    ],
};


pub const FAF: LanguageCode = LanguageCode {
    name: "Fagani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "faf",
    individual_languages: &[
    ],
};


pub const FAG: LanguageCode = LanguageCode {
    name: "Finongan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fag",
    individual_languages: &[
    ],
};


pub const FAH: LanguageCode = LanguageCode {
    name: "Baissa Fali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fah",
    individual_languages: &[
    ],
};


pub const FAI: LanguageCode = LanguageCode {
    name: "Faiwol",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fai",
    individual_languages: &[
    ],
};


pub const FAJ: LanguageCode = LanguageCode {
    name: "Faita",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "faj",
    individual_languages: &[
    ],
};


pub const FAK: LanguageCode = LanguageCode {
    name: "Fang (Cameroon)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fak",
    individual_languages: &[
    ],
};


pub const FAL: LanguageCode = LanguageCode {
    name: "South Fali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fal",
    individual_languages: &[
    ],
};


pub const FAM: LanguageCode = LanguageCode {
    name: "Fam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fam",
    individual_languages: &[
    ],
};


pub const FAP: LanguageCode = LanguageCode {
    name: "Paloor",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fap",
    individual_languages: &[
    ],
};


pub const FAR: LanguageCode = LanguageCode {
    name: "Fataleka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "far",
    individual_languages: &[
    ],
};


pub const FAU: LanguageCode = LanguageCode {
    name: "Fayu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fau",
    individual_languages: &[
    ],
};


pub const FAX: LanguageCode = LanguageCode {
    name: "Fala",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fax",
    individual_languages: &[
    ],
};


pub const FAY: LanguageCode = LanguageCode {
    name: "Southwestern Fars",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fay",
    individual_languages: &[
    ],
};


pub const FAZ: LanguageCode = LanguageCode {
    name: "Northwestern Fars",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "faz",
    individual_languages: &[
    ],
};


pub const FBL: LanguageCode = LanguageCode {
    name: "West Albay Bikol",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fbl",
    individual_languages: &[
    ],
};


pub const FCS: LanguageCode = LanguageCode {
    name: "Quebec Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fcs",
    individual_languages: &[
    ],
};


pub const FER: LanguageCode = LanguageCode {
    name: "Feroge",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fer",
    individual_languages: &[
    ],
};


pub const FFI: LanguageCode = LanguageCode {
    name: "Foia Foia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ffi",
    individual_languages: &[
    ],
};


pub const FFM: LanguageCode = LanguageCode {
    name: "Maasina Fulfulde",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ffm",
    individual_languages: &[
    ],
};


pub const FGR: LanguageCode = LanguageCode {
    name: "Fongoro",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fgr",
    individual_languages: &[
    ],
};


pub const FIA: LanguageCode = LanguageCode {
    name: "Nobiin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fia",
    individual_languages: &[
    ],
};


pub const FIE: LanguageCode = LanguageCode {
    name: "Fyer",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fie",
    individual_languages: &[
    ],
};


pub const FIF: LanguageCode = LanguageCode {
    name: "Faifi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fif",
    individual_languages: &[
    ],
};


pub const FIP: LanguageCode = LanguageCode {
    name: "Fipa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fip",
    individual_languages: &[
    ],
};


pub const FIR: LanguageCode = LanguageCode {
    name: "Firan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fir",
    individual_languages: &[
    ],
};


pub const FIT: LanguageCode = LanguageCode {
    name: "Tornedalen Finnish",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fit",
    individual_languages: &[
    ],
};


pub const FIW: LanguageCode = LanguageCode {
    name: "Fiwaga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fiw",
    individual_languages: &[
    ],
};


pub const FKK: LanguageCode = LanguageCode {
    name: "Kirya-Konzəl",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fkk",
    individual_languages: &[
    ],
};


pub const FKV: LanguageCode = LanguageCode {
    name: "Kven Finnish",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fkv",
    individual_languages: &[
    ],
};


pub const FLA: LanguageCode = LanguageCode {
    name: "Kalispel-Pend d'Oreille",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fla",
    individual_languages: &[
    ],
};


pub const FLH: LanguageCode = LanguageCode {
    name: "Foau",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "flh",
    individual_languages: &[
    ],
};


pub const FLI: LanguageCode = LanguageCode {
    name: "Fali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fli",
    individual_languages: &[
    ],
};


pub const FLL: LanguageCode = LanguageCode {
    name: "North Fali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fll",
    individual_languages: &[
    ],
};


pub const FLN: LanguageCode = LanguageCode {
    name: "Flinders Island",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fln",
    individual_languages: &[
    ],
};


pub const FLR: LanguageCode = LanguageCode {
    name: "Fuliiru",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "flr",
    individual_languages: &[
    ],
};


pub const FLY: LanguageCode = LanguageCode {
    name: "Flaaitaal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fly",
    individual_languages: &[
    ],
};


pub const FMP: LanguageCode = LanguageCode {
    name: "Fe'fe'",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fmp",
    individual_languages: &[
    ],
};


pub const FMU: LanguageCode = LanguageCode {
    name: "Far Western Muria",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fmu",
    individual_languages: &[
    ],
};


pub const FNB: LanguageCode = LanguageCode {
    name: "Fanbak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fnb",
    individual_languages: &[
    ],
};


pub const FNG: LanguageCode = LanguageCode {
    name: "Fanagalo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fng",
    individual_languages: &[
    ],
};


pub const FNI: LanguageCode = LanguageCode {
    name: "Fania",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fni",
    individual_languages: &[
    ],
};


pub const FOD: LanguageCode = LanguageCode {
    name: "Foodo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fod",
    individual_languages: &[
    ],
};


pub const FOI: LanguageCode = LanguageCode {
    name: "Foi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "foi",
    individual_languages: &[
    ],
};


pub const FOM: LanguageCode = LanguageCode {
    name: "Foma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fom",
    individual_languages: &[
    ],
};


pub const FOR: LanguageCode = LanguageCode {
    name: "Fore",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "for",
    individual_languages: &[
    ],
};


pub const FOS: LanguageCode = LanguageCode {
    name: "Siraya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fos",
    individual_languages: &[
    ],
};


pub const FPE: LanguageCode = LanguageCode {
    name: "Fernando Po Creole English",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fpe",
    individual_languages: &[
    ],
};


pub const FQS: LanguageCode = LanguageCode {
    name: "Fas",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fqs",
    individual_languages: &[
    ],
};


pub const FRC: LanguageCode = LanguageCode {
    name: "Cajun French",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "frc",
    individual_languages: &[
    ],
};


pub const FRD: LanguageCode = LanguageCode {
    name: "Fordata",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "frd",
    individual_languages: &[
    ],
};


pub const FRK: LanguageCode = LanguageCode {
    name: "Frankish",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "frk",
    individual_languages: &[
    ],
};


pub const FRP: LanguageCode = LanguageCode {
    name: "Arpitan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "frp",
    individual_languages: &[
    ],
};


pub const FRQ: LanguageCode = LanguageCode {
    name: "Forak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "frq",
    individual_languages: &[
    ],
};


pub const FRT: LanguageCode = LanguageCode {
    name: "Fortsenal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "frt",
    individual_languages: &[
    ],
};


pub const FSE: LanguageCode = LanguageCode {
    name: "Finnish Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fse",
    individual_languages: &[
    ],
};


pub const FSL: LanguageCode = LanguageCode {
    name: "French Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fsl",
    individual_languages: &[
    ],
};


pub const FSS: LanguageCode = LanguageCode {
    name: "Finland-Swedish Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fss",
    individual_languages: &[
    ],
};


pub const FUB: LanguageCode = LanguageCode {
    name: "Adamawa Fulfulde",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fub",
    individual_languages: &[
    ],
};


pub const FUC: LanguageCode = LanguageCode {
    name: "Pulaar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fuc",
    individual_languages: &[
    ],
};


pub const FUD: LanguageCode = LanguageCode {
    name: "East Futuna",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fud",
    individual_languages: &[
    ],
};


pub const FUE: LanguageCode = LanguageCode {
    name: "Borgu Fulfulde",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fue",
    individual_languages: &[
    ],
};


pub const FUF: LanguageCode = LanguageCode {
    name: "Pular",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fuf",
    individual_languages: &[
    ],
};


pub const FUH: LanguageCode = LanguageCode {
    name: "Western Niger Fulfulde",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fuh",
    individual_languages: &[
    ],
};


pub const FUI: LanguageCode = LanguageCode {
    name: "Bagirmi Fulfulde",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fui",
    individual_languages: &[
    ],
};


pub const FUJ: LanguageCode = LanguageCode {
    name: "Ko",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fuj",
    individual_languages: &[
    ],
};


pub const FUM: LanguageCode = LanguageCode {
    name: "Fum",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fum",
    individual_languages: &[
    ],
};


pub const FUN: LanguageCode = LanguageCode {
    name: "Fulniô",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fun",
    individual_languages: &[
    ],
};


pub const FUQ: LanguageCode = LanguageCode {
    name: "Central-Eastern Niger Fulfulde",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fuq",
    individual_languages: &[
    ],
};


pub const FUT: LanguageCode = LanguageCode {
    name: "Futuna-Aniwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fut",
    individual_languages: &[
    ],
};


pub const FUU: LanguageCode = LanguageCode {
    name: "Furu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fuu",
    individual_languages: &[
    ],
};


pub const FUV: LanguageCode = LanguageCode {
    name: "Nigerian Fulfulde",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fuv",
    individual_languages: &[
    ],
};


pub const FUY: LanguageCode = LanguageCode {
    name: "Fuyug",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fuy",
    individual_languages: &[
    ],
};


pub const FVR: LanguageCode = LanguageCode {
    name: "Fur",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fvr",
    individual_languages: &[
    ],
};


pub const FWA: LanguageCode = LanguageCode {
    name: "Fwâi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fwa",
    individual_languages: &[
    ],
};


pub const FWE: LanguageCode = LanguageCode {
    name: "Fwe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "fwe",
    individual_languages: &[
    ],
};


pub const GAB: LanguageCode = LanguageCode {
    name: "Gabri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gab",
    individual_languages: &[
    ],
};


pub const GAC: LanguageCode = LanguageCode {
    name: "Mixed Great Andamanese",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gac",
    individual_languages: &[
    ],
};


pub const GAD: LanguageCode = LanguageCode {
    name: "Gaddang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gad",
    individual_languages: &[
    ],
};


pub const GAE: LanguageCode = LanguageCode {
    name: "Guarequena",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gae",
    individual_languages: &[
    ],
};


pub const GAF: LanguageCode = LanguageCode {
    name: "Gende",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gaf",
    individual_languages: &[
    ],
};


pub const GAG: LanguageCode = LanguageCode {
    name: "Gagauz",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gag",
    individual_languages: &[
    ],
};


pub const GAH: LanguageCode = LanguageCode {
    name: "Alekano",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gah",
    individual_languages: &[
    ],
};


pub const GAI: LanguageCode = LanguageCode {
    name: "Borei",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gai",
    individual_languages: &[
    ],
};


pub const GAJ: LanguageCode = LanguageCode {
    name: "Gadsup",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gaj",
    individual_languages: &[
    ],
};


pub const GAK: LanguageCode = LanguageCode {
    name: "Gamkonora",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gak",
    individual_languages: &[
    ],
};


pub const GAL: LanguageCode = LanguageCode {
    name: "Galolen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gal",
    individual_languages: &[
    ],
};


pub const GAM: LanguageCode = LanguageCode {
    name: "Kandawo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gam",
    individual_languages: &[
    ],
};


pub const GAN: LanguageCode = LanguageCode {
    name: "Gan Chinese",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gan",
    individual_languages: &[
    ],
};


pub const GAO: LanguageCode = LanguageCode {
    name: "Gants",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gao",
    individual_languages: &[
    ],
};


pub const GAP: LanguageCode = LanguageCode {
    name: "Gal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gap",
    individual_languages: &[
    ],
};


pub const GAQ: LanguageCode = LanguageCode {
    name: "Gata'",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gaq",
    individual_languages: &[
    ],
};


pub const GAR: LanguageCode = LanguageCode {
    name: "Galeya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gar",
    individual_languages: &[
    ],
};


pub const GAS: LanguageCode = LanguageCode {
    name: "Adiwasi Garasia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gas",
    individual_languages: &[
    ],
};


pub const GAT: LanguageCode = LanguageCode {
    name: "Kenati",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gat",
    individual_languages: &[
    ],
};


pub const GAU: LanguageCode = LanguageCode {
    name: "Mudhili Gadaba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gau",
    individual_languages: &[
    ],
};


pub const GAW: LanguageCode = LanguageCode {
    name: "Nobonob",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gaw",
    individual_languages: &[
    ],
};


pub const GAX: LanguageCode = LanguageCode {
    name: "Borana-Arsi-Guji Oromo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gax",
    individual_languages: &[
    ],
};


pub const GAZ: LanguageCode = LanguageCode {
    name: "West Central Oromo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gaz",
    individual_languages: &[
    ],
};


pub const GBB: LanguageCode = LanguageCode {
    name: "Kaytetye",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gbb",
    individual_languages: &[
    ],
};


pub const GBD: LanguageCode = LanguageCode {
    name: "Karajarri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gbd",
    individual_languages: &[
    ],
};


pub const GBE: LanguageCode = LanguageCode {
    name: "Niksek",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gbe",
    individual_languages: &[
    ],
};


pub const GBF: LanguageCode = LanguageCode {
    name: "Gaikundi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gbf",
    individual_languages: &[
    ],
};


pub const GBG: LanguageCode = LanguageCode {
    name: "Gbanziri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gbg",
    individual_languages: &[
    ],
};


pub const GBH: LanguageCode = LanguageCode {
    name: "Defi Gbe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gbh",
    individual_languages: &[
    ],
};


pub const GBI: LanguageCode = LanguageCode {
    name: "Galela",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gbi",
    individual_languages: &[
    ],
};


pub const GBJ: LanguageCode = LanguageCode {
    name: "Bodo Gadaba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gbj",
    individual_languages: &[
    ],
};


pub const GBK: LanguageCode = LanguageCode {
    name: "Gaddi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gbk",
    individual_languages: &[
    ],
};


pub const GBL: LanguageCode = LanguageCode {
    name: "Gamit",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gbl",
    individual_languages: &[
    ],
};


pub const GBM: LanguageCode = LanguageCode {
    name: "Garhwali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gbm",
    individual_languages: &[
    ],
};


pub const GBN: LanguageCode = LanguageCode {
    name: "Mo'da",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gbn",
    individual_languages: &[
    ],
};


pub const GBO: LanguageCode = LanguageCode {
    name: "Northern Grebo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gbo",
    individual_languages: &[
    ],
};


pub const GBP: LanguageCode = LanguageCode {
    name: "Gbaya-Bossangoa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gbp",
    individual_languages: &[
    ],
};


pub const GBQ: LanguageCode = LanguageCode {
    name: "Gbaya-Bozoum",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gbq",
    individual_languages: &[
    ],
};


pub const GBR: LanguageCode = LanguageCode {
    name: "Gbagyi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gbr",
    individual_languages: &[
    ],
};


pub const GBS: LanguageCode = LanguageCode {
    name: "Gbesi Gbe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gbs",
    individual_languages: &[
    ],
};


pub const GBU: LanguageCode = LanguageCode {
    name: "Gagadu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gbu",
    individual_languages: &[
    ],
};


pub const GBV: LanguageCode = LanguageCode {
    name: "Gbanu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gbv",
    individual_languages: &[
    ],
};


pub const GBW: LanguageCode = LanguageCode {
    name: "Gabi-Gabi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gbw",
    individual_languages: &[
    ],
};


pub const GBX: LanguageCode = LanguageCode {
    name: "Eastern Xwla Gbe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gbx",
    individual_languages: &[
    ],
};


pub const GBY: LanguageCode = LanguageCode {
    name: "Gbari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gby",
    individual_languages: &[
    ],
};


pub const GBZ: LanguageCode = LanguageCode {
    name: "Zoroastrian Dari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gbz",
    individual_languages: &[
    ],
};


pub const GCC: LanguageCode = LanguageCode {
    name: "Mali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gcc",
    individual_languages: &[
    ],
};


pub const GCD: LanguageCode = LanguageCode {
    name: "Ganggalida",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gcd",
    individual_languages: &[
    ],
};


pub const GCE: LanguageCode = LanguageCode {
    name: "Galice",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gce",
    individual_languages: &[
    ],
};


pub const GCF: LanguageCode = LanguageCode {
    name: "Guadeloupean Creole French",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gcf",
    individual_languages: &[
    ],
};


pub const GCL: LanguageCode = LanguageCode {
    name: "Grenadian Creole English",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gcl",
    individual_languages: &[
    ],
};


pub const GCN: LanguageCode = LanguageCode {
    name: "Gaina",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gcn",
    individual_languages: &[
    ],
};


pub const GCR: LanguageCode = LanguageCode {
    name: "Guianese Creole French",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gcr",
    individual_languages: &[
    ],
};


pub const GCT: LanguageCode = LanguageCode {
    name: "Colonia Tovar German",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gct",
    individual_languages: &[
    ],
};


pub const GDA: LanguageCode = LanguageCode {
    name: "Gade Lohar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gda",
    individual_languages: &[
    ],
};


pub const GDB: LanguageCode = LanguageCode {
    name: "Pottangi Ollar Gadaba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gdb",
    individual_languages: &[
    ],
};


pub const GDC: LanguageCode = LanguageCode {
    name: "Gugu Badhun",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gdc",
    individual_languages: &[
    ],
};


pub const GDD: LanguageCode = LanguageCode {
    name: "Gedaged",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gdd",
    individual_languages: &[
    ],
};


pub const GDE: LanguageCode = LanguageCode {
    name: "Gude",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gde",
    individual_languages: &[
    ],
};


pub const GDF: LanguageCode = LanguageCode {
    name: "Guduf-Gava",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gdf",
    individual_languages: &[
    ],
};


pub const GDG: LanguageCode = LanguageCode {
    name: "Ga'dang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gdg",
    individual_languages: &[
    ],
};


pub const GDH: LanguageCode = LanguageCode {
    name: "Gadjerawang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gdh",
    individual_languages: &[
    ],
};


pub const GDI: LanguageCode = LanguageCode {
    name: "Gundi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gdi",
    individual_languages: &[
    ],
};


pub const GDJ: LanguageCode = LanguageCode {
    name: "Gurdjar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gdj",
    individual_languages: &[
    ],
};


pub const GDK: LanguageCode = LanguageCode {
    name: "Gadang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gdk",
    individual_languages: &[
    ],
};


pub const GDL: LanguageCode = LanguageCode {
    name: "Dirasha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gdl",
    individual_languages: &[
    ],
};


pub const GDM: LanguageCode = LanguageCode {
    name: "Laal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gdm",
    individual_languages: &[
    ],
};


pub const GDN: LanguageCode = LanguageCode {
    name: "Umanakaina",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gdn",
    individual_languages: &[
    ],
};


pub const GDO: LanguageCode = LanguageCode {
    name: "Ghodoberi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gdo",
    individual_languages: &[
    ],
};


pub const GDQ: LanguageCode = LanguageCode {
    name: "Mehri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gdq",
    individual_languages: &[
    ],
};


pub const GDR: LanguageCode = LanguageCode {
    name: "Wipi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gdr",
    individual_languages: &[
    ],
};


pub const GDS: LanguageCode = LanguageCode {
    name: "Ghandruk Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gds",
    individual_languages: &[
    ],
};


pub const GDT: LanguageCode = LanguageCode {
    name: "Kungardutyi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gdt",
    individual_languages: &[
    ],
};


pub const GDU: LanguageCode = LanguageCode {
    name: "Gudu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gdu",
    individual_languages: &[
    ],
};


pub const GDX: LanguageCode = LanguageCode {
    name: "Godwari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gdx",
    individual_languages: &[
    ],
};


pub const GEA: LanguageCode = LanguageCode {
    name: "Geruma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gea",
    individual_languages: &[
    ],
};


pub const GEB: LanguageCode = LanguageCode {
    name: "Kire",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "geb",
    individual_languages: &[
    ],
};


pub const GEC: LanguageCode = LanguageCode {
    name: "Gboloo Grebo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gec",
    individual_languages: &[
    ],
};


pub const GED: LanguageCode = LanguageCode {
    name: "Gade",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ged",
    individual_languages: &[
    ],
};


pub const GEF: LanguageCode = LanguageCode {
    name: "Gerai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gef",
    individual_languages: &[
    ],
};


pub const GEG: LanguageCode = LanguageCode {
    name: "Gengle",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "geg",
    individual_languages: &[
    ],
};


pub const GEH: LanguageCode = LanguageCode {
    name: "Hutterite German",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "geh",
    individual_languages: &[
    ],
};


pub const GEI: LanguageCode = LanguageCode {
    name: "Gebe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gei",
    individual_languages: &[
    ],
};


pub const GEJ: LanguageCode = LanguageCode {
    name: "Gen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gej",
    individual_languages: &[
    ],
};


pub const GEK: LanguageCode = LanguageCode {
    name: "Ywom",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gek",
    individual_languages: &[
    ],
};


pub const GEL: LanguageCode = LanguageCode {
    name: "ut-Ma'in",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gel",
    individual_languages: &[
    ],
};


pub const GEQ: LanguageCode = LanguageCode {
    name: "Geme",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "geq",
    individual_languages: &[
    ],
};


pub const GES: LanguageCode = LanguageCode {
    name: "Geser-Gorom",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ges",
    individual_languages: &[
    ],
};


pub const GEV: LanguageCode = LanguageCode {
    name: "Eviya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gev",
    individual_languages: &[
    ],
};


pub const GEW: LanguageCode = LanguageCode {
    name: "Gera",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gew",
    individual_languages: &[
    ],
};


pub const GEX: LanguageCode = LanguageCode {
    name: "Garre",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gex",
    individual_languages: &[
    ],
};


pub const GEY: LanguageCode = LanguageCode {
    name: "Enya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gey",
    individual_languages: &[
    ],
};


pub const GFK: LanguageCode = LanguageCode {
    name: "Patpatar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gfk",
    individual_languages: &[
    ],
};


pub const GFT: LanguageCode = LanguageCode {
    name: "Gafat",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gft",
    individual_languages: &[
    ],
};


pub const GGA: LanguageCode = LanguageCode {
    name: "Gao",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gga",
    individual_languages: &[
    ],
};


pub const GGB: LanguageCode = LanguageCode {
    name: "Gbii",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ggb",
    individual_languages: &[
    ],
};


pub const GGD: LanguageCode = LanguageCode {
    name: "Gugadj",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ggd",
    individual_languages: &[
    ],
};


pub const GGE: LanguageCode = LanguageCode {
    name: "Gurr-goni",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gge",
    individual_languages: &[
    ],
};


pub const GGG: LanguageCode = LanguageCode {
    name: "Gurgula",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ggg",
    individual_languages: &[
    ],
};


pub const GGK: LanguageCode = LanguageCode {
    name: "Kungarakany",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ggk",
    individual_languages: &[
    ],
};


pub const GGL: LanguageCode = LanguageCode {
    name: "Ganglau",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ggl",
    individual_languages: &[
    ],
};


pub const GGT: LanguageCode = LanguageCode {
    name: "Gitua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ggt",
    individual_languages: &[
    ],
};


pub const GGU: LanguageCode = LanguageCode {
    name: "Gagu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ggu",
    individual_languages: &[
    ],
};


pub const GGW: LanguageCode = LanguageCode {
    name: "Gogodala",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ggw",
    individual_languages: &[
    ],
};


pub const GHA: LanguageCode = LanguageCode {
    name: "Ghadamès",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gha",
    individual_languages: &[
    ],
};


pub const GHC: LanguageCode = LanguageCode {
    name: "Hiberno-Scottish Gaelic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ghc",
    individual_languages: &[
    ],
};


pub const GHE: LanguageCode = LanguageCode {
    name: "Southern Ghale",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ghe",
    individual_languages: &[
    ],
};


pub const GHH: LanguageCode = LanguageCode {
    name: "Northern Ghale",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ghh",
    individual_languages: &[
    ],
};


pub const GHK: LanguageCode = LanguageCode {
    name: "Geko Karen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ghk",
    individual_languages: &[
    ],
};


pub const GHL: LanguageCode = LanguageCode {
    name: "Ghulfan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ghl",
    individual_languages: &[
    ],
};


pub const GHN: LanguageCode = LanguageCode {
    name: "Ghanongga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ghn",
    individual_languages: &[
    ],
};


pub const GHO: LanguageCode = LanguageCode {
    name: "Ghomara",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gho",
    individual_languages: &[
    ],
};


pub const GHR: LanguageCode = LanguageCode {
    name: "Ghera",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ghr",
    individual_languages: &[
    ],
};


pub const GHS: LanguageCode = LanguageCode {
    name: "Guhu-Samane",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ghs",
    individual_languages: &[
    ],
};


pub const GHT: LanguageCode = LanguageCode {
    name: "Kuke",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ght",
    individual_languages: &[
    ],
};


pub const GIA: LanguageCode = LanguageCode {
    name: "Kija",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gia",
    individual_languages: &[
    ],
};


pub const GIB: LanguageCode = LanguageCode {
    name: "Gibanawa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gib",
    individual_languages: &[
    ],
};


pub const GIC: LanguageCode = LanguageCode {
    name: "Gail",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gic",
    individual_languages: &[
    ],
};


pub const GID: LanguageCode = LanguageCode {
    name: "Gidar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gid",
    individual_languages: &[
    ],
};


pub const GIE: LanguageCode = LanguageCode {
    name: "Gaɓogbo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gie",
    individual_languages: &[
    ],
};


pub const GIG: LanguageCode = LanguageCode {
    name: "Goaria",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gig",
    individual_languages: &[
    ],
};


pub const GIH: LanguageCode = LanguageCode {
    name: "Githabul",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gih",
    individual_languages: &[
    ],
};


pub const GII: LanguageCode = LanguageCode {
    name: "Girirra",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gii",
    individual_languages: &[
    ],
};


pub const GIM: LanguageCode = LanguageCode {
    name: "Gimi (Eastern Highlands)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gim",
    individual_languages: &[
    ],
};


pub const GIN: LanguageCode = LanguageCode {
    name: "Hinukh",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gin",
    individual_languages: &[
    ],
};


pub const GIP: LanguageCode = LanguageCode {
    name: "Gimi (West New Britain)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gip",
    individual_languages: &[
    ],
};


pub const GIQ: LanguageCode = LanguageCode {
    name: "Green Gelao",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "giq",
    individual_languages: &[
    ],
};


pub const GIR: LanguageCode = LanguageCode {
    name: "Red Gelao",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gir",
    individual_languages: &[
    ],
};


pub const GIS: LanguageCode = LanguageCode {
    name: "North Giziga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gis",
    individual_languages: &[
    ],
};


pub const GIT: LanguageCode = LanguageCode {
    name: "Gitxsan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "git",
    individual_languages: &[
    ],
};


pub const GIU: LanguageCode = LanguageCode {
    name: "Mulao",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "giu",
    individual_languages: &[
    ],
};


pub const GIW: LanguageCode = LanguageCode {
    name: "White Gelao",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "giw",
    individual_languages: &[
    ],
};


pub const GIX: LanguageCode = LanguageCode {
    name: "Gilima",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gix",
    individual_languages: &[
    ],
};


pub const GIY: LanguageCode = LanguageCode {
    name: "Giyug",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "giy",
    individual_languages: &[
    ],
};


pub const GIZ: LanguageCode = LanguageCode {
    name: "South Giziga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "giz",
    individual_languages: &[
    ],
};


pub const GJK: LanguageCode = LanguageCode {
    name: "Kachi Koli",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gjk",
    individual_languages: &[
    ],
};


pub const GJM: LanguageCode = LanguageCode {
    name: "Gunditjmara",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gjm",
    individual_languages: &[
    ],
};


pub const GJN: LanguageCode = LanguageCode {
    name: "Gonja",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gjn",
    individual_languages: &[
    ],
};


pub const GJR: LanguageCode = LanguageCode {
    name: "Gurindji Kriol",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gjr",
    individual_languages: &[
    ],
};


pub const GJU: LanguageCode = LanguageCode {
    name: "Gujari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gju",
    individual_languages: &[
    ],
};


pub const GKA: LanguageCode = LanguageCode {
    name: "Guya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gka",
    individual_languages: &[
    ],
};


pub const GKD: LanguageCode = LanguageCode {
    name: "Magɨ (Madang Province)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gkd",
    individual_languages: &[
    ],
};


pub const GKE: LanguageCode = LanguageCode {
    name: "Ndai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gke",
    individual_languages: &[
    ],
};


pub const GKN: LanguageCode = LanguageCode {
    name: "Gokana",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gkn",
    individual_languages: &[
    ],
};


pub const GKO: LanguageCode = LanguageCode {
    name: "Kok-Nar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gko",
    individual_languages: &[
    ],
};


pub const GKP: LanguageCode = LanguageCode {
    name: "Guinea Kpelle",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gkp",
    individual_languages: &[
    ],
};


pub const GKU: LanguageCode = LanguageCode {
    name: "ǂUngkue",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gku",
    individual_languages: &[
    ],
};


pub const GLB: LanguageCode = LanguageCode {
    name: "Belning",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "glb",
    individual_languages: &[
    ],
};


pub const GLC: LanguageCode = LanguageCode {
    name: "Bon Gula",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "glc",
    individual_languages: &[
    ],
};


pub const GLD: LanguageCode = LanguageCode {
    name: "Nanai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gld",
    individual_languages: &[
    ],
};


pub const GLH: LanguageCode = LanguageCode {
    name: "Northwest Pashai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "glh",
    individual_languages: &[
    ],
};


pub const GLJ: LanguageCode = LanguageCode {
    name: "Gula Iro",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "glj",
    individual_languages: &[
    ],
};


pub const GLK: LanguageCode = LanguageCode {
    name: "Gilaki",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "glk",
    individual_languages: &[
    ],
};


pub const GLL: LanguageCode = LanguageCode {
    name: "Garlali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gll",
    individual_languages: &[
    ],
};


pub const GLO: LanguageCode = LanguageCode {
    name: "Galambu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "glo",
    individual_languages: &[
    ],
};


pub const GLR: LanguageCode = LanguageCode {
    name: "Glaro-Twabo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "glr",
    individual_languages: &[
    ],
};


pub const GLU: LanguageCode = LanguageCode {
    name: "Gula (Chad)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "glu",
    individual_languages: &[
    ],
};


pub const GLW: LanguageCode = LanguageCode {
    name: "Glavda",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "glw",
    individual_languages: &[
    ],
};


pub const GLY: LanguageCode = LanguageCode {
    name: "Gule",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gly",
    individual_languages: &[
    ],
};


pub const GMA: LanguageCode = LanguageCode {
    name: "Gambera",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gma",
    individual_languages: &[
    ],
};


pub const GMB: LanguageCode = LanguageCode {
    name: "Gula'alaa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gmb",
    individual_languages: &[
    ],
};


pub const GMD: LanguageCode = LanguageCode {
    name: "Mághdì",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gmd",
    individual_languages: &[
    ],
};


pub const GMG: LanguageCode = LanguageCode {
    name: "Magɨyi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gmg",
    individual_languages: &[
    ],
};


pub const GML: LanguageCode = LanguageCode {
    name: "Middle Low German",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gml",
    individual_languages: &[
    ],
};


pub const GMM: LanguageCode = LanguageCode {
    name: "Gbaya-Mbodomo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gmm",
    individual_languages: &[
    ],
};


pub const GMN: LanguageCode = LanguageCode {
    name: "Gimnime",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gmn",
    individual_languages: &[
    ],
};


pub const GMR: LanguageCode = LanguageCode {
    name: "Mirning",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gmr",
    individual_languages: &[
    ],
};


pub const GMU: LanguageCode = LanguageCode {
    name: "Gumalu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gmu",
    individual_languages: &[
    ],
};


pub const GMV: LanguageCode = LanguageCode {
    name: "Gamo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gmv",
    individual_languages: &[
    ],
};


pub const GMX: LanguageCode = LanguageCode {
    name: "Magoma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gmx",
    individual_languages: &[
    ],
};


pub const GMY: LanguageCode = LanguageCode {
    name: "Mycenaean Greek",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gmy",
    individual_languages: &[
    ],
};


pub const GMZ: LanguageCode = LanguageCode {
    name: "Mgbolizhia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gmz",
    individual_languages: &[
    ],
};


pub const GNA: LanguageCode = LanguageCode {
    name: "Kaansa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gna",
    individual_languages: &[
    ],
};


pub const GNB: LanguageCode = LanguageCode {
    name: "Gangte",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gnb",
    individual_languages: &[
    ],
};


pub const GNC: LanguageCode = LanguageCode {
    name: "Guanche",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gnc",
    individual_languages: &[
    ],
};


pub const GND: LanguageCode = LanguageCode {
    name: "Zulgo-Gemzek",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gnd",
    individual_languages: &[
    ],
};


pub const GNE: LanguageCode = LanguageCode {
    name: "Ganang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gne",
    individual_languages: &[
    ],
};


pub const GNG: LanguageCode = LanguageCode {
    name: "Ngangam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gng",
    individual_languages: &[
    ],
};


pub const GNH: LanguageCode = LanguageCode {
    name: "Lere",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gnh",
    individual_languages: &[
    ],
};


pub const GNI: LanguageCode = LanguageCode {
    name: "Gooniyandi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gni",
    individual_languages: &[
    ],
};


pub const GNJ: LanguageCode = LanguageCode {
    name: "Ngen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gnj",
    individual_languages: &[
    ],
};


pub const GNK: LanguageCode = LanguageCode {
    name: "ǁGana",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gnk",
    individual_languages: &[
    ],
};


pub const GNL: LanguageCode = LanguageCode {
    name: "Gangulu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gnl",
    individual_languages: &[
    ],
};


pub const GNM: LanguageCode = LanguageCode {
    name: "Ginuman",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gnm",
    individual_languages: &[
    ],
};


pub const GNN: LanguageCode = LanguageCode {
    name: "Gumatj",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gnn",
    individual_languages: &[
    ],
};


pub const GNO: LanguageCode = LanguageCode {
    name: "Northern Gondi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gno",
    individual_languages: &[
    ],
};


pub const GNQ: LanguageCode = LanguageCode {
    name: "Gana",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gnq",
    individual_languages: &[
    ],
};


pub const GNR: LanguageCode = LanguageCode {
    name: "Gureng Gureng",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gnr",
    individual_languages: &[
    ],
};


pub const GNT: LanguageCode = LanguageCode {
    name: "Guntai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gnt",
    individual_languages: &[
    ],
};


pub const GNU: LanguageCode = LanguageCode {
    name: "Gnau",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gnu",
    individual_languages: &[
    ],
};


pub const GNW: LanguageCode = LanguageCode {
    name: "Western Bolivian Guaraní",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gnw",
    individual_languages: &[
    ],
};


pub const GNZ: LanguageCode = LanguageCode {
    name: "Ganzi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gnz",
    individual_languages: &[
    ],
};


pub const GOA: LanguageCode = LanguageCode {
    name: "Guro",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "goa",
    individual_languages: &[
    ],
};


pub const GOB: LanguageCode = LanguageCode {
    name: "Playero",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gob",
    individual_languages: &[
    ],
};


pub const GOC: LanguageCode = LanguageCode {
    name: "Gorakor",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "goc",
    individual_languages: &[
    ],
};


pub const GOD: LanguageCode = LanguageCode {
    name: "Godié",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "god",
    individual_languages: &[
    ],
};


pub const GOE: LanguageCode = LanguageCode {
    name: "Gongduk",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "goe",
    individual_languages: &[
    ],
};


pub const GOF: LanguageCode = LanguageCode {
    name: "Gofa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gof",
    individual_languages: &[
    ],
};


pub const GOG: LanguageCode = LanguageCode {
    name: "Gogo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gog",
    individual_languages: &[
    ],
};


pub const GOI: LanguageCode = LanguageCode {
    name: "Gobasi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "goi",
    individual_languages: &[
    ],
};


pub const GOJ: LanguageCode = LanguageCode {
    name: "Gowlan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "goj",
    individual_languages: &[
    ],
};


pub const GOK: LanguageCode = LanguageCode {
    name: "Gowli",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gok",
    individual_languages: &[
    ],
};


pub const GOL: LanguageCode = LanguageCode {
    name: "Gola",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gol",
    individual_languages: &[
    ],
};


pub const GOM: LanguageCode = LanguageCode {
    name: "Goan Konkani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gom",
    individual_languages: &[
    ],
};


pub const GOO: LanguageCode = LanguageCode {
    name: "Gone Dau",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "goo",
    individual_languages: &[
    ],
};


pub const GOP: LanguageCode = LanguageCode {
    name: "Yeretuar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gop",
    individual_languages: &[
    ],
};


pub const GOQ: LanguageCode = LanguageCode {
    name: "Gorap",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "goq",
    individual_languages: &[
    ],
};


pub const GOS: LanguageCode = LanguageCode {
    name: "Gronings",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gos",
    individual_languages: &[
    ],
};


pub const GOU: LanguageCode = LanguageCode {
    name: "Gavar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gou",
    individual_languages: &[
    ],
};


pub const GOV: LanguageCode = LanguageCode {
    name: "Goo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gov",
    individual_languages: &[
    ],
};


pub const GOW: LanguageCode = LanguageCode {
    name: "Gorowa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gow",
    individual_languages: &[
    ],
};


pub const GOX: LanguageCode = LanguageCode {
    name: "Gobu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gox",
    individual_languages: &[
    ],
};


pub const GOY: LanguageCode = LanguageCode {
    name: "Goundo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "goy",
    individual_languages: &[
    ],
};


pub const GOZ: LanguageCode = LanguageCode {
    name: "Gozarkhani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "goz",
    individual_languages: &[
    ],
};


pub const GPA: LanguageCode = LanguageCode {
    name: "Gupa-Abawa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gpa",
    individual_languages: &[
    ],
};


pub const GPE: LanguageCode = LanguageCode {
    name: "Ghanaian Pidgin English",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gpe",
    individual_languages: &[
    ],
};


pub const GPN: LanguageCode = LanguageCode {
    name: "Taiap",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gpn",
    individual_languages: &[
    ],
};


pub const GQA: LanguageCode = LanguageCode {
    name: "Ga'anda",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gqa",
    individual_languages: &[
    ],
};


pub const GQI: LanguageCode = LanguageCode {
    name: "Guiqiong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gqi",
    individual_languages: &[
    ],
};


pub const GQN: LanguageCode = LanguageCode {
    name: "Guana (Brazil)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gqn",
    individual_languages: &[
    ],
};


pub const GQR: LanguageCode = LanguageCode {
    name: "Gor",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gqr",
    individual_languages: &[
    ],
};


pub const GQU: LanguageCode = LanguageCode {
    name: "Qau",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gqu",
    individual_languages: &[
    ],
};


pub const GRA: LanguageCode = LanguageCode {
    name: "Rajput Garasia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gra",
    individual_languages: &[
    ],
};


pub const GRD: LanguageCode = LanguageCode {
    name: "Guruntum-Mbaaru",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "grd",
    individual_languages: &[
    ],
};


pub const GRG: LanguageCode = LanguageCode {
    name: "Madi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "grg",
    individual_languages: &[
    ],
};


pub const GRH: LanguageCode = LanguageCode {
    name: "Gbiri-Niragu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "grh",
    individual_languages: &[
    ],
};


pub const GRI: LanguageCode = LanguageCode {
    name: "Ghari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gri",
    individual_languages: &[
    ],
};


pub const GRJ: LanguageCode = LanguageCode {
    name: "Southern Grebo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "grj",
    individual_languages: &[
    ],
};


pub const GRM: LanguageCode = LanguageCode {
    name: "Kota Marudu Talantang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "grm",
    individual_languages: &[
    ],
};


pub const GRO: LanguageCode = LanguageCode {
    name: "Groma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gro",
    individual_languages: &[
    ],
};


pub const GRQ: LanguageCode = LanguageCode {
    name: "Gorovu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "grq",
    individual_languages: &[
    ],
};


pub const GRR: LanguageCode = LanguageCode {
    name: "Taznatit",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "grr",
    individual_languages: &[
    ],
};


pub const GRS: LanguageCode = LanguageCode {
    name: "Gresi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "grs",
    individual_languages: &[
    ],
};


pub const GRT: LanguageCode = LanguageCode {
    name: "Garo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "grt",
    individual_languages: &[
    ],
};


pub const GRU: LanguageCode = LanguageCode {
    name: "Kistane",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gru",
    individual_languages: &[
    ],
};


pub const GRV: LanguageCode = LanguageCode {
    name: "Central Grebo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "grv",
    individual_languages: &[
    ],
};


pub const GRW: LanguageCode = LanguageCode {
    name: "Gweda",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "grw",
    individual_languages: &[
    ],
};


pub const GRX: LanguageCode = LanguageCode {
    name: "Guriaso",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "grx",
    individual_languages: &[
    ],
};


pub const GRY: LanguageCode = LanguageCode {
    name: "Barclayville Grebo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gry",
    individual_languages: &[
    ],
};


pub const GRZ: LanguageCode = LanguageCode {
    name: "Guramalum",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "grz",
    individual_languages: &[
    ],
};


pub const GSE: LanguageCode = LanguageCode {
    name: "Ghanaian Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gse",
    individual_languages: &[
    ],
};


pub const GSG: LanguageCode = LanguageCode {
    name: "German Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gsg",
    individual_languages: &[
    ],
};


pub const GSL: LanguageCode = LanguageCode {
    name: "Gusilay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gsl",
    individual_languages: &[
    ],
};


pub const GSM: LanguageCode = LanguageCode {
    name: "Guatemalan Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gsm",
    individual_languages: &[
    ],
};


pub const GSN: LanguageCode = LanguageCode {
    name: "Nema",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gsn",
    individual_languages: &[
    ],
};


pub const GSO: LanguageCode = LanguageCode {
    name: "Southwest Gbaya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gso",
    individual_languages: &[
    ],
};


pub const GSP: LanguageCode = LanguageCode {
    name: "Wasembo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gsp",
    individual_languages: &[
    ],
};


pub const GSS: LanguageCode = LanguageCode {
    name: "Greek Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gss",
    individual_languages: &[
    ],
};


pub const GTA: LanguageCode = LanguageCode {
    name: "Guató",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gta",
    individual_languages: &[
    ],
};


pub const GTU: LanguageCode = LanguageCode {
    name: "Aghu-Tharnggala",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gtu",
    individual_languages: &[
    ],
};


pub const GUA: LanguageCode = LanguageCode {
    name: "Shiki",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gua",
    individual_languages: &[
    ],
};


pub const GUB: LanguageCode = LanguageCode {
    name: "Guajajára",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gub",
    individual_languages: &[
    ],
};


pub const GUC: LanguageCode = LanguageCode {
    name: "Wayuu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "guc",
    individual_languages: &[
    ],
};


pub const GUD: LanguageCode = LanguageCode {
    name: "Yocoboué Dida",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gud",
    individual_languages: &[
    ],
};


pub const GUE: LanguageCode = LanguageCode {
    name: "Gurindji",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gue",
    individual_languages: &[
    ],
};


pub const GUF: LanguageCode = LanguageCode {
    name: "Gupapuyngu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "guf",
    individual_languages: &[
    ],
};


pub const GUG: LanguageCode = LanguageCode {
    name: "Paraguayan Guaraní",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gug",
    individual_languages: &[
    ],
};


pub const GUH: LanguageCode = LanguageCode {
    name: "Guahibo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "guh",
    individual_languages: &[
    ],
};


pub const GUI: LanguageCode = LanguageCode {
    name: "Eastern Bolivian Guaraní",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gui",
    individual_languages: &[
    ],
};


pub const GUK: LanguageCode = LanguageCode {
    name: "Gumuz",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "guk",
    individual_languages: &[
    ],
};


pub const GUL: LanguageCode = LanguageCode {
    name: "Sea Island Creole English",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gul",
    individual_languages: &[
    ],
};


pub const GUM: LanguageCode = LanguageCode {
    name: "Guambiano",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gum",
    individual_languages: &[
    ],
};


pub const GUN: LanguageCode = LanguageCode {
    name: "Mbyá Guaraní",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gun",
    individual_languages: &[
    ],
};


pub const GUO: LanguageCode = LanguageCode {
    name: "Guayabero",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "guo",
    individual_languages: &[
    ],
};


pub const GUP: LanguageCode = LanguageCode {
    name: "Gunwinggu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gup",
    individual_languages: &[
    ],
};


pub const GUQ: LanguageCode = LanguageCode {
    name: "Aché",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "guq",
    individual_languages: &[
    ],
};


pub const GUR: LanguageCode = LanguageCode {
    name: "Farefare",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gur",
    individual_languages: &[
    ],
};


pub const GUS: LanguageCode = LanguageCode {
    name: "Guinean Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gus",
    individual_languages: &[
    ],
};


pub const GUT: LanguageCode = LanguageCode {
    name: "Maléku Jaíka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gut",
    individual_languages: &[
    ],
};


pub const GUU: LanguageCode = LanguageCode {
    name: "Yanomamö",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "guu",
    individual_languages: &[
    ],
};


pub const GUW: LanguageCode = LanguageCode {
    name: "Gun",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "guw",
    individual_languages: &[
    ],
};


pub const GUX: LanguageCode = LanguageCode {
    name: "Gourmanchéma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gux",
    individual_languages: &[
    ],
};


pub const GUZ: LanguageCode = LanguageCode {
    name: "Gusii",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "guz",
    individual_languages: &[
    ],
};


pub const GVA: LanguageCode = LanguageCode {
    name: "Guana (Paraguay)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gva",
    individual_languages: &[
    ],
};


pub const GVC: LanguageCode = LanguageCode {
    name: "Guanano",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gvc",
    individual_languages: &[
    ],
};


pub const GVE: LanguageCode = LanguageCode {
    name: "Duwet",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gve",
    individual_languages: &[
    ],
};


pub const GVF: LanguageCode = LanguageCode {
    name: "Golin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gvf",
    individual_languages: &[
    ],
};


pub const GVJ: LanguageCode = LanguageCode {
    name: "Guajá",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gvj",
    individual_languages: &[
    ],
};


pub const GVL: LanguageCode = LanguageCode {
    name: "Gulay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gvl",
    individual_languages: &[
    ],
};


pub const GVM: LanguageCode = LanguageCode {
    name: "Gurmana",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gvm",
    individual_languages: &[
    ],
};


pub const GVN: LanguageCode = LanguageCode {
    name: "Kuku-Yalanji",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gvn",
    individual_languages: &[
    ],
};


pub const GVO: LanguageCode = LanguageCode {
    name: "Gavião Do Jiparaná",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gvo",
    individual_languages: &[
    ],
};


pub const GVP: LanguageCode = LanguageCode {
    name: "Pará Gavião",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gvp",
    individual_languages: &[
    ],
};


pub const GVR: LanguageCode = LanguageCode {
    name: "Gurung",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gvr",
    individual_languages: &[
    ],
};


pub const GVS: LanguageCode = LanguageCode {
    name: "Gumawana",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gvs",
    individual_languages: &[
    ],
};


pub const GVY: LanguageCode = LanguageCode {
    name: "Guyani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gvy",
    individual_languages: &[
    ],
};


pub const GWA: LanguageCode = LanguageCode {
    name: "Mbato",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gwa",
    individual_languages: &[
    ],
};


pub const GWB: LanguageCode = LanguageCode {
    name: "Gwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gwb",
    individual_languages: &[
    ],
};


pub const GWC: LanguageCode = LanguageCode {
    name: "Gawri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gwc",
    individual_languages: &[
    ],
};


pub const GWD: LanguageCode = LanguageCode {
    name: "Gawwada",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gwd",
    individual_languages: &[
    ],
};


pub const GWE: LanguageCode = LanguageCode {
    name: "Gweno",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gwe",
    individual_languages: &[
    ],
};


pub const GWF: LanguageCode = LanguageCode {
    name: "Gowro",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gwf",
    individual_languages: &[
    ],
};


pub const GWG: LanguageCode = LanguageCode {
    name: "Moo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gwg",
    individual_languages: &[
    ],
};


pub const GWJ: LanguageCode = LanguageCode {
    name: "ǀGwi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gwj",
    individual_languages: &[
    ],
};


pub const GWM: LanguageCode = LanguageCode {
    name: "Awngthim",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gwm",
    individual_languages: &[
    ],
};


pub const GWN: LanguageCode = LanguageCode {
    name: "Gwandara",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gwn",
    individual_languages: &[
    ],
};


pub const GWR: LanguageCode = LanguageCode {
    name: "Gwere",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gwr",
    individual_languages: &[
    ],
};


pub const GWT: LanguageCode = LanguageCode {
    name: "Gawar-Bati",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gwt",
    individual_languages: &[
    ],
};


pub const GWU: LanguageCode = LanguageCode {
    name: "Guwamu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gwu",
    individual_languages: &[
    ],
};


pub const GWW: LanguageCode = LanguageCode {
    name: "Kwini",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gww",
    individual_languages: &[
    ],
};


pub const GWX: LanguageCode = LanguageCode {
    name: "Gua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gwx",
    individual_languages: &[
    ],
};


pub const GXX: LanguageCode = LanguageCode {
    name: "Wè Southern",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gxx",
    individual_languages: &[
    ],
};


pub const GYA: LanguageCode = LanguageCode {
    name: "Northwest Gbaya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gya",
    individual_languages: &[
    ],
};


pub const GYB: LanguageCode = LanguageCode {
    name: "Garus",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gyb",
    individual_languages: &[
    ],
};


pub const GYD: LanguageCode = LanguageCode {
    name: "Kayardild",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gyd",
    individual_languages: &[
    ],
};


pub const GYE: LanguageCode = LanguageCode {
    name: "Gyem",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gye",
    individual_languages: &[
    ],
};


pub const GYF: LanguageCode = LanguageCode {
    name: "Gungabula",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gyf",
    individual_languages: &[
    ],
};


pub const GYG: LanguageCode = LanguageCode {
    name: "Gbayi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gyg",
    individual_languages: &[
    ],
};


pub const GYI: LanguageCode = LanguageCode {
    name: "Gyele",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gyi",
    individual_languages: &[
    ],
};


pub const GYL: LanguageCode = LanguageCode {
    name: "Gayil",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gyl",
    individual_languages: &[
    ],
};


pub const GYM: LanguageCode = LanguageCode {
    name: "Ngäbere",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gym",
    individual_languages: &[
    ],
};


pub const GYN: LanguageCode = LanguageCode {
    name: "Guyanese Creole English",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gyn",
    individual_languages: &[
    ],
};


pub const GYO: LanguageCode = LanguageCode {
    name: "Gyalsumdo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gyo",
    individual_languages: &[
    ],
};


pub const GYR: LanguageCode = LanguageCode {
    name: "Guarayu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gyr",
    individual_languages: &[
    ],
};


pub const GYY: LanguageCode = LanguageCode {
    name: "Gunya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gyy",
    individual_languages: &[
    ],
};


pub const GYZ: LanguageCode = LanguageCode {
    name: "Geji",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gyz",
    individual_languages: &[
    ],
};


pub const GZA: LanguageCode = LanguageCode {
    name: "Ganza",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gza",
    individual_languages: &[
    ],
};


pub const GZI: LanguageCode = LanguageCode {
    name: "Gazi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gzi",
    individual_languages: &[
    ],
};


pub const GZN: LanguageCode = LanguageCode {
    name: "Gane",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "gzn",
    individual_languages: &[
    ],
};


pub const HAA: LanguageCode = LanguageCode {
    name: "Han",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "haa",
    individual_languages: &[
    ],
};


pub const HAB: LanguageCode = LanguageCode {
    name: "Hanoi Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hab",
    individual_languages: &[
    ],
};


pub const HAC: LanguageCode = LanguageCode {
    name: "Gurani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hac",
    individual_languages: &[
    ],
};


pub const HAD: LanguageCode = LanguageCode {
    name: "Hatam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "had",
    individual_languages: &[
    ],
};


pub const HAE: LanguageCode = LanguageCode {
    name: "Eastern Oromo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hae",
    individual_languages: &[
    ],
};


pub const HAF: LanguageCode = LanguageCode {
    name: "Haiphong Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "haf",
    individual_languages: &[
    ],
};


pub const HAG: LanguageCode = LanguageCode {
    name: "Hanga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hag",
    individual_languages: &[
    ],
};


pub const HAH: LanguageCode = LanguageCode {
    name: "Hahon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hah",
    individual_languages: &[
    ],
};


pub const HAJ: LanguageCode = LanguageCode {
    name: "Hajong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "haj",
    individual_languages: &[
    ],
};


pub const HAK: LanguageCode = LanguageCode {
    name: "Hakka Chinese",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hak",
    individual_languages: &[
    ],
};


pub const HAL: LanguageCode = LanguageCode {
    name: "Halang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hal",
    individual_languages: &[
    ],
};


pub const HAM: LanguageCode = LanguageCode {
    name: "Hewa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ham",
    individual_languages: &[
    ],
};


pub const HAN: LanguageCode = LanguageCode {
    name: "Hangaza",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "han",
    individual_languages: &[
    ],
};


pub const HAO: LanguageCode = LanguageCode {
    name: "Hakö",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hao",
    individual_languages: &[
    ],
};


pub const HAP: LanguageCode = LanguageCode {
    name: "Hupla",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hap",
    individual_languages: &[
    ],
};


pub const HAQ: LanguageCode = LanguageCode {
    name: "Ha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "haq",
    individual_languages: &[
    ],
};


pub const HAR: LanguageCode = LanguageCode {
    name: "Harari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "har",
    individual_languages: &[
    ],
};


pub const HAS: LanguageCode = LanguageCode {
    name: "Haisla",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "has",
    individual_languages: &[
    ],
};


pub const HAV: LanguageCode = LanguageCode {
    name: "Havu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hav",
    individual_languages: &[
    ],
};


pub const HAX: LanguageCode = LanguageCode {
    name: "Southern Haida",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hax",
    individual_languages: &[
    ],
};


pub const HAY: LanguageCode = LanguageCode {
    name: "Haya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hay",
    individual_languages: &[
    ],
};


pub const HAZ: LanguageCode = LanguageCode {
    name: "Hazaragi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "haz",
    individual_languages: &[
    ],
};


pub const HBA: LanguageCode = LanguageCode {
    name: "Hamba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hba",
    individual_languages: &[
    ],
};


pub const HBB: LanguageCode = LanguageCode {
    name: "Huba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hbb",
    individual_languages: &[
    ],
};


pub const HBN: LanguageCode = LanguageCode {
    name: "Heiban",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hbn",
    individual_languages: &[
    ],
};


pub const HBO: LanguageCode = LanguageCode {
    name: "Ancient Hebrew",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hbo",
    individual_languages: &[
    ],
};


pub const SH: LanguageCode = LanguageCode {
    name: "Serbo-Croatian",
    code: "sh",
    code_2t: "",
    code_2b: "",
    code_3: "hbs",
    individual_languages: &[
        IndividualLanguages {
            name: "Bosnian",
            code: "bos",
        },
        IndividualLanguages {
            name: "Montenegrin",
            code: "cnr",
        },
        IndividualLanguages {
            name: "Croatian",
            code: "hrv",
        },
        IndividualLanguages {
            name: "Serbian",
            code: "srp",
        },
    ],
};


pub const HBU: LanguageCode = LanguageCode {
    name: "Habu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hbu",
    individual_languages: &[
    ],
};


pub const HCA: LanguageCode = LanguageCode {
    name: "Andaman Creole Hindi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hca",
    individual_languages: &[
    ],
};


pub const HCH: LanguageCode = LanguageCode {
    name: "Huichol",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hch",
    individual_languages: &[
    ],
};


pub const HDN: LanguageCode = LanguageCode {
    name: "Northern Haida",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hdn",
    individual_languages: &[
    ],
};


pub const HDS: LanguageCode = LanguageCode {
    name: "Honduras Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hds",
    individual_languages: &[
    ],
};


pub const HDY: LanguageCode = LanguageCode {
    name: "Hadiyya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hdy",
    individual_languages: &[
    ],
};


pub const HEA: LanguageCode = LanguageCode {
    name: "Northern Qiandong Miao",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hea",
    individual_languages: &[
    ],
};


pub const HED: LanguageCode = LanguageCode {
    name: "Herdé",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hed",
    individual_languages: &[
    ],
};


pub const HEG: LanguageCode = LanguageCode {
    name: "Helong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "heg",
    individual_languages: &[
    ],
};


pub const HEH: LanguageCode = LanguageCode {
    name: "Hehe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "heh",
    individual_languages: &[
    ],
};


pub const HEI: LanguageCode = LanguageCode {
    name: "Heiltsuk",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hei",
    individual_languages: &[
    ],
};


pub const HEM: LanguageCode = LanguageCode {
    name: "Hemba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hem",
    individual_languages: &[
    ],
};


pub const HGM: LanguageCode = LanguageCode {
    name: "Haiǁom",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hgm",
    individual_languages: &[
    ],
};


pub const HGW: LanguageCode = LanguageCode {
    name: "Haigwai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hgw",
    individual_languages: &[
    ],
};


pub const HHI: LanguageCode = LanguageCode {
    name: "Hoia Hoia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hhi",
    individual_languages: &[
    ],
};


pub const HHR: LanguageCode = LanguageCode {
    name: "Kerak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hhr",
    individual_languages: &[
    ],
};


pub const HHY: LanguageCode = LanguageCode {
    name: "Hoyahoya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hhy",
    individual_languages: &[
    ],
};


pub const HIA: LanguageCode = LanguageCode {
    name: "Lamang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hia",
    individual_languages: &[
    ],
};


pub const HIB: LanguageCode = LanguageCode {
    name: "Hibito",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hib",
    individual_languages: &[
    ],
};


pub const HID: LanguageCode = LanguageCode {
    name: "Hidatsa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hid",
    individual_languages: &[
    ],
};


pub const HIF: LanguageCode = LanguageCode {
    name: "Fiji Hindi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hif",
    individual_languages: &[
    ],
};


pub const HIG: LanguageCode = LanguageCode {
    name: "Kamwe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hig",
    individual_languages: &[
    ],
};


pub const HIH: LanguageCode = LanguageCode {
    name: "Pamosu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hih",
    individual_languages: &[
    ],
};


pub const HII: LanguageCode = LanguageCode {
    name: "Hinduri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hii",
    individual_languages: &[
    ],
};


pub const HIJ: LanguageCode = LanguageCode {
    name: "Hijuk",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hij",
    individual_languages: &[
    ],
};


pub const HIK: LanguageCode = LanguageCode {
    name: "Seit-Kaitetu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hik",
    individual_languages: &[
    ],
};


pub const HIO: LanguageCode = LanguageCode {
    name: "Tsoa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hio",
    individual_languages: &[
    ],
};


pub const HIR: LanguageCode = LanguageCode {
    name: "Himarimã",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hir",
    individual_languages: &[
    ],
};


pub const HIW: LanguageCode = LanguageCode {
    name: "Hiw",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hiw",
    individual_languages: &[
    ],
};


pub const HIX: LanguageCode = LanguageCode {
    name: "Hixkaryána",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hix",
    individual_languages: &[
    ],
};


pub const HJI: LanguageCode = LanguageCode {
    name: "Haji",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hji",
    individual_languages: &[
    ],
};


pub const HKA: LanguageCode = LanguageCode {
    name: "Kahe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hka",
    individual_languages: &[
    ],
};


pub const HKE: LanguageCode = LanguageCode {
    name: "Hunde",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hke",
    individual_languages: &[
    ],
};


pub const HKH: LanguageCode = LanguageCode {
    name: "Khah",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hkh",
    individual_languages: &[
    ],
};


pub const HKK: LanguageCode = LanguageCode {
    name: "Hunjara-Kaina Ke",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hkk",
    individual_languages: &[
    ],
};


pub const HKN: LanguageCode = LanguageCode {
    name: "Mel-Khaonh",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hkn",
    individual_languages: &[
    ],
};


pub const HKS: LanguageCode = LanguageCode {
    name: "Hong Kong Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hks",
    individual_languages: &[
    ],
};


pub const HLA: LanguageCode = LanguageCode {
    name: "Halia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hla",
    individual_languages: &[
    ],
};


pub const HLB: LanguageCode = LanguageCode {
    name: "Halbi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hlb",
    individual_languages: &[
    ],
};


pub const HLD: LanguageCode = LanguageCode {
    name: "Halang Doan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hld",
    individual_languages: &[
    ],
};


pub const HLE: LanguageCode = LanguageCode {
    name: "Hlersu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hle",
    individual_languages: &[
    ],
};


pub const HLT: LanguageCode = LanguageCode {
    name: "Matu Chin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hlt",
    individual_languages: &[
    ],
};


pub const HLU: LanguageCode = LanguageCode {
    name: "Hieroglyphic Luwian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hlu",
    individual_languages: &[
    ],
};


pub const HMA: LanguageCode = LanguageCode {
    name: "Southern Mashan Hmong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hma",
    individual_languages: &[
    ],
};


pub const HMB: LanguageCode = LanguageCode {
    name: "Humburi Senni Songhay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hmb",
    individual_languages: &[
    ],
};


pub const HMC: LanguageCode = LanguageCode {
    name: "Central Huishui Hmong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hmc",
    individual_languages: &[
    ],
};


pub const HMD: LanguageCode = LanguageCode {
    name: "Large Flowery Miao",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hmd",
    individual_languages: &[
    ],
};


pub const HME: LanguageCode = LanguageCode {
    name: "Eastern Huishui Hmong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hme",
    individual_languages: &[
    ],
};


pub const HMF: LanguageCode = LanguageCode {
    name: "Hmong Don",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hmf",
    individual_languages: &[
    ],
};


pub const HMG: LanguageCode = LanguageCode {
    name: "Southwestern Guiyang Hmong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hmg",
    individual_languages: &[
    ],
};


pub const HMH: LanguageCode = LanguageCode {
    name: "Southwestern Huishui Hmong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hmh",
    individual_languages: &[
    ],
};


pub const HMI: LanguageCode = LanguageCode {
    name: "Northern Huishui Hmong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hmi",
    individual_languages: &[
    ],
};


pub const HMJ: LanguageCode = LanguageCode {
    name: "Ge",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hmj",
    individual_languages: &[
    ],
};


pub const HMK: LanguageCode = LanguageCode {
    name: "Maek",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hmk",
    individual_languages: &[
    ],
};


pub const HML: LanguageCode = LanguageCode {
    name: "Luopohe Hmong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hml",
    individual_languages: &[
    ],
};


pub const HMM: LanguageCode = LanguageCode {
    name: "Central Mashan Hmong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hmm",
    individual_languages: &[
    ],
};


pub const HMP: LanguageCode = LanguageCode {
    name: "Northern Mashan Hmong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hmp",
    individual_languages: &[
    ],
};


pub const HMQ: LanguageCode = LanguageCode {
    name: "Eastern Qiandong Miao",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hmq",
    individual_languages: &[
    ],
};


pub const HMR: LanguageCode = LanguageCode {
    name: "Hmar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hmr",
    individual_languages: &[
    ],
};


pub const HMS: LanguageCode = LanguageCode {
    name: "Southern Qiandong Miao",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hms",
    individual_languages: &[
    ],
};


pub const HMT: LanguageCode = LanguageCode {
    name: "Hamtai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hmt",
    individual_languages: &[
    ],
};


pub const HMU: LanguageCode = LanguageCode {
    name: "Hamap",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hmu",
    individual_languages: &[
    ],
};


pub const HMV: LanguageCode = LanguageCode {
    name: "Hmong Dô",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hmv",
    individual_languages: &[
    ],
};


pub const HMW: LanguageCode = LanguageCode {
    name: "Western Mashan Hmong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hmw",
    individual_languages: &[
    ],
};


pub const HMY: LanguageCode = LanguageCode {
    name: "Southern Guiyang Hmong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hmy",
    individual_languages: &[
    ],
};


pub const HMZ: LanguageCode = LanguageCode {
    name: "Hmong Shua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hmz",
    individual_languages: &[
    ],
};


pub const HNA: LanguageCode = LanguageCode {
    name: "Mina (Cameroon)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hna",
    individual_languages: &[
    ],
};


pub const HND: LanguageCode = LanguageCode {
    name: "Southern Hindko",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hnd",
    individual_languages: &[
    ],
};


pub const HNE: LanguageCode = LanguageCode {
    name: "Chhattisgarhi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hne",
    individual_languages: &[
    ],
};


pub const HNG: LanguageCode = LanguageCode {
    name: "Hungu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hng",
    individual_languages: &[
    ],
};


pub const HNH: LanguageCode = LanguageCode {
    name: "ǁAni",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hnh",
    individual_languages: &[
    ],
};


pub const HNI: LanguageCode = LanguageCode {
    name: "Hani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hni",
    individual_languages: &[
    ],
};


pub const HNJ: LanguageCode = LanguageCode {
    name: "Hmong Njua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hnj",
    individual_languages: &[
    ],
};


pub const HNN: LanguageCode = LanguageCode {
    name: "Hanunoo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hnn",
    individual_languages: &[
    ],
};


pub const HNO: LanguageCode = LanguageCode {
    name: "Northern Hindko",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hno",
    individual_languages: &[
    ],
};


pub const HNS: LanguageCode = LanguageCode {
    name: "Caribbean Hindustani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hns",
    individual_languages: &[
    ],
};


pub const HNU: LanguageCode = LanguageCode {
    name: "Hung",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hnu",
    individual_languages: &[
    ],
};


pub const HOA: LanguageCode = LanguageCode {
    name: "Hoava",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hoa",
    individual_languages: &[
    ],
};


pub const HOB: LanguageCode = LanguageCode {
    name: "Mari (Madang Province)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hob",
    individual_languages: &[
    ],
};


pub const HOC: LanguageCode = LanguageCode {
    name: "Ho",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hoc",
    individual_languages: &[
    ],
};


pub const HOD: LanguageCode = LanguageCode {
    name: "Holma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hod",
    individual_languages: &[
    ],
};


pub const HOE: LanguageCode = LanguageCode {
    name: "Horom",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hoe",
    individual_languages: &[
    ],
};


pub const HOH: LanguageCode = LanguageCode {
    name: "Hobyót",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hoh",
    individual_languages: &[
    ],
};


pub const HOI: LanguageCode = LanguageCode {
    name: "Holikachuk",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hoi",
    individual_languages: &[
    ],
};


pub const HOJ: LanguageCode = LanguageCode {
    name: "Hadothi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hoj",
    individual_languages: &[
    ],
};


pub const HOL: LanguageCode = LanguageCode {
    name: "Holu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hol",
    individual_languages: &[
    ],
};


pub const HOM: LanguageCode = LanguageCode {
    name: "Homa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hom",
    individual_languages: &[
    ],
};


pub const HOO: LanguageCode = LanguageCode {
    name: "Holoholo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hoo",
    individual_languages: &[
    ],
};


pub const HOP: LanguageCode = LanguageCode {
    name: "Hopi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hop",
    individual_languages: &[
    ],
};


pub const HOR: LanguageCode = LanguageCode {
    name: "Horo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hor",
    individual_languages: &[
    ],
};


pub const HOS: LanguageCode = LanguageCode {
    name: "Ho Chi Minh City Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hos",
    individual_languages: &[
    ],
};


pub const HOT: LanguageCode = LanguageCode {
    name: "Hote",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hot",
    individual_languages: &[
    ],
};


pub const HOV: LanguageCode = LanguageCode {
    name: "Hovongan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hov",
    individual_languages: &[
    ],
};


pub const HOW: LanguageCode = LanguageCode {
    name: "Honi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "how",
    individual_languages: &[
    ],
};


pub const HOY: LanguageCode = LanguageCode {
    name: "Holiya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hoy",
    individual_languages: &[
    ],
};


pub const HOZ: LanguageCode = LanguageCode {
    name: "Hozo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hoz",
    individual_languages: &[
    ],
};


pub const HPO: LanguageCode = LanguageCode {
    name: "Hpon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hpo",
    individual_languages: &[
    ],
};


pub const HPS: LanguageCode = LanguageCode {
    name: "Hawai'i Sign Language (HSL)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hps",
    individual_languages: &[
    ],
};


pub const HRA: LanguageCode = LanguageCode {
    name: "Hrangkhol",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hra",
    individual_languages: &[
    ],
};


pub const HRC: LanguageCode = LanguageCode {
    name: "Niwer Mil",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hrc",
    individual_languages: &[
    ],
};


pub const HRE: LanguageCode = LanguageCode {
    name: "Hre",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hre",
    individual_languages: &[
    ],
};


pub const HRK: LanguageCode = LanguageCode {
    name: "Haruku",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hrk",
    individual_languages: &[
    ],
};


pub const HRM: LanguageCode = LanguageCode {
    name: "Horned Miao",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hrm",
    individual_languages: &[
    ],
};


pub const HRO: LanguageCode = LanguageCode {
    name: "Haroi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hro",
    individual_languages: &[
    ],
};


pub const HRP: LanguageCode = LanguageCode {
    name: "Nhirrpi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hrp",
    individual_languages: &[
    ],
};


pub const HRT: LanguageCode = LanguageCode {
    name: "Hértevin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hrt",
    individual_languages: &[
    ],
};


pub const HRU: LanguageCode = LanguageCode {
    name: "Hruso",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hru",
    individual_languages: &[
    ],
};


pub const HRW: LanguageCode = LanguageCode {
    name: "Warwar Feni",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hrw",
    individual_languages: &[
    ],
};


pub const HRX: LanguageCode = LanguageCode {
    name: "Hunsrik",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hrx",
    individual_languages: &[
    ],
};


pub const HRZ: LanguageCode = LanguageCode {
    name: "Harzani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hrz",
    individual_languages: &[
    ],
};


pub const HSH: LanguageCode = LanguageCode {
    name: "Hungarian Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hsh",
    individual_languages: &[
    ],
};


pub const HSL: LanguageCode = LanguageCode {
    name: "Hausa Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hsl",
    individual_languages: &[
    ],
};


pub const HSN: LanguageCode = LanguageCode {
    name: "Xiang Chinese",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hsn",
    individual_languages: &[
    ],
};


pub const HSS: LanguageCode = LanguageCode {
    name: "Harsusi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hss",
    individual_languages: &[
    ],
};


pub const HTI: LanguageCode = LanguageCode {
    name: "Hoti",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hti",
    individual_languages: &[
    ],
};


pub const HTO: LanguageCode = LanguageCode {
    name: "Minica Huitoto",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hto",
    individual_languages: &[
    ],
};


pub const HTS: LanguageCode = LanguageCode {
    name: "Hadza",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hts",
    individual_languages: &[
    ],
};


pub const HTU: LanguageCode = LanguageCode {
    name: "Hitu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "htu",
    individual_languages: &[
    ],
};


pub const HTX: LanguageCode = LanguageCode {
    name: "Middle Hittite",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "htx",
    individual_languages: &[
    ],
};


pub const HUB: LanguageCode = LanguageCode {
    name: "Huambisa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hub",
    individual_languages: &[
    ],
};


pub const HUC: LanguageCode = LanguageCode {
    name: "ǂHua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "huc",
    individual_languages: &[
    ],
};


pub const HUD: LanguageCode = LanguageCode {
    name: "Huaulu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hud",
    individual_languages: &[
    ],
};


pub const HUE: LanguageCode = LanguageCode {
    name: "San Francisco Del Mar Huave",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hue",
    individual_languages: &[
    ],
};


pub const HUF: LanguageCode = LanguageCode {
    name: "Humene",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "huf",
    individual_languages: &[
    ],
};


pub const HUG: LanguageCode = LanguageCode {
    name: "Huachipaeri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hug",
    individual_languages: &[
    ],
};


pub const HUH: LanguageCode = LanguageCode {
    name: "Huilliche",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "huh",
    individual_languages: &[
    ],
};


pub const HUI: LanguageCode = LanguageCode {
    name: "Huli",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hui",
    individual_languages: &[
    ],
};


pub const HUJ: LanguageCode = LanguageCode {
    name: "Northern Guiyang Hmong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "huj",
    individual_languages: &[
    ],
};


pub const HUK: LanguageCode = LanguageCode {
    name: "Hulung",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "huk",
    individual_languages: &[
    ],
};


pub const HUL: LanguageCode = LanguageCode {
    name: "Hula",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hul",
    individual_languages: &[
    ],
};


pub const HUM: LanguageCode = LanguageCode {
    name: "Hungana",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hum",
    individual_languages: &[
    ],
};


pub const HUO: LanguageCode = LanguageCode {
    name: "Hu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "huo",
    individual_languages: &[
    ],
};


pub const HUQ: LanguageCode = LanguageCode {
    name: "Tsat",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "huq",
    individual_languages: &[
    ],
};


pub const HUR: LanguageCode = LanguageCode {
    name: "Halkomelem",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hur",
    individual_languages: &[
    ],
};


pub const HUS: LanguageCode = LanguageCode {
    name: "Huastec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hus",
    individual_languages: &[
    ],
};


pub const HUT: LanguageCode = LanguageCode {
    name: "Humla",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hut",
    individual_languages: &[
    ],
};


pub const HUU: LanguageCode = LanguageCode {
    name: "Murui Huitoto",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "huu",
    individual_languages: &[
    ],
};


pub const HUV: LanguageCode = LanguageCode {
    name: "San Mateo Del Mar Huave",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "huv",
    individual_languages: &[
    ],
};


pub const HUW: LanguageCode = LanguageCode {
    name: "Hukumina",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "huw",
    individual_languages: &[
    ],
};


pub const HUX: LanguageCode = LanguageCode {
    name: "Nüpode Huitoto",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hux",
    individual_languages: &[
    ],
};


pub const HUY: LanguageCode = LanguageCode {
    name: "Hulaulá",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "huy",
    individual_languages: &[
    ],
};


pub const HUZ: LanguageCode = LanguageCode {
    name: "Hunzib",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "huz",
    individual_languages: &[
    ],
};


pub const HVC: LanguageCode = LanguageCode {
    name: "Haitian Vodoun Culture Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hvc",
    individual_languages: &[
    ],
};


pub const HVE: LanguageCode = LanguageCode {
    name: "San Dionisio Del Mar Huave",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hve",
    individual_languages: &[
    ],
};


pub const HVK: LanguageCode = LanguageCode {
    name: "Haveke",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hvk",
    individual_languages: &[
    ],
};


pub const HVN: LanguageCode = LanguageCode {
    name: "Sabu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hvn",
    individual_languages: &[
    ],
};


pub const HVV: LanguageCode = LanguageCode {
    name: "Santa María Del Mar Huave",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hvv",
    individual_languages: &[
    ],
};


pub const HWA: LanguageCode = LanguageCode {
    name: "Wané",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hwa",
    individual_languages: &[
    ],
};


pub const HWC: LanguageCode = LanguageCode {
    name: "Hawai'i Creole English",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hwc",
    individual_languages: &[
    ],
};


pub const HWO: LanguageCode = LanguageCode {
    name: "Hwana",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hwo",
    individual_languages: &[
    ],
};


pub const HYA: LanguageCode = LanguageCode {
    name: "Hya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hya",
    individual_languages: &[
    ],
};


pub const HYW: LanguageCode = LanguageCode {
    name: "Western Armenian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "hyw",
    individual_languages: &[
    ],
};


pub const IAI: LanguageCode = LanguageCode {
    name: "Iaai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "iai",
    individual_languages: &[
    ],
};


pub const IAN: LanguageCode = LanguageCode {
    name: "Iatmul",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ian",
    individual_languages: &[
    ],
};


pub const IAR: LanguageCode = LanguageCode {
    name: "Purari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "iar",
    individual_languages: &[
    ],
};


pub const IBB: LanguageCode = LanguageCode {
    name: "Ibibio",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ibb",
    individual_languages: &[
    ],
};


pub const IBD: LanguageCode = LanguageCode {
    name: "Iwaidja",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ibd",
    individual_languages: &[
    ],
};


pub const IBE: LanguageCode = LanguageCode {
    name: "Akpes",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ibe",
    individual_languages: &[
    ],
};


pub const IBG: LanguageCode = LanguageCode {
    name: "Ibanag",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ibg",
    individual_languages: &[
    ],
};


pub const IBH: LanguageCode = LanguageCode {
    name: "Bih",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ibh",
    individual_languages: &[
    ],
};


pub const IBL: LanguageCode = LanguageCode {
    name: "Ibaloi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ibl",
    individual_languages: &[
    ],
};


pub const IBM: LanguageCode = LanguageCode {
    name: "Agoi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ibm",
    individual_languages: &[
    ],
};


pub const IBN: LanguageCode = LanguageCode {
    name: "Ibino",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ibn",
    individual_languages: &[
    ],
};


pub const IBR: LanguageCode = LanguageCode {
    name: "Ibuoro",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ibr",
    individual_languages: &[
    ],
};


pub const IBU: LanguageCode = LanguageCode {
    name: "Ibu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ibu",
    individual_languages: &[
    ],
};


pub const IBY: LanguageCode = LanguageCode {
    name: "Ibani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "iby",
    individual_languages: &[
    ],
};


pub const ICA: LanguageCode = LanguageCode {
    name: "Ede Ica",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ica",
    individual_languages: &[
    ],
};


pub const ICH: LanguageCode = LanguageCode {
    name: "Etkywan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ich",
    individual_languages: &[
    ],
};


pub const ICL: LanguageCode = LanguageCode {
    name: "Icelandic Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "icl",
    individual_languages: &[
    ],
};


pub const ICR: LanguageCode = LanguageCode {
    name: "Islander Creole English",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "icr",
    individual_languages: &[
    ],
};


pub const IDA: LanguageCode = LanguageCode {
    name: "Idakho-Isukha-Tiriki",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ida",
    individual_languages: &[
    ],
};


pub const IDB: LanguageCode = LanguageCode {
    name: "Indo-Portuguese",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "idb",
    individual_languages: &[
    ],
};


pub const IDC: LanguageCode = LanguageCode {
    name: "Idon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "idc",
    individual_languages: &[
    ],
};


pub const IDD: LanguageCode = LanguageCode {
    name: "Ede Idaca",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "idd",
    individual_languages: &[
    ],
};


pub const IDE: LanguageCode = LanguageCode {
    name: "Idere",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ide",
    individual_languages: &[
    ],
};


pub const IDI: LanguageCode = LanguageCode {
    name: "Idi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "idi",
    individual_languages: &[
    ],
};


pub const IDR: LanguageCode = LanguageCode {
    name: "Indri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "idr",
    individual_languages: &[
    ],
};


pub const IDS: LanguageCode = LanguageCode {
    name: "Idesa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ids",
    individual_languages: &[
    ],
};


pub const IDT: LanguageCode = LanguageCode {
    name: "Idaté",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "idt",
    individual_languages: &[
    ],
};


pub const IDU: LanguageCode = LanguageCode {
    name: "Idoma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "idu",
    individual_languages: &[
    ],
};


pub const IFA: LanguageCode = LanguageCode {
    name: "Amganad Ifugao",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ifa",
    individual_languages: &[
    ],
};


pub const IFB: LanguageCode = LanguageCode {
    name: "Batad Ifugao",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ifb",
    individual_languages: &[
    ],
};


pub const IFE: LanguageCode = LanguageCode {
    name: "Ifè",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ife",
    individual_languages: &[
    ],
};


pub const IFF: LanguageCode = LanguageCode {
    name: "Ifo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "iff",
    individual_languages: &[
    ],
};


pub const IFK: LanguageCode = LanguageCode {
    name: "Tuwali Ifugao",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ifk",
    individual_languages: &[
    ],
};


pub const IFM: LanguageCode = LanguageCode {
    name: "Teke-Fuumu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ifm",
    individual_languages: &[
    ],
};


pub const IFU: LanguageCode = LanguageCode {
    name: "Mayoyao Ifugao",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ifu",
    individual_languages: &[
    ],
};


pub const IFY: LanguageCode = LanguageCode {
    name: "Keley-I Kallahan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ify",
    individual_languages: &[
    ],
};


pub const IGB: LanguageCode = LanguageCode {
    name: "Ebira",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "igb",
    individual_languages: &[
    ],
};


pub const IGE: LanguageCode = LanguageCode {
    name: "Igede",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ige",
    individual_languages: &[
    ],
};


pub const IGG: LanguageCode = LanguageCode {
    name: "Igana",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "igg",
    individual_languages: &[
    ],
};


pub const IGL: LanguageCode = LanguageCode {
    name: "Igala",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "igl",
    individual_languages: &[
    ],
};


pub const IGM: LanguageCode = LanguageCode {
    name: "Kanggape",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "igm",
    individual_languages: &[
    ],
};


pub const IGN: LanguageCode = LanguageCode {
    name: "Ignaciano",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ign",
    individual_languages: &[
    ],
};


pub const IGO: LanguageCode = LanguageCode {
    name: "Isebe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "igo",
    individual_languages: &[
    ],
};


pub const IGS: LanguageCode = LanguageCode {
    name: "Interglossa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "igs",
    individual_languages: &[
    ],
};


pub const IGW: LanguageCode = LanguageCode {
    name: "Igwe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "igw",
    individual_languages: &[
    ],
};


pub const IHB: LanguageCode = LanguageCode {
    name: "Iha Based Pidgin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ihb",
    individual_languages: &[
    ],
};


pub const IHI: LanguageCode = LanguageCode {
    name: "Ihievbe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ihi",
    individual_languages: &[
    ],
};


pub const IHP: LanguageCode = LanguageCode {
    name: "Iha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ihp",
    individual_languages: &[
    ],
};


pub const IHW: LanguageCode = LanguageCode {
    name: "Bidhawal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ihw",
    individual_languages: &[
    ],
};


pub const IIN: LanguageCode = LanguageCode {
    name: "Thiin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "iin",
    individual_languages: &[
    ],
};


pub const IJC: LanguageCode = LanguageCode {
    name: "Izon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ijc",
    individual_languages: &[
    ],
};


pub const IJE: LanguageCode = LanguageCode {
    name: "Biseni",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ije",
    individual_languages: &[
    ],
};


pub const IJJ: LanguageCode = LanguageCode {
    name: "Ede Ije",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ijj",
    individual_languages: &[
    ],
};


pub const IJN: LanguageCode = LanguageCode {
    name: "Kalabari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ijn",
    individual_languages: &[
    ],
};


pub const IJS: LanguageCode = LanguageCode {
    name: "Southeast Ijo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ijs",
    individual_languages: &[
    ],
};


pub const IKE: LanguageCode = LanguageCode {
    name: "Eastern Canadian Inuktitut",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ike",
    individual_languages: &[
    ],
};


pub const IKI: LanguageCode = LanguageCode {
    name: "Iko",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "iki",
    individual_languages: &[
    ],
};


pub const IKK: LanguageCode = LanguageCode {
    name: "Ika",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ikk",
    individual_languages: &[
    ],
};


pub const IKL: LanguageCode = LanguageCode {
    name: "Ikulu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ikl",
    individual_languages: &[
    ],
};


pub const IKO: LanguageCode = LanguageCode {
    name: "Olulumo-Ikom",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "iko",
    individual_languages: &[
    ],
};


pub const IKP: LanguageCode = LanguageCode {
    name: "Ikpeshi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ikp",
    individual_languages: &[
    ],
};


pub const IKR: LanguageCode = LanguageCode {
    name: "Ikaranggal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ikr",
    individual_languages: &[
    ],
};


pub const IKS: LanguageCode = LanguageCode {
    name: "Inuit Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "iks",
    individual_languages: &[
    ],
};


pub const IKT: LanguageCode = LanguageCode {
    name: "Inuinnaqtun",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ikt",
    individual_languages: &[
    ],
};


pub const IKV: LanguageCode = LanguageCode {
    name: "Iku-Gora-Ankwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ikv",
    individual_languages: &[
    ],
};


pub const IKW: LanguageCode = LanguageCode {
    name: "Ikwere",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ikw",
    individual_languages: &[
    ],
};


pub const IKX: LanguageCode = LanguageCode {
    name: "Ik",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ikx",
    individual_languages: &[
    ],
};


pub const IKZ: LanguageCode = LanguageCode {
    name: "Ikizu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ikz",
    individual_languages: &[
    ],
};


pub const ILA: LanguageCode = LanguageCode {
    name: "Ile Ape",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ila",
    individual_languages: &[
    ],
};


pub const ILB: LanguageCode = LanguageCode {
    name: "Ila",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ilb",
    individual_languages: &[
    ],
};


pub const ILG: LanguageCode = LanguageCode {
    name: "Garig-Ilgar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ilg",
    individual_languages: &[
    ],
};


pub const ILI: LanguageCode = LanguageCode {
    name: "Ili Turki",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ili",
    individual_languages: &[
    ],
};


pub const ILK: LanguageCode = LanguageCode {
    name: "Ilongot",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ilk",
    individual_languages: &[
    ],
};


pub const ILM: LanguageCode = LanguageCode {
    name: "Iranun (Malaysia)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ilm",
    individual_languages: &[
    ],
};


pub const ILP: LanguageCode = LanguageCode {
    name: "Iranun (Philippines)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ilp",
    individual_languages: &[
    ],
};


pub const ILS: LanguageCode = LanguageCode {
    name: "International Sign",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ils",
    individual_languages: &[
    ],
};


pub const ILU: LanguageCode = LanguageCode {
    name: "Ili'uun",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ilu",
    individual_languages: &[
    ],
};


pub const ILV: LanguageCode = LanguageCode {
    name: "Ilue",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ilv",
    individual_languages: &[
    ],
};


pub const IMA: LanguageCode = LanguageCode {
    name: "Mala Malasar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ima",
    individual_languages: &[
    ],
};


pub const IMI: LanguageCode = LanguageCode {
    name: "Anamgura",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "imi",
    individual_languages: &[
    ],
};


pub const IML: LanguageCode = LanguageCode {
    name: "Miluk",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "iml",
    individual_languages: &[
    ],
};


pub const IMN: LanguageCode = LanguageCode {
    name: "Imonda",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "imn",
    individual_languages: &[
    ],
};


pub const IMO: LanguageCode = LanguageCode {
    name: "Imbongu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "imo",
    individual_languages: &[
    ],
};


pub const IMR: LanguageCode = LanguageCode {
    name: "Imroing",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "imr",
    individual_languages: &[
    ],
};


pub const IMS: LanguageCode = LanguageCode {
    name: "Marsian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ims",
    individual_languages: &[
    ],
};


pub const IMT: LanguageCode = LanguageCode {
    name: "Imotong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "imt",
    individual_languages: &[
    ],
};


pub const IMY: LanguageCode = LanguageCode {
    name: "Milyan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "imy",
    individual_languages: &[
    ],
};


pub const INB: LanguageCode = LanguageCode {
    name: "Inga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "inb",
    individual_languages: &[
    ],
};


pub const ING: LanguageCode = LanguageCode {
    name: "Degexit'an",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ing",
    individual_languages: &[
    ],
};


pub const INJ: LanguageCode = LanguageCode {
    name: "Jungle Inga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "inj",
    individual_languages: &[
    ],
};


pub const INL: LanguageCode = LanguageCode {
    name: "Indonesian Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "inl",
    individual_languages: &[
    ],
};


pub const INM: LanguageCode = LanguageCode {
    name: "Minaean",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "inm",
    individual_languages: &[
    ],
};


pub const INN: LanguageCode = LanguageCode {
    name: "Isinai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "inn",
    individual_languages: &[
    ],
};


pub const INO: LanguageCode = LanguageCode {
    name: "Inoke-Yate",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ino",
    individual_languages: &[
    ],
};


pub const INP: LanguageCode = LanguageCode {
    name: "Iñapari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "inp",
    individual_languages: &[
    ],
};


pub const INS: LanguageCode = LanguageCode {
    name: "Indian Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ins",
    individual_languages: &[
    ],
};


pub const INT: LanguageCode = LanguageCode {
    name: "Intha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "int",
    individual_languages: &[
    ],
};


pub const INZ: LanguageCode = LanguageCode {
    name: "Ineseño",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "inz",
    individual_languages: &[
    ],
};


pub const IOR: LanguageCode = LanguageCode {
    name: "Inor",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ior",
    individual_languages: &[
    ],
};


pub const IOU: LanguageCode = LanguageCode {
    name: "Tuma-Irumu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "iou",
    individual_languages: &[
    ],
};


pub const IOW: LanguageCode = LanguageCode {
    name: "Iowa-Oto",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "iow",
    individual_languages: &[
    ],
};


pub const IPI: LanguageCode = LanguageCode {
    name: "Ipili",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ipi",
    individual_languages: &[
    ],
};


pub const IPO: LanguageCode = LanguageCode {
    name: "Ipiko",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ipo",
    individual_languages: &[
    ],
};


pub const IQU: LanguageCode = LanguageCode {
    name: "Iquito",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "iqu",
    individual_languages: &[
    ],
};


pub const IQW: LanguageCode = LanguageCode {
    name: "Ikwo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "iqw",
    individual_languages: &[
    ],
};


pub const IRE: LanguageCode = LanguageCode {
    name: "Iresim",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ire",
    individual_languages: &[
    ],
};


pub const IRH: LanguageCode = LanguageCode {
    name: "Irarutu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "irh",
    individual_languages: &[
    ],
};


pub const IRI: LanguageCode = LanguageCode {
    name: "Rigwe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "iri",
    individual_languages: &[
    ],
};


pub const IRK: LanguageCode = LanguageCode {
    name: "Iraqw",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "irk",
    individual_languages: &[
    ],
};


pub const IRN: LanguageCode = LanguageCode {
    name: "Irántxe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "irn",
    individual_languages: &[
    ],
};


pub const IRR: LanguageCode = LanguageCode {
    name: "Ir",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "irr",
    individual_languages: &[
    ],
};


pub const IRU: LanguageCode = LanguageCode {
    name: "Irula",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "iru",
    individual_languages: &[
    ],
};


pub const IRX: LanguageCode = LanguageCode {
    name: "Kamberau",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "irx",
    individual_languages: &[
    ],
};


pub const IRY: LanguageCode = LanguageCode {
    name: "Iraya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "iry",
    individual_languages: &[
    ],
};


pub const ISA: LanguageCode = LanguageCode {
    name: "Isabi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "isa",
    individual_languages: &[
    ],
};


pub const ISC: LanguageCode = LanguageCode {
    name: "Isconahua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "isc",
    individual_languages: &[
    ],
};


pub const ISD: LanguageCode = LanguageCode {
    name: "Isnag",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "isd",
    individual_languages: &[
    ],
};


pub const ISE: LanguageCode = LanguageCode {
    name: "Italian Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ise",
    individual_languages: &[
    ],
};


pub const ISG: LanguageCode = LanguageCode {
    name: "Irish Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "isg",
    individual_languages: &[
    ],
};


pub const ISH: LanguageCode = LanguageCode {
    name: "Esan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ish",
    individual_languages: &[
    ],
};


pub const ISI: LanguageCode = LanguageCode {
    name: "Nkem-Nkum",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "isi",
    individual_languages: &[
    ],
};


pub const ISK: LanguageCode = LanguageCode {
    name: "Ishkashimi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "isk",
    individual_languages: &[
    ],
};


pub const ISM: LanguageCode = LanguageCode {
    name: "Masimasi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ism",
    individual_languages: &[
    ],
};


pub const ISN: LanguageCode = LanguageCode {
    name: "Isanzu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "isn",
    individual_languages: &[
    ],
};


pub const ISO: LanguageCode = LanguageCode {
    name: "Isoko",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "iso",
    individual_languages: &[
    ],
};


pub const ISR: LanguageCode = LanguageCode {
    name: "Israeli Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "isr",
    individual_languages: &[
    ],
};


pub const IST: LanguageCode = LanguageCode {
    name: "Istriot",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ist",
    individual_languages: &[
    ],
};


pub const ISU: LanguageCode = LanguageCode {
    name: "Isu (Menchum Division)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "isu",
    individual_languages: &[
    ],
};


pub const ITB: LanguageCode = LanguageCode {
    name: "Binongan Itneg",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "itb",
    individual_languages: &[
    ],
};


pub const ITD: LanguageCode = LanguageCode {
    name: "Southern Tidung",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "itd",
    individual_languages: &[
    ],
};


pub const ITE: LanguageCode = LanguageCode {
    name: "Itene",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ite",
    individual_languages: &[
    ],
};


pub const ITI: LanguageCode = LanguageCode {
    name: "Inlaod Itneg",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "iti",
    individual_languages: &[
    ],
};


pub const ITK: LanguageCode = LanguageCode {
    name: "Judeo-Italian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "itk",
    individual_languages: &[
    ],
};


pub const ITL: LanguageCode = LanguageCode {
    name: "Itelmen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "itl",
    individual_languages: &[
    ],
};


pub const ITM: LanguageCode = LanguageCode {
    name: "Itu Mbon Uzo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "itm",
    individual_languages: &[
    ],
};


pub const ITO: LanguageCode = LanguageCode {
    name: "Itonama",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ito",
    individual_languages: &[
    ],
};


pub const ITR: LanguageCode = LanguageCode {
    name: "Iteri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "itr",
    individual_languages: &[
    ],
};


pub const ITS: LanguageCode = LanguageCode {
    name: "Isekiri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "its",
    individual_languages: &[
    ],
};


pub const ITT: LanguageCode = LanguageCode {
    name: "Maeng Itneg",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "itt",
    individual_languages: &[
    ],
};


pub const ITV: LanguageCode = LanguageCode {
    name: "Itawit",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "itv",
    individual_languages: &[
    ],
};


pub const ITW: LanguageCode = LanguageCode {
    name: "Ito",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "itw",
    individual_languages: &[
    ],
};


pub const ITX: LanguageCode = LanguageCode {
    name: "Itik",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "itx",
    individual_languages: &[
    ],
};


pub const ITY: LanguageCode = LanguageCode {
    name: "Moyadan Itneg",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ity",
    individual_languages: &[
    ],
};


pub const ITZ: LanguageCode = LanguageCode {
    name: "Itzá",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "itz",
    individual_languages: &[
    ],
};


pub const IUM: LanguageCode = LanguageCode {
    name: "Iu Mien",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ium",
    individual_languages: &[
    ],
};


pub const IVB: LanguageCode = LanguageCode {
    name: "Ibatan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ivb",
    individual_languages: &[
    ],
};


pub const IVV: LanguageCode = LanguageCode {
    name: "Ivatan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ivv",
    individual_languages: &[
    ],
};


pub const IWK: LanguageCode = LanguageCode {
    name: "I-Wak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "iwk",
    individual_languages: &[
    ],
};


pub const IWM: LanguageCode = LanguageCode {
    name: "Iwam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "iwm",
    individual_languages: &[
    ],
};


pub const IWO: LanguageCode = LanguageCode {
    name: "Iwur",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "iwo",
    individual_languages: &[
    ],
};


pub const IWS: LanguageCode = LanguageCode {
    name: "Sepik Iwam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "iws",
    individual_languages: &[
    ],
};


pub const IXC: LanguageCode = LanguageCode {
    name: "Ixcatec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ixc",
    individual_languages: &[
    ],
};


pub const IXL: LanguageCode = LanguageCode {
    name: "Ixil",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ixl",
    individual_languages: &[
    ],
};


pub const IYA: LanguageCode = LanguageCode {
    name: "Iyayu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "iya",
    individual_languages: &[
    ],
};


pub const IYO: LanguageCode = LanguageCode {
    name: "Mesaka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "iyo",
    individual_languages: &[
    ],
};


pub const IYX: LanguageCode = LanguageCode {
    name: "Yaka (Congo)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "iyx",
    individual_languages: &[
    ],
};


pub const IZH: LanguageCode = LanguageCode {
    name: "Ingrian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "izh",
    individual_languages: &[
    ],
};


pub const IZR: LanguageCode = LanguageCode {
    name: "Izere",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "izr",
    individual_languages: &[
    ],
};


pub const IZZ: LanguageCode = LanguageCode {
    name: "Izii",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "izz",
    individual_languages: &[
    ],
};


pub const JAA: LanguageCode = LanguageCode {
    name: "Jamamadí",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jaa",
    individual_languages: &[
    ],
};


pub const JAB: LanguageCode = LanguageCode {
    name: "Hyam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jab",
    individual_languages: &[
    ],
};


pub const JAC: LanguageCode = LanguageCode {
    name: "Popti'",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jac",
    individual_languages: &[
    ],
};


pub const JAD: LanguageCode = LanguageCode {
    name: "Jahanka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jad",
    individual_languages: &[
    ],
};


pub const JAE: LanguageCode = LanguageCode {
    name: "Yabem",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jae",
    individual_languages: &[
    ],
};


pub const JAF: LanguageCode = LanguageCode {
    name: "Jara",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jaf",
    individual_languages: &[
    ],
};


pub const JAH: LanguageCode = LanguageCode {
    name: "Jah Hut",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jah",
    individual_languages: &[
    ],
};


pub const JAJ: LanguageCode = LanguageCode {
    name: "Zazao",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jaj",
    individual_languages: &[
    ],
};


pub const JAK: LanguageCode = LanguageCode {
    name: "Jakun",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jak",
    individual_languages: &[
    ],
};


pub const JAL: LanguageCode = LanguageCode {
    name: "Yalahatan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jal",
    individual_languages: &[
    ],
};


pub const JAM: LanguageCode = LanguageCode {
    name: "Jamaican Creole English",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jam",
    individual_languages: &[
    ],
};


pub const JAN: LanguageCode = LanguageCode {
    name: "Jandai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jan",
    individual_languages: &[
    ],
};


pub const JAO: LanguageCode = LanguageCode {
    name: "Yanyuwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jao",
    individual_languages: &[
    ],
};


pub const JAQ: LanguageCode = LanguageCode {
    name: "Yaqay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jaq",
    individual_languages: &[
    ],
};


pub const JAS: LanguageCode = LanguageCode {
    name: "New Caledonian Javanese",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jas",
    individual_languages: &[
    ],
};


pub const JAT: LanguageCode = LanguageCode {
    name: "Jakati",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jat",
    individual_languages: &[
    ],
};


pub const JAU: LanguageCode = LanguageCode {
    name: "Yaur",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jau",
    individual_languages: &[
    ],
};


pub const JAX: LanguageCode = LanguageCode {
    name: "Jambi Malay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jax",
    individual_languages: &[
    ],
};


pub const JAY: LanguageCode = LanguageCode {
    name: "Yan-nhangu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jay",
    individual_languages: &[
    ],
};


pub const JAZ: LanguageCode = LanguageCode {
    name: "Jawe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jaz",
    individual_languages: &[
    ],
};


pub const JBE: LanguageCode = LanguageCode {
    name: "Judeo-Berber",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jbe",
    individual_languages: &[
    ],
};


pub const JBI: LanguageCode = LanguageCode {
    name: "Badjiri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jbi",
    individual_languages: &[
    ],
};


pub const JBJ: LanguageCode = LanguageCode {
    name: "Arandai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jbj",
    individual_languages: &[
    ],
};


pub const JBK: LanguageCode = LanguageCode {
    name: "Barikewa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jbk",
    individual_languages: &[
    ],
};


pub const JBM: LanguageCode = LanguageCode {
    name: "Bijim",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jbm",
    individual_languages: &[
    ],
};


pub const JBN: LanguageCode = LanguageCode {
    name: "Nafusi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jbn",
    individual_languages: &[
    ],
};


pub const JBR: LanguageCode = LanguageCode {
    name: "Jofotek-Bromnya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jbr",
    individual_languages: &[
    ],
};


pub const JBT: LanguageCode = LanguageCode {
    name: "Jabutí",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jbt",
    individual_languages: &[
    ],
};


pub const JBU: LanguageCode = LanguageCode {
    name: "Jukun Takum",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jbu",
    individual_languages: &[
    ],
};


pub const JBW: LanguageCode = LanguageCode {
    name: "Yawijibaya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jbw",
    individual_languages: &[
    ],
};


pub const JCS: LanguageCode = LanguageCode {
    name: "Jamaican Country Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jcs",
    individual_languages: &[
    ],
};


pub const JCT: LanguageCode = LanguageCode {
    name: "Krymchak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jct",
    individual_languages: &[
    ],
};


pub const JDA: LanguageCode = LanguageCode {
    name: "Jad",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jda",
    individual_languages: &[
    ],
};


pub const JDG: LanguageCode = LanguageCode {
    name: "Jadgali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jdg",
    individual_languages: &[
    ],
};


pub const JDT: LanguageCode = LanguageCode {
    name: "Judeo-Tat",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jdt",
    individual_languages: &[
    ],
};


pub const JEB: LanguageCode = LanguageCode {
    name: "Jebero",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jeb",
    individual_languages: &[
    ],
};


pub const JEE: LanguageCode = LanguageCode {
    name: "Jerung",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jee",
    individual_languages: &[
    ],
};


pub const JEH: LanguageCode = LanguageCode {
    name: "Jeh",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jeh",
    individual_languages: &[
    ],
};


pub const JEI: LanguageCode = LanguageCode {
    name: "Yei",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jei",
    individual_languages: &[
    ],
};


pub const JEK: LanguageCode = LanguageCode {
    name: "Jeri Kuo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jek",
    individual_languages: &[
    ],
};


pub const JEL: LanguageCode = LanguageCode {
    name: "Yelmek",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jel",
    individual_languages: &[
    ],
};


pub const JEN: LanguageCode = LanguageCode {
    name: "Dza",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jen",
    individual_languages: &[
    ],
};


pub const JER: LanguageCode = LanguageCode {
    name: "Jere",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jer",
    individual_languages: &[
    ],
};


pub const JET: LanguageCode = LanguageCode {
    name: "Manem",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jet",
    individual_languages: &[
    ],
};


pub const JEU: LanguageCode = LanguageCode {
    name: "Jonkor Bourmataguil",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jeu",
    individual_languages: &[
    ],
};


pub const JGB: LanguageCode = LanguageCode {
    name: "Ngbee",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jgb",
    individual_languages: &[
    ],
};


pub const JGE: LanguageCode = LanguageCode {
    name: "Judeo-Georgian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jge",
    individual_languages: &[
    ],
};


pub const JGK: LanguageCode = LanguageCode {
    name: "Gwak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jgk",
    individual_languages: &[
    ],
};


pub const JGO: LanguageCode = LanguageCode {
    name: "Ngomba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jgo",
    individual_languages: &[
    ],
};


pub const JHI: LanguageCode = LanguageCode {
    name: "Jehai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jhi",
    individual_languages: &[
    ],
};


pub const JHS: LanguageCode = LanguageCode {
    name: "Jhankot Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jhs",
    individual_languages: &[
    ],
};


pub const JIA: LanguageCode = LanguageCode {
    name: "Jina",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jia",
    individual_languages: &[
    ],
};


pub const JIB: LanguageCode = LanguageCode {
    name: "Jibu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jib",
    individual_languages: &[
    ],
};


pub const JIC: LanguageCode = LanguageCode {
    name: "Tol",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jic",
    individual_languages: &[
    ],
};


pub const JID: LanguageCode = LanguageCode {
    name: "Bu (Kaduna State)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jid",
    individual_languages: &[
    ],
};


pub const JIE: LanguageCode = LanguageCode {
    name: "Jilbe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jie",
    individual_languages: &[
    ],
};


pub const JIG: LanguageCode = LanguageCode {
    name: "Jingulu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jig",
    individual_languages: &[
    ],
};


pub const JIH: LanguageCode = LanguageCode {
    name: "sTodsde",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jih",
    individual_languages: &[
    ],
};


pub const JII: LanguageCode = LanguageCode {
    name: "Jiiddu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jii",
    individual_languages: &[
    ],
};


pub const JIL: LanguageCode = LanguageCode {
    name: "Jilim",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jil",
    individual_languages: &[
    ],
};


pub const JIM: LanguageCode = LanguageCode {
    name: "Jimi (Cameroon)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jim",
    individual_languages: &[
    ],
};


pub const JIO: LanguageCode = LanguageCode {
    name: "Jiamao",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jio",
    individual_languages: &[
    ],
};


pub const JIQ: LanguageCode = LanguageCode {
    name: "Guanyinqiao",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jiq",
    individual_languages: &[
    ],
};


pub const JIT: LanguageCode = LanguageCode {
    name: "Jita",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jit",
    individual_languages: &[
    ],
};


pub const JIU: LanguageCode = LanguageCode {
    name: "Youle Jinuo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jiu",
    individual_languages: &[
    ],
};


pub const JIV: LanguageCode = LanguageCode {
    name: "Shuar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jiv",
    individual_languages: &[
    ],
};


pub const JIY: LanguageCode = LanguageCode {
    name: "Buyuan Jinuo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jiy",
    individual_languages: &[
    ],
};


pub const JJE: LanguageCode = LanguageCode {
    name: "Jejueo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jje",
    individual_languages: &[
    ],
};


pub const JJR: LanguageCode = LanguageCode {
    name: "Bankal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jjr",
    individual_languages: &[
    ],
};


pub const JKA: LanguageCode = LanguageCode {
    name: "Kaera",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jka",
    individual_languages: &[
    ],
};


pub const JKM: LanguageCode = LanguageCode {
    name: "Mobwa Karen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jkm",
    individual_languages: &[
    ],
};


pub const JKO: LanguageCode = LanguageCode {
    name: "Kubo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jko",
    individual_languages: &[
    ],
};


pub const JKP: LanguageCode = LanguageCode {
    name: "Paku Karen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jkp",
    individual_languages: &[
    ],
};


pub const JKR: LanguageCode = LanguageCode {
    name: "Koro (India)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jkr",
    individual_languages: &[
    ],
};


pub const JKS: LanguageCode = LanguageCode {
    name: "Amami Koniya Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jks",
    individual_languages: &[
    ],
};


pub const JKU: LanguageCode = LanguageCode {
    name: "Labir",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jku",
    individual_languages: &[
    ],
};


pub const JLE: LanguageCode = LanguageCode {
    name: "Ngile",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jle",
    individual_languages: &[
    ],
};


pub const JLS: LanguageCode = LanguageCode {
    name: "Jamaican Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jls",
    individual_languages: &[
    ],
};


pub const JMA: LanguageCode = LanguageCode {
    name: "Dima",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jma",
    individual_languages: &[
    ],
};


pub const JMB: LanguageCode = LanguageCode {
    name: "Zumbun",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jmb",
    individual_languages: &[
    ],
};


pub const JMC: LanguageCode = LanguageCode {
    name: "Machame",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jmc",
    individual_languages: &[
    ],
};


pub const JMD: LanguageCode = LanguageCode {
    name: "Yamdena",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jmd",
    individual_languages: &[
    ],
};


pub const JMI: LanguageCode = LanguageCode {
    name: "Jimi (Nigeria)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jmi",
    individual_languages: &[
    ],
};


pub const JML: LanguageCode = LanguageCode {
    name: "Jumli",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jml",
    individual_languages: &[
    ],
};


pub const JMN: LanguageCode = LanguageCode {
    name: "Makuri Naga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jmn",
    individual_languages: &[
    ],
};


pub const JMR: LanguageCode = LanguageCode {
    name: "Kamara",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jmr",
    individual_languages: &[
    ],
};


pub const JMS: LanguageCode = LanguageCode {
    name: "Mashi (Nigeria)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jms",
    individual_languages: &[
    ],
};


pub const JMW: LanguageCode = LanguageCode {
    name: "Mouwase",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jmw",
    individual_languages: &[
    ],
};


pub const JMX: LanguageCode = LanguageCode {
    name: "Western Juxtlahuaca Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jmx",
    individual_languages: &[
    ],
};


pub const JNA: LanguageCode = LanguageCode {
    name: "Jangshung",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jna",
    individual_languages: &[
    ],
};


pub const JND: LanguageCode = LanguageCode {
    name: "Jandavra",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jnd",
    individual_languages: &[
    ],
};


pub const JNG: LanguageCode = LanguageCode {
    name: "Yangman",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jng",
    individual_languages: &[
    ],
};


pub const JNI: LanguageCode = LanguageCode {
    name: "Janji",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jni",
    individual_languages: &[
    ],
};


pub const JNJ: LanguageCode = LanguageCode {
    name: "Yemsa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jnj",
    individual_languages: &[
    ],
};


pub const JNL: LanguageCode = LanguageCode {
    name: "Rawat",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jnl",
    individual_languages: &[
    ],
};


pub const JNS: LanguageCode = LanguageCode {
    name: "Jaunsari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jns",
    individual_languages: &[
    ],
};


pub const JOB: LanguageCode = LanguageCode {
    name: "Joba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "job",
    individual_languages: &[
    ],
};


pub const JOD: LanguageCode = LanguageCode {
    name: "Wojenaka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jod",
    individual_languages: &[
    ],
};


pub const JOG: LanguageCode = LanguageCode {
    name: "Jogi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jog",
    individual_languages: &[
    ],
};


pub const JOR: LanguageCode = LanguageCode {
    name: "Jorá",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jor",
    individual_languages: &[
    ],
};


pub const JOS: LanguageCode = LanguageCode {
    name: "Jordanian Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jos",
    individual_languages: &[
    ],
};


pub const JOW: LanguageCode = LanguageCode {
    name: "Jowulu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jow",
    individual_languages: &[
    ],
};


pub const JPA: LanguageCode = LanguageCode {
    name: "Jewish Palestinian Aramaic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jpa",
    individual_languages: &[
    ],
};


pub const JQR: LanguageCode = LanguageCode {
    name: "Jaqaru",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jqr",
    individual_languages: &[
    ],
};


pub const JRA: LanguageCode = LanguageCode {
    name: "Jarai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jra",
    individual_languages: &[
    ],
};


pub const JRR: LanguageCode = LanguageCode {
    name: "Jiru",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jrr",
    individual_languages: &[
    ],
};


pub const JRT: LanguageCode = LanguageCode {
    name: "Jakattoe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jrt",
    individual_languages: &[
    ],
};


pub const JRU: LanguageCode = LanguageCode {
    name: "Japrería",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jru",
    individual_languages: &[
    ],
};


pub const JSL: LanguageCode = LanguageCode {
    name: "Japanese Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jsl",
    individual_languages: &[
    ],
};


pub const JUA: LanguageCode = LanguageCode {
    name: "Júma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jua",
    individual_languages: &[
    ],
};


pub const JUB: LanguageCode = LanguageCode {
    name: "Wannu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jub",
    individual_languages: &[
    ],
};


pub const JUC: LanguageCode = LanguageCode {
    name: "Jurchen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "juc",
    individual_languages: &[
    ],
};


pub const JUD: LanguageCode = LanguageCode {
    name: "Worodougou",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jud",
    individual_languages: &[
    ],
};


pub const JUH: LanguageCode = LanguageCode {
    name: "Hõne",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "juh",
    individual_languages: &[
    ],
};


pub const JUI: LanguageCode = LanguageCode {
    name: "Ngadjuri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jui",
    individual_languages: &[
    ],
};


pub const JUK: LanguageCode = LanguageCode {
    name: "Wapan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "juk",
    individual_languages: &[
    ],
};


pub const JUL: LanguageCode = LanguageCode {
    name: "Jirel",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jul",
    individual_languages: &[
    ],
};


pub const JUM: LanguageCode = LanguageCode {
    name: "Jumjum",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jum",
    individual_languages: &[
    ],
};


pub const JUN: LanguageCode = LanguageCode {
    name: "Juang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jun",
    individual_languages: &[
    ],
};


pub const JUO: LanguageCode = LanguageCode {
    name: "Jiba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "juo",
    individual_languages: &[
    ],
};


pub const JUP: LanguageCode = LanguageCode {
    name: "Hupdë",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jup",
    individual_languages: &[
    ],
};


pub const JUR: LanguageCode = LanguageCode {
    name: "Jurúna",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jur",
    individual_languages: &[
    ],
};


pub const JUS: LanguageCode = LanguageCode {
    name: "Jumla Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jus",
    individual_languages: &[
    ],
};


pub const JUT: LanguageCode = LanguageCode {
    name: "Jutish",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jut",
    individual_languages: &[
    ],
};


pub const JUU: LanguageCode = LanguageCode {
    name: "Ju",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "juu",
    individual_languages: &[
    ],
};


pub const JUW: LanguageCode = LanguageCode {
    name: "Wãpha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "juw",
    individual_languages: &[
    ],
};


pub const JUY: LanguageCode = LanguageCode {
    name: "Juray",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "juy",
    individual_languages: &[
    ],
};


pub const JVD: LanguageCode = LanguageCode {
    name: "Javindo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jvd",
    individual_languages: &[
    ],
};


pub const JVN: LanguageCode = LanguageCode {
    name: "Caribbean Javanese",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jvn",
    individual_languages: &[
    ],
};


pub const JWI: LanguageCode = LanguageCode {
    name: "Jwira-Pepesa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jwi",
    individual_languages: &[
    ],
};


pub const JYA: LanguageCode = LanguageCode {
    name: "Jiarong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jya",
    individual_languages: &[
    ],
};


pub const JYE: LanguageCode = LanguageCode {
    name: "Judeo-Yemeni Arabic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jye",
    individual_languages: &[
    ],
};


pub const JYY: LanguageCode = LanguageCode {
    name: "Jaya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "jyy",
    individual_languages: &[
    ],
};


pub const KAD: LanguageCode = LanguageCode {
    name: "Adara",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kad",
    individual_languages: &[
    ],
};


pub const KAE: LanguageCode = LanguageCode {
    name: "Ketangalan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kae",
    individual_languages: &[
    ],
};


pub const KAF: LanguageCode = LanguageCode {
    name: "Katso",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kaf",
    individual_languages: &[
    ],
};


pub const KAG: LanguageCode = LanguageCode {
    name: "Kajaman",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kag",
    individual_languages: &[
    ],
};


pub const KAH: LanguageCode = LanguageCode {
    name: "Kara (Central African Republic)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kah",
    individual_languages: &[
    ],
};


pub const KAI: LanguageCode = LanguageCode {
    name: "Karekare",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kai",
    individual_languages: &[
    ],
};


pub const KAJ: LanguageCode = LanguageCode {
    name: "Jju",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kaj",
    individual_languages: &[
    ],
};


pub const KAK: LanguageCode = LanguageCode {
    name: "Kalanguya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kak",
    individual_languages: &[
    ],
};


pub const KAO: LanguageCode = LanguageCode {
    name: "Xaasongaxango",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kao",
    individual_languages: &[
    ],
};


pub const KAP: LanguageCode = LanguageCode {
    name: "Bezhta",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kap",
    individual_languages: &[
    ],
};


pub const KAQ: LanguageCode = LanguageCode {
    name: "Capanahua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kaq",
    individual_languages: &[
    ],
};


pub const KAV: LanguageCode = LanguageCode {
    name: "Katukína",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kav",
    individual_languages: &[
    ],
};


pub const KAX: LanguageCode = LanguageCode {
    name: "Kao",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kax",
    individual_languages: &[
    ],
};


pub const KAY: LanguageCode = LanguageCode {
    name: "Kamayurá",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kay",
    individual_languages: &[
    ],
};


pub const KBA: LanguageCode = LanguageCode {
    name: "Kalarko",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kba",
    individual_languages: &[
    ],
};


pub const KBB: LanguageCode = LanguageCode {
    name: "Kaxuiâna",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kbb",
    individual_languages: &[
    ],
};


pub const KBC: LanguageCode = LanguageCode {
    name: "Kadiwéu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kbc",
    individual_languages: &[
    ],
};


pub const KBE: LanguageCode = LanguageCode {
    name: "Kanju",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kbe",
    individual_languages: &[
    ],
};


pub const KBG: LanguageCode = LanguageCode {
    name: "Khamba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kbg",
    individual_languages: &[
    ],
};


pub const KBH: LanguageCode = LanguageCode {
    name: "Camsá",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kbh",
    individual_languages: &[
    ],
};


pub const KBI: LanguageCode = LanguageCode {
    name: "Kaptiau",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kbi",
    individual_languages: &[
    ],
};


pub const KBJ: LanguageCode = LanguageCode {
    name: "Kari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kbj",
    individual_languages: &[
    ],
};


pub const KBK: LanguageCode = LanguageCode {
    name: "Grass Koiari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kbk",
    individual_languages: &[
    ],
};


pub const KBL: LanguageCode = LanguageCode {
    name: "Kanembu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kbl",
    individual_languages: &[
    ],
};


pub const KBM: LanguageCode = LanguageCode {
    name: "Iwal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kbm",
    individual_languages: &[
    ],
};


pub const KBN: LanguageCode = LanguageCode {
    name: "Kare (Central African Republic)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kbn",
    individual_languages: &[
    ],
};


pub const KBO: LanguageCode = LanguageCode {
    name: "Keliko",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kbo",
    individual_languages: &[
    ],
};


pub const KBP: LanguageCode = LanguageCode {
    name: "Kabiyè",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kbp",
    individual_languages: &[
    ],
};


pub const KBQ: LanguageCode = LanguageCode {
    name: "Kamano",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kbq",
    individual_languages: &[
    ],
};


pub const KBR: LanguageCode = LanguageCode {
    name: "Kafa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kbr",
    individual_languages: &[
    ],
};


pub const KBS: LanguageCode = LanguageCode {
    name: "Kande",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kbs",
    individual_languages: &[
    ],
};


pub const KBT: LanguageCode = LanguageCode {
    name: "Abadi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kbt",
    individual_languages: &[
    ],
};


pub const KBU: LanguageCode = LanguageCode {
    name: "Kabutra",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kbu",
    individual_languages: &[
    ],
};


pub const KBV: LanguageCode = LanguageCode {
    name: "Dera (Indonesia)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kbv",
    individual_languages: &[
    ],
};


pub const KBW: LanguageCode = LanguageCode {
    name: "Kaiep",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kbw",
    individual_languages: &[
    ],
};


pub const KBX: LanguageCode = LanguageCode {
    name: "Ap Ma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kbx",
    individual_languages: &[
    ],
};


pub const KBY: LanguageCode = LanguageCode {
    name: "Manga Kanuri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kby",
    individual_languages: &[
    ],
};


pub const KBZ: LanguageCode = LanguageCode {
    name: "Duhwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kbz",
    individual_languages: &[
    ],
};


pub const KCA: LanguageCode = LanguageCode {
    name: "Khanty",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kca",
    individual_languages: &[
    ],
};


pub const KCB: LanguageCode = LanguageCode {
    name: "Kawacha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kcb",
    individual_languages: &[
    ],
};


pub const KCC: LanguageCode = LanguageCode {
    name: "Lubila",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kcc",
    individual_languages: &[
    ],
};


pub const KCD: LanguageCode = LanguageCode {
    name: "Ngkâlmpw Kanum",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kcd",
    individual_languages: &[
    ],
};


pub const KCE: LanguageCode = LanguageCode {
    name: "Kaivi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kce",
    individual_languages: &[
    ],
};


pub const KCF: LanguageCode = LanguageCode {
    name: "Ukaan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kcf",
    individual_languages: &[
    ],
};


pub const KCG: LanguageCode = LanguageCode {
    name: "Tyap",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kcg",
    individual_languages: &[
    ],
};


pub const KCH: LanguageCode = LanguageCode {
    name: "Vono",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kch",
    individual_languages: &[
    ],
};


pub const KCI: LanguageCode = LanguageCode {
    name: "Kamantan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kci",
    individual_languages: &[
    ],
};


pub const KCJ: LanguageCode = LanguageCode {
    name: "Kobiana",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kcj",
    individual_languages: &[
    ],
};


pub const KCK: LanguageCode = LanguageCode {
    name: "Kalanga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kck",
    individual_languages: &[
    ],
};


pub const KCL: LanguageCode = LanguageCode {
    name: "Kela (Papua New Guinea)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kcl",
    individual_languages: &[
    ],
};


pub const KCM: LanguageCode = LanguageCode {
    name: "Gula (Central African Republic)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kcm",
    individual_languages: &[
    ],
};


pub const KCN: LanguageCode = LanguageCode {
    name: "Nubi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kcn",
    individual_languages: &[
    ],
};


pub const KCO: LanguageCode = LanguageCode {
    name: "Kinalakna",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kco",
    individual_languages: &[
    ],
};


pub const KCP: LanguageCode = LanguageCode {
    name: "Kanga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kcp",
    individual_languages: &[
    ],
};


pub const KCQ: LanguageCode = LanguageCode {
    name: "Kamo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kcq",
    individual_languages: &[
    ],
};


pub const KCR: LanguageCode = LanguageCode {
    name: "Katla",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kcr",
    individual_languages: &[
    ],
};


pub const KCS: LanguageCode = LanguageCode {
    name: "Koenoem",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kcs",
    individual_languages: &[
    ],
};


pub const KCT: LanguageCode = LanguageCode {
    name: "Kaian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kct",
    individual_languages: &[
    ],
};


pub const KCU: LanguageCode = LanguageCode {
    name: "Kami (Tanzania)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kcu",
    individual_languages: &[
    ],
};


pub const KCV: LanguageCode = LanguageCode {
    name: "Kete",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kcv",
    individual_languages: &[
    ],
};


pub const KCW: LanguageCode = LanguageCode {
    name: "Kabwari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kcw",
    individual_languages: &[
    ],
};


pub const KCX: LanguageCode = LanguageCode {
    name: "Kachama-Ganjule",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kcx",
    individual_languages: &[
    ],
};


pub const KCY: LanguageCode = LanguageCode {
    name: "Korandje",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kcy",
    individual_languages: &[
    ],
};


pub const KCZ: LanguageCode = LanguageCode {
    name: "Konongo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kcz",
    individual_languages: &[
    ],
};


pub const KDA: LanguageCode = LanguageCode {
    name: "Worimi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kda",
    individual_languages: &[
    ],
};


pub const KDC: LanguageCode = LanguageCode {
    name: "Kutu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kdc",
    individual_languages: &[
    ],
};


pub const KDD: LanguageCode = LanguageCode {
    name: "Yankunytjatjara",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kdd",
    individual_languages: &[
    ],
};


pub const KDE: LanguageCode = LanguageCode {
    name: "Makonde",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kde",
    individual_languages: &[
    ],
};


pub const KDF: LanguageCode = LanguageCode {
    name: "Mamusi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kdf",
    individual_languages: &[
    ],
};


pub const KDG: LanguageCode = LanguageCode {
    name: "Seba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kdg",
    individual_languages: &[
    ],
};


pub const KDH: LanguageCode = LanguageCode {
    name: "Tem",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kdh",
    individual_languages: &[
    ],
};


pub const KDI: LanguageCode = LanguageCode {
    name: "Kumam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kdi",
    individual_languages: &[
    ],
};


pub const KDJ: LanguageCode = LanguageCode {
    name: "Karamojong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kdj",
    individual_languages: &[
    ],
};


pub const KDK: LanguageCode = LanguageCode {
    name: "Numèè",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kdk",
    individual_languages: &[
    ],
};


pub const KDL: LanguageCode = LanguageCode {
    name: "Tsikimba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kdl",
    individual_languages: &[
    ],
};


pub const KDM: LanguageCode = LanguageCode {
    name: "Kagoma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kdm",
    individual_languages: &[
    ],
};


pub const KDN: LanguageCode = LanguageCode {
    name: "Kunda",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kdn",
    individual_languages: &[
    ],
};


pub const KDP: LanguageCode = LanguageCode {
    name: "Kaningdon-Nindem",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kdp",
    individual_languages: &[
    ],
};


pub const KDQ: LanguageCode = LanguageCode {
    name: "Koch",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kdq",
    individual_languages: &[
    ],
};


pub const KDR: LanguageCode = LanguageCode {
    name: "Karaim",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kdr",
    individual_languages: &[
    ],
};


pub const KDT: LanguageCode = LanguageCode {
    name: "Kuy",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kdt",
    individual_languages: &[
    ],
};


pub const KDU: LanguageCode = LanguageCode {
    name: "Kadaru",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kdu",
    individual_languages: &[
    ],
};


pub const KDW: LanguageCode = LanguageCode {
    name: "Koneraw",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kdw",
    individual_languages: &[
    ],
};


pub const KDX: LanguageCode = LanguageCode {
    name: "Kam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kdx",
    individual_languages: &[
    ],
};


pub const KDY: LanguageCode = LanguageCode {
    name: "Keder",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kdy",
    individual_languages: &[
    ],
};


pub const KDZ: LanguageCode = LanguageCode {
    name: "Kwaja",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kdz",
    individual_languages: &[
    ],
};


pub const KEA: LanguageCode = LanguageCode {
    name: "Kabuverdianu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kea",
    individual_languages: &[
    ],
};


pub const KEB: LanguageCode = LanguageCode {
    name: "Kélé",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "keb",
    individual_languages: &[
    ],
};


pub const KEC: LanguageCode = LanguageCode {
    name: "Keiga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kec",
    individual_languages: &[
    ],
};


pub const KED: LanguageCode = LanguageCode {
    name: "Kerewe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ked",
    individual_languages: &[
    ],
};


pub const KEE: LanguageCode = LanguageCode {
    name: "Eastern Keres",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kee",
    individual_languages: &[
    ],
};


pub const KEF: LanguageCode = LanguageCode {
    name: "Kpessi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kef",
    individual_languages: &[
    ],
};


pub const KEG: LanguageCode = LanguageCode {
    name: "Tese",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "keg",
    individual_languages: &[
    ],
};


pub const KEH: LanguageCode = LanguageCode {
    name: "Keak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "keh",
    individual_languages: &[
    ],
};


pub const KEI: LanguageCode = LanguageCode {
    name: "Kei",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kei",
    individual_languages: &[
    ],
};


pub const KEJ: LanguageCode = LanguageCode {
    name: "Kadar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kej",
    individual_languages: &[
    ],
};


pub const KEK: LanguageCode = LanguageCode {
    name: "Kekchí",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kek",
    individual_languages: &[
    ],
};


pub const KEL: LanguageCode = LanguageCode {
    name: "Kela (Democratic Republic of Congo)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kel",
    individual_languages: &[
    ],
};


pub const KEM: LanguageCode = LanguageCode {
    name: "Kemak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kem",
    individual_languages: &[
    ],
};


pub const KEN: LanguageCode = LanguageCode {
    name: "Kenyang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ken",
    individual_languages: &[
    ],
};


pub const KEO: LanguageCode = LanguageCode {
    name: "Kakwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "keo",
    individual_languages: &[
    ],
};


pub const KEP: LanguageCode = LanguageCode {
    name: "Kaikadi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kep",
    individual_languages: &[
    ],
};


pub const KEQ: LanguageCode = LanguageCode {
    name: "Kamar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "keq",
    individual_languages: &[
    ],
};


pub const KER: LanguageCode = LanguageCode {
    name: "Kera",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ker",
    individual_languages: &[
    ],
};


pub const KES: LanguageCode = LanguageCode {
    name: "Kugbo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kes",
    individual_languages: &[
    ],
};


pub const KET: LanguageCode = LanguageCode {
    name: "Ket",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ket",
    individual_languages: &[
    ],
};


pub const KEU: LanguageCode = LanguageCode {
    name: "Akebu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "keu",
    individual_languages: &[
    ],
};


pub const KEV: LanguageCode = LanguageCode {
    name: "Kanikkaran",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kev",
    individual_languages: &[
    ],
};


pub const KEW: LanguageCode = LanguageCode {
    name: "West Kewa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kew",
    individual_languages: &[
    ],
};


pub const KEX: LanguageCode = LanguageCode {
    name: "Kukna",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kex",
    individual_languages: &[
    ],
};


pub const KEY: LanguageCode = LanguageCode {
    name: "Kupia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "key",
    individual_languages: &[
    ],
};


pub const KEZ: LanguageCode = LanguageCode {
    name: "Kukele",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kez",
    individual_languages: &[
    ],
};


pub const KFA: LanguageCode = LanguageCode {
    name: "Kodava",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kfa",
    individual_languages: &[
    ],
};


pub const KFB: LanguageCode = LanguageCode {
    name: "Northwestern Kolami",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kfb",
    individual_languages: &[
    ],
};


pub const KFC: LanguageCode = LanguageCode {
    name: "Konda-Dora",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kfc",
    individual_languages: &[
    ],
};


pub const KFD: LanguageCode = LanguageCode {
    name: "Korra Koraga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kfd",
    individual_languages: &[
    ],
};


pub const KFE: LanguageCode = LanguageCode {
    name: "Kota (India)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kfe",
    individual_languages: &[
    ],
};


pub const KFF: LanguageCode = LanguageCode {
    name: "Koya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kff",
    individual_languages: &[
    ],
};


pub const KFG: LanguageCode = LanguageCode {
    name: "Kudiya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kfg",
    individual_languages: &[
    ],
};


pub const KFH: LanguageCode = LanguageCode {
    name: "Kurichiya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kfh",
    individual_languages: &[
    ],
};


pub const KFI: LanguageCode = LanguageCode {
    name: "Kannada Kurumba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kfi",
    individual_languages: &[
    ],
};


pub const KFJ: LanguageCode = LanguageCode {
    name: "Kemiehua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kfj",
    individual_languages: &[
    ],
};


pub const KFK: LanguageCode = LanguageCode {
    name: "Kinnauri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kfk",
    individual_languages: &[
    ],
};


pub const KFL: LanguageCode = LanguageCode {
    name: "Kung",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kfl",
    individual_languages: &[
    ],
};


pub const KFM: LanguageCode = LanguageCode {
    name: "Khunsari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kfm",
    individual_languages: &[
    ],
};


pub const KFN: LanguageCode = LanguageCode {
    name: "Kuk",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kfn",
    individual_languages: &[
    ],
};


pub const KFO: LanguageCode = LanguageCode {
    name: "Koro (Côte d'Ivoire)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kfo",
    individual_languages: &[
    ],
};


pub const KFP: LanguageCode = LanguageCode {
    name: "Korwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kfp",
    individual_languages: &[
    ],
};


pub const KFQ: LanguageCode = LanguageCode {
    name: "Korku",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kfq",
    individual_languages: &[
    ],
};


pub const KFR: LanguageCode = LanguageCode {
    name: "Kachhi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kfr",
    individual_languages: &[
    ],
};


pub const KFS: LanguageCode = LanguageCode {
    name: "Bilaspuri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kfs",
    individual_languages: &[
    ],
};


pub const KFT: LanguageCode = LanguageCode {
    name: "Kanjari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kft",
    individual_languages: &[
    ],
};


pub const KFU: LanguageCode = LanguageCode {
    name: "Katkari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kfu",
    individual_languages: &[
    ],
};


pub const KFV: LanguageCode = LanguageCode {
    name: "Kurmukar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kfv",
    individual_languages: &[
    ],
};


pub const KFW: LanguageCode = LanguageCode {
    name: "Kharam Naga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kfw",
    individual_languages: &[
    ],
};


pub const KFX: LanguageCode = LanguageCode {
    name: "Kullu Pahari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kfx",
    individual_languages: &[
    ],
};


pub const KFY: LanguageCode = LanguageCode {
    name: "Kumaoni",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kfy",
    individual_languages: &[
    ],
};


pub const KFZ: LanguageCode = LanguageCode {
    name: "Koromfé",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kfz",
    individual_languages: &[
    ],
};


pub const KGA: LanguageCode = LanguageCode {
    name: "Koyaga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kga",
    individual_languages: &[
    ],
};


pub const KGB: LanguageCode = LanguageCode {
    name: "Kawe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kgb",
    individual_languages: &[
    ],
};


pub const KGE: LanguageCode = LanguageCode {
    name: "Komering",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kge",
    individual_languages: &[
    ],
};


pub const KGF: LanguageCode = LanguageCode {
    name: "Kube",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kgf",
    individual_languages: &[
    ],
};


pub const KGG: LanguageCode = LanguageCode {
    name: "Kusunda",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kgg",
    individual_languages: &[
    ],
};


pub const KGI: LanguageCode = LanguageCode {
    name: "Selangor Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kgi",
    individual_languages: &[
    ],
};


pub const KGJ: LanguageCode = LanguageCode {
    name: "Gamale Kham",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kgj",
    individual_languages: &[
    ],
};


pub const KGK: LanguageCode = LanguageCode {
    name: "Kaiwá",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kgk",
    individual_languages: &[
    ],
};


pub const KGL: LanguageCode = LanguageCode {
    name: "Kunggari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kgl",
    individual_languages: &[
    ],
};


pub const KGM: LanguageCode = LanguageCode {
    name: "Karipúna",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kgm",
    individual_languages: &[
    ],
};


pub const KGN: LanguageCode = LanguageCode {
    name: "Karingani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kgn",
    individual_languages: &[
    ],
};


pub const KGO: LanguageCode = LanguageCode {
    name: "Krongo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kgo",
    individual_languages: &[
    ],
};


pub const KGP: LanguageCode = LanguageCode {
    name: "Kaingang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kgp",
    individual_languages: &[
    ],
};


pub const KGQ: LanguageCode = LanguageCode {
    name: "Kamoro",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kgq",
    individual_languages: &[
    ],
};


pub const KGR: LanguageCode = LanguageCode {
    name: "Abun",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kgr",
    individual_languages: &[
    ],
};


pub const KGS: LanguageCode = LanguageCode {
    name: "Kumbainggar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kgs",
    individual_languages: &[
    ],
};


pub const KGT: LanguageCode = LanguageCode {
    name: "Somyev",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kgt",
    individual_languages: &[
    ],
};


pub const KGU: LanguageCode = LanguageCode {
    name: "Kobol",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kgu",
    individual_languages: &[
    ],
};


pub const KGV: LanguageCode = LanguageCode {
    name: "Karas",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kgv",
    individual_languages: &[
    ],
};


pub const KGW: LanguageCode = LanguageCode {
    name: "Karon Dori",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kgw",
    individual_languages: &[
    ],
};


pub const KGX: LanguageCode = LanguageCode {
    name: "Kamaru",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kgx",
    individual_languages: &[
    ],
};


pub const KGY: LanguageCode = LanguageCode {
    name: "Kyerung",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kgy",
    individual_languages: &[
    ],
};


pub const KHB: LanguageCode = LanguageCode {
    name: "Lü",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "khb",
    individual_languages: &[
    ],
};


pub const KHC: LanguageCode = LanguageCode {
    name: "Tukang Besi North",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "khc",
    individual_languages: &[
    ],
};


pub const KHD: LanguageCode = LanguageCode {
    name: "Bädi Kanum",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "khd",
    individual_languages: &[
    ],
};


pub const KHE: LanguageCode = LanguageCode {
    name: "Korowai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "khe",
    individual_languages: &[
    ],
};


pub const KHF: LanguageCode = LanguageCode {
    name: "Khuen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "khf",
    individual_languages: &[
    ],
};


pub const KHG: LanguageCode = LanguageCode {
    name: "Khams Tibetan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "khg",
    individual_languages: &[
    ],
};


pub const KHH: LanguageCode = LanguageCode {
    name: "Kehu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "khh",
    individual_languages: &[
    ],
};


pub const KHJ: LanguageCode = LanguageCode {
    name: "Kuturmi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "khj",
    individual_languages: &[
    ],
};


pub const KHK: LanguageCode = LanguageCode {
    name: "Halh Mongolian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "khk",
    individual_languages: &[
    ],
};


pub const KHL: LanguageCode = LanguageCode {
    name: "Lusi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "khl",
    individual_languages: &[
    ],
};


pub const KHN: LanguageCode = LanguageCode {
    name: "Khandesi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "khn",
    individual_languages: &[
    ],
};


pub const KHP: LanguageCode = LanguageCode {
    name: "Kapori",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "khp",
    individual_languages: &[
    ],
};


pub const KHQ: LanguageCode = LanguageCode {
    name: "Koyra Chiini Songhay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "khq",
    individual_languages: &[
    ],
};


pub const KHR: LanguageCode = LanguageCode {
    name: "Kharia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "khr",
    individual_languages: &[
    ],
};


pub const KHS: LanguageCode = LanguageCode {
    name: "Kasua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "khs",
    individual_languages: &[
    ],
};


pub const KHT: LanguageCode = LanguageCode {
    name: "Khamti",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kht",
    individual_languages: &[
    ],
};


pub const KHU: LanguageCode = LanguageCode {
    name: "Nkhumbi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "khu",
    individual_languages: &[
    ],
};


pub const KHV: LanguageCode = LanguageCode {
    name: "Khvarshi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "khv",
    individual_languages: &[
    ],
};


pub const KHW: LanguageCode = LanguageCode {
    name: "Khowar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "khw",
    individual_languages: &[
    ],
};


pub const KHX: LanguageCode = LanguageCode {
    name: "Kanu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "khx",
    individual_languages: &[
    ],
};


pub const KHY: LanguageCode = LanguageCode {
    name: "Kele (Democratic Republic of Congo)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "khy",
    individual_languages: &[
    ],
};


pub const KHZ: LanguageCode = LanguageCode {
    name: "Keapara",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "khz",
    individual_languages: &[
    ],
};


pub const KIA: LanguageCode = LanguageCode {
    name: "Kim",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kia",
    individual_languages: &[
    ],
};


pub const KIB: LanguageCode = LanguageCode {
    name: "Koalib",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kib",
    individual_languages: &[
    ],
};


pub const KIC: LanguageCode = LanguageCode {
    name: "Kickapoo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kic",
    individual_languages: &[
    ],
};


pub const KID: LanguageCode = LanguageCode {
    name: "Koshin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kid",
    individual_languages: &[
    ],
};


pub const KIE: LanguageCode = LanguageCode {
    name: "Kibet",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kie",
    individual_languages: &[
    ],
};


pub const KIF: LanguageCode = LanguageCode {
    name: "Eastern Parbate Kham",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kif",
    individual_languages: &[
    ],
};


pub const KIG: LanguageCode = LanguageCode {
    name: "Kimaama",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kig",
    individual_languages: &[
    ],
};


pub const KIH: LanguageCode = LanguageCode {
    name: "Kilmeri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kih",
    individual_languages: &[
    ],
};


pub const KII: LanguageCode = LanguageCode {
    name: "Kitsai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kii",
    individual_languages: &[
    ],
};


pub const KIJ: LanguageCode = LanguageCode {
    name: "Kilivila",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kij",
    individual_languages: &[
    ],
};


pub const KIL: LanguageCode = LanguageCode {
    name: "Kariya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kil",
    individual_languages: &[
    ],
};


pub const KIM: LanguageCode = LanguageCode {
    name: "Karagas",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kim",
    individual_languages: &[
    ],
};


pub const KIO: LanguageCode = LanguageCode {
    name: "Kiowa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kio",
    individual_languages: &[
    ],
};


pub const KIP: LanguageCode = LanguageCode {
    name: "Sheshi Kham",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kip",
    individual_languages: &[
    ],
};


pub const KIQ: LanguageCode = LanguageCode {
    name: "Kosadle",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kiq",
    individual_languages: &[
    ],
};


pub const KIS: LanguageCode = LanguageCode {
    name: "Kis",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kis",
    individual_languages: &[
    ],
};


pub const KIT: LanguageCode = LanguageCode {
    name: "Agob",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kit",
    individual_languages: &[
    ],
};


pub const KIU: LanguageCode = LanguageCode {
    name: "Kirmanjki (individual language)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kiu",
    individual_languages: &[
    ],
};


pub const KIV: LanguageCode = LanguageCode {
    name: "Kimbu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kiv",
    individual_languages: &[
    ],
};


pub const KIW: LanguageCode = LanguageCode {
    name: "Northeast Kiwai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kiw",
    individual_languages: &[
    ],
};


pub const KIX: LanguageCode = LanguageCode {
    name: "Khiamniungan Naga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kix",
    individual_languages: &[
    ],
};


pub const KIY: LanguageCode = LanguageCode {
    name: "Kirikiri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kiy",
    individual_languages: &[
    ],
};


pub const KIZ: LanguageCode = LanguageCode {
    name: "Kisi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kiz",
    individual_languages: &[
    ],
};


pub const KJA: LanguageCode = LanguageCode {
    name: "Mlap",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kja",
    individual_languages: &[
    ],
};


pub const KJB: LanguageCode = LanguageCode {
    name: "Q'anjob'al",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kjb",
    individual_languages: &[
    ],
};


pub const KJC: LanguageCode = LanguageCode {
    name: "Coastal Konjo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kjc",
    individual_languages: &[
    ],
};


pub const KJD: LanguageCode = LanguageCode {
    name: "Southern Kiwai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kjd",
    individual_languages: &[
    ],
};


pub const KJE: LanguageCode = LanguageCode {
    name: "Kisar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kje",
    individual_languages: &[
    ],
};


pub const KJG: LanguageCode = LanguageCode {
    name: "Khmu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kjg",
    individual_languages: &[
    ],
};


pub const KJH: LanguageCode = LanguageCode {
    name: "Khakas",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kjh",
    individual_languages: &[
    ],
};


pub const KJI: LanguageCode = LanguageCode {
    name: "Zabana",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kji",
    individual_languages: &[
    ],
};


pub const KJJ: LanguageCode = LanguageCode {
    name: "Khinalugh",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kjj",
    individual_languages: &[
    ],
};


pub const KJK: LanguageCode = LanguageCode {
    name: "Highland Konjo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kjk",
    individual_languages: &[
    ],
};


pub const KJL: LanguageCode = LanguageCode {
    name: "Western Parbate Kham",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kjl",
    individual_languages: &[
    ],
};


pub const KJM: LanguageCode = LanguageCode {
    name: "Kháng",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kjm",
    individual_languages: &[
    ],
};


pub const KJN: LanguageCode = LanguageCode {
    name: "Kunjen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kjn",
    individual_languages: &[
    ],
};


pub const KJO: LanguageCode = LanguageCode {
    name: "Harijan Kinnauri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kjo",
    individual_languages: &[
    ],
};


pub const KJP: LanguageCode = LanguageCode {
    name: "Pwo Eastern Karen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kjp",
    individual_languages: &[
    ],
};


pub const KJQ: LanguageCode = LanguageCode {
    name: "Western Keres",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kjq",
    individual_languages: &[
    ],
};


pub const KJR: LanguageCode = LanguageCode {
    name: "Kurudu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kjr",
    individual_languages: &[
    ],
};


pub const KJS: LanguageCode = LanguageCode {
    name: "East Kewa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kjs",
    individual_languages: &[
    ],
};


pub const KJT: LanguageCode = LanguageCode {
    name: "Phrae Pwo Karen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kjt",
    individual_languages: &[
    ],
};


pub const KJU: LanguageCode = LanguageCode {
    name: "Kashaya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kju",
    individual_languages: &[
    ],
};


pub const KJV: LanguageCode = LanguageCode {
    name: "Kaikavian Literary Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kjv",
    individual_languages: &[
    ],
};


pub const KJX: LanguageCode = LanguageCode {
    name: "Ramopa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kjx",
    individual_languages: &[
    ],
};


pub const KJY: LanguageCode = LanguageCode {
    name: "Erave",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kjy",
    individual_languages: &[
    ],
};


pub const KJZ: LanguageCode = LanguageCode {
    name: "Bumthangkha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kjz",
    individual_languages: &[
    ],
};


pub const KKA: LanguageCode = LanguageCode {
    name: "Kakanda",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kka",
    individual_languages: &[
    ],
};


pub const KKB: LanguageCode = LanguageCode {
    name: "Kwerisa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kkb",
    individual_languages: &[
    ],
};


pub const KKC: LanguageCode = LanguageCode {
    name: "Odoodee",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kkc",
    individual_languages: &[
    ],
};


pub const KKD: LanguageCode = LanguageCode {
    name: "Kinuku",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kkd",
    individual_languages: &[
    ],
};


pub const KKE: LanguageCode = LanguageCode {
    name: "Kakabe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kke",
    individual_languages: &[
    ],
};


pub const KKF: LanguageCode = LanguageCode {
    name: "Kalaktang Monpa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kkf",
    individual_languages: &[
    ],
};


pub const KKG: LanguageCode = LanguageCode {
    name: "Mabaka Valley Kalinga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kkg",
    individual_languages: &[
    ],
};


pub const KKH: LanguageCode = LanguageCode {
    name: "Khün",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kkh",
    individual_languages: &[
    ],
};


pub const KKI: LanguageCode = LanguageCode {
    name: "Kagulu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kki",
    individual_languages: &[
    ],
};


pub const KKJ: LanguageCode = LanguageCode {
    name: "Kako",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kkj",
    individual_languages: &[
    ],
};


pub const KKK: LanguageCode = LanguageCode {
    name: "Kokota",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kkk",
    individual_languages: &[
    ],
};


pub const KKL: LanguageCode = LanguageCode {
    name: "Kosarek Yale",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kkl",
    individual_languages: &[
    ],
};


pub const KKM: LanguageCode = LanguageCode {
    name: "Kiong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kkm",
    individual_languages: &[
    ],
};


pub const KKN: LanguageCode = LanguageCode {
    name: "Kon Keu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kkn",
    individual_languages: &[
    ],
};


pub const KKO: LanguageCode = LanguageCode {
    name: "Karko",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kko",
    individual_languages: &[
    ],
};


pub const KKP: LanguageCode = LanguageCode {
    name: "Gugubera",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kkp",
    individual_languages: &[
    ],
};


pub const KKQ: LanguageCode = LanguageCode {
    name: "Kaeku",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kkq",
    individual_languages: &[
    ],
};


pub const KKR: LanguageCode = LanguageCode {
    name: "Kir-Balar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kkr",
    individual_languages: &[
    ],
};


pub const KKS: LanguageCode = LanguageCode {
    name: "Giiwo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kks",
    individual_languages: &[
    ],
};


pub const KKT: LanguageCode = LanguageCode {
    name: "Koi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kkt",
    individual_languages: &[
    ],
};


pub const KKU: LanguageCode = LanguageCode {
    name: "Tumi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kku",
    individual_languages: &[
    ],
};


pub const KKV: LanguageCode = LanguageCode {
    name: "Kangean",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kkv",
    individual_languages: &[
    ],
};


pub const KKW: LanguageCode = LanguageCode {
    name: "Teke-Kukuya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kkw",
    individual_languages: &[
    ],
};


pub const KKX: LanguageCode = LanguageCode {
    name: "Kohin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kkx",
    individual_languages: &[
    ],
};


pub const KKY: LanguageCode = LanguageCode {
    name: "Guugu Yimidhirr",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kky",
    individual_languages: &[
    ],
};


pub const KKZ: LanguageCode = LanguageCode {
    name: "Kaska",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kkz",
    individual_languages: &[
    ],
};


pub const KLA: LanguageCode = LanguageCode {
    name: "Klamath-Modoc",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kla",
    individual_languages: &[
    ],
};


pub const KLB: LanguageCode = LanguageCode {
    name: "Kiliwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "klb",
    individual_languages: &[
    ],
};


pub const KLC: LanguageCode = LanguageCode {
    name: "Kolbila",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "klc",
    individual_languages: &[
    ],
};


pub const KLD: LanguageCode = LanguageCode {
    name: "Gamilaraay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kld",
    individual_languages: &[
    ],
};


pub const KLE: LanguageCode = LanguageCode {
    name: "Kulung (Nepal)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kle",
    individual_languages: &[
    ],
};


pub const KLF: LanguageCode = LanguageCode {
    name: "Kendeje",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "klf",
    individual_languages: &[
    ],
};


pub const KLG: LanguageCode = LanguageCode {
    name: "Tagakaulo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "klg",
    individual_languages: &[
    ],
};


pub const KLH: LanguageCode = LanguageCode {
    name: "Weliki",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "klh",
    individual_languages: &[
    ],
};


pub const KLI: LanguageCode = LanguageCode {
    name: "Kalumpang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kli",
    individual_languages: &[
    ],
};


pub const KLJ: LanguageCode = LanguageCode {
    name: "Khalaj",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "klj",
    individual_languages: &[
    ],
};


pub const KLK: LanguageCode = LanguageCode {
    name: "Kono (Nigeria)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "klk",
    individual_languages: &[
    ],
};


pub const KLL: LanguageCode = LanguageCode {
    name: "Kagan Kalagan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kll",
    individual_languages: &[
    ],
};


pub const KLM: LanguageCode = LanguageCode {
    name: "Migum",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "klm",
    individual_languages: &[
    ],
};


pub const KLN: LanguageCode = LanguageCode {
    name: "Kalenjin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kln",
    individual_languages: &[
        IndividualLanguages {
            name: "Markweeta",
            code: "enb",
        },
        IndividualLanguages {
            name: "Keiyo",
            code: "eyo",
        },
        IndividualLanguages {
            name: "Nandi",
            code: "niq",
        },
        IndividualLanguages {
            name: "Okiek",
            code: "oki",
        },
        IndividualLanguages {
            name: "Pökoot",
            code: "pko",
        },
        IndividualLanguages {
            name: "Kipsigis",
            code: "sgc",
        },
        IndividualLanguages {
            name: "Sabaot",
            code: "spy",
        },
        IndividualLanguages {
            name: "Terik",
            code: "tec",
        },
        IndividualLanguages {
            name: "Tugen",
            code: "tuy",
        },
    ],
};


pub const KLO: LanguageCode = LanguageCode {
    name: "Kapya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "klo",
    individual_languages: &[
    ],
};


pub const KLP: LanguageCode = LanguageCode {
    name: "Kamasa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "klp",
    individual_languages: &[
    ],
};


pub const KLQ: LanguageCode = LanguageCode {
    name: "Rumu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "klq",
    individual_languages: &[
    ],
};


pub const KLR: LanguageCode = LanguageCode {
    name: "Khaling",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "klr",
    individual_languages: &[
    ],
};


pub const KLS: LanguageCode = LanguageCode {
    name: "Kalasha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kls",
    individual_languages: &[
    ],
};


pub const KLT: LanguageCode = LanguageCode {
    name: "Nukna",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "klt",
    individual_languages: &[
    ],
};


pub const KLU: LanguageCode = LanguageCode {
    name: "Klao",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "klu",
    individual_languages: &[
    ],
};


pub const KLV: LanguageCode = LanguageCode {
    name: "Maskelynes",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "klv",
    individual_languages: &[
    ],
};


pub const KLW: LanguageCode = LanguageCode {
    name: "Tado",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "klw",
    individual_languages: &[
    ],
};


pub const KLX: LanguageCode = LanguageCode {
    name: "Koluwawa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "klx",
    individual_languages: &[
    ],
};


pub const KLY: LanguageCode = LanguageCode {
    name: "Kalao",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kly",
    individual_languages: &[
    ],
};


pub const KLZ: LanguageCode = LanguageCode {
    name: "Kabola",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "klz",
    individual_languages: &[
    ],
};


pub const KMA: LanguageCode = LanguageCode {
    name: "Konni",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kma",
    individual_languages: &[
    ],
};


pub const KMC: LanguageCode = LanguageCode {
    name: "Southern Dong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kmc",
    individual_languages: &[
    ],
};


pub const KMD: LanguageCode = LanguageCode {
    name: "Majukayang Kalinga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kmd",
    individual_languages: &[
    ],
};


pub const KME: LanguageCode = LanguageCode {
    name: "Bakole",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kme",
    individual_languages: &[
    ],
};


pub const KMF: LanguageCode = LanguageCode {
    name: "Kare (Papua New Guinea)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kmf",
    individual_languages: &[
    ],
};


pub const KMG: LanguageCode = LanguageCode {
    name: "Kâte",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kmg",
    individual_languages: &[
    ],
};


pub const KMH: LanguageCode = LanguageCode {
    name: "Kalam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kmh",
    individual_languages: &[
    ],
};


pub const KMI: LanguageCode = LanguageCode {
    name: "Kami (Nigeria)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kmi",
    individual_languages: &[
    ],
};


pub const KMJ: LanguageCode = LanguageCode {
    name: "Kumarbhag Paharia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kmj",
    individual_languages: &[
    ],
};


pub const KMK: LanguageCode = LanguageCode {
    name: "Limos Kalinga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kmk",
    individual_languages: &[
    ],
};


pub const KML: LanguageCode = LanguageCode {
    name: "Tanudan Kalinga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kml",
    individual_languages: &[
    ],
};


pub const KMM: LanguageCode = LanguageCode {
    name: "Kom (India)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kmm",
    individual_languages: &[
    ],
};


pub const KMN: LanguageCode = LanguageCode {
    name: "Awtuw",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kmn",
    individual_languages: &[
    ],
};


pub const KMO: LanguageCode = LanguageCode {
    name: "Kwoma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kmo",
    individual_languages: &[
    ],
};


pub const KMP: LanguageCode = LanguageCode {
    name: "Gimme",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kmp",
    individual_languages: &[
    ],
};


pub const KMQ: LanguageCode = LanguageCode {
    name: "Kwama",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kmq",
    individual_languages: &[
    ],
};


pub const KMR: LanguageCode = LanguageCode {
    name: "Northern Kurdish",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kmr",
    individual_languages: &[
    ],
};


pub const KMS: LanguageCode = LanguageCode {
    name: "Kamasau",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kms",
    individual_languages: &[
    ],
};


pub const KMT: LanguageCode = LanguageCode {
    name: "Kemtuik",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kmt",
    individual_languages: &[
    ],
};


pub const KMU: LanguageCode = LanguageCode {
    name: "Kanite",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kmu",
    individual_languages: &[
    ],
};


pub const KMV: LanguageCode = LanguageCode {
    name: "Karipúna Creole French",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kmv",
    individual_languages: &[
    ],
};


pub const KMW: LanguageCode = LanguageCode {
    name: "Komo (Democratic Republic of Congo)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kmw",
    individual_languages: &[
    ],
};


pub const KMX: LanguageCode = LanguageCode {
    name: "Waboda",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kmx",
    individual_languages: &[
    ],
};


pub const KMY: LanguageCode = LanguageCode {
    name: "Koma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kmy",
    individual_languages: &[
    ],
};


pub const KMZ: LanguageCode = LanguageCode {
    name: "Khorasani Turkish",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kmz",
    individual_languages: &[
    ],
};


pub const KNA: LanguageCode = LanguageCode {
    name: "Dera (Nigeria)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kna",
    individual_languages: &[
    ],
};


pub const KNB: LanguageCode = LanguageCode {
    name: "Lubuagan Kalinga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "knb",
    individual_languages: &[
    ],
};


pub const KNC: LanguageCode = LanguageCode {
    name: "Central Kanuri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "knc",
    individual_languages: &[
    ],
};


pub const KND: LanguageCode = LanguageCode {
    name: "Konda",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "knd",
    individual_languages: &[
    ],
};


pub const KNE: LanguageCode = LanguageCode {
    name: "Kankanaey",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kne",
    individual_languages: &[
    ],
};


pub const KNF: LanguageCode = LanguageCode {
    name: "Mankanya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "knf",
    individual_languages: &[
    ],
};


pub const KNG: LanguageCode = LanguageCode {
    name: "Koongo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kng",
    individual_languages: &[
    ],
};


pub const KNI: LanguageCode = LanguageCode {
    name: "Kanufi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kni",
    individual_languages: &[
    ],
};


pub const KNJ: LanguageCode = LanguageCode {
    name: "Western Kanjobal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "knj",
    individual_languages: &[
    ],
};


pub const KNK: LanguageCode = LanguageCode {
    name: "Kuranko",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "knk",
    individual_languages: &[
    ],
};


pub const KNL: LanguageCode = LanguageCode {
    name: "Keninjal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "knl",
    individual_languages: &[
    ],
};


pub const KNM: LanguageCode = LanguageCode {
    name: "Kanamarí",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "knm",
    individual_languages: &[
    ],
};


pub const KNN: LanguageCode = LanguageCode {
    name: "Konkani (individual language)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "knn",
    individual_languages: &[
    ],
};


pub const KNO: LanguageCode = LanguageCode {
    name: "Kono (Sierra Leone)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kno",
    individual_languages: &[
    ],
};


pub const KNP: LanguageCode = LanguageCode {
    name: "Kwanja",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "knp",
    individual_languages: &[
    ],
};


pub const KNQ: LanguageCode = LanguageCode {
    name: "Kintaq",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "knq",
    individual_languages: &[
    ],
};


pub const KNR: LanguageCode = LanguageCode {
    name: "Kaningra",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "knr",
    individual_languages: &[
    ],
};


pub const KNS: LanguageCode = LanguageCode {
    name: "Kensiu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kns",
    individual_languages: &[
    ],
};


pub const KNT: LanguageCode = LanguageCode {
    name: "Panoan Katukína",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "knt",
    individual_languages: &[
    ],
};


pub const KNU: LanguageCode = LanguageCode {
    name: "Kono (Guinea)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "knu",
    individual_languages: &[
    ],
};


pub const KNV: LanguageCode = LanguageCode {
    name: "Tabo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "knv",
    individual_languages: &[
    ],
};


pub const KNW: LanguageCode = LanguageCode {
    name: "Kung-Ekoka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "knw",
    individual_languages: &[
    ],
};


pub const KNX: LanguageCode = LanguageCode {
    name: "Kendayan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "knx",
    individual_languages: &[
    ],
};


pub const KNY: LanguageCode = LanguageCode {
    name: "Kanyok",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kny",
    individual_languages: &[
    ],
};


pub const KNZ: LanguageCode = LanguageCode {
    name: "Kalamsé",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "knz",
    individual_languages: &[
    ],
};


pub const KOA: LanguageCode = LanguageCode {
    name: "Konomala",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "koa",
    individual_languages: &[
    ],
};


pub const KOC: LanguageCode = LanguageCode {
    name: "Kpati",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "koc",
    individual_languages: &[
    ],
};


pub const KOD: LanguageCode = LanguageCode {
    name: "Kodi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kod",
    individual_languages: &[
    ],
};


pub const KOE: LanguageCode = LanguageCode {
    name: "Kacipo-Bale Suri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "koe",
    individual_languages: &[
    ],
};


pub const KOF: LanguageCode = LanguageCode {
    name: "Kubi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kof",
    individual_languages: &[
    ],
};


pub const KOG: LanguageCode = LanguageCode {
    name: "Cogui",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kog",
    individual_languages: &[
    ],
};


pub const KOH: LanguageCode = LanguageCode {
    name: "Koyo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "koh",
    individual_languages: &[
    ],
};


pub const KOI: LanguageCode = LanguageCode {
    name: "Komi-Permyak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "koi",
    individual_languages: &[
    ],
};


pub const KOL: LanguageCode = LanguageCode {
    name: "Kol (Papua New Guinea)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kol",
    individual_languages: &[
    ],
};


pub const KOO: LanguageCode = LanguageCode {
    name: "Konzo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "koo",
    individual_languages: &[
    ],
};


pub const KOP: LanguageCode = LanguageCode {
    name: "Waube",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kop",
    individual_languages: &[
    ],
};


pub const KOQ: LanguageCode = LanguageCode {
    name: "Kota (Gabon)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "koq",
    individual_languages: &[
    ],
};


pub const KOT: LanguageCode = LanguageCode {
    name: "Lagwan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kot",
    individual_languages: &[
    ],
};


pub const KOU: LanguageCode = LanguageCode {
    name: "Koke",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kou",
    individual_languages: &[
    ],
};


pub const KOV: LanguageCode = LanguageCode {
    name: "Kudu-Camo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kov",
    individual_languages: &[
    ],
};


pub const KOW: LanguageCode = LanguageCode {
    name: "Kugama",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kow",
    individual_languages: &[
    ],
};


pub const KOY: LanguageCode = LanguageCode {
    name: "Koyukon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "koy",
    individual_languages: &[
    ],
};


pub const KOZ: LanguageCode = LanguageCode {
    name: "Korak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "koz",
    individual_languages: &[
    ],
};


pub const KPA: LanguageCode = LanguageCode {
    name: "Kutto",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kpa",
    individual_languages: &[
    ],
};


pub const KPB: LanguageCode = LanguageCode {
    name: "Mullu Kurumba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kpb",
    individual_languages: &[
    ],
};


pub const KPC: LanguageCode = LanguageCode {
    name: "Curripaco",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kpc",
    individual_languages: &[
    ],
};


pub const KPD: LanguageCode = LanguageCode {
    name: "Koba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kpd",
    individual_languages: &[
    ],
};


pub const KPF: LanguageCode = LanguageCode {
    name: "Komba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kpf",
    individual_languages: &[
    ],
};


pub const KPG: LanguageCode = LanguageCode {
    name: "Kapingamarangi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kpg",
    individual_languages: &[
    ],
};


pub const KPH: LanguageCode = LanguageCode {
    name: "Kplang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kph",
    individual_languages: &[
    ],
};


pub const KPI: LanguageCode = LanguageCode {
    name: "Kofei",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kpi",
    individual_languages: &[
    ],
};


pub const KPJ: LanguageCode = LanguageCode {
    name: "Karajá",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kpj",
    individual_languages: &[
    ],
};


pub const KPK: LanguageCode = LanguageCode {
    name: "Kpan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kpk",
    individual_languages: &[
    ],
};


pub const KPL: LanguageCode = LanguageCode {
    name: "Kpala",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kpl",
    individual_languages: &[
    ],
};


pub const KPM: LanguageCode = LanguageCode {
    name: "Koho",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kpm",
    individual_languages: &[
    ],
};


pub const KPN: LanguageCode = LanguageCode {
    name: "Kepkiriwát",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kpn",
    individual_languages: &[
    ],
};


pub const KPO: LanguageCode = LanguageCode {
    name: "Ikposo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kpo",
    individual_languages: &[
    ],
};


pub const KPQ: LanguageCode = LanguageCode {
    name: "Korupun-Sela",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kpq",
    individual_languages: &[
    ],
};


pub const KPR: LanguageCode = LanguageCode {
    name: "Korafe-Yegha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kpr",
    individual_languages: &[
    ],
};


pub const KPS: LanguageCode = LanguageCode {
    name: "Tehit",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kps",
    individual_languages: &[
    ],
};


pub const KPT: LanguageCode = LanguageCode {
    name: "Karata",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kpt",
    individual_languages: &[
    ],
};


pub const KPU: LanguageCode = LanguageCode {
    name: "Kafoa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kpu",
    individual_languages: &[
    ],
};


pub const KPV: LanguageCode = LanguageCode {
    name: "Komi-Zyrian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kpv",
    individual_languages: &[
    ],
};


pub const KPW: LanguageCode = LanguageCode {
    name: "Kobon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kpw",
    individual_languages: &[
    ],
};


pub const KPX: LanguageCode = LanguageCode {
    name: "Mountain Koiali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kpx",
    individual_languages: &[
    ],
};


pub const KPY: LanguageCode = LanguageCode {
    name: "Koryak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kpy",
    individual_languages: &[
    ],
};


pub const KPZ: LanguageCode = LanguageCode {
    name: "Kupsabiny",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kpz",
    individual_languages: &[
    ],
};


pub const KQA: LanguageCode = LanguageCode {
    name: "Mum",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kqa",
    individual_languages: &[
    ],
};


pub const KQB: LanguageCode = LanguageCode {
    name: "Kovai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kqb",
    individual_languages: &[
    ],
};


pub const KQC: LanguageCode = LanguageCode {
    name: "Doromu-Koki",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kqc",
    individual_languages: &[
    ],
};


pub const KQD: LanguageCode = LanguageCode {
    name: "Koy Sanjaq Surat",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kqd",
    individual_languages: &[
    ],
};


pub const KQE: LanguageCode = LanguageCode {
    name: "Kalagan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kqe",
    individual_languages: &[
    ],
};


pub const KQF: LanguageCode = LanguageCode {
    name: "Kakabai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kqf",
    individual_languages: &[
    ],
};


pub const KQG: LanguageCode = LanguageCode {
    name: "Khe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kqg",
    individual_languages: &[
    ],
};


pub const KQH: LanguageCode = LanguageCode {
    name: "Kisankasa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kqh",
    individual_languages: &[
    ],
};


pub const KQI: LanguageCode = LanguageCode {
    name: "Koitabu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kqi",
    individual_languages: &[
    ],
};


pub const KQJ: LanguageCode = LanguageCode {
    name: "Koromira",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kqj",
    individual_languages: &[
    ],
};


pub const KQK: LanguageCode = LanguageCode {
    name: "Kotafon Gbe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kqk",
    individual_languages: &[
    ],
};


pub const KQL: LanguageCode = LanguageCode {
    name: "Kyenele",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kql",
    individual_languages: &[
    ],
};


pub const KQM: LanguageCode = LanguageCode {
    name: "Khisa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kqm",
    individual_languages: &[
    ],
};


pub const KQN: LanguageCode = LanguageCode {
    name: "Kaonde",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kqn",
    individual_languages: &[
    ],
};


pub const KQO: LanguageCode = LanguageCode {
    name: "Eastern Krahn",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kqo",
    individual_languages: &[
    ],
};


pub const KQP: LanguageCode = LanguageCode {
    name: "Kimré",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kqp",
    individual_languages: &[
    ],
};


pub const KQQ: LanguageCode = LanguageCode {
    name: "Krenak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kqq",
    individual_languages: &[
    ],
};


pub const KQR: LanguageCode = LanguageCode {
    name: "Kimaragang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kqr",
    individual_languages: &[
    ],
};


pub const KQS: LanguageCode = LanguageCode {
    name: "Northern Kissi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kqs",
    individual_languages: &[
    ],
};


pub const KQT: LanguageCode = LanguageCode {
    name: "Klias River Kadazan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kqt",
    individual_languages: &[
    ],
};


pub const KQU: LanguageCode = LanguageCode {
    name: "Seroa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kqu",
    individual_languages: &[
    ],
};


pub const KQV: LanguageCode = LanguageCode {
    name: "Okolod",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kqv",
    individual_languages: &[
    ],
};


pub const KQW: LanguageCode = LanguageCode {
    name: "Kandas",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kqw",
    individual_languages: &[
    ],
};


pub const KQX: LanguageCode = LanguageCode {
    name: "Mser",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kqx",
    individual_languages: &[
    ],
};


pub const KQY: LanguageCode = LanguageCode {
    name: "Koorete",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kqy",
    individual_languages: &[
    ],
};


pub const KQZ: LanguageCode = LanguageCode {
    name: "Korana",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kqz",
    individual_languages: &[
    ],
};


pub const KRA: LanguageCode = LanguageCode {
    name: "Kumhali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kra",
    individual_languages: &[
    ],
};


pub const KRB: LanguageCode = LanguageCode {
    name: "Karkin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "krb",
    individual_languages: &[
    ],
};


pub const KRD: LanguageCode = LanguageCode {
    name: "Kairui-Midiki",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "krd",
    individual_languages: &[
    ],
};


pub const KRE: LanguageCode = LanguageCode {
    name: "Panará",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kre",
    individual_languages: &[
    ],
};


pub const KRF: LanguageCode = LanguageCode {
    name: "Koro (Vanuatu)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "krf",
    individual_languages: &[
    ],
};


pub const KRH: LanguageCode = LanguageCode {
    name: "Kurama",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "krh",
    individual_languages: &[
    ],
};


pub const KRI: LanguageCode = LanguageCode {
    name: "Krio",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kri",
    individual_languages: &[
    ],
};


pub const KRJ: LanguageCode = LanguageCode {
    name: "Kinaray-A",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "krj",
    individual_languages: &[
    ],
};


pub const KRK: LanguageCode = LanguageCode {
    name: "Kerek",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "krk",
    individual_languages: &[
    ],
};


pub const KRN: LanguageCode = LanguageCode {
    name: "Sapo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "krn",
    individual_languages: &[
    ],
};


pub const KRP: LanguageCode = LanguageCode {
    name: "Korop",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "krp",
    individual_languages: &[
    ],
};


pub const KRR: LanguageCode = LanguageCode {
    name: "Krung",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "krr",
    individual_languages: &[
    ],
};


pub const KRS: LanguageCode = LanguageCode {
    name: "Gbaya (Sudan)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "krs",
    individual_languages: &[
    ],
};


pub const KRT: LanguageCode = LanguageCode {
    name: "Tumari Kanuri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "krt",
    individual_languages: &[
    ],
};


pub const KRV: LanguageCode = LanguageCode {
    name: "Kavet",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "krv",
    individual_languages: &[
    ],
};


pub const KRW: LanguageCode = LanguageCode {
    name: "Western Krahn",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "krw",
    individual_languages: &[
    ],
};


pub const KRX: LanguageCode = LanguageCode {
    name: "Karon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "krx",
    individual_languages: &[
    ],
};


pub const KRY: LanguageCode = LanguageCode {
    name: "Kryts",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kry",
    individual_languages: &[
    ],
};


pub const KRZ: LanguageCode = LanguageCode {
    name: "Sota Kanum",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "krz",
    individual_languages: &[
    ],
};


pub const KSA: LanguageCode = LanguageCode {
    name: "Shuwa-Zamani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ksa",
    individual_languages: &[
    ],
};


pub const KSB: LanguageCode = LanguageCode {
    name: "Shambala",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ksb",
    individual_languages: &[
    ],
};


pub const KSC: LanguageCode = LanguageCode {
    name: "Southern Kalinga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ksc",
    individual_languages: &[
    ],
};


pub const KSD: LanguageCode = LanguageCode {
    name: "Kuanua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ksd",
    individual_languages: &[
    ],
};


pub const KSE: LanguageCode = LanguageCode {
    name: "Kuni",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kse",
    individual_languages: &[
    ],
};


pub const KSF: LanguageCode = LanguageCode {
    name: "Bafia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ksf",
    individual_languages: &[
    ],
};


pub const KSG: LanguageCode = LanguageCode {
    name: "Kusaghe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ksg",
    individual_languages: &[
    ],
};


pub const KSH: LanguageCode = LanguageCode {
    name: "Kölsch",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ksh",
    individual_languages: &[
    ],
};


pub const KSI: LanguageCode = LanguageCode {
    name: "Krisa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ksi",
    individual_languages: &[
    ],
};


pub const KSJ: LanguageCode = LanguageCode {
    name: "Uare",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ksj",
    individual_languages: &[
    ],
};


pub const KSK: LanguageCode = LanguageCode {
    name: "Kansa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ksk",
    individual_languages: &[
    ],
};


pub const KSL: LanguageCode = LanguageCode {
    name: "Kumalu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ksl",
    individual_languages: &[
    ],
};


pub const KSM: LanguageCode = LanguageCode {
    name: "Kumba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ksm",
    individual_languages: &[
    ],
};


pub const KSN: LanguageCode = LanguageCode {
    name: "Kasiguranin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ksn",
    individual_languages: &[
    ],
};


pub const KSO: LanguageCode = LanguageCode {
    name: "Kofa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kso",
    individual_languages: &[
    ],
};


pub const KSP: LanguageCode = LanguageCode {
    name: "Kaba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ksp",
    individual_languages: &[
    ],
};


pub const KSQ: LanguageCode = LanguageCode {
    name: "Kwaami",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ksq",
    individual_languages: &[
    ],
};


pub const KSR: LanguageCode = LanguageCode {
    name: "Borong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ksr",
    individual_languages: &[
    ],
};


pub const KSS: LanguageCode = LanguageCode {
    name: "Southern Kisi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kss",
    individual_languages: &[
    ],
};


pub const KST: LanguageCode = LanguageCode {
    name: "Winyé",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kst",
    individual_languages: &[
    ],
};


pub const KSU: LanguageCode = LanguageCode {
    name: "Khamyang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ksu",
    individual_languages: &[
    ],
};


pub const KSV: LanguageCode = LanguageCode {
    name: "Kusu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ksv",
    individual_languages: &[
    ],
};


pub const KSW: LanguageCode = LanguageCode {
    name: "S'gaw Karen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ksw",
    individual_languages: &[
    ],
};


pub const KSX: LanguageCode = LanguageCode {
    name: "Kedang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ksx",
    individual_languages: &[
    ],
};


pub const KSY: LanguageCode = LanguageCode {
    name: "Kharia Thar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ksy",
    individual_languages: &[
    ],
};


pub const KSZ: LanguageCode = LanguageCode {
    name: "Kodaku",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ksz",
    individual_languages: &[
    ],
};


pub const KTA: LanguageCode = LanguageCode {
    name: "Katua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kta",
    individual_languages: &[
    ],
};


pub const KTB: LanguageCode = LanguageCode {
    name: "Kambaata",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ktb",
    individual_languages: &[
    ],
};


pub const KTC: LanguageCode = LanguageCode {
    name: "Kholok",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ktc",
    individual_languages: &[
    ],
};


pub const KTD: LanguageCode = LanguageCode {
    name: "Kokata",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ktd",
    individual_languages: &[
    ],
};


pub const KTE: LanguageCode = LanguageCode {
    name: "Nubri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kte",
    individual_languages: &[
    ],
};


pub const KTF: LanguageCode = LanguageCode {
    name: "Kwami",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ktf",
    individual_languages: &[
    ],
};


pub const KTG: LanguageCode = LanguageCode {
    name: "Kalkutung",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ktg",
    individual_languages: &[
    ],
};


pub const KTH: LanguageCode = LanguageCode {
    name: "Karanga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kth",
    individual_languages: &[
    ],
};


pub const KTI: LanguageCode = LanguageCode {
    name: "North Muyu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kti",
    individual_languages: &[
    ],
};


pub const KTJ: LanguageCode = LanguageCode {
    name: "Plapo Krumen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ktj",
    individual_languages: &[
    ],
};


pub const KTK: LanguageCode = LanguageCode {
    name: "Kaniet",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ktk",
    individual_languages: &[
    ],
};


pub const KTL: LanguageCode = LanguageCode {
    name: "Koroshi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ktl",
    individual_languages: &[
    ],
};


pub const KTM: LanguageCode = LanguageCode {
    name: "Kurti",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ktm",
    individual_languages: &[
    ],
};


pub const KTN: LanguageCode = LanguageCode {
    name: "Karitiâna",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ktn",
    individual_languages: &[
    ],
};


pub const KTO: LanguageCode = LanguageCode {
    name: "Kuot",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kto",
    individual_languages: &[
    ],
};


pub const KTP: LanguageCode = LanguageCode {
    name: "Kaduo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ktp",
    individual_languages: &[
    ],
};


pub const KTQ: LanguageCode = LanguageCode {
    name: "Katabaga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ktq",
    individual_languages: &[
    ],
};


pub const KTS: LanguageCode = LanguageCode {
    name: "South Muyu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kts",
    individual_languages: &[
    ],
};


pub const KTT: LanguageCode = LanguageCode {
    name: "Ketum",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ktt",
    individual_languages: &[
    ],
};


pub const KTU: LanguageCode = LanguageCode {
    name: "Kituba (Democratic Republic of Congo)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ktu",
    individual_languages: &[
    ],
};


pub const KTV: LanguageCode = LanguageCode {
    name: "Eastern Katu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ktv",
    individual_languages: &[
    ],
};


pub const KTW: LanguageCode = LanguageCode {
    name: "Kato",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ktw",
    individual_languages: &[
    ],
};


pub const KTX: LanguageCode = LanguageCode {
    name: "Kaxararí",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ktx",
    individual_languages: &[
    ],
};


pub const KTY: LanguageCode = LanguageCode {
    name: "Kango (Bas-Uélé District)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kty",
    individual_languages: &[
    ],
};


pub const KTZ: LanguageCode = LanguageCode {
    name: "Juǀʼhoan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ktz",
    individual_languages: &[
    ],
};


pub const KUB: LanguageCode = LanguageCode {
    name: "Kutep",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kub",
    individual_languages: &[
    ],
};


pub const KUC: LanguageCode = LanguageCode {
    name: "Kwinsu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kuc",
    individual_languages: &[
    ],
};


pub const KUD: LanguageCode = LanguageCode {
    name: "'Auhelawa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kud",
    individual_languages: &[
    ],
};


pub const KUE: LanguageCode = LanguageCode {
    name: "Kuman (Papua New Guinea)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kue",
    individual_languages: &[
    ],
};


pub const KUF: LanguageCode = LanguageCode {
    name: "Western Katu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kuf",
    individual_languages: &[
    ],
};


pub const KUG: LanguageCode = LanguageCode {
    name: "Kupa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kug",
    individual_languages: &[
    ],
};


pub const KUH: LanguageCode = LanguageCode {
    name: "Kushi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kuh",
    individual_languages: &[
    ],
};


pub const KUI: LanguageCode = LanguageCode {
    name: "Kuikúro-Kalapálo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kui",
    individual_languages: &[
    ],
};


pub const KUJ: LanguageCode = LanguageCode {
    name: "Kuria",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kuj",
    individual_languages: &[
    ],
};


pub const KUK: LanguageCode = LanguageCode {
    name: "Kepo'",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kuk",
    individual_languages: &[
    ],
};


pub const KUL: LanguageCode = LanguageCode {
    name: "Kulere",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kul",
    individual_languages: &[
    ],
};


pub const KUN: LanguageCode = LanguageCode {
    name: "Kunama",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kun",
    individual_languages: &[
    ],
};


pub const KUO: LanguageCode = LanguageCode {
    name: "Kumukio",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kuo",
    individual_languages: &[
    ],
};


pub const KUP: LanguageCode = LanguageCode {
    name: "Kunimaipa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kup",
    individual_languages: &[
    ],
};


pub const KUQ: LanguageCode = LanguageCode {
    name: "Karipuna",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kuq",
    individual_languages: &[
    ],
};


pub const KUS: LanguageCode = LanguageCode {
    name: "Kusaal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kus",
    individual_languages: &[
    ],
};


pub const KUU: LanguageCode = LanguageCode {
    name: "Upper Kuskokwim",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kuu",
    individual_languages: &[
    ],
};


pub const KUV: LanguageCode = LanguageCode {
    name: "Kur",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kuv",
    individual_languages: &[
    ],
};


pub const KUW: LanguageCode = LanguageCode {
    name: "Kpagua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kuw",
    individual_languages: &[
    ],
};


pub const KUX: LanguageCode = LanguageCode {
    name: "Kukatja",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kux",
    individual_languages: &[
    ],
};


pub const KUY: LanguageCode = LanguageCode {
    name: "Kuuku-Ya'u",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kuy",
    individual_languages: &[
    ],
};


pub const KUZ: LanguageCode = LanguageCode {
    name: "Kunza",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kuz",
    individual_languages: &[
    ],
};


pub const KVA: LanguageCode = LanguageCode {
    name: "Bagvalal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kva",
    individual_languages: &[
    ],
};


pub const KVB: LanguageCode = LanguageCode {
    name: "Kubu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kvb",
    individual_languages: &[
    ],
};


pub const KVC: LanguageCode = LanguageCode {
    name: "Kove",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kvc",
    individual_languages: &[
    ],
};


pub const KVD: LanguageCode = LanguageCode {
    name: "Kui (Indonesia)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kvd",
    individual_languages: &[
    ],
};


pub const KVE: LanguageCode = LanguageCode {
    name: "Kalabakan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kve",
    individual_languages: &[
    ],
};


pub const KVF: LanguageCode = LanguageCode {
    name: "Kabalai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kvf",
    individual_languages: &[
    ],
};


pub const KVG: LanguageCode = LanguageCode {
    name: "Kuni-Boazi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kvg",
    individual_languages: &[
    ],
};


pub const KVH: LanguageCode = LanguageCode {
    name: "Komodo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kvh",
    individual_languages: &[
    ],
};


pub const KVI: LanguageCode = LanguageCode {
    name: "Kwang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kvi",
    individual_languages: &[
    ],
};


pub const KVJ: LanguageCode = LanguageCode {
    name: "Psikye",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kvj",
    individual_languages: &[
    ],
};


pub const KVK: LanguageCode = LanguageCode {
    name: "Korean Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kvk",
    individual_languages: &[
    ],
};


pub const KVL: LanguageCode = LanguageCode {
    name: "Kayaw",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kvl",
    individual_languages: &[
    ],
};


pub const KVM: LanguageCode = LanguageCode {
    name: "Kendem",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kvm",
    individual_languages: &[
    ],
};


pub const KVN: LanguageCode = LanguageCode {
    name: "Border Kuna",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kvn",
    individual_languages: &[
    ],
};


pub const KVO: LanguageCode = LanguageCode {
    name: "Dobel",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kvo",
    individual_languages: &[
    ],
};


pub const KVP: LanguageCode = LanguageCode {
    name: "Kompane",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kvp",
    individual_languages: &[
    ],
};


pub const KVQ: LanguageCode = LanguageCode {
    name: "Geba Karen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kvq",
    individual_languages: &[
    ],
};


pub const KVR: LanguageCode = LanguageCode {
    name: "Kerinci",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kvr",
    individual_languages: &[
    ],
};


pub const KVT: LanguageCode = LanguageCode {
    name: "Lahta Karen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kvt",
    individual_languages: &[
    ],
};


pub const KVU: LanguageCode = LanguageCode {
    name: "Yinbaw Karen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kvu",
    individual_languages: &[
    ],
};


pub const KVV: LanguageCode = LanguageCode {
    name: "Kola",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kvv",
    individual_languages: &[
    ],
};


pub const KVW: LanguageCode = LanguageCode {
    name: "Wersing",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kvw",
    individual_languages: &[
    ],
};


pub const KVX: LanguageCode = LanguageCode {
    name: "Parkari Koli",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kvx",
    individual_languages: &[
    ],
};


pub const KVY: LanguageCode = LanguageCode {
    name: "Yintale Karen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kvy",
    individual_languages: &[
    ],
};


pub const KVZ: LanguageCode = LanguageCode {
    name: "Tsakwambo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kvz",
    individual_languages: &[
    ],
};


pub const KWA: LanguageCode = LanguageCode {
    name: "Dâw",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kwa",
    individual_languages: &[
    ],
};


pub const KWB: LanguageCode = LanguageCode {
    name: "Kwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kwb",
    individual_languages: &[
    ],
};


pub const KWC: LanguageCode = LanguageCode {
    name: "Likwala",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kwc",
    individual_languages: &[
    ],
};


pub const KWD: LanguageCode = LanguageCode {
    name: "Kwaio",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kwd",
    individual_languages: &[
    ],
};


pub const KWE: LanguageCode = LanguageCode {
    name: "Kwerba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kwe",
    individual_languages: &[
    ],
};


pub const KWF: LanguageCode = LanguageCode {
    name: "Kwara'ae",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kwf",
    individual_languages: &[
    ],
};


pub const KWG: LanguageCode = LanguageCode {
    name: "Sara Kaba Deme",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kwg",
    individual_languages: &[
    ],
};


pub const KWH: LanguageCode = LanguageCode {
    name: "Kowiai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kwh",
    individual_languages: &[
    ],
};


pub const KWI: LanguageCode = LanguageCode {
    name: "Awa-Cuaiquer",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kwi",
    individual_languages: &[
    ],
};


pub const KWJ: LanguageCode = LanguageCode {
    name: "Kwanga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kwj",
    individual_languages: &[
    ],
};


pub const KWK: LanguageCode = LanguageCode {
    name: "Kwakiutl",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kwk",
    individual_languages: &[
    ],
};


pub const KWL: LanguageCode = LanguageCode {
    name: "Kofyar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kwl",
    individual_languages: &[
    ],
};


pub const KWM: LanguageCode = LanguageCode {
    name: "Kwambi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kwm",
    individual_languages: &[
    ],
};


pub const KWN: LanguageCode = LanguageCode {
    name: "Kwangali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kwn",
    individual_languages: &[
    ],
};


pub const KWO: LanguageCode = LanguageCode {
    name: "Kwomtari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kwo",
    individual_languages: &[
    ],
};


pub const KWP: LanguageCode = LanguageCode {
    name: "Kodia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kwp",
    individual_languages: &[
    ],
};


pub const KWR: LanguageCode = LanguageCode {
    name: "Kwer",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kwr",
    individual_languages: &[
    ],
};


pub const KWS: LanguageCode = LanguageCode {
    name: "Kwese",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kws",
    individual_languages: &[
    ],
};


pub const KWT: LanguageCode = LanguageCode {
    name: "Kwesten",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kwt",
    individual_languages: &[
    ],
};


pub const KWU: LanguageCode = LanguageCode {
    name: "Kwakum",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kwu",
    individual_languages: &[
    ],
};


pub const KWV: LanguageCode = LanguageCode {
    name: "Sara Kaba Náà",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kwv",
    individual_languages: &[
    ],
};


pub const KWW: LanguageCode = LanguageCode {
    name: "Kwinti",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kww",
    individual_languages: &[
    ],
};


pub const KWX: LanguageCode = LanguageCode {
    name: "Khirwar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kwx",
    individual_languages: &[
    ],
};


pub const KWY: LanguageCode = LanguageCode {
    name: "San Salvador Kongo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kwy",
    individual_languages: &[
    ],
};


pub const KWZ: LanguageCode = LanguageCode {
    name: "Kwadi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kwz",
    individual_languages: &[
    ],
};


pub const KXA: LanguageCode = LanguageCode {
    name: "Kairiru",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kxa",
    individual_languages: &[
    ],
};


pub const KXB: LanguageCode = LanguageCode {
    name: "Krobu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kxb",
    individual_languages: &[
    ],
};


pub const KXC: LanguageCode = LanguageCode {
    name: "Konso",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kxc",
    individual_languages: &[
    ],
};


pub const KXD: LanguageCode = LanguageCode {
    name: "Brunei",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kxd",
    individual_languages: &[
    ],
};


pub const KXF: LanguageCode = LanguageCode {
    name: "Manumanaw Karen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kxf",
    individual_languages: &[
    ],
};


pub const KXH: LanguageCode = LanguageCode {
    name: "Karo (Ethiopia)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kxh",
    individual_languages: &[
    ],
};


pub const KXI: LanguageCode = LanguageCode {
    name: "Keningau Murut",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kxi",
    individual_languages: &[
    ],
};


pub const KXJ: LanguageCode = LanguageCode {
    name: "Kulfa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kxj",
    individual_languages: &[
    ],
};


pub const KXK: LanguageCode = LanguageCode {
    name: "Zayein Karen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kxk",
    individual_languages: &[
    ],
};


pub const KXM: LanguageCode = LanguageCode {
    name: "Northern Khmer",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kxm",
    individual_languages: &[
    ],
};


pub const KXN: LanguageCode = LanguageCode {
    name: "Kanowit-Tanjong Melanau",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kxn",
    individual_languages: &[
    ],
};


pub const KXO: LanguageCode = LanguageCode {
    name: "Kanoé",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kxo",
    individual_languages: &[
    ],
};


pub const KXP: LanguageCode = LanguageCode {
    name: "Wadiyara Koli",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kxp",
    individual_languages: &[
    ],
};


pub const KXQ: LanguageCode = LanguageCode {
    name: "Smärky Kanum",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kxq",
    individual_languages: &[
    ],
};


pub const KXR: LanguageCode = LanguageCode {
    name: "Koro (Papua New Guinea)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kxr",
    individual_languages: &[
    ],
};


pub const KXS: LanguageCode = LanguageCode {
    name: "Kangjia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kxs",
    individual_languages: &[
    ],
};


pub const KXT: LanguageCode = LanguageCode {
    name: "Koiwat",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kxt",
    individual_languages: &[
    ],
};


pub const KXV: LanguageCode = LanguageCode {
    name: "Kuvi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kxv",
    individual_languages: &[
    ],
};


pub const KXW: LanguageCode = LanguageCode {
    name: "Konai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kxw",
    individual_languages: &[
    ],
};


pub const KXX: LanguageCode = LanguageCode {
    name: "Likuba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kxx",
    individual_languages: &[
    ],
};


pub const KXY: LanguageCode = LanguageCode {
    name: "Kayong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kxy",
    individual_languages: &[
    ],
};


pub const KXZ: LanguageCode = LanguageCode {
    name: "Kerewo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kxz",
    individual_languages: &[
    ],
};


pub const KYA: LanguageCode = LanguageCode {
    name: "Kwaya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kya",
    individual_languages: &[
    ],
};


pub const KYB: LanguageCode = LanguageCode {
    name: "Butbut Kalinga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kyb",
    individual_languages: &[
    ],
};


pub const KYC: LanguageCode = LanguageCode {
    name: "Kyaka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kyc",
    individual_languages: &[
    ],
};


pub const KYD: LanguageCode = LanguageCode {
    name: "Karey",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kyd",
    individual_languages: &[
    ],
};


pub const KYE: LanguageCode = LanguageCode {
    name: "Krache",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kye",
    individual_languages: &[
    ],
};


pub const KYF: LanguageCode = LanguageCode {
    name: "Kouya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kyf",
    individual_languages: &[
    ],
};


pub const KYG: LanguageCode = LanguageCode {
    name: "Keyagana",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kyg",
    individual_languages: &[
    ],
};


pub const KYH: LanguageCode = LanguageCode {
    name: "Karok",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kyh",
    individual_languages: &[
    ],
};


pub const KYI: LanguageCode = LanguageCode {
    name: "Kiput",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kyi",
    individual_languages: &[
    ],
};


pub const KYJ: LanguageCode = LanguageCode {
    name: "Karao",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kyj",
    individual_languages: &[
    ],
};


pub const KYK: LanguageCode = LanguageCode {
    name: "Kamayo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kyk",
    individual_languages: &[
    ],
};


pub const KYL: LanguageCode = LanguageCode {
    name: "Kalapuya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kyl",
    individual_languages: &[
    ],
};


pub const KYM: LanguageCode = LanguageCode {
    name: "Kpatili",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kym",
    individual_languages: &[
    ],
};


pub const KYN: LanguageCode = LanguageCode {
    name: "Northern Binukidnon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kyn",
    individual_languages: &[
    ],
};


pub const KYO: LanguageCode = LanguageCode {
    name: "Kelon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kyo",
    individual_languages: &[
    ],
};


pub const KYP: LanguageCode = LanguageCode {
    name: "Kang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kyp",
    individual_languages: &[
    ],
};


pub const KYQ: LanguageCode = LanguageCode {
    name: "Kenga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kyq",
    individual_languages: &[
    ],
};


pub const KYR: LanguageCode = LanguageCode {
    name: "Kuruáya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kyr",
    individual_languages: &[
    ],
};


pub const KYS: LanguageCode = LanguageCode {
    name: "Baram Kayan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kys",
    individual_languages: &[
    ],
};


pub const KYT: LanguageCode = LanguageCode {
    name: "Kayagar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kyt",
    individual_languages: &[
    ],
};


pub const KYU: LanguageCode = LanguageCode {
    name: "Western Kayah",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kyu",
    individual_languages: &[
    ],
};


pub const KYV: LanguageCode = LanguageCode {
    name: "Kayort",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kyv",
    individual_languages: &[
    ],
};


pub const KYW: LanguageCode = LanguageCode {
    name: "Kudmali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kyw",
    individual_languages: &[
    ],
};


pub const KYX: LanguageCode = LanguageCode {
    name: "Rapoisi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kyx",
    individual_languages: &[
    ],
};


pub const KYY: LanguageCode = LanguageCode {
    name: "Kambaira",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kyy",
    individual_languages: &[
    ],
};


pub const KYZ: LanguageCode = LanguageCode {
    name: "Kayabí",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kyz",
    individual_languages: &[
    ],
};


pub const KZA: LanguageCode = LanguageCode {
    name: "Western Karaboro",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kza",
    individual_languages: &[
    ],
};


pub const KZB: LanguageCode = LanguageCode {
    name: "Kaibobo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kzb",
    individual_languages: &[
    ],
};


pub const KZC: LanguageCode = LanguageCode {
    name: "Bondoukou Kulango",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kzc",
    individual_languages: &[
    ],
};


pub const KZD: LanguageCode = LanguageCode {
    name: "Kadai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kzd",
    individual_languages: &[
    ],
};


pub const KZE: LanguageCode = LanguageCode {
    name: "Kosena",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kze",
    individual_languages: &[
    ],
};


pub const KZF: LanguageCode = LanguageCode {
    name: "Da'a Kaili",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kzf",
    individual_languages: &[
    ],
};


pub const KZG: LanguageCode = LanguageCode {
    name: "Kikai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kzg",
    individual_languages: &[
    ],
};


pub const KZI: LanguageCode = LanguageCode {
    name: "Kelabit",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kzi",
    individual_languages: &[
    ],
};


pub const KZK: LanguageCode = LanguageCode {
    name: "Kazukuru",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kzk",
    individual_languages: &[
    ],
};


pub const KZL: LanguageCode = LanguageCode {
    name: "Kayeli",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kzl",
    individual_languages: &[
    ],
};


pub const KZM: LanguageCode = LanguageCode {
    name: "Kais",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kzm",
    individual_languages: &[
    ],
};


pub const KZN: LanguageCode = LanguageCode {
    name: "Kokola",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kzn",
    individual_languages: &[
    ],
};


pub const KZO: LanguageCode = LanguageCode {
    name: "Kaningi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kzo",
    individual_languages: &[
    ],
};


pub const KZP: LanguageCode = LanguageCode {
    name: "Kaidipang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kzp",
    individual_languages: &[
    ],
};


pub const KZQ: LanguageCode = LanguageCode {
    name: "Kaike",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kzq",
    individual_languages: &[
    ],
};


pub const KZR: LanguageCode = LanguageCode {
    name: "Karang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kzr",
    individual_languages: &[
    ],
};


pub const KZS: LanguageCode = LanguageCode {
    name: "Sugut Dusun",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kzs",
    individual_languages: &[
    ],
};


pub const KZU: LanguageCode = LanguageCode {
    name: "Kayupulau",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kzu",
    individual_languages: &[
    ],
};


pub const KZV: LanguageCode = LanguageCode {
    name: "Komyandaret",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kzv",
    individual_languages: &[
    ],
};


pub const KZW: LanguageCode = LanguageCode {
    name: "Karirí-Xocó",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kzw",
    individual_languages: &[
    ],
};


pub const KZX: LanguageCode = LanguageCode {
    name: "Kamarian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kzx",
    individual_languages: &[
    ],
};


pub const KZY: LanguageCode = LanguageCode {
    name: "Kango (Tshopo District)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kzy",
    individual_languages: &[
    ],
};


pub const KZZ: LanguageCode = LanguageCode {
    name: "Kalabra",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "kzz",
    individual_languages: &[
    ],
};


pub const LAA: LanguageCode = LanguageCode {
    name: "Southern Subanen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "laa",
    individual_languages: &[
    ],
};


pub const LAB: LanguageCode = LanguageCode {
    name: "Linear A",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lab",
    individual_languages: &[
    ],
};


pub const LAC: LanguageCode = LanguageCode {
    name: "Lacandon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lac",
    individual_languages: &[
    ],
};


pub const LAE: LanguageCode = LanguageCode {
    name: "Pattani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lae",
    individual_languages: &[
    ],
};


pub const LAF: LanguageCode = LanguageCode {
    name: "Lafofa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "laf",
    individual_languages: &[
    ],
};


pub const LAG: LanguageCode = LanguageCode {
    name: "Langi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lag",
    individual_languages: &[
    ],
};


pub const LAI: LanguageCode = LanguageCode {
    name: "Lambya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lai",
    individual_languages: &[
    ],
};


pub const LAJ: LanguageCode = LanguageCode {
    name: "Lango (Uganda)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "laj",
    individual_languages: &[
    ],
};


pub const LAL: LanguageCode = LanguageCode {
    name: "Lalia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lal",
    individual_languages: &[
    ],
};


pub const LAN: LanguageCode = LanguageCode {
    name: "Laru",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lan",
    individual_languages: &[
    ],
};


pub const LAP: LanguageCode = LanguageCode {
    name: "Laka (Chad)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lap",
    individual_languages: &[
    ],
};


pub const LAQ: LanguageCode = LanguageCode {
    name: "Qabiao",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "laq",
    individual_languages: &[
    ],
};


pub const LAR: LanguageCode = LanguageCode {
    name: "Larteh",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lar",
    individual_languages: &[
    ],
};


pub const LAS: LanguageCode = LanguageCode {
    name: "Lama (Togo)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "las",
    individual_languages: &[
    ],
};


pub const LAU: LanguageCode = LanguageCode {
    name: "Laba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lau",
    individual_languages: &[
    ],
};


pub const LAW: LanguageCode = LanguageCode {
    name: "Lauje",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "law",
    individual_languages: &[
    ],
};


pub const LAX: LanguageCode = LanguageCode {
    name: "Tiwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lax",
    individual_languages: &[
    ],
};


pub const LAY: LanguageCode = LanguageCode {
    name: "Lama Bai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lay",
    individual_languages: &[
    ],
};


pub const LAZ: LanguageCode = LanguageCode {
    name: "Aribwatsa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "laz",
    individual_languages: &[
    ],
};


pub const LBB: LanguageCode = LanguageCode {
    name: "Label",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lbb",
    individual_languages: &[
    ],
};


pub const LBC: LanguageCode = LanguageCode {
    name: "Lakkia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lbc",
    individual_languages: &[
    ],
};


pub const LBE: LanguageCode = LanguageCode {
    name: "Lak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lbe",
    individual_languages: &[
    ],
};


pub const LBF: LanguageCode = LanguageCode {
    name: "Tinani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lbf",
    individual_languages: &[
    ],
};


pub const LBG: LanguageCode = LanguageCode {
    name: "Laopang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lbg",
    individual_languages: &[
    ],
};


pub const LBI: LanguageCode = LanguageCode {
    name: "La'bi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lbi",
    individual_languages: &[
    ],
};


pub const LBJ: LanguageCode = LanguageCode {
    name: "Ladakhi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lbj",
    individual_languages: &[
    ],
};


pub const LBK: LanguageCode = LanguageCode {
    name: "Central Bontok",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lbk",
    individual_languages: &[
    ],
};


pub const LBL: LanguageCode = LanguageCode {
    name: "Libon Bikol",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lbl",
    individual_languages: &[
    ],
};


pub const LBM: LanguageCode = LanguageCode {
    name: "Lodhi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lbm",
    individual_languages: &[
    ],
};


pub const LBN: LanguageCode = LanguageCode {
    name: "Rmeet",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lbn",
    individual_languages: &[
    ],
};


pub const LBO: LanguageCode = LanguageCode {
    name: "Laven",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lbo",
    individual_languages: &[
    ],
};


pub const LBQ: LanguageCode = LanguageCode {
    name: "Wampar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lbq",
    individual_languages: &[
    ],
};


pub const LBR: LanguageCode = LanguageCode {
    name: "Lohorung",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lbr",
    individual_languages: &[
    ],
};


pub const LBS: LanguageCode = LanguageCode {
    name: "Libyan Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lbs",
    individual_languages: &[
    ],
};


pub const LBT: LanguageCode = LanguageCode {
    name: "Lachi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lbt",
    individual_languages: &[
    ],
};


pub const LBU: LanguageCode = LanguageCode {
    name: "Labu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lbu",
    individual_languages: &[
    ],
};


pub const LBV: LanguageCode = LanguageCode {
    name: "Lavatbura-Lamusong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lbv",
    individual_languages: &[
    ],
};


pub const LBW: LanguageCode = LanguageCode {
    name: "Tolaki",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lbw",
    individual_languages: &[
    ],
};


pub const LBX: LanguageCode = LanguageCode {
    name: "Lawangan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lbx",
    individual_languages: &[
    ],
};


pub const LBY: LanguageCode = LanguageCode {
    name: "Lamalama",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lby",
    individual_languages: &[
    ],
};


pub const LBZ: LanguageCode = LanguageCode {
    name: "Lardil",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lbz",
    individual_languages: &[
    ],
};


pub const LCC: LanguageCode = LanguageCode {
    name: "Legenyem",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lcc",
    individual_languages: &[
    ],
};


pub const LCD: LanguageCode = LanguageCode {
    name: "Lola",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lcd",
    individual_languages: &[
    ],
};


pub const LCE: LanguageCode = LanguageCode {
    name: "Loncong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lce",
    individual_languages: &[
    ],
};


pub const LCF: LanguageCode = LanguageCode {
    name: "Lubu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lcf",
    individual_languages: &[
    ],
};


pub const LCH: LanguageCode = LanguageCode {
    name: "Luchazi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lch",
    individual_languages: &[
    ],
};


pub const LCL: LanguageCode = LanguageCode {
    name: "Lisela",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lcl",
    individual_languages: &[
    ],
};


pub const LCM: LanguageCode = LanguageCode {
    name: "Tungag",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lcm",
    individual_languages: &[
    ],
};


pub const LCP: LanguageCode = LanguageCode {
    name: "Western Lawa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lcp",
    individual_languages: &[
    ],
};


pub const LCQ: LanguageCode = LanguageCode {
    name: "Luhu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lcq",
    individual_languages: &[
    ],
};


pub const LCS: LanguageCode = LanguageCode {
    name: "Lisabata-Nuniali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lcs",
    individual_languages: &[
    ],
};


pub const LDA: LanguageCode = LanguageCode {
    name: "Kla-Dan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lda",
    individual_languages: &[
    ],
};


pub const LDB: LanguageCode = LanguageCode {
    name: "Dũya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ldb",
    individual_languages: &[
    ],
};


pub const LDD: LanguageCode = LanguageCode {
    name: "Luri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ldd",
    individual_languages: &[
    ],
};


pub const LDG: LanguageCode = LanguageCode {
    name: "Lenyima",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ldg",
    individual_languages: &[
    ],
};


pub const LDH: LanguageCode = LanguageCode {
    name: "Lamja-Dengsa-Tola",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ldh",
    individual_languages: &[
    ],
};


pub const LDI: LanguageCode = LanguageCode {
    name: "Laari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ldi",
    individual_languages: &[
    ],
};


pub const LDJ: LanguageCode = LanguageCode {
    name: "Lemoro",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ldj",
    individual_languages: &[
    ],
};


pub const LDK: LanguageCode = LanguageCode {
    name: "Leelau",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ldk",
    individual_languages: &[
    ],
};


pub const LDL: LanguageCode = LanguageCode {
    name: "Kaan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ldl",
    individual_languages: &[
    ],
};


pub const LDM: LanguageCode = LanguageCode {
    name: "Landoma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ldm",
    individual_languages: &[
    ],
};


pub const LDN: LanguageCode = LanguageCode {
    name: "Láadan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ldn",
    individual_languages: &[
    ],
};


pub const LDO: LanguageCode = LanguageCode {
    name: "Loo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ldo",
    individual_languages: &[
    ],
};


pub const LDP: LanguageCode = LanguageCode {
    name: "Tso",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ldp",
    individual_languages: &[
    ],
};


pub const LDQ: LanguageCode = LanguageCode {
    name: "Lufu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ldq",
    individual_languages: &[
    ],
};


pub const LEA: LanguageCode = LanguageCode {
    name: "Lega-Shabunda",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lea",
    individual_languages: &[
    ],
};


pub const LEB: LanguageCode = LanguageCode {
    name: "Lala-Bisa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "leb",
    individual_languages: &[
    ],
};


pub const LEC: LanguageCode = LanguageCode {
    name: "Leco",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lec",
    individual_languages: &[
    ],
};


pub const LED: LanguageCode = LanguageCode {
    name: "Lendu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "led",
    individual_languages: &[
    ],
};


pub const LEE: LanguageCode = LanguageCode {
    name: "Lyélé",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lee",
    individual_languages: &[
    ],
};


pub const LEF: LanguageCode = LanguageCode {
    name: "Lelemi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lef",
    individual_languages: &[
    ],
};


pub const LEH: LanguageCode = LanguageCode {
    name: "Lenje",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "leh",
    individual_languages: &[
    ],
};


pub const LEI: LanguageCode = LanguageCode {
    name: "Lemio",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lei",
    individual_languages: &[
    ],
};


pub const LEJ: LanguageCode = LanguageCode {
    name: "Lengola",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lej",
    individual_languages: &[
    ],
};


pub const LEK: LanguageCode = LanguageCode {
    name: "Leipon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lek",
    individual_languages: &[
    ],
};


pub const LEL: LanguageCode = LanguageCode {
    name: "Lele (Democratic Republic of Congo)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lel",
    individual_languages: &[
    ],
};


pub const LEM: LanguageCode = LanguageCode {
    name: "Nomaande",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lem",
    individual_languages: &[
    ],
};


pub const LEN: LanguageCode = LanguageCode {
    name: "Lenca",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "len",
    individual_languages: &[
    ],
};


pub const LEO: LanguageCode = LanguageCode {
    name: "Leti (Cameroon)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "leo",
    individual_languages: &[
    ],
};


pub const LEP: LanguageCode = LanguageCode {
    name: "Lepcha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lep",
    individual_languages: &[
    ],
};


pub const LEQ: LanguageCode = LanguageCode {
    name: "Lembena",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "leq",
    individual_languages: &[
    ],
};


pub const LER: LanguageCode = LanguageCode {
    name: "Lenkau",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ler",
    individual_languages: &[
    ],
};


pub const LES: LanguageCode = LanguageCode {
    name: "Lese",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "les",
    individual_languages: &[
    ],
};


pub const LET: LanguageCode = LanguageCode {
    name: "Lesing-Gelimi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "let",
    individual_languages: &[
    ],
};


pub const LEU: LanguageCode = LanguageCode {
    name: "Kara (Papua New Guinea)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "leu",
    individual_languages: &[
    ],
};


pub const LEV: LanguageCode = LanguageCode {
    name: "Lamma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lev",
    individual_languages: &[
    ],
};


pub const LEW: LanguageCode = LanguageCode {
    name: "Ledo Kaili",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lew",
    individual_languages: &[
    ],
};


pub const LEX: LanguageCode = LanguageCode {
    name: "Luang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lex",
    individual_languages: &[
    ],
};


pub const LEY: LanguageCode = LanguageCode {
    name: "Lemolang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ley",
    individual_languages: &[
    ],
};


pub const LFA: LanguageCode = LanguageCode {
    name: "Lefa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lfa",
    individual_languages: &[
    ],
};


pub const LFN: LanguageCode = LanguageCode {
    name: "Lingua Franca Nova",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lfn",
    individual_languages: &[
    ],
};


pub const LGA: LanguageCode = LanguageCode {
    name: "Lungga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lga",
    individual_languages: &[
    ],
};


pub const LGB: LanguageCode = LanguageCode {
    name: "Laghu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lgb",
    individual_languages: &[
    ],
};


pub const LGG: LanguageCode = LanguageCode {
    name: "Lugbara",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lgg",
    individual_languages: &[
    ],
};


pub const LGH: LanguageCode = LanguageCode {
    name: "Laghuu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lgh",
    individual_languages: &[
    ],
};


pub const LGI: LanguageCode = LanguageCode {
    name: "Lengilu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lgi",
    individual_languages: &[
    ],
};


pub const LGK: LanguageCode = LanguageCode {
    name: "Lingarak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lgk",
    individual_languages: &[
    ],
};


pub const LGL: LanguageCode = LanguageCode {
    name: "Wala",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lgl",
    individual_languages: &[
    ],
};


pub const LGM: LanguageCode = LanguageCode {
    name: "Lega-Mwenga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lgm",
    individual_languages: &[
    ],
};


pub const LGN: LanguageCode = LanguageCode {
    name: "T'apo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lgn",
    individual_languages: &[
    ],
};


pub const LGO: LanguageCode = LanguageCode {
    name: "Lango (South Sudan)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lgo",
    individual_languages: &[
    ],
};


pub const LGQ: LanguageCode = LanguageCode {
    name: "Logba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lgq",
    individual_languages: &[
    ],
};


pub const LGR: LanguageCode = LanguageCode {
    name: "Lengo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lgr",
    individual_languages: &[
    ],
};


pub const LGT: LanguageCode = LanguageCode {
    name: "Pahi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lgt",
    individual_languages: &[
    ],
};


pub const LGU: LanguageCode = LanguageCode {
    name: "Longgu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lgu",
    individual_languages: &[
    ],
};


pub const LGZ: LanguageCode = LanguageCode {
    name: "Ligenza",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lgz",
    individual_languages: &[
    ],
};


pub const LHA: LanguageCode = LanguageCode {
    name: "Laha (Viet Nam)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lha",
    individual_languages: &[
    ],
};


pub const LHH: LanguageCode = LanguageCode {
    name: "Laha (Indonesia)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lhh",
    individual_languages: &[
    ],
};


pub const LHI: LanguageCode = LanguageCode {
    name: "Lahu Shi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lhi",
    individual_languages: &[
    ],
};


pub const LHL: LanguageCode = LanguageCode {
    name: "Lahul Lohar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lhl",
    individual_languages: &[
    ],
};


pub const LHM: LanguageCode = LanguageCode {
    name: "Lhomi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lhm",
    individual_languages: &[
    ],
};


pub const LHN: LanguageCode = LanguageCode {
    name: "Lahanan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lhn",
    individual_languages: &[
    ],
};


pub const LHP: LanguageCode = LanguageCode {
    name: "Lhokpu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lhp",
    individual_languages: &[
    ],
};


pub const LHS: LanguageCode = LanguageCode {
    name: "Mlahsö",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lhs",
    individual_languages: &[
    ],
};


pub const LHT: LanguageCode = LanguageCode {
    name: "Lo-Toga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lht",
    individual_languages: &[
    ],
};


pub const LHU: LanguageCode = LanguageCode {
    name: "Lahu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lhu",
    individual_languages: &[
    ],
};


pub const LIA: LanguageCode = LanguageCode {
    name: "West-Central Limba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lia",
    individual_languages: &[
    ],
};


pub const LIB: LanguageCode = LanguageCode {
    name: "Likum",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lib",
    individual_languages: &[
    ],
};


pub const LIC: LanguageCode = LanguageCode {
    name: "Hlai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lic",
    individual_languages: &[
    ],
};


pub const LID: LanguageCode = LanguageCode {
    name: "Nyindrou",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lid",
    individual_languages: &[
    ],
};


pub const LIE: LanguageCode = LanguageCode {
    name: "Likila",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lie",
    individual_languages: &[
    ],
};


pub const LIF: LanguageCode = LanguageCode {
    name: "Limbu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lif",
    individual_languages: &[
    ],
};


pub const LIG: LanguageCode = LanguageCode {
    name: "Ligbi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lig",
    individual_languages: &[
    ],
};


pub const LIH: LanguageCode = LanguageCode {
    name: "Lihir",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lih",
    individual_languages: &[
    ],
};


pub const LIJ: LanguageCode = LanguageCode {
    name: "Ligurian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lij",
    individual_languages: &[
    ],
};


pub const LIK: LanguageCode = LanguageCode {
    name: "Lika",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lik",
    individual_languages: &[
    ],
};


pub const LIL: LanguageCode = LanguageCode {
    name: "Lillooet",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lil",
    individual_languages: &[
    ],
};


pub const LIO: LanguageCode = LanguageCode {
    name: "Liki",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lio",
    individual_languages: &[
    ],
};


pub const LIP: LanguageCode = LanguageCode {
    name: "Sekpele",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lip",
    individual_languages: &[
    ],
};


pub const LIQ: LanguageCode = LanguageCode {
    name: "Libido",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "liq",
    individual_languages: &[
    ],
};


pub const LIR: LanguageCode = LanguageCode {
    name: "Liberian English",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lir",
    individual_languages: &[
    ],
};


pub const LIS: LanguageCode = LanguageCode {
    name: "Lisu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lis",
    individual_languages: &[
    ],
};


pub const LIU: LanguageCode = LanguageCode {
    name: "Logorik",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "liu",
    individual_languages: &[
    ],
};


pub const LIV: LanguageCode = LanguageCode {
    name: "Liv",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "liv",
    individual_languages: &[
    ],
};


pub const LIW: LanguageCode = LanguageCode {
    name: "Col",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "liw",
    individual_languages: &[
    ],
};


pub const LIX: LanguageCode = LanguageCode {
    name: "Liabuku",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lix",
    individual_languages: &[
    ],
};


pub const LIY: LanguageCode = LanguageCode {
    name: "Banda-Bambari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "liy",
    individual_languages: &[
    ],
};


pub const LIZ: LanguageCode = LanguageCode {
    name: "Libinza",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "liz",
    individual_languages: &[
    ],
};


pub const LJA: LanguageCode = LanguageCode {
    name: "Golpa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lja",
    individual_languages: &[
    ],
};


pub const LJE: LanguageCode = LanguageCode {
    name: "Rampi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lje",
    individual_languages: &[
    ],
};


pub const LJI: LanguageCode = LanguageCode {
    name: "Laiyolo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lji",
    individual_languages: &[
    ],
};


pub const LJL: LanguageCode = LanguageCode {
    name: "Li'o",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ljl",
    individual_languages: &[
    ],
};


pub const LJP: LanguageCode = LanguageCode {
    name: "Lampung Api",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ljp",
    individual_languages: &[
    ],
};


pub const LJW: LanguageCode = LanguageCode {
    name: "Yirandali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ljw",
    individual_languages: &[
    ],
};


pub const LJX: LanguageCode = LanguageCode {
    name: "Yuru",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ljx",
    individual_languages: &[
    ],
};


pub const LKA: LanguageCode = LanguageCode {
    name: "Lakalei",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lka",
    individual_languages: &[
    ],
};


pub const LKB: LanguageCode = LanguageCode {
    name: "Kabras",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lkb",
    individual_languages: &[
    ],
};


pub const LKC: LanguageCode = LanguageCode {
    name: "Kucong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lkc",
    individual_languages: &[
    ],
};


pub const LKD: LanguageCode = LanguageCode {
    name: "Lakondê",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lkd",
    individual_languages: &[
    ],
};


pub const LKE: LanguageCode = LanguageCode {
    name: "Kenyi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lke",
    individual_languages: &[
    ],
};


pub const LKH: LanguageCode = LanguageCode {
    name: "Lakha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lkh",
    individual_languages: &[
    ],
};


pub const LKI: LanguageCode = LanguageCode {
    name: "Laki",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lki",
    individual_languages: &[
    ],
};


pub const LKJ: LanguageCode = LanguageCode {
    name: "Remun",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lkj",
    individual_languages: &[
    ],
};


pub const LKL: LanguageCode = LanguageCode {
    name: "Laeko-Libuat",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lkl",
    individual_languages: &[
    ],
};


pub const LKM: LanguageCode = LanguageCode {
    name: "Kalaamaya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lkm",
    individual_languages: &[
    ],
};


pub const LKN: LanguageCode = LanguageCode {
    name: "Lakon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lkn",
    individual_languages: &[
    ],
};


pub const LKO: LanguageCode = LanguageCode {
    name: "Khayo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lko",
    individual_languages: &[
    ],
};


pub const LKR: LanguageCode = LanguageCode {
    name: "Päri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lkr",
    individual_languages: &[
    ],
};


pub const LKS: LanguageCode = LanguageCode {
    name: "Kisa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lks",
    individual_languages: &[
    ],
};


pub const LKT: LanguageCode = LanguageCode {
    name: "Lakota",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lkt",
    individual_languages: &[
    ],
};


pub const LKU: LanguageCode = LanguageCode {
    name: "Kungkari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lku",
    individual_languages: &[
    ],
};


pub const LKY: LanguageCode = LanguageCode {
    name: "Lokoya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lky",
    individual_languages: &[
    ],
};


pub const LLA: LanguageCode = LanguageCode {
    name: "Lala-Roba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lla",
    individual_languages: &[
    ],
};


pub const LLB: LanguageCode = LanguageCode {
    name: "Lolo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "llb",
    individual_languages: &[
    ],
};


pub const LLC: LanguageCode = LanguageCode {
    name: "Lele (Guinea)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "llc",
    individual_languages: &[
    ],
};


pub const LLD: LanguageCode = LanguageCode {
    name: "Ladin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lld",
    individual_languages: &[
    ],
};


pub const LLE: LanguageCode = LanguageCode {
    name: "Lele (Papua New Guinea)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lle",
    individual_languages: &[
    ],
};


pub const LLF: LanguageCode = LanguageCode {
    name: "Hermit",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "llf",
    individual_languages: &[
    ],
};


pub const LLG: LanguageCode = LanguageCode {
    name: "Lole",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "llg",
    individual_languages: &[
    ],
};


pub const LLH: LanguageCode = LanguageCode {
    name: "Lamu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "llh",
    individual_languages: &[
    ],
};


pub const LLI: LanguageCode = LanguageCode {
    name: "Teke-Laali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lli",
    individual_languages: &[
    ],
};


pub const LLJ: LanguageCode = LanguageCode {
    name: "Ladji Ladji",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "llj",
    individual_languages: &[
    ],
};


pub const LLK: LanguageCode = LanguageCode {
    name: "Lelak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "llk",
    individual_languages: &[
    ],
};


pub const LLL: LanguageCode = LanguageCode {
    name: "Lilau",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lll",
    individual_languages: &[
    ],
};


pub const LLM: LanguageCode = LanguageCode {
    name: "Lasalimu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "llm",
    individual_languages: &[
    ],
};


pub const LLN: LanguageCode = LanguageCode {
    name: "Lele (Chad)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lln",
    individual_languages: &[
    ],
};


pub const LLP: LanguageCode = LanguageCode {
    name: "North Efate",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "llp",
    individual_languages: &[
    ],
};


pub const LLQ: LanguageCode = LanguageCode {
    name: "Lolak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "llq",
    individual_languages: &[
    ],
};


pub const LLS: LanguageCode = LanguageCode {
    name: "Lithuanian Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lls",
    individual_languages: &[
    ],
};


pub const LLU: LanguageCode = LanguageCode {
    name: "Lau",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "llu",
    individual_languages: &[
    ],
};


pub const LLX: LanguageCode = LanguageCode {
    name: "Lauan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "llx",
    individual_languages: &[
    ],
};


pub const LMA: LanguageCode = LanguageCode {
    name: "East Limba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lma",
    individual_languages: &[
    ],
};


pub const LMB: LanguageCode = LanguageCode {
    name: "Merei",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lmb",
    individual_languages: &[
    ],
};


pub const LMC: LanguageCode = LanguageCode {
    name: "Limilngan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lmc",
    individual_languages: &[
    ],
};


pub const LMD: LanguageCode = LanguageCode {
    name: "Lumun",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lmd",
    individual_languages: &[
    ],
};


pub const LME: LanguageCode = LanguageCode {
    name: "Pévé",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lme",
    individual_languages: &[
    ],
};


pub const LMF: LanguageCode = LanguageCode {
    name: "South Lembata",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lmf",
    individual_languages: &[
    ],
};


pub const LMG: LanguageCode = LanguageCode {
    name: "Lamogai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lmg",
    individual_languages: &[
    ],
};


pub const LMH: LanguageCode = LanguageCode {
    name: "Lambichhong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lmh",
    individual_languages: &[
    ],
};


pub const LMI: LanguageCode = LanguageCode {
    name: "Lombi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lmi",
    individual_languages: &[
    ],
};


pub const LMJ: LanguageCode = LanguageCode {
    name: "West Lembata",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lmj",
    individual_languages: &[
    ],
};


pub const LMK: LanguageCode = LanguageCode {
    name: "Lamkang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lmk",
    individual_languages: &[
    ],
};


pub const LML: LanguageCode = LanguageCode {
    name: "Hano",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lml",
    individual_languages: &[
    ],
};


pub const LMN: LanguageCode = LanguageCode {
    name: "Lambadi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lmn",
    individual_languages: &[
    ],
};


pub const LMO: LanguageCode = LanguageCode {
    name: "Lombard",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lmo",
    individual_languages: &[
    ],
};


pub const LMP: LanguageCode = LanguageCode {
    name: "Limbum",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lmp",
    individual_languages: &[
    ],
};


pub const LMQ: LanguageCode = LanguageCode {
    name: "Lamatuka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lmq",
    individual_languages: &[
    ],
};


pub const LMR: LanguageCode = LanguageCode {
    name: "Lamalera",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lmr",
    individual_languages: &[
    ],
};


pub const LMU: LanguageCode = LanguageCode {
    name: "Lamenu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lmu",
    individual_languages: &[
    ],
};


pub const LMV: LanguageCode = LanguageCode {
    name: "Lomaiviti",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lmv",
    individual_languages: &[
    ],
};


pub const LMW: LanguageCode = LanguageCode {
    name: "Lake Miwok",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lmw",
    individual_languages: &[
    ],
};


pub const LMX: LanguageCode = LanguageCode {
    name: "Laimbue",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lmx",
    individual_languages: &[
    ],
};


pub const LMY: LanguageCode = LanguageCode {
    name: "Lamboya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lmy",
    individual_languages: &[
    ],
};


pub const LNA: LanguageCode = LanguageCode {
    name: "Langbashe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lna",
    individual_languages: &[
    ],
};


pub const LNB: LanguageCode = LanguageCode {
    name: "Mbalanhu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lnb",
    individual_languages: &[
    ],
};


pub const LND: LanguageCode = LanguageCode {
    name: "Lundayeh",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lnd",
    individual_languages: &[
    ],
};


pub const LNG: LanguageCode = LanguageCode {
    name: "Langobardic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lng",
    individual_languages: &[
    ],
};


pub const LNH: LanguageCode = LanguageCode {
    name: "Lanoh",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lnh",
    individual_languages: &[
    ],
};


pub const LNI: LanguageCode = LanguageCode {
    name: "Daantanai'",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lni",
    individual_languages: &[
    ],
};


pub const LNJ: LanguageCode = LanguageCode {
    name: "Leningitij",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lnj",
    individual_languages: &[
    ],
};


pub const LNL: LanguageCode = LanguageCode {
    name: "South Central Banda",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lnl",
    individual_languages: &[
    ],
};


pub const LNM: LanguageCode = LanguageCode {
    name: "Langam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lnm",
    individual_languages: &[
    ],
};


pub const LNN: LanguageCode = LanguageCode {
    name: "Lorediakarkar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lnn",
    individual_languages: &[
    ],
};


pub const LNS: LanguageCode = LanguageCode {
    name: "Lamnso'",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lns",
    individual_languages: &[
    ],
};


pub const LNU: LanguageCode = LanguageCode {
    name: "Longuda",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lnu",
    individual_languages: &[
    ],
};


pub const LNW: LanguageCode = LanguageCode {
    name: "Lanima",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lnw",
    individual_languages: &[
    ],
};


pub const LNZ: LanguageCode = LanguageCode {
    name: "Lonzo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lnz",
    individual_languages: &[
    ],
};


pub const LOA: LanguageCode = LanguageCode {
    name: "Loloda",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "loa",
    individual_languages: &[
    ],
};


pub const LOB: LanguageCode = LanguageCode {
    name: "Lobi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lob",
    individual_languages: &[
    ],
};


pub const LOC: LanguageCode = LanguageCode {
    name: "Inonhan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "loc",
    individual_languages: &[
    ],
};


pub const LOE: LanguageCode = LanguageCode {
    name: "Saluan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "loe",
    individual_languages: &[
    ],
};


pub const LOF: LanguageCode = LanguageCode {
    name: "Logol",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lof",
    individual_languages: &[
    ],
};


pub const LOG: LanguageCode = LanguageCode {
    name: "Logo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "log",
    individual_languages: &[
    ],
};


pub const LOH: LanguageCode = LanguageCode {
    name: "Narim",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "loh",
    individual_languages: &[
    ],
};


pub const LOI: LanguageCode = LanguageCode {
    name: "Loma (Côte d'Ivoire)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "loi",
    individual_languages: &[
    ],
};


pub const LOJ: LanguageCode = LanguageCode {
    name: "Lou",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "loj",
    individual_languages: &[
    ],
};


pub const LOK: LanguageCode = LanguageCode {
    name: "Loko",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lok",
    individual_languages: &[
    ],
};


pub const LOM: LanguageCode = LanguageCode {
    name: "Loma (Liberia)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lom",
    individual_languages: &[
    ],
};


pub const LON: LanguageCode = LanguageCode {
    name: "Malawi Lomwe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lon",
    individual_languages: &[
    ],
};


pub const LOO: LanguageCode = LanguageCode {
    name: "Lombo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "loo",
    individual_languages: &[
    ],
};


pub const LOP: LanguageCode = LanguageCode {
    name: "Lopa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lop",
    individual_languages: &[
    ],
};


pub const LOQ: LanguageCode = LanguageCode {
    name: "Lobala",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "loq",
    individual_languages: &[
    ],
};


pub const LOR: LanguageCode = LanguageCode {
    name: "Téén",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lor",
    individual_languages: &[
    ],
};


pub const LOS: LanguageCode = LanguageCode {
    name: "Loniu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "los",
    individual_languages: &[
    ],
};


pub const LOT: LanguageCode = LanguageCode {
    name: "Otuho",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lot",
    individual_languages: &[
    ],
};


pub const LOU: LanguageCode = LanguageCode {
    name: "Louisiana Creole",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lou",
    individual_languages: &[
    ],
};


pub const LOV: LanguageCode = LanguageCode {
    name: "Lopi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lov",
    individual_languages: &[
    ],
};


pub const LOW: LanguageCode = LanguageCode {
    name: "Tampias Lobu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "low",
    individual_languages: &[
    ],
};


pub const LOX: LanguageCode = LanguageCode {
    name: "Loun",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lox",
    individual_languages: &[
    ],
};


pub const LOY: LanguageCode = LanguageCode {
    name: "Loke",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "loy",
    individual_languages: &[
    ],
};


pub const LPA: LanguageCode = LanguageCode {
    name: "Lelepa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lpa",
    individual_languages: &[
    ],
};


pub const LPE: LanguageCode = LanguageCode {
    name: "Lepki",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lpe",
    individual_languages: &[
    ],
};


pub const LPN: LanguageCode = LanguageCode {
    name: "Long Phuri Naga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lpn",
    individual_languages: &[
    ],
};


pub const LPO: LanguageCode = LanguageCode {
    name: "Lipo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lpo",
    individual_languages: &[
    ],
};


pub const LPX: LanguageCode = LanguageCode {
    name: "Lopit",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lpx",
    individual_languages: &[
    ],
};


pub const LQR: LanguageCode = LanguageCode {
    name: "Logir",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lqr",
    individual_languages: &[
    ],
};


pub const LRA: LanguageCode = LanguageCode {
    name: "Rara Bakati'",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lra",
    individual_languages: &[
    ],
};


pub const LRC: LanguageCode = LanguageCode {
    name: "Northern Luri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lrc",
    individual_languages: &[
    ],
};


pub const LRE: LanguageCode = LanguageCode {
    name: "Laurentian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lre",
    individual_languages: &[
    ],
};


pub const LRG: LanguageCode = LanguageCode {
    name: "Laragia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lrg",
    individual_languages: &[
    ],
};


pub const LRI: LanguageCode = LanguageCode {
    name: "Marachi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lri",
    individual_languages: &[
    ],
};


pub const LRK: LanguageCode = LanguageCode {
    name: "Loarki",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lrk",
    individual_languages: &[
    ],
};


pub const LRL: LanguageCode = LanguageCode {
    name: "Lari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lrl",
    individual_languages: &[
    ],
};


pub const LRM: LanguageCode = LanguageCode {
    name: "Marama",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lrm",
    individual_languages: &[
    ],
};


pub const LRN: LanguageCode = LanguageCode {
    name: "Lorang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lrn",
    individual_languages: &[
    ],
};


pub const LRO: LanguageCode = LanguageCode {
    name: "Laro",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lro",
    individual_languages: &[
    ],
};


pub const LRR: LanguageCode = LanguageCode {
    name: "Southern Yamphu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lrr",
    individual_languages: &[
    ],
};


pub const LRT: LanguageCode = LanguageCode {
    name: "Larantuka Malay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lrt",
    individual_languages: &[
    ],
};


pub const LRV: LanguageCode = LanguageCode {
    name: "Larevat",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lrv",
    individual_languages: &[
    ],
};


pub const LRZ: LanguageCode = LanguageCode {
    name: "Lemerig",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lrz",
    individual_languages: &[
    ],
};


pub const LSA: LanguageCode = LanguageCode {
    name: "Lasgerdi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lsa",
    individual_languages: &[
    ],
};


pub const LSB: LanguageCode = LanguageCode {
    name: "Burundian Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lsb",
    individual_languages: &[
    ],
};


pub const LSC: LanguageCode = LanguageCode {
    name: "Albarradas Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lsc",
    individual_languages: &[
    ],
};


pub const LSD: LanguageCode = LanguageCode {
    name: "Lishana Deni",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lsd",
    individual_languages: &[
    ],
};


pub const LSE: LanguageCode = LanguageCode {
    name: "Lusengo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lse",
    individual_languages: &[
    ],
};


pub const LSH: LanguageCode = LanguageCode {
    name: "Lish",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lsh",
    individual_languages: &[
    ],
};


pub const LSI: LanguageCode = LanguageCode {
    name: "Lashi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lsi",
    individual_languages: &[
    ],
};


pub const LSL: LanguageCode = LanguageCode {
    name: "Latvian Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lsl",
    individual_languages: &[
    ],
};


pub const LSM: LanguageCode = LanguageCode {
    name: "Saamia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lsm",
    individual_languages: &[
    ],
};


pub const LSN: LanguageCode = LanguageCode {
    name: "Tibetan Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lsn",
    individual_languages: &[
    ],
};


pub const LSO: LanguageCode = LanguageCode {
    name: "Laos Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lso",
    individual_languages: &[
    ],
};


pub const LSP: LanguageCode = LanguageCode {
    name: "Panamanian Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lsp",
    individual_languages: &[
    ],
};


pub const LSR: LanguageCode = LanguageCode {
    name: "Aruop",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lsr",
    individual_languages: &[
    ],
};


pub const LSS: LanguageCode = LanguageCode {
    name: "Lasi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lss",
    individual_languages: &[
    ],
};


pub const LST: LanguageCode = LanguageCode {
    name: "Trinidad and Tobago Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lst",
    individual_languages: &[
    ],
};


pub const LSV: LanguageCode = LanguageCode {
    name: "Sivia Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lsv",
    individual_languages: &[
    ],
};


pub const LSW: LanguageCode = LanguageCode {
    name: "Seychelles Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lsw",
    individual_languages: &[
    ],
};


pub const LSY: LanguageCode = LanguageCode {
    name: "Mauritian Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lsy",
    individual_languages: &[
    ],
};


pub const LTC: LanguageCode = LanguageCode {
    name: "Late Middle Chinese",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ltc",
    individual_languages: &[
    ],
};


pub const LTG: LanguageCode = LanguageCode {
    name: "Latgalian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ltg",
    individual_languages: &[
    ],
};


pub const LTH: LanguageCode = LanguageCode {
    name: "Thur",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lth",
    individual_languages: &[
    ],
};


pub const LTI: LanguageCode = LanguageCode {
    name: "Leti (Indonesia)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lti",
    individual_languages: &[
    ],
};


pub const LTN: LanguageCode = LanguageCode {
    name: "Latundê",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ltn",
    individual_languages: &[
    ],
};


pub const LTO: LanguageCode = LanguageCode {
    name: "Tsotso",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lto",
    individual_languages: &[
    ],
};


pub const LTS: LanguageCode = LanguageCode {
    name: "Tachoni",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lts",
    individual_languages: &[
    ],
};


pub const LTU: LanguageCode = LanguageCode {
    name: "Latu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ltu",
    individual_languages: &[
    ],
};


pub const LUC: LanguageCode = LanguageCode {
    name: "Aringa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "luc",
    individual_languages: &[
    ],
};


pub const LUD: LanguageCode = LanguageCode {
    name: "Ludian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lud",
    individual_languages: &[
    ],
};


pub const LUE: LanguageCode = LanguageCode {
    name: "Luvale",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lue",
    individual_languages: &[
    ],
};


pub const LUF: LanguageCode = LanguageCode {
    name: "Laua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "luf",
    individual_languages: &[
    ],
};


pub const LUJ: LanguageCode = LanguageCode {
    name: "Luna",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "luj",
    individual_languages: &[
    ],
};


pub const LUK: LanguageCode = LanguageCode {
    name: "Lunanakha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "luk",
    individual_languages: &[
    ],
};


pub const LUL: LanguageCode = LanguageCode {
    name: "Olu'bo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lul",
    individual_languages: &[
    ],
};


pub const LUM: LanguageCode = LanguageCode {
    name: "Luimbi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lum",
    individual_languages: &[
    ],
};


pub const LUP: LanguageCode = LanguageCode {
    name: "Lumbu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lup",
    individual_languages: &[
    ],
};


pub const LUQ: LanguageCode = LanguageCode {
    name: "Lucumi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "luq",
    individual_languages: &[
    ],
};


pub const LUR: LanguageCode = LanguageCode {
    name: "Laura",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lur",
    individual_languages: &[
    ],
};


pub const LUT: LanguageCode = LanguageCode {
    name: "Lushootseed",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lut",
    individual_languages: &[
    ],
};


pub const LUU: LanguageCode = LanguageCode {
    name: "Lumba-Yakkha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "luu",
    individual_languages: &[
    ],
};


pub const LUV: LanguageCode = LanguageCode {
    name: "Luwati",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "luv",
    individual_languages: &[
    ],
};


pub const LUW: LanguageCode = LanguageCode {
    name: "Luo (Cameroon)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "luw",
    individual_languages: &[
    ],
};


pub const LUY: LanguageCode = LanguageCode {
    name: "Luyia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "luy",
    individual_languages: &[
        IndividualLanguages {
            name: "Lubukusu",
            code: "bxk",
        },
        IndividualLanguages {
            name: "Luidakho-Luisukha-Lutirichi",
            code: "ida",
        },
        IndividualLanguages {
            name: "Lukabaras",
            code: "lkb",
        },
        IndividualLanguages {
            name: "Olukhayo",
            code: "lko",
        },
        IndividualLanguages {
            name: "Olushisa",
            code: "lks",
        },
        IndividualLanguages {
            name: "Olumarachi",
            code: "lri",
        },
        IndividualLanguages {
            name: "Olumarama",
            code: "lrm",
        },
        IndividualLanguages {
            name: "Saamia",
            code: "lsm",
        },
        IndividualLanguages {
            name: "Tsotso",
            code: "lto",
        },
        IndividualLanguages {
            name: "Tachoni",
            code: "lts",
        },
        IndividualLanguages {
            name: "Wanga",
            code: "lwg",
        },
        IndividualLanguages {
            name: "East Nyala",
            code: "nle",
        },
        IndividualLanguages {
            name: "Olunyole",
            code: "nyd",
        },
        IndividualLanguages {
            name: "Lulogooli",
            code: "rag",
        },
    ],
};


pub const LUZ: LanguageCode = LanguageCode {
    name: "Southern Luri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "luz",
    individual_languages: &[
    ],
};


pub const LVA: LanguageCode = LanguageCode {
    name: "Maku'a",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lva",
    individual_languages: &[
    ],
};


pub const LVI: LanguageCode = LanguageCode {
    name: "Lavi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lvi",
    individual_languages: &[
    ],
};


pub const LVK: LanguageCode = LanguageCode {
    name: "Lavukaleve",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lvk",
    individual_languages: &[
    ],
};


pub const LVS: LanguageCode = LanguageCode {
    name: "Standard Latvian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lvs",
    individual_languages: &[
    ],
};


pub const LVU: LanguageCode = LanguageCode {
    name: "Levuka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lvu",
    individual_languages: &[
    ],
};


pub const LWA: LanguageCode = LanguageCode {
    name: "Lwalu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lwa",
    individual_languages: &[
    ],
};


pub const LWE: LanguageCode = LanguageCode {
    name: "Lewo Eleng",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lwe",
    individual_languages: &[
    ],
};


pub const LWG: LanguageCode = LanguageCode {
    name: "Wanga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lwg",
    individual_languages: &[
    ],
};


pub const LWH: LanguageCode = LanguageCode {
    name: "White Lachi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lwh",
    individual_languages: &[
    ],
};


pub const LWL: LanguageCode = LanguageCode {
    name: "Eastern Lawa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lwl",
    individual_languages: &[
    ],
};


pub const LWM: LanguageCode = LanguageCode {
    name: "Laomian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lwm",
    individual_languages: &[
    ],
};


pub const LWO: LanguageCode = LanguageCode {
    name: "Luwo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lwo",
    individual_languages: &[
    ],
};


pub const LWS: LanguageCode = LanguageCode {
    name: "Malawian Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lws",
    individual_languages: &[
    ],
};


pub const LWT: LanguageCode = LanguageCode {
    name: "Lewotobi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lwt",
    individual_languages: &[
    ],
};


pub const LWU: LanguageCode = LanguageCode {
    name: "Lawu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lwu",
    individual_languages: &[
    ],
};


pub const LWW: LanguageCode = LanguageCode {
    name: "Lewo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lww",
    individual_languages: &[
    ],
};


pub const LXM: LanguageCode = LanguageCode {
    name: "Lakurumau",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lxm",
    individual_languages: &[
    ],
};


pub const LYA: LanguageCode = LanguageCode {
    name: "Layakha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lya",
    individual_languages: &[
    ],
};


pub const LYG: LanguageCode = LanguageCode {
    name: "Lyngngam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lyg",
    individual_languages: &[
    ],
};


pub const LYN: LanguageCode = LanguageCode {
    name: "Luyana",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lyn",
    individual_languages: &[
    ],
};


pub const LZH: LanguageCode = LanguageCode {
    name: "Literary Chinese",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lzh",
    individual_languages: &[
    ],
};


pub const LZL: LanguageCode = LanguageCode {
    name: "Litzlitz",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lzl",
    individual_languages: &[
    ],
};


pub const LZN: LanguageCode = LanguageCode {
    name: "Leinong Naga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lzn",
    individual_languages: &[
    ],
};


pub const LZZ: LanguageCode = LanguageCode {
    name: "Laz",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "lzz",
    individual_languages: &[
    ],
};


pub const MAA: LanguageCode = LanguageCode {
    name: "San Jerónimo Tecóatl Mazatec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "maa",
    individual_languages: &[
    ],
};


pub const MAB: LanguageCode = LanguageCode {
    name: "Yutanduchi Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mab",
    individual_languages: &[
    ],
};


pub const MAE: LanguageCode = LanguageCode {
    name: "Bo-Rukul",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mae",
    individual_languages: &[
    ],
};


pub const MAF: LanguageCode = LanguageCode {
    name: "Mafa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "maf",
    individual_languages: &[
    ],
};


pub const MAJ: LanguageCode = LanguageCode {
    name: "Jalapa De Díaz Mazatec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "maj",
    individual_languages: &[
    ],
};


pub const MAM: LanguageCode = LanguageCode {
    name: "Mam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mam",
    individual_languages: &[
    ],
};


pub const MAQ: LanguageCode = LanguageCode {
    name: "Chiquihuitlán Mazatec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "maq",
    individual_languages: &[
    ],
};


pub const MAT: LanguageCode = LanguageCode {
    name: "San Francisco Matlatzinca",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mat",
    individual_languages: &[
    ],
};


pub const MAU: LanguageCode = LanguageCode {
    name: "Huautla Mazatec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mau",
    individual_languages: &[
    ],
};


pub const MAV: LanguageCode = LanguageCode {
    name: "Sateré-Mawé",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mav",
    individual_languages: &[
    ],
};


pub const MAW: LanguageCode = LanguageCode {
    name: "Mampruli",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "maw",
    individual_languages: &[
    ],
};


pub const MAX: LanguageCode = LanguageCode {
    name: "North Moluccan Malay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "max",
    individual_languages: &[
    ],
};


pub const MAZ: LanguageCode = LanguageCode {
    name: "Central Mazahua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "maz",
    individual_languages: &[
    ],
};


pub const MBA: LanguageCode = LanguageCode {
    name: "Higaonon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mba",
    individual_languages: &[
    ],
};


pub const MBB: LanguageCode = LanguageCode {
    name: "Western Bukidnon Manobo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mbb",
    individual_languages: &[
    ],
};


pub const MBC: LanguageCode = LanguageCode {
    name: "Macushi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mbc",
    individual_languages: &[
    ],
};


pub const MBD: LanguageCode = LanguageCode {
    name: "Dibabawon Manobo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mbd",
    individual_languages: &[
    ],
};


pub const MBE: LanguageCode = LanguageCode {
    name: "Molale",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mbe",
    individual_languages: &[
    ],
};


pub const MBF: LanguageCode = LanguageCode {
    name: "Baba Malay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mbf",
    individual_languages: &[
    ],
};


pub const MBH: LanguageCode = LanguageCode {
    name: "Mangseng",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mbh",
    individual_languages: &[
    ],
};


pub const MBI: LanguageCode = LanguageCode {
    name: "Ilianen Manobo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mbi",
    individual_languages: &[
    ],
};


pub const MBJ: LanguageCode = LanguageCode {
    name: "Nadëb",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mbj",
    individual_languages: &[
    ],
};


pub const MBK: LanguageCode = LanguageCode {
    name: "Malol",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mbk",
    individual_languages: &[
    ],
};


pub const MBL: LanguageCode = LanguageCode {
    name: "Maxakalí",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mbl",
    individual_languages: &[
    ],
};


pub const MBM: LanguageCode = LanguageCode {
    name: "Ombamba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mbm",
    individual_languages: &[
    ],
};


pub const MBN: LanguageCode = LanguageCode {
    name: "Macaguán",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mbn",
    individual_languages: &[
    ],
};


pub const MBO: LanguageCode = LanguageCode {
    name: "Mbo (Cameroon)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mbo",
    individual_languages: &[
    ],
};


pub const MBP: LanguageCode = LanguageCode {
    name: "Malayo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mbp",
    individual_languages: &[
    ],
};


pub const MBQ: LanguageCode = LanguageCode {
    name: "Maisin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mbq",
    individual_languages: &[
    ],
};


pub const MBR: LanguageCode = LanguageCode {
    name: "Nukak Makú",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mbr",
    individual_languages: &[
    ],
};


pub const MBS: LanguageCode = LanguageCode {
    name: "Sarangani Manobo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mbs",
    individual_languages: &[
    ],
};


pub const MBT: LanguageCode = LanguageCode {
    name: "Matigsalug Manobo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mbt",
    individual_languages: &[
    ],
};


pub const MBU: LanguageCode = LanguageCode {
    name: "Mbula-Bwazza",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mbu",
    individual_languages: &[
    ],
};


pub const MBV: LanguageCode = LanguageCode {
    name: "Mbulungish",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mbv",
    individual_languages: &[
    ],
};


pub const MBW: LanguageCode = LanguageCode {
    name: "Maring",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mbw",
    individual_languages: &[
    ],
};


pub const MBX: LanguageCode = LanguageCode {
    name: "Mari (East Sepik Province)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mbx",
    individual_languages: &[
    ],
};


pub const MBY: LanguageCode = LanguageCode {
    name: "Memoni",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mby",
    individual_languages: &[
    ],
};


pub const MBZ: LanguageCode = LanguageCode {
    name: "Amoltepec Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mbz",
    individual_languages: &[
    ],
};


pub const MCA: LanguageCode = LanguageCode {
    name: "Maca",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mca",
    individual_languages: &[
    ],
};


pub const MCB: LanguageCode = LanguageCode {
    name: "Machiguenga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mcb",
    individual_languages: &[
    ],
};


pub const MCC: LanguageCode = LanguageCode {
    name: "Bitur",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mcc",
    individual_languages: &[
    ],
};


pub const MCD: LanguageCode = LanguageCode {
    name: "Sharanahua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mcd",
    individual_languages: &[
    ],
};


pub const MCE: LanguageCode = LanguageCode {
    name: "Itundujia Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mce",
    individual_languages: &[
    ],
};


pub const MCF: LanguageCode = LanguageCode {
    name: "Matsés",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mcf",
    individual_languages: &[
    ],
};


pub const MCG: LanguageCode = LanguageCode {
    name: "Mapoyo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mcg",
    individual_languages: &[
    ],
};


pub const MCH: LanguageCode = LanguageCode {
    name: "Maquiritari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mch",
    individual_languages: &[
    ],
};


pub const MCI: LanguageCode = LanguageCode {
    name: "Mese",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mci",
    individual_languages: &[
    ],
};


pub const MCJ: LanguageCode = LanguageCode {
    name: "Mvanip",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mcj",
    individual_languages: &[
    ],
};


pub const MCK: LanguageCode = LanguageCode {
    name: "Mbunda",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mck",
    individual_languages: &[
    ],
};


pub const MCL: LanguageCode = LanguageCode {
    name: "Macaguaje",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mcl",
    individual_languages: &[
    ],
};


pub const MCM: LanguageCode = LanguageCode {
    name: "Malaccan Creole Portuguese",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mcm",
    individual_languages: &[
    ],
};


pub const MCN: LanguageCode = LanguageCode {
    name: "Masana",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mcn",
    individual_languages: &[
    ],
};


pub const MCO: LanguageCode = LanguageCode {
    name: "Coatlán Mixe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mco",
    individual_languages: &[
    ],
};


pub const MCP: LanguageCode = LanguageCode {
    name: "Makaa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mcp",
    individual_languages: &[
    ],
};


pub const MCQ: LanguageCode = LanguageCode {
    name: "Ese",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mcq",
    individual_languages: &[
    ],
};


pub const MCR: LanguageCode = LanguageCode {
    name: "Menya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mcr",
    individual_languages: &[
    ],
};


pub const MCS: LanguageCode = LanguageCode {
    name: "Mambai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mcs",
    individual_languages: &[
    ],
};


pub const MCT: LanguageCode = LanguageCode {
    name: "Mengisa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mct",
    individual_languages: &[
    ],
};


pub const MCU: LanguageCode = LanguageCode {
    name: "Cameroon Mambila",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mcu",
    individual_languages: &[
    ],
};


pub const MCV: LanguageCode = LanguageCode {
    name: "Minanibai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mcv",
    individual_languages: &[
    ],
};


pub const MCW: LanguageCode = LanguageCode {
    name: "Mawa (Chad)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mcw",
    individual_languages: &[
    ],
};


pub const MCX: LanguageCode = LanguageCode {
    name: "Mpiemo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mcx",
    individual_languages: &[
    ],
};


pub const MCY: LanguageCode = LanguageCode {
    name: "South Watut",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mcy",
    individual_languages: &[
    ],
};


pub const MCZ: LanguageCode = LanguageCode {
    name: "Mawan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mcz",
    individual_languages: &[
    ],
};


pub const MDA: LanguageCode = LanguageCode {
    name: "Mada (Nigeria)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mda",
    individual_languages: &[
    ],
};


pub const MDB: LanguageCode = LanguageCode {
    name: "Morigi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mdb",
    individual_languages: &[
    ],
};


pub const MDC: LanguageCode = LanguageCode {
    name: "Male (Papua New Guinea)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mdc",
    individual_languages: &[
    ],
};


pub const MDD: LanguageCode = LanguageCode {
    name: "Mbum",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mdd",
    individual_languages: &[
    ],
};


pub const MDE: LanguageCode = LanguageCode {
    name: "Maba (Chad)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mde",
    individual_languages: &[
    ],
};


pub const MDG: LanguageCode = LanguageCode {
    name: "Massalat",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mdg",
    individual_languages: &[
    ],
};


pub const MDH: LanguageCode = LanguageCode {
    name: "Maguindanaon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mdh",
    individual_languages: &[
    ],
};


pub const MDI: LanguageCode = LanguageCode {
    name: "Mamvu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mdi",
    individual_languages: &[
    ],
};


pub const MDJ: LanguageCode = LanguageCode {
    name: "Mangbetu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mdj",
    individual_languages: &[
    ],
};


pub const MDK: LanguageCode = LanguageCode {
    name: "Mangbutu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mdk",
    individual_languages: &[
    ],
};


pub const MDL: LanguageCode = LanguageCode {
    name: "Maltese Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mdl",
    individual_languages: &[
    ],
};


pub const MDM: LanguageCode = LanguageCode {
    name: "Mayogo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mdm",
    individual_languages: &[
    ],
};


pub const MDN: LanguageCode = LanguageCode {
    name: "Mbati",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mdn",
    individual_languages: &[
    ],
};


pub const MDP: LanguageCode = LanguageCode {
    name: "Mbala",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mdp",
    individual_languages: &[
    ],
};


pub const MDQ: LanguageCode = LanguageCode {
    name: "Mbole",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mdq",
    individual_languages: &[
    ],
};


pub const MDS: LanguageCode = LanguageCode {
    name: "Maria (Papua New Guinea)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mds",
    individual_languages: &[
    ],
};


pub const MDT: LanguageCode = LanguageCode {
    name: "Mbere",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mdt",
    individual_languages: &[
    ],
};


pub const MDU: LanguageCode = LanguageCode {
    name: "Mboko",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mdu",
    individual_languages: &[
    ],
};


pub const MDV: LanguageCode = LanguageCode {
    name: "Santa Lucía Monteverde Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mdv",
    individual_languages: &[
    ],
};


pub const MDW: LanguageCode = LanguageCode {
    name: "Mbosi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mdw",
    individual_languages: &[
    ],
};


pub const MDX: LanguageCode = LanguageCode {
    name: "Dizin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mdx",
    individual_languages: &[
    ],
};


pub const MDY: LanguageCode = LanguageCode {
    name: "Male (Ethiopia)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mdy",
    individual_languages: &[
    ],
};


pub const MDZ: LanguageCode = LanguageCode {
    name: "Suruí Do Pará",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mdz",
    individual_languages: &[
    ],
};


pub const MEA: LanguageCode = LanguageCode {
    name: "Menka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mea",
    individual_languages: &[
    ],
};


pub const MEB: LanguageCode = LanguageCode {
    name: "Ikobi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "meb",
    individual_languages: &[
    ],
};


pub const MEC: LanguageCode = LanguageCode {
    name: "Marra",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mec",
    individual_languages: &[
    ],
};


pub const MED: LanguageCode = LanguageCode {
    name: "Melpa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "med",
    individual_languages: &[
    ],
};


pub const MEE: LanguageCode = LanguageCode {
    name: "Mengen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mee",
    individual_languages: &[
    ],
};


pub const MEF: LanguageCode = LanguageCode {
    name: "Megam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mef",
    individual_languages: &[
    ],
};


pub const MEH: LanguageCode = LanguageCode {
    name: "Southwestern Tlaxiaco Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "meh",
    individual_languages: &[
    ],
};


pub const MEI: LanguageCode = LanguageCode {
    name: "Midob",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mei",
    individual_languages: &[
    ],
};


pub const MEJ: LanguageCode = LanguageCode {
    name: "Meyah",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mej",
    individual_languages: &[
    ],
};


pub const MEK: LanguageCode = LanguageCode {
    name: "Mekeo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mek",
    individual_languages: &[
    ],
};


pub const MEL: LanguageCode = LanguageCode {
    name: "Central Melanau",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mel",
    individual_languages: &[
    ],
};


pub const MEM: LanguageCode = LanguageCode {
    name: "Mangala",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mem",
    individual_languages: &[
    ],
};


pub const MEO: LanguageCode = LanguageCode {
    name: "Kedah Malay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "meo",
    individual_languages: &[
    ],
};


pub const MEP: LanguageCode = LanguageCode {
    name: "Miriwoong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mep",
    individual_languages: &[
    ],
};


pub const MEQ: LanguageCode = LanguageCode {
    name: "Merey",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "meq",
    individual_languages: &[
    ],
};


pub const MER: LanguageCode = LanguageCode {
    name: "Meru",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mer",
    individual_languages: &[
    ],
};


pub const MES: LanguageCode = LanguageCode {
    name: "Masmaje",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mes",
    individual_languages: &[
    ],
};


pub const MET: LanguageCode = LanguageCode {
    name: "Mato",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "met",
    individual_languages: &[
    ],
};


pub const MEU: LanguageCode = LanguageCode {
    name: "Motu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "meu",
    individual_languages: &[
    ],
};


pub const MEV: LanguageCode = LanguageCode {
    name: "Mano",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mev",
    individual_languages: &[
    ],
};


pub const MEW: LanguageCode = LanguageCode {
    name: "Maaka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mew",
    individual_languages: &[
    ],
};


pub const MEY: LanguageCode = LanguageCode {
    name: "Hassaniyya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mey",
    individual_languages: &[
    ],
};


pub const MEZ: LanguageCode = LanguageCode {
    name: "Menominee",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mez",
    individual_languages: &[
    ],
};


pub const MFA: LanguageCode = LanguageCode {
    name: "Pattani Malay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mfa",
    individual_languages: &[
    ],
};


pub const MFB: LanguageCode = LanguageCode {
    name: "Bangka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mfb",
    individual_languages: &[
    ],
};


pub const MFC: LanguageCode = LanguageCode {
    name: "Mba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mfc",
    individual_languages: &[
    ],
};


pub const MFD: LanguageCode = LanguageCode {
    name: "Mendankwe-Nkwen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mfd",
    individual_languages: &[
    ],
};


pub const MFE: LanguageCode = LanguageCode {
    name: "Morisyen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mfe",
    individual_languages: &[
    ],
};


pub const MFF: LanguageCode = LanguageCode {
    name: "Naki",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mff",
    individual_languages: &[
    ],
};


pub const MFG: LanguageCode = LanguageCode {
    name: "Mogofin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mfg",
    individual_languages: &[
    ],
};


pub const MFH: LanguageCode = LanguageCode {
    name: "Matal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mfh",
    individual_languages: &[
    ],
};


pub const MFI: LanguageCode = LanguageCode {
    name: "Wandala",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mfi",
    individual_languages: &[
    ],
};


pub const MFJ: LanguageCode = LanguageCode {
    name: "Mefele",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mfj",
    individual_languages: &[
    ],
};


pub const MFK: LanguageCode = LanguageCode {
    name: "North Mofu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mfk",
    individual_languages: &[
    ],
};


pub const MFL: LanguageCode = LanguageCode {
    name: "Putai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mfl",
    individual_languages: &[
    ],
};


pub const MFM: LanguageCode = LanguageCode {
    name: "Marghi South",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mfm",
    individual_languages: &[
    ],
};


pub const MFN: LanguageCode = LanguageCode {
    name: "Cross River Mbembe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mfn",
    individual_languages: &[
    ],
};


pub const MFO: LanguageCode = LanguageCode {
    name: "Mbe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mfo",
    individual_languages: &[
    ],
};


pub const MFP: LanguageCode = LanguageCode {
    name: "Makassar Malay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mfp",
    individual_languages: &[
    ],
};


pub const MFQ: LanguageCode = LanguageCode {
    name: "Moba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mfq",
    individual_languages: &[
    ],
};


pub const MFR: LanguageCode = LanguageCode {
    name: "Marrithiyel",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mfr",
    individual_languages: &[
    ],
};


pub const MFS: LanguageCode = LanguageCode {
    name: "Mexican Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mfs",
    individual_languages: &[
    ],
};


pub const MFT: LanguageCode = LanguageCode {
    name: "Mokerang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mft",
    individual_languages: &[
    ],
};


pub const MFU: LanguageCode = LanguageCode {
    name: "Mbwela",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mfu",
    individual_languages: &[
    ],
};


pub const MFV: LanguageCode = LanguageCode {
    name: "Mandjak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mfv",
    individual_languages: &[
    ],
};


pub const MFW: LanguageCode = LanguageCode {
    name: "Mulaha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mfw",
    individual_languages: &[
    ],
};


pub const MFX: LanguageCode = LanguageCode {
    name: "Melo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mfx",
    individual_languages: &[
    ],
};


pub const MFY: LanguageCode = LanguageCode {
    name: "Mayo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mfy",
    individual_languages: &[
    ],
};


pub const MFZ: LanguageCode = LanguageCode {
    name: "Mabaan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mfz",
    individual_languages: &[
    ],
};


pub const MGB: LanguageCode = LanguageCode {
    name: "Mararit",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mgb",
    individual_languages: &[
    ],
};


pub const MGC: LanguageCode = LanguageCode {
    name: "Morokodo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mgc",
    individual_languages: &[
    ],
};


pub const MGD: LanguageCode = LanguageCode {
    name: "Moru",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mgd",
    individual_languages: &[
    ],
};


pub const MGE: LanguageCode = LanguageCode {
    name: "Mango",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mge",
    individual_languages: &[
    ],
};


pub const MGF: LanguageCode = LanguageCode {
    name: "Maklew",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mgf",
    individual_languages: &[
    ],
};


pub const MGG: LanguageCode = LanguageCode {
    name: "Mpumpong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mgg",
    individual_languages: &[
    ],
};


pub const MGH: LanguageCode = LanguageCode {
    name: "Makhuwa-Meetto",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mgh",
    individual_languages: &[
    ],
};


pub const MGI: LanguageCode = LanguageCode {
    name: "Lijili",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mgi",
    individual_languages: &[
    ],
};


pub const MGJ: LanguageCode = LanguageCode {
    name: "Abureni",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mgj",
    individual_languages: &[
    ],
};


pub const MGK: LanguageCode = LanguageCode {
    name: "Mawes",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mgk",
    individual_languages: &[
    ],
};


pub const MGL: LanguageCode = LanguageCode {
    name: "Maleu-Kilenge",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mgl",
    individual_languages: &[
    ],
};


pub const MGM: LanguageCode = LanguageCode {
    name: "Mambae",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mgm",
    individual_languages: &[
    ],
};


pub const MGN: LanguageCode = LanguageCode {
    name: "Mbangi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mgn",
    individual_languages: &[
    ],
};


pub const MGO: LanguageCode = LanguageCode {
    name: "Meta'",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mgo",
    individual_languages: &[
    ],
};


pub const MGP: LanguageCode = LanguageCode {
    name: "Eastern Magar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mgp",
    individual_languages: &[
    ],
};


pub const MGQ: LanguageCode = LanguageCode {
    name: "Malila",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mgq",
    individual_languages: &[
    ],
};


pub const MGR: LanguageCode = LanguageCode {
    name: "Mambwe-Lungu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mgr",
    individual_languages: &[
    ],
};


pub const MGS: LanguageCode = LanguageCode {
    name: "Manda (Tanzania)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mgs",
    individual_languages: &[
    ],
};


pub const MGT: LanguageCode = LanguageCode {
    name: "Mongol",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mgt",
    individual_languages: &[
    ],
};


pub const MGU: LanguageCode = LanguageCode {
    name: "Mailu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mgu",
    individual_languages: &[
    ],
};


pub const MGV: LanguageCode = LanguageCode {
    name: "Matengo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mgv",
    individual_languages: &[
    ],
};


pub const MGW: LanguageCode = LanguageCode {
    name: "Matumbi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mgw",
    individual_languages: &[
    ],
};


pub const MGY: LanguageCode = LanguageCode {
    name: "Mbunga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mgy",
    individual_languages: &[
    ],
};


pub const MGZ: LanguageCode = LanguageCode {
    name: "Mbugwe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mgz",
    individual_languages: &[
    ],
};


pub const MHA: LanguageCode = LanguageCode {
    name: "Manda (India)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mha",
    individual_languages: &[
    ],
};


pub const MHB: LanguageCode = LanguageCode {
    name: "Mahongwe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mhb",
    individual_languages: &[
    ],
};


pub const MHC: LanguageCode = LanguageCode {
    name: "Mocho",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mhc",
    individual_languages: &[
    ],
};


pub const MHD: LanguageCode = LanguageCode {
    name: "Mbugu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mhd",
    individual_languages: &[
    ],
};


pub const MHE: LanguageCode = LanguageCode {
    name: "Besisi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mhe",
    individual_languages: &[
    ],
};


pub const MHF: LanguageCode = LanguageCode {
    name: "Mamaa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mhf",
    individual_languages: &[
    ],
};


pub const MHG: LanguageCode = LanguageCode {
    name: "Margu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mhg",
    individual_languages: &[
    ],
};


pub const MHI: LanguageCode = LanguageCode {
    name: "Ma'di",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mhi",
    individual_languages: &[
    ],
};


pub const MHJ: LanguageCode = LanguageCode {
    name: "Mogholi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mhj",
    individual_languages: &[
    ],
};


pub const MHK: LanguageCode = LanguageCode {
    name: "Mungaka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mhk",
    individual_languages: &[
    ],
};


pub const MHL: LanguageCode = LanguageCode {
    name: "Mauwake",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mhl",
    individual_languages: &[
    ],
};


pub const MHM: LanguageCode = LanguageCode {
    name: "Makhuwa-Moniga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mhm",
    individual_languages: &[
    ],
};


pub const MHN: LanguageCode = LanguageCode {
    name: "Mócheno",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mhn",
    individual_languages: &[
    ],
};


pub const MHO: LanguageCode = LanguageCode {
    name: "Mashi (Zambia)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mho",
    individual_languages: &[
    ],
};


pub const MHP: LanguageCode = LanguageCode {
    name: "Balinese Malay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mhp",
    individual_languages: &[
    ],
};


pub const MHQ: LanguageCode = LanguageCode {
    name: "Mandan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mhq",
    individual_languages: &[
    ],
};


pub const MHR: LanguageCode = LanguageCode {
    name: "Eastern Mari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mhr",
    individual_languages: &[
    ],
};


pub const MHS: LanguageCode = LanguageCode {
    name: "Buru (Indonesia)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mhs",
    individual_languages: &[
    ],
};


pub const MHT: LanguageCode = LanguageCode {
    name: "Mandahuaca",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mht",
    individual_languages: &[
    ],
};


pub const MHU: LanguageCode = LanguageCode {
    name: "Digaro-Mishmi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mhu",
    individual_languages: &[
    ],
};


pub const MHW: LanguageCode = LanguageCode {
    name: "Mbukushu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mhw",
    individual_languages: &[
    ],
};


pub const MHX: LanguageCode = LanguageCode {
    name: "Maru",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mhx",
    individual_languages: &[
    ],
};


pub const MHY: LanguageCode = LanguageCode {
    name: "Ma'anyan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mhy",
    individual_languages: &[
    ],
};


pub const MHZ: LanguageCode = LanguageCode {
    name: "Mor (Mor Islands)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mhz",
    individual_languages: &[
    ],
};


pub const MIA: LanguageCode = LanguageCode {
    name: "Miami",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mia",
    individual_languages: &[
    ],
};


pub const MIB: LanguageCode = LanguageCode {
    name: "Atatláhuca Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mib",
    individual_languages: &[
    ],
};


pub const MID: LanguageCode = LanguageCode {
    name: "Mandaic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mid",
    individual_languages: &[
    ],
};


pub const MIE: LanguageCode = LanguageCode {
    name: "Ocotepec Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mie",
    individual_languages: &[
    ],
};


pub const MIF: LanguageCode = LanguageCode {
    name: "Mofu-Gudur",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mif",
    individual_languages: &[
    ],
};


pub const MIG: LanguageCode = LanguageCode {
    name: "San Miguel El Grande Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mig",
    individual_languages: &[
    ],
};


pub const MIH: LanguageCode = LanguageCode {
    name: "Chayuco Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mih",
    individual_languages: &[
    ],
};


pub const MII: LanguageCode = LanguageCode {
    name: "Chigmecatitlán Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mii",
    individual_languages: &[
    ],
};


pub const MIJ: LanguageCode = LanguageCode {
    name: "Abar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mij",
    individual_languages: &[
    ],
};


pub const MIK: LanguageCode = LanguageCode {
    name: "Mikasuki",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mik",
    individual_languages: &[
    ],
};


pub const MIL: LanguageCode = LanguageCode {
    name: "Peñoles Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mil",
    individual_languages: &[
    ],
};


pub const MIM: LanguageCode = LanguageCode {
    name: "Alacatlatzala Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mim",
    individual_languages: &[
    ],
};


pub const MIO: LanguageCode = LanguageCode {
    name: "Pinotepa Nacional Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mio",
    individual_languages: &[
    ],
};


pub const MIP: LanguageCode = LanguageCode {
    name: "Apasco-Apoala Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mip",
    individual_languages: &[
    ],
};


pub const MIQ: LanguageCode = LanguageCode {
    name: "Mískito",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "miq",
    individual_languages: &[
    ],
};


pub const MIR: LanguageCode = LanguageCode {
    name: "Isthmus Mixe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mir",
    individual_languages: &[
    ],
};


pub const MIT: LanguageCode = LanguageCode {
    name: "Southern Puebla Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mit",
    individual_languages: &[
    ],
};


pub const MIU: LanguageCode = LanguageCode {
    name: "Cacaloxtepec Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "miu",
    individual_languages: &[
    ],
};


pub const MIW: LanguageCode = LanguageCode {
    name: "Akoye",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "miw",
    individual_languages: &[
    ],
};


pub const MIX: LanguageCode = LanguageCode {
    name: "Mixtepec Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mix",
    individual_languages: &[
    ],
};


pub const MIY: LanguageCode = LanguageCode {
    name: "Ayutla Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "miy",
    individual_languages: &[
    ],
};


pub const MIZ: LanguageCode = LanguageCode {
    name: "Coatzospan Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "miz",
    individual_languages: &[
    ],
};


pub const MJB: LanguageCode = LanguageCode {
    name: "Makalero",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mjb",
    individual_languages: &[
    ],
};


pub const MJC: LanguageCode = LanguageCode {
    name: "San Juan Colorado Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mjc",
    individual_languages: &[
    ],
};


pub const MJD: LanguageCode = LanguageCode {
    name: "Northwest Maidu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mjd",
    individual_languages: &[
    ],
};


pub const MJE: LanguageCode = LanguageCode {
    name: "Muskum",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mje",
    individual_languages: &[
    ],
};


pub const MJG: LanguageCode = LanguageCode {
    name: "Tu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mjg",
    individual_languages: &[
    ],
};


pub const MJH: LanguageCode = LanguageCode {
    name: "Mwera (Nyasa)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mjh",
    individual_languages: &[
    ],
};


pub const MJI: LanguageCode = LanguageCode {
    name: "Kim Mun",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mji",
    individual_languages: &[
    ],
};


pub const MJJ: LanguageCode = LanguageCode {
    name: "Mawak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mjj",
    individual_languages: &[
    ],
};


pub const MJK: LanguageCode = LanguageCode {
    name: "Matukar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mjk",
    individual_languages: &[
    ],
};


pub const MJL: LanguageCode = LanguageCode {
    name: "Mandeali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mjl",
    individual_languages: &[
    ],
};


pub const MJM: LanguageCode = LanguageCode {
    name: "Medebur",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mjm",
    individual_languages: &[
    ],
};


pub const MJN: LanguageCode = LanguageCode {
    name: "Ma (Papua New Guinea)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mjn",
    individual_languages: &[
    ],
};


pub const MJO: LanguageCode = LanguageCode {
    name: "Malankuravan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mjo",
    individual_languages: &[
    ],
};


pub const MJP: LanguageCode = LanguageCode {
    name: "Malapandaram",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mjp",
    individual_languages: &[
    ],
};


pub const MJQ: LanguageCode = LanguageCode {
    name: "Malaryan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mjq",
    individual_languages: &[
    ],
};


pub const MJR: LanguageCode = LanguageCode {
    name: "Malavedan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mjr",
    individual_languages: &[
    ],
};


pub const MJS: LanguageCode = LanguageCode {
    name: "Miship",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mjs",
    individual_languages: &[
    ],
};


pub const MJT: LanguageCode = LanguageCode {
    name: "Sauria Paharia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mjt",
    individual_languages: &[
    ],
};


pub const MJU: LanguageCode = LanguageCode {
    name: "Manna-Dora",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mju",
    individual_languages: &[
    ],
};


pub const MJV: LanguageCode = LanguageCode {
    name: "Mannan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mjv",
    individual_languages: &[
    ],
};


pub const MJW: LanguageCode = LanguageCode {
    name: "Karbi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mjw",
    individual_languages: &[
    ],
};


pub const MJX: LanguageCode = LanguageCode {
    name: "Mahali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mjx",
    individual_languages: &[
    ],
};


pub const MJY: LanguageCode = LanguageCode {
    name: "Mahican",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mjy",
    individual_languages: &[
    ],
};


pub const MJZ: LanguageCode = LanguageCode {
    name: "Majhi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mjz",
    individual_languages: &[
    ],
};


pub const MKA: LanguageCode = LanguageCode {
    name: "Mbre",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mka",
    individual_languages: &[
    ],
};


pub const MKB: LanguageCode = LanguageCode {
    name: "Mal Paharia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mkb",
    individual_languages: &[
    ],
};


pub const MKC: LanguageCode = LanguageCode {
    name: "Siliput",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mkc",
    individual_languages: &[
    ],
};


pub const MKE: LanguageCode = LanguageCode {
    name: "Mawchi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mke",
    individual_languages: &[
    ],
};


pub const MKF: LanguageCode = LanguageCode {
    name: "Miya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mkf",
    individual_languages: &[
    ],
};


pub const MKG: LanguageCode = LanguageCode {
    name: "Mak (China)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mkg",
    individual_languages: &[
    ],
};


pub const MKI: LanguageCode = LanguageCode {
    name: "Dhatki",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mki",
    individual_languages: &[
    ],
};


pub const MKJ: LanguageCode = LanguageCode {
    name: "Mokilese",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mkj",
    individual_languages: &[
    ],
};


pub const MKK: LanguageCode = LanguageCode {
    name: "Byep",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mkk",
    individual_languages: &[
    ],
};


pub const MKL: LanguageCode = LanguageCode {
    name: "Mokole",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mkl",
    individual_languages: &[
    ],
};


pub const MKM: LanguageCode = LanguageCode {
    name: "Moklen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mkm",
    individual_languages: &[
    ],
};


pub const MKN: LanguageCode = LanguageCode {
    name: "Kupang Malay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mkn",
    individual_languages: &[
    ],
};


pub const MKO: LanguageCode = LanguageCode {
    name: "Mingang Doso",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mko",
    individual_languages: &[
    ],
};


pub const MKP: LanguageCode = LanguageCode {
    name: "Moikodi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mkp",
    individual_languages: &[
    ],
};


pub const MKQ: LanguageCode = LanguageCode {
    name: "Bay Miwok",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mkq",
    individual_languages: &[
    ],
};


pub const MKR: LanguageCode = LanguageCode {
    name: "Malas",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mkr",
    individual_languages: &[
    ],
};


pub const MKS: LanguageCode = LanguageCode {
    name: "Silacayoapan Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mks",
    individual_languages: &[
    ],
};


pub const MKT: LanguageCode = LanguageCode {
    name: "Vamale",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mkt",
    individual_languages: &[
    ],
};


pub const MKU: LanguageCode = LanguageCode {
    name: "Konyanka Maninka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mku",
    individual_languages: &[
    ],
};


pub const MKV: LanguageCode = LanguageCode {
    name: "Mafea",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mkv",
    individual_languages: &[
    ],
};


pub const MKW: LanguageCode = LanguageCode {
    name: "Kituba (Congo)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mkw",
    individual_languages: &[
    ],
};


pub const MKX: LanguageCode = LanguageCode {
    name: "Kinamiging Manobo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mkx",
    individual_languages: &[
    ],
};


pub const MKY: LanguageCode = LanguageCode {
    name: "East Makian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mky",
    individual_languages: &[
    ],
};


pub const MKZ: LanguageCode = LanguageCode {
    name: "Makasae",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mkz",
    individual_languages: &[
    ],
};


pub const MLA: LanguageCode = LanguageCode {
    name: "Malo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mla",
    individual_languages: &[
    ],
};


pub const MLB: LanguageCode = LanguageCode {
    name: "Mbule",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mlb",
    individual_languages: &[
    ],
};


pub const MLC: LanguageCode = LanguageCode {
    name: "Cao Lan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mlc",
    individual_languages: &[
    ],
};


pub const MLE: LanguageCode = LanguageCode {
    name: "Manambu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mle",
    individual_languages: &[
    ],
};


pub const MLF: LanguageCode = LanguageCode {
    name: "Mal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mlf",
    individual_languages: &[
    ],
};


pub const MLH: LanguageCode = LanguageCode {
    name: "Mape",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mlh",
    individual_languages: &[
    ],
};


pub const MLI: LanguageCode = LanguageCode {
    name: "Malimpung",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mli",
    individual_languages: &[
    ],
};


pub const MLJ: LanguageCode = LanguageCode {
    name: "Miltu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mlj",
    individual_languages: &[
    ],
};


pub const MLK: LanguageCode = LanguageCode {
    name: "Ilwana",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mlk",
    individual_languages: &[
    ],
};


pub const MLL: LanguageCode = LanguageCode {
    name: "Malua Bay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mll",
    individual_languages: &[
    ],
};


pub const MLM: LanguageCode = LanguageCode {
    name: "Mulam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mlm",
    individual_languages: &[
    ],
};


pub const MLN: LanguageCode = LanguageCode {
    name: "Malango",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mln",
    individual_languages: &[
    ],
};


pub const MLO: LanguageCode = LanguageCode {
    name: "Mlomp",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mlo",
    individual_languages: &[
    ],
};


pub const MLP: LanguageCode = LanguageCode {
    name: "Bargam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mlp",
    individual_languages: &[
    ],
};


pub const MLQ: LanguageCode = LanguageCode {
    name: "Western Maninkakan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mlq",
    individual_languages: &[
    ],
};


pub const MLR: LanguageCode = LanguageCode {
    name: "Vame",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mlr",
    individual_languages: &[
    ],
};


pub const MLS: LanguageCode = LanguageCode {
    name: "Masalit",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mls",
    individual_languages: &[
    ],
};


pub const MLU: LanguageCode = LanguageCode {
    name: "To'abaita",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mlu",
    individual_languages: &[
    ],
};


pub const MLV: LanguageCode = LanguageCode {
    name: "Motlav",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mlv",
    individual_languages: &[
    ],
};


pub const MLW: LanguageCode = LanguageCode {
    name: "Moloko",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mlw",
    individual_languages: &[
    ],
};


pub const MLX: LanguageCode = LanguageCode {
    name: "Malfaxal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mlx",
    individual_languages: &[
    ],
};


pub const MLZ: LanguageCode = LanguageCode {
    name: "Malaynon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mlz",
    individual_languages: &[
    ],
};


pub const MMA: LanguageCode = LanguageCode {
    name: "Mama",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mma",
    individual_languages: &[
    ],
};


pub const MMB: LanguageCode = LanguageCode {
    name: "Momina",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mmb",
    individual_languages: &[
    ],
};


pub const MMC: LanguageCode = LanguageCode {
    name: "Michoacán Mazahua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mmc",
    individual_languages: &[
    ],
};


pub const MMD: LanguageCode = LanguageCode {
    name: "Maonan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mmd",
    individual_languages: &[
    ],
};


pub const MME: LanguageCode = LanguageCode {
    name: "Mae",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mme",
    individual_languages: &[
    ],
};


pub const MMF: LanguageCode = LanguageCode {
    name: "Mundat",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mmf",
    individual_languages: &[
    ],
};


pub const MMG: LanguageCode = LanguageCode {
    name: "North Ambrym",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mmg",
    individual_languages: &[
    ],
};


pub const MMH: LanguageCode = LanguageCode {
    name: "Mehináku",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mmh",
    individual_languages: &[
    ],
};


pub const MMI: LanguageCode = LanguageCode {
    name: "Musar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mmi",
    individual_languages: &[
    ],
};


pub const MMJ: LanguageCode = LanguageCode {
    name: "Majhwar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mmj",
    individual_languages: &[
    ],
};


pub const MMK: LanguageCode = LanguageCode {
    name: "Mukha-Dora",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mmk",
    individual_languages: &[
    ],
};


pub const MML: LanguageCode = LanguageCode {
    name: "Man Met",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mml",
    individual_languages: &[
    ],
};


pub const MMM: LanguageCode = LanguageCode {
    name: "Maii",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mmm",
    individual_languages: &[
    ],
};


pub const MMN: LanguageCode = LanguageCode {
    name: "Mamanwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mmn",
    individual_languages: &[
    ],
};


pub const MMO: LanguageCode = LanguageCode {
    name: "Mangga Buang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mmo",
    individual_languages: &[
    ],
};


pub const MMP: LanguageCode = LanguageCode {
    name: "Siawi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mmp",
    individual_languages: &[
    ],
};


pub const MMQ: LanguageCode = LanguageCode {
    name: "Musak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mmq",
    individual_languages: &[
    ],
};


pub const MMR: LanguageCode = LanguageCode {
    name: "Western Xiangxi Miao",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mmr",
    individual_languages: &[
    ],
};


pub const MMT: LanguageCode = LanguageCode {
    name: "Malalamai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mmt",
    individual_languages: &[
    ],
};


pub const MMU: LanguageCode = LanguageCode {
    name: "Mmaala",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mmu",
    individual_languages: &[
    ],
};


pub const MMV: LanguageCode = LanguageCode {
    name: "Miriti",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mmv",
    individual_languages: &[
    ],
};


pub const MMW: LanguageCode = LanguageCode {
    name: "Emae",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mmw",
    individual_languages: &[
    ],
};


pub const MMX: LanguageCode = LanguageCode {
    name: "Madak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mmx",
    individual_languages: &[
    ],
};


pub const MMY: LanguageCode = LanguageCode {
    name: "Migaama",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mmy",
    individual_languages: &[
    ],
};


pub const MMZ: LanguageCode = LanguageCode {
    name: "Mabaale",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mmz",
    individual_languages: &[
    ],
};


pub const MNA: LanguageCode = LanguageCode {
    name: "Mbula",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mna",
    individual_languages: &[
    ],
};


pub const MNB: LanguageCode = LanguageCode {
    name: "Muna",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mnb",
    individual_languages: &[
    ],
};


pub const MND: LanguageCode = LanguageCode {
    name: "Mondé",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mnd",
    individual_languages: &[
    ],
};


pub const MNE: LanguageCode = LanguageCode {
    name: "Naba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mne",
    individual_languages: &[
    ],
};


pub const MNF: LanguageCode = LanguageCode {
    name: "Mundani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mnf",
    individual_languages: &[
    ],
};


pub const MNG: LanguageCode = LanguageCode {
    name: "Eastern Mnong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mng",
    individual_languages: &[
    ],
};


pub const MNH: LanguageCode = LanguageCode {
    name: "Mono (Democratic Republic of Congo)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mnh",
    individual_languages: &[
    ],
};


pub const MNJ: LanguageCode = LanguageCode {
    name: "Munji",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mnj",
    individual_languages: &[
    ],
};


pub const MNK: LanguageCode = LanguageCode {
    name: "Mandinka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mnk",
    individual_languages: &[
    ],
};


pub const MNL: LanguageCode = LanguageCode {
    name: "Tiale",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mnl",
    individual_languages: &[
    ],
};


pub const MNM: LanguageCode = LanguageCode {
    name: "Mapena",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mnm",
    individual_languages: &[
    ],
};


pub const MNN: LanguageCode = LanguageCode {
    name: "Southern Mnong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mnn",
    individual_languages: &[
    ],
};


pub const MNP: LanguageCode = LanguageCode {
    name: "Min Bei Chinese",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mnp",
    individual_languages: &[
    ],
};


pub const MNQ: LanguageCode = LanguageCode {
    name: "Minriq",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mnq",
    individual_languages: &[
    ],
};


pub const MNR: LanguageCode = LanguageCode {
    name: "Mono (USA)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mnr",
    individual_languages: &[
    ],
};


pub const MNS: LanguageCode = LanguageCode {
    name: "Mansi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mns",
    individual_languages: &[
    ],
};


pub const MNU: LanguageCode = LanguageCode {
    name: "Mer",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mnu",
    individual_languages: &[
    ],
};


pub const MNV: LanguageCode = LanguageCode {
    name: "Rennell-Bellona",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mnv",
    individual_languages: &[
    ],
};


pub const MNW: LanguageCode = LanguageCode {
    name: "Mon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mnw",
    individual_languages: &[
    ],
};


pub const MNX: LanguageCode = LanguageCode {
    name: "Manikion",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mnx",
    individual_languages: &[
    ],
};


pub const MNY: LanguageCode = LanguageCode {
    name: "Manyawa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mny",
    individual_languages: &[
    ],
};


pub const MNZ: LanguageCode = LanguageCode {
    name: "Moni",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mnz",
    individual_languages: &[
    ],
};


pub const MOA: LanguageCode = LanguageCode {
    name: "Mwan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "moa",
    individual_languages: &[
    ],
};


pub const MOC: LanguageCode = LanguageCode {
    name: "Mocoví",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "moc",
    individual_languages: &[
    ],
};


pub const MOD: LanguageCode = LanguageCode {
    name: "Mobilian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mod",
    individual_languages: &[
    ],
};


pub const MOE: LanguageCode = LanguageCode {
    name: "Innu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "moe",
    individual_languages: &[
    ],
};


pub const MOG: LanguageCode = LanguageCode {
    name: "Mongondow",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mog",
    individual_languages: &[
    ],
};


pub const MOI: LanguageCode = LanguageCode {
    name: "Mboi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "moi",
    individual_languages: &[
    ],
};


pub const MOJ: LanguageCode = LanguageCode {
    name: "Monzombo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "moj",
    individual_languages: &[
    ],
};


pub const MOK: LanguageCode = LanguageCode {
    name: "Morori",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mok",
    individual_languages: &[
    ],
};


pub const MOM: LanguageCode = LanguageCode {
    name: "Mangue",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mom",
    individual_languages: &[
    ],
};


pub const MOO: LanguageCode = LanguageCode {
    name: "Monom",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "moo",
    individual_languages: &[
    ],
};


pub const MOP: LanguageCode = LanguageCode {
    name: "Mopán Maya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mop",
    individual_languages: &[
    ],
};


pub const MOQ: LanguageCode = LanguageCode {
    name: "Mor (Bomberai Peninsula)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "moq",
    individual_languages: &[
    ],
};


pub const MOR: LanguageCode = LanguageCode {
    name: "Moro",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mor",
    individual_languages: &[
    ],
};


pub const MOT: LanguageCode = LanguageCode {
    name: "Barí",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mot",
    individual_languages: &[
    ],
};


pub const MOU: LanguageCode = LanguageCode {
    name: "Mogum",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mou",
    individual_languages: &[
    ],
};


pub const MOV: LanguageCode = LanguageCode {
    name: "Mohave",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mov",
    individual_languages: &[
    ],
};


pub const MOW: LanguageCode = LanguageCode {
    name: "Moi (Congo)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mow",
    individual_languages: &[
    ],
};


pub const MOX: LanguageCode = LanguageCode {
    name: "Molima",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mox",
    individual_languages: &[
    ],
};


pub const MOY: LanguageCode = LanguageCode {
    name: "Shekkacho",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "moy",
    individual_languages: &[
    ],
};


pub const MOZ: LanguageCode = LanguageCode {
    name: "Mukulu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "moz",
    individual_languages: &[
    ],
};


pub const MPA: LanguageCode = LanguageCode {
    name: "Mpoto",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mpa",
    individual_languages: &[
    ],
};


pub const MPB: LanguageCode = LanguageCode {
    name: "Malak Malak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mpb",
    individual_languages: &[
    ],
};


pub const MPC: LanguageCode = LanguageCode {
    name: "Mangarrayi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mpc",
    individual_languages: &[
    ],
};


pub const MPD: LanguageCode = LanguageCode {
    name: "Machinere",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mpd",
    individual_languages: &[
    ],
};


pub const MPE: LanguageCode = LanguageCode {
    name: "Majang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mpe",
    individual_languages: &[
    ],
};


pub const MPG: LanguageCode = LanguageCode {
    name: "Marba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mpg",
    individual_languages: &[
    ],
};


pub const MPH: LanguageCode = LanguageCode {
    name: "Maung",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mph",
    individual_languages: &[
    ],
};


pub const MPI: LanguageCode = LanguageCode {
    name: "Mpade",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mpi",
    individual_languages: &[
    ],
};


pub const MPJ: LanguageCode = LanguageCode {
    name: "Martu Wangka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mpj",
    individual_languages: &[
    ],
};


pub const MPK: LanguageCode = LanguageCode {
    name: "Mbara (Chad)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mpk",
    individual_languages: &[
    ],
};


pub const MPL: LanguageCode = LanguageCode {
    name: "Middle Watut",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mpl",
    individual_languages: &[
    ],
};


pub const MPM: LanguageCode = LanguageCode {
    name: "Yosondúa Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mpm",
    individual_languages: &[
    ],
};


pub const MPN: LanguageCode = LanguageCode {
    name: "Mindiri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mpn",
    individual_languages: &[
    ],
};


pub const MPO: LanguageCode = LanguageCode {
    name: "Miu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mpo",
    individual_languages: &[
    ],
};


pub const MPP: LanguageCode = LanguageCode {
    name: "Migabac",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mpp",
    individual_languages: &[
    ],
};


pub const MPQ: LanguageCode = LanguageCode {
    name: "Matís",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mpq",
    individual_languages: &[
    ],
};


pub const MPR: LanguageCode = LanguageCode {
    name: "Vangunu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mpr",
    individual_languages: &[
    ],
};


pub const MPS: LanguageCode = LanguageCode {
    name: "Dadibi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mps",
    individual_languages: &[
    ],
};


pub const MPT: LanguageCode = LanguageCode {
    name: "Mian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mpt",
    individual_languages: &[
    ],
};


pub const MPU: LanguageCode = LanguageCode {
    name: "Makuráp",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mpu",
    individual_languages: &[
    ],
};


pub const MPV: LanguageCode = LanguageCode {
    name: "Mungkip",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mpv",
    individual_languages: &[
    ],
};


pub const MPW: LanguageCode = LanguageCode {
    name: "Mapidian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mpw",
    individual_languages: &[
    ],
};


pub const MPX: LanguageCode = LanguageCode {
    name: "Misima-Panaeati",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mpx",
    individual_languages: &[
    ],
};


pub const MPY: LanguageCode = LanguageCode {
    name: "Mapia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mpy",
    individual_languages: &[
    ],
};


pub const MPZ: LanguageCode = LanguageCode {
    name: "Mpi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mpz",
    individual_languages: &[
    ],
};


pub const MQA: LanguageCode = LanguageCode {
    name: "Maba (Indonesia)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mqa",
    individual_languages: &[
    ],
};


pub const MQB: LanguageCode = LanguageCode {
    name: "Mbuko",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mqb",
    individual_languages: &[
    ],
};


pub const MQC: LanguageCode = LanguageCode {
    name: "Mangole",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mqc",
    individual_languages: &[
    ],
};


pub const MQE: LanguageCode = LanguageCode {
    name: "Matepi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mqe",
    individual_languages: &[
    ],
};


pub const MQF: LanguageCode = LanguageCode {
    name: "Momuna",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mqf",
    individual_languages: &[
    ],
};


pub const MQG: LanguageCode = LanguageCode {
    name: "Kota Bangun Kutai Malay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mqg",
    individual_languages: &[
    ],
};


pub const MQH: LanguageCode = LanguageCode {
    name: "Tlazoyaltepec Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mqh",
    individual_languages: &[
    ],
};


pub const MQI: LanguageCode = LanguageCode {
    name: "Mariri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mqi",
    individual_languages: &[
    ],
};


pub const MQJ: LanguageCode = LanguageCode {
    name: "Mamasa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mqj",
    individual_languages: &[
    ],
};


pub const MQK: LanguageCode = LanguageCode {
    name: "Rajah Kabunsuwan Manobo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mqk",
    individual_languages: &[
    ],
};


pub const MQL: LanguageCode = LanguageCode {
    name: "Mbelime",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mql",
    individual_languages: &[
    ],
};


pub const MQM: LanguageCode = LanguageCode {
    name: "South Marquesan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mqm",
    individual_languages: &[
    ],
};


pub const MQN: LanguageCode = LanguageCode {
    name: "Moronene",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mqn",
    individual_languages: &[
    ],
};


pub const MQO: LanguageCode = LanguageCode {
    name: "Modole",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mqo",
    individual_languages: &[
    ],
};


pub const MQP: LanguageCode = LanguageCode {
    name: "Manipa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mqp",
    individual_languages: &[
    ],
};


pub const MQQ: LanguageCode = LanguageCode {
    name: "Minokok",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mqq",
    individual_languages: &[
    ],
};


pub const MQR: LanguageCode = LanguageCode {
    name: "Mander",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mqr",
    individual_languages: &[
    ],
};


pub const MQS: LanguageCode = LanguageCode {
    name: "West Makian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mqs",
    individual_languages: &[
    ],
};


pub const MQT: LanguageCode = LanguageCode {
    name: "Mok",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mqt",
    individual_languages: &[
    ],
};


pub const MQU: LanguageCode = LanguageCode {
    name: "Mandari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mqu",
    individual_languages: &[
    ],
};


pub const MQV: LanguageCode = LanguageCode {
    name: "Mosimo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mqv",
    individual_languages: &[
    ],
};


pub const MQW: LanguageCode = LanguageCode {
    name: "Murupi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mqw",
    individual_languages: &[
    ],
};


pub const MQX: LanguageCode = LanguageCode {
    name: "Mamuju",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mqx",
    individual_languages: &[
    ],
};


pub const MQY: LanguageCode = LanguageCode {
    name: "Manggarai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mqy",
    individual_languages: &[
    ],
};


pub const MQZ: LanguageCode = LanguageCode {
    name: "Pano",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mqz",
    individual_languages: &[
    ],
};


pub const MRA: LanguageCode = LanguageCode {
    name: "Mlabri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mra",
    individual_languages: &[
    ],
};


pub const MRB: LanguageCode = LanguageCode {
    name: "Marino",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mrb",
    individual_languages: &[
    ],
};


pub const MRC: LanguageCode = LanguageCode {
    name: "Maricopa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mrc",
    individual_languages: &[
    ],
};


pub const MRD: LanguageCode = LanguageCode {
    name: "Western Magar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mrd",
    individual_languages: &[
    ],
};


pub const MRE: LanguageCode = LanguageCode {
    name: "Martha's Vineyard Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mre",
    individual_languages: &[
    ],
};


pub const MRF: LanguageCode = LanguageCode {
    name: "Elseng",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mrf",
    individual_languages: &[
    ],
};


pub const MRG: LanguageCode = LanguageCode {
    name: "Mising",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mrg",
    individual_languages: &[
    ],
};


pub const MRH: LanguageCode = LanguageCode {
    name: "Mara Chin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mrh",
    individual_languages: &[
    ],
};


pub const MRJ: LanguageCode = LanguageCode {
    name: "Western Mari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mrj",
    individual_languages: &[
    ],
};


pub const MRK: LanguageCode = LanguageCode {
    name: "Hmwaveke",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mrk",
    individual_languages: &[
    ],
};


pub const MRL: LanguageCode = LanguageCode {
    name: "Mortlockese",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mrl",
    individual_languages: &[
    ],
};


pub const MRM: LanguageCode = LanguageCode {
    name: "Merlav",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mrm",
    individual_languages: &[
    ],
};


pub const MRN: LanguageCode = LanguageCode {
    name: "Cheke Holo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mrn",
    individual_languages: &[
    ],
};


pub const MRO: LanguageCode = LanguageCode {
    name: "Mru",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mro",
    individual_languages: &[
    ],
};


pub const MRP: LanguageCode = LanguageCode {
    name: "Morouas",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mrp",
    individual_languages: &[
    ],
};


pub const MRQ: LanguageCode = LanguageCode {
    name: "North Marquesan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mrq",
    individual_languages: &[
    ],
};


pub const MRR: LanguageCode = LanguageCode {
    name: "Maria (India)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mrr",
    individual_languages: &[
    ],
};


pub const MRS: LanguageCode = LanguageCode {
    name: "Maragus",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mrs",
    individual_languages: &[
    ],
};


pub const MRT: LanguageCode = LanguageCode {
    name: "Marghi Central",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mrt",
    individual_languages: &[
    ],
};


pub const MRU: LanguageCode = LanguageCode {
    name: "Mono (Cameroon)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mru",
    individual_languages: &[
    ],
};


pub const MRV: LanguageCode = LanguageCode {
    name: "Mangareva",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mrv",
    individual_languages: &[
    ],
};


pub const MRW: LanguageCode = LanguageCode {
    name: "Maranao",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mrw",
    individual_languages: &[
    ],
};


pub const MRX: LanguageCode = LanguageCode {
    name: "Maremgi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mrx",
    individual_languages: &[
    ],
};


pub const MRY: LanguageCode = LanguageCode {
    name: "Mandaya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mry",
    individual_languages: &[
    ],
};


pub const MRZ: LanguageCode = LanguageCode {
    name: "Marind",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mrz",
    individual_languages: &[
    ],
};


pub const MSB: LanguageCode = LanguageCode {
    name: "Masbatenyo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "msb",
    individual_languages: &[
    ],
};


pub const MSC: LanguageCode = LanguageCode {
    name: "Sankaran Maninka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "msc",
    individual_languages: &[
    ],
};


pub const MSD: LanguageCode = LanguageCode {
    name: "Yucatec Maya Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "msd",
    individual_languages: &[
    ],
};


pub const MSE: LanguageCode = LanguageCode {
    name: "Musey",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mse",
    individual_languages: &[
    ],
};


pub const MSF: LanguageCode = LanguageCode {
    name: "Mekwei",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "msf",
    individual_languages: &[
    ],
};


pub const MSG: LanguageCode = LanguageCode {
    name: "Moraid",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "msg",
    individual_languages: &[
    ],
};


pub const MSH: LanguageCode = LanguageCode {
    name: "Masikoro Malagasy",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "msh",
    individual_languages: &[
    ],
};


pub const MSI: LanguageCode = LanguageCode {
    name: "Sabah Malay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "msi",
    individual_languages: &[
    ],
};


pub const MSJ: LanguageCode = LanguageCode {
    name: "Ma (Democratic Republic of Congo)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "msj",
    individual_languages: &[
    ],
};


pub const MSK: LanguageCode = LanguageCode {
    name: "Mansaka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "msk",
    individual_languages: &[
    ],
};


pub const MSL: LanguageCode = LanguageCode {
    name: "Molof",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "msl",
    individual_languages: &[
    ],
};


pub const MSM: LanguageCode = LanguageCode {
    name: "Agusan Manobo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "msm",
    individual_languages: &[
    ],
};


pub const MSN: LanguageCode = LanguageCode {
    name: "Vurës",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "msn",
    individual_languages: &[
    ],
};


pub const MSO: LanguageCode = LanguageCode {
    name: "Mombum",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mso",
    individual_languages: &[
    ],
};


pub const MSP: LanguageCode = LanguageCode {
    name: "Maritsauá",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "msp",
    individual_languages: &[
    ],
};


pub const MSQ: LanguageCode = LanguageCode {
    name: "Caac",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "msq",
    individual_languages: &[
    ],
};


pub const MSR: LanguageCode = LanguageCode {
    name: "Mongolian Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "msr",
    individual_languages: &[
    ],
};


pub const MSS: LanguageCode = LanguageCode {
    name: "West Masela",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mss",
    individual_languages: &[
    ],
};


pub const MSU: LanguageCode = LanguageCode {
    name: "Musom",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "msu",
    individual_languages: &[
    ],
};


pub const MSV: LanguageCode = LanguageCode {
    name: "Maslam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "msv",
    individual_languages: &[
    ],
};


pub const MSW: LanguageCode = LanguageCode {
    name: "Mansoanka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "msw",
    individual_languages: &[
    ],
};


pub const MSX: LanguageCode = LanguageCode {
    name: "Moresada",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "msx",
    individual_languages: &[
    ],
};


pub const MSY: LanguageCode = LanguageCode {
    name: "Aruamu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "msy",
    individual_languages: &[
    ],
};


pub const MSZ: LanguageCode = LanguageCode {
    name: "Momare",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "msz",
    individual_languages: &[
    ],
};


pub const MTA: LanguageCode = LanguageCode {
    name: "Cotabato Manobo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mta",
    individual_languages: &[
    ],
};


pub const MTB: LanguageCode = LanguageCode {
    name: "Anyin Morofo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mtb",
    individual_languages: &[
    ],
};


pub const MTC: LanguageCode = LanguageCode {
    name: "Munit",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mtc",
    individual_languages: &[
    ],
};


pub const MTD: LanguageCode = LanguageCode {
    name: "Mualang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mtd",
    individual_languages: &[
    ],
};


pub const MTE: LanguageCode = LanguageCode {
    name: "Mono (Solomon Islands)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mte",
    individual_languages: &[
    ],
};


pub const MTF: LanguageCode = LanguageCode {
    name: "Murik (Papua New Guinea)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mtf",
    individual_languages: &[
    ],
};


pub const MTG: LanguageCode = LanguageCode {
    name: "Una",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mtg",
    individual_languages: &[
    ],
};


pub const MTH: LanguageCode = LanguageCode {
    name: "Munggui",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mth",
    individual_languages: &[
    ],
};


pub const MTI: LanguageCode = LanguageCode {
    name: "Maiwa (Papua New Guinea)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mti",
    individual_languages: &[
    ],
};


pub const MTJ: LanguageCode = LanguageCode {
    name: "Moskona",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mtj",
    individual_languages: &[
    ],
};


pub const MTK: LanguageCode = LanguageCode {
    name: "Mbe'",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mtk",
    individual_languages: &[
    ],
};


pub const MTL: LanguageCode = LanguageCode {
    name: "Montol",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mtl",
    individual_languages: &[
    ],
};


pub const MTM: LanguageCode = LanguageCode {
    name: "Mator",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mtm",
    individual_languages: &[
    ],
};


pub const MTN: LanguageCode = LanguageCode {
    name: "Matagalpa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mtn",
    individual_languages: &[
    ],
};


pub const MTO: LanguageCode = LanguageCode {
    name: "Totontepec Mixe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mto",
    individual_languages: &[
    ],
};


pub const MTP: LanguageCode = LanguageCode {
    name: "Wichí Lhamtés Nocten",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mtp",
    individual_languages: &[
    ],
};


pub const MTQ: LanguageCode = LanguageCode {
    name: "Muong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mtq",
    individual_languages: &[
    ],
};


pub const MTR: LanguageCode = LanguageCode {
    name: "Mewari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mtr",
    individual_languages: &[
    ],
};


pub const MTS: LanguageCode = LanguageCode {
    name: "Yora",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mts",
    individual_languages: &[
    ],
};


pub const MTT: LanguageCode = LanguageCode {
    name: "Mota",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mtt",
    individual_languages: &[
    ],
};


pub const MTU: LanguageCode = LanguageCode {
    name: "Tututepec Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mtu",
    individual_languages: &[
    ],
};


pub const MTV: LanguageCode = LanguageCode {
    name: "Asaro'o",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mtv",
    individual_languages: &[
    ],
};


pub const MTW: LanguageCode = LanguageCode {
    name: "Southern Binukidnon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mtw",
    individual_languages: &[
    ],
};


pub const MTX: LanguageCode = LanguageCode {
    name: "Tidaá Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mtx",
    individual_languages: &[
    ],
};


pub const MTY: LanguageCode = LanguageCode {
    name: "Nabi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mty",
    individual_languages: &[
    ],
};


pub const MUA: LanguageCode = LanguageCode {
    name: "Mundang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mua",
    individual_languages: &[
    ],
};


pub const MUB: LanguageCode = LanguageCode {
    name: "Mubi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mub",
    individual_languages: &[
    ],
};


pub const MUC: LanguageCode = LanguageCode {
    name: "Ajumbu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "muc",
    individual_languages: &[
    ],
};


pub const MUD: LanguageCode = LanguageCode {
    name: "Mednyj Aleut",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mud",
    individual_languages: &[
    ],
};


pub const MUE: LanguageCode = LanguageCode {
    name: "Media Lengua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mue",
    individual_languages: &[
    ],
};


pub const MUG: LanguageCode = LanguageCode {
    name: "Musgu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mug",
    individual_languages: &[
    ],
};


pub const MUH: LanguageCode = LanguageCode {
    name: "Mündü",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "muh",
    individual_languages: &[
    ],
};


pub const MUI: LanguageCode = LanguageCode {
    name: "Musi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mui",
    individual_languages: &[
    ],
};


pub const MUJ: LanguageCode = LanguageCode {
    name: "Mabire",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "muj",
    individual_languages: &[
    ],
};


pub const MUK: LanguageCode = LanguageCode {
    name: "Mugom",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "muk",
    individual_languages: &[
    ],
};


pub const MUM: LanguageCode = LanguageCode {
    name: "Maiwala",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mum",
    individual_languages: &[
    ],
};


pub const MUO: LanguageCode = LanguageCode {
    name: "Nyong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "muo",
    individual_languages: &[
    ],
};


pub const MUP: LanguageCode = LanguageCode {
    name: "Malvi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mup",
    individual_languages: &[
    ],
};


pub const MUQ: LanguageCode = LanguageCode {
    name: "Eastern Xiangxi Miao",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "muq",
    individual_languages: &[
    ],
};


pub const MUR: LanguageCode = LanguageCode {
    name: "Murle",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mur",
    individual_languages: &[
    ],
};


pub const MUT: LanguageCode = LanguageCode {
    name: "Western Muria",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mut",
    individual_languages: &[
    ],
};


pub const MUU: LanguageCode = LanguageCode {
    name: "Yaaku",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "muu",
    individual_languages: &[
    ],
};


pub const MUV: LanguageCode = LanguageCode {
    name: "Muthuvan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "muv",
    individual_languages: &[
    ],
};


pub const MUX: LanguageCode = LanguageCode {
    name: "Bo-Ung",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mux",
    individual_languages: &[
    ],
};


pub const MUY: LanguageCode = LanguageCode {
    name: "Muyang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "muy",
    individual_languages: &[
    ],
};


pub const MUZ: LanguageCode = LanguageCode {
    name: "Mursi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "muz",
    individual_languages: &[
    ],
};


pub const MVA: LanguageCode = LanguageCode {
    name: "Manam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mva",
    individual_languages: &[
    ],
};


pub const MVB: LanguageCode = LanguageCode {
    name: "Mattole",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mvb",
    individual_languages: &[
    ],
};


pub const MVD: LanguageCode = LanguageCode {
    name: "Mamboru",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mvd",
    individual_languages: &[
    ],
};


pub const MVE: LanguageCode = LanguageCode {
    name: "Marwari (Pakistan)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mve",
    individual_languages: &[
    ],
};


pub const MVF: LanguageCode = LanguageCode {
    name: "Peripheral Mongolian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mvf",
    individual_languages: &[
    ],
};


pub const MVG: LanguageCode = LanguageCode {
    name: "Yucuañe Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mvg",
    individual_languages: &[
    ],
};


pub const MVH: LanguageCode = LanguageCode {
    name: "Mulgi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mvh",
    individual_languages: &[
    ],
};


pub const MVI: LanguageCode = LanguageCode {
    name: "Miyako",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mvi",
    individual_languages: &[
    ],
};


pub const MVK: LanguageCode = LanguageCode {
    name: "Mekmek",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mvk",
    individual_languages: &[
    ],
};


pub const MVL: LanguageCode = LanguageCode {
    name: "Mbara (Australia)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mvl",
    individual_languages: &[
    ],
};


pub const MVN: LanguageCode = LanguageCode {
    name: "Minaveha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mvn",
    individual_languages: &[
    ],
};


pub const MVO: LanguageCode = LanguageCode {
    name: "Marovo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mvo",
    individual_languages: &[
    ],
};


pub const MVP: LanguageCode = LanguageCode {
    name: "Duri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mvp",
    individual_languages: &[
    ],
};


pub const MVQ: LanguageCode = LanguageCode {
    name: "Moere",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mvq",
    individual_languages: &[
    ],
};


pub const MVR: LanguageCode = LanguageCode {
    name: "Marau",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mvr",
    individual_languages: &[
    ],
};


pub const MVS: LanguageCode = LanguageCode {
    name: "Massep",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mvs",
    individual_languages: &[
    ],
};


pub const MVT: LanguageCode = LanguageCode {
    name: "Mpotovoro",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mvt",
    individual_languages: &[
    ],
};


pub const MVU: LanguageCode = LanguageCode {
    name: "Marfa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mvu",
    individual_languages: &[
    ],
};


pub const MVV: LanguageCode = LanguageCode {
    name: "Tagal Murut",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mvv",
    individual_languages: &[
    ],
};


pub const MVW: LanguageCode = LanguageCode {
    name: "Machinga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mvw",
    individual_languages: &[
    ],
};


pub const MVX: LanguageCode = LanguageCode {
    name: "Meoswar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mvx",
    individual_languages: &[
    ],
};


pub const MVY: LanguageCode = LanguageCode {
    name: "Indus Kohistani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mvy",
    individual_languages: &[
    ],
};


pub const MVZ: LanguageCode = LanguageCode {
    name: "Mesqan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mvz",
    individual_languages: &[
    ],
};


pub const MWA: LanguageCode = LanguageCode {
    name: "Mwatebu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mwa",
    individual_languages: &[
    ],
};


pub const MWB: LanguageCode = LanguageCode {
    name: "Juwal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mwb",
    individual_languages: &[
    ],
};


pub const MWC: LanguageCode = LanguageCode {
    name: "Are",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mwc",
    individual_languages: &[
    ],
};


pub const MWE: LanguageCode = LanguageCode {
    name: "Mwera (Chimwera)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mwe",
    individual_languages: &[
    ],
};


pub const MWF: LanguageCode = LanguageCode {
    name: "Murrinh-Patha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mwf",
    individual_languages: &[
    ],
};


pub const MWG: LanguageCode = LanguageCode {
    name: "Aiklep",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mwg",
    individual_languages: &[
    ],
};


pub const MWH: LanguageCode = LanguageCode {
    name: "Mouk-Aria",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mwh",
    individual_languages: &[
    ],
};


pub const MWI: LanguageCode = LanguageCode {
    name: "Labo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mwi",
    individual_languages: &[
    ],
};


pub const MWK: LanguageCode = LanguageCode {
    name: "Kita Maninkakan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mwk",
    individual_languages: &[
    ],
};


pub const MWM: LanguageCode = LanguageCode {
    name: "Sar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mwm",
    individual_languages: &[
    ],
};


pub const MWN: LanguageCode = LanguageCode {
    name: "Nyamwanga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mwn",
    individual_languages: &[
    ],
};


pub const MWO: LanguageCode = LanguageCode {
    name: "Central Maewo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mwo",
    individual_languages: &[
    ],
};


pub const MWP: LanguageCode = LanguageCode {
    name: "Kala Lagaw Ya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mwp",
    individual_languages: &[
    ],
};


pub const MWQ: LanguageCode = LanguageCode {
    name: "Mün Chin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mwq",
    individual_languages: &[
    ],
};


pub const MWS: LanguageCode = LanguageCode {
    name: "Mwimbi-Muthambi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mws",
    individual_languages: &[
    ],
};


pub const MWT: LanguageCode = LanguageCode {
    name: "Moken",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mwt",
    individual_languages: &[
    ],
};


pub const MWU: LanguageCode = LanguageCode {
    name: "Mittu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mwu",
    individual_languages: &[
    ],
};


pub const MWV: LanguageCode = LanguageCode {
    name: "Mentawai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mwv",
    individual_languages: &[
    ],
};


pub const MWW: LanguageCode = LanguageCode {
    name: "Hmong Daw",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mww",
    individual_languages: &[
    ],
};


pub const MWZ: LanguageCode = LanguageCode {
    name: "Moingi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mwz",
    individual_languages: &[
    ],
};


pub const MXA: LanguageCode = LanguageCode {
    name: "Northwest Oaxaca Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mxa",
    individual_languages: &[
    ],
};


pub const MXB: LanguageCode = LanguageCode {
    name: "Tezoatlán Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mxb",
    individual_languages: &[
    ],
};


pub const MXC: LanguageCode = LanguageCode {
    name: "Manyika",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mxc",
    individual_languages: &[
    ],
};


pub const MXD: LanguageCode = LanguageCode {
    name: "Modang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mxd",
    individual_languages: &[
    ],
};


pub const MXE: LanguageCode = LanguageCode {
    name: "Mele-Fila",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mxe",
    individual_languages: &[
    ],
};


pub const MXF: LanguageCode = LanguageCode {
    name: "Malgbe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mxf",
    individual_languages: &[
    ],
};


pub const MXG: LanguageCode = LanguageCode {
    name: "Mbangala",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mxg",
    individual_languages: &[
    ],
};


pub const MXH: LanguageCode = LanguageCode {
    name: "Mvuba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mxh",
    individual_languages: &[
    ],
};


pub const MXI: LanguageCode = LanguageCode {
    name: "Mozarabic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mxi",
    individual_languages: &[
    ],
};


pub const MXJ: LanguageCode = LanguageCode {
    name: "Miju-Mishmi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mxj",
    individual_languages: &[
    ],
};


pub const MXK: LanguageCode = LanguageCode {
    name: "Monumbo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mxk",
    individual_languages: &[
    ],
};


pub const MXL: LanguageCode = LanguageCode {
    name: "Maxi Gbe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mxl",
    individual_languages: &[
    ],
};


pub const MXM: LanguageCode = LanguageCode {
    name: "Meramera",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mxm",
    individual_languages: &[
    ],
};


pub const MXN: LanguageCode = LanguageCode {
    name: "Moi (Indonesia)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mxn",
    individual_languages: &[
    ],
};


pub const MXO: LanguageCode = LanguageCode {
    name: "Mbowe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mxo",
    individual_languages: &[
    ],
};


pub const MXP: LanguageCode = LanguageCode {
    name: "Tlahuitoltepec Mixe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mxp",
    individual_languages: &[
    ],
};


pub const MXQ: LanguageCode = LanguageCode {
    name: "Juquila Mixe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mxq",
    individual_languages: &[
    ],
};


pub const MXR: LanguageCode = LanguageCode {
    name: "Murik (Malaysia)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mxr",
    individual_languages: &[
    ],
};


pub const MXS: LanguageCode = LanguageCode {
    name: "Huitepec Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mxs",
    individual_languages: &[
    ],
};


pub const MXT: LanguageCode = LanguageCode {
    name: "Jamiltepec Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mxt",
    individual_languages: &[
    ],
};


pub const MXU: LanguageCode = LanguageCode {
    name: "Mada (Cameroon)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mxu",
    individual_languages: &[
    ],
};


pub const MXV: LanguageCode = LanguageCode {
    name: "Metlatónoc Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mxv",
    individual_languages: &[
    ],
};


pub const MXW: LanguageCode = LanguageCode {
    name: "Namo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mxw",
    individual_languages: &[
    ],
};


pub const MXX: LanguageCode = LanguageCode {
    name: "Mahou",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mxx",
    individual_languages: &[
    ],
};


pub const MXY: LanguageCode = LanguageCode {
    name: "Southeastern Nochixtlán Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mxy",
    individual_languages: &[
    ],
};


pub const MXZ: LanguageCode = LanguageCode {
    name: "Central Masela",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mxz",
    individual_languages: &[
    ],
};


pub const MYB: LanguageCode = LanguageCode {
    name: "Mbay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "myb",
    individual_languages: &[
    ],
};


pub const MYC: LanguageCode = LanguageCode {
    name: "Mayeka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "myc",
    individual_languages: &[
    ],
};


pub const MYE: LanguageCode = LanguageCode {
    name: "Myene",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mye",
    individual_languages: &[
    ],
};


pub const MYF: LanguageCode = LanguageCode {
    name: "Bambassi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "myf",
    individual_languages: &[
    ],
};


pub const MYG: LanguageCode = LanguageCode {
    name: "Manta",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "myg",
    individual_languages: &[
    ],
};


pub const MYH: LanguageCode = LanguageCode {
    name: "Makah",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "myh",
    individual_languages: &[
    ],
};


pub const MYJ: LanguageCode = LanguageCode {
    name: "Mangayat",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "myj",
    individual_languages: &[
    ],
};


pub const MYK: LanguageCode = LanguageCode {
    name: "Mamara Senoufo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "myk",
    individual_languages: &[
    ],
};


pub const MYL: LanguageCode = LanguageCode {
    name: "Moma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "myl",
    individual_languages: &[
    ],
};


pub const MYM: LanguageCode = LanguageCode {
    name: "Me'en",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mym",
    individual_languages: &[
    ],
};


pub const MYO: LanguageCode = LanguageCode {
    name: "Anfillo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "myo",
    individual_languages: &[
    ],
};


pub const MYP: LanguageCode = LanguageCode {
    name: "Pirahã",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "myp",
    individual_languages: &[
    ],
};


pub const MYR: LanguageCode = LanguageCode {
    name: "Muniche",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "myr",
    individual_languages: &[
    ],
};


pub const MYS: LanguageCode = LanguageCode {
    name: "Mesmes",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mys",
    individual_languages: &[
    ],
};


pub const MYU: LanguageCode = LanguageCode {
    name: "Mundurukú",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "myu",
    individual_languages: &[
    ],
};


pub const MYW: LanguageCode = LanguageCode {
    name: "Muyuw",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "myw",
    individual_languages: &[
    ],
};


pub const MYX: LanguageCode = LanguageCode {
    name: "Masaaba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "myx",
    individual_languages: &[
    ],
};


pub const MYY: LanguageCode = LanguageCode {
    name: "Macuna",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "myy",
    individual_languages: &[
    ],
};


pub const MYZ: LanguageCode = LanguageCode {
    name: "Classical Mandaic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "myz",
    individual_languages: &[
    ],
};


pub const MZA: LanguageCode = LanguageCode {
    name: "Santa María Zacatepec Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mza",
    individual_languages: &[
    ],
};


pub const MZB: LanguageCode = LanguageCode {
    name: "Tumzabt",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mzb",
    individual_languages: &[
    ],
};


pub const MZC: LanguageCode = LanguageCode {
    name: "Madagascar Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mzc",
    individual_languages: &[
    ],
};


pub const MZD: LanguageCode = LanguageCode {
    name: "Malimba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mzd",
    individual_languages: &[
    ],
};


pub const MZE: LanguageCode = LanguageCode {
    name: "Morawa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mze",
    individual_languages: &[
    ],
};


pub const MZG: LanguageCode = LanguageCode {
    name: "Monastic Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mzg",
    individual_languages: &[
    ],
};


pub const MZH: LanguageCode = LanguageCode {
    name: "Wichí Lhamtés Güisnay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mzh",
    individual_languages: &[
    ],
};


pub const MZI: LanguageCode = LanguageCode {
    name: "Ixcatlán Mazatec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mzi",
    individual_languages: &[
    ],
};


pub const MZJ: LanguageCode = LanguageCode {
    name: "Manya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mzj",
    individual_languages: &[
    ],
};


pub const MZK: LanguageCode = LanguageCode {
    name: "Nigeria Mambila",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mzk",
    individual_languages: &[
    ],
};


pub const MZL: LanguageCode = LanguageCode {
    name: "Mazatlán Mixe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mzl",
    individual_languages: &[
    ],
};


pub const MZM: LanguageCode = LanguageCode {
    name: "Mumuye",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mzm",
    individual_languages: &[
    ],
};


pub const MZN: LanguageCode = LanguageCode {
    name: "Mazanderani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mzn",
    individual_languages: &[
    ],
};


pub const MZO: LanguageCode = LanguageCode {
    name: "Matipuhy",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mzo",
    individual_languages: &[
    ],
};


pub const MZP: LanguageCode = LanguageCode {
    name: "Movima",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mzp",
    individual_languages: &[
    ],
};


pub const MZQ: LanguageCode = LanguageCode {
    name: "Mori Atas",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mzq",
    individual_languages: &[
    ],
};


pub const MZR: LanguageCode = LanguageCode {
    name: "Marúbo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mzr",
    individual_languages: &[
    ],
};


pub const MZS: LanguageCode = LanguageCode {
    name: "Macanese",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mzs",
    individual_languages: &[
    ],
};


pub const MZT: LanguageCode = LanguageCode {
    name: "Mintil",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mzt",
    individual_languages: &[
    ],
};


pub const MZU: LanguageCode = LanguageCode {
    name: "Inapang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mzu",
    individual_languages: &[
    ],
};


pub const MZV: LanguageCode = LanguageCode {
    name: "Manza",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mzv",
    individual_languages: &[
    ],
};


pub const MZW: LanguageCode = LanguageCode {
    name: "Deg",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mzw",
    individual_languages: &[
    ],
};


pub const MZX: LanguageCode = LanguageCode {
    name: "Mawayana",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mzx",
    individual_languages: &[
    ],
};


pub const MZY: LanguageCode = LanguageCode {
    name: "Mozambican Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mzy",
    individual_languages: &[
    ],
};


pub const MZZ: LanguageCode = LanguageCode {
    name: "Maiadomu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "mzz",
    individual_languages: &[
    ],
};


pub const NAA: LanguageCode = LanguageCode {
    name: "Namla",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "naa",
    individual_languages: &[
    ],
};


pub const NAB: LanguageCode = LanguageCode {
    name: "Southern Nambikuára",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nab",
    individual_languages: &[
    ],
};


pub const NAC: LanguageCode = LanguageCode {
    name: "Narak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nac",
    individual_languages: &[
    ],
};


pub const NAE: LanguageCode = LanguageCode {
    name: "Naka'ela",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nae",
    individual_languages: &[
    ],
};


pub const NAF: LanguageCode = LanguageCode {
    name: "Nabak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "naf",
    individual_languages: &[
    ],
};


pub const NAG: LanguageCode = LanguageCode {
    name: "Naga Pidgin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nag",
    individual_languages: &[
    ],
};


pub const NAJ: LanguageCode = LanguageCode {
    name: "Nalu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "naj",
    individual_languages: &[
    ],
};


pub const NAK: LanguageCode = LanguageCode {
    name: "Nakanai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nak",
    individual_languages: &[
    ],
};


pub const NAL: LanguageCode = LanguageCode {
    name: "Nalik",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nal",
    individual_languages: &[
    ],
};


pub const NAM: LanguageCode = LanguageCode {
    name: "Ngan'gityemerri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nam",
    individual_languages: &[
    ],
};


pub const NAN: LanguageCode = LanguageCode {
    name: "Min Nan Chinese",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nan",
    individual_languages: &[
    ],
};


pub const NAO: LanguageCode = LanguageCode {
    name: "Naaba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nao",
    individual_languages: &[
    ],
};


pub const NAQ: LanguageCode = LanguageCode {
    name: "Khoekhoe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "naq",
    individual_languages: &[
    ],
};


pub const NAR: LanguageCode = LanguageCode {
    name: "Iguta",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nar",
    individual_languages: &[
    ],
};


pub const NAS: LanguageCode = LanguageCode {
    name: "Naasioi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nas",
    individual_languages: &[
    ],
};


pub const NAT: LanguageCode = LanguageCode {
    name: "Ca̱hungwa̱rya̱",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nat",
    individual_languages: &[
    ],
};


pub const NAW: LanguageCode = LanguageCode {
    name: "Nawuri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "naw",
    individual_languages: &[
    ],
};


pub const NAX: LanguageCode = LanguageCode {
    name: "Nakwi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nax",
    individual_languages: &[
    ],
};


pub const NAY: LanguageCode = LanguageCode {
    name: "Ngarrindjeri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nay",
    individual_languages: &[
    ],
};


pub const NAZ: LanguageCode = LanguageCode {
    name: "Coatepec Nahuatl",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "naz",
    individual_languages: &[
    ],
};


pub const NBA: LanguageCode = LanguageCode {
    name: "Nyemba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nba",
    individual_languages: &[
    ],
};


pub const NBB: LanguageCode = LanguageCode {
    name: "Ndoe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nbb",
    individual_languages: &[
    ],
};


pub const NBC: LanguageCode = LanguageCode {
    name: "Chang Naga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nbc",
    individual_languages: &[
    ],
};


pub const NBD: LanguageCode = LanguageCode {
    name: "Ngbinda",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nbd",
    individual_languages: &[
    ],
};


pub const NBE: LanguageCode = LanguageCode {
    name: "Konyak Naga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nbe",
    individual_languages: &[
    ],
};


pub const NBG: LanguageCode = LanguageCode {
    name: "Nagarchal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nbg",
    individual_languages: &[
    ],
};


pub const NBH: LanguageCode = LanguageCode {
    name: "Ngamo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nbh",
    individual_languages: &[
    ],
};


pub const NBI: LanguageCode = LanguageCode {
    name: "Mao Naga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nbi",
    individual_languages: &[
    ],
};


pub const NBJ: LanguageCode = LanguageCode {
    name: "Ngarinyman",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nbj",
    individual_languages: &[
    ],
};


pub const NBK: LanguageCode = LanguageCode {
    name: "Nake",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nbk",
    individual_languages: &[
    ],
};


pub const NBM: LanguageCode = LanguageCode {
    name: "Ngbaka Ma'bo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nbm",
    individual_languages: &[
    ],
};


pub const NBN: LanguageCode = LanguageCode {
    name: "Kuri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nbn",
    individual_languages: &[
    ],
};


pub const NBO: LanguageCode = LanguageCode {
    name: "Nkukoli",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nbo",
    individual_languages: &[
    ],
};


pub const NBP: LanguageCode = LanguageCode {
    name: "Nnam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nbp",
    individual_languages: &[
    ],
};


pub const NBQ: LanguageCode = LanguageCode {
    name: "Nggem",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nbq",
    individual_languages: &[
    ],
};


pub const NBR: LanguageCode = LanguageCode {
    name: "Numana",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nbr",
    individual_languages: &[
    ],
};


pub const NBS: LanguageCode = LanguageCode {
    name: "Namibian Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nbs",
    individual_languages: &[
    ],
};


pub const NBT: LanguageCode = LanguageCode {
    name: "Na",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nbt",
    individual_languages: &[
    ],
};


pub const NBU: LanguageCode = LanguageCode {
    name: "Rongmei Naga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nbu",
    individual_languages: &[
    ],
};


pub const NBV: LanguageCode = LanguageCode {
    name: "Ngamambo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nbv",
    individual_languages: &[
    ],
};


pub const NBW: LanguageCode = LanguageCode {
    name: "Southern Ngbandi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nbw",
    individual_languages: &[
    ],
};


pub const NBY: LanguageCode = LanguageCode {
    name: "Ningera",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nby",
    individual_languages: &[
    ],
};


pub const NCA: LanguageCode = LanguageCode {
    name: "Iyo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nca",
    individual_languages: &[
    ],
};


pub const NCB: LanguageCode = LanguageCode {
    name: "Central Nicobarese",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ncb",
    individual_languages: &[
    ],
};


pub const NCC: LanguageCode = LanguageCode {
    name: "Ponam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ncc",
    individual_languages: &[
    ],
};


pub const NCD: LanguageCode = LanguageCode {
    name: "Nachering",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ncd",
    individual_languages: &[
    ],
};


pub const NCE: LanguageCode = LanguageCode {
    name: "Yale",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nce",
    individual_languages: &[
    ],
};


pub const NCF: LanguageCode = LanguageCode {
    name: "Notsi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ncf",
    individual_languages: &[
    ],
};


pub const NCG: LanguageCode = LanguageCode {
    name: "Nisga'a",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ncg",
    individual_languages: &[
    ],
};


pub const NCH: LanguageCode = LanguageCode {
    name: "Central Huasteca Nahuatl",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nch",
    individual_languages: &[
    ],
};


pub const NCI: LanguageCode = LanguageCode {
    name: "Classical Nahuatl",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nci",
    individual_languages: &[
    ],
};


pub const NCJ: LanguageCode = LanguageCode {
    name: "Northern Puebla Nahuatl",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ncj",
    individual_languages: &[
    ],
};


pub const NCK: LanguageCode = LanguageCode {
    name: "Na-kara",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nck",
    individual_languages: &[
    ],
};


pub const NCL: LanguageCode = LanguageCode {
    name: "Michoacán Nahuatl",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ncl",
    individual_languages: &[
    ],
};


pub const NCM: LanguageCode = LanguageCode {
    name: "Nambo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ncm",
    individual_languages: &[
    ],
};


pub const NCN: LanguageCode = LanguageCode {
    name: "Nauna",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ncn",
    individual_languages: &[
    ],
};


pub const NCO: LanguageCode = LanguageCode {
    name: "Sibe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nco",
    individual_languages: &[
    ],
};


pub const NCQ: LanguageCode = LanguageCode {
    name: "Northern Katang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ncq",
    individual_languages: &[
    ],
};


pub const NCR: LanguageCode = LanguageCode {
    name: "Ncane",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ncr",
    individual_languages: &[
    ],
};


pub const NCS: LanguageCode = LanguageCode {
    name: "Nicaraguan Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ncs",
    individual_languages: &[
    ],
};


pub const NCT: LanguageCode = LanguageCode {
    name: "Chothe Naga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nct",
    individual_languages: &[
    ],
};


pub const NCU: LanguageCode = LanguageCode {
    name: "Chumburung",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ncu",
    individual_languages: &[
    ],
};


pub const NCX: LanguageCode = LanguageCode {
    name: "Central Puebla Nahuatl",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ncx",
    individual_languages: &[
    ],
};


pub const NCZ: LanguageCode = LanguageCode {
    name: "Natchez",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ncz",
    individual_languages: &[
    ],
};


pub const NDA: LanguageCode = LanguageCode {
    name: "Ndasa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nda",
    individual_languages: &[
    ],
};


pub const NDB: LanguageCode = LanguageCode {
    name: "Kenswei Nsei",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ndb",
    individual_languages: &[
    ],
};


pub const NDC: LanguageCode = LanguageCode {
    name: "Ndau",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ndc",
    individual_languages: &[
    ],
};


pub const NDD: LanguageCode = LanguageCode {
    name: "Nde-Nsele-Nta",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ndd",
    individual_languages: &[
    ],
};


pub const NDF: LanguageCode = LanguageCode {
    name: "Nadruvian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ndf",
    individual_languages: &[
    ],
};


pub const NDG: LanguageCode = LanguageCode {
    name: "Ndengereko",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ndg",
    individual_languages: &[
    ],
};


pub const NDH: LanguageCode = LanguageCode {
    name: "Ndali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ndh",
    individual_languages: &[
    ],
};


pub const NDI: LanguageCode = LanguageCode {
    name: "Samba Leko",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ndi",
    individual_languages: &[
    ],
};


pub const NDJ: LanguageCode = LanguageCode {
    name: "Ndamba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ndj",
    individual_languages: &[
    ],
};


pub const NDK: LanguageCode = LanguageCode {
    name: "Ndaka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ndk",
    individual_languages: &[
    ],
};


pub const NDL: LanguageCode = LanguageCode {
    name: "Ndolo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ndl",
    individual_languages: &[
    ],
};


pub const NDM: LanguageCode = LanguageCode {
    name: "Ndam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ndm",
    individual_languages: &[
    ],
};


pub const NDN: LanguageCode = LanguageCode {
    name: "Ngundi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ndn",
    individual_languages: &[
    ],
};


pub const NDP: LanguageCode = LanguageCode {
    name: "Ndo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ndp",
    individual_languages: &[
    ],
};


pub const NDQ: LanguageCode = LanguageCode {
    name: "Ndombe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ndq",
    individual_languages: &[
    ],
};


pub const NDR: LanguageCode = LanguageCode {
    name: "Ndoola",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ndr",
    individual_languages: &[
    ],
};


pub const NDT: LanguageCode = LanguageCode {
    name: "Ndunga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ndt",
    individual_languages: &[
    ],
};


pub const NDU: LanguageCode = LanguageCode {
    name: "Dugun",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ndu",
    individual_languages: &[
    ],
};


pub const NDV: LanguageCode = LanguageCode {
    name: "Ndut",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ndv",
    individual_languages: &[
    ],
};


pub const NDW: LanguageCode = LanguageCode {
    name: "Ndobo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ndw",
    individual_languages: &[
    ],
};


pub const NDX: LanguageCode = LanguageCode {
    name: "Nduga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ndx",
    individual_languages: &[
    ],
};


pub const NDY: LanguageCode = LanguageCode {
    name: "Lutos",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ndy",
    individual_languages: &[
    ],
};


pub const NDZ: LanguageCode = LanguageCode {
    name: "Ndogo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ndz",
    individual_languages: &[
    ],
};


pub const NEA: LanguageCode = LanguageCode {
    name: "Eastern Ngad'a",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nea",
    individual_languages: &[
    ],
};


pub const NEB: LanguageCode = LanguageCode {
    name: "Toura (Côte d'Ivoire)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "neb",
    individual_languages: &[
    ],
};


pub const NEC: LanguageCode = LanguageCode {
    name: "Nedebang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nec",
    individual_languages: &[
    ],
};


pub const NED: LanguageCode = LanguageCode {
    name: "Nde-Gbite",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ned",
    individual_languages: &[
    ],
};


pub const NEE: LanguageCode = LanguageCode {
    name: "Nêlêmwa-Nixumwak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nee",
    individual_languages: &[
    ],
};


pub const NEF: LanguageCode = LanguageCode {
    name: "Nefamese",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nef",
    individual_languages: &[
    ],
};


pub const NEG: LanguageCode = LanguageCode {
    name: "Negidal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "neg",
    individual_languages: &[
    ],
};


pub const NEH: LanguageCode = LanguageCode {
    name: "Nyenkha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "neh",
    individual_languages: &[
    ],
};


pub const NEI: LanguageCode = LanguageCode {
    name: "Neo-Hittite",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nei",
    individual_languages: &[
    ],
};


pub const NEJ: LanguageCode = LanguageCode {
    name: "Neko",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nej",
    individual_languages: &[
    ],
};


pub const NEK: LanguageCode = LanguageCode {
    name: "Neku",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nek",
    individual_languages: &[
    ],
};


pub const NEM: LanguageCode = LanguageCode {
    name: "Nemi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nem",
    individual_languages: &[
    ],
};


pub const NEN: LanguageCode = LanguageCode {
    name: "Nengone",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nen",
    individual_languages: &[
    ],
};


pub const NEO: LanguageCode = LanguageCode {
    name: "Ná-Meo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "neo",
    individual_languages: &[
    ],
};


pub const NEQ: LanguageCode = LanguageCode {
    name: "North Central Mixe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "neq",
    individual_languages: &[
    ],
};


pub const NER: LanguageCode = LanguageCode {
    name: "Yahadian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ner",
    individual_languages: &[
    ],
};


pub const NES: LanguageCode = LanguageCode {
    name: "Bhoti Kinnauri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nes",
    individual_languages: &[
    ],
};


pub const NET: LanguageCode = LanguageCode {
    name: "Nete",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "net",
    individual_languages: &[
    ],
};


pub const NEU: LanguageCode = LanguageCode {
    name: "Neo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "neu",
    individual_languages: &[
    ],
};


pub const NEV: LanguageCode = LanguageCode {
    name: "Nyaheun",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nev",
    individual_languages: &[
    ],
};


pub const NEX: LanguageCode = LanguageCode {
    name: "Neme",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nex",
    individual_languages: &[
    ],
};


pub const NEY: LanguageCode = LanguageCode {
    name: "Neyo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ney",
    individual_languages: &[
    ],
};


pub const NEZ: LanguageCode = LanguageCode {
    name: "Nez Perce",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nez",
    individual_languages: &[
    ],
};


pub const NFA: LanguageCode = LanguageCode {
    name: "Dhao",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nfa",
    individual_languages: &[
    ],
};


pub const NFD: LanguageCode = LanguageCode {
    name: "Ahwai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nfd",
    individual_languages: &[
    ],
};


pub const NFL: LanguageCode = LanguageCode {
    name: "Ayiwo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nfl",
    individual_languages: &[
    ],
};


pub const NFR: LanguageCode = LanguageCode {
    name: "Nafaanra",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nfr",
    individual_languages: &[
    ],
};


pub const NFU: LanguageCode = LanguageCode {
    name: "Mfumte",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nfu",
    individual_languages: &[
    ],
};


pub const NGA: LanguageCode = LanguageCode {
    name: "Ngbaka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nga",
    individual_languages: &[
    ],
};


pub const NGB: LanguageCode = LanguageCode {
    name: "Northern Ngbandi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ngb",
    individual_languages: &[
    ],
};


pub const NGC: LanguageCode = LanguageCode {
    name: "Ngombe (Democratic Republic of Congo)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ngc",
    individual_languages: &[
    ],
};


pub const NGD: LanguageCode = LanguageCode {
    name: "Ngando (Central African Republic)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ngd",
    individual_languages: &[
    ],
};


pub const NGE: LanguageCode = LanguageCode {
    name: "Ngemba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nge",
    individual_languages: &[
    ],
};


pub const NGG: LanguageCode = LanguageCode {
    name: "Ngbaka Manza",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ngg",
    individual_languages: &[
    ],
};


pub const NGH: LanguageCode = LanguageCode {
    name: "Nǁng",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ngh",
    individual_languages: &[
    ],
};


pub const NGI: LanguageCode = LanguageCode {
    name: "Ngizim",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ngi",
    individual_languages: &[
    ],
};


pub const NGJ: LanguageCode = LanguageCode {
    name: "Ngie",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ngj",
    individual_languages: &[
    ],
};


pub const NGK: LanguageCode = LanguageCode {
    name: "Dalabon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ngk",
    individual_languages: &[
    ],
};


pub const NGL: LanguageCode = LanguageCode {
    name: "Lomwe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ngl",
    individual_languages: &[
    ],
};


pub const NGM: LanguageCode = LanguageCode {
    name: "Ngatik Men's Creole",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ngm",
    individual_languages: &[
    ],
};


pub const NGN: LanguageCode = LanguageCode {
    name: "Ngwo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ngn",
    individual_languages: &[
    ],
};


pub const NGP: LanguageCode = LanguageCode {
    name: "Ngulu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ngp",
    individual_languages: &[
    ],
};


pub const NGQ: LanguageCode = LanguageCode {
    name: "Ngurimi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ngq",
    individual_languages: &[
    ],
};


pub const NGR: LanguageCode = LanguageCode {
    name: "Engdewu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ngr",
    individual_languages: &[
    ],
};


pub const NGS: LanguageCode = LanguageCode {
    name: "Gvoko",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ngs",
    individual_languages: &[
    ],
};


pub const NGT: LanguageCode = LanguageCode {
    name: "Kriang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ngt",
    individual_languages: &[
    ],
};


pub const NGU: LanguageCode = LanguageCode {
    name: "Guerrero Nahuatl",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ngu",
    individual_languages: &[
    ],
};


pub const NGV: LanguageCode = LanguageCode {
    name: "Nagumi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ngv",
    individual_languages: &[
    ],
};


pub const NGW: LanguageCode = LanguageCode {
    name: "Ngwaba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ngw",
    individual_languages: &[
    ],
};


pub const NGX: LanguageCode = LanguageCode {
    name: "Nggwahyi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ngx",
    individual_languages: &[
    ],
};


pub const NGY: LanguageCode = LanguageCode {
    name: "Tibea",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ngy",
    individual_languages: &[
    ],
};


pub const NGZ: LanguageCode = LanguageCode {
    name: "Ngungwel",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ngz",
    individual_languages: &[
    ],
};


pub const NHA: LanguageCode = LanguageCode {
    name: "Nhanda",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nha",
    individual_languages: &[
    ],
};


pub const NHB: LanguageCode = LanguageCode {
    name: "Beng",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nhb",
    individual_languages: &[
    ],
};


pub const NHC: LanguageCode = LanguageCode {
    name: "Tabasco Nahuatl",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nhc",
    individual_languages: &[
    ],
};


pub const NHD: LanguageCode = LanguageCode {
    name: "Chiripá",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nhd",
    individual_languages: &[
    ],
};


pub const NHE: LanguageCode = LanguageCode {
    name: "Eastern Huasteca Nahuatl",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nhe",
    individual_languages: &[
    ],
};


pub const NHF: LanguageCode = LanguageCode {
    name: "Nhuwala",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nhf",
    individual_languages: &[
    ],
};


pub const NHG: LanguageCode = LanguageCode {
    name: "Tetelcingo Nahuatl",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nhg",
    individual_languages: &[
    ],
};


pub const NHH: LanguageCode = LanguageCode {
    name: "Nahari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nhh",
    individual_languages: &[
    ],
};


pub const NHI: LanguageCode = LanguageCode {
    name: "Zacatlán-Ahuacatlán-Tepetzintla Nahuatl",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nhi",
    individual_languages: &[
    ],
};


pub const NHK: LanguageCode = LanguageCode {
    name: "Isthmus-Cosoleacaque Nahuatl",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nhk",
    individual_languages: &[
    ],
};


pub const NHM: LanguageCode = LanguageCode {
    name: "Morelos Nahuatl",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nhm",
    individual_languages: &[
    ],
};


pub const NHN: LanguageCode = LanguageCode {
    name: "Central Nahuatl",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nhn",
    individual_languages: &[
    ],
};


pub const NHO: LanguageCode = LanguageCode {
    name: "Takuu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nho",
    individual_languages: &[
    ],
};


pub const NHP: LanguageCode = LanguageCode {
    name: "Isthmus-Pajapan Nahuatl",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nhp",
    individual_languages: &[
    ],
};


pub const NHQ: LanguageCode = LanguageCode {
    name: "Huaxcaleca Nahuatl",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nhq",
    individual_languages: &[
    ],
};


pub const NHR: LanguageCode = LanguageCode {
    name: "Naro",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nhr",
    individual_languages: &[
    ],
};


pub const NHT: LanguageCode = LanguageCode {
    name: "Ometepec Nahuatl",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nht",
    individual_languages: &[
    ],
};


pub const NHU: LanguageCode = LanguageCode {
    name: "Noone",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nhu",
    individual_languages: &[
    ],
};


pub const NHV: LanguageCode = LanguageCode {
    name: "Temascaltepec Nahuatl",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nhv",
    individual_languages: &[
    ],
};


pub const NHW: LanguageCode = LanguageCode {
    name: "Western Huasteca Nahuatl",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nhw",
    individual_languages: &[
    ],
};


pub const NHX: LanguageCode = LanguageCode {
    name: "Isthmus-Mecayapan Nahuatl",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nhx",
    individual_languages: &[
    ],
};


pub const NHY: LanguageCode = LanguageCode {
    name: "Northern Oaxaca Nahuatl",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nhy",
    individual_languages: &[
    ],
};


pub const NHZ: LanguageCode = LanguageCode {
    name: "Santa María La Alta Nahuatl",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nhz",
    individual_languages: &[
    ],
};


pub const NIB: LanguageCode = LanguageCode {
    name: "Nakame",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nib",
    individual_languages: &[
    ],
};


pub const NID: LanguageCode = LanguageCode {
    name: "Ngandi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nid",
    individual_languages: &[
    ],
};


pub const NIE: LanguageCode = LanguageCode {
    name: "Niellim",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nie",
    individual_languages: &[
    ],
};


pub const NIF: LanguageCode = LanguageCode {
    name: "Nek",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nif",
    individual_languages: &[
    ],
};


pub const NIG: LanguageCode = LanguageCode {
    name: "Ngalakgan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nig",
    individual_languages: &[
    ],
};


pub const NIH: LanguageCode = LanguageCode {
    name: "Nyiha (Tanzania)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nih",
    individual_languages: &[
    ],
};


pub const NII: LanguageCode = LanguageCode {
    name: "Nii",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nii",
    individual_languages: &[
    ],
};


pub const NIJ: LanguageCode = LanguageCode {
    name: "Ngaju",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nij",
    individual_languages: &[
    ],
};


pub const NIK: LanguageCode = LanguageCode {
    name: "Southern Nicobarese",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nik",
    individual_languages: &[
    ],
};


pub const NIL: LanguageCode = LanguageCode {
    name: "Nila",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nil",
    individual_languages: &[
    ],
};


pub const NIM: LanguageCode = LanguageCode {
    name: "Nilamba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nim",
    individual_languages: &[
    ],
};


pub const NIN: LanguageCode = LanguageCode {
    name: "Ninzo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nin",
    individual_languages: &[
    ],
};


pub const NIO: LanguageCode = LanguageCode {
    name: "Nganasan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nio",
    individual_languages: &[
    ],
};


pub const NIQ: LanguageCode = LanguageCode {
    name: "Nandi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "niq",
    individual_languages: &[
    ],
};


pub const NIR: LanguageCode = LanguageCode {
    name: "Nimboran",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nir",
    individual_languages: &[
    ],
};


pub const NIS: LanguageCode = LanguageCode {
    name: "Nimi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nis",
    individual_languages: &[
    ],
};


pub const NIT: LanguageCode = LanguageCode {
    name: "Southeastern Kolami",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nit",
    individual_languages: &[
    ],
};


pub const NIV: LanguageCode = LanguageCode {
    name: "Gilyak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "niv",
    individual_languages: &[
    ],
};


pub const NIW: LanguageCode = LanguageCode {
    name: "Nimo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "niw",
    individual_languages: &[
    ],
};


pub const NIX: LanguageCode = LanguageCode {
    name: "Hema",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nix",
    individual_languages: &[
    ],
};


pub const NIY: LanguageCode = LanguageCode {
    name: "Ngiti",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "niy",
    individual_languages: &[
    ],
};


pub const NIZ: LanguageCode = LanguageCode {
    name: "Ningil",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "niz",
    individual_languages: &[
    ],
};


pub const NJA: LanguageCode = LanguageCode {
    name: "Nzanyi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nja",
    individual_languages: &[
    ],
};


pub const NJB: LanguageCode = LanguageCode {
    name: "Nocte Naga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "njb",
    individual_languages: &[
    ],
};


pub const NJD: LanguageCode = LanguageCode {
    name: "Ndonde Hamba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "njd",
    individual_languages: &[
    ],
};


pub const NJH: LanguageCode = LanguageCode {
    name: "Lotha Naga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "njh",
    individual_languages: &[
    ],
};


pub const NJI: LanguageCode = LanguageCode {
    name: "Gudanji",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nji",
    individual_languages: &[
    ],
};


pub const NJJ: LanguageCode = LanguageCode {
    name: "Njen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "njj",
    individual_languages: &[
    ],
};


pub const NJL: LanguageCode = LanguageCode {
    name: "Njalgulgule",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "njl",
    individual_languages: &[
    ],
};


pub const NJM: LanguageCode = LanguageCode {
    name: "Angami Naga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "njm",
    individual_languages: &[
    ],
};


pub const NJN: LanguageCode = LanguageCode {
    name: "Liangmai Naga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "njn",
    individual_languages: &[
    ],
};


pub const NJO: LanguageCode = LanguageCode {
    name: "Ao Naga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "njo",
    individual_languages: &[
    ],
};


pub const NJR: LanguageCode = LanguageCode {
    name: "Njerep",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "njr",
    individual_languages: &[
    ],
};


pub const NJS: LanguageCode = LanguageCode {
    name: "Nisa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "njs",
    individual_languages: &[
    ],
};


pub const NJT: LanguageCode = LanguageCode {
    name: "Ndyuka-Trio Pidgin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "njt",
    individual_languages: &[
    ],
};


pub const NJU: LanguageCode = LanguageCode {
    name: "Ngadjunmaya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nju",
    individual_languages: &[
    ],
};


pub const NJX: LanguageCode = LanguageCode {
    name: "Kunyi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "njx",
    individual_languages: &[
    ],
};


pub const NJY: LanguageCode = LanguageCode {
    name: "Njyem",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "njy",
    individual_languages: &[
    ],
};


pub const NJZ: LanguageCode = LanguageCode {
    name: "Nyishi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "njz",
    individual_languages: &[
    ],
};


pub const NKA: LanguageCode = LanguageCode {
    name: "Nkoya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nka",
    individual_languages: &[
    ],
};


pub const NKB: LanguageCode = LanguageCode {
    name: "Khoibu Naga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nkb",
    individual_languages: &[
    ],
};


pub const NKC: LanguageCode = LanguageCode {
    name: "Nkongho",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nkc",
    individual_languages: &[
    ],
};


pub const NKD: LanguageCode = LanguageCode {
    name: "Koireng",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nkd",
    individual_languages: &[
    ],
};


pub const NKE: LanguageCode = LanguageCode {
    name: "Duke",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nke",
    individual_languages: &[
    ],
};


pub const NKF: LanguageCode = LanguageCode {
    name: "Inpui Naga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nkf",
    individual_languages: &[
    ],
};


pub const NKG: LanguageCode = LanguageCode {
    name: "Nekgini",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nkg",
    individual_languages: &[
    ],
};


pub const NKH: LanguageCode = LanguageCode {
    name: "Khezha Naga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nkh",
    individual_languages: &[
    ],
};


pub const NKI: LanguageCode = LanguageCode {
    name: "Thangal Naga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nki",
    individual_languages: &[
    ],
};


pub const NKJ: LanguageCode = LanguageCode {
    name: "Nakai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nkj",
    individual_languages: &[
    ],
};


pub const NKK: LanguageCode = LanguageCode {
    name: "Nokuku",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nkk",
    individual_languages: &[
    ],
};


pub const NKM: LanguageCode = LanguageCode {
    name: "Namat",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nkm",
    individual_languages: &[
    ],
};


pub const NKN: LanguageCode = LanguageCode {
    name: "Nkangala",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nkn",
    individual_languages: &[
    ],
};


pub const NKO: LanguageCode = LanguageCode {
    name: "Nkonya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nko",
    individual_languages: &[
    ],
};


pub const NKP: LanguageCode = LanguageCode {
    name: "Niuatoputapu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nkp",
    individual_languages: &[
    ],
};


pub const NKQ: LanguageCode = LanguageCode {
    name: "Nkami",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nkq",
    individual_languages: &[
    ],
};


pub const NKR: LanguageCode = LanguageCode {
    name: "Nukuoro",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nkr",
    individual_languages: &[
    ],
};


pub const NKS: LanguageCode = LanguageCode {
    name: "North Asmat",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nks",
    individual_languages: &[
    ],
};


pub const NKT: LanguageCode = LanguageCode {
    name: "Nyika (Tanzania)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nkt",
    individual_languages: &[
    ],
};


pub const NKU: LanguageCode = LanguageCode {
    name: "Bouna Kulango",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nku",
    individual_languages: &[
    ],
};


pub const NKV: LanguageCode = LanguageCode {
    name: "Nyika (Malawi and Zambia)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nkv",
    individual_languages: &[
    ],
};


pub const NKW: LanguageCode = LanguageCode {
    name: "Nkutu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nkw",
    individual_languages: &[
    ],
};


pub const NKX: LanguageCode = LanguageCode {
    name: "Nkoroo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nkx",
    individual_languages: &[
    ],
};


pub const NKZ: LanguageCode = LanguageCode {
    name: "Nkari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nkz",
    individual_languages: &[
    ],
};


pub const NLA: LanguageCode = LanguageCode {
    name: "Ngombale",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nla",
    individual_languages: &[
    ],
};


pub const NLC: LanguageCode = LanguageCode {
    name: "Nalca",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nlc",
    individual_languages: &[
    ],
};


pub const NLE: LanguageCode = LanguageCode {
    name: "East Nyala",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nle",
    individual_languages: &[
    ],
};


pub const NLG: LanguageCode = LanguageCode {
    name: "Gela",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nlg",
    individual_languages: &[
    ],
};


pub const NLI: LanguageCode = LanguageCode {
    name: "Grangali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nli",
    individual_languages: &[
    ],
};


pub const NLJ: LanguageCode = LanguageCode {
    name: "Nyali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nlj",
    individual_languages: &[
    ],
};


pub const NLK: LanguageCode = LanguageCode {
    name: "Ninia Yali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nlk",
    individual_languages: &[
    ],
};


pub const NLL: LanguageCode = LanguageCode {
    name: "Nihali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nll",
    individual_languages: &[
    ],
};


pub const NLM: LanguageCode = LanguageCode {
    name: "Mankiyali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nlm",
    individual_languages: &[
    ],
};


pub const NLO: LanguageCode = LanguageCode {
    name: "Ngul",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nlo",
    individual_languages: &[
    ],
};


pub const NLQ: LanguageCode = LanguageCode {
    name: "Lao Naga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nlq",
    individual_languages: &[
    ],
};


pub const NLU: LanguageCode = LanguageCode {
    name: "Nchumbulu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nlu",
    individual_languages: &[
    ],
};


pub const NLV: LanguageCode = LanguageCode {
    name: "Orizaba Nahuatl",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nlv",
    individual_languages: &[
    ],
};


pub const NLW: LanguageCode = LanguageCode {
    name: "Walangama",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nlw",
    individual_languages: &[
    ],
};


pub const NLX: LanguageCode = LanguageCode {
    name: "Nahali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nlx",
    individual_languages: &[
    ],
};


pub const NLY: LanguageCode = LanguageCode {
    name: "Nyamal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nly",
    individual_languages: &[
    ],
};


pub const NLZ: LanguageCode = LanguageCode {
    name: "Nalögo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nlz",
    individual_languages: &[
    ],
};


pub const NMA: LanguageCode = LanguageCode {
    name: "Maram Naga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nma",
    individual_languages: &[
    ],
};


pub const NMB: LanguageCode = LanguageCode {
    name: "Big Nambas",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nmb",
    individual_languages: &[
    ],
};


pub const NMC: LanguageCode = LanguageCode {
    name: "Ngam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nmc",
    individual_languages: &[
    ],
};


pub const NMD: LanguageCode = LanguageCode {
    name: "Ndumu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nmd",
    individual_languages: &[
    ],
};


pub const NME: LanguageCode = LanguageCode {
    name: "Mzieme Naga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nme",
    individual_languages: &[
    ],
};


pub const NMF: LanguageCode = LanguageCode {
    name: "Tangkhul Naga (India)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nmf",
    individual_languages: &[
    ],
};


pub const NMG: LanguageCode = LanguageCode {
    name: "Kwasio",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nmg",
    individual_languages: &[
    ],
};


pub const NMH: LanguageCode = LanguageCode {
    name: "Monsang Naga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nmh",
    individual_languages: &[
    ],
};


pub const NMI: LanguageCode = LanguageCode {
    name: "Nyam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nmi",
    individual_languages: &[
    ],
};


pub const NMJ: LanguageCode = LanguageCode {
    name: "Ngombe (Central African Republic)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nmj",
    individual_languages: &[
    ],
};


pub const NMK: LanguageCode = LanguageCode {
    name: "Namakura",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nmk",
    individual_languages: &[
    ],
};


pub const NML: LanguageCode = LanguageCode {
    name: "Ndemli",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nml",
    individual_languages: &[
    ],
};


pub const NMM: LanguageCode = LanguageCode {
    name: "Manangba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nmm",
    individual_languages: &[
    ],
};


pub const NMN: LanguageCode = LanguageCode {
    name: "ǃXóõ",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nmn",
    individual_languages: &[
    ],
};


pub const NMO: LanguageCode = LanguageCode {
    name: "Moyon Naga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nmo",
    individual_languages: &[
    ],
};


pub const NMP: LanguageCode = LanguageCode {
    name: "Nimanbur",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nmp",
    individual_languages: &[
    ],
};


pub const NMQ: LanguageCode = LanguageCode {
    name: "Nambya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nmq",
    individual_languages: &[
    ],
};


pub const NMR: LanguageCode = LanguageCode {
    name: "Nimbari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nmr",
    individual_languages: &[
    ],
};


pub const NMS: LanguageCode = LanguageCode {
    name: "Letemboi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nms",
    individual_languages: &[
    ],
};


pub const NMT: LanguageCode = LanguageCode {
    name: "Namonuito",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nmt",
    individual_languages: &[
    ],
};


pub const NMU: LanguageCode = LanguageCode {
    name: "Northeast Maidu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nmu",
    individual_languages: &[
    ],
};


pub const NMV: LanguageCode = LanguageCode {
    name: "Ngamini",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nmv",
    individual_languages: &[
    ],
};


pub const NMW: LanguageCode = LanguageCode {
    name: "Nimoa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nmw",
    individual_languages: &[
    ],
};


pub const NMX: LanguageCode = LanguageCode {
    name: "Nama (Papua New Guinea)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nmx",
    individual_languages: &[
    ],
};


pub const NMY: LanguageCode = LanguageCode {
    name: "Namuyi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nmy",
    individual_languages: &[
    ],
};


pub const NMZ: LanguageCode = LanguageCode {
    name: "Nawdm",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nmz",
    individual_languages: &[
    ],
};


pub const NNA: LanguageCode = LanguageCode {
    name: "Nyangumarta",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nna",
    individual_languages: &[
    ],
};


pub const NNB: LanguageCode = LanguageCode {
    name: "Nande",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nnb",
    individual_languages: &[
    ],
};


pub const NNC: LanguageCode = LanguageCode {
    name: "Nancere",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nnc",
    individual_languages: &[
    ],
};


pub const NND: LanguageCode = LanguageCode {
    name: "West Ambae",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nnd",
    individual_languages: &[
    ],
};


pub const NNE: LanguageCode = LanguageCode {
    name: "Ngandyera",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nne",
    individual_languages: &[
    ],
};


pub const NNF: LanguageCode = LanguageCode {
    name: "Ngaing",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nnf",
    individual_languages: &[
    ],
};


pub const NNG: LanguageCode = LanguageCode {
    name: "Maring Naga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nng",
    individual_languages: &[
    ],
};


pub const NNH: LanguageCode = LanguageCode {
    name: "Ngiemboon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nnh",
    individual_languages: &[
    ],
};


pub const NNI: LanguageCode = LanguageCode {
    name: "North Nuaulu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nni",
    individual_languages: &[
    ],
};


pub const NNJ: LanguageCode = LanguageCode {
    name: "Nyangatom",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nnj",
    individual_languages: &[
    ],
};


pub const NNK: LanguageCode = LanguageCode {
    name: "Nankina",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nnk",
    individual_languages: &[
    ],
};


pub const NNL: LanguageCode = LanguageCode {
    name: "Northern Rengma Naga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nnl",
    individual_languages: &[
    ],
};


pub const NNM: LanguageCode = LanguageCode {
    name: "Namia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nnm",
    individual_languages: &[
    ],
};


pub const NNN: LanguageCode = LanguageCode {
    name: "Ngete",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nnn",
    individual_languages: &[
    ],
};


pub const NNP: LanguageCode = LanguageCode {
    name: "Wancho Naga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nnp",
    individual_languages: &[
    ],
};


pub const NNQ: LanguageCode = LanguageCode {
    name: "Ngindo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nnq",
    individual_languages: &[
    ],
};


pub const NNR: LanguageCode = LanguageCode {
    name: "Narungga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nnr",
    individual_languages: &[
    ],
};


pub const NNT: LanguageCode = LanguageCode {
    name: "Nanticoke",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nnt",
    individual_languages: &[
    ],
};


pub const NNU: LanguageCode = LanguageCode {
    name: "Dwang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nnu",
    individual_languages: &[
    ],
};


pub const NNV: LanguageCode = LanguageCode {
    name: "Nugunu (Australia)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nnv",
    individual_languages: &[
    ],
};


pub const NNW: LanguageCode = LanguageCode {
    name: "Southern Nuni",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nnw",
    individual_languages: &[
    ],
};


pub const NNY: LanguageCode = LanguageCode {
    name: "Nyangga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nny",
    individual_languages: &[
    ],
};


pub const NNZ: LanguageCode = LanguageCode {
    name: "Nda'nda'",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nnz",
    individual_languages: &[
    ],
};


pub const NOA: LanguageCode = LanguageCode {
    name: "Woun Meu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "noa",
    individual_languages: &[
    ],
};


pub const NOC: LanguageCode = LanguageCode {
    name: "Nuk",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "noc",
    individual_languages: &[
    ],
};


pub const NOD: LanguageCode = LanguageCode {
    name: "Northern Thai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nod",
    individual_languages: &[
    ],
};


pub const NOE: LanguageCode = LanguageCode {
    name: "Nimadi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "noe",
    individual_languages: &[
    ],
};


pub const NOF: LanguageCode = LanguageCode {
    name: "Nomane",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nof",
    individual_languages: &[
    ],
};


pub const NOH: LanguageCode = LanguageCode {
    name: "Nomu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "noh",
    individual_languages: &[
    ],
};


pub const NOI: LanguageCode = LanguageCode {
    name: "Noiri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "noi",
    individual_languages: &[
    ],
};


pub const NOJ: LanguageCode = LanguageCode {
    name: "Nonuya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "noj",
    individual_languages: &[
    ],
};


pub const NOK: LanguageCode = LanguageCode {
    name: "Nooksack",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nok",
    individual_languages: &[
    ],
};


pub const NOL: LanguageCode = LanguageCode {
    name: "Nomlaki",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nol",
    individual_languages: &[
    ],
};


pub const NOM: LanguageCode = LanguageCode {
    name: "Nocamán",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nom",
    individual_languages: &[
    ],
};


pub const NOP: LanguageCode = LanguageCode {
    name: "Numanggang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nop",
    individual_languages: &[
    ],
};


pub const NOQ: LanguageCode = LanguageCode {
    name: "Ngongo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "noq",
    individual_languages: &[
    ],
};


pub const NOS: LanguageCode = LanguageCode {
    name: "Eastern Nisu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nos",
    individual_languages: &[
    ],
};


pub const NOT: LanguageCode = LanguageCode {
    name: "Nomatsiguenga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "not",
    individual_languages: &[
    ],
};


pub const NOU: LanguageCode = LanguageCode {
    name: "Ewage-Notu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nou",
    individual_languages: &[
    ],
};


pub const NOV: LanguageCode = LanguageCode {
    name: "Novial",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nov",
    individual_languages: &[
    ],
};


pub const NOW: LanguageCode = LanguageCode {
    name: "Nyambo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "now",
    individual_languages: &[
    ],
};


pub const NOY: LanguageCode = LanguageCode {
    name: "Noy",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "noy",
    individual_languages: &[
    ],
};


pub const NOZ: LanguageCode = LanguageCode {
    name: "Nayi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "noz",
    individual_languages: &[
    ],
};


pub const NPA: LanguageCode = LanguageCode {
    name: "Nar Phu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "npa",
    individual_languages: &[
    ],
};


pub const NPB: LanguageCode = LanguageCode {
    name: "Nupbikha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "npb",
    individual_languages: &[
    ],
};


pub const NPG: LanguageCode = LanguageCode {
    name: "Ponyo-Gongwang Naga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "npg",
    individual_languages: &[
    ],
};


pub const NPH: LanguageCode = LanguageCode {
    name: "Phom Naga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nph",
    individual_languages: &[
    ],
};


pub const NPI: LanguageCode = LanguageCode {
    name: "Nepali (individual language)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "npi",
    individual_languages: &[
    ],
};


pub const NPL: LanguageCode = LanguageCode {
    name: "Southeastern Puebla Nahuatl",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "npl",
    individual_languages: &[
    ],
};


pub const NPN: LanguageCode = LanguageCode {
    name: "Mondropolon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "npn",
    individual_languages: &[
    ],
};


pub const NPO: LanguageCode = LanguageCode {
    name: "Pochuri Naga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "npo",
    individual_languages: &[
    ],
};


pub const NPS: LanguageCode = LanguageCode {
    name: "Nipsan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nps",
    individual_languages: &[
    ],
};


pub const NPU: LanguageCode = LanguageCode {
    name: "Puimei Naga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "npu",
    individual_languages: &[
    ],
};


pub const NPX: LanguageCode = LanguageCode {
    name: "Noipx",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "npx",
    individual_languages: &[
    ],
};


pub const NPY: LanguageCode = LanguageCode {
    name: "Napu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "npy",
    individual_languages: &[
    ],
};


pub const NQG: LanguageCode = LanguageCode {
    name: "Southern Nago",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nqg",
    individual_languages: &[
    ],
};


pub const NQK: LanguageCode = LanguageCode {
    name: "Kura Ede Nago",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nqk",
    individual_languages: &[
    ],
};


pub const NQL: LanguageCode = LanguageCode {
    name: "Ngendelengo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nql",
    individual_languages: &[
    ],
};


pub const NQM: LanguageCode = LanguageCode {
    name: "Ndom",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nqm",
    individual_languages: &[
    ],
};


pub const NQN: LanguageCode = LanguageCode {
    name: "Nen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nqn",
    individual_languages: &[
    ],
};


pub const NQQ: LanguageCode = LanguageCode {
    name: "Kyan-Karyaw Naga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nqq",
    individual_languages: &[
    ],
};


pub const NQT: LanguageCode = LanguageCode {
    name: "Nteng",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nqt",
    individual_languages: &[
    ],
};


pub const NQY: LanguageCode = LanguageCode {
    name: "Akyaung Ari Naga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nqy",
    individual_languages: &[
    ],
};


pub const NRA: LanguageCode = LanguageCode {
    name: "Ngom",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nra",
    individual_languages: &[
    ],
};


pub const NRB: LanguageCode = LanguageCode {
    name: "Nara",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nrb",
    individual_languages: &[
    ],
};


pub const NRC: LanguageCode = LanguageCode {
    name: "Noric",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nrc",
    individual_languages: &[
    ],
};


pub const NRE: LanguageCode = LanguageCode {
    name: "Southern Rengma Naga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nre",
    individual_languages: &[
    ],
};


pub const NRF: LanguageCode = LanguageCode {
    name: "Jèrriais",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nrf",
    individual_languages: &[
    ],
};


pub const NRG: LanguageCode = LanguageCode {
    name: "Narango",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nrg",
    individual_languages: &[
    ],
};


pub const NRI: LanguageCode = LanguageCode {
    name: "Chokri Naga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nri",
    individual_languages: &[
    ],
};


pub const NRK: LanguageCode = LanguageCode {
    name: "Ngarla",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nrk",
    individual_languages: &[
    ],
};


pub const NRL: LanguageCode = LanguageCode {
    name: "Ngarluma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nrl",
    individual_languages: &[
    ],
};


pub const NRM: LanguageCode = LanguageCode {
    name: "Narom",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nrm",
    individual_languages: &[
    ],
};


pub const NRN: LanguageCode = LanguageCode {
    name: "Norn",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nrn",
    individual_languages: &[
    ],
};


pub const NRP: LanguageCode = LanguageCode {
    name: "North Picene",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nrp",
    individual_languages: &[
    ],
};


pub const NRR: LanguageCode = LanguageCode {
    name: "Norra",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nrr",
    individual_languages: &[
    ],
};


pub const NRT: LanguageCode = LanguageCode {
    name: "Northern Kalapuya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nrt",
    individual_languages: &[
    ],
};


pub const NRU: LanguageCode = LanguageCode {
    name: "Narua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nru",
    individual_languages: &[
    ],
};


pub const NRX: LanguageCode = LanguageCode {
    name: "Ngurmbur",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nrx",
    individual_languages: &[
    ],
};


pub const NRZ: LanguageCode = LanguageCode {
    name: "Lala",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nrz",
    individual_languages: &[
    ],
};


pub const NSA: LanguageCode = LanguageCode {
    name: "Sangtam Naga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nsa",
    individual_languages: &[
    ],
};


pub const NSB: LanguageCode = LanguageCode {
    name: "Lower Nossob",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nsb",
    individual_languages: &[
    ],
};


pub const NSC: LanguageCode = LanguageCode {
    name: "Nshi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nsc",
    individual_languages: &[
    ],
};


pub const NSD: LanguageCode = LanguageCode {
    name: "Southern Nisu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nsd",
    individual_languages: &[
    ],
};


pub const NSE: LanguageCode = LanguageCode {
    name: "Nsenga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nse",
    individual_languages: &[
    ],
};


pub const NSF: LanguageCode = LanguageCode {
    name: "Northwestern Nisu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nsf",
    individual_languages: &[
    ],
};


pub const NSG: LanguageCode = LanguageCode {
    name: "Ngasa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nsg",
    individual_languages: &[
    ],
};


pub const NSH: LanguageCode = LanguageCode {
    name: "Ngoshie",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nsh",
    individual_languages: &[
    ],
};


pub const NSI: LanguageCode = LanguageCode {
    name: "Nigerian Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nsi",
    individual_languages: &[
    ],
};


pub const NSK: LanguageCode = LanguageCode {
    name: "Naskapi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nsk",
    individual_languages: &[
    ],
};


pub const NSL: LanguageCode = LanguageCode {
    name: "Norwegian Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nsl",
    individual_languages: &[
    ],
};


pub const NSM: LanguageCode = LanguageCode {
    name: "Sumi Naga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nsm",
    individual_languages: &[
    ],
};


pub const NSN: LanguageCode = LanguageCode {
    name: "Nehan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nsn",
    individual_languages: &[
    ],
};


pub const NSP: LanguageCode = LanguageCode {
    name: "Nepalese Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nsp",
    individual_languages: &[
    ],
};


pub const NSQ: LanguageCode = LanguageCode {
    name: "Northern Sierra Miwok",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nsq",
    individual_languages: &[
    ],
};


pub const NSR: LanguageCode = LanguageCode {
    name: "Maritime Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nsr",
    individual_languages: &[
    ],
};


pub const NSS: LanguageCode = LanguageCode {
    name: "Nali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nss",
    individual_languages: &[
    ],
};


pub const NST: LanguageCode = LanguageCode {
    name: "Tase Naga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nst",
    individual_languages: &[
    ],
};


pub const NSU: LanguageCode = LanguageCode {
    name: "Sierra Negra Nahuatl",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nsu",
    individual_languages: &[
    ],
};


pub const NSV: LanguageCode = LanguageCode {
    name: "Southwestern Nisu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nsv",
    individual_languages: &[
    ],
};


pub const NSW: LanguageCode = LanguageCode {
    name: "Navut",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nsw",
    individual_languages: &[
    ],
};


pub const NSX: LanguageCode = LanguageCode {
    name: "Nsongo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nsx",
    individual_languages: &[
    ],
};


pub const NSY: LanguageCode = LanguageCode {
    name: "Nasal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nsy",
    individual_languages: &[
    ],
};


pub const NSZ: LanguageCode = LanguageCode {
    name: "Nisenan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nsz",
    individual_languages: &[
    ],
};


pub const NTD: LanguageCode = LanguageCode {
    name: "Northern Tidung",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ntd",
    individual_languages: &[
    ],
};


pub const NTE: LanguageCode = LanguageCode {
    name: "Nathembo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nte",
    individual_languages: &[
    ],
};


pub const NTG: LanguageCode = LanguageCode {
    name: "Ngantangarra",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ntg",
    individual_languages: &[
    ],
};


pub const NTI: LanguageCode = LanguageCode {
    name: "Natioro",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nti",
    individual_languages: &[
    ],
};


pub const NTJ: LanguageCode = LanguageCode {
    name: "Ngaanyatjarra",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ntj",
    individual_languages: &[
    ],
};


pub const NTK: LanguageCode = LanguageCode {
    name: "Ikoma-Nata-Isenye",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ntk",
    individual_languages: &[
    ],
};


pub const NTM: LanguageCode = LanguageCode {
    name: "Nateni",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ntm",
    individual_languages: &[
    ],
};


pub const NTO: LanguageCode = LanguageCode {
    name: "Ntomba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nto",
    individual_languages: &[
    ],
};


pub const NTP: LanguageCode = LanguageCode {
    name: "Northern Tepehuan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ntp",
    individual_languages: &[
    ],
};


pub const NTR: LanguageCode = LanguageCode {
    name: "Delo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ntr",
    individual_languages: &[
    ],
};


pub const NTU: LanguageCode = LanguageCode {
    name: "Natügu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ntu",
    individual_languages: &[
    ],
};


pub const NTW: LanguageCode = LanguageCode {
    name: "Nottoway",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ntw",
    individual_languages: &[
    ],
};


pub const NTX: LanguageCode = LanguageCode {
    name: "Tangkhul Naga (Myanmar)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ntx",
    individual_languages: &[
    ],
};


pub const NTY: LanguageCode = LanguageCode {
    name: "Mantsi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nty",
    individual_languages: &[
    ],
};


pub const NTZ: LanguageCode = LanguageCode {
    name: "Natanzi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ntz",
    individual_languages: &[
    ],
};


pub const NUA: LanguageCode = LanguageCode {
    name: "Yuanga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nua",
    individual_languages: &[
    ],
};


pub const NUC: LanguageCode = LanguageCode {
    name: "Nukuini",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nuc",
    individual_languages: &[
    ],
};


pub const NUD: LanguageCode = LanguageCode {
    name: "Ngala",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nud",
    individual_languages: &[
    ],
};


pub const NUE: LanguageCode = LanguageCode {
    name: "Ngundu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nue",
    individual_languages: &[
    ],
};


pub const NUF: LanguageCode = LanguageCode {
    name: "Nusu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nuf",
    individual_languages: &[
    ],
};


pub const NUG: LanguageCode = LanguageCode {
    name: "Nungali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nug",
    individual_languages: &[
    ],
};


pub const NUH: LanguageCode = LanguageCode {
    name: "Ndunda",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nuh",
    individual_languages: &[
    ],
};


pub const NUI: LanguageCode = LanguageCode {
    name: "Ngumbi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nui",
    individual_languages: &[
    ],
};


pub const NUJ: LanguageCode = LanguageCode {
    name: "Nyole",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nuj",
    individual_languages: &[
    ],
};


pub const NUK: LanguageCode = LanguageCode {
    name: "Nuu-chah-nulth",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nuk",
    individual_languages: &[
    ],
};


pub const NUL: LanguageCode = LanguageCode {
    name: "Nusa Laut",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nul",
    individual_languages: &[
    ],
};


pub const NUM: LanguageCode = LanguageCode {
    name: "Niuafo'ou",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "num",
    individual_languages: &[
    ],
};


pub const NUN: LanguageCode = LanguageCode {
    name: "Anong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nun",
    individual_languages: &[
    ],
};


pub const NUO: LanguageCode = LanguageCode {
    name: "Nguôn",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nuo",
    individual_languages: &[
    ],
};


pub const NUP: LanguageCode = LanguageCode {
    name: "Nupe-Nupe-Tako",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nup",
    individual_languages: &[
    ],
};


pub const NUQ: LanguageCode = LanguageCode {
    name: "Nukumanu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nuq",
    individual_languages: &[
    ],
};


pub const NUR: LanguageCode = LanguageCode {
    name: "Nukuria",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nur",
    individual_languages: &[
    ],
};


pub const NUS: LanguageCode = LanguageCode {
    name: "Nuer",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nus",
    individual_languages: &[
    ],
};


pub const NUT: LanguageCode = LanguageCode {
    name: "Nung (Viet Nam)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nut",
    individual_languages: &[
    ],
};


pub const NUU: LanguageCode = LanguageCode {
    name: "Ngbundu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nuu",
    individual_languages: &[
    ],
};


pub const NUV: LanguageCode = LanguageCode {
    name: "Northern Nuni",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nuv",
    individual_languages: &[
    ],
};


pub const NUW: LanguageCode = LanguageCode {
    name: "Nguluwan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nuw",
    individual_languages: &[
    ],
};


pub const NUX: LanguageCode = LanguageCode {
    name: "Mehek",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nux",
    individual_languages: &[
    ],
};


pub const NUY: LanguageCode = LanguageCode {
    name: "Nunggubuyu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nuy",
    individual_languages: &[
    ],
};


pub const NUZ: LanguageCode = LanguageCode {
    name: "Tlamacazapa Nahuatl",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nuz",
    individual_languages: &[
    ],
};


pub const NVH: LanguageCode = LanguageCode {
    name: "Nasarian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nvh",
    individual_languages: &[
    ],
};


pub const NVM: LanguageCode = LanguageCode {
    name: "Namiae",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nvm",
    individual_languages: &[
    ],
};


pub const NVO: LanguageCode = LanguageCode {
    name: "Nyokon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nvo",
    individual_languages: &[
    ],
};


pub const NWA: LanguageCode = LanguageCode {
    name: "Nawathinehena",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nwa",
    individual_languages: &[
    ],
};


pub const NWB: LanguageCode = LanguageCode {
    name: "Nyabwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nwb",
    individual_languages: &[
    ],
};


pub const NWE: LanguageCode = LanguageCode {
    name: "Ngwe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nwe",
    individual_languages: &[
    ],
};


pub const NWG: LanguageCode = LanguageCode {
    name: "Ngayawung",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nwg",
    individual_languages: &[
    ],
};


pub const NWI: LanguageCode = LanguageCode {
    name: "Southwest Tanna",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nwi",
    individual_languages: &[
    ],
};


pub const NWM: LanguageCode = LanguageCode {
    name: "Nyamusa-Molo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nwm",
    individual_languages: &[
    ],
};


pub const NWO: LanguageCode = LanguageCode {
    name: "Nauo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nwo",
    individual_languages: &[
    ],
};


pub const NWR: LanguageCode = LanguageCode {
    name: "Nawaru",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nwr",
    individual_languages: &[
    ],
};


pub const NWW: LanguageCode = LanguageCode {
    name: "Ndwewe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nww",
    individual_languages: &[
    ],
};


pub const NWX: LanguageCode = LanguageCode {
    name: "Middle Newar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nwx",
    individual_languages: &[
    ],
};


pub const NWY: LanguageCode = LanguageCode {
    name: "Nottoway-Meherrin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nwy",
    individual_languages: &[
    ],
};


pub const NXA: LanguageCode = LanguageCode {
    name: "Nauete",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nxa",
    individual_languages: &[
    ],
};


pub const NXD: LanguageCode = LanguageCode {
    name: "Ngando (Democratic Republic of Congo)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nxd",
    individual_languages: &[
    ],
};


pub const NXE: LanguageCode = LanguageCode {
    name: "Nage",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nxe",
    individual_languages: &[
    ],
};


pub const NXG: LanguageCode = LanguageCode {
    name: "Ngad'a",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nxg",
    individual_languages: &[
    ],
};


pub const NXI: LanguageCode = LanguageCode {
    name: "Nindi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nxi",
    individual_languages: &[
    ],
};


pub const NXK: LanguageCode = LanguageCode {
    name: "Koki Naga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nxk",
    individual_languages: &[
    ],
};


pub const NXL: LanguageCode = LanguageCode {
    name: "South Nuaulu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nxl",
    individual_languages: &[
    ],
};


pub const NXM: LanguageCode = LanguageCode {
    name: "Numidian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nxm",
    individual_languages: &[
    ],
};


pub const NXN: LanguageCode = LanguageCode {
    name: "Ngawun",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nxn",
    individual_languages: &[
    ],
};


pub const NXO: LanguageCode = LanguageCode {
    name: "Ndambomo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nxo",
    individual_languages: &[
    ],
};


pub const NXQ: LanguageCode = LanguageCode {
    name: "Naxi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nxq",
    individual_languages: &[
    ],
};


pub const NXR: LanguageCode = LanguageCode {
    name: "Ninggerum",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nxr",
    individual_languages: &[
    ],
};


pub const NXX: LanguageCode = LanguageCode {
    name: "Nafri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nxx",
    individual_languages: &[
    ],
};


pub const NYB: LanguageCode = LanguageCode {
    name: "Nyangbo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nyb",
    individual_languages: &[
    ],
};


pub const NYC: LanguageCode = LanguageCode {
    name: "Nyanga-li",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nyc",
    individual_languages: &[
    ],
};


pub const NYD: LanguageCode = LanguageCode {
    name: "Nyore",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nyd",
    individual_languages: &[
    ],
};


pub const NYE: LanguageCode = LanguageCode {
    name: "Nyengo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nye",
    individual_languages: &[
    ],
};


pub const NYF: LanguageCode = LanguageCode {
    name: "Giryama",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nyf",
    individual_languages: &[
    ],
};


pub const NYG: LanguageCode = LanguageCode {
    name: "Nyindu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nyg",
    individual_languages: &[
    ],
};


pub const NYH: LanguageCode = LanguageCode {
    name: "Nyikina",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nyh",
    individual_languages: &[
    ],
};


pub const NYI: LanguageCode = LanguageCode {
    name: "Ama (Sudan)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nyi",
    individual_languages: &[
    ],
};


pub const NYJ: LanguageCode = LanguageCode {
    name: "Nyanga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nyj",
    individual_languages: &[
    ],
};


pub const NYK: LanguageCode = LanguageCode {
    name: "Nyaneka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nyk",
    individual_languages: &[
    ],
};


pub const NYL: LanguageCode = LanguageCode {
    name: "Nyeu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nyl",
    individual_languages: &[
    ],
};


pub const NYP: LanguageCode = LanguageCode {
    name: "Nyang'i",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nyp",
    individual_languages: &[
    ],
};


pub const NYQ: LanguageCode = LanguageCode {
    name: "Nayini",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nyq",
    individual_languages: &[
    ],
};


pub const NYR: LanguageCode = LanguageCode {
    name: "Nyiha (Malawi)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nyr",
    individual_languages: &[
    ],
};


pub const NYS: LanguageCode = LanguageCode {
    name: "Nyungar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nys",
    individual_languages: &[
    ],
};


pub const NYT: LanguageCode = LanguageCode {
    name: "Nyawaygi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nyt",
    individual_languages: &[
    ],
};


pub const NYU: LanguageCode = LanguageCode {
    name: "Nyungwe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nyu",
    individual_languages: &[
    ],
};


pub const NYV: LanguageCode = LanguageCode {
    name: "Nyulnyul",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nyv",
    individual_languages: &[
    ],
};


pub const NYW: LanguageCode = LanguageCode {
    name: "Nyaw",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nyw",
    individual_languages: &[
    ],
};


pub const NYX: LanguageCode = LanguageCode {
    name: "Nganyaywana",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nyx",
    individual_languages: &[
    ],
};


pub const NYY: LanguageCode = LanguageCode {
    name: "Nyakyusa-Ngonde",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nyy",
    individual_languages: &[
    ],
};


pub const NZA: LanguageCode = LanguageCode {
    name: "Tigon Mbembe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nza",
    individual_languages: &[
    ],
};


pub const NZB: LanguageCode = LanguageCode {
    name: "Njebi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nzb",
    individual_languages: &[
    ],
};


pub const NZD: LanguageCode = LanguageCode {
    name: "Nzadi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nzd",
    individual_languages: &[
    ],
};


pub const NZK: LanguageCode = LanguageCode {
    name: "Nzakara",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nzk",
    individual_languages: &[
    ],
};


pub const NZM: LanguageCode = LanguageCode {
    name: "Zeme Naga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nzm",
    individual_languages: &[
    ],
};


pub const NZS: LanguageCode = LanguageCode {
    name: "New Zealand Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nzs",
    individual_languages: &[
    ],
};


pub const NZU: LanguageCode = LanguageCode {
    name: "Teke-Nzikou",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nzu",
    individual_languages: &[
    ],
};


pub const NZY: LanguageCode = LanguageCode {
    name: "Nzakambay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nzy",
    individual_languages: &[
    ],
};


pub const NZZ: LanguageCode = LanguageCode {
    name: "Nanga Dama Dogon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "nzz",
    individual_languages: &[
    ],
};


pub const OAA: LanguageCode = LanguageCode {
    name: "Orok",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "oaa",
    individual_languages: &[
    ],
};


pub const OAC: LanguageCode = LanguageCode {
    name: "Oroch",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "oac",
    individual_languages: &[
    ],
};


pub const OAR: LanguageCode = LanguageCode {
    name: "Old Aramaic (up to 700 BCE)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "oar",
    individual_languages: &[
    ],
};


pub const OAV: LanguageCode = LanguageCode {
    name: "Old Avar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "oav",
    individual_languages: &[
    ],
};


pub const OBI: LanguageCode = LanguageCode {
    name: "Obispeño",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "obi",
    individual_languages: &[
    ],
};


pub const OBK: LanguageCode = LanguageCode {
    name: "Southern Bontok",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "obk",
    individual_languages: &[
    ],
};


pub const OBL: LanguageCode = LanguageCode {
    name: "Oblo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "obl",
    individual_languages: &[
    ],
};


pub const OBM: LanguageCode = LanguageCode {
    name: "Moabite",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "obm",
    individual_languages: &[
    ],
};


pub const OBO: LanguageCode = LanguageCode {
    name: "Obo Manobo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "obo",
    individual_languages: &[
    ],
};


pub const OBR: LanguageCode = LanguageCode {
    name: "Old Burmese",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "obr",
    individual_languages: &[
    ],
};


pub const OBT: LanguageCode = LanguageCode {
    name: "Old Breton",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "obt",
    individual_languages: &[
    ],
};


pub const OBU: LanguageCode = LanguageCode {
    name: "Obulom",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "obu",
    individual_languages: &[
    ],
};


pub const OCA: LanguageCode = LanguageCode {
    name: "Ocaina",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "oca",
    individual_languages: &[
    ],
};


pub const OCH: LanguageCode = LanguageCode {
    name: "Old Chinese",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "och",
    individual_languages: &[
    ],
};


pub const OCM: LanguageCode = LanguageCode {
    name: "Old Cham",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ocm",
    individual_languages: &[
    ],
};


pub const OCO: LanguageCode = LanguageCode {
    name: "Old Cornish",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "oco",
    individual_languages: &[
    ],
};


pub const OCU: LanguageCode = LanguageCode {
    name: "Atzingo Matlatzinca",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ocu",
    individual_languages: &[
    ],
};


pub const ODA: LanguageCode = LanguageCode {
    name: "Odut",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "oda",
    individual_languages: &[
    ],
};


pub const ODK: LanguageCode = LanguageCode {
    name: "Od",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "odk",
    individual_languages: &[
    ],
};


pub const ODT: LanguageCode = LanguageCode {
    name: "Old Dutch",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "odt",
    individual_languages: &[
    ],
};


pub const ODU: LanguageCode = LanguageCode {
    name: "Odual",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "odu",
    individual_languages: &[
    ],
};


pub const OFO: LanguageCode = LanguageCode {
    name: "Ofo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ofo",
    individual_languages: &[
    ],
};


pub const OFS: LanguageCode = LanguageCode {
    name: "Old Frisian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ofs",
    individual_languages: &[
    ],
};


pub const OFU: LanguageCode = LanguageCode {
    name: "Efutop",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ofu",
    individual_languages: &[
    ],
};


pub const OGB: LanguageCode = LanguageCode {
    name: "Ogbia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ogb",
    individual_languages: &[
    ],
};


pub const OGC: LanguageCode = LanguageCode {
    name: "Ogbah",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ogc",
    individual_languages: &[
    ],
};


pub const OGE: LanguageCode = LanguageCode {
    name: "Old Georgian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "oge",
    individual_languages: &[
    ],
};


pub const OGG: LanguageCode = LanguageCode {
    name: "Ogbogolo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ogg",
    individual_languages: &[
    ],
};


pub const OGO: LanguageCode = LanguageCode {
    name: "Khana",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ogo",
    individual_languages: &[
    ],
};


pub const OGU: LanguageCode = LanguageCode {
    name: "Ogbronuagum",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ogu",
    individual_languages: &[
    ],
};


pub const OHT: LanguageCode = LanguageCode {
    name: "Old Hittite",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "oht",
    individual_languages: &[
    ],
};


pub const OHU: LanguageCode = LanguageCode {
    name: "Old Hungarian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ohu",
    individual_languages: &[
    ],
};


pub const OIA: LanguageCode = LanguageCode {
    name: "Oirata",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "oia",
    individual_languages: &[
    ],
};


pub const OIE: LanguageCode = LanguageCode {
    name: "Okolie",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "oie",
    individual_languages: &[
    ],
};


pub const OIN: LanguageCode = LanguageCode {
    name: "Inebu One",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "oin",
    individual_languages: &[
    ],
};


pub const OJB: LanguageCode = LanguageCode {
    name: "Northwestern Ojibwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ojb",
    individual_languages: &[
    ],
};


pub const OJC: LanguageCode = LanguageCode {
    name: "Central Ojibwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ojc",
    individual_languages: &[
    ],
};


pub const OJG: LanguageCode = LanguageCode {
    name: "Eastern Ojibwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ojg",
    individual_languages: &[
    ],
};


pub const OJP: LanguageCode = LanguageCode {
    name: "Old Japanese",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ojp",
    individual_languages: &[
    ],
};


pub const OJS: LanguageCode = LanguageCode {
    name: "Severn Ojibwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ojs",
    individual_languages: &[
    ],
};


pub const OJV: LanguageCode = LanguageCode {
    name: "Ontong Java",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ojv",
    individual_languages: &[
    ],
};


pub const OJW: LanguageCode = LanguageCode {
    name: "Western Ojibwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ojw",
    individual_languages: &[
    ],
};


pub const OKA: LanguageCode = LanguageCode {
    name: "Okanagan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "oka",
    individual_languages: &[
    ],
};


pub const OKB: LanguageCode = LanguageCode {
    name: "Okobo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "okb",
    individual_languages: &[
    ],
};


pub const OKC: LanguageCode = LanguageCode {
    name: "Kobo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "okc",
    individual_languages: &[
    ],
};


pub const OKD: LanguageCode = LanguageCode {
    name: "Okodia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "okd",
    individual_languages: &[
    ],
};


pub const OKE: LanguageCode = LanguageCode {
    name: "Okpe (Southwestern Edo)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "oke",
    individual_languages: &[
    ],
};


pub const OKG: LanguageCode = LanguageCode {
    name: "Koko Babangk",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "okg",
    individual_languages: &[
    ],
};


pub const OKH: LanguageCode = LanguageCode {
    name: "Koresh-e Rostam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "okh",
    individual_languages: &[
    ],
};


pub const OKI: LanguageCode = LanguageCode {
    name: "Okiek",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "oki",
    individual_languages: &[
    ],
};


pub const OKJ: LanguageCode = LanguageCode {
    name: "Oko-Juwoi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "okj",
    individual_languages: &[
    ],
};


pub const OKK: LanguageCode = LanguageCode {
    name: "Kwamtim One",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "okk",
    individual_languages: &[
    ],
};


pub const OKL: LanguageCode = LanguageCode {
    name: "Old Kentish Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "okl",
    individual_languages: &[
    ],
};


pub const OKM: LanguageCode = LanguageCode {
    name: "Middle Korean (10th-16th cent.)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "okm",
    individual_languages: &[
    ],
};


pub const OKN: LanguageCode = LanguageCode {
    name: "Oki-No-Erabu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "okn",
    individual_languages: &[
    ],
};


pub const OKO: LanguageCode = LanguageCode {
    name: "Old Korean (3rd-9th cent.)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "oko",
    individual_languages: &[
    ],
};


pub const OKR: LanguageCode = LanguageCode {
    name: "Kirike",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "okr",
    individual_languages: &[
    ],
};


pub const OKS: LanguageCode = LanguageCode {
    name: "Oko-Eni-Osayen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "oks",
    individual_languages: &[
    ],
};


pub const OKU: LanguageCode = LanguageCode {
    name: "Oku",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "oku",
    individual_languages: &[
    ],
};


pub const OKV: LanguageCode = LanguageCode {
    name: "Orokaiva",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "okv",
    individual_languages: &[
    ],
};


pub const OKX: LanguageCode = LanguageCode {
    name: "Okpe (Northwestern Edo)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "okx",
    individual_languages: &[
    ],
};


pub const OKZ: LanguageCode = LanguageCode {
    name: "Old Khmer",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "okz",
    individual_languages: &[
    ],
};


pub const OLA: LanguageCode = LanguageCode {
    name: "Walungge",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ola",
    individual_languages: &[
    ],
};


pub const OLD: LanguageCode = LanguageCode {
    name: "Mochi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "old",
    individual_languages: &[
    ],
};


pub const OLE: LanguageCode = LanguageCode {
    name: "Olekha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ole",
    individual_languages: &[
    ],
};


pub const OLK: LanguageCode = LanguageCode {
    name: "Olkol",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "olk",
    individual_languages: &[
    ],
};


pub const OLM: LanguageCode = LanguageCode {
    name: "Oloma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "olm",
    individual_languages: &[
    ],
};


pub const OLO: LanguageCode = LanguageCode {
    name: "Livvi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "olo",
    individual_languages: &[
    ],
};


pub const OLR: LanguageCode = LanguageCode {
    name: "Olrat",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "olr",
    individual_languages: &[
    ],
};


pub const OLT: LanguageCode = LanguageCode {
    name: "Old Lithuanian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "olt",
    individual_languages: &[
    ],
};


pub const OLU: LanguageCode = LanguageCode {
    name: "Kuvale",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "olu",
    individual_languages: &[
    ],
};


pub const OMA: LanguageCode = LanguageCode {
    name: "Omaha-Ponca",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "oma",
    individual_languages: &[
    ],
};


pub const OMB: LanguageCode = LanguageCode {
    name: "East Ambae",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "omb",
    individual_languages: &[
    ],
};


pub const OMC: LanguageCode = LanguageCode {
    name: "Mochica",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "omc",
    individual_languages: &[
    ],
};


pub const OMG: LanguageCode = LanguageCode {
    name: "Omagua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "omg",
    individual_languages: &[
    ],
};


pub const OMI: LanguageCode = LanguageCode {
    name: "Omi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "omi",
    individual_languages: &[
    ],
};


pub const OMK: LanguageCode = LanguageCode {
    name: "Omok",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "omk",
    individual_languages: &[
    ],
};


pub const OML: LanguageCode = LanguageCode {
    name: "Ombo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "oml",
    individual_languages: &[
    ],
};


pub const OMN: LanguageCode = LanguageCode {
    name: "Minoan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "omn",
    individual_languages: &[
    ],
};


pub const OMO: LanguageCode = LanguageCode {
    name: "Utarmbung",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "omo",
    individual_languages: &[
    ],
};


pub const OMP: LanguageCode = LanguageCode {
    name: "Old Manipuri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "omp",
    individual_languages: &[
    ],
};


pub const OMR: LanguageCode = LanguageCode {
    name: "Old Marathi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "omr",
    individual_languages: &[
    ],
};


pub const OMT: LanguageCode = LanguageCode {
    name: "Omotik",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "omt",
    individual_languages: &[
    ],
};


pub const OMU: LanguageCode = LanguageCode {
    name: "Omurano",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "omu",
    individual_languages: &[
    ],
};


pub const OMW: LanguageCode = LanguageCode {
    name: "South Tairora",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "omw",
    individual_languages: &[
    ],
};


pub const OMX: LanguageCode = LanguageCode {
    name: "Old Mon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "omx",
    individual_languages: &[
    ],
};


pub const OMY: LanguageCode = LanguageCode {
    name: "Old Malay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "omy",
    individual_languages: &[
    ],
};


pub const ONA: LanguageCode = LanguageCode {
    name: "Ona",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ona",
    individual_languages: &[
    ],
};


pub const ONB: LanguageCode = LanguageCode {
    name: "Lingao",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "onb",
    individual_languages: &[
    ],
};


pub const ONE: LanguageCode = LanguageCode {
    name: "Oneida",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "one",
    individual_languages: &[
    ],
};


pub const ONG: LanguageCode = LanguageCode {
    name: "Olo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ong",
    individual_languages: &[
    ],
};


pub const ONI: LanguageCode = LanguageCode {
    name: "Onin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "oni",
    individual_languages: &[
    ],
};


pub const ONJ: LanguageCode = LanguageCode {
    name: "Onjob",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "onj",
    individual_languages: &[
    ],
};


pub const ONK: LanguageCode = LanguageCode {
    name: "Kabore One",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "onk",
    individual_languages: &[
    ],
};


pub const ONN: LanguageCode = LanguageCode {
    name: "Onobasulu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "onn",
    individual_languages: &[
    ],
};


pub const ONO: LanguageCode = LanguageCode {
    name: "Onondaga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ono",
    individual_languages: &[
    ],
};


pub const ONP: LanguageCode = LanguageCode {
    name: "Sartang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "onp",
    individual_languages: &[
    ],
};


pub const ONR: LanguageCode = LanguageCode {
    name: "Northern One",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "onr",
    individual_languages: &[
    ],
};


pub const ONS: LanguageCode = LanguageCode {
    name: "Ono",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ons",
    individual_languages: &[
    ],
};


pub const ONT: LanguageCode = LanguageCode {
    name: "Ontenu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ont",
    individual_languages: &[
    ],
};


pub const ONU: LanguageCode = LanguageCode {
    name: "Unua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "onu",
    individual_languages: &[
    ],
};


pub const ONW: LanguageCode = LanguageCode {
    name: "Old Nubian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "onw",
    individual_languages: &[
    ],
};


pub const ONX: LanguageCode = LanguageCode {
    name: "Onin Based Pidgin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "onx",
    individual_languages: &[
    ],
};


pub const OOD: LanguageCode = LanguageCode {
    name: "Tohono O'odham",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ood",
    individual_languages: &[
    ],
};


pub const OOG: LanguageCode = LanguageCode {
    name: "Ong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "oog",
    individual_languages: &[
    ],
};


pub const OON: LanguageCode = LanguageCode {
    name: "Önge",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "oon",
    individual_languages: &[
    ],
};


pub const OOR: LanguageCode = LanguageCode {
    name: "Oorlams",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "oor",
    individual_languages: &[
    ],
};


pub const OOS: LanguageCode = LanguageCode {
    name: "Old Ossetic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "oos",
    individual_languages: &[
    ],
};


pub const OPA: LanguageCode = LanguageCode {
    name: "Okpamheri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "opa",
    individual_languages: &[
    ],
};


pub const OPK: LanguageCode = LanguageCode {
    name: "Kopkaka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "opk",
    individual_languages: &[
    ],
};


pub const OPM: LanguageCode = LanguageCode {
    name: "Oksapmin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "opm",
    individual_languages: &[
    ],
};


pub const OPO: LanguageCode = LanguageCode {
    name: "Opao",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "opo",
    individual_languages: &[
    ],
};


pub const OPT: LanguageCode = LanguageCode {
    name: "Opata",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "opt",
    individual_languages: &[
    ],
};


pub const OPY: LanguageCode = LanguageCode {
    name: "Ofayé",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "opy",
    individual_languages: &[
    ],
};


pub const ORA: LanguageCode = LanguageCode {
    name: "Oroha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ora",
    individual_languages: &[
    ],
};


pub const ORC: LanguageCode = LanguageCode {
    name: "Orma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "orc",
    individual_languages: &[
    ],
};


pub const ORE: LanguageCode = LanguageCode {
    name: "Orejón",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ore",
    individual_languages: &[
    ],
};


pub const ORG: LanguageCode = LanguageCode {
    name: "Oring",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "org",
    individual_languages: &[
    ],
};


pub const ORH: LanguageCode = LanguageCode {
    name: "Oroqen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "orh",
    individual_languages: &[
    ],
};


pub const ORN: LanguageCode = LanguageCode {
    name: "Orang Kanaq",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "orn",
    individual_languages: &[
    ],
};


pub const ORO: LanguageCode = LanguageCode {
    name: "Orokolo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "oro",
    individual_languages: &[
    ],
};


pub const ORR: LanguageCode = LanguageCode {
    name: "Oruma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "orr",
    individual_languages: &[
    ],
};


pub const ORS: LanguageCode = LanguageCode {
    name: "Orang Seletar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ors",
    individual_languages: &[
    ],
};


pub const ORT: LanguageCode = LanguageCode {
    name: "Adivasi Oriya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ort",
    individual_languages: &[
    ],
};


pub const ORU: LanguageCode = LanguageCode {
    name: "Ormuri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "oru",
    individual_languages: &[
    ],
};


pub const ORV: LanguageCode = LanguageCode {
    name: "Old Russian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "orv",
    individual_languages: &[
    ],
};


pub const ORW: LanguageCode = LanguageCode {
    name: "Oro Win",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "orw",
    individual_languages: &[
    ],
};


pub const ORX: LanguageCode = LanguageCode {
    name: "Oro",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "orx",
    individual_languages: &[
    ],
};


pub const ORY: LanguageCode = LanguageCode {
    name: "Odia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ory",
    individual_languages: &[
    ],
};


pub const ORZ: LanguageCode = LanguageCode {
    name: "Ormu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "orz",
    individual_languages: &[
    ],
};


pub const OSC: LanguageCode = LanguageCode {
    name: "Oscan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "osc",
    individual_languages: &[
    ],
};


pub const OSI: LanguageCode = LanguageCode {
    name: "Osing",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "osi",
    individual_languages: &[
    ],
};


pub const OSN: LanguageCode = LanguageCode {
    name: "Old Sundanese",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "osn",
    individual_languages: &[
    ],
};


pub const OSO: LanguageCode = LanguageCode {
    name: "Ososo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "oso",
    individual_languages: &[
    ],
};


pub const OSP: LanguageCode = LanguageCode {
    name: "Old Spanish",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "osp",
    individual_languages: &[
    ],
};


pub const OST: LanguageCode = LanguageCode {
    name: "Osatu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ost",
    individual_languages: &[
    ],
};


pub const OSU: LanguageCode = LanguageCode {
    name: "Southern One",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "osu",
    individual_languages: &[
    ],
};


pub const OSX: LanguageCode = LanguageCode {
    name: "Old Saxon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "osx",
    individual_languages: &[
    ],
};


pub const OTB: LanguageCode = LanguageCode {
    name: "Old Tibetan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "otb",
    individual_languages: &[
    ],
};


pub const OTD: LanguageCode = LanguageCode {
    name: "Ot Danum",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "otd",
    individual_languages: &[
    ],
};


pub const OTE: LanguageCode = LanguageCode {
    name: "Mezquital Otomi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ote",
    individual_languages: &[
    ],
};


pub const OTI: LanguageCode = LanguageCode {
    name: "Oti",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "oti",
    individual_languages: &[
    ],
};


pub const OTK: LanguageCode = LanguageCode {
    name: "Old Turkish",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "otk",
    individual_languages: &[
    ],
};


pub const OTL: LanguageCode = LanguageCode {
    name: "Tilapa Otomi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "otl",
    individual_languages: &[
    ],
};


pub const OTM: LanguageCode = LanguageCode {
    name: "Eastern Highland Otomi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "otm",
    individual_languages: &[
    ],
};


pub const OTN: LanguageCode = LanguageCode {
    name: "Tenango Otomi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "otn",
    individual_languages: &[
    ],
};


pub const OTQ: LanguageCode = LanguageCode {
    name: "Querétaro Otomi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "otq",
    individual_languages: &[
    ],
};


pub const OTR: LanguageCode = LanguageCode {
    name: "Otoro",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "otr",
    individual_languages: &[
    ],
};


pub const OTS: LanguageCode = LanguageCode {
    name: "Estado de México Otomi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ots",
    individual_languages: &[
    ],
};


pub const OTT: LanguageCode = LanguageCode {
    name: "Temoaya Otomi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ott",
    individual_languages: &[
    ],
};


pub const OTU: LanguageCode = LanguageCode {
    name: "Otuke",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "otu",
    individual_languages: &[
    ],
};


pub const OTW: LanguageCode = LanguageCode {
    name: "Ottawa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "otw",
    individual_languages: &[
    ],
};


pub const OTX: LanguageCode = LanguageCode {
    name: "Texcatepec Otomi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "otx",
    individual_languages: &[
    ],
};


pub const OTY: LanguageCode = LanguageCode {
    name: "Old Tamil",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "oty",
    individual_languages: &[
    ],
};


pub const OTZ: LanguageCode = LanguageCode {
    name: "Ixtenco Otomi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "otz",
    individual_languages: &[
    ],
};


pub const OUA: LanguageCode = LanguageCode {
    name: "Tagargrent",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "oua",
    individual_languages: &[
    ],
};


pub const OUB: LanguageCode = LanguageCode {
    name: "Glio-Oubi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "oub",
    individual_languages: &[
    ],
};


pub const OUE: LanguageCode = LanguageCode {
    name: "Oune",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "oue",
    individual_languages: &[
    ],
};


pub const OUI: LanguageCode = LanguageCode {
    name: "Old Uighur",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "oui",
    individual_languages: &[
    ],
};


pub const OUM: LanguageCode = LanguageCode {
    name: "Ouma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "oum",
    individual_languages: &[
    ],
};


pub const OVD: LanguageCode = LanguageCode {
    name: "Elfdalian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ovd",
    individual_languages: &[
    ],
};


pub const OWI: LanguageCode = LanguageCode {
    name: "Owiniga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "owi",
    individual_languages: &[
    ],
};


pub const OWL: LanguageCode = LanguageCode {
    name: "Old Welsh",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "owl",
    individual_languages: &[
    ],
};


pub const OYB: LanguageCode = LanguageCode {
    name: "Oy",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "oyb",
    individual_languages: &[
    ],
};


pub const OYD: LanguageCode = LanguageCode {
    name: "Oyda",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "oyd",
    individual_languages: &[
    ],
};


pub const OYM: LanguageCode = LanguageCode {
    name: "Wayampi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "oym",
    individual_languages: &[
    ],
};


pub const OYY: LanguageCode = LanguageCode {
    name: "Oya'oya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "oyy",
    individual_languages: &[
    ],
};


pub const OZM: LanguageCode = LanguageCode {
    name: "Koonzime",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ozm",
    individual_languages: &[
    ],
};


pub const PAB: LanguageCode = LanguageCode {
    name: "Parecís",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pab",
    individual_languages: &[
    ],
};


pub const PAC: LanguageCode = LanguageCode {
    name: "Pacoh",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pac",
    individual_languages: &[
    ],
};


pub const PAD: LanguageCode = LanguageCode {
    name: "Paumarí",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pad",
    individual_languages: &[
    ],
};


pub const PAE: LanguageCode = LanguageCode {
    name: "Pagibete",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pae",
    individual_languages: &[
    ],
};


pub const PAF: LanguageCode = LanguageCode {
    name: "Paranawát",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "paf",
    individual_languages: &[
    ],
};


pub const PAH: LanguageCode = LanguageCode {
    name: "Tenharim",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pah",
    individual_languages: &[
    ],
};


pub const PAI: LanguageCode = LanguageCode {
    name: "Pe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pai",
    individual_languages: &[
    ],
};


pub const PAK: LanguageCode = LanguageCode {
    name: "Parakanã",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pak",
    individual_languages: &[
    ],
};


pub const PAO: LanguageCode = LanguageCode {
    name: "Northern Paiute",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pao",
    individual_languages: &[
    ],
};


pub const PAQ: LanguageCode = LanguageCode {
    name: "Parya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "paq",
    individual_languages: &[
    ],
};


pub const PAR: LanguageCode = LanguageCode {
    name: "Panamint",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "par",
    individual_languages: &[
    ],
};


pub const PAS: LanguageCode = LanguageCode {
    name: "Papasena",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pas",
    individual_languages: &[
    ],
};


pub const PAV: LanguageCode = LanguageCode {
    name: "Pakaásnovos",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pav",
    individual_languages: &[
    ],
};


pub const PAW: LanguageCode = LanguageCode {
    name: "Pawnee",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "paw",
    individual_languages: &[
    ],
};


pub const PAX: LanguageCode = LanguageCode {
    name: "Pankararé",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pax",
    individual_languages: &[
    ],
};


pub const PAY: LanguageCode = LanguageCode {
    name: "Pech",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pay",
    individual_languages: &[
    ],
};


pub const PAZ: LanguageCode = LanguageCode {
    name: "Pankararú",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "paz",
    individual_languages: &[
    ],
};


pub const PBB: LanguageCode = LanguageCode {
    name: "Páez",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pbb",
    individual_languages: &[
    ],
};


pub const PBC: LanguageCode = LanguageCode {
    name: "Patamona",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pbc",
    individual_languages: &[
    ],
};


pub const PBE: LanguageCode = LanguageCode {
    name: "Mezontla Popoloca",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pbe",
    individual_languages: &[
    ],
};


pub const PBF: LanguageCode = LanguageCode {
    name: "Coyotepec Popoloca",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pbf",
    individual_languages: &[
    ],
};


pub const PBG: LanguageCode = LanguageCode {
    name: "Paraujano",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pbg",
    individual_languages: &[
    ],
};


pub const PBH: LanguageCode = LanguageCode {
    name: "E'ñapa Woromaipu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pbh",
    individual_languages: &[
    ],
};


pub const PBI: LanguageCode = LanguageCode {
    name: "Parkwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pbi",
    individual_languages: &[
    ],
};


pub const PBL: LanguageCode = LanguageCode {
    name: "Mak (Nigeria)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pbl",
    individual_languages: &[
    ],
};


pub const PBM: LanguageCode = LanguageCode {
    name: "Puebla Mazatec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pbm",
    individual_languages: &[
    ],
};


pub const PBN: LanguageCode = LanguageCode {
    name: "Kpasam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pbn",
    individual_languages: &[
    ],
};


pub const PBO: LanguageCode = LanguageCode {
    name: "Papel",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pbo",
    individual_languages: &[
    ],
};


pub const PBP: LanguageCode = LanguageCode {
    name: "Badyara",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pbp",
    individual_languages: &[
    ],
};


pub const PBR: LanguageCode = LanguageCode {
    name: "Pangwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pbr",
    individual_languages: &[
    ],
};


pub const PBS: LanguageCode = LanguageCode {
    name: "Central Pame",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pbs",
    individual_languages: &[
    ],
};


pub const PBT: LanguageCode = LanguageCode {
    name: "Southern Pashto",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pbt",
    individual_languages: &[
    ],
};


pub const PBU: LanguageCode = LanguageCode {
    name: "Northern Pashto",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pbu",
    individual_languages: &[
    ],
};


pub const PBV: LanguageCode = LanguageCode {
    name: "Pnar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pbv",
    individual_languages: &[
    ],
};


pub const PBY: LanguageCode = LanguageCode {
    name: "Pyu (Papua New Guinea)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pby",
    individual_languages: &[
    ],
};


pub const PCA: LanguageCode = LanguageCode {
    name: "Santa Inés Ahuatempan Popoloca",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pca",
    individual_languages: &[
    ],
};


pub const PCB: LanguageCode = LanguageCode {
    name: "Pear",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pcb",
    individual_languages: &[
    ],
};


pub const PCC: LanguageCode = LanguageCode {
    name: "Bouyei",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pcc",
    individual_languages: &[
    ],
};


pub const PCD: LanguageCode = LanguageCode {
    name: "Picard",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pcd",
    individual_languages: &[
    ],
};


pub const PCE: LanguageCode = LanguageCode {
    name: "Ruching Palaung",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pce",
    individual_languages: &[
    ],
};


pub const PCF: LanguageCode = LanguageCode {
    name: "Paliyan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pcf",
    individual_languages: &[
    ],
};


pub const PCG: LanguageCode = LanguageCode {
    name: "Paniya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pcg",
    individual_languages: &[
    ],
};


pub const PCH: LanguageCode = LanguageCode {
    name: "Pardhan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pch",
    individual_languages: &[
    ],
};


pub const PCI: LanguageCode = LanguageCode {
    name: "Duruwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pci",
    individual_languages: &[
    ],
};


pub const PCJ: LanguageCode = LanguageCode {
    name: "Parenga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pcj",
    individual_languages: &[
    ],
};


pub const PCK: LanguageCode = LanguageCode {
    name: "Paite Chin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pck",
    individual_languages: &[
    ],
};


pub const PCL: LanguageCode = LanguageCode {
    name: "Pardhi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pcl",
    individual_languages: &[
    ],
};


pub const PCM: LanguageCode = LanguageCode {
    name: "Nigerian Pidgin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pcm",
    individual_languages: &[
    ],
};


pub const PCN: LanguageCode = LanguageCode {
    name: "Piti",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pcn",
    individual_languages: &[
    ],
};


pub const PCP: LanguageCode = LanguageCode {
    name: "Pacahuara",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pcp",
    individual_languages: &[
    ],
};


pub const PCW: LanguageCode = LanguageCode {
    name: "Pyapun",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pcw",
    individual_languages: &[
    ],
};


pub const PDA: LanguageCode = LanguageCode {
    name: "Anam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pda",
    individual_languages: &[
    ],
};


pub const PDC: LanguageCode = LanguageCode {
    name: "Pennsylvania German",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pdc",
    individual_languages: &[
    ],
};


pub const PDI: LanguageCode = LanguageCode {
    name: "Pa Di",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pdi",
    individual_languages: &[
    ],
};


pub const PDN: LanguageCode = LanguageCode {
    name: "Podena",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pdn",
    individual_languages: &[
    ],
};


pub const PDO: LanguageCode = LanguageCode {
    name: "Padoe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pdo",
    individual_languages: &[
    ],
};


pub const PDT: LanguageCode = LanguageCode {
    name: "Plautdietsch",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pdt",
    individual_languages: &[
    ],
};


pub const PDU: LanguageCode = LanguageCode {
    name: "Kayan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pdu",
    individual_languages: &[
    ],
};


pub const PEA: LanguageCode = LanguageCode {
    name: "Peranakan Indonesian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pea",
    individual_languages: &[
    ],
};


pub const PEB: LanguageCode = LanguageCode {
    name: "Eastern Pomo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "peb",
    individual_languages: &[
    ],
};


pub const PED: LanguageCode = LanguageCode {
    name: "Mala (Papua New Guinea)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ped",
    individual_languages: &[
    ],
};


pub const PEE: LanguageCode = LanguageCode {
    name: "Taje",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pee",
    individual_languages: &[
    ],
};


pub const PEF: LanguageCode = LanguageCode {
    name: "Northeastern Pomo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pef",
    individual_languages: &[
    ],
};


pub const PEG: LanguageCode = LanguageCode {
    name: "Pengo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "peg",
    individual_languages: &[
    ],
};


pub const PEH: LanguageCode = LanguageCode {
    name: "Bonan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "peh",
    individual_languages: &[
    ],
};


pub const PEI: LanguageCode = LanguageCode {
    name: "Chichimeca-Jonaz",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pei",
    individual_languages: &[
    ],
};


pub const PEJ: LanguageCode = LanguageCode {
    name: "Northern Pomo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pej",
    individual_languages: &[
    ],
};


pub const PEK: LanguageCode = LanguageCode {
    name: "Penchal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pek",
    individual_languages: &[
    ],
};


pub const PEL: LanguageCode = LanguageCode {
    name: "Pekal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pel",
    individual_languages: &[
    ],
};


pub const PEM: LanguageCode = LanguageCode {
    name: "Phende",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pem",
    individual_languages: &[
    ],
};


pub const PEP: LanguageCode = LanguageCode {
    name: "Kunja",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pep",
    individual_languages: &[
    ],
};


pub const PEQ: LanguageCode = LanguageCode {
    name: "Southern Pomo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "peq",
    individual_languages: &[
    ],
};


pub const PES: LanguageCode = LanguageCode {
    name: "Iranian Persian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pes",
    individual_languages: &[
    ],
};


pub const PEV: LanguageCode = LanguageCode {
    name: "Pémono",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pev",
    individual_languages: &[
    ],
};


pub const PEX: LanguageCode = LanguageCode {
    name: "Petats",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pex",
    individual_languages: &[
    ],
};


pub const PEY: LanguageCode = LanguageCode {
    name: "Petjo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pey",
    individual_languages: &[
    ],
};


pub const PEZ: LanguageCode = LanguageCode {
    name: "Eastern Penan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pez",
    individual_languages: &[
    ],
};


pub const PFA: LanguageCode = LanguageCode {
    name: "Pááfang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pfa",
    individual_languages: &[
    ],
};


pub const PFE: LanguageCode = LanguageCode {
    name: "Pere",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pfe",
    individual_languages: &[
    ],
};


pub const PFL: LanguageCode = LanguageCode {
    name: "Pfaelzisch",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pfl",
    individual_languages: &[
    ],
};


pub const PGA: LanguageCode = LanguageCode {
    name: "Sudanese Creole Arabic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pga",
    individual_languages: &[
    ],
};


pub const PGD: LanguageCode = LanguageCode {
    name: "Gāndhārī",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pgd",
    individual_languages: &[
    ],
};


pub const PGG: LanguageCode = LanguageCode {
    name: "Pangwali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pgg",
    individual_languages: &[
    ],
};


pub const PGI: LanguageCode = LanguageCode {
    name: "Pagi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pgi",
    individual_languages: &[
    ],
};


pub const PGK: LanguageCode = LanguageCode {
    name: "Rerep",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pgk",
    individual_languages: &[
    ],
};


pub const PGL: LanguageCode = LanguageCode {
    name: "Primitive Irish",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pgl",
    individual_languages: &[
    ],
};


pub const PGN: LanguageCode = LanguageCode {
    name: "Paelignian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pgn",
    individual_languages: &[
    ],
};


pub const PGS: LanguageCode = LanguageCode {
    name: "Pangseng",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pgs",
    individual_languages: &[
    ],
};


pub const PGU: LanguageCode = LanguageCode {
    name: "Pagu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pgu",
    individual_languages: &[
    ],
};


pub const PGZ: LanguageCode = LanguageCode {
    name: "Papua New Guinean Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pgz",
    individual_languages: &[
    ],
};


pub const PHA: LanguageCode = LanguageCode {
    name: "Pa-Hng",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pha",
    individual_languages: &[
    ],
};


pub const PHD: LanguageCode = LanguageCode {
    name: "Phudagi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "phd",
    individual_languages: &[
    ],
};


pub const PHG: LanguageCode = LanguageCode {
    name: "Phuong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "phg",
    individual_languages: &[
    ],
};


pub const PHH: LanguageCode = LanguageCode {
    name: "Phukha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "phh",
    individual_languages: &[
    ],
};


pub const PHJ: LanguageCode = LanguageCode {
    name: "Pahari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "phj",
    individual_languages: &[
    ],
};


pub const PHK: LanguageCode = LanguageCode {
    name: "Phake",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "phk",
    individual_languages: &[
    ],
};


pub const PHL: LanguageCode = LanguageCode {
    name: "Phalura",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "phl",
    individual_languages: &[
    ],
};


pub const PHM: LanguageCode = LanguageCode {
    name: "Phimbi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "phm",
    individual_languages: &[
    ],
};


pub const PHO: LanguageCode = LanguageCode {
    name: "Phunoi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pho",
    individual_languages: &[
    ],
};


pub const PHQ: LanguageCode = LanguageCode {
    name: "Phana'",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "phq",
    individual_languages: &[
    ],
};


pub const PHR: LanguageCode = LanguageCode {
    name: "Pahari-Potwari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "phr",
    individual_languages: &[
    ],
};


pub const PHT: LanguageCode = LanguageCode {
    name: "Phu Thai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pht",
    individual_languages: &[
    ],
};


pub const PHU: LanguageCode = LanguageCode {
    name: "Phuan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "phu",
    individual_languages: &[
    ],
};


pub const PHV: LanguageCode = LanguageCode {
    name: "Pahlavani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "phv",
    individual_languages: &[
    ],
};


pub const PHW: LanguageCode = LanguageCode {
    name: "Phangduwali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "phw",
    individual_languages: &[
    ],
};


pub const PIA: LanguageCode = LanguageCode {
    name: "Pima Bajo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pia",
    individual_languages: &[
    ],
};


pub const PIB: LanguageCode = LanguageCode {
    name: "Yine",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pib",
    individual_languages: &[
    ],
};


pub const PIC: LanguageCode = LanguageCode {
    name: "Pinji",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pic",
    individual_languages: &[
    ],
};


pub const PID: LanguageCode = LanguageCode {
    name: "Piaroa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pid",
    individual_languages: &[
    ],
};


pub const PIE: LanguageCode = LanguageCode {
    name: "Piro",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pie",
    individual_languages: &[
    ],
};


pub const PIF: LanguageCode = LanguageCode {
    name: "Pingelapese",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pif",
    individual_languages: &[
    ],
};


pub const PIG: LanguageCode = LanguageCode {
    name: "Pisabo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pig",
    individual_languages: &[
    ],
};


pub const PIH: LanguageCode = LanguageCode {
    name: "Pitcairn-Norfolk",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pih",
    individual_languages: &[
    ],
};


pub const PIJ: LanguageCode = LanguageCode {
    name: "Pijao",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pij",
    individual_languages: &[
    ],
};


pub const PIL: LanguageCode = LanguageCode {
    name: "Yom",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pil",
    individual_languages: &[
    ],
};


pub const PIM: LanguageCode = LanguageCode {
    name: "Powhatan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pim",
    individual_languages: &[
    ],
};


pub const PIN: LanguageCode = LanguageCode {
    name: "Piame",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pin",
    individual_languages: &[
    ],
};


pub const PIO: LanguageCode = LanguageCode {
    name: "Piapoco",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pio",
    individual_languages: &[
    ],
};


pub const PIP: LanguageCode = LanguageCode {
    name: "Pero",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pip",
    individual_languages: &[
    ],
};


pub const PIR: LanguageCode = LanguageCode {
    name: "Piratapuyo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pir",
    individual_languages: &[
    ],
};


pub const PIS: LanguageCode = LanguageCode {
    name: "Pijin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pis",
    individual_languages: &[
    ],
};


pub const PIT: LanguageCode = LanguageCode {
    name: "Pitta Pitta",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pit",
    individual_languages: &[
    ],
};


pub const PIU: LanguageCode = LanguageCode {
    name: "Pintupi-Luritja",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "piu",
    individual_languages: &[
    ],
};


pub const PIV: LanguageCode = LanguageCode {
    name: "Pileni",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "piv",
    individual_languages: &[
    ],
};


pub const PIW: LanguageCode = LanguageCode {
    name: "Pimbwe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "piw",
    individual_languages: &[
    ],
};


pub const PIX: LanguageCode = LanguageCode {
    name: "Piu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pix",
    individual_languages: &[
    ],
};


pub const PIY: LanguageCode = LanguageCode {
    name: "Piya-Kwonci",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "piy",
    individual_languages: &[
    ],
};


pub const PIZ: LanguageCode = LanguageCode {
    name: "Pije",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "piz",
    individual_languages: &[
    ],
};


pub const PJT: LanguageCode = LanguageCode {
    name: "Pitjantjatjara",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pjt",
    individual_languages: &[
    ],
};


pub const PKA: LanguageCode = LanguageCode {
    name: "Ardhamāgadhī Prākrit",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pka",
    individual_languages: &[
    ],
};


pub const PKB: LanguageCode = LanguageCode {
    name: "Pokomo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pkb",
    individual_languages: &[
    ],
};


pub const PKC: LanguageCode = LanguageCode {
    name: "Paekche",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pkc",
    individual_languages: &[
    ],
};


pub const PKG: LanguageCode = LanguageCode {
    name: "Pak-Tong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pkg",
    individual_languages: &[
    ],
};


pub const PKH: LanguageCode = LanguageCode {
    name: "Pankhu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pkh",
    individual_languages: &[
    ],
};


pub const PKN: LanguageCode = LanguageCode {
    name: "Pakanha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pkn",
    individual_languages: &[
    ],
};


pub const PKO: LanguageCode = LanguageCode {
    name: "Pökoot",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pko",
    individual_languages: &[
    ],
};


pub const PKP: LanguageCode = LanguageCode {
    name: "Pukapuka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pkp",
    individual_languages: &[
    ],
};


pub const PKR: LanguageCode = LanguageCode {
    name: "Attapady Kurumba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pkr",
    individual_languages: &[
    ],
};


pub const PKS: LanguageCode = LanguageCode {
    name: "Pakistan Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pks",
    individual_languages: &[
    ],
};


pub const PKT: LanguageCode = LanguageCode {
    name: "Maleng",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pkt",
    individual_languages: &[
    ],
};


pub const PKU: LanguageCode = LanguageCode {
    name: "Paku",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pku",
    individual_languages: &[
    ],
};


pub const PLA: LanguageCode = LanguageCode {
    name: "Miani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pla",
    individual_languages: &[
    ],
};


pub const PLB: LanguageCode = LanguageCode {
    name: "Polonombauk",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "plb",
    individual_languages: &[
    ],
};


pub const PLC: LanguageCode = LanguageCode {
    name: "Central Palawano",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "plc",
    individual_languages: &[
    ],
};


pub const PLD: LanguageCode = LanguageCode {
    name: "Polari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pld",
    individual_languages: &[
    ],
};


pub const PLE: LanguageCode = LanguageCode {
    name: "Palu'e",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ple",
    individual_languages: &[
    ],
};


pub const PLG: LanguageCode = LanguageCode {
    name: "Pilagá",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "plg",
    individual_languages: &[
    ],
};


pub const PLH: LanguageCode = LanguageCode {
    name: "Paulohi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "plh",
    individual_languages: &[
    ],
};


pub const PLJ: LanguageCode = LanguageCode {
    name: "Polci",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "plj",
    individual_languages: &[
    ],
};


pub const PLK: LanguageCode = LanguageCode {
    name: "Kohistani Shina",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "plk",
    individual_languages: &[
    ],
};


pub const PLL: LanguageCode = LanguageCode {
    name: "Shwe Palaung",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pll",
    individual_languages: &[
    ],
};


pub const PLN: LanguageCode = LanguageCode {
    name: "Palenquero",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pln",
    individual_languages: &[
    ],
};


pub const PLO: LanguageCode = LanguageCode {
    name: "Oluta Popoluca",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "plo",
    individual_languages: &[
    ],
};


pub const PLQ: LanguageCode = LanguageCode {
    name: "Palaic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "plq",
    individual_languages: &[
    ],
};


pub const PLR: LanguageCode = LanguageCode {
    name: "Palaka Senoufo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "plr",
    individual_languages: &[
    ],
};


pub const PLS: LanguageCode = LanguageCode {
    name: "San Marcos Tlacoyalco Popoloca",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pls",
    individual_languages: &[
    ],
};


pub const PLT: LanguageCode = LanguageCode {
    name: "Plateau Malagasy",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "plt",
    individual_languages: &[
    ],
};


pub const PLU: LanguageCode = LanguageCode {
    name: "Palikúr",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "plu",
    individual_languages: &[
    ],
};


pub const PLV: LanguageCode = LanguageCode {
    name: "Southwest Palawano",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "plv",
    individual_languages: &[
    ],
};


pub const PLW: LanguageCode = LanguageCode {
    name: "Brooke's Point Palawano",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "plw",
    individual_languages: &[
    ],
};


pub const PLY: LanguageCode = LanguageCode {
    name: "Bolyu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ply",
    individual_languages: &[
    ],
};


pub const PLZ: LanguageCode = LanguageCode {
    name: "Paluan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "plz",
    individual_languages: &[
    ],
};


pub const PMA: LanguageCode = LanguageCode {
    name: "Paama",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pma",
    individual_languages: &[
    ],
};


pub const PMB: LanguageCode = LanguageCode {
    name: "Pambia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pmb",
    individual_languages: &[
    ],
};


pub const PMD: LanguageCode = LanguageCode {
    name: "Pallanganmiddang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pmd",
    individual_languages: &[
    ],
};


pub const PME: LanguageCode = LanguageCode {
    name: "Pwaamei",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pme",
    individual_languages: &[
    ],
};


pub const PMF: LanguageCode = LanguageCode {
    name: "Pamona",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pmf",
    individual_languages: &[
    ],
};


pub const PMH: LanguageCode = LanguageCode {
    name: "Māhārāṣṭri Prākrit",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pmh",
    individual_languages: &[
    ],
};


pub const PMI: LanguageCode = LanguageCode {
    name: "Northern Pumi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pmi",
    individual_languages: &[
    ],
};


pub const PMJ: LanguageCode = LanguageCode {
    name: "Southern Pumi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pmj",
    individual_languages: &[
    ],
};


pub const PMK: LanguageCode = LanguageCode {
    name: "Pamlico",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pmk",
    individual_languages: &[
    ],
};


pub const PML: LanguageCode = LanguageCode {
    name: "Lingua Franca",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pml",
    individual_languages: &[
    ],
};


pub const PMM: LanguageCode = LanguageCode {
    name: "Pomo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pmm",
    individual_languages: &[
    ],
};


pub const PMN: LanguageCode = LanguageCode {
    name: "Pam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pmn",
    individual_languages: &[
    ],
};


pub const PMO: LanguageCode = LanguageCode {
    name: "Pom",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pmo",
    individual_languages: &[
    ],
};


pub const PMQ: LanguageCode = LanguageCode {
    name: "Northern Pame",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pmq",
    individual_languages: &[
    ],
};


pub const PMR: LanguageCode = LanguageCode {
    name: "Paynamar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pmr",
    individual_languages: &[
    ],
};


pub const PMS: LanguageCode = LanguageCode {
    name: "Piemontese",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pms",
    individual_languages: &[
    ],
};


pub const PMT: LanguageCode = LanguageCode {
    name: "Tuamotuan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pmt",
    individual_languages: &[
    ],
};


pub const PMW: LanguageCode = LanguageCode {
    name: "Plains Miwok",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pmw",
    individual_languages: &[
    ],
};


pub const PMX: LanguageCode = LanguageCode {
    name: "Poumei Naga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pmx",
    individual_languages: &[
    ],
};


pub const PMY: LanguageCode = LanguageCode {
    name: "Papuan Malay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pmy",
    individual_languages: &[
    ],
};


pub const PMZ: LanguageCode = LanguageCode {
    name: "Southern Pame",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pmz",
    individual_languages: &[
    ],
};


pub const PNA: LanguageCode = LanguageCode {
    name: "Punan Bah-Biau",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pna",
    individual_languages: &[
    ],
};


pub const PNB: LanguageCode = LanguageCode {
    name: "Western Panjabi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pnb",
    individual_languages: &[
    ],
};


pub const PNC: LanguageCode = LanguageCode {
    name: "Pannei",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pnc",
    individual_languages: &[
    ],
};


pub const PND: LanguageCode = LanguageCode {
    name: "Mpinda",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pnd",
    individual_languages: &[
    ],
};


pub const PNE: LanguageCode = LanguageCode {
    name: "Western Penan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pne",
    individual_languages: &[
    ],
};


pub const PNG: LanguageCode = LanguageCode {
    name: "Pangu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "png",
    individual_languages: &[
    ],
};


pub const PNH: LanguageCode = LanguageCode {
    name: "Penrhyn",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pnh",
    individual_languages: &[
    ],
};


pub const PNI: LanguageCode = LanguageCode {
    name: "Aoheng",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pni",
    individual_languages: &[
    ],
};


pub const PNJ: LanguageCode = LanguageCode {
    name: "Pinjarup",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pnj",
    individual_languages: &[
    ],
};


pub const PNK: LanguageCode = LanguageCode {
    name: "Paunaka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pnk",
    individual_languages: &[
    ],
};


pub const PNL: LanguageCode = LanguageCode {
    name: "Paleni",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pnl",
    individual_languages: &[
    ],
};


pub const PNM: LanguageCode = LanguageCode {
    name: "Punan Batu 1",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pnm",
    individual_languages: &[
    ],
};


pub const PNN: LanguageCode = LanguageCode {
    name: "Pinai-Hagahai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pnn",
    individual_languages: &[
    ],
};


pub const PNO: LanguageCode = LanguageCode {
    name: "Panobo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pno",
    individual_languages: &[
    ],
};


pub const PNP: LanguageCode = LanguageCode {
    name: "Pancana",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pnp",
    individual_languages: &[
    ],
};


pub const PNQ: LanguageCode = LanguageCode {
    name: "Pana (Burkina Faso)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pnq",
    individual_languages: &[
    ],
};


pub const PNR: LanguageCode = LanguageCode {
    name: "Panim",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pnr",
    individual_languages: &[
    ],
};


pub const PNS: LanguageCode = LanguageCode {
    name: "Ponosakan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pns",
    individual_languages: &[
    ],
};


pub const PNT: LanguageCode = LanguageCode {
    name: "Pontic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pnt",
    individual_languages: &[
    ],
};


pub const PNU: LanguageCode = LanguageCode {
    name: "Jiongnai Bunu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pnu",
    individual_languages: &[
    ],
};


pub const PNV: LanguageCode = LanguageCode {
    name: "Pinigura",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pnv",
    individual_languages: &[
    ],
};


pub const PNW: LanguageCode = LanguageCode {
    name: "Banyjima",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pnw",
    individual_languages: &[
    ],
};


pub const PNX: LanguageCode = LanguageCode {
    name: "Phong-Kniang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pnx",
    individual_languages: &[
    ],
};


pub const PNY: LanguageCode = LanguageCode {
    name: "Pinyin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pny",
    individual_languages: &[
    ],
};


pub const PNZ: LanguageCode = LanguageCode {
    name: "Pana (Central African Republic)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pnz",
    individual_languages: &[
    ],
};


pub const POC: LanguageCode = LanguageCode {
    name: "Poqomam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "poc",
    individual_languages: &[
    ],
};


pub const POE: LanguageCode = LanguageCode {
    name: "San Juan Atzingo Popoloca",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "poe",
    individual_languages: &[
    ],
};


pub const POF: LanguageCode = LanguageCode {
    name: "Poke",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pof",
    individual_languages: &[
    ],
};


pub const POG: LanguageCode = LanguageCode {
    name: "Potiguára",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pog",
    individual_languages: &[
    ],
};


pub const POH: LanguageCode = LanguageCode {
    name: "Poqomchi'",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "poh",
    individual_languages: &[
    ],
};


pub const POI: LanguageCode = LanguageCode {
    name: "Highland Popoluca",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "poi",
    individual_languages: &[
    ],
};


pub const POK: LanguageCode = LanguageCode {
    name: "Pokangá",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pok",
    individual_languages: &[
    ],
};


pub const POM: LanguageCode = LanguageCode {
    name: "Southeastern Pomo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pom",
    individual_languages: &[
    ],
};


pub const POO: LanguageCode = LanguageCode {
    name: "Central Pomo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "poo",
    individual_languages: &[
    ],
};


pub const POP: LanguageCode = LanguageCode {
    name: "Pwapwâ",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pop",
    individual_languages: &[
    ],
};


pub const POQ: LanguageCode = LanguageCode {
    name: "Texistepec Popoluca",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "poq",
    individual_languages: &[
    ],
};


pub const POS: LanguageCode = LanguageCode {
    name: "Sayula Popoluca",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pos",
    individual_languages: &[
    ],
};


pub const POT: LanguageCode = LanguageCode {
    name: "Potawatomi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pot",
    individual_languages: &[
    ],
};


pub const POV: LanguageCode = LanguageCode {
    name: "Upper Guinea Crioulo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pov",
    individual_languages: &[
    ],
};


pub const POW: LanguageCode = LanguageCode {
    name: "San Felipe Otlaltepec Popoloca",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pow",
    individual_languages: &[
    ],
};


pub const POX: LanguageCode = LanguageCode {
    name: "Polabian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pox",
    individual_languages: &[
    ],
};


pub const POY: LanguageCode = LanguageCode {
    name: "Pogolo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "poy",
    individual_languages: &[
    ],
};


pub const PPE: LanguageCode = LanguageCode {
    name: "Papi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ppe",
    individual_languages: &[
    ],
};


pub const PPI: LanguageCode = LanguageCode {
    name: "Paipai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ppi",
    individual_languages: &[
    ],
};


pub const PPK: LanguageCode = LanguageCode {
    name: "Uma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ppk",
    individual_languages: &[
    ],
};


pub const PPL: LanguageCode = LanguageCode {
    name: "Pipil",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ppl",
    individual_languages: &[
    ],
};


pub const PPM: LanguageCode = LanguageCode {
    name: "Papuma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ppm",
    individual_languages: &[
    ],
};


pub const PPN: LanguageCode = LanguageCode {
    name: "Papapana",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ppn",
    individual_languages: &[
    ],
};


pub const PPO: LanguageCode = LanguageCode {
    name: "Folopa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ppo",
    individual_languages: &[
    ],
};


pub const PPP: LanguageCode = LanguageCode {
    name: "Pelende",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ppp",
    individual_languages: &[
    ],
};


pub const PPQ: LanguageCode = LanguageCode {
    name: "Pei",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ppq",
    individual_languages: &[
    ],
};


pub const PPS: LanguageCode = LanguageCode {
    name: "San Luís Temalacayuca Popoloca",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pps",
    individual_languages: &[
    ],
};


pub const PPT: LanguageCode = LanguageCode {
    name: "Pare",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ppt",
    individual_languages: &[
    ],
};


pub const PPU: LanguageCode = LanguageCode {
    name: "Papora",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ppu",
    individual_languages: &[
    ],
};


pub const PQA: LanguageCode = LanguageCode {
    name: "Pa'a",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pqa",
    individual_languages: &[
    ],
};


pub const PQM: LanguageCode = LanguageCode {
    name: "Malecite-Passamaquoddy",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pqm",
    individual_languages: &[
    ],
};


pub const PRC: LanguageCode = LanguageCode {
    name: "Parachi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "prc",
    individual_languages: &[
    ],
};


pub const PRD: LanguageCode = LanguageCode {
    name: "Parsi-Dari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "prd",
    individual_languages: &[
    ],
};


pub const PRE: LanguageCode = LanguageCode {
    name: "Principense",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pre",
    individual_languages: &[
    ],
};


pub const PRF: LanguageCode = LanguageCode {
    name: "Paranan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "prf",
    individual_languages: &[
    ],
};


pub const PRG: LanguageCode = LanguageCode {
    name: "Prussian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "prg",
    individual_languages: &[
    ],
};


pub const PRH: LanguageCode = LanguageCode {
    name: "Porohanon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "prh",
    individual_languages: &[
    ],
};


pub const PRI: LanguageCode = LanguageCode {
    name: "Paicî",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pri",
    individual_languages: &[
    ],
};


pub const PRK: LanguageCode = LanguageCode {
    name: "Parauk",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "prk",
    individual_languages: &[
    ],
};


pub const PRL: LanguageCode = LanguageCode {
    name: "Peruvian Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "prl",
    individual_languages: &[
    ],
};


pub const PRM: LanguageCode = LanguageCode {
    name: "Kibiri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "prm",
    individual_languages: &[
    ],
};


pub const PRN: LanguageCode = LanguageCode {
    name: "Prasuni",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "prn",
    individual_languages: &[
    ],
};


pub const PRP: LanguageCode = LanguageCode {
    name: "Parsi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "prp",
    individual_languages: &[
    ],
};


pub const PRQ: LanguageCode = LanguageCode {
    name: "Ashéninka Perené",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "prq",
    individual_languages: &[
    ],
};


pub const PRR: LanguageCode = LanguageCode {
    name: "Puri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "prr",
    individual_languages: &[
    ],
};


pub const PRS: LanguageCode = LanguageCode {
    name: "Dari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "prs",
    individual_languages: &[
    ],
};


pub const PRT: LanguageCode = LanguageCode {
    name: "Phai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "prt",
    individual_languages: &[
    ],
};


pub const PRU: LanguageCode = LanguageCode {
    name: "Puragi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pru",
    individual_languages: &[
    ],
};


pub const PRW: LanguageCode = LanguageCode {
    name: "Parawen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "prw",
    individual_languages: &[
    ],
};


pub const PRX: LanguageCode = LanguageCode {
    name: "Purik",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "prx",
    individual_languages: &[
    ],
};


pub const PRZ: LanguageCode = LanguageCode {
    name: "Providencia Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "prz",
    individual_languages: &[
    ],
};


pub const PSA: LanguageCode = LanguageCode {
    name: "Asue Awyu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "psa",
    individual_languages: &[
    ],
};


pub const PSC: LanguageCode = LanguageCode {
    name: "Iranian Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "psc",
    individual_languages: &[
    ],
};


pub const PSD: LanguageCode = LanguageCode {
    name: "Plains Indian Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "psd",
    individual_languages: &[
    ],
};


pub const PSE: LanguageCode = LanguageCode {
    name: "Central Malay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pse",
    individual_languages: &[
    ],
};


pub const PSG: LanguageCode = LanguageCode {
    name: "Penang Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "psg",
    individual_languages: &[
    ],
};


pub const PSH: LanguageCode = LanguageCode {
    name: "Southwest Pashai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "psh",
    individual_languages: &[
    ],
};


pub const PSI: LanguageCode = LanguageCode {
    name: "Southeast Pashai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "psi",
    individual_languages: &[
    ],
};


pub const PSL: LanguageCode = LanguageCode {
    name: "Puerto Rican Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "psl",
    individual_languages: &[
    ],
};


pub const PSM: LanguageCode = LanguageCode {
    name: "Pauserna",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "psm",
    individual_languages: &[
    ],
};


pub const PSN: LanguageCode = LanguageCode {
    name: "Panasuan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "psn",
    individual_languages: &[
    ],
};


pub const PSO: LanguageCode = LanguageCode {
    name: "Polish Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pso",
    individual_languages: &[
    ],
};


pub const PSP: LanguageCode = LanguageCode {
    name: "Philippine Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "psp",
    individual_languages: &[
    ],
};


pub const PSQ: LanguageCode = LanguageCode {
    name: "Pasi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "psq",
    individual_languages: &[
    ],
};


pub const PSR: LanguageCode = LanguageCode {
    name: "Portuguese Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "psr",
    individual_languages: &[
    ],
};


pub const PSS: LanguageCode = LanguageCode {
    name: "Kaulong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pss",
    individual_languages: &[
    ],
};


pub const PST: LanguageCode = LanguageCode {
    name: "Central Pashto",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pst",
    individual_languages: &[
    ],
};


pub const PSU: LanguageCode = LanguageCode {
    name: "Sauraseni Prākrit",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "psu",
    individual_languages: &[
    ],
};


pub const PSW: LanguageCode = LanguageCode {
    name: "Port Sandwich",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "psw",
    individual_languages: &[
    ],
};


pub const PSY: LanguageCode = LanguageCode {
    name: "Piscataway",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "psy",
    individual_languages: &[
    ],
};


pub const PTA: LanguageCode = LanguageCode {
    name: "Pai Tavytera",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pta",
    individual_languages: &[
    ],
};


pub const PTH: LanguageCode = LanguageCode {
    name: "Pataxó Hã-Ha-Hãe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pth",
    individual_languages: &[
    ],
};


pub const PTI: LanguageCode = LanguageCode {
    name: "Pindiini",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pti",
    individual_languages: &[
    ],
};


pub const PTN: LanguageCode = LanguageCode {
    name: "Patani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ptn",
    individual_languages: &[
    ],
};


pub const PTO: LanguageCode = LanguageCode {
    name: "Zo'é",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pto",
    individual_languages: &[
    ],
};


pub const PTP: LanguageCode = LanguageCode {
    name: "Patep",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ptp",
    individual_languages: &[
    ],
};


pub const PTQ: LanguageCode = LanguageCode {
    name: "Pattapu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ptq",
    individual_languages: &[
    ],
};


pub const PTR: LanguageCode = LanguageCode {
    name: "Piamatsina",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ptr",
    individual_languages: &[
    ],
};


pub const PTT: LanguageCode = LanguageCode {
    name: "Enrekang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ptt",
    individual_languages: &[
    ],
};


pub const PTU: LanguageCode = LanguageCode {
    name: "Bambam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ptu",
    individual_languages: &[
    ],
};


pub const PTV: LanguageCode = LanguageCode {
    name: "Port Vato",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ptv",
    individual_languages: &[
    ],
};


pub const PTW: LanguageCode = LanguageCode {
    name: "Pentlatch",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ptw",
    individual_languages: &[
    ],
};


pub const PTY: LanguageCode = LanguageCode {
    name: "Pathiya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pty",
    individual_languages: &[
    ],
};


pub const PUA: LanguageCode = LanguageCode {
    name: "Western Highland Purepecha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pua",
    individual_languages: &[
    ],
};


pub const PUB: LanguageCode = LanguageCode {
    name: "Purum",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pub",
    individual_languages: &[
    ],
};


pub const PUC: LanguageCode = LanguageCode {
    name: "Punan Merap",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "puc",
    individual_languages: &[
    ],
};


pub const PUD: LanguageCode = LanguageCode {
    name: "Punan Aput",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pud",
    individual_languages: &[
    ],
};


pub const PUE: LanguageCode = LanguageCode {
    name: "Puelche",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pue",
    individual_languages: &[
    ],
};


pub const PUF: LanguageCode = LanguageCode {
    name: "Punan Merah",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "puf",
    individual_languages: &[
    ],
};


pub const PUG: LanguageCode = LanguageCode {
    name: "Phuie",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pug",
    individual_languages: &[
    ],
};


pub const PUI: LanguageCode = LanguageCode {
    name: "Puinave",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pui",
    individual_languages: &[
    ],
};


pub const PUJ: LanguageCode = LanguageCode {
    name: "Punan Tubu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "puj",
    individual_languages: &[
    ],
};


pub const PUM: LanguageCode = LanguageCode {
    name: "Puma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pum",
    individual_languages: &[
    ],
};


pub const PUO: LanguageCode = LanguageCode {
    name: "Puoc",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "puo",
    individual_languages: &[
    ],
};


pub const PUP: LanguageCode = LanguageCode {
    name: "Pulabu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pup",
    individual_languages: &[
    ],
};


pub const PUQ: LanguageCode = LanguageCode {
    name: "Puquina",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "puq",
    individual_languages: &[
    ],
};


pub const PUR: LanguageCode = LanguageCode {
    name: "Puruborá",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pur",
    individual_languages: &[
    ],
};


pub const PUT: LanguageCode = LanguageCode {
    name: "Putoh",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "put",
    individual_languages: &[
    ],
};


pub const PUU: LanguageCode = LanguageCode {
    name: "Punu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "puu",
    individual_languages: &[
    ],
};


pub const PUW: LanguageCode = LanguageCode {
    name: "Puluwatese",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "puw",
    individual_languages: &[
    ],
};


pub const PUX: LanguageCode = LanguageCode {
    name: "Puare",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pux",
    individual_languages: &[
    ],
};


pub const PUY: LanguageCode = LanguageCode {
    name: "Purisimeño",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "puy",
    individual_languages: &[
    ],
};


pub const PWA: LanguageCode = LanguageCode {
    name: "Pawaia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pwa",
    individual_languages: &[
    ],
};


pub const PWB: LanguageCode = LanguageCode {
    name: "Panawa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pwb",
    individual_languages: &[
    ],
};


pub const PWG: LanguageCode = LanguageCode {
    name: "Gapapaiwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pwg",
    individual_languages: &[
    ],
};


pub const PWI: LanguageCode = LanguageCode {
    name: "Patwin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pwi",
    individual_languages: &[
    ],
};


pub const PWM: LanguageCode = LanguageCode {
    name: "Molbog",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pwm",
    individual_languages: &[
    ],
};


pub const PWN: LanguageCode = LanguageCode {
    name: "Paiwan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pwn",
    individual_languages: &[
    ],
};


pub const PWO: LanguageCode = LanguageCode {
    name: "Pwo Western Karen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pwo",
    individual_languages: &[
    ],
};


pub const PWR: LanguageCode = LanguageCode {
    name: "Powari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pwr",
    individual_languages: &[
    ],
};


pub const PWW: LanguageCode = LanguageCode {
    name: "Pwo Northern Karen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pww",
    individual_languages: &[
    ],
};


pub const PXM: LanguageCode = LanguageCode {
    name: "Quetzaltepec Mixe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pxm",
    individual_languages: &[
    ],
};


pub const PYE: LanguageCode = LanguageCode {
    name: "Pye Krumen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pye",
    individual_languages: &[
    ],
};


pub const PYM: LanguageCode = LanguageCode {
    name: "Fyam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pym",
    individual_languages: &[
    ],
};


pub const PYN: LanguageCode = LanguageCode {
    name: "Poyanáwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pyn",
    individual_languages: &[
    ],
};


pub const PYS: LanguageCode = LanguageCode {
    name: "Paraguayan Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pys",
    individual_languages: &[
    ],
};


pub const PYU: LanguageCode = LanguageCode {
    name: "Puyuma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pyu",
    individual_languages: &[
    ],
};


pub const PYX: LanguageCode = LanguageCode {
    name: "Pyu (Myanmar)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pyx",
    individual_languages: &[
    ],
};


pub const PYY: LanguageCode = LanguageCode {
    name: "Pyen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pyy",
    individual_languages: &[
    ],
};


pub const PZH: LanguageCode = LanguageCode {
    name: "Pazeh",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pzh",
    individual_languages: &[
    ],
};


pub const PZN: LanguageCode = LanguageCode {
    name: "Jejara Naga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "pzn",
    individual_languages: &[
    ],
};


pub const QUA: LanguageCode = LanguageCode {
    name: "Quapaw",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "qua",
    individual_languages: &[
    ],
};


pub const QUB: LanguageCode = LanguageCode {
    name: "Huallaga Huánuco Quechua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "qub",
    individual_languages: &[
    ],
};


pub const QUC: LanguageCode = LanguageCode {
    name: "K'iche'",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "quc",
    individual_languages: &[
    ],
};


pub const QUD: LanguageCode = LanguageCode {
    name: "Calderón Highland Quichua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "qud",
    individual_languages: &[
    ],
};


pub const QUF: LanguageCode = LanguageCode {
    name: "Lambayeque Quechua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "quf",
    individual_languages: &[
    ],
};


pub const QUG: LanguageCode = LanguageCode {
    name: "Chimborazo Highland Quichua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "qug",
    individual_languages: &[
    ],
};


pub const QUH: LanguageCode = LanguageCode {
    name: "South Bolivian Quechua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "quh",
    individual_languages: &[
    ],
};


pub const QUI: LanguageCode = LanguageCode {
    name: "Quileute",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "qui",
    individual_languages: &[
    ],
};


pub const QUK: LanguageCode = LanguageCode {
    name: "Chachapoyas Quechua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "quk",
    individual_languages: &[
    ],
};


pub const QUL: LanguageCode = LanguageCode {
    name: "North Bolivian Quechua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "qul",
    individual_languages: &[
    ],
};


pub const QUM: LanguageCode = LanguageCode {
    name: "Sipacapense",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "qum",
    individual_languages: &[
    ],
};


pub const QUN: LanguageCode = LanguageCode {
    name: "Quinault",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "qun",
    individual_languages: &[
    ],
};


pub const QUP: LanguageCode = LanguageCode {
    name: "Southern Pastaza Quechua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "qup",
    individual_languages: &[
    ],
};


pub const QUQ: LanguageCode = LanguageCode {
    name: "Quinqui",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "quq",
    individual_languages: &[
    ],
};


pub const QUR: LanguageCode = LanguageCode {
    name: "Yanahuanca Pasco Quechua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "qur",
    individual_languages: &[
    ],
};


pub const QUS: LanguageCode = LanguageCode {
    name: "Santiago del Estero Quichua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "qus",
    individual_languages: &[
    ],
};


pub const QUV: LanguageCode = LanguageCode {
    name: "Sacapulteco",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "quv",
    individual_languages: &[
    ],
};


pub const QUW: LanguageCode = LanguageCode {
    name: "Tena Lowland Quichua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "quw",
    individual_languages: &[
    ],
};


pub const QUX: LanguageCode = LanguageCode {
    name: "Yauyos Quechua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "qux",
    individual_languages: &[
    ],
};


pub const QUY: LanguageCode = LanguageCode {
    name: "Ayacucho Quechua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "quy",
    individual_languages: &[
    ],
};


pub const QUZ: LanguageCode = LanguageCode {
    name: "Cusco Quechua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "quz",
    individual_languages: &[
    ],
};


pub const QVA: LanguageCode = LanguageCode {
    name: "Ambo-Pasco Quechua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "qva",
    individual_languages: &[
    ],
};


pub const QVC: LanguageCode = LanguageCode {
    name: "Cajamarca Quechua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "qvc",
    individual_languages: &[
    ],
};


pub const QVE: LanguageCode = LanguageCode {
    name: "Eastern Apurímac Quechua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "qve",
    individual_languages: &[
    ],
};


pub const QVH: LanguageCode = LanguageCode {
    name: "Huamalíes-Dos de Mayo Huánuco Quechua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "qvh",
    individual_languages: &[
    ],
};


pub const QVI: LanguageCode = LanguageCode {
    name: "Imbabura Highland Quichua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "qvi",
    individual_languages: &[
    ],
};


pub const QVJ: LanguageCode = LanguageCode {
    name: "Loja Highland Quichua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "qvj",
    individual_languages: &[
    ],
};


pub const QVL: LanguageCode = LanguageCode {
    name: "Cajatambo North Lima Quechua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "qvl",
    individual_languages: &[
    ],
};


pub const QVM: LanguageCode = LanguageCode {
    name: "Margos-Yarowilca-Lauricocha Quechua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "qvm",
    individual_languages: &[
    ],
};


pub const QVN: LanguageCode = LanguageCode {
    name: "North Junín Quechua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "qvn",
    individual_languages: &[
    ],
};


pub const QVO: LanguageCode = LanguageCode {
    name: "Napo Lowland Quechua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "qvo",
    individual_languages: &[
    ],
};


pub const QVP: LanguageCode = LanguageCode {
    name: "Pacaraos Quechua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "qvp",
    individual_languages: &[
    ],
};


pub const QVS: LanguageCode = LanguageCode {
    name: "San Martín Quechua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "qvs",
    individual_languages: &[
    ],
};


pub const QVW: LanguageCode = LanguageCode {
    name: "Huaylla Wanca Quechua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "qvw",
    individual_languages: &[
    ],
};


pub const QVY: LanguageCode = LanguageCode {
    name: "Queyu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "qvy",
    individual_languages: &[
    ],
};


pub const QVZ: LanguageCode = LanguageCode {
    name: "Northern Pastaza Quichua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "qvz",
    individual_languages: &[
    ],
};


pub const QWA: LanguageCode = LanguageCode {
    name: "Corongo Ancash Quechua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "qwa",
    individual_languages: &[
    ],
};


pub const QWC: LanguageCode = LanguageCode {
    name: "Classical Quechua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "qwc",
    individual_languages: &[
    ],
};


pub const QWH: LanguageCode = LanguageCode {
    name: "Huaylas Ancash Quechua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "qwh",
    individual_languages: &[
    ],
};


pub const QWM: LanguageCode = LanguageCode {
    name: "Kuman (Russia)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "qwm",
    individual_languages: &[
    ],
};


pub const QWS: LanguageCode = LanguageCode {
    name: "Sihuas Ancash Quechua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "qws",
    individual_languages: &[
    ],
};


pub const QWT: LanguageCode = LanguageCode {
    name: "Kwalhioqua-Tlatskanai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "qwt",
    individual_languages: &[
    ],
};


pub const QXA: LanguageCode = LanguageCode {
    name: "Chiquián Ancash Quechua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "qxa",
    individual_languages: &[
    ],
};


pub const QXC: LanguageCode = LanguageCode {
    name: "Chincha Quechua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "qxc",
    individual_languages: &[
    ],
};


pub const QXH: LanguageCode = LanguageCode {
    name: "Panao Huánuco Quechua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "qxh",
    individual_languages: &[
    ],
};


pub const QXL: LanguageCode = LanguageCode {
    name: "Salasaca Highland Quichua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "qxl",
    individual_languages: &[
    ],
};


pub const QXN: LanguageCode = LanguageCode {
    name: "Northern Conchucos Ancash Quechua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "qxn",
    individual_languages: &[
    ],
};


pub const QXO: LanguageCode = LanguageCode {
    name: "Southern Conchucos Ancash Quechua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "qxo",
    individual_languages: &[
    ],
};


pub const QXP: LanguageCode = LanguageCode {
    name: "Puno Quechua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "qxp",
    individual_languages: &[
    ],
};


pub const QXQ: LanguageCode = LanguageCode {
    name: "Qashqa'i",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "qxq",
    individual_languages: &[
    ],
};


pub const QXR: LanguageCode = LanguageCode {
    name: "Cañar Highland Quichua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "qxr",
    individual_languages: &[
    ],
};


pub const QXS: LanguageCode = LanguageCode {
    name: "Southern Qiang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "qxs",
    individual_languages: &[
    ],
};


pub const QXT: LanguageCode = LanguageCode {
    name: "Santa Ana de Tusi Pasco Quechua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "qxt",
    individual_languages: &[
    ],
};


pub const QXU: LanguageCode = LanguageCode {
    name: "Arequipa-La Unión Quechua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "qxu",
    individual_languages: &[
    ],
};


pub const QXW: LanguageCode = LanguageCode {
    name: "Jauja Wanca Quechua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "qxw",
    individual_languages: &[
    ],
};


pub const QYA: LanguageCode = LanguageCode {
    name: "Quenya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "qya",
    individual_languages: &[
    ],
};


pub const QYP: LanguageCode = LanguageCode {
    name: "Quiripi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "qyp",
    individual_languages: &[
    ],
};


pub const RAA: LanguageCode = LanguageCode {
    name: "Dungmali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "raa",
    individual_languages: &[
    ],
};


pub const RAB: LanguageCode = LanguageCode {
    name: "Camling",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rab",
    individual_languages: &[
    ],
};


pub const RAC: LanguageCode = LanguageCode {
    name: "Rasawa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rac",
    individual_languages: &[
    ],
};


pub const RAD: LanguageCode = LanguageCode {
    name: "Rade",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rad",
    individual_languages: &[
    ],
};


pub const RAF: LanguageCode = LanguageCode {
    name: "Western Meohang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "raf",
    individual_languages: &[
    ],
};


pub const RAG: LanguageCode = LanguageCode {
    name: "Logooli",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rag",
    individual_languages: &[
    ],
};


pub const RAH: LanguageCode = LanguageCode {
    name: "Rabha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rah",
    individual_languages: &[
    ],
};


pub const RAI: LanguageCode = LanguageCode {
    name: "Ramoaaina",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rai",
    individual_languages: &[
    ],
};


pub const RAK: LanguageCode = LanguageCode {
    name: "Tulu-Bohuai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rak",
    individual_languages: &[
    ],
};


pub const RAL: LanguageCode = LanguageCode {
    name: "Ralte",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ral",
    individual_languages: &[
    ],
};


pub const RAM: LanguageCode = LanguageCode {
    name: "Canela",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ram",
    individual_languages: &[
    ],
};


pub const RAN: LanguageCode = LanguageCode {
    name: "Riantana",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ran",
    individual_languages: &[
    ],
};


pub const RAO: LanguageCode = LanguageCode {
    name: "Rao",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rao",
    individual_languages: &[
    ],
};


pub const RAQ: LanguageCode = LanguageCode {
    name: "Saam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "raq",
    individual_languages: &[
    ],
};


pub const RAS: LanguageCode = LanguageCode {
    name: "Tegali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ras",
    individual_languages: &[
    ],
};


pub const RAT: LanguageCode = LanguageCode {
    name: "Razajerdi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rat",
    individual_languages: &[
    ],
};


pub const RAU: LanguageCode = LanguageCode {
    name: "Raute",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rau",
    individual_languages: &[
    ],
};


pub const RAV: LanguageCode = LanguageCode {
    name: "Sampang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rav",
    individual_languages: &[
    ],
};


pub const RAW: LanguageCode = LanguageCode {
    name: "Rawang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "raw",
    individual_languages: &[
    ],
};


pub const RAX: LanguageCode = LanguageCode {
    name: "Rang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rax",
    individual_languages: &[
    ],
};


pub const RAY: LanguageCode = LanguageCode {
    name: "Rapa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ray",
    individual_languages: &[
    ],
};


pub const RAZ: LanguageCode = LanguageCode {
    name: "Rahambuu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "raz",
    individual_languages: &[
    ],
};


pub const RBB: LanguageCode = LanguageCode {
    name: "Rumai Palaung",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rbb",
    individual_languages: &[
    ],
};


pub const RBK: LanguageCode = LanguageCode {
    name: "Northern Bontok",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rbk",
    individual_languages: &[
    ],
};


pub const RBL: LanguageCode = LanguageCode {
    name: "Miraya Bikol",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rbl",
    individual_languages: &[
    ],
};


pub const RBP: LanguageCode = LanguageCode {
    name: "Barababaraba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rbp",
    individual_languages: &[
    ],
};


pub const RCF: LanguageCode = LanguageCode {
    name: "Réunion Creole French",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rcf",
    individual_languages: &[
    ],
};


pub const RDB: LanguageCode = LanguageCode {
    name: "Rudbari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rdb",
    individual_languages: &[
    ],
};


pub const REA: LanguageCode = LanguageCode {
    name: "Rerau",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rea",
    individual_languages: &[
    ],
};


pub const REB: LanguageCode = LanguageCode {
    name: "Rembong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "reb",
    individual_languages: &[
    ],
};


pub const REE: LanguageCode = LanguageCode {
    name: "Rejang Kayan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ree",
    individual_languages: &[
    ],
};


pub const REG: LanguageCode = LanguageCode {
    name: "Kara (Tanzania)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "reg",
    individual_languages: &[
    ],
};


pub const REI: LanguageCode = LanguageCode {
    name: "Reli",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rei",
    individual_languages: &[
    ],
};


pub const REJ: LanguageCode = LanguageCode {
    name: "Rejang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rej",
    individual_languages: &[
    ],
};


pub const REL: LanguageCode = LanguageCode {
    name: "Rendille",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rel",
    individual_languages: &[
    ],
};


pub const REM: LanguageCode = LanguageCode {
    name: "Remo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rem",
    individual_languages: &[
    ],
};


pub const REN: LanguageCode = LanguageCode {
    name: "Rengao",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ren",
    individual_languages: &[
    ],
};


pub const RER: LanguageCode = LanguageCode {
    name: "Rer Bare",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rer",
    individual_languages: &[
    ],
};


pub const RES: LanguageCode = LanguageCode {
    name: "Reshe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "res",
    individual_languages: &[
    ],
};


pub const RET: LanguageCode = LanguageCode {
    name: "Retta",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ret",
    individual_languages: &[
    ],
};


pub const REY: LanguageCode = LanguageCode {
    name: "Reyesano",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rey",
    individual_languages: &[
    ],
};


pub const RGA: LanguageCode = LanguageCode {
    name: "Roria",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rga",
    individual_languages: &[
    ],
};


pub const RGE: LanguageCode = LanguageCode {
    name: "Romano-Greek",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rge",
    individual_languages: &[
    ],
};


pub const RGK: LanguageCode = LanguageCode {
    name: "Rangkas",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rgk",
    individual_languages: &[
    ],
};


pub const RGN: LanguageCode = LanguageCode {
    name: "Romagnol",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rgn",
    individual_languages: &[
    ],
};


pub const RGR: LanguageCode = LanguageCode {
    name: "Resígaro",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rgr",
    individual_languages: &[
    ],
};


pub const RGS: LanguageCode = LanguageCode {
    name: "Southern Roglai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rgs",
    individual_languages: &[
    ],
};


pub const RGU: LanguageCode = LanguageCode {
    name: "Ringgou",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rgu",
    individual_languages: &[
    ],
};


pub const RHG: LanguageCode = LanguageCode {
    name: "Rohingya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rhg",
    individual_languages: &[
    ],
};


pub const RHP: LanguageCode = LanguageCode {
    name: "Yahang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rhp",
    individual_languages: &[
    ],
};


pub const RIA: LanguageCode = LanguageCode {
    name: "Riang (India)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ria",
    individual_languages: &[
    ],
};


pub const RIB: LanguageCode = LanguageCode {
    name: "Bribri Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rib",
    individual_languages: &[
    ],
};


pub const RIF: LanguageCode = LanguageCode {
    name: "Tarifit",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rif",
    individual_languages: &[
    ],
};


pub const RIL: LanguageCode = LanguageCode {
    name: "Riang Lang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ril",
    individual_languages: &[
    ],
};


pub const RIM: LanguageCode = LanguageCode {
    name: "Nyaturu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rim",
    individual_languages: &[
    ],
};


pub const RIN: LanguageCode = LanguageCode {
    name: "Nungu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rin",
    individual_languages: &[
    ],
};


pub const RIR: LanguageCode = LanguageCode {
    name: "Ribun",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rir",
    individual_languages: &[
    ],
};


pub const RIT: LanguageCode = LanguageCode {
    name: "Ritharrngu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rit",
    individual_languages: &[
    ],
};


pub const RIU: LanguageCode = LanguageCode {
    name: "Riung",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "riu",
    individual_languages: &[
    ],
};


pub const RJG: LanguageCode = LanguageCode {
    name: "Rajong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rjg",
    individual_languages: &[
    ],
};


pub const RJI: LanguageCode = LanguageCode {
    name: "Raji",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rji",
    individual_languages: &[
    ],
};


pub const RJS: LanguageCode = LanguageCode {
    name: "Rajbanshi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rjs",
    individual_languages: &[
    ],
};


pub const RKA: LanguageCode = LanguageCode {
    name: "Kraol",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rka",
    individual_languages: &[
    ],
};


pub const RKB: LanguageCode = LanguageCode {
    name: "Rikbaktsa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rkb",
    individual_languages: &[
    ],
};


pub const RKH: LanguageCode = LanguageCode {
    name: "Rakahanga-Manihiki",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rkh",
    individual_languages: &[
    ],
};


pub const RKI: LanguageCode = LanguageCode {
    name: "Rakhine",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rki",
    individual_languages: &[
    ],
};


pub const RKM: LanguageCode = LanguageCode {
    name: "Marka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rkm",
    individual_languages: &[
    ],
};


pub const RKT: LanguageCode = LanguageCode {
    name: "Rangpuri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rkt",
    individual_languages: &[
    ],
};


pub const RKW: LanguageCode = LanguageCode {
    name: "Arakwal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rkw",
    individual_languages: &[
    ],
};


pub const RMA: LanguageCode = LanguageCode {
    name: "Rama",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rma",
    individual_languages: &[
    ],
};


pub const RMB: LanguageCode = LanguageCode {
    name: "Rembarrnga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rmb",
    individual_languages: &[
    ],
};


pub const RMC: LanguageCode = LanguageCode {
    name: "Carpathian Romani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rmc",
    individual_languages: &[
    ],
};


pub const RMD: LanguageCode = LanguageCode {
    name: "Traveller Danish",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rmd",
    individual_languages: &[
    ],
};


pub const RME: LanguageCode = LanguageCode {
    name: "Angloromani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rme",
    individual_languages: &[
    ],
};


pub const RMF: LanguageCode = LanguageCode {
    name: "Kalo Finnish Romani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rmf",
    individual_languages: &[
    ],
};


pub const RMG: LanguageCode = LanguageCode {
    name: "Traveller Norwegian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rmg",
    individual_languages: &[
    ],
};


pub const RMH: LanguageCode = LanguageCode {
    name: "Murkim",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rmh",
    individual_languages: &[
    ],
};


pub const RMI: LanguageCode = LanguageCode {
    name: "Lomavren",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rmi",
    individual_languages: &[
    ],
};


pub const RMK: LanguageCode = LanguageCode {
    name: "Romkun",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rmk",
    individual_languages: &[
    ],
};


pub const RML: LanguageCode = LanguageCode {
    name: "Baltic Romani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rml",
    individual_languages: &[
    ],
};


pub const RMM: LanguageCode = LanguageCode {
    name: "Roma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rmm",
    individual_languages: &[
    ],
};


pub const RMN: LanguageCode = LanguageCode {
    name: "Balkan Romani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rmn",
    individual_languages: &[
    ],
};


pub const RMO: LanguageCode = LanguageCode {
    name: "Sinte Romani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rmo",
    individual_languages: &[
    ],
};


pub const RMP: LanguageCode = LanguageCode {
    name: "Rempi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rmp",
    individual_languages: &[
    ],
};


pub const RMQ: LanguageCode = LanguageCode {
    name: "Caló",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rmq",
    individual_languages: &[
    ],
};


pub const RMS: LanguageCode = LanguageCode {
    name: "Romanian Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rms",
    individual_languages: &[
    ],
};


pub const RMT: LanguageCode = LanguageCode {
    name: "Domari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rmt",
    individual_languages: &[
    ],
};


pub const RMU: LanguageCode = LanguageCode {
    name: "Tavringer Romani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rmu",
    individual_languages: &[
    ],
};


pub const RMV: LanguageCode = LanguageCode {
    name: "Romanova",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rmv",
    individual_languages: &[
    ],
};


pub const RMW: LanguageCode = LanguageCode {
    name: "Welsh Romani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rmw",
    individual_languages: &[
    ],
};


pub const RMX: LanguageCode = LanguageCode {
    name: "Romam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rmx",
    individual_languages: &[
    ],
};


pub const RMY: LanguageCode = LanguageCode {
    name: "Vlax Romani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rmy",
    individual_languages: &[
    ],
};


pub const RMZ: LanguageCode = LanguageCode {
    name: "Marma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rmz",
    individual_languages: &[
    ],
};


pub const RNB: LanguageCode = LanguageCode {
    name: "Brunca Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rnb",
    individual_languages: &[
    ],
};


pub const RND: LanguageCode = LanguageCode {
    name: "Ruund",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rnd",
    individual_languages: &[
    ],
};


pub const RNG: LanguageCode = LanguageCode {
    name: "Ronga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rng",
    individual_languages: &[
    ],
};


pub const RNL: LanguageCode = LanguageCode {
    name: "Ranglong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rnl",
    individual_languages: &[
    ],
};


pub const RNN: LanguageCode = LanguageCode {
    name: "Roon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rnn",
    individual_languages: &[
    ],
};


pub const RNP: LanguageCode = LanguageCode {
    name: "Rongpo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rnp",
    individual_languages: &[
    ],
};


pub const RNR: LanguageCode = LanguageCode {
    name: "Nari Nari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rnr",
    individual_languages: &[
    ],
};


pub const RNW: LanguageCode = LanguageCode {
    name: "Rungwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rnw",
    individual_languages: &[
    ],
};


pub const ROB: LanguageCode = LanguageCode {
    name: "Tae'",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rob",
    individual_languages: &[
    ],
};


pub const ROC: LanguageCode = LanguageCode {
    name: "Cacgia Roglai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "roc",
    individual_languages: &[
    ],
};


pub const ROD: LanguageCode = LanguageCode {
    name: "Rogo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rod",
    individual_languages: &[
    ],
};


pub const ROE: LanguageCode = LanguageCode {
    name: "Ronji",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "roe",
    individual_languages: &[
    ],
};


pub const ROF: LanguageCode = LanguageCode {
    name: "Rombo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rof",
    individual_languages: &[
    ],
};


pub const ROG: LanguageCode = LanguageCode {
    name: "Northern Roglai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rog",
    individual_languages: &[
    ],
};


pub const ROL: LanguageCode = LanguageCode {
    name: "Romblomanon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rol",
    individual_languages: &[
    ],
};


pub const ROO: LanguageCode = LanguageCode {
    name: "Rotokas",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "roo",
    individual_languages: &[
    ],
};


pub const ROP: LanguageCode = LanguageCode {
    name: "Kriol",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rop",
    individual_languages: &[
    ],
};


pub const ROR: LanguageCode = LanguageCode {
    name: "Rongga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ror",
    individual_languages: &[
    ],
};


pub const ROU: LanguageCode = LanguageCode {
    name: "Runga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rou",
    individual_languages: &[
    ],
};


pub const ROW: LanguageCode = LanguageCode {
    name: "Dela-Oenale",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "row",
    individual_languages: &[
    ],
};


pub const RPN: LanguageCode = LanguageCode {
    name: "Repanbitip",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rpn",
    individual_languages: &[
    ],
};


pub const RPT: LanguageCode = LanguageCode {
    name: "Rapting",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rpt",
    individual_languages: &[
    ],
};


pub const RRI: LanguageCode = LanguageCode {
    name: "Ririo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rri",
    individual_languages: &[
    ],
};


pub const RRO: LanguageCode = LanguageCode {
    name: "Waima",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rro",
    individual_languages: &[
    ],
};


pub const RRT: LanguageCode = LanguageCode {
    name: "Arritinngithigh",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rrt",
    individual_languages: &[
    ],
};


pub const RSB: LanguageCode = LanguageCode {
    name: "Romano-Serbian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rsb",
    individual_languages: &[
    ],
};


pub const RSK: LanguageCode = LanguageCode {
    name: "Ruthenian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rsk",
    individual_languages: &[
    ],
};


pub const RSL: LanguageCode = LanguageCode {
    name: "Russian Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rsl",
    individual_languages: &[
    ],
};


pub const RSM: LanguageCode = LanguageCode {
    name: "Miriwoong Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rsm",
    individual_languages: &[
    ],
};


pub const RSN: LanguageCode = LanguageCode {
    name: "Rwandan Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rsn",
    individual_languages: &[
    ],
};


pub const RTC: LanguageCode = LanguageCode {
    name: "Rungtu Chin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rtc",
    individual_languages: &[
    ],
};


pub const RTH: LanguageCode = LanguageCode {
    name: "Ratahan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rth",
    individual_languages: &[
    ],
};


pub const RTM: LanguageCode = LanguageCode {
    name: "Rotuman",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rtm",
    individual_languages: &[
    ],
};


pub const RTS: LanguageCode = LanguageCode {
    name: "Yurats",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rts",
    individual_languages: &[
    ],
};


pub const RTW: LanguageCode = LanguageCode {
    name: "Rathawi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rtw",
    individual_languages: &[
    ],
};


pub const RUB: LanguageCode = LanguageCode {
    name: "Gungu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rub",
    individual_languages: &[
    ],
};


pub const RUC: LanguageCode = LanguageCode {
    name: "Ruuli",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ruc",
    individual_languages: &[
    ],
};


pub const RUE: LanguageCode = LanguageCode {
    name: "Rusyn",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rue",
    individual_languages: &[
    ],
};


pub const RUF: LanguageCode = LanguageCode {
    name: "Luguru",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ruf",
    individual_languages: &[
    ],
};


pub const RUG: LanguageCode = LanguageCode {
    name: "Roviana",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rug",
    individual_languages: &[
    ],
};


pub const RUH: LanguageCode = LanguageCode {
    name: "Ruga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ruh",
    individual_languages: &[
    ],
};


pub const RUI: LanguageCode = LanguageCode {
    name: "Rufiji",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rui",
    individual_languages: &[
    ],
};


pub const RUK: LanguageCode = LanguageCode {
    name: "Che",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ruk",
    individual_languages: &[
    ],
};


pub const RUO: LanguageCode = LanguageCode {
    name: "Istro Romanian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ruo",
    individual_languages: &[
    ],
};


pub const RUQ: LanguageCode = LanguageCode {
    name: "Megleno Romanian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ruq",
    individual_languages: &[
    ],
};


pub const RUT: LanguageCode = LanguageCode {
    name: "Rutul",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rut",
    individual_languages: &[
    ],
};


pub const RUU: LanguageCode = LanguageCode {
    name: "Lanas Lobu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ruu",
    individual_languages: &[
    ],
};


pub const RUY: LanguageCode = LanguageCode {
    name: "Mala (Nigeria)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ruy",
    individual_languages: &[
    ],
};


pub const RUZ: LanguageCode = LanguageCode {
    name: "Ruma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ruz",
    individual_languages: &[
    ],
};


pub const RWA: LanguageCode = LanguageCode {
    name: "Rawo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rwa",
    individual_languages: &[
    ],
};


pub const RWK: LanguageCode = LanguageCode {
    name: "Rwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rwk",
    individual_languages: &[
    ],
};


pub const RWL: LanguageCode = LanguageCode {
    name: "Ruwila",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rwl",
    individual_languages: &[
    ],
};


pub const RWM: LanguageCode = LanguageCode {
    name: "Amba (Uganda)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rwm",
    individual_languages: &[
    ],
};


pub const RWO: LanguageCode = LanguageCode {
    name: "Rawa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rwo",
    individual_languages: &[
    ],
};


pub const RWR: LanguageCode = LanguageCode {
    name: "Marwari (India)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rwr",
    individual_languages: &[
    ],
};


pub const RXD: LanguageCode = LanguageCode {
    name: "Ngardi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rxd",
    individual_languages: &[
    ],
};


pub const RXW: LanguageCode = LanguageCode {
    name: "Karuwali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rxw",
    individual_languages: &[
    ],
};


pub const RYN: LanguageCode = LanguageCode {
    name: "Northern Amami-Oshima",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ryn",
    individual_languages: &[
    ],
};


pub const RYS: LanguageCode = LanguageCode {
    name: "Yaeyama",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rys",
    individual_languages: &[
    ],
};


pub const RYU: LanguageCode = LanguageCode {
    name: "Central Okinawan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ryu",
    individual_languages: &[
    ],
};


pub const RZH: LanguageCode = LanguageCode {
    name: "Rāziḥī",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "rzh",
    individual_languages: &[
    ],
};


pub const SAA: LanguageCode = LanguageCode {
    name: "Saba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "saa",
    individual_languages: &[
    ],
};


pub const SAB: LanguageCode = LanguageCode {
    name: "Buglere",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sab",
    individual_languages: &[
    ],
};


pub const SAC: LanguageCode = LanguageCode {
    name: "Meskwaki",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sac",
    individual_languages: &[
    ],
};


pub const SAE: LanguageCode = LanguageCode {
    name: "Sabanê",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sae",
    individual_languages: &[
    ],
};


pub const SAF: LanguageCode = LanguageCode {
    name: "Safaliba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "saf",
    individual_languages: &[
    ],
};


pub const SAJ: LanguageCode = LanguageCode {
    name: "Sahu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "saj",
    individual_languages: &[
    ],
};


pub const SAK: LanguageCode = LanguageCode {
    name: "Sake",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sak",
    individual_languages: &[
    ],
};


pub const SAO: LanguageCode = LanguageCode {
    name: "Sause",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sao",
    individual_languages: &[
    ],
};


pub const SAQ: LanguageCode = LanguageCode {
    name: "Samburu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "saq",
    individual_languages: &[
    ],
};


pub const SAR: LanguageCode = LanguageCode {
    name: "Saraveca",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sar",
    individual_languages: &[
    ],
};


pub const SAU: LanguageCode = LanguageCode {
    name: "Saleman",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sau",
    individual_languages: &[
    ],
};


pub const SAV: LanguageCode = LanguageCode {
    name: "Saafi-Saafi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sav",
    individual_languages: &[
    ],
};


pub const SAW: LanguageCode = LanguageCode {
    name: "Sawi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "saw",
    individual_languages: &[
    ],
};


pub const SAX: LanguageCode = LanguageCode {
    name: "Sa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sax",
    individual_languages: &[
    ],
};


pub const SAY: LanguageCode = LanguageCode {
    name: "Saya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "say",
    individual_languages: &[
    ],
};


pub const SAZ: LanguageCode = LanguageCode {
    name: "Saurashtra",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "saz",
    individual_languages: &[
    ],
};


pub const SBA: LanguageCode = LanguageCode {
    name: "Ngambay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sba",
    individual_languages: &[
    ],
};


pub const SBB: LanguageCode = LanguageCode {
    name: "Simbo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sbb",
    individual_languages: &[
    ],
};


pub const SBC: LanguageCode = LanguageCode {
    name: "Kele (Papua New Guinea)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sbc",
    individual_languages: &[
    ],
};


pub const SBD: LanguageCode = LanguageCode {
    name: "Southern Samo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sbd",
    individual_languages: &[
    ],
};


pub const SBE: LanguageCode = LanguageCode {
    name: "Saliba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sbe",
    individual_languages: &[
    ],
};


pub const SBF: LanguageCode = LanguageCode {
    name: "Chabu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sbf",
    individual_languages: &[
    ],
};


pub const SBG: LanguageCode = LanguageCode {
    name: "Seget",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sbg",
    individual_languages: &[
    ],
};


pub const SBH: LanguageCode = LanguageCode {
    name: "Sori-Harengan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sbh",
    individual_languages: &[
    ],
};


pub const SBI: LanguageCode = LanguageCode {
    name: "Seti",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sbi",
    individual_languages: &[
    ],
};


pub const SBJ: LanguageCode = LanguageCode {
    name: "Surbakhal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sbj",
    individual_languages: &[
    ],
};


pub const SBK: LanguageCode = LanguageCode {
    name: "Safwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sbk",
    individual_languages: &[
    ],
};


pub const SBL: LanguageCode = LanguageCode {
    name: "Botolan Sambal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sbl",
    individual_languages: &[
    ],
};


pub const SBM: LanguageCode = LanguageCode {
    name: "Sagala",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sbm",
    individual_languages: &[
    ],
};


pub const SBN: LanguageCode = LanguageCode {
    name: "Sindhi Bhil",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sbn",
    individual_languages: &[
    ],
};


pub const SBO: LanguageCode = LanguageCode {
    name: "Sabüm",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sbo",
    individual_languages: &[
    ],
};


pub const SBP: LanguageCode = LanguageCode {
    name: "Sangu (Tanzania)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sbp",
    individual_languages: &[
    ],
};


pub const SBQ: LanguageCode = LanguageCode {
    name: "Sileibi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sbq",
    individual_languages: &[
    ],
};


pub const SBR: LanguageCode = LanguageCode {
    name: "Sembakung Murut",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sbr",
    individual_languages: &[
    ],
};


pub const SBS: LanguageCode = LanguageCode {
    name: "Subiya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sbs",
    individual_languages: &[
    ],
};


pub const SBT: LanguageCode = LanguageCode {
    name: "Kimki",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sbt",
    individual_languages: &[
    ],
};


pub const SBU: LanguageCode = LanguageCode {
    name: "Stod Bhoti",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sbu",
    individual_languages: &[
    ],
};


pub const SBV: LanguageCode = LanguageCode {
    name: "Sabine",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sbv",
    individual_languages: &[
    ],
};


pub const SBW: LanguageCode = LanguageCode {
    name: "Simba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sbw",
    individual_languages: &[
    ],
};


pub const SBX: LanguageCode = LanguageCode {
    name: "Seberuang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sbx",
    individual_languages: &[
    ],
};


pub const SBY: LanguageCode = LanguageCode {
    name: "Soli",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sby",
    individual_languages: &[
    ],
};


pub const SBZ: LanguageCode = LanguageCode {
    name: "Sara Kaba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sbz",
    individual_languages: &[
    ],
};


pub const SCB: LanguageCode = LanguageCode {
    name: "Chut",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "scb",
    individual_languages: &[
    ],
};


pub const SCE: LanguageCode = LanguageCode {
    name: "Dongxiang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sce",
    individual_languages: &[
    ],
};


pub const SCF: LanguageCode = LanguageCode {
    name: "San Miguel Creole French",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "scf",
    individual_languages: &[
    ],
};


pub const SCG: LanguageCode = LanguageCode {
    name: "Sanggau",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "scg",
    individual_languages: &[
    ],
};


pub const SCH: LanguageCode = LanguageCode {
    name: "Sakachep",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sch",
    individual_languages: &[
    ],
};


pub const SCI: LanguageCode = LanguageCode {
    name: "Sri Lankan Creole Malay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sci",
    individual_languages: &[
    ],
};


pub const SCK: LanguageCode = LanguageCode {
    name: "Sadri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sck",
    individual_languages: &[
    ],
};


pub const SCL: LanguageCode = LanguageCode {
    name: "Shina",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "scl",
    individual_languages: &[
    ],
};


pub const SCP: LanguageCode = LanguageCode {
    name: "Hyolmo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "scp",
    individual_languages: &[
    ],
};


pub const SCQ: LanguageCode = LanguageCode {
    name: "Sa'och",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "scq",
    individual_languages: &[
    ],
};


pub const SCS: LanguageCode = LanguageCode {
    name: "North Slavey",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "scs",
    individual_languages: &[
    ],
};


pub const SCT: LanguageCode = LanguageCode {
    name: "Southern Katang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sct",
    individual_languages: &[
    ],
};


pub const SCU: LanguageCode = LanguageCode {
    name: "Shumcho",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "scu",
    individual_languages: &[
    ],
};


pub const SCV: LanguageCode = LanguageCode {
    name: "Sheni",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "scv",
    individual_languages: &[
    ],
};


pub const SCW: LanguageCode = LanguageCode {
    name: "Sha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "scw",
    individual_languages: &[
    ],
};


pub const SCX: LanguageCode = LanguageCode {
    name: "Sicel",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "scx",
    individual_languages: &[
    ],
};


pub const SDA: LanguageCode = LanguageCode {
    name: "Toraja-Sa'dan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sda",
    individual_languages: &[
    ],
};


pub const SDB: LanguageCode = LanguageCode {
    name: "Shabak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sdb",
    individual_languages: &[
    ],
};


pub const SDC: LanguageCode = LanguageCode {
    name: "Sassarese Sardinian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sdc",
    individual_languages: &[
    ],
};


pub const SDE: LanguageCode = LanguageCode {
    name: "Surubu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sde",
    individual_languages: &[
    ],
};


pub const SDF: LanguageCode = LanguageCode {
    name: "Sarli",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sdf",
    individual_languages: &[
    ],
};


pub const SDG: LanguageCode = LanguageCode {
    name: "Savi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sdg",
    individual_languages: &[
    ],
};


pub const SDH: LanguageCode = LanguageCode {
    name: "Southern Kurdish",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sdh",
    individual_languages: &[
    ],
};


pub const SDJ: LanguageCode = LanguageCode {
    name: "Suundi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sdj",
    individual_languages: &[
    ],
};


pub const SDK: LanguageCode = LanguageCode {
    name: "Sos Kundi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sdk",
    individual_languages: &[
    ],
};


pub const SDL: LanguageCode = LanguageCode {
    name: "Saudi Arabian Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sdl",
    individual_languages: &[
    ],
};


pub const SDN: LanguageCode = LanguageCode {
    name: "Gallurese Sardinian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sdn",
    individual_languages: &[
    ],
};


pub const SDO: LanguageCode = LanguageCode {
    name: "Bukar-Sadung Bidayuh",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sdo",
    individual_languages: &[
    ],
};


pub const SDP: LanguageCode = LanguageCode {
    name: "Sherdukpen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sdp",
    individual_languages: &[
    ],
};


pub const SDQ: LanguageCode = LanguageCode {
    name: "Semandang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sdq",
    individual_languages: &[
    ],
};


pub const SDR: LanguageCode = LanguageCode {
    name: "Oraon Sadri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sdr",
    individual_languages: &[
    ],
};


pub const SDS: LanguageCode = LanguageCode {
    name: "Sened",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sds",
    individual_languages: &[
    ],
};


pub const SDT: LanguageCode = LanguageCode {
    name: "Shuadit",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sdt",
    individual_languages: &[
    ],
};


pub const SDU: LanguageCode = LanguageCode {
    name: "Sarudu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sdu",
    individual_languages: &[
    ],
};


pub const SDX: LanguageCode = LanguageCode {
    name: "Sibu Melanau",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sdx",
    individual_languages: &[
    ],
};


pub const SDZ: LanguageCode = LanguageCode {
    name: "Sallands",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sdz",
    individual_languages: &[
    ],
};


pub const SEA: LanguageCode = LanguageCode {
    name: "Semai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sea",
    individual_languages: &[
    ],
};


pub const SEB: LanguageCode = LanguageCode {
    name: "Shempire Senoufo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "seb",
    individual_languages: &[
    ],
};


pub const SEC: LanguageCode = LanguageCode {
    name: "Sechelt",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sec",
    individual_languages: &[
    ],
};


pub const SED: LanguageCode = LanguageCode {
    name: "Sedang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sed",
    individual_languages: &[
    ],
};


pub const SEE: LanguageCode = LanguageCode {
    name: "Seneca",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "see",
    individual_languages: &[
    ],
};


pub const SEF: LanguageCode = LanguageCode {
    name: "Cebaara Senoufo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sef",
    individual_languages: &[
    ],
};


pub const SEG: LanguageCode = LanguageCode {
    name: "Segeju",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "seg",
    individual_languages: &[
    ],
};


pub const SEH: LanguageCode = LanguageCode {
    name: "Sena",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "seh",
    individual_languages: &[
    ],
};


pub const SEI: LanguageCode = LanguageCode {
    name: "Seri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sei",
    individual_languages: &[
    ],
};


pub const SEJ: LanguageCode = LanguageCode {
    name: "Sene",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sej",
    individual_languages: &[
    ],
};


pub const SEK: LanguageCode = LanguageCode {
    name: "Sekani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sek",
    individual_languages: &[
    ],
};


pub const SEN: LanguageCode = LanguageCode {
    name: "Nanerigé Sénoufo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sen",
    individual_languages: &[
    ],
};


pub const SEO: LanguageCode = LanguageCode {
    name: "Suarmin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "seo",
    individual_languages: &[
    ],
};


pub const SEP: LanguageCode = LanguageCode {
    name: "Sìcìté Sénoufo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sep",
    individual_languages: &[
    ],
};


pub const SEQ: LanguageCode = LanguageCode {
    name: "Senara Sénoufo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "seq",
    individual_languages: &[
    ],
};


pub const SER: LanguageCode = LanguageCode {
    name: "Serrano",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ser",
    individual_languages: &[
    ],
};


pub const SES: LanguageCode = LanguageCode {
    name: "Koyraboro Senni Songhai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ses",
    individual_languages: &[
    ],
};


pub const SET: LanguageCode = LanguageCode {
    name: "Sentani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "set",
    individual_languages: &[
    ],
};


pub const SEU: LanguageCode = LanguageCode {
    name: "Serui-Laut",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "seu",
    individual_languages: &[
    ],
};


pub const SEV: LanguageCode = LanguageCode {
    name: "Nyarafolo Senoufo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sev",
    individual_languages: &[
    ],
};


pub const SEW: LanguageCode = LanguageCode {
    name: "Sewa Bay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sew",
    individual_languages: &[
    ],
};


pub const SEY: LanguageCode = LanguageCode {
    name: "Secoya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sey",
    individual_languages: &[
    ],
};


pub const SEZ: LanguageCode = LanguageCode {
    name: "Senthang Chin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sez",
    individual_languages: &[
    ],
};


pub const SFB: LanguageCode = LanguageCode {
    name: "Langue des signes de Belgique Francophone",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sfb",
    individual_languages: &[
    ],
};


pub const SFE: LanguageCode = LanguageCode {
    name: "Eastern Subanen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sfe",
    individual_languages: &[
    ],
};


pub const SFM: LanguageCode = LanguageCode {
    name: "Small Flowery Miao",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sfm",
    individual_languages: &[
    ],
};


pub const SFS: LanguageCode = LanguageCode {
    name: "South African Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sfs",
    individual_languages: &[
    ],
};


pub const SFW: LanguageCode = LanguageCode {
    name: "Sehwi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sfw",
    individual_languages: &[
    ],
};


pub const SGB: LanguageCode = LanguageCode {
    name: "Mag-antsi Ayta",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sgb",
    individual_languages: &[
    ],
};


pub const SGC: LanguageCode = LanguageCode {
    name: "Kipsigis",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sgc",
    individual_languages: &[
    ],
};


pub const SGD: LanguageCode = LanguageCode {
    name: "Surigaonon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sgd",
    individual_languages: &[
    ],
};


pub const SGE: LanguageCode = LanguageCode {
    name: "Segai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sge",
    individual_languages: &[
    ],
};


pub const SGG: LanguageCode = LanguageCode {
    name: "Swiss-German Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sgg",
    individual_languages: &[
    ],
};


pub const SGH: LanguageCode = LanguageCode {
    name: "Shughni",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sgh",
    individual_languages: &[
    ],
};


pub const SGI: LanguageCode = LanguageCode {
    name: "Suga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sgi",
    individual_languages: &[
    ],
};


pub const SGJ: LanguageCode = LanguageCode {
    name: "Surgujia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sgj",
    individual_languages: &[
    ],
};


pub const SGK: LanguageCode = LanguageCode {
    name: "Sangkong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sgk",
    individual_languages: &[
    ],
};


pub const SGM: LanguageCode = LanguageCode {
    name: "Singa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sgm",
    individual_languages: &[
    ],
};


pub const SGP: LanguageCode = LanguageCode {
    name: "Singpho",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sgp",
    individual_languages: &[
    ],
};


pub const SGR: LanguageCode = LanguageCode {
    name: "Sangisari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sgr",
    individual_languages: &[
    ],
};


pub const SGS: LanguageCode = LanguageCode {
    name: "Samogitian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sgs",
    individual_languages: &[
    ],
};


pub const SGT: LanguageCode = LanguageCode {
    name: "Brokpake",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sgt",
    individual_languages: &[
    ],
};


pub const SGU: LanguageCode = LanguageCode {
    name: "Salas",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sgu",
    individual_languages: &[
    ],
};


pub const SGW: LanguageCode = LanguageCode {
    name: "Sebat Bet Gurage",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sgw",
    individual_languages: &[
    ],
};


pub const SGX: LanguageCode = LanguageCode {
    name: "Sierra Leone Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sgx",
    individual_languages: &[
    ],
};


pub const SGY: LanguageCode = LanguageCode {
    name: "Sanglechi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sgy",
    individual_languages: &[
    ],
};


pub const SGZ: LanguageCode = LanguageCode {
    name: "Sursurunga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sgz",
    individual_languages: &[
    ],
};


pub const SHA: LanguageCode = LanguageCode {
    name: "Shall-Zwall",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sha",
    individual_languages: &[
    ],
};


pub const SHB: LanguageCode = LanguageCode {
    name: "Ninam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "shb",
    individual_languages: &[
    ],
};


pub const SHC: LanguageCode = LanguageCode {
    name: "Sonde",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "shc",
    individual_languages: &[
    ],
};


pub const SHD: LanguageCode = LanguageCode {
    name: "Kundal Shahi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "shd",
    individual_languages: &[
    ],
};


pub const SHE: LanguageCode = LanguageCode {
    name: "Sheko",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "she",
    individual_languages: &[
    ],
};


pub const SHG: LanguageCode = LanguageCode {
    name: "Shua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "shg",
    individual_languages: &[
    ],
};


pub const SHH: LanguageCode = LanguageCode {
    name: "Shoshoni",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "shh",
    individual_languages: &[
    ],
};


pub const SHI: LanguageCode = LanguageCode {
    name: "Tachelhit",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "shi",
    individual_languages: &[
    ],
};


pub const SHJ: LanguageCode = LanguageCode {
    name: "Shatt",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "shj",
    individual_languages: &[
    ],
};


pub const SHK: LanguageCode = LanguageCode {
    name: "Shilluk",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "shk",
    individual_languages: &[
    ],
};


pub const SHL: LanguageCode = LanguageCode {
    name: "Shendu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "shl",
    individual_languages: &[
    ],
};


pub const SHM: LanguageCode = LanguageCode {
    name: "Shahrudi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "shm",
    individual_languages: &[
    ],
};


pub const SHO: LanguageCode = LanguageCode {
    name: "Shanga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sho",
    individual_languages: &[
    ],
};


pub const SHP: LanguageCode = LanguageCode {
    name: "Shipibo-Conibo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "shp",
    individual_languages: &[
    ],
};


pub const SHQ: LanguageCode = LanguageCode {
    name: "Sala",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "shq",
    individual_languages: &[
    ],
};


pub const SHR: LanguageCode = LanguageCode {
    name: "Shi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "shr",
    individual_languages: &[
    ],
};


pub const SHS: LanguageCode = LanguageCode {
    name: "Shuswap",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "shs",
    individual_languages: &[
    ],
};


pub const SHT: LanguageCode = LanguageCode {
    name: "Shasta",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sht",
    individual_languages: &[
    ],
};


pub const SHU: LanguageCode = LanguageCode {
    name: "Chadian Arabic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "shu",
    individual_languages: &[
    ],
};


pub const SHV: LanguageCode = LanguageCode {
    name: "Shehri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "shv",
    individual_languages: &[
    ],
};


pub const SHW: LanguageCode = LanguageCode {
    name: "Shwai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "shw",
    individual_languages: &[
    ],
};


pub const SHX: LanguageCode = LanguageCode {
    name: "She",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "shx",
    individual_languages: &[
    ],
};


pub const SHY: LanguageCode = LanguageCode {
    name: "Tachawit",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "shy",
    individual_languages: &[
    ],
};


pub const SHZ: LanguageCode = LanguageCode {
    name: "Syenara Senoufo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "shz",
    individual_languages: &[
    ],
};


pub const SIA: LanguageCode = LanguageCode {
    name: "Akkala Sami",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sia",
    individual_languages: &[
    ],
};


pub const SIB: LanguageCode = LanguageCode {
    name: "Sebop",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sib",
    individual_languages: &[
    ],
};


pub const SIE: LanguageCode = LanguageCode {
    name: "Simaa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sie",
    individual_languages: &[
    ],
};


pub const SIF: LanguageCode = LanguageCode {
    name: "Siamou",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sif",
    individual_languages: &[
    ],
};


pub const SIG: LanguageCode = LanguageCode {
    name: "Paasaal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sig",
    individual_languages: &[
    ],
};


pub const SIH: LanguageCode = LanguageCode {
    name: "Zire",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sih",
    individual_languages: &[
    ],
};


pub const SII: LanguageCode = LanguageCode {
    name: "Shom Peng",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sii",
    individual_languages: &[
    ],
};


pub const SIJ: LanguageCode = LanguageCode {
    name: "Numbami",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sij",
    individual_languages: &[
    ],
};


pub const SIK: LanguageCode = LanguageCode {
    name: "Sikiana",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sik",
    individual_languages: &[
    ],
};


pub const SIL: LanguageCode = LanguageCode {
    name: "Tumulung Sisaala",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sil",
    individual_languages: &[
    ],
};


pub const SIM: LanguageCode = LanguageCode {
    name: "Mende (Papua New Guinea)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sim",
    individual_languages: &[
    ],
};


pub const SIP: LanguageCode = LanguageCode {
    name: "Sikkimese",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sip",
    individual_languages: &[
    ],
};


pub const SIQ: LanguageCode = LanguageCode {
    name: "Sonia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "siq",
    individual_languages: &[
    ],
};


pub const SIR: LanguageCode = LanguageCode {
    name: "Siri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sir",
    individual_languages: &[
    ],
};


pub const SIS: LanguageCode = LanguageCode {
    name: "Siuslaw",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sis",
    individual_languages: &[
    ],
};


pub const SIU: LanguageCode = LanguageCode {
    name: "Sinagen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "siu",
    individual_languages: &[
    ],
};


pub const SIV: LanguageCode = LanguageCode {
    name: "Sumariup",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "siv",
    individual_languages: &[
    ],
};


pub const SIW: LanguageCode = LanguageCode {
    name: "Siwai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "siw",
    individual_languages: &[
    ],
};


pub const SIX: LanguageCode = LanguageCode {
    name: "Sumau",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "six",
    individual_languages: &[
    ],
};


pub const SIY: LanguageCode = LanguageCode {
    name: "Sivandi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "siy",
    individual_languages: &[
    ],
};


pub const SIZ: LanguageCode = LanguageCode {
    name: "Siwi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "siz",
    individual_languages: &[
    ],
};


pub const SJA: LanguageCode = LanguageCode {
    name: "Epena",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sja",
    individual_languages: &[
    ],
};


pub const SJB: LanguageCode = LanguageCode {
    name: "Sajau Basap",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sjb",
    individual_languages: &[
    ],
};


pub const SJD: LanguageCode = LanguageCode {
    name: "Kildin Sami",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sjd",
    individual_languages: &[
    ],
};


pub const SJE: LanguageCode = LanguageCode {
    name: "Pite Sami",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sje",
    individual_languages: &[
    ],
};


pub const SJG: LanguageCode = LanguageCode {
    name: "Assangori",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sjg",
    individual_languages: &[
    ],
};


pub const SJK: LanguageCode = LanguageCode {
    name: "Kemi Sami",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sjk",
    individual_languages: &[
    ],
};


pub const SJL: LanguageCode = LanguageCode {
    name: "Sajalong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sjl",
    individual_languages: &[
    ],
};


pub const SJM: LanguageCode = LanguageCode {
    name: "Mapun",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sjm",
    individual_languages: &[
    ],
};


pub const SJN: LanguageCode = LanguageCode {
    name: "Sindarin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sjn",
    individual_languages: &[
    ],
};


pub const SJO: LanguageCode = LanguageCode {
    name: "Xibe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sjo",
    individual_languages: &[
    ],
};


pub const SJP: LanguageCode = LanguageCode {
    name: "Surjapuri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sjp",
    individual_languages: &[
    ],
};


pub const SJR: LanguageCode = LanguageCode {
    name: "Siar-Lak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sjr",
    individual_languages: &[
    ],
};


pub const SJS: LanguageCode = LanguageCode {
    name: "Senhaja De Srair",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sjs",
    individual_languages: &[
    ],
};


pub const SJT: LanguageCode = LanguageCode {
    name: "Ter Sami",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sjt",
    individual_languages: &[
    ],
};


pub const SJU: LanguageCode = LanguageCode {
    name: "Ume Sami",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sju",
    individual_languages: &[
    ],
};


pub const SJW: LanguageCode = LanguageCode {
    name: "Shawnee",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sjw",
    individual_languages: &[
    ],
};


pub const SKA: LanguageCode = LanguageCode {
    name: "Skagit",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ska",
    individual_languages: &[
    ],
};


pub const SKB: LanguageCode = LanguageCode {
    name: "Saek",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "skb",
    individual_languages: &[
    ],
};


pub const SKC: LanguageCode = LanguageCode {
    name: "Ma Manda",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "skc",
    individual_languages: &[
    ],
};


pub const SKD: LanguageCode = LanguageCode {
    name: "Southern Sierra Miwok",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "skd",
    individual_languages: &[
    ],
};


pub const SKE: LanguageCode = LanguageCode {
    name: "Seke (Vanuatu)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ske",
    individual_languages: &[
    ],
};


pub const SKF: LanguageCode = LanguageCode {
    name: "Sakirabiá",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "skf",
    individual_languages: &[
    ],
};


pub const SKG: LanguageCode = LanguageCode {
    name: "Sakalava Malagasy",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "skg",
    individual_languages: &[
    ],
};


pub const SKH: LanguageCode = LanguageCode {
    name: "Sikule",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "skh",
    individual_languages: &[
    ],
};


pub const SKI: LanguageCode = LanguageCode {
    name: "Sika",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ski",
    individual_languages: &[
    ],
};


pub const SKJ: LanguageCode = LanguageCode {
    name: "Seke (Nepal)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "skj",
    individual_languages: &[
    ],
};


pub const SKM: LanguageCode = LanguageCode {
    name: "Kutong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "skm",
    individual_languages: &[
    ],
};


pub const SKN: LanguageCode = LanguageCode {
    name: "Kolibugan Subanon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "skn",
    individual_languages: &[
    ],
};


pub const SKO: LanguageCode = LanguageCode {
    name: "Seko Tengah",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sko",
    individual_languages: &[
    ],
};


pub const SKP: LanguageCode = LanguageCode {
    name: "Sekapan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "skp",
    individual_languages: &[
    ],
};


pub const SKQ: LanguageCode = LanguageCode {
    name: "Sininkere",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "skq",
    individual_languages: &[
    ],
};


pub const SKR: LanguageCode = LanguageCode {
    name: "Saraiki",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "skr",
    individual_languages: &[
    ],
};


pub const SKS: LanguageCode = LanguageCode {
    name: "Maia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sks",
    individual_languages: &[
    ],
};


pub const SKT: LanguageCode = LanguageCode {
    name: "Sakata",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "skt",
    individual_languages: &[
    ],
};


pub const SKU: LanguageCode = LanguageCode {
    name: "Sakao",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sku",
    individual_languages: &[
    ],
};


pub const SKV: LanguageCode = LanguageCode {
    name: "Skou",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "skv",
    individual_languages: &[
    ],
};


pub const SKW: LanguageCode = LanguageCode {
    name: "Skepi Creole Dutch",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "skw",
    individual_languages: &[
    ],
};


pub const SKX: LanguageCode = LanguageCode {
    name: "Seko Padang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "skx",
    individual_languages: &[
    ],
};


pub const SKY: LanguageCode = LanguageCode {
    name: "Sikaiana",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sky",
    individual_languages: &[
    ],
};


pub const SKZ: LanguageCode = LanguageCode {
    name: "Sekar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "skz",
    individual_languages: &[
    ],
};


pub const SLC: LanguageCode = LanguageCode {
    name: "Sáliba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "slc",
    individual_languages: &[
    ],
};


pub const SLD: LanguageCode = LanguageCode {
    name: "Sissala",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sld",
    individual_languages: &[
    ],
};


pub const SLE: LanguageCode = LanguageCode {
    name: "Sholaga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sle",
    individual_languages: &[
    ],
};


pub const SLF: LanguageCode = LanguageCode {
    name: "Swiss-Italian Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "slf",
    individual_languages: &[
    ],
};


pub const SLG: LanguageCode = LanguageCode {
    name: "Selungai Murut",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "slg",
    individual_languages: &[
    ],
};


pub const SLH: LanguageCode = LanguageCode {
    name: "Southern Puget Sound Salish",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "slh",
    individual_languages: &[
    ],
};


pub const SLI: LanguageCode = LanguageCode {
    name: "Lower Silesian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sli",
    individual_languages: &[
    ],
};


pub const SLJ: LanguageCode = LanguageCode {
    name: "Salumá",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "slj",
    individual_languages: &[
    ],
};


pub const SLL: LanguageCode = LanguageCode {
    name: "Salt-Yui",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sll",
    individual_languages: &[
    ],
};


pub const SLM: LanguageCode = LanguageCode {
    name: "Pangutaran Sama",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "slm",
    individual_languages: &[
    ],
};


pub const SLN: LanguageCode = LanguageCode {
    name: "Salinan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sln",
    individual_languages: &[
    ],
};


pub const SLP: LanguageCode = LanguageCode {
    name: "Lamaholot",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "slp",
    individual_languages: &[
    ],
};


pub const SLQ: LanguageCode = LanguageCode {
    name: "Salchuq",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "slq",
    individual_languages: &[
    ],
};


pub const SLR: LanguageCode = LanguageCode {
    name: "Salar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "slr",
    individual_languages: &[
    ],
};


pub const SLS: LanguageCode = LanguageCode {
    name: "Singapore Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sls",
    individual_languages: &[
    ],
};


pub const SLT: LanguageCode = LanguageCode {
    name: "Sila",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "slt",
    individual_languages: &[
    ],
};


pub const SLU: LanguageCode = LanguageCode {
    name: "Selaru",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "slu",
    individual_languages: &[
    ],
};


pub const SLW: LanguageCode = LanguageCode {
    name: "Sialum",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "slw",
    individual_languages: &[
    ],
};


pub const SLX: LanguageCode = LanguageCode {
    name: "Salampasu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "slx",
    individual_languages: &[
    ],
};


pub const SLY: LanguageCode = LanguageCode {
    name: "Selayar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sly",
    individual_languages: &[
    ],
};


pub const SLZ: LanguageCode = LanguageCode {
    name: "Ma'ya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "slz",
    individual_languages: &[
    ],
};


pub const SMB: LanguageCode = LanguageCode {
    name: "Simbari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "smb",
    individual_languages: &[
    ],
};


pub const SMC: LanguageCode = LanguageCode {
    name: "Som",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "smc",
    individual_languages: &[
    ],
};


pub const SMF: LanguageCode = LanguageCode {
    name: "Auwe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "smf",
    individual_languages: &[
    ],
};


pub const SMG: LanguageCode = LanguageCode {
    name: "Simbali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "smg",
    individual_languages: &[
    ],
};


pub const SMH: LanguageCode = LanguageCode {
    name: "Samei",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "smh",
    individual_languages: &[
    ],
};


pub const SMK: LanguageCode = LanguageCode {
    name: "Bolinao",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "smk",
    individual_languages: &[
    ],
};


pub const SML: LanguageCode = LanguageCode {
    name: "Central Sama",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sml",
    individual_languages: &[
    ],
};


pub const SMM: LanguageCode = LanguageCode {
    name: "Musasa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "smm",
    individual_languages: &[
    ],
};


pub const SMP: LanguageCode = LanguageCode {
    name: "Samaritan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "smp",
    individual_languages: &[
    ],
};


pub const SMQ: LanguageCode = LanguageCode {
    name: "Samo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "smq",
    individual_languages: &[
    ],
};


pub const SMR: LanguageCode = LanguageCode {
    name: "Simeulue",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "smr",
    individual_languages: &[
    ],
};


pub const SMT: LanguageCode = LanguageCode {
    name: "Simte",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "smt",
    individual_languages: &[
    ],
};


pub const SMU: LanguageCode = LanguageCode {
    name: "Somray",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "smu",
    individual_languages: &[
    ],
};


pub const SMV: LanguageCode = LanguageCode {
    name: "Samvedi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "smv",
    individual_languages: &[
    ],
};


pub const SMW: LanguageCode = LanguageCode {
    name: "Sumbawa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "smw",
    individual_languages: &[
    ],
};


pub const SMX: LanguageCode = LanguageCode {
    name: "Samba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "smx",
    individual_languages: &[
    ],
};


pub const SMY: LanguageCode = LanguageCode {
    name: "Semnani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "smy",
    individual_languages: &[
    ],
};


pub const SMZ: LanguageCode = LanguageCode {
    name: "Simeku",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "smz",
    individual_languages: &[
    ],
};


pub const SNC: LanguageCode = LanguageCode {
    name: "Sinaugoro",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "snc",
    individual_languages: &[
    ],
};


pub const SNE: LanguageCode = LanguageCode {
    name: "Bau Bidayuh",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sne",
    individual_languages: &[
    ],
};


pub const SNF: LanguageCode = LanguageCode {
    name: "Noon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "snf",
    individual_languages: &[
    ],
};


pub const SNG: LanguageCode = LanguageCode {
    name: "Sanga (Democratic Republic of Congo)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sng",
    individual_languages: &[
    ],
};


pub const SNI: LanguageCode = LanguageCode {
    name: "Sensi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sni",
    individual_languages: &[
    ],
};


pub const SNJ: LanguageCode = LanguageCode {
    name: "Riverain Sango",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "snj",
    individual_languages: &[
    ],
};


pub const SNL: LanguageCode = LanguageCode {
    name: "Sangil",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "snl",
    individual_languages: &[
    ],
};


pub const SNM: LanguageCode = LanguageCode {
    name: "Southern Ma'di",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "snm",
    individual_languages: &[
    ],
};


pub const SNN: LanguageCode = LanguageCode {
    name: "Siona",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "snn",
    individual_languages: &[
    ],
};


pub const SNO: LanguageCode = LanguageCode {
    name: "Snohomish",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sno",
    individual_languages: &[
    ],
};


pub const SNP: LanguageCode = LanguageCode {
    name: "Siane",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "snp",
    individual_languages: &[
    ],
};


pub const SNQ: LanguageCode = LanguageCode {
    name: "Sangu (Gabon)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "snq",
    individual_languages: &[
    ],
};


pub const SNR: LanguageCode = LanguageCode {
    name: "Sihan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "snr",
    individual_languages: &[
    ],
};


pub const SNS: LanguageCode = LanguageCode {
    name: "South West Bay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sns",
    individual_languages: &[
    ],
};


pub const SNU: LanguageCode = LanguageCode {
    name: "Senggi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "snu",
    individual_languages: &[
    ],
};


pub const SNV: LanguageCode = LanguageCode {
    name: "Sa'ban",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "snv",
    individual_languages: &[
    ],
};


pub const SNW: LanguageCode = LanguageCode {
    name: "Selee",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "snw",
    individual_languages: &[
    ],
};


pub const SNX: LanguageCode = LanguageCode {
    name: "Sam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "snx",
    individual_languages: &[
    ],
};


pub const SNY: LanguageCode = LanguageCode {
    name: "Saniyo-Hiyewe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sny",
    individual_languages: &[
    ],
};


pub const SNZ: LanguageCode = LanguageCode {
    name: "Kou",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "snz",
    individual_languages: &[
    ],
};


pub const SOA: LanguageCode = LanguageCode {
    name: "Thai Song",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "soa",
    individual_languages: &[
    ],
};


pub const SOB: LanguageCode = LanguageCode {
    name: "Sobei",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sob",
    individual_languages: &[
    ],
};


pub const SOC: LanguageCode = LanguageCode {
    name: "So (Democratic Republic of Congo)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "soc",
    individual_languages: &[
    ],
};


pub const SOD: LanguageCode = LanguageCode {
    name: "Songoora",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sod",
    individual_languages: &[
    ],
};


pub const SOE: LanguageCode = LanguageCode {
    name: "Songomeno",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "soe",
    individual_languages: &[
    ],
};


pub const SOH: LanguageCode = LanguageCode {
    name: "Aka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "soh",
    individual_languages: &[
    ],
};


pub const SOI: LanguageCode = LanguageCode {
    name: "Sonha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "soi",
    individual_languages: &[
    ],
};


pub const SOJ: LanguageCode = LanguageCode {
    name: "Soi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "soj",
    individual_languages: &[
    ],
};


pub const SOK: LanguageCode = LanguageCode {
    name: "Sokoro",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sok",
    individual_languages: &[
    ],
};


pub const SOL: LanguageCode = LanguageCode {
    name: "Solos",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sol",
    individual_languages: &[
    ],
};


pub const SOO: LanguageCode = LanguageCode {
    name: "Songo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "soo",
    individual_languages: &[
    ],
};


pub const SOP: LanguageCode = LanguageCode {
    name: "Songe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sop",
    individual_languages: &[
    ],
};


pub const SOQ: LanguageCode = LanguageCode {
    name: "Kanasi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "soq",
    individual_languages: &[
    ],
};


pub const SOR: LanguageCode = LanguageCode {
    name: "Somrai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sor",
    individual_languages: &[
    ],
};


pub const SOS: LanguageCode = LanguageCode {
    name: "Seeku",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sos",
    individual_languages: &[
    ],
};


pub const SOU: LanguageCode = LanguageCode {
    name: "Southern Thai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sou",
    individual_languages: &[
    ],
};


pub const SOV: LanguageCode = LanguageCode {
    name: "Sonsorol",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sov",
    individual_languages: &[
    ],
};


pub const SOW: LanguageCode = LanguageCode {
    name: "Sowanda",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sow",
    individual_languages: &[
    ],
};


pub const SOX: LanguageCode = LanguageCode {
    name: "Swo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sox",
    individual_languages: &[
    ],
};


pub const SOY: LanguageCode = LanguageCode {
    name: "Miyobe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "soy",
    individual_languages: &[
    ],
};


pub const SOZ: LanguageCode = LanguageCode {
    name: "Temi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "soz",
    individual_languages: &[
    ],
};


pub const SPB: LanguageCode = LanguageCode {
    name: "Sepa (Indonesia)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "spb",
    individual_languages: &[
    ],
};


pub const SPC: LanguageCode = LanguageCode {
    name: "Sapé",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "spc",
    individual_languages: &[
    ],
};


pub const SPD: LanguageCode = LanguageCode {
    name: "Saep",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "spd",
    individual_languages: &[
    ],
};


pub const SPE: LanguageCode = LanguageCode {
    name: "Sepa (Papua New Guinea)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "spe",
    individual_languages: &[
    ],
};


pub const SPG: LanguageCode = LanguageCode {
    name: "Sian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "spg",
    individual_languages: &[
    ],
};


pub const SPI: LanguageCode = LanguageCode {
    name: "Saponi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "spi",
    individual_languages: &[
    ],
};


pub const SPK: LanguageCode = LanguageCode {
    name: "Sengo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "spk",
    individual_languages: &[
    ],
};


pub const SPL: LanguageCode = LanguageCode {
    name: "Selepet",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "spl",
    individual_languages: &[
    ],
};


pub const SPM: LanguageCode = LanguageCode {
    name: "Akukem",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "spm",
    individual_languages: &[
    ],
};


pub const SPN: LanguageCode = LanguageCode {
    name: "Sanapaná",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "spn",
    individual_languages: &[
    ],
};


pub const SPO: LanguageCode = LanguageCode {
    name: "Spokane",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "spo",
    individual_languages: &[
    ],
};


pub const SPP: LanguageCode = LanguageCode {
    name: "Supyire Senoufo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "spp",
    individual_languages: &[
    ],
};


pub const SPQ: LanguageCode = LanguageCode {
    name: "Loreto-Ucayali Spanish",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "spq",
    individual_languages: &[
    ],
};


pub const SPR: LanguageCode = LanguageCode {
    name: "Saparua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "spr",
    individual_languages: &[
    ],
};


pub const SPS: LanguageCode = LanguageCode {
    name: "Saposa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sps",
    individual_languages: &[
    ],
};


pub const SPT: LanguageCode = LanguageCode {
    name: "Spiti Bhoti",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "spt",
    individual_languages: &[
    ],
};


pub const SPU: LanguageCode = LanguageCode {
    name: "Sapuan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "spu",
    individual_languages: &[
    ],
};


pub const SPV: LanguageCode = LanguageCode {
    name: "Sambalpuri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "spv",
    individual_languages: &[
    ],
};


pub const SPX: LanguageCode = LanguageCode {
    name: "South Picene",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "spx",
    individual_languages: &[
    ],
};


pub const SPY: LanguageCode = LanguageCode {
    name: "Sabaot",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "spy",
    individual_languages: &[
    ],
};


pub const SQA: LanguageCode = LanguageCode {
    name: "Shama-Sambuga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sqa",
    individual_languages: &[
    ],
};


pub const SQH: LanguageCode = LanguageCode {
    name: "Shau",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sqh",
    individual_languages: &[
    ],
};


pub const SQK: LanguageCode = LanguageCode {
    name: "Albanian Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sqk",
    individual_languages: &[
    ],
};


pub const SQM: LanguageCode = LanguageCode {
    name: "Suma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sqm",
    individual_languages: &[
    ],
};


pub const SQN: LanguageCode = LanguageCode {
    name: "Susquehannock",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sqn",
    individual_languages: &[
    ],
};


pub const SQO: LanguageCode = LanguageCode {
    name: "Sorkhei",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sqo",
    individual_languages: &[
    ],
};


pub const SQQ: LanguageCode = LanguageCode {
    name: "Sou",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sqq",
    individual_languages: &[
    ],
};


pub const SQR: LanguageCode = LanguageCode {
    name: "Siculo Arabic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sqr",
    individual_languages: &[
    ],
};


pub const SQS: LanguageCode = LanguageCode {
    name: "Sri Lankan Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sqs",
    individual_languages: &[
    ],
};


pub const SQT: LanguageCode = LanguageCode {
    name: "Soqotri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sqt",
    individual_languages: &[
    ],
};


pub const SQU: LanguageCode = LanguageCode {
    name: "Squamish",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "squ",
    individual_languages: &[
    ],
};


pub const SQX: LanguageCode = LanguageCode {
    name: "Kufr Qassem Sign Language (KQSL)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sqx",
    individual_languages: &[
    ],
};


pub const SRA: LanguageCode = LanguageCode {
    name: "Saruga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sra",
    individual_languages: &[
    ],
};


pub const SRB: LanguageCode = LanguageCode {
    name: "Sora",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "srb",
    individual_languages: &[
    ],
};


pub const SRC: LanguageCode = LanguageCode {
    name: "Logudorese Sardinian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "src",
    individual_languages: &[
    ],
};


pub const SRE: LanguageCode = LanguageCode {
    name: "Sara",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sre",
    individual_languages: &[
    ],
};


pub const SRF: LanguageCode = LanguageCode {
    name: "Nafi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "srf",
    individual_languages: &[
    ],
};


pub const SRG: LanguageCode = LanguageCode {
    name: "Sulod",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "srg",
    individual_languages: &[
    ],
};


pub const SRH: LanguageCode = LanguageCode {
    name: "Sarikoli",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "srh",
    individual_languages: &[
    ],
};


pub const SRI: LanguageCode = LanguageCode {
    name: "Siriano",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sri",
    individual_languages: &[
    ],
};


pub const SRK: LanguageCode = LanguageCode {
    name: "Serudung Murut",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "srk",
    individual_languages: &[
    ],
};


pub const SRL: LanguageCode = LanguageCode {
    name: "Isirawa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "srl",
    individual_languages: &[
    ],
};


pub const SRM: LanguageCode = LanguageCode {
    name: "Saramaccan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "srm",
    individual_languages: &[
    ],
};


pub const SRO: LanguageCode = LanguageCode {
    name: "Campidanese Sardinian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sro",
    individual_languages: &[
    ],
};


pub const SRQ: LanguageCode = LanguageCode {
    name: "Sirionó",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "srq",
    individual_languages: &[
    ],
};


pub const SRS: LanguageCode = LanguageCode {
    name: "Sarsi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "srs",
    individual_languages: &[
    ],
};


pub const SRT: LanguageCode = LanguageCode {
    name: "Sauri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "srt",
    individual_languages: &[
    ],
};


pub const SRU: LanguageCode = LanguageCode {
    name: "Suruí",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sru",
    individual_languages: &[
    ],
};


pub const SRV: LanguageCode = LanguageCode {
    name: "Southern Sorsoganon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "srv",
    individual_languages: &[
    ],
};


pub const SRW: LanguageCode = LanguageCode {
    name: "Serua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "srw",
    individual_languages: &[
    ],
};


pub const SRX: LanguageCode = LanguageCode {
    name: "Sirmauri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "srx",
    individual_languages: &[
    ],
};


pub const SRY: LanguageCode = LanguageCode {
    name: "Sera",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sry",
    individual_languages: &[
    ],
};


pub const SRZ: LanguageCode = LanguageCode {
    name: "Shahmirzadi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "srz",
    individual_languages: &[
    ],
};


pub const SSB: LanguageCode = LanguageCode {
    name: "Southern Sama",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ssb",
    individual_languages: &[
    ],
};


pub const SSC: LanguageCode = LanguageCode {
    name: "Suba-Simbiti",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ssc",
    individual_languages: &[
    ],
};


pub const SSD: LanguageCode = LanguageCode {
    name: "Siroi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ssd",
    individual_languages: &[
    ],
};


pub const SSE: LanguageCode = LanguageCode {
    name: "Balangingi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sse",
    individual_languages: &[
    ],
};


pub const SSF: LanguageCode = LanguageCode {
    name: "Thao",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ssf",
    individual_languages: &[
    ],
};


pub const SSG: LanguageCode = LanguageCode {
    name: "Seimat",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ssg",
    individual_languages: &[
    ],
};


pub const SSH: LanguageCode = LanguageCode {
    name: "Shihhi Arabic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ssh",
    individual_languages: &[
    ],
};


pub const SSI: LanguageCode = LanguageCode {
    name: "Sansi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ssi",
    individual_languages: &[
    ],
};


pub const SSJ: LanguageCode = LanguageCode {
    name: "Sausi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ssj",
    individual_languages: &[
    ],
};


pub const SSK: LanguageCode = LanguageCode {
    name: "Sunam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ssk",
    individual_languages: &[
    ],
};


pub const SSL: LanguageCode = LanguageCode {
    name: "Western Sisaala",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ssl",
    individual_languages: &[
    ],
};


pub const SSM: LanguageCode = LanguageCode {
    name: "Semnam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ssm",
    individual_languages: &[
    ],
};


pub const SSN: LanguageCode = LanguageCode {
    name: "Waata",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ssn",
    individual_languages: &[
    ],
};


pub const SSO: LanguageCode = LanguageCode {
    name: "Sissano",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sso",
    individual_languages: &[
    ],
};


pub const SSP: LanguageCode = LanguageCode {
    name: "Spanish Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ssp",
    individual_languages: &[
    ],
};


pub const SSQ: LanguageCode = LanguageCode {
    name: "So'a",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ssq",
    individual_languages: &[
    ],
};


pub const SSR: LanguageCode = LanguageCode {
    name: "Swiss-French Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ssr",
    individual_languages: &[
    ],
};


pub const SSS: LanguageCode = LanguageCode {
    name: "Sô",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sss",
    individual_languages: &[
    ],
};


pub const SST: LanguageCode = LanguageCode {
    name: "Sinasina",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sst",
    individual_languages: &[
    ],
};


pub const SSU: LanguageCode = LanguageCode {
    name: "Susuami",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ssu",
    individual_languages: &[
    ],
};


pub const SSV: LanguageCode = LanguageCode {
    name: "Shark Bay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ssv",
    individual_languages: &[
    ],
};


pub const SSX: LanguageCode = LanguageCode {
    name: "Samberigi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ssx",
    individual_languages: &[
    ],
};


pub const SSY: LanguageCode = LanguageCode {
    name: "Saho",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ssy",
    individual_languages: &[
    ],
};


pub const SSZ: LanguageCode = LanguageCode {
    name: "Sengseng",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ssz",
    individual_languages: &[
    ],
};


pub const STA: LanguageCode = LanguageCode {
    name: "Settla",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sta",
    individual_languages: &[
    ],
};


pub const STB: LanguageCode = LanguageCode {
    name: "Northern Subanen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "stb",
    individual_languages: &[
    ],
};


pub const STD: LanguageCode = LanguageCode {
    name: "Sentinel",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "std",
    individual_languages: &[
    ],
};


pub const STE: LanguageCode = LanguageCode {
    name: "Liana-Seti",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ste",
    individual_languages: &[
    ],
};


pub const STF: LanguageCode = LanguageCode {
    name: "Seta",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "stf",
    individual_languages: &[
    ],
};


pub const STG: LanguageCode = LanguageCode {
    name: "Trieng",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "stg",
    individual_languages: &[
    ],
};


pub const STH: LanguageCode = LanguageCode {
    name: "Shelta",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sth",
    individual_languages: &[
    ],
};


pub const STI: LanguageCode = LanguageCode {
    name: "Bulo Stieng",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sti",
    individual_languages: &[
    ],
};


pub const STJ: LanguageCode = LanguageCode {
    name: "Matya Samo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "stj",
    individual_languages: &[
    ],
};


pub const STK: LanguageCode = LanguageCode {
    name: "Arammba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "stk",
    individual_languages: &[
    ],
};


pub const STL: LanguageCode = LanguageCode {
    name: "Stellingwerfs",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "stl",
    individual_languages: &[
    ],
};


pub const STM: LanguageCode = LanguageCode {
    name: "Setaman",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "stm",
    individual_languages: &[
    ],
};


pub const STN: LanguageCode = LanguageCode {
    name: "Owa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "stn",
    individual_languages: &[
    ],
};


pub const STO: LanguageCode = LanguageCode {
    name: "Stoney",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sto",
    individual_languages: &[
    ],
};


pub const STP: LanguageCode = LanguageCode {
    name: "Southeastern Tepehuan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "stp",
    individual_languages: &[
    ],
};


pub const STQ: LanguageCode = LanguageCode {
    name: "Saterfriesisch",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "stq",
    individual_languages: &[
    ],
};


pub const STR: LanguageCode = LanguageCode {
    name: "Straits Salish",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "str",
    individual_languages: &[
    ],
};


pub const STS: LanguageCode = LanguageCode {
    name: "Shumashti",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sts",
    individual_languages: &[
    ],
};


pub const STT: LanguageCode = LanguageCode {
    name: "Budeh Stieng",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "stt",
    individual_languages: &[
    ],
};


pub const STU: LanguageCode = LanguageCode {
    name: "Samtao",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "stu",
    individual_languages: &[
    ],
};


pub const STV: LanguageCode = LanguageCode {
    name: "Silt'e",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "stv",
    individual_languages: &[
    ],
};


pub const STW: LanguageCode = LanguageCode {
    name: "Satawalese",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "stw",
    individual_languages: &[
    ],
};


pub const STY: LanguageCode = LanguageCode {
    name: "Siberian Tatar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sty",
    individual_languages: &[
    ],
};


pub const SUA: LanguageCode = LanguageCode {
    name: "Sulka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sua",
    individual_languages: &[
    ],
};


pub const SUB: LanguageCode = LanguageCode {
    name: "Suku",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sub",
    individual_languages: &[
    ],
};


pub const SUC: LanguageCode = LanguageCode {
    name: "Western Subanon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "suc",
    individual_languages: &[
    ],
};


pub const SUE: LanguageCode = LanguageCode {
    name: "Suena",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sue",
    individual_languages: &[
    ],
};


pub const SUG: LanguageCode = LanguageCode {
    name: "Suganga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sug",
    individual_languages: &[
    ],
};


pub const SUI: LanguageCode = LanguageCode {
    name: "Suki",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sui",
    individual_languages: &[
    ],
};


pub const SUJ: LanguageCode = LanguageCode {
    name: "Shubi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "suj",
    individual_languages: &[
    ],
};


pub const SUO: LanguageCode = LanguageCode {
    name: "Bouni",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "suo",
    individual_languages: &[
    ],
};


pub const SUQ: LanguageCode = LanguageCode {
    name: "Tirmaga-Chai Suri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "suq",
    individual_languages: &[
    ],
};


pub const SUR: LanguageCode = LanguageCode {
    name: "Mwaghavul",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sur",
    individual_languages: &[
    ],
};


pub const SUT: LanguageCode = LanguageCode {
    name: "Subtiaba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sut",
    individual_languages: &[
    ],
};


pub const SUV: LanguageCode = LanguageCode {
    name: "Puroik",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "suv",
    individual_languages: &[
    ],
};


pub const SUW: LanguageCode = LanguageCode {
    name: "Sumbwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "suw",
    individual_languages: &[
    ],
};


pub const SUY: LanguageCode = LanguageCode {
    name: "Suyá",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "suy",
    individual_languages: &[
    ],
};


pub const SUZ: LanguageCode = LanguageCode {
    name: "Sunwar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "suz",
    individual_languages: &[
    ],
};


pub const SVA: LanguageCode = LanguageCode {
    name: "Svan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sva",
    individual_languages: &[
    ],
};


pub const SVB: LanguageCode = LanguageCode {
    name: "Ulau-Suain",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "svb",
    individual_languages: &[
    ],
};


pub const SVC: LanguageCode = LanguageCode {
    name: "Vincentian Creole English",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "svc",
    individual_languages: &[
    ],
};


pub const SVE: LanguageCode = LanguageCode {
    name: "Serili",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sve",
    individual_languages: &[
    ],
};


pub const SVK: LanguageCode = LanguageCode {
    name: "Slovakian Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "svk",
    individual_languages: &[
    ],
};


pub const SVM: LanguageCode = LanguageCode {
    name: "Slavomolisano",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "svm",
    individual_languages: &[
    ],
};


pub const SVS: LanguageCode = LanguageCode {
    name: "Savosavo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "svs",
    individual_languages: &[
    ],
};


pub const SVX: LanguageCode = LanguageCode {
    name: "Skalvian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "svx",
    individual_languages: &[
    ],
};


pub const SWB: LanguageCode = LanguageCode {
    name: "Maore Comorian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "swb",
    individual_languages: &[
    ],
};


pub const SWC: LanguageCode = LanguageCode {
    name: "Congo Swahili",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "swc",
    individual_languages: &[
    ],
};


pub const SWF: LanguageCode = LanguageCode {
    name: "Sere",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "swf",
    individual_languages: &[
    ],
};


pub const SWG: LanguageCode = LanguageCode {
    name: "Swabian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "swg",
    individual_languages: &[
    ],
};


pub const SWH: LanguageCode = LanguageCode {
    name: "Swahili (individual language)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "swh",
    individual_languages: &[
    ],
};


pub const SWI: LanguageCode = LanguageCode {
    name: "Sui",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "swi",
    individual_languages: &[
    ],
};


pub const SWJ: LanguageCode = LanguageCode {
    name: "Sira",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "swj",
    individual_languages: &[
    ],
};


pub const SWK: LanguageCode = LanguageCode {
    name: "Malawi Sena",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "swk",
    individual_languages: &[
    ],
};


pub const SWL: LanguageCode = LanguageCode {
    name: "Swedish Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "swl",
    individual_languages: &[
    ],
};


pub const SWM: LanguageCode = LanguageCode {
    name: "Samosa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "swm",
    individual_languages: &[
    ],
};


pub const SWN: LanguageCode = LanguageCode {
    name: "Sawknah",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "swn",
    individual_languages: &[
    ],
};


pub const SWO: LanguageCode = LanguageCode {
    name: "Shanenawa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "swo",
    individual_languages: &[
    ],
};


pub const SWP: LanguageCode = LanguageCode {
    name: "Suau",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "swp",
    individual_languages: &[
    ],
};


pub const SWQ: LanguageCode = LanguageCode {
    name: "Sharwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "swq",
    individual_languages: &[
    ],
};


pub const SWR: LanguageCode = LanguageCode {
    name: "Saweru",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "swr",
    individual_languages: &[
    ],
};


pub const SWS: LanguageCode = LanguageCode {
    name: "Seluwasan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sws",
    individual_languages: &[
    ],
};


pub const SWT: LanguageCode = LanguageCode {
    name: "Sawila",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "swt",
    individual_languages: &[
    ],
};


pub const SWU: LanguageCode = LanguageCode {
    name: "Suwawa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "swu",
    individual_languages: &[
    ],
};


pub const SWV: LanguageCode = LanguageCode {
    name: "Shekhawati",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "swv",
    individual_languages: &[
    ],
};


pub const SWW: LanguageCode = LanguageCode {
    name: "Sowa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sww",
    individual_languages: &[
    ],
};


pub const SWX: LanguageCode = LanguageCode {
    name: "Suruahá",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "swx",
    individual_languages: &[
    ],
};


pub const SWY: LanguageCode = LanguageCode {
    name: "Sarua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "swy",
    individual_languages: &[
    ],
};


pub const SXB: LanguageCode = LanguageCode {
    name: "Suba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sxb",
    individual_languages: &[
    ],
};


pub const SXC: LanguageCode = LanguageCode {
    name: "Sicanian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sxc",
    individual_languages: &[
    ],
};


pub const SXE: LanguageCode = LanguageCode {
    name: "Sighu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sxe",
    individual_languages: &[
    ],
};


pub const SXG: LanguageCode = LanguageCode {
    name: "Shuhi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sxg",
    individual_languages: &[
    ],
};


pub const SXK: LanguageCode = LanguageCode {
    name: "Southern Kalapuya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sxk",
    individual_languages: &[
    ],
};


pub const SXL: LanguageCode = LanguageCode {
    name: "Selian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sxl",
    individual_languages: &[
    ],
};


pub const SXM: LanguageCode = LanguageCode {
    name: "Samre",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sxm",
    individual_languages: &[
    ],
};


pub const SXN: LanguageCode = LanguageCode {
    name: "Sangir",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sxn",
    individual_languages: &[
    ],
};


pub const SXO: LanguageCode = LanguageCode {
    name: "Sorothaptic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sxo",
    individual_languages: &[
    ],
};


pub const SXR: LanguageCode = LanguageCode {
    name: "Saaroa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sxr",
    individual_languages: &[
    ],
};


pub const SXS: LanguageCode = LanguageCode {
    name: "Sasaru",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sxs",
    individual_languages: &[
    ],
};


pub const SXU: LanguageCode = LanguageCode {
    name: "Upper Saxon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sxu",
    individual_languages: &[
    ],
};


pub const SXW: LanguageCode = LanguageCode {
    name: "Saxwe Gbe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sxw",
    individual_languages: &[
    ],
};


pub const SYA: LanguageCode = LanguageCode {
    name: "Siang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sya",
    individual_languages: &[
    ],
};


pub const SYB: LanguageCode = LanguageCode {
    name: "Central Subanen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "syb",
    individual_languages: &[
    ],
};


pub const SYI: LanguageCode = LanguageCode {
    name: "Seki",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "syi",
    individual_languages: &[
    ],
};


pub const SYK: LanguageCode = LanguageCode {
    name: "Sukur",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "syk",
    individual_languages: &[
    ],
};


pub const SYL: LanguageCode = LanguageCode {
    name: "Sylheti",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "syl",
    individual_languages: &[
    ],
};


pub const SYM: LanguageCode = LanguageCode {
    name: "Maya Samo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sym",
    individual_languages: &[
    ],
};


pub const SYN: LanguageCode = LanguageCode {
    name: "Senaya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "syn",
    individual_languages: &[
    ],
};


pub const SYO: LanguageCode = LanguageCode {
    name: "Suoy",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "syo",
    individual_languages: &[
    ],
};


pub const SYS: LanguageCode = LanguageCode {
    name: "Sinyar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sys",
    individual_languages: &[
    ],
};


pub const SYW: LanguageCode = LanguageCode {
    name: "Kagate",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "syw",
    individual_languages: &[
    ],
};


pub const SYX: LanguageCode = LanguageCode {
    name: "Samay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "syx",
    individual_languages: &[
    ],
};


pub const SYY: LanguageCode = LanguageCode {
    name: "Al-Sayyid Bedouin Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "syy",
    individual_languages: &[
    ],
};


pub const SZA: LanguageCode = LanguageCode {
    name: "Semelai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sza",
    individual_languages: &[
    ],
};


pub const SZB: LanguageCode = LanguageCode {
    name: "Ngalum",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "szb",
    individual_languages: &[
    ],
};


pub const SZC: LanguageCode = LanguageCode {
    name: "Semaq Beri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "szc",
    individual_languages: &[
    ],
};


pub const SZD: LanguageCode = LanguageCode {
    name: "Seru",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "szd",
    individual_languages: &[
    ],
};


pub const SZE: LanguageCode = LanguageCode {
    name: "Seze",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "sze",
    individual_languages: &[
    ],
};


pub const SZG: LanguageCode = LanguageCode {
    name: "Sengele",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "szg",
    individual_languages: &[
    ],
};


pub const SZL: LanguageCode = LanguageCode {
    name: "Silesian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "szl",
    individual_languages: &[
    ],
};


pub const SZN: LanguageCode = LanguageCode {
    name: "Sula",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "szn",
    individual_languages: &[
    ],
};


pub const SZP: LanguageCode = LanguageCode {
    name: "Suabo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "szp",
    individual_languages: &[
    ],
};


pub const SZS: LanguageCode = LanguageCode {
    name: "Solomon Islands Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "szs",
    individual_languages: &[
    ],
};


pub const SZV: LanguageCode = LanguageCode {
    name: "Isu (Fako Division)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "szv",
    individual_languages: &[
    ],
};


pub const SZW: LanguageCode = LanguageCode {
    name: "Sawai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "szw",
    individual_languages: &[
    ],
};


pub const SZY: LanguageCode = LanguageCode {
    name: "Sakizaya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "szy",
    individual_languages: &[
    ],
};


pub const TAA: LanguageCode = LanguageCode {
    name: "Lower Tanana",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "taa",
    individual_languages: &[
    ],
};


pub const TAB: LanguageCode = LanguageCode {
    name: "Tabassaran",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tab",
    individual_languages: &[
    ],
};


pub const TAC: LanguageCode = LanguageCode {
    name: "Lowland Tarahumara",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tac",
    individual_languages: &[
    ],
};


pub const TAD: LanguageCode = LanguageCode {
    name: "Tause",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tad",
    individual_languages: &[
    ],
};


pub const TAE: LanguageCode = LanguageCode {
    name: "Tariana",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tae",
    individual_languages: &[
    ],
};


pub const TAF: LanguageCode = LanguageCode {
    name: "Tapirapé",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "taf",
    individual_languages: &[
    ],
};


pub const TAG: LanguageCode = LanguageCode {
    name: "Tagoi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tag",
    individual_languages: &[
    ],
};


pub const TAJ: LanguageCode = LanguageCode {
    name: "Eastern Tamang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "taj",
    individual_languages: &[
    ],
};


pub const TAK: LanguageCode = LanguageCode {
    name: "Tala",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tak",
    individual_languages: &[
    ],
};


pub const TAL: LanguageCode = LanguageCode {
    name: "Tal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tal",
    individual_languages: &[
    ],
};


pub const TAN: LanguageCode = LanguageCode {
    name: "Tangale",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tan",
    individual_languages: &[
    ],
};


pub const TAO: LanguageCode = LanguageCode {
    name: "Yami",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tao",
    individual_languages: &[
    ],
};


pub const TAP: LanguageCode = LanguageCode {
    name: "Taabwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tap",
    individual_languages: &[
    ],
};


pub const TAQ: LanguageCode = LanguageCode {
    name: "Tamasheq",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "taq",
    individual_languages: &[
    ],
};


pub const TAR: LanguageCode = LanguageCode {
    name: "Central Tarahumara",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tar",
    individual_languages: &[
    ],
};


pub const TAS: LanguageCode = LanguageCode {
    name: "Tay Boi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tas",
    individual_languages: &[
    ],
};


pub const TAU: LanguageCode = LanguageCode {
    name: "Upper Tanana",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tau",
    individual_languages: &[
    ],
};


pub const TAV: LanguageCode = LanguageCode {
    name: "Tatuyo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tav",
    individual_languages: &[
    ],
};


pub const TAW: LanguageCode = LanguageCode {
    name: "Tai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "taw",
    individual_languages: &[
    ],
};


pub const TAX: LanguageCode = LanguageCode {
    name: "Tamki",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tax",
    individual_languages: &[
    ],
};


pub const TAY: LanguageCode = LanguageCode {
    name: "Atayal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tay",
    individual_languages: &[
    ],
};


pub const TAZ: LanguageCode = LanguageCode {
    name: "Tocho",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "taz",
    individual_languages: &[
    ],
};


pub const TBA: LanguageCode = LanguageCode {
    name: "Aikanã",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tba",
    individual_languages: &[
    ],
};


pub const TBC: LanguageCode = LanguageCode {
    name: "Takia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tbc",
    individual_languages: &[
    ],
};


pub const TBD: LanguageCode = LanguageCode {
    name: "Kaki Ae",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tbd",
    individual_languages: &[
    ],
};


pub const TBE: LanguageCode = LanguageCode {
    name: "Tanimbili",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tbe",
    individual_languages: &[
    ],
};


pub const TBF: LanguageCode = LanguageCode {
    name: "Mandara",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tbf",
    individual_languages: &[
    ],
};


pub const TBG: LanguageCode = LanguageCode {
    name: "North Tairora",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tbg",
    individual_languages: &[
    ],
};


pub const TBH: LanguageCode = LanguageCode {
    name: "Dharawal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tbh",
    individual_languages: &[
    ],
};


pub const TBI: LanguageCode = LanguageCode {
    name: "Gaam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tbi",
    individual_languages: &[
    ],
};


pub const TBJ: LanguageCode = LanguageCode {
    name: "Tiang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tbj",
    individual_languages: &[
    ],
};


pub const TBK: LanguageCode = LanguageCode {
    name: "Calamian Tagbanwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tbk",
    individual_languages: &[
    ],
};


pub const TBL: LanguageCode = LanguageCode {
    name: "Tboli",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tbl",
    individual_languages: &[
    ],
};


pub const TBM: LanguageCode = LanguageCode {
    name: "Tagbu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tbm",
    individual_languages: &[
    ],
};


pub const TBN: LanguageCode = LanguageCode {
    name: "Barro Negro Tunebo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tbn",
    individual_languages: &[
    ],
};


pub const TBO: LanguageCode = LanguageCode {
    name: "Tawala",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tbo",
    individual_languages: &[
    ],
};


pub const TBP: LanguageCode = LanguageCode {
    name: "Taworta",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tbp",
    individual_languages: &[
    ],
};


pub const TBR: LanguageCode = LanguageCode {
    name: "Tumtum",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tbr",
    individual_languages: &[
    ],
};


pub const TBS: LanguageCode = LanguageCode {
    name: "Tanguat",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tbs",
    individual_languages: &[
    ],
};


pub const TBT: LanguageCode = LanguageCode {
    name: "Tembo (Kitembo)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tbt",
    individual_languages: &[
    ],
};


pub const TBU: LanguageCode = LanguageCode {
    name: "Tubar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tbu",
    individual_languages: &[
    ],
};


pub const TBV: LanguageCode = LanguageCode {
    name: "Tobo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tbv",
    individual_languages: &[
    ],
};


pub const TBW: LanguageCode = LanguageCode {
    name: "Tagbanwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tbw",
    individual_languages: &[
    ],
};


pub const TBX: LanguageCode = LanguageCode {
    name: "Kapin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tbx",
    individual_languages: &[
    ],
};


pub const TBY: LanguageCode = LanguageCode {
    name: "Tabaru",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tby",
    individual_languages: &[
    ],
};


pub const TBZ: LanguageCode = LanguageCode {
    name: "Ditammari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tbz",
    individual_languages: &[
    ],
};


pub const TCA: LanguageCode = LanguageCode {
    name: "Ticuna",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tca",
    individual_languages: &[
    ],
};


pub const TCB: LanguageCode = LanguageCode {
    name: "Tanacross",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tcb",
    individual_languages: &[
    ],
};


pub const TCC: LanguageCode = LanguageCode {
    name: "Datooga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tcc",
    individual_languages: &[
    ],
};


pub const TCD: LanguageCode = LanguageCode {
    name: "Tafi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tcd",
    individual_languages: &[
    ],
};


pub const TCE: LanguageCode = LanguageCode {
    name: "Southern Tutchone",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tce",
    individual_languages: &[
    ],
};


pub const TCF: LanguageCode = LanguageCode {
    name: "Malinaltepec Me'phaa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tcf",
    individual_languages: &[
    ],
};


pub const TCG: LanguageCode = LanguageCode {
    name: "Tamagario",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tcg",
    individual_languages: &[
    ],
};


pub const TCH: LanguageCode = LanguageCode {
    name: "Turks And Caicos Creole English",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tch",
    individual_languages: &[
    ],
};


pub const TCI: LanguageCode = LanguageCode {
    name: "Wára",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tci",
    individual_languages: &[
    ],
};


pub const TCK: LanguageCode = LanguageCode {
    name: "Tchitchege",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tck",
    individual_languages: &[
    ],
};


pub const TCL: LanguageCode = LanguageCode {
    name: "Taman (Myanmar)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tcl",
    individual_languages: &[
    ],
};


pub const TCM: LanguageCode = LanguageCode {
    name: "Tanahmerah",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tcm",
    individual_languages: &[
    ],
};


pub const TCN: LanguageCode = LanguageCode {
    name: "Tichurong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tcn",
    individual_languages: &[
    ],
};


pub const TCO: LanguageCode = LanguageCode {
    name: "Taungyo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tco",
    individual_languages: &[
    ],
};


pub const TCP: LanguageCode = LanguageCode {
    name: "Tawr Chin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tcp",
    individual_languages: &[
    ],
};


pub const TCQ: LanguageCode = LanguageCode {
    name: "Kaiy",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tcq",
    individual_languages: &[
    ],
};


pub const TCS: LanguageCode = LanguageCode {
    name: "Torres Strait Creole",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tcs",
    individual_languages: &[
    ],
};


pub const TCT: LanguageCode = LanguageCode {
    name: "T'en",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tct",
    individual_languages: &[
    ],
};


pub const TCU: LanguageCode = LanguageCode {
    name: "Southeastern Tarahumara",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tcu",
    individual_languages: &[
    ],
};


pub const TCW: LanguageCode = LanguageCode {
    name: "Tecpatlán Totonac",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tcw",
    individual_languages: &[
    ],
};


pub const TCX: LanguageCode = LanguageCode {
    name: "Toda",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tcx",
    individual_languages: &[
    ],
};


pub const TCY: LanguageCode = LanguageCode {
    name: "Tulu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tcy",
    individual_languages: &[
    ],
};


pub const TCZ: LanguageCode = LanguageCode {
    name: "Thado Chin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tcz",
    individual_languages: &[
    ],
};


pub const TDA: LanguageCode = LanguageCode {
    name: "Tagdal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tda",
    individual_languages: &[
    ],
};


pub const TDB: LanguageCode = LanguageCode {
    name: "Panchpargania",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tdb",
    individual_languages: &[
    ],
};


pub const TDC: LanguageCode = LanguageCode {
    name: "Emberá-Tadó",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tdc",
    individual_languages: &[
    ],
};


pub const TDD: LanguageCode = LanguageCode {
    name: "Tai Nüa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tdd",
    individual_languages: &[
    ],
};


pub const TDE: LanguageCode = LanguageCode {
    name: "Tiranige Diga Dogon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tde",
    individual_languages: &[
    ],
};


pub const TDF: LanguageCode = LanguageCode {
    name: "Talieng",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tdf",
    individual_languages: &[
    ],
};


pub const TDG: LanguageCode = LanguageCode {
    name: "Western Tamang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tdg",
    individual_languages: &[
    ],
};


pub const TDH: LanguageCode = LanguageCode {
    name: "Thulung",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tdh",
    individual_languages: &[
    ],
};


pub const TDI: LanguageCode = LanguageCode {
    name: "Tomadino",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tdi",
    individual_languages: &[
    ],
};


pub const TDJ: LanguageCode = LanguageCode {
    name: "Tajio",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tdj",
    individual_languages: &[
    ],
};


pub const TDK: LanguageCode = LanguageCode {
    name: "Tambas",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tdk",
    individual_languages: &[
    ],
};


pub const TDL: LanguageCode = LanguageCode {
    name: "Sur",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tdl",
    individual_languages: &[
    ],
};


pub const TDM: LanguageCode = LanguageCode {
    name: "Taruma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tdm",
    individual_languages: &[
    ],
};


pub const TDN: LanguageCode = LanguageCode {
    name: "Tondano",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tdn",
    individual_languages: &[
    ],
};


pub const TDO: LanguageCode = LanguageCode {
    name: "Teme",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tdo",
    individual_languages: &[
    ],
};


pub const TDQ: LanguageCode = LanguageCode {
    name: "Tita",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tdq",
    individual_languages: &[
    ],
};


pub const TDR: LanguageCode = LanguageCode {
    name: "Todrah",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tdr",
    individual_languages: &[
    ],
};


pub const TDS: LanguageCode = LanguageCode {
    name: "Doutai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tds",
    individual_languages: &[
    ],
};


pub const TDT: LanguageCode = LanguageCode {
    name: "Tetun Dili",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tdt",
    individual_languages: &[
    ],
};


pub const TDV: LanguageCode = LanguageCode {
    name: "Toro",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tdv",
    individual_languages: &[
    ],
};


pub const TDX: LanguageCode = LanguageCode {
    name: "Tandroy-Mahafaly Malagasy",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tdx",
    individual_languages: &[
    ],
};


pub const TDY: LanguageCode = LanguageCode {
    name: "Tadyawan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tdy",
    individual_languages: &[
    ],
};


pub const TEA: LanguageCode = LanguageCode {
    name: "Temiar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tea",
    individual_languages: &[
    ],
};


pub const TEB: LanguageCode = LanguageCode {
    name: "Tetete",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "teb",
    individual_languages: &[
    ],
};


pub const TEC: LanguageCode = LanguageCode {
    name: "Terik",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tec",
    individual_languages: &[
    ],
};


pub const TED: LanguageCode = LanguageCode {
    name: "Tepo Krumen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ted",
    individual_languages: &[
    ],
};


pub const TEE: LanguageCode = LanguageCode {
    name: "Huehuetla Tepehua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tee",
    individual_languages: &[
    ],
};


pub const TEF: LanguageCode = LanguageCode {
    name: "Teressa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tef",
    individual_languages: &[
    ],
};


pub const TEG: LanguageCode = LanguageCode {
    name: "Teke-Tege",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "teg",
    individual_languages: &[
    ],
};


pub const TEH: LanguageCode = LanguageCode {
    name: "Tehuelche",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "teh",
    individual_languages: &[
    ],
};


pub const TEI: LanguageCode = LanguageCode {
    name: "Torricelli",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tei",
    individual_languages: &[
    ],
};


pub const TEK: LanguageCode = LanguageCode {
    name: "Ibali Teke",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tek",
    individual_languages: &[
    ],
};


pub const TEN: LanguageCode = LanguageCode {
    name: "Tama (Colombia)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ten",
    individual_languages: &[
    ],
};


pub const TEO: LanguageCode = LanguageCode {
    name: "Teso",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "teo",
    individual_languages: &[
    ],
};


pub const TEP: LanguageCode = LanguageCode {
    name: "Tepecano",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tep",
    individual_languages: &[
    ],
};


pub const TEQ: LanguageCode = LanguageCode {
    name: "Temein",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "teq",
    individual_languages: &[
    ],
};


pub const TES: LanguageCode = LanguageCode {
    name: "Tengger",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tes",
    individual_languages: &[
    ],
};


pub const TEU: LanguageCode = LanguageCode {
    name: "Soo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "teu",
    individual_languages: &[
    ],
};


pub const TEV: LanguageCode = LanguageCode {
    name: "Teor",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tev",
    individual_languages: &[
    ],
};


pub const TEW: LanguageCode = LanguageCode {
    name: "Tewa (USA)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tew",
    individual_languages: &[
    ],
};


pub const TEX: LanguageCode = LanguageCode {
    name: "Tennet",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tex",
    individual_languages: &[
    ],
};


pub const TEY: LanguageCode = LanguageCode {
    name: "Tulishi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tey",
    individual_languages: &[
    ],
};


pub const TEZ: LanguageCode = LanguageCode {
    name: "Tetserret",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tez",
    individual_languages: &[
    ],
};


pub const TFI: LanguageCode = LanguageCode {
    name: "Tofin Gbe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tfi",
    individual_languages: &[
    ],
};


pub const TFN: LanguageCode = LanguageCode {
    name: "Tanaina",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tfn",
    individual_languages: &[
    ],
};


pub const TFO: LanguageCode = LanguageCode {
    name: "Tefaro",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tfo",
    individual_languages: &[
    ],
};


pub const TFR: LanguageCode = LanguageCode {
    name: "Teribe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tfr",
    individual_languages: &[
    ],
};


pub const TFT: LanguageCode = LanguageCode {
    name: "Ternate",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tft",
    individual_languages: &[
    ],
};


pub const TGA: LanguageCode = LanguageCode {
    name: "Sagalla",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tga",
    individual_languages: &[
    ],
};


pub const TGB: LanguageCode = LanguageCode {
    name: "Tobilung",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tgb",
    individual_languages: &[
    ],
};


pub const TGC: LanguageCode = LanguageCode {
    name: "Tigak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tgc",
    individual_languages: &[
    ],
};


pub const TGD: LanguageCode = LanguageCode {
    name: "Ciwogai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tgd",
    individual_languages: &[
    ],
};


pub const TGE: LanguageCode = LanguageCode {
    name: "Eastern Gorkha Tamang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tge",
    individual_languages: &[
    ],
};


pub const TGF: LanguageCode = LanguageCode {
    name: "Chalikha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tgf",
    individual_languages: &[
    ],
};


pub const TGH: LanguageCode = LanguageCode {
    name: "Tobagonian Creole English",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tgh",
    individual_languages: &[
    ],
};


pub const TGI: LanguageCode = LanguageCode {
    name: "Lawunuia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tgi",
    individual_languages: &[
    ],
};


pub const TGJ: LanguageCode = LanguageCode {
    name: "Tagin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tgj",
    individual_languages: &[
    ],
};


pub const TGN: LanguageCode = LanguageCode {
    name: "Tandaganon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tgn",
    individual_languages: &[
    ],
};


pub const TGO: LanguageCode = LanguageCode {
    name: "Sudest",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tgo",
    individual_languages: &[
    ],
};


pub const TGP: LanguageCode = LanguageCode {
    name: "Tangoa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tgp",
    individual_languages: &[
    ],
};


pub const TGQ: LanguageCode = LanguageCode {
    name: "Tring",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tgq",
    individual_languages: &[
    ],
};


pub const TGR: LanguageCode = LanguageCode {
    name: "Tareng",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tgr",
    individual_languages: &[
    ],
};


pub const TGS: LanguageCode = LanguageCode {
    name: "Nume",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tgs",
    individual_languages: &[
    ],
};


pub const TGT: LanguageCode = LanguageCode {
    name: "Central Tagbanwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tgt",
    individual_languages: &[
    ],
};


pub const TGU: LanguageCode = LanguageCode {
    name: "Tanggu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tgu",
    individual_languages: &[
    ],
};


pub const TGV: LanguageCode = LanguageCode {
    name: "Tingui-Boto",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tgv",
    individual_languages: &[
    ],
};


pub const TGW: LanguageCode = LanguageCode {
    name: "Tagwana Senoufo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tgw",
    individual_languages: &[
    ],
};


pub const TGX: LanguageCode = LanguageCode {
    name: "Tagish",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tgx",
    individual_languages: &[
    ],
};


pub const TGY: LanguageCode = LanguageCode {
    name: "Togoyo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tgy",
    individual_languages: &[
    ],
};


pub const TGZ: LanguageCode = LanguageCode {
    name: "Tagalaka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tgz",
    individual_languages: &[
    ],
};


pub const THD: LanguageCode = LanguageCode {
    name: "Kuuk Thaayorre",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "thd",
    individual_languages: &[
    ],
};


pub const THE: LanguageCode = LanguageCode {
    name: "Chitwania Tharu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "the",
    individual_languages: &[
    ],
};


pub const THF: LanguageCode = LanguageCode {
    name: "Thangmi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "thf",
    individual_languages: &[
    ],
};


pub const THH: LanguageCode = LanguageCode {
    name: "Northern Tarahumara",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "thh",
    individual_languages: &[
    ],
};


pub const THI: LanguageCode = LanguageCode {
    name: "Tai Long",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "thi",
    individual_languages: &[
    ],
};


pub const THK: LanguageCode = LanguageCode {
    name: "Tharaka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "thk",
    individual_languages: &[
    ],
};


pub const THL: LanguageCode = LanguageCode {
    name: "Dangaura Tharu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "thl",
    individual_languages: &[
    ],
};


pub const THM: LanguageCode = LanguageCode {
    name: "Aheu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "thm",
    individual_languages: &[
    ],
};


pub const THN: LanguageCode = LanguageCode {
    name: "Thachanadan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "thn",
    individual_languages: &[
    ],
};


pub const THP: LanguageCode = LanguageCode {
    name: "Thompson",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "thp",
    individual_languages: &[
    ],
};


pub const THQ: LanguageCode = LanguageCode {
    name: "Kochila Tharu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "thq",
    individual_languages: &[
    ],
};


pub const THR: LanguageCode = LanguageCode {
    name: "Rana Tharu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "thr",
    individual_languages: &[
    ],
};


pub const THS: LanguageCode = LanguageCode {
    name: "Thakali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ths",
    individual_languages: &[
    ],
};


pub const THT: LanguageCode = LanguageCode {
    name: "Tahltan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tht",
    individual_languages: &[
    ],
};


pub const THU: LanguageCode = LanguageCode {
    name: "Thuri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "thu",
    individual_languages: &[
    ],
};


pub const THV: LanguageCode = LanguageCode {
    name: "Tahaggart Tamahaq",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "thv",
    individual_languages: &[
    ],
};


pub const THY: LanguageCode = LanguageCode {
    name: "Tha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "thy",
    individual_languages: &[
    ],
};


pub const THZ: LanguageCode = LanguageCode {
    name: "Tayart Tamajeq",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "thz",
    individual_languages: &[
    ],
};


pub const TIA: LanguageCode = LanguageCode {
    name: "Tidikelt Tamazight",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tia",
    individual_languages: &[
    ],
};


pub const TIC: LanguageCode = LanguageCode {
    name: "Tira",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tic",
    individual_languages: &[
    ],
};


pub const TIF: LanguageCode = LanguageCode {
    name: "Tifal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tif",
    individual_languages: &[
    ],
};


pub const TIH: LanguageCode = LanguageCode {
    name: "Timugon Murut",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tih",
    individual_languages: &[
    ],
};


pub const TII: LanguageCode = LanguageCode {
    name: "Tiene",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tii",
    individual_languages: &[
    ],
};


pub const TIJ: LanguageCode = LanguageCode {
    name: "Tilung",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tij",
    individual_languages: &[
    ],
};


pub const TIK: LanguageCode = LanguageCode {
    name: "Tikar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tik",
    individual_languages: &[
    ],
};


pub const TIL: LanguageCode = LanguageCode {
    name: "Tillamook",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "til",
    individual_languages: &[
    ],
};


pub const TIM: LanguageCode = LanguageCode {
    name: "Timbe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tim",
    individual_languages: &[
    ],
};


pub const TIN: LanguageCode = LanguageCode {
    name: "Tindi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tin",
    individual_languages: &[
    ],
};


pub const TIO: LanguageCode = LanguageCode {
    name: "Teop",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tio",
    individual_languages: &[
    ],
};


pub const TIP: LanguageCode = LanguageCode {
    name: "Trimuris",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tip",
    individual_languages: &[
    ],
};


pub const TIQ: LanguageCode = LanguageCode {
    name: "Tiéfo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tiq",
    individual_languages: &[
    ],
};


pub const TIS: LanguageCode = LanguageCode {
    name: "Masadiit Itneg",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tis",
    individual_languages: &[
    ],
};


pub const TIT: LanguageCode = LanguageCode {
    name: "Tinigua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tit",
    individual_languages: &[
    ],
};


pub const TIU: LanguageCode = LanguageCode {
    name: "Adasen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tiu",
    individual_languages: &[
    ],
};


pub const TIW: LanguageCode = LanguageCode {
    name: "Tiwi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tiw",
    individual_languages: &[
    ],
};


pub const TIX: LanguageCode = LanguageCode {
    name: "Southern Tiwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tix",
    individual_languages: &[
    ],
};


pub const TIY: LanguageCode = LanguageCode {
    name: "Tiruray",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tiy",
    individual_languages: &[
    ],
};


pub const TIZ: LanguageCode = LanguageCode {
    name: "Tai Hongjin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tiz",
    individual_languages: &[
    ],
};


pub const TJA: LanguageCode = LanguageCode {
    name: "Tajuasohn",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tja",
    individual_languages: &[
    ],
};


pub const TJG: LanguageCode = LanguageCode {
    name: "Tunjung",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tjg",
    individual_languages: &[
    ],
};


pub const TJI: LanguageCode = LanguageCode {
    name: "Northern Tujia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tji",
    individual_languages: &[
    ],
};


pub const TJJ: LanguageCode = LanguageCode {
    name: "Tjungundji",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tjj",
    individual_languages: &[
    ],
};


pub const TJL: LanguageCode = LanguageCode {
    name: "Tai Laing",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tjl",
    individual_languages: &[
    ],
};


pub const TJM: LanguageCode = LanguageCode {
    name: "Timucua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tjm",
    individual_languages: &[
    ],
};


pub const TJN: LanguageCode = LanguageCode {
    name: "Tonjon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tjn",
    individual_languages: &[
    ],
};


pub const TJO: LanguageCode = LanguageCode {
    name: "Temacine Tamazight",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tjo",
    individual_languages: &[
    ],
};


pub const TJP: LanguageCode = LanguageCode {
    name: "Tjupany",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tjp",
    individual_languages: &[
    ],
};


pub const TJS: LanguageCode = LanguageCode {
    name: "Southern Tujia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tjs",
    individual_languages: &[
    ],
};


pub const TJU: LanguageCode = LanguageCode {
    name: "Tjurruru",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tju",
    individual_languages: &[
    ],
};


pub const TJW: LanguageCode = LanguageCode {
    name: "Djabwurrung",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tjw",
    individual_languages: &[
    ],
};


pub const TKA: LanguageCode = LanguageCode {
    name: "Truká",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tka",
    individual_languages: &[
    ],
};


pub const TKB: LanguageCode = LanguageCode {
    name: "Buksa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tkb",
    individual_languages: &[
    ],
};


pub const TKD: LanguageCode = LanguageCode {
    name: "Tukudede",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tkd",
    individual_languages: &[
    ],
};


pub const TKE: LanguageCode = LanguageCode {
    name: "Takwane",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tke",
    individual_languages: &[
    ],
};


pub const TKF: LanguageCode = LanguageCode {
    name: "Tukumanféd",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tkf",
    individual_languages: &[
    ],
};


pub const TKG: LanguageCode = LanguageCode {
    name: "Tesaka Malagasy",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tkg",
    individual_languages: &[
    ],
};


pub const TKM: LanguageCode = LanguageCode {
    name: "Takelma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tkm",
    individual_languages: &[
    ],
};


pub const TKN: LanguageCode = LanguageCode {
    name: "Toku-No-Shima",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tkn",
    individual_languages: &[
    ],
};


pub const TKP: LanguageCode = LanguageCode {
    name: "Tikopia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tkp",
    individual_languages: &[
    ],
};


pub const TKQ: LanguageCode = LanguageCode {
    name: "Tee",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tkq",
    individual_languages: &[
    ],
};


pub const TKR: LanguageCode = LanguageCode {
    name: "Tsakhur",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tkr",
    individual_languages: &[
    ],
};


pub const TKS: LanguageCode = LanguageCode {
    name: "Takestani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tks",
    individual_languages: &[
    ],
};


pub const TKT: LanguageCode = LanguageCode {
    name: "Kathoriya Tharu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tkt",
    individual_languages: &[
    ],
};


pub const TKU: LanguageCode = LanguageCode {
    name: "Upper Necaxa Totonac",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tku",
    individual_languages: &[
    ],
};


pub const TKV: LanguageCode = LanguageCode {
    name: "Mur Pano",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tkv",
    individual_languages: &[
    ],
};


pub const TKW: LanguageCode = LanguageCode {
    name: "Teanu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tkw",
    individual_languages: &[
    ],
};


pub const TKX: LanguageCode = LanguageCode {
    name: "Tangko",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tkx",
    individual_languages: &[
    ],
};


pub const TKZ: LanguageCode = LanguageCode {
    name: "Takua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tkz",
    individual_languages: &[
    ],
};


pub const TLA: LanguageCode = LanguageCode {
    name: "Southwestern Tepehuan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tla",
    individual_languages: &[
    ],
};


pub const TLB: LanguageCode = LanguageCode {
    name: "Tobelo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tlb",
    individual_languages: &[
    ],
};


pub const TLC: LanguageCode = LanguageCode {
    name: "Yecuatla Totonac",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tlc",
    individual_languages: &[
    ],
};


pub const TLD: LanguageCode = LanguageCode {
    name: "Talaud",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tld",
    individual_languages: &[
    ],
};


pub const TLF: LanguageCode = LanguageCode {
    name: "Telefol",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tlf",
    individual_languages: &[
    ],
};


pub const TLG: LanguageCode = LanguageCode {
    name: "Tofanma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tlg",
    individual_languages: &[
    ],
};


pub const TLJ: LanguageCode = LanguageCode {
    name: "Talinga-Bwisi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tlj",
    individual_languages: &[
    ],
};


pub const TLK: LanguageCode = LanguageCode {
    name: "Taloki",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tlk",
    individual_languages: &[
    ],
};


pub const TLL: LanguageCode = LanguageCode {
    name: "Tetela",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tll",
    individual_languages: &[
    ],
};


pub const TLM: LanguageCode = LanguageCode {
    name: "Tolomako",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tlm",
    individual_languages: &[
    ],
};


pub const TLN: LanguageCode = LanguageCode {
    name: "Talondo'",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tln",
    individual_languages: &[
    ],
};


pub const TLO: LanguageCode = LanguageCode {
    name: "Talodi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tlo",
    individual_languages: &[
    ],
};


pub const TLP: LanguageCode = LanguageCode {
    name: "Filomena Mata-Coahuitlán Totonac",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tlp",
    individual_languages: &[
    ],
};


pub const TLQ: LanguageCode = LanguageCode {
    name: "Tai Loi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tlq",
    individual_languages: &[
    ],
};


pub const TLR: LanguageCode = LanguageCode {
    name: "Talise",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tlr",
    individual_languages: &[
    ],
};


pub const TLS: LanguageCode = LanguageCode {
    name: "Tambotalo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tls",
    individual_languages: &[
    ],
};


pub const TLT: LanguageCode = LanguageCode {
    name: "Sou Nama",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tlt",
    individual_languages: &[
    ],
};


pub const TLU: LanguageCode = LanguageCode {
    name: "Tulehu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tlu",
    individual_languages: &[
    ],
};


pub const TLV: LanguageCode = LanguageCode {
    name: "Taliabu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tlv",
    individual_languages: &[
    ],
};


pub const TLX: LanguageCode = LanguageCode {
    name: "Khehek",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tlx",
    individual_languages: &[
    ],
};


pub const TLY: LanguageCode = LanguageCode {
    name: "Talysh",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tly",
    individual_languages: &[
    ],
};


pub const TMA: LanguageCode = LanguageCode {
    name: "Tama (Chad)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tma",
    individual_languages: &[
    ],
};


pub const TMB: LanguageCode = LanguageCode {
    name: "Katbol",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tmb",
    individual_languages: &[
    ],
};


pub const TMC: LanguageCode = LanguageCode {
    name: "Tumak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tmc",
    individual_languages: &[
    ],
};


pub const TMD: LanguageCode = LanguageCode {
    name: "Haruai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tmd",
    individual_languages: &[
    ],
};


pub const TME: LanguageCode = LanguageCode {
    name: "Tremembé",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tme",
    individual_languages: &[
    ],
};


pub const TMF: LanguageCode = LanguageCode {
    name: "Toba-Maskoy",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tmf",
    individual_languages: &[
    ],
};


pub const TMG: LanguageCode = LanguageCode {
    name: "Ternateño",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tmg",
    individual_languages: &[
    ],
};


pub const TMI: LanguageCode = LanguageCode {
    name: "Tutuba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tmi",
    individual_languages: &[
    ],
};


pub const TMJ: LanguageCode = LanguageCode {
    name: "Samarokena",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tmj",
    individual_languages: &[
    ],
};


pub const TMK: LanguageCode = LanguageCode {
    name: "Northwestern Tamang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tmk",
    individual_languages: &[
    ],
};


pub const TML: LanguageCode = LanguageCode {
    name: "Tamnim Citak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tml",
    individual_languages: &[
    ],
};


pub const TMM: LanguageCode = LanguageCode {
    name: "Tai Thanh",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tmm",
    individual_languages: &[
    ],
};


pub const TMN: LanguageCode = LanguageCode {
    name: "Taman (Indonesia)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tmn",
    individual_languages: &[
    ],
};


pub const TMO: LanguageCode = LanguageCode {
    name: "Temoq",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tmo",
    individual_languages: &[
    ],
};


pub const TMQ: LanguageCode = LanguageCode {
    name: "Tumleo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tmq",
    individual_languages: &[
    ],
};


pub const TMR: LanguageCode = LanguageCode {
    name: "Jewish Babylonian Aramaic (ca. 200-1200 CE)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tmr",
    individual_languages: &[
    ],
};


pub const TMS: LanguageCode = LanguageCode {
    name: "Tima",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tms",
    individual_languages: &[
    ],
};


pub const TMT: LanguageCode = LanguageCode {
    name: "Tasmate",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tmt",
    individual_languages: &[
    ],
};


pub const TMU: LanguageCode = LanguageCode {
    name: "Iau",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tmu",
    individual_languages: &[
    ],
};


pub const TMV: LanguageCode = LanguageCode {
    name: "Tembo (Motembo)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tmv",
    individual_languages: &[
    ],
};


pub const TMW: LanguageCode = LanguageCode {
    name: "Temuan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tmw",
    individual_languages: &[
    ],
};


pub const TMY: LanguageCode = LanguageCode {
    name: "Tami",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tmy",
    individual_languages: &[
    ],
};


pub const TMZ: LanguageCode = LanguageCode {
    name: "Tamanaku",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tmz",
    individual_languages: &[
    ],
};


pub const TNA: LanguageCode = LanguageCode {
    name: "Tacana",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tna",
    individual_languages: &[
    ],
};


pub const TNB: LanguageCode = LanguageCode {
    name: "Western Tunebo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tnb",
    individual_languages: &[
    ],
};


pub const TNC: LanguageCode = LanguageCode {
    name: "Tanimuca-Retuarã",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tnc",
    individual_languages: &[
    ],
};


pub const TND: LanguageCode = LanguageCode {
    name: "Angosturas Tunebo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tnd",
    individual_languages: &[
    ],
};


pub const TNG: LanguageCode = LanguageCode {
    name: "Tobanga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tng",
    individual_languages: &[
    ],
};


pub const TNH: LanguageCode = LanguageCode {
    name: "Maiani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tnh",
    individual_languages: &[
    ],
};


pub const TNI: LanguageCode = LanguageCode {
    name: "Tandia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tni",
    individual_languages: &[
    ],
};


pub const TNK: LanguageCode = LanguageCode {
    name: "Kwamera",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tnk",
    individual_languages: &[
    ],
};


pub const TNL: LanguageCode = LanguageCode {
    name: "Lenakel",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tnl",
    individual_languages: &[
    ],
};


pub const TNM: LanguageCode = LanguageCode {
    name: "Tabla",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tnm",
    individual_languages: &[
    ],
};


pub const TNN: LanguageCode = LanguageCode {
    name: "North Tanna",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tnn",
    individual_languages: &[
    ],
};


pub const TNO: LanguageCode = LanguageCode {
    name: "Toromono",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tno",
    individual_languages: &[
    ],
};


pub const TNP: LanguageCode = LanguageCode {
    name: "Whitesands",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tnp",
    individual_languages: &[
    ],
};


pub const TNQ: LanguageCode = LanguageCode {
    name: "Taino",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tnq",
    individual_languages: &[
    ],
};


pub const TNR: LanguageCode = LanguageCode {
    name: "Ménik",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tnr",
    individual_languages: &[
    ],
};


pub const TNS: LanguageCode = LanguageCode {
    name: "Tenis",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tns",
    individual_languages: &[
    ],
};


pub const TNT: LanguageCode = LanguageCode {
    name: "Tontemboan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tnt",
    individual_languages: &[
    ],
};


pub const TNU: LanguageCode = LanguageCode {
    name: "Tay Khang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tnu",
    individual_languages: &[
    ],
};


pub const TNV: LanguageCode = LanguageCode {
    name: "Tangchangya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tnv",
    individual_languages: &[
    ],
};


pub const TNW: LanguageCode = LanguageCode {
    name: "Tonsawang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tnw",
    individual_languages: &[
    ],
};


pub const TNX: LanguageCode = LanguageCode {
    name: "Tanema",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tnx",
    individual_languages: &[
    ],
};


pub const TNY: LanguageCode = LanguageCode {
    name: "Tongwe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tny",
    individual_languages: &[
    ],
};


pub const TNZ: LanguageCode = LanguageCode {
    name: "Ten'edn",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tnz",
    individual_languages: &[
    ],
};


pub const TOB: LanguageCode = LanguageCode {
    name: "Toba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tob",
    individual_languages: &[
    ],
};


pub const TOC: LanguageCode = LanguageCode {
    name: "Coyutla Totonac",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "toc",
    individual_languages: &[
    ],
};


pub const TOD: LanguageCode = LanguageCode {
    name: "Toma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tod",
    individual_languages: &[
    ],
};


pub const TOF: LanguageCode = LanguageCode {
    name: "Gizrra",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tof",
    individual_languages: &[
    ],
};


pub const TOH: LanguageCode = LanguageCode {
    name: "Gitonga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "toh",
    individual_languages: &[
    ],
};


pub const TOI: LanguageCode = LanguageCode {
    name: "Tonga (Zambia)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "toi",
    individual_languages: &[
    ],
};


pub const TOJ: LanguageCode = LanguageCode {
    name: "Tojolabal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "toj",
    individual_languages: &[
    ],
};


pub const TOK: LanguageCode = LanguageCode {
    name: "Toki Pona",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tok",
    individual_languages: &[
    ],
};


pub const TOL: LanguageCode = LanguageCode {
    name: "Tolowa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tol",
    individual_languages: &[
    ],
};


pub const TOM: LanguageCode = LanguageCode {
    name: "Tombulu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tom",
    individual_languages: &[
    ],
};


pub const TOO: LanguageCode = LanguageCode {
    name: "Xicotepec De Juárez Totonac",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "too",
    individual_languages: &[
    ],
};


pub const TOP: LanguageCode = LanguageCode {
    name: "Papantla Totonac",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "top",
    individual_languages: &[
    ],
};


pub const TOQ: LanguageCode = LanguageCode {
    name: "Toposa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "toq",
    individual_languages: &[
    ],
};


pub const TOR: LanguageCode = LanguageCode {
    name: "Togbo-Vara Banda",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tor",
    individual_languages: &[
    ],
};


pub const TOS: LanguageCode = LanguageCode {
    name: "Highland Totonac",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tos",
    individual_languages: &[
    ],
};


pub const TOU: LanguageCode = LanguageCode {
    name: "Tho",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tou",
    individual_languages: &[
    ],
};


pub const TOV: LanguageCode = LanguageCode {
    name: "Upper Taromi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tov",
    individual_languages: &[
    ],
};


pub const TOW: LanguageCode = LanguageCode {
    name: "Jemez",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tow",
    individual_languages: &[
    ],
};


pub const TOX: LanguageCode = LanguageCode {
    name: "Tobian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tox",
    individual_languages: &[
    ],
};


pub const TOY: LanguageCode = LanguageCode {
    name: "Topoiyo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "toy",
    individual_languages: &[
    ],
};


pub const TOZ: LanguageCode = LanguageCode {
    name: "To",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "toz",
    individual_languages: &[
    ],
};


pub const TPA: LanguageCode = LanguageCode {
    name: "Taupota",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tpa",
    individual_languages: &[
    ],
};


pub const TPC: LanguageCode = LanguageCode {
    name: "Azoyú Me'phaa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tpc",
    individual_languages: &[
    ],
};


pub const TPE: LanguageCode = LanguageCode {
    name: "Tippera",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tpe",
    individual_languages: &[
    ],
};


pub const TPF: LanguageCode = LanguageCode {
    name: "Tarpia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tpf",
    individual_languages: &[
    ],
};


pub const TPG: LanguageCode = LanguageCode {
    name: "Kula",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tpg",
    individual_languages: &[
    ],
};


pub const TPJ: LanguageCode = LanguageCode {
    name: "Tapieté",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tpj",
    individual_languages: &[
    ],
};


pub const TPK: LanguageCode = LanguageCode {
    name: "Tupinikin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tpk",
    individual_languages: &[
    ],
};


pub const TPL: LanguageCode = LanguageCode {
    name: "Tlacoapa Me'phaa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tpl",
    individual_languages: &[
    ],
};


pub const TPM: LanguageCode = LanguageCode {
    name: "Tampulma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tpm",
    individual_languages: &[
    ],
};


pub const TPN: LanguageCode = LanguageCode {
    name: "Tupinambá",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tpn",
    individual_languages: &[
    ],
};


pub const TPO: LanguageCode = LanguageCode {
    name: "Tai Pao",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tpo",
    individual_languages: &[
    ],
};


pub const TPP: LanguageCode = LanguageCode {
    name: "Pisaflores Tepehua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tpp",
    individual_languages: &[
    ],
};


pub const TPQ: LanguageCode = LanguageCode {
    name: "Tukpa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tpq",
    individual_languages: &[
    ],
};


pub const TPR: LanguageCode = LanguageCode {
    name: "Tuparí",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tpr",
    individual_languages: &[
    ],
};


pub const TPT: LanguageCode = LanguageCode {
    name: "Tlachichilco Tepehua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tpt",
    individual_languages: &[
    ],
};


pub const TPU: LanguageCode = LanguageCode {
    name: "Tampuan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tpu",
    individual_languages: &[
    ],
};


pub const TPV: LanguageCode = LanguageCode {
    name: "Tanapag",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tpv",
    individual_languages: &[
    ],
};


pub const TPW: LanguageCode = LanguageCode {
    name: "Tupí",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tpw",
    individual_languages: &[
    ],
};


pub const TPX: LanguageCode = LanguageCode {
    name: "Acatepec Me'phaa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tpx",
    individual_languages: &[
    ],
};


pub const TPY: LanguageCode = LanguageCode {
    name: "Trumai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tpy",
    individual_languages: &[
    ],
};


pub const TPZ: LanguageCode = LanguageCode {
    name: "Tinputz",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tpz",
    individual_languages: &[
    ],
};


pub const TQB: LanguageCode = LanguageCode {
    name: "Tembé",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tqb",
    individual_languages: &[
    ],
};


pub const TQL: LanguageCode = LanguageCode {
    name: "Lehali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tql",
    individual_languages: &[
    ],
};


pub const TQM: LanguageCode = LanguageCode {
    name: "Turumsa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tqm",
    individual_languages: &[
    ],
};


pub const TQN: LanguageCode = LanguageCode {
    name: "Tenino",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tqn",
    individual_languages: &[
    ],
};


pub const TQO: LanguageCode = LanguageCode {
    name: "Toaripi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tqo",
    individual_languages: &[
    ],
};


pub const TQP: LanguageCode = LanguageCode {
    name: "Tomoip",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tqp",
    individual_languages: &[
    ],
};


pub const TQQ: LanguageCode = LanguageCode {
    name: "Tunni",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tqq",
    individual_languages: &[
    ],
};


pub const TQR: LanguageCode = LanguageCode {
    name: "Torona",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tqr",
    individual_languages: &[
    ],
};


pub const TQT: LanguageCode = LanguageCode {
    name: "Western Totonac",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tqt",
    individual_languages: &[
    ],
};


pub const TQU: LanguageCode = LanguageCode {
    name: "Touo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tqu",
    individual_languages: &[
    ],
};


pub const TQW: LanguageCode = LanguageCode {
    name: "Tonkawa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tqw",
    individual_languages: &[
    ],
};


pub const TRA: LanguageCode = LanguageCode {
    name: "Tirahi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tra",
    individual_languages: &[
    ],
};


pub const TRB: LanguageCode = LanguageCode {
    name: "Terebu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "trb",
    individual_languages: &[
    ],
};


pub const TRC: LanguageCode = LanguageCode {
    name: "Copala Triqui",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "trc",
    individual_languages: &[
    ],
};


pub const TRD: LanguageCode = LanguageCode {
    name: "Turi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "trd",
    individual_languages: &[
    ],
};


pub const TRE: LanguageCode = LanguageCode {
    name: "East Tarangan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tre",
    individual_languages: &[
    ],
};


pub const TRF: LanguageCode = LanguageCode {
    name: "Trinidadian Creole English",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "trf",
    individual_languages: &[
    ],
};


pub const TRG: LanguageCode = LanguageCode {
    name: "Lishán Didán",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "trg",
    individual_languages: &[
    ],
};


pub const TRH: LanguageCode = LanguageCode {
    name: "Turaka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "trh",
    individual_languages: &[
    ],
};


pub const TRI: LanguageCode = LanguageCode {
    name: "Trió",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tri",
    individual_languages: &[
    ],
};


pub const TRJ: LanguageCode = LanguageCode {
    name: "Toram",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "trj",
    individual_languages: &[
    ],
};


pub const TRL: LanguageCode = LanguageCode {
    name: "Traveller Scottish",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "trl",
    individual_languages: &[
    ],
};


pub const TRM: LanguageCode = LanguageCode {
    name: "Tregami",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "trm",
    individual_languages: &[
    ],
};


pub const TRN: LanguageCode = LanguageCode {
    name: "Trinitario",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "trn",
    individual_languages: &[
    ],
};


pub const TRO: LanguageCode = LanguageCode {
    name: "Tarao Naga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tro",
    individual_languages: &[
    ],
};


pub const TRP: LanguageCode = LanguageCode {
    name: "Kok Borok",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "trp",
    individual_languages: &[
    ],
};


pub const TRQ: LanguageCode = LanguageCode {
    name: "San Martín Itunyoso Triqui",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "trq",
    individual_languages: &[
    ],
};


pub const TRR: LanguageCode = LanguageCode {
    name: "Taushiro",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "trr",
    individual_languages: &[
    ],
};


pub const TRS: LanguageCode = LanguageCode {
    name: "Chicahuaxtla Triqui",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "trs",
    individual_languages: &[
    ],
};


pub const TRT: LanguageCode = LanguageCode {
    name: "Tunggare",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "trt",
    individual_languages: &[
    ],
};


pub const TRU: LanguageCode = LanguageCode {
    name: "Turoyo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tru",
    individual_languages: &[
    ],
};


pub const TRV: LanguageCode = LanguageCode {
    name: "Sediq",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "trv",
    individual_languages: &[
    ],
};


pub const TRW: LanguageCode = LanguageCode {
    name: "Torwali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "trw",
    individual_languages: &[
    ],
};


pub const TRX: LanguageCode = LanguageCode {
    name: "Tringgus-Sembaan Bidayuh",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "trx",
    individual_languages: &[
    ],
};


pub const TRY: LanguageCode = LanguageCode {
    name: "Turung",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "try",
    individual_languages: &[
    ],
};


pub const TRZ: LanguageCode = LanguageCode {
    name: "Torá",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "trz",
    individual_languages: &[
    ],
};


pub const TSA: LanguageCode = LanguageCode {
    name: "Tsaangi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tsa",
    individual_languages: &[
    ],
};


pub const TSB: LanguageCode = LanguageCode {
    name: "Tsamai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tsb",
    individual_languages: &[
    ],
};


pub const TSC: LanguageCode = LanguageCode {
    name: "Tswa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tsc",
    individual_languages: &[
    ],
};


pub const TSD: LanguageCode = LanguageCode {
    name: "Tsakonian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tsd",
    individual_languages: &[
    ],
};


pub const TSE: LanguageCode = LanguageCode {
    name: "Tunisian Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tse",
    individual_languages: &[
    ],
};


pub const TSG: LanguageCode = LanguageCode {
    name: "Tausug",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tsg",
    individual_languages: &[
    ],
};


pub const TSH: LanguageCode = LanguageCode {
    name: "Tsuvan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tsh",
    individual_languages: &[
    ],
};


pub const TSJ: LanguageCode = LanguageCode {
    name: "Tshangla",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tsj",
    individual_languages: &[
    ],
};


pub const TSK: LanguageCode = LanguageCode {
    name: "Tseku",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tsk",
    individual_languages: &[
    ],
};


pub const TSL: LanguageCode = LanguageCode {
    name: "Ts'ün-Lao",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tsl",
    individual_languages: &[
    ],
};


pub const TSM: LanguageCode = LanguageCode {
    name: "Turkish Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tsm",
    individual_languages: &[
    ],
};


pub const TSP: LanguageCode = LanguageCode {
    name: "Northern Toussian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tsp",
    individual_languages: &[
    ],
};


pub const TSQ: LanguageCode = LanguageCode {
    name: "Thai Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tsq",
    individual_languages: &[
    ],
};


pub const TSR: LanguageCode = LanguageCode {
    name: "Akei",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tsr",
    individual_languages: &[
    ],
};


pub const TSS: LanguageCode = LanguageCode {
    name: "Taiwan Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tss",
    individual_languages: &[
    ],
};


pub const TST: LanguageCode = LanguageCode {
    name: "Tondi Songway Kiini",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tst",
    individual_languages: &[
    ],
};


pub const TSU: LanguageCode = LanguageCode {
    name: "Tsou",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tsu",
    individual_languages: &[
    ],
};


pub const TSV: LanguageCode = LanguageCode {
    name: "Tsogo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tsv",
    individual_languages: &[
    ],
};


pub const TSW: LanguageCode = LanguageCode {
    name: "Tsishingini",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tsw",
    individual_languages: &[
    ],
};


pub const TSX: LanguageCode = LanguageCode {
    name: "Mubami",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tsx",
    individual_languages: &[
    ],
};


pub const TSY: LanguageCode = LanguageCode {
    name: "Tebul Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tsy",
    individual_languages: &[
    ],
};


pub const TSZ: LanguageCode = LanguageCode {
    name: "Purepecha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tsz",
    individual_languages: &[
    ],
};


pub const TTA: LanguageCode = LanguageCode {
    name: "Tutelo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tta",
    individual_languages: &[
    ],
};


pub const TTB: LanguageCode = LanguageCode {
    name: "Gaa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ttb",
    individual_languages: &[
    ],
};


pub const TTC: LanguageCode = LanguageCode {
    name: "Tektiteko",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ttc",
    individual_languages: &[
    ],
};


pub const TTD: LanguageCode = LanguageCode {
    name: "Tauade",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ttd",
    individual_languages: &[
    ],
};


pub const TTE: LanguageCode = LanguageCode {
    name: "Bwanabwana",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tte",
    individual_languages: &[
    ],
};


pub const TTF: LanguageCode = LanguageCode {
    name: "Tuotomb",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ttf",
    individual_languages: &[
    ],
};


pub const TTG: LanguageCode = LanguageCode {
    name: "Tutong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ttg",
    individual_languages: &[
    ],
};


pub const TTH: LanguageCode = LanguageCode {
    name: "Upper Ta'oih",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tth",
    individual_languages: &[
    ],
};


pub const TTI: LanguageCode = LanguageCode {
    name: "Tobati",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tti",
    individual_languages: &[
    ],
};


pub const TTJ: LanguageCode = LanguageCode {
    name: "Tooro",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ttj",
    individual_languages: &[
    ],
};


pub const TTK: LanguageCode = LanguageCode {
    name: "Totoro",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ttk",
    individual_languages: &[
    ],
};


pub const TTL: LanguageCode = LanguageCode {
    name: "Totela",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ttl",
    individual_languages: &[
    ],
};


pub const TTM: LanguageCode = LanguageCode {
    name: "Northern Tutchone",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ttm",
    individual_languages: &[
    ],
};


pub const TTN: LanguageCode = LanguageCode {
    name: "Towei",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ttn",
    individual_languages: &[
    ],
};


pub const TTO: LanguageCode = LanguageCode {
    name: "Lower Ta'oih",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tto",
    individual_languages: &[
    ],
};


pub const TTP: LanguageCode = LanguageCode {
    name: "Tombelala",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ttp",
    individual_languages: &[
    ],
};


pub const TTQ: LanguageCode = LanguageCode {
    name: "Tawallammat Tamajaq",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ttq",
    individual_languages: &[
    ],
};


pub const TTR: LanguageCode = LanguageCode {
    name: "Tera",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ttr",
    individual_languages: &[
    ],
};


pub const TTS: LanguageCode = LanguageCode {
    name: "Northeastern Thai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tts",
    individual_languages: &[
    ],
};


pub const TTT: LanguageCode = LanguageCode {
    name: "Muslim Tat",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ttt",
    individual_languages: &[
    ],
};


pub const TTU: LanguageCode = LanguageCode {
    name: "Torau",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ttu",
    individual_languages: &[
    ],
};


pub const TTV: LanguageCode = LanguageCode {
    name: "Titan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ttv",
    individual_languages: &[
    ],
};


pub const TTW: LanguageCode = LanguageCode {
    name: "Long Wat",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ttw",
    individual_languages: &[
    ],
};


pub const TTY: LanguageCode = LanguageCode {
    name: "Sikaritai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tty",
    individual_languages: &[
    ],
};


pub const TTZ: LanguageCode = LanguageCode {
    name: "Tsum",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ttz",
    individual_languages: &[
    ],
};


pub const TUA: LanguageCode = LanguageCode {
    name: "Wiarumus",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tua",
    individual_languages: &[
    ],
};


pub const TUB: LanguageCode = LanguageCode {
    name: "Tübatulabal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tub",
    individual_languages: &[
    ],
};


pub const TUC: LanguageCode = LanguageCode {
    name: "Mutu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tuc",
    individual_languages: &[
    ],
};


pub const TUD: LanguageCode = LanguageCode {
    name: "Tuxá",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tud",
    individual_languages: &[
    ],
};


pub const TUE: LanguageCode = LanguageCode {
    name: "Tuyuca",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tue",
    individual_languages: &[
    ],
};


pub const TUF: LanguageCode = LanguageCode {
    name: "Central Tunebo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tuf",
    individual_languages: &[
    ],
};


pub const TUG: LanguageCode = LanguageCode {
    name: "Tunia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tug",
    individual_languages: &[
    ],
};


pub const TUH: LanguageCode = LanguageCode {
    name: "Taulil",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tuh",
    individual_languages: &[
    ],
};


pub const TUI: LanguageCode = LanguageCode {
    name: "Tupuri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tui",
    individual_languages: &[
    ],
};


pub const TUJ: LanguageCode = LanguageCode {
    name: "Tugutil",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tuj",
    individual_languages: &[
    ],
};


pub const TUL: LanguageCode = LanguageCode {
    name: "Tula",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tul",
    individual_languages: &[
    ],
};


pub const TUN: LanguageCode = LanguageCode {
    name: "Tunica",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tun",
    individual_languages: &[
    ],
};


pub const TUO: LanguageCode = LanguageCode {
    name: "Tucano",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tuo",
    individual_languages: &[
    ],
};


pub const TUQ: LanguageCode = LanguageCode {
    name: "Tedaga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tuq",
    individual_languages: &[
    ],
};


pub const TUS: LanguageCode = LanguageCode {
    name: "Tuscarora",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tus",
    individual_languages: &[
    ],
};


pub const TUU: LanguageCode = LanguageCode {
    name: "Tututni",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tuu",
    individual_languages: &[
    ],
};


pub const TUV: LanguageCode = LanguageCode {
    name: "Turkana",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tuv",
    individual_languages: &[
    ],
};


pub const TUX: LanguageCode = LanguageCode {
    name: "Tuxináwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tux",
    individual_languages: &[
    ],
};


pub const TUY: LanguageCode = LanguageCode {
    name: "Tugen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tuy",
    individual_languages: &[
    ],
};


pub const TUZ: LanguageCode = LanguageCode {
    name: "Turka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tuz",
    individual_languages: &[
    ],
};


pub const TVA: LanguageCode = LanguageCode {
    name: "Vaghua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tva",
    individual_languages: &[
    ],
};


pub const TVD: LanguageCode = LanguageCode {
    name: "Tsuvadi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tvd",
    individual_languages: &[
    ],
};


pub const TVE: LanguageCode = LanguageCode {
    name: "Te'un",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tve",
    individual_languages: &[
    ],
};


pub const TVK: LanguageCode = LanguageCode {
    name: "Southeast Ambrym",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tvk",
    individual_languages: &[
    ],
};


pub const TVM: LanguageCode = LanguageCode {
    name: "Tela-Masbuar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tvm",
    individual_languages: &[
    ],
};


pub const TVN: LanguageCode = LanguageCode {
    name: "Tavoyan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tvn",
    individual_languages: &[
    ],
};


pub const TVO: LanguageCode = LanguageCode {
    name: "Tidore",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tvo",
    individual_languages: &[
    ],
};


pub const TVS: LanguageCode = LanguageCode {
    name: "Taveta",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tvs",
    individual_languages: &[
    ],
};


pub const TVT: LanguageCode = LanguageCode {
    name: "Tutsa Naga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tvt",
    individual_languages: &[
    ],
};


pub const TVU: LanguageCode = LanguageCode {
    name: "Tunen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tvu",
    individual_languages: &[
    ],
};


pub const TVW: LanguageCode = LanguageCode {
    name: "Sedoa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tvw",
    individual_languages: &[
    ],
};


pub const TVX: LanguageCode = LanguageCode {
    name: "Taivoan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tvx",
    individual_languages: &[
    ],
};


pub const TVY: LanguageCode = LanguageCode {
    name: "Timor Pidgin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tvy",
    individual_languages: &[
    ],
};


pub const TWA: LanguageCode = LanguageCode {
    name: "Twana",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "twa",
    individual_languages: &[
    ],
};


pub const TWB: LanguageCode = LanguageCode {
    name: "Western Tawbuid",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "twb",
    individual_languages: &[
    ],
};


pub const TWC: LanguageCode = LanguageCode {
    name: "Teshenawa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "twc",
    individual_languages: &[
    ],
};


pub const TWD: LanguageCode = LanguageCode {
    name: "Twents",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "twd",
    individual_languages: &[
    ],
};


pub const TWE: LanguageCode = LanguageCode {
    name: "Tewa (Indonesia)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "twe",
    individual_languages: &[
    ],
};


pub const TWF: LanguageCode = LanguageCode {
    name: "Northern Tiwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "twf",
    individual_languages: &[
    ],
};


pub const TWG: LanguageCode = LanguageCode {
    name: "Tereweng",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "twg",
    individual_languages: &[
    ],
};


pub const TWH: LanguageCode = LanguageCode {
    name: "Tai Dón",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "twh",
    individual_languages: &[
    ],
};


pub const TWL: LanguageCode = LanguageCode {
    name: "Tawara",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "twl",
    individual_languages: &[
    ],
};


pub const TWM: LanguageCode = LanguageCode {
    name: "Tawang Monpa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "twm",
    individual_languages: &[
    ],
};


pub const TWN: LanguageCode = LanguageCode {
    name: "Twendi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "twn",
    individual_languages: &[
    ],
};


pub const TWO: LanguageCode = LanguageCode {
    name: "Tswapong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "two",
    individual_languages: &[
    ],
};


pub const TWP: LanguageCode = LanguageCode {
    name: "Ere",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "twp",
    individual_languages: &[
    ],
};


pub const TWQ: LanguageCode = LanguageCode {
    name: "Tasawaq",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "twq",
    individual_languages: &[
    ],
};


pub const TWR: LanguageCode = LanguageCode {
    name: "Southwestern Tarahumara",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "twr",
    individual_languages: &[
    ],
};


pub const TWT: LanguageCode = LanguageCode {
    name: "Turiwára",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "twt",
    individual_languages: &[
    ],
};


pub const TWU: LanguageCode = LanguageCode {
    name: "Termanu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "twu",
    individual_languages: &[
    ],
};


pub const TWW: LanguageCode = LanguageCode {
    name: "Tuwari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tww",
    individual_languages: &[
    ],
};


pub const TWX: LanguageCode = LanguageCode {
    name: "Tewe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "twx",
    individual_languages: &[
    ],
};


pub const TWY: LanguageCode = LanguageCode {
    name: "Tawoyan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "twy",
    individual_languages: &[
    ],
};


pub const TXA: LanguageCode = LanguageCode {
    name: "Tombonuo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "txa",
    individual_languages: &[
    ],
};


pub const TXB: LanguageCode = LanguageCode {
    name: "Tokharian B",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "txb",
    individual_languages: &[
    ],
};


pub const TXC: LanguageCode = LanguageCode {
    name: "Tsetsaut",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "txc",
    individual_languages: &[
    ],
};


pub const TXE: LanguageCode = LanguageCode {
    name: "Totoli",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "txe",
    individual_languages: &[
    ],
};


pub const TXG: LanguageCode = LanguageCode {
    name: "Tangut",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "txg",
    individual_languages: &[
    ],
};


pub const TXH: LanguageCode = LanguageCode {
    name: "Thracian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "txh",
    individual_languages: &[
    ],
};


pub const TXI: LanguageCode = LanguageCode {
    name: "Ikpeng",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "txi",
    individual_languages: &[
    ],
};


pub const TXJ: LanguageCode = LanguageCode {
    name: "Tarjumo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "txj",
    individual_languages: &[
    ],
};


pub const TXM: LanguageCode = LanguageCode {
    name: "Tomini",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "txm",
    individual_languages: &[
    ],
};


pub const TXN: LanguageCode = LanguageCode {
    name: "West Tarangan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "txn",
    individual_languages: &[
    ],
};


pub const TXO: LanguageCode = LanguageCode {
    name: "Toto",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "txo",
    individual_languages: &[
    ],
};


pub const TXQ: LanguageCode = LanguageCode {
    name: "Tii",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "txq",
    individual_languages: &[
    ],
};


pub const TXR: LanguageCode = LanguageCode {
    name: "Tartessian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "txr",
    individual_languages: &[
    ],
};


pub const TXS: LanguageCode = LanguageCode {
    name: "Tonsea",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "txs",
    individual_languages: &[
    ],
};


pub const TXT: LanguageCode = LanguageCode {
    name: "Citak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "txt",
    individual_languages: &[
    ],
};


pub const TXU: LanguageCode = LanguageCode {
    name: "Kayapó",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "txu",
    individual_languages: &[
    ],
};


pub const TXX: LanguageCode = LanguageCode {
    name: "Tatana",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "txx",
    individual_languages: &[
    ],
};


pub const TXY: LanguageCode = LanguageCode {
    name: "Tanosy Malagasy",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "txy",
    individual_languages: &[
    ],
};


pub const TYA: LanguageCode = LanguageCode {
    name: "Tauya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tya",
    individual_languages: &[
    ],
};


pub const TYE: LanguageCode = LanguageCode {
    name: "Kyanga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tye",
    individual_languages: &[
    ],
};


pub const TYH: LanguageCode = LanguageCode {
    name: "O'du",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tyh",
    individual_languages: &[
    ],
};


pub const TYI: LanguageCode = LanguageCode {
    name: "Teke-Tsaayi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tyi",
    individual_languages: &[
    ],
};


pub const TYJ: LanguageCode = LanguageCode {
    name: "Tai Do",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tyj",
    individual_languages: &[
    ],
};


pub const TYL: LanguageCode = LanguageCode {
    name: "Thu Lao",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tyl",
    individual_languages: &[
    ],
};


pub const TYN: LanguageCode = LanguageCode {
    name: "Kombai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tyn",
    individual_languages: &[
    ],
};


pub const TYP: LanguageCode = LanguageCode {
    name: "Thaypan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "typ",
    individual_languages: &[
    ],
};


pub const TYR: LanguageCode = LanguageCode {
    name: "Tai Daeng",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tyr",
    individual_languages: &[
    ],
};


pub const TYS: LanguageCode = LanguageCode {
    name: "Tày Sa Pa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tys",
    individual_languages: &[
    ],
};


pub const TYT: LanguageCode = LanguageCode {
    name: "Tày Tac",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tyt",
    individual_languages: &[
    ],
};


pub const TYU: LanguageCode = LanguageCode {
    name: "Kua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tyu",
    individual_languages: &[
    ],
};


pub const TYX: LanguageCode = LanguageCode {
    name: "Teke-Tyee",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tyx",
    individual_languages: &[
    ],
};


pub const TYY: LanguageCode = LanguageCode {
    name: "Tiyaa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tyy",
    individual_languages: &[
    ],
};


pub const TYZ: LanguageCode = LanguageCode {
    name: "Tày",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tyz",
    individual_languages: &[
    ],
};


pub const TZA: LanguageCode = LanguageCode {
    name: "Tanzanian Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tza",
    individual_languages: &[
    ],
};


pub const TZH: LanguageCode = LanguageCode {
    name: "Tzeltal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tzh",
    individual_languages: &[
    ],
};


pub const TZJ: LanguageCode = LanguageCode {
    name: "Tz'utujil",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tzj",
    individual_languages: &[
    ],
};


pub const TZL: LanguageCode = LanguageCode {
    name: "Talossan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tzl",
    individual_languages: &[
    ],
};


pub const TZM: LanguageCode = LanguageCode {
    name: "Central Atlas Tamazight",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tzm",
    individual_languages: &[
    ],
};


pub const TZN: LanguageCode = LanguageCode {
    name: "Tugun",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tzn",
    individual_languages: &[
    ],
};


pub const TZO: LanguageCode = LanguageCode {
    name: "Tzotzil",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tzo",
    individual_languages: &[
    ],
};


pub const TZX: LanguageCode = LanguageCode {
    name: "Tabriak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "tzx",
    individual_languages: &[
    ],
};


pub const UAM: LanguageCode = LanguageCode {
    name: "Uamué",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "uam",
    individual_languages: &[
    ],
};


pub const UAN: LanguageCode = LanguageCode {
    name: "Kuan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "uan",
    individual_languages: &[
    ],
};


pub const UAR: LanguageCode = LanguageCode {
    name: "Tairuma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "uar",
    individual_languages: &[
    ],
};


pub const UBA: LanguageCode = LanguageCode {
    name: "Ubang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "uba",
    individual_languages: &[
    ],
};


pub const UBI: LanguageCode = LanguageCode {
    name: "Ubi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ubi",
    individual_languages: &[
    ],
};


pub const UBL: LanguageCode = LanguageCode {
    name: "Buhi'non Bikol",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ubl",
    individual_languages: &[
    ],
};


pub const UBR: LanguageCode = LanguageCode {
    name: "Ubir",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ubr",
    individual_languages: &[
    ],
};


pub const UBU: LanguageCode = LanguageCode {
    name: "Umbu-Ungu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ubu",
    individual_languages: &[
    ],
};


pub const UBY: LanguageCode = LanguageCode {
    name: "Ubykh",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "uby",
    individual_languages: &[
    ],
};


pub const UDA: LanguageCode = LanguageCode {
    name: "Uda",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "uda",
    individual_languages: &[
    ],
};


pub const UDE: LanguageCode = LanguageCode {
    name: "Udihe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ude",
    individual_languages: &[
    ],
};


pub const UDG: LanguageCode = LanguageCode {
    name: "Muduga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "udg",
    individual_languages: &[
    ],
};


pub const UDI: LanguageCode = LanguageCode {
    name: "Udi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "udi",
    individual_languages: &[
    ],
};


pub const UDJ: LanguageCode = LanguageCode {
    name: "Ujir",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "udj",
    individual_languages: &[
    ],
};


pub const UDL: LanguageCode = LanguageCode {
    name: "Wuzlam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "udl",
    individual_languages: &[
    ],
};


pub const UDU: LanguageCode = LanguageCode {
    name: "Uduk",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "udu",
    individual_languages: &[
    ],
};


pub const UES: LanguageCode = LanguageCode {
    name: "Kioko",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ues",
    individual_languages: &[
    ],
};


pub const UFI: LanguageCode = LanguageCode {
    name: "Ufim",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ufi",
    individual_languages: &[
    ],
};


pub const UGB: LanguageCode = LanguageCode {
    name: "Kuku-Ugbanh",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ugb",
    individual_languages: &[
    ],
};


pub const UGE: LanguageCode = LanguageCode {
    name: "Ughele",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "uge",
    individual_languages: &[
    ],
};


pub const UGH: LanguageCode = LanguageCode {
    name: "Kubachi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ugh",
    individual_languages: &[
    ],
};


pub const UGN: LanguageCode = LanguageCode {
    name: "Ugandan Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ugn",
    individual_languages: &[
    ],
};


pub const UGO: LanguageCode = LanguageCode {
    name: "Ugong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ugo",
    individual_languages: &[
    ],
};


pub const UGY: LanguageCode = LanguageCode {
    name: "Uruguayan Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ugy",
    individual_languages: &[
    ],
};


pub const UHA: LanguageCode = LanguageCode {
    name: "Uhami",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "uha",
    individual_languages: &[
    ],
};


pub const UHN: LanguageCode = LanguageCode {
    name: "Damal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "uhn",
    individual_languages: &[
    ],
};


pub const UIS: LanguageCode = LanguageCode {
    name: "Uisai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "uis",
    individual_languages: &[
    ],
};


pub const UIV: LanguageCode = LanguageCode {
    name: "Iyive",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "uiv",
    individual_languages: &[
    ],
};


pub const UJI: LanguageCode = LanguageCode {
    name: "Tanjijili",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "uji",
    individual_languages: &[
    ],
};


pub const UKA: LanguageCode = LanguageCode {
    name: "Kaburi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "uka",
    individual_languages: &[
    ],
};


pub const UKG: LanguageCode = LanguageCode {
    name: "Ukuriguma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ukg",
    individual_languages: &[
    ],
};


pub const UKH: LanguageCode = LanguageCode {
    name: "Ukhwejo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ukh",
    individual_languages: &[
    ],
};


pub const UKI: LanguageCode = LanguageCode {
    name: "Kui (India)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "uki",
    individual_languages: &[
    ],
};


pub const UKK: LanguageCode = LanguageCode {
    name: "Muak Sa-aak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ukk",
    individual_languages: &[
    ],
};


pub const UKL: LanguageCode = LanguageCode {
    name: "Ukrainian Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ukl",
    individual_languages: &[
    ],
};


pub const UKP: LanguageCode = LanguageCode {
    name: "Ukpe-Bayobiri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ukp",
    individual_languages: &[
    ],
};


pub const UKQ: LanguageCode = LanguageCode {
    name: "Ukwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ukq",
    individual_languages: &[
    ],
};


pub const UKS: LanguageCode = LanguageCode {
    name: "Urubú-Kaapor Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "uks",
    individual_languages: &[
    ],
};


pub const UKU: LanguageCode = LanguageCode {
    name: "Ukue",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "uku",
    individual_languages: &[
    ],
};


pub const UKV: LanguageCode = LanguageCode {
    name: "Kuku",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ukv",
    individual_languages: &[
    ],
};


pub const UKW: LanguageCode = LanguageCode {
    name: "Ukwuani-Aboh-Ndoni",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ukw",
    individual_languages: &[
    ],
};


pub const UKY: LanguageCode = LanguageCode {
    name: "Kuuk-Yak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "uky",
    individual_languages: &[
    ],
};


pub const ULA: LanguageCode = LanguageCode {
    name: "Fungwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ula",
    individual_languages: &[
    ],
};


pub const ULB: LanguageCode = LanguageCode {
    name: "Ulukwumi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ulb",
    individual_languages: &[
    ],
};


pub const ULC: LanguageCode = LanguageCode {
    name: "Ulch",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ulc",
    individual_languages: &[
    ],
};


pub const ULE: LanguageCode = LanguageCode {
    name: "Lule",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ule",
    individual_languages: &[
    ],
};


pub const ULF: LanguageCode = LanguageCode {
    name: "Usku",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ulf",
    individual_languages: &[
    ],
};


pub const ULI: LanguageCode = LanguageCode {
    name: "Ulithian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "uli",
    individual_languages: &[
    ],
};


pub const ULK: LanguageCode = LanguageCode {
    name: "Meriam Mir",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ulk",
    individual_languages: &[
    ],
};


pub const ULL: LanguageCode = LanguageCode {
    name: "Ullatan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ull",
    individual_languages: &[
    ],
};


pub const ULM: LanguageCode = LanguageCode {
    name: "Ulumanda'",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ulm",
    individual_languages: &[
    ],
};


pub const ULN: LanguageCode = LanguageCode {
    name: "Unserdeutsch",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "uln",
    individual_languages: &[
    ],
};


pub const ULU: LanguageCode = LanguageCode {
    name: "Uma' Lung",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ulu",
    individual_languages: &[
    ],
};


pub const ULW: LanguageCode = LanguageCode {
    name: "Ulwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ulw",
    individual_languages: &[
    ],
};


pub const UMA: LanguageCode = LanguageCode {
    name: "Umatilla",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "uma",
    individual_languages: &[
    ],
};


pub const UMC: LanguageCode = LanguageCode {
    name: "Marrucinian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "umc",
    individual_languages: &[
    ],
};


pub const UMD: LanguageCode = LanguageCode {
    name: "Umbindhamu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "umd",
    individual_languages: &[
    ],
};


pub const UMG: LanguageCode = LanguageCode {
    name: "Morrobalama",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "umg",
    individual_languages: &[
    ],
};


pub const UMI: LanguageCode = LanguageCode {
    name: "Ukit",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "umi",
    individual_languages: &[
    ],
};


pub const UMM: LanguageCode = LanguageCode {
    name: "Umon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "umm",
    individual_languages: &[
    ],
};


pub const UMN: LanguageCode = LanguageCode {
    name: "Makyan Naga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "umn",
    individual_languages: &[
    ],
};


pub const UMO: LanguageCode = LanguageCode {
    name: "Umotína",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "umo",
    individual_languages: &[
    ],
};


pub const UMP: LanguageCode = LanguageCode {
    name: "Umpila",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ump",
    individual_languages: &[
    ],
};


pub const UMR: LanguageCode = LanguageCode {
    name: "Umbugarla",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "umr",
    individual_languages: &[
    ],
};


pub const UMS: LanguageCode = LanguageCode {
    name: "Pendau",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ums",
    individual_languages: &[
    ],
};


pub const UMU: LanguageCode = LanguageCode {
    name: "Munsee",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "umu",
    individual_languages: &[
    ],
};


pub const UNA: LanguageCode = LanguageCode {
    name: "North Watut",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "una",
    individual_languages: &[
    ],
};


pub const UNE: LanguageCode = LanguageCode {
    name: "Uneme",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "une",
    individual_languages: &[
    ],
};


pub const UNG: LanguageCode = LanguageCode {
    name: "Ngarinyin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ung",
    individual_languages: &[
    ],
};


pub const UNI: LanguageCode = LanguageCode {
    name: "Uni",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "uni",
    individual_languages: &[
    ],
};


pub const UNK: LanguageCode = LanguageCode {
    name: "Enawené-Nawé",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "unk",
    individual_languages: &[
    ],
};


pub const UNM: LanguageCode = LanguageCode {
    name: "Unami",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "unm",
    individual_languages: &[
    ],
};


pub const UNN: LanguageCode = LanguageCode {
    name: "Kurnai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "unn",
    individual_languages: &[
    ],
};


pub const UNR: LanguageCode = LanguageCode {
    name: "Mundari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "unr",
    individual_languages: &[
    ],
};


pub const UNU: LanguageCode = LanguageCode {
    name: "Unubahe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "unu",
    individual_languages: &[
    ],
};


pub const UNX: LanguageCode = LanguageCode {
    name: "Munda",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "unx",
    individual_languages: &[
    ],
};


pub const UNZ: LanguageCode = LanguageCode {
    name: "Unde Kaili",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "unz",
    individual_languages: &[
    ],
};


pub const UON: LanguageCode = LanguageCode {
    name: "Kulon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "uon",
    individual_languages: &[
    ],
};


pub const UPI: LanguageCode = LanguageCode {
    name: "Umeda",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "upi",
    individual_languages: &[
    ],
};


pub const UPV: LanguageCode = LanguageCode {
    name: "Uripiv-Wala-Rano-Atchin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "upv",
    individual_languages: &[
    ],
};


pub const URA: LanguageCode = LanguageCode {
    name: "Urarina",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ura",
    individual_languages: &[
    ],
};


pub const URB: LanguageCode = LanguageCode {
    name: "Urubú-Kaapor",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "urb",
    individual_languages: &[
    ],
};


pub const URC: LanguageCode = LanguageCode {
    name: "Urningangg",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "urc",
    individual_languages: &[
    ],
};


pub const URE: LanguageCode = LanguageCode {
    name: "Uru",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ure",
    individual_languages: &[
    ],
};


pub const URF: LanguageCode = LanguageCode {
    name: "Uradhi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "urf",
    individual_languages: &[
    ],
};


pub const URG: LanguageCode = LanguageCode {
    name: "Urigina",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "urg",
    individual_languages: &[
    ],
};


pub const URH: LanguageCode = LanguageCode {
    name: "Urhobo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "urh",
    individual_languages: &[
    ],
};


pub const URI: LanguageCode = LanguageCode {
    name: "Urim",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "uri",
    individual_languages: &[
    ],
};


pub const URK: LanguageCode = LanguageCode {
    name: "Urak Lawoi'",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "urk",
    individual_languages: &[
    ],
};


pub const URL: LanguageCode = LanguageCode {
    name: "Urali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "url",
    individual_languages: &[
    ],
};


pub const URM: LanguageCode = LanguageCode {
    name: "Urapmin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "urm",
    individual_languages: &[
    ],
};


pub const URN: LanguageCode = LanguageCode {
    name: "Uruangnirin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "urn",
    individual_languages: &[
    ],
};


pub const URO: LanguageCode = LanguageCode {
    name: "Ura (Papua New Guinea)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "uro",
    individual_languages: &[
    ],
};


pub const URP: LanguageCode = LanguageCode {
    name: "Uru-Pa-In",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "urp",
    individual_languages: &[
    ],
};


pub const URR: LanguageCode = LanguageCode {
    name: "Lehalurup",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "urr",
    individual_languages: &[
    ],
};


pub const URT: LanguageCode = LanguageCode {
    name: "Urat",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "urt",
    individual_languages: &[
    ],
};


pub const URU: LanguageCode = LanguageCode {
    name: "Urumi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "uru",
    individual_languages: &[
    ],
};


pub const URV: LanguageCode = LanguageCode {
    name: "Uruava",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "urv",
    individual_languages: &[
    ],
};


pub const URW: LanguageCode = LanguageCode {
    name: "Sop",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "urw",
    individual_languages: &[
    ],
};


pub const URX: LanguageCode = LanguageCode {
    name: "Urimo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "urx",
    individual_languages: &[
    ],
};


pub const URY: LanguageCode = LanguageCode {
    name: "Orya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ury",
    individual_languages: &[
    ],
};


pub const URZ: LanguageCode = LanguageCode {
    name: "Uru-Eu-Wau-Wau",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "urz",
    individual_languages: &[
    ],
};


pub const USA: LanguageCode = LanguageCode {
    name: "Usarufa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "usa",
    individual_languages: &[
    ],
};


pub const USH: LanguageCode = LanguageCode {
    name: "Ushojo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ush",
    individual_languages: &[
    ],
};


pub const USI: LanguageCode = LanguageCode {
    name: "Usui",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "usi",
    individual_languages: &[
    ],
};


pub const USK: LanguageCode = LanguageCode {
    name: "Usaghade",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "usk",
    individual_languages: &[
    ],
};


pub const USP: LanguageCode = LanguageCode {
    name: "Uspanteco",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "usp",
    individual_languages: &[
    ],
};


pub const USS: LanguageCode = LanguageCode {
    name: "us-Saare",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "uss",
    individual_languages: &[
    ],
};


pub const USU: LanguageCode = LanguageCode {
    name: "Uya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "usu",
    individual_languages: &[
    ],
};


pub const UTA: LanguageCode = LanguageCode {
    name: "Otank",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "uta",
    individual_languages: &[
    ],
};


pub const UTE: LanguageCode = LanguageCode {
    name: "Ute-Southern Paiute",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ute",
    individual_languages: &[
    ],
};


pub const UTH: LanguageCode = LanguageCode {
    name: "ut-Hun",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "uth",
    individual_languages: &[
    ],
};


pub const UTP: LanguageCode = LanguageCode {
    name: "Amba (Solomon Islands)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "utp",
    individual_languages: &[
    ],
};


pub const UTR: LanguageCode = LanguageCode {
    name: "Etulo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "utr",
    individual_languages: &[
    ],
};


pub const UTU: LanguageCode = LanguageCode {
    name: "Utu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "utu",
    individual_languages: &[
    ],
};


pub const UUM: LanguageCode = LanguageCode {
    name: "Urum",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "uum",
    individual_languages: &[
    ],
};


pub const UUR: LanguageCode = LanguageCode {
    name: "Ura (Vanuatu)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "uur",
    individual_languages: &[
    ],
};


pub const UUU: LanguageCode = LanguageCode {
    name: "U",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "uuu",
    individual_languages: &[
    ],
};


pub const UVE: LanguageCode = LanguageCode {
    name: "West Uvean",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "uve",
    individual_languages: &[
    ],
};


pub const UVH: LanguageCode = LanguageCode {
    name: "Uri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "uvh",
    individual_languages: &[
    ],
};


pub const UVL: LanguageCode = LanguageCode {
    name: "Lote",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "uvl",
    individual_languages: &[
    ],
};


pub const UWA: LanguageCode = LanguageCode {
    name: "Kuku-Uwanh",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "uwa",
    individual_languages: &[
    ],
};


pub const UYA: LanguageCode = LanguageCode {
    name: "Doko-Uyanga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "uya",
    individual_languages: &[
    ],
};


pub const UZN: LanguageCode = LanguageCode {
    name: "Northern Uzbek",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "uzn",
    individual_languages: &[
    ],
};


pub const UZS: LanguageCode = LanguageCode {
    name: "Southern Uzbek",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "uzs",
    individual_languages: &[
    ],
};


pub const VAA: LanguageCode = LanguageCode {
    name: "Vaagri Booli",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vaa",
    individual_languages: &[
    ],
};


pub const VAE: LanguageCode = LanguageCode {
    name: "Vale",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vae",
    individual_languages: &[
    ],
};


pub const VAF: LanguageCode = LanguageCode {
    name: "Vafsi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vaf",
    individual_languages: &[
    ],
};


pub const VAG: LanguageCode = LanguageCode {
    name: "Vagla",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vag",
    individual_languages: &[
    ],
};


pub const VAH: LanguageCode = LanguageCode {
    name: "Varhadi-Nagpuri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vah",
    individual_languages: &[
    ],
};


pub const VAJ: LanguageCode = LanguageCode {
    name: "Sekele",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vaj",
    individual_languages: &[
    ],
};


pub const VAL: LanguageCode = LanguageCode {
    name: "Vehes",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "val",
    individual_languages: &[
    ],
};


pub const VAM: LanguageCode = LanguageCode {
    name: "Vanimo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vam",
    individual_languages: &[
    ],
};


pub const VAN: LanguageCode = LanguageCode {
    name: "Valman",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "van",
    individual_languages: &[
    ],
};


pub const VAO: LanguageCode = LanguageCode {
    name: "Vao",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vao",
    individual_languages: &[
    ],
};


pub const VAP: LanguageCode = LanguageCode {
    name: "Vaiphei",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vap",
    individual_languages: &[
    ],
};


pub const VAR: LanguageCode = LanguageCode {
    name: "Huarijio",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "var",
    individual_languages: &[
    ],
};


pub const VAS: LanguageCode = LanguageCode {
    name: "Vasavi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vas",
    individual_languages: &[
    ],
};


pub const VAU: LanguageCode = LanguageCode {
    name: "Vanuma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vau",
    individual_languages: &[
    ],
};


pub const VAV: LanguageCode = LanguageCode {
    name: "Varli",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vav",
    individual_languages: &[
    ],
};


pub const VAY: LanguageCode = LanguageCode {
    name: "Wayu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vay",
    individual_languages: &[
    ],
};


pub const VBB: LanguageCode = LanguageCode {
    name: "Southeast Babar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vbb",
    individual_languages: &[
    ],
};


pub const VBK: LanguageCode = LanguageCode {
    name: "Southwestern Bontok",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vbk",
    individual_languages: &[
    ],
};


pub const VEC: LanguageCode = LanguageCode {
    name: "Venetian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vec",
    individual_languages: &[
    ],
};


pub const VED: LanguageCode = LanguageCode {
    name: "Veddah",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ved",
    individual_languages: &[
    ],
};


pub const VEL: LanguageCode = LanguageCode {
    name: "Veluws",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vel",
    individual_languages: &[
    ],
};


pub const VEM: LanguageCode = LanguageCode {
    name: "Vemgo-Mabas",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vem",
    individual_languages: &[
    ],
};


pub const VEO: LanguageCode = LanguageCode {
    name: "Ventureño",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "veo",
    individual_languages: &[
    ],
};


pub const VEP: LanguageCode = LanguageCode {
    name: "Veps",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vep",
    individual_languages: &[
    ],
};


pub const VER: LanguageCode = LanguageCode {
    name: "Mom Jango",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ver",
    individual_languages: &[
    ],
};


pub const VGR: LanguageCode = LanguageCode {
    name: "Vaghri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vgr",
    individual_languages: &[
    ],
};


pub const VGT: LanguageCode = LanguageCode {
    name: "Vlaamse Gebarentaal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vgt",
    individual_languages: &[
    ],
};


pub const VIC: LanguageCode = LanguageCode {
    name: "Virgin Islands Creole English",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vic",
    individual_languages: &[
    ],
};


pub const VID: LanguageCode = LanguageCode {
    name: "Vidunda",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vid",
    individual_languages: &[
    ],
};


pub const VIF: LanguageCode = LanguageCode {
    name: "Vili",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vif",
    individual_languages: &[
    ],
};


pub const VIG: LanguageCode = LanguageCode {
    name: "Viemo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vig",
    individual_languages: &[
    ],
};


pub const VIL: LanguageCode = LanguageCode {
    name: "Vilela",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vil",
    individual_languages: &[
    ],
};


pub const VIN: LanguageCode = LanguageCode {
    name: "Vinza",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vin",
    individual_languages: &[
    ],
};


pub const VIS: LanguageCode = LanguageCode {
    name: "Vishavan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vis",
    individual_languages: &[
    ],
};


pub const VIT: LanguageCode = LanguageCode {
    name: "Viti",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vit",
    individual_languages: &[
    ],
};


pub const VIV: LanguageCode = LanguageCode {
    name: "Iduna",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "viv",
    individual_languages: &[
    ],
};


pub const VKA: LanguageCode = LanguageCode {
    name: "Kariyarra",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vka",
    individual_languages: &[
    ],
};


pub const VKJ: LanguageCode = LanguageCode {
    name: "Kujarge",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vkj",
    individual_languages: &[
    ],
};


pub const VKK: LanguageCode = LanguageCode {
    name: "Kaur",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vkk",
    individual_languages: &[
    ],
};


pub const VKL: LanguageCode = LanguageCode {
    name: "Kulisusu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vkl",
    individual_languages: &[
    ],
};


pub const VKM: LanguageCode = LanguageCode {
    name: "Kamakan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vkm",
    individual_languages: &[
    ],
};


pub const VKN: LanguageCode = LanguageCode {
    name: "Koro Nulu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vkn",
    individual_languages: &[
    ],
};


pub const VKO: LanguageCode = LanguageCode {
    name: "Kodeoha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vko",
    individual_languages: &[
    ],
};


pub const VKP: LanguageCode = LanguageCode {
    name: "Korlai Creole Portuguese",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vkp",
    individual_languages: &[
    ],
};


pub const VKT: LanguageCode = LanguageCode {
    name: "Tenggarong Kutai Malay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vkt",
    individual_languages: &[
    ],
};


pub const VKU: LanguageCode = LanguageCode {
    name: "Kurrama",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vku",
    individual_languages: &[
    ],
};


pub const VKZ: LanguageCode = LanguageCode {
    name: "Koro Zuba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vkz",
    individual_languages: &[
    ],
};


pub const VLP: LanguageCode = LanguageCode {
    name: "Valpei",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vlp",
    individual_languages: &[
    ],
};


pub const VLS: LanguageCode = LanguageCode {
    name: "Vlaams",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vls",
    individual_languages: &[
    ],
};


pub const VMA: LanguageCode = LanguageCode {
    name: "Martuyhunira",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vma",
    individual_languages: &[
    ],
};


pub const VMB: LanguageCode = LanguageCode {
    name: "Barbaram",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vmb",
    individual_languages: &[
    ],
};


pub const VMC: LanguageCode = LanguageCode {
    name: "Juxtlahuaca Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vmc",
    individual_languages: &[
    ],
};


pub const VMD: LanguageCode = LanguageCode {
    name: "Mudu Koraga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vmd",
    individual_languages: &[
    ],
};


pub const VME: LanguageCode = LanguageCode {
    name: "East Masela",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vme",
    individual_languages: &[
    ],
};


pub const VMF: LanguageCode = LanguageCode {
    name: "Mainfränkisch",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vmf",
    individual_languages: &[
    ],
};


pub const VMG: LanguageCode = LanguageCode {
    name: "Lungalunga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vmg",
    individual_languages: &[
    ],
};


pub const VMH: LanguageCode = LanguageCode {
    name: "Maraghei",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vmh",
    individual_languages: &[
    ],
};


pub const VMI: LanguageCode = LanguageCode {
    name: "Miwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vmi",
    individual_languages: &[
    ],
};


pub const VMJ: LanguageCode = LanguageCode {
    name: "Ixtayutla Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vmj",
    individual_languages: &[
    ],
};


pub const VMK: LanguageCode = LanguageCode {
    name: "Makhuwa-Shirima",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vmk",
    individual_languages: &[
    ],
};


pub const VML: LanguageCode = LanguageCode {
    name: "Malgana",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vml",
    individual_languages: &[
    ],
};


pub const VMM: LanguageCode = LanguageCode {
    name: "Mitlatongo Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vmm",
    individual_languages: &[
    ],
};


pub const VMP: LanguageCode = LanguageCode {
    name: "Soyaltepec Mazatec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vmp",
    individual_languages: &[
    ],
};


pub const VMQ: LanguageCode = LanguageCode {
    name: "Soyaltepec Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vmq",
    individual_languages: &[
    ],
};


pub const VMR: LanguageCode = LanguageCode {
    name: "Marenje",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vmr",
    individual_languages: &[
    ],
};


pub const VMS: LanguageCode = LanguageCode {
    name: "Moksela",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vms",
    individual_languages: &[
    ],
};


pub const VMU: LanguageCode = LanguageCode {
    name: "Muluridyi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vmu",
    individual_languages: &[
    ],
};


pub const VMV: LanguageCode = LanguageCode {
    name: "Valley Maidu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vmv",
    individual_languages: &[
    ],
};


pub const VMW: LanguageCode = LanguageCode {
    name: "Makhuwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vmw",
    individual_languages: &[
    ],
};


pub const VMX: LanguageCode = LanguageCode {
    name: "Tamazola Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vmx",
    individual_languages: &[
    ],
};


pub const VMY: LanguageCode = LanguageCode {
    name: "Ayautla Mazatec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vmy",
    individual_languages: &[
    ],
};


pub const VMZ: LanguageCode = LanguageCode {
    name: "Mazatlán Mazatec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vmz",
    individual_languages: &[
    ],
};


pub const VNK: LanguageCode = LanguageCode {
    name: "Vano",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vnk",
    individual_languages: &[
    ],
};


pub const VNM: LanguageCode = LanguageCode {
    name: "Vinmavis",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vnm",
    individual_languages: &[
    ],
};


pub const VNP: LanguageCode = LanguageCode {
    name: "Vunapu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vnp",
    individual_languages: &[
    ],
};


pub const VOR: LanguageCode = LanguageCode {
    name: "Voro",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vor",
    individual_languages: &[
    ],
};


pub const VRA: LanguageCode = LanguageCode {
    name: "Vera'a",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vra",
    individual_languages: &[
    ],
};


pub const VRO: LanguageCode = LanguageCode {
    name: "Võro",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vro",
    individual_languages: &[
    ],
};


pub const VRS: LanguageCode = LanguageCode {
    name: "Varisi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vrs",
    individual_languages: &[
    ],
};


pub const VRT: LanguageCode = LanguageCode {
    name: "Burmbar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vrt",
    individual_languages: &[
    ],
};


pub const VSI: LanguageCode = LanguageCode {
    name: "Moldova Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vsi",
    individual_languages: &[
    ],
};


pub const VSL: LanguageCode = LanguageCode {
    name: "Venezuelan Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vsl",
    individual_languages: &[
    ],
};


pub const VSV: LanguageCode = LanguageCode {
    name: "Valencian Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vsv",
    individual_languages: &[
    ],
};


pub const VTO: LanguageCode = LanguageCode {
    name: "Vitou",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vto",
    individual_languages: &[
    ],
};


pub const VUM: LanguageCode = LanguageCode {
    name: "Vumbu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vum",
    individual_languages: &[
    ],
};


pub const VUN: LanguageCode = LanguageCode {
    name: "Vunjo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vun",
    individual_languages: &[
    ],
};


pub const VUT: LanguageCode = LanguageCode {
    name: "Vute",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vut",
    individual_languages: &[
    ],
};


pub const VWA: LanguageCode = LanguageCode {
    name: "Awa (China)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "vwa",
    individual_languages: &[
    ],
};


pub const WAA: LanguageCode = LanguageCode {
    name: "Walla Walla",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "waa",
    individual_languages: &[
    ],
};


pub const WAB: LanguageCode = LanguageCode {
    name: "Wab",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wab",
    individual_languages: &[
    ],
};


pub const WAC: LanguageCode = LanguageCode {
    name: "Wasco-Wishram",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wac",
    individual_languages: &[
    ],
};


pub const WAD: LanguageCode = LanguageCode {
    name: "Wamesa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wad",
    individual_languages: &[
    ],
};


pub const WAE: LanguageCode = LanguageCode {
    name: "Walser",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wae",
    individual_languages: &[
    ],
};


pub const WAF: LanguageCode = LanguageCode {
    name: "Wakoná",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "waf",
    individual_languages: &[
    ],
};


pub const WAG: LanguageCode = LanguageCode {
    name: "Wa'ema",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wag",
    individual_languages: &[
    ],
};


pub const WAH: LanguageCode = LanguageCode {
    name: "Watubela",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wah",
    individual_languages: &[
    ],
};


pub const WAI: LanguageCode = LanguageCode {
    name: "Wares",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wai",
    individual_languages: &[
    ],
};


pub const WAJ: LanguageCode = LanguageCode {
    name: "Waffa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "waj",
    individual_languages: &[
    ],
};


pub const WAM: LanguageCode = LanguageCode {
    name: "Wampanoag",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wam",
    individual_languages: &[
    ],
};


pub const WAN: LanguageCode = LanguageCode {
    name: "Wan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wan",
    individual_languages: &[
    ],
};


pub const WAO: LanguageCode = LanguageCode {
    name: "Wappo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wao",
    individual_languages: &[
    ],
};


pub const WAP: LanguageCode = LanguageCode {
    name: "Wapishana",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wap",
    individual_languages: &[
    ],
};


pub const WAQ: LanguageCode = LanguageCode {
    name: "Wagiman",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "waq",
    individual_languages: &[
    ],
};


pub const WAT: LanguageCode = LanguageCode {
    name: "Kaninuwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wat",
    individual_languages: &[
    ],
};


pub const WAU: LanguageCode = LanguageCode {
    name: "Waurá",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wau",
    individual_languages: &[
    ],
};


pub const WAV: LanguageCode = LanguageCode {
    name: "Waka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wav",
    individual_languages: &[
    ],
};


pub const WAW: LanguageCode = LanguageCode {
    name: "Waiwai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "waw",
    individual_languages: &[
    ],
};


pub const WAX: LanguageCode = LanguageCode {
    name: "Watam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wax",
    individual_languages: &[
    ],
};


pub const WAY: LanguageCode = LanguageCode {
    name: "Wayana",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "way",
    individual_languages: &[
    ],
};


pub const WAZ: LanguageCode = LanguageCode {
    name: "Wampur",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "waz",
    individual_languages: &[
    ],
};


pub const WBA: LanguageCode = LanguageCode {
    name: "Warao",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wba",
    individual_languages: &[
    ],
};


pub const WBB: LanguageCode = LanguageCode {
    name: "Wabo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wbb",
    individual_languages: &[
    ],
};


pub const WBE: LanguageCode = LanguageCode {
    name: "Waritai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wbe",
    individual_languages: &[
    ],
};


pub const WBF: LanguageCode = LanguageCode {
    name: "Wara",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wbf",
    individual_languages: &[
    ],
};


pub const WBH: LanguageCode = LanguageCode {
    name: "Wanda",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wbh",
    individual_languages: &[
    ],
};


pub const WBI: LanguageCode = LanguageCode {
    name: "Vwanji",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wbi",
    individual_languages: &[
    ],
};


pub const WBJ: LanguageCode = LanguageCode {
    name: "Alagwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wbj",
    individual_languages: &[
    ],
};


pub const WBK: LanguageCode = LanguageCode {
    name: "Waigali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wbk",
    individual_languages: &[
    ],
};


pub const WBL: LanguageCode = LanguageCode {
    name: "Wakhi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wbl",
    individual_languages: &[
    ],
};


pub const WBM: LanguageCode = LanguageCode {
    name: "Wa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wbm",
    individual_languages: &[
    ],
};


pub const WBP: LanguageCode = LanguageCode {
    name: "Warlpiri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wbp",
    individual_languages: &[
    ],
};


pub const WBQ: LanguageCode = LanguageCode {
    name: "Waddar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wbq",
    individual_languages: &[
    ],
};


pub const WBR: LanguageCode = LanguageCode {
    name: "Wagdi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wbr",
    individual_languages: &[
    ],
};


pub const WBS: LanguageCode = LanguageCode {
    name: "West Bengal Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wbs",
    individual_languages: &[
    ],
};


pub const WBT: LanguageCode = LanguageCode {
    name: "Warnman",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wbt",
    individual_languages: &[
    ],
};


pub const WBV: LanguageCode = LanguageCode {
    name: "Wajarri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wbv",
    individual_languages: &[
    ],
};


pub const WBW: LanguageCode = LanguageCode {
    name: "Woi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wbw",
    individual_languages: &[
    ],
};


pub const WCA: LanguageCode = LanguageCode {
    name: "Yanomámi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wca",
    individual_languages: &[
    ],
};


pub const WCI: LanguageCode = LanguageCode {
    name: "Waci Gbe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wci",
    individual_languages: &[
    ],
};


pub const WDD: LanguageCode = LanguageCode {
    name: "Wandji",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wdd",
    individual_languages: &[
    ],
};


pub const WDG: LanguageCode = LanguageCode {
    name: "Wadaginam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wdg",
    individual_languages: &[
    ],
};


pub const WDJ: LanguageCode = LanguageCode {
    name: "Wadjiginy",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wdj",
    individual_languages: &[
    ],
};


pub const WDK: LanguageCode = LanguageCode {
    name: "Wadikali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wdk",
    individual_languages: &[
    ],
};


pub const WDT: LanguageCode = LanguageCode {
    name: "Wendat",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wdt",
    individual_languages: &[
    ],
};


pub const WDU: LanguageCode = LanguageCode {
    name: "Wadjigu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wdu",
    individual_languages: &[
    ],
};


pub const WDY: LanguageCode = LanguageCode {
    name: "Wadjabangayi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wdy",
    individual_languages: &[
    ],
};


pub const WEA: LanguageCode = LanguageCode {
    name: "Wewaw",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wea",
    individual_languages: &[
    ],
};


pub const WEC: LanguageCode = LanguageCode {
    name: "Wè Western",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wec",
    individual_languages: &[
    ],
};


pub const WED: LanguageCode = LanguageCode {
    name: "Wedau",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wed",
    individual_languages: &[
    ],
};


pub const WEG: LanguageCode = LanguageCode {
    name: "Wergaia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "weg",
    individual_languages: &[
    ],
};


pub const WEH: LanguageCode = LanguageCode {
    name: "Weh",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "weh",
    individual_languages: &[
    ],
};


pub const WEI: LanguageCode = LanguageCode {
    name: "Kiunum",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wei",
    individual_languages: &[
    ],
};


pub const WEM: LanguageCode = LanguageCode {
    name: "Weme Gbe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wem",
    individual_languages: &[
    ],
};


pub const WEO: LanguageCode = LanguageCode {
    name: "Wemale",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "weo",
    individual_languages: &[
    ],
};


pub const WEP: LanguageCode = LanguageCode {
    name: "Westphalien",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wep",
    individual_languages: &[
    ],
};


pub const WER: LanguageCode = LanguageCode {
    name: "Weri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wer",
    individual_languages: &[
    ],
};


pub const WES: LanguageCode = LanguageCode {
    name: "Cameroon Pidgin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wes",
    individual_languages: &[
    ],
};


pub const WET: LanguageCode = LanguageCode {
    name: "Perai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wet",
    individual_languages: &[
    ],
};


pub const WEU: LanguageCode = LanguageCode {
    name: "Rawngtu Chin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "weu",
    individual_languages: &[
    ],
};


pub const WEW: LanguageCode = LanguageCode {
    name: "Wejewa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wew",
    individual_languages: &[
    ],
};


pub const WFG: LanguageCode = LanguageCode {
    name: "Yafi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wfg",
    individual_languages: &[
    ],
};


pub const WGA: LanguageCode = LanguageCode {
    name: "Wagaya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wga",
    individual_languages: &[
    ],
};


pub const WGB: LanguageCode = LanguageCode {
    name: "Wagawaga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wgb",
    individual_languages: &[
    ],
};


pub const WGG: LanguageCode = LanguageCode {
    name: "Wangkangurru",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wgg",
    individual_languages: &[
    ],
};


pub const WGI: LanguageCode = LanguageCode {
    name: "Wahgi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wgi",
    individual_languages: &[
    ],
};


pub const WGO: LanguageCode = LanguageCode {
    name: "Waigeo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wgo",
    individual_languages: &[
    ],
};


pub const WGU: LanguageCode = LanguageCode {
    name: "Wirangu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wgu",
    individual_languages: &[
    ],
};


pub const WGY: LanguageCode = LanguageCode {
    name: "Warrgamay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wgy",
    individual_languages: &[
    ],
};


pub const WHA: LanguageCode = LanguageCode {
    name: "Sou Upaa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wha",
    individual_languages: &[
    ],
};


pub const WHG: LanguageCode = LanguageCode {
    name: "North Wahgi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "whg",
    individual_languages: &[
    ],
};


pub const WHK: LanguageCode = LanguageCode {
    name: "Wahau Kenyah",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "whk",
    individual_languages: &[
    ],
};


pub const WHU: LanguageCode = LanguageCode {
    name: "Wahau Kayan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "whu",
    individual_languages: &[
    ],
};


pub const WIB: LanguageCode = LanguageCode {
    name: "Southern Toussian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wib",
    individual_languages: &[
    ],
};


pub const WIC: LanguageCode = LanguageCode {
    name: "Wichita",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wic",
    individual_languages: &[
    ],
};


pub const WIE: LanguageCode = LanguageCode {
    name: "Wik-Epa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wie",
    individual_languages: &[
    ],
};


pub const WIF: LanguageCode = LanguageCode {
    name: "Wik-Keyangan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wif",
    individual_languages: &[
    ],
};


pub const WIG: LanguageCode = LanguageCode {
    name: "Wik Ngathan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wig",
    individual_languages: &[
    ],
};


pub const WIH: LanguageCode = LanguageCode {
    name: "Wik-Me'anha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wih",
    individual_languages: &[
    ],
};


pub const WII: LanguageCode = LanguageCode {
    name: "Minidien",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wii",
    individual_languages: &[
    ],
};


pub const WIJ: LanguageCode = LanguageCode {
    name: "Wik-Iiyanh",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wij",
    individual_languages: &[
    ],
};


pub const WIK: LanguageCode = LanguageCode {
    name: "Wikalkan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wik",
    individual_languages: &[
    ],
};


pub const WIL: LanguageCode = LanguageCode {
    name: "Wilawila",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wil",
    individual_languages: &[
    ],
};


pub const WIM: LanguageCode = LanguageCode {
    name: "Wik-Mungkan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wim",
    individual_languages: &[
    ],
};


pub const WIN: LanguageCode = LanguageCode {
    name: "Ho-Chunk",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "win",
    individual_languages: &[
    ],
};


pub const WIR: LanguageCode = LanguageCode {
    name: "Wiraféd",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wir",
    individual_languages: &[
    ],
};


pub const WIU: LanguageCode = LanguageCode {
    name: "Wiru",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wiu",
    individual_languages: &[
    ],
};


pub const WIV: LanguageCode = LanguageCode {
    name: "Vitu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wiv",
    individual_languages: &[
    ],
};


pub const WIY: LanguageCode = LanguageCode {
    name: "Wiyot",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wiy",
    individual_languages: &[
    ],
};


pub const WJA: LanguageCode = LanguageCode {
    name: "Waja",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wja",
    individual_languages: &[
    ],
};


pub const WJI: LanguageCode = LanguageCode {
    name: "Warji",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wji",
    individual_languages: &[
    ],
};


pub const WKA: LanguageCode = LanguageCode {
    name: "Kw'adza",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wka",
    individual_languages: &[
    ],
};


pub const WKB: LanguageCode = LanguageCode {
    name: "Kumbaran",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wkb",
    individual_languages: &[
    ],
};


pub const WKD: LanguageCode = LanguageCode {
    name: "Wakde",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wkd",
    individual_languages: &[
    ],
};


pub const WKL: LanguageCode = LanguageCode {
    name: "Kalanadi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wkl",
    individual_languages: &[
    ],
};


pub const WKR: LanguageCode = LanguageCode {
    name: "Keerray-Woorroong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wkr",
    individual_languages: &[
    ],
};


pub const WKU: LanguageCode = LanguageCode {
    name: "Kunduvadi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wku",
    individual_languages: &[
    ],
};


pub const WKW: LanguageCode = LanguageCode {
    name: "Wakawaka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wkw",
    individual_languages: &[
    ],
};


pub const WKY: LanguageCode = LanguageCode {
    name: "Wangkayutyuru",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wky",
    individual_languages: &[
    ],
};


pub const WLA: LanguageCode = LanguageCode {
    name: "Walio",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wla",
    individual_languages: &[
    ],
};


pub const WLC: LanguageCode = LanguageCode {
    name: "Mwali Comorian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wlc",
    individual_languages: &[
    ],
};


pub const WLE: LanguageCode = LanguageCode {
    name: "Wolane",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wle",
    individual_languages: &[
    ],
};


pub const WLG: LanguageCode = LanguageCode {
    name: "Kunbarlang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wlg",
    individual_languages: &[
    ],
};


pub const WLH: LanguageCode = LanguageCode {
    name: "Welaun",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wlh",
    individual_languages: &[
    ],
};


pub const WLI: LanguageCode = LanguageCode {
    name: "Waioli",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wli",
    individual_languages: &[
    ],
};


pub const WLK: LanguageCode = LanguageCode {
    name: "Wailaki",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wlk",
    individual_languages: &[
    ],
};


pub const WLL: LanguageCode = LanguageCode {
    name: "Wali (Sudan)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wll",
    individual_languages: &[
    ],
};


pub const WLM: LanguageCode = LanguageCode {
    name: "Middle Welsh",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wlm",
    individual_languages: &[
    ],
};


pub const WLO: LanguageCode = LanguageCode {
    name: "Wolio",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wlo",
    individual_languages: &[
    ],
};


pub const WLR: LanguageCode = LanguageCode {
    name: "Wailapa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wlr",
    individual_languages: &[
    ],
};


pub const WLS: LanguageCode = LanguageCode {
    name: "Wallisian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wls",
    individual_languages: &[
    ],
};


pub const WLU: LanguageCode = LanguageCode {
    name: "Wuliwuli",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wlu",
    individual_languages: &[
    ],
};


pub const WLV: LanguageCode = LanguageCode {
    name: "Wichí Lhamtés Vejoz",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wlv",
    individual_languages: &[
    ],
};


pub const WLW: LanguageCode = LanguageCode {
    name: "Walak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wlw",
    individual_languages: &[
    ],
};


pub const WLX: LanguageCode = LanguageCode {
    name: "Wali (Ghana)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wlx",
    individual_languages: &[
    ],
};


pub const WLY: LanguageCode = LanguageCode {
    name: "Waling",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wly",
    individual_languages: &[
    ],
};


pub const WMA: LanguageCode = LanguageCode {
    name: "Mawa (Nigeria)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wma",
    individual_languages: &[
    ],
};


pub const WMB: LanguageCode = LanguageCode {
    name: "Wambaya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wmb",
    individual_languages: &[
    ],
};


pub const WMC: LanguageCode = LanguageCode {
    name: "Wamas",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wmc",
    individual_languages: &[
    ],
};


pub const WMD: LanguageCode = LanguageCode {
    name: "Mamaindé",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wmd",
    individual_languages: &[
    ],
};


pub const WME: LanguageCode = LanguageCode {
    name: "Wambule",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wme",
    individual_languages: &[
    ],
};


pub const WMG: LanguageCode = LanguageCode {
    name: "Western Minyag",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wmg",
    individual_languages: &[
    ],
};


pub const WMH: LanguageCode = LanguageCode {
    name: "Waima'a",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wmh",
    individual_languages: &[
    ],
};


pub const WMI: LanguageCode = LanguageCode {
    name: "Wamin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wmi",
    individual_languages: &[
    ],
};


pub const WMM: LanguageCode = LanguageCode {
    name: "Maiwa (Indonesia)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wmm",
    individual_languages: &[
    ],
};


pub const WMN: LanguageCode = LanguageCode {
    name: "Waamwang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wmn",
    individual_languages: &[
    ],
};


pub const WMO: LanguageCode = LanguageCode {
    name: "Wom (Papua New Guinea)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wmo",
    individual_languages: &[
    ],
};


pub const WMS: LanguageCode = LanguageCode {
    name: "Wambon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wms",
    individual_languages: &[
    ],
};


pub const WMT: LanguageCode = LanguageCode {
    name: "Walmajarri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wmt",
    individual_languages: &[
    ],
};


pub const WMW: LanguageCode = LanguageCode {
    name: "Mwani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wmw",
    individual_languages: &[
    ],
};


pub const WMX: LanguageCode = LanguageCode {
    name: "Womo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wmx",
    individual_languages: &[
    ],
};


pub const WNB: LanguageCode = LanguageCode {
    name: "Wanambre",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wnb",
    individual_languages: &[
    ],
};


pub const WNC: LanguageCode = LanguageCode {
    name: "Wantoat",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wnc",
    individual_languages: &[
    ],
};


pub const WND: LanguageCode = LanguageCode {
    name: "Wandarang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wnd",
    individual_languages: &[
    ],
};


pub const WNE: LanguageCode = LanguageCode {
    name: "Waneci",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wne",
    individual_languages: &[
    ],
};


pub const WNG: LanguageCode = LanguageCode {
    name: "Wanggom",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wng",
    individual_languages: &[
    ],
};


pub const WNI: LanguageCode = LanguageCode {
    name: "Ndzwani Comorian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wni",
    individual_languages: &[
    ],
};


pub const WNK: LanguageCode = LanguageCode {
    name: "Wanukaka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wnk",
    individual_languages: &[
    ],
};


pub const WNM: LanguageCode = LanguageCode {
    name: "Wanggamala",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wnm",
    individual_languages: &[
    ],
};


pub const WNN: LanguageCode = LanguageCode {
    name: "Wunumara",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wnn",
    individual_languages: &[
    ],
};


pub const WNO: LanguageCode = LanguageCode {
    name: "Wano",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wno",
    individual_languages: &[
    ],
};


pub const WNP: LanguageCode = LanguageCode {
    name: "Wanap",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wnp",
    individual_languages: &[
    ],
};


pub const WNU: LanguageCode = LanguageCode {
    name: "Usan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wnu",
    individual_languages: &[
    ],
};


pub const WNW: LanguageCode = LanguageCode {
    name: "Wintu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wnw",
    individual_languages: &[
    ],
};


pub const WNY: LanguageCode = LanguageCode {
    name: "Wanyi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wny",
    individual_languages: &[
    ],
};


pub const WOA: LanguageCode = LanguageCode {
    name: "Kuwema",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "woa",
    individual_languages: &[
    ],
};


pub const WOB: LanguageCode = LanguageCode {
    name: "Wè Northern",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wob",
    individual_languages: &[
    ],
};


pub const WOC: LanguageCode = LanguageCode {
    name: "Wogeo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "woc",
    individual_languages: &[
    ],
};


pub const WOD: LanguageCode = LanguageCode {
    name: "Wolani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wod",
    individual_languages: &[
    ],
};


pub const WOE: LanguageCode = LanguageCode {
    name: "Woleaian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "woe",
    individual_languages: &[
    ],
};


pub const WOF: LanguageCode = LanguageCode {
    name: "Gambian Wolof",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wof",
    individual_languages: &[
    ],
};


pub const WOG: LanguageCode = LanguageCode {
    name: "Wogamusin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wog",
    individual_languages: &[
    ],
};


pub const WOI: LanguageCode = LanguageCode {
    name: "Kamang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "woi",
    individual_languages: &[
    ],
};


pub const WOK: LanguageCode = LanguageCode {
    name: "Longto",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wok",
    individual_languages: &[
    ],
};


pub const WOM: LanguageCode = LanguageCode {
    name: "Wom (Nigeria)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wom",
    individual_languages: &[
    ],
};


pub const WON: LanguageCode = LanguageCode {
    name: "Wongo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "won",
    individual_languages: &[
    ],
};


pub const WOO: LanguageCode = LanguageCode {
    name: "Manombai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "woo",
    individual_languages: &[
    ],
};


pub const WOR: LanguageCode = LanguageCode {
    name: "Woria",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wor",
    individual_languages: &[
    ],
};


pub const WOS: LanguageCode = LanguageCode {
    name: "Hanga Hundi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wos",
    individual_languages: &[
    ],
};


pub const WOW: LanguageCode = LanguageCode {
    name: "Wawonii",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wow",
    individual_languages: &[
    ],
};


pub const WOY: LanguageCode = LanguageCode {
    name: "Weyto",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "woy",
    individual_languages: &[
    ],
};


pub const WPC: LanguageCode = LanguageCode {
    name: "Maco",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wpc",
    individual_languages: &[
    ],
};


pub const WRB: LanguageCode = LanguageCode {
    name: "Waluwarra",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wrb",
    individual_languages: &[
    ],
};


pub const WRG: LanguageCode = LanguageCode {
    name: "Warungu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wrg",
    individual_languages: &[
    ],
};


pub const WRH: LanguageCode = LanguageCode {
    name: "Wiradjuri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wrh",
    individual_languages: &[
    ],
};


pub const WRI: LanguageCode = LanguageCode {
    name: "Wariyangga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wri",
    individual_languages: &[
    ],
};


pub const WRK: LanguageCode = LanguageCode {
    name: "Garrwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wrk",
    individual_languages: &[
    ],
};


pub const WRL: LanguageCode = LanguageCode {
    name: "Warlmanpa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wrl",
    individual_languages: &[
    ],
};


pub const WRM: LanguageCode = LanguageCode {
    name: "Warumungu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wrm",
    individual_languages: &[
    ],
};


pub const WRN: LanguageCode = LanguageCode {
    name: "Warnang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wrn",
    individual_languages: &[
    ],
};


pub const WRO: LanguageCode = LanguageCode {
    name: "Worrorra",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wro",
    individual_languages: &[
    ],
};


pub const WRP: LanguageCode = LanguageCode {
    name: "Waropen",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wrp",
    individual_languages: &[
    ],
};


pub const WRR: LanguageCode = LanguageCode {
    name: "Wardaman",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wrr",
    individual_languages: &[
    ],
};


pub const WRS: LanguageCode = LanguageCode {
    name: "Waris",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wrs",
    individual_languages: &[
    ],
};


pub const WRU: LanguageCode = LanguageCode {
    name: "Waru",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wru",
    individual_languages: &[
    ],
};


pub const WRV: LanguageCode = LanguageCode {
    name: "Waruna",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wrv",
    individual_languages: &[
    ],
};


pub const WRW: LanguageCode = LanguageCode {
    name: "Gugu Warra",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wrw",
    individual_languages: &[
    ],
};


pub const WRX: LanguageCode = LanguageCode {
    name: "Wae Rana",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wrx",
    individual_languages: &[
    ],
};


pub const WRY: LanguageCode = LanguageCode {
    name: "Merwari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wry",
    individual_languages: &[
    ],
};


pub const WRZ: LanguageCode = LanguageCode {
    name: "Waray (Australia)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wrz",
    individual_languages: &[
    ],
};


pub const WSA: LanguageCode = LanguageCode {
    name: "Warembori",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wsa",
    individual_languages: &[
    ],
};


pub const WSG: LanguageCode = LanguageCode {
    name: "Adilabad Gondi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wsg",
    individual_languages: &[
    ],
};


pub const WSI: LanguageCode = LanguageCode {
    name: "Wusi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wsi",
    individual_languages: &[
    ],
};


pub const WSK: LanguageCode = LanguageCode {
    name: "Waskia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wsk",
    individual_languages: &[
    ],
};


pub const WSR: LanguageCode = LanguageCode {
    name: "Owenia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wsr",
    individual_languages: &[
    ],
};


pub const WSS: LanguageCode = LanguageCode {
    name: "Wasa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wss",
    individual_languages: &[
    ],
};


pub const WSU: LanguageCode = LanguageCode {
    name: "Wasu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wsu",
    individual_languages: &[
    ],
};


pub const WSV: LanguageCode = LanguageCode {
    name: "Wotapuri-Katarqalai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wsv",
    individual_languages: &[
    ],
};


pub const WTF: LanguageCode = LanguageCode {
    name: "Watiwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wtf",
    individual_languages: &[
    ],
};


pub const WTH: LanguageCode = LanguageCode {
    name: "Wathawurrung",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wth",
    individual_languages: &[
    ],
};


pub const WTI: LanguageCode = LanguageCode {
    name: "Berta",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wti",
    individual_languages: &[
    ],
};


pub const WTK: LanguageCode = LanguageCode {
    name: "Watakataui",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wtk",
    individual_languages: &[
    ],
};


pub const WTM: LanguageCode = LanguageCode {
    name: "Mewati",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wtm",
    individual_languages: &[
    ],
};


pub const WTW: LanguageCode = LanguageCode {
    name: "Wotu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wtw",
    individual_languages: &[
    ],
};


pub const WUA: LanguageCode = LanguageCode {
    name: "Wikngenchera",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wua",
    individual_languages: &[
    ],
};


pub const WUB: LanguageCode = LanguageCode {
    name: "Wunambal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wub",
    individual_languages: &[
    ],
};


pub const WUD: LanguageCode = LanguageCode {
    name: "Wudu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wud",
    individual_languages: &[
    ],
};


pub const WUH: LanguageCode = LanguageCode {
    name: "Wutunhua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wuh",
    individual_languages: &[
    ],
};


pub const WUL: LanguageCode = LanguageCode {
    name: "Silimo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wul",
    individual_languages: &[
    ],
};


pub const WUM: LanguageCode = LanguageCode {
    name: "Wumbvu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wum",
    individual_languages: &[
    ],
};


pub const WUN: LanguageCode = LanguageCode {
    name: "Bungu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wun",
    individual_languages: &[
    ],
};


pub const WUR: LanguageCode = LanguageCode {
    name: "Wurrugu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wur",
    individual_languages: &[
    ],
};


pub const WUT: LanguageCode = LanguageCode {
    name: "Wutung",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wut",
    individual_languages: &[
    ],
};


pub const WUU: LanguageCode = LanguageCode {
    name: "Wu Chinese",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wuu",
    individual_languages: &[
    ],
};


pub const WUV: LanguageCode = LanguageCode {
    name: "Wuvulu-Aua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wuv",
    individual_languages: &[
    ],
};


pub const WUX: LanguageCode = LanguageCode {
    name: "Wulna",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wux",
    individual_languages: &[
    ],
};


pub const WUY: LanguageCode = LanguageCode {
    name: "Wauyai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wuy",
    individual_languages: &[
    ],
};


pub const WWA: LanguageCode = LanguageCode {
    name: "Waama",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wwa",
    individual_languages: &[
    ],
};


pub const WWB: LanguageCode = LanguageCode {
    name: "Wakabunga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wwb",
    individual_languages: &[
    ],
};


pub const WWO: LanguageCode = LanguageCode {
    name: "Wetamut",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wwo",
    individual_languages: &[
    ],
};


pub const WWR: LanguageCode = LanguageCode {
    name: "Warrwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wwr",
    individual_languages: &[
    ],
};


pub const WWW: LanguageCode = LanguageCode {
    name: "Wawa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "www",
    individual_languages: &[
    ],
};


pub const WXA: LanguageCode = LanguageCode {
    name: "Waxianghua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wxa",
    individual_languages: &[
    ],
};


pub const WXW: LanguageCode = LanguageCode {
    name: "Wardandi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wxw",
    individual_languages: &[
    ],
};


pub const WYB: LanguageCode = LanguageCode {
    name: "Wangaaybuwan-Ngiyambaa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wyb",
    individual_languages: &[
    ],
};


pub const WYI: LanguageCode = LanguageCode {
    name: "Woiwurrung",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wyi",
    individual_languages: &[
    ],
};


pub const WYM: LanguageCode = LanguageCode {
    name: "Wymysorys",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wym",
    individual_languages: &[
    ],
};


pub const WYN: LanguageCode = LanguageCode {
    name: "Wyandot",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wyn",
    individual_languages: &[
    ],
};


pub const WYR: LanguageCode = LanguageCode {
    name: "Wayoró",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wyr",
    individual_languages: &[
    ],
};


pub const WYY: LanguageCode = LanguageCode {
    name: "Western Fijian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "wyy",
    individual_languages: &[
    ],
};


pub const XAA: LanguageCode = LanguageCode {
    name: "Andalusian Arabic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xaa",
    individual_languages: &[
    ],
};


pub const XAB: LanguageCode = LanguageCode {
    name: "Sambe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xab",
    individual_languages: &[
    ],
};


pub const XAC: LanguageCode = LanguageCode {
    name: "Kachari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xac",
    individual_languages: &[
    ],
};


pub const XAD: LanguageCode = LanguageCode {
    name: "Adai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xad",
    individual_languages: &[
    ],
};


pub const XAE: LanguageCode = LanguageCode {
    name: "Aequian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xae",
    individual_languages: &[
    ],
};


pub const XAG: LanguageCode = LanguageCode {
    name: "Aghwan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xag",
    individual_languages: &[
    ],
};


pub const XAI: LanguageCode = LanguageCode {
    name: "Kaimbé",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xai",
    individual_languages: &[
    ],
};


pub const XAJ: LanguageCode = LanguageCode {
    name: "Ararandewára",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xaj",
    individual_languages: &[
    ],
};


pub const XAK: LanguageCode = LanguageCode {
    name: "Máku",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xak",
    individual_languages: &[
    ],
};


pub const XAM: LanguageCode = LanguageCode {
    name: "ǀXam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xam",
    individual_languages: &[
    ],
};


pub const XAN: LanguageCode = LanguageCode {
    name: "Xamtanga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xan",
    individual_languages: &[
    ],
};


pub const XAO: LanguageCode = LanguageCode {
    name: "Khao",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xao",
    individual_languages: &[
    ],
};


pub const XAP: LanguageCode = LanguageCode {
    name: "Apalachee",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xap",
    individual_languages: &[
    ],
};


pub const XAQ: LanguageCode = LanguageCode {
    name: "Aquitanian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xaq",
    individual_languages: &[
    ],
};


pub const XAR: LanguageCode = LanguageCode {
    name: "Karami",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xar",
    individual_languages: &[
    ],
};


pub const XAS: LanguageCode = LanguageCode {
    name: "Kamas",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xas",
    individual_languages: &[
    ],
};


pub const XAT: LanguageCode = LanguageCode {
    name: "Katawixi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xat",
    individual_languages: &[
    ],
};


pub const XAU: LanguageCode = LanguageCode {
    name: "Kauwera",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xau",
    individual_languages: &[
    ],
};


pub const XAV: LanguageCode = LanguageCode {
    name: "Xavánte",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xav",
    individual_languages: &[
    ],
};


pub const XAW: LanguageCode = LanguageCode {
    name: "Kawaiisu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xaw",
    individual_languages: &[
    ],
};


pub const XAY: LanguageCode = LanguageCode {
    name: "Kayan Mahakam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xay",
    individual_languages: &[
    ],
};


pub const XBB: LanguageCode = LanguageCode {
    name: "Lower Burdekin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xbb",
    individual_languages: &[
    ],
};


pub const XBC: LanguageCode = LanguageCode {
    name: "Bactrian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xbc",
    individual_languages: &[
    ],
};


pub const XBD: LanguageCode = LanguageCode {
    name: "Bindal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xbd",
    individual_languages: &[
    ],
};


pub const XBE: LanguageCode = LanguageCode {
    name: "Bigambal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xbe",
    individual_languages: &[
    ],
};


pub const XBG: LanguageCode = LanguageCode {
    name: "Bunganditj",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xbg",
    individual_languages: &[
    ],
};


pub const XBI: LanguageCode = LanguageCode {
    name: "Kombio",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xbi",
    individual_languages: &[
    ],
};


pub const XBJ: LanguageCode = LanguageCode {
    name: "Birrpayi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xbj",
    individual_languages: &[
    ],
};


pub const XBM: LanguageCode = LanguageCode {
    name: "Middle Breton",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xbm",
    individual_languages: &[
    ],
};


pub const XBN: LanguageCode = LanguageCode {
    name: "Kenaboi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xbn",
    individual_languages: &[
    ],
};


pub const XBO: LanguageCode = LanguageCode {
    name: "Bolgarian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xbo",
    individual_languages: &[
    ],
};


pub const XBP: LanguageCode = LanguageCode {
    name: "Bibbulman",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xbp",
    individual_languages: &[
    ],
};


pub const XBR: LanguageCode = LanguageCode {
    name: "Kambera",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xbr",
    individual_languages: &[
    ],
};


pub const XBW: LanguageCode = LanguageCode {
    name: "Kambiwá",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xbw",
    individual_languages: &[
    ],
};


pub const XBY: LanguageCode = LanguageCode {
    name: "Batjala",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xby",
    individual_languages: &[
    ],
};


pub const XCB: LanguageCode = LanguageCode {
    name: "Cumbric",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xcb",
    individual_languages: &[
    ],
};


pub const XCC: LanguageCode = LanguageCode {
    name: "Camunic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xcc",
    individual_languages: &[
    ],
};


pub const XCE: LanguageCode = LanguageCode {
    name: "Celtiberian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xce",
    individual_languages: &[
    ],
};


pub const XCG: LanguageCode = LanguageCode {
    name: "Cisalpine Gaulish",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xcg",
    individual_languages: &[
    ],
};


pub const XCH: LanguageCode = LanguageCode {
    name: "Chemakum",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xch",
    individual_languages: &[
    ],
};


pub const XCL: LanguageCode = LanguageCode {
    name: "Classical Armenian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xcl",
    individual_languages: &[
    ],
};


pub const XCM: LanguageCode = LanguageCode {
    name: "Comecrudo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xcm",
    individual_languages: &[
    ],
};


pub const XCN: LanguageCode = LanguageCode {
    name: "Cotoname",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xcn",
    individual_languages: &[
    ],
};


pub const XCO: LanguageCode = LanguageCode {
    name: "Chorasmian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xco",
    individual_languages: &[
    ],
};


pub const XCR: LanguageCode = LanguageCode {
    name: "Carian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xcr",
    individual_languages: &[
    ],
};


pub const XCT: LanguageCode = LanguageCode {
    name: "Classical Tibetan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xct",
    individual_languages: &[
    ],
};


pub const XCU: LanguageCode = LanguageCode {
    name: "Curonian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xcu",
    individual_languages: &[
    ],
};


pub const XCV: LanguageCode = LanguageCode {
    name: "Chuvantsy",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xcv",
    individual_languages: &[
    ],
};


pub const XCW: LanguageCode = LanguageCode {
    name: "Coahuilteco",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xcw",
    individual_languages: &[
    ],
};


pub const XCY: LanguageCode = LanguageCode {
    name: "Cayuse",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xcy",
    individual_languages: &[
    ],
};


pub const XDA: LanguageCode = LanguageCode {
    name: "Darkinyung",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xda",
    individual_languages: &[
    ],
};


pub const XDC: LanguageCode = LanguageCode {
    name: "Dacian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xdc",
    individual_languages: &[
    ],
};


pub const XDK: LanguageCode = LanguageCode {
    name: "Dharuk",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xdk",
    individual_languages: &[
    ],
};


pub const XDM: LanguageCode = LanguageCode {
    name: "Edomite",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xdm",
    individual_languages: &[
    ],
};


pub const XDO: LanguageCode = LanguageCode {
    name: "Kwandu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xdo",
    individual_languages: &[
    ],
};


pub const XDQ: LanguageCode = LanguageCode {
    name: "Kaitag",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xdq",
    individual_languages: &[
    ],
};


pub const XDY: LanguageCode = LanguageCode {
    name: "Malayic Dayak",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xdy",
    individual_languages: &[
    ],
};


pub const XEB: LanguageCode = LanguageCode {
    name: "Eblan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xeb",
    individual_languages: &[
    ],
};


pub const XED: LanguageCode = LanguageCode {
    name: "Hdi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xed",
    individual_languages: &[
    ],
};


pub const XEG: LanguageCode = LanguageCode {
    name: "ǁXegwi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xeg",
    individual_languages: &[
    ],
};


pub const XEL: LanguageCode = LanguageCode {
    name: "Kelo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xel",
    individual_languages: &[
    ],
};


pub const XEM: LanguageCode = LanguageCode {
    name: "Kembayan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xem",
    individual_languages: &[
    ],
};


pub const XEP: LanguageCode = LanguageCode {
    name: "Epi-Olmec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xep",
    individual_languages: &[
    ],
};


pub const XER: LanguageCode = LanguageCode {
    name: "Xerénte",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xer",
    individual_languages: &[
    ],
};


pub const XES: LanguageCode = LanguageCode {
    name: "Kesawai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xes",
    individual_languages: &[
    ],
};


pub const XET: LanguageCode = LanguageCode {
    name: "Xetá",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xet",
    individual_languages: &[
    ],
};


pub const XEU: LanguageCode = LanguageCode {
    name: "Keoru-Ahia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xeu",
    individual_languages: &[
    ],
};


pub const XFA: LanguageCode = LanguageCode {
    name: "Faliscan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xfa",
    individual_languages: &[
    ],
};


pub const XGA: LanguageCode = LanguageCode {
    name: "Galatian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xga",
    individual_languages: &[
    ],
};


pub const XGB: LanguageCode = LanguageCode {
    name: "Gbin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xgb",
    individual_languages: &[
    ],
};


pub const XGD: LanguageCode = LanguageCode {
    name: "Gudang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xgd",
    individual_languages: &[
    ],
};


pub const XGF: LanguageCode = LanguageCode {
    name: "Gabrielino-Fernandeño",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xgf",
    individual_languages: &[
    ],
};


pub const XGG: LanguageCode = LanguageCode {
    name: "Goreng",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xgg",
    individual_languages: &[
    ],
};


pub const XGI: LanguageCode = LanguageCode {
    name: "Garingbal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xgi",
    individual_languages: &[
    ],
};


pub const XGL: LanguageCode = LanguageCode {
    name: "Galindan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xgl",
    individual_languages: &[
    ],
};


pub const XGM: LanguageCode = LanguageCode {
    name: "Dharumbal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xgm",
    individual_languages: &[
    ],
};


pub const XGR: LanguageCode = LanguageCode {
    name: "Garza",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xgr",
    individual_languages: &[
    ],
};


pub const XGU: LanguageCode = LanguageCode {
    name: "Unggumi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xgu",
    individual_languages: &[
    ],
};


pub const XGW: LanguageCode = LanguageCode {
    name: "Guwa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xgw",
    individual_languages: &[
    ],
};


pub const XHA: LanguageCode = LanguageCode {
    name: "Harami",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xha",
    individual_languages: &[
    ],
};


pub const XHC: LanguageCode = LanguageCode {
    name: "Hunnic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xhc",
    individual_languages: &[
    ],
};


pub const XHD: LanguageCode = LanguageCode {
    name: "Hadrami",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xhd",
    individual_languages: &[
    ],
};


pub const XHE: LanguageCode = LanguageCode {
    name: "Khetrani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xhe",
    individual_languages: &[
    ],
};


pub const XHM: LanguageCode = LanguageCode {
    name: "Middle Khmer (1400 to 1850 CE)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xhm",
    individual_languages: &[
    ],
};


pub const XHR: LanguageCode = LanguageCode {
    name: "Hernican",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xhr",
    individual_languages: &[
    ],
};


pub const XHT: LanguageCode = LanguageCode {
    name: "Hattic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xht",
    individual_languages: &[
    ],
};


pub const XHU: LanguageCode = LanguageCode {
    name: "Hurrian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xhu",
    individual_languages: &[
    ],
};


pub const XHV: LanguageCode = LanguageCode {
    name: "Khua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xhv",
    individual_languages: &[
    ],
};


pub const XIB: LanguageCode = LanguageCode {
    name: "Iberian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xib",
    individual_languages: &[
    ],
};


pub const XII: LanguageCode = LanguageCode {
    name: "Xiri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xii",
    individual_languages: &[
    ],
};


pub const XIL: LanguageCode = LanguageCode {
    name: "Illyrian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xil",
    individual_languages: &[
    ],
};


pub const XIN: LanguageCode = LanguageCode {
    name: "Xinca",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xin",
    individual_languages: &[
    ],
};


pub const XIR: LanguageCode = LanguageCode {
    name: "Xiriâna",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xir",
    individual_languages: &[
    ],
};


pub const XIS: LanguageCode = LanguageCode {
    name: "Kisan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xis",
    individual_languages: &[
    ],
};


pub const XIV: LanguageCode = LanguageCode {
    name: "Indus Valley Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xiv",
    individual_languages: &[
    ],
};


pub const XIY: LanguageCode = LanguageCode {
    name: "Xipaya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xiy",
    individual_languages: &[
    ],
};


pub const XJB: LanguageCode = LanguageCode {
    name: "Minjungbal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xjb",
    individual_languages: &[
    ],
};


pub const XJT: LanguageCode = LanguageCode {
    name: "Jaitmatang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xjt",
    individual_languages: &[
    ],
};


pub const XKA: LanguageCode = LanguageCode {
    name: "Kalkoti",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xka",
    individual_languages: &[
    ],
};


pub const XKB: LanguageCode = LanguageCode {
    name: "Northern Nago",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xkb",
    individual_languages: &[
    ],
};


pub const XKC: LanguageCode = LanguageCode {
    name: "Kho'ini",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xkc",
    individual_languages: &[
    ],
};


pub const XKD: LanguageCode = LanguageCode {
    name: "Mendalam Kayan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xkd",
    individual_languages: &[
    ],
};


pub const XKE: LanguageCode = LanguageCode {
    name: "Kereho",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xke",
    individual_languages: &[
    ],
};


pub const XKF: LanguageCode = LanguageCode {
    name: "Khengkha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xkf",
    individual_languages: &[
    ],
};


pub const XKG: LanguageCode = LanguageCode {
    name: "Kagoro",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xkg",
    individual_languages: &[
    ],
};


pub const XKI: LanguageCode = LanguageCode {
    name: "Kenyan Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xki",
    individual_languages: &[
    ],
};


pub const XKJ: LanguageCode = LanguageCode {
    name: "Kajali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xkj",
    individual_languages: &[
    ],
};


pub const XKK: LanguageCode = LanguageCode {
    name: "Kachok",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xkk",
    individual_languages: &[
    ],
};


pub const XKL: LanguageCode = LanguageCode {
    name: "Mainstream Kenyah",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xkl",
    individual_languages: &[
    ],
};


pub const XKN: LanguageCode = LanguageCode {
    name: "Kayan River Kayan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xkn",
    individual_languages: &[
    ],
};


pub const XKO: LanguageCode = LanguageCode {
    name: "Kiorr",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xko",
    individual_languages: &[
    ],
};


pub const XKP: LanguageCode = LanguageCode {
    name: "Kabatei",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xkp",
    individual_languages: &[
    ],
};


pub const XKQ: LanguageCode = LanguageCode {
    name: "Koroni",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xkq",
    individual_languages: &[
    ],
};


pub const XKR: LanguageCode = LanguageCode {
    name: "Xakriabá",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xkr",
    individual_languages: &[
    ],
};


pub const XKS: LanguageCode = LanguageCode {
    name: "Kumbewaha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xks",
    individual_languages: &[
    ],
};


pub const XKT: LanguageCode = LanguageCode {
    name: "Kantosi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xkt",
    individual_languages: &[
    ],
};


pub const XKU: LanguageCode = LanguageCode {
    name: "Kaamba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xku",
    individual_languages: &[
    ],
};


pub const XKV: LanguageCode = LanguageCode {
    name: "Kgalagadi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xkv",
    individual_languages: &[
    ],
};


pub const XKW: LanguageCode = LanguageCode {
    name: "Kembra",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xkw",
    individual_languages: &[
    ],
};


pub const XKX: LanguageCode = LanguageCode {
    name: "Karore",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xkx",
    individual_languages: &[
    ],
};


pub const XKY: LanguageCode = LanguageCode {
    name: "Uma' Lasan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xky",
    individual_languages: &[
    ],
};


pub const XKZ: LanguageCode = LanguageCode {
    name: "Kurtokha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xkz",
    individual_languages: &[
    ],
};


pub const XLA: LanguageCode = LanguageCode {
    name: "Kamula",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xla",
    individual_languages: &[
    ],
};


pub const XLB: LanguageCode = LanguageCode {
    name: "Loup B",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xlb",
    individual_languages: &[
    ],
};


pub const XLC: LanguageCode = LanguageCode {
    name: "Lycian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xlc",
    individual_languages: &[
    ],
};


pub const XLD: LanguageCode = LanguageCode {
    name: "Lydian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xld",
    individual_languages: &[
    ],
};


pub const XLE: LanguageCode = LanguageCode {
    name: "Lemnian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xle",
    individual_languages: &[
    ],
};


pub const XLG: LanguageCode = LanguageCode {
    name: "Ligurian (Ancient)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xlg",
    individual_languages: &[
    ],
};


pub const XLI: LanguageCode = LanguageCode {
    name: "Liburnian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xli",
    individual_languages: &[
    ],
};


pub const XLN: LanguageCode = LanguageCode {
    name: "Alanic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xln",
    individual_languages: &[
    ],
};


pub const XLO: LanguageCode = LanguageCode {
    name: "Loup A",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xlo",
    individual_languages: &[
    ],
};


pub const XLP: LanguageCode = LanguageCode {
    name: "Lepontic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xlp",
    individual_languages: &[
    ],
};


pub const XLS: LanguageCode = LanguageCode {
    name: "Lusitanian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xls",
    individual_languages: &[
    ],
};


pub const XLU: LanguageCode = LanguageCode {
    name: "Cuneiform Luwian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xlu",
    individual_languages: &[
    ],
};


pub const XLY: LanguageCode = LanguageCode {
    name: "Elymian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xly",
    individual_languages: &[
    ],
};


pub const XMA: LanguageCode = LanguageCode {
    name: "Mushungulu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xma",
    individual_languages: &[
    ],
};


pub const XMB: LanguageCode = LanguageCode {
    name: "Mbonga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xmb",
    individual_languages: &[
    ],
};


pub const XMC: LanguageCode = LanguageCode {
    name: "Makhuwa-Marrevone",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xmc",
    individual_languages: &[
    ],
};


pub const XMD: LanguageCode = LanguageCode {
    name: "Mbudum",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xmd",
    individual_languages: &[
    ],
};


pub const XME: LanguageCode = LanguageCode {
    name: "Median",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xme",
    individual_languages: &[
    ],
};


pub const XMF: LanguageCode = LanguageCode {
    name: "Mingrelian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xmf",
    individual_languages: &[
    ],
};


pub const XMG: LanguageCode = LanguageCode {
    name: "Mengaka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xmg",
    individual_languages: &[
    ],
};


pub const XMH: LanguageCode = LanguageCode {
    name: "Kugu-Muminh",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xmh",
    individual_languages: &[
    ],
};


pub const XMJ: LanguageCode = LanguageCode {
    name: "Majera",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xmj",
    individual_languages: &[
    ],
};


pub const XMK: LanguageCode = LanguageCode {
    name: "Ancient Macedonian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xmk",
    individual_languages: &[
    ],
};


pub const XML: LanguageCode = LanguageCode {
    name: "Malaysian Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xml",
    individual_languages: &[
    ],
};


pub const XMM: LanguageCode = LanguageCode {
    name: "Manado Malay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xmm",
    individual_languages: &[
    ],
};


pub const XMN: LanguageCode = LanguageCode {
    name: "Manichaean Middle Persian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xmn",
    individual_languages: &[
    ],
};


pub const XMO: LanguageCode = LanguageCode {
    name: "Morerebi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xmo",
    individual_languages: &[
    ],
};


pub const XMP: LanguageCode = LanguageCode {
    name: "Kuku-Mu'inh",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xmp",
    individual_languages: &[
    ],
};


pub const XMQ: LanguageCode = LanguageCode {
    name: "Kuku-Mangk",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xmq",
    individual_languages: &[
    ],
};


pub const XMR: LanguageCode = LanguageCode {
    name: "Meroitic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xmr",
    individual_languages: &[
    ],
};


pub const XMS: LanguageCode = LanguageCode {
    name: "Moroccan Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xms",
    individual_languages: &[
    ],
};


pub const XMT: LanguageCode = LanguageCode {
    name: "Matbat",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xmt",
    individual_languages: &[
    ],
};


pub const XMU: LanguageCode = LanguageCode {
    name: "Kamu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xmu",
    individual_languages: &[
    ],
};


pub const XMV: LanguageCode = LanguageCode {
    name: "Antankarana Malagasy",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xmv",
    individual_languages: &[
    ],
};


pub const XMW: LanguageCode = LanguageCode {
    name: "Tsimihety Malagasy",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xmw",
    individual_languages: &[
    ],
};


pub const XMX: LanguageCode = LanguageCode {
    name: "Salawati",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xmx",
    individual_languages: &[
    ],
};


pub const XMY: LanguageCode = LanguageCode {
    name: "Mayaguduna",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xmy",
    individual_languages: &[
    ],
};


pub const XMZ: LanguageCode = LanguageCode {
    name: "Mori Bawah",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xmz",
    individual_languages: &[
    ],
};


pub const XNA: LanguageCode = LanguageCode {
    name: "Ancient North Arabian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xna",
    individual_languages: &[
    ],
};


pub const XNB: LanguageCode = LanguageCode {
    name: "Kanakanabu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xnb",
    individual_languages: &[
    ],
};


pub const XNG: LanguageCode = LanguageCode {
    name: "Middle Mongolian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xng",
    individual_languages: &[
    ],
};


pub const XNH: LanguageCode = LanguageCode {
    name: "Kuanhua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xnh",
    individual_languages: &[
    ],
};


pub const XNI: LanguageCode = LanguageCode {
    name: "Ngarigu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xni",
    individual_languages: &[
    ],
};


pub const XNJ: LanguageCode = LanguageCode {
    name: "Ngoni (Tanzania)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xnj",
    individual_languages: &[
    ],
};


pub const XNK: LanguageCode = LanguageCode {
    name: "Nganakarti",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xnk",
    individual_languages: &[
    ],
};


pub const XNM: LanguageCode = LanguageCode {
    name: "Ngumbarl",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xnm",
    individual_languages: &[
    ],
};


pub const XNN: LanguageCode = LanguageCode {
    name: "Northern Kankanay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xnn",
    individual_languages: &[
    ],
};


pub const XNO: LanguageCode = LanguageCode {
    name: "Anglo-Norman",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xno",
    individual_languages: &[
    ],
};


pub const XNQ: LanguageCode = LanguageCode {
    name: "Ngoni (Mozambique)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xnq",
    individual_languages: &[
    ],
};


pub const XNR: LanguageCode = LanguageCode {
    name: "Kangri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xnr",
    individual_languages: &[
    ],
};


pub const XNS: LanguageCode = LanguageCode {
    name: "Kanashi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xns",
    individual_languages: &[
    ],
};


pub const XNT: LanguageCode = LanguageCode {
    name: "Narragansett",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xnt",
    individual_languages: &[
    ],
};


pub const XNU: LanguageCode = LanguageCode {
    name: "Nukunul",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xnu",
    individual_languages: &[
    ],
};


pub const XNY: LanguageCode = LanguageCode {
    name: "Nyiyaparli",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xny",
    individual_languages: &[
    ],
};


pub const XNZ: LanguageCode = LanguageCode {
    name: "Kenzi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xnz",
    individual_languages: &[
    ],
};


pub const XOC: LanguageCode = LanguageCode {
    name: "O'chi'chi'",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xoc",
    individual_languages: &[
    ],
};


pub const XOD: LanguageCode = LanguageCode {
    name: "Kokoda",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xod",
    individual_languages: &[
    ],
};


pub const XOG: LanguageCode = LanguageCode {
    name: "Soga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xog",
    individual_languages: &[
    ],
};


pub const XOI: LanguageCode = LanguageCode {
    name: "Kominimung",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xoi",
    individual_languages: &[
    ],
};


pub const XOK: LanguageCode = LanguageCode {
    name: "Xokleng",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xok",
    individual_languages: &[
    ],
};


pub const XOM: LanguageCode = LanguageCode {
    name: "Komo (Sudan)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xom",
    individual_languages: &[
    ],
};


pub const XON: LanguageCode = LanguageCode {
    name: "Konkomba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xon",
    individual_languages: &[
    ],
};


pub const XOO: LanguageCode = LanguageCode {
    name: "Xukurú",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xoo",
    individual_languages: &[
    ],
};


pub const XOP: LanguageCode = LanguageCode {
    name: "Kopar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xop",
    individual_languages: &[
    ],
};


pub const XOR: LanguageCode = LanguageCode {
    name: "Korubo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xor",
    individual_languages: &[
    ],
};


pub const XOW: LanguageCode = LanguageCode {
    name: "Kowaki",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xow",
    individual_languages: &[
    ],
};


pub const XPA: LanguageCode = LanguageCode {
    name: "Pirriya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xpa",
    individual_languages: &[
    ],
};


pub const XPB: LanguageCode = LanguageCode {
    name: "Northeastern Tasmanian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xpb",
    individual_languages: &[
    ],
};


pub const XPC: LanguageCode = LanguageCode {
    name: "Pecheneg",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xpc",
    individual_languages: &[
    ],
};


pub const XPD: LanguageCode = LanguageCode {
    name: "Oyster Bay Tasmanian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xpd",
    individual_languages: &[
    ],
};


pub const XPE: LanguageCode = LanguageCode {
    name: "Liberia Kpelle",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xpe",
    individual_languages: &[
    ],
};


pub const XPF: LanguageCode = LanguageCode {
    name: "Southeast Tasmanian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xpf",
    individual_languages: &[
    ],
};


pub const XPG: LanguageCode = LanguageCode {
    name: "Phrygian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xpg",
    individual_languages: &[
    ],
};


pub const XPH: LanguageCode = LanguageCode {
    name: "North Midlands Tasmanian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xph",
    individual_languages: &[
    ],
};


pub const XPI: LanguageCode = LanguageCode {
    name: "Pictish",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xpi",
    individual_languages: &[
    ],
};


pub const XPJ: LanguageCode = LanguageCode {
    name: "Mpalitjanh",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xpj",
    individual_languages: &[
    ],
};


pub const XPK: LanguageCode = LanguageCode {
    name: "Kulina Pano",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xpk",
    individual_languages: &[
    ],
};


pub const XPL: LanguageCode = LanguageCode {
    name: "Port Sorell Tasmanian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xpl",
    individual_languages: &[
    ],
};


pub const XPM: LanguageCode = LanguageCode {
    name: "Pumpokol",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xpm",
    individual_languages: &[
    ],
};


pub const XPN: LanguageCode = LanguageCode {
    name: "Kapinawá",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xpn",
    individual_languages: &[
    ],
};


pub const XPO: LanguageCode = LanguageCode {
    name: "Pochutec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xpo",
    individual_languages: &[
    ],
};


pub const XPP: LanguageCode = LanguageCode {
    name: "Puyo-Paekche",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xpp",
    individual_languages: &[
    ],
};


pub const XPQ: LanguageCode = LanguageCode {
    name: "Mohegan-Pequot",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xpq",
    individual_languages: &[
    ],
};


pub const XPR: LanguageCode = LanguageCode {
    name: "Parthian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xpr",
    individual_languages: &[
    ],
};


pub const XPS: LanguageCode = LanguageCode {
    name: "Pisidian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xps",
    individual_languages: &[
    ],
};


pub const XPT: LanguageCode = LanguageCode {
    name: "Punthamara",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xpt",
    individual_languages: &[
    ],
};


pub const XPU: LanguageCode = LanguageCode {
    name: "Punic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xpu",
    individual_languages: &[
    ],
};


pub const XPV: LanguageCode = LanguageCode {
    name: "Northern Tasmanian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xpv",
    individual_languages: &[
    ],
};


pub const XPW: LanguageCode = LanguageCode {
    name: "Northwestern Tasmanian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xpw",
    individual_languages: &[
    ],
};


pub const XPX: LanguageCode = LanguageCode {
    name: "Southwestern Tasmanian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xpx",
    individual_languages: &[
    ],
};


pub const XPY: LanguageCode = LanguageCode {
    name: "Puyo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xpy",
    individual_languages: &[
    ],
};


pub const XPZ: LanguageCode = LanguageCode {
    name: "Bruny Island Tasmanian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xpz",
    individual_languages: &[
    ],
};


pub const XQA: LanguageCode = LanguageCode {
    name: "Karakhanid",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xqa",
    individual_languages: &[
    ],
};


pub const XQT: LanguageCode = LanguageCode {
    name: "Qatabanian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xqt",
    individual_languages: &[
    ],
};


pub const XRA: LanguageCode = LanguageCode {
    name: "Krahô",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xra",
    individual_languages: &[
    ],
};


pub const XRB: LanguageCode = LanguageCode {
    name: "Eastern Karaboro",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xrb",
    individual_languages: &[
    ],
};


pub const XRD: LanguageCode = LanguageCode {
    name: "Gundungurra",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xrd",
    individual_languages: &[
    ],
};


pub const XRE: LanguageCode = LanguageCode {
    name: "Kreye",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xre",
    individual_languages: &[
    ],
};


pub const XRG: LanguageCode = LanguageCode {
    name: "Minang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xrg",
    individual_languages: &[
    ],
};


pub const XRI: LanguageCode = LanguageCode {
    name: "Krikati-Timbira",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xri",
    individual_languages: &[
    ],
};


pub const XRM: LanguageCode = LanguageCode {
    name: "Armazic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xrm",
    individual_languages: &[
    ],
};


pub const XRN: LanguageCode = LanguageCode {
    name: "Arin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xrn",
    individual_languages: &[
    ],
};


pub const XRR: LanguageCode = LanguageCode {
    name: "Raetic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xrr",
    individual_languages: &[
    ],
};


pub const XRT: LanguageCode = LanguageCode {
    name: "Aranama-Tamique",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xrt",
    individual_languages: &[
    ],
};


pub const XRU: LanguageCode = LanguageCode {
    name: "Marriammu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xru",
    individual_languages: &[
    ],
};


pub const XRW: LanguageCode = LanguageCode {
    name: "Karawa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xrw",
    individual_languages: &[
    ],
};


pub const XSA: LanguageCode = LanguageCode {
    name: "Sabaean",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xsa",
    individual_languages: &[
    ],
};


pub const XSB: LanguageCode = LanguageCode {
    name: "Sambal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xsb",
    individual_languages: &[
    ],
};


pub const XSC: LanguageCode = LanguageCode {
    name: "Scythian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xsc",
    individual_languages: &[
    ],
};


pub const XSD: LanguageCode = LanguageCode {
    name: "Sidetic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xsd",
    individual_languages: &[
    ],
};


pub const XSE: LanguageCode = LanguageCode {
    name: "Sempan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xse",
    individual_languages: &[
    ],
};


pub const XSH: LanguageCode = LanguageCode {
    name: "Shamang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xsh",
    individual_languages: &[
    ],
};


pub const XSI: LanguageCode = LanguageCode {
    name: "Sio",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xsi",
    individual_languages: &[
    ],
};


pub const XSJ: LanguageCode = LanguageCode {
    name: "Subi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xsj",
    individual_languages: &[
    ],
};


pub const XSL: LanguageCode = LanguageCode {
    name: "South Slavey",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xsl",
    individual_languages: &[
    ],
};


pub const XSM: LanguageCode = LanguageCode {
    name: "Kasem",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xsm",
    individual_languages: &[
    ],
};


pub const XSN: LanguageCode = LanguageCode {
    name: "Sanga (Nigeria)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xsn",
    individual_languages: &[
    ],
};


pub const XSO: LanguageCode = LanguageCode {
    name: "Solano",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xso",
    individual_languages: &[
    ],
};


pub const XSP: LanguageCode = LanguageCode {
    name: "Silopi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xsp",
    individual_languages: &[
    ],
};


pub const XSQ: LanguageCode = LanguageCode {
    name: "Makhuwa-Saka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xsq",
    individual_languages: &[
    ],
};


pub const XSR: LanguageCode = LanguageCode {
    name: "Sherpa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xsr",
    individual_languages: &[
    ],
};


pub const XSS: LanguageCode = LanguageCode {
    name: "Assan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xss",
    individual_languages: &[
    ],
};


pub const XSU: LanguageCode = LanguageCode {
    name: "Sanumá",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xsu",
    individual_languages: &[
    ],
};


pub const XSV: LanguageCode = LanguageCode {
    name: "Sudovian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xsv",
    individual_languages: &[
    ],
};


pub const XSY: LanguageCode = LanguageCode {
    name: "Saisiyat",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xsy",
    individual_languages: &[
    ],
};


pub const XTA: LanguageCode = LanguageCode {
    name: "Alcozauca Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xta",
    individual_languages: &[
    ],
};


pub const XTB: LanguageCode = LanguageCode {
    name: "Chazumba Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xtb",
    individual_languages: &[
    ],
};


pub const XTC: LanguageCode = LanguageCode {
    name: "Katcha-Kadugli-Miri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xtc",
    individual_languages: &[
    ],
};


pub const XTD: LanguageCode = LanguageCode {
    name: "Diuxi-Tilantongo Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xtd",
    individual_languages: &[
    ],
};


pub const XTE: LanguageCode = LanguageCode {
    name: "Ketengban",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xte",
    individual_languages: &[
    ],
};


pub const XTG: LanguageCode = LanguageCode {
    name: "Transalpine Gaulish",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xtg",
    individual_languages: &[
    ],
};


pub const XTH: LanguageCode = LanguageCode {
    name: "Yitha Yitha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xth",
    individual_languages: &[
    ],
};


pub const XTI: LanguageCode = LanguageCode {
    name: "Sinicahua Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xti",
    individual_languages: &[
    ],
};


pub const XTJ: LanguageCode = LanguageCode {
    name: "San Juan Teita Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xtj",
    individual_languages: &[
    ],
};


pub const XTL: LanguageCode = LanguageCode {
    name: "Tijaltepec Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xtl",
    individual_languages: &[
    ],
};


pub const XTM: LanguageCode = LanguageCode {
    name: "Magdalena Peñasco Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xtm",
    individual_languages: &[
    ],
};


pub const XTN: LanguageCode = LanguageCode {
    name: "Northern Tlaxiaco Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xtn",
    individual_languages: &[
    ],
};


pub const XTO: LanguageCode = LanguageCode {
    name: "Tokharian A",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xto",
    individual_languages: &[
    ],
};


pub const XTP: LanguageCode = LanguageCode {
    name: "San Miguel Piedras Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xtp",
    individual_languages: &[
    ],
};


pub const XTQ: LanguageCode = LanguageCode {
    name: "Tumshuqese",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xtq",
    individual_languages: &[
    ],
};


pub const XTR: LanguageCode = LanguageCode {
    name: "Early Tripuri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xtr",
    individual_languages: &[
    ],
};


pub const XTS: LanguageCode = LanguageCode {
    name: "Sindihui Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xts",
    individual_languages: &[
    ],
};


pub const XTT: LanguageCode = LanguageCode {
    name: "Tacahua Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xtt",
    individual_languages: &[
    ],
};


pub const XTU: LanguageCode = LanguageCode {
    name: "Cuyamecalco Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xtu",
    individual_languages: &[
    ],
};


pub const XTV: LanguageCode = LanguageCode {
    name: "Thawa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xtv",
    individual_languages: &[
    ],
};


pub const XTW: LanguageCode = LanguageCode {
    name: "Tawandê",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xtw",
    individual_languages: &[
    ],
};


pub const XTY: LanguageCode = LanguageCode {
    name: "Yoloxochitl Mixtec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xty",
    individual_languages: &[
    ],
};


pub const XUA: LanguageCode = LanguageCode {
    name: "Alu Kurumba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xua",
    individual_languages: &[
    ],
};


pub const XUB: LanguageCode = LanguageCode {
    name: "Betta Kurumba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xub",
    individual_languages: &[
    ],
};


pub const XUD: LanguageCode = LanguageCode {
    name: "Umiida",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xud",
    individual_languages: &[
    ],
};


pub const XUG: LanguageCode = LanguageCode {
    name: "Kunigami",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xug",
    individual_languages: &[
    ],
};


pub const XUJ: LanguageCode = LanguageCode {
    name: "Jennu Kurumba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xuj",
    individual_languages: &[
    ],
};


pub const XUL: LanguageCode = LanguageCode {
    name: "Ngunawal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xul",
    individual_languages: &[
    ],
};


pub const XUM: LanguageCode = LanguageCode {
    name: "Umbrian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xum",
    individual_languages: &[
    ],
};


pub const XUN: LanguageCode = LanguageCode {
    name: "Unggaranggu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xun",
    individual_languages: &[
    ],
};


pub const XUO: LanguageCode = LanguageCode {
    name: "Kuo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xuo",
    individual_languages: &[
    ],
};


pub const XUP: LanguageCode = LanguageCode {
    name: "Upper Umpqua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xup",
    individual_languages: &[
    ],
};


pub const XUR: LanguageCode = LanguageCode {
    name: "Urartian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xur",
    individual_languages: &[
    ],
};


pub const XUT: LanguageCode = LanguageCode {
    name: "Kuthant",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xut",
    individual_languages: &[
    ],
};


pub const XUU: LanguageCode = LanguageCode {
    name: "Kxoe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xuu",
    individual_languages: &[
    ],
};


pub const XVE: LanguageCode = LanguageCode {
    name: "Venetic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xve",
    individual_languages: &[
    ],
};


pub const XVI: LanguageCode = LanguageCode {
    name: "Kamviri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xvi",
    individual_languages: &[
    ],
};


pub const XVN: LanguageCode = LanguageCode {
    name: "Vandalic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xvn",
    individual_languages: &[
    ],
};


pub const XVO: LanguageCode = LanguageCode {
    name: "Volscian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xvo",
    individual_languages: &[
    ],
};


pub const XVS: LanguageCode = LanguageCode {
    name: "Vestinian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xvs",
    individual_languages: &[
    ],
};


pub const XWA: LanguageCode = LanguageCode {
    name: "Kwaza",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xwa",
    individual_languages: &[
    ],
};


pub const XWC: LanguageCode = LanguageCode {
    name: "Woccon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xwc",
    individual_languages: &[
    ],
};


pub const XWD: LanguageCode = LanguageCode {
    name: "Wadi Wadi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xwd",
    individual_languages: &[
    ],
};


pub const XWE: LanguageCode = LanguageCode {
    name: "Xwela Gbe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xwe",
    individual_languages: &[
    ],
};


pub const XWG: LanguageCode = LanguageCode {
    name: "Kwegu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xwg",
    individual_languages: &[
    ],
};


pub const XWJ: LanguageCode = LanguageCode {
    name: "Wajuk",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xwj",
    individual_languages: &[
    ],
};


pub const XWK: LanguageCode = LanguageCode {
    name: "Wangkumara",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xwk",
    individual_languages: &[
    ],
};


pub const XWL: LanguageCode = LanguageCode {
    name: "Western Xwla Gbe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xwl",
    individual_languages: &[
    ],
};


pub const XWO: LanguageCode = LanguageCode {
    name: "Written Oirat",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xwo",
    individual_languages: &[
    ],
};


pub const XWR: LanguageCode = LanguageCode {
    name: "Kwerba Mamberamo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xwr",
    individual_languages: &[
    ],
};


pub const XWT: LanguageCode = LanguageCode {
    name: "Wotjobaluk",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xwt",
    individual_languages: &[
    ],
};


pub const XWW: LanguageCode = LanguageCode {
    name: "Wemba Wemba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xww",
    individual_languages: &[
    ],
};


pub const XXB: LanguageCode = LanguageCode {
    name: "Boro (Ghana)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xxb",
    individual_languages: &[
    ],
};


pub const XXK: LanguageCode = LanguageCode {
    name: "Ke'o",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xxk",
    individual_languages: &[
    ],
};


pub const XXM: LanguageCode = LanguageCode {
    name: "Minkin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xxm",
    individual_languages: &[
    ],
};


pub const XXR: LanguageCode = LanguageCode {
    name: "Koropó",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xxr",
    individual_languages: &[
    ],
};


pub const XXT: LanguageCode = LanguageCode {
    name: "Tambora",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xxt",
    individual_languages: &[
    ],
};


pub const XYA: LanguageCode = LanguageCode {
    name: "Yaygir",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xya",
    individual_languages: &[
    ],
};


pub const XYB: LanguageCode = LanguageCode {
    name: "Yandjibara",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xyb",
    individual_languages: &[
    ],
};


pub const XYJ: LanguageCode = LanguageCode {
    name: "Mayi-Yapi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xyj",
    individual_languages: &[
    ],
};


pub const XYK: LanguageCode = LanguageCode {
    name: "Mayi-Kulan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xyk",
    individual_languages: &[
    ],
};


pub const XYL: LanguageCode = LanguageCode {
    name: "Yalakalore",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xyl",
    individual_languages: &[
    ],
};


pub const XYT: LanguageCode = LanguageCode {
    name: "Mayi-Thakurti",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xyt",
    individual_languages: &[
    ],
};


pub const XYY: LanguageCode = LanguageCode {
    name: "Yorta Yorta",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xyy",
    individual_languages: &[
    ],
};


pub const XZH: LanguageCode = LanguageCode {
    name: "Zhang-Zhung",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xzh",
    individual_languages: &[
    ],
};


pub const XZM: LanguageCode = LanguageCode {
    name: "Zemgalian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xzm",
    individual_languages: &[
    ],
};


pub const XZP: LanguageCode = LanguageCode {
    name: "Ancient Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "xzp",
    individual_languages: &[
    ],
};


pub const YAA: LanguageCode = LanguageCode {
    name: "Yaminahua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yaa",
    individual_languages: &[
    ],
};


pub const YAB: LanguageCode = LanguageCode {
    name: "Yuhup",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yab",
    individual_languages: &[
    ],
};


pub const YAC: LanguageCode = LanguageCode {
    name: "Pass Valley Yali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yac",
    individual_languages: &[
    ],
};


pub const YAD: LanguageCode = LanguageCode {
    name: "Yagua",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yad",
    individual_languages: &[
    ],
};


pub const YAE: LanguageCode = LanguageCode {
    name: "Pumé",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yae",
    individual_languages: &[
    ],
};


pub const YAF: LanguageCode = LanguageCode {
    name: "Yaka (Democratic Republic of Congo)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yaf",
    individual_languages: &[
    ],
};


pub const YAG: LanguageCode = LanguageCode {
    name: "Yámana",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yag",
    individual_languages: &[
    ],
};


pub const YAH: LanguageCode = LanguageCode {
    name: "Yazgulyam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yah",
    individual_languages: &[
    ],
};


pub const YAI: LanguageCode = LanguageCode {
    name: "Yagnobi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yai",
    individual_languages: &[
    ],
};


pub const YAJ: LanguageCode = LanguageCode {
    name: "Banda-Yangere",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yaj",
    individual_languages: &[
    ],
};


pub const YAK: LanguageCode = LanguageCode {
    name: "Yakama",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yak",
    individual_languages: &[
    ],
};


pub const YAL: LanguageCode = LanguageCode {
    name: "Yalunka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yal",
    individual_languages: &[
    ],
};


pub const YAM: LanguageCode = LanguageCode {
    name: "Yamba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yam",
    individual_languages: &[
    ],
};


pub const YAN: LanguageCode = LanguageCode {
    name: "Mayangna",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yan",
    individual_languages: &[
    ],
};


pub const YAQ: LanguageCode = LanguageCode {
    name: "Yaqui",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yaq",
    individual_languages: &[
    ],
};


pub const YAR: LanguageCode = LanguageCode {
    name: "Yabarana",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yar",
    individual_languages: &[
    ],
};


pub const YAS: LanguageCode = LanguageCode {
    name: "Nugunu (Cameroon)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yas",
    individual_languages: &[
    ],
};


pub const YAT: LanguageCode = LanguageCode {
    name: "Yambeta",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yat",
    individual_languages: &[
    ],
};


pub const YAU: LanguageCode = LanguageCode {
    name: "Yuwana",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yau",
    individual_languages: &[
    ],
};


pub const YAV: LanguageCode = LanguageCode {
    name: "Yangben",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yav",
    individual_languages: &[
    ],
};


pub const YAW: LanguageCode = LanguageCode {
    name: "Yawalapití",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yaw",
    individual_languages: &[
    ],
};


pub const YAX: LanguageCode = LanguageCode {
    name: "Yauma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yax",
    individual_languages: &[
    ],
};


pub const YAY: LanguageCode = LanguageCode {
    name: "Agwagwune",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yay",
    individual_languages: &[
    ],
};


pub const YAZ: LanguageCode = LanguageCode {
    name: "Lokaa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yaz",
    individual_languages: &[
    ],
};


pub const YBA: LanguageCode = LanguageCode {
    name: "Yala",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yba",
    individual_languages: &[
    ],
};


pub const YBB: LanguageCode = LanguageCode {
    name: "Yemba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ybb",
    individual_languages: &[
    ],
};


pub const YBE: LanguageCode = LanguageCode {
    name: "West Yugur",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ybe",
    individual_languages: &[
    ],
};


pub const YBH: LanguageCode = LanguageCode {
    name: "Yakha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ybh",
    individual_languages: &[
    ],
};


pub const YBI: LanguageCode = LanguageCode {
    name: "Yamphu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ybi",
    individual_languages: &[
    ],
};


pub const YBJ: LanguageCode = LanguageCode {
    name: "Hasha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ybj",
    individual_languages: &[
    ],
};


pub const YBK: LanguageCode = LanguageCode {
    name: "Bokha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ybk",
    individual_languages: &[
    ],
};


pub const YBL: LanguageCode = LanguageCode {
    name: "Yukuben",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ybl",
    individual_languages: &[
    ],
};


pub const YBM: LanguageCode = LanguageCode {
    name: "Yaben",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ybm",
    individual_languages: &[
    ],
};


pub const YBN: LanguageCode = LanguageCode {
    name: "Yabaâna",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ybn",
    individual_languages: &[
    ],
};


pub const YBO: LanguageCode = LanguageCode {
    name: "Yabong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ybo",
    individual_languages: &[
    ],
};


pub const YBX: LanguageCode = LanguageCode {
    name: "Yawiyo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ybx",
    individual_languages: &[
    ],
};


pub const YBY: LanguageCode = LanguageCode {
    name: "Yaweyuha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yby",
    individual_languages: &[
    ],
};


pub const YCH: LanguageCode = LanguageCode {
    name: "Chesu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ych",
    individual_languages: &[
    ],
};


pub const YCL: LanguageCode = LanguageCode {
    name: "Lolopo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ycl",
    individual_languages: &[
    ],
};


pub const YCN: LanguageCode = LanguageCode {
    name: "Yucuna",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ycn",
    individual_languages: &[
    ],
};


pub const YCP: LanguageCode = LanguageCode {
    name: "Chepya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ycp",
    individual_languages: &[
    ],
};


pub const YDA: LanguageCode = LanguageCode {
    name: "Yanda",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yda",
    individual_languages: &[
    ],
};


pub const YDD: LanguageCode = LanguageCode {
    name: "Eastern Yiddish",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ydd",
    individual_languages: &[
    ],
};


pub const YDE: LanguageCode = LanguageCode {
    name: "Yangum Dey",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yde",
    individual_languages: &[
    ],
};


pub const YDG: LanguageCode = LanguageCode {
    name: "Yidgha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ydg",
    individual_languages: &[
    ],
};


pub const YDK: LanguageCode = LanguageCode {
    name: "Yoidik",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ydk",
    individual_languages: &[
    ],
};


pub const YEA: LanguageCode = LanguageCode {
    name: "Ravula",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yea",
    individual_languages: &[
    ],
};


pub const YEC: LanguageCode = LanguageCode {
    name: "Yeniche",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yec",
    individual_languages: &[
    ],
};


pub const YEE: LanguageCode = LanguageCode {
    name: "Yimas",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yee",
    individual_languages: &[
    ],
};


pub const YEI: LanguageCode = LanguageCode {
    name: "Yeni",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yei",
    individual_languages: &[
    ],
};


pub const YEJ: LanguageCode = LanguageCode {
    name: "Yevanic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yej",
    individual_languages: &[
    ],
};


pub const YEL: LanguageCode = LanguageCode {
    name: "Yela",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yel",
    individual_languages: &[
    ],
};


pub const YER: LanguageCode = LanguageCode {
    name: "Tarok",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yer",
    individual_languages: &[
    ],
};


pub const YES: LanguageCode = LanguageCode {
    name: "Nyankpa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yes",
    individual_languages: &[
    ],
};


pub const YET: LanguageCode = LanguageCode {
    name: "Yetfa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yet",
    individual_languages: &[
    ],
};


pub const YEU: LanguageCode = LanguageCode {
    name: "Yerukula",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yeu",
    individual_languages: &[
    ],
};


pub const YEV: LanguageCode = LanguageCode {
    name: "Yapunda",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yev",
    individual_languages: &[
    ],
};


pub const YEY: LanguageCode = LanguageCode {
    name: "Yeyi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yey",
    individual_languages: &[
    ],
};


pub const YGA: LanguageCode = LanguageCode {
    name: "Malyangapa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yga",
    individual_languages: &[
    ],
};


pub const YGI: LanguageCode = LanguageCode {
    name: "Yiningayi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ygi",
    individual_languages: &[
    ],
};


pub const YGL: LanguageCode = LanguageCode {
    name: "Yangum Gel",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ygl",
    individual_languages: &[
    ],
};


pub const YGM: LanguageCode = LanguageCode {
    name: "Yagomi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ygm",
    individual_languages: &[
    ],
};


pub const YGP: LanguageCode = LanguageCode {
    name: "Gepo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ygp",
    individual_languages: &[
    ],
};


pub const YGR: LanguageCode = LanguageCode {
    name: "Yagaria",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ygr",
    individual_languages: &[
    ],
};


pub const YGS: LanguageCode = LanguageCode {
    name: "Yolŋu Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ygs",
    individual_languages: &[
    ],
};


pub const YGU: LanguageCode = LanguageCode {
    name: "Yugul",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ygu",
    individual_languages: &[
    ],
};


pub const YGW: LanguageCode = LanguageCode {
    name: "Yagwoia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ygw",
    individual_languages: &[
    ],
};


pub const YHA: LanguageCode = LanguageCode {
    name: "Baha Buyang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yha",
    individual_languages: &[
    ],
};


pub const YHD: LanguageCode = LanguageCode {
    name: "Judeo-Iraqi Arabic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yhd",
    individual_languages: &[
    ],
};


pub const YHL: LanguageCode = LanguageCode {
    name: "Hlepho Phowa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yhl",
    individual_languages: &[
    ],
};


pub const YHS: LanguageCode = LanguageCode {
    name: "Yan-nhaŋu Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yhs",
    individual_languages: &[
    ],
};


pub const YIA: LanguageCode = LanguageCode {
    name: "Yinggarda",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yia",
    individual_languages: &[
    ],
};


pub const YIF: LanguageCode = LanguageCode {
    name: "Ache",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yif",
    individual_languages: &[
    ],
};


pub const YIG: LanguageCode = LanguageCode {
    name: "Wusa Nasu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yig",
    individual_languages: &[
    ],
};


pub const YIH: LanguageCode = LanguageCode {
    name: "Western Yiddish",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yih",
    individual_languages: &[
    ],
};


pub const YII: LanguageCode = LanguageCode {
    name: "Yidiny",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yii",
    individual_languages: &[
    ],
};


pub const YIJ: LanguageCode = LanguageCode {
    name: "Yindjibarndi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yij",
    individual_languages: &[
    ],
};


pub const YIK: LanguageCode = LanguageCode {
    name: "Dongshanba Lalo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yik",
    individual_languages: &[
    ],
};


pub const YIL: LanguageCode = LanguageCode {
    name: "Yindjilandji",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yil",
    individual_languages: &[
    ],
};


pub const YIM: LanguageCode = LanguageCode {
    name: "Yimchungru Naga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yim",
    individual_languages: &[
    ],
};


pub const YIN: LanguageCode = LanguageCode {
    name: "Riang Lai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yin",
    individual_languages: &[
    ],
};


pub const YIP: LanguageCode = LanguageCode {
    name: "Pholo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yip",
    individual_languages: &[
    ],
};


pub const YIQ: LanguageCode = LanguageCode {
    name: "Miqie",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yiq",
    individual_languages: &[
    ],
};


pub const YIR: LanguageCode = LanguageCode {
    name: "North Awyu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yir",
    individual_languages: &[
    ],
};


pub const YIS: LanguageCode = LanguageCode {
    name: "Yis",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yis",
    individual_languages: &[
    ],
};


pub const YIT: LanguageCode = LanguageCode {
    name: "Eastern Lalu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yit",
    individual_languages: &[
    ],
};


pub const YIU: LanguageCode = LanguageCode {
    name: "Awu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yiu",
    individual_languages: &[
    ],
};


pub const YIV: LanguageCode = LanguageCode {
    name: "Northern Nisu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yiv",
    individual_languages: &[
    ],
};


pub const YIX: LanguageCode = LanguageCode {
    name: "Axi Yi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yix",
    individual_languages: &[
    ],
};


pub const YIZ: LanguageCode = LanguageCode {
    name: "Azhe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yiz",
    individual_languages: &[
    ],
};


pub const YKA: LanguageCode = LanguageCode {
    name: "Yakan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yka",
    individual_languages: &[
    ],
};


pub const YKG: LanguageCode = LanguageCode {
    name: "Northern Yukaghir",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ykg",
    individual_languages: &[
    ],
};


pub const YKI: LanguageCode = LanguageCode {
    name: "Yoke",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yki",
    individual_languages: &[
    ],
};


pub const YKK: LanguageCode = LanguageCode {
    name: "Yakaikeke",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ykk",
    individual_languages: &[
    ],
};


pub const YKL: LanguageCode = LanguageCode {
    name: "Khlula",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ykl",
    individual_languages: &[
    ],
};


pub const YKM: LanguageCode = LanguageCode {
    name: "Kap",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ykm",
    individual_languages: &[
    ],
};


pub const YKN: LanguageCode = LanguageCode {
    name: "Kua-nsi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ykn",
    individual_languages: &[
    ],
};


pub const YKO: LanguageCode = LanguageCode {
    name: "Yasa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yko",
    individual_languages: &[
    ],
};


pub const YKR: LanguageCode = LanguageCode {
    name: "Yekora",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ykr",
    individual_languages: &[
    ],
};


pub const YKT: LanguageCode = LanguageCode {
    name: "Kathu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ykt",
    individual_languages: &[
    ],
};


pub const YKU: LanguageCode = LanguageCode {
    name: "Kuamasi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yku",
    individual_languages: &[
    ],
};


pub const YKY: LanguageCode = LanguageCode {
    name: "Yakoma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yky",
    individual_languages: &[
    ],
};


pub const YLA: LanguageCode = LanguageCode {
    name: "Yaul",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yla",
    individual_languages: &[
    ],
};


pub const YLB: LanguageCode = LanguageCode {
    name: "Yaleba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ylb",
    individual_languages: &[
    ],
};


pub const YLE: LanguageCode = LanguageCode {
    name: "Yele",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yle",
    individual_languages: &[
    ],
};


pub const YLG: LanguageCode = LanguageCode {
    name: "Yelogu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ylg",
    individual_languages: &[
    ],
};


pub const YLI: LanguageCode = LanguageCode {
    name: "Angguruk Yali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yli",
    individual_languages: &[
    ],
};


pub const YLL: LanguageCode = LanguageCode {
    name: "Yil",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yll",
    individual_languages: &[
    ],
};


pub const YLM: LanguageCode = LanguageCode {
    name: "Limi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ylm",
    individual_languages: &[
    ],
};


pub const YLN: LanguageCode = LanguageCode {
    name: "Langnian Buyang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yln",
    individual_languages: &[
    ],
};


pub const YLO: LanguageCode = LanguageCode {
    name: "Naluo Yi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ylo",
    individual_languages: &[
    ],
};


pub const YLR: LanguageCode = LanguageCode {
    name: "Yalarnnga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ylr",
    individual_languages: &[
    ],
};


pub const YLU: LanguageCode = LanguageCode {
    name: "Aribwaung",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ylu",
    individual_languages: &[
    ],
};


pub const YLY: LanguageCode = LanguageCode {
    name: "Nyâlayu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yly",
    individual_languages: &[
    ],
};


pub const YMB: LanguageCode = LanguageCode {
    name: "Yambes",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ymb",
    individual_languages: &[
    ],
};


pub const YMC: LanguageCode = LanguageCode {
    name: "Southern Muji",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ymc",
    individual_languages: &[
    ],
};


pub const YMD: LanguageCode = LanguageCode {
    name: "Muda",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ymd",
    individual_languages: &[
    ],
};


pub const YME: LanguageCode = LanguageCode {
    name: "Yameo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yme",
    individual_languages: &[
    ],
};


pub const YMG: LanguageCode = LanguageCode {
    name: "Yamongeri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ymg",
    individual_languages: &[
    ],
};


pub const YMH: LanguageCode = LanguageCode {
    name: "Mili",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ymh",
    individual_languages: &[
    ],
};


pub const YMI: LanguageCode = LanguageCode {
    name: "Moji",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ymi",
    individual_languages: &[
    ],
};


pub const YMK: LanguageCode = LanguageCode {
    name: "Makwe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ymk",
    individual_languages: &[
    ],
};


pub const YML: LanguageCode = LanguageCode {
    name: "Iamalele",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yml",
    individual_languages: &[
    ],
};


pub const YMM: LanguageCode = LanguageCode {
    name: "Maay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ymm",
    individual_languages: &[
    ],
};


pub const YMN: LanguageCode = LanguageCode {
    name: "Yamna",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ymn",
    individual_languages: &[
    ],
};


pub const YMO: LanguageCode = LanguageCode {
    name: "Yangum Mon",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ymo",
    individual_languages: &[
    ],
};


pub const YMP: LanguageCode = LanguageCode {
    name: "Yamap",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ymp",
    individual_languages: &[
    ],
};


pub const YMQ: LanguageCode = LanguageCode {
    name: "Qila Muji",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ymq",
    individual_languages: &[
    ],
};


pub const YMR: LanguageCode = LanguageCode {
    name: "Malasar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ymr",
    individual_languages: &[
    ],
};


pub const YMS: LanguageCode = LanguageCode {
    name: "Mysian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yms",
    individual_languages: &[
    ],
};


pub const YMX: LanguageCode = LanguageCode {
    name: "Northern Muji",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ymx",
    individual_languages: &[
    ],
};


pub const YMZ: LanguageCode = LanguageCode {
    name: "Muzi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ymz",
    individual_languages: &[
    ],
};


pub const YNA: LanguageCode = LanguageCode {
    name: "Aluo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yna",
    individual_languages: &[
    ],
};


pub const YND: LanguageCode = LanguageCode {
    name: "Yandruwandha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ynd",
    individual_languages: &[
    ],
};


pub const YNE: LanguageCode = LanguageCode {
    name: "Lang'e",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yne",
    individual_languages: &[
    ],
};


pub const YNG: LanguageCode = LanguageCode {
    name: "Yango",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yng",
    individual_languages: &[
    ],
};


pub const YNK: LanguageCode = LanguageCode {
    name: "Naukan Yupik",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ynk",
    individual_languages: &[
    ],
};


pub const YNL: LanguageCode = LanguageCode {
    name: "Yangulam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ynl",
    individual_languages: &[
    ],
};


pub const YNN: LanguageCode = LanguageCode {
    name: "Yana",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ynn",
    individual_languages: &[
    ],
};


pub const YNO: LanguageCode = LanguageCode {
    name: "Yong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yno",
    individual_languages: &[
    ],
};


pub const YNQ: LanguageCode = LanguageCode {
    name: "Yendang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ynq",
    individual_languages: &[
    ],
};


pub const YNS: LanguageCode = LanguageCode {
    name: "Yansi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yns",
    individual_languages: &[
    ],
};


pub const YNU: LanguageCode = LanguageCode {
    name: "Yahuna",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ynu",
    individual_languages: &[
    ],
};


pub const YOB: LanguageCode = LanguageCode {
    name: "Yoba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yob",
    individual_languages: &[
    ],
};


pub const YOG: LanguageCode = LanguageCode {
    name: "Yogad",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yog",
    individual_languages: &[
    ],
};


pub const YOI: LanguageCode = LanguageCode {
    name: "Yonaguni",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yoi",
    individual_languages: &[
    ],
};


pub const YOK: LanguageCode = LanguageCode {
    name: "Yokuts",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yok",
    individual_languages: &[
    ],
};


pub const YOL: LanguageCode = LanguageCode {
    name: "Yola",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yol",
    individual_languages: &[
    ],
};


pub const YOM: LanguageCode = LanguageCode {
    name: "Yombe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yom",
    individual_languages: &[
    ],
};


pub const YON: LanguageCode = LanguageCode {
    name: "Yongkom",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yon",
    individual_languages: &[
    ],
};


pub const YOT: LanguageCode = LanguageCode {
    name: "Yotti",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yot",
    individual_languages: &[
    ],
};


pub const YOX: LanguageCode = LanguageCode {
    name: "Yoron",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yox",
    individual_languages: &[
    ],
};


pub const YOY: LanguageCode = LanguageCode {
    name: "Yoy",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yoy",
    individual_languages: &[
    ],
};


pub const YPA: LanguageCode = LanguageCode {
    name: "Phala",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ypa",
    individual_languages: &[
    ],
};


pub const YPB: LanguageCode = LanguageCode {
    name: "Labo Phowa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ypb",
    individual_languages: &[
    ],
};


pub const YPG: LanguageCode = LanguageCode {
    name: "Phola",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ypg",
    individual_languages: &[
    ],
};


pub const YPH: LanguageCode = LanguageCode {
    name: "Phupha",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yph",
    individual_languages: &[
    ],
};


pub const YPM: LanguageCode = LanguageCode {
    name: "Phuma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ypm",
    individual_languages: &[
    ],
};


pub const YPN: LanguageCode = LanguageCode {
    name: "Ani Phowa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ypn",
    individual_languages: &[
    ],
};


pub const YPO: LanguageCode = LanguageCode {
    name: "Alo Phola",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ypo",
    individual_languages: &[
    ],
};


pub const YPP: LanguageCode = LanguageCode {
    name: "Phupa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ypp",
    individual_languages: &[
    ],
};


pub const YPZ: LanguageCode = LanguageCode {
    name: "Phuza",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ypz",
    individual_languages: &[
    ],
};


pub const YRA: LanguageCode = LanguageCode {
    name: "Yerakai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yra",
    individual_languages: &[
    ],
};


pub const YRB: LanguageCode = LanguageCode {
    name: "Yareba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yrb",
    individual_languages: &[
    ],
};


pub const YRE: LanguageCode = LanguageCode {
    name: "Yaouré",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yre",
    individual_languages: &[
    ],
};


pub const YRK: LanguageCode = LanguageCode {
    name: "Nenets",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yrk",
    individual_languages: &[
    ],
};


pub const YRL: LanguageCode = LanguageCode {
    name: "Nhengatu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yrl",
    individual_languages: &[
    ],
};


pub const YRM: LanguageCode = LanguageCode {
    name: "Yirrk-Mel",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yrm",
    individual_languages: &[
    ],
};


pub const YRN: LanguageCode = LanguageCode {
    name: "Yerong",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yrn",
    individual_languages: &[
    ],
};


pub const YRO: LanguageCode = LanguageCode {
    name: "Yaroamë",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yro",
    individual_languages: &[
    ],
};


pub const YRS: LanguageCode = LanguageCode {
    name: "Yarsun",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yrs",
    individual_languages: &[
    ],
};


pub const YRW: LanguageCode = LanguageCode {
    name: "Yarawata",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yrw",
    individual_languages: &[
    ],
};


pub const YRY: LanguageCode = LanguageCode {
    name: "Yarluyandi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yry",
    individual_languages: &[
    ],
};


pub const YSC: LanguageCode = LanguageCode {
    name: "Yassic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ysc",
    individual_languages: &[
    ],
};


pub const YSD: LanguageCode = LanguageCode {
    name: "Samatao",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ysd",
    individual_languages: &[
    ],
};


pub const YSG: LanguageCode = LanguageCode {
    name: "Sonaga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ysg",
    individual_languages: &[
    ],
};


pub const YSL: LanguageCode = LanguageCode {
    name: "Yugoslavian Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ysl",
    individual_languages: &[
    ],
};


pub const YSM: LanguageCode = LanguageCode {
    name: "Myanmar Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ysm",
    individual_languages: &[
    ],
};


pub const YSN: LanguageCode = LanguageCode {
    name: "Sani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ysn",
    individual_languages: &[
    ],
};


pub const YSO: LanguageCode = LanguageCode {
    name: "Nisi (China)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yso",
    individual_languages: &[
    ],
};


pub const YSP: LanguageCode = LanguageCode {
    name: "Southern Lolopo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ysp",
    individual_languages: &[
    ],
};


pub const YSR: LanguageCode = LanguageCode {
    name: "Sirenik Yupik",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ysr",
    individual_languages: &[
    ],
};


pub const YSS: LanguageCode = LanguageCode {
    name: "Yessan-Mayo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yss",
    individual_languages: &[
    ],
};


pub const YSY: LanguageCode = LanguageCode {
    name: "Sanie",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ysy",
    individual_languages: &[
    ],
};


pub const YTA: LanguageCode = LanguageCode {
    name: "Talu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yta",
    individual_languages: &[
    ],
};


pub const YTL: LanguageCode = LanguageCode {
    name: "Tanglang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ytl",
    individual_languages: &[
    ],
};


pub const YTP: LanguageCode = LanguageCode {
    name: "Thopho",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ytp",
    individual_languages: &[
    ],
};


pub const YTW: LanguageCode = LanguageCode {
    name: "Yout Wam",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ytw",
    individual_languages: &[
    ],
};


pub const YTY: LanguageCode = LanguageCode {
    name: "Yatay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yty",
    individual_languages: &[
    ],
};


pub const YUA: LanguageCode = LanguageCode {
    name: "Yucateco",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yua",
    individual_languages: &[
    ],
};


pub const YUB: LanguageCode = LanguageCode {
    name: "Yugambal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yub",
    individual_languages: &[
    ],
};


pub const YUC: LanguageCode = LanguageCode {
    name: "Yuchi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yuc",
    individual_languages: &[
    ],
};


pub const YUD: LanguageCode = LanguageCode {
    name: "Judeo-Tripolitanian Arabic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yud",
    individual_languages: &[
    ],
};


pub const YUE: LanguageCode = LanguageCode {
    name: "Yue Chinese",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yue",
    individual_languages: &[
    ],
};


pub const YUF: LanguageCode = LanguageCode {
    name: "Havasupai-Walapai-Yavapai",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yuf",
    individual_languages: &[
    ],
};


pub const YUG: LanguageCode = LanguageCode {
    name: "Yug",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yug",
    individual_languages: &[
    ],
};


pub const YUI: LanguageCode = LanguageCode {
    name: "Yurutí",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yui",
    individual_languages: &[
    ],
};


pub const YUJ: LanguageCode = LanguageCode {
    name: "Karkar-Yuri",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yuj",
    individual_languages: &[
    ],
};


pub const YUK: LanguageCode = LanguageCode {
    name: "Yuki",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yuk",
    individual_languages: &[
    ],
};


pub const YUL: LanguageCode = LanguageCode {
    name: "Yulu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yul",
    individual_languages: &[
    ],
};


pub const YUM: LanguageCode = LanguageCode {
    name: "Quechan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yum",
    individual_languages: &[
    ],
};


pub const YUN: LanguageCode = LanguageCode {
    name: "Bena (Nigeria)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yun",
    individual_languages: &[
    ],
};


pub const YUP: LanguageCode = LanguageCode {
    name: "Yukpa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yup",
    individual_languages: &[
    ],
};


pub const YUQ: LanguageCode = LanguageCode {
    name: "Yuqui",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yuq",
    individual_languages: &[
    ],
};


pub const YUR: LanguageCode = LanguageCode {
    name: "Yurok",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yur",
    individual_languages: &[
    ],
};


pub const YUT: LanguageCode = LanguageCode {
    name: "Yopno",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yut",
    individual_languages: &[
    ],
};


pub const YUW: LanguageCode = LanguageCode {
    name: "Yau (Morobe Province)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yuw",
    individual_languages: &[
    ],
};


pub const YUX: LanguageCode = LanguageCode {
    name: "Southern Yukaghir",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yux",
    individual_languages: &[
    ],
};


pub const YUY: LanguageCode = LanguageCode {
    name: "East Yugur",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yuy",
    individual_languages: &[
    ],
};


pub const YUZ: LanguageCode = LanguageCode {
    name: "Yuracare",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yuz",
    individual_languages: &[
    ],
};


pub const YVA: LanguageCode = LanguageCode {
    name: "Yawa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yva",
    individual_languages: &[
    ],
};


pub const YVT: LanguageCode = LanguageCode {
    name: "Yavitero",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yvt",
    individual_languages: &[
    ],
};


pub const YWA: LanguageCode = LanguageCode {
    name: "Kalou",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ywa",
    individual_languages: &[
    ],
};


pub const YWG: LanguageCode = LanguageCode {
    name: "Yinhawangka",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ywg",
    individual_languages: &[
    ],
};


pub const YWL: LanguageCode = LanguageCode {
    name: "Western Lalu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ywl",
    individual_languages: &[
    ],
};


pub const YWN: LanguageCode = LanguageCode {
    name: "Yawanawa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ywn",
    individual_languages: &[
    ],
};


pub const YWQ: LanguageCode = LanguageCode {
    name: "Wuding-Luquan Yi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ywq",
    individual_languages: &[
    ],
};


pub const YWR: LanguageCode = LanguageCode {
    name: "Yawuru",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ywr",
    individual_languages: &[
    ],
};


pub const YWT: LanguageCode = LanguageCode {
    name: "Xishanba Lalo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ywt",
    individual_languages: &[
    ],
};


pub const YWU: LanguageCode = LanguageCode {
    name: "Wumeng Nasu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ywu",
    individual_languages: &[
    ],
};


pub const YWW: LanguageCode = LanguageCode {
    name: "Yawarawarga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yww",
    individual_languages: &[
    ],
};


pub const YXA: LanguageCode = LanguageCode {
    name: "Mayawali",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yxa",
    individual_languages: &[
    ],
};


pub const YXG: LanguageCode = LanguageCode {
    name: "Yagara",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yxg",
    individual_languages: &[
    ],
};


pub const YXL: LanguageCode = LanguageCode {
    name: "Yardliyawarra",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yxl",
    individual_languages: &[
    ],
};


pub const YXM: LanguageCode = LanguageCode {
    name: "Yinwum",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yxm",
    individual_languages: &[
    ],
};


pub const YXU: LanguageCode = LanguageCode {
    name: "Yuyu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yxu",
    individual_languages: &[
    ],
};


pub const YXY: LanguageCode = LanguageCode {
    name: "Yabula Yabula",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yxy",
    individual_languages: &[
    ],
};


pub const YYR: LanguageCode = LanguageCode {
    name: "Yir Yoront",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yyr",
    individual_languages: &[
    ],
};


pub const YYU: LanguageCode = LanguageCode {
    name: "Yau (Sandaun Province)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yyu",
    individual_languages: &[
    ],
};


pub const YYZ: LanguageCode = LanguageCode {
    name: "Ayizi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yyz",
    individual_languages: &[
    ],
};


pub const YZG: LanguageCode = LanguageCode {
    name: "E'ma Buyang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yzg",
    individual_languages: &[
    ],
};


pub const YZK: LanguageCode = LanguageCode {
    name: "Zokhuo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "yzk",
    individual_languages: &[
    ],
};


pub const ZAA: LanguageCode = LanguageCode {
    name: "Sierra de Juárez Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zaa",
    individual_languages: &[
    ],
};


pub const ZAB: LanguageCode = LanguageCode {
    name: "Western Tlacolula Valley Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zab",
    individual_languages: &[
    ],
};


pub const ZAC: LanguageCode = LanguageCode {
    name: "Ocotlán Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zac",
    individual_languages: &[
    ],
};


pub const ZAD: LanguageCode = LanguageCode {
    name: "Cajonos Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zad",
    individual_languages: &[
    ],
};


pub const ZAE: LanguageCode = LanguageCode {
    name: "Yareni Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zae",
    individual_languages: &[
    ],
};


pub const ZAF: LanguageCode = LanguageCode {
    name: "Ayoquesco Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zaf",
    individual_languages: &[
    ],
};


pub const ZAG: LanguageCode = LanguageCode {
    name: "Zaghawa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zag",
    individual_languages: &[
    ],
};


pub const ZAH: LanguageCode = LanguageCode {
    name: "Zangwal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zah",
    individual_languages: &[
    ],
};


pub const ZAI: LanguageCode = LanguageCode {
    name: "Isthmus Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zai",
    individual_languages: &[
    ],
};


pub const ZAJ: LanguageCode = LanguageCode {
    name: "Zaramo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zaj",
    individual_languages: &[
    ],
};


pub const ZAK: LanguageCode = LanguageCode {
    name: "Zanaki",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zak",
    individual_languages: &[
    ],
};


pub const ZAL: LanguageCode = LanguageCode {
    name: "Zauzou",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zal",
    individual_languages: &[
    ],
};


pub const ZAM: LanguageCode = LanguageCode {
    name: "Miahuatlán Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zam",
    individual_languages: &[
    ],
};


pub const ZAO: LanguageCode = LanguageCode {
    name: "Ozolotepec Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zao",
    individual_languages: &[
    ],
};


pub const ZAQ: LanguageCode = LanguageCode {
    name: "Aloápam Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zaq",
    individual_languages: &[
    ],
};


pub const ZAR: LanguageCode = LanguageCode {
    name: "Rincón Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zar",
    individual_languages: &[
    ],
};


pub const ZAS: LanguageCode = LanguageCode {
    name: "Santo Domingo Albarradas Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zas",
    individual_languages: &[
    ],
};


pub const ZAT: LanguageCode = LanguageCode {
    name: "Tabaa Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zat",
    individual_languages: &[
    ],
};


pub const ZAU: LanguageCode = LanguageCode {
    name: "Zangskari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zau",
    individual_languages: &[
    ],
};


pub const ZAV: LanguageCode = LanguageCode {
    name: "Yatzachi Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zav",
    individual_languages: &[
    ],
};


pub const ZAW: LanguageCode = LanguageCode {
    name: "Mitla Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zaw",
    individual_languages: &[
    ],
};


pub const ZAX: LanguageCode = LanguageCode {
    name: "Xadani Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zax",
    individual_languages: &[
    ],
};


pub const ZAY: LanguageCode = LanguageCode {
    name: "Zayse-Zergulla",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zay",
    individual_languages: &[
    ],
};


pub const ZAZ: LanguageCode = LanguageCode {
    name: "Zari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zaz",
    individual_languages: &[
    ],
};


pub const ZBA: LanguageCode = LanguageCode {
    name: "Balaibalan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zba",
    individual_languages: &[
    ],
};


pub const ZBC: LanguageCode = LanguageCode {
    name: "Central Berawan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zbc",
    individual_languages: &[
    ],
};


pub const ZBE: LanguageCode = LanguageCode {
    name: "East Berawan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zbe",
    individual_languages: &[
    ],
};


pub const ZBT: LanguageCode = LanguageCode {
    name: "Batui",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zbt",
    individual_languages: &[
    ],
};


pub const ZBU: LanguageCode = LanguageCode {
    name: "Bu (Bauchi State)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zbu",
    individual_languages: &[
    ],
};


pub const ZBW: LanguageCode = LanguageCode {
    name: "West Berawan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zbw",
    individual_languages: &[
    ],
};


pub const ZCA: LanguageCode = LanguageCode {
    name: "Coatecas Altas Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zca",
    individual_languages: &[
    ],
};


pub const ZCD: LanguageCode = LanguageCode {
    name: "Las Delicias Zapotec",
    code: "",
    code_2t: "zcd",
    code_2b: "zcd",
    code_3: "zcd",
    individual_languages: &[
    ],
};


pub const ZCH: LanguageCode = LanguageCode {
    name: "Central Hongshuihe Zhuang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zch",
    individual_languages: &[
    ],
};


pub const ZDJ: LanguageCode = LanguageCode {
    name: "Ngazidja Comorian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zdj",
    individual_languages: &[
    ],
};


pub const ZEA: LanguageCode = LanguageCode {
    name: "Zeeuws",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zea",
    individual_languages: &[
    ],
};


pub const ZEG: LanguageCode = LanguageCode {
    name: "Zenag",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zeg",
    individual_languages: &[
    ],
};


pub const ZEH: LanguageCode = LanguageCode {
    name: "Eastern Hongshuihe Zhuang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zeh",
    individual_languages: &[
    ],
};


pub const ZGA: LanguageCode = LanguageCode {
    name: "Kinga",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zga",
    individual_languages: &[
    ],
};


pub const ZGB: LanguageCode = LanguageCode {
    name: "Guibei Zhuang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zgb",
    individual_languages: &[
    ],
};


pub const ZGM: LanguageCode = LanguageCode {
    name: "Minz Zhuang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zgm",
    individual_languages: &[
    ],
};


pub const ZGN: LanguageCode = LanguageCode {
    name: "Guibian Zhuang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zgn",
    individual_languages: &[
    ],
};


pub const ZGR: LanguageCode = LanguageCode {
    name: "Magori",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zgr",
    individual_languages: &[
    ],
};


pub const ZHB: LanguageCode = LanguageCode {
    name: "Zhaba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zhb",
    individual_languages: &[
    ],
};


pub const ZHD: LanguageCode = LanguageCode {
    name: "Dai Zhuang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zhd",
    individual_languages: &[
    ],
};


pub const ZHI: LanguageCode = LanguageCode {
    name: "Zhire",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zhi",
    individual_languages: &[
    ],
};


pub const ZHN: LanguageCode = LanguageCode {
    name: "Nong Zhuang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zhn",
    individual_languages: &[
    ],
};


pub const ZHW: LanguageCode = LanguageCode {
    name: "Zhoa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zhw",
    individual_languages: &[
    ],
};


pub const ZIA: LanguageCode = LanguageCode {
    name: "Zia",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zia",
    individual_languages: &[
    ],
};


pub const ZIB: LanguageCode = LanguageCode {
    name: "Zimbabwe Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zib",
    individual_languages: &[
    ],
};


pub const ZIK: LanguageCode = LanguageCode {
    name: "Zimakani",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zik",
    individual_languages: &[
    ],
};


pub const ZIL: LanguageCode = LanguageCode {
    name: "Zialo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zil",
    individual_languages: &[
    ],
};


pub const ZIM: LanguageCode = LanguageCode {
    name: "Mesme",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zim",
    individual_languages: &[
    ],
};


pub const ZIN: LanguageCode = LanguageCode {
    name: "Zinza",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zin",
    individual_languages: &[
    ],
};


pub const ZIW: LanguageCode = LanguageCode {
    name: "Zigula",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ziw",
    individual_languages: &[
    ],
};


pub const ZIZ: LanguageCode = LanguageCode {
    name: "Zizilivakan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ziz",
    individual_languages: &[
    ],
};


pub const ZKA: LanguageCode = LanguageCode {
    name: "Kaimbulawa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zka",
    individual_languages: &[
    ],
};


pub const ZKB: LanguageCode = LanguageCode {
    name: "Koibal",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zkb",
    individual_languages: &[
    ],
};


pub const ZKD: LanguageCode = LanguageCode {
    name: "Kadu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zkd",
    individual_languages: &[
    ],
};


pub const ZKG: LanguageCode = LanguageCode {
    name: "Koguryo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zkg",
    individual_languages: &[
    ],
};


pub const ZKH: LanguageCode = LanguageCode {
    name: "Khorezmian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zkh",
    individual_languages: &[
    ],
};


pub const ZKK: LanguageCode = LanguageCode {
    name: "Karankawa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zkk",
    individual_languages: &[
    ],
};


pub const ZKN: LanguageCode = LanguageCode {
    name: "Kanan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zkn",
    individual_languages: &[
    ],
};


pub const ZKO: LanguageCode = LanguageCode {
    name: "Kott",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zko",
    individual_languages: &[
    ],
};


pub const ZKP: LanguageCode = LanguageCode {
    name: "São Paulo Kaingáng",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zkp",
    individual_languages: &[
    ],
};


pub const ZKR: LanguageCode = LanguageCode {
    name: "Zakhring",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zkr",
    individual_languages: &[
    ],
};


pub const ZKT: LanguageCode = LanguageCode {
    name: "Kitan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zkt",
    individual_languages: &[
    ],
};


pub const ZKU: LanguageCode = LanguageCode {
    name: "Kaurna",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zku",
    individual_languages: &[
    ],
};


pub const ZKV: LanguageCode = LanguageCode {
    name: "Krevinian",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zkv",
    individual_languages: &[
    ],
};


pub const ZKZ: LanguageCode = LanguageCode {
    name: "Khazar",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zkz",
    individual_languages: &[
    ],
};


pub const ZLA: LanguageCode = LanguageCode {
    name: "Zula",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zla",
    individual_languages: &[
    ],
};


pub const ZLJ: LanguageCode = LanguageCode {
    name: "Liujiang Zhuang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zlj",
    individual_languages: &[
    ],
};


pub const ZLM: LanguageCode = LanguageCode {
    name: "Malay (individual language)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zlm",
    individual_languages: &[
    ],
};


pub const ZLN: LanguageCode = LanguageCode {
    name: "Lianshan Zhuang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zln",
    individual_languages: &[
    ],
};


pub const ZLQ: LanguageCode = LanguageCode {
    name: "Liuqian Zhuang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zlq",
    individual_languages: &[
    ],
};


pub const ZMA: LanguageCode = LanguageCode {
    name: "Manda (Australia)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zma",
    individual_languages: &[
    ],
};


pub const ZMB: LanguageCode = LanguageCode {
    name: "Zimba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zmb",
    individual_languages: &[
    ],
};


pub const ZMC: LanguageCode = LanguageCode {
    name: "Margany",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zmc",
    individual_languages: &[
    ],
};


pub const ZMD: LanguageCode = LanguageCode {
    name: "Maridan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zmd",
    individual_languages: &[
    ],
};


pub const ZME: LanguageCode = LanguageCode {
    name: "Mangerr",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zme",
    individual_languages: &[
    ],
};


pub const ZMF: LanguageCode = LanguageCode {
    name: "Mfinu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zmf",
    individual_languages: &[
    ],
};


pub const ZMG: LanguageCode = LanguageCode {
    name: "Marti Ke",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zmg",
    individual_languages: &[
    ],
};


pub const ZMH: LanguageCode = LanguageCode {
    name: "Makolkol",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zmh",
    individual_languages: &[
    ],
};


pub const ZMI: LanguageCode = LanguageCode {
    name: "Negeri Sembilan Malay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zmi",
    individual_languages: &[
    ],
};


pub const ZMJ: LanguageCode = LanguageCode {
    name: "Maridjabin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zmj",
    individual_languages: &[
    ],
};


pub const ZMK: LanguageCode = LanguageCode {
    name: "Mandandanyi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zmk",
    individual_languages: &[
    ],
};


pub const ZML: LanguageCode = LanguageCode {
    name: "Matngala",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zml",
    individual_languages: &[
    ],
};


pub const ZMM: LanguageCode = LanguageCode {
    name: "Marimanindji",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zmm",
    individual_languages: &[
    ],
};


pub const ZMN: LanguageCode = LanguageCode {
    name: "Mbangwe",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zmn",
    individual_languages: &[
    ],
};


pub const ZMO: LanguageCode = LanguageCode {
    name: "Molo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zmo",
    individual_languages: &[
    ],
};


pub const ZMP: LanguageCode = LanguageCode {
    name: "Mpuono",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zmp",
    individual_languages: &[
    ],
};


pub const ZMQ: LanguageCode = LanguageCode {
    name: "Mituku",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zmq",
    individual_languages: &[
    ],
};


pub const ZMR: LanguageCode = LanguageCode {
    name: "Maranunggu",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zmr",
    individual_languages: &[
    ],
};


pub const ZMS: LanguageCode = LanguageCode {
    name: "Mbesa",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zms",
    individual_languages: &[
    ],
};


pub const ZMT: LanguageCode = LanguageCode {
    name: "Maringarr",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zmt",
    individual_languages: &[
    ],
};


pub const ZMU: LanguageCode = LanguageCode {
    name: "Muruwari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zmu",
    individual_languages: &[
    ],
};


pub const ZMV: LanguageCode = LanguageCode {
    name: "Mbariman-Gudhinma",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zmv",
    individual_languages: &[
    ],
};


pub const ZMW: LanguageCode = LanguageCode {
    name: "Mbo (Democratic Republic of Congo)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zmw",
    individual_languages: &[
    ],
};


pub const ZMX: LanguageCode = LanguageCode {
    name: "Bomitaba",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zmx",
    individual_languages: &[
    ],
};


pub const ZMY: LanguageCode = LanguageCode {
    name: "Mariyedi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zmy",
    individual_languages: &[
    ],
};


pub const ZMZ: LanguageCode = LanguageCode {
    name: "Mbandja",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zmz",
    individual_languages: &[
    ],
};


pub const ZNA: LanguageCode = LanguageCode {
    name: "Zan Gula",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zna",
    individual_languages: &[
    ],
};


pub const ZNE: LanguageCode = LanguageCode {
    name: "Zande (individual language)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zne",
    individual_languages: &[
    ],
};


pub const ZNG: LanguageCode = LanguageCode {
    name: "Mang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zng",
    individual_languages: &[
    ],
};


pub const ZNK: LanguageCode = LanguageCode {
    name: "Manangkari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "znk",
    individual_languages: &[
    ],
};


pub const ZNS: LanguageCode = LanguageCode {
    name: "Mangas",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zns",
    individual_languages: &[
    ],
};


pub const ZOC: LanguageCode = LanguageCode {
    name: "Copainalá Zoque",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zoc",
    individual_languages: &[
    ],
};


pub const ZOH: LanguageCode = LanguageCode {
    name: "Chimalapa Zoque",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zoh",
    individual_languages: &[
    ],
};


pub const ZOM: LanguageCode = LanguageCode {
    name: "Zou",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zom",
    individual_languages: &[
    ],
};


pub const ZOO: LanguageCode = LanguageCode {
    name: "Asunción Mixtepec Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zoo",
    individual_languages: &[
    ],
};


pub const ZOQ: LanguageCode = LanguageCode {
    name: "Tabasco Zoque",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zoq",
    individual_languages: &[
    ],
};


pub const ZOR: LanguageCode = LanguageCode {
    name: "Rayón Zoque",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zor",
    individual_languages: &[
    ],
};


pub const ZOS: LanguageCode = LanguageCode {
    name: "Francisco León Zoque",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zos",
    individual_languages: &[
    ],
};


pub const ZPA: LanguageCode = LanguageCode {
    name: "Lachiguiri Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zpa",
    individual_languages: &[
    ],
};


pub const ZPB: LanguageCode = LanguageCode {
    name: "Yautepec Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zpb",
    individual_languages: &[
    ],
};


pub const ZPC: LanguageCode = LanguageCode {
    name: "Choapan Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zpc",
    individual_languages: &[
    ],
};


pub const ZPD: LanguageCode = LanguageCode {
    name: "Southeastern Ixtlán Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zpd",
    individual_languages: &[
    ],
};


pub const ZPE: LanguageCode = LanguageCode {
    name: "Petapa Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zpe",
    individual_languages: &[
    ],
};


pub const ZPF: LanguageCode = LanguageCode {
    name: "San Pedro Quiatoni Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zpf",
    individual_languages: &[
    ],
};


pub const ZPG: LanguageCode = LanguageCode {
    name: "Guevea De Humboldt Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zpg",
    individual_languages: &[
    ],
};


pub const ZPH: LanguageCode = LanguageCode {
    name: "Totomachapan Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zph",
    individual_languages: &[
    ],
};


pub const ZPI: LanguageCode = LanguageCode {
    name: "Santa María Quiegolani Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zpi",
    individual_languages: &[
    ],
};


pub const ZPJ: LanguageCode = LanguageCode {
    name: "Quiavicuzas Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zpj",
    individual_languages: &[
    ],
};


pub const ZPK: LanguageCode = LanguageCode {
    name: "Tlacolulita Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zpk",
    individual_languages: &[
    ],
};


pub const ZPL: LanguageCode = LanguageCode {
    name: "Lachixío Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zpl",
    individual_languages: &[
    ],
};


pub const ZPM: LanguageCode = LanguageCode {
    name: "Mixtepec Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zpm",
    individual_languages: &[
    ],
};


pub const ZPN: LanguageCode = LanguageCode {
    name: "Santa Inés Yatzechi Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zpn",
    individual_languages: &[
    ],
};


pub const ZPO: LanguageCode = LanguageCode {
    name: "Amatlán Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zpo",
    individual_languages: &[
    ],
};


pub const ZPP: LanguageCode = LanguageCode {
    name: "El Alto Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zpp",
    individual_languages: &[
    ],
};


pub const ZPQ: LanguageCode = LanguageCode {
    name: "Zoogocho Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zpq",
    individual_languages: &[
    ],
};


pub const ZPR: LanguageCode = LanguageCode {
    name: "Santiago Xanica Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zpr",
    individual_languages: &[
    ],
};


pub const ZPS: LanguageCode = LanguageCode {
    name: "Coatlán Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zps",
    individual_languages: &[
    ],
};


pub const ZPT: LanguageCode = LanguageCode {
    name: "San Vicente Coatlán Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zpt",
    individual_languages: &[
    ],
};


pub const ZPU: LanguageCode = LanguageCode {
    name: "Yalálag Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zpu",
    individual_languages: &[
    ],
};


pub const ZPV: LanguageCode = LanguageCode {
    name: "Chichicapan Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zpv",
    individual_languages: &[
    ],
};


pub const ZPW: LanguageCode = LanguageCode {
    name: "Zaniza Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zpw",
    individual_languages: &[
    ],
};


pub const ZPX: LanguageCode = LanguageCode {
    name: "San Baltazar Loxicha Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zpx",
    individual_languages: &[
    ],
};


pub const ZPY: LanguageCode = LanguageCode {
    name: "Mazaltepec Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zpy",
    individual_languages: &[
    ],
};


pub const ZPZ: LanguageCode = LanguageCode {
    name: "Texmelucan Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zpz",
    individual_languages: &[
    ],
};


pub const ZQE: LanguageCode = LanguageCode {
    name: "Qiubei Zhuang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zqe",
    individual_languages: &[
    ],
};


pub const ZRA: LanguageCode = LanguageCode {
    name: "Kara (Korea)",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zra",
    individual_languages: &[
    ],
};


pub const ZRG: LanguageCode = LanguageCode {
    name: "Mirgan",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zrg",
    individual_languages: &[
    ],
};


pub const ZRN: LanguageCode = LanguageCode {
    name: "Zerenkel",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zrn",
    individual_languages: &[
    ],
};


pub const ZRO: LanguageCode = LanguageCode {
    name: "Záparo",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zro",
    individual_languages: &[
    ],
};


pub const ZRP: LanguageCode = LanguageCode {
    name: "Zarphatic",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zrp",
    individual_languages: &[
    ],
};


pub const ZRS: LanguageCode = LanguageCode {
    name: "Mairasi",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zrs",
    individual_languages: &[
    ],
};


pub const ZSA: LanguageCode = LanguageCode {
    name: "Sarasira",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zsa",
    individual_languages: &[
    ],
};


pub const ZSK: LanguageCode = LanguageCode {
    name: "Kaskean",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zsk",
    individual_languages: &[
    ],
};


pub const ZSL: LanguageCode = LanguageCode {
    name: "Zambian Sign Language",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zsl",
    individual_languages: &[
    ],
};


pub const ZSM: LanguageCode = LanguageCode {
    name: "Standard Malay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zsm",
    individual_languages: &[
    ],
};


pub const ZSR: LanguageCode = LanguageCode {
    name: "Southern Rincon Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zsr",
    individual_languages: &[
    ],
};


pub const ZSU: LanguageCode = LanguageCode {
    name: "Sukurum",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zsu",
    individual_languages: &[
    ],
};


pub const ZTE: LanguageCode = LanguageCode {
    name: "Elotepec Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zte",
    individual_languages: &[
    ],
};


pub const ZTG: LanguageCode = LanguageCode {
    name: "Xanaguía Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ztg",
    individual_languages: &[
    ],
};


pub const ZTL: LanguageCode = LanguageCode {
    name: "Lapaguía-Guivini Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ztl",
    individual_languages: &[
    ],
};


pub const ZTM: LanguageCode = LanguageCode {
    name: "San Agustín Mixtepec Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ztm",
    individual_languages: &[
    ],
};


pub const ZTN: LanguageCode = LanguageCode {
    name: "Santa Catarina Albarradas Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ztn",
    individual_languages: &[
    ],
};


pub const ZTP: LanguageCode = LanguageCode {
    name: "Loxicha Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ztp",
    individual_languages: &[
    ],
};


pub const ZTQ: LanguageCode = LanguageCode {
    name: "Quioquitani-Quierí Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ztq",
    individual_languages: &[
    ],
};


pub const ZTS: LanguageCode = LanguageCode {
    name: "Tilquiapan Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zts",
    individual_languages: &[
    ],
};


pub const ZTT: LanguageCode = LanguageCode {
    name: "Tejalapan Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ztt",
    individual_languages: &[
    ],
};


pub const ZTU: LanguageCode = LanguageCode {
    name: "Güilá Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ztu",
    individual_languages: &[
    ],
};


pub const ZTX: LanguageCode = LanguageCode {
    name: "Zaachila Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "ztx",
    individual_languages: &[
    ],
};


pub const ZTY: LanguageCode = LanguageCode {
    name: "Yatee Zapotec",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zty",
    individual_languages: &[
    ],
};


pub const ZUA: LanguageCode = LanguageCode {
    name: "Zeem",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zua",
    individual_languages: &[
    ],
};


pub const ZUH: LanguageCode = LanguageCode {
    name: "Tokano",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zuh",
    individual_languages: &[
    ],
};


pub const ZUM: LanguageCode = LanguageCode {
    name: "Kumzari",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zum",
    individual_languages: &[
    ],
};


pub const ZUY: LanguageCode = LanguageCode {
    name: "Zumaya",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zuy",
    individual_languages: &[
    ],
};


pub const ZWA: LanguageCode = LanguageCode {
    name: "Zay",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zwa",
    individual_languages: &[
    ],
};


pub const ZYB: LanguageCode = LanguageCode {
    name: "Yongbei Zhuang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zyb",
    individual_languages: &[
    ],
};


pub const ZYG: LanguageCode = LanguageCode {
    name: "Yang Zhuang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zyg",
    individual_languages: &[
    ],
};


pub const ZYJ: LanguageCode = LanguageCode {
    name: "Youjiang Zhuang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zyj",
    individual_languages: &[
    ],
};


pub const ZYN: LanguageCode = LanguageCode {
    name: "Yongnan Zhuang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zyn",
    individual_languages: &[
    ],
};


pub const ZYP: LanguageCode = LanguageCode {
    name: "Zyphe Chin",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zyp",
    individual_languages: &[
    ],
};


pub const ZZJ: LanguageCode = LanguageCode {
    name: "Zuojiang Zhuang",
    code: "",
    code_2t: "",
    code_2b: "",
    code_3: "zzj",
    individual_languages: &[
    ],
};


pub const ALL_CODES: & [LanguageCode] = &[
AB,
AA,
AF,
AK,
SQ,
AM,
AR,
AN,
HY,
AS,
AV,
AE,
AY,
AZ,
BM,
BA,
EU,
BE,
BN,
BI,
BS,
BR,
BG,
MY,
CA,
CH,
CE,
NY,
ZH,
CU,
CV,
KW,
CO,
CR,
HR,
CS,
DA,
DV,
NL,
DZ,
EN,
EO,
ET,
EE,
FO,
FJ,
FI,
FR,
FY,
FF,
GD,
GL,
LG,
KA,
DE,
EL,
KL,
GN,
GU,
HT,
HA,
HE,
HZ,
HI,
HO,
HU,
IS,
IO,
IG,
ID,
IA,
IE,
IU,
IK,
GA,
IT,
JA,
JV,
KN,
KR,
KS,
KK,
KM,
KI,
RW,
KY,
KV,
KG,
KO,
KJ,
KU,
LO,
LA,
LV,
LI,
LN,
LT,
LU,
LB,
MK,
MG,
MS,
ML,
MT,
GV,
MI,
MR,
MH,
MN,
NA,
NV,
ND,
NR,
NG,
NE,
NO,
NB,
NN,
II,
OC,
OJ,
OR,
OM,
OS,
PI,
PS,
FA,
PL,
PT,
PA,
QU,
RO,
RM,
RN,
RU,
SE,
SM,
SG,
SA,
SC,
SR,
SN,
SD,
SI,
SK,
SL,
SO,
ST,
ES,
SU,
SW,
SS,
SV,
TL,
TY,
TG,
TA,
TT,
TE,
TH,
BO,
TI,
TO,
TS,
TN,
TR,
TK,
TW,
UG,
UK,
UR,
UZ,
VE,
VI,
VO,
WA,
CY,
WO,
XH,
YI,
YO,
ZA,
ZU,
ACE,
ACH,
ADA,
ADY,
AFA,
AFH,
AIN,
AKK,
ALE,
ALG,
ALT,
ANG,
ANP,
APA,
ARC,
ARN,
ARP,
ART,
ARW,
AST,
ATH,
AUS,
AWA,
BAD,
BAI,
BAL,
BAN,
BAS,
BAT,
BEJ,
BEM,
BER,
BHO,
BIH,
BIK,
BIN,
BLA,
BNT,
BRA,
BTK,
BUA,
BUG,
BYN,
CAD,
CAI,
CAR,
CAU,
CEB,
CEL,
CHB,
CHG,
CHK,
CHM,
CHN,
CHO,
CHP,
CHR,
CHY,
CMC,
CNR,
COP,
CPE,
CPF,
CPP,
CRH,
CRP,
CSB,
CUS,
DAK,
DAR,
DAY,
DEL,
DEN,
DGR,
DIN,
DOI,
DRA,
DSB,
DUA,
DUM,
DYU,
EFI,
EGY,
EKA,
ELX,
ENM,
EWO,
FAN,
FAT,
FIL,
FIU,
FON,
FRM,
FRO,
FRR,
FRS,
FUR,
GAA,
GAY,
GBA,
GEM,
GEZ,
GIL,
GMH,
GOH,
GON,
GOR,
GOT,
GRB,
GRC,
GSW,
GWI,
HAI,
HAW,
HIL,
HIM,
HIT,
HMN,
HSB,
HUP,
IBA,
IJO,
ILO,
INC,
INE,
INH,
IRA,
IRO,
JBO,
JPR,
JRB,
KAA,
KAB,
KAC,
KAM,
KAR,
KAW,
KBD,
KHA,
KHI,
KHO,
KMB,
KOK,
KOS,
KPE,
KRC,
KRL,
KRO,
KRU,
KUM,
KUT,
LAD,
LAH,
LAM,
LEZ,
LOL,
LOZ,
LUA,
LUI,
LUN,
LUO,
LUS,
MAD,
MAG,
MAI,
MAK,
MAN,
MAP,
MAS,
MDF,
MDR,
MEN,
MGA,
MIC,
MIN,
MIS,
MKH,
MNC,
MNI,
MNO,
MOH,
MOS,
MUL,
MUN,
MUS,
MWL,
MWR,
MYN,
MYV,
NAH,
NAI,
NAP,
NDS,
NEW,
NIA,
NIC,
NIU,
NOG,
NON,
NQO,
NSO,
NUB,
NWC,
NYM,
NYN,
NYO,
NZI,
OSA,
OTA,
OTO,
PAA,
PAG,
PAL,
PAM,
PAP,
PAU,
PEO,
PHI,
PHN,
PON,
PRA,
PRO,
QAA,
RAJ,
RAP,
RAR,
ROA,
ROM,
RUP,
SAD,
SAH,
SAI,
SAL,
SAM,
SAS,
SAT,
SCN,
SCO,
SEL,
SEM,
SGA,
SGN,
SHN,
SID,
SIO,
SIT,
SLA,
SMA,
SMI,
SMJ,
SMN,
SMS,
SNK,
SOG,
SON,
SRN,
SRR,
SSA,
SUK,
SUS,
SUX,
SYC,
SYR,
TAI,
TEM,
TER,
TET,
TIG,
TIV,
TKL,
TLH,
TLI,
TMH,
TOG,
TPI,
TSI,
TUM,
TUP,
TUT,
TVL,
TYV,
UDM,
UGA,
UMB,
UND,
VAI,
VOT,
WAK,
WAL,
WAR,
WAS,
WEN,
XAL,
YAO,
YAP,
YPK,
ZAP,
ZBL,
ZEN,
ZGH,
ZND,
ZUN,
ZXX,
ZZA,
AAA,
AAB,
AAC,
AAD,
AAE,
AAF,
AAG,
AAH,
AAI,
AAK,
AAL,
AAN,
AAO,
AAP,
AAQ,
AAS,
AAT,
AAU,
AAW,
AAX,
AAZ,
ABA,
ABB,
ABC,
ABD,
ABE,
ABF,
ABG,
ABH,
ABI,
ABJ,
ABL,
ABM,
ABN,
ABO,
ABP,
ABQ,
ABR,
ABS,
ABT,
ABU,
ABV,
ABW,
ABX,
ABY,
ABZ,
ACA,
ACB,
ACD,
ACF,
ACI,
ACK,
ACL,
ACM,
ACN,
ACP,
ACQ,
ACR,
ACS,
ACT,
ACU,
ACV,
ACW,
ACX,
ACY,
ACZ,
ADB,
ADD,
ADE,
ADF,
ADG,
ADH,
ADI,
ADJ,
ADL,
ADN,
ADO,
ADQ,
ADR,
ADS,
ADT,
ADU,
ADW,
ADX,
ADZ,
AEA,
AEB,
AEC,
AED,
AEE,
AEK,
AEL,
AEM,
AEN,
AEQ,
AER,
AES,
AEU,
AEW,
AEY,
AEZ,
AFB,
AFD,
AFE,
AFG,
AFI,
AFK,
AFN,
AFO,
AFP,
AFS,
AFT,
AFU,
AFZ,
AGA,
AGB,
AGC,
AGD,
AGE,
AGF,
AGG,
AGH,
AGI,
AGJ,
AGK,
AGL,
AGM,
AGN,
AGO,
AGQ,
AGR,
AGS,
AGT,
AGU,
AGV,
AGW,
AGX,
AGY,
AGZ,
AHA,
AHB,
AHG,
AHH,
AHI,
AHK,
AHL,
AHM,
AHN,
AHO,
AHP,
AHR,
AHS,
AHT,
AIA,
AIB,
AIC,
AID,
AIE,
AIF,
AIG,
AIH,
AII,
AIJ,
AIK,
AIL,
AIM,
AIO,
AIP,
AIQ,
AIR,
AIT,
AIW,
AIX,
AIY,
AJA,
AJG,
AJI,
AJN,
AJP,
AJS,
AJU,
AJW,
AJZ,
AKB,
AKC,
AKD,
AKE,
AKF,
AKG,
AKH,
AKI,
AKJ,
AKL,
AKM,
AKO,
AKP,
AKQ,
AKR,
AKS,
AKT,
AKU,
AKV,
AKW,
AKX,
AKY,
AKZ,
ALA,
ALC,
ALD,
ALF,
ALH,
ALI,
ALJ,
ALK,
ALL,
ALM,
ALN,
ALO,
ALP,
ALQ,
ALR,
ALS,
ALU,
ALW,
ALX,
ALY,
ALZ,
AMA,
AMB,
AMC,
AME,
AMF,
AMG,
AMI,
AMJ,
AMK,
AML,
AMM,
AMN,
AMO,
AMP,
AMQ,
AMR,
AMS,
AMT,
AMU,
AMV,
AMW,
AMX,
AMY,
AMZ,
ANA,
ANB,
ANC,
AND,
ANE,
ANF,
ANH,
ANI,
ANJ,
ANK,
ANL,
ANM,
ANN,
ANO,
ANQ,
ANR,
ANS,
ANT,
ANU,
ANV,
ANW,
ANX,
ANY,
ANZ,
AOA,
AOB,
AOC,
AOD,
AOE,
AOF,
AOG,
AOI,
AOJ,
AOK,
AOL,
AOM,
AON,
AOR,
AOS,
AOT,
AOU,
AOX,
AOZ,
APB,
APC,
APD,
APE,
APF,
APG,
APH,
API,
APJ,
APK,
APL,
APM,
APN,
APO,
APP,
APQ,
APR,
APS,
APT,
APU,
APV,
APW,
APX,
APY,
APZ,
AQC,
AQD,
AQG,
AQK,
AQM,
AQN,
AQP,
AQR,
AQT,
AQZ,
ARB,
ARD,
ARE,
ARH,
ARI,
ARJ,
ARK,
ARL,
ARO,
ARQ,
ARR,
ARS,
ARU,
ARV,
ARX,
ARY,
ARZ,
ASA,
ASB,
ASC,
ASE,
ASF,
ASG,
ASH,
ASI,
ASJ,
ASK,
ASL,
ASN,
ASO,
ASP,
ASQ,
ASR,
ASS,
ASU,
ASV,
ASW,
ASX,
ASY,
ASZ,
ATA,
ATB,
ATC,
ATD,
ATE,
ATG,
ATI,
ATJ,
ATK,
ATL,
ATM,
ATN,
ATO,
ATP,
ATQ,
ATR,
ATS,
ATT,
ATU,
ATV,
ATW,
ATX,
ATY,
ATZ,
AUA,
AUB,
AUC,
AUD,
AUG,
AUH,
AUI,
AUJ,
AUK,
AUL,
AUM,
AUN,
AUO,
AUP,
AUQ,
AUR,
AUT,
AUU,
AUW,
AUX,
AUY,
AUZ,
AVB,
AVD,
AVI,
AVK,
AVL,
AVM,
AVN,
AVO,
AVS,
AVT,
AVU,
AVV,
AWB,
AWC,
AWE,
AWG,
AWH,
AWI,
AWK,
AWM,
AWN,
AWO,
AWR,
AWS,
AWT,
AWU,
AWV,
AWW,
AWX,
AWY,
AXB,
AXE,
AXG,
AXK,
AXL,
AXM,
AXX,
AYA,
AYB,
AYC,
AYD,
AYE,
AYG,
AYH,
AYI,
AYK,
AYL,
AYN,
AYO,
AYP,
AYQ,
AYR,
AYS,
AYT,
AYU,
AYZ,
AZA,
AZB,
AZD,
AZG,
AZJ,
AZM,
AZN,
AZO,
AZT,
AZZ,
BAA,
BAB,
BAC,
BAE,
BAF,
BAG,
BAH,
BAJ,
BAO,
BAP,
BAR,
BAU,
BAV,
BAW,
BAX,
BAY,
BBA,
BBB,
BBC,
BBD,
BBE,
BBF,
BBG,
BBH,
BBI,
BBJ,
BBK,
BBL,
BBM,
BBN,
BBO,
BBP,
BBQ,
BBR,
BBS,
BBT,
BBU,
BBV,
BBW,
BBX,
BBY,
BCA,
BCB,
BCC,
BCD,
BCE,
BCF,
BCG,
BCH,
BCI,
BCJ,
BCK,
BCL,
BCM,
BCN,
BCO,
BCP,
BCQ,
BCR,
BCS,
BCT,
BCU,
BCV,
BCW,
BCY,
BCZ,
BDA,
BDB,
BDC,
BDD,
BDE,
BDF,
BDG,
BDH,
BDI,
BDJ,
BDK,
BDL,
BDM,
BDN,
BDO,
BDP,
BDQ,
BDR,
BDS,
BDT,
BDU,
BDV,
BDW,
BDX,
BDY,
BDZ,
BEA,
BEB,
BEC,
BED,
BEE,
BEF,
BEG,
BEH,
BEI,
BEK,
BEO,
BEP,
BEQ,
BES,
BET,
BEU,
BEV,
BEW,
BEX,
BEY,
BEZ,
BFA,
BFB,
BFC,
BFD,
BFE,
BFF,
BFG,
BFH,
BFI,
BFJ,
BFK,
BFL,
BFM,
BFN,
BFO,
BFP,
BFQ,
BFR,
BFS,
BFT,
BFU,
BFW,
BFX,
BFY,
BFZ,
BGA,
BGB,
BGC,
BGD,
BGE,
BGF,
BGG,
BGI,
BGJ,
BGK,
BGL,
BGN,
BGO,
BGP,
BGQ,
BGR,
BGS,
BGT,
BGU,
BGV,
BGW,
BGX,
BGY,
BGZ,
BHA,
BHB,
BHC,
BHD,
BHE,
BHF,
BHG,
BHH,
BHI,
BHJ,
BHL,
BHM,
BHN,
BHP,
BHQ,
BHR,
BHS,
BHT,
BHU,
BHV,
BHW,
BHX,
BHY,
BHZ,
BIA,
BIB,
BID,
BIE,
BIF,
BIG,
BIL,
BIM,
BIO,
BIP,
BIQ,
BIR,
BIT,
BIU,
BIV,
BIW,
BIX,
BIY,
BIZ,
BJA,
BJB,
BJC,
BJE,
BJF,
BJG,
BJH,
BJI,
BJJ,
BJK,
BJL,
BJM,
BJN,
BJO,
BJP,
BJR,
BJS,
BJT,
BJU,
BJV,
BJW,
BJX,
BJY,
BJZ,
BKA,
BKC,
BKD,
BKF,
BKG,
BKH,
BKI,
BKJ,
BKK,
BKL,
BKM,
BKN,
BKO,
BKP,
BKQ,
BKR,
BKS,
BKT,
BKU,
BKV,
BKW,
BKX,
BKY,
BKZ,
BLB,
BLC,
BLD,
BLE,
BLF,
BLH,
BLI,
BLJ,
BLK,
BLL,
BLM,
BLN,
BLO,
BLP,
BLQ,
BLR,
BLS,
BLT,
BLV,
BLW,
BLX,
BLY,
BLZ,
BMA,
BMB,
BMC,
BMD,
BME,
BMF,
BMG,
BMH,
BMI,
BMJ,
BMK,
BML,
BMM,
BMN,
BMO,
BMP,
BMQ,
BMR,
BMS,
BMT,
BMU,
BMV,
BMW,
BMX,
BMZ,
BNA,
BNB,
BNC,
BND,
BNE,
BNF,
BNG,
BNI,
BNJ,
BNK,
BNL,
BNM,
BNN,
BNO,
BNP,
BNQ,
BNR,
BNS,
BNU,
BNV,
BNW,
BNX,
BNY,
BNZ,
BOA,
BOB,
BOE,
BOF,
BOG,
BOH,
BOI,
BOJ,
BOK,
BOL,
BOM,
BON,
BOO,
BOP,
BOQ,
BOR,
BOT,
BOU,
BOV,
BOW,
BOX,
BOY,
BOZ,
BPA,
BPC,
BPD,
BPE,
BPG,
BPH,
BPI,
BPJ,
BPK,
BPL,
BPM,
BPN,
BPO,
BPP,
BPQ,
BPR,
BPS,
BPT,
BPU,
BPV,
BPW,
BPX,
BPY,
BPZ,
BQA,
BQB,
BQC,
BQD,
BQF,
BQG,
BQH,
BQI,
BQJ,
BQK,
BQL,
BQM,
BQN,
BQO,
BQP,
BQQ,
BQR,
BQS,
BQT,
BQU,
BQV,
BQW,
BQX,
BQY,
BQZ,
BRB,
BRC,
BRD,
BRF,
BRG,
BRH,
BRI,
BRJ,
BRK,
BRL,
BRM,
BRN,
BRO,
BRP,
BRQ,
BRR,
BRS,
BRT,
BRU,
BRV,
BRW,
BRX,
BRY,
BRZ,
BSA,
BSB,
BSC,
BSE,
BSF,
BSG,
BSH,
BSI,
BSJ,
BSK,
BSL,
BSM,
BSN,
BSO,
BSP,
BSQ,
BSR,
BSS,
BST,
BSU,
BSV,
BSW,
BSX,
BSY,
BTA,
BTC,
BTD,
BTE,
BTF,
BTG,
BTH,
BTI,
BTJ,
BTM,
BTN,
BTO,
BTP,
BTQ,
BTR,
BTS,
BTT,
BTU,
BTV,
BTW,
BTX,
BTY,
BTZ,
BUB,
BUC,
BUD,
BUE,
BUF,
BUH,
BUI,
BUJ,
BUK,
BUM,
BUN,
BUO,
BUP,
BUQ,
BUS,
BUT,
BUU,
BUV,
BUW,
BUX,
BUY,
BUZ,
BVA,
BVB,
BVC,
BVD,
BVE,
BVF,
BVG,
BVH,
BVI,
BVJ,
BVK,
BVL,
BVM,
BVN,
BVO,
BVP,
BVQ,
BVR,
BVT,
BVU,
BVV,
BVW,
BVX,
BVY,
BVZ,
BWA,
BWB,
BWC,
BWD,
BWE,
BWF,
BWG,
BWH,
BWI,
BWJ,
BWK,
BWL,
BWM,
BWN,
BWO,
BWP,
BWQ,
BWR,
BWS,
BWT,
BWU,
BWW,
BWX,
BWY,
BWZ,
BXA,
BXB,
BXC,
BXD,
BXE,
BXF,
BXG,
BXH,
BXI,
BXJ,
BXK,
BXL,
BXM,
BXN,
BXO,
BXP,
BXQ,
BXR,
BXS,
BXU,
BXV,
BXW,
BXZ,
BYA,
BYB,
BYC,
BYD,
BYE,
BYF,
BYG,
BYH,
BYI,
BYJ,
BYK,
BYL,
BYM,
BYO,
BYP,
BYQ,
BYR,
BYS,
BYT,
BYV,
BYW,
BYX,
BYZ,
BZA,
BZB,
BZC,
BZD,
BZE,
BZF,
BZG,
BZH,
BZI,
BZJ,
BZK,
BZL,
BZM,
BZN,
BZO,
BZP,
BZQ,
BZR,
BZS,
BZT,
BZU,
BZV,
BZW,
BZX,
BZY,
BZZ,
CAA,
CAB,
CAC,
CAE,
CAF,
CAG,
CAH,
CAJ,
CAK,
CAL,
CAM,
CAN,
CAO,
CAP,
CAQ,
CAS,
CAV,
CAW,
CAX,
CAY,
CAZ,
CBB,
CBC,
CBD,
CBG,
CBI,
CBJ,
CBK,
CBL,
CBN,
CBO,
CBQ,
CBR,
CBS,
CBT,
CBU,
CBV,
CBW,
CBY,
CCC,
CCD,
CCE,
CCG,
CCH,
CCJ,
CCL,
CCM,
CCO,
CCP,
CCR,
CDA,
CDE,
CDF,
CDH,
CDI,
CDJ,
CDM,
CDN,
CDO,
CDR,
CDS,
CDY,
CDZ,
CEA,
CEG,
CEK,
CEN,
CET,
CEY,
CFA,
CFD,
CFG,
CFM,
CGA,
CGC,
CGG,
CGK,
CHC,
CHD,
CHF,
CHH,
CHJ,
CHL,
CHQ,
CHT,
CHW,
CHX,
CHZ,
CIA,
CIB,
CIC,
CID,
CIE,
CIH,
CIK,
CIM,
CIN,
CIP,
CIR,
CIW,
CIY,
CJA,
CJE,
CJH,
CJI,
CJK,
CJM,
CJN,
CJO,
CJP,
CJS,
CJV,
CJY,
CKB,
CKH,
CKL,
CKM,
CKN,
CKO,
CKQ,
CKR,
CKS,
CKT,
CKU,
CKV,
CKX,
CKY,
CKZ,
CLA,
CLC,
CLD,
CLE,
CLH,
CLI,
CLJ,
CLK,
CLL,
CLM,
CLO,
CLT,
CLU,
CLW,
CLY,
CMA,
CME,
CMG,
CMI,
CML,
CMM,
CMN,
CMO,
CMR,
CMS,
CMT,
CNA,
CNB,
CNC,
CNG,
CNH,
CNI,
CNK,
CNL,
CNO,
CNP,
CNQ,
CNS,
CNT,
CNU,
CNW,
CNX,
COA,
COB,
COC,
COD,
COE,
COF,
COG,
COH,
COJ,
COK,
COL,
COM,
CON,
COO,
COQ,
COT,
COU,
COV,
COW,
COX,
COZ,
CPA,
CPB,
CPC,
CPG,
CPI,
CPN,
CPO,
CPS,
CPU,
CPX,
CPY,
CQD,
CRA,
CRB,
CRC,
CRD,
CRF,
CRG,
CRI,
CRJ,
CRK,
CRL,
CRM,
CRN,
CRO,
CRQ,
CRR,
CRS,
CRT,
CRV,
CRW,
CRX,
CRY,
CRZ,
CSA,
CSC,
CSD,
CSE,
CSF,
CSG,
CSH,
CSI,
CSJ,
CSK,
CSL,
CSM,
CSN,
CSO,
CSP,
CSQ,
CSR,
CSS,
CST,
CSV,
CSW,
CSX,
CSY,
CSZ,
CTA,
CTC,
CTD,
CTE,
CTG,
CTH,
CTL,
CTM,
CTN,
CTO,
CTP,
CTS,
CTT,
CTU,
CTY,
CTZ,
CUA,
CUB,
CUC,
CUH,
CUI,
CUJ,
CUK,
CUL,
CUO,
CUP,
CUQ,
CUR,
CUT,
CUU,
CUV,
CUW,
CUX,
CUY,
CVG,
CVN,
CWA,
CWB,
CWD,
CWE,
CWG,
CWT,
CYA,
CYB,
CYO,
CZH,
CZK,
CZN,
CZO,
CZT,
DAA,
DAC,
DAD,
DAE,
DAG,
DAH,
DAI,
DAJ,
DAL,
DAM,
DAO,
DAQ,
DAS,
DAU,
DAV,
DAW,
DAX,
DAZ,
DBA,
DBB,
DBD,
DBE,
DBF,
DBG,
DBI,
DBJ,
DBL,
DBM,
DBN,
DBO,
DBP,
DBQ,
DBR,
DBT,
DBU,
DBV,
DBW,
DBY,
DCC,
DCR,
DDA,
DDD,
DDE,
DDG,
DDI,
DDJ,
DDN,
DDO,
DDR,
DDS,
DDW,
DEC,
DED,
DEE,
DEF,
DEG,
DEH,
DEI,
DEK,
DEM,
DEP,
DEQ,
DER,
DES,
DEV,
DEZ,
DGA,
DGB,
DGC,
DGD,
DGE,
DGG,
DGH,
DGI,
DGK,
DGL,
DGN,
DGO,
DGS,
DGT,
DGW,
DGX,
DGZ,
DHD,
DHG,
DHI,
DHL,
DHM,
DHN,
DHO,
DHR,
DHS,
DHU,
DHV,
DHW,
DHX,
DIA,
DIB,
DIC,
DID,
DIF,
DIG,
DIH,
DII,
DIJ,
DIK,
DIL,
DIM,
DIO,
DIP,
DIQ,
DIR,
DIS,
DIU,
DIW,
DIX,
DIY,
DIZ,
DJA,
DJB,
DJC,
DJD,
DJE,
DJF,
DJI,
DJJ,
DJK,
DJM,
DJN,
DJO,
DJR,
DJU,
DJW,
DKA,
DKG,
DKK,
DKR,
DKS,
DKX,
DLG,
DLK,
DLM,
DLN,
DMA,
DMB,
DMC,
DMD,
DME,
DMF,
DMG,
DMK,
DML,
DMM,
DMO,
DMR,
DMS,
DMU,
DMV,
DMW,
DMX,
DMY,
DNA,
DND,
DNE,
DNG,
DNI,
DNJ,
DNK,
DNN,
DNO,
DNR,
DNT,
DNU,
DNV,
DNW,
DNY,
DOA,
DOB,
DOC,
DOE,
DOF,
DOH,
DOK,
DOL,
DON,
DOO,
DOP,
DOQ,
DOR,
DOS,
DOT,
DOV,
DOW,
DOX,
DOY,
DOZ,
DPP,
DRB,
DRC,
DRD,
DRE,
DRG,
DRI,
DRL,
DRN,
DRO,
DRQ,
DRS,
DRT,
DRU,
DRY,
DSE,
DSH,
DSI,
DSL,
DSN,
DSO,
DSQ,
DSZ,
DTA,
DTB,
DTD,
DTH,
DTI,
DTK,
DTM,
DTN,
DTO,
DTP,
DTR,
DTS,
DTT,
DTU,
DTY,
DUB,
DUC,
DUE,
DUF,
DUG,
DUH,
DUI,
DUK,
DUL,
DUN,
DUO,
DUP,
DUQ,
DUR,
DUS,
DUU,
DUV,
DUW,
DUX,
DUY,
DUZ,
DVA,
DWA,
DWK,
DWR,
DWS,
DWU,
DWW,
DWY,
DWZ,
DYA,
DYB,
DYD,
DYG,
DYI,
DYM,
DYN,
DYO,
DYY,
DZA,
DZE,
DZG,
DZL,
DZN,
EAA,
EBC,
EBG,
EBK,
EBO,
EBR,
EBU,
ECR,
ECS,
ECY,
EEE,
EFA,
EFE,
EGA,
EGL,
EGM,
EGO,
EHS,
EHU,
EIP,
EIT,
EIV,
EJA,
EKE,
EKG,
EKI,
EKK,
EKL,
EKM,
EKO,
EKP,
EKR,
EKY,
ELE,
ELH,
ELI,
ELK,
ELM,
ELO,
ELU,
EMA,
EMB,
EME,
EMG,
EMI,
EMK,
EMM,
EMN,
EMP,
EMQ,
EMS,
EMU,
EMW,
EMX,
EMY,
EMZ,
ENA,
ENB,
ENC,
END,
ENF,
ENH,
ENL,
ENN,
ENO,
ENQ,
ENR,
ENU,
ENV,
ENW,
ENX,
EOT,
EPI,
ERA,
ERG,
ERH,
ERI,
ERK,
ERO,
ERR,
ERS,
ERT,
ERW,
ESE,
ESG,
ESH,
ESI,
ESK,
ESL,
ESM,
ESN,
ESO,
ESQ,
ESS,
ESU,
ESY,
ETB,
ETC,
ETH,
ETN,
ETO,
ETR,
ETS,
ETT,
ETU,
ETX,
ETZ,
EVE,
EVH,
EVN,
EXT,
EYA,
EYO,
EZA,
EZE,
FAA,
FAB,
FAD,
FAF,
FAG,
FAH,
FAI,
FAJ,
FAK,
FAL,
FAM,
FAP,
FAR,
FAU,
FAX,
FAY,
FAZ,
FBL,
FCS,
FER,
FFI,
FFM,
FGR,
FIA,
FIE,
FIF,
FIP,
FIR,
FIT,
FIW,
FKK,
FKV,
FLA,
FLH,
FLI,
FLL,
FLN,
FLR,
FLY,
FMP,
FMU,
FNB,
FNG,
FNI,
FOD,
FOI,
FOM,
FOR,
FOS,
FPE,
FQS,
FRC,
FRD,
FRK,
FRP,
FRQ,
FRT,
FSE,
FSL,
FSS,
FUB,
FUC,
FUD,
FUE,
FUF,
FUH,
FUI,
FUJ,
FUM,
FUN,
FUQ,
FUT,
FUU,
FUV,
FUY,
FVR,
FWA,
FWE,
GAB,
GAC,
GAD,
GAE,
GAF,
GAG,
GAH,
GAI,
GAJ,
GAK,
GAL,
GAM,
GAN,
GAO,
GAP,
GAQ,
GAR,
GAS,
GAT,
GAU,
GAW,
GAX,
GAZ,
GBB,
GBD,
GBE,
GBF,
GBG,
GBH,
GBI,
GBJ,
GBK,
GBL,
GBM,
GBN,
GBO,
GBP,
GBQ,
GBR,
GBS,
GBU,
GBV,
GBW,
GBX,
GBY,
GBZ,
GCC,
GCD,
GCE,
GCF,
GCL,
GCN,
GCR,
GCT,
GDA,
GDB,
GDC,
GDD,
GDE,
GDF,
GDG,
GDH,
GDI,
GDJ,
GDK,
GDL,
GDM,
GDN,
GDO,
GDQ,
GDR,
GDS,
GDT,
GDU,
GDX,
GEA,
GEB,
GEC,
GED,
GEF,
GEG,
GEH,
GEI,
GEJ,
GEK,
GEL,
GEQ,
GES,
GEV,
GEW,
GEX,
GEY,
GFK,
GFT,
GGA,
GGB,
GGD,
GGE,
GGG,
GGK,
GGL,
GGT,
GGU,
GGW,
GHA,
GHC,
GHE,
GHH,
GHK,
GHL,
GHN,
GHO,
GHR,
GHS,
GHT,
GIA,
GIB,
GIC,
GID,
GIE,
GIG,
GIH,
GII,
GIM,
GIN,
GIP,
GIQ,
GIR,
GIS,
GIT,
GIU,
GIW,
GIX,
GIY,
GIZ,
GJK,
GJM,
GJN,
GJR,
GJU,
GKA,
GKD,
GKE,
GKN,
GKO,
GKP,
GKU,
GLB,
GLC,
GLD,
GLH,
GLJ,
GLK,
GLL,
GLO,
GLR,
GLU,
GLW,
GLY,
GMA,
GMB,
GMD,
GMG,
GML,
GMM,
GMN,
GMR,
GMU,
GMV,
GMX,
GMY,
GMZ,
GNA,
GNB,
GNC,
GND,
GNE,
GNG,
GNH,
GNI,
GNJ,
GNK,
GNL,
GNM,
GNN,
GNO,
GNQ,
GNR,
GNT,
GNU,
GNW,
GNZ,
GOA,
GOB,
GOC,
GOD,
GOE,
GOF,
GOG,
GOI,
GOJ,
GOK,
GOL,
GOM,
GOO,
GOP,
GOQ,
GOS,
GOU,
GOV,
GOW,
GOX,
GOY,
GOZ,
GPA,
GPE,
GPN,
GQA,
GQI,
GQN,
GQR,
GQU,
GRA,
GRD,
GRG,
GRH,
GRI,
GRJ,
GRM,
GRO,
GRQ,
GRR,
GRS,
GRT,
GRU,
GRV,
GRW,
GRX,
GRY,
GRZ,
GSE,
GSG,
GSL,
GSM,
GSN,
GSO,
GSP,
GSS,
GTA,
GTU,
GUA,
GUB,
GUC,
GUD,
GUE,
GUF,
GUG,
GUH,
GUI,
GUK,
GUL,
GUM,
GUN,
GUO,
GUP,
GUQ,
GUR,
GUS,
GUT,
GUU,
GUW,
GUX,
GUZ,
GVA,
GVC,
GVE,
GVF,
GVJ,
GVL,
GVM,
GVN,
GVO,
GVP,
GVR,
GVS,
GVY,
GWA,
GWB,
GWC,
GWD,
GWE,
GWF,
GWG,
GWJ,
GWM,
GWN,
GWR,
GWT,
GWU,
GWW,
GWX,
GXX,
GYA,
GYB,
GYD,
GYE,
GYF,
GYG,
GYI,
GYL,
GYM,
GYN,
GYO,
GYR,
GYY,
GYZ,
GZA,
GZI,
GZN,
HAA,
HAB,
HAC,
HAD,
HAE,
HAF,
HAG,
HAH,
HAJ,
HAK,
HAL,
HAM,
HAN,
HAO,
HAP,
HAQ,
HAR,
HAS,
HAV,
HAX,
HAY,
HAZ,
HBA,
HBB,
HBN,
HBO,
SH,
HBU,
HCA,
HCH,
HDN,
HDS,
HDY,
HEA,
HED,
HEG,
HEH,
HEI,
HEM,
HGM,
HGW,
HHI,
HHR,
HHY,
HIA,
HIB,
HID,
HIF,
HIG,
HIH,
HII,
HIJ,
HIK,
HIO,
HIR,
HIW,
HIX,
HJI,
HKA,
HKE,
HKH,
HKK,
HKN,
HKS,
HLA,
HLB,
HLD,
HLE,
HLT,
HLU,
HMA,
HMB,
HMC,
HMD,
HME,
HMF,
HMG,
HMH,
HMI,
HMJ,
HMK,
HML,
HMM,
HMP,
HMQ,
HMR,
HMS,
HMT,
HMU,
HMV,
HMW,
HMY,
HMZ,
HNA,
HND,
HNE,
HNG,
HNH,
HNI,
HNJ,
HNN,
HNO,
HNS,
HNU,
HOA,
HOB,
HOC,
HOD,
HOE,
HOH,
HOI,
HOJ,
HOL,
HOM,
HOO,
HOP,
HOR,
HOS,
HOT,
HOV,
HOW,
HOY,
HOZ,
HPO,
HPS,
HRA,
HRC,
HRE,
HRK,
HRM,
HRO,
HRP,
HRT,
HRU,
HRW,
HRX,
HRZ,
HSH,
HSL,
HSN,
HSS,
HTI,
HTO,
HTS,
HTU,
HTX,
HUB,
HUC,
HUD,
HUE,
HUF,
HUG,
HUH,
HUI,
HUJ,
HUK,
HUL,
HUM,
HUO,
HUQ,
HUR,
HUS,
HUT,
HUU,
HUV,
HUW,
HUX,
HUY,
HUZ,
HVC,
HVE,
HVK,
HVN,
HVV,
HWA,
HWC,
HWO,
HYA,
HYW,
IAI,
IAN,
IAR,
IBB,
IBD,
IBE,
IBG,
IBH,
IBL,
IBM,
IBN,
IBR,
IBU,
IBY,
ICA,
ICH,
ICL,
ICR,
IDA,
IDB,
IDC,
IDD,
IDE,
IDI,
IDR,
IDS,
IDT,
IDU,
IFA,
IFB,
IFE,
IFF,
IFK,
IFM,
IFU,
IFY,
IGB,
IGE,
IGG,
IGL,
IGM,
IGN,
IGO,
IGS,
IGW,
IHB,
IHI,
IHP,
IHW,
IIN,
IJC,
IJE,
IJJ,
IJN,
IJS,
IKE,
IKI,
IKK,
IKL,
IKO,
IKP,
IKR,
IKS,
IKT,
IKV,
IKW,
IKX,
IKZ,
ILA,
ILB,
ILG,
ILI,
ILK,
ILM,
ILP,
ILS,
ILU,
ILV,
IMA,
IMI,
IML,
IMN,
IMO,
IMR,
IMS,
IMT,
IMY,
INB,
ING,
INJ,
INL,
INM,
INN,
INO,
INP,
INS,
INT,
INZ,
IOR,
IOU,
IOW,
IPI,
IPO,
IQU,
IQW,
IRE,
IRH,
IRI,
IRK,
IRN,
IRR,
IRU,
IRX,
IRY,
ISA,
ISC,
ISD,
ISE,
ISG,
ISH,
ISI,
ISK,
ISM,
ISN,
ISO,
ISR,
IST,
ISU,
ITB,
ITD,
ITE,
ITI,
ITK,
ITL,
ITM,
ITO,
ITR,
ITS,
ITT,
ITV,
ITW,
ITX,
ITY,
ITZ,
IUM,
IVB,
IVV,
IWK,
IWM,
IWO,
IWS,
IXC,
IXL,
IYA,
IYO,
IYX,
IZH,
IZR,
IZZ,
JAA,
JAB,
JAC,
JAD,
JAE,
JAF,
JAH,
JAJ,
JAK,
JAL,
JAM,
JAN,
JAO,
JAQ,
JAS,
JAT,
JAU,
JAX,
JAY,
JAZ,
JBE,
JBI,
JBJ,
JBK,
JBM,
JBN,
JBR,
JBT,
JBU,
JBW,
JCS,
JCT,
JDA,
JDG,
JDT,
JEB,
JEE,
JEH,
JEI,
JEK,
JEL,
JEN,
JER,
JET,
JEU,
JGB,
JGE,
JGK,
JGO,
JHI,
JHS,
JIA,
JIB,
JIC,
JID,
JIE,
JIG,
JIH,
JII,
JIL,
JIM,
JIO,
JIQ,
JIT,
JIU,
JIV,
JIY,
JJE,
JJR,
JKA,
JKM,
JKO,
JKP,
JKR,
JKS,
JKU,
JLE,
JLS,
JMA,
JMB,
JMC,
JMD,
JMI,
JML,
JMN,
JMR,
JMS,
JMW,
JMX,
JNA,
JND,
JNG,
JNI,
JNJ,
JNL,
JNS,
JOB,
JOD,
JOG,
JOR,
JOS,
JOW,
JPA,
JQR,
JRA,
JRR,
JRT,
JRU,
JSL,
JUA,
JUB,
JUC,
JUD,
JUH,
JUI,
JUK,
JUL,
JUM,
JUN,
JUO,
JUP,
JUR,
JUS,
JUT,
JUU,
JUW,
JUY,
JVD,
JVN,
JWI,
JYA,
JYE,
JYY,
KAD,
KAE,
KAF,
KAG,
KAH,
KAI,
KAJ,
KAK,
KAO,
KAP,
KAQ,
KAV,
KAX,
KAY,
KBA,
KBB,
KBC,
KBE,
KBG,
KBH,
KBI,
KBJ,
KBK,
KBL,
KBM,
KBN,
KBO,
KBP,
KBQ,
KBR,
KBS,
KBT,
KBU,
KBV,
KBW,
KBX,
KBY,
KBZ,
KCA,
KCB,
KCC,
KCD,
KCE,
KCF,
KCG,
KCH,
KCI,
KCJ,
KCK,
KCL,
KCM,
KCN,
KCO,
KCP,
KCQ,
KCR,
KCS,
KCT,
KCU,
KCV,
KCW,
KCX,
KCY,
KCZ,
KDA,
KDC,
KDD,
KDE,
KDF,
KDG,
KDH,
KDI,
KDJ,
KDK,
KDL,
KDM,
KDN,
KDP,
KDQ,
KDR,
KDT,
KDU,
KDW,
KDX,
KDY,
KDZ,
KEA,
KEB,
KEC,
KED,
KEE,
KEF,
KEG,
KEH,
KEI,
KEJ,
KEK,
KEL,
KEM,
KEN,
KEO,
KEP,
KEQ,
KER,
KES,
KET,
KEU,
KEV,
KEW,
KEX,
KEY,
KEZ,
KFA,
KFB,
KFC,
KFD,
KFE,
KFF,
KFG,
KFH,
KFI,
KFJ,
KFK,
KFL,
KFM,
KFN,
KFO,
KFP,
KFQ,
KFR,
KFS,
KFT,
KFU,
KFV,
KFW,
KFX,
KFY,
KFZ,
KGA,
KGB,
KGE,
KGF,
KGG,
KGI,
KGJ,
KGK,
KGL,
KGM,
KGN,
KGO,
KGP,
KGQ,
KGR,
KGS,
KGT,
KGU,
KGV,
KGW,
KGX,
KGY,
KHB,
KHC,
KHD,
KHE,
KHF,
KHG,
KHH,
KHJ,
KHK,
KHL,
KHN,
KHP,
KHQ,
KHR,
KHS,
KHT,
KHU,
KHV,
KHW,
KHX,
KHY,
KHZ,
KIA,
KIB,
KIC,
KID,
KIE,
KIF,
KIG,
KIH,
KII,
KIJ,
KIL,
KIM,
KIO,
KIP,
KIQ,
KIS,
KIT,
KIU,
KIV,
KIW,
KIX,
KIY,
KIZ,
KJA,
KJB,
KJC,
KJD,
KJE,
KJG,
KJH,
KJI,
KJJ,
KJK,
KJL,
KJM,
KJN,
KJO,
KJP,
KJQ,
KJR,
KJS,
KJT,
KJU,
KJV,
KJX,
KJY,
KJZ,
KKA,
KKB,
KKC,
KKD,
KKE,
KKF,
KKG,
KKH,
KKI,
KKJ,
KKK,
KKL,
KKM,
KKN,
KKO,
KKP,
KKQ,
KKR,
KKS,
KKT,
KKU,
KKV,
KKW,
KKX,
KKY,
KKZ,
KLA,
KLB,
KLC,
KLD,
KLE,
KLF,
KLG,
KLH,
KLI,
KLJ,
KLK,
KLL,
KLM,
KLN,
KLO,
KLP,
KLQ,
KLR,
KLS,
KLT,
KLU,
KLV,
KLW,
KLX,
KLY,
KLZ,
KMA,
KMC,
KMD,
KME,
KMF,
KMG,
KMH,
KMI,
KMJ,
KMK,
KML,
KMM,
KMN,
KMO,
KMP,
KMQ,
KMR,
KMS,
KMT,
KMU,
KMV,
KMW,
KMX,
KMY,
KMZ,
KNA,
KNB,
KNC,
KND,
KNE,
KNF,
KNG,
KNI,
KNJ,
KNK,
KNL,
KNM,
KNN,
KNO,
KNP,
KNQ,
KNR,
KNS,
KNT,
KNU,
KNV,
KNW,
KNX,
KNY,
KNZ,
KOA,
KOC,
KOD,
KOE,
KOF,
KOG,
KOH,
KOI,
KOL,
KOO,
KOP,
KOQ,
KOT,
KOU,
KOV,
KOW,
KOY,
KOZ,
KPA,
KPB,
KPC,
KPD,
KPF,
KPG,
KPH,
KPI,
KPJ,
KPK,
KPL,
KPM,
KPN,
KPO,
KPQ,
KPR,
KPS,
KPT,
KPU,
KPV,
KPW,
KPX,
KPY,
KPZ,
KQA,
KQB,
KQC,
KQD,
KQE,
KQF,
KQG,
KQH,
KQI,
KQJ,
KQK,
KQL,
KQM,
KQN,
KQO,
KQP,
KQQ,
KQR,
KQS,
KQT,
KQU,
KQV,
KQW,
KQX,
KQY,
KQZ,
KRA,
KRB,
KRD,
KRE,
KRF,
KRH,
KRI,
KRJ,
KRK,
KRN,
KRP,
KRR,
KRS,
KRT,
KRV,
KRW,
KRX,
KRY,
KRZ,
KSA,
KSB,
KSC,
KSD,
KSE,
KSF,
KSG,
KSH,
KSI,
KSJ,
KSK,
KSL,
KSM,
KSN,
KSO,
KSP,
KSQ,
KSR,
KSS,
KST,
KSU,
KSV,
KSW,
KSX,
KSY,
KSZ,
KTA,
KTB,
KTC,
KTD,
KTE,
KTF,
KTG,
KTH,
KTI,
KTJ,
KTK,
KTL,
KTM,
KTN,
KTO,
KTP,
KTQ,
KTS,
KTT,
KTU,
KTV,
KTW,
KTX,
KTY,
KTZ,
KUB,
KUC,
KUD,
KUE,
KUF,
KUG,
KUH,
KUI,
KUJ,
KUK,
KUL,
KUN,
KUO,
KUP,
KUQ,
KUS,
KUU,
KUV,
KUW,
KUX,
KUY,
KUZ,
KVA,
KVB,
KVC,
KVD,
KVE,
KVF,
KVG,
KVH,
KVI,
KVJ,
KVK,
KVL,
KVM,
KVN,
KVO,
KVP,
KVQ,
KVR,
KVT,
KVU,
KVV,
KVW,
KVX,
KVY,
KVZ,
KWA,
KWB,
KWC,
KWD,
KWE,
KWF,
KWG,
KWH,
KWI,
KWJ,
KWK,
KWL,
KWM,
KWN,
KWO,
KWP,
KWR,
KWS,
KWT,
KWU,
KWV,
KWW,
KWX,
KWY,
KWZ,
KXA,
KXB,
KXC,
KXD,
KXF,
KXH,
KXI,
KXJ,
KXK,
KXM,
KXN,
KXO,
KXP,
KXQ,
KXR,
KXS,
KXT,
KXV,
KXW,
KXX,
KXY,
KXZ,
KYA,
KYB,
KYC,
KYD,
KYE,
KYF,
KYG,
KYH,
KYI,
KYJ,
KYK,
KYL,
KYM,
KYN,
KYO,
KYP,
KYQ,
KYR,
KYS,
KYT,
KYU,
KYV,
KYW,
KYX,
KYY,
KYZ,
KZA,
KZB,
KZC,
KZD,
KZE,
KZF,
KZG,
KZI,
KZK,
KZL,
KZM,
KZN,
KZO,
KZP,
KZQ,
KZR,
KZS,
KZU,
KZV,
KZW,
KZX,
KZY,
KZZ,
LAA,
LAB,
LAC,
LAE,
LAF,
LAG,
LAI,
LAJ,
LAL,
LAN,
LAP,
LAQ,
LAR,
LAS,
LAU,
LAW,
LAX,
LAY,
LAZ,
LBB,
LBC,
LBE,
LBF,
LBG,
LBI,
LBJ,
LBK,
LBL,
LBM,
LBN,
LBO,
LBQ,
LBR,
LBS,
LBT,
LBU,
LBV,
LBW,
LBX,
LBY,
LBZ,
LCC,
LCD,
LCE,
LCF,
LCH,
LCL,
LCM,
LCP,
LCQ,
LCS,
LDA,
LDB,
LDD,
LDG,
LDH,
LDI,
LDJ,
LDK,
LDL,
LDM,
LDN,
LDO,
LDP,
LDQ,
LEA,
LEB,
LEC,
LED,
LEE,
LEF,
LEH,
LEI,
LEJ,
LEK,
LEL,
LEM,
LEN,
LEO,
LEP,
LEQ,
LER,
LES,
LET,
LEU,
LEV,
LEW,
LEX,
LEY,
LFA,
LFN,
LGA,
LGB,
LGG,
LGH,
LGI,
LGK,
LGL,
LGM,
LGN,
LGO,
LGQ,
LGR,
LGT,
LGU,
LGZ,
LHA,
LHH,
LHI,
LHL,
LHM,
LHN,
LHP,
LHS,
LHT,
LHU,
LIA,
LIB,
LIC,
LID,
LIE,
LIF,
LIG,
LIH,
LIJ,
LIK,
LIL,
LIO,
LIP,
LIQ,
LIR,
LIS,
LIU,
LIV,
LIW,
LIX,
LIY,
LIZ,
LJA,
LJE,
LJI,
LJL,
LJP,
LJW,
LJX,
LKA,
LKB,
LKC,
LKD,
LKE,
LKH,
LKI,
LKJ,
LKL,
LKM,
LKN,
LKO,
LKR,
LKS,
LKT,
LKU,
LKY,
LLA,
LLB,
LLC,
LLD,
LLE,
LLF,
LLG,
LLH,
LLI,
LLJ,
LLK,
LLL,
LLM,
LLN,
LLP,
LLQ,
LLS,
LLU,
LLX,
LMA,
LMB,
LMC,
LMD,
LME,
LMF,
LMG,
LMH,
LMI,
LMJ,
LMK,
LML,
LMN,
LMO,
LMP,
LMQ,
LMR,
LMU,
LMV,
LMW,
LMX,
LMY,
LNA,
LNB,
LND,
LNG,
LNH,
LNI,
LNJ,
LNL,
LNM,
LNN,
LNS,
LNU,
LNW,
LNZ,
LOA,
LOB,
LOC,
LOE,
LOF,
LOG,
LOH,
LOI,
LOJ,
LOK,
LOM,
LON,
LOO,
LOP,
LOQ,
LOR,
LOS,
LOT,
LOU,
LOV,
LOW,
LOX,
LOY,
LPA,
LPE,
LPN,
LPO,
LPX,
LQR,
LRA,
LRC,
LRE,
LRG,
LRI,
LRK,
LRL,
LRM,
LRN,
LRO,
LRR,
LRT,
LRV,
LRZ,
LSA,
LSB,
LSC,
LSD,
LSE,
LSH,
LSI,
LSL,
LSM,
LSN,
LSO,
LSP,
LSR,
LSS,
LST,
LSV,
LSW,
LSY,
LTC,
LTG,
LTH,
LTI,
LTN,
LTO,
LTS,
LTU,
LUC,
LUD,
LUE,
LUF,
LUJ,
LUK,
LUL,
LUM,
LUP,
LUQ,
LUR,
LUT,
LUU,
LUV,
LUW,
LUY,
LUZ,
LVA,
LVI,
LVK,
LVS,
LVU,
LWA,
LWE,
LWG,
LWH,
LWL,
LWM,
LWO,
LWS,
LWT,
LWU,
LWW,
LXM,
LYA,
LYG,
LYN,
LZH,
LZL,
LZN,
LZZ,
MAA,
MAB,
MAE,
MAF,
MAJ,
MAM,
MAQ,
MAT,
MAU,
MAV,
MAW,
MAX,
MAZ,
MBA,
MBB,
MBC,
MBD,
MBE,
MBF,
MBH,
MBI,
MBJ,
MBK,
MBL,
MBM,
MBN,
MBO,
MBP,
MBQ,
MBR,
MBS,
MBT,
MBU,
MBV,
MBW,
MBX,
MBY,
MBZ,
MCA,
MCB,
MCC,
MCD,
MCE,
MCF,
MCG,
MCH,
MCI,
MCJ,
MCK,
MCL,
MCM,
MCN,
MCO,
MCP,
MCQ,
MCR,
MCS,
MCT,
MCU,
MCV,
MCW,
MCX,
MCY,
MCZ,
MDA,
MDB,
MDC,
MDD,
MDE,
MDG,
MDH,
MDI,
MDJ,
MDK,
MDL,
MDM,
MDN,
MDP,
MDQ,
MDS,
MDT,
MDU,
MDV,
MDW,
MDX,
MDY,
MDZ,
MEA,
MEB,
MEC,
MED,
MEE,
MEF,
MEH,
MEI,
MEJ,
MEK,
MEL,
MEM,
MEO,
MEP,
MEQ,
MER,
MES,
MET,
MEU,
MEV,
MEW,
MEY,
MEZ,
MFA,
MFB,
MFC,
MFD,
MFE,
MFF,
MFG,
MFH,
MFI,
MFJ,
MFK,
MFL,
MFM,
MFN,
MFO,
MFP,
MFQ,
MFR,
MFS,
MFT,
MFU,
MFV,
MFW,
MFX,
MFY,
MFZ,
MGB,
MGC,
MGD,
MGE,
MGF,
MGG,
MGH,
MGI,
MGJ,
MGK,
MGL,
MGM,
MGN,
MGO,
MGP,
MGQ,
MGR,
MGS,
MGT,
MGU,
MGV,
MGW,
MGY,
MGZ,
MHA,
MHB,
MHC,
MHD,
MHE,
MHF,
MHG,
MHI,
MHJ,
MHK,
MHL,
MHM,
MHN,
MHO,
MHP,
MHQ,
MHR,
MHS,
MHT,
MHU,
MHW,
MHX,
MHY,
MHZ,
MIA,
MIB,
MID,
MIE,
MIF,
MIG,
MIH,
MII,
MIJ,
MIK,
MIL,
MIM,
MIO,
MIP,
MIQ,
MIR,
MIT,
MIU,
MIW,
MIX,
MIY,
MIZ,
MJB,
MJC,
MJD,
MJE,
MJG,
MJH,
MJI,
MJJ,
MJK,
MJL,
MJM,
MJN,
MJO,
MJP,
MJQ,
MJR,
MJS,
MJT,
MJU,
MJV,
MJW,
MJX,
MJY,
MJZ,
MKA,
MKB,
MKC,
MKE,
MKF,
MKG,
MKI,
MKJ,
MKK,
MKL,
MKM,
MKN,
MKO,
MKP,
MKQ,
MKR,
MKS,
MKT,
MKU,
MKV,
MKW,
MKX,
MKY,
MKZ,
MLA,
MLB,
MLC,
MLE,
MLF,
MLH,
MLI,
MLJ,
MLK,
MLL,
MLM,
MLN,
MLO,
MLP,
MLQ,
MLR,
MLS,
MLU,
MLV,
MLW,
MLX,
MLZ,
MMA,
MMB,
MMC,
MMD,
MME,
MMF,
MMG,
MMH,
MMI,
MMJ,
MMK,
MML,
MMM,
MMN,
MMO,
MMP,
MMQ,
MMR,
MMT,
MMU,
MMV,
MMW,
MMX,
MMY,
MMZ,
MNA,
MNB,
MND,
MNE,
MNF,
MNG,
MNH,
MNJ,
MNK,
MNL,
MNM,
MNN,
MNP,
MNQ,
MNR,
MNS,
MNU,
MNV,
MNW,
MNX,
MNY,
MNZ,
MOA,
MOC,
MOD,
MOE,
MOG,
MOI,
MOJ,
MOK,
MOM,
MOO,
MOP,
MOQ,
MOR,
MOT,
MOU,
MOV,
MOW,
MOX,
MOY,
MOZ,
MPA,
MPB,
MPC,
MPD,
MPE,
MPG,
MPH,
MPI,
MPJ,
MPK,
MPL,
MPM,
MPN,
MPO,
MPP,
MPQ,
MPR,
MPS,
MPT,
MPU,
MPV,
MPW,
MPX,
MPY,
MPZ,
MQA,
MQB,
MQC,
MQE,
MQF,
MQG,
MQH,
MQI,
MQJ,
MQK,
MQL,
MQM,
MQN,
MQO,
MQP,
MQQ,
MQR,
MQS,
MQT,
MQU,
MQV,
MQW,
MQX,
MQY,
MQZ,
MRA,
MRB,
MRC,
MRD,
MRE,
MRF,
MRG,
MRH,
MRJ,
MRK,
MRL,
MRM,
MRN,
MRO,
MRP,
MRQ,
MRR,
MRS,
MRT,
MRU,
MRV,
MRW,
MRX,
MRY,
MRZ,
MSB,
MSC,
MSD,
MSE,
MSF,
MSG,
MSH,
MSI,
MSJ,
MSK,
MSL,
MSM,
MSN,
MSO,
MSP,
MSQ,
MSR,
MSS,
MSU,
MSV,
MSW,
MSX,
MSY,
MSZ,
MTA,
MTB,
MTC,
MTD,
MTE,
MTF,
MTG,
MTH,
MTI,
MTJ,
MTK,
MTL,
MTM,
MTN,
MTO,
MTP,
MTQ,
MTR,
MTS,
MTT,
MTU,
MTV,
MTW,
MTX,
MTY,
MUA,
MUB,
MUC,
MUD,
MUE,
MUG,
MUH,
MUI,
MUJ,
MUK,
MUM,
MUO,
MUP,
MUQ,
MUR,
MUT,
MUU,
MUV,
MUX,
MUY,
MUZ,
MVA,
MVB,
MVD,
MVE,
MVF,
MVG,
MVH,
MVI,
MVK,
MVL,
MVN,
MVO,
MVP,
MVQ,
MVR,
MVS,
MVT,
MVU,
MVV,
MVW,
MVX,
MVY,
MVZ,
MWA,
MWB,
MWC,
MWE,
MWF,
MWG,
MWH,
MWI,
MWK,
MWM,
MWN,
MWO,
MWP,
MWQ,
MWS,
MWT,
MWU,
MWV,
MWW,
MWZ,
MXA,
MXB,
MXC,
MXD,
MXE,
MXF,
MXG,
MXH,
MXI,
MXJ,
MXK,
MXL,
MXM,
MXN,
MXO,
MXP,
MXQ,
MXR,
MXS,
MXT,
MXU,
MXV,
MXW,
MXX,
MXY,
MXZ,
MYB,
MYC,
MYE,
MYF,
MYG,
MYH,
MYJ,
MYK,
MYL,
MYM,
MYO,
MYP,
MYR,
MYS,
MYU,
MYW,
MYX,
MYY,
MYZ,
MZA,
MZB,
MZC,
MZD,
MZE,
MZG,
MZH,
MZI,
MZJ,
MZK,
MZL,
MZM,
MZN,
MZO,
MZP,
MZQ,
MZR,
MZS,
MZT,
MZU,
MZV,
MZW,
MZX,
MZY,
MZZ,
NAA,
NAB,
NAC,
NAE,
NAF,
NAG,
NAJ,
NAK,
NAL,
NAM,
NAN,
NAO,
NAQ,
NAR,
NAS,
NAT,
NAW,
NAX,
NAY,
NAZ,
NBA,
NBB,
NBC,
NBD,
NBE,
NBG,
NBH,
NBI,
NBJ,
NBK,
NBM,
NBN,
NBO,
NBP,
NBQ,
NBR,
NBS,
NBT,
NBU,
NBV,
NBW,
NBY,
NCA,
NCB,
NCC,
NCD,
NCE,
NCF,
NCG,
NCH,
NCI,
NCJ,
NCK,
NCL,
NCM,
NCN,
NCO,
NCQ,
NCR,
NCS,
NCT,
NCU,
NCX,
NCZ,
NDA,
NDB,
NDC,
NDD,
NDF,
NDG,
NDH,
NDI,
NDJ,
NDK,
NDL,
NDM,
NDN,
NDP,
NDQ,
NDR,
NDT,
NDU,
NDV,
NDW,
NDX,
NDY,
NDZ,
NEA,
NEB,
NEC,
NED,
NEE,
NEF,
NEG,
NEH,
NEI,
NEJ,
NEK,
NEM,
NEN,
NEO,
NEQ,
NER,
NES,
NET,
NEU,
NEV,
NEX,
NEY,
NEZ,
NFA,
NFD,
NFL,
NFR,
NFU,
NGA,
NGB,
NGC,
NGD,
NGE,
NGG,
NGH,
NGI,
NGJ,
NGK,
NGL,
NGM,
NGN,
NGP,
NGQ,
NGR,
NGS,
NGT,
NGU,
NGV,
NGW,
NGX,
NGY,
NGZ,
NHA,
NHB,
NHC,
NHD,
NHE,
NHF,
NHG,
NHH,
NHI,
NHK,
NHM,
NHN,
NHO,
NHP,
NHQ,
NHR,
NHT,
NHU,
NHV,
NHW,
NHX,
NHY,
NHZ,
NIB,
NID,
NIE,
NIF,
NIG,
NIH,
NII,
NIJ,
NIK,
NIL,
NIM,
NIN,
NIO,
NIQ,
NIR,
NIS,
NIT,
NIV,
NIW,
NIX,
NIY,
NIZ,
NJA,
NJB,
NJD,
NJH,
NJI,
NJJ,
NJL,
NJM,
NJN,
NJO,
NJR,
NJS,
NJT,
NJU,
NJX,
NJY,
NJZ,
NKA,
NKB,
NKC,
NKD,
NKE,
NKF,
NKG,
NKH,
NKI,
NKJ,
NKK,
NKM,
NKN,
NKO,
NKP,
NKQ,
NKR,
NKS,
NKT,
NKU,
NKV,
NKW,
NKX,
NKZ,
NLA,
NLC,
NLE,
NLG,
NLI,
NLJ,
NLK,
NLL,
NLM,
NLO,
NLQ,
NLU,
NLV,
NLW,
NLX,
NLY,
NLZ,
NMA,
NMB,
NMC,
NMD,
NME,
NMF,
NMG,
NMH,
NMI,
NMJ,
NMK,
NML,
NMM,
NMN,
NMO,
NMP,
NMQ,
NMR,
NMS,
NMT,
NMU,
NMV,
NMW,
NMX,
NMY,
NMZ,
NNA,
NNB,
NNC,
NND,
NNE,
NNF,
NNG,
NNH,
NNI,
NNJ,
NNK,
NNL,
NNM,
NNN,
NNP,
NNQ,
NNR,
NNT,
NNU,
NNV,
NNW,
NNY,
NNZ,
NOA,
NOC,
NOD,
NOE,
NOF,
NOH,
NOI,
NOJ,
NOK,
NOL,
NOM,
NOP,
NOQ,
NOS,
NOT,
NOU,
NOV,
NOW,
NOY,
NOZ,
NPA,
NPB,
NPG,
NPH,
NPI,
NPL,
NPN,
NPO,
NPS,
NPU,
NPX,
NPY,
NQG,
NQK,
NQL,
NQM,
NQN,
NQQ,
NQT,
NQY,
NRA,
NRB,
NRC,
NRE,
NRF,
NRG,
NRI,
NRK,
NRL,
NRM,
NRN,
NRP,
NRR,
NRT,
NRU,
NRX,
NRZ,
NSA,
NSB,
NSC,
NSD,
NSE,
NSF,
NSG,
NSH,
NSI,
NSK,
NSL,
NSM,
NSN,
NSP,
NSQ,
NSR,
NSS,
NST,
NSU,
NSV,
NSW,
NSX,
NSY,
NSZ,
NTD,
NTE,
NTG,
NTI,
NTJ,
NTK,
NTM,
NTO,
NTP,
NTR,
NTU,
NTW,
NTX,
NTY,
NTZ,
NUA,
NUC,
NUD,
NUE,
NUF,
NUG,
NUH,
NUI,
NUJ,
NUK,
NUL,
NUM,
NUN,
NUO,
NUP,
NUQ,
NUR,
NUS,
NUT,
NUU,
NUV,
NUW,
NUX,
NUY,
NUZ,
NVH,
NVM,
NVO,
NWA,
NWB,
NWE,
NWG,
NWI,
NWM,
NWO,
NWR,
NWW,
NWX,
NWY,
NXA,
NXD,
NXE,
NXG,
NXI,
NXK,
NXL,
NXM,
NXN,
NXO,
NXQ,
NXR,
NXX,
NYB,
NYC,
NYD,
NYE,
NYF,
NYG,
NYH,
NYI,
NYJ,
NYK,
NYL,
NYP,
NYQ,
NYR,
NYS,
NYT,
NYU,
NYV,
NYW,
NYX,
NYY,
NZA,
NZB,
NZD,
NZK,
NZM,
NZS,
NZU,
NZY,
NZZ,
OAA,
OAC,
OAR,
OAV,
OBI,
OBK,
OBL,
OBM,
OBO,
OBR,
OBT,
OBU,
OCA,
OCH,
OCM,
OCO,
OCU,
ODA,
ODK,
ODT,
ODU,
OFO,
OFS,
OFU,
OGB,
OGC,
OGE,
OGG,
OGO,
OGU,
OHT,
OHU,
OIA,
OIE,
OIN,
OJB,
OJC,
OJG,
OJP,
OJS,
OJV,
OJW,
OKA,
OKB,
OKC,
OKD,
OKE,
OKG,
OKH,
OKI,
OKJ,
OKK,
OKL,
OKM,
OKN,
OKO,
OKR,
OKS,
OKU,
OKV,
OKX,
OKZ,
OLA,
OLD,
OLE,
OLK,
OLM,
OLO,
OLR,
OLT,
OLU,
OMA,
OMB,
OMC,
OMG,
OMI,
OMK,
OML,
OMN,
OMO,
OMP,
OMR,
OMT,
OMU,
OMW,
OMX,
OMY,
ONA,
ONB,
ONE,
ONG,
ONI,
ONJ,
ONK,
ONN,
ONO,
ONP,
ONR,
ONS,
ONT,
ONU,
ONW,
ONX,
OOD,
OOG,
OON,
OOR,
OOS,
OPA,
OPK,
OPM,
OPO,
OPT,
OPY,
ORA,
ORC,
ORE,
ORG,
ORH,
ORN,
ORO,
ORR,
ORS,
ORT,
ORU,
ORV,
ORW,
ORX,
ORY,
ORZ,
OSC,
OSI,
OSN,
OSO,
OSP,
OST,
OSU,
OSX,
OTB,
OTD,
OTE,
OTI,
OTK,
OTL,
OTM,
OTN,
OTQ,
OTR,
OTS,
OTT,
OTU,
OTW,
OTX,
OTY,
OTZ,
OUA,
OUB,
OUE,
OUI,
OUM,
OVD,
OWI,
OWL,
OYB,
OYD,
OYM,
OYY,
OZM,
PAB,
PAC,
PAD,
PAE,
PAF,
PAH,
PAI,
PAK,
PAO,
PAQ,
PAR,
PAS,
PAV,
PAW,
PAX,
PAY,
PAZ,
PBB,
PBC,
PBE,
PBF,
PBG,
PBH,
PBI,
PBL,
PBM,
PBN,
PBO,
PBP,
PBR,
PBS,
PBT,
PBU,
PBV,
PBY,
PCA,
PCB,
PCC,
PCD,
PCE,
PCF,
PCG,
PCH,
PCI,
PCJ,
PCK,
PCL,
PCM,
PCN,
PCP,
PCW,
PDA,
PDC,
PDI,
PDN,
PDO,
PDT,
PDU,
PEA,
PEB,
PED,
PEE,
PEF,
PEG,
PEH,
PEI,
PEJ,
PEK,
PEL,
PEM,
PEP,
PEQ,
PES,
PEV,
PEX,
PEY,
PEZ,
PFA,
PFE,
PFL,
PGA,
PGD,
PGG,
PGI,
PGK,
PGL,
PGN,
PGS,
PGU,
PGZ,
PHA,
PHD,
PHG,
PHH,
PHJ,
PHK,
PHL,
PHM,
PHO,
PHQ,
PHR,
PHT,
PHU,
PHV,
PHW,
PIA,
PIB,
PIC,
PID,
PIE,
PIF,
PIG,
PIH,
PIJ,
PIL,
PIM,
PIN,
PIO,
PIP,
PIR,
PIS,
PIT,
PIU,
PIV,
PIW,
PIX,
PIY,
PIZ,
PJT,
PKA,
PKB,
PKC,
PKG,
PKH,
PKN,
PKO,
PKP,
PKR,
PKS,
PKT,
PKU,
PLA,
PLB,
PLC,
PLD,
PLE,
PLG,
PLH,
PLJ,
PLK,
PLL,
PLN,
PLO,
PLQ,
PLR,
PLS,
PLT,
PLU,
PLV,
PLW,
PLY,
PLZ,
PMA,
PMB,
PMD,
PME,
PMF,
PMH,
PMI,
PMJ,
PMK,
PML,
PMM,
PMN,
PMO,
PMQ,
PMR,
PMS,
PMT,
PMW,
PMX,
PMY,
PMZ,
PNA,
PNB,
PNC,
PND,
PNE,
PNG,
PNH,
PNI,
PNJ,
PNK,
PNL,
PNM,
PNN,
PNO,
PNP,
PNQ,
PNR,
PNS,
PNT,
PNU,
PNV,
PNW,
PNX,
PNY,
PNZ,
POC,
POE,
POF,
POG,
POH,
POI,
POK,
POM,
POO,
POP,
POQ,
POS,
POT,
POV,
POW,
POX,
POY,
PPE,
PPI,
PPK,
PPL,
PPM,
PPN,
PPO,
PPP,
PPQ,
PPS,
PPT,
PPU,
PQA,
PQM,
PRC,
PRD,
PRE,
PRF,
PRG,
PRH,
PRI,
PRK,
PRL,
PRM,
PRN,
PRP,
PRQ,
PRR,
PRS,
PRT,
PRU,
PRW,
PRX,
PRZ,
PSA,
PSC,
PSD,
PSE,
PSG,
PSH,
PSI,
PSL,
PSM,
PSN,
PSO,
PSP,
PSQ,
PSR,
PSS,
PST,
PSU,
PSW,
PSY,
PTA,
PTH,
PTI,
PTN,
PTO,
PTP,
PTQ,
PTR,
PTT,
PTU,
PTV,
PTW,
PTY,
PUA,
PUB,
PUC,
PUD,
PUE,
PUF,
PUG,
PUI,
PUJ,
PUM,
PUO,
PUP,
PUQ,
PUR,
PUT,
PUU,
PUW,
PUX,
PUY,
PWA,
PWB,
PWG,
PWI,
PWM,
PWN,
PWO,
PWR,
PWW,
PXM,
PYE,
PYM,
PYN,
PYS,
PYU,
PYX,
PYY,
PZH,
PZN,
QUA,
QUB,
QUC,
QUD,
QUF,
QUG,
QUH,
QUI,
QUK,
QUL,
QUM,
QUN,
QUP,
QUQ,
QUR,
QUS,
QUV,
QUW,
QUX,
QUY,
QUZ,
QVA,
QVC,
QVE,
QVH,
QVI,
QVJ,
QVL,
QVM,
QVN,
QVO,
QVP,
QVS,
QVW,
QVY,
QVZ,
QWA,
QWC,
QWH,
QWM,
QWS,
QWT,
QXA,
QXC,
QXH,
QXL,
QXN,
QXO,
QXP,
QXQ,
QXR,
QXS,
QXT,
QXU,
QXW,
QYA,
QYP,
RAA,
RAB,
RAC,
RAD,
RAF,
RAG,
RAH,
RAI,
RAK,
RAL,
RAM,
RAN,
RAO,
RAQ,
RAS,
RAT,
RAU,
RAV,
RAW,
RAX,
RAY,
RAZ,
RBB,
RBK,
RBL,
RBP,
RCF,
RDB,
REA,
REB,
REE,
REG,
REI,
REJ,
REL,
REM,
REN,
RER,
RES,
RET,
REY,
RGA,
RGE,
RGK,
RGN,
RGR,
RGS,
RGU,
RHG,
RHP,
RIA,
RIB,
RIF,
RIL,
RIM,
RIN,
RIR,
RIT,
RIU,
RJG,
RJI,
RJS,
RKA,
RKB,
RKH,
RKI,
RKM,
RKT,
RKW,
RMA,
RMB,
RMC,
RMD,
RME,
RMF,
RMG,
RMH,
RMI,
RMK,
RML,
RMM,
RMN,
RMO,
RMP,
RMQ,
RMS,
RMT,
RMU,
RMV,
RMW,
RMX,
RMY,
RMZ,
RNB,
RND,
RNG,
RNL,
RNN,
RNP,
RNR,
RNW,
ROB,
ROC,
ROD,
ROE,
ROF,
ROG,
ROL,
ROO,
ROP,
ROR,
ROU,
ROW,
RPN,
RPT,
RRI,
RRO,
RRT,
RSB,
RSK,
RSL,
RSM,
RSN,
RTC,
RTH,
RTM,
RTS,
RTW,
RUB,
RUC,
RUE,
RUF,
RUG,
RUH,
RUI,
RUK,
RUO,
RUQ,
RUT,
RUU,
RUY,
RUZ,
RWA,
RWK,
RWL,
RWM,
RWO,
RWR,
RXD,
RXW,
RYN,
RYS,
RYU,
RZH,
SAA,
SAB,
SAC,
SAE,
SAF,
SAJ,
SAK,
SAO,
SAQ,
SAR,
SAU,
SAV,
SAW,
SAX,
SAY,
SAZ,
SBA,
SBB,
SBC,
SBD,
SBE,
SBF,
SBG,
SBH,
SBI,
SBJ,
SBK,
SBL,
SBM,
SBN,
SBO,
SBP,
SBQ,
SBR,
SBS,
SBT,
SBU,
SBV,
SBW,
SBX,
SBY,
SBZ,
SCB,
SCE,
SCF,
SCG,
SCH,
SCI,
SCK,
SCL,
SCP,
SCQ,
SCS,
SCT,
SCU,
SCV,
SCW,
SCX,
SDA,
SDB,
SDC,
SDE,
SDF,
SDG,
SDH,
SDJ,
SDK,
SDL,
SDN,
SDO,
SDP,
SDQ,
SDR,
SDS,
SDT,
SDU,
SDX,
SDZ,
SEA,
SEB,
SEC,
SED,
SEE,
SEF,
SEG,
SEH,
SEI,
SEJ,
SEK,
SEN,
SEO,
SEP,
SEQ,
SER,
SES,
SET,
SEU,
SEV,
SEW,
SEY,
SEZ,
SFB,
SFE,
SFM,
SFS,
SFW,
SGB,
SGC,
SGD,
SGE,
SGG,
SGH,
SGI,
SGJ,
SGK,
SGM,
SGP,
SGR,
SGS,
SGT,
SGU,
SGW,
SGX,
SGY,
SGZ,
SHA,
SHB,
SHC,
SHD,
SHE,
SHG,
SHH,
SHI,
SHJ,
SHK,
SHL,
SHM,
SHO,
SHP,
SHQ,
SHR,
SHS,
SHT,
SHU,
SHV,
SHW,
SHX,
SHY,
SHZ,
SIA,
SIB,
SIE,
SIF,
SIG,
SIH,
SII,
SIJ,
SIK,
SIL,
SIM,
SIP,
SIQ,
SIR,
SIS,
SIU,
SIV,
SIW,
SIX,
SIY,
SIZ,
SJA,
SJB,
SJD,
SJE,
SJG,
SJK,
SJL,
SJM,
SJN,
SJO,
SJP,
SJR,
SJS,
SJT,
SJU,
SJW,
SKA,
SKB,
SKC,
SKD,
SKE,
SKF,
SKG,
SKH,
SKI,
SKJ,
SKM,
SKN,
SKO,
SKP,
SKQ,
SKR,
SKS,
SKT,
SKU,
SKV,
SKW,
SKX,
SKY,
SKZ,
SLC,
SLD,
SLE,
SLF,
SLG,
SLH,
SLI,
SLJ,
SLL,
SLM,
SLN,
SLP,
SLQ,
SLR,
SLS,
SLT,
SLU,
SLW,
SLX,
SLY,
SLZ,
SMB,
SMC,
SMF,
SMG,
SMH,
SMK,
SML,
SMM,
SMP,
SMQ,
SMR,
SMT,
SMU,
SMV,
SMW,
SMX,
SMY,
SMZ,
SNC,
SNE,
SNF,
SNG,
SNI,
SNJ,
SNL,
SNM,
SNN,
SNO,
SNP,
SNQ,
SNR,
SNS,
SNU,
SNV,
SNW,
SNX,
SNY,
SNZ,
SOA,
SOB,
SOC,
SOD,
SOE,
SOH,
SOI,
SOJ,
SOK,
SOL,
SOO,
SOP,
SOQ,
SOR,
SOS,
SOU,
SOV,
SOW,
SOX,
SOY,
SOZ,
SPB,
SPC,
SPD,
SPE,
SPG,
SPI,
SPK,
SPL,
SPM,
SPN,
SPO,
SPP,
SPQ,
SPR,
SPS,
SPT,
SPU,
SPV,
SPX,
SPY,
SQA,
SQH,
SQK,
SQM,
SQN,
SQO,
SQQ,
SQR,
SQS,
SQT,
SQU,
SQX,
SRA,
SRB,
SRC,
SRE,
SRF,
SRG,
SRH,
SRI,
SRK,
SRL,
SRM,
SRO,
SRQ,
SRS,
SRT,
SRU,
SRV,
SRW,
SRX,
SRY,
SRZ,
SSB,
SSC,
SSD,
SSE,
SSF,
SSG,
SSH,
SSI,
SSJ,
SSK,
SSL,
SSM,
SSN,
SSO,
SSP,
SSQ,
SSR,
SSS,
SST,
SSU,
SSV,
SSX,
SSY,
SSZ,
STA,
STB,
STD,
STE,
STF,
STG,
STH,
STI,
STJ,
STK,
STL,
STM,
STN,
STO,
STP,
STQ,
STR,
STS,
STT,
STU,
STV,
STW,
STY,
SUA,
SUB,
SUC,
SUE,
SUG,
SUI,
SUJ,
SUO,
SUQ,
SUR,
SUT,
SUV,
SUW,
SUY,
SUZ,
SVA,
SVB,
SVC,
SVE,
SVK,
SVM,
SVS,
SVX,
SWB,
SWC,
SWF,
SWG,
SWH,
SWI,
SWJ,
SWK,
SWL,
SWM,
SWN,
SWO,
SWP,
SWQ,
SWR,
SWS,
SWT,
SWU,
SWV,
SWW,
SWX,
SWY,
SXB,
SXC,
SXE,
SXG,
SXK,
SXL,
SXM,
SXN,
SXO,
SXR,
SXS,
SXU,
SXW,
SYA,
SYB,
SYI,
SYK,
SYL,
SYM,
SYN,
SYO,
SYS,
SYW,
SYX,
SYY,
SZA,
SZB,
SZC,
SZD,
SZE,
SZG,
SZL,
SZN,
SZP,
SZS,
SZV,
SZW,
SZY,
TAA,
TAB,
TAC,
TAD,
TAE,
TAF,
TAG,
TAJ,
TAK,
TAL,
TAN,
TAO,
TAP,
TAQ,
TAR,
TAS,
TAU,
TAV,
TAW,
TAX,
TAY,
TAZ,
TBA,
TBC,
TBD,
TBE,
TBF,
TBG,
TBH,
TBI,
TBJ,
TBK,
TBL,
TBM,
TBN,
TBO,
TBP,
TBR,
TBS,
TBT,
TBU,
TBV,
TBW,
TBX,
TBY,
TBZ,
TCA,
TCB,
TCC,
TCD,
TCE,
TCF,
TCG,
TCH,
TCI,
TCK,
TCL,
TCM,
TCN,
TCO,
TCP,
TCQ,
TCS,
TCT,
TCU,
TCW,
TCX,
TCY,
TCZ,
TDA,
TDB,
TDC,
TDD,
TDE,
TDF,
TDG,
TDH,
TDI,
TDJ,
TDK,
TDL,
TDM,
TDN,
TDO,
TDQ,
TDR,
TDS,
TDT,
TDV,
TDX,
TDY,
TEA,
TEB,
TEC,
TED,
TEE,
TEF,
TEG,
TEH,
TEI,
TEK,
TEN,
TEO,
TEP,
TEQ,
TES,
TEU,
TEV,
TEW,
TEX,
TEY,
TEZ,
TFI,
TFN,
TFO,
TFR,
TFT,
TGA,
TGB,
TGC,
TGD,
TGE,
TGF,
TGH,
TGI,
TGJ,
TGN,
TGO,
TGP,
TGQ,
TGR,
TGS,
TGT,
TGU,
TGV,
TGW,
TGX,
TGY,
TGZ,
THD,
THE,
THF,
THH,
THI,
THK,
THL,
THM,
THN,
THP,
THQ,
THR,
THS,
THT,
THU,
THV,
THY,
THZ,
TIA,
TIC,
TIF,
TIH,
TII,
TIJ,
TIK,
TIL,
TIM,
TIN,
TIO,
TIP,
TIQ,
TIS,
TIT,
TIU,
TIW,
TIX,
TIY,
TIZ,
TJA,
TJG,
TJI,
TJJ,
TJL,
TJM,
TJN,
TJO,
TJP,
TJS,
TJU,
TJW,
TKA,
TKB,
TKD,
TKE,
TKF,
TKG,
TKM,
TKN,
TKP,
TKQ,
TKR,
TKS,
TKT,
TKU,
TKV,
TKW,
TKX,
TKZ,
TLA,
TLB,
TLC,
TLD,
TLF,
TLG,
TLJ,
TLK,
TLL,
TLM,
TLN,
TLO,
TLP,
TLQ,
TLR,
TLS,
TLT,
TLU,
TLV,
TLX,
TLY,
TMA,
TMB,
TMC,
TMD,
TME,
TMF,
TMG,
TMI,
TMJ,
TMK,
TML,
TMM,
TMN,
TMO,
TMQ,
TMR,
TMS,
TMT,
TMU,
TMV,
TMW,
TMY,
TMZ,
TNA,
TNB,
TNC,
TND,
TNG,
TNH,
TNI,
TNK,
TNL,
TNM,
TNN,
TNO,
TNP,
TNQ,
TNR,
TNS,
TNT,
TNU,
TNV,
TNW,
TNX,
TNY,
TNZ,
TOB,
TOC,
TOD,
TOF,
TOH,
TOI,
TOJ,
TOK,
TOL,
TOM,
TOO,
TOP,
TOQ,
TOR,
TOS,
TOU,
TOV,
TOW,
TOX,
TOY,
TOZ,
TPA,
TPC,
TPE,
TPF,
TPG,
TPJ,
TPK,
TPL,
TPM,
TPN,
TPO,
TPP,
TPQ,
TPR,
TPT,
TPU,
TPV,
TPW,
TPX,
TPY,
TPZ,
TQB,
TQL,
TQM,
TQN,
TQO,
TQP,
TQQ,
TQR,
TQT,
TQU,
TQW,
TRA,
TRB,
TRC,
TRD,
TRE,
TRF,
TRG,
TRH,
TRI,
TRJ,
TRL,
TRM,
TRN,
TRO,
TRP,
TRQ,
TRR,
TRS,
TRT,
TRU,
TRV,
TRW,
TRX,
TRY,
TRZ,
TSA,
TSB,
TSC,
TSD,
TSE,
TSG,
TSH,
TSJ,
TSK,
TSL,
TSM,
TSP,
TSQ,
TSR,
TSS,
TST,
TSU,
TSV,
TSW,
TSX,
TSY,
TSZ,
TTA,
TTB,
TTC,
TTD,
TTE,
TTF,
TTG,
TTH,
TTI,
TTJ,
TTK,
TTL,
TTM,
TTN,
TTO,
TTP,
TTQ,
TTR,
TTS,
TTT,
TTU,
TTV,
TTW,
TTY,
TTZ,
TUA,
TUB,
TUC,
TUD,
TUE,
TUF,
TUG,
TUH,
TUI,
TUJ,
TUL,
TUN,
TUO,
TUQ,
TUS,
TUU,
TUV,
TUX,
TUY,
TUZ,
TVA,
TVD,
TVE,
TVK,
TVM,
TVN,
TVO,
TVS,
TVT,
TVU,
TVW,
TVX,
TVY,
TWA,
TWB,
TWC,
TWD,
TWE,
TWF,
TWG,
TWH,
TWL,
TWM,
TWN,
TWO,
TWP,
TWQ,
TWR,
TWT,
TWU,
TWW,
TWX,
TWY,
TXA,
TXB,
TXC,
TXE,
TXG,
TXH,
TXI,
TXJ,
TXM,
TXN,
TXO,
TXQ,
TXR,
TXS,
TXT,
TXU,
TXX,
TXY,
TYA,
TYE,
TYH,
TYI,
TYJ,
TYL,
TYN,
TYP,
TYR,
TYS,
TYT,
TYU,
TYX,
TYY,
TYZ,
TZA,
TZH,
TZJ,
TZL,
TZM,
TZN,
TZO,
TZX,
UAM,
UAN,
UAR,
UBA,
UBI,
UBL,
UBR,
UBU,
UBY,
UDA,
UDE,
UDG,
UDI,
UDJ,
UDL,
UDU,
UES,
UFI,
UGB,
UGE,
UGH,
UGN,
UGO,
UGY,
UHA,
UHN,
UIS,
UIV,
UJI,
UKA,
UKG,
UKH,
UKI,
UKK,
UKL,
UKP,
UKQ,
UKS,
UKU,
UKV,
UKW,
UKY,
ULA,
ULB,
ULC,
ULE,
ULF,
ULI,
ULK,
ULL,
ULM,
ULN,
ULU,
ULW,
UMA,
UMC,
UMD,
UMG,
UMI,
UMM,
UMN,
UMO,
UMP,
UMR,
UMS,
UMU,
UNA,
UNE,
UNG,
UNI,
UNK,
UNM,
UNN,
UNR,
UNU,
UNX,
UNZ,
UON,
UPI,
UPV,
URA,
URB,
URC,
URE,
URF,
URG,
URH,
URI,
URK,
URL,
URM,
URN,
URO,
URP,
URR,
URT,
URU,
URV,
URW,
URX,
URY,
URZ,
USA,
USH,
USI,
USK,
USP,
USS,
USU,
UTA,
UTE,
UTH,
UTP,
UTR,
UTU,
UUM,
UUR,
UUU,
UVE,
UVH,
UVL,
UWA,
UYA,
UZN,
UZS,
VAA,
VAE,
VAF,
VAG,
VAH,
VAJ,
VAL,
VAM,
VAN,
VAO,
VAP,
VAR,
VAS,
VAU,
VAV,
VAY,
VBB,
VBK,
VEC,
VED,
VEL,
VEM,
VEO,
VEP,
VER,
VGR,
VGT,
VIC,
VID,
VIF,
VIG,
VIL,
VIN,
VIS,
VIT,
VIV,
VKA,
VKJ,
VKK,
VKL,
VKM,
VKN,
VKO,
VKP,
VKT,
VKU,
VKZ,
VLP,
VLS,
VMA,
VMB,
VMC,
VMD,
VME,
VMF,
VMG,
VMH,
VMI,
VMJ,
VMK,
VML,
VMM,
VMP,
VMQ,
VMR,
VMS,
VMU,
VMV,
VMW,
VMX,
VMY,
VMZ,
VNK,
VNM,
VNP,
VOR,
VRA,
VRO,
VRS,
VRT,
VSI,
VSL,
VSV,
VTO,
VUM,
VUN,
VUT,
VWA,
WAA,
WAB,
WAC,
WAD,
WAE,
WAF,
WAG,
WAH,
WAI,
WAJ,
WAM,
WAN,
WAO,
WAP,
WAQ,
WAT,
WAU,
WAV,
WAW,
WAX,
WAY,
WAZ,
WBA,
WBB,
WBE,
WBF,
WBH,
WBI,
WBJ,
WBK,
WBL,
WBM,
WBP,
WBQ,
WBR,
WBS,
WBT,
WBV,
WBW,
WCA,
WCI,
WDD,
WDG,
WDJ,
WDK,
WDT,
WDU,
WDY,
WEA,
WEC,
WED,
WEG,
WEH,
WEI,
WEM,
WEO,
WEP,
WER,
WES,
WET,
WEU,
WEW,
WFG,
WGA,
WGB,
WGG,
WGI,
WGO,
WGU,
WGY,
WHA,
WHG,
WHK,
WHU,
WIB,
WIC,
WIE,
WIF,
WIG,
WIH,
WII,
WIJ,
WIK,
WIL,
WIM,
WIN,
WIR,
WIU,
WIV,
WIY,
WJA,
WJI,
WKA,
WKB,
WKD,
WKL,
WKR,
WKU,
WKW,
WKY,
WLA,
WLC,
WLE,
WLG,
WLH,
WLI,
WLK,
WLL,
WLM,
WLO,
WLR,
WLS,
WLU,
WLV,
WLW,
WLX,
WLY,
WMA,
WMB,
WMC,
WMD,
WME,
WMG,
WMH,
WMI,
WMM,
WMN,
WMO,
WMS,
WMT,
WMW,
WMX,
WNB,
WNC,
WND,
WNE,
WNG,
WNI,
WNK,
WNM,
WNN,
WNO,
WNP,
WNU,
WNW,
WNY,
WOA,
WOB,
WOC,
WOD,
WOE,
WOF,
WOG,
WOI,
WOK,
WOM,
WON,
WOO,
WOR,
WOS,
WOW,
WOY,
WPC,
WRB,
WRG,
WRH,
WRI,
WRK,
WRL,
WRM,
WRN,
WRO,
WRP,
WRR,
WRS,
WRU,
WRV,
WRW,
WRX,
WRY,
WRZ,
WSA,
WSG,
WSI,
WSK,
WSR,
WSS,
WSU,
WSV,
WTF,
WTH,
WTI,
WTK,
WTM,
WTW,
WUA,
WUB,
WUD,
WUH,
WUL,
WUM,
WUN,
WUR,
WUT,
WUU,
WUV,
WUX,
WUY,
WWA,
WWB,
WWO,
WWR,
WWW,
WXA,
WXW,
WYB,
WYI,
WYM,
WYN,
WYR,
WYY,
XAA,
XAB,
XAC,
XAD,
XAE,
XAG,
XAI,
XAJ,
XAK,
XAM,
XAN,
XAO,
XAP,
XAQ,
XAR,
XAS,
XAT,
XAU,
XAV,
XAW,
XAY,
XBB,
XBC,
XBD,
XBE,
XBG,
XBI,
XBJ,
XBM,
XBN,
XBO,
XBP,
XBR,
XBW,
XBY,
XCB,
XCC,
XCE,
XCG,
XCH,
XCL,
XCM,
XCN,
XCO,
XCR,
XCT,
XCU,
XCV,
XCW,
XCY,
XDA,
XDC,
XDK,
XDM,
XDO,
XDQ,
XDY,
XEB,
XED,
XEG,
XEL,
XEM,
XEP,
XER,
XES,
XET,
XEU,
XFA,
XGA,
XGB,
XGD,
XGF,
XGG,
XGI,
XGL,
XGM,
XGR,
XGU,
XGW,
XHA,
XHC,
XHD,
XHE,
XHM,
XHR,
XHT,
XHU,
XHV,
XIB,
XII,
XIL,
XIN,
XIR,
XIS,
XIV,
XIY,
XJB,
XJT,
XKA,
XKB,
XKC,
XKD,
XKE,
XKF,
XKG,
XKI,
XKJ,
XKK,
XKL,
XKN,
XKO,
XKP,
XKQ,
XKR,
XKS,
XKT,
XKU,
XKV,
XKW,
XKX,
XKY,
XKZ,
XLA,
XLB,
XLC,
XLD,
XLE,
XLG,
XLI,
XLN,
XLO,
XLP,
XLS,
XLU,
XLY,
XMA,
XMB,
XMC,
XMD,
XME,
XMF,
XMG,
XMH,
XMJ,
XMK,
XML,
XMM,
XMN,
XMO,
XMP,
XMQ,
XMR,
XMS,
XMT,
XMU,
XMV,
XMW,
XMX,
XMY,
XMZ,
XNA,
XNB,
XNG,
XNH,
XNI,
XNJ,
XNK,
XNM,
XNN,
XNO,
XNQ,
XNR,
XNS,
XNT,
XNU,
XNY,
XNZ,
XOC,
XOD,
XOG,
XOI,
XOK,
XOM,
XON,
XOO,
XOP,
XOR,
XOW,
XPA,
XPB,
XPC,
XPD,
XPE,
XPF,
XPG,
XPH,
XPI,
XPJ,
XPK,
XPL,
XPM,
XPN,
XPO,
XPP,
XPQ,
XPR,
XPS,
XPT,
XPU,
XPV,
XPW,
XPX,
XPY,
XPZ,
XQA,
XQT,
XRA,
XRB,
XRD,
XRE,
XRG,
XRI,
XRM,
XRN,
XRR,
XRT,
XRU,
XRW,
XSA,
XSB,
XSC,
XSD,
XSE,
XSH,
XSI,
XSJ,
XSL,
XSM,
XSN,
XSO,
XSP,
XSQ,
XSR,
XSS,
XSU,
XSV,
XSY,
XTA,
XTB,
XTC,
XTD,
XTE,
XTG,
XTH,
XTI,
XTJ,
XTL,
XTM,
XTN,
XTO,
XTP,
XTQ,
XTR,
XTS,
XTT,
XTU,
XTV,
XTW,
XTY,
XUA,
XUB,
XUD,
XUG,
XUJ,
XUL,
XUM,
XUN,
XUO,
XUP,
XUR,
XUT,
XUU,
XVE,
XVI,
XVN,
XVO,
XVS,
XWA,
XWC,
XWD,
XWE,
XWG,
XWJ,
XWK,
XWL,
XWO,
XWR,
XWT,
XWW,
XXB,
XXK,
XXM,
XXR,
XXT,
XYA,
XYB,
XYJ,
XYK,
XYL,
XYT,
XYY,
XZH,
XZM,
XZP,
YAA,
YAB,
YAC,
YAD,
YAE,
YAF,
YAG,
YAH,
YAI,
YAJ,
YAK,
YAL,
YAM,
YAN,
YAQ,
YAR,
YAS,
YAT,
YAU,
YAV,
YAW,
YAX,
YAY,
YAZ,
YBA,
YBB,
YBE,
YBH,
YBI,
YBJ,
YBK,
YBL,
YBM,
YBN,
YBO,
YBX,
YBY,
YCH,
YCL,
YCN,
YCP,
YDA,
YDD,
YDE,
YDG,
YDK,
YEA,
YEC,
YEE,
YEI,
YEJ,
YEL,
YER,
YES,
YET,
YEU,
YEV,
YEY,
YGA,
YGI,
YGL,
YGM,
YGP,
YGR,
YGS,
YGU,
YGW,
YHA,
YHD,
YHL,
YHS,
YIA,
YIF,
YIG,
YIH,
YII,
YIJ,
YIK,
YIL,
YIM,
YIN,
YIP,
YIQ,
YIR,
YIS,
YIT,
YIU,
YIV,
YIX,
YIZ,
YKA,
YKG,
YKI,
YKK,
YKL,
YKM,
YKN,
YKO,
YKR,
YKT,
YKU,
YKY,
YLA,
YLB,
YLE,
YLG,
YLI,
YLL,
YLM,
YLN,
YLO,
YLR,
YLU,
YLY,
YMB,
YMC,
YMD,
YME,
YMG,
YMH,
YMI,
YMK,
YML,
YMM,
YMN,
YMO,
YMP,
YMQ,
YMR,
YMS,
YMX,
YMZ,
YNA,
YND,
YNE,
YNG,
YNK,
YNL,
YNN,
YNO,
YNQ,
YNS,
YNU,
YOB,
YOG,
YOI,
YOK,
YOL,
YOM,
YON,
YOT,
YOX,
YOY,
YPA,
YPB,
YPG,
YPH,
YPM,
YPN,
YPO,
YPP,
YPZ,
YRA,
YRB,
YRE,
YRK,
YRL,
YRM,
YRN,
YRO,
YRS,
YRW,
YRY,
YSC,
YSD,
YSG,
YSL,
YSM,
YSN,
YSO,
YSP,
YSR,
YSS,
YSY,
YTA,
YTL,
YTP,
YTW,
YTY,
YUA,
YUB,
YUC,
YUD,
YUE,
YUF,
YUG,
YUI,
YUJ,
YUK,
YUL,
YUM,
YUN,
YUP,
YUQ,
YUR,
YUT,
YUW,
YUX,
YUY,
YUZ,
YVA,
YVT,
YWA,
YWG,
YWL,
YWN,
YWQ,
YWR,
YWT,
YWU,
YWW,
YXA,
YXG,
YXL,
YXM,
YXU,
YXY,
YYR,
YYU,
YYZ,
YZG,
YZK,
ZAA,
ZAB,
ZAC,
ZAD,
ZAE,
ZAF,
ZAG,
ZAH,
ZAI,
ZAJ,
ZAK,
ZAL,
ZAM,
ZAO,
ZAQ,
ZAR,
ZAS,
ZAT,
ZAU,
ZAV,
ZAW,
ZAX,
ZAY,
ZAZ,
ZBA,
ZBC,
ZBE,
ZBT,
ZBU,
ZBW,
ZCA,
ZCD,
ZCH,
ZDJ,
ZEA,
ZEG,
ZEH,
ZGA,
ZGB,
ZGM,
ZGN,
ZGR,
ZHB,
ZHD,
ZHI,
ZHN,
ZHW,
ZIA,
ZIB,
ZIK,
ZIL,
ZIM,
ZIN,
ZIW,
ZIZ,
ZKA,
ZKB,
ZKD,
ZKG,
ZKH,
ZKK,
ZKN,
ZKO,
ZKP,
ZKR,
ZKT,
ZKU,
ZKV,
ZKZ,
ZLA,
ZLJ,
ZLM,
ZLN,
ZLQ,
ZMA,
ZMB,
ZMC,
ZMD,
ZME,
ZMF,
ZMG,
ZMH,
ZMI,
ZMJ,
ZMK,
ZML,
ZMM,
ZMN,
ZMO,
ZMP,
ZMQ,
ZMR,
ZMS,
ZMT,
ZMU,
ZMV,
ZMW,
ZMX,
ZMY,
ZMZ,
ZNA,
ZNE,
ZNG,
ZNK,
ZNS,
ZOC,
ZOH,
ZOM,
ZOO,
ZOQ,
ZOR,
ZOS,
ZPA,
ZPB,
ZPC,
ZPD,
ZPE,
ZPF,
ZPG,
ZPH,
ZPI,
ZPJ,
ZPK,
ZPL,
ZPM,
ZPN,
ZPO,
ZPP,
ZPQ,
ZPR,
ZPS,
ZPT,
ZPU,
ZPV,
ZPW,
ZPX,
ZPY,
ZPZ,
ZQE,
ZRA,
ZRG,
ZRN,
ZRO,
ZRP,
ZRS,
ZSA,
ZSK,
ZSL,
ZSM,
ZSR,
ZSU,
ZTE,
ZTG,
ZTL,
ZTM,
ZTN,
ZTP,
ZTQ,
ZTS,
ZTT,
ZTU,
ZTX,
ZTY,
ZUA,
ZUH,
ZUM,
ZUY,
ZWA,
ZYB,
ZYG,
ZYJ,
ZYN,
ZYP,
ZZJ,

];


pub const ALL_1: & [LanguageCode] = &[
AB,
AA,
AF,
AK,
SQ,
AM,
AR,
AN,
HY,
AS,
AV,
AE,
AY,
AZ,
BM,
BA,
EU,
BE,
BN,
BI,
BS,
BR,
BG,
MY,
CA,
CH,
CE,
NY,
ZH,
CU,
CV,
KW,
CO,
CR,
HR,
CS,
DA,
DV,
NL,
DZ,
EN,
EO,
ET,
EE,
FO,
FJ,
FI,
FR,
FY,
FF,
GD,
GL,
LG,
KA,
DE,
EL,
KL,
GN,
GU,
HT,
HA,
HE,
HZ,
HI,
HO,
HU,
IS,
IO,
IG,
ID,
IA,
IE,
IU,
IK,
GA,
IT,
JA,
JV,
KN,
KR,
KS,
KK,
KM,
KI,
RW,
KY,
KV,
KG,
KO,
KJ,
KU,
LO,
LA,
LV,
LI,
LN,
LT,
LU,
LB,
MK,
MG,
MS,
ML,
MT,
GV,
MI,
MR,
MH,
MN,
NA,
NV,
ND,
NR,
NG,
NE,
NO,
NB,
NN,
II,
OC,
OJ,
OR,
OM,
OS,
PI,
PS,
FA,
PL,
PT,
PA,
QU,
RO,
RM,
RN,
RU,
SE,
SM,
SG,
SA,
SC,
SR,
SN,
SD,
SI,
SK,
SL,
SO,
ST,
ES,
SU,
SW,
SS,
SV,
TL,
TY,
TG,
TA,
TT,
TE,
TH,
BO,
TI,
TO,
TS,
TN,
TR,
TK,
TW,
UG,
UK,
UR,
UZ,
VE,
VI,
VO,
WA,
CY,
WO,
XH,
YI,
YO,
ZA,
ZU,
SH,

];


pub const ALL_2T: & [LanguageCode] = &[
AB,
AA,
AF,
AK,
SQ,
AM,
AR,
AN,
HY,
AS,
AV,
AE,
AY,
AZ,
BM,
BA,
EU,
BE,
BN,
BI,
BS,
BR,
BG,
MY,
CA,
CH,
CE,
NY,
ZH,
CU,
CV,
KW,
CO,
CR,
HR,
CS,
DA,
DV,
NL,
DZ,
EN,
EO,
ET,
EE,
FO,
FJ,
FI,
FR,
FY,
FF,
GD,
GL,
LG,
KA,
DE,
EL,
KL,
GN,
GU,
HT,
HA,
HE,
HZ,
HI,
HO,
HU,
IS,
IO,
IG,
ID,
IA,
IE,
IU,
IK,
GA,
IT,
JA,
JV,
KN,
KR,
KS,
KK,
KM,
KI,
RW,
KY,
KV,
KG,
KO,
KJ,
KU,
LO,
LA,
LV,
LI,
LN,
LT,
LU,
LB,
MK,
MG,
MS,
ML,
MT,
GV,
MI,
MR,
MH,
MN,
NA,
NV,
ND,
NR,
NG,
NE,
NO,
NB,
NN,
II,
OC,
OJ,
OR,
OM,
OS,
PI,
PS,
FA,
PL,
PT,
PA,
QU,
RO,
RM,
RN,
RU,
SE,
SM,
SG,
SA,
SC,
SR,
SN,
SD,
SI,
SK,
SL,
SO,
ST,
ES,
SU,
SW,
SS,
SV,
TL,
TY,
TG,
TA,
TT,
TE,
TH,
BO,
TI,
TO,
TS,
TN,
TR,
TK,
TW,
UG,
UK,
UR,
UZ,
VE,
VI,
VO,
WA,
CY,
WO,
XH,
YI,
YO,
ZA,
ZU,
ACE,
ACH,
ADA,
ADY,
AFA,
AFH,
AIN,
AKK,
ALE,
ALG,
ALT,
ANG,
ANP,
APA,
ARC,
ARN,
ARP,
ART,
ARW,
AST,
ATH,
AUS,
AWA,
BAD,
BAI,
BAL,
BAN,
BAS,
BAT,
BEJ,
BEM,
BER,
BHO,
BIH,
BIK,
BIN,
BLA,
BNT,
BRA,
BTK,
BUA,
BUG,
BYN,
CAD,
CAI,
CAR,
CAU,
CEB,
CEL,
CHB,
CHG,
CHK,
CHM,
CHN,
CHO,
CHP,
CHR,
CHY,
CMC,
CNR,
COP,
CPE,
CPF,
CPP,
CRH,
CRP,
CSB,
CUS,
DAK,
DAR,
DAY,
DEL,
DEN,
DGR,
DIN,
DOI,
DRA,
DSB,
DUA,
DUM,
DYU,
EFI,
EGY,
EKA,
ELX,
ENM,
EWO,
FAN,
FAT,
FIL,
FIU,
FON,
FRM,
FRO,
FRR,
FRS,
FUR,
GAA,
GAY,
GBA,
GEM,
GEZ,
GIL,
GMH,
GOH,
GON,
GOR,
GOT,
GRB,
GRC,
GSW,
GWI,
HAI,
HAW,
HIL,
HIM,
HIT,
HMN,
HSB,
HUP,
IBA,
IJO,
ILO,
INC,
INE,
INH,
IRA,
IRO,
JBO,
JPR,
JRB,
KAA,
KAB,
KAC,
KAM,
KAR,
KAW,
KBD,
KHA,
KHI,
KHO,
KMB,
KOK,
KOS,
KPE,
KRC,
KRL,
KRO,
KRU,
KUM,
KUT,
LAD,
LAH,
LAM,
LEZ,
LOL,
LOZ,
LUA,
LUI,
LUN,
LUO,
LUS,
MAD,
MAG,
MAI,
MAK,
MAN,
MAP,
MAS,
MDF,
MDR,
MEN,
MGA,
MIC,
MIN,
MIS,
MKH,
MNC,
MNI,
MNO,
MOH,
MOS,
MUL,
MUN,
MUS,
MWL,
MWR,
MYN,
MYV,
NAH,
NAI,
NAP,
NDS,
NEW,
NIA,
NIC,
NIU,
NOG,
NON,
NQO,
NSO,
NUB,
NWC,
NYM,
NYN,
NYO,
NZI,
OSA,
OTA,
OTO,
PAA,
PAG,
PAL,
PAM,
PAP,
PAU,
PEO,
PHI,
PHN,
PON,
PRA,
PRO,
QAA,
RAJ,
RAP,
RAR,
ROA,
ROM,
RUP,
SAD,
SAH,
SAI,
SAL,
SAM,
SAS,
SAT,
SCN,
SCO,
SEL,
SEM,
SGA,
SGN,
SHN,
SID,
SIO,
SIT,
SLA,
SMA,
SMI,
SMJ,
SMN,
SMS,
SNK,
SOG,
SON,
SRN,
SRR,
SSA,
SUK,
SUS,
SUX,
SYC,
SYR,
TAI,
TEM,
TER,
TET,
TIG,
TIV,
TKL,
TLH,
TLI,
TMH,
TOG,
TPI,
TSI,
TUM,
TUP,
TUT,
TVL,
TYV,
UDM,
UGA,
UMB,
UND,
VAI,
VOT,
WAK,
WAL,
WAR,
WAS,
WEN,
XAL,
YAO,
YAP,
YPK,
ZAP,
ZBL,
ZEN,
ZGH,
ZND,
ZUN,
ZXX,
ZZA,
ZCD,

];


pub const ALL_2B: & [LanguageCode] = &[
AB,
AA,
AF,
AK,
SQ,
AM,
AR,
AN,
HY,
AS,
AV,
AE,
AY,
AZ,
BM,
BA,
EU,
BE,
BN,
BI,
BS,
BR,
BG,
MY,
CA,
CH,
CE,
NY,
ZH,
CU,
CV,
KW,
CO,
CR,
HR,
CS,
DA,
DV,
NL,
DZ,
EN,
EO,
ET,
EE,
FO,
FJ,
FI,
FR,
FY,
FF,
GD,
GL,
LG,
KA,
DE,
EL,
KL,
GN,
GU,
HT,
HA,
HE,
HZ,
HI,
HO,
HU,
IS,
IO,
IG,
ID,
IA,
IE,
IU,
IK,
GA,
IT,
JA,
JV,
KN,
KR,
KS,
KK,
KM,
KI,
RW,
KY,
KV,
KG,
KO,
KJ,
KU,
LO,
LA,
LV,
LI,
LN,
LT,
LU,
LB,
MK,
MG,
MS,
ML,
MT,
GV,
MI,
MR,
MH,
MN,
NA,
NV,
ND,
NR,
NG,
NE,
NO,
NB,
NN,
II,
OC,
OJ,
OR,
OM,
OS,
PI,
PS,
FA,
PL,
PT,
PA,
QU,
RO,
RM,
RN,
RU,
SE,
SM,
SG,
SA,
SC,
SR,
SN,
SD,
SI,
SK,
SL,
SO,
ST,
ES,
SU,
SW,
SS,
SV,
TL,
TY,
TG,
TA,
TT,
TE,
TH,
BO,
TI,
TO,
TS,
TN,
TR,
TK,
TW,
UG,
UK,
UR,
UZ,
VE,
VI,
VO,
WA,
CY,
WO,
XH,
YI,
YO,
ZA,
ZU,
ACE,
ACH,
ADA,
ADY,
AFA,
AFH,
AIN,
AKK,
ALE,
ALG,
ALT,
ANG,
ANP,
APA,
ARC,
ARN,
ARP,
ART,
ARW,
AST,
ATH,
AUS,
AWA,
BAD,
BAI,
BAL,
BAN,
BAS,
BAT,
BEJ,
BEM,
BER,
BHO,
BIH,
BIK,
BIN,
BLA,
BNT,
BRA,
BTK,
BUA,
BUG,
BYN,
CAD,
CAI,
CAR,
CAU,
CEB,
CEL,
CHB,
CHG,
CHK,
CHM,
CHN,
CHO,
CHP,
CHR,
CHY,
CMC,
CNR,
COP,
CPE,
CPF,
CPP,
CRH,
CRP,
CSB,
CUS,
DAK,
DAR,
DAY,
DEL,
DEN,
DGR,
DIN,
DOI,
DRA,
DSB,
DUA,
DUM,
DYU,
EFI,
EGY,
EKA,
ELX,
ENM,
EWO,
FAN,
FAT,
FIL,
FIU,
FON,
FRM,
FRO,
FRR,
FRS,
FUR,
GAA,
GAY,
GBA,
GEM,
GEZ,
GIL,
GMH,
GOH,
GON,
GOR,
GOT,
GRB,
GRC,
GSW,
GWI,
HAI,
HAW,
HIL,
HIM,
HIT,
HMN,
HSB,
HUP,
IBA,
IJO,
ILO,
INC,
INE,
INH,
IRA,
IRO,
JBO,
JPR,
JRB,
KAA,
KAB,
KAC,
KAM,
KAR,
KAW,
KBD,
KHA,
KHI,
KHO,
KMB,
KOK,
KOS,
KPE,
KRC,
KRL,
KRO,
KRU,
KUM,
KUT,
LAD,
LAH,
LAM,
LEZ,
LOL,
LOZ,
LUA,
LUI,
LUN,
LUO,
LUS,
MAD,
MAG,
MAI,
MAK,
MAN,
MAP,
MAS,
MDF,
MDR,
MEN,
MGA,
MIC,
MIN,
MIS,
MKH,
MNC,
MNI,
MNO,
MOH,
MOS,
MUL,
MUN,
MUS,
MWL,
MWR,
MYN,
MYV,
NAH,
NAI,
NAP,
NDS,
NEW,
NIA,
NIC,
NIU,
NOG,
NON,
NQO,
NSO,
NUB,
NWC,
NYM,
NYN,
NYO,
NZI,
OSA,
OTA,
OTO,
PAA,
PAG,
PAL,
PAM,
PAP,
PAU,
PEO,
PHI,
PHN,
PON,
PRA,
PRO,
QAA,
RAJ,
RAP,
RAR,
ROA,
ROM,
RUP,
SAD,
SAH,
SAI,
SAL,
SAM,
SAS,
SAT,
SCN,
SCO,
SEL,
SEM,
SGA,
SGN,
SHN,
SID,
SIO,
SIT,
SLA,
SMA,
SMI,
SMJ,
SMN,
SMS,
SNK,
SOG,
SON,
SRN,
SRR,
SSA,
SUK,
SUS,
SUX,
SYC,
SYR,
TAI,
TEM,
TER,
TET,
TIG,
TIV,
TKL,
TLH,
TLI,
TMH,
TOG,
TPI,
TSI,
TUM,
TUP,
TUT,
TVL,
TYV,
UDM,
UGA,
UMB,
UND,
VAI,
VOT,
WAK,
WAL,
WAR,
WAS,
WEN,
XAL,
YAO,
YAP,
YPK,
ZAP,
ZBL,
ZEN,
ZGH,
ZND,
ZUN,
ZXX,
ZZA,
ZCD,

];


pub const ALL_3: & [LanguageCode] = &[
AB,
AA,
AF,
AK,
SQ,
AM,
AR,
AN,
HY,
AS,
AV,
AE,
AY,
AZ,
BM,
BA,
EU,
BE,
BN,
BI,
BS,
BR,
BG,
MY,
CA,
CH,
CE,
NY,
ZH,
CU,
CV,
KW,
CO,
CR,
HR,
CS,
DA,
DV,
NL,
DZ,
EN,
EO,
ET,
EE,
FO,
FJ,
FI,
FR,
FY,
FF,
GD,
GL,
LG,
KA,
DE,
EL,
KL,
GN,
GU,
HT,
HA,
HE,
HZ,
HI,
HO,
HU,
IS,
IO,
IG,
ID,
IA,
IE,
IU,
IK,
GA,
IT,
JA,
JV,
KN,
KR,
KS,
KK,
KM,
KI,
RW,
KY,
KV,
KG,
KO,
KJ,
KU,
LO,
LA,
LV,
LI,
LN,
LT,
LU,
LB,
MK,
MG,
MS,
ML,
MT,
GV,
MI,
MR,
MH,
MN,
NA,
NV,
ND,
NR,
NG,
NE,
NO,
NB,
NN,
II,
OC,
OJ,
OR,
OM,
OS,
PI,
PS,
FA,
PL,
PT,
PA,
QU,
RO,
RM,
RN,
RU,
SE,
SM,
SG,
SA,
SC,
SR,
SN,
SD,
SI,
SK,
SL,
SO,
ST,
ES,
SU,
SW,
SS,
SV,
TL,
TY,
TG,
TA,
TT,
TE,
TH,
BO,
TI,
TO,
TS,
TN,
TR,
TK,
TW,
UG,
UK,
UR,
UZ,
VE,
VI,
VO,
WA,
CY,
WO,
XH,
YI,
YO,
ZA,
ZU,
ACE,
ACH,
ADA,
ADY,
AFH,
AIN,
AKK,
ALE,
ALT,
ANG,
ANP,
ARC,
ARN,
ARP,
ARW,
AST,
AWA,
BAL,
BAN,
BAS,
BEJ,
BEM,
BHO,
BIK,
BIN,
BLA,
BRA,
BUA,
BUG,
BYN,
CAD,
CAR,
CEB,
CHB,
CHG,
CHK,
CHM,
CHN,
CHO,
CHP,
CHR,
CHY,
CNR,
COP,
CRH,
CSB,
DAK,
DAR,
DEL,
DEN,
DGR,
DIN,
DOI,
DSB,
DUA,
DUM,
DYU,
EFI,
EGY,
EKA,
ELX,
ENM,
EWO,
FAN,
FAT,
FIL,
FON,
FRM,
FRO,
FRR,
FRS,
FUR,
GAA,
GAY,
GBA,
GEZ,
GIL,
GMH,
GOH,
GON,
GOR,
GOT,
GRB,
GRC,
GSW,
GWI,
HAI,
HAW,
HIL,
HIT,
HMN,
HSB,
HUP,
IBA,
ILO,
INH,
JBO,
JPR,
JRB,
KAA,
KAB,
KAC,
KAM,
KAW,
KBD,
KHA,
KHO,
KMB,
KOK,
KOS,
KPE,
KRC,
KRL,
KRU,
KUM,
KUT,
LAD,
LAH,
LAM,
LEZ,
LOL,
LOZ,
LUA,
LUI,
LUN,
LUO,
LUS,
MAD,
MAG,
MAI,
MAK,
MAN,
MAS,
MDF,
MDR,
MEN,
MGA,
MIC,
MIN,
MIS,
MNC,
MNI,
MOH,
MOS,
MUL,
MUS,
MWL,
MWR,
MYV,
NAP,
NDS,
NEW,
NIA,
NIU,
NOG,
NON,
NQO,
NSO,
NWC,
NYM,
NYN,
NYO,
NZI,
OSA,
OTA,
PAG,
PAL,
PAM,
PAP,
PAU,
PEO,
PHN,
PON,
PRO,
QAA,
RAJ,
RAP,
RAR,
ROM,
RUP,
SAD,
SAH,
SAM,
SAS,
SAT,
SCN,
SCO,
SEL,
SGA,
SHN,
SID,
SMA,
SMJ,
SMN,
SMS,
SNK,
SOG,
SRN,
SRR,
SUK,
SUS,
SUX,
SYC,
SYR,
TEM,
TER,
TET,
TIG,
TIV,
TKL,
TLH,
TLI,
TMH,
TOG,
TPI,
TSI,
TUM,
TVL,
TYV,
UDM,
UGA,
UMB,
UND,
VAI,
VOT,
WAL,
WAR,
WAS,
XAL,
YAO,
YAP,
ZAP,
ZBL,
ZEN,
ZGH,
ZUN,
ZXX,
ZZA,
AAA,
AAB,
AAC,
AAD,
AAE,
AAF,
AAG,
AAH,
AAI,
AAK,
AAL,
AAN,
AAO,
AAP,
AAQ,
AAS,
AAT,
AAU,
AAW,
AAX,
AAZ,
ABA,
ABB,
ABC,
ABD,
ABE,
ABF,
ABG,
ABH,
ABI,
ABJ,
ABL,
ABM,
ABN,
ABO,
ABP,
ABQ,
ABR,
ABS,
ABT,
ABU,
ABV,
ABW,
ABX,
ABY,
ABZ,
ACA,
ACB,
ACD,
ACF,
ACI,
ACK,
ACL,
ACM,
ACN,
ACP,
ACQ,
ACR,
ACS,
ACT,
ACU,
ACV,
ACW,
ACX,
ACY,
ACZ,
ADB,
ADD,
ADE,
ADF,
ADG,
ADH,
ADI,
ADJ,
ADL,
ADN,
ADO,
ADQ,
ADR,
ADS,
ADT,
ADU,
ADW,
ADX,
ADZ,
AEA,
AEB,
AEC,
AED,
AEE,
AEK,
AEL,
AEM,
AEN,
AEQ,
AER,
AES,
AEU,
AEW,
AEY,
AEZ,
AFB,
AFD,
AFE,
AFG,
AFI,
AFK,
AFN,
AFO,
AFP,
AFS,
AFT,
AFU,
AFZ,
AGA,
AGB,
AGC,
AGD,
AGE,
AGF,
AGG,
AGH,
AGI,
AGJ,
AGK,
AGL,
AGM,
AGN,
AGO,
AGQ,
AGR,
AGS,
AGT,
AGU,
AGV,
AGW,
AGX,
AGY,
AGZ,
AHA,
AHB,
AHG,
AHH,
AHI,
AHK,
AHL,
AHM,
AHN,
AHO,
AHP,
AHR,
AHS,
AHT,
AIA,
AIB,
AIC,
AID,
AIE,
AIF,
AIG,
AIH,
AII,
AIJ,
AIK,
AIL,
AIM,
AIO,
AIP,
AIQ,
AIR,
AIT,
AIW,
AIX,
AIY,
AJA,
AJG,
AJI,
AJN,
AJP,
AJS,
AJU,
AJW,
AJZ,
AKB,
AKC,
AKD,
AKE,
AKF,
AKG,
AKH,
AKI,
AKJ,
AKL,
AKM,
AKO,
AKP,
AKQ,
AKR,
AKS,
AKT,
AKU,
AKV,
AKW,
AKX,
AKY,
AKZ,
ALA,
ALC,
ALD,
ALF,
ALH,
ALI,
ALJ,
ALK,
ALL,
ALM,
ALN,
ALO,
ALP,
ALQ,
ALR,
ALS,
ALU,
ALW,
ALX,
ALY,
ALZ,
AMA,
AMB,
AMC,
AME,
AMF,
AMG,
AMI,
AMJ,
AMK,
AML,
AMM,
AMN,
AMO,
AMP,
AMQ,
AMR,
AMS,
AMT,
AMU,
AMV,
AMW,
AMX,
AMY,
AMZ,
ANA,
ANB,
ANC,
AND,
ANE,
ANF,
ANH,
ANI,
ANJ,
ANK,
ANL,
ANM,
ANN,
ANO,
ANQ,
ANR,
ANS,
ANT,
ANU,
ANV,
ANW,
ANX,
ANY,
ANZ,
AOA,
AOB,
AOC,
AOD,
AOE,
AOF,
AOG,
AOI,
AOJ,
AOK,
AOL,
AOM,
AON,
AOR,
AOS,
AOT,
AOU,
AOX,
AOZ,
APB,
APC,
APD,
APE,
APF,
APG,
APH,
API,
APJ,
APK,
APL,
APM,
APN,
APO,
APP,
APQ,
APR,
APS,
APT,
APU,
APV,
APW,
APX,
APY,
APZ,
AQC,
AQD,
AQG,
AQK,
AQM,
AQN,
AQP,
AQR,
AQT,
AQZ,
ARB,
ARD,
ARE,
ARH,
ARI,
ARJ,
ARK,
ARL,
ARO,
ARQ,
ARR,
ARS,
ARU,
ARV,
ARX,
ARY,
ARZ,
ASA,
ASB,
ASC,
ASE,
ASF,
ASG,
ASH,
ASI,
ASJ,
ASK,
ASL,
ASN,
ASO,
ASP,
ASQ,
ASR,
ASS,
ASU,
ASV,
ASW,
ASX,
ASY,
ASZ,
ATA,
ATB,
ATC,
ATD,
ATE,
ATG,
ATI,
ATJ,
ATK,
ATL,
ATM,
ATN,
ATO,
ATP,
ATQ,
ATR,
ATS,
ATT,
ATU,
ATV,
ATW,
ATX,
ATY,
ATZ,
AUA,
AUB,
AUC,
AUD,
AUG,
AUH,
AUI,
AUJ,
AUK,
AUL,
AUM,
AUN,
AUO,
AUP,
AUQ,
AUR,
AUT,
AUU,
AUW,
AUX,
AUY,
AUZ,
AVB,
AVD,
AVI,
AVK,
AVL,
AVM,
AVN,
AVO,
AVS,
AVT,
AVU,
AVV,
AWB,
AWC,
AWE,
AWG,
AWH,
AWI,
AWK,
AWM,
AWN,
AWO,
AWR,
AWS,
AWT,
AWU,
AWV,
AWW,
AWX,
AWY,
AXB,
AXE,
AXG,
AXK,
AXL,
AXM,
AXX,
AYA,
AYB,
AYC,
AYD,
AYE,
AYG,
AYH,
AYI,
AYK,
AYL,
AYN,
AYO,
AYP,
AYQ,
AYR,
AYS,
AYT,
AYU,
AYZ,
AZA,
AZB,
AZD,
AZG,
AZJ,
AZM,
AZN,
AZO,
AZT,
AZZ,
BAA,
BAB,
BAC,
BAE,
BAF,
BAG,
BAH,
BAJ,
BAO,
BAP,
BAR,
BAU,
BAV,
BAW,
BAX,
BAY,
BBA,
BBB,
BBC,
BBD,
BBE,
BBF,
BBG,
BBH,
BBI,
BBJ,
BBK,
BBL,
BBM,
BBN,
BBO,
BBP,
BBQ,
BBR,
BBS,
BBT,
BBU,
BBV,
BBW,
BBX,
BBY,
BCA,
BCB,
BCC,
BCD,
BCE,
BCF,
BCG,
BCH,
BCI,
BCJ,
BCK,
BCL,
BCM,
BCN,
BCO,
BCP,
BCQ,
BCR,
BCS,
BCT,
BCU,
BCV,
BCW,
BCY,
BCZ,
BDA,
BDB,
BDC,
BDD,
BDE,
BDF,
BDG,
BDH,
BDI,
BDJ,
BDK,
BDL,
BDM,
BDN,
BDO,
BDP,
BDQ,
BDR,
BDS,
BDT,
BDU,
BDV,
BDW,
BDX,
BDY,
BDZ,
BEA,
BEB,
BEC,
BED,
BEE,
BEF,
BEG,
BEH,
BEI,
BEK,
BEO,
BEP,
BEQ,
BES,
BET,
BEU,
BEV,
BEW,
BEX,
BEY,
BEZ,
BFA,
BFB,
BFC,
BFD,
BFE,
BFF,
BFG,
BFH,
BFI,
BFJ,
BFK,
BFL,
BFM,
BFN,
BFO,
BFP,
BFQ,
BFR,
BFS,
BFT,
BFU,
BFW,
BFX,
BFY,
BFZ,
BGA,
BGB,
BGC,
BGD,
BGE,
BGF,
BGG,
BGI,
BGJ,
BGK,
BGL,
BGN,
BGO,
BGP,
BGQ,
BGR,
BGS,
BGT,
BGU,
BGV,
BGW,
BGX,
BGY,
BGZ,
BHA,
BHB,
BHC,
BHD,
BHE,
BHF,
BHG,
BHH,
BHI,
BHJ,
BHL,
BHM,
BHN,
BHP,
BHQ,
BHR,
BHS,
BHT,
BHU,
BHV,
BHW,
BHX,
BHY,
BHZ,
BIA,
BIB,
BID,
BIE,
BIF,
BIG,
BIL,
BIM,
BIO,
BIP,
BIQ,
BIR,
BIT,
BIU,
BIV,
BIW,
BIX,
BIY,
BIZ,
BJA,
BJB,
BJC,
BJE,
BJF,
BJG,
BJH,
BJI,
BJJ,
BJK,
BJL,
BJM,
BJN,
BJO,
BJP,
BJR,
BJS,
BJT,
BJU,
BJV,
BJW,
BJX,
BJY,
BJZ,
BKA,
BKC,
BKD,
BKF,
BKG,
BKH,
BKI,
BKJ,
BKK,
BKL,
BKM,
BKN,
BKO,
BKP,
BKQ,
BKR,
BKS,
BKT,
BKU,
BKV,
BKW,
BKX,
BKY,
BKZ,
BLB,
BLC,
BLD,
BLE,
BLF,
BLH,
BLI,
BLJ,
BLK,
BLL,
BLM,
BLN,
BLO,
BLP,
BLQ,
BLR,
BLS,
BLT,
BLV,
BLW,
BLX,
BLY,
BLZ,
BMA,
BMB,
BMC,
BMD,
BME,
BMF,
BMG,
BMH,
BMI,
BMJ,
BMK,
BML,
BMM,
BMN,
BMO,
BMP,
BMQ,
BMR,
BMS,
BMT,
BMU,
BMV,
BMW,
BMX,
BMZ,
BNA,
BNB,
BNC,
BND,
BNE,
BNF,
BNG,
BNI,
BNJ,
BNK,
BNL,
BNM,
BNN,
BNO,
BNP,
BNQ,
BNR,
BNS,
BNU,
BNV,
BNW,
BNX,
BNY,
BNZ,
BOA,
BOB,
BOE,
BOF,
BOG,
BOH,
BOI,
BOJ,
BOK,
BOL,
BOM,
BON,
BOO,
BOP,
BOQ,
BOR,
BOT,
BOU,
BOV,
BOW,
BOX,
BOY,
BOZ,
BPA,
BPC,
BPD,
BPE,
BPG,
BPH,
BPI,
BPJ,
BPK,
BPL,
BPM,
BPN,
BPO,
BPP,
BPQ,
BPR,
BPS,
BPT,
BPU,
BPV,
BPW,
BPX,
BPY,
BPZ,
BQA,
BQB,
BQC,
BQD,
BQF,
BQG,
BQH,
BQI,
BQJ,
BQK,
BQL,
BQM,
BQN,
BQO,
BQP,
BQQ,
BQR,
BQS,
BQT,
BQU,
BQV,
BQW,
BQX,
BQY,
BQZ,
BRB,
BRC,
BRD,
BRF,
BRG,
BRH,
BRI,
BRJ,
BRK,
BRL,
BRM,
BRN,
BRO,
BRP,
BRQ,
BRR,
BRS,
BRT,
BRU,
BRV,
BRW,
BRX,
BRY,
BRZ,
BSA,
BSB,
BSC,
BSE,
BSF,
BSG,
BSH,
BSI,
BSJ,
BSK,
BSL,
BSM,
BSN,
BSO,
BSP,
BSQ,
BSR,
BSS,
BST,
BSU,
BSV,
BSW,
BSX,
BSY,
BTA,
BTC,
BTD,
BTE,
BTF,
BTG,
BTH,
BTI,
BTJ,
BTM,
BTN,
BTO,
BTP,
BTQ,
BTR,
BTS,
BTT,
BTU,
BTV,
BTW,
BTX,
BTY,
BTZ,
BUB,
BUC,
BUD,
BUE,
BUF,
BUH,
BUI,
BUJ,
BUK,
BUM,
BUN,
BUO,
BUP,
BUQ,
BUS,
BUT,
BUU,
BUV,
BUW,
BUX,
BUY,
BUZ,
BVA,
BVB,
BVC,
BVD,
BVE,
BVF,
BVG,
BVH,
BVI,
BVJ,
BVK,
BVL,
BVM,
BVN,
BVO,
BVP,
BVQ,
BVR,
BVT,
BVU,
BVV,
BVW,
BVX,
BVY,
BVZ,
BWA,
BWB,
BWC,
BWD,
BWE,
BWF,
BWG,
BWH,
BWI,
BWJ,
BWK,
BWL,
BWM,
BWN,
BWO,
BWP,
BWQ,
BWR,
BWS,
BWT,
BWU,
BWW,
BWX,
BWY,
BWZ,
BXA,
BXB,
BXC,
BXD,
BXE,
BXF,
BXG,
BXH,
BXI,
BXJ,
BXK,
BXL,
BXM,
BXN,
BXO,
BXP,
BXQ,
BXR,
BXS,
BXU,
BXV,
BXW,
BXZ,
BYA,
BYB,
BYC,
BYD,
BYE,
BYF,
BYG,
BYH,
BYI,
BYJ,
BYK,
BYL,
BYM,
BYO,
BYP,
BYQ,
BYR,
BYS,
BYT,
BYV,
BYW,
BYX,
BYZ,
BZA,
BZB,
BZC,
BZD,
BZE,
BZF,
BZG,
BZH,
BZI,
BZJ,
BZK,
BZL,
BZM,
BZN,
BZO,
BZP,
BZQ,
BZR,
BZS,
BZT,
BZU,
BZV,
BZW,
BZX,
BZY,
BZZ,
CAA,
CAB,
CAC,
CAE,
CAF,
CAG,
CAH,
CAJ,
CAK,
CAL,
CAM,
CAN,
CAO,
CAP,
CAQ,
CAS,
CAV,
CAW,
CAX,
CAY,
CAZ,
CBB,
CBC,
CBD,
CBG,
CBI,
CBJ,
CBK,
CBL,
CBN,
CBO,
CBQ,
CBR,
CBS,
CBT,
CBU,
CBV,
CBW,
CBY,
CCC,
CCD,
CCE,
CCG,
CCH,
CCJ,
CCL,
CCM,
CCO,
CCP,
CCR,
CDA,
CDE,
CDF,
CDH,
CDI,
CDJ,
CDM,
CDN,
CDO,
CDR,
CDS,
CDY,
CDZ,
CEA,
CEG,
CEK,
CEN,
CET,
CEY,
CFA,
CFD,
CFG,
CFM,
CGA,
CGC,
CGG,
CGK,
CHC,
CHD,
CHF,
CHH,
CHJ,
CHL,
CHQ,
CHT,
CHW,
CHX,
CHZ,
CIA,
CIB,
CIC,
CID,
CIE,
CIH,
CIK,
CIM,
CIN,
CIP,
CIR,
CIW,
CIY,
CJA,
CJE,
CJH,
CJI,
CJK,
CJM,
CJN,
CJO,
CJP,
CJS,
CJV,
CJY,
CKB,
CKH,
CKL,
CKM,
CKN,
CKO,
CKQ,
CKR,
CKS,
CKT,
CKU,
CKV,
CKX,
CKY,
CKZ,
CLA,
CLC,
CLD,
CLE,
CLH,
CLI,
CLJ,
CLK,
CLL,
CLM,
CLO,
CLT,
CLU,
CLW,
CLY,
CMA,
CME,
CMG,
CMI,
CML,
CMM,
CMN,
CMO,
CMR,
CMS,
CMT,
CNA,
CNB,
CNC,
CNG,
CNH,
CNI,
CNK,
CNL,
CNO,
CNP,
CNQ,
CNS,
CNT,
CNU,
CNW,
CNX,
COA,
COB,
COC,
COD,
COE,
COF,
COG,
COH,
COJ,
COK,
COL,
COM,
CON,
COO,
COQ,
COT,
COU,
COV,
COW,
COX,
COZ,
CPA,
CPB,
CPC,
CPG,
CPI,
CPN,
CPO,
CPS,
CPU,
CPX,
CPY,
CQD,
CRA,
CRB,
CRC,
CRD,
CRF,
CRG,
CRI,
CRJ,
CRK,
CRL,
CRM,
CRN,
CRO,
CRQ,
CRR,
CRS,
CRT,
CRV,
CRW,
CRX,
CRY,
CRZ,
CSA,
CSC,
CSD,
CSE,
CSF,
CSG,
CSH,
CSI,
CSJ,
CSK,
CSL,
CSM,
CSN,
CSO,
CSP,
CSQ,
CSR,
CSS,
CST,
CSV,
CSW,
CSX,
CSY,
CSZ,
CTA,
CTC,
CTD,
CTE,
CTG,
CTH,
CTL,
CTM,
CTN,
CTO,
CTP,
CTS,
CTT,
CTU,
CTY,
CTZ,
CUA,
CUB,
CUC,
CUH,
CUI,
CUJ,
CUK,
CUL,
CUO,
CUP,
CUQ,
CUR,
CUT,
CUU,
CUV,
CUW,
CUX,
CUY,
CVG,
CVN,
CWA,
CWB,
CWD,
CWE,
CWG,
CWT,
CYA,
CYB,
CYO,
CZH,
CZK,
CZN,
CZO,
CZT,
DAA,
DAC,
DAD,
DAE,
DAG,
DAH,
DAI,
DAJ,
DAL,
DAM,
DAO,
DAQ,
DAS,
DAU,
DAV,
DAW,
DAX,
DAZ,
DBA,
DBB,
DBD,
DBE,
DBF,
DBG,
DBI,
DBJ,
DBL,
DBM,
DBN,
DBO,
DBP,
DBQ,
DBR,
DBT,
DBU,
DBV,
DBW,
DBY,
DCC,
DCR,
DDA,
DDD,
DDE,
DDG,
DDI,
DDJ,
DDN,
DDO,
DDR,
DDS,
DDW,
DEC,
DED,
DEE,
DEF,
DEG,
DEH,
DEI,
DEK,
DEM,
DEP,
DEQ,
DER,
DES,
DEV,
DEZ,
DGA,
DGB,
DGC,
DGD,
DGE,
DGG,
DGH,
DGI,
DGK,
DGL,
DGN,
DGO,
DGS,
DGT,
DGW,
DGX,
DGZ,
DHD,
DHG,
DHI,
DHL,
DHM,
DHN,
DHO,
DHR,
DHS,
DHU,
DHV,
DHW,
DHX,
DIA,
DIB,
DIC,
DID,
DIF,
DIG,
DIH,
DII,
DIJ,
DIK,
DIL,
DIM,
DIO,
DIP,
DIQ,
DIR,
DIS,
DIU,
DIW,
DIX,
DIY,
DIZ,
DJA,
DJB,
DJC,
DJD,
DJE,
DJF,
DJI,
DJJ,
DJK,
DJM,
DJN,
DJO,
DJR,
DJU,
DJW,
DKA,
DKG,
DKK,
DKR,
DKS,
DKX,
DLG,
DLK,
DLM,
DLN,
DMA,
DMB,
DMC,
DMD,
DME,
DMF,
DMG,
DMK,
DML,
DMM,
DMO,
DMR,
DMS,
DMU,
DMV,
DMW,
DMX,
DMY,
DNA,
DND,
DNE,
DNG,
DNI,
DNJ,
DNK,
DNN,
DNO,
DNR,
DNT,
DNU,
DNV,
DNW,
DNY,
DOA,
DOB,
DOC,
DOE,
DOF,
DOH,
DOK,
DOL,
DON,
DOO,
DOP,
DOQ,
DOR,
DOS,
DOT,
DOV,
DOW,
DOX,
DOY,
DOZ,
DPP,
DRB,
DRC,
DRD,
DRE,
DRG,
DRI,
DRL,
DRN,
DRO,
DRQ,
DRS,
DRT,
DRU,
DRY,
DSE,
DSH,
DSI,
DSL,
DSN,
DSO,
DSQ,
DSZ,
DTA,
DTB,
DTD,
DTH,
DTI,
DTK,
DTM,
DTN,
DTO,
DTP,
DTR,
DTS,
DTT,
DTU,
DTY,
DUB,
DUC,
DUE,
DUF,
DUG,
DUH,
DUI,
DUK,
DUL,
DUN,
DUO,
DUP,
DUQ,
DUR,
DUS,
DUU,
DUV,
DUW,
DUX,
DUY,
DUZ,
DVA,
DWA,
DWK,
DWR,
DWS,
DWU,
DWW,
DWY,
DWZ,
DYA,
DYB,
DYD,
DYG,
DYI,
DYM,
DYN,
DYO,
DYY,
DZA,
DZE,
DZG,
DZL,
DZN,
EAA,
EBC,
EBG,
EBK,
EBO,
EBR,
EBU,
ECR,
ECS,
ECY,
EEE,
EFA,
EFE,
EGA,
EGL,
EGM,
EGO,
EHS,
EHU,
EIP,
EIT,
EIV,
EJA,
EKE,
EKG,
EKI,
EKK,
EKL,
EKM,
EKO,
EKP,
EKR,
EKY,
ELE,
ELH,
ELI,
ELK,
ELM,
ELO,
ELU,
EMA,
EMB,
EME,
EMG,
EMI,
EMK,
EMM,
EMN,
EMP,
EMQ,
EMS,
EMU,
EMW,
EMX,
EMY,
EMZ,
ENA,
ENB,
ENC,
END,
ENF,
ENH,
ENL,
ENN,
ENO,
ENQ,
ENR,
ENU,
ENV,
ENW,
ENX,
EOT,
EPI,
ERA,
ERG,
ERH,
ERI,
ERK,
ERO,
ERR,
ERS,
ERT,
ERW,
ESE,
ESG,
ESH,
ESI,
ESK,
ESL,
ESM,
ESN,
ESO,
ESQ,
ESS,
ESU,
ESY,
ETB,
ETC,
ETH,
ETN,
ETO,
ETR,
ETS,
ETT,
ETU,
ETX,
ETZ,
EVE,
EVH,
EVN,
EXT,
EYA,
EYO,
EZA,
EZE,
FAA,
FAB,
FAD,
FAF,
FAG,
FAH,
FAI,
FAJ,
FAK,
FAL,
FAM,
FAP,
FAR,
FAU,
FAX,
FAY,
FAZ,
FBL,
FCS,
FER,
FFI,
FFM,
FGR,
FIA,
FIE,
FIF,
FIP,
FIR,
FIT,
FIW,
FKK,
FKV,
FLA,
FLH,
FLI,
FLL,
FLN,
FLR,
FLY,
FMP,
FMU,
FNB,
FNG,
FNI,
FOD,
FOI,
FOM,
FOR,
FOS,
FPE,
FQS,
FRC,
FRD,
FRK,
FRP,
FRQ,
FRT,
FSE,
FSL,
FSS,
FUB,
FUC,
FUD,
FUE,
FUF,
FUH,
FUI,
FUJ,
FUM,
FUN,
FUQ,
FUT,
FUU,
FUV,
FUY,
FVR,
FWA,
FWE,
GAB,
GAC,
GAD,
GAE,
GAF,
GAG,
GAH,
GAI,
GAJ,
GAK,
GAL,
GAM,
GAN,
GAO,
GAP,
GAQ,
GAR,
GAS,
GAT,
GAU,
GAW,
GAX,
GAZ,
GBB,
GBD,
GBE,
GBF,
GBG,
GBH,
GBI,
GBJ,
GBK,
GBL,
GBM,
GBN,
GBO,
GBP,
GBQ,
GBR,
GBS,
GBU,
GBV,
GBW,
GBX,
GBY,
GBZ,
GCC,
GCD,
GCE,
GCF,
GCL,
GCN,
GCR,
GCT,
GDA,
GDB,
GDC,
GDD,
GDE,
GDF,
GDG,
GDH,
GDI,
GDJ,
GDK,
GDL,
GDM,
GDN,
GDO,
GDQ,
GDR,
GDS,
GDT,
GDU,
GDX,
GEA,
GEB,
GEC,
GED,
GEF,
GEG,
GEH,
GEI,
GEJ,
GEK,
GEL,
GEQ,
GES,
GEV,
GEW,
GEX,
GEY,
GFK,
GFT,
GGA,
GGB,
GGD,
GGE,
GGG,
GGK,
GGL,
GGT,
GGU,
GGW,
GHA,
GHC,
GHE,
GHH,
GHK,
GHL,
GHN,
GHO,
GHR,
GHS,
GHT,
GIA,
GIB,
GIC,
GID,
GIE,
GIG,
GIH,
GII,
GIM,
GIN,
GIP,
GIQ,
GIR,
GIS,
GIT,
GIU,
GIW,
GIX,
GIY,
GIZ,
GJK,
GJM,
GJN,
GJR,
GJU,
GKA,
GKD,
GKE,
GKN,
GKO,
GKP,
GKU,
GLB,
GLC,
GLD,
GLH,
GLJ,
GLK,
GLL,
GLO,
GLR,
GLU,
GLW,
GLY,
GMA,
GMB,
GMD,
GMG,
GML,
GMM,
GMN,
GMR,
GMU,
GMV,
GMX,
GMY,
GMZ,
GNA,
GNB,
GNC,
GND,
GNE,
GNG,
GNH,
GNI,
GNJ,
GNK,
GNL,
GNM,
GNN,
GNO,
GNQ,
GNR,
GNT,
GNU,
GNW,
GNZ,
GOA,
GOB,
GOC,
GOD,
GOE,
GOF,
GOG,
GOI,
GOJ,
GOK,
GOL,
GOM,
GOO,
GOP,
GOQ,
GOS,
GOU,
GOV,
GOW,
GOX,
GOY,
GOZ,
GPA,
GPE,
GPN,
GQA,
GQI,
GQN,
GQR,
GQU,
GRA,
GRD,
GRG,
GRH,
GRI,
GRJ,
GRM,
GRO,
GRQ,
GRR,
GRS,
GRT,
GRU,
GRV,
GRW,
GRX,
GRY,
GRZ,
GSE,
GSG,
GSL,
GSM,
GSN,
GSO,
GSP,
GSS,
GTA,
GTU,
GUA,
GUB,
GUC,
GUD,
GUE,
GUF,
GUG,
GUH,
GUI,
GUK,
GUL,
GUM,
GUN,
GUO,
GUP,
GUQ,
GUR,
GUS,
GUT,
GUU,
GUW,
GUX,
GUZ,
GVA,
GVC,
GVE,
GVF,
GVJ,
GVL,
GVM,
GVN,
GVO,
GVP,
GVR,
GVS,
GVY,
GWA,
GWB,
GWC,
GWD,
GWE,
GWF,
GWG,
GWJ,
GWM,
GWN,
GWR,
GWT,
GWU,
GWW,
GWX,
GXX,
GYA,
GYB,
GYD,
GYE,
GYF,
GYG,
GYI,
GYL,
GYM,
GYN,
GYO,
GYR,
GYY,
GYZ,
GZA,
GZI,
GZN,
HAA,
HAB,
HAC,
HAD,
HAE,
HAF,
HAG,
HAH,
HAJ,
HAK,
HAL,
HAM,
HAN,
HAO,
HAP,
HAQ,
HAR,
HAS,
HAV,
HAX,
HAY,
HAZ,
HBA,
HBB,
HBN,
HBO,
SH,
HBU,
HCA,
HCH,
HDN,
HDS,
HDY,
HEA,
HED,
HEG,
HEH,
HEI,
HEM,
HGM,
HGW,
HHI,
HHR,
HHY,
HIA,
HIB,
HID,
HIF,
HIG,
HIH,
HII,
HIJ,
HIK,
HIO,
HIR,
HIW,
HIX,
HJI,
HKA,
HKE,
HKH,
HKK,
HKN,
HKS,
HLA,
HLB,
HLD,
HLE,
HLT,
HLU,
HMA,
HMB,
HMC,
HMD,
HME,
HMF,
HMG,
HMH,
HMI,
HMJ,
HMK,
HML,
HMM,
HMP,
HMQ,
HMR,
HMS,
HMT,
HMU,
HMV,
HMW,
HMY,
HMZ,
HNA,
HND,
HNE,
HNG,
HNH,
HNI,
HNJ,
HNN,
HNO,
HNS,
HNU,
HOA,
HOB,
HOC,
HOD,
HOE,
HOH,
HOI,
HOJ,
HOL,
HOM,
HOO,
HOP,
HOR,
HOS,
HOT,
HOV,
HOW,
HOY,
HOZ,
HPO,
HPS,
HRA,
HRC,
HRE,
HRK,
HRM,
HRO,
HRP,
HRT,
HRU,
HRW,
HRX,
HRZ,
HSH,
HSL,
HSN,
HSS,
HTI,
HTO,
HTS,
HTU,
HTX,
HUB,
HUC,
HUD,
HUE,
HUF,
HUG,
HUH,
HUI,
HUJ,
HUK,
HUL,
HUM,
HUO,
HUQ,
HUR,
HUS,
HUT,
HUU,
HUV,
HUW,
HUX,
HUY,
HUZ,
HVC,
HVE,
HVK,
HVN,
HVV,
HWA,
HWC,
HWO,
HYA,
HYW,
IAI,
IAN,
IAR,
IBB,
IBD,
IBE,
IBG,
IBH,
IBL,
IBM,
IBN,
IBR,
IBU,
IBY,
ICA,
ICH,
ICL,
ICR,
IDA,
IDB,
IDC,
IDD,
IDE,
IDI,
IDR,
IDS,
IDT,
IDU,
IFA,
IFB,
IFE,
IFF,
IFK,
IFM,
IFU,
IFY,
IGB,
IGE,
IGG,
IGL,
IGM,
IGN,
IGO,
IGS,
IGW,
IHB,
IHI,
IHP,
IHW,
IIN,
IJC,
IJE,
IJJ,
IJN,
IJS,
IKE,
IKI,
IKK,
IKL,
IKO,
IKP,
IKR,
IKS,
IKT,
IKV,
IKW,
IKX,
IKZ,
ILA,
ILB,
ILG,
ILI,
ILK,
ILM,
ILP,
ILS,
ILU,
ILV,
IMA,
IMI,
IML,
IMN,
IMO,
IMR,
IMS,
IMT,
IMY,
INB,
ING,
INJ,
INL,
INM,
INN,
INO,
INP,
INS,
INT,
INZ,
IOR,
IOU,
IOW,
IPI,
IPO,
IQU,
IQW,
IRE,
IRH,
IRI,
IRK,
IRN,
IRR,
IRU,
IRX,
IRY,
ISA,
ISC,
ISD,
ISE,
ISG,
ISH,
ISI,
ISK,
ISM,
ISN,
ISO,
ISR,
IST,
ISU,
ITB,
ITD,
ITE,
ITI,
ITK,
ITL,
ITM,
ITO,
ITR,
ITS,
ITT,
ITV,
ITW,
ITX,
ITY,
ITZ,
IUM,
IVB,
IVV,
IWK,
IWM,
IWO,
IWS,
IXC,
IXL,
IYA,
IYO,
IYX,
IZH,
IZR,
IZZ,
JAA,
JAB,
JAC,
JAD,
JAE,
JAF,
JAH,
JAJ,
JAK,
JAL,
JAM,
JAN,
JAO,
JAQ,
JAS,
JAT,
JAU,
JAX,
JAY,
JAZ,
JBE,
JBI,
JBJ,
JBK,
JBM,
JBN,
JBR,
JBT,
JBU,
JBW,
JCS,
JCT,
JDA,
JDG,
JDT,
JEB,
JEE,
JEH,
JEI,
JEK,
JEL,
JEN,
JER,
JET,
JEU,
JGB,
JGE,
JGK,
JGO,
JHI,
JHS,
JIA,
JIB,
JIC,
JID,
JIE,
JIG,
JIH,
JII,
JIL,
JIM,
JIO,
JIQ,
JIT,
JIU,
JIV,
JIY,
JJE,
JJR,
JKA,
JKM,
JKO,
JKP,
JKR,
JKS,
JKU,
JLE,
JLS,
JMA,
JMB,
JMC,
JMD,
JMI,
JML,
JMN,
JMR,
JMS,
JMW,
JMX,
JNA,
JND,
JNG,
JNI,
JNJ,
JNL,
JNS,
JOB,
JOD,
JOG,
JOR,
JOS,
JOW,
JPA,
JQR,
JRA,
JRR,
JRT,
JRU,
JSL,
JUA,
JUB,
JUC,
JUD,
JUH,
JUI,
JUK,
JUL,
JUM,
JUN,
JUO,
JUP,
JUR,
JUS,
JUT,
JUU,
JUW,
JUY,
JVD,
JVN,
JWI,
JYA,
JYE,
JYY,
KAD,
KAE,
KAF,
KAG,
KAH,
KAI,
KAJ,
KAK,
KAO,
KAP,
KAQ,
KAV,
KAX,
KAY,
KBA,
KBB,
KBC,
KBE,
KBG,
KBH,
KBI,
KBJ,
KBK,
KBL,
KBM,
KBN,
KBO,
KBP,
KBQ,
KBR,
KBS,
KBT,
KBU,
KBV,
KBW,
KBX,
KBY,
KBZ,
KCA,
KCB,
KCC,
KCD,
KCE,
KCF,
KCG,
KCH,
KCI,
KCJ,
KCK,
KCL,
KCM,
KCN,
KCO,
KCP,
KCQ,
KCR,
KCS,
KCT,
KCU,
KCV,
KCW,
KCX,
KCY,
KCZ,
KDA,
KDC,
KDD,
KDE,
KDF,
KDG,
KDH,
KDI,
KDJ,
KDK,
KDL,
KDM,
KDN,
KDP,
KDQ,
KDR,
KDT,
KDU,
KDW,
KDX,
KDY,
KDZ,
KEA,
KEB,
KEC,
KED,
KEE,
KEF,
KEG,
KEH,
KEI,
KEJ,
KEK,
KEL,
KEM,
KEN,
KEO,
KEP,
KEQ,
KER,
KES,
KET,
KEU,
KEV,
KEW,
KEX,
KEY,
KEZ,
KFA,
KFB,
KFC,
KFD,
KFE,
KFF,
KFG,
KFH,
KFI,
KFJ,
KFK,
KFL,
KFM,
KFN,
KFO,
KFP,
KFQ,
KFR,
KFS,
KFT,
KFU,
KFV,
KFW,
KFX,
KFY,
KFZ,
KGA,
KGB,
KGE,
KGF,
KGG,
KGI,
KGJ,
KGK,
KGL,
KGM,
KGN,
KGO,
KGP,
KGQ,
KGR,
KGS,
KGT,
KGU,
KGV,
KGW,
KGX,
KGY,
KHB,
KHC,
KHD,
KHE,
KHF,
KHG,
KHH,
KHJ,
KHK,
KHL,
KHN,
KHP,
KHQ,
KHR,
KHS,
KHT,
KHU,
KHV,
KHW,
KHX,
KHY,
KHZ,
KIA,
KIB,
KIC,
KID,
KIE,
KIF,
KIG,
KIH,
KII,
KIJ,
KIL,
KIM,
KIO,
KIP,
KIQ,
KIS,
KIT,
KIU,
KIV,
KIW,
KIX,
KIY,
KIZ,
KJA,
KJB,
KJC,
KJD,
KJE,
KJG,
KJH,
KJI,
KJJ,
KJK,
KJL,
KJM,
KJN,
KJO,
KJP,
KJQ,
KJR,
KJS,
KJT,
KJU,
KJV,
KJX,
KJY,
KJZ,
KKA,
KKB,
KKC,
KKD,
KKE,
KKF,
KKG,
KKH,
KKI,
KKJ,
KKK,
KKL,
KKM,
KKN,
KKO,
KKP,
KKQ,
KKR,
KKS,
KKT,
KKU,
KKV,
KKW,
KKX,
KKY,
KKZ,
KLA,
KLB,
KLC,
KLD,
KLE,
KLF,
KLG,
KLH,
KLI,
KLJ,
KLK,
KLL,
KLM,
KLN,
KLO,
KLP,
KLQ,
KLR,
KLS,
KLT,
KLU,
KLV,
KLW,
KLX,
KLY,
KLZ,
KMA,
KMC,
KMD,
KME,
KMF,
KMG,
KMH,
KMI,
KMJ,
KMK,
KML,
KMM,
KMN,
KMO,
KMP,
KMQ,
KMR,
KMS,
KMT,
KMU,
KMV,
KMW,
KMX,
KMY,
KMZ,
KNA,
KNB,
KNC,
KND,
KNE,
KNF,
KNG,
KNI,
KNJ,
KNK,
KNL,
KNM,
KNN,
KNO,
KNP,
KNQ,
KNR,
KNS,
KNT,
KNU,
KNV,
KNW,
KNX,
KNY,
KNZ,
KOA,
KOC,
KOD,
KOE,
KOF,
KOG,
KOH,
KOI,
KOL,
KOO,
KOP,
KOQ,
KOT,
KOU,
KOV,
KOW,
KOY,
KOZ,
KPA,
KPB,
KPC,
KPD,
KPF,
KPG,
KPH,
KPI,
KPJ,
KPK,
KPL,
KPM,
KPN,
KPO,
KPQ,
KPR,
KPS,
KPT,
KPU,
KPV,
KPW,
KPX,
KPY,
KPZ,
KQA,
KQB,
KQC,
KQD,
KQE,
KQF,
KQG,
KQH,
KQI,
KQJ,
KQK,
KQL,
KQM,
KQN,
KQO,
KQP,
KQQ,
KQR,
KQS,
KQT,
KQU,
KQV,
KQW,
KQX,
KQY,
KQZ,
KRA,
KRB,
KRD,
KRE,
KRF,
KRH,
KRI,
KRJ,
KRK,
KRN,
KRP,
KRR,
KRS,
KRT,
KRV,
KRW,
KRX,
KRY,
KRZ,
KSA,
KSB,
KSC,
KSD,
KSE,
KSF,
KSG,
KSH,
KSI,
KSJ,
KSK,
KSL,
KSM,
KSN,
KSO,
KSP,
KSQ,
KSR,
KSS,
KST,
KSU,
KSV,
KSW,
KSX,
KSY,
KSZ,
KTA,
KTB,
KTC,
KTD,
KTE,
KTF,
KTG,
KTH,
KTI,
KTJ,
KTK,
KTL,
KTM,
KTN,
KTO,
KTP,
KTQ,
KTS,
KTT,
KTU,
KTV,
KTW,
KTX,
KTY,
KTZ,
KUB,
KUC,
KUD,
KUE,
KUF,
KUG,
KUH,
KUI,
KUJ,
KUK,
KUL,
KUN,
KUO,
KUP,
KUQ,
KUS,
KUU,
KUV,
KUW,
KUX,
KUY,
KUZ,
KVA,
KVB,
KVC,
KVD,
KVE,
KVF,
KVG,
KVH,
KVI,
KVJ,
KVK,
KVL,
KVM,
KVN,
KVO,
KVP,
KVQ,
KVR,
KVT,
KVU,
KVV,
KVW,
KVX,
KVY,
KVZ,
KWA,
KWB,
KWC,
KWD,
KWE,
KWF,
KWG,
KWH,
KWI,
KWJ,
KWK,
KWL,
KWM,
KWN,
KWO,
KWP,
KWR,
KWS,
KWT,
KWU,
KWV,
KWW,
KWX,
KWY,
KWZ,
KXA,
KXB,
KXC,
KXD,
KXF,
KXH,
KXI,
KXJ,
KXK,
KXM,
KXN,
KXO,
KXP,
KXQ,
KXR,
KXS,
KXT,
KXV,
KXW,
KXX,
KXY,
KXZ,
KYA,
KYB,
KYC,
KYD,
KYE,
KYF,
KYG,
KYH,
KYI,
KYJ,
KYK,
KYL,
KYM,
KYN,
KYO,
KYP,
KYQ,
KYR,
KYS,
KYT,
KYU,
KYV,
KYW,
KYX,
KYY,
KYZ,
KZA,
KZB,
KZC,
KZD,
KZE,
KZF,
KZG,
KZI,
KZK,
KZL,
KZM,
KZN,
KZO,
KZP,
KZQ,
KZR,
KZS,
KZU,
KZV,
KZW,
KZX,
KZY,
KZZ,
LAA,
LAB,
LAC,
LAE,
LAF,
LAG,
LAI,
LAJ,
LAL,
LAN,
LAP,
LAQ,
LAR,
LAS,
LAU,
LAW,
LAX,
LAY,
LAZ,
LBB,
LBC,
LBE,
LBF,
LBG,
LBI,
LBJ,
LBK,
LBL,
LBM,
LBN,
LBO,
LBQ,
LBR,
LBS,
LBT,
LBU,
LBV,
LBW,
LBX,
LBY,
LBZ,
LCC,
LCD,
LCE,
LCF,
LCH,
LCL,
LCM,
LCP,
LCQ,
LCS,
LDA,
LDB,
LDD,
LDG,
LDH,
LDI,
LDJ,
LDK,
LDL,
LDM,
LDN,
LDO,
LDP,
LDQ,
LEA,
LEB,
LEC,
LED,
LEE,
LEF,
LEH,
LEI,
LEJ,
LEK,
LEL,
LEM,
LEN,
LEO,
LEP,
LEQ,
LER,
LES,
LET,
LEU,
LEV,
LEW,
LEX,
LEY,
LFA,
LFN,
LGA,
LGB,
LGG,
LGH,
LGI,
LGK,
LGL,
LGM,
LGN,
LGO,
LGQ,
LGR,
LGT,
LGU,
LGZ,
LHA,
LHH,
LHI,
LHL,
LHM,
LHN,
LHP,
LHS,
LHT,
LHU,
LIA,
LIB,
LIC,
LID,
LIE,
LIF,
LIG,
LIH,
LIJ,
LIK,
LIL,
LIO,
LIP,
LIQ,
LIR,
LIS,
LIU,
LIV,
LIW,
LIX,
LIY,
LIZ,
LJA,
LJE,
LJI,
LJL,
LJP,
LJW,
LJX,
LKA,
LKB,
LKC,
LKD,
LKE,
LKH,
LKI,
LKJ,
LKL,
LKM,
LKN,
LKO,
LKR,
LKS,
LKT,
LKU,
LKY,
LLA,
LLB,
LLC,
LLD,
LLE,
LLF,
LLG,
LLH,
LLI,
LLJ,
LLK,
LLL,
LLM,
LLN,
LLP,
LLQ,
LLS,
LLU,
LLX,
LMA,
LMB,
LMC,
LMD,
LME,
LMF,
LMG,
LMH,
LMI,
LMJ,
LMK,
LML,
LMN,
LMO,
LMP,
LMQ,
LMR,
LMU,
LMV,
LMW,
LMX,
LMY,
LNA,
LNB,
LND,
LNG,
LNH,
LNI,
LNJ,
LNL,
LNM,
LNN,
LNS,
LNU,
LNW,
LNZ,
LOA,
LOB,
LOC,
LOE,
LOF,
LOG,
LOH,
LOI,
LOJ,
LOK,
LOM,
LON,
LOO,
LOP,
LOQ,
LOR,
LOS,
LOT,
LOU,
LOV,
LOW,
LOX,
LOY,
LPA,
LPE,
LPN,
LPO,
LPX,
LQR,
LRA,
LRC,
LRE,
LRG,
LRI,
LRK,
LRL,
LRM,
LRN,
LRO,
LRR,
LRT,
LRV,
LRZ,
LSA,
LSB,
LSC,
LSD,
LSE,
LSH,
LSI,
LSL,
LSM,
LSN,
LSO,
LSP,
LSR,
LSS,
LST,
LSV,
LSW,
LSY,
LTC,
LTG,
LTH,
LTI,
LTN,
LTO,
LTS,
LTU,
LUC,
LUD,
LUE,
LUF,
LUJ,
LUK,
LUL,
LUM,
LUP,
LUQ,
LUR,
LUT,
LUU,
LUV,
LUW,
LUY,
LUZ,
LVA,
LVI,
LVK,
LVS,
LVU,
LWA,
LWE,
LWG,
LWH,
LWL,
LWM,
LWO,
LWS,
LWT,
LWU,
LWW,
LXM,
LYA,
LYG,
LYN,
LZH,
LZL,
LZN,
LZZ,
MAA,
MAB,
MAE,
MAF,
MAJ,
MAM,
MAQ,
MAT,
MAU,
MAV,
MAW,
MAX,
MAZ,
MBA,
MBB,
MBC,
MBD,
MBE,
MBF,
MBH,
MBI,
MBJ,
MBK,
MBL,
MBM,
MBN,
MBO,
MBP,
MBQ,
MBR,
MBS,
MBT,
MBU,
MBV,
MBW,
MBX,
MBY,
MBZ,
MCA,
MCB,
MCC,
MCD,
MCE,
MCF,
MCG,
MCH,
MCI,
MCJ,
MCK,
MCL,
MCM,
MCN,
MCO,
MCP,
MCQ,
MCR,
MCS,
MCT,
MCU,
MCV,
MCW,
MCX,
MCY,
MCZ,
MDA,
MDB,
MDC,
MDD,
MDE,
MDG,
MDH,
MDI,
MDJ,
MDK,
MDL,
MDM,
MDN,
MDP,
MDQ,
MDS,
MDT,
MDU,
MDV,
MDW,
MDX,
MDY,
MDZ,
MEA,
MEB,
MEC,
MED,
MEE,
MEF,
MEH,
MEI,
MEJ,
MEK,
MEL,
MEM,
MEO,
MEP,
MEQ,
MER,
MES,
MET,
MEU,
MEV,
MEW,
MEY,
MEZ,
MFA,
MFB,
MFC,
MFD,
MFE,
MFF,
MFG,
MFH,
MFI,
MFJ,
MFK,
MFL,
MFM,
MFN,
MFO,
MFP,
MFQ,
MFR,
MFS,
MFT,
MFU,
MFV,
MFW,
MFX,
MFY,
MFZ,
MGB,
MGC,
MGD,
MGE,
MGF,
MGG,
MGH,
MGI,
MGJ,
MGK,
MGL,
MGM,
MGN,
MGO,
MGP,
MGQ,
MGR,
MGS,
MGT,
MGU,
MGV,
MGW,
MGY,
MGZ,
MHA,
MHB,
MHC,
MHD,
MHE,
MHF,
MHG,
MHI,
MHJ,
MHK,
MHL,
MHM,
MHN,
MHO,
MHP,
MHQ,
MHR,
MHS,
MHT,
MHU,
MHW,
MHX,
MHY,
MHZ,
MIA,
MIB,
MID,
MIE,
MIF,
MIG,
MIH,
MII,
MIJ,
MIK,
MIL,
MIM,
MIO,
MIP,
MIQ,
MIR,
MIT,
MIU,
MIW,
MIX,
MIY,
MIZ,
MJB,
MJC,
MJD,
MJE,
MJG,
MJH,
MJI,
MJJ,
MJK,
MJL,
MJM,
MJN,
MJO,
MJP,
MJQ,
MJR,
MJS,
MJT,
MJU,
MJV,
MJW,
MJX,
MJY,
MJZ,
MKA,
MKB,
MKC,
MKE,
MKF,
MKG,
MKI,
MKJ,
MKK,
MKL,
MKM,
MKN,
MKO,
MKP,
MKQ,
MKR,
MKS,
MKT,
MKU,
MKV,
MKW,
MKX,
MKY,
MKZ,
MLA,
MLB,
MLC,
MLE,
MLF,
MLH,
MLI,
MLJ,
MLK,
MLL,
MLM,
MLN,
MLO,
MLP,
MLQ,
MLR,
MLS,
MLU,
MLV,
MLW,
MLX,
MLZ,
MMA,
MMB,
MMC,
MMD,
MME,
MMF,
MMG,
MMH,
MMI,
MMJ,
MMK,
MML,
MMM,
MMN,
MMO,
MMP,
MMQ,
MMR,
MMT,
MMU,
MMV,
MMW,
MMX,
MMY,
MMZ,
MNA,
MNB,
MND,
MNE,
MNF,
MNG,
MNH,
MNJ,
MNK,
MNL,
MNM,
MNN,
MNP,
MNQ,
MNR,
MNS,
MNU,
MNV,
MNW,
MNX,
MNY,
MNZ,
MOA,
MOC,
MOD,
MOE,
MOG,
MOI,
MOJ,
MOK,
MOM,
MOO,
MOP,
MOQ,
MOR,
MOT,
MOU,
MOV,
MOW,
MOX,
MOY,
MOZ,
MPA,
MPB,
MPC,
MPD,
MPE,
MPG,
MPH,
MPI,
MPJ,
MPK,
MPL,
MPM,
MPN,
MPO,
MPP,
MPQ,
MPR,
MPS,
MPT,
MPU,
MPV,
MPW,
MPX,
MPY,
MPZ,
MQA,
MQB,
MQC,
MQE,
MQF,
MQG,
MQH,
MQI,
MQJ,
MQK,
MQL,
MQM,
MQN,
MQO,
MQP,
MQQ,
MQR,
MQS,
MQT,
MQU,
MQV,
MQW,
MQX,
MQY,
MQZ,
MRA,
MRB,
MRC,
MRD,
MRE,
MRF,
MRG,
MRH,
MRJ,
MRK,
MRL,
MRM,
MRN,
MRO,
MRP,
MRQ,
MRR,
MRS,
MRT,
MRU,
MRV,
MRW,
MRX,
MRY,
MRZ,
MSB,
MSC,
MSD,
MSE,
MSF,
MSG,
MSH,
MSI,
MSJ,
MSK,
MSL,
MSM,
MSN,
MSO,
MSP,
MSQ,
MSR,
MSS,
MSU,
MSV,
MSW,
MSX,
MSY,
MSZ,
MTA,
MTB,
MTC,
MTD,
MTE,
MTF,
MTG,
MTH,
MTI,
MTJ,
MTK,
MTL,
MTM,
MTN,
MTO,
MTP,
MTQ,
MTR,
MTS,
MTT,
MTU,
MTV,
MTW,
MTX,
MTY,
MUA,
MUB,
MUC,
MUD,
MUE,
MUG,
MUH,
MUI,
MUJ,
MUK,
MUM,
MUO,
MUP,
MUQ,
MUR,
MUT,
MUU,
MUV,
MUX,
MUY,
MUZ,
MVA,
MVB,
MVD,
MVE,
MVF,
MVG,
MVH,
MVI,
MVK,
MVL,
MVN,
MVO,
MVP,
MVQ,
MVR,
MVS,
MVT,
MVU,
MVV,
MVW,
MVX,
MVY,
MVZ,
MWA,
MWB,
MWC,
MWE,
MWF,
MWG,
MWH,
MWI,
MWK,
MWM,
MWN,
MWO,
MWP,
MWQ,
MWS,
MWT,
MWU,
MWV,
MWW,
MWZ,
MXA,
MXB,
MXC,
MXD,
MXE,
MXF,
MXG,
MXH,
MXI,
MXJ,
MXK,
MXL,
MXM,
MXN,
MXO,
MXP,
MXQ,
MXR,
MXS,
MXT,
MXU,
MXV,
MXW,
MXX,
MXY,
MXZ,
MYB,
MYC,
MYE,
MYF,
MYG,
MYH,
MYJ,
MYK,
MYL,
MYM,
MYO,
MYP,
MYR,
MYS,
MYU,
MYW,
MYX,
MYY,
MYZ,
MZA,
MZB,
MZC,
MZD,
MZE,
MZG,
MZH,
MZI,
MZJ,
MZK,
MZL,
MZM,
MZN,
MZO,
MZP,
MZQ,
MZR,
MZS,
MZT,
MZU,
MZV,
MZW,
MZX,
MZY,
MZZ,
NAA,
NAB,
NAC,
NAE,
NAF,
NAG,
NAJ,
NAK,
NAL,
NAM,
NAN,
NAO,
NAQ,
NAR,
NAS,
NAT,
NAW,
NAX,
NAY,
NAZ,
NBA,
NBB,
NBC,
NBD,
NBE,
NBG,
NBH,
NBI,
NBJ,
NBK,
NBM,
NBN,
NBO,
NBP,
NBQ,
NBR,
NBS,
NBT,
NBU,
NBV,
NBW,
NBY,
NCA,
NCB,
NCC,
NCD,
NCE,
NCF,
NCG,
NCH,
NCI,
NCJ,
NCK,
NCL,
NCM,
NCN,
NCO,
NCQ,
NCR,
NCS,
NCT,
NCU,
NCX,
NCZ,
NDA,
NDB,
NDC,
NDD,
NDF,
NDG,
NDH,
NDI,
NDJ,
NDK,
NDL,
NDM,
NDN,
NDP,
NDQ,
NDR,
NDT,
NDU,
NDV,
NDW,
NDX,
NDY,
NDZ,
NEA,
NEB,
NEC,
NED,
NEE,
NEF,
NEG,
NEH,
NEI,
NEJ,
NEK,
NEM,
NEN,
NEO,
NEQ,
NER,
NES,
NET,
NEU,
NEV,
NEX,
NEY,
NEZ,
NFA,
NFD,
NFL,
NFR,
NFU,
NGA,
NGB,
NGC,
NGD,
NGE,
NGG,
NGH,
NGI,
NGJ,
NGK,
NGL,
NGM,
NGN,
NGP,
NGQ,
NGR,
NGS,
NGT,
NGU,
NGV,
NGW,
NGX,
NGY,
NGZ,
NHA,
NHB,
NHC,
NHD,
NHE,
NHF,
NHG,
NHH,
NHI,
NHK,
NHM,
NHN,
NHO,
NHP,
NHQ,
NHR,
NHT,
NHU,
NHV,
NHW,
NHX,
NHY,
NHZ,
NIB,
NID,
NIE,
NIF,
NIG,
NIH,
NII,
NIJ,
NIK,
NIL,
NIM,
NIN,
NIO,
NIQ,
NIR,
NIS,
NIT,
NIV,
NIW,
NIX,
NIY,
NIZ,
NJA,
NJB,
NJD,
NJH,
NJI,
NJJ,
NJL,
NJM,
NJN,
NJO,
NJR,
NJS,
NJT,
NJU,
NJX,
NJY,
NJZ,
NKA,
NKB,
NKC,
NKD,
NKE,
NKF,
NKG,
NKH,
NKI,
NKJ,
NKK,
NKM,
NKN,
NKO,
NKP,
NKQ,
NKR,
NKS,
NKT,
NKU,
NKV,
NKW,
NKX,
NKZ,
NLA,
NLC,
NLE,
NLG,
NLI,
NLJ,
NLK,
NLL,
NLM,
NLO,
NLQ,
NLU,
NLV,
NLW,
NLX,
NLY,
NLZ,
NMA,
NMB,
NMC,
NMD,
NME,
NMF,
NMG,
NMH,
NMI,
NMJ,
NMK,
NML,
NMM,
NMN,
NMO,
NMP,
NMQ,
NMR,
NMS,
NMT,
NMU,
NMV,
NMW,
NMX,
NMY,
NMZ,
NNA,
NNB,
NNC,
NND,
NNE,
NNF,
NNG,
NNH,
NNI,
NNJ,
NNK,
NNL,
NNM,
NNN,
NNP,
NNQ,
NNR,
NNT,
NNU,
NNV,
NNW,
NNY,
NNZ,
NOA,
NOC,
NOD,
NOE,
NOF,
NOH,
NOI,
NOJ,
NOK,
NOL,
NOM,
NOP,
NOQ,
NOS,
NOT,
NOU,
NOV,
NOW,
NOY,
NOZ,
NPA,
NPB,
NPG,
NPH,
NPI,
NPL,
NPN,
NPO,
NPS,
NPU,
NPX,
NPY,
NQG,
NQK,
NQL,
NQM,
NQN,
NQQ,
NQT,
NQY,
NRA,
NRB,
NRC,
NRE,
NRF,
NRG,
NRI,
NRK,
NRL,
NRM,
NRN,
NRP,
NRR,
NRT,
NRU,
NRX,
NRZ,
NSA,
NSB,
NSC,
NSD,
NSE,
NSF,
NSG,
NSH,
NSI,
NSK,
NSL,
NSM,
NSN,
NSP,
NSQ,
NSR,
NSS,
NST,
NSU,
NSV,
NSW,
NSX,
NSY,
NSZ,
NTD,
NTE,
NTG,
NTI,
NTJ,
NTK,
NTM,
NTO,
NTP,
NTR,
NTU,
NTW,
NTX,
NTY,
NTZ,
NUA,
NUC,
NUD,
NUE,
NUF,
NUG,
NUH,
NUI,
NUJ,
NUK,
NUL,
NUM,
NUN,
NUO,
NUP,
NUQ,
NUR,
NUS,
NUT,
NUU,
NUV,
NUW,
NUX,
NUY,
NUZ,
NVH,
NVM,
NVO,
NWA,
NWB,
NWE,
NWG,
NWI,
NWM,
NWO,
NWR,
NWW,
NWX,
NWY,
NXA,
NXD,
NXE,
NXG,
NXI,
NXK,
NXL,
NXM,
NXN,
NXO,
NXQ,
NXR,
NXX,
NYB,
NYC,
NYD,
NYE,
NYF,
NYG,
NYH,
NYI,
NYJ,
NYK,
NYL,
NYP,
NYQ,
NYR,
NYS,
NYT,
NYU,
NYV,
NYW,
NYX,
NYY,
NZA,
NZB,
NZD,
NZK,
NZM,
NZS,
NZU,
NZY,
NZZ,
OAA,
OAC,
OAR,
OAV,
OBI,
OBK,
OBL,
OBM,
OBO,
OBR,
OBT,
OBU,
OCA,
OCH,
OCM,
OCO,
OCU,
ODA,
ODK,
ODT,
ODU,
OFO,
OFS,
OFU,
OGB,
OGC,
OGE,
OGG,
OGO,
OGU,
OHT,
OHU,
OIA,
OIE,
OIN,
OJB,
OJC,
OJG,
OJP,
OJS,
OJV,
OJW,
OKA,
OKB,
OKC,
OKD,
OKE,
OKG,
OKH,
OKI,
OKJ,
OKK,
OKL,
OKM,
OKN,
OKO,
OKR,
OKS,
OKU,
OKV,
OKX,
OKZ,
OLA,
OLD,
OLE,
OLK,
OLM,
OLO,
OLR,
OLT,
OLU,
OMA,
OMB,
OMC,
OMG,
OMI,
OMK,
OML,
OMN,
OMO,
OMP,
OMR,
OMT,
OMU,
OMW,
OMX,
OMY,
ONA,
ONB,
ONE,
ONG,
ONI,
ONJ,
ONK,
ONN,
ONO,
ONP,
ONR,
ONS,
ONT,
ONU,
ONW,
ONX,
OOD,
OOG,
OON,
OOR,
OOS,
OPA,
OPK,
OPM,
OPO,
OPT,
OPY,
ORA,
ORC,
ORE,
ORG,
ORH,
ORN,
ORO,
ORR,
ORS,
ORT,
ORU,
ORV,
ORW,
ORX,
ORY,
ORZ,
OSC,
OSI,
OSN,
OSO,
OSP,
OST,
OSU,
OSX,
OTB,
OTD,
OTE,
OTI,
OTK,
OTL,
OTM,
OTN,
OTQ,
OTR,
OTS,
OTT,
OTU,
OTW,
OTX,
OTY,
OTZ,
OUA,
OUB,
OUE,
OUI,
OUM,
OVD,
OWI,
OWL,
OYB,
OYD,
OYM,
OYY,
OZM,
PAB,
PAC,
PAD,
PAE,
PAF,
PAH,
PAI,
PAK,
PAO,
PAQ,
PAR,
PAS,
PAV,
PAW,
PAX,
PAY,
PAZ,
PBB,
PBC,
PBE,
PBF,
PBG,
PBH,
PBI,
PBL,
PBM,
PBN,
PBO,
PBP,
PBR,
PBS,
PBT,
PBU,
PBV,
PBY,
PCA,
PCB,
PCC,
PCD,
PCE,
PCF,
PCG,
PCH,
PCI,
PCJ,
PCK,
PCL,
PCM,
PCN,
PCP,
PCW,
PDA,
PDC,
PDI,
PDN,
PDO,
PDT,
PDU,
PEA,
PEB,
PED,
PEE,
PEF,
PEG,
PEH,
PEI,
PEJ,
PEK,
PEL,
PEM,
PEP,
PEQ,
PES,
PEV,
PEX,
PEY,
PEZ,
PFA,
PFE,
PFL,
PGA,
PGD,
PGG,
PGI,
PGK,
PGL,
PGN,
PGS,
PGU,
PGZ,
PHA,
PHD,
PHG,
PHH,
PHJ,
PHK,
PHL,
PHM,
PHO,
PHQ,
PHR,
PHT,
PHU,
PHV,
PHW,
PIA,
PIB,
PIC,
PID,
PIE,
PIF,
PIG,
PIH,
PIJ,
PIL,
PIM,
PIN,
PIO,
PIP,
PIR,
PIS,
PIT,
PIU,
PIV,
PIW,
PIX,
PIY,
PIZ,
PJT,
PKA,
PKB,
PKC,
PKG,
PKH,
PKN,
PKO,
PKP,
PKR,
PKS,
PKT,
PKU,
PLA,
PLB,
PLC,
PLD,
PLE,
PLG,
PLH,
PLJ,
PLK,
PLL,
PLN,
PLO,
PLQ,
PLR,
PLS,
PLT,
PLU,
PLV,
PLW,
PLY,
PLZ,
PMA,
PMB,
PMD,
PME,
PMF,
PMH,
PMI,
PMJ,
PMK,
PML,
PMM,
PMN,
PMO,
PMQ,
PMR,
PMS,
PMT,
PMW,
PMX,
PMY,
PMZ,
PNA,
PNB,
PNC,
PND,
PNE,
PNG,
PNH,
PNI,
PNJ,
PNK,
PNL,
PNM,
PNN,
PNO,
PNP,
PNQ,
PNR,
PNS,
PNT,
PNU,
PNV,
PNW,
PNX,
PNY,
PNZ,
POC,
POE,
POF,
POG,
POH,
POI,
POK,
POM,
POO,
POP,
POQ,
POS,
POT,
POV,
POW,
POX,
POY,
PPE,
PPI,
PPK,
PPL,
PPM,
PPN,
PPO,
PPP,
PPQ,
PPS,
PPT,
PPU,
PQA,
PQM,
PRC,
PRD,
PRE,
PRF,
PRG,
PRH,
PRI,
PRK,
PRL,
PRM,
PRN,
PRP,
PRQ,
PRR,
PRS,
PRT,
PRU,
PRW,
PRX,
PRZ,
PSA,
PSC,
PSD,
PSE,
PSG,
PSH,
PSI,
PSL,
PSM,
PSN,
PSO,
PSP,
PSQ,
PSR,
PSS,
PST,
PSU,
PSW,
PSY,
PTA,
PTH,
PTI,
PTN,
PTO,
PTP,
PTQ,
PTR,
PTT,
PTU,
PTV,
PTW,
PTY,
PUA,
PUB,
PUC,
PUD,
PUE,
PUF,
PUG,
PUI,
PUJ,
PUM,
PUO,
PUP,
PUQ,
PUR,
PUT,
PUU,
PUW,
PUX,
PUY,
PWA,
PWB,
PWG,
PWI,
PWM,
PWN,
PWO,
PWR,
PWW,
PXM,
PYE,
PYM,
PYN,
PYS,
PYU,
PYX,
PYY,
PZH,
PZN,
QUA,
QUB,
QUC,
QUD,
QUF,
QUG,
QUH,
QUI,
QUK,
QUL,
QUM,
QUN,
QUP,
QUQ,
QUR,
QUS,
QUV,
QUW,
QUX,
QUY,
QUZ,
QVA,
QVC,
QVE,
QVH,
QVI,
QVJ,
QVL,
QVM,
QVN,
QVO,
QVP,
QVS,
QVW,
QVY,
QVZ,
QWA,
QWC,
QWH,
QWM,
QWS,
QWT,
QXA,
QXC,
QXH,
QXL,
QXN,
QXO,
QXP,
QXQ,
QXR,
QXS,
QXT,
QXU,
QXW,
QYA,
QYP,
RAA,
RAB,
RAC,
RAD,
RAF,
RAG,
RAH,
RAI,
RAK,
RAL,
RAM,
RAN,
RAO,
RAQ,
RAS,
RAT,
RAU,
RAV,
RAW,
RAX,
RAY,
RAZ,
RBB,
RBK,
RBL,
RBP,
RCF,
RDB,
REA,
REB,
REE,
REG,
REI,
REJ,
REL,
REM,
REN,
RER,
RES,
RET,
REY,
RGA,
RGE,
RGK,
RGN,
RGR,
RGS,
RGU,
RHG,
RHP,
RIA,
RIB,
RIF,
RIL,
RIM,
RIN,
RIR,
RIT,
RIU,
RJG,
RJI,
RJS,
RKA,
RKB,
RKH,
RKI,
RKM,
RKT,
RKW,
RMA,
RMB,
RMC,
RMD,
RME,
RMF,
RMG,
RMH,
RMI,
RMK,
RML,
RMM,
RMN,
RMO,
RMP,
RMQ,
RMS,
RMT,
RMU,
RMV,
RMW,
RMX,
RMY,
RMZ,
RNB,
RND,
RNG,
RNL,
RNN,
RNP,
RNR,
RNW,
ROB,
ROC,
ROD,
ROE,
ROF,
ROG,
ROL,
ROO,
ROP,
ROR,
ROU,
ROW,
RPN,
RPT,
RRI,
RRO,
RRT,
RSB,
RSK,
RSL,
RSM,
RSN,
RTC,
RTH,
RTM,
RTS,
RTW,
RUB,
RUC,
RUE,
RUF,
RUG,
RUH,
RUI,
RUK,
RUO,
RUQ,
RUT,
RUU,
RUY,
RUZ,
RWA,
RWK,
RWL,
RWM,
RWO,
RWR,
RXD,
RXW,
RYN,
RYS,
RYU,
RZH,
SAA,
SAB,
SAC,
SAE,
SAF,
SAJ,
SAK,
SAO,
SAQ,
SAR,
SAU,
SAV,
SAW,
SAX,
SAY,
SAZ,
SBA,
SBB,
SBC,
SBD,
SBE,
SBF,
SBG,
SBH,
SBI,
SBJ,
SBK,
SBL,
SBM,
SBN,
SBO,
SBP,
SBQ,
SBR,
SBS,
SBT,
SBU,
SBV,
SBW,
SBX,
SBY,
SBZ,
SCB,
SCE,
SCF,
SCG,
SCH,
SCI,
SCK,
SCL,
SCP,
SCQ,
SCS,
SCT,
SCU,
SCV,
SCW,
SCX,
SDA,
SDB,
SDC,
SDE,
SDF,
SDG,
SDH,
SDJ,
SDK,
SDL,
SDN,
SDO,
SDP,
SDQ,
SDR,
SDS,
SDT,
SDU,
SDX,
SDZ,
SEA,
SEB,
SEC,
SED,
SEE,
SEF,
SEG,
SEH,
SEI,
SEJ,
SEK,
SEN,
SEO,
SEP,
SEQ,
SER,
SES,
SET,
SEU,
SEV,
SEW,
SEY,
SEZ,
SFB,
SFE,
SFM,
SFS,
SFW,
SGB,
SGC,
SGD,
SGE,
SGG,
SGH,
SGI,
SGJ,
SGK,
SGM,
SGP,
SGR,
SGS,
SGT,
SGU,
SGW,
SGX,
SGY,
SGZ,
SHA,
SHB,
SHC,
SHD,
SHE,
SHG,
SHH,
SHI,
SHJ,
SHK,
SHL,
SHM,
SHO,
SHP,
SHQ,
SHR,
SHS,
SHT,
SHU,
SHV,
SHW,
SHX,
SHY,
SHZ,
SIA,
SIB,
SIE,
SIF,
SIG,
SIH,
SII,
SIJ,
SIK,
SIL,
SIM,
SIP,
SIQ,
SIR,
SIS,
SIU,
SIV,
SIW,
SIX,
SIY,
SIZ,
SJA,
SJB,
SJD,
SJE,
SJG,
SJK,
SJL,
SJM,
SJN,
SJO,
SJP,
SJR,
SJS,
SJT,
SJU,
SJW,
SKA,
SKB,
SKC,
SKD,
SKE,
SKF,
SKG,
SKH,
SKI,
SKJ,
SKM,
SKN,
SKO,
SKP,
SKQ,
SKR,
SKS,
SKT,
SKU,
SKV,
SKW,
SKX,
SKY,
SKZ,
SLC,
SLD,
SLE,
SLF,
SLG,
SLH,
SLI,
SLJ,
SLL,
SLM,
SLN,
SLP,
SLQ,
SLR,
SLS,
SLT,
SLU,
SLW,
SLX,
SLY,
SLZ,
SMB,
SMC,
SMF,
SMG,
SMH,
SMK,
SML,
SMM,
SMP,
SMQ,
SMR,
SMT,
SMU,
SMV,
SMW,
SMX,
SMY,
SMZ,
SNC,
SNE,
SNF,
SNG,
SNI,
SNJ,
SNL,
SNM,
SNN,
SNO,
SNP,
SNQ,
SNR,
SNS,
SNU,
SNV,
SNW,
SNX,
SNY,
SNZ,
SOA,
SOB,
SOC,
SOD,
SOE,
SOH,
SOI,
SOJ,
SOK,
SOL,
SOO,
SOP,
SOQ,
SOR,
SOS,
SOU,
SOV,
SOW,
SOX,
SOY,
SOZ,
SPB,
SPC,
SPD,
SPE,
SPG,
SPI,
SPK,
SPL,
SPM,
SPN,
SPO,
SPP,
SPQ,
SPR,
SPS,
SPT,
SPU,
SPV,
SPX,
SPY,
SQA,
SQH,
SQK,
SQM,
SQN,
SQO,
SQQ,
SQR,
SQS,
SQT,
SQU,
SQX,
SRA,
SRB,
SRC,
SRE,
SRF,
SRG,
SRH,
SRI,
SRK,
SRL,
SRM,
SRO,
SRQ,
SRS,
SRT,
SRU,
SRV,
SRW,
SRX,
SRY,
SRZ,
SSB,
SSC,
SSD,
SSE,
SSF,
SSG,
SSH,
SSI,
SSJ,
SSK,
SSL,
SSM,
SSN,
SSO,
SSP,
SSQ,
SSR,
SSS,
SST,
SSU,
SSV,
SSX,
SSY,
SSZ,
STA,
STB,
STD,
STE,
STF,
STG,
STH,
STI,
STJ,
STK,
STL,
STM,
STN,
STO,
STP,
STQ,
STR,
STS,
STT,
STU,
STV,
STW,
STY,
SUA,
SUB,
SUC,
SUE,
SUG,
SUI,
SUJ,
SUO,
SUQ,
SUR,
SUT,
SUV,
SUW,
SUY,
SUZ,
SVA,
SVB,
SVC,
SVE,
SVK,
SVM,
SVS,
SVX,
SWB,
SWC,
SWF,
SWG,
SWH,
SWI,
SWJ,
SWK,
SWL,
SWM,
SWN,
SWO,
SWP,
SWQ,
SWR,
SWS,
SWT,
SWU,
SWV,
SWW,
SWX,
SWY,
SXB,
SXC,
SXE,
SXG,
SXK,
SXL,
SXM,
SXN,
SXO,
SXR,
SXS,
SXU,
SXW,
SYA,
SYB,
SYI,
SYK,
SYL,
SYM,
SYN,
SYO,
SYS,
SYW,
SYX,
SYY,
SZA,
SZB,
SZC,
SZD,
SZE,
SZG,
SZL,
SZN,
SZP,
SZS,
SZV,
SZW,
SZY,
TAA,
TAB,
TAC,
TAD,
TAE,
TAF,
TAG,
TAJ,
TAK,
TAL,
TAN,
TAO,
TAP,
TAQ,
TAR,
TAS,
TAU,
TAV,
TAW,
TAX,
TAY,
TAZ,
TBA,
TBC,
TBD,
TBE,
TBF,
TBG,
TBH,
TBI,
TBJ,
TBK,
TBL,
TBM,
TBN,
TBO,
TBP,
TBR,
TBS,
TBT,
TBU,
TBV,
TBW,
TBX,
TBY,
TBZ,
TCA,
TCB,
TCC,
TCD,
TCE,
TCF,
TCG,
TCH,
TCI,
TCK,
TCL,
TCM,
TCN,
TCO,
TCP,
TCQ,
TCS,
TCT,
TCU,
TCW,
TCX,
TCY,
TCZ,
TDA,
TDB,
TDC,
TDD,
TDE,
TDF,
TDG,
TDH,
TDI,
TDJ,
TDK,
TDL,
TDM,
TDN,
TDO,
TDQ,
TDR,
TDS,
TDT,
TDV,
TDX,
TDY,
TEA,
TEB,
TEC,
TED,
TEE,
TEF,
TEG,
TEH,
TEI,
TEK,
TEN,
TEO,
TEP,
TEQ,
TES,
TEU,
TEV,
TEW,
TEX,
TEY,
TEZ,
TFI,
TFN,
TFO,
TFR,
TFT,
TGA,
TGB,
TGC,
TGD,
TGE,
TGF,
TGH,
TGI,
TGJ,
TGN,
TGO,
TGP,
TGQ,
TGR,
TGS,
TGT,
TGU,
TGV,
TGW,
TGX,
TGY,
TGZ,
THD,
THE,
THF,
THH,
THI,
THK,
THL,
THM,
THN,
THP,
THQ,
THR,
THS,
THT,
THU,
THV,
THY,
THZ,
TIA,
TIC,
TIF,
TIH,
TII,
TIJ,
TIK,
TIL,
TIM,
TIN,
TIO,
TIP,
TIQ,
TIS,
TIT,
TIU,
TIW,
TIX,
TIY,
TIZ,
TJA,
TJG,
TJI,
TJJ,
TJL,
TJM,
TJN,
TJO,
TJP,
TJS,
TJU,
TJW,
TKA,
TKB,
TKD,
TKE,
TKF,
TKG,
TKM,
TKN,
TKP,
TKQ,
TKR,
TKS,
TKT,
TKU,
TKV,
TKW,
TKX,
TKZ,
TLA,
TLB,
TLC,
TLD,
TLF,
TLG,
TLJ,
TLK,
TLL,
TLM,
TLN,
TLO,
TLP,
TLQ,
TLR,
TLS,
TLT,
TLU,
TLV,
TLX,
TLY,
TMA,
TMB,
TMC,
TMD,
TME,
TMF,
TMG,
TMI,
TMJ,
TMK,
TML,
TMM,
TMN,
TMO,
TMQ,
TMR,
TMS,
TMT,
TMU,
TMV,
TMW,
TMY,
TMZ,
TNA,
TNB,
TNC,
TND,
TNG,
TNH,
TNI,
TNK,
TNL,
TNM,
TNN,
TNO,
TNP,
TNQ,
TNR,
TNS,
TNT,
TNU,
TNV,
TNW,
TNX,
TNY,
TNZ,
TOB,
TOC,
TOD,
TOF,
TOH,
TOI,
TOJ,
TOK,
TOL,
TOM,
TOO,
TOP,
TOQ,
TOR,
TOS,
TOU,
TOV,
TOW,
TOX,
TOY,
TOZ,
TPA,
TPC,
TPE,
TPF,
TPG,
TPJ,
TPK,
TPL,
TPM,
TPN,
TPO,
TPP,
TPQ,
TPR,
TPT,
TPU,
TPV,
TPW,
TPX,
TPY,
TPZ,
TQB,
TQL,
TQM,
TQN,
TQO,
TQP,
TQQ,
TQR,
TQT,
TQU,
TQW,
TRA,
TRB,
TRC,
TRD,
TRE,
TRF,
TRG,
TRH,
TRI,
TRJ,
TRL,
TRM,
TRN,
TRO,
TRP,
TRQ,
TRR,
TRS,
TRT,
TRU,
TRV,
TRW,
TRX,
TRY,
TRZ,
TSA,
TSB,
TSC,
TSD,
TSE,
TSG,
TSH,
TSJ,
TSK,
TSL,
TSM,
TSP,
TSQ,
TSR,
TSS,
TST,
TSU,
TSV,
TSW,
TSX,
TSY,
TSZ,
TTA,
TTB,
TTC,
TTD,
TTE,
TTF,
TTG,
TTH,
TTI,
TTJ,
TTK,
TTL,
TTM,
TTN,
TTO,
TTP,
TTQ,
TTR,
TTS,
TTT,
TTU,
TTV,
TTW,
TTY,
TTZ,
TUA,
TUB,
TUC,
TUD,
TUE,
TUF,
TUG,
TUH,
TUI,
TUJ,
TUL,
TUN,
TUO,
TUQ,
TUS,
TUU,
TUV,
TUX,
TUY,
TUZ,
TVA,
TVD,
TVE,
TVK,
TVM,
TVN,
TVO,
TVS,
TVT,
TVU,
TVW,
TVX,
TVY,
TWA,
TWB,
TWC,
TWD,
TWE,
TWF,
TWG,
TWH,
TWL,
TWM,
TWN,
TWO,
TWP,
TWQ,
TWR,
TWT,
TWU,
TWW,
TWX,
TWY,
TXA,
TXB,
TXC,
TXE,
TXG,
TXH,
TXI,
TXJ,
TXM,
TXN,
TXO,
TXQ,
TXR,
TXS,
TXT,
TXU,
TXX,
TXY,
TYA,
TYE,
TYH,
TYI,
TYJ,
TYL,
TYN,
TYP,
TYR,
TYS,
TYT,
TYU,
TYX,
TYY,
TYZ,
TZA,
TZH,
TZJ,
TZL,
TZM,
TZN,
TZO,
TZX,
UAM,
UAN,
UAR,
UBA,
UBI,
UBL,
UBR,
UBU,
UBY,
UDA,
UDE,
UDG,
UDI,
UDJ,
UDL,
UDU,
UES,
UFI,
UGB,
UGE,
UGH,
UGN,
UGO,
UGY,
UHA,
UHN,
UIS,
UIV,
UJI,
UKA,
UKG,
UKH,
UKI,
UKK,
UKL,
UKP,
UKQ,
UKS,
UKU,
UKV,
UKW,
UKY,
ULA,
ULB,
ULC,
ULE,
ULF,
ULI,
ULK,
ULL,
ULM,
ULN,
ULU,
ULW,
UMA,
UMC,
UMD,
UMG,
UMI,
UMM,
UMN,
UMO,
UMP,
UMR,
UMS,
UMU,
UNA,
UNE,
UNG,
UNI,
UNK,
UNM,
UNN,
UNR,
UNU,
UNX,
UNZ,
UON,
UPI,
UPV,
URA,
URB,
URC,
URE,
URF,
URG,
URH,
URI,
URK,
URL,
URM,
URN,
URO,
URP,
URR,
URT,
URU,
URV,
URW,
URX,
URY,
URZ,
USA,
USH,
USI,
USK,
USP,
USS,
USU,
UTA,
UTE,
UTH,
UTP,
UTR,
UTU,
UUM,
UUR,
UUU,
UVE,
UVH,
UVL,
UWA,
UYA,
UZN,
UZS,
VAA,
VAE,
VAF,
VAG,
VAH,
VAJ,
VAL,
VAM,
VAN,
VAO,
VAP,
VAR,
VAS,
VAU,
VAV,
VAY,
VBB,
VBK,
VEC,
VED,
VEL,
VEM,
VEO,
VEP,
VER,
VGR,
VGT,
VIC,
VID,
VIF,
VIG,
VIL,
VIN,
VIS,
VIT,
VIV,
VKA,
VKJ,
VKK,
VKL,
VKM,
VKN,
VKO,
VKP,
VKT,
VKU,
VKZ,
VLP,
VLS,
VMA,
VMB,
VMC,
VMD,
VME,
VMF,
VMG,
VMH,
VMI,
VMJ,
VMK,
VML,
VMM,
VMP,
VMQ,
VMR,
VMS,
VMU,
VMV,
VMW,
VMX,
VMY,
VMZ,
VNK,
VNM,
VNP,
VOR,
VRA,
VRO,
VRS,
VRT,
VSI,
VSL,
VSV,
VTO,
VUM,
VUN,
VUT,
VWA,
WAA,
WAB,
WAC,
WAD,
WAE,
WAF,
WAG,
WAH,
WAI,
WAJ,
WAM,
WAN,
WAO,
WAP,
WAQ,
WAT,
WAU,
WAV,
WAW,
WAX,
WAY,
WAZ,
WBA,
WBB,
WBE,
WBF,
WBH,
WBI,
WBJ,
WBK,
WBL,
WBM,
WBP,
WBQ,
WBR,
WBS,
WBT,
WBV,
WBW,
WCA,
WCI,
WDD,
WDG,
WDJ,
WDK,
WDT,
WDU,
WDY,
WEA,
WEC,
WED,
WEG,
WEH,
WEI,
WEM,
WEO,
WEP,
WER,
WES,
WET,
WEU,
WEW,
WFG,
WGA,
WGB,
WGG,
WGI,
WGO,
WGU,
WGY,
WHA,
WHG,
WHK,
WHU,
WIB,
WIC,
WIE,
WIF,
WIG,
WIH,
WII,
WIJ,
WIK,
WIL,
WIM,
WIN,
WIR,
WIU,
WIV,
WIY,
WJA,
WJI,
WKA,
WKB,
WKD,
WKL,
WKR,
WKU,
WKW,
WKY,
WLA,
WLC,
WLE,
WLG,
WLH,
WLI,
WLK,
WLL,
WLM,
WLO,
WLR,
WLS,
WLU,
WLV,
WLW,
WLX,
WLY,
WMA,
WMB,
WMC,
WMD,
WME,
WMG,
WMH,
WMI,
WMM,
WMN,
WMO,
WMS,
WMT,
WMW,
WMX,
WNB,
WNC,
WND,
WNE,
WNG,
WNI,
WNK,
WNM,
WNN,
WNO,
WNP,
WNU,
WNW,
WNY,
WOA,
WOB,
WOC,
WOD,
WOE,
WOF,
WOG,
WOI,
WOK,
WOM,
WON,
WOO,
WOR,
WOS,
WOW,
WOY,
WPC,
WRB,
WRG,
WRH,
WRI,
WRK,
WRL,
WRM,
WRN,
WRO,
WRP,
WRR,
WRS,
WRU,
WRV,
WRW,
WRX,
WRY,
WRZ,
WSA,
WSG,
WSI,
WSK,
WSR,
WSS,
WSU,
WSV,
WTF,
WTH,
WTI,
WTK,
WTM,
WTW,
WUA,
WUB,
WUD,
WUH,
WUL,
WUM,
WUN,
WUR,
WUT,
WUU,
WUV,
WUX,
WUY,
WWA,
WWB,
WWO,
WWR,
WWW,
WXA,
WXW,
WYB,
WYI,
WYM,
WYN,
WYR,
WYY,
XAA,
XAB,
XAC,
XAD,
XAE,
XAG,
XAI,
XAJ,
XAK,
XAM,
XAN,
XAO,
XAP,
XAQ,
XAR,
XAS,
XAT,
XAU,
XAV,
XAW,
XAY,
XBB,
XBC,
XBD,
XBE,
XBG,
XBI,
XBJ,
XBM,
XBN,
XBO,
XBP,
XBR,
XBW,
XBY,
XCB,
XCC,
XCE,
XCG,
XCH,
XCL,
XCM,
XCN,
XCO,
XCR,
XCT,
XCU,
XCV,
XCW,
XCY,
XDA,
XDC,
XDK,
XDM,
XDO,
XDQ,
XDY,
XEB,
XED,
XEG,
XEL,
XEM,
XEP,
XER,
XES,
XET,
XEU,
XFA,
XGA,
XGB,
XGD,
XGF,
XGG,
XGI,
XGL,
XGM,
XGR,
XGU,
XGW,
XHA,
XHC,
XHD,
XHE,
XHM,
XHR,
XHT,
XHU,
XHV,
XIB,
XII,
XIL,
XIN,
XIR,
XIS,
XIV,
XIY,
XJB,
XJT,
XKA,
XKB,
XKC,
XKD,
XKE,
XKF,
XKG,
XKI,
XKJ,
XKK,
XKL,
XKN,
XKO,
XKP,
XKQ,
XKR,
XKS,
XKT,
XKU,
XKV,
XKW,
XKX,
XKY,
XKZ,
XLA,
XLB,
XLC,
XLD,
XLE,
XLG,
XLI,
XLN,
XLO,
XLP,
XLS,
XLU,
XLY,
XMA,
XMB,
XMC,
XMD,
XME,
XMF,
XMG,
XMH,
XMJ,
XMK,
XML,
XMM,
XMN,
XMO,
XMP,
XMQ,
XMR,
XMS,
XMT,
XMU,
XMV,
XMW,
XMX,
XMY,
XMZ,
XNA,
XNB,
XNG,
XNH,
XNI,
XNJ,
XNK,
XNM,
XNN,
XNO,
XNQ,
XNR,
XNS,
XNT,
XNU,
XNY,
XNZ,
XOC,
XOD,
XOG,
XOI,
XOK,
XOM,
XON,
XOO,
XOP,
XOR,
XOW,
XPA,
XPB,
XPC,
XPD,
XPE,
XPF,
XPG,
XPH,
XPI,
XPJ,
XPK,
XPL,
XPM,
XPN,
XPO,
XPP,
XPQ,
XPR,
XPS,
XPT,
XPU,
XPV,
XPW,
XPX,
XPY,
XPZ,
XQA,
XQT,
XRA,
XRB,
XRD,
XRE,
XRG,
XRI,
XRM,
XRN,
XRR,
XRT,
XRU,
XRW,
XSA,
XSB,
XSC,
XSD,
XSE,
XSH,
XSI,
XSJ,
XSL,
XSM,
XSN,
XSO,
XSP,
XSQ,
XSR,
XSS,
XSU,
XSV,
XSY,
XTA,
XTB,
XTC,
XTD,
XTE,
XTG,
XTH,
XTI,
XTJ,
XTL,
XTM,
XTN,
XTO,
XTP,
XTQ,
XTR,
XTS,
XTT,
XTU,
XTV,
XTW,
XTY,
XUA,
XUB,
XUD,
XUG,
XUJ,
XUL,
XUM,
XUN,
XUO,
XUP,
XUR,
XUT,
XUU,
XVE,
XVI,
XVN,
XVO,
XVS,
XWA,
XWC,
XWD,
XWE,
XWG,
XWJ,
XWK,
XWL,
XWO,
XWR,
XWT,
XWW,
XXB,
XXK,
XXM,
XXR,
XXT,
XYA,
XYB,
XYJ,
XYK,
XYL,
XYT,
XYY,
XZH,
XZM,
XZP,
YAA,
YAB,
YAC,
YAD,
YAE,
YAF,
YAG,
YAH,
YAI,
YAJ,
YAK,
YAL,
YAM,
YAN,
YAQ,
YAR,
YAS,
YAT,
YAU,
YAV,
YAW,
YAX,
YAY,
YAZ,
YBA,
YBB,
YBE,
YBH,
YBI,
YBJ,
YBK,
YBL,
YBM,
YBN,
YBO,
YBX,
YBY,
YCH,
YCL,
YCN,
YCP,
YDA,
YDD,
YDE,
YDG,
YDK,
YEA,
YEC,
YEE,
YEI,
YEJ,
YEL,
YER,
YES,
YET,
YEU,
YEV,
YEY,
YGA,
YGI,
YGL,
YGM,
YGP,
YGR,
YGS,
YGU,
YGW,
YHA,
YHD,
YHL,
YHS,
YIA,
YIF,
YIG,
YIH,
YII,
YIJ,
YIK,
YIL,
YIM,
YIN,
YIP,
YIQ,
YIR,
YIS,
YIT,
YIU,
YIV,
YIX,
YIZ,
YKA,
YKG,
YKI,
YKK,
YKL,
YKM,
YKN,
YKO,
YKR,
YKT,
YKU,
YKY,
YLA,
YLB,
YLE,
YLG,
YLI,
YLL,
YLM,
YLN,
YLO,
YLR,
YLU,
YLY,
YMB,
YMC,
YMD,
YME,
YMG,
YMH,
YMI,
YMK,
YML,
YMM,
YMN,
YMO,
YMP,
YMQ,
YMR,
YMS,
YMX,
YMZ,
YNA,
YND,
YNE,
YNG,
YNK,
YNL,
YNN,
YNO,
YNQ,
YNS,
YNU,
YOB,
YOG,
YOI,
YOK,
YOL,
YOM,
YON,
YOT,
YOX,
YOY,
YPA,
YPB,
YPG,
YPH,
YPM,
YPN,
YPO,
YPP,
YPZ,
YRA,
YRB,
YRE,
YRK,
YRL,
YRM,
YRN,
YRO,
YRS,
YRW,
YRY,
YSC,
YSD,
YSG,
YSL,
YSM,
YSN,
YSO,
YSP,
YSR,
YSS,
YSY,
YTA,
YTL,
YTP,
YTW,
YTY,
YUA,
YUB,
YUC,
YUD,
YUE,
YUF,
YUG,
YUI,
YUJ,
YUK,
YUL,
YUM,
YUN,
YUP,
YUQ,
YUR,
YUT,
YUW,
YUX,
YUY,
YUZ,
YVA,
YVT,
YWA,
YWG,
YWL,
YWN,
YWQ,
YWR,
YWT,
YWU,
YWW,
YXA,
YXG,
YXL,
YXM,
YXU,
YXY,
YYR,
YYU,
YYZ,
YZG,
YZK,
ZAA,
ZAB,
ZAC,
ZAD,
ZAE,
ZAF,
ZAG,
ZAH,
ZAI,
ZAJ,
ZAK,
ZAL,
ZAM,
ZAO,
ZAQ,
ZAR,
ZAS,
ZAT,
ZAU,
ZAV,
ZAW,
ZAX,
ZAY,
ZAZ,
ZBA,
ZBC,
ZBE,
ZBT,
ZBU,
ZBW,
ZCA,
ZCD,
ZCH,
ZDJ,
ZEA,
ZEG,
ZEH,
ZGA,
ZGB,
ZGM,
ZGN,
ZGR,
ZHB,
ZHD,
ZHI,
ZHN,
ZHW,
ZIA,
ZIB,
ZIK,
ZIL,
ZIM,
ZIN,
ZIW,
ZIZ,
ZKA,
ZKB,
ZKD,
ZKG,
ZKH,
ZKK,
ZKN,
ZKO,
ZKP,
ZKR,
ZKT,
ZKU,
ZKV,
ZKZ,
ZLA,
ZLJ,
ZLM,
ZLN,
ZLQ,
ZMA,
ZMB,
ZMC,
ZMD,
ZME,
ZMF,
ZMG,
ZMH,
ZMI,
ZMJ,
ZMK,
ZML,
ZMM,
ZMN,
ZMO,
ZMP,
ZMQ,
ZMR,
ZMS,
ZMT,
ZMU,
ZMV,
ZMW,
ZMX,
ZMY,
ZMZ,
ZNA,
ZNE,
ZNG,
ZNK,
ZNS,
ZOC,
ZOH,
ZOM,
ZOO,
ZOQ,
ZOR,
ZOS,
ZPA,
ZPB,
ZPC,
ZPD,
ZPE,
ZPF,
ZPG,
ZPH,
ZPI,
ZPJ,
ZPK,
ZPL,
ZPM,
ZPN,
ZPO,
ZPP,
ZPQ,
ZPR,
ZPS,
ZPT,
ZPU,
ZPV,
ZPW,
ZPX,
ZPY,
ZPZ,
ZQE,
ZRA,
ZRG,
ZRN,
ZRO,
ZRP,
ZRS,
ZSA,
ZSK,
ZSL,
ZSM,
ZSR,
ZSU,
ZTE,
ZTG,
ZTL,
ZTM,
ZTN,
ZTP,
ZTQ,
ZTS,
ZTT,
ZTU,
ZTX,
ZTY,
ZUA,
ZUH,
ZUM,
ZUY,
ZWA,
ZYB,
ZYG,
ZYJ,
ZYN,
ZYP,
ZZJ,

];


pub const ALL_MAP: Map<&str, LanguageCode> = phf_map! {
"ab" => AB,
"aa" => AA,
"af" => AF,
"ak" => AK,
"sq" => SQ,
"am" => AM,
"ar" => AR,
"an" => AN,
"hy" => HY,
"as" => AS,
"av" => AV,
"ae" => AE,
"ay" => AY,
"az" => AZ,
"bm" => BM,
"ba" => BA,
"eu" => EU,
"be" => BE,
"bn" => BN,
"bi" => BI,
"bs" => BS,
"br" => BR,
"bg" => BG,
"my" => MY,
"ca" => CA,
"ch" => CH,
"ce" => CE,
"ny" => NY,
"zh" => ZH,
"cu" => CU,
"cv" => CV,
"kw" => KW,
"co" => CO,
"cr" => CR,
"hr" => HR,
"cs" => CS,
"da" => DA,
"dv" => DV,
"nl" => NL,
"dz" => DZ,
"en" => EN,
"eo" => EO,
"et" => ET,
"ee" => EE,
"fo" => FO,
"fj" => FJ,
"fi" => FI,
"fr" => FR,
"fy" => FY,
"ff" => FF,
"gd" => GD,
"gl" => GL,
"lg" => LG,
"ka" => KA,
"de" => DE,
"el" => EL,
"kl" => KL,
"gn" => GN,
"gu" => GU,
"ht" => HT,
"ha" => HA,
"he" => HE,
"hz" => HZ,
"hi" => HI,
"ho" => HO,
"hu" => HU,
"is" => IS,
"io" => IO,
"ig" => IG,
"id" => ID,
"ia" => IA,
"ie" => IE,
"iu" => IU,
"ik" => IK,
"ga" => GA,
"it" => IT,
"ja" => JA,
"jv" => JV,
"kn" => KN,
"kr" => KR,
"ks" => KS,
"kk" => KK,
"km" => KM,
"ki" => KI,
"rw" => RW,
"ky" => KY,
"kv" => KV,
"kg" => KG,
"ko" => KO,
"kj" => KJ,
"ku" => KU,
"lo" => LO,
"la" => LA,
"lv" => LV,
"li" => LI,
"ln" => LN,
"lt" => LT,
"lu" => LU,
"lb" => LB,
"mk" => MK,
"mg" => MG,
"ms" => MS,
"ml" => ML,
"mt" => MT,
"gv" => GV,
"mi" => MI,
"mr" => MR,
"mh" => MH,
"mn" => MN,
"na" => NA,
"nv" => NV,
"nd" => ND,
"nr" => NR,
"ng" => NG,
"ne" => NE,
"no" => NO,
"nb" => NB,
"nn" => NN,
"ii" => II,
"oc" => OC,
"oj" => OJ,
"or" => OR,
"om" => OM,
"os" => OS,
"pi" => PI,
"ps" => PS,
"fa" => FA,
"pl" => PL,
"pt" => PT,
"pa" => PA,
"qu" => QU,
"ro" => RO,
"rm" => RM,
"rn" => RN,
"ru" => RU,
"se" => SE,
"sm" => SM,
"sg" => SG,
"sa" => SA,
"sc" => SC,
"sr" => SR,
"sn" => SN,
"sd" => SD,
"si" => SI,
"sk" => SK,
"sl" => SL,
"so" => SO,
"st" => ST,
"es" => ES,
"su" => SU,
"sw" => SW,
"ss" => SS,
"sv" => SV,
"tl" => TL,
"ty" => TY,
"tg" => TG,
"ta" => TA,
"tt" => TT,
"te" => TE,
"th" => TH,
"bo" => BO,
"ti" => TI,
"to" => TO,
"ts" => TS,
"tn" => TN,
"tr" => TR,
"tk" => TK,
"tw" => TW,
"ug" => UG,
"uk" => UK,
"ur" => UR,
"uz" => UZ,
"ve" => VE,
"vi" => VI,
"vo" => VO,
"wa" => WA,
"cy" => CY,
"wo" => WO,
"xh" => XH,
"yi" => YI,
"yo" => YO,
"za" => ZA,
"zu" => ZU,
"ace" => ACE,
"ach" => ACH,
"ada" => ADA,
"ady" => ADY,
"afa" => AFA,
"afh" => AFH,
"ain" => AIN,
"akk" => AKK,
"ale" => ALE,
"alg" => ALG,
"alt" => ALT,
"ang" => ANG,
"anp" => ANP,
"apa" => APA,
"arc" => ARC,
"arn" => ARN,
"arp" => ARP,
"art" => ART,
"arw" => ARW,
"ast" => AST,
"ath" => ATH,
"aus" => AUS,
"awa" => AWA,
"bad" => BAD,
"bai" => BAI,
"bal" => BAL,
"ban" => BAN,
"bas" => BAS,
"bat" => BAT,
"bej" => BEJ,
"bem" => BEM,
"ber" => BER,
"bho" => BHO,
"bih" => BIH,
"bik" => BIK,
"bin" => BIN,
"bla" => BLA,
"bnt" => BNT,
"bra" => BRA,
"btk" => BTK,
"bua" => BUA,
"bug" => BUG,
"byn" => BYN,
"cad" => CAD,
"cai" => CAI,
"car" => CAR,
"cau" => CAU,
"ceb" => CEB,
"cel" => CEL,
"chb" => CHB,
"chg" => CHG,
"chk" => CHK,
"chm" => CHM,
"chn" => CHN,
"cho" => CHO,
"chp" => CHP,
"chr" => CHR,
"chy" => CHY,
"cmc" => CMC,
"cnr" => CNR,
"cop" => COP,
"cpe" => CPE,
"cpf" => CPF,
"cpp" => CPP,
"crh" => CRH,
"crp" => CRP,
"csb" => CSB,
"cus" => CUS,
"dak" => DAK,
"dar" => DAR,
"day" => DAY,
"del" => DEL,
"den" => DEN,
"dgr" => DGR,
"din" => DIN,
"doi" => DOI,
"dra" => DRA,
"dsb" => DSB,
"dua" => DUA,
"dum" => DUM,
"dyu" => DYU,
"efi" => EFI,
"egy" => EGY,
"eka" => EKA,
"elx" => ELX,
"enm" => ENM,
"ewo" => EWO,
"fan" => FAN,
"fat" => FAT,
"fil" => FIL,
"fiu" => FIU,
"fon" => FON,
"frm" => FRM,
"fro" => FRO,
"frr" => FRR,
"frs" => FRS,
"fur" => FUR,
"gaa" => GAA,
"gay" => GAY,
"gba" => GBA,
"gem" => GEM,
"gez" => GEZ,
"gil" => GIL,
"gmh" => GMH,
"goh" => GOH,
"gon" => GON,
"gor" => GOR,
"got" => GOT,
"grb" => GRB,
"grc" => GRC,
"gsw" => GSW,
"gwi" => GWI,
"hai" => HAI,
"haw" => HAW,
"hil" => HIL,
"him" => HIM,
"hit" => HIT,
"hmn" => HMN,
"hsb" => HSB,
"hup" => HUP,
"iba" => IBA,
"ijo" => IJO,
"ilo" => ILO,
"inc" => INC,
"ine" => INE,
"inh" => INH,
"ira" => IRA,
"iro" => IRO,
"jbo" => JBO,
"jpr" => JPR,
"jrb" => JRB,
"kaa" => KAA,
"kab" => KAB,
"kac" => KAC,
"kam" => KAM,
"kar" => KAR,
"kaw" => KAW,
"kbd" => KBD,
"kha" => KHA,
"khi" => KHI,
"kho" => KHO,
"kmb" => KMB,
"kok" => KOK,
"kos" => KOS,
"kpe" => KPE,
"krc" => KRC,
"krl" => KRL,
"kro" => KRO,
"kru" => KRU,
"kum" => KUM,
"kut" => KUT,
"lad" => LAD,
"lah" => LAH,
"lam" => LAM,
"lez" => LEZ,
"lol" => LOL,
"loz" => LOZ,
"lua" => LUA,
"lui" => LUI,
"lun" => LUN,
"luo" => LUO,
"lus" => LUS,
"mad" => MAD,
"mag" => MAG,
"mai" => MAI,
"mak" => MAK,
"man" => MAN,
"map" => MAP,
"mas" => MAS,
"mdf" => MDF,
"mdr" => MDR,
"men" => MEN,
"mga" => MGA,
"mic" => MIC,
"min" => MIN,
"mis" => MIS,
"mkh" => MKH,
"mnc" => MNC,
"mni" => MNI,
"mno" => MNO,
"moh" => MOH,
"mos" => MOS,
"mul" => MUL,
"mun" => MUN,
"mus" => MUS,
"mwl" => MWL,
"mwr" => MWR,
"myn" => MYN,
"myv" => MYV,
"nah" => NAH,
"nai" => NAI,
"nap" => NAP,
"nds" => NDS,
"new" => NEW,
"nia" => NIA,
"nic" => NIC,
"niu" => NIU,
"nog" => NOG,
"non" => NON,
"nqo" => NQO,
"nso" => NSO,
"nub" => NUB,
"nwc" => NWC,
"nym" => NYM,
"nyn" => NYN,
"nyo" => NYO,
"nzi" => NZI,
"osa" => OSA,
"ota" => OTA,
"oto" => OTO,
"paa" => PAA,
"pag" => PAG,
"pal" => PAL,
"pam" => PAM,
"pap" => PAP,
"pau" => PAU,
"peo" => PEO,
"phi" => PHI,
"phn" => PHN,
"pon" => PON,
"pra" => PRA,
"pro" => PRO,
"qaa" => QAA,
"raj" => RAJ,
"rap" => RAP,
"rar" => RAR,
"roa" => ROA,
"rom" => ROM,
"rup" => RUP,
"sad" => SAD,
"sah" => SAH,
"sai" => SAI,
"sal" => SAL,
"sam" => SAM,
"sas" => SAS,
"sat" => SAT,
"scn" => SCN,
"sco" => SCO,
"sel" => SEL,
"sem" => SEM,
"sga" => SGA,
"sgn" => SGN,
"shn" => SHN,
"sid" => SID,
"sio" => SIO,
"sit" => SIT,
"sla" => SLA,
"sma" => SMA,
"smi" => SMI,
"smj" => SMJ,
"smn" => SMN,
"sms" => SMS,
"snk" => SNK,
"sog" => SOG,
"son" => SON,
"srn" => SRN,
"srr" => SRR,
"ssa" => SSA,
"suk" => SUK,
"sus" => SUS,
"sux" => SUX,
"syc" => SYC,
"syr" => SYR,
"tai" => TAI,
"tem" => TEM,
"ter" => TER,
"tet" => TET,
"tig" => TIG,
"tiv" => TIV,
"tkl" => TKL,
"tlh" => TLH,
"tli" => TLI,
"tmh" => TMH,
"tog" => TOG,
"tpi" => TPI,
"tsi" => TSI,
"tum" => TUM,
"tup" => TUP,
"tut" => TUT,
"tvl" => TVL,
"tyv" => TYV,
"udm" => UDM,
"uga" => UGA,
"umb" => UMB,
"und" => UND,
"vai" => VAI,
"vot" => VOT,
"wak" => WAK,
"wal" => WAL,
"war" => WAR,
"was" => WAS,
"wen" => WEN,
"xal" => XAL,
"yao" => YAO,
"yap" => YAP,
"ypk" => YPK,
"zap" => ZAP,
"zbl" => ZBL,
"zen" => ZEN,
"zgh" => ZGH,
"znd" => ZND,
"zun" => ZUN,
"zxx" => ZXX,
"zza" => ZZA,
"aaa" => AAA,
"aab" => AAB,
"aac" => AAC,
"aad" => AAD,
"aae" => AAE,
"aaf" => AAF,
"aag" => AAG,
"aah" => AAH,
"aai" => AAI,
"aak" => AAK,
"aal" => AAL,
"aan" => AAN,
"aao" => AAO,
"aap" => AAP,
"aaq" => AAQ,
"aas" => AAS,
"aat" => AAT,
"aau" => AAU,
"aaw" => AAW,
"aax" => AAX,
"aaz" => AAZ,
"aba" => ABA,
"abb" => ABB,
"abc" => ABC,
"abd" => ABD,
"abe" => ABE,
"abf" => ABF,
"abg" => ABG,
"abh" => ABH,
"abi" => ABI,
"abj" => ABJ,
"abl" => ABL,
"abm" => ABM,
"abn" => ABN,
"abo" => ABO,
"abp" => ABP,
"abq" => ABQ,
"abr" => ABR,
"abs" => ABS,
"abt" => ABT,
"abu" => ABU,
"abv" => ABV,
"abw" => ABW,
"abx" => ABX,
"aby" => ABY,
"abz" => ABZ,
"aca" => ACA,
"acb" => ACB,
"acd" => ACD,
"acf" => ACF,
"aci" => ACI,
"ack" => ACK,
"acl" => ACL,
"acm" => ACM,
"acn" => ACN,
"acp" => ACP,
"acq" => ACQ,
"acr" => ACR,
"acs" => ACS,
"act" => ACT,
"acu" => ACU,
"acv" => ACV,
"acw" => ACW,
"acx" => ACX,
"acy" => ACY,
"acz" => ACZ,
"adb" => ADB,
"add" => ADD,
"ade" => ADE,
"adf" => ADF,
"adg" => ADG,
"adh" => ADH,
"adi" => ADI,
"adj" => ADJ,
"adl" => ADL,
"adn" => ADN,
"ado" => ADO,
"adq" => ADQ,
"adr" => ADR,
"ads" => ADS,
"adt" => ADT,
"adu" => ADU,
"adw" => ADW,
"adx" => ADX,
"adz" => ADZ,
"aea" => AEA,
"aeb" => AEB,
"aec" => AEC,
"aed" => AED,
"aee" => AEE,
"aek" => AEK,
"ael" => AEL,
"aem" => AEM,
"aen" => AEN,
"aeq" => AEQ,
"aer" => AER,
"aes" => AES,
"aeu" => AEU,
"aew" => AEW,
"aey" => AEY,
"aez" => AEZ,
"afb" => AFB,
"afd" => AFD,
"afe" => AFE,
"afg" => AFG,
"afi" => AFI,
"afk" => AFK,
"afn" => AFN,
"afo" => AFO,
"afp" => AFP,
"afs" => AFS,
"aft" => AFT,
"afu" => AFU,
"afz" => AFZ,
"aga" => AGA,
"agb" => AGB,
"agc" => AGC,
"agd" => AGD,
"age" => AGE,
"agf" => AGF,
"agg" => AGG,
"agh" => AGH,
"agi" => AGI,
"agj" => AGJ,
"agk" => AGK,
"agl" => AGL,
"agm" => AGM,
"agn" => AGN,
"ago" => AGO,
"agq" => AGQ,
"agr" => AGR,
"ags" => AGS,
"agt" => AGT,
"agu" => AGU,
"agv" => AGV,
"agw" => AGW,
"agx" => AGX,
"agy" => AGY,
"agz" => AGZ,
"aha" => AHA,
"ahb" => AHB,
"ahg" => AHG,
"ahh" => AHH,
"ahi" => AHI,
"ahk" => AHK,
"ahl" => AHL,
"ahm" => AHM,
"ahn" => AHN,
"aho" => AHO,
"ahp" => AHP,
"ahr" => AHR,
"ahs" => AHS,
"aht" => AHT,
"aia" => AIA,
"aib" => AIB,
"aic" => AIC,
"aid" => AID,
"aie" => AIE,
"aif" => AIF,
"aig" => AIG,
"aih" => AIH,
"aii" => AII,
"aij" => AIJ,
"aik" => AIK,
"ail" => AIL,
"aim" => AIM,
"aio" => AIO,
"aip" => AIP,
"aiq" => AIQ,
"air" => AIR,
"ait" => AIT,
"aiw" => AIW,
"aix" => AIX,
"aiy" => AIY,
"aja" => AJA,
"ajg" => AJG,
"aji" => AJI,
"ajn" => AJN,
"ajp" => AJP,
"ajs" => AJS,
"aju" => AJU,
"ajw" => AJW,
"ajz" => AJZ,
"akb" => AKB,
"akc" => AKC,
"akd" => AKD,
"ake" => AKE,
"akf" => AKF,
"akg" => AKG,
"akh" => AKH,
"aki" => AKI,
"akj" => AKJ,
"akl" => AKL,
"akm" => AKM,
"ako" => AKO,
"akp" => AKP,
"akq" => AKQ,
"akr" => AKR,
"aks" => AKS,
"akt" => AKT,
"aku" => AKU,
"akv" => AKV,
"akw" => AKW,
"akx" => AKX,
"aky" => AKY,
"akz" => AKZ,
"ala" => ALA,
"alc" => ALC,
"ald" => ALD,
"alf" => ALF,
"alh" => ALH,
"ali" => ALI,
"alj" => ALJ,
"alk" => ALK,
"all" => ALL,
"alm" => ALM,
"aln" => ALN,
"alo" => ALO,
"alp" => ALP,
"alq" => ALQ,
"alr" => ALR,
"als" => ALS,
"alu" => ALU,
"alw" => ALW,
"alx" => ALX,
"aly" => ALY,
"alz" => ALZ,
"ama" => AMA,
"amb" => AMB,
"amc" => AMC,
"ame" => AME,
"amf" => AMF,
"amg" => AMG,
"ami" => AMI,
"amj" => AMJ,
"amk" => AMK,
"aml" => AML,
"amm" => AMM,
"amn" => AMN,
"amo" => AMO,
"amp" => AMP,
"amq" => AMQ,
"amr" => AMR,
"ams" => AMS,
"amt" => AMT,
"amu" => AMU,
"amv" => AMV,
"amw" => AMW,
"amx" => AMX,
"amy" => AMY,
"amz" => AMZ,
"ana" => ANA,
"anb" => ANB,
"anc" => ANC,
"and" => AND,
"ane" => ANE,
"anf" => ANF,
"anh" => ANH,
"ani" => ANI,
"anj" => ANJ,
"ank" => ANK,
"anl" => ANL,
"anm" => ANM,
"ann" => ANN,
"ano" => ANO,
"anq" => ANQ,
"anr" => ANR,
"ans" => ANS,
"ant" => ANT,
"anu" => ANU,
"anv" => ANV,
"anw" => ANW,
"anx" => ANX,
"any" => ANY,
"anz" => ANZ,
"aoa" => AOA,
"aob" => AOB,
"aoc" => AOC,
"aod" => AOD,
"aoe" => AOE,
"aof" => AOF,
"aog" => AOG,
"aoi" => AOI,
"aoj" => AOJ,
"aok" => AOK,
"aol" => AOL,
"aom" => AOM,
"aon" => AON,
"aor" => AOR,
"aos" => AOS,
"aot" => AOT,
"aou" => AOU,
"aox" => AOX,
"aoz" => AOZ,
"apb" => APB,
"apc" => APC,
"apd" => APD,
"ape" => APE,
"apf" => APF,
"apg" => APG,
"aph" => APH,
"api" => API,
"apj" => APJ,
"apk" => APK,
"apl" => APL,
"apm" => APM,
"apn" => APN,
"apo" => APO,
"app" => APP,
"apq" => APQ,
"apr" => APR,
"aps" => APS,
"apt" => APT,
"apu" => APU,
"apv" => APV,
"apw" => APW,
"apx" => APX,
"apy" => APY,
"apz" => APZ,
"aqc" => AQC,
"aqd" => AQD,
"aqg" => AQG,
"aqk" => AQK,
"aqm" => AQM,
"aqn" => AQN,
"aqp" => AQP,
"aqr" => AQR,
"aqt" => AQT,
"aqz" => AQZ,
"arb" => ARB,
"ard" => ARD,
"are" => ARE,
"arh" => ARH,
"ari" => ARI,
"arj" => ARJ,
"ark" => ARK,
"arl" => ARL,
"aro" => ARO,
"arq" => ARQ,
"arr" => ARR,
"ars" => ARS,
"aru" => ARU,
"arv" => ARV,
"arx" => ARX,
"ary" => ARY,
"arz" => ARZ,
"asa" => ASA,
"asb" => ASB,
"asc" => ASC,
"ase" => ASE,
"asf" => ASF,
"asg" => ASG,
"ash" => ASH,
"asi" => ASI,
"asj" => ASJ,
"ask" => ASK,
"asl" => ASL,
"asn" => ASN,
"aso" => ASO,
"asp" => ASP,
"asq" => ASQ,
"asr" => ASR,
"ass" => ASS,
"asu" => ASU,
"asv" => ASV,
"asw" => ASW,
"asx" => ASX,
"asy" => ASY,
"asz" => ASZ,
"ata" => ATA,
"atb" => ATB,
"atc" => ATC,
"atd" => ATD,
"ate" => ATE,
"atg" => ATG,
"ati" => ATI,
"atj" => ATJ,
"atk" => ATK,
"atl" => ATL,
"atm" => ATM,
"atn" => ATN,
"ato" => ATO,
"atp" => ATP,
"atq" => ATQ,
"atr" => ATR,
"ats" => ATS,
"att" => ATT,
"atu" => ATU,
"atv" => ATV,
"atw" => ATW,
"atx" => ATX,
"aty" => ATY,
"atz" => ATZ,
"aua" => AUA,
"aub" => AUB,
"auc" => AUC,
"aud" => AUD,
"aug" => AUG,
"auh" => AUH,
"aui" => AUI,
"auj" => AUJ,
"auk" => AUK,
"aul" => AUL,
"aum" => AUM,
"aun" => AUN,
"auo" => AUO,
"aup" => AUP,
"auq" => AUQ,
"aur" => AUR,
"aut" => AUT,
"auu" => AUU,
"auw" => AUW,
"aux" => AUX,
"auy" => AUY,
"auz" => AUZ,
"avb" => AVB,
"avd" => AVD,
"avi" => AVI,
"avk" => AVK,
"avl" => AVL,
"avm" => AVM,
"avn" => AVN,
"avo" => AVO,
"avs" => AVS,
"avt" => AVT,
"avu" => AVU,
"avv" => AVV,
"awb" => AWB,
"awc" => AWC,
"awe" => AWE,
"awg" => AWG,
"awh" => AWH,
"awi" => AWI,
"awk" => AWK,
"awm" => AWM,
"awn" => AWN,
"awo" => AWO,
"awr" => AWR,
"aws" => AWS,
"awt" => AWT,
"awu" => AWU,
"awv" => AWV,
"aww" => AWW,
"awx" => AWX,
"awy" => AWY,
"axb" => AXB,
"axe" => AXE,
"axg" => AXG,
"axk" => AXK,
"axl" => AXL,
"axm" => AXM,
"axx" => AXX,
"aya" => AYA,
"ayb" => AYB,
"ayc" => AYC,
"ayd" => AYD,
"aye" => AYE,
"ayg" => AYG,
"ayh" => AYH,
"ayi" => AYI,
"ayk" => AYK,
"ayl" => AYL,
"ayn" => AYN,
"ayo" => AYO,
"ayp" => AYP,
"ayq" => AYQ,
"ayr" => AYR,
"ays" => AYS,
"ayt" => AYT,
"ayu" => AYU,
"ayz" => AYZ,
"aza" => AZA,
"azb" => AZB,
"azd" => AZD,
"azg" => AZG,
"azj" => AZJ,
"azm" => AZM,
"azn" => AZN,
"azo" => AZO,
"azt" => AZT,
"azz" => AZZ,
"baa" => BAA,
"bab" => BAB,
"bac" => BAC,
"bae" => BAE,
"baf" => BAF,
"bag" => BAG,
"bah" => BAH,
"baj" => BAJ,
"bao" => BAO,
"bap" => BAP,
"bar" => BAR,
"bau" => BAU,
"bav" => BAV,
"baw" => BAW,
"bax" => BAX,
"bay" => BAY,
"bba" => BBA,
"bbb" => BBB,
"bbc" => BBC,
"bbd" => BBD,
"bbe" => BBE,
"bbf" => BBF,
"bbg" => BBG,
"bbh" => BBH,
"bbi" => BBI,
"bbj" => BBJ,
"bbk" => BBK,
"bbl" => BBL,
"bbm" => BBM,
"bbn" => BBN,
"bbo" => BBO,
"bbp" => BBP,
"bbq" => BBQ,
"bbr" => BBR,
"bbs" => BBS,
"bbt" => BBT,
"bbu" => BBU,
"bbv" => BBV,
"bbw" => BBW,
"bbx" => BBX,
"bby" => BBY,
"bca" => BCA,
"bcb" => BCB,
"bcc" => BCC,
"bcd" => BCD,
"bce" => BCE,
"bcf" => BCF,
"bcg" => BCG,
"bch" => BCH,
"bci" => BCI,
"bcj" => BCJ,
"bck" => BCK,
"bcl" => BCL,
"bcm" => BCM,
"bcn" => BCN,
"bco" => BCO,
"bcp" => BCP,
"bcq" => BCQ,
"bcr" => BCR,
"bcs" => BCS,
"bct" => BCT,
"bcu" => BCU,
"bcv" => BCV,
"bcw" => BCW,
"bcy" => BCY,
"bcz" => BCZ,
"bda" => BDA,
"bdb" => BDB,
"bdc" => BDC,
"bdd" => BDD,
"bde" => BDE,
"bdf" => BDF,
"bdg" => BDG,
"bdh" => BDH,
"bdi" => BDI,
"bdj" => BDJ,
"bdk" => BDK,
"bdl" => BDL,
"bdm" => BDM,
"bdn" => BDN,
"bdo" => BDO,
"bdp" => BDP,
"bdq" => BDQ,
"bdr" => BDR,
"bds" => BDS,
"bdt" => BDT,
"bdu" => BDU,
"bdv" => BDV,
"bdw" => BDW,
"bdx" => BDX,
"bdy" => BDY,
"bdz" => BDZ,
"bea" => BEA,
"beb" => BEB,
"bec" => BEC,
"bed" => BED,
"bee" => BEE,
"bef" => BEF,
"beg" => BEG,
"beh" => BEH,
"bei" => BEI,
"bek" => BEK,
"beo" => BEO,
"bep" => BEP,
"beq" => BEQ,
"bes" => BES,
"bet" => BET,
"beu" => BEU,
"bev" => BEV,
"bew" => BEW,
"bex" => BEX,
"bey" => BEY,
"bez" => BEZ,
"bfa" => BFA,
"bfb" => BFB,
"bfc" => BFC,
"bfd" => BFD,
"bfe" => BFE,
"bff" => BFF,
"bfg" => BFG,
"bfh" => BFH,
"bfi" => BFI,
"bfj" => BFJ,
"bfk" => BFK,
"bfl" => BFL,
"bfm" => BFM,
"bfn" => BFN,
"bfo" => BFO,
"bfp" => BFP,
"bfq" => BFQ,
"bfr" => BFR,
"bfs" => BFS,
"bft" => BFT,
"bfu" => BFU,
"bfw" => BFW,
"bfx" => BFX,
"bfy" => BFY,
"bfz" => BFZ,
"bga" => BGA,
"bgb" => BGB,
"bgc" => BGC,
"bgd" => BGD,
"bge" => BGE,
"bgf" => BGF,
"bgg" => BGG,
"bgi" => BGI,
"bgj" => BGJ,
"bgk" => BGK,
"bgl" => BGL,
"bgn" => BGN,
"bgo" => BGO,
"bgp" => BGP,
"bgq" => BGQ,
"bgr" => BGR,
"bgs" => BGS,
"bgt" => BGT,
"bgu" => BGU,
"bgv" => BGV,
"bgw" => BGW,
"bgx" => BGX,
"bgy" => BGY,
"bgz" => BGZ,
"bha" => BHA,
"bhb" => BHB,
"bhc" => BHC,
"bhd" => BHD,
"bhe" => BHE,
"bhf" => BHF,
"bhg" => BHG,
"bhh" => BHH,
"bhi" => BHI,
"bhj" => BHJ,
"bhl" => BHL,
"bhm" => BHM,
"bhn" => BHN,
"bhp" => BHP,
"bhq" => BHQ,
"bhr" => BHR,
"bhs" => BHS,
"bht" => BHT,
"bhu" => BHU,
"bhv" => BHV,
"bhw" => BHW,
"bhx" => BHX,
"bhy" => BHY,
"bhz" => BHZ,
"bia" => BIA,
"bib" => BIB,
"bid" => BID,
"bie" => BIE,
"bif" => BIF,
"big" => BIG,
"bil" => BIL,
"bim" => BIM,
"bio" => BIO,
"bip" => BIP,
"biq" => BIQ,
"bir" => BIR,
"bit" => BIT,
"biu" => BIU,
"biv" => BIV,
"biw" => BIW,
"bix" => BIX,
"biy" => BIY,
"biz" => BIZ,
"bja" => BJA,
"bjb" => BJB,
"bjc" => BJC,
"bje" => BJE,
"bjf" => BJF,
"bjg" => BJG,
"bjh" => BJH,
"bji" => BJI,
"bjj" => BJJ,
"bjk" => BJK,
"bjl" => BJL,
"bjm" => BJM,
"bjn" => BJN,
"bjo" => BJO,
"bjp" => BJP,
"bjr" => BJR,
"bjs" => BJS,
"bjt" => BJT,
"bju" => BJU,
"bjv" => BJV,
"bjw" => BJW,
"bjx" => BJX,
"bjy" => BJY,
"bjz" => BJZ,
"bka" => BKA,
"bkc" => BKC,
"bkd" => BKD,
"bkf" => BKF,
"bkg" => BKG,
"bkh" => BKH,
"bki" => BKI,
"bkj" => BKJ,
"bkk" => BKK,
"bkl" => BKL,
"bkm" => BKM,
"bkn" => BKN,
"bko" => BKO,
"bkp" => BKP,
"bkq" => BKQ,
"bkr" => BKR,
"bks" => BKS,
"bkt" => BKT,
"bku" => BKU,
"bkv" => BKV,
"bkw" => BKW,
"bkx" => BKX,
"bky" => BKY,
"bkz" => BKZ,
"blb" => BLB,
"blc" => BLC,
"bld" => BLD,
"ble" => BLE,
"blf" => BLF,
"blh" => BLH,
"bli" => BLI,
"blj" => BLJ,
"blk" => BLK,
"bll" => BLL,
"blm" => BLM,
"bln" => BLN,
"blo" => BLO,
"blp" => BLP,
"blq" => BLQ,
"blr" => BLR,
"bls" => BLS,
"blt" => BLT,
"blv" => BLV,
"blw" => BLW,
"blx" => BLX,
"bly" => BLY,
"blz" => BLZ,
"bma" => BMA,
"bmb" => BMB,
"bmc" => BMC,
"bmd" => BMD,
"bme" => BME,
"bmf" => BMF,
"bmg" => BMG,
"bmh" => BMH,
"bmi" => BMI,
"bmj" => BMJ,
"bmk" => BMK,
"bml" => BML,
"bmm" => BMM,
"bmn" => BMN,
"bmo" => BMO,
"bmp" => BMP,
"bmq" => BMQ,
"bmr" => BMR,
"bms" => BMS,
"bmt" => BMT,
"bmu" => BMU,
"bmv" => BMV,
"bmw" => BMW,
"bmx" => BMX,
"bmz" => BMZ,
"bna" => BNA,
"bnb" => BNB,
"bnc" => BNC,
"bnd" => BND,
"bne" => BNE,
"bnf" => BNF,
"bng" => BNG,
"bni" => BNI,
"bnj" => BNJ,
"bnk" => BNK,
"bnl" => BNL,
"bnm" => BNM,
"bnn" => BNN,
"bno" => BNO,
"bnp" => BNP,
"bnq" => BNQ,
"bnr" => BNR,
"bns" => BNS,
"bnu" => BNU,
"bnv" => BNV,
"bnw" => BNW,
"bnx" => BNX,
"bny" => BNY,
"bnz" => BNZ,
"boa" => BOA,
"bob" => BOB,
"boe" => BOE,
"bof" => BOF,
"bog" => BOG,
"boh" => BOH,
"boi" => BOI,
"boj" => BOJ,
"bok" => BOK,
"bol" => BOL,
"bom" => BOM,
"bon" => BON,
"boo" => BOO,
"bop" => BOP,
"boq" => BOQ,
"bor" => BOR,
"bot" => BOT,
"bou" => BOU,
"bov" => BOV,
"bow" => BOW,
"box" => BOX,
"boy" => BOY,
"boz" => BOZ,
"bpa" => BPA,
"bpc" => BPC,
"bpd" => BPD,
"bpe" => BPE,
"bpg" => BPG,
"bph" => BPH,
"bpi" => BPI,
"bpj" => BPJ,
"bpk" => BPK,
"bpl" => BPL,
"bpm" => BPM,
"bpn" => BPN,
"bpo" => BPO,
"bpp" => BPP,
"bpq" => BPQ,
"bpr" => BPR,
"bps" => BPS,
"bpt" => BPT,
"bpu" => BPU,
"bpv" => BPV,
"bpw" => BPW,
"bpx" => BPX,
"bpy" => BPY,
"bpz" => BPZ,
"bqa" => BQA,
"bqb" => BQB,
"bqc" => BQC,
"bqd" => BQD,
"bqf" => BQF,
"bqg" => BQG,
"bqh" => BQH,
"bqi" => BQI,
"bqj" => BQJ,
"bqk" => BQK,
"bql" => BQL,
"bqm" => BQM,
"bqn" => BQN,
"bqo" => BQO,
"bqp" => BQP,
"bqq" => BQQ,
"bqr" => BQR,
"bqs" => BQS,
"bqt" => BQT,
"bqu" => BQU,
"bqv" => BQV,
"bqw" => BQW,
"bqx" => BQX,
"bqy" => BQY,
"bqz" => BQZ,
"brb" => BRB,
"brc" => BRC,
"brd" => BRD,
"brf" => BRF,
"brg" => BRG,
"brh" => BRH,
"bri" => BRI,
"brj" => BRJ,
"brk" => BRK,
"brl" => BRL,
"brm" => BRM,
"brn" => BRN,
"bro" => BRO,
"brp" => BRP,
"brq" => BRQ,
"brr" => BRR,
"brs" => BRS,
"brt" => BRT,
"bru" => BRU,
"brv" => BRV,
"brw" => BRW,
"brx" => BRX,
"bry" => BRY,
"brz" => BRZ,
"bsa" => BSA,
"bsb" => BSB,
"bsc" => BSC,
"bse" => BSE,
"bsf" => BSF,
"bsg" => BSG,
"bsh" => BSH,
"bsi" => BSI,
"bsj" => BSJ,
"bsk" => BSK,
"bsl" => BSL,
"bsm" => BSM,
"bsn" => BSN,
"bso" => BSO,
"bsp" => BSP,
"bsq" => BSQ,
"bsr" => BSR,
"bss" => BSS,
"bst" => BST,
"bsu" => BSU,
"bsv" => BSV,
"bsw" => BSW,
"bsx" => BSX,
"bsy" => BSY,
"bta" => BTA,
"btc" => BTC,
"btd" => BTD,
"bte" => BTE,
"btf" => BTF,
"btg" => BTG,
"bth" => BTH,
"bti" => BTI,
"btj" => BTJ,
"btm" => BTM,
"btn" => BTN,
"bto" => BTO,
"btp" => BTP,
"btq" => BTQ,
"btr" => BTR,
"bts" => BTS,
"btt" => BTT,
"btu" => BTU,
"btv" => BTV,
"btw" => BTW,
"btx" => BTX,
"bty" => BTY,
"btz" => BTZ,
"bub" => BUB,
"buc" => BUC,
"bud" => BUD,
"bue" => BUE,
"buf" => BUF,
"buh" => BUH,
"bui" => BUI,
"buj" => BUJ,
"buk" => BUK,
"bum" => BUM,
"bun" => BUN,
"buo" => BUO,
"bup" => BUP,
"buq" => BUQ,
"bus" => BUS,
"but" => BUT,
"buu" => BUU,
"buv" => BUV,
"buw" => BUW,
"bux" => BUX,
"buy" => BUY,
"buz" => BUZ,
"bva" => BVA,
"bvb" => BVB,
"bvc" => BVC,
"bvd" => BVD,
"bve" => BVE,
"bvf" => BVF,
"bvg" => BVG,
"bvh" => BVH,
"bvi" => BVI,
"bvj" => BVJ,
"bvk" => BVK,
"bvl" => BVL,
"bvm" => BVM,
"bvn" => BVN,
"bvo" => BVO,
"bvp" => BVP,
"bvq" => BVQ,
"bvr" => BVR,
"bvt" => BVT,
"bvu" => BVU,
"bvv" => BVV,
"bvw" => BVW,
"bvx" => BVX,
"bvy" => BVY,
"bvz" => BVZ,
"bwa" => BWA,
"bwb" => BWB,
"bwc" => BWC,
"bwd" => BWD,
"bwe" => BWE,
"bwf" => BWF,
"bwg" => BWG,
"bwh" => BWH,
"bwi" => BWI,
"bwj" => BWJ,
"bwk" => BWK,
"bwl" => BWL,
"bwm" => BWM,
"bwn" => BWN,
"bwo" => BWO,
"bwp" => BWP,
"bwq" => BWQ,
"bwr" => BWR,
"bws" => BWS,
"bwt" => BWT,
"bwu" => BWU,
"bww" => BWW,
"bwx" => BWX,
"bwy" => BWY,
"bwz" => BWZ,
"bxa" => BXA,
"bxb" => BXB,
"bxc" => BXC,
"bxd" => BXD,
"bxe" => BXE,
"bxf" => BXF,
"bxg" => BXG,
"bxh" => BXH,
"bxi" => BXI,
"bxj" => BXJ,
"bxk" => BXK,
"bxl" => BXL,
"bxm" => BXM,
"bxn" => BXN,
"bxo" => BXO,
"bxp" => BXP,
"bxq" => BXQ,
"bxr" => BXR,
"bxs" => BXS,
"bxu" => BXU,
"bxv" => BXV,
"bxw" => BXW,
"bxz" => BXZ,
"bya" => BYA,
"byb" => BYB,
"byc" => BYC,
"byd" => BYD,
"bye" => BYE,
"byf" => BYF,
"byg" => BYG,
"byh" => BYH,
"byi" => BYI,
"byj" => BYJ,
"byk" => BYK,
"byl" => BYL,
"bym" => BYM,
"byo" => BYO,
"byp" => BYP,
"byq" => BYQ,
"byr" => BYR,
"bys" => BYS,
"byt" => BYT,
"byv" => BYV,
"byw" => BYW,
"byx" => BYX,
"byz" => BYZ,
"bza" => BZA,
"bzb" => BZB,
"bzc" => BZC,
"bzd" => BZD,
"bze" => BZE,
"bzf" => BZF,
"bzg" => BZG,
"bzh" => BZH,
"bzi" => BZI,
"bzj" => BZJ,
"bzk" => BZK,
"bzl" => BZL,
"bzm" => BZM,
"bzn" => BZN,
"bzo" => BZO,
"bzp" => BZP,
"bzq" => BZQ,
"bzr" => BZR,
"bzs" => BZS,
"bzt" => BZT,
"bzu" => BZU,
"bzv" => BZV,
"bzw" => BZW,
"bzx" => BZX,
"bzy" => BZY,
"bzz" => BZZ,
"caa" => CAA,
"cab" => CAB,
"cac" => CAC,
"cae" => CAE,
"caf" => CAF,
"cag" => CAG,
"cah" => CAH,
"caj" => CAJ,
"cak" => CAK,
"cal" => CAL,
"cam" => CAM,
"can" => CAN,
"cao" => CAO,
"cap" => CAP,
"caq" => CAQ,
"cas" => CAS,
"cav" => CAV,
"caw" => CAW,
"cax" => CAX,
"cay" => CAY,
"caz" => CAZ,
"cbb" => CBB,
"cbc" => CBC,
"cbd" => CBD,
"cbg" => CBG,
"cbi" => CBI,
"cbj" => CBJ,
"cbk" => CBK,
"cbl" => CBL,
"cbn" => CBN,
"cbo" => CBO,
"cbq" => CBQ,
"cbr" => CBR,
"cbs" => CBS,
"cbt" => CBT,
"cbu" => CBU,
"cbv" => CBV,
"cbw" => CBW,
"cby" => CBY,
"ccc" => CCC,
"ccd" => CCD,
"cce" => CCE,
"ccg" => CCG,
"cch" => CCH,
"ccj" => CCJ,
"ccl" => CCL,
"ccm" => CCM,
"cco" => CCO,
"ccp" => CCP,
"ccr" => CCR,
"cda" => CDA,
"cde" => CDE,
"cdf" => CDF,
"cdh" => CDH,
"cdi" => CDI,
"cdj" => CDJ,
"cdm" => CDM,
"cdn" => CDN,
"cdo" => CDO,
"cdr" => CDR,
"cds" => CDS,
"cdy" => CDY,
"cdz" => CDZ,
"cea" => CEA,
"ceg" => CEG,
"cek" => CEK,
"cen" => CEN,
"cet" => CET,
"cey" => CEY,
"cfa" => CFA,
"cfd" => CFD,
"cfg" => CFG,
"cfm" => CFM,
"cga" => CGA,
"cgc" => CGC,
"cgg" => CGG,
"cgk" => CGK,
"chc" => CHC,
"chd" => CHD,
"chf" => CHF,
"chh" => CHH,
"chj" => CHJ,
"chl" => CHL,
"chq" => CHQ,
"cht" => CHT,
"chw" => CHW,
"chx" => CHX,
"chz" => CHZ,
"cia" => CIA,
"cib" => CIB,
"cic" => CIC,
"cid" => CID,
"cie" => CIE,
"cih" => CIH,
"cik" => CIK,
"cim" => CIM,
"cin" => CIN,
"cip" => CIP,
"cir" => CIR,
"ciw" => CIW,
"ciy" => CIY,
"cja" => CJA,
"cje" => CJE,
"cjh" => CJH,
"cji" => CJI,
"cjk" => CJK,
"cjm" => CJM,
"cjn" => CJN,
"cjo" => CJO,
"cjp" => CJP,
"cjs" => CJS,
"cjv" => CJV,
"cjy" => CJY,
"ckb" => CKB,
"ckh" => CKH,
"ckl" => CKL,
"ckm" => CKM,
"ckn" => CKN,
"cko" => CKO,
"ckq" => CKQ,
"ckr" => CKR,
"cks" => CKS,
"ckt" => CKT,
"cku" => CKU,
"ckv" => CKV,
"ckx" => CKX,
"cky" => CKY,
"ckz" => CKZ,
"cla" => CLA,
"clc" => CLC,
"cld" => CLD,
"cle" => CLE,
"clh" => CLH,
"cli" => CLI,
"clj" => CLJ,
"clk" => CLK,
"cll" => CLL,
"clm" => CLM,
"clo" => CLO,
"clt" => CLT,
"clu" => CLU,
"clw" => CLW,
"cly" => CLY,
"cma" => CMA,
"cme" => CME,
"cmg" => CMG,
"cmi" => CMI,
"cml" => CML,
"cmm" => CMM,
"cmn" => CMN,
"cmo" => CMO,
"cmr" => CMR,
"cms" => CMS,
"cmt" => CMT,
"cna" => CNA,
"cnb" => CNB,
"cnc" => CNC,
"cng" => CNG,
"cnh" => CNH,
"cni" => CNI,
"cnk" => CNK,
"cnl" => CNL,
"cno" => CNO,
"cnp" => CNP,
"cnq" => CNQ,
"cns" => CNS,
"cnt" => CNT,
"cnu" => CNU,
"cnw" => CNW,
"cnx" => CNX,
"coa" => COA,
"cob" => COB,
"coc" => COC,
"cod" => COD,
"coe" => COE,
"cof" => COF,
"cog" => COG,
"coh" => COH,
"coj" => COJ,
"cok" => COK,
"col" => COL,
"com" => COM,
"con" => CON,
"coo" => COO,
"coq" => COQ,
"cot" => COT,
"cou" => COU,
"cov" => COV,
"cow" => COW,
"cox" => COX,
"coz" => COZ,
"cpa" => CPA,
"cpb" => CPB,
"cpc" => CPC,
"cpg" => CPG,
"cpi" => CPI,
"cpn" => CPN,
"cpo" => CPO,
"cps" => CPS,
"cpu" => CPU,
"cpx" => CPX,
"cpy" => CPY,
"cqd" => CQD,
"cra" => CRA,
"crb" => CRB,
"crc" => CRC,
"crd" => CRD,
"crf" => CRF,
"crg" => CRG,
"cri" => CRI,
"crj" => CRJ,
"crk" => CRK,
"crl" => CRL,
"crm" => CRM,
"crn" => CRN,
"cro" => CRO,
"crq" => CRQ,
"crr" => CRR,
"crs" => CRS,
"crt" => CRT,
"crv" => CRV,
"crw" => CRW,
"crx" => CRX,
"cry" => CRY,
"crz" => CRZ,
"csa" => CSA,
"csc" => CSC,
"csd" => CSD,
"cse" => CSE,
"csf" => CSF,
"csg" => CSG,
"csh" => CSH,
"csi" => CSI,
"csj" => CSJ,
"csk" => CSK,
"csl" => CSL,
"csm" => CSM,
"csn" => CSN,
"cso" => CSO,
"csp" => CSP,
"csq" => CSQ,
"csr" => CSR,
"css" => CSS,
"cst" => CST,
"csv" => CSV,
"csw" => CSW,
"csx" => CSX,
"csy" => CSY,
"csz" => CSZ,
"cta" => CTA,
"ctc" => CTC,
"ctd" => CTD,
"cte" => CTE,
"ctg" => CTG,
"cth" => CTH,
"ctl" => CTL,
"ctm" => CTM,
"ctn" => CTN,
"cto" => CTO,
"ctp" => CTP,
"cts" => CTS,
"ctt" => CTT,
"ctu" => CTU,
"cty" => CTY,
"ctz" => CTZ,
"cua" => CUA,
"cub" => CUB,
"cuc" => CUC,
"cuh" => CUH,
"cui" => CUI,
"cuj" => CUJ,
"cuk" => CUK,
"cul" => CUL,
"cuo" => CUO,
"cup" => CUP,
"cuq" => CUQ,
"cur" => CUR,
"cut" => CUT,
"cuu" => CUU,
"cuv" => CUV,
"cuw" => CUW,
"cux" => CUX,
"cuy" => CUY,
"cvg" => CVG,
"cvn" => CVN,
"cwa" => CWA,
"cwb" => CWB,
"cwd" => CWD,
"cwe" => CWE,
"cwg" => CWG,
"cwt" => CWT,
"cya" => CYA,
"cyb" => CYB,
"cyo" => CYO,
"czh" => CZH,
"czk" => CZK,
"czn" => CZN,
"czo" => CZO,
"czt" => CZT,
"daa" => DAA,
"dac" => DAC,
"dad" => DAD,
"dae" => DAE,
"dag" => DAG,
"dah" => DAH,
"dai" => DAI,
"daj" => DAJ,
"dal" => DAL,
"dam" => DAM,
"dao" => DAO,
"daq" => DAQ,
"das" => DAS,
"dau" => DAU,
"dav" => DAV,
"daw" => DAW,
"dax" => DAX,
"daz" => DAZ,
"dba" => DBA,
"dbb" => DBB,
"dbd" => DBD,
"dbe" => DBE,
"dbf" => DBF,
"dbg" => DBG,
"dbi" => DBI,
"dbj" => DBJ,
"dbl" => DBL,
"dbm" => DBM,
"dbn" => DBN,
"dbo" => DBO,
"dbp" => DBP,
"dbq" => DBQ,
"dbr" => DBR,
"dbt" => DBT,
"dbu" => DBU,
"dbv" => DBV,
"dbw" => DBW,
"dby" => DBY,
"dcc" => DCC,
"dcr" => DCR,
"dda" => DDA,
"ddd" => DDD,
"dde" => DDE,
"ddg" => DDG,
"ddi" => DDI,
"ddj" => DDJ,
"ddn" => DDN,
"ddo" => DDO,
"ddr" => DDR,
"dds" => DDS,
"ddw" => DDW,
"dec" => DEC,
"ded" => DED,
"dee" => DEE,
"def" => DEF,
"deg" => DEG,
"deh" => DEH,
"dei" => DEI,
"dek" => DEK,
"dem" => DEM,
"dep" => DEP,
"deq" => DEQ,
"der" => DER,
"des" => DES,
"dev" => DEV,
"dez" => DEZ,
"dga" => DGA,
"dgb" => DGB,
"dgc" => DGC,
"dgd" => DGD,
"dge" => DGE,
"dgg" => DGG,
"dgh" => DGH,
"dgi" => DGI,
"dgk" => DGK,
"dgl" => DGL,
"dgn" => DGN,
"dgo" => DGO,
"dgs" => DGS,
"dgt" => DGT,
"dgw" => DGW,
"dgx" => DGX,
"dgz" => DGZ,
"dhd" => DHD,
"dhg" => DHG,
"dhi" => DHI,
"dhl" => DHL,
"dhm" => DHM,
"dhn" => DHN,
"dho" => DHO,
"dhr" => DHR,
"dhs" => DHS,
"dhu" => DHU,
"dhv" => DHV,
"dhw" => DHW,
"dhx" => DHX,
"dia" => DIA,
"dib" => DIB,
"dic" => DIC,
"did" => DID,
"dif" => DIF,
"dig" => DIG,
"dih" => DIH,
"dii" => DII,
"dij" => DIJ,
"dik" => DIK,
"dil" => DIL,
"dim" => DIM,
"dio" => DIO,
"dip" => DIP,
"diq" => DIQ,
"dir" => DIR,
"dis" => DIS,
"diu" => DIU,
"diw" => DIW,
"dix" => DIX,
"diy" => DIY,
"diz" => DIZ,
"dja" => DJA,
"djb" => DJB,
"djc" => DJC,
"djd" => DJD,
"dje" => DJE,
"djf" => DJF,
"dji" => DJI,
"djj" => DJJ,
"djk" => DJK,
"djm" => DJM,
"djn" => DJN,
"djo" => DJO,
"djr" => DJR,
"dju" => DJU,
"djw" => DJW,
"dka" => DKA,
"dkg" => DKG,
"dkk" => DKK,
"dkr" => DKR,
"dks" => DKS,
"dkx" => DKX,
"dlg" => DLG,
"dlk" => DLK,
"dlm" => DLM,
"dln" => DLN,
"dma" => DMA,
"dmb" => DMB,
"dmc" => DMC,
"dmd" => DMD,
"dme" => DME,
"dmf" => DMF,
"dmg" => DMG,
"dmk" => DMK,
"dml" => DML,
"dmm" => DMM,
"dmo" => DMO,
"dmr" => DMR,
"dms" => DMS,
"dmu" => DMU,
"dmv" => DMV,
"dmw" => DMW,
"dmx" => DMX,
"dmy" => DMY,
"dna" => DNA,
"dnd" => DND,
"dne" => DNE,
"dng" => DNG,
"dni" => DNI,
"dnj" => DNJ,
"dnk" => DNK,
"dnn" => DNN,
"dno" => DNO,
"dnr" => DNR,
"dnt" => DNT,
"dnu" => DNU,
"dnv" => DNV,
"dnw" => DNW,
"dny" => DNY,
"doa" => DOA,
"dob" => DOB,
"doc" => DOC,
"doe" => DOE,
"dof" => DOF,
"doh" => DOH,
"dok" => DOK,
"dol" => DOL,
"don" => DON,
"doo" => DOO,
"dop" => DOP,
"doq" => DOQ,
"dor" => DOR,
"dos" => DOS,
"dot" => DOT,
"dov" => DOV,
"dow" => DOW,
"dox" => DOX,
"doy" => DOY,
"doz" => DOZ,
"dpp" => DPP,
"drb" => DRB,
"drc" => DRC,
"drd" => DRD,
"dre" => DRE,
"drg" => DRG,
"dri" => DRI,
"drl" => DRL,
"drn" => DRN,
"dro" => DRO,
"drq" => DRQ,
"drs" => DRS,
"drt" => DRT,
"dru" => DRU,
"dry" => DRY,
"dse" => DSE,
"dsh" => DSH,
"dsi" => DSI,
"dsl" => DSL,
"dsn" => DSN,
"dso" => DSO,
"dsq" => DSQ,
"dsz" => DSZ,
"dta" => DTA,
"dtb" => DTB,
"dtd" => DTD,
"dth" => DTH,
"dti" => DTI,
"dtk" => DTK,
"dtm" => DTM,
"dtn" => DTN,
"dto" => DTO,
"dtp" => DTP,
"dtr" => DTR,
"dts" => DTS,
"dtt" => DTT,
"dtu" => DTU,
"dty" => DTY,
"dub" => DUB,
"duc" => DUC,
"due" => DUE,
"duf" => DUF,
"dug" => DUG,
"duh" => DUH,
"dui" => DUI,
"duk" => DUK,
"dul" => DUL,
"dun" => DUN,
"duo" => DUO,
"dup" => DUP,
"duq" => DUQ,
"dur" => DUR,
"dus" => DUS,
"duu" => DUU,
"duv" => DUV,
"duw" => DUW,
"dux" => DUX,
"duy" => DUY,
"duz" => DUZ,
"dva" => DVA,
"dwa" => DWA,
"dwk" => DWK,
"dwr" => DWR,
"dws" => DWS,
"dwu" => DWU,
"dww" => DWW,
"dwy" => DWY,
"dwz" => DWZ,
"dya" => DYA,
"dyb" => DYB,
"dyd" => DYD,
"dyg" => DYG,
"dyi" => DYI,
"dym" => DYM,
"dyn" => DYN,
"dyo" => DYO,
"dyy" => DYY,
"dza" => DZA,
"dze" => DZE,
"dzg" => DZG,
"dzl" => DZL,
"dzn" => DZN,
"eaa" => EAA,
"ebc" => EBC,
"ebg" => EBG,
"ebk" => EBK,
"ebo" => EBO,
"ebr" => EBR,
"ebu" => EBU,
"ecr" => ECR,
"ecs" => ECS,
"ecy" => ECY,
"eee" => EEE,
"efa" => EFA,
"efe" => EFE,
"ega" => EGA,
"egl" => EGL,
"egm" => EGM,
"ego" => EGO,
"ehs" => EHS,
"ehu" => EHU,
"eip" => EIP,
"eit" => EIT,
"eiv" => EIV,
"eja" => EJA,
"eke" => EKE,
"ekg" => EKG,
"eki" => EKI,
"ekk" => EKK,
"ekl" => EKL,
"ekm" => EKM,
"eko" => EKO,
"ekp" => EKP,
"ekr" => EKR,
"eky" => EKY,
"ele" => ELE,
"elh" => ELH,
"eli" => ELI,
"elk" => ELK,
"elm" => ELM,
"elo" => ELO,
"elu" => ELU,
"ema" => EMA,
"emb" => EMB,
"eme" => EME,
"emg" => EMG,
"emi" => EMI,
"emk" => EMK,
"emm" => EMM,
"emn" => EMN,
"emp" => EMP,
"emq" => EMQ,
"ems" => EMS,
"emu" => EMU,
"emw" => EMW,
"emx" => EMX,
"emy" => EMY,
"emz" => EMZ,
"ena" => ENA,
"enb" => ENB,
"enc" => ENC,
"end" => END,
"enf" => ENF,
"enh" => ENH,
"enl" => ENL,
"enn" => ENN,
"eno" => ENO,
"enq" => ENQ,
"enr" => ENR,
"enu" => ENU,
"env" => ENV,
"enw" => ENW,
"enx" => ENX,
"eot" => EOT,
"epi" => EPI,
"era" => ERA,
"erg" => ERG,
"erh" => ERH,
"eri" => ERI,
"erk" => ERK,
"ero" => ERO,
"err" => ERR,
"ers" => ERS,
"ert" => ERT,
"erw" => ERW,
"ese" => ESE,
"esg" => ESG,
"esh" => ESH,
"esi" => ESI,
"esk" => ESK,
"esl" => ESL,
"esm" => ESM,
"esn" => ESN,
"eso" => ESO,
"esq" => ESQ,
"ess" => ESS,
"esu" => ESU,
"esy" => ESY,
"etb" => ETB,
"etc" => ETC,
"eth" => ETH,
"etn" => ETN,
"eto" => ETO,
"etr" => ETR,
"ets" => ETS,
"ett" => ETT,
"etu" => ETU,
"etx" => ETX,
"etz" => ETZ,
"eve" => EVE,
"evh" => EVH,
"evn" => EVN,
"ext" => EXT,
"eya" => EYA,
"eyo" => EYO,
"eza" => EZA,
"eze" => EZE,
"faa" => FAA,
"fab" => FAB,
"fad" => FAD,
"faf" => FAF,
"fag" => FAG,
"fah" => FAH,
"fai" => FAI,
"faj" => FAJ,
"fak" => FAK,
"fal" => FAL,
"fam" => FAM,
"fap" => FAP,
"far" => FAR,
"fau" => FAU,
"fax" => FAX,
"fay" => FAY,
"faz" => FAZ,
"fbl" => FBL,
"fcs" => FCS,
"fer" => FER,
"ffi" => FFI,
"ffm" => FFM,
"fgr" => FGR,
"fia" => FIA,
"fie" => FIE,
"fif" => FIF,
"fip" => FIP,
"fir" => FIR,
"fit" => FIT,
"fiw" => FIW,
"fkk" => FKK,
"fkv" => FKV,
"fla" => FLA,
"flh" => FLH,
"fli" => FLI,
"fll" => FLL,
"fln" => FLN,
"flr" => FLR,
"fly" => FLY,
"fmp" => FMP,
"fmu" => FMU,
"fnb" => FNB,
"fng" => FNG,
"fni" => FNI,
"fod" => FOD,
"foi" => FOI,
"fom" => FOM,
"for" => FOR,
"fos" => FOS,
"fpe" => FPE,
"fqs" => FQS,
"frc" => FRC,
"frd" => FRD,
"frk" => FRK,
"frp" => FRP,
"frq" => FRQ,
"frt" => FRT,
"fse" => FSE,
"fsl" => FSL,
"fss" => FSS,
"fub" => FUB,
"fuc" => FUC,
"fud" => FUD,
"fue" => FUE,
"fuf" => FUF,
"fuh" => FUH,
"fui" => FUI,
"fuj" => FUJ,
"fum" => FUM,
"fun" => FUN,
"fuq" => FUQ,
"fut" => FUT,
"fuu" => FUU,
"fuv" => FUV,
"fuy" => FUY,
"fvr" => FVR,
"fwa" => FWA,
"fwe" => FWE,
"gab" => GAB,
"gac" => GAC,
"gad" => GAD,
"gae" => GAE,
"gaf" => GAF,
"gag" => GAG,
"gah" => GAH,
"gai" => GAI,
"gaj" => GAJ,
"gak" => GAK,
"gal" => GAL,
"gam" => GAM,
"gan" => GAN,
"gao" => GAO,
"gap" => GAP,
"gaq" => GAQ,
"gar" => GAR,
"gas" => GAS,
"gat" => GAT,
"gau" => GAU,
"gaw" => GAW,
"gax" => GAX,
"gaz" => GAZ,
"gbb" => GBB,
"gbd" => GBD,
"gbe" => GBE,
"gbf" => GBF,
"gbg" => GBG,
"gbh" => GBH,
"gbi" => GBI,
"gbj" => GBJ,
"gbk" => GBK,
"gbl" => GBL,
"gbm" => GBM,
"gbn" => GBN,
"gbo" => GBO,
"gbp" => GBP,
"gbq" => GBQ,
"gbr" => GBR,
"gbs" => GBS,
"gbu" => GBU,
"gbv" => GBV,
"gbw" => GBW,
"gbx" => GBX,
"gby" => GBY,
"gbz" => GBZ,
"gcc" => GCC,
"gcd" => GCD,
"gce" => GCE,
"gcf" => GCF,
"gcl" => GCL,
"gcn" => GCN,
"gcr" => GCR,
"gct" => GCT,
"gda" => GDA,
"gdb" => GDB,
"gdc" => GDC,
"gdd" => GDD,
"gde" => GDE,
"gdf" => GDF,
"gdg" => GDG,
"gdh" => GDH,
"gdi" => GDI,
"gdj" => GDJ,
"gdk" => GDK,
"gdl" => GDL,
"gdm" => GDM,
"gdn" => GDN,
"gdo" => GDO,
"gdq" => GDQ,
"gdr" => GDR,
"gds" => GDS,
"gdt" => GDT,
"gdu" => GDU,
"gdx" => GDX,
"gea" => GEA,
"geb" => GEB,
"gec" => GEC,
"ged" => GED,
"gef" => GEF,
"geg" => GEG,
"geh" => GEH,
"gei" => GEI,
"gej" => GEJ,
"gek" => GEK,
"gel" => GEL,
"geq" => GEQ,
"ges" => GES,
"gev" => GEV,
"gew" => GEW,
"gex" => GEX,
"gey" => GEY,
"gfk" => GFK,
"gft" => GFT,
"gga" => GGA,
"ggb" => GGB,
"ggd" => GGD,
"gge" => GGE,
"ggg" => GGG,
"ggk" => GGK,
"ggl" => GGL,
"ggt" => GGT,
"ggu" => GGU,
"ggw" => GGW,
"gha" => GHA,
"ghc" => GHC,
"ghe" => GHE,
"ghh" => GHH,
"ghk" => GHK,
"ghl" => GHL,
"ghn" => GHN,
"gho" => GHO,
"ghr" => GHR,
"ghs" => GHS,
"ght" => GHT,
"gia" => GIA,
"gib" => GIB,
"gic" => GIC,
"gid" => GID,
"gie" => GIE,
"gig" => GIG,
"gih" => GIH,
"gii" => GII,
"gim" => GIM,
"gin" => GIN,
"gip" => GIP,
"giq" => GIQ,
"gir" => GIR,
"gis" => GIS,
"git" => GIT,
"giu" => GIU,
"giw" => GIW,
"gix" => GIX,
"giy" => GIY,
"giz" => GIZ,
"gjk" => GJK,
"gjm" => GJM,
"gjn" => GJN,
"gjr" => GJR,
"gju" => GJU,
"gka" => GKA,
"gkd" => GKD,
"gke" => GKE,
"gkn" => GKN,
"gko" => GKO,
"gkp" => GKP,
"gku" => GKU,
"glb" => GLB,
"glc" => GLC,
"gld" => GLD,
"glh" => GLH,
"glj" => GLJ,
"glk" => GLK,
"gll" => GLL,
"glo" => GLO,
"glr" => GLR,
"glu" => GLU,
"glw" => GLW,
"gly" => GLY,
"gma" => GMA,
"gmb" => GMB,
"gmd" => GMD,
"gmg" => GMG,
"gml" => GML,
"gmm" => GMM,
"gmn" => GMN,
"gmr" => GMR,
"gmu" => GMU,
"gmv" => GMV,
"gmx" => GMX,
"gmy" => GMY,
"gmz" => GMZ,
"gna" => GNA,
"gnb" => GNB,
"gnc" => GNC,
"gnd" => GND,
"gne" => GNE,
"gng" => GNG,
"gnh" => GNH,
"gni" => GNI,
"gnj" => GNJ,
"gnk" => GNK,
"gnl" => GNL,
"gnm" => GNM,
"gnn" => GNN,
"gno" => GNO,
"gnq" => GNQ,
"gnr" => GNR,
"gnt" => GNT,
"gnu" => GNU,
"gnw" => GNW,
"gnz" => GNZ,
"goa" => GOA,
"gob" => GOB,
"goc" => GOC,
"god" => GOD,
"goe" => GOE,
"gof" => GOF,
"gog" => GOG,
"goi" => GOI,
"goj" => GOJ,
"gok" => GOK,
"gol" => GOL,
"gom" => GOM,
"goo" => GOO,
"gop" => GOP,
"goq" => GOQ,
"gos" => GOS,
"gou" => GOU,
"gov" => GOV,
"gow" => GOW,
"gox" => GOX,
"goy" => GOY,
"goz" => GOZ,
"gpa" => GPA,
"gpe" => GPE,
"gpn" => GPN,
"gqa" => GQA,
"gqi" => GQI,
"gqn" => GQN,
"gqr" => GQR,
"gqu" => GQU,
"gra" => GRA,
"grd" => GRD,
"grg" => GRG,
"grh" => GRH,
"gri" => GRI,
"grj" => GRJ,
"grm" => GRM,
"gro" => GRO,
"grq" => GRQ,
"grr" => GRR,
"grs" => GRS,
"grt" => GRT,
"gru" => GRU,
"grv" => GRV,
"grw" => GRW,
"grx" => GRX,
"gry" => GRY,
"grz" => GRZ,
"gse" => GSE,
"gsg" => GSG,
"gsl" => GSL,
"gsm" => GSM,
"gsn" => GSN,
"gso" => GSO,
"gsp" => GSP,
"gss" => GSS,
"gta" => GTA,
"gtu" => GTU,
"gua" => GUA,
"gub" => GUB,
"guc" => GUC,
"gud" => GUD,
"gue" => GUE,
"guf" => GUF,
"gug" => GUG,
"guh" => GUH,
"gui" => GUI,
"guk" => GUK,
"gul" => GUL,
"gum" => GUM,
"gun" => GUN,
"guo" => GUO,
"gup" => GUP,
"guq" => GUQ,
"gur" => GUR,
"gus" => GUS,
"gut" => GUT,
"guu" => GUU,
"guw" => GUW,
"gux" => GUX,
"guz" => GUZ,
"gva" => GVA,
"gvc" => GVC,
"gve" => GVE,
"gvf" => GVF,
"gvj" => GVJ,
"gvl" => GVL,
"gvm" => GVM,
"gvn" => GVN,
"gvo" => GVO,
"gvp" => GVP,
"gvr" => GVR,
"gvs" => GVS,
"gvy" => GVY,
"gwa" => GWA,
"gwb" => GWB,
"gwc" => GWC,
"gwd" => GWD,
"gwe" => GWE,
"gwf" => GWF,
"gwg" => GWG,
"gwj" => GWJ,
"gwm" => GWM,
"gwn" => GWN,
"gwr" => GWR,
"gwt" => GWT,
"gwu" => GWU,
"gww" => GWW,
"gwx" => GWX,
"gxx" => GXX,
"gya" => GYA,
"gyb" => GYB,
"gyd" => GYD,
"gye" => GYE,
"gyf" => GYF,
"gyg" => GYG,
"gyi" => GYI,
"gyl" => GYL,
"gym" => GYM,
"gyn" => GYN,
"gyo" => GYO,
"gyr" => GYR,
"gyy" => GYY,
"gyz" => GYZ,
"gza" => GZA,
"gzi" => GZI,
"gzn" => GZN,
"haa" => HAA,
"hab" => HAB,
"hac" => HAC,
"had" => HAD,
"hae" => HAE,
"haf" => HAF,
"hag" => HAG,
"hah" => HAH,
"haj" => HAJ,
"hak" => HAK,
"hal" => HAL,
"ham" => HAM,
"han" => HAN,
"hao" => HAO,
"hap" => HAP,
"haq" => HAQ,
"har" => HAR,
"has" => HAS,
"hav" => HAV,
"hax" => HAX,
"hay" => HAY,
"haz" => HAZ,
"hba" => HBA,
"hbb" => HBB,
"hbn" => HBN,
"hbo" => HBO,
"sh" => SH,
"hbu" => HBU,
"hca" => HCA,
"hch" => HCH,
"hdn" => HDN,
"hds" => HDS,
"hdy" => HDY,
"hea" => HEA,
"hed" => HED,
"heg" => HEG,
"heh" => HEH,
"hei" => HEI,
"hem" => HEM,
"hgm" => HGM,
"hgw" => HGW,
"hhi" => HHI,
"hhr" => HHR,
"hhy" => HHY,
"hia" => HIA,
"hib" => HIB,
"hid" => HID,
"hif" => HIF,
"hig" => HIG,
"hih" => HIH,
"hii" => HII,
"hij" => HIJ,
"hik" => HIK,
"hio" => HIO,
"hir" => HIR,
"hiw" => HIW,
"hix" => HIX,
"hji" => HJI,
"hka" => HKA,
"hke" => HKE,
"hkh" => HKH,
"hkk" => HKK,
"hkn" => HKN,
"hks" => HKS,
"hla" => HLA,
"hlb" => HLB,
"hld" => HLD,
"hle" => HLE,
"hlt" => HLT,
"hlu" => HLU,
"hma" => HMA,
"hmb" => HMB,
"hmc" => HMC,
"hmd" => HMD,
"hme" => HME,
"hmf" => HMF,
"hmg" => HMG,
"hmh" => HMH,
"hmi" => HMI,
"hmj" => HMJ,
"hmk" => HMK,
"hml" => HML,
"hmm" => HMM,
"hmp" => HMP,
"hmq" => HMQ,
"hmr" => HMR,
"hms" => HMS,
"hmt" => HMT,
"hmu" => HMU,
"hmv" => HMV,
"hmw" => HMW,
"hmy" => HMY,
"hmz" => HMZ,
"hna" => HNA,
"hnd" => HND,
"hne" => HNE,
"hng" => HNG,
"hnh" => HNH,
"hni" => HNI,
"hnj" => HNJ,
"hnn" => HNN,
"hno" => HNO,
"hns" => HNS,
"hnu" => HNU,
"hoa" => HOA,
"hob" => HOB,
"hoc" => HOC,
"hod" => HOD,
"hoe" => HOE,
"hoh" => HOH,
"hoi" => HOI,
"hoj" => HOJ,
"hol" => HOL,
"hom" => HOM,
"hoo" => HOO,
"hop" => HOP,
"hor" => HOR,
"hos" => HOS,
"hot" => HOT,
"hov" => HOV,
"how" => HOW,
"hoy" => HOY,
"hoz" => HOZ,
"hpo" => HPO,
"hps" => HPS,
"hra" => HRA,
"hrc" => HRC,
"hre" => HRE,
"hrk" => HRK,
"hrm" => HRM,
"hro" => HRO,
"hrp" => HRP,
"hrt" => HRT,
"hru" => HRU,
"hrw" => HRW,
"hrx" => HRX,
"hrz" => HRZ,
"hsh" => HSH,
"hsl" => HSL,
"hsn" => HSN,
"hss" => HSS,
"hti" => HTI,
"hto" => HTO,
"hts" => HTS,
"htu" => HTU,
"htx" => HTX,
"hub" => HUB,
"huc" => HUC,
"hud" => HUD,
"hue" => HUE,
"huf" => HUF,
"hug" => HUG,
"huh" => HUH,
"hui" => HUI,
"huj" => HUJ,
"huk" => HUK,
"hul" => HUL,
"hum" => HUM,
"huo" => HUO,
"huq" => HUQ,
"hur" => HUR,
"hus" => HUS,
"hut" => HUT,
"huu" => HUU,
"huv" => HUV,
"huw" => HUW,
"hux" => HUX,
"huy" => HUY,
"huz" => HUZ,
"hvc" => HVC,
"hve" => HVE,
"hvk" => HVK,
"hvn" => HVN,
"hvv" => HVV,
"hwa" => HWA,
"hwc" => HWC,
"hwo" => HWO,
"hya" => HYA,
"hyw" => HYW,
"iai" => IAI,
"ian" => IAN,
"iar" => IAR,
"ibb" => IBB,
"ibd" => IBD,
"ibe" => IBE,
"ibg" => IBG,
"ibh" => IBH,
"ibl" => IBL,
"ibm" => IBM,
"ibn" => IBN,
"ibr" => IBR,
"ibu" => IBU,
"iby" => IBY,
"ica" => ICA,
"ich" => ICH,
"icl" => ICL,
"icr" => ICR,
"ida" => IDA,
"idb" => IDB,
"idc" => IDC,
"idd" => IDD,
"ide" => IDE,
"idi" => IDI,
"idr" => IDR,
"ids" => IDS,
"idt" => IDT,
"idu" => IDU,
"ifa" => IFA,
"ifb" => IFB,
"ife" => IFE,
"iff" => IFF,
"ifk" => IFK,
"ifm" => IFM,
"ifu" => IFU,
"ify" => IFY,
"igb" => IGB,
"ige" => IGE,
"igg" => IGG,
"igl" => IGL,
"igm" => IGM,
"ign" => IGN,
"igo" => IGO,
"igs" => IGS,
"igw" => IGW,
"ihb" => IHB,
"ihi" => IHI,
"ihp" => IHP,
"ihw" => IHW,
"iin" => IIN,
"ijc" => IJC,
"ije" => IJE,
"ijj" => IJJ,
"ijn" => IJN,
"ijs" => IJS,
"ike" => IKE,
"iki" => IKI,
"ikk" => IKK,
"ikl" => IKL,
"iko" => IKO,
"ikp" => IKP,
"ikr" => IKR,
"iks" => IKS,
"ikt" => IKT,
"ikv" => IKV,
"ikw" => IKW,
"ikx" => IKX,
"ikz" => IKZ,
"ila" => ILA,
"ilb" => ILB,
"ilg" => ILG,
"ili" => ILI,
"ilk" => ILK,
"ilm" => ILM,
"ilp" => ILP,
"ils" => ILS,
"ilu" => ILU,
"ilv" => ILV,
"ima" => IMA,
"imi" => IMI,
"iml" => IML,
"imn" => IMN,
"imo" => IMO,
"imr" => IMR,
"ims" => IMS,
"imt" => IMT,
"imy" => IMY,
"inb" => INB,
"ing" => ING,
"inj" => INJ,
"inl" => INL,
"inm" => INM,
"inn" => INN,
"ino" => INO,
"inp" => INP,
"ins" => INS,
"int" => INT,
"inz" => INZ,
"ior" => IOR,
"iou" => IOU,
"iow" => IOW,
"ipi" => IPI,
"ipo" => IPO,
"iqu" => IQU,
"iqw" => IQW,
"ire" => IRE,
"irh" => IRH,
"iri" => IRI,
"irk" => IRK,
"irn" => IRN,
"irr" => IRR,
"iru" => IRU,
"irx" => IRX,
"iry" => IRY,
"isa" => ISA,
"isc" => ISC,
"isd" => ISD,
"ise" => ISE,
"isg" => ISG,
"ish" => ISH,
"isi" => ISI,
"isk" => ISK,
"ism" => ISM,
"isn" => ISN,
"iso" => ISO,
"isr" => ISR,
"ist" => IST,
"isu" => ISU,
"itb" => ITB,
"itd" => ITD,
"ite" => ITE,
"iti" => ITI,
"itk" => ITK,
"itl" => ITL,
"itm" => ITM,
"ito" => ITO,
"itr" => ITR,
"its" => ITS,
"itt" => ITT,
"itv" => ITV,
"itw" => ITW,
"itx" => ITX,
"ity" => ITY,
"itz" => ITZ,
"ium" => IUM,
"ivb" => IVB,
"ivv" => IVV,
"iwk" => IWK,
"iwm" => IWM,
"iwo" => IWO,
"iws" => IWS,
"ixc" => IXC,
"ixl" => IXL,
"iya" => IYA,
"iyo" => IYO,
"iyx" => IYX,
"izh" => IZH,
"izr" => IZR,
"izz" => IZZ,
"jaa" => JAA,
"jab" => JAB,
"jac" => JAC,
"jad" => JAD,
"jae" => JAE,
"jaf" => JAF,
"jah" => JAH,
"jaj" => JAJ,
"jak" => JAK,
"jal" => JAL,
"jam" => JAM,
"jan" => JAN,
"jao" => JAO,
"jaq" => JAQ,
"jas" => JAS,
"jat" => JAT,
"jau" => JAU,
"jax" => JAX,
"jay" => JAY,
"jaz" => JAZ,
"jbe" => JBE,
"jbi" => JBI,
"jbj" => JBJ,
"jbk" => JBK,
"jbm" => JBM,
"jbn" => JBN,
"jbr" => JBR,
"jbt" => JBT,
"jbu" => JBU,
"jbw" => JBW,
"jcs" => JCS,
"jct" => JCT,
"jda" => JDA,
"jdg" => JDG,
"jdt" => JDT,
"jeb" => JEB,
"jee" => JEE,
"jeh" => JEH,
"jei" => JEI,
"jek" => JEK,
"jel" => JEL,
"jen" => JEN,
"jer" => JER,
"jet" => JET,
"jeu" => JEU,
"jgb" => JGB,
"jge" => JGE,
"jgk" => JGK,
"jgo" => JGO,
"jhi" => JHI,
"jhs" => JHS,
"jia" => JIA,
"jib" => JIB,
"jic" => JIC,
"jid" => JID,
"jie" => JIE,
"jig" => JIG,
"jih" => JIH,
"jii" => JII,
"jil" => JIL,
"jim" => JIM,
"jio" => JIO,
"jiq" => JIQ,
"jit" => JIT,
"jiu" => JIU,
"jiv" => JIV,
"jiy" => JIY,
"jje" => JJE,
"jjr" => JJR,
"jka" => JKA,
"jkm" => JKM,
"jko" => JKO,
"jkp" => JKP,
"jkr" => JKR,
"jks" => JKS,
"jku" => JKU,
"jle" => JLE,
"jls" => JLS,
"jma" => JMA,
"jmb" => JMB,
"jmc" => JMC,
"jmd" => JMD,
"jmi" => JMI,
"jml" => JML,
"jmn" => JMN,
"jmr" => JMR,
"jms" => JMS,
"jmw" => JMW,
"jmx" => JMX,
"jna" => JNA,
"jnd" => JND,
"jng" => JNG,
"jni" => JNI,
"jnj" => JNJ,
"jnl" => JNL,
"jns" => JNS,
"job" => JOB,
"jod" => JOD,
"jog" => JOG,
"jor" => JOR,
"jos" => JOS,
"jow" => JOW,
"jpa" => JPA,
"jqr" => JQR,
"jra" => JRA,
"jrr" => JRR,
"jrt" => JRT,
"jru" => JRU,
"jsl" => JSL,
"jua" => JUA,
"jub" => JUB,
"juc" => JUC,
"jud" => JUD,
"juh" => JUH,
"jui" => JUI,
"juk" => JUK,
"jul" => JUL,
"jum" => JUM,
"jun" => JUN,
"juo" => JUO,
"jup" => JUP,
"jur" => JUR,
"jus" => JUS,
"jut" => JUT,
"juu" => JUU,
"juw" => JUW,
"juy" => JUY,
"jvd" => JVD,
"jvn" => JVN,
"jwi" => JWI,
"jya" => JYA,
"jye" => JYE,
"jyy" => JYY,
"kad" => KAD,
"kae" => KAE,
"kaf" => KAF,
"kag" => KAG,
"kah" => KAH,
"kai" => KAI,
"kaj" => KAJ,
"kak" => KAK,
"kao" => KAO,
"kap" => KAP,
"kaq" => KAQ,
"kav" => KAV,
"kax" => KAX,
"kay" => KAY,
"kba" => KBA,
"kbb" => KBB,
"kbc" => KBC,
"kbe" => KBE,
"kbg" => KBG,
"kbh" => KBH,
"kbi" => KBI,
"kbj" => KBJ,
"kbk" => KBK,
"kbl" => KBL,
"kbm" => KBM,
"kbn" => KBN,
"kbo" => KBO,
"kbp" => KBP,
"kbq" => KBQ,
"kbr" => KBR,
"kbs" => KBS,
"kbt" => KBT,
"kbu" => KBU,
"kbv" => KBV,
"kbw" => KBW,
"kbx" => KBX,
"kby" => KBY,
"kbz" => KBZ,
"kca" => KCA,
"kcb" => KCB,
"kcc" => KCC,
"kcd" => KCD,
"kce" => KCE,
"kcf" => KCF,
"kcg" => KCG,
"kch" => KCH,
"kci" => KCI,
"kcj" => KCJ,
"kck" => KCK,
"kcl" => KCL,
"kcm" => KCM,
"kcn" => KCN,
"kco" => KCO,
"kcp" => KCP,
"kcq" => KCQ,
"kcr" => KCR,
"kcs" => KCS,
"kct" => KCT,
"kcu" => KCU,
"kcv" => KCV,
"kcw" => KCW,
"kcx" => KCX,
"kcy" => KCY,
"kcz" => KCZ,
"kda" => KDA,
"kdc" => KDC,
"kdd" => KDD,
"kde" => KDE,
"kdf" => KDF,
"kdg" => KDG,
"kdh" => KDH,
"kdi" => KDI,
"kdj" => KDJ,
"kdk" => KDK,
"kdl" => KDL,
"kdm" => KDM,
"kdn" => KDN,
"kdp" => KDP,
"kdq" => KDQ,
"kdr" => KDR,
"kdt" => KDT,
"kdu" => KDU,
"kdw" => KDW,
"kdx" => KDX,
"kdy" => KDY,
"kdz" => KDZ,
"kea" => KEA,
"keb" => KEB,
"kec" => KEC,
"ked" => KED,
"kee" => KEE,
"kef" => KEF,
"keg" => KEG,
"keh" => KEH,
"kei" => KEI,
"kej" => KEJ,
"kek" => KEK,
"kel" => KEL,
"kem" => KEM,
"ken" => KEN,
"keo" => KEO,
"kep" => KEP,
"keq" => KEQ,
"ker" => KER,
"kes" => KES,
"ket" => KET,
"keu" => KEU,
"kev" => KEV,
"kew" => KEW,
"kex" => KEX,
"key" => KEY,
"kez" => KEZ,
"kfa" => KFA,
"kfb" => KFB,
"kfc" => KFC,
"kfd" => KFD,
"kfe" => KFE,
"kff" => KFF,
"kfg" => KFG,
"kfh" => KFH,
"kfi" => KFI,
"kfj" => KFJ,
"kfk" => KFK,
"kfl" => KFL,
"kfm" => KFM,
"kfn" => KFN,
"kfo" => KFO,
"kfp" => KFP,
"kfq" => KFQ,
"kfr" => KFR,
"kfs" => KFS,
"kft" => KFT,
"kfu" => KFU,
"kfv" => KFV,
"kfw" => KFW,
"kfx" => KFX,
"kfy" => KFY,
"kfz" => KFZ,
"kga" => KGA,
"kgb" => KGB,
"kge" => KGE,
"kgf" => KGF,
"kgg" => KGG,
"kgi" => KGI,
"kgj" => KGJ,
"kgk" => KGK,
"kgl" => KGL,
"kgm" => KGM,
"kgn" => KGN,
"kgo" => KGO,
"kgp" => KGP,
"kgq" => KGQ,
"kgr" => KGR,
"kgs" => KGS,
"kgt" => KGT,
"kgu" => KGU,
"kgv" => KGV,
"kgw" => KGW,
"kgx" => KGX,
"kgy" => KGY,
"khb" => KHB,
"khc" => KHC,
"khd" => KHD,
"khe" => KHE,
"khf" => KHF,
"khg" => KHG,
"khh" => KHH,
"khj" => KHJ,
"khk" => KHK,
"khl" => KHL,
"khn" => KHN,
"khp" => KHP,
"khq" => KHQ,
"khr" => KHR,
"khs" => KHS,
"kht" => KHT,
"khu" => KHU,
"khv" => KHV,
"khw" => KHW,
"khx" => KHX,
"khy" => KHY,
"khz" => KHZ,
"kia" => KIA,
"kib" => KIB,
"kic" => KIC,
"kid" => KID,
"kie" => KIE,
"kif" => KIF,
"kig" => KIG,
"kih" => KIH,
"kii" => KII,
"kij" => KIJ,
"kil" => KIL,
"kim" => KIM,
"kio" => KIO,
"kip" => KIP,
"kiq" => KIQ,
"kis" => KIS,
"kit" => KIT,
"kiu" => KIU,
"kiv" => KIV,
"kiw" => KIW,
"kix" => KIX,
"kiy" => KIY,
"kiz" => KIZ,
"kja" => KJA,
"kjb" => KJB,
"kjc" => KJC,
"kjd" => KJD,
"kje" => KJE,
"kjg" => KJG,
"kjh" => KJH,
"kji" => KJI,
"kjj" => KJJ,
"kjk" => KJK,
"kjl" => KJL,
"kjm" => KJM,
"kjn" => KJN,
"kjo" => KJO,
"kjp" => KJP,
"kjq" => KJQ,
"kjr" => KJR,
"kjs" => KJS,
"kjt" => KJT,
"kju" => KJU,
"kjv" => KJV,
"kjx" => KJX,
"kjy" => KJY,
"kjz" => KJZ,
"kka" => KKA,
"kkb" => KKB,
"kkc" => KKC,
"kkd" => KKD,
"kke" => KKE,
"kkf" => KKF,
"kkg" => KKG,
"kkh" => KKH,
"kki" => KKI,
"kkj" => KKJ,
"kkk" => KKK,
"kkl" => KKL,
"kkm" => KKM,
"kkn" => KKN,
"kko" => KKO,
"kkp" => KKP,
"kkq" => KKQ,
"kkr" => KKR,
"kks" => KKS,
"kkt" => KKT,
"kku" => KKU,
"kkv" => KKV,
"kkw" => KKW,
"kkx" => KKX,
"kky" => KKY,
"kkz" => KKZ,
"kla" => KLA,
"klb" => KLB,
"klc" => KLC,
"kld" => KLD,
"kle" => KLE,
"klf" => KLF,
"klg" => KLG,
"klh" => KLH,
"kli" => KLI,
"klj" => KLJ,
"klk" => KLK,
"kll" => KLL,
"klm" => KLM,
"kln" => KLN,
"klo" => KLO,
"klp" => KLP,
"klq" => KLQ,
"klr" => KLR,
"kls" => KLS,
"klt" => KLT,
"klu" => KLU,
"klv" => KLV,
"klw" => KLW,
"klx" => KLX,
"kly" => KLY,
"klz" => KLZ,
"kma" => KMA,
"kmc" => KMC,
"kmd" => KMD,
"kme" => KME,
"kmf" => KMF,
"kmg" => KMG,
"kmh" => KMH,
"kmi" => KMI,
"kmj" => KMJ,
"kmk" => KMK,
"kml" => KML,
"kmm" => KMM,
"kmn" => KMN,
"kmo" => KMO,
"kmp" => KMP,
"kmq" => KMQ,
"kmr" => KMR,
"kms" => KMS,
"kmt" => KMT,
"kmu" => KMU,
"kmv" => KMV,
"kmw" => KMW,
"kmx" => KMX,
"kmy" => KMY,
"kmz" => KMZ,
"kna" => KNA,
"knb" => KNB,
"knc" => KNC,
"knd" => KND,
"kne" => KNE,
"knf" => KNF,
"kng" => KNG,
"kni" => KNI,
"knj" => KNJ,
"knk" => KNK,
"knl" => KNL,
"knm" => KNM,
"knn" => KNN,
"kno" => KNO,
"knp" => KNP,
"knq" => KNQ,
"knr" => KNR,
"kns" => KNS,
"knt" => KNT,
"knu" => KNU,
"knv" => KNV,
"knw" => KNW,
"knx" => KNX,
"kny" => KNY,
"knz" => KNZ,
"koa" => KOA,
"koc" => KOC,
"kod" => KOD,
"koe" => KOE,
"kof" => KOF,
"kog" => KOG,
"koh" => KOH,
"koi" => KOI,
"kol" => KOL,
"koo" => KOO,
"kop" => KOP,
"koq" => KOQ,
"kot" => KOT,
"kou" => KOU,
"kov" => KOV,
"kow" => KOW,
"koy" => KOY,
"koz" => KOZ,
"kpa" => KPA,
"kpb" => KPB,
"kpc" => KPC,
"kpd" => KPD,
"kpf" => KPF,
"kpg" => KPG,
"kph" => KPH,
"kpi" => KPI,
"kpj" => KPJ,
"kpk" => KPK,
"kpl" => KPL,
"kpm" => KPM,
"kpn" => KPN,
"kpo" => KPO,
"kpq" => KPQ,
"kpr" => KPR,
"kps" => KPS,
"kpt" => KPT,
"kpu" => KPU,
"kpv" => KPV,
"kpw" => KPW,
"kpx" => KPX,
"kpy" => KPY,
"kpz" => KPZ,
"kqa" => KQA,
"kqb" => KQB,
"kqc" => KQC,
"kqd" => KQD,
"kqe" => KQE,
"kqf" => KQF,
"kqg" => KQG,
"kqh" => KQH,
"kqi" => KQI,
"kqj" => KQJ,
"kqk" => KQK,
"kql" => KQL,
"kqm" => KQM,
"kqn" => KQN,
"kqo" => KQO,
"kqp" => KQP,
"kqq" => KQQ,
"kqr" => KQR,
"kqs" => KQS,
"kqt" => KQT,
"kqu" => KQU,
"kqv" => KQV,
"kqw" => KQW,
"kqx" => KQX,
"kqy" => KQY,
"kqz" => KQZ,
"kra" => KRA,
"krb" => KRB,
"krd" => KRD,
"kre" => KRE,
"krf" => KRF,
"krh" => KRH,
"kri" => KRI,
"krj" => KRJ,
"krk" => KRK,
"krn" => KRN,
"krp" => KRP,
"krr" => KRR,
"krs" => KRS,
"krt" => KRT,
"krv" => KRV,
"krw" => KRW,
"krx" => KRX,
"kry" => KRY,
"krz" => KRZ,
"ksa" => KSA,
"ksb" => KSB,
"ksc" => KSC,
"ksd" => KSD,
"kse" => KSE,
"ksf" => KSF,
"ksg" => KSG,
"ksh" => KSH,
"ksi" => KSI,
"ksj" => KSJ,
"ksk" => KSK,
"ksl" => KSL,
"ksm" => KSM,
"ksn" => KSN,
"kso" => KSO,
"ksp" => KSP,
"ksq" => KSQ,
"ksr" => KSR,
"kss" => KSS,
"kst" => KST,
"ksu" => KSU,
"ksv" => KSV,
"ksw" => KSW,
"ksx" => KSX,
"ksy" => KSY,
"ksz" => KSZ,
"kta" => KTA,
"ktb" => KTB,
"ktc" => KTC,
"ktd" => KTD,
"kte" => KTE,
"ktf" => KTF,
"ktg" => KTG,
"kth" => KTH,
"kti" => KTI,
"ktj" => KTJ,
"ktk" => KTK,
"ktl" => KTL,
"ktm" => KTM,
"ktn" => KTN,
"kto" => KTO,
"ktp" => KTP,
"ktq" => KTQ,
"kts" => KTS,
"ktt" => KTT,
"ktu" => KTU,
"ktv" => KTV,
"ktw" => KTW,
"ktx" => KTX,
"kty" => KTY,
"ktz" => KTZ,
"kub" => KUB,
"kuc" => KUC,
"kud" => KUD,
"kue" => KUE,
"kuf" => KUF,
"kug" => KUG,
"kuh" => KUH,
"kui" => KUI,
"kuj" => KUJ,
"kuk" => KUK,
"kul" => KUL,
"kun" => KUN,
"kuo" => KUO,
"kup" => KUP,
"kuq" => KUQ,
"kus" => KUS,
"kuu" => KUU,
"kuv" => KUV,
"kuw" => KUW,
"kux" => KUX,
"kuy" => KUY,
"kuz" => KUZ,
"kva" => KVA,
"kvb" => KVB,
"kvc" => KVC,
"kvd" => KVD,
"kve" => KVE,
"kvf" => KVF,
"kvg" => KVG,
"kvh" => KVH,
"kvi" => KVI,
"kvj" => KVJ,
"kvk" => KVK,
"kvl" => KVL,
"kvm" => KVM,
"kvn" => KVN,
"kvo" => KVO,
"kvp" => KVP,
"kvq" => KVQ,
"kvr" => KVR,
"kvt" => KVT,
"kvu" => KVU,
"kvv" => KVV,
"kvw" => KVW,
"kvx" => KVX,
"kvy" => KVY,
"kvz" => KVZ,
"kwa" => KWA,
"kwb" => KWB,
"kwc" => KWC,
"kwd" => KWD,
"kwe" => KWE,
"kwf" => KWF,
"kwg" => KWG,
"kwh" => KWH,
"kwi" => KWI,
"kwj" => KWJ,
"kwk" => KWK,
"kwl" => KWL,
"kwm" => KWM,
"kwn" => KWN,
"kwo" => KWO,
"kwp" => KWP,
"kwr" => KWR,
"kws" => KWS,
"kwt" => KWT,
"kwu" => KWU,
"kwv" => KWV,
"kww" => KWW,
"kwx" => KWX,
"kwy" => KWY,
"kwz" => KWZ,
"kxa" => KXA,
"kxb" => KXB,
"kxc" => KXC,
"kxd" => KXD,
"kxf" => KXF,
"kxh" => KXH,
"kxi" => KXI,
"kxj" => KXJ,
"kxk" => KXK,
"kxm" => KXM,
"kxn" => KXN,
"kxo" => KXO,
"kxp" => KXP,
"kxq" => KXQ,
"kxr" => KXR,
"kxs" => KXS,
"kxt" => KXT,
"kxv" => KXV,
"kxw" => KXW,
"kxx" => KXX,
"kxy" => KXY,
"kxz" => KXZ,
"kya" => KYA,
"kyb" => KYB,
"kyc" => KYC,
"kyd" => KYD,
"kye" => KYE,
"kyf" => KYF,
"kyg" => KYG,
"kyh" => KYH,
"kyi" => KYI,
"kyj" => KYJ,
"kyk" => KYK,
"kyl" => KYL,
"kym" => KYM,
"kyn" => KYN,
"kyo" => KYO,
"kyp" => KYP,
"kyq" => KYQ,
"kyr" => KYR,
"kys" => KYS,
"kyt" => KYT,
"kyu" => KYU,
"kyv" => KYV,
"kyw" => KYW,
"kyx" => KYX,
"kyy" => KYY,
"kyz" => KYZ,
"kza" => KZA,
"kzb" => KZB,
"kzc" => KZC,
"kzd" => KZD,
"kze" => KZE,
"kzf" => KZF,
"kzg" => KZG,
"kzi" => KZI,
"kzk" => KZK,
"kzl" => KZL,
"kzm" => KZM,
"kzn" => KZN,
"kzo" => KZO,
"kzp" => KZP,
"kzq" => KZQ,
"kzr" => KZR,
"kzs" => KZS,
"kzu" => KZU,
"kzv" => KZV,
"kzw" => KZW,
"kzx" => KZX,
"kzy" => KZY,
"kzz" => KZZ,
"laa" => LAA,
"lab" => LAB,
"lac" => LAC,
"lae" => LAE,
"laf" => LAF,
"lag" => LAG,
"lai" => LAI,
"laj" => LAJ,
"lal" => LAL,
"lan" => LAN,
"lap" => LAP,
"laq" => LAQ,
"lar" => LAR,
"las" => LAS,
"lau" => LAU,
"law" => LAW,
"lax" => LAX,
"lay" => LAY,
"laz" => LAZ,
"lbb" => LBB,
"lbc" => LBC,
"lbe" => LBE,
"lbf" => LBF,
"lbg" => LBG,
"lbi" => LBI,
"lbj" => LBJ,
"lbk" => LBK,
"lbl" => LBL,
"lbm" => LBM,
"lbn" => LBN,
"lbo" => LBO,
"lbq" => LBQ,
"lbr" => LBR,
"lbs" => LBS,
"lbt" => LBT,
"lbu" => LBU,
"lbv" => LBV,
"lbw" => LBW,
"lbx" => LBX,
"lby" => LBY,
"lbz" => LBZ,
"lcc" => LCC,
"lcd" => LCD,
"lce" => LCE,
"lcf" => LCF,
"lch" => LCH,
"lcl" => LCL,
"lcm" => LCM,
"lcp" => LCP,
"lcq" => LCQ,
"lcs" => LCS,
"lda" => LDA,
"ldb" => LDB,
"ldd" => LDD,
"ldg" => LDG,
"ldh" => LDH,
"ldi" => LDI,
"ldj" => LDJ,
"ldk" => LDK,
"ldl" => LDL,
"ldm" => LDM,
"ldn" => LDN,
"ldo" => LDO,
"ldp" => LDP,
"ldq" => LDQ,
"lea" => LEA,
"leb" => LEB,
"lec" => LEC,
"led" => LED,
"lee" => LEE,
"lef" => LEF,
"leh" => LEH,
"lei" => LEI,
"lej" => LEJ,
"lek" => LEK,
"lel" => LEL,
"lem" => LEM,
"len" => LEN,
"leo" => LEO,
"lep" => LEP,
"leq" => LEQ,
"ler" => LER,
"les" => LES,
"let" => LET,
"leu" => LEU,
"lev" => LEV,
"lew" => LEW,
"lex" => LEX,
"ley" => LEY,
"lfa" => LFA,
"lfn" => LFN,
"lga" => LGA,
"lgb" => LGB,
"lgg" => LGG,
"lgh" => LGH,
"lgi" => LGI,
"lgk" => LGK,
"lgl" => LGL,
"lgm" => LGM,
"lgn" => LGN,
"lgo" => LGO,
"lgq" => LGQ,
"lgr" => LGR,
"lgt" => LGT,
"lgu" => LGU,
"lgz" => LGZ,
"lha" => LHA,
"lhh" => LHH,
"lhi" => LHI,
"lhl" => LHL,
"lhm" => LHM,
"lhn" => LHN,
"lhp" => LHP,
"lhs" => LHS,
"lht" => LHT,
"lhu" => LHU,
"lia" => LIA,
"lib" => LIB,
"lic" => LIC,
"lid" => LID,
"lie" => LIE,
"lif" => LIF,
"lig" => LIG,
"lih" => LIH,
"lij" => LIJ,
"lik" => LIK,
"lil" => LIL,
"lio" => LIO,
"lip" => LIP,
"liq" => LIQ,
"lir" => LIR,
"lis" => LIS,
"liu" => LIU,
"liv" => LIV,
"liw" => LIW,
"lix" => LIX,
"liy" => LIY,
"liz" => LIZ,
"lja" => LJA,
"lje" => LJE,
"lji" => LJI,
"ljl" => LJL,
"ljp" => LJP,
"ljw" => LJW,
"ljx" => LJX,
"lka" => LKA,
"lkb" => LKB,
"lkc" => LKC,
"lkd" => LKD,
"lke" => LKE,
"lkh" => LKH,
"lki" => LKI,
"lkj" => LKJ,
"lkl" => LKL,
"lkm" => LKM,
"lkn" => LKN,
"lko" => LKO,
"lkr" => LKR,
"lks" => LKS,
"lkt" => LKT,
"lku" => LKU,
"lky" => LKY,
"lla" => LLA,
"llb" => LLB,
"llc" => LLC,
"lld" => LLD,
"lle" => LLE,
"llf" => LLF,
"llg" => LLG,
"llh" => LLH,
"lli" => LLI,
"llj" => LLJ,
"llk" => LLK,
"lll" => LLL,
"llm" => LLM,
"lln" => LLN,
"llp" => LLP,
"llq" => LLQ,
"lls" => LLS,
"llu" => LLU,
"llx" => LLX,
"lma" => LMA,
"lmb" => LMB,
"lmc" => LMC,
"lmd" => LMD,
"lme" => LME,
"lmf" => LMF,
"lmg" => LMG,
"lmh" => LMH,
"lmi" => LMI,
"lmj" => LMJ,
"lmk" => LMK,
"lml" => LML,
"lmn" => LMN,
"lmo" => LMO,
"lmp" => LMP,
"lmq" => LMQ,
"lmr" => LMR,
"lmu" => LMU,
"lmv" => LMV,
"lmw" => LMW,
"lmx" => LMX,
"lmy" => LMY,
"lna" => LNA,
"lnb" => LNB,
"lnd" => LND,
"lng" => LNG,
"lnh" => LNH,
"lni" => LNI,
"lnj" => LNJ,
"lnl" => LNL,
"lnm" => LNM,
"lnn" => LNN,
"lns" => LNS,
"lnu" => LNU,
"lnw" => LNW,
"lnz" => LNZ,
"loa" => LOA,
"lob" => LOB,
"loc" => LOC,
"loe" => LOE,
"lof" => LOF,
"log" => LOG,
"loh" => LOH,
"loi" => LOI,
"loj" => LOJ,
"lok" => LOK,
"lom" => LOM,
"lon" => LON,
"loo" => LOO,
"lop" => LOP,
"loq" => LOQ,
"lor" => LOR,
"los" => LOS,
"lot" => LOT,
"lou" => LOU,
"lov" => LOV,
"low" => LOW,
"lox" => LOX,
"loy" => LOY,
"lpa" => LPA,
"lpe" => LPE,
"lpn" => LPN,
"lpo" => LPO,
"lpx" => LPX,
"lqr" => LQR,
"lra" => LRA,
"lrc" => LRC,
"lre" => LRE,
"lrg" => LRG,
"lri" => LRI,
"lrk" => LRK,
"lrl" => LRL,
"lrm" => LRM,
"lrn" => LRN,
"lro" => LRO,
"lrr" => LRR,
"lrt" => LRT,
"lrv" => LRV,
"lrz" => LRZ,
"lsa" => LSA,
"lsb" => LSB,
"lsc" => LSC,
"lsd" => LSD,
"lse" => LSE,
"lsh" => LSH,
"lsi" => LSI,
"lsl" => LSL,
"lsm" => LSM,
"lsn" => LSN,
"lso" => LSO,
"lsp" => LSP,
"lsr" => LSR,
"lss" => LSS,
"lst" => LST,
"lsv" => LSV,
"lsw" => LSW,
"lsy" => LSY,
"ltc" => LTC,
"ltg" => LTG,
"lth" => LTH,
"lti" => LTI,
"ltn" => LTN,
"lto" => LTO,
"lts" => LTS,
"ltu" => LTU,
"luc" => LUC,
"lud" => LUD,
"lue" => LUE,
"luf" => LUF,
"luj" => LUJ,
"luk" => LUK,
"lul" => LUL,
"lum" => LUM,
"lup" => LUP,
"luq" => LUQ,
"lur" => LUR,
"lut" => LUT,
"luu" => LUU,
"luv" => LUV,
"luw" => LUW,
"luy" => LUY,
"luz" => LUZ,
"lva" => LVA,
"lvi" => LVI,
"lvk" => LVK,
"lvs" => LVS,
"lvu" => LVU,
"lwa" => LWA,
"lwe" => LWE,
"lwg" => LWG,
"lwh" => LWH,
"lwl" => LWL,
"lwm" => LWM,
"lwo" => LWO,
"lws" => LWS,
"lwt" => LWT,
"lwu" => LWU,
"lww" => LWW,
"lxm" => LXM,
"lya" => LYA,
"lyg" => LYG,
"lyn" => LYN,
"lzh" => LZH,
"lzl" => LZL,
"lzn" => LZN,
"lzz" => LZZ,
"maa" => MAA,
"mab" => MAB,
"mae" => MAE,
"maf" => MAF,
"maj" => MAJ,
"mam" => MAM,
"maq" => MAQ,
"mat" => MAT,
"mau" => MAU,
"mav" => MAV,
"maw" => MAW,
"max" => MAX,
"maz" => MAZ,
"mba" => MBA,
"mbb" => MBB,
"mbc" => MBC,
"mbd" => MBD,
"mbe" => MBE,
"mbf" => MBF,
"mbh" => MBH,
"mbi" => MBI,
"mbj" => MBJ,
"mbk" => MBK,
"mbl" => MBL,
"mbm" => MBM,
"mbn" => MBN,
"mbo" => MBO,
"mbp" => MBP,
"mbq" => MBQ,
"mbr" => MBR,
"mbs" => MBS,
"mbt" => MBT,
"mbu" => MBU,
"mbv" => MBV,
"mbw" => MBW,
"mbx" => MBX,
"mby" => MBY,
"mbz" => MBZ,
"mca" => MCA,
"mcb" => MCB,
"mcc" => MCC,
"mcd" => MCD,
"mce" => MCE,
"mcf" => MCF,
"mcg" => MCG,
"mch" => MCH,
"mci" => MCI,
"mcj" => MCJ,
"mck" => MCK,
"mcl" => MCL,
"mcm" => MCM,
"mcn" => MCN,
"mco" => MCO,
"mcp" => MCP,
"mcq" => MCQ,
"mcr" => MCR,
"mcs" => MCS,
"mct" => MCT,
"mcu" => MCU,
"mcv" => MCV,
"mcw" => MCW,
"mcx" => MCX,
"mcy" => MCY,
"mcz" => MCZ,
"mda" => MDA,
"mdb" => MDB,
"mdc" => MDC,
"mdd" => MDD,
"mde" => MDE,
"mdg" => MDG,
"mdh" => MDH,
"mdi" => MDI,
"mdj" => MDJ,
"mdk" => MDK,
"mdl" => MDL,
"mdm" => MDM,
"mdn" => MDN,
"mdp" => MDP,
"mdq" => MDQ,
"mds" => MDS,
"mdt" => MDT,
"mdu" => MDU,
"mdv" => MDV,
"mdw" => MDW,
"mdx" => MDX,
"mdy" => MDY,
"mdz" => MDZ,
"mea" => MEA,
"meb" => MEB,
"mec" => MEC,
"med" => MED,
"mee" => MEE,
"mef" => MEF,
"meh" => MEH,
"mei" => MEI,
"mej" => MEJ,
"mek" => MEK,
"mel" => MEL,
"mem" => MEM,
"meo" => MEO,
"mep" => MEP,
"meq" => MEQ,
"mer" => MER,
"mes" => MES,
"met" => MET,
"meu" => MEU,
"mev" => MEV,
"mew" => MEW,
"mey" => MEY,
"mez" => MEZ,
"mfa" => MFA,
"mfb" => MFB,
"mfc" => MFC,
"mfd" => MFD,
"mfe" => MFE,
"mff" => MFF,
"mfg" => MFG,
"mfh" => MFH,
"mfi" => MFI,
"mfj" => MFJ,
"mfk" => MFK,
"mfl" => MFL,
"mfm" => MFM,
"mfn" => MFN,
"mfo" => MFO,
"mfp" => MFP,
"mfq" => MFQ,
"mfr" => MFR,
"mfs" => MFS,
"mft" => MFT,
"mfu" => MFU,
"mfv" => MFV,
"mfw" => MFW,
"mfx" => MFX,
"mfy" => MFY,
"mfz" => MFZ,
"mgb" => MGB,
"mgc" => MGC,
"mgd" => MGD,
"mge" => MGE,
"mgf" => MGF,
"mgg" => MGG,
"mgh" => MGH,
"mgi" => MGI,
"mgj" => MGJ,
"mgk" => MGK,
"mgl" => MGL,
"mgm" => MGM,
"mgn" => MGN,
"mgo" => MGO,
"mgp" => MGP,
"mgq" => MGQ,
"mgr" => MGR,
"mgs" => MGS,
"mgt" => MGT,
"mgu" => MGU,
"mgv" => MGV,
"mgw" => MGW,
"mgy" => MGY,
"mgz" => MGZ,
"mha" => MHA,
"mhb" => MHB,
"mhc" => MHC,
"mhd" => MHD,
"mhe" => MHE,
"mhf" => MHF,
"mhg" => MHG,
"mhi" => MHI,
"mhj" => MHJ,
"mhk" => MHK,
"mhl" => MHL,
"mhm" => MHM,
"mhn" => MHN,
"mho" => MHO,
"mhp" => MHP,
"mhq" => MHQ,
"mhr" => MHR,
"mhs" => MHS,
"mht" => MHT,
"mhu" => MHU,
"mhw" => MHW,
"mhx" => MHX,
"mhy" => MHY,
"mhz" => MHZ,
"mia" => MIA,
"mib" => MIB,
"mid" => MID,
"mie" => MIE,
"mif" => MIF,
"mig" => MIG,
"mih" => MIH,
"mii" => MII,
"mij" => MIJ,
"mik" => MIK,
"mil" => MIL,
"mim" => MIM,
"mio" => MIO,
"mip" => MIP,
"miq" => MIQ,
"mir" => MIR,
"mit" => MIT,
"miu" => MIU,
"miw" => MIW,
"mix" => MIX,
"miy" => MIY,
"miz" => MIZ,
"mjb" => MJB,
"mjc" => MJC,
"mjd" => MJD,
"mje" => MJE,
"mjg" => MJG,
"mjh" => MJH,
"mji" => MJI,
"mjj" => MJJ,
"mjk" => MJK,
"mjl" => MJL,
"mjm" => MJM,
"mjn" => MJN,
"mjo" => MJO,
"mjp" => MJP,
"mjq" => MJQ,
"mjr" => MJR,
"mjs" => MJS,
"mjt" => MJT,
"mju" => MJU,
"mjv" => MJV,
"mjw" => MJW,
"mjx" => MJX,
"mjy" => MJY,
"mjz" => MJZ,
"mka" => MKA,
"mkb" => MKB,
"mkc" => MKC,
"mke" => MKE,
"mkf" => MKF,
"mkg" => MKG,
"mki" => MKI,
"mkj" => MKJ,
"mkk" => MKK,
"mkl" => MKL,
"mkm" => MKM,
"mkn" => MKN,
"mko" => MKO,
"mkp" => MKP,
"mkq" => MKQ,
"mkr" => MKR,
"mks" => MKS,
"mkt" => MKT,
"mku" => MKU,
"mkv" => MKV,
"mkw" => MKW,
"mkx" => MKX,
"mky" => MKY,
"mkz" => MKZ,
"mla" => MLA,
"mlb" => MLB,
"mlc" => MLC,
"mle" => MLE,
"mlf" => MLF,
"mlh" => MLH,
"mli" => MLI,
"mlj" => MLJ,
"mlk" => MLK,
"mll" => MLL,
"mlm" => MLM,
"mln" => MLN,
"mlo" => MLO,
"mlp" => MLP,
"mlq" => MLQ,
"mlr" => MLR,
"mls" => MLS,
"mlu" => MLU,
"mlv" => MLV,
"mlw" => MLW,
"mlx" => MLX,
"mlz" => MLZ,
"mma" => MMA,
"mmb" => MMB,
"mmc" => MMC,
"mmd" => MMD,
"mme" => MME,
"mmf" => MMF,
"mmg" => MMG,
"mmh" => MMH,
"mmi" => MMI,
"mmj" => MMJ,
"mmk" => MMK,
"mml" => MML,
"mmm" => MMM,
"mmn" => MMN,
"mmo" => MMO,
"mmp" => MMP,
"mmq" => MMQ,
"mmr" => MMR,
"mmt" => MMT,
"mmu" => MMU,
"mmv" => MMV,
"mmw" => MMW,
"mmx" => MMX,
"mmy" => MMY,
"mmz" => MMZ,
"mna" => MNA,
"mnb" => MNB,
"mnd" => MND,
"mne" => MNE,
"mnf" => MNF,
"mng" => MNG,
"mnh" => MNH,
"mnj" => MNJ,
"mnk" => MNK,
"mnl" => MNL,
"mnm" => MNM,
"mnn" => MNN,
"mnp" => MNP,
"mnq" => MNQ,
"mnr" => MNR,
"mns" => MNS,
"mnu" => MNU,
"mnv" => MNV,
"mnw" => MNW,
"mnx" => MNX,
"mny" => MNY,
"mnz" => MNZ,
"moa" => MOA,
"moc" => MOC,
"mod" => MOD,
"moe" => MOE,
"mog" => MOG,
"moi" => MOI,
"moj" => MOJ,
"mok" => MOK,
"mom" => MOM,
"moo" => MOO,
"mop" => MOP,
"moq" => MOQ,
"mor" => MOR,
"mot" => MOT,
"mou" => MOU,
"mov" => MOV,
"mow" => MOW,
"mox" => MOX,
"moy" => MOY,
"moz" => MOZ,
"mpa" => MPA,
"mpb" => MPB,
"mpc" => MPC,
"mpd" => MPD,
"mpe" => MPE,
"mpg" => MPG,
"mph" => MPH,
"mpi" => MPI,
"mpj" => MPJ,
"mpk" => MPK,
"mpl" => MPL,
"mpm" => MPM,
"mpn" => MPN,
"mpo" => MPO,
"mpp" => MPP,
"mpq" => MPQ,
"mpr" => MPR,
"mps" => MPS,
"mpt" => MPT,
"mpu" => MPU,
"mpv" => MPV,
"mpw" => MPW,
"mpx" => MPX,
"mpy" => MPY,
"mpz" => MPZ,
"mqa" => MQA,
"mqb" => MQB,
"mqc" => MQC,
"mqe" => MQE,
"mqf" => MQF,
"mqg" => MQG,
"mqh" => MQH,
"mqi" => MQI,
"mqj" => MQJ,
"mqk" => MQK,
"mql" => MQL,
"mqm" => MQM,
"mqn" => MQN,
"mqo" => MQO,
"mqp" => MQP,
"mqq" => MQQ,
"mqr" => MQR,
"mqs" => MQS,
"mqt" => MQT,
"mqu" => MQU,
"mqv" => MQV,
"mqw" => MQW,
"mqx" => MQX,
"mqy" => MQY,
"mqz" => MQZ,
"mra" => MRA,
"mrb" => MRB,
"mrc" => MRC,
"mrd" => MRD,
"mre" => MRE,
"mrf" => MRF,
"mrg" => MRG,
"mrh" => MRH,
"mrj" => MRJ,
"mrk" => MRK,
"mrl" => MRL,
"mrm" => MRM,
"mrn" => MRN,
"mro" => MRO,
"mrp" => MRP,
"mrq" => MRQ,
"mrr" => MRR,
"mrs" => MRS,
"mrt" => MRT,
"mru" => MRU,
"mrv" => MRV,
"mrw" => MRW,
"mrx" => MRX,
"mry" => MRY,
"mrz" => MRZ,
"msb" => MSB,
"msc" => MSC,
"msd" => MSD,
"mse" => MSE,
"msf" => MSF,
"msg" => MSG,
"msh" => MSH,
"msi" => MSI,
"msj" => MSJ,
"msk" => MSK,
"msl" => MSL,
"msm" => MSM,
"msn" => MSN,
"mso" => MSO,
"msp" => MSP,
"msq" => MSQ,
"msr" => MSR,
"mss" => MSS,
"msu" => MSU,
"msv" => MSV,
"msw" => MSW,
"msx" => MSX,
"msy" => MSY,
"msz" => MSZ,
"mta" => MTA,
"mtb" => MTB,
"mtc" => MTC,
"mtd" => MTD,
"mte" => MTE,
"mtf" => MTF,
"mtg" => MTG,
"mth" => MTH,
"mti" => MTI,
"mtj" => MTJ,
"mtk" => MTK,
"mtl" => MTL,
"mtm" => MTM,
"mtn" => MTN,
"mto" => MTO,
"mtp" => MTP,
"mtq" => MTQ,
"mtr" => MTR,
"mts" => MTS,
"mtt" => MTT,
"mtu" => MTU,
"mtv" => MTV,
"mtw" => MTW,
"mtx" => MTX,
"mty" => MTY,
"mua" => MUA,
"mub" => MUB,
"muc" => MUC,
"mud" => MUD,
"mue" => MUE,
"mug" => MUG,
"muh" => MUH,
"mui" => MUI,
"muj" => MUJ,
"muk" => MUK,
"mum" => MUM,
"muo" => MUO,
"mup" => MUP,
"muq" => MUQ,
"mur" => MUR,
"mut" => MUT,
"muu" => MUU,
"muv" => MUV,
"mux" => MUX,
"muy" => MUY,
"muz" => MUZ,
"mva" => MVA,
"mvb" => MVB,
"mvd" => MVD,
"mve" => MVE,
"mvf" => MVF,
"mvg" => MVG,
"mvh" => MVH,
"mvi" => MVI,
"mvk" => MVK,
"mvl" => MVL,
"mvn" => MVN,
"mvo" => MVO,
"mvp" => MVP,
"mvq" => MVQ,
"mvr" => MVR,
"mvs" => MVS,
"mvt" => MVT,
"mvu" => MVU,
"mvv" => MVV,
"mvw" => MVW,
"mvx" => MVX,
"mvy" => MVY,
"mvz" => MVZ,
"mwa" => MWA,
"mwb" => MWB,
"mwc" => MWC,
"mwe" => MWE,
"mwf" => MWF,
"mwg" => MWG,
"mwh" => MWH,
"mwi" => MWI,
"mwk" => MWK,
"mwm" => MWM,
"mwn" => MWN,
"mwo" => MWO,
"mwp" => MWP,
"mwq" => MWQ,
"mws" => MWS,
"mwt" => MWT,
"mwu" => MWU,
"mwv" => MWV,
"mww" => MWW,
"mwz" => MWZ,
"mxa" => MXA,
"mxb" => MXB,
"mxc" => MXC,
"mxd" => MXD,
"mxe" => MXE,
"mxf" => MXF,
"mxg" => MXG,
"mxh" => MXH,
"mxi" => MXI,
"mxj" => MXJ,
"mxk" => MXK,
"mxl" => MXL,
"mxm" => MXM,
"mxn" => MXN,
"mxo" => MXO,
"mxp" => MXP,
"mxq" => MXQ,
"mxr" => MXR,
"mxs" => MXS,
"mxt" => MXT,
"mxu" => MXU,
"mxv" => MXV,
"mxw" => MXW,
"mxx" => MXX,
"mxy" => MXY,
"mxz" => MXZ,
"myb" => MYB,
"myc" => MYC,
"mye" => MYE,
"myf" => MYF,
"myg" => MYG,
"myh" => MYH,
"myj" => MYJ,
"myk" => MYK,
"myl" => MYL,
"mym" => MYM,
"myo" => MYO,
"myp" => MYP,
"myr" => MYR,
"mys" => MYS,
"myu" => MYU,
"myw" => MYW,
"myx" => MYX,
"myy" => MYY,
"myz" => MYZ,
"mza" => MZA,
"mzb" => MZB,
"mzc" => MZC,
"mzd" => MZD,
"mze" => MZE,
"mzg" => MZG,
"mzh" => MZH,
"mzi" => MZI,
"mzj" => MZJ,
"mzk" => MZK,
"mzl" => MZL,
"mzm" => MZM,
"mzn" => MZN,
"mzo" => MZO,
"mzp" => MZP,
"mzq" => MZQ,
"mzr" => MZR,
"mzs" => MZS,
"mzt" => MZT,
"mzu" => MZU,
"mzv" => MZV,
"mzw" => MZW,
"mzx" => MZX,
"mzy" => MZY,
"mzz" => MZZ,
"naa" => NAA,
"nab" => NAB,
"nac" => NAC,
"nae" => NAE,
"naf" => NAF,
"nag" => NAG,
"naj" => NAJ,
"nak" => NAK,
"nal" => NAL,
"nam" => NAM,
"nan" => NAN,
"nao" => NAO,
"naq" => NAQ,
"nar" => NAR,
"nas" => NAS,
"nat" => NAT,
"naw" => NAW,
"nax" => NAX,
"nay" => NAY,
"naz" => NAZ,
"nba" => NBA,
"nbb" => NBB,
"nbc" => NBC,
"nbd" => NBD,
"nbe" => NBE,
"nbg" => NBG,
"nbh" => NBH,
"nbi" => NBI,
"nbj" => NBJ,
"nbk" => NBK,
"nbm" => NBM,
"nbn" => NBN,
"nbo" => NBO,
"nbp" => NBP,
"nbq" => NBQ,
"nbr" => NBR,
"nbs" => NBS,
"nbt" => NBT,
"nbu" => NBU,
"nbv" => NBV,
"nbw" => NBW,
"nby" => NBY,
"nca" => NCA,
"ncb" => NCB,
"ncc" => NCC,
"ncd" => NCD,
"nce" => NCE,
"ncf" => NCF,
"ncg" => NCG,
"nch" => NCH,
"nci" => NCI,
"ncj" => NCJ,
"nck" => NCK,
"ncl" => NCL,
"ncm" => NCM,
"ncn" => NCN,
"nco" => NCO,
"ncq" => NCQ,
"ncr" => NCR,
"ncs" => NCS,
"nct" => NCT,
"ncu" => NCU,
"ncx" => NCX,
"ncz" => NCZ,
"nda" => NDA,
"ndb" => NDB,
"ndc" => NDC,
"ndd" => NDD,
"ndf" => NDF,
"ndg" => NDG,
"ndh" => NDH,
"ndi" => NDI,
"ndj" => NDJ,
"ndk" => NDK,
"ndl" => NDL,
"ndm" => NDM,
"ndn" => NDN,
"ndp" => NDP,
"ndq" => NDQ,
"ndr" => NDR,
"ndt" => NDT,
"ndu" => NDU,
"ndv" => NDV,
"ndw" => NDW,
"ndx" => NDX,
"ndy" => NDY,
"ndz" => NDZ,
"nea" => NEA,
"neb" => NEB,
"nec" => NEC,
"ned" => NED,
"nee" => NEE,
"nef" => NEF,
"neg" => NEG,
"neh" => NEH,
"nei" => NEI,
"nej" => NEJ,
"nek" => NEK,
"nem" => NEM,
"nen" => NEN,
"neo" => NEO,
"neq" => NEQ,
"ner" => NER,
"nes" => NES,
"net" => NET,
"neu" => NEU,
"nev" => NEV,
"nex" => NEX,
"ney" => NEY,
"nez" => NEZ,
"nfa" => NFA,
"nfd" => NFD,
"nfl" => NFL,
"nfr" => NFR,
"nfu" => NFU,
"nga" => NGA,
"ngb" => NGB,
"ngc" => NGC,
"ngd" => NGD,
"nge" => NGE,
"ngg" => NGG,
"ngh" => NGH,
"ngi" => NGI,
"ngj" => NGJ,
"ngk" => NGK,
"ngl" => NGL,
"ngm" => NGM,
"ngn" => NGN,
"ngp" => NGP,
"ngq" => NGQ,
"ngr" => NGR,
"ngs" => NGS,
"ngt" => NGT,
"ngu" => NGU,
"ngv" => NGV,
"ngw" => NGW,
"ngx" => NGX,
"ngy" => NGY,
"ngz" => NGZ,
"nha" => NHA,
"nhb" => NHB,
"nhc" => NHC,
"nhd" => NHD,
"nhe" => NHE,
"nhf" => NHF,
"nhg" => NHG,
"nhh" => NHH,
"nhi" => NHI,
"nhk" => NHK,
"nhm" => NHM,
"nhn" => NHN,
"nho" => NHO,
"nhp" => NHP,
"nhq" => NHQ,
"nhr" => NHR,
"nht" => NHT,
"nhu" => NHU,
"nhv" => NHV,
"nhw" => NHW,
"nhx" => NHX,
"nhy" => NHY,
"nhz" => NHZ,
"nib" => NIB,
"nid" => NID,
"nie" => NIE,
"nif" => NIF,
"nig" => NIG,
"nih" => NIH,
"nii" => NII,
"nij" => NIJ,
"nik" => NIK,
"nil" => NIL,
"nim" => NIM,
"nin" => NIN,
"nio" => NIO,
"niq" => NIQ,
"nir" => NIR,
"nis" => NIS,
"nit" => NIT,
"niv" => NIV,
"niw" => NIW,
"nix" => NIX,
"niy" => NIY,
"niz" => NIZ,
"nja" => NJA,
"njb" => NJB,
"njd" => NJD,
"njh" => NJH,
"nji" => NJI,
"njj" => NJJ,
"njl" => NJL,
"njm" => NJM,
"njn" => NJN,
"njo" => NJO,
"njr" => NJR,
"njs" => NJS,
"njt" => NJT,
"nju" => NJU,
"njx" => NJX,
"njy" => NJY,
"njz" => NJZ,
"nka" => NKA,
"nkb" => NKB,
"nkc" => NKC,
"nkd" => NKD,
"nke" => NKE,
"nkf" => NKF,
"nkg" => NKG,
"nkh" => NKH,
"nki" => NKI,
"nkj" => NKJ,
"nkk" => NKK,
"nkm" => NKM,
"nkn" => NKN,
"nko" => NKO,
"nkp" => NKP,
"nkq" => NKQ,
"nkr" => NKR,
"nks" => NKS,
"nkt" => NKT,
"nku" => NKU,
"nkv" => NKV,
"nkw" => NKW,
"nkx" => NKX,
"nkz" => NKZ,
"nla" => NLA,
"nlc" => NLC,
"nle" => NLE,
"nlg" => NLG,
"nli" => NLI,
"nlj" => NLJ,
"nlk" => NLK,
"nll" => NLL,
"nlm" => NLM,
"nlo" => NLO,
"nlq" => NLQ,
"nlu" => NLU,
"nlv" => NLV,
"nlw" => NLW,
"nlx" => NLX,
"nly" => NLY,
"nlz" => NLZ,
"nma" => NMA,
"nmb" => NMB,
"nmc" => NMC,
"nmd" => NMD,
"nme" => NME,
"nmf" => NMF,
"nmg" => NMG,
"nmh" => NMH,
"nmi" => NMI,
"nmj" => NMJ,
"nmk" => NMK,
"nml" => NML,
"nmm" => NMM,
"nmn" => NMN,
"nmo" => NMO,
"nmp" => NMP,
"nmq" => NMQ,
"nmr" => NMR,
"nms" => NMS,
"nmt" => NMT,
"nmu" => NMU,
"nmv" => NMV,
"nmw" => NMW,
"nmx" => NMX,
"nmy" => NMY,
"nmz" => NMZ,
"nna" => NNA,
"nnb" => NNB,
"nnc" => NNC,
"nnd" => NND,
"nne" => NNE,
"nnf" => NNF,
"nng" => NNG,
"nnh" => NNH,
"nni" => NNI,
"nnj" => NNJ,
"nnk" => NNK,
"nnl" => NNL,
"nnm" => NNM,
"nnn" => NNN,
"nnp" => NNP,
"nnq" => NNQ,
"nnr" => NNR,
"nnt" => NNT,
"nnu" => NNU,
"nnv" => NNV,
"nnw" => NNW,
"nny" => NNY,
"nnz" => NNZ,
"noa" => NOA,
"noc" => NOC,
"nod" => NOD,
"noe" => NOE,
"nof" => NOF,
"noh" => NOH,
"noi" => NOI,
"noj" => NOJ,
"nok" => NOK,
"nol" => NOL,
"nom" => NOM,
"nop" => NOP,
"noq" => NOQ,
"nos" => NOS,
"not" => NOT,
"nou" => NOU,
"nov" => NOV,
"now" => NOW,
"noy" => NOY,
"noz" => NOZ,
"npa" => NPA,
"npb" => NPB,
"npg" => NPG,
"nph" => NPH,
"npi" => NPI,
"npl" => NPL,
"npn" => NPN,
"npo" => NPO,
"nps" => NPS,
"npu" => NPU,
"npx" => NPX,
"npy" => NPY,
"nqg" => NQG,
"nqk" => NQK,
"nql" => NQL,
"nqm" => NQM,
"nqn" => NQN,
"nqq" => NQQ,
"nqt" => NQT,
"nqy" => NQY,
"nra" => NRA,
"nrb" => NRB,
"nrc" => NRC,
"nre" => NRE,
"nrf" => NRF,
"nrg" => NRG,
"nri" => NRI,
"nrk" => NRK,
"nrl" => NRL,
"nrm" => NRM,
"nrn" => NRN,
"nrp" => NRP,
"nrr" => NRR,
"nrt" => NRT,
"nru" => NRU,
"nrx" => NRX,
"nrz" => NRZ,
"nsa" => NSA,
"nsb" => NSB,
"nsc" => NSC,
"nsd" => NSD,
"nse" => NSE,
"nsf" => NSF,
"nsg" => NSG,
"nsh" => NSH,
"nsi" => NSI,
"nsk" => NSK,
"nsl" => NSL,
"nsm" => NSM,
"nsn" => NSN,
"nsp" => NSP,
"nsq" => NSQ,
"nsr" => NSR,
"nss" => NSS,
"nst" => NST,
"nsu" => NSU,
"nsv" => NSV,
"nsw" => NSW,
"nsx" => NSX,
"nsy" => NSY,
"nsz" => NSZ,
"ntd" => NTD,
"nte" => NTE,
"ntg" => NTG,
"nti" => NTI,
"ntj" => NTJ,
"ntk" => NTK,
"ntm" => NTM,
"nto" => NTO,
"ntp" => NTP,
"ntr" => NTR,
"ntu" => NTU,
"ntw" => NTW,
"ntx" => NTX,
"nty" => NTY,
"ntz" => NTZ,
"nua" => NUA,
"nuc" => NUC,
"nud" => NUD,
"nue" => NUE,
"nuf" => NUF,
"nug" => NUG,
"nuh" => NUH,
"nui" => NUI,
"nuj" => NUJ,
"nuk" => NUK,
"nul" => NUL,
"num" => NUM,
"nun" => NUN,
"nuo" => NUO,
"nup" => NUP,
"nuq" => NUQ,
"nur" => NUR,
"nus" => NUS,
"nut" => NUT,
"nuu" => NUU,
"nuv" => NUV,
"nuw" => NUW,
"nux" => NUX,
"nuy" => NUY,
"nuz" => NUZ,
"nvh" => NVH,
"nvm" => NVM,
"nvo" => NVO,
"nwa" => NWA,
"nwb" => NWB,
"nwe" => NWE,
"nwg" => NWG,
"nwi" => NWI,
"nwm" => NWM,
"nwo" => NWO,
"nwr" => NWR,
"nww" => NWW,
"nwx" => NWX,
"nwy" => NWY,
"nxa" => NXA,
"nxd" => NXD,
"nxe" => NXE,
"nxg" => NXG,
"nxi" => NXI,
"nxk" => NXK,
"nxl" => NXL,
"nxm" => NXM,
"nxn" => NXN,
"nxo" => NXO,
"nxq" => NXQ,
"nxr" => NXR,
"nxx" => NXX,
"nyb" => NYB,
"nyc" => NYC,
"nyd" => NYD,
"nye" => NYE,
"nyf" => NYF,
"nyg" => NYG,
"nyh" => NYH,
"nyi" => NYI,
"nyj" => NYJ,
"nyk" => NYK,
"nyl" => NYL,
"nyp" => NYP,
"nyq" => NYQ,
"nyr" => NYR,
"nys" => NYS,
"nyt" => NYT,
"nyu" => NYU,
"nyv" => NYV,
"nyw" => NYW,
"nyx" => NYX,
"nyy" => NYY,
"nza" => NZA,
"nzb" => NZB,
"nzd" => NZD,
"nzk" => NZK,
"nzm" => NZM,
"nzs" => NZS,
"nzu" => NZU,
"nzy" => NZY,
"nzz" => NZZ,
"oaa" => OAA,
"oac" => OAC,
"oar" => OAR,
"oav" => OAV,
"obi" => OBI,
"obk" => OBK,
"obl" => OBL,
"obm" => OBM,
"obo" => OBO,
"obr" => OBR,
"obt" => OBT,
"obu" => OBU,
"oca" => OCA,
"och" => OCH,
"ocm" => OCM,
"oco" => OCO,
"ocu" => OCU,
"oda" => ODA,
"odk" => ODK,
"odt" => ODT,
"odu" => ODU,
"ofo" => OFO,
"ofs" => OFS,
"ofu" => OFU,
"ogb" => OGB,
"ogc" => OGC,
"oge" => OGE,
"ogg" => OGG,
"ogo" => OGO,
"ogu" => OGU,
"oht" => OHT,
"ohu" => OHU,
"oia" => OIA,
"oie" => OIE,
"oin" => OIN,
"ojb" => OJB,
"ojc" => OJC,
"ojg" => OJG,
"ojp" => OJP,
"ojs" => OJS,
"ojv" => OJV,
"ojw" => OJW,
"oka" => OKA,
"okb" => OKB,
"okc" => OKC,
"okd" => OKD,
"oke" => OKE,
"okg" => OKG,
"okh" => OKH,
"oki" => OKI,
"okj" => OKJ,
"okk" => OKK,
"okl" => OKL,
"okm" => OKM,
"okn" => OKN,
"oko" => OKO,
"okr" => OKR,
"oks" => OKS,
"oku" => OKU,
"okv" => OKV,
"okx" => OKX,
"okz" => OKZ,
"ola" => OLA,
"old" => OLD,
"ole" => OLE,
"olk" => OLK,
"olm" => OLM,
"olo" => OLO,
"olr" => OLR,
"olt" => OLT,
"olu" => OLU,
"oma" => OMA,
"omb" => OMB,
"omc" => OMC,
"omg" => OMG,
"omi" => OMI,
"omk" => OMK,
"oml" => OML,
"omn" => OMN,
"omo" => OMO,
"omp" => OMP,
"omr" => OMR,
"omt" => OMT,
"omu" => OMU,
"omw" => OMW,
"omx" => OMX,
"omy" => OMY,
"ona" => ONA,
"onb" => ONB,
"one" => ONE,
"ong" => ONG,
"oni" => ONI,
"onj" => ONJ,
"onk" => ONK,
"onn" => ONN,
"ono" => ONO,
"onp" => ONP,
"onr" => ONR,
"ons" => ONS,
"ont" => ONT,
"onu" => ONU,
"onw" => ONW,
"onx" => ONX,
"ood" => OOD,
"oog" => OOG,
"oon" => OON,
"oor" => OOR,
"oos" => OOS,
"opa" => OPA,
"opk" => OPK,
"opm" => OPM,
"opo" => OPO,
"opt" => OPT,
"opy" => OPY,
"ora" => ORA,
"orc" => ORC,
"ore" => ORE,
"org" => ORG,
"orh" => ORH,
"orn" => ORN,
"oro" => ORO,
"orr" => ORR,
"ors" => ORS,
"ort" => ORT,
"oru" => ORU,
"orv" => ORV,
"orw" => ORW,
"orx" => ORX,
"ory" => ORY,
"orz" => ORZ,
"osc" => OSC,
"osi" => OSI,
"osn" => OSN,
"oso" => OSO,
"osp" => OSP,
"ost" => OST,
"osu" => OSU,
"osx" => OSX,
"otb" => OTB,
"otd" => OTD,
"ote" => OTE,
"oti" => OTI,
"otk" => OTK,
"otl" => OTL,
"otm" => OTM,
"otn" => OTN,
"otq" => OTQ,
"otr" => OTR,
"ots" => OTS,
"ott" => OTT,
"otu" => OTU,
"otw" => OTW,
"otx" => OTX,
"oty" => OTY,
"otz" => OTZ,
"oua" => OUA,
"oub" => OUB,
"oue" => OUE,
"oui" => OUI,
"oum" => OUM,
"ovd" => OVD,
"owi" => OWI,
"owl" => OWL,
"oyb" => OYB,
"oyd" => OYD,
"oym" => OYM,
"oyy" => OYY,
"ozm" => OZM,
"pab" => PAB,
"pac" => PAC,
"pad" => PAD,
"pae" => PAE,
"paf" => PAF,
"pah" => PAH,
"pai" => PAI,
"pak" => PAK,
"pao" => PAO,
"paq" => PAQ,
"par" => PAR,
"pas" => PAS,
"pav" => PAV,
"paw" => PAW,
"pax" => PAX,
"pay" => PAY,
"paz" => PAZ,
"pbb" => PBB,
"pbc" => PBC,
"pbe" => PBE,
"pbf" => PBF,
"pbg" => PBG,
"pbh" => PBH,
"pbi" => PBI,
"pbl" => PBL,
"pbm" => PBM,
"pbn" => PBN,
"pbo" => PBO,
"pbp" => PBP,
"pbr" => PBR,
"pbs" => PBS,
"pbt" => PBT,
"pbu" => PBU,
"pbv" => PBV,
"pby" => PBY,
"pca" => PCA,
"pcb" => PCB,
"pcc" => PCC,
"pcd" => PCD,
"pce" => PCE,
"pcf" => PCF,
"pcg" => PCG,
"pch" => PCH,
"pci" => PCI,
"pcj" => PCJ,
"pck" => PCK,
"pcl" => PCL,
"pcm" => PCM,
"pcn" => PCN,
"pcp" => PCP,
"pcw" => PCW,
"pda" => PDA,
"pdc" => PDC,
"pdi" => PDI,
"pdn" => PDN,
"pdo" => PDO,
"pdt" => PDT,
"pdu" => PDU,
"pea" => PEA,
"peb" => PEB,
"ped" => PED,
"pee" => PEE,
"pef" => PEF,
"peg" => PEG,
"peh" => PEH,
"pei" => PEI,
"pej" => PEJ,
"pek" => PEK,
"pel" => PEL,
"pem" => PEM,
"pep" => PEP,
"peq" => PEQ,
"pes" => PES,
"pev" => PEV,
"pex" => PEX,
"pey" => PEY,
"pez" => PEZ,
"pfa" => PFA,
"pfe" => PFE,
"pfl" => PFL,
"pga" => PGA,
"pgd" => PGD,
"pgg" => PGG,
"pgi" => PGI,
"pgk" => PGK,
"pgl" => PGL,
"pgn" => PGN,
"pgs" => PGS,
"pgu" => PGU,
"pgz" => PGZ,
"pha" => PHA,
"phd" => PHD,
"phg" => PHG,
"phh" => PHH,
"phj" => PHJ,
"phk" => PHK,
"phl" => PHL,
"phm" => PHM,
"pho" => PHO,
"phq" => PHQ,
"phr" => PHR,
"pht" => PHT,
"phu" => PHU,
"phv" => PHV,
"phw" => PHW,
"pia" => PIA,
"pib" => PIB,
"pic" => PIC,
"pid" => PID,
"pie" => PIE,
"pif" => PIF,
"pig" => PIG,
"pih" => PIH,
"pij" => PIJ,
"pil" => PIL,
"pim" => PIM,
"pin" => PIN,
"pio" => PIO,
"pip" => PIP,
"pir" => PIR,
"pis" => PIS,
"pit" => PIT,
"piu" => PIU,
"piv" => PIV,
"piw" => PIW,
"pix" => PIX,
"piy" => PIY,
"piz" => PIZ,
"pjt" => PJT,
"pka" => PKA,
"pkb" => PKB,
"pkc" => PKC,
"pkg" => PKG,
"pkh" => PKH,
"pkn" => PKN,
"pko" => PKO,
"pkp" => PKP,
"pkr" => PKR,
"pks" => PKS,
"pkt" => PKT,
"pku" => PKU,
"pla" => PLA,
"plb" => PLB,
"plc" => PLC,
"pld" => PLD,
"ple" => PLE,
"plg" => PLG,
"plh" => PLH,
"plj" => PLJ,
"plk" => PLK,
"pll" => PLL,
"pln" => PLN,
"plo" => PLO,
"plq" => PLQ,
"plr" => PLR,
"pls" => PLS,
"plt" => PLT,
"plu" => PLU,
"plv" => PLV,
"plw" => PLW,
"ply" => PLY,
"plz" => PLZ,
"pma" => PMA,
"pmb" => PMB,
"pmd" => PMD,
"pme" => PME,
"pmf" => PMF,
"pmh" => PMH,
"pmi" => PMI,
"pmj" => PMJ,
"pmk" => PMK,
"pml" => PML,
"pmm" => PMM,
"pmn" => PMN,
"pmo" => PMO,
"pmq" => PMQ,
"pmr" => PMR,
"pms" => PMS,
"pmt" => PMT,
"pmw" => PMW,
"pmx" => PMX,
"pmy" => PMY,
"pmz" => PMZ,
"pna" => PNA,
"pnb" => PNB,
"pnc" => PNC,
"pnd" => PND,
"pne" => PNE,
"png" => PNG,
"pnh" => PNH,
"pni" => PNI,
"pnj" => PNJ,
"pnk" => PNK,
"pnl" => PNL,
"pnm" => PNM,
"pnn" => PNN,
"pno" => PNO,
"pnp" => PNP,
"pnq" => PNQ,
"pnr" => PNR,
"pns" => PNS,
"pnt" => PNT,
"pnu" => PNU,
"pnv" => PNV,
"pnw" => PNW,
"pnx" => PNX,
"pny" => PNY,
"pnz" => PNZ,
"poc" => POC,
"poe" => POE,
"pof" => POF,
"pog" => POG,
"poh" => POH,
"poi" => POI,
"pok" => POK,
"pom" => POM,
"poo" => POO,
"pop" => POP,
"poq" => POQ,
"pos" => POS,
"pot" => POT,
"pov" => POV,
"pow" => POW,
"pox" => POX,
"poy" => POY,
"ppe" => PPE,
"ppi" => PPI,
"ppk" => PPK,
"ppl" => PPL,
"ppm" => PPM,
"ppn" => PPN,
"ppo" => PPO,
"ppp" => PPP,
"ppq" => PPQ,
"pps" => PPS,
"ppt" => PPT,
"ppu" => PPU,
"pqa" => PQA,
"pqm" => PQM,
"prc" => PRC,
"prd" => PRD,
"pre" => PRE,
"prf" => PRF,
"prg" => PRG,
"prh" => PRH,
"pri" => PRI,
"prk" => PRK,
"prl" => PRL,
"prm" => PRM,
"prn" => PRN,
"prp" => PRP,
"prq" => PRQ,
"prr" => PRR,
"prs" => PRS,
"prt" => PRT,
"pru" => PRU,
"prw" => PRW,
"prx" => PRX,
"prz" => PRZ,
"psa" => PSA,
"psc" => PSC,
"psd" => PSD,
"pse" => PSE,
"psg" => PSG,
"psh" => PSH,
"psi" => PSI,
"psl" => PSL,
"psm" => PSM,
"psn" => PSN,
"pso" => PSO,
"psp" => PSP,
"psq" => PSQ,
"psr" => PSR,
"pss" => PSS,
"pst" => PST,
"psu" => PSU,
"psw" => PSW,
"psy" => PSY,
"pta" => PTA,
"pth" => PTH,
"pti" => PTI,
"ptn" => PTN,
"pto" => PTO,
"ptp" => PTP,
"ptq" => PTQ,
"ptr" => PTR,
"ptt" => PTT,
"ptu" => PTU,
"ptv" => PTV,
"ptw" => PTW,
"pty" => PTY,
"pua" => PUA,
"pub" => PUB,
"puc" => PUC,
"pud" => PUD,
"pue" => PUE,
"puf" => PUF,
"pug" => PUG,
"pui" => PUI,
"puj" => PUJ,
"pum" => PUM,
"puo" => PUO,
"pup" => PUP,
"puq" => PUQ,
"pur" => PUR,
"put" => PUT,
"puu" => PUU,
"puw" => PUW,
"pux" => PUX,
"puy" => PUY,
"pwa" => PWA,
"pwb" => PWB,
"pwg" => PWG,
"pwi" => PWI,
"pwm" => PWM,
"pwn" => PWN,
"pwo" => PWO,
"pwr" => PWR,
"pww" => PWW,
"pxm" => PXM,
"pye" => PYE,
"pym" => PYM,
"pyn" => PYN,
"pys" => PYS,
"pyu" => PYU,
"pyx" => PYX,
"pyy" => PYY,
"pzh" => PZH,
"pzn" => PZN,
"qua" => QUA,
"qub" => QUB,
"quc" => QUC,
"qud" => QUD,
"quf" => QUF,
"qug" => QUG,
"quh" => QUH,
"qui" => QUI,
"quk" => QUK,
"qul" => QUL,
"qum" => QUM,
"qun" => QUN,
"qup" => QUP,
"quq" => QUQ,
"qur" => QUR,
"qus" => QUS,
"quv" => QUV,
"quw" => QUW,
"qux" => QUX,
"quy" => QUY,
"quz" => QUZ,
"qva" => QVA,
"qvc" => QVC,
"qve" => QVE,
"qvh" => QVH,
"qvi" => QVI,
"qvj" => QVJ,
"qvl" => QVL,
"qvm" => QVM,
"qvn" => QVN,
"qvo" => QVO,
"qvp" => QVP,
"qvs" => QVS,
"qvw" => QVW,
"qvy" => QVY,
"qvz" => QVZ,
"qwa" => QWA,
"qwc" => QWC,
"qwh" => QWH,
"qwm" => QWM,
"qws" => QWS,
"qwt" => QWT,
"qxa" => QXA,
"qxc" => QXC,
"qxh" => QXH,
"qxl" => QXL,
"qxn" => QXN,
"qxo" => QXO,
"qxp" => QXP,
"qxq" => QXQ,
"qxr" => QXR,
"qxs" => QXS,
"qxt" => QXT,
"qxu" => QXU,
"qxw" => QXW,
"qya" => QYA,
"qyp" => QYP,
"raa" => RAA,
"rab" => RAB,
"rac" => RAC,
"rad" => RAD,
"raf" => RAF,
"rag" => RAG,
"rah" => RAH,
"rai" => RAI,
"rak" => RAK,
"ral" => RAL,
"ram" => RAM,
"ran" => RAN,
"rao" => RAO,
"raq" => RAQ,
"ras" => RAS,
"rat" => RAT,
"rau" => RAU,
"rav" => RAV,
"raw" => RAW,
"rax" => RAX,
"ray" => RAY,
"raz" => RAZ,
"rbb" => RBB,
"rbk" => RBK,
"rbl" => RBL,
"rbp" => RBP,
"rcf" => RCF,
"rdb" => RDB,
"rea" => REA,
"reb" => REB,
"ree" => REE,
"reg" => REG,
"rei" => REI,
"rej" => REJ,
"rel" => REL,
"rem" => REM,
"ren" => REN,
"rer" => RER,
"res" => RES,
"ret" => RET,
"rey" => REY,
"rga" => RGA,
"rge" => RGE,
"rgk" => RGK,
"rgn" => RGN,
"rgr" => RGR,
"rgs" => RGS,
"rgu" => RGU,
"rhg" => RHG,
"rhp" => RHP,
"ria" => RIA,
"rib" => RIB,
"rif" => RIF,
"ril" => RIL,
"rim" => RIM,
"rin" => RIN,
"rir" => RIR,
"rit" => RIT,
"riu" => RIU,
"rjg" => RJG,
"rji" => RJI,
"rjs" => RJS,
"rka" => RKA,
"rkb" => RKB,
"rkh" => RKH,
"rki" => RKI,
"rkm" => RKM,
"rkt" => RKT,
"rkw" => RKW,
"rma" => RMA,
"rmb" => RMB,
"rmc" => RMC,
"rmd" => RMD,
"rme" => RME,
"rmf" => RMF,
"rmg" => RMG,
"rmh" => RMH,
"rmi" => RMI,
"rmk" => RMK,
"rml" => RML,
"rmm" => RMM,
"rmn" => RMN,
"rmo" => RMO,
"rmp" => RMP,
"rmq" => RMQ,
"rms" => RMS,
"rmt" => RMT,
"rmu" => RMU,
"rmv" => RMV,
"rmw" => RMW,
"rmx" => RMX,
"rmy" => RMY,
"rmz" => RMZ,
"rnb" => RNB,
"rnd" => RND,
"rng" => RNG,
"rnl" => RNL,
"rnn" => RNN,
"rnp" => RNP,
"rnr" => RNR,
"rnw" => RNW,
"rob" => ROB,
"roc" => ROC,
"rod" => ROD,
"roe" => ROE,
"rof" => ROF,
"rog" => ROG,
"rol" => ROL,
"roo" => ROO,
"rop" => ROP,
"ror" => ROR,
"rou" => ROU,
"row" => ROW,
"rpn" => RPN,
"rpt" => RPT,
"rri" => RRI,
"rro" => RRO,
"rrt" => RRT,
"rsb" => RSB,
"rsk" => RSK,
"rsl" => RSL,
"rsm" => RSM,
"rsn" => RSN,
"rtc" => RTC,
"rth" => RTH,
"rtm" => RTM,
"rts" => RTS,
"rtw" => RTW,
"rub" => RUB,
"ruc" => RUC,
"rue" => RUE,
"ruf" => RUF,
"rug" => RUG,
"ruh" => RUH,
"rui" => RUI,
"ruk" => RUK,
"ruo" => RUO,
"ruq" => RUQ,
"rut" => RUT,
"ruu" => RUU,
"ruy" => RUY,
"ruz" => RUZ,
"rwa" => RWA,
"rwk" => RWK,
"rwl" => RWL,
"rwm" => RWM,
"rwo" => RWO,
"rwr" => RWR,
"rxd" => RXD,
"rxw" => RXW,
"ryn" => RYN,
"rys" => RYS,
"ryu" => RYU,
"rzh" => RZH,
"saa" => SAA,
"sab" => SAB,
"sac" => SAC,
"sae" => SAE,
"saf" => SAF,
"saj" => SAJ,
"sak" => SAK,
"sao" => SAO,
"saq" => SAQ,
"sar" => SAR,
"sau" => SAU,
"sav" => SAV,
"saw" => SAW,
"sax" => SAX,
"say" => SAY,
"saz" => SAZ,
"sba" => SBA,
"sbb" => SBB,
"sbc" => SBC,
"sbd" => SBD,
"sbe" => SBE,
"sbf" => SBF,
"sbg" => SBG,
"sbh" => SBH,
"sbi" => SBI,
"sbj" => SBJ,
"sbk" => SBK,
"sbl" => SBL,
"sbm" => SBM,
"sbn" => SBN,
"sbo" => SBO,
"sbp" => SBP,
"sbq" => SBQ,
"sbr" => SBR,
"sbs" => SBS,
"sbt" => SBT,
"sbu" => SBU,
"sbv" => SBV,
"sbw" => SBW,
"sbx" => SBX,
"sby" => SBY,
"sbz" => SBZ,
"scb" => SCB,
"sce" => SCE,
"scf" => SCF,
"scg" => SCG,
"sch" => SCH,
"sci" => SCI,
"sck" => SCK,
"scl" => SCL,
"scp" => SCP,
"scq" => SCQ,
"scs" => SCS,
"sct" => SCT,
"scu" => SCU,
"scv" => SCV,
"scw" => SCW,
"scx" => SCX,
"sda" => SDA,
"sdb" => SDB,
"sdc" => SDC,
"sde" => SDE,
"sdf" => SDF,
"sdg" => SDG,
"sdh" => SDH,
"sdj" => SDJ,
"sdk" => SDK,
"sdl" => SDL,
"sdn" => SDN,
"sdo" => SDO,
"sdp" => SDP,
"sdq" => SDQ,
"sdr" => SDR,
"sds" => SDS,
"sdt" => SDT,
"sdu" => SDU,
"sdx" => SDX,
"sdz" => SDZ,
"sea" => SEA,
"seb" => SEB,
"sec" => SEC,
"sed" => SED,
"see" => SEE,
"sef" => SEF,
"seg" => SEG,
"seh" => SEH,
"sei" => SEI,
"sej" => SEJ,
"sek" => SEK,
"sen" => SEN,
"seo" => SEO,
"sep" => SEP,
"seq" => SEQ,
"ser" => SER,
"ses" => SES,
"set" => SET,
"seu" => SEU,
"sev" => SEV,
"sew" => SEW,
"sey" => SEY,
"sez" => SEZ,
"sfb" => SFB,
"sfe" => SFE,
"sfm" => SFM,
"sfs" => SFS,
"sfw" => SFW,
"sgb" => SGB,
"sgc" => SGC,
"sgd" => SGD,
"sge" => SGE,
"sgg" => SGG,
"sgh" => SGH,
"sgi" => SGI,
"sgj" => SGJ,
"sgk" => SGK,
"sgm" => SGM,
"sgp" => SGP,
"sgr" => SGR,
"sgs" => SGS,
"sgt" => SGT,
"sgu" => SGU,
"sgw" => SGW,
"sgx" => SGX,
"sgy" => SGY,
"sgz" => SGZ,
"sha" => SHA,
"shb" => SHB,
"shc" => SHC,
"shd" => SHD,
"she" => SHE,
"shg" => SHG,
"shh" => SHH,
"shi" => SHI,
"shj" => SHJ,
"shk" => SHK,
"shl" => SHL,
"shm" => SHM,
"sho" => SHO,
"shp" => SHP,
"shq" => SHQ,
"shr" => SHR,
"shs" => SHS,
"sht" => SHT,
"shu" => SHU,
"shv" => SHV,
"shw" => SHW,
"shx" => SHX,
"shy" => SHY,
"shz" => SHZ,
"sia" => SIA,
"sib" => SIB,
"sie" => SIE,
"sif" => SIF,
"sig" => SIG,
"sih" => SIH,
"sii" => SII,
"sij" => SIJ,
"sik" => SIK,
"sil" => SIL,
"sim" => SIM,
"sip" => SIP,
"siq" => SIQ,
"sir" => SIR,
"sis" => SIS,
"siu" => SIU,
"siv" => SIV,
"siw" => SIW,
"six" => SIX,
"siy" => SIY,
"siz" => SIZ,
"sja" => SJA,
"sjb" => SJB,
"sjd" => SJD,
"sje" => SJE,
"sjg" => SJG,
"sjk" => SJK,
"sjl" => SJL,
"sjm" => SJM,
"sjn" => SJN,
"sjo" => SJO,
"sjp" => SJP,
"sjr" => SJR,
"sjs" => SJS,
"sjt" => SJT,
"sju" => SJU,
"sjw" => SJW,
"ska" => SKA,
"skb" => SKB,
"skc" => SKC,
"skd" => SKD,
"ske" => SKE,
"skf" => SKF,
"skg" => SKG,
"skh" => SKH,
"ski" => SKI,
"skj" => SKJ,
"skm" => SKM,
"skn" => SKN,
"sko" => SKO,
"skp" => SKP,
"skq" => SKQ,
"skr" => SKR,
"sks" => SKS,
"skt" => SKT,
"sku" => SKU,
"skv" => SKV,
"skw" => SKW,
"skx" => SKX,
"sky" => SKY,
"skz" => SKZ,
"slc" => SLC,
"sld" => SLD,
"sle" => SLE,
"slf" => SLF,
"slg" => SLG,
"slh" => SLH,
"sli" => SLI,
"slj" => SLJ,
"sll" => SLL,
"slm" => SLM,
"sln" => SLN,
"slp" => SLP,
"slq" => SLQ,
"slr" => SLR,
"sls" => SLS,
"slt" => SLT,
"slu" => SLU,
"slw" => SLW,
"slx" => SLX,
"sly" => SLY,
"slz" => SLZ,
"smb" => SMB,
"smc" => SMC,
"smf" => SMF,
"smg" => SMG,
"smh" => SMH,
"smk" => SMK,
"sml" => SML,
"smm" => SMM,
"smp" => SMP,
"smq" => SMQ,
"smr" => SMR,
"smt" => SMT,
"smu" => SMU,
"smv" => SMV,
"smw" => SMW,
"smx" => SMX,
"smy" => SMY,
"smz" => SMZ,
"snc" => SNC,
"sne" => SNE,
"snf" => SNF,
"sng" => SNG,
"sni" => SNI,
"snj" => SNJ,
"snl" => SNL,
"snm" => SNM,
"snn" => SNN,
"sno" => SNO,
"snp" => SNP,
"snq" => SNQ,
"snr" => SNR,
"sns" => SNS,
"snu" => SNU,
"snv" => SNV,
"snw" => SNW,
"snx" => SNX,
"sny" => SNY,
"snz" => SNZ,
"soa" => SOA,
"sob" => SOB,
"soc" => SOC,
"sod" => SOD,
"soe" => SOE,
"soh" => SOH,
"soi" => SOI,
"soj" => SOJ,
"sok" => SOK,
"sol" => SOL,
"soo" => SOO,
"sop" => SOP,
"soq" => SOQ,
"sor" => SOR,
"sos" => SOS,
"sou" => SOU,
"sov" => SOV,
"sow" => SOW,
"sox" => SOX,
"soy" => SOY,
"soz" => SOZ,
"spb" => SPB,
"spc" => SPC,
"spd" => SPD,
"spe" => SPE,
"spg" => SPG,
"spi" => SPI,
"spk" => SPK,
"spl" => SPL,
"spm" => SPM,
"spn" => SPN,
"spo" => SPO,
"spp" => SPP,
"spq" => SPQ,
"spr" => SPR,
"sps" => SPS,
"spt" => SPT,
"spu" => SPU,
"spv" => SPV,
"spx" => SPX,
"spy" => SPY,
"sqa" => SQA,
"sqh" => SQH,
"sqk" => SQK,
"sqm" => SQM,
"sqn" => SQN,
"sqo" => SQO,
"sqq" => SQQ,
"sqr" => SQR,
"sqs" => SQS,
"sqt" => SQT,
"squ" => SQU,
"sqx" => SQX,
"sra" => SRA,
"srb" => SRB,
"src" => SRC,
"sre" => SRE,
"srf" => SRF,
"srg" => SRG,
"srh" => SRH,
"sri" => SRI,
"srk" => SRK,
"srl" => SRL,
"srm" => SRM,
"sro" => SRO,
"srq" => SRQ,
"srs" => SRS,
"srt" => SRT,
"sru" => SRU,
"srv" => SRV,
"srw" => SRW,
"srx" => SRX,
"sry" => SRY,
"srz" => SRZ,
"ssb" => SSB,
"ssc" => SSC,
"ssd" => SSD,
"sse" => SSE,
"ssf" => SSF,
"ssg" => SSG,
"ssh" => SSH,
"ssi" => SSI,
"ssj" => SSJ,
"ssk" => SSK,
"ssl" => SSL,
"ssm" => SSM,
"ssn" => SSN,
"sso" => SSO,
"ssp" => SSP,
"ssq" => SSQ,
"ssr" => SSR,
"sss" => SSS,
"sst" => SST,
"ssu" => SSU,
"ssv" => SSV,
"ssx" => SSX,
"ssy" => SSY,
"ssz" => SSZ,
"sta" => STA,
"stb" => STB,
"std" => STD,
"ste" => STE,
"stf" => STF,
"stg" => STG,
"sth" => STH,
"sti" => STI,
"stj" => STJ,
"stk" => STK,
"stl" => STL,
"stm" => STM,
"stn" => STN,
"sto" => STO,
"stp" => STP,
"stq" => STQ,
"str" => STR,
"sts" => STS,
"stt" => STT,
"stu" => STU,
"stv" => STV,
"stw" => STW,
"sty" => STY,
"sua" => SUA,
"sub" => SUB,
"suc" => SUC,
"sue" => SUE,
"sug" => SUG,
"sui" => SUI,
"suj" => SUJ,
"suo" => SUO,
"suq" => SUQ,
"sur" => SUR,
"sut" => SUT,
"suv" => SUV,
"suw" => SUW,
"suy" => SUY,
"suz" => SUZ,
"sva" => SVA,
"svb" => SVB,
"svc" => SVC,
"sve" => SVE,
"svk" => SVK,
"svm" => SVM,
"svs" => SVS,
"svx" => SVX,
"swb" => SWB,
"swc" => SWC,
"swf" => SWF,
"swg" => SWG,
"swh" => SWH,
"swi" => SWI,
"swj" => SWJ,
"swk" => SWK,
"swl" => SWL,
"swm" => SWM,
"swn" => SWN,
"swo" => SWO,
"swp" => SWP,
"swq" => SWQ,
"swr" => SWR,
"sws" => SWS,
"swt" => SWT,
"swu" => SWU,
"swv" => SWV,
"sww" => SWW,
"swx" => SWX,
"swy" => SWY,
"sxb" => SXB,
"sxc" => SXC,
"sxe" => SXE,
"sxg" => SXG,
"sxk" => SXK,
"sxl" => SXL,
"sxm" => SXM,
"sxn" => SXN,
"sxo" => SXO,
"sxr" => SXR,
"sxs" => SXS,
"sxu" => SXU,
"sxw" => SXW,
"sya" => SYA,
"syb" => SYB,
"syi" => SYI,
"syk" => SYK,
"syl" => SYL,
"sym" => SYM,
"syn" => SYN,
"syo" => SYO,
"sys" => SYS,
"syw" => SYW,
"syx" => SYX,
"syy" => SYY,
"sza" => SZA,
"szb" => SZB,
"szc" => SZC,
"szd" => SZD,
"sze" => SZE,
"szg" => SZG,
"szl" => SZL,
"szn" => SZN,
"szp" => SZP,
"szs" => SZS,
"szv" => SZV,
"szw" => SZW,
"szy" => SZY,
"taa" => TAA,
"tab" => TAB,
"tac" => TAC,
"tad" => TAD,
"tae" => TAE,
"taf" => TAF,
"tag" => TAG,
"taj" => TAJ,
"tak" => TAK,
"tal" => TAL,
"tan" => TAN,
"tao" => TAO,
"tap" => TAP,
"taq" => TAQ,
"tar" => TAR,
"tas" => TAS,
"tau" => TAU,
"tav" => TAV,
"taw" => TAW,
"tax" => TAX,
"tay" => TAY,
"taz" => TAZ,
"tba" => TBA,
"tbc" => TBC,
"tbd" => TBD,
"tbe" => TBE,
"tbf" => TBF,
"tbg" => TBG,
"tbh" => TBH,
"tbi" => TBI,
"tbj" => TBJ,
"tbk" => TBK,
"tbl" => TBL,
"tbm" => TBM,
"tbn" => TBN,
"tbo" => TBO,
"tbp" => TBP,
"tbr" => TBR,
"tbs" => TBS,
"tbt" => TBT,
"tbu" => TBU,
"tbv" => TBV,
"tbw" => TBW,
"tbx" => TBX,
"tby" => TBY,
"tbz" => TBZ,
"tca" => TCA,
"tcb" => TCB,
"tcc" => TCC,
"tcd" => TCD,
"tce" => TCE,
"tcf" => TCF,
"tcg" => TCG,
"tch" => TCH,
"tci" => TCI,
"tck" => TCK,
"tcl" => TCL,
"tcm" => TCM,
"tcn" => TCN,
"tco" => TCO,
"tcp" => TCP,
"tcq" => TCQ,
"tcs" => TCS,
"tct" => TCT,
"tcu" => TCU,
"tcw" => TCW,
"tcx" => TCX,
"tcy" => TCY,
"tcz" => TCZ,
"tda" => TDA,
"tdb" => TDB,
"tdc" => TDC,
"tdd" => TDD,
"tde" => TDE,
"tdf" => TDF,
"tdg" => TDG,
"tdh" => TDH,
"tdi" => TDI,
"tdj" => TDJ,
"tdk" => TDK,
"tdl" => TDL,
"tdm" => TDM,
"tdn" => TDN,
"tdo" => TDO,
"tdq" => TDQ,
"tdr" => TDR,
"tds" => TDS,
"tdt" => TDT,
"tdv" => TDV,
"tdx" => TDX,
"tdy" => TDY,
"tea" => TEA,
"teb" => TEB,
"tec" => TEC,
"ted" => TED,
"tee" => TEE,
"tef" => TEF,
"teg" => TEG,
"teh" => TEH,
"tei" => TEI,
"tek" => TEK,
"ten" => TEN,
"teo" => TEO,
"tep" => TEP,
"teq" => TEQ,
"tes" => TES,
"teu" => TEU,
"tev" => TEV,
"tew" => TEW,
"tex" => TEX,
"tey" => TEY,
"tez" => TEZ,
"tfi" => TFI,
"tfn" => TFN,
"tfo" => TFO,
"tfr" => TFR,
"tft" => TFT,
"tga" => TGA,
"tgb" => TGB,
"tgc" => TGC,
"tgd" => TGD,
"tge" => TGE,
"tgf" => TGF,
"tgh" => TGH,
"tgi" => TGI,
"tgj" => TGJ,
"tgn" => TGN,
"tgo" => TGO,
"tgp" => TGP,
"tgq" => TGQ,
"tgr" => TGR,
"tgs" => TGS,
"tgt" => TGT,
"tgu" => TGU,
"tgv" => TGV,
"tgw" => TGW,
"tgx" => TGX,
"tgy" => TGY,
"tgz" => TGZ,
"thd" => THD,
"the" => THE,
"thf" => THF,
"thh" => THH,
"thi" => THI,
"thk" => THK,
"thl" => THL,
"thm" => THM,
"thn" => THN,
"thp" => THP,
"thq" => THQ,
"thr" => THR,
"ths" => THS,
"tht" => THT,
"thu" => THU,
"thv" => THV,
"thy" => THY,
"thz" => THZ,
"tia" => TIA,
"tic" => TIC,
"tif" => TIF,
"tih" => TIH,
"tii" => TII,
"tij" => TIJ,
"tik" => TIK,
"til" => TIL,
"tim" => TIM,
"tin" => TIN,
"tio" => TIO,
"tip" => TIP,
"tiq" => TIQ,
"tis" => TIS,
"tit" => TIT,
"tiu" => TIU,
"tiw" => TIW,
"tix" => TIX,
"tiy" => TIY,
"tiz" => TIZ,
"tja" => TJA,
"tjg" => TJG,
"tji" => TJI,
"tjj" => TJJ,
"tjl" => TJL,
"tjm" => TJM,
"tjn" => TJN,
"tjo" => TJO,
"tjp" => TJP,
"tjs" => TJS,
"tju" => TJU,
"tjw" => TJW,
"tka" => TKA,
"tkb" => TKB,
"tkd" => TKD,
"tke" => TKE,
"tkf" => TKF,
"tkg" => TKG,
"tkm" => TKM,
"tkn" => TKN,
"tkp" => TKP,
"tkq" => TKQ,
"tkr" => TKR,
"tks" => TKS,
"tkt" => TKT,
"tku" => TKU,
"tkv" => TKV,
"tkw" => TKW,
"tkx" => TKX,
"tkz" => TKZ,
"tla" => TLA,
"tlb" => TLB,
"tlc" => TLC,
"tld" => TLD,
"tlf" => TLF,
"tlg" => TLG,
"tlj" => TLJ,
"tlk" => TLK,
"tll" => TLL,
"tlm" => TLM,
"tln" => TLN,
"tlo" => TLO,
"tlp" => TLP,
"tlq" => TLQ,
"tlr" => TLR,
"tls" => TLS,
"tlt" => TLT,
"tlu" => TLU,
"tlv" => TLV,
"tlx" => TLX,
"tly" => TLY,
"tma" => TMA,
"tmb" => TMB,
"tmc" => TMC,
"tmd" => TMD,
"tme" => TME,
"tmf" => TMF,
"tmg" => TMG,
"tmi" => TMI,
"tmj" => TMJ,
"tmk" => TMK,
"tml" => TML,
"tmm" => TMM,
"tmn" => TMN,
"tmo" => TMO,
"tmq" => TMQ,
"tmr" => TMR,
"tms" => TMS,
"tmt" => TMT,
"tmu" => TMU,
"tmv" => TMV,
"tmw" => TMW,
"tmy" => TMY,
"tmz" => TMZ,
"tna" => TNA,
"tnb" => TNB,
"tnc" => TNC,
"tnd" => TND,
"tng" => TNG,
"tnh" => TNH,
"tni" => TNI,
"tnk" => TNK,
"tnl" => TNL,
"tnm" => TNM,
"tnn" => TNN,
"tno" => TNO,
"tnp" => TNP,
"tnq" => TNQ,
"tnr" => TNR,
"tns" => TNS,
"tnt" => TNT,
"tnu" => TNU,
"tnv" => TNV,
"tnw" => TNW,
"tnx" => TNX,
"tny" => TNY,
"tnz" => TNZ,
"tob" => TOB,
"toc" => TOC,
"tod" => TOD,
"tof" => TOF,
"toh" => TOH,
"toi" => TOI,
"toj" => TOJ,
"tok" => TOK,
"tol" => TOL,
"tom" => TOM,
"too" => TOO,
"top" => TOP,
"toq" => TOQ,
"tor" => TOR,
"tos" => TOS,
"tou" => TOU,
"tov" => TOV,
"tow" => TOW,
"tox" => TOX,
"toy" => TOY,
"toz" => TOZ,
"tpa" => TPA,
"tpc" => TPC,
"tpe" => TPE,
"tpf" => TPF,
"tpg" => TPG,
"tpj" => TPJ,
"tpk" => TPK,
"tpl" => TPL,
"tpm" => TPM,
"tpn" => TPN,
"tpo" => TPO,
"tpp" => TPP,
"tpq" => TPQ,
"tpr" => TPR,
"tpt" => TPT,
"tpu" => TPU,
"tpv" => TPV,
"tpw" => TPW,
"tpx" => TPX,
"tpy" => TPY,
"tpz" => TPZ,
"tqb" => TQB,
"tql" => TQL,
"tqm" => TQM,
"tqn" => TQN,
"tqo" => TQO,
"tqp" => TQP,
"tqq" => TQQ,
"tqr" => TQR,
"tqt" => TQT,
"tqu" => TQU,
"tqw" => TQW,
"tra" => TRA,
"trb" => TRB,
"trc" => TRC,
"trd" => TRD,
"tre" => TRE,
"trf" => TRF,
"trg" => TRG,
"trh" => TRH,
"tri" => TRI,
"trj" => TRJ,
"trl" => TRL,
"trm" => TRM,
"trn" => TRN,
"tro" => TRO,
"trp" => TRP,
"trq" => TRQ,
"trr" => TRR,
"trs" => TRS,
"trt" => TRT,
"tru" => TRU,
"trv" => TRV,
"trw" => TRW,
"trx" => TRX,
"try" => TRY,
"trz" => TRZ,
"tsa" => TSA,
"tsb" => TSB,
"tsc" => TSC,
"tsd" => TSD,
"tse" => TSE,
"tsg" => TSG,
"tsh" => TSH,
"tsj" => TSJ,
"tsk" => TSK,
"tsl" => TSL,
"tsm" => TSM,
"tsp" => TSP,
"tsq" => TSQ,
"tsr" => TSR,
"tss" => TSS,
"tst" => TST,
"tsu" => TSU,
"tsv" => TSV,
"tsw" => TSW,
"tsx" => TSX,
"tsy" => TSY,
"tsz" => TSZ,
"tta" => TTA,
"ttb" => TTB,
"ttc" => TTC,
"ttd" => TTD,
"tte" => TTE,
"ttf" => TTF,
"ttg" => TTG,
"tth" => TTH,
"tti" => TTI,
"ttj" => TTJ,
"ttk" => TTK,
"ttl" => TTL,
"ttm" => TTM,
"ttn" => TTN,
"tto" => TTO,
"ttp" => TTP,
"ttq" => TTQ,
"ttr" => TTR,
"tts" => TTS,
"ttt" => TTT,
"ttu" => TTU,
"ttv" => TTV,
"ttw" => TTW,
"tty" => TTY,
"ttz" => TTZ,
"tua" => TUA,
"tub" => TUB,
"tuc" => TUC,
"tud" => TUD,
"tue" => TUE,
"tuf" => TUF,
"tug" => TUG,
"tuh" => TUH,
"tui" => TUI,
"tuj" => TUJ,
"tul" => TUL,
"tun" => TUN,
"tuo" => TUO,
"tuq" => TUQ,
"tus" => TUS,
"tuu" => TUU,
"tuv" => TUV,
"tux" => TUX,
"tuy" => TUY,
"tuz" => TUZ,
"tva" => TVA,
"tvd" => TVD,
"tve" => TVE,
"tvk" => TVK,
"tvm" => TVM,
"tvn" => TVN,
"tvo" => TVO,
"tvs" => TVS,
"tvt" => TVT,
"tvu" => TVU,
"tvw" => TVW,
"tvx" => TVX,
"tvy" => TVY,
"twa" => TWA,
"twb" => TWB,
"twc" => TWC,
"twd" => TWD,
"twe" => TWE,
"twf" => TWF,
"twg" => TWG,
"twh" => TWH,
"twl" => TWL,
"twm" => TWM,
"twn" => TWN,
"two" => TWO,
"twp" => TWP,
"twq" => TWQ,
"twr" => TWR,
"twt" => TWT,
"twu" => TWU,
"tww" => TWW,
"twx" => TWX,
"twy" => TWY,
"txa" => TXA,
"txb" => TXB,
"txc" => TXC,
"txe" => TXE,
"txg" => TXG,
"txh" => TXH,
"txi" => TXI,
"txj" => TXJ,
"txm" => TXM,
"txn" => TXN,
"txo" => TXO,
"txq" => TXQ,
"txr" => TXR,
"txs" => TXS,
"txt" => TXT,
"txu" => TXU,
"txx" => TXX,
"txy" => TXY,
"tya" => TYA,
"tye" => TYE,
"tyh" => TYH,
"tyi" => TYI,
"tyj" => TYJ,
"tyl" => TYL,
"tyn" => TYN,
"typ" => TYP,
"tyr" => TYR,
"tys" => TYS,
"tyt" => TYT,
"tyu" => TYU,
"tyx" => TYX,
"tyy" => TYY,
"tyz" => TYZ,
"tza" => TZA,
"tzh" => TZH,
"tzj" => TZJ,
"tzl" => TZL,
"tzm" => TZM,
"tzn" => TZN,
"tzo" => TZO,
"tzx" => TZX,
"uam" => UAM,
"uan" => UAN,
"uar" => UAR,
"uba" => UBA,
"ubi" => UBI,
"ubl" => UBL,
"ubr" => UBR,
"ubu" => UBU,
"uby" => UBY,
"uda" => UDA,
"ude" => UDE,
"udg" => UDG,
"udi" => UDI,
"udj" => UDJ,
"udl" => UDL,
"udu" => UDU,
"ues" => UES,
"ufi" => UFI,
"ugb" => UGB,
"uge" => UGE,
"ugh" => UGH,
"ugn" => UGN,
"ugo" => UGO,
"ugy" => UGY,
"uha" => UHA,
"uhn" => UHN,
"uis" => UIS,
"uiv" => UIV,
"uji" => UJI,
"uka" => UKA,
"ukg" => UKG,
"ukh" => UKH,
"uki" => UKI,
"ukk" => UKK,
"ukl" => UKL,
"ukp" => UKP,
"ukq" => UKQ,
"uks" => UKS,
"uku" => UKU,
"ukv" => UKV,
"ukw" => UKW,
"uky" => UKY,
"ula" => ULA,
"ulb" => ULB,
"ulc" => ULC,
"ule" => ULE,
"ulf" => ULF,
"uli" => ULI,
"ulk" => ULK,
"ull" => ULL,
"ulm" => ULM,
"uln" => ULN,
"ulu" => ULU,
"ulw" => ULW,
"uma" => UMA,
"umc" => UMC,
"umd" => UMD,
"umg" => UMG,
"umi" => UMI,
"umm" => UMM,
"umn" => UMN,
"umo" => UMO,
"ump" => UMP,
"umr" => UMR,
"ums" => UMS,
"umu" => UMU,
"una" => UNA,
"une" => UNE,
"ung" => UNG,
"uni" => UNI,
"unk" => UNK,
"unm" => UNM,
"unn" => UNN,
"unr" => UNR,
"unu" => UNU,
"unx" => UNX,
"unz" => UNZ,
"uon" => UON,
"upi" => UPI,
"upv" => UPV,
"ura" => URA,
"urb" => URB,
"urc" => URC,
"ure" => URE,
"urf" => URF,
"urg" => URG,
"urh" => URH,
"uri" => URI,
"urk" => URK,
"url" => URL,
"urm" => URM,
"urn" => URN,
"uro" => URO,
"urp" => URP,
"urr" => URR,
"urt" => URT,
"uru" => URU,
"urv" => URV,
"urw" => URW,
"urx" => URX,
"ury" => URY,
"urz" => URZ,
"usa" => USA,
"ush" => USH,
"usi" => USI,
"usk" => USK,
"usp" => USP,
"uss" => USS,
"usu" => USU,
"uta" => UTA,
"ute" => UTE,
"uth" => UTH,
"utp" => UTP,
"utr" => UTR,
"utu" => UTU,
"uum" => UUM,
"uur" => UUR,
"uuu" => UUU,
"uve" => UVE,
"uvh" => UVH,
"uvl" => UVL,
"uwa" => UWA,
"uya" => UYA,
"uzn" => UZN,
"uzs" => UZS,
"vaa" => VAA,
"vae" => VAE,
"vaf" => VAF,
"vag" => VAG,
"vah" => VAH,
"vaj" => VAJ,
"val" => VAL,
"vam" => VAM,
"van" => VAN,
"vao" => VAO,
"vap" => VAP,
"var" => VAR,
"vas" => VAS,
"vau" => VAU,
"vav" => VAV,
"vay" => VAY,
"vbb" => VBB,
"vbk" => VBK,
"vec" => VEC,
"ved" => VED,
"vel" => VEL,
"vem" => VEM,
"veo" => VEO,
"vep" => VEP,
"ver" => VER,
"vgr" => VGR,
"vgt" => VGT,
"vic" => VIC,
"vid" => VID,
"vif" => VIF,
"vig" => VIG,
"vil" => VIL,
"vin" => VIN,
"vis" => VIS,
"vit" => VIT,
"viv" => VIV,
"vka" => VKA,
"vkj" => VKJ,
"vkk" => VKK,
"vkl" => VKL,
"vkm" => VKM,
"vkn" => VKN,
"vko" => VKO,
"vkp" => VKP,
"vkt" => VKT,
"vku" => VKU,
"vkz" => VKZ,
"vlp" => VLP,
"vls" => VLS,
"vma" => VMA,
"vmb" => VMB,
"vmc" => VMC,
"vmd" => VMD,
"vme" => VME,
"vmf" => VMF,
"vmg" => VMG,
"vmh" => VMH,
"vmi" => VMI,
"vmj" => VMJ,
"vmk" => VMK,
"vml" => VML,
"vmm" => VMM,
"vmp" => VMP,
"vmq" => VMQ,
"vmr" => VMR,
"vms" => VMS,
"vmu" => VMU,
"vmv" => VMV,
"vmw" => VMW,
"vmx" => VMX,
"vmy" => VMY,
"vmz" => VMZ,
"vnk" => VNK,
"vnm" => VNM,
"vnp" => VNP,
"vor" => VOR,
"vra" => VRA,
"vro" => VRO,
"vrs" => VRS,
"vrt" => VRT,
"vsi" => VSI,
"vsl" => VSL,
"vsv" => VSV,
"vto" => VTO,
"vum" => VUM,
"vun" => VUN,
"vut" => VUT,
"vwa" => VWA,
"waa" => WAA,
"wab" => WAB,
"wac" => WAC,
"wad" => WAD,
"wae" => WAE,
"waf" => WAF,
"wag" => WAG,
"wah" => WAH,
"wai" => WAI,
"waj" => WAJ,
"wam" => WAM,
"wan" => WAN,
"wao" => WAO,
"wap" => WAP,
"waq" => WAQ,
"wat" => WAT,
"wau" => WAU,
"wav" => WAV,
"waw" => WAW,
"wax" => WAX,
"way" => WAY,
"waz" => WAZ,
"wba" => WBA,
"wbb" => WBB,
"wbe" => WBE,
"wbf" => WBF,
"wbh" => WBH,
"wbi" => WBI,
"wbj" => WBJ,
"wbk" => WBK,
"wbl" => WBL,
"wbm" => WBM,
"wbp" => WBP,
"wbq" => WBQ,
"wbr" => WBR,
"wbs" => WBS,
"wbt" => WBT,
"wbv" => WBV,
"wbw" => WBW,
"wca" => WCA,
"wci" => WCI,
"wdd" => WDD,
"wdg" => WDG,
"wdj" => WDJ,
"wdk" => WDK,
"wdt" => WDT,
"wdu" => WDU,
"wdy" => WDY,
"wea" => WEA,
"wec" => WEC,
"wed" => WED,
"weg" => WEG,
"weh" => WEH,
"wei" => WEI,
"wem" => WEM,
"weo" => WEO,
"wep" => WEP,
"wer" => WER,
"wes" => WES,
"wet" => WET,
"weu" => WEU,
"wew" => WEW,
"wfg" => WFG,
"wga" => WGA,
"wgb" => WGB,
"wgg" => WGG,
"wgi" => WGI,
"wgo" => WGO,
"wgu" => WGU,
"wgy" => WGY,
"wha" => WHA,
"whg" => WHG,
"whk" => WHK,
"whu" => WHU,
"wib" => WIB,
"wic" => WIC,
"wie" => WIE,
"wif" => WIF,
"wig" => WIG,
"wih" => WIH,
"wii" => WII,
"wij" => WIJ,
"wik" => WIK,
"wil" => WIL,
"wim" => WIM,
"win" => WIN,
"wir" => WIR,
"wiu" => WIU,
"wiv" => WIV,
"wiy" => WIY,
"wja" => WJA,
"wji" => WJI,
"wka" => WKA,
"wkb" => WKB,
"wkd" => WKD,
"wkl" => WKL,
"wkr" => WKR,
"wku" => WKU,
"wkw" => WKW,
"wky" => WKY,
"wla" => WLA,
"wlc" => WLC,
"wle" => WLE,
"wlg" => WLG,
"wlh" => WLH,
"wli" => WLI,
"wlk" => WLK,
"wll" => WLL,
"wlm" => WLM,
"wlo" => WLO,
"wlr" => WLR,
"wls" => WLS,
"wlu" => WLU,
"wlv" => WLV,
"wlw" => WLW,
"wlx" => WLX,
"wly" => WLY,
"wma" => WMA,
"wmb" => WMB,
"wmc" => WMC,
"wmd" => WMD,
"wme" => WME,
"wmg" => WMG,
"wmh" => WMH,
"wmi" => WMI,
"wmm" => WMM,
"wmn" => WMN,
"wmo" => WMO,
"wms" => WMS,
"wmt" => WMT,
"wmw" => WMW,
"wmx" => WMX,
"wnb" => WNB,
"wnc" => WNC,
"wnd" => WND,
"wne" => WNE,
"wng" => WNG,
"wni" => WNI,
"wnk" => WNK,
"wnm" => WNM,
"wnn" => WNN,
"wno" => WNO,
"wnp" => WNP,
"wnu" => WNU,
"wnw" => WNW,
"wny" => WNY,
"woa" => WOA,
"wob" => WOB,
"woc" => WOC,
"wod" => WOD,
"woe" => WOE,
"wof" => WOF,
"wog" => WOG,
"woi" => WOI,
"wok" => WOK,
"wom" => WOM,
"won" => WON,
"woo" => WOO,
"wor" => WOR,
"wos" => WOS,
"wow" => WOW,
"woy" => WOY,
"wpc" => WPC,
"wrb" => WRB,
"wrg" => WRG,
"wrh" => WRH,
"wri" => WRI,
"wrk" => WRK,
"wrl" => WRL,
"wrm" => WRM,
"wrn" => WRN,
"wro" => WRO,
"wrp" => WRP,
"wrr" => WRR,
"wrs" => WRS,
"wru" => WRU,
"wrv" => WRV,
"wrw" => WRW,
"wrx" => WRX,
"wry" => WRY,
"wrz" => WRZ,
"wsa" => WSA,
"wsg" => WSG,
"wsi" => WSI,
"wsk" => WSK,
"wsr" => WSR,
"wss" => WSS,
"wsu" => WSU,
"wsv" => WSV,
"wtf" => WTF,
"wth" => WTH,
"wti" => WTI,
"wtk" => WTK,
"wtm" => WTM,
"wtw" => WTW,
"wua" => WUA,
"wub" => WUB,
"wud" => WUD,
"wuh" => WUH,
"wul" => WUL,
"wum" => WUM,
"wun" => WUN,
"wur" => WUR,
"wut" => WUT,
"wuu" => WUU,
"wuv" => WUV,
"wux" => WUX,
"wuy" => WUY,
"wwa" => WWA,
"wwb" => WWB,
"wwo" => WWO,
"wwr" => WWR,
"www" => WWW,
"wxa" => WXA,
"wxw" => WXW,
"wyb" => WYB,
"wyi" => WYI,
"wym" => WYM,
"wyn" => WYN,
"wyr" => WYR,
"wyy" => WYY,
"xaa" => XAA,
"xab" => XAB,
"xac" => XAC,
"xad" => XAD,
"xae" => XAE,
"xag" => XAG,
"xai" => XAI,
"xaj" => XAJ,
"xak" => XAK,
"xam" => XAM,
"xan" => XAN,
"xao" => XAO,
"xap" => XAP,
"xaq" => XAQ,
"xar" => XAR,
"xas" => XAS,
"xat" => XAT,
"xau" => XAU,
"xav" => XAV,
"xaw" => XAW,
"xay" => XAY,
"xbb" => XBB,
"xbc" => XBC,
"xbd" => XBD,
"xbe" => XBE,
"xbg" => XBG,
"xbi" => XBI,
"xbj" => XBJ,
"xbm" => XBM,
"xbn" => XBN,
"xbo" => XBO,
"xbp" => XBP,
"xbr" => XBR,
"xbw" => XBW,
"xby" => XBY,
"xcb" => XCB,
"xcc" => XCC,
"xce" => XCE,
"xcg" => XCG,
"xch" => XCH,
"xcl" => XCL,
"xcm" => XCM,
"xcn" => XCN,
"xco" => XCO,
"xcr" => XCR,
"xct" => XCT,
"xcu" => XCU,
"xcv" => XCV,
"xcw" => XCW,
"xcy" => XCY,
"xda" => XDA,
"xdc" => XDC,
"xdk" => XDK,
"xdm" => XDM,
"xdo" => XDO,
"xdq" => XDQ,
"xdy" => XDY,
"xeb" => XEB,
"xed" => XED,
"xeg" => XEG,
"xel" => XEL,
"xem" => XEM,
"xep" => XEP,
"xer" => XER,
"xes" => XES,
"xet" => XET,
"xeu" => XEU,
"xfa" => XFA,
"xga" => XGA,
"xgb" => XGB,
"xgd" => XGD,
"xgf" => XGF,
"xgg" => XGG,
"xgi" => XGI,
"xgl" => XGL,
"xgm" => XGM,
"xgr" => XGR,
"xgu" => XGU,
"xgw" => XGW,
"xha" => XHA,
"xhc" => XHC,
"xhd" => XHD,
"xhe" => XHE,
"xhm" => XHM,
"xhr" => XHR,
"xht" => XHT,
"xhu" => XHU,
"xhv" => XHV,
"xib" => XIB,
"xii" => XII,
"xil" => XIL,
"xin" => XIN,
"xir" => XIR,
"xis" => XIS,
"xiv" => XIV,
"xiy" => XIY,
"xjb" => XJB,
"xjt" => XJT,
"xka" => XKA,
"xkb" => XKB,
"xkc" => XKC,
"xkd" => XKD,
"xke" => XKE,
"xkf" => XKF,
"xkg" => XKG,
"xki" => XKI,
"xkj" => XKJ,
"xkk" => XKK,
"xkl" => XKL,
"xkn" => XKN,
"xko" => XKO,
"xkp" => XKP,
"xkq" => XKQ,
"xkr" => XKR,
"xks" => XKS,
"xkt" => XKT,
"xku" => XKU,
"xkv" => XKV,
"xkw" => XKW,
"xkx" => XKX,
"xky" => XKY,
"xkz" => XKZ,
"xla" => XLA,
"xlb" => XLB,
"xlc" => XLC,
"xld" => XLD,
"xle" => XLE,
"xlg" => XLG,
"xli" => XLI,
"xln" => XLN,
"xlo" => XLO,
"xlp" => XLP,
"xls" => XLS,
"xlu" => XLU,
"xly" => XLY,
"xma" => XMA,
"xmb" => XMB,
"xmc" => XMC,
"xmd" => XMD,
"xme" => XME,
"xmf" => XMF,
"xmg" => XMG,
"xmh" => XMH,
"xmj" => XMJ,
"xmk" => XMK,
"xml" => XML,
"xmm" => XMM,
"xmn" => XMN,
"xmo" => XMO,
"xmp" => XMP,
"xmq" => XMQ,
"xmr" => XMR,
"xms" => XMS,
"xmt" => XMT,
"xmu" => XMU,
"xmv" => XMV,
"xmw" => XMW,
"xmx" => XMX,
"xmy" => XMY,
"xmz" => XMZ,
"xna" => XNA,
"xnb" => XNB,
"xng" => XNG,
"xnh" => XNH,
"xni" => XNI,
"xnj" => XNJ,
"xnk" => XNK,
"xnm" => XNM,
"xnn" => XNN,
"xno" => XNO,
"xnq" => XNQ,
"xnr" => XNR,
"xns" => XNS,
"xnt" => XNT,
"xnu" => XNU,
"xny" => XNY,
"xnz" => XNZ,
"xoc" => XOC,
"xod" => XOD,
"xog" => XOG,
"xoi" => XOI,
"xok" => XOK,
"xom" => XOM,
"xon" => XON,
"xoo" => XOO,
"xop" => XOP,
"xor" => XOR,
"xow" => XOW,
"xpa" => XPA,
"xpb" => XPB,
"xpc" => XPC,
"xpd" => XPD,
"xpe" => XPE,
"xpf" => XPF,
"xpg" => XPG,
"xph" => XPH,
"xpi" => XPI,
"xpj" => XPJ,
"xpk" => XPK,
"xpl" => XPL,
"xpm" => XPM,
"xpn" => XPN,
"xpo" => XPO,
"xpp" => XPP,
"xpq" => XPQ,
"xpr" => XPR,
"xps" => XPS,
"xpt" => XPT,
"xpu" => XPU,
"xpv" => XPV,
"xpw" => XPW,
"xpx" => XPX,
"xpy" => XPY,
"xpz" => XPZ,
"xqa" => XQA,
"xqt" => XQT,
"xra" => XRA,
"xrb" => XRB,
"xrd" => XRD,
"xre" => XRE,
"xrg" => XRG,
"xri" => XRI,
"xrm" => XRM,
"xrn" => XRN,
"xrr" => XRR,
"xrt" => XRT,
"xru" => XRU,
"xrw" => XRW,
"xsa" => XSA,
"xsb" => XSB,
"xsc" => XSC,
"xsd" => XSD,
"xse" => XSE,
"xsh" => XSH,
"xsi" => XSI,
"xsj" => XSJ,
"xsl" => XSL,
"xsm" => XSM,
"xsn" => XSN,
"xso" => XSO,
"xsp" => XSP,
"xsq" => XSQ,
"xsr" => XSR,
"xss" => XSS,
"xsu" => XSU,
"xsv" => XSV,
"xsy" => XSY,
"xta" => XTA,
"xtb" => XTB,
"xtc" => XTC,
"xtd" => XTD,
"xte" => XTE,
"xtg" => XTG,
"xth" => XTH,
"xti" => XTI,
"xtj" => XTJ,
"xtl" => XTL,
"xtm" => XTM,
"xtn" => XTN,
"xto" => XTO,
"xtp" => XTP,
"xtq" => XTQ,
"xtr" => XTR,
"xts" => XTS,
"xtt" => XTT,
"xtu" => XTU,
"xtv" => XTV,
"xtw" => XTW,
"xty" => XTY,
"xua" => XUA,
"xub" => XUB,
"xud" => XUD,
"xug" => XUG,
"xuj" => XUJ,
"xul" => XUL,
"xum" => XUM,
"xun" => XUN,
"xuo" => XUO,
"xup" => XUP,
"xur" => XUR,
"xut" => XUT,
"xuu" => XUU,
"xve" => XVE,
"xvi" => XVI,
"xvn" => XVN,
"xvo" => XVO,
"xvs" => XVS,
"xwa" => XWA,
"xwc" => XWC,
"xwd" => XWD,
"xwe" => XWE,
"xwg" => XWG,
"xwj" => XWJ,
"xwk" => XWK,
"xwl" => XWL,
"xwo" => XWO,
"xwr" => XWR,
"xwt" => XWT,
"xww" => XWW,
"xxb" => XXB,
"xxk" => XXK,
"xxm" => XXM,
"xxr" => XXR,
"xxt" => XXT,
"xya" => XYA,
"xyb" => XYB,
"xyj" => XYJ,
"xyk" => XYK,
"xyl" => XYL,
"xyt" => XYT,
"xyy" => XYY,
"xzh" => XZH,
"xzm" => XZM,
"xzp" => XZP,
"yaa" => YAA,
"yab" => YAB,
"yac" => YAC,
"yad" => YAD,
"yae" => YAE,
"yaf" => YAF,
"yag" => YAG,
"yah" => YAH,
"yai" => YAI,
"yaj" => YAJ,
"yak" => YAK,
"yal" => YAL,
"yam" => YAM,
"yan" => YAN,
"yaq" => YAQ,
"yar" => YAR,
"yas" => YAS,
"yat" => YAT,
"yau" => YAU,
"yav" => YAV,
"yaw" => YAW,
"yax" => YAX,
"yay" => YAY,
"yaz" => YAZ,
"yba" => YBA,
"ybb" => YBB,
"ybe" => YBE,
"ybh" => YBH,
"ybi" => YBI,
"ybj" => YBJ,
"ybk" => YBK,
"ybl" => YBL,
"ybm" => YBM,
"ybn" => YBN,
"ybo" => YBO,
"ybx" => YBX,
"yby" => YBY,
"ych" => YCH,
"ycl" => YCL,
"ycn" => YCN,
"ycp" => YCP,
"yda" => YDA,
"ydd" => YDD,
"yde" => YDE,
"ydg" => YDG,
"ydk" => YDK,
"yea" => YEA,
"yec" => YEC,
"yee" => YEE,
"yei" => YEI,
"yej" => YEJ,
"yel" => YEL,
"yer" => YER,
"yes" => YES,
"yet" => YET,
"yeu" => YEU,
"yev" => YEV,
"yey" => YEY,
"yga" => YGA,
"ygi" => YGI,
"ygl" => YGL,
"ygm" => YGM,
"ygp" => YGP,
"ygr" => YGR,
"ygs" => YGS,
"ygu" => YGU,
"ygw" => YGW,
"yha" => YHA,
"yhd" => YHD,
"yhl" => YHL,
"yhs" => YHS,
"yia" => YIA,
"yif" => YIF,
"yig" => YIG,
"yih" => YIH,
"yii" => YII,
"yij" => YIJ,
"yik" => YIK,
"yil" => YIL,
"yim" => YIM,
"yin" => YIN,
"yip" => YIP,
"yiq" => YIQ,
"yir" => YIR,
"yis" => YIS,
"yit" => YIT,
"yiu" => YIU,
"yiv" => YIV,
"yix" => YIX,
"yiz" => YIZ,
"yka" => YKA,
"ykg" => YKG,
"yki" => YKI,
"ykk" => YKK,
"ykl" => YKL,
"ykm" => YKM,
"ykn" => YKN,
"yko" => YKO,
"ykr" => YKR,
"ykt" => YKT,
"yku" => YKU,
"yky" => YKY,
"yla" => YLA,
"ylb" => YLB,
"yle" => YLE,
"ylg" => YLG,
"yli" => YLI,
"yll" => YLL,
"ylm" => YLM,
"yln" => YLN,
"ylo" => YLO,
"ylr" => YLR,
"ylu" => YLU,
"yly" => YLY,
"ymb" => YMB,
"ymc" => YMC,
"ymd" => YMD,
"yme" => YME,
"ymg" => YMG,
"ymh" => YMH,
"ymi" => YMI,
"ymk" => YMK,
"yml" => YML,
"ymm" => YMM,
"ymn" => YMN,
"ymo" => YMO,
"ymp" => YMP,
"ymq" => YMQ,
"ymr" => YMR,
"yms" => YMS,
"ymx" => YMX,
"ymz" => YMZ,
"yna" => YNA,
"ynd" => YND,
"yne" => YNE,
"yng" => YNG,
"ynk" => YNK,
"ynl" => YNL,
"ynn" => YNN,
"yno" => YNO,
"ynq" => YNQ,
"yns" => YNS,
"ynu" => YNU,
"yob" => YOB,
"yog" => YOG,
"yoi" => YOI,
"yok" => YOK,
"yol" => YOL,
"yom" => YOM,
"yon" => YON,
"yot" => YOT,
"yox" => YOX,
"yoy" => YOY,
"ypa" => YPA,
"ypb" => YPB,
"ypg" => YPG,
"yph" => YPH,
"ypm" => YPM,
"ypn" => YPN,
"ypo" => YPO,
"ypp" => YPP,
"ypz" => YPZ,
"yra" => YRA,
"yrb" => YRB,
"yre" => YRE,
"yrk" => YRK,
"yrl" => YRL,
"yrm" => YRM,
"yrn" => YRN,
"yro" => YRO,
"yrs" => YRS,
"yrw" => YRW,
"yry" => YRY,
"ysc" => YSC,
"ysd" => YSD,
"ysg" => YSG,
"ysl" => YSL,
"ysm" => YSM,
"ysn" => YSN,
"yso" => YSO,
"ysp" => YSP,
"ysr" => YSR,
"yss" => YSS,
"ysy" => YSY,
"yta" => YTA,
"ytl" => YTL,
"ytp" => YTP,
"ytw" => YTW,
"yty" => YTY,
"yua" => YUA,
"yub" => YUB,
"yuc" => YUC,
"yud" => YUD,
"yue" => YUE,
"yuf" => YUF,
"yug" => YUG,
"yui" => YUI,
"yuj" => YUJ,
"yuk" => YUK,
"yul" => YUL,
"yum" => YUM,
"yun" => YUN,
"yup" => YUP,
"yuq" => YUQ,
"yur" => YUR,
"yut" => YUT,
"yuw" => YUW,
"yux" => YUX,
"yuy" => YUY,
"yuz" => YUZ,
"yva" => YVA,
"yvt" => YVT,
"ywa" => YWA,
"ywg" => YWG,
"ywl" => YWL,
"ywn" => YWN,
"ywq" => YWQ,
"ywr" => YWR,
"ywt" => YWT,
"ywu" => YWU,
"yww" => YWW,
"yxa" => YXA,
"yxg" => YXG,
"yxl" => YXL,
"yxm" => YXM,
"yxu" => YXU,
"yxy" => YXY,
"yyr" => YYR,
"yyu" => YYU,
"yyz" => YYZ,
"yzg" => YZG,
"yzk" => YZK,
"zaa" => ZAA,
"zab" => ZAB,
"zac" => ZAC,
"zad" => ZAD,
"zae" => ZAE,
"zaf" => ZAF,
"zag" => ZAG,
"zah" => ZAH,
"zai" => ZAI,
"zaj" => ZAJ,
"zak" => ZAK,
"zal" => ZAL,
"zam" => ZAM,
"zao" => ZAO,
"zaq" => ZAQ,
"zar" => ZAR,
"zas" => ZAS,
"zat" => ZAT,
"zau" => ZAU,
"zav" => ZAV,
"zaw" => ZAW,
"zax" => ZAX,
"zay" => ZAY,
"zaz" => ZAZ,
"zba" => ZBA,
"zbc" => ZBC,
"zbe" => ZBE,
"zbt" => ZBT,
"zbu" => ZBU,
"zbw" => ZBW,
"zca" => ZCA,
"zcd" => ZCD,
"zch" => ZCH,
"zdj" => ZDJ,
"zea" => ZEA,
"zeg" => ZEG,
"zeh" => ZEH,
"zga" => ZGA,
"zgb" => ZGB,
"zgm" => ZGM,
"zgn" => ZGN,
"zgr" => ZGR,
"zhb" => ZHB,
"zhd" => ZHD,
"zhi" => ZHI,
"zhn" => ZHN,
"zhw" => ZHW,
"zia" => ZIA,
"zib" => ZIB,
"zik" => ZIK,
"zil" => ZIL,
"zim" => ZIM,
"zin" => ZIN,
"ziw" => ZIW,
"ziz" => ZIZ,
"zka" => ZKA,
"zkb" => ZKB,
"zkd" => ZKD,
"zkg" => ZKG,
"zkh" => ZKH,
"zkk" => ZKK,
"zkn" => ZKN,
"zko" => ZKO,
"zkp" => ZKP,
"zkr" => ZKR,
"zkt" => ZKT,
"zku" => ZKU,
"zkv" => ZKV,
"zkz" => ZKZ,
"zla" => ZLA,
"zlj" => ZLJ,
"zlm" => ZLM,
"zln" => ZLN,
"zlq" => ZLQ,
"zma" => ZMA,
"zmb" => ZMB,
"zmc" => ZMC,
"zmd" => ZMD,
"zme" => ZME,
"zmf" => ZMF,
"zmg" => ZMG,
"zmh" => ZMH,
"zmi" => ZMI,
"zmj" => ZMJ,
"zmk" => ZMK,
"zml" => ZML,
"zmm" => ZMM,
"zmn" => ZMN,
"zmo" => ZMO,
"zmp" => ZMP,
"zmq" => ZMQ,
"zmr" => ZMR,
"zms" => ZMS,
"zmt" => ZMT,
"zmu" => ZMU,
"zmv" => ZMV,
"zmw" => ZMW,
"zmx" => ZMX,
"zmy" => ZMY,
"zmz" => ZMZ,
"zna" => ZNA,
"zne" => ZNE,
"zng" => ZNG,
"znk" => ZNK,
"zns" => ZNS,
"zoc" => ZOC,
"zoh" => ZOH,
"zom" => ZOM,
"zoo" => ZOO,
"zoq" => ZOQ,
"zor" => ZOR,
"zos" => ZOS,
"zpa" => ZPA,
"zpb" => ZPB,
"zpc" => ZPC,
"zpd" => ZPD,
"zpe" => ZPE,
"zpf" => ZPF,
"zpg" => ZPG,
"zph" => ZPH,
"zpi" => ZPI,
"zpj" => ZPJ,
"zpk" => ZPK,
"zpl" => ZPL,
"zpm" => ZPM,
"zpn" => ZPN,
"zpo" => ZPO,
"zpp" => ZPP,
"zpq" => ZPQ,
"zpr" => ZPR,
"zps" => ZPS,
"zpt" => ZPT,
"zpu" => ZPU,
"zpv" => ZPV,
"zpw" => ZPW,
"zpx" => ZPX,
"zpy" => ZPY,
"zpz" => ZPZ,
"zqe" => ZQE,
"zra" => ZRA,
"zrg" => ZRG,
"zrn" => ZRN,
"zro" => ZRO,
"zrp" => ZRP,
"zrs" => ZRS,
"zsa" => ZSA,
"zsk" => ZSK,
"zsl" => ZSL,
"zsm" => ZSM,
"zsr" => ZSR,
"zsu" => ZSU,
"zte" => ZTE,
"ztg" => ZTG,
"ztl" => ZTL,
"ztm" => ZTM,
"ztn" => ZTN,
"ztp" => ZTP,
"ztq" => ZTQ,
"zts" => ZTS,
"ztt" => ZTT,
"ztu" => ZTU,
"ztx" => ZTX,
"zty" => ZTY,
"zua" => ZUA,
"zuh" => ZUH,
"zum" => ZUM,
"zuy" => ZUY,
"zwa" => ZWA,
"zyb" => ZYB,
"zyg" => ZYG,
"zyj" => ZYJ,
"zyn" => ZYN,
"zyp" => ZYP,
"zzj" => ZZJ,

};


pub const ALL_1_MAP: Map<&str, LanguageCode> = phf_map! {
"ab" => AB,
"aa" => AA,
"af" => AF,
"ak" => AK,
"sq" => SQ,
"am" => AM,
"ar" => AR,
"an" => AN,
"hy" => HY,
"as" => AS,
"av" => AV,
"ae" => AE,
"ay" => AY,
"az" => AZ,
"bm" => BM,
"ba" => BA,
"eu" => EU,
"be" => BE,
"bn" => BN,
"bi" => BI,
"bs" => BS,
"br" => BR,
"bg" => BG,
"my" => MY,
"ca" => CA,
"ch" => CH,
"ce" => CE,
"ny" => NY,
"zh" => ZH,
"cu" => CU,
"cv" => CV,
"kw" => KW,
"co" => CO,
"cr" => CR,
"hr" => HR,
"cs" => CS,
"da" => DA,
"dv" => DV,
"nl" => NL,
"dz" => DZ,
"en" => EN,
"eo" => EO,
"et" => ET,
"ee" => EE,
"fo" => FO,
"fj" => FJ,
"fi" => FI,
"fr" => FR,
"fy" => FY,
"ff" => FF,
"gd" => GD,
"gl" => GL,
"lg" => LG,
"ka" => KA,
"de" => DE,
"el" => EL,
"kl" => KL,
"gn" => GN,
"gu" => GU,
"ht" => HT,
"ha" => HA,
"he" => HE,
"hz" => HZ,
"hi" => HI,
"ho" => HO,
"hu" => HU,
"is" => IS,
"io" => IO,
"ig" => IG,
"id" => ID,
"ia" => IA,
"ie" => IE,
"iu" => IU,
"ik" => IK,
"ga" => GA,
"it" => IT,
"ja" => JA,
"jv" => JV,
"kn" => KN,
"kr" => KR,
"ks" => KS,
"kk" => KK,
"km" => KM,
"ki" => KI,
"rw" => RW,
"ky" => KY,
"kv" => KV,
"kg" => KG,
"ko" => KO,
"kj" => KJ,
"ku" => KU,
"lo" => LO,
"la" => LA,
"lv" => LV,
"li" => LI,
"ln" => LN,
"lt" => LT,
"lu" => LU,
"lb" => LB,
"mk" => MK,
"mg" => MG,
"ms" => MS,
"ml" => ML,
"mt" => MT,
"gv" => GV,
"mi" => MI,
"mr" => MR,
"mh" => MH,
"mn" => MN,
"na" => NA,
"nv" => NV,
"nd" => ND,
"nr" => NR,
"ng" => NG,
"ne" => NE,
"no" => NO,
"nb" => NB,
"nn" => NN,
"ii" => II,
"oc" => OC,
"oj" => OJ,
"or" => OR,
"om" => OM,
"os" => OS,
"pi" => PI,
"ps" => PS,
"fa" => FA,
"pl" => PL,
"pt" => PT,
"pa" => PA,
"qu" => QU,
"ro" => RO,
"rm" => RM,
"rn" => RN,
"ru" => RU,
"se" => SE,
"sm" => SM,
"sg" => SG,
"sa" => SA,
"sc" => SC,
"sr" => SR,
"sn" => SN,
"sd" => SD,
"si" => SI,
"sk" => SK,
"sl" => SL,
"so" => SO,
"st" => ST,
"es" => ES,
"su" => SU,
"sw" => SW,
"ss" => SS,
"sv" => SV,
"tl" => TL,
"ty" => TY,
"tg" => TG,
"ta" => TA,
"tt" => TT,
"te" => TE,
"th" => TH,
"bo" => BO,
"ti" => TI,
"to" => TO,
"ts" => TS,
"tn" => TN,
"tr" => TR,
"tk" => TK,
"tw" => TW,
"ug" => UG,
"uk" => UK,
"ur" => UR,
"uz" => UZ,
"ve" => VE,
"vi" => VI,
"vo" => VO,
"wa" => WA,
"cy" => CY,
"wo" => WO,
"xh" => XH,
"yi" => YI,
"yo" => YO,
"za" => ZA,
"zu" => ZU,
"sh" => SH,

};


pub const ALL_2B_MAP: Map<&str, LanguageCode> = phf_map! {
"abk" => AB,
"aar" => AA,
"afr" => AF,
"aka" => AK,
"alb" => SQ,
"amh" => AM,
"ara" => AR,
"arg" => AN,
"arm" => HY,
"asm" => AS,
"ava" => AV,
"ave" => AE,
"aym" => AY,
"aze" => AZ,
"bam" => BM,
"bak" => BA,
"baq" => EU,
"bel" => BE,
"ben" => BN,
"bis" => BI,
"bos" => BS,
"bre" => BR,
"bul" => BG,
"bur" => MY,
"cat" => CA,
"cha" => CH,
"che" => CE,
"nya" => NY,
"chi" => ZH,
"chu" => CU,
"chv" => CV,
"cor" => KW,
"cos" => CO,
"cre" => CR,
"hrv" => HR,
"cze" => CS,
"dan" => DA,
"div" => DV,
"dut" => NL,
"dzo" => DZ,
"eng" => EN,
"epo" => EO,
"est" => ET,
"ewe" => EE,
"fao" => FO,
"fij" => FJ,
"fin" => FI,
"fre" => FR,
"fry" => FY,
"ful" => FF,
"gla" => GD,
"glg" => GL,
"lug" => LG,
"geo" => KA,
"ger" => DE,
"gre" => EL,
"kal" => KL,
"grn" => GN,
"guj" => GU,
"hat" => HT,
"hau" => HA,
"heb" => HE,
"her" => HZ,
"hin" => HI,
"hmo" => HO,
"hun" => HU,
"ice" => IS,
"ido" => IO,
"ibo" => IG,
"ind" => ID,
"ina" => IA,
"ile" => IE,
"iku" => IU,
"ipk" => IK,
"gle" => GA,
"ita" => IT,
"jpn" => JA,
"jav" => JV,
"kan" => KN,
"kau" => KR,
"kas" => KS,
"kaz" => KK,
"khm" => KM,
"kik" => KI,
"kin" => RW,
"kir" => KY,
"kom" => KV,
"kon" => KG,
"kor" => KO,
"kua" => KJ,
"kur" => KU,
"lao" => LO,
"lat" => LA,
"lav" => LV,
"lim" => LI,
"lin" => LN,
"lit" => LT,
"lub" => LU,
"ltz" => LB,
"mac" => MK,
"mlg" => MG,
"may" => MS,
"mal" => ML,
"mlt" => MT,
"glv" => GV,
"mao" => MI,
"mar" => MR,
"mah" => MH,
"mon" => MN,
"nau" => NA,
"nav" => NV,
"nde" => ND,
"nbl" => NR,
"ndo" => NG,
"nep" => NE,
"nor" => NO,
"nob" => NB,
"nno" => NN,
"iii" => II,
"oci" => OC,
"oji" => OJ,
"ori" => OR,
"orm" => OM,
"oss" => OS,
"pli" => PI,
"pus" => PS,
"per" => FA,
"pol" => PL,
"por" => PT,
"pan" => PA,
"que" => QU,
"rum" => RO,
"roh" => RM,
"run" => RN,
"rus" => RU,
"sme" => SE,
"smo" => SM,
"sag" => SG,
"san" => SA,
"srd" => SC,
"srp" => SR,
"sna" => SN,
"snd" => SD,
"sin" => SI,
"slo" => SK,
"slv" => SL,
"som" => SO,
"sot" => ST,
"spa" => ES,
"sun" => SU,
"swa" => SW,
"ssw" => SS,
"swe" => SV,
"tgl" => TL,
"tah" => TY,
"tgk" => TG,
"tam" => TA,
"tat" => TT,
"tel" => TE,
"tha" => TH,
"tib" => BO,
"tir" => TI,
"ton" => TO,
"tso" => TS,
"tsn" => TN,
"tur" => TR,
"tuk" => TK,
"twi" => TW,
"uig" => UG,
"ukr" => UK,
"urd" => UR,
"uzb" => UZ,
"ven" => VE,
"vie" => VI,
"vol" => VO,
"wln" => WA,
"wel" => CY,
"wol" => WO,
"xho" => XH,
"yid" => YI,
"yor" => YO,
"zha" => ZA,
"zul" => ZU,
"ace" => ACE,
"ach" => ACH,
"ada" => ADA,
"ady" => ADY,
"afa" => AFA,
"afh" => AFH,
"ain" => AIN,
"akk" => AKK,
"ale" => ALE,
"alg" => ALG,
"alt" => ALT,
"ang" => ANG,
"anp" => ANP,
"apa" => APA,
"arc" => ARC,
"arn" => ARN,
"arp" => ARP,
"art" => ART,
"arw" => ARW,
"ast" => AST,
"ath" => ATH,
"aus" => AUS,
"awa" => AWA,
"bad" => BAD,
"bai" => BAI,
"bal" => BAL,
"ban" => BAN,
"bas" => BAS,
"bat" => BAT,
"bej" => BEJ,
"bem" => BEM,
"ber" => BER,
"bho" => BHO,
"bih" => BIH,
"bik" => BIK,
"bin" => BIN,
"bla" => BLA,
"bnt" => BNT,
"bra" => BRA,
"btk" => BTK,
"bua" => BUA,
"bug" => BUG,
"byn" => BYN,
"cad" => CAD,
"cai" => CAI,
"car" => CAR,
"cau" => CAU,
"ceb" => CEB,
"cel" => CEL,
"chb" => CHB,
"chg" => CHG,
"chk" => CHK,
"chm" => CHM,
"chn" => CHN,
"cho" => CHO,
"chp" => CHP,
"chr" => CHR,
"chy" => CHY,
"cmc" => CMC,
"cnr" => CNR,
"cop" => COP,
"cpe" => CPE,
"cpf" => CPF,
"cpp" => CPP,
"crh" => CRH,
"crp" => CRP,
"csb" => CSB,
"cus" => CUS,
"dak" => DAK,
"dar" => DAR,
"day" => DAY,
"del" => DEL,
"den" => DEN,
"dgr" => DGR,
"din" => DIN,
"doi" => DOI,
"dra" => DRA,
"dsb" => DSB,
"dua" => DUA,
"dum" => DUM,
"dyu" => DYU,
"efi" => EFI,
"egy" => EGY,
"eka" => EKA,
"elx" => ELX,
"enm" => ENM,
"ewo" => EWO,
"fan" => FAN,
"fat" => FAT,
"fil" => FIL,
"fiu" => FIU,
"fon" => FON,
"frm" => FRM,
"fro" => FRO,
"frr" => FRR,
"frs" => FRS,
"fur" => FUR,
"gaa" => GAA,
"gay" => GAY,
"gba" => GBA,
"gem" => GEM,
"gez" => GEZ,
"gil" => GIL,
"gmh" => GMH,
"goh" => GOH,
"gon" => GON,
"gor" => GOR,
"got" => GOT,
"grb" => GRB,
"grc" => GRC,
"gsw" => GSW,
"gwi" => GWI,
"hai" => HAI,
"haw" => HAW,
"hil" => HIL,
"him" => HIM,
"hit" => HIT,
"hmn" => HMN,
"hsb" => HSB,
"hup" => HUP,
"iba" => IBA,
"ijo" => IJO,
"ilo" => ILO,
"inc" => INC,
"ine" => INE,
"inh" => INH,
"ira" => IRA,
"iro" => IRO,
"jbo" => JBO,
"jpr" => JPR,
"jrb" => JRB,
"kaa" => KAA,
"kab" => KAB,
"kac" => KAC,
"kam" => KAM,
"kar" => KAR,
"kaw" => KAW,
"kbd" => KBD,
"kha" => KHA,
"khi" => KHI,
"kho" => KHO,
"kmb" => KMB,
"kok" => KOK,
"kos" => KOS,
"kpe" => KPE,
"krc" => KRC,
"krl" => KRL,
"kro" => KRO,
"kru" => KRU,
"kum" => KUM,
"kut" => KUT,
"lad" => LAD,
"lah" => LAH,
"lam" => LAM,
"lez" => LEZ,
"lol" => LOL,
"loz" => LOZ,
"lua" => LUA,
"lui" => LUI,
"lun" => LUN,
"luo" => LUO,
"lus" => LUS,
"mad" => MAD,
"mag" => MAG,
"mai" => MAI,
"mak" => MAK,
"man" => MAN,
"map" => MAP,
"mas" => MAS,
"mdf" => MDF,
"mdr" => MDR,
"men" => MEN,
"mga" => MGA,
"mic" => MIC,
"min" => MIN,
"mis" => MIS,
"mkh" => MKH,
"mnc" => MNC,
"mni" => MNI,
"mno" => MNO,
"moh" => MOH,
"mos" => MOS,
"mul" => MUL,
"mun" => MUN,
"mus" => MUS,
"mwl" => MWL,
"mwr" => MWR,
"myn" => MYN,
"myv" => MYV,
"nah" => NAH,
"nai" => NAI,
"nap" => NAP,
"nds" => NDS,
"new" => NEW,
"nia" => NIA,
"nic" => NIC,
"niu" => NIU,
"nog" => NOG,
"non" => NON,
"nqo" => NQO,
"nso" => NSO,
"nub" => NUB,
"nwc" => NWC,
"nym" => NYM,
"nyn" => NYN,
"nyo" => NYO,
"nzi" => NZI,
"osa" => OSA,
"ota" => OTA,
"oto" => OTO,
"paa" => PAA,
"pag" => PAG,
"pal" => PAL,
"pam" => PAM,
"pap" => PAP,
"pau" => PAU,
"peo" => PEO,
"phi" => PHI,
"phn" => PHN,
"pon" => PON,
"pra" => PRA,
"pro" => PRO,
"qaa-qtz" => QAA,
"raj" => RAJ,
"rap" => RAP,
"rar" => RAR,
"roa" => ROA,
"rom" => ROM,
"rup" => RUP,
"sad" => SAD,
"sah" => SAH,
"sai" => SAI,
"sal" => SAL,
"sam" => SAM,
"sas" => SAS,
"sat" => SAT,
"scn" => SCN,
"sco" => SCO,
"sel" => SEL,
"sem" => SEM,
"sga" => SGA,
"sgn" => SGN,
"shn" => SHN,
"sid" => SID,
"sio" => SIO,
"sit" => SIT,
"sla" => SLA,
"sma" => SMA,
"smi" => SMI,
"smj" => SMJ,
"smn" => SMN,
"sms" => SMS,
"snk" => SNK,
"sog" => SOG,
"son" => SON,
"srn" => SRN,
"srr" => SRR,
"ssa" => SSA,
"suk" => SUK,
"sus" => SUS,
"sux" => SUX,
"syc" => SYC,
"syr" => SYR,
"tai" => TAI,
"tem" => TEM,
"ter" => TER,
"tet" => TET,
"tig" => TIG,
"tiv" => TIV,
"tkl" => TKL,
"tlh" => TLH,
"tli" => TLI,
"tmh" => TMH,
"tog" => TOG,
"tpi" => TPI,
"tsi" => TSI,
"tum" => TUM,
"tup" => TUP,
"tut" => TUT,
"tvl" => TVL,
"tyv" => TYV,
"udm" => UDM,
"uga" => UGA,
"umb" => UMB,
"und" => UND,
"vai" => VAI,
"vot" => VOT,
"wak" => WAK,
"wal" => WAL,
"war" => WAR,
"was" => WAS,
"wen" => WEN,
"xal" => XAL,
"yao" => YAO,
"yap" => YAP,
"ypk" => YPK,
"zap" => ZAP,
"zbl" => ZBL,
"zen" => ZEN,
"zgh" => ZGH,
"znd" => ZND,
"zun" => ZUN,
"zxx" => ZXX,
"zza" => ZZA,
"zcd" => ZCD,

};


pub const ALL_2T_MAP: Map<&str, LanguageCode> = phf_map! {
"abk" => AB,
"aar" => AA,
"afr" => AF,
"aka" => AK,
"amh" => AM,
"ara" => AR,
"arg" => AN,
"asm" => AS,
"ava" => AV,
"ave" => AE,
"aym" => AY,
"aze" => AZ,
"bam" => BM,
"bak" => BA,
"bel" => BE,
"ben" => BN,
"bis" => BI,
"bos" => BS,
"bre" => BR,
"bul" => BG,
"cat" => CA,
"cha" => CH,
"che" => CE,
"nya" => NY,
"chu" => CU,
"chv" => CV,
"cor" => KW,
"cos" => CO,
"cre" => CR,
"hrv" => HR,
"dan" => DA,
"div" => DV,
"dzo" => DZ,
"eng" => EN,
"epo" => EO,
"est" => ET,
"ewe" => EE,
"fao" => FO,
"fij" => FJ,
"fin" => FI,
"fry" => FY,
"ful" => FF,
"gla" => GD,
"glg" => GL,
"lug" => LG,
"kal" => KL,
"grn" => GN,
"guj" => GU,
"hat" => HT,
"hau" => HA,
"heb" => HE,
"her" => HZ,
"hin" => HI,
"hmo" => HO,
"hun" => HU,
"ido" => IO,
"ibo" => IG,
"ind" => ID,
"ina" => IA,
"ile" => IE,
"iku" => IU,
"ipk" => IK,
"gle" => GA,
"ita" => IT,
"jpn" => JA,
"jav" => JV,
"kan" => KN,
"kau" => KR,
"kas" => KS,
"kaz" => KK,
"khm" => KM,
"kik" => KI,
"kin" => RW,
"kir" => KY,
"kom" => KV,
"kon" => KG,
"kor" => KO,
"kua" => KJ,
"kur" => KU,
"lao" => LO,
"lat" => LA,
"lav" => LV,
"lim" => LI,
"lin" => LN,
"lit" => LT,
"lub" => LU,
"ltz" => LB,
"mlg" => MG,
"mal" => ML,
"mlt" => MT,
"glv" => GV,
"mar" => MR,
"mah" => MH,
"mon" => MN,
"nau" => NA,
"nav" => NV,
"nde" => ND,
"nbl" => NR,
"ndo" => NG,
"nep" => NE,
"nor" => NO,
"nob" => NB,
"nno" => NN,
"iii" => II,
"oci" => OC,
"oji" => OJ,
"ori" => OR,
"orm" => OM,
"oss" => OS,
"pli" => PI,
"pus" => PS,
"pol" => PL,
"por" => PT,
"pan" => PA,
"que" => QU,
"roh" => RM,
"run" => RN,
"rus" => RU,
"sme" => SE,
"smo" => SM,
"sag" => SG,
"san" => SA,
"srd" => SC,
"srp" => SR,
"sna" => SN,
"snd" => SD,
"sin" => SI,
"slv" => SL,
"som" => SO,
"sot" => ST,
"spa" => ES,
"sun" => SU,
"swa" => SW,
"ssw" => SS,
"swe" => SV,
"tgl" => TL,
"tah" => TY,
"tgk" => TG,
"tam" => TA,
"tat" => TT,
"tel" => TE,
"tha" => TH,
"tir" => TI,
"ton" => TO,
"tso" => TS,
"tsn" => TN,
"tur" => TR,
"tuk" => TK,
"twi" => TW,
"uig" => UG,
"ukr" => UK,
"urd" => UR,
"uzb" => UZ,
"ven" => VE,
"vie" => VI,
"vol" => VO,
"wln" => WA,
"wol" => WO,
"xho" => XH,
"yid" => YI,
"yor" => YO,
"zha" => ZA,
"zul" => ZU,
"ace" => ACE,
"ach" => ACH,
"ada" => ADA,
"ady" => ADY,
"afa" => AFA,
"afh" => AFH,
"ain" => AIN,
"akk" => AKK,
"ale" => ALE,
"alg" => ALG,
"alt" => ALT,
"ang" => ANG,
"anp" => ANP,
"apa" => APA,
"arc" => ARC,
"arn" => ARN,
"arp" => ARP,
"art" => ART,
"arw" => ARW,
"ast" => AST,
"ath" => ATH,
"aus" => AUS,
"awa" => AWA,
"bad" => BAD,
"bai" => BAI,
"bal" => BAL,
"ban" => BAN,
"bas" => BAS,
"bat" => BAT,
"bej" => BEJ,
"bem" => BEM,
"ber" => BER,
"bho" => BHO,
"bih" => BIH,
"bik" => BIK,
"bin" => BIN,
"bla" => BLA,
"bnt" => BNT,
"bra" => BRA,
"btk" => BTK,
"bua" => BUA,
"bug" => BUG,
"byn" => BYN,
"cad" => CAD,
"cai" => CAI,
"car" => CAR,
"cau" => CAU,
"ceb" => CEB,
"cel" => CEL,
"chb" => CHB,
"chg" => CHG,
"chk" => CHK,
"chm" => CHM,
"chn" => CHN,
"cho" => CHO,
"chp" => CHP,
"chr" => CHR,
"chy" => CHY,
"cmc" => CMC,
"cnr" => CNR,
"cop" => COP,
"cpe" => CPE,
"cpf" => CPF,
"cpp" => CPP,
"crh" => CRH,
"crp" => CRP,
"csb" => CSB,
"cus" => CUS,
"dak" => DAK,
"dar" => DAR,
"day" => DAY,
"del" => DEL,
"den" => DEN,
"dgr" => DGR,
"din" => DIN,
"doi" => DOI,
"dra" => DRA,
"dsb" => DSB,
"dua" => DUA,
"dum" => DUM,
"dyu" => DYU,
"efi" => EFI,
"egy" => EGY,
"eka" => EKA,
"elx" => ELX,
"enm" => ENM,
"ewo" => EWO,
"fan" => FAN,
"fat" => FAT,
"fil" => FIL,
"fiu" => FIU,
"fon" => FON,
"frm" => FRM,
"fro" => FRO,
"frr" => FRR,
"frs" => FRS,
"fur" => FUR,
"gaa" => GAA,
"gay" => GAY,
"gba" => GBA,
"gem" => GEM,
"gez" => GEZ,
"gil" => GIL,
"gmh" => GMH,
"goh" => GOH,
"gon" => GON,
"gor" => GOR,
"got" => GOT,
"grb" => GRB,
"grc" => GRC,
"gsw" => GSW,
"gwi" => GWI,
"hai" => HAI,
"haw" => HAW,
"hil" => HIL,
"him" => HIM,
"hit" => HIT,
"hmn" => HMN,
"hsb" => HSB,
"hup" => HUP,
"iba" => IBA,
"ijo" => IJO,
"ilo" => ILO,
"inc" => INC,
"ine" => INE,
"inh" => INH,
"ira" => IRA,
"iro" => IRO,
"jbo" => JBO,
"jpr" => JPR,
"jrb" => JRB,
"kaa" => KAA,
"kab" => KAB,
"kac" => KAC,
"kam" => KAM,
"kar" => KAR,
"kaw" => KAW,
"kbd" => KBD,
"kha" => KHA,
"khi" => KHI,
"kho" => KHO,
"kmb" => KMB,
"kok" => KOK,
"kos" => KOS,
"kpe" => KPE,
"krc" => KRC,
"krl" => KRL,
"kro" => KRO,
"kru" => KRU,
"kum" => KUM,
"kut" => KUT,
"lad" => LAD,
"lah" => LAH,
"lam" => LAM,
"lez" => LEZ,
"lol" => LOL,
"loz" => LOZ,
"lua" => LUA,
"lui" => LUI,
"lun" => LUN,
"luo" => LUO,
"lus" => LUS,
"mad" => MAD,
"mag" => MAG,
"mai" => MAI,
"mak" => MAK,
"man" => MAN,
"map" => MAP,
"mas" => MAS,
"mdf" => MDF,
"mdr" => MDR,
"men" => MEN,
"mga" => MGA,
"mic" => MIC,
"min" => MIN,
"mis" => MIS,
"mkh" => MKH,
"mnc" => MNC,
"mni" => MNI,
"mno" => MNO,
"moh" => MOH,
"mos" => MOS,
"mul" => MUL,
"mun" => MUN,
"mus" => MUS,
"mwl" => MWL,
"mwr" => MWR,
"myn" => MYN,
"myv" => MYV,
"nah" => NAH,
"nai" => NAI,
"nap" => NAP,
"nds" => NDS,
"new" => NEW,
"nia" => NIA,
"nic" => NIC,
"niu" => NIU,
"nog" => NOG,
"non" => NON,
"nqo" => NQO,
"nso" => NSO,
"nub" => NUB,
"nwc" => NWC,
"nym" => NYM,
"nyn" => NYN,
"nyo" => NYO,
"nzi" => NZI,
"osa" => OSA,
"ota" => OTA,
"oto" => OTO,
"paa" => PAA,
"pag" => PAG,
"pal" => PAL,
"pam" => PAM,
"pap" => PAP,
"pau" => PAU,
"peo" => PEO,
"phi" => PHI,
"phn" => PHN,
"pon" => PON,
"pra" => PRA,
"pro" => PRO,
"qaa-qtz" => QAA,
"raj" => RAJ,
"rap" => RAP,
"rar" => RAR,
"roa" => ROA,
"rom" => ROM,
"rup" => RUP,
"sad" => SAD,
"sah" => SAH,
"sai" => SAI,
"sal" => SAL,
"sam" => SAM,
"sas" => SAS,
"sat" => SAT,
"scn" => SCN,
"sco" => SCO,
"sel" => SEL,
"sem" => SEM,
"sga" => SGA,
"sgn" => SGN,
"shn" => SHN,
"sid" => SID,
"sio" => SIO,
"sit" => SIT,
"sla" => SLA,
"sma" => SMA,
"smi" => SMI,
"smj" => SMJ,
"smn" => SMN,
"sms" => SMS,
"snk" => SNK,
"sog" => SOG,
"son" => SON,
"srn" => SRN,
"srr" => SRR,
"ssa" => SSA,
"suk" => SUK,
"sus" => SUS,
"sux" => SUX,
"syc" => SYC,
"syr" => SYR,
"tai" => TAI,
"tem" => TEM,
"ter" => TER,
"tet" => TET,
"tig" => TIG,
"tiv" => TIV,
"tkl" => TKL,
"tlh" => TLH,
"tli" => TLI,
"tmh" => TMH,
"tog" => TOG,
"tpi" => TPI,
"tsi" => TSI,
"tum" => TUM,
"tup" => TUP,
"tut" => TUT,
"tvl" => TVL,
"tyv" => TYV,
"udm" => UDM,
"uga" => UGA,
"umb" => UMB,
"und" => UND,
"vai" => VAI,
"vot" => VOT,
"wak" => WAK,
"wal" => WAL,
"war" => WAR,
"was" => WAS,
"wen" => WEN,
"xal" => XAL,
"yao" => YAO,
"yap" => YAP,
"ypk" => YPK,
"zap" => ZAP,
"zbl" => ZBL,
"zen" => ZEN,
"zgh" => ZGH,
"znd" => ZND,
"zun" => ZUN,
"zxx" => ZXX,
"zza" => ZZA,
"zcd" => ZCD,

};


pub const ALL_3_MAP: Map<&str, LanguageCode> = phf_map! {
"abk" => AB,
"aar" => AA,
"afr" => AF,
"aka" => AK,
"sqi" => SQ,
"amh" => AM,
"ara" => AR,
"arg" => AN,
"hye" => HY,
"asm" => AS,
"ava" => AV,
"ave" => AE,
"aym" => AY,
"aze" => AZ,
"bam" => BM,
"bak" => BA,
"eus" => EU,
"bel" => BE,
"ben" => BN,
"bis" => BI,
"bos" => BS,
"bre" => BR,
"bul" => BG,
"mya" => MY,
"cat" => CA,
"cha" => CH,
"che" => CE,
"nya" => NY,
"zho" => ZH,
"chu" => CU,
"chv" => CV,
"cor" => KW,
"cos" => CO,
"cre" => CR,
"hrv" => HR,
"ces" => CS,
"dan" => DA,
"div" => DV,
"nld" => NL,
"dzo" => DZ,
"eng" => EN,
"epo" => EO,
"est" => ET,
"ewe" => EE,
"fao" => FO,
"fij" => FJ,
"fin" => FI,
"fra" => FR,
"fry" => FY,
"ful" => FF,
"gla" => GD,
"glg" => GL,
"lug" => LG,
"kat" => KA,
"deu" => DE,
"ell" => EL,
"kal" => KL,
"grn" => GN,
"guj" => GU,
"hat" => HT,
"hau" => HA,
"heb" => HE,
"her" => HZ,
"hin" => HI,
"hmo" => HO,
"hun" => HU,
"isl" => IS,
"ido" => IO,
"ibo" => IG,
"ind" => ID,
"ina" => IA,
"ile" => IE,
"iku" => IU,
"ipk" => IK,
"gle" => GA,
"ita" => IT,
"jpn" => JA,
"jav" => JV,
"kan" => KN,
"kau" => KR,
"kas" => KS,
"kaz" => KK,
"khm" => KM,
"kik" => KI,
"kin" => RW,
"kir" => KY,
"kom" => KV,
"kon" => KG,
"kor" => KO,
"kua" => KJ,
"kur" => KU,
"lao" => LO,
"lat" => LA,
"lav" => LV,
"lim" => LI,
"lin" => LN,
"lit" => LT,
"lub" => LU,
"ltz" => LB,
"mkd" => MK,
"mlg" => MG,
"msa" => MS,
"mal" => ML,
"mlt" => MT,
"glv" => GV,
"mri" => MI,
"mar" => MR,
"mah" => MH,
"mon" => MN,
"nau" => NA,
"nav" => NV,
"nde" => ND,
"nbl" => NR,
"ndo" => NG,
"nep" => NE,
"nor" => NO,
"nob" => NB,
"nno" => NN,
"iii" => II,
"oci" => OC,
"oji" => OJ,
"ori" => OR,
"orm" => OM,
"oss" => OS,
"pli" => PI,
"pus" => PS,
"fas" => FA,
"pol" => PL,
"por" => PT,
"pan" => PA,
"que" => QU,
"ron" => RO,
"roh" => RM,
"run" => RN,
"rus" => RU,
"sme" => SE,
"smo" => SM,
"sag" => SG,
"san" => SA,
"srd" => SC,
"srp" => SR,
"sna" => SN,
"snd" => SD,
"sin" => SI,
"slk" => SK,
"slv" => SL,
"som" => SO,
"sot" => ST,
"spa" => ES,
"sun" => SU,
"swa" => SW,
"ssw" => SS,
"swe" => SV,
"tgl" => TL,
"tah" => TY,
"tgk" => TG,
"tam" => TA,
"tat" => TT,
"tel" => TE,
"tha" => TH,
"bod" => BO,
"tir" => TI,
"ton" => TO,
"tso" => TS,
"tsn" => TN,
"tur" => TR,
"tuk" => TK,
"twi" => TW,
"uig" => UG,
"ukr" => UK,
"urd" => UR,
"uzb" => UZ,
"ven" => VE,
"vie" => VI,
"vol" => VO,
"wln" => WA,
"cym" => CY,
"wol" => WO,
"xho" => XH,
"yid" => YI,
"yor" => YO,
"zha" => ZA,
"zul" => ZU,
"ace" => ACE,
"ach" => ACH,
"ada" => ADA,
"ady" => ADY,
"afh" => AFH,
"ain" => AIN,
"akk" => AKK,
"ale" => ALE,
"alt" => ALT,
"ang" => ANG,
"anp" => ANP,
"arc" => ARC,
"arn" => ARN,
"arp" => ARP,
"arw" => ARW,
"ast" => AST,
"awa" => AWA,
"bal" => BAL,
"ban" => BAN,
"bas" => BAS,
"bej" => BEJ,
"bem" => BEM,
"bho" => BHO,
"bik" => BIK,
"bin" => BIN,
"bla" => BLA,
"bra" => BRA,
"bua" => BUA,
"bug" => BUG,
"byn" => BYN,
"cad" => CAD,
"car" => CAR,
"ceb" => CEB,
"chb" => CHB,
"chg" => CHG,
"chk" => CHK,
"chm" => CHM,
"chn" => CHN,
"cho" => CHO,
"chp" => CHP,
"chr" => CHR,
"chy" => CHY,
"cnr" => CNR,
"cop" => COP,
"crh" => CRH,
"csb" => CSB,
"dak" => DAK,
"dar" => DAR,
"del" => DEL,
"den" => DEN,
"dgr" => DGR,
"din" => DIN,
"doi" => DOI,
"dsb" => DSB,
"dua" => DUA,
"dum" => DUM,
"dyu" => DYU,
"efi" => EFI,
"egy" => EGY,
"eka" => EKA,
"elx" => ELX,
"enm" => ENM,
"ewo" => EWO,
"fan" => FAN,
"fat" => FAT,
"fil" => FIL,
"fon" => FON,
"frm" => FRM,
"fro" => FRO,
"frr" => FRR,
"frs" => FRS,
"fur" => FUR,
"gaa" => GAA,
"gay" => GAY,
"gba" => GBA,
"gez" => GEZ,
"gil" => GIL,
"gmh" => GMH,
"goh" => GOH,
"gon" => GON,
"gor" => GOR,
"got" => GOT,
"grb" => GRB,
"grc" => GRC,
"gsw" => GSW,
"gwi" => GWI,
"hai" => HAI,
"haw" => HAW,
"hil" => HIL,
"hit" => HIT,
"hmn" => HMN,
"hsb" => HSB,
"hup" => HUP,
"iba" => IBA,
"ilo" => ILO,
"inh" => INH,
"jbo" => JBO,
"jpr" => JPR,
"jrb" => JRB,
"kaa" => KAA,
"kab" => KAB,
"kac" => KAC,
"kam" => KAM,
"kaw" => KAW,
"kbd" => KBD,
"kha" => KHA,
"kho" => KHO,
"kmb" => KMB,
"kok" => KOK,
"kos" => KOS,
"kpe" => KPE,
"krc" => KRC,
"krl" => KRL,
"kru" => KRU,
"kum" => KUM,
"kut" => KUT,
"lad" => LAD,
"lah" => LAH,
"lam" => LAM,
"lez" => LEZ,
"lol" => LOL,
"loz" => LOZ,
"lua" => LUA,
"lui" => LUI,
"lun" => LUN,
"luo" => LUO,
"lus" => LUS,
"mad" => MAD,
"mag" => MAG,
"mai" => MAI,
"mak" => MAK,
"man" => MAN,
"mas" => MAS,
"mdf" => MDF,
"mdr" => MDR,
"men" => MEN,
"mga" => MGA,
"mic" => MIC,
"min" => MIN,
"mis" => MIS,
"mnc" => MNC,
"mni" => MNI,
"moh" => MOH,
"mos" => MOS,
"mul" => MUL,
"mus" => MUS,
"mwl" => MWL,
"mwr" => MWR,
"myv" => MYV,
"nap" => NAP,
"nds" => NDS,
"new" => NEW,
"nia" => NIA,
"niu" => NIU,
"nog" => NOG,
"non" => NON,
"nqo" => NQO,
"nso" => NSO,
"nwc" => NWC,
"nym" => NYM,
"nyn" => NYN,
"nyo" => NYO,
"nzi" => NZI,
"osa" => OSA,
"ota" => OTA,
"pag" => PAG,
"pal" => PAL,
"pam" => PAM,
"pap" => PAP,
"pau" => PAU,
"peo" => PEO,
"phn" => PHN,
"pon" => PON,
"pro" => PRO,
"qaa-qtz" => QAA,
"raj" => RAJ,
"rap" => RAP,
"rar" => RAR,
"rom" => ROM,
"rup" => RUP,
"sad" => SAD,
"sah" => SAH,
"sam" => SAM,
"sas" => SAS,
"sat" => SAT,
"scn" => SCN,
"sco" => SCO,
"sel" => SEL,
"sga" => SGA,
"shn" => SHN,
"sid" => SID,
"sma" => SMA,
"smj" => SMJ,
"smn" => SMN,
"sms" => SMS,
"snk" => SNK,
"sog" => SOG,
"srn" => SRN,
"srr" => SRR,
"suk" => SUK,
"sus" => SUS,
"sux" => SUX,
"syc" => SYC,
"syr" => SYR,
"tem" => TEM,
"ter" => TER,
"tet" => TET,
"tig" => TIG,
"tiv" => TIV,
"tkl" => TKL,
"tlh" => TLH,
"tli" => TLI,
"tmh" => TMH,
"tog" => TOG,
"tpi" => TPI,
"tsi" => TSI,
"tum" => TUM,
"tvl" => TVL,
"tyv" => TYV,
"udm" => UDM,
"uga" => UGA,
"umb" => UMB,
"und" => UND,
"vai" => VAI,
"vot" => VOT,
"wal" => WAL,
"war" => WAR,
"was" => WAS,
"xal" => XAL,
"yao" => YAO,
"yap" => YAP,
"zap" => ZAP,
"zbl" => ZBL,
"zen" => ZEN,
"zgh" => ZGH,
"zun" => ZUN,
"zxx" => ZXX,
"zza" => ZZA,
"aaa" => AAA,
"aab" => AAB,
"aac" => AAC,
"aad" => AAD,
"aae" => AAE,
"aaf" => AAF,
"aag" => AAG,
"aah" => AAH,
"aai" => AAI,
"aak" => AAK,
"aal" => AAL,
"aan" => AAN,
"aao" => AAO,
"aap" => AAP,
"aaq" => AAQ,
"aas" => AAS,
"aat" => AAT,
"aau" => AAU,
"aaw" => AAW,
"aax" => AAX,
"aaz" => AAZ,
"aba" => ABA,
"abb" => ABB,
"abc" => ABC,
"abd" => ABD,
"abe" => ABE,
"abf" => ABF,
"abg" => ABG,
"abh" => ABH,
"abi" => ABI,
"abj" => ABJ,
"abl" => ABL,
"abm" => ABM,
"abn" => ABN,
"abo" => ABO,
"abp" => ABP,
"abq" => ABQ,
"abr" => ABR,
"abs" => ABS,
"abt" => ABT,
"abu" => ABU,
"abv" => ABV,
"abw" => ABW,
"abx" => ABX,
"aby" => ABY,
"abz" => ABZ,
"aca" => ACA,
"acb" => ACB,
"acd" => ACD,
"acf" => ACF,
"aci" => ACI,
"ack" => ACK,
"acl" => ACL,
"acm" => ACM,
"acn" => ACN,
"acp" => ACP,
"acq" => ACQ,
"acr" => ACR,
"acs" => ACS,
"act" => ACT,
"acu" => ACU,
"acv" => ACV,
"acw" => ACW,
"acx" => ACX,
"acy" => ACY,
"acz" => ACZ,
"adb" => ADB,
"add" => ADD,
"ade" => ADE,
"adf" => ADF,
"adg" => ADG,
"adh" => ADH,
"adi" => ADI,
"adj" => ADJ,
"adl" => ADL,
"adn" => ADN,
"ado" => ADO,
"adq" => ADQ,
"adr" => ADR,
"ads" => ADS,
"adt" => ADT,
"adu" => ADU,
"adw" => ADW,
"adx" => ADX,
"adz" => ADZ,
"aea" => AEA,
"aeb" => AEB,
"aec" => AEC,
"aed" => AED,
"aee" => AEE,
"aek" => AEK,
"ael" => AEL,
"aem" => AEM,
"aen" => AEN,
"aeq" => AEQ,
"aer" => AER,
"aes" => AES,
"aeu" => AEU,
"aew" => AEW,
"aey" => AEY,
"aez" => AEZ,
"afb" => AFB,
"afd" => AFD,
"afe" => AFE,
"afg" => AFG,
"afi" => AFI,
"afk" => AFK,
"afn" => AFN,
"afo" => AFO,
"afp" => AFP,
"afs" => AFS,
"aft" => AFT,
"afu" => AFU,
"afz" => AFZ,
"aga" => AGA,
"agb" => AGB,
"agc" => AGC,
"agd" => AGD,
"age" => AGE,
"agf" => AGF,
"agg" => AGG,
"agh" => AGH,
"agi" => AGI,
"agj" => AGJ,
"agk" => AGK,
"agl" => AGL,
"agm" => AGM,
"agn" => AGN,
"ago" => AGO,
"agq" => AGQ,
"agr" => AGR,
"ags" => AGS,
"agt" => AGT,
"agu" => AGU,
"agv" => AGV,
"agw" => AGW,
"agx" => AGX,
"agy" => AGY,
"agz" => AGZ,
"aha" => AHA,
"ahb" => AHB,
"ahg" => AHG,
"ahh" => AHH,
"ahi" => AHI,
"ahk" => AHK,
"ahl" => AHL,
"ahm" => AHM,
"ahn" => AHN,
"aho" => AHO,
"ahp" => AHP,
"ahr" => AHR,
"ahs" => AHS,
"aht" => AHT,
"aia" => AIA,
"aib" => AIB,
"aic" => AIC,
"aid" => AID,
"aie" => AIE,
"aif" => AIF,
"aig" => AIG,
"aih" => AIH,
"aii" => AII,
"aij" => AIJ,
"aik" => AIK,
"ail" => AIL,
"aim" => AIM,
"aio" => AIO,
"aip" => AIP,
"aiq" => AIQ,
"air" => AIR,
"ait" => AIT,
"aiw" => AIW,
"aix" => AIX,
"aiy" => AIY,
"aja" => AJA,
"ajg" => AJG,
"aji" => AJI,
"ajn" => AJN,
"ajp" => AJP,
"ajs" => AJS,
"aju" => AJU,
"ajw" => AJW,
"ajz" => AJZ,
"akb" => AKB,
"akc" => AKC,
"akd" => AKD,
"ake" => AKE,
"akf" => AKF,
"akg" => AKG,
"akh" => AKH,
"aki" => AKI,
"akj" => AKJ,
"akl" => AKL,
"akm" => AKM,
"ako" => AKO,
"akp" => AKP,
"akq" => AKQ,
"akr" => AKR,
"aks" => AKS,
"akt" => AKT,
"aku" => AKU,
"akv" => AKV,
"akw" => AKW,
"akx" => AKX,
"aky" => AKY,
"akz" => AKZ,
"ala" => ALA,
"alc" => ALC,
"ald" => ALD,
"alf" => ALF,
"alh" => ALH,
"ali" => ALI,
"alj" => ALJ,
"alk" => ALK,
"all" => ALL,
"alm" => ALM,
"aln" => ALN,
"alo" => ALO,
"alp" => ALP,
"alq" => ALQ,
"alr" => ALR,
"als" => ALS,
"alu" => ALU,
"alw" => ALW,
"alx" => ALX,
"aly" => ALY,
"alz" => ALZ,
"ama" => AMA,
"amb" => AMB,
"amc" => AMC,
"ame" => AME,
"amf" => AMF,
"amg" => AMG,
"ami" => AMI,
"amj" => AMJ,
"amk" => AMK,
"aml" => AML,
"amm" => AMM,
"amn" => AMN,
"amo" => AMO,
"amp" => AMP,
"amq" => AMQ,
"amr" => AMR,
"ams" => AMS,
"amt" => AMT,
"amu" => AMU,
"amv" => AMV,
"amw" => AMW,
"amx" => AMX,
"amy" => AMY,
"amz" => AMZ,
"ana" => ANA,
"anb" => ANB,
"anc" => ANC,
"and" => AND,
"ane" => ANE,
"anf" => ANF,
"anh" => ANH,
"ani" => ANI,
"anj" => ANJ,
"ank" => ANK,
"anl" => ANL,
"anm" => ANM,
"ann" => ANN,
"ano" => ANO,
"anq" => ANQ,
"anr" => ANR,
"ans" => ANS,
"ant" => ANT,
"anu" => ANU,
"anv" => ANV,
"anw" => ANW,
"anx" => ANX,
"any" => ANY,
"anz" => ANZ,
"aoa" => AOA,
"aob" => AOB,
"aoc" => AOC,
"aod" => AOD,
"aoe" => AOE,
"aof" => AOF,
"aog" => AOG,
"aoi" => AOI,
"aoj" => AOJ,
"aok" => AOK,
"aol" => AOL,
"aom" => AOM,
"aon" => AON,
"aor" => AOR,
"aos" => AOS,
"aot" => AOT,
"aou" => AOU,
"aox" => AOX,
"aoz" => AOZ,
"apb" => APB,
"apc" => APC,
"apd" => APD,
"ape" => APE,
"apf" => APF,
"apg" => APG,
"aph" => APH,
"api" => API,
"apj" => APJ,
"apk" => APK,
"apl" => APL,
"apm" => APM,
"apn" => APN,
"apo" => APO,
"app" => APP,
"apq" => APQ,
"apr" => APR,
"aps" => APS,
"apt" => APT,
"apu" => APU,
"apv" => APV,
"apw" => APW,
"apx" => APX,
"apy" => APY,
"apz" => APZ,
"aqc" => AQC,
"aqd" => AQD,
"aqg" => AQG,
"aqk" => AQK,
"aqm" => AQM,
"aqn" => AQN,
"aqp" => AQP,
"aqr" => AQR,
"aqt" => AQT,
"aqz" => AQZ,
"arb" => ARB,
"ard" => ARD,
"are" => ARE,
"arh" => ARH,
"ari" => ARI,
"arj" => ARJ,
"ark" => ARK,
"arl" => ARL,
"aro" => ARO,
"arq" => ARQ,
"arr" => ARR,
"ars" => ARS,
"aru" => ARU,
"arv" => ARV,
"arx" => ARX,
"ary" => ARY,
"arz" => ARZ,
"asa" => ASA,
"asb" => ASB,
"asc" => ASC,
"ase" => ASE,
"asf" => ASF,
"asg" => ASG,
"ash" => ASH,
"asi" => ASI,
"asj" => ASJ,
"ask" => ASK,
"asl" => ASL,
"asn" => ASN,
"aso" => ASO,
"asp" => ASP,
"asq" => ASQ,
"asr" => ASR,
"ass" => ASS,
"asu" => ASU,
"asv" => ASV,
"asw" => ASW,
"asx" => ASX,
"asy" => ASY,
"asz" => ASZ,
"ata" => ATA,
"atb" => ATB,
"atc" => ATC,
"atd" => ATD,
"ate" => ATE,
"atg" => ATG,
"ati" => ATI,
"atj" => ATJ,
"atk" => ATK,
"atl" => ATL,
"atm" => ATM,
"atn" => ATN,
"ato" => ATO,
"atp" => ATP,
"atq" => ATQ,
"atr" => ATR,
"ats" => ATS,
"att" => ATT,
"atu" => ATU,
"atv" => ATV,
"atw" => ATW,
"atx" => ATX,
"aty" => ATY,
"atz" => ATZ,
"aua" => AUA,
"aub" => AUB,
"auc" => AUC,
"aud" => AUD,
"aug" => AUG,
"auh" => AUH,
"aui" => AUI,
"auj" => AUJ,
"auk" => AUK,
"aul" => AUL,
"aum" => AUM,
"aun" => AUN,
"auo" => AUO,
"aup" => AUP,
"auq" => AUQ,
"aur" => AUR,
"aut" => AUT,
"auu" => AUU,
"auw" => AUW,
"aux" => AUX,
"auy" => AUY,
"auz" => AUZ,
"avb" => AVB,
"avd" => AVD,
"avi" => AVI,
"avk" => AVK,
"avl" => AVL,
"avm" => AVM,
"avn" => AVN,
"avo" => AVO,
"avs" => AVS,
"avt" => AVT,
"avu" => AVU,
"avv" => AVV,
"awb" => AWB,
"awc" => AWC,
"awe" => AWE,
"awg" => AWG,
"awh" => AWH,
"awi" => AWI,
"awk" => AWK,
"awm" => AWM,
"awn" => AWN,
"awo" => AWO,
"awr" => AWR,
"aws" => AWS,
"awt" => AWT,
"awu" => AWU,
"awv" => AWV,
"aww" => AWW,
"awx" => AWX,
"awy" => AWY,
"axb" => AXB,
"axe" => AXE,
"axg" => AXG,
"axk" => AXK,
"axl" => AXL,
"axm" => AXM,
"axx" => AXX,
"aya" => AYA,
"ayb" => AYB,
"ayc" => AYC,
"ayd" => AYD,
"aye" => AYE,
"ayg" => AYG,
"ayh" => AYH,
"ayi" => AYI,
"ayk" => AYK,
"ayl" => AYL,
"ayn" => AYN,
"ayo" => AYO,
"ayp" => AYP,
"ayq" => AYQ,
"ayr" => AYR,
"ays" => AYS,
"ayt" => AYT,
"ayu" => AYU,
"ayz" => AYZ,
"aza" => AZA,
"azb" => AZB,
"azd" => AZD,
"azg" => AZG,
"azj" => AZJ,
"azm" => AZM,
"azn" => AZN,
"azo" => AZO,
"azt" => AZT,
"azz" => AZZ,
"baa" => BAA,
"bab" => BAB,
"bac" => BAC,
"bae" => BAE,
"baf" => BAF,
"bag" => BAG,
"bah" => BAH,
"baj" => BAJ,
"bao" => BAO,
"bap" => BAP,
"bar" => BAR,
"bau" => BAU,
"bav" => BAV,
"baw" => BAW,
"bax" => BAX,
"bay" => BAY,
"bba" => BBA,
"bbb" => BBB,
"bbc" => BBC,
"bbd" => BBD,
"bbe" => BBE,
"bbf" => BBF,
"bbg" => BBG,
"bbh" => BBH,
"bbi" => BBI,
"bbj" => BBJ,
"bbk" => BBK,
"bbl" => BBL,
"bbm" => BBM,
"bbn" => BBN,
"bbo" => BBO,
"bbp" => BBP,
"bbq" => BBQ,
"bbr" => BBR,
"bbs" => BBS,
"bbt" => BBT,
"bbu" => BBU,
"bbv" => BBV,
"bbw" => BBW,
"bbx" => BBX,
"bby" => BBY,
"bca" => BCA,
"bcb" => BCB,
"bcc" => BCC,
"bcd" => BCD,
"bce" => BCE,
"bcf" => BCF,
"bcg" => BCG,
"bch" => BCH,
"bci" => BCI,
"bcj" => BCJ,
"bck" => BCK,
"bcl" => BCL,
"bcm" => BCM,
"bcn" => BCN,
"bco" => BCO,
"bcp" => BCP,
"bcq" => BCQ,
"bcr" => BCR,
"bcs" => BCS,
"bct" => BCT,
"bcu" => BCU,
"bcv" => BCV,
"bcw" => BCW,
"bcy" => BCY,
"bcz" => BCZ,
"bda" => BDA,
"bdb" => BDB,
"bdc" => BDC,
"bdd" => BDD,
"bde" => BDE,
"bdf" => BDF,
"bdg" => BDG,
"bdh" => BDH,
"bdi" => BDI,
"bdj" => BDJ,
"bdk" => BDK,
"bdl" => BDL,
"bdm" => BDM,
"bdn" => BDN,
"bdo" => BDO,
"bdp" => BDP,
"bdq" => BDQ,
"bdr" => BDR,
"bds" => BDS,
"bdt" => BDT,
"bdu" => BDU,
"bdv" => BDV,
"bdw" => BDW,
"bdx" => BDX,
"bdy" => BDY,
"bdz" => BDZ,
"bea" => BEA,
"beb" => BEB,
"bec" => BEC,
"bed" => BED,
"bee" => BEE,
"bef" => BEF,
"beg" => BEG,
"beh" => BEH,
"bei" => BEI,
"bek" => BEK,
"beo" => BEO,
"bep" => BEP,
"beq" => BEQ,
"bes" => BES,
"bet" => BET,
"beu" => BEU,
"bev" => BEV,
"bew" => BEW,
"bex" => BEX,
"bey" => BEY,
"bez" => BEZ,
"bfa" => BFA,
"bfb" => BFB,
"bfc" => BFC,
"bfd" => BFD,
"bfe" => BFE,
"bff" => BFF,
"bfg" => BFG,
"bfh" => BFH,
"bfi" => BFI,
"bfj" => BFJ,
"bfk" => BFK,
"bfl" => BFL,
"bfm" => BFM,
"bfn" => BFN,
"bfo" => BFO,
"bfp" => BFP,
"bfq" => BFQ,
"bfr" => BFR,
"bfs" => BFS,
"bft" => BFT,
"bfu" => BFU,
"bfw" => BFW,
"bfx" => BFX,
"bfy" => BFY,
"bfz" => BFZ,
"bga" => BGA,
"bgb" => BGB,
"bgc" => BGC,
"bgd" => BGD,
"bge" => BGE,
"bgf" => BGF,
"bgg" => BGG,
"bgi" => BGI,
"bgj" => BGJ,
"bgk" => BGK,
"bgl" => BGL,
"bgn" => BGN,
"bgo" => BGO,
"bgp" => BGP,
"bgq" => BGQ,
"bgr" => BGR,
"bgs" => BGS,
"bgt" => BGT,
"bgu" => BGU,
"bgv" => BGV,
"bgw" => BGW,
"bgx" => BGX,
"bgy" => BGY,
"bgz" => BGZ,
"bha" => BHA,
"bhb" => BHB,
"bhc" => BHC,
"bhd" => BHD,
"bhe" => BHE,
"bhf" => BHF,
"bhg" => BHG,
"bhh" => BHH,
"bhi" => BHI,
"bhj" => BHJ,
"bhl" => BHL,
"bhm" => BHM,
"bhn" => BHN,
"bhp" => BHP,
"bhq" => BHQ,
"bhr" => BHR,
"bhs" => BHS,
"bht" => BHT,
"bhu" => BHU,
"bhv" => BHV,
"bhw" => BHW,
"bhx" => BHX,
"bhy" => BHY,
"bhz" => BHZ,
"bia" => BIA,
"bib" => BIB,
"bid" => BID,
"bie" => BIE,
"bif" => BIF,
"big" => BIG,
"bil" => BIL,
"bim" => BIM,
"bio" => BIO,
"bip" => BIP,
"biq" => BIQ,
"bir" => BIR,
"bit" => BIT,
"biu" => BIU,
"biv" => BIV,
"biw" => BIW,
"bix" => BIX,
"biy" => BIY,
"biz" => BIZ,
"bja" => BJA,
"bjb" => BJB,
"bjc" => BJC,
"bje" => BJE,
"bjf" => BJF,
"bjg" => BJG,
"bjh" => BJH,
"bji" => BJI,
"bjj" => BJJ,
"bjk" => BJK,
"bjl" => BJL,
"bjm" => BJM,
"bjn" => BJN,
"bjo" => BJO,
"bjp" => BJP,
"bjr" => BJR,
"bjs" => BJS,
"bjt" => BJT,
"bju" => BJU,
"bjv" => BJV,
"bjw" => BJW,
"bjx" => BJX,
"bjy" => BJY,
"bjz" => BJZ,
"bka" => BKA,
"bkc" => BKC,
"bkd" => BKD,
"bkf" => BKF,
"bkg" => BKG,
"bkh" => BKH,
"bki" => BKI,
"bkj" => BKJ,
"bkk" => BKK,
"bkl" => BKL,
"bkm" => BKM,
"bkn" => BKN,
"bko" => BKO,
"bkp" => BKP,
"bkq" => BKQ,
"bkr" => BKR,
"bks" => BKS,
"bkt" => BKT,
"bku" => BKU,
"bkv" => BKV,
"bkw" => BKW,
"bkx" => BKX,
"bky" => BKY,
"bkz" => BKZ,
"blb" => BLB,
"blc" => BLC,
"bld" => BLD,
"ble" => BLE,
"blf" => BLF,
"blh" => BLH,
"bli" => BLI,
"blj" => BLJ,
"blk" => BLK,
"bll" => BLL,
"blm" => BLM,
"bln" => BLN,
"blo" => BLO,
"blp" => BLP,
"blq" => BLQ,
"blr" => BLR,
"bls" => BLS,
"blt" => BLT,
"blv" => BLV,
"blw" => BLW,
"blx" => BLX,
"bly" => BLY,
"blz" => BLZ,
"bma" => BMA,
"bmb" => BMB,
"bmc" => BMC,
"bmd" => BMD,
"bme" => BME,
"bmf" => BMF,
"bmg" => BMG,
"bmh" => BMH,
"bmi" => BMI,
"bmj" => BMJ,
"bmk" => BMK,
"bml" => BML,
"bmm" => BMM,
"bmn" => BMN,
"bmo" => BMO,
"bmp" => BMP,
"bmq" => BMQ,
"bmr" => BMR,
"bms" => BMS,
"bmt" => BMT,
"bmu" => BMU,
"bmv" => BMV,
"bmw" => BMW,
"bmx" => BMX,
"bmz" => BMZ,
"bna" => BNA,
"bnb" => BNB,
"bnc" => BNC,
"bnd" => BND,
"bne" => BNE,
"bnf" => BNF,
"bng" => BNG,
"bni" => BNI,
"bnj" => BNJ,
"bnk" => BNK,
"bnl" => BNL,
"bnm" => BNM,
"bnn" => BNN,
"bno" => BNO,
"bnp" => BNP,
"bnq" => BNQ,
"bnr" => BNR,
"bns" => BNS,
"bnu" => BNU,
"bnv" => BNV,
"bnw" => BNW,
"bnx" => BNX,
"bny" => BNY,
"bnz" => BNZ,
"boa" => BOA,
"bob" => BOB,
"boe" => BOE,
"bof" => BOF,
"bog" => BOG,
"boh" => BOH,
"boi" => BOI,
"boj" => BOJ,
"bok" => BOK,
"bol" => BOL,
"bom" => BOM,
"bon" => BON,
"boo" => BOO,
"bop" => BOP,
"boq" => BOQ,
"bor" => BOR,
"bot" => BOT,
"bou" => BOU,
"bov" => BOV,
"bow" => BOW,
"box" => BOX,
"boy" => BOY,
"boz" => BOZ,
"bpa" => BPA,
"bpc" => BPC,
"bpd" => BPD,
"bpe" => BPE,
"bpg" => BPG,
"bph" => BPH,
"bpi" => BPI,
"bpj" => BPJ,
"bpk" => BPK,
"bpl" => BPL,
"bpm" => BPM,
"bpn" => BPN,
"bpo" => BPO,
"bpp" => BPP,
"bpq" => BPQ,
"bpr" => BPR,
"bps" => BPS,
"bpt" => BPT,
"bpu" => BPU,
"bpv" => BPV,
"bpw" => BPW,
"bpx" => BPX,
"bpy" => BPY,
"bpz" => BPZ,
"bqa" => BQA,
"bqb" => BQB,
"bqc" => BQC,
"bqd" => BQD,
"bqf" => BQF,
"bqg" => BQG,
"bqh" => BQH,
"bqi" => BQI,
"bqj" => BQJ,
"bqk" => BQK,
"bql" => BQL,
"bqm" => BQM,
"bqn" => BQN,
"bqo" => BQO,
"bqp" => BQP,
"bqq" => BQQ,
"bqr" => BQR,
"bqs" => BQS,
"bqt" => BQT,
"bqu" => BQU,
"bqv" => BQV,
"bqw" => BQW,
"bqx" => BQX,
"bqy" => BQY,
"bqz" => BQZ,
"brb" => BRB,
"brc" => BRC,
"brd" => BRD,
"brf" => BRF,
"brg" => BRG,
"brh" => BRH,
"bri" => BRI,
"brj" => BRJ,
"brk" => BRK,
"brl" => BRL,
"brm" => BRM,
"brn" => BRN,
"bro" => BRO,
"brp" => BRP,
"brq" => BRQ,
"brr" => BRR,
"brs" => BRS,
"brt" => BRT,
"bru" => BRU,
"brv" => BRV,
"brw" => BRW,
"brx" => BRX,
"bry" => BRY,
"brz" => BRZ,
"bsa" => BSA,
"bsb" => BSB,
"bsc" => BSC,
"bse" => BSE,
"bsf" => BSF,
"bsg" => BSG,
"bsh" => BSH,
"bsi" => BSI,
"bsj" => BSJ,
"bsk" => BSK,
"bsl" => BSL,
"bsm" => BSM,
"bsn" => BSN,
"bso" => BSO,
"bsp" => BSP,
"bsq" => BSQ,
"bsr" => BSR,
"bss" => BSS,
"bst" => BST,
"bsu" => BSU,
"bsv" => BSV,
"bsw" => BSW,
"bsx" => BSX,
"bsy" => BSY,
"bta" => BTA,
"btc" => BTC,
"btd" => BTD,
"bte" => BTE,
"btf" => BTF,
"btg" => BTG,
"bth" => BTH,
"bti" => BTI,
"btj" => BTJ,
"btm" => BTM,
"btn" => BTN,
"bto" => BTO,
"btp" => BTP,
"btq" => BTQ,
"btr" => BTR,
"bts" => BTS,
"btt" => BTT,
"btu" => BTU,
"btv" => BTV,
"btw" => BTW,
"btx" => BTX,
"bty" => BTY,
"btz" => BTZ,
"bub" => BUB,
"buc" => BUC,
"bud" => BUD,
"bue" => BUE,
"buf" => BUF,
"buh" => BUH,
"bui" => BUI,
"buj" => BUJ,
"buk" => BUK,
"bum" => BUM,
"bun" => BUN,
"buo" => BUO,
"bup" => BUP,
"buq" => BUQ,
"bus" => BUS,
"but" => BUT,
"buu" => BUU,
"buv" => BUV,
"buw" => BUW,
"bux" => BUX,
"buy" => BUY,
"buz" => BUZ,
"bva" => BVA,
"bvb" => BVB,
"bvc" => BVC,
"bvd" => BVD,
"bve" => BVE,
"bvf" => BVF,
"bvg" => BVG,
"bvh" => BVH,
"bvi" => BVI,
"bvj" => BVJ,
"bvk" => BVK,
"bvl" => BVL,
"bvm" => BVM,
"bvn" => BVN,
"bvo" => BVO,
"bvp" => BVP,
"bvq" => BVQ,
"bvr" => BVR,
"bvt" => BVT,
"bvu" => BVU,
"bvv" => BVV,
"bvw" => BVW,
"bvx" => BVX,
"bvy" => BVY,
"bvz" => BVZ,
"bwa" => BWA,
"bwb" => BWB,
"bwc" => BWC,
"bwd" => BWD,
"bwe" => BWE,
"bwf" => BWF,
"bwg" => BWG,
"bwh" => BWH,
"bwi" => BWI,
"bwj" => BWJ,
"bwk" => BWK,
"bwl" => BWL,
"bwm" => BWM,
"bwn" => BWN,
"bwo" => BWO,
"bwp" => BWP,
"bwq" => BWQ,
"bwr" => BWR,
"bws" => BWS,
"bwt" => BWT,
"bwu" => BWU,
"bww" => BWW,
"bwx" => BWX,
"bwy" => BWY,
"bwz" => BWZ,
"bxa" => BXA,
"bxb" => BXB,
"bxc" => BXC,
"bxd" => BXD,
"bxe" => BXE,
"bxf" => BXF,
"bxg" => BXG,
"bxh" => BXH,
"bxi" => BXI,
"bxj" => BXJ,
"bxk" => BXK,
"bxl" => BXL,
"bxm" => BXM,
"bxn" => BXN,
"bxo" => BXO,
"bxp" => BXP,
"bxq" => BXQ,
"bxr" => BXR,
"bxs" => BXS,
"bxu" => BXU,
"bxv" => BXV,
"bxw" => BXW,
"bxz" => BXZ,
"bya" => BYA,
"byb" => BYB,
"byc" => BYC,
"byd" => BYD,
"bye" => BYE,
"byf" => BYF,
"byg" => BYG,
"byh" => BYH,
"byi" => BYI,
"byj" => BYJ,
"byk" => BYK,
"byl" => BYL,
"bym" => BYM,
"byo" => BYO,
"byp" => BYP,
"byq" => BYQ,
"byr" => BYR,
"bys" => BYS,
"byt" => BYT,
"byv" => BYV,
"byw" => BYW,
"byx" => BYX,
"byz" => BYZ,
"bza" => BZA,
"bzb" => BZB,
"bzc" => BZC,
"bzd" => BZD,
"bze" => BZE,
"bzf" => BZF,
"bzg" => BZG,
"bzh" => BZH,
"bzi" => BZI,
"bzj" => BZJ,
"bzk" => BZK,
"bzl" => BZL,
"bzm" => BZM,
"bzn" => BZN,
"bzo" => BZO,
"bzp" => BZP,
"bzq" => BZQ,
"bzr" => BZR,
"bzs" => BZS,
"bzt" => BZT,
"bzu" => BZU,
"bzv" => BZV,
"bzw" => BZW,
"bzx" => BZX,
"bzy" => BZY,
"bzz" => BZZ,
"caa" => CAA,
"cab" => CAB,
"cac" => CAC,
"cae" => CAE,
"caf" => CAF,
"cag" => CAG,
"cah" => CAH,
"caj" => CAJ,
"cak" => CAK,
"cal" => CAL,
"cam" => CAM,
"can" => CAN,
"cao" => CAO,
"cap" => CAP,
"caq" => CAQ,
"cas" => CAS,
"cav" => CAV,
"caw" => CAW,
"cax" => CAX,
"cay" => CAY,
"caz" => CAZ,
"cbb" => CBB,
"cbc" => CBC,
"cbd" => CBD,
"cbg" => CBG,
"cbi" => CBI,
"cbj" => CBJ,
"cbk" => CBK,
"cbl" => CBL,
"cbn" => CBN,
"cbo" => CBO,
"cbq" => CBQ,
"cbr" => CBR,
"cbs" => CBS,
"cbt" => CBT,
"cbu" => CBU,
"cbv" => CBV,
"cbw" => CBW,
"cby" => CBY,
"ccc" => CCC,
"ccd" => CCD,
"cce" => CCE,
"ccg" => CCG,
"cch" => CCH,
"ccj" => CCJ,
"ccl" => CCL,
"ccm" => CCM,
"cco" => CCO,
"ccp" => CCP,
"ccr" => CCR,
"cda" => CDA,
"cde" => CDE,
"cdf" => CDF,
"cdh" => CDH,
"cdi" => CDI,
"cdj" => CDJ,
"cdm" => CDM,
"cdn" => CDN,
"cdo" => CDO,
"cdr" => CDR,
"cds" => CDS,
"cdy" => CDY,
"cdz" => CDZ,
"cea" => CEA,
"ceg" => CEG,
"cek" => CEK,
"cen" => CEN,
"cet" => CET,
"cey" => CEY,
"cfa" => CFA,
"cfd" => CFD,
"cfg" => CFG,
"cfm" => CFM,
"cga" => CGA,
"cgc" => CGC,
"cgg" => CGG,
"cgk" => CGK,
"chc" => CHC,
"chd" => CHD,
"chf" => CHF,
"chh" => CHH,
"chj" => CHJ,
"chl" => CHL,
"chq" => CHQ,
"cht" => CHT,
"chw" => CHW,
"chx" => CHX,
"chz" => CHZ,
"cia" => CIA,
"cib" => CIB,
"cic" => CIC,
"cid" => CID,
"cie" => CIE,
"cih" => CIH,
"cik" => CIK,
"cim" => CIM,
"cin" => CIN,
"cip" => CIP,
"cir" => CIR,
"ciw" => CIW,
"ciy" => CIY,
"cja" => CJA,
"cje" => CJE,
"cjh" => CJH,
"cji" => CJI,
"cjk" => CJK,
"cjm" => CJM,
"cjn" => CJN,
"cjo" => CJO,
"cjp" => CJP,
"cjs" => CJS,
"cjv" => CJV,
"cjy" => CJY,
"ckb" => CKB,
"ckh" => CKH,
"ckl" => CKL,
"ckm" => CKM,
"ckn" => CKN,
"cko" => CKO,
"ckq" => CKQ,
"ckr" => CKR,
"cks" => CKS,
"ckt" => CKT,
"cku" => CKU,
"ckv" => CKV,
"ckx" => CKX,
"cky" => CKY,
"ckz" => CKZ,
"cla" => CLA,
"clc" => CLC,
"cld" => CLD,
"cle" => CLE,
"clh" => CLH,
"cli" => CLI,
"clj" => CLJ,
"clk" => CLK,
"cll" => CLL,
"clm" => CLM,
"clo" => CLO,
"clt" => CLT,
"clu" => CLU,
"clw" => CLW,
"cly" => CLY,
"cma" => CMA,
"cme" => CME,
"cmg" => CMG,
"cmi" => CMI,
"cml" => CML,
"cmm" => CMM,
"cmn" => CMN,
"cmo" => CMO,
"cmr" => CMR,
"cms" => CMS,
"cmt" => CMT,
"cna" => CNA,
"cnb" => CNB,
"cnc" => CNC,
"cng" => CNG,
"cnh" => CNH,
"cni" => CNI,
"cnk" => CNK,
"cnl" => CNL,
"cno" => CNO,
"cnp" => CNP,
"cnq" => CNQ,
"cns" => CNS,
"cnt" => CNT,
"cnu" => CNU,
"cnw" => CNW,
"cnx" => CNX,
"coa" => COA,
"cob" => COB,
"coc" => COC,
"cod" => COD,
"coe" => COE,
"cof" => COF,
"cog" => COG,
"coh" => COH,
"coj" => COJ,
"cok" => COK,
"col" => COL,
"com" => COM,
"con" => CON,
"coo" => COO,
"coq" => COQ,
"cot" => COT,
"cou" => COU,
"cov" => COV,
"cow" => COW,
"cox" => COX,
"coz" => COZ,
"cpa" => CPA,
"cpb" => CPB,
"cpc" => CPC,
"cpg" => CPG,
"cpi" => CPI,
"cpn" => CPN,
"cpo" => CPO,
"cps" => CPS,
"cpu" => CPU,
"cpx" => CPX,
"cpy" => CPY,
"cqd" => CQD,
"cra" => CRA,
"crb" => CRB,
"crc" => CRC,
"crd" => CRD,
"crf" => CRF,
"crg" => CRG,
"cri" => CRI,
"crj" => CRJ,
"crk" => CRK,
"crl" => CRL,
"crm" => CRM,
"crn" => CRN,
"cro" => CRO,
"crq" => CRQ,
"crr" => CRR,
"crs" => CRS,
"crt" => CRT,
"crv" => CRV,
"crw" => CRW,
"crx" => CRX,
"cry" => CRY,
"crz" => CRZ,
"csa" => CSA,
"csc" => CSC,
"csd" => CSD,
"cse" => CSE,
"csf" => CSF,
"csg" => CSG,
"csh" => CSH,
"csi" => CSI,
"csj" => CSJ,
"csk" => CSK,
"csl" => CSL,
"csm" => CSM,
"csn" => CSN,
"cso" => CSO,
"csp" => CSP,
"csq" => CSQ,
"csr" => CSR,
"css" => CSS,
"cst" => CST,
"csv" => CSV,
"csw" => CSW,
"csx" => CSX,
"csy" => CSY,
"csz" => CSZ,
"cta" => CTA,
"ctc" => CTC,
"ctd" => CTD,
"cte" => CTE,
"ctg" => CTG,
"cth" => CTH,
"ctl" => CTL,
"ctm" => CTM,
"ctn" => CTN,
"cto" => CTO,
"ctp" => CTP,
"cts" => CTS,
"ctt" => CTT,
"ctu" => CTU,
"cty" => CTY,
"ctz" => CTZ,
"cua" => CUA,
"cub" => CUB,
"cuc" => CUC,
"cuh" => CUH,
"cui" => CUI,
"cuj" => CUJ,
"cuk" => CUK,
"cul" => CUL,
"cuo" => CUO,
"cup" => CUP,
"cuq" => CUQ,
"cur" => CUR,
"cut" => CUT,
"cuu" => CUU,
"cuv" => CUV,
"cuw" => CUW,
"cux" => CUX,
"cuy" => CUY,
"cvg" => CVG,
"cvn" => CVN,
"cwa" => CWA,
"cwb" => CWB,
"cwd" => CWD,
"cwe" => CWE,
"cwg" => CWG,
"cwt" => CWT,
"cya" => CYA,
"cyb" => CYB,
"cyo" => CYO,
"czh" => CZH,
"czk" => CZK,
"czn" => CZN,
"czo" => CZO,
"czt" => CZT,
"daa" => DAA,
"dac" => DAC,
"dad" => DAD,
"dae" => DAE,
"dag" => DAG,
"dah" => DAH,
"dai" => DAI,
"daj" => DAJ,
"dal" => DAL,
"dam" => DAM,
"dao" => DAO,
"daq" => DAQ,
"das" => DAS,
"dau" => DAU,
"dav" => DAV,
"daw" => DAW,
"dax" => DAX,
"daz" => DAZ,
"dba" => DBA,
"dbb" => DBB,
"dbd" => DBD,
"dbe" => DBE,
"dbf" => DBF,
"dbg" => DBG,
"dbi" => DBI,
"dbj" => DBJ,
"dbl" => DBL,
"dbm" => DBM,
"dbn" => DBN,
"dbo" => DBO,
"dbp" => DBP,
"dbq" => DBQ,
"dbr" => DBR,
"dbt" => DBT,
"dbu" => DBU,
"dbv" => DBV,
"dbw" => DBW,
"dby" => DBY,
"dcc" => DCC,
"dcr" => DCR,
"dda" => DDA,
"ddd" => DDD,
"dde" => DDE,
"ddg" => DDG,
"ddi" => DDI,
"ddj" => DDJ,
"ddn" => DDN,
"ddo" => DDO,
"ddr" => DDR,
"dds" => DDS,
"ddw" => DDW,
"dec" => DEC,
"ded" => DED,
"dee" => DEE,
"def" => DEF,
"deg" => DEG,
"deh" => DEH,
"dei" => DEI,
"dek" => DEK,
"dem" => DEM,
"dep" => DEP,
"deq" => DEQ,
"der" => DER,
"des" => DES,
"dev" => DEV,
"dez" => DEZ,
"dga" => DGA,
"dgb" => DGB,
"dgc" => DGC,
"dgd" => DGD,
"dge" => DGE,
"dgg" => DGG,
"dgh" => DGH,
"dgi" => DGI,
"dgk" => DGK,
"dgl" => DGL,
"dgn" => DGN,
"dgo" => DGO,
"dgs" => DGS,
"dgt" => DGT,
"dgw" => DGW,
"dgx" => DGX,
"dgz" => DGZ,
"dhd" => DHD,
"dhg" => DHG,
"dhi" => DHI,
"dhl" => DHL,
"dhm" => DHM,
"dhn" => DHN,
"dho" => DHO,
"dhr" => DHR,
"dhs" => DHS,
"dhu" => DHU,
"dhv" => DHV,
"dhw" => DHW,
"dhx" => DHX,
"dia" => DIA,
"dib" => DIB,
"dic" => DIC,
"did" => DID,
"dif" => DIF,
"dig" => DIG,
"dih" => DIH,
"dii" => DII,
"dij" => DIJ,
"dik" => DIK,
"dil" => DIL,
"dim" => DIM,
"dio" => DIO,
"dip" => DIP,
"diq" => DIQ,
"dir" => DIR,
"dis" => DIS,
"diu" => DIU,
"diw" => DIW,
"dix" => DIX,
"diy" => DIY,
"diz" => DIZ,
"dja" => DJA,
"djb" => DJB,
"djc" => DJC,
"djd" => DJD,
"dje" => DJE,
"djf" => DJF,
"dji" => DJI,
"djj" => DJJ,
"djk" => DJK,
"djm" => DJM,
"djn" => DJN,
"djo" => DJO,
"djr" => DJR,
"dju" => DJU,
"djw" => DJW,
"dka" => DKA,
"dkg" => DKG,
"dkk" => DKK,
"dkr" => DKR,
"dks" => DKS,
"dkx" => DKX,
"dlg" => DLG,
"dlk" => DLK,
"dlm" => DLM,
"dln" => DLN,
"dma" => DMA,
"dmb" => DMB,
"dmc" => DMC,
"dmd" => DMD,
"dme" => DME,
"dmf" => DMF,
"dmg" => DMG,
"dmk" => DMK,
"dml" => DML,
"dmm" => DMM,
"dmo" => DMO,
"dmr" => DMR,
"dms" => DMS,
"dmu" => DMU,
"dmv" => DMV,
"dmw" => DMW,
"dmx" => DMX,
"dmy" => DMY,
"dna" => DNA,
"dnd" => DND,
"dne" => DNE,
"dng" => DNG,
"dni" => DNI,
"dnj" => DNJ,
"dnk" => DNK,
"dnn" => DNN,
"dno" => DNO,
"dnr" => DNR,
"dnt" => DNT,
"dnu" => DNU,
"dnv" => DNV,
"dnw" => DNW,
"dny" => DNY,
"doa" => DOA,
"dob" => DOB,
"doc" => DOC,
"doe" => DOE,
"dof" => DOF,
"doh" => DOH,
"dok" => DOK,
"dol" => DOL,
"don" => DON,
"doo" => DOO,
"dop" => DOP,
"doq" => DOQ,
"dor" => DOR,
"dos" => DOS,
"dot" => DOT,
"dov" => DOV,
"dow" => DOW,
"dox" => DOX,
"doy" => DOY,
"doz" => DOZ,
"dpp" => DPP,
"drb" => DRB,
"drc" => DRC,
"drd" => DRD,
"dre" => DRE,
"drg" => DRG,
"dri" => DRI,
"drl" => DRL,
"drn" => DRN,
"dro" => DRO,
"drq" => DRQ,
"drs" => DRS,
"drt" => DRT,
"dru" => DRU,
"dry" => DRY,
"dse" => DSE,
"dsh" => DSH,
"dsi" => DSI,
"dsl" => DSL,
"dsn" => DSN,
"dso" => DSO,
"dsq" => DSQ,
"dsz" => DSZ,
"dta" => DTA,
"dtb" => DTB,
"dtd" => DTD,
"dth" => DTH,
"dti" => DTI,
"dtk" => DTK,
"dtm" => DTM,
"dtn" => DTN,
"dto" => DTO,
"dtp" => DTP,
"dtr" => DTR,
"dts" => DTS,
"dtt" => DTT,
"dtu" => DTU,
"dty" => DTY,
"dub" => DUB,
"duc" => DUC,
"due" => DUE,
"duf" => DUF,
"dug" => DUG,
"duh" => DUH,
"dui" => DUI,
"duk" => DUK,
"dul" => DUL,
"dun" => DUN,
"duo" => DUO,
"dup" => DUP,
"duq" => DUQ,
"dur" => DUR,
"dus" => DUS,
"duu" => DUU,
"duv" => DUV,
"duw" => DUW,
"dux" => DUX,
"duy" => DUY,
"duz" => DUZ,
"dva" => DVA,
"dwa" => DWA,
"dwk" => DWK,
"dwr" => DWR,
"dws" => DWS,
"dwu" => DWU,
"dww" => DWW,
"dwy" => DWY,
"dwz" => DWZ,
"dya" => DYA,
"dyb" => DYB,
"dyd" => DYD,
"dyg" => DYG,
"dyi" => DYI,
"dym" => DYM,
"dyn" => DYN,
"dyo" => DYO,
"dyy" => DYY,
"dza" => DZA,
"dze" => DZE,
"dzg" => DZG,
"dzl" => DZL,
"dzn" => DZN,
"eaa" => EAA,
"ebc" => EBC,
"ebg" => EBG,
"ebk" => EBK,
"ebo" => EBO,
"ebr" => EBR,
"ebu" => EBU,
"ecr" => ECR,
"ecs" => ECS,
"ecy" => ECY,
"eee" => EEE,
"efa" => EFA,
"efe" => EFE,
"ega" => EGA,
"egl" => EGL,
"egm" => EGM,
"ego" => EGO,
"ehs" => EHS,
"ehu" => EHU,
"eip" => EIP,
"eit" => EIT,
"eiv" => EIV,
"eja" => EJA,
"eke" => EKE,
"ekg" => EKG,
"eki" => EKI,
"ekk" => EKK,
"ekl" => EKL,
"ekm" => EKM,
"eko" => EKO,
"ekp" => EKP,
"ekr" => EKR,
"eky" => EKY,
"ele" => ELE,
"elh" => ELH,
"eli" => ELI,
"elk" => ELK,
"elm" => ELM,
"elo" => ELO,
"elu" => ELU,
"ema" => EMA,
"emb" => EMB,
"eme" => EME,
"emg" => EMG,
"emi" => EMI,
"emk" => EMK,
"emm" => EMM,
"emn" => EMN,
"emp" => EMP,
"emq" => EMQ,
"ems" => EMS,
"emu" => EMU,
"emw" => EMW,
"emx" => EMX,
"emy" => EMY,
"emz" => EMZ,
"ena" => ENA,
"enb" => ENB,
"enc" => ENC,
"end" => END,
"enf" => ENF,
"enh" => ENH,
"enl" => ENL,
"enn" => ENN,
"eno" => ENO,
"enq" => ENQ,
"enr" => ENR,
"enu" => ENU,
"env" => ENV,
"enw" => ENW,
"enx" => ENX,
"eot" => EOT,
"epi" => EPI,
"era" => ERA,
"erg" => ERG,
"erh" => ERH,
"eri" => ERI,
"erk" => ERK,
"ero" => ERO,
"err" => ERR,
"ers" => ERS,
"ert" => ERT,
"erw" => ERW,
"ese" => ESE,
"esg" => ESG,
"esh" => ESH,
"esi" => ESI,
"esk" => ESK,
"esl" => ESL,
"esm" => ESM,
"esn" => ESN,
"eso" => ESO,
"esq" => ESQ,
"ess" => ESS,
"esu" => ESU,
"esy" => ESY,
"etb" => ETB,
"etc" => ETC,
"eth" => ETH,
"etn" => ETN,
"eto" => ETO,
"etr" => ETR,
"ets" => ETS,
"ett" => ETT,
"etu" => ETU,
"etx" => ETX,
"etz" => ETZ,
"eve" => EVE,
"evh" => EVH,
"evn" => EVN,
"ext" => EXT,
"eya" => EYA,
"eyo" => EYO,
"eza" => EZA,
"eze" => EZE,
"faa" => FAA,
"fab" => FAB,
"fad" => FAD,
"faf" => FAF,
"fag" => FAG,
"fah" => FAH,
"fai" => FAI,
"faj" => FAJ,
"fak" => FAK,
"fal" => FAL,
"fam" => FAM,
"fap" => FAP,
"far" => FAR,
"fau" => FAU,
"fax" => FAX,
"fay" => FAY,
"faz" => FAZ,
"fbl" => FBL,
"fcs" => FCS,
"fer" => FER,
"ffi" => FFI,
"ffm" => FFM,
"fgr" => FGR,
"fia" => FIA,
"fie" => FIE,
"fif" => FIF,
"fip" => FIP,
"fir" => FIR,
"fit" => FIT,
"fiw" => FIW,
"fkk" => FKK,
"fkv" => FKV,
"fla" => FLA,
"flh" => FLH,
"fli" => FLI,
"fll" => FLL,
"fln" => FLN,
"flr" => FLR,
"fly" => FLY,
"fmp" => FMP,
"fmu" => FMU,
"fnb" => FNB,
"fng" => FNG,
"fni" => FNI,
"fod" => FOD,
"foi" => FOI,
"fom" => FOM,
"for" => FOR,
"fos" => FOS,
"fpe" => FPE,
"fqs" => FQS,
"frc" => FRC,
"frd" => FRD,
"frk" => FRK,
"frp" => FRP,
"frq" => FRQ,
"frt" => FRT,
"fse" => FSE,
"fsl" => FSL,
"fss" => FSS,
"fub" => FUB,
"fuc" => FUC,
"fud" => FUD,
"fue" => FUE,
"fuf" => FUF,
"fuh" => FUH,
"fui" => FUI,
"fuj" => FUJ,
"fum" => FUM,
"fun" => FUN,
"fuq" => FUQ,
"fut" => FUT,
"fuu" => FUU,
"fuv" => FUV,
"fuy" => FUY,
"fvr" => FVR,
"fwa" => FWA,
"fwe" => FWE,
"gab" => GAB,
"gac" => GAC,
"gad" => GAD,
"gae" => GAE,
"gaf" => GAF,
"gag" => GAG,
"gah" => GAH,
"gai" => GAI,
"gaj" => GAJ,
"gak" => GAK,
"gal" => GAL,
"gam" => GAM,
"gan" => GAN,
"gao" => GAO,
"gap" => GAP,
"gaq" => GAQ,
"gar" => GAR,
"gas" => GAS,
"gat" => GAT,
"gau" => GAU,
"gaw" => GAW,
"gax" => GAX,
"gaz" => GAZ,
"gbb" => GBB,
"gbd" => GBD,
"gbe" => GBE,
"gbf" => GBF,
"gbg" => GBG,
"gbh" => GBH,
"gbi" => GBI,
"gbj" => GBJ,
"gbk" => GBK,
"gbl" => GBL,
"gbm" => GBM,
"gbn" => GBN,
"gbo" => GBO,
"gbp" => GBP,
"gbq" => GBQ,
"gbr" => GBR,
"gbs" => GBS,
"gbu" => GBU,
"gbv" => GBV,
"gbw" => GBW,
"gbx" => GBX,
"gby" => GBY,
"gbz" => GBZ,
"gcc" => GCC,
"gcd" => GCD,
"gce" => GCE,
"gcf" => GCF,
"gcl" => GCL,
"gcn" => GCN,
"gcr" => GCR,
"gct" => GCT,
"gda" => GDA,
"gdb" => GDB,
"gdc" => GDC,
"gdd" => GDD,
"gde" => GDE,
"gdf" => GDF,
"gdg" => GDG,
"gdh" => GDH,
"gdi" => GDI,
"gdj" => GDJ,
"gdk" => GDK,
"gdl" => GDL,
"gdm" => GDM,
"gdn" => GDN,
"gdo" => GDO,
"gdq" => GDQ,
"gdr" => GDR,
"gds" => GDS,
"gdt" => GDT,
"gdu" => GDU,
"gdx" => GDX,
"gea" => GEA,
"geb" => GEB,
"gec" => GEC,
"ged" => GED,
"gef" => GEF,
"geg" => GEG,
"geh" => GEH,
"gei" => GEI,
"gej" => GEJ,
"gek" => GEK,
"gel" => GEL,
"geq" => GEQ,
"ges" => GES,
"gev" => GEV,
"gew" => GEW,
"gex" => GEX,
"gey" => GEY,
"gfk" => GFK,
"gft" => GFT,
"gga" => GGA,
"ggb" => GGB,
"ggd" => GGD,
"gge" => GGE,
"ggg" => GGG,
"ggk" => GGK,
"ggl" => GGL,
"ggt" => GGT,
"ggu" => GGU,
"ggw" => GGW,
"gha" => GHA,
"ghc" => GHC,
"ghe" => GHE,
"ghh" => GHH,
"ghk" => GHK,
"ghl" => GHL,
"ghn" => GHN,
"gho" => GHO,
"ghr" => GHR,
"ghs" => GHS,
"ght" => GHT,
"gia" => GIA,
"gib" => GIB,
"gic" => GIC,
"gid" => GID,
"gie" => GIE,
"gig" => GIG,
"gih" => GIH,
"gii" => GII,
"gim" => GIM,
"gin" => GIN,
"gip" => GIP,
"giq" => GIQ,
"gir" => GIR,
"gis" => GIS,
"git" => GIT,
"giu" => GIU,
"giw" => GIW,
"gix" => GIX,
"giy" => GIY,
"giz" => GIZ,
"gjk" => GJK,
"gjm" => GJM,
"gjn" => GJN,
"gjr" => GJR,
"gju" => GJU,
"gka" => GKA,
"gkd" => GKD,
"gke" => GKE,
"gkn" => GKN,
"gko" => GKO,
"gkp" => GKP,
"gku" => GKU,
"glb" => GLB,
"glc" => GLC,
"gld" => GLD,
"glh" => GLH,
"glj" => GLJ,
"glk" => GLK,
"gll" => GLL,
"glo" => GLO,
"glr" => GLR,
"glu" => GLU,
"glw" => GLW,
"gly" => GLY,
"gma" => GMA,
"gmb" => GMB,
"gmd" => GMD,
"gmg" => GMG,
"gml" => GML,
"gmm" => GMM,
"gmn" => GMN,
"gmr" => GMR,
"gmu" => GMU,
"gmv" => GMV,
"gmx" => GMX,
"gmy" => GMY,
"gmz" => GMZ,
"gna" => GNA,
"gnb" => GNB,
"gnc" => GNC,
"gnd" => GND,
"gne" => GNE,
"gng" => GNG,
"gnh" => GNH,
"gni" => GNI,
"gnj" => GNJ,
"gnk" => GNK,
"gnl" => GNL,
"gnm" => GNM,
"gnn" => GNN,
"gno" => GNO,
"gnq" => GNQ,
"gnr" => GNR,
"gnt" => GNT,
"gnu" => GNU,
"gnw" => GNW,
"gnz" => GNZ,
"goa" => GOA,
"gob" => GOB,
"goc" => GOC,
"god" => GOD,
"goe" => GOE,
"gof" => GOF,
"gog" => GOG,
"goi" => GOI,
"goj" => GOJ,
"gok" => GOK,
"gol" => GOL,
"gom" => GOM,
"goo" => GOO,
"gop" => GOP,
"goq" => GOQ,
"gos" => GOS,
"gou" => GOU,
"gov" => GOV,
"gow" => GOW,
"gox" => GOX,
"goy" => GOY,
"goz" => GOZ,
"gpa" => GPA,
"gpe" => GPE,
"gpn" => GPN,
"gqa" => GQA,
"gqi" => GQI,
"gqn" => GQN,
"gqr" => GQR,
"gqu" => GQU,
"gra" => GRA,
"grd" => GRD,
"grg" => GRG,
"grh" => GRH,
"gri" => GRI,
"grj" => GRJ,
"grm" => GRM,
"gro" => GRO,
"grq" => GRQ,
"grr" => GRR,
"grs" => GRS,
"grt" => GRT,
"gru" => GRU,
"grv" => GRV,
"grw" => GRW,
"grx" => GRX,
"gry" => GRY,
"grz" => GRZ,
"gse" => GSE,
"gsg" => GSG,
"gsl" => GSL,
"gsm" => GSM,
"gsn" => GSN,
"gso" => GSO,
"gsp" => GSP,
"gss" => GSS,
"gta" => GTA,
"gtu" => GTU,
"gua" => GUA,
"gub" => GUB,
"guc" => GUC,
"gud" => GUD,
"gue" => GUE,
"guf" => GUF,
"gug" => GUG,
"guh" => GUH,
"gui" => GUI,
"guk" => GUK,
"gul" => GUL,
"gum" => GUM,
"gun" => GUN,
"guo" => GUO,
"gup" => GUP,
"guq" => GUQ,
"gur" => GUR,
"gus" => GUS,
"gut" => GUT,
"guu" => GUU,
"guw" => GUW,
"gux" => GUX,
"guz" => GUZ,
"gva" => GVA,
"gvc" => GVC,
"gve" => GVE,
"gvf" => GVF,
"gvj" => GVJ,
"gvl" => GVL,
"gvm" => GVM,
"gvn" => GVN,
"gvo" => GVO,
"gvp" => GVP,
"gvr" => GVR,
"gvs" => GVS,
"gvy" => GVY,
"gwa" => GWA,
"gwb" => GWB,
"gwc" => GWC,
"gwd" => GWD,
"gwe" => GWE,
"gwf" => GWF,
"gwg" => GWG,
"gwj" => GWJ,
"gwm" => GWM,
"gwn" => GWN,
"gwr" => GWR,
"gwt" => GWT,
"gwu" => GWU,
"gww" => GWW,
"gwx" => GWX,
"gxx" => GXX,
"gya" => GYA,
"gyb" => GYB,
"gyd" => GYD,
"gye" => GYE,
"gyf" => GYF,
"gyg" => GYG,
"gyi" => GYI,
"gyl" => GYL,
"gym" => GYM,
"gyn" => GYN,
"gyo" => GYO,
"gyr" => GYR,
"gyy" => GYY,
"gyz" => GYZ,
"gza" => GZA,
"gzi" => GZI,
"gzn" => GZN,
"haa" => HAA,
"hab" => HAB,
"hac" => HAC,
"had" => HAD,
"hae" => HAE,
"haf" => HAF,
"hag" => HAG,
"hah" => HAH,
"haj" => HAJ,
"hak" => HAK,
"hal" => HAL,
"ham" => HAM,
"han" => HAN,
"hao" => HAO,
"hap" => HAP,
"haq" => HAQ,
"har" => HAR,
"has" => HAS,
"hav" => HAV,
"hax" => HAX,
"hay" => HAY,
"haz" => HAZ,
"hba" => HBA,
"hbb" => HBB,
"hbn" => HBN,
"hbo" => HBO,
"hbs" => SH,
"hbu" => HBU,
"hca" => HCA,
"hch" => HCH,
"hdn" => HDN,
"hds" => HDS,
"hdy" => HDY,
"hea" => HEA,
"hed" => HED,
"heg" => HEG,
"heh" => HEH,
"hei" => HEI,
"hem" => HEM,
"hgm" => HGM,
"hgw" => HGW,
"hhi" => HHI,
"hhr" => HHR,
"hhy" => HHY,
"hia" => HIA,
"hib" => HIB,
"hid" => HID,
"hif" => HIF,
"hig" => HIG,
"hih" => HIH,
"hii" => HII,
"hij" => HIJ,
"hik" => HIK,
"hio" => HIO,
"hir" => HIR,
"hiw" => HIW,
"hix" => HIX,
"hji" => HJI,
"hka" => HKA,
"hke" => HKE,
"hkh" => HKH,
"hkk" => HKK,
"hkn" => HKN,
"hks" => HKS,
"hla" => HLA,
"hlb" => HLB,
"hld" => HLD,
"hle" => HLE,
"hlt" => HLT,
"hlu" => HLU,
"hma" => HMA,
"hmb" => HMB,
"hmc" => HMC,
"hmd" => HMD,
"hme" => HME,
"hmf" => HMF,
"hmg" => HMG,
"hmh" => HMH,
"hmi" => HMI,
"hmj" => HMJ,
"hmk" => HMK,
"hml" => HML,
"hmm" => HMM,
"hmp" => HMP,
"hmq" => HMQ,
"hmr" => HMR,
"hms" => HMS,
"hmt" => HMT,
"hmu" => HMU,
"hmv" => HMV,
"hmw" => HMW,
"hmy" => HMY,
"hmz" => HMZ,
"hna" => HNA,
"hnd" => HND,
"hne" => HNE,
"hng" => HNG,
"hnh" => HNH,
"hni" => HNI,
"hnj" => HNJ,
"hnn" => HNN,
"hno" => HNO,
"hns" => HNS,
"hnu" => HNU,
"hoa" => HOA,
"hob" => HOB,
"hoc" => HOC,
"hod" => HOD,
"hoe" => HOE,
"hoh" => HOH,
"hoi" => HOI,
"hoj" => HOJ,
"hol" => HOL,
"hom" => HOM,
"hoo" => HOO,
"hop" => HOP,
"hor" => HOR,
"hos" => HOS,
"hot" => HOT,
"hov" => HOV,
"how" => HOW,
"hoy" => HOY,
"hoz" => HOZ,
"hpo" => HPO,
"hps" => HPS,
"hra" => HRA,
"hrc" => HRC,
"hre" => HRE,
"hrk" => HRK,
"hrm" => HRM,
"hro" => HRO,
"hrp" => HRP,
"hrt" => HRT,
"hru" => HRU,
"hrw" => HRW,
"hrx" => HRX,
"hrz" => HRZ,
"hsh" => HSH,
"hsl" => HSL,
"hsn" => HSN,
"hss" => HSS,
"hti" => HTI,
"hto" => HTO,
"hts" => HTS,
"htu" => HTU,
"htx" => HTX,
"hub" => HUB,
"huc" => HUC,
"hud" => HUD,
"hue" => HUE,
"huf" => HUF,
"hug" => HUG,
"huh" => HUH,
"hui" => HUI,
"huj" => HUJ,
"huk" => HUK,
"hul" => HUL,
"hum" => HUM,
"huo" => HUO,
"huq" => HUQ,
"hur" => HUR,
"hus" => HUS,
"hut" => HUT,
"huu" => HUU,
"huv" => HUV,
"huw" => HUW,
"hux" => HUX,
"huy" => HUY,
"huz" => HUZ,
"hvc" => HVC,
"hve" => HVE,
"hvk" => HVK,
"hvn" => HVN,
"hvv" => HVV,
"hwa" => HWA,
"hwc" => HWC,
"hwo" => HWO,
"hya" => HYA,
"hyw" => HYW,
"iai" => IAI,
"ian" => IAN,
"iar" => IAR,
"ibb" => IBB,
"ibd" => IBD,
"ibe" => IBE,
"ibg" => IBG,
"ibh" => IBH,
"ibl" => IBL,
"ibm" => IBM,
"ibn" => IBN,
"ibr" => IBR,
"ibu" => IBU,
"iby" => IBY,
"ica" => ICA,
"ich" => ICH,
"icl" => ICL,
"icr" => ICR,
"ida" => IDA,
"idb" => IDB,
"idc" => IDC,
"idd" => IDD,
"ide" => IDE,
"idi" => IDI,
"idr" => IDR,
"ids" => IDS,
"idt" => IDT,
"idu" => IDU,
"ifa" => IFA,
"ifb" => IFB,
"ife" => IFE,
"iff" => IFF,
"ifk" => IFK,
"ifm" => IFM,
"ifu" => IFU,
"ify" => IFY,
"igb" => IGB,
"ige" => IGE,
"igg" => IGG,
"igl" => IGL,
"igm" => IGM,
"ign" => IGN,
"igo" => IGO,
"igs" => IGS,
"igw" => IGW,
"ihb" => IHB,
"ihi" => IHI,
"ihp" => IHP,
"ihw" => IHW,
"iin" => IIN,
"ijc" => IJC,
"ije" => IJE,
"ijj" => IJJ,
"ijn" => IJN,
"ijs" => IJS,
"ike" => IKE,
"iki" => IKI,
"ikk" => IKK,
"ikl" => IKL,
"iko" => IKO,
"ikp" => IKP,
"ikr" => IKR,
"iks" => IKS,
"ikt" => IKT,
"ikv" => IKV,
"ikw" => IKW,
"ikx" => IKX,
"ikz" => IKZ,
"ila" => ILA,
"ilb" => ILB,
"ilg" => ILG,
"ili" => ILI,
"ilk" => ILK,
"ilm" => ILM,
"ilp" => ILP,
"ils" => ILS,
"ilu" => ILU,
"ilv" => ILV,
"ima" => IMA,
"imi" => IMI,
"iml" => IML,
"imn" => IMN,
"imo" => IMO,
"imr" => IMR,
"ims" => IMS,
"imt" => IMT,
"imy" => IMY,
"inb" => INB,
"ing" => ING,
"inj" => INJ,
"inl" => INL,
"inm" => INM,
"inn" => INN,
"ino" => INO,
"inp" => INP,
"ins" => INS,
"int" => INT,
"inz" => INZ,
"ior" => IOR,
"iou" => IOU,
"iow" => IOW,
"ipi" => IPI,
"ipo" => IPO,
"iqu" => IQU,
"iqw" => IQW,
"ire" => IRE,
"irh" => IRH,
"iri" => IRI,
"irk" => IRK,
"irn" => IRN,
"irr" => IRR,
"iru" => IRU,
"irx" => IRX,
"iry" => IRY,
"isa" => ISA,
"isc" => ISC,
"isd" => ISD,
"ise" => ISE,
"isg" => ISG,
"ish" => ISH,
"isi" => ISI,
"isk" => ISK,
"ism" => ISM,
"isn" => ISN,
"iso" => ISO,
"isr" => ISR,
"ist" => IST,
"isu" => ISU,
"itb" => ITB,
"itd" => ITD,
"ite" => ITE,
"iti" => ITI,
"itk" => ITK,
"itl" => ITL,
"itm" => ITM,
"ito" => ITO,
"itr" => ITR,
"its" => ITS,
"itt" => ITT,
"itv" => ITV,
"itw" => ITW,
"itx" => ITX,
"ity" => ITY,
"itz" => ITZ,
"ium" => IUM,
"ivb" => IVB,
"ivv" => IVV,
"iwk" => IWK,
"iwm" => IWM,
"iwo" => IWO,
"iws" => IWS,
"ixc" => IXC,
"ixl" => IXL,
"iya" => IYA,
"iyo" => IYO,
"iyx" => IYX,
"izh" => IZH,
"izr" => IZR,
"izz" => IZZ,
"jaa" => JAA,
"jab" => JAB,
"jac" => JAC,
"jad" => JAD,
"jae" => JAE,
"jaf" => JAF,
"jah" => JAH,
"jaj" => JAJ,
"jak" => JAK,
"jal" => JAL,
"jam" => JAM,
"jan" => JAN,
"jao" => JAO,
"jaq" => JAQ,
"jas" => JAS,
"jat" => JAT,
"jau" => JAU,
"jax" => JAX,
"jay" => JAY,
"jaz" => JAZ,
"jbe" => JBE,
"jbi" => JBI,
"jbj" => JBJ,
"jbk" => JBK,
"jbm" => JBM,
"jbn" => JBN,
"jbr" => JBR,
"jbt" => JBT,
"jbu" => JBU,
"jbw" => JBW,
"jcs" => JCS,
"jct" => JCT,
"jda" => JDA,
"jdg" => JDG,
"jdt" => JDT,
"jeb" => JEB,
"jee" => JEE,
"jeh" => JEH,
"jei" => JEI,
"jek" => JEK,
"jel" => JEL,
"jen" => JEN,
"jer" => JER,
"jet" => JET,
"jeu" => JEU,
"jgb" => JGB,
"jge" => JGE,
"jgk" => JGK,
"jgo" => JGO,
"jhi" => JHI,
"jhs" => JHS,
"jia" => JIA,
"jib" => JIB,
"jic" => JIC,
"jid" => JID,
"jie" => JIE,
"jig" => JIG,
"jih" => JIH,
"jii" => JII,
"jil" => JIL,
"jim" => JIM,
"jio" => JIO,
"jiq" => JIQ,
"jit" => JIT,
"jiu" => JIU,
"jiv" => JIV,
"jiy" => JIY,
"jje" => JJE,
"jjr" => JJR,
"jka" => JKA,
"jkm" => JKM,
"jko" => JKO,
"jkp" => JKP,
"jkr" => JKR,
"jks" => JKS,
"jku" => JKU,
"jle" => JLE,
"jls" => JLS,
"jma" => JMA,
"jmb" => JMB,
"jmc" => JMC,
"jmd" => JMD,
"jmi" => JMI,
"jml" => JML,
"jmn" => JMN,
"jmr" => JMR,
"jms" => JMS,
"jmw" => JMW,
"jmx" => JMX,
"jna" => JNA,
"jnd" => JND,
"jng" => JNG,
"jni" => JNI,
"jnj" => JNJ,
"jnl" => JNL,
"jns" => JNS,
"job" => JOB,
"jod" => JOD,
"jog" => JOG,
"jor" => JOR,
"jos" => JOS,
"jow" => JOW,
"jpa" => JPA,
"jqr" => JQR,
"jra" => JRA,
"jrr" => JRR,
"jrt" => JRT,
"jru" => JRU,
"jsl" => JSL,
"jua" => JUA,
"jub" => JUB,
"juc" => JUC,
"jud" => JUD,
"juh" => JUH,
"jui" => JUI,
"juk" => JUK,
"jul" => JUL,
"jum" => JUM,
"jun" => JUN,
"juo" => JUO,
"jup" => JUP,
"jur" => JUR,
"jus" => JUS,
"jut" => JUT,
"juu" => JUU,
"juw" => JUW,
"juy" => JUY,
"jvd" => JVD,
"jvn" => JVN,
"jwi" => JWI,
"jya" => JYA,
"jye" => JYE,
"jyy" => JYY,
"kad" => KAD,
"kae" => KAE,
"kaf" => KAF,
"kag" => KAG,
"kah" => KAH,
"kai" => KAI,
"kaj" => KAJ,
"kak" => KAK,
"kao" => KAO,
"kap" => KAP,
"kaq" => KAQ,
"kav" => KAV,
"kax" => KAX,
"kay" => KAY,
"kba" => KBA,
"kbb" => KBB,
"kbc" => KBC,
"kbe" => KBE,
"kbg" => KBG,
"kbh" => KBH,
"kbi" => KBI,
"kbj" => KBJ,
"kbk" => KBK,
"kbl" => KBL,
"kbm" => KBM,
"kbn" => KBN,
"kbo" => KBO,
"kbp" => KBP,
"kbq" => KBQ,
"kbr" => KBR,
"kbs" => KBS,
"kbt" => KBT,
"kbu" => KBU,
"kbv" => KBV,
"kbw" => KBW,
"kbx" => KBX,
"kby" => KBY,
"kbz" => KBZ,
"kca" => KCA,
"kcb" => KCB,
"kcc" => KCC,
"kcd" => KCD,
"kce" => KCE,
"kcf" => KCF,
"kcg" => KCG,
"kch" => KCH,
"kci" => KCI,
"kcj" => KCJ,
"kck" => KCK,
"kcl" => KCL,
"kcm" => KCM,
"kcn" => KCN,
"kco" => KCO,
"kcp" => KCP,
"kcq" => KCQ,
"kcr" => KCR,
"kcs" => KCS,
"kct" => KCT,
"kcu" => KCU,
"kcv" => KCV,
"kcw" => KCW,
"kcx" => KCX,
"kcy" => KCY,
"kcz" => KCZ,
"kda" => KDA,
"kdc" => KDC,
"kdd" => KDD,
"kde" => KDE,
"kdf" => KDF,
"kdg" => KDG,
"kdh" => KDH,
"kdi" => KDI,
"kdj" => KDJ,
"kdk" => KDK,
"kdl" => KDL,
"kdm" => KDM,
"kdn" => KDN,
"kdp" => KDP,
"kdq" => KDQ,
"kdr" => KDR,
"kdt" => KDT,
"kdu" => KDU,
"kdw" => KDW,
"kdx" => KDX,
"kdy" => KDY,
"kdz" => KDZ,
"kea" => KEA,
"keb" => KEB,
"kec" => KEC,
"ked" => KED,
"kee" => KEE,
"kef" => KEF,
"keg" => KEG,
"keh" => KEH,
"kei" => KEI,
"kej" => KEJ,
"kek" => KEK,
"kel" => KEL,
"kem" => KEM,
"ken" => KEN,
"keo" => KEO,
"kep" => KEP,
"keq" => KEQ,
"ker" => KER,
"kes" => KES,
"ket" => KET,
"keu" => KEU,
"kev" => KEV,
"kew" => KEW,
"kex" => KEX,
"key" => KEY,
"kez" => KEZ,
"kfa" => KFA,
"kfb" => KFB,
"kfc" => KFC,
"kfd" => KFD,
"kfe" => KFE,
"kff" => KFF,
"kfg" => KFG,
"kfh" => KFH,
"kfi" => KFI,
"kfj" => KFJ,
"kfk" => KFK,
"kfl" => KFL,
"kfm" => KFM,
"kfn" => KFN,
"kfo" => KFO,
"kfp" => KFP,
"kfq" => KFQ,
"kfr" => KFR,
"kfs" => KFS,
"kft" => KFT,
"kfu" => KFU,
"kfv" => KFV,
"kfw" => KFW,
"kfx" => KFX,
"kfy" => KFY,
"kfz" => KFZ,
"kga" => KGA,
"kgb" => KGB,
"kge" => KGE,
"kgf" => KGF,
"kgg" => KGG,
"kgi" => KGI,
"kgj" => KGJ,
"kgk" => KGK,
"kgl" => KGL,
"kgm" => KGM,
"kgn" => KGN,
"kgo" => KGO,
"kgp" => KGP,
"kgq" => KGQ,
"kgr" => KGR,
"kgs" => KGS,
"kgt" => KGT,
"kgu" => KGU,
"kgv" => KGV,
"kgw" => KGW,
"kgx" => KGX,
"kgy" => KGY,
"khb" => KHB,
"khc" => KHC,
"khd" => KHD,
"khe" => KHE,
"khf" => KHF,
"khg" => KHG,
"khh" => KHH,
"khj" => KHJ,
"khk" => KHK,
"khl" => KHL,
"khn" => KHN,
"khp" => KHP,
"khq" => KHQ,
"khr" => KHR,
"khs" => KHS,
"kht" => KHT,
"khu" => KHU,
"khv" => KHV,
"khw" => KHW,
"khx" => KHX,
"khy" => KHY,
"khz" => KHZ,
"kia" => KIA,
"kib" => KIB,
"kic" => KIC,
"kid" => KID,
"kie" => KIE,
"kif" => KIF,
"kig" => KIG,
"kih" => KIH,
"kii" => KII,
"kij" => KIJ,
"kil" => KIL,
"kim" => KIM,
"kio" => KIO,
"kip" => KIP,
"kiq" => KIQ,
"kis" => KIS,
"kit" => KIT,
"kiu" => KIU,
"kiv" => KIV,
"kiw" => KIW,
"kix" => KIX,
"kiy" => KIY,
"kiz" => KIZ,
"kja" => KJA,
"kjb" => KJB,
"kjc" => KJC,
"kjd" => KJD,
"kje" => KJE,
"kjg" => KJG,
"kjh" => KJH,
"kji" => KJI,
"kjj" => KJJ,
"kjk" => KJK,
"kjl" => KJL,
"kjm" => KJM,
"kjn" => KJN,
"kjo" => KJO,
"kjp" => KJP,
"kjq" => KJQ,
"kjr" => KJR,
"kjs" => KJS,
"kjt" => KJT,
"kju" => KJU,
"kjv" => KJV,
"kjx" => KJX,
"kjy" => KJY,
"kjz" => KJZ,
"kka" => KKA,
"kkb" => KKB,
"kkc" => KKC,
"kkd" => KKD,
"kke" => KKE,
"kkf" => KKF,
"kkg" => KKG,
"kkh" => KKH,
"kki" => KKI,
"kkj" => KKJ,
"kkk" => KKK,
"kkl" => KKL,
"kkm" => KKM,
"kkn" => KKN,
"kko" => KKO,
"kkp" => KKP,
"kkq" => KKQ,
"kkr" => KKR,
"kks" => KKS,
"kkt" => KKT,
"kku" => KKU,
"kkv" => KKV,
"kkw" => KKW,
"kkx" => KKX,
"kky" => KKY,
"kkz" => KKZ,
"kla" => KLA,
"klb" => KLB,
"klc" => KLC,
"kld" => KLD,
"kle" => KLE,
"klf" => KLF,
"klg" => KLG,
"klh" => KLH,
"kli" => KLI,
"klj" => KLJ,
"klk" => KLK,
"kll" => KLL,
"klm" => KLM,
"kln" => KLN,
"klo" => KLO,
"klp" => KLP,
"klq" => KLQ,
"klr" => KLR,
"kls" => KLS,
"klt" => KLT,
"klu" => KLU,
"klv" => KLV,
"klw" => KLW,
"klx" => KLX,
"kly" => KLY,
"klz" => KLZ,
"kma" => KMA,
"kmc" => KMC,
"kmd" => KMD,
"kme" => KME,
"kmf" => KMF,
"kmg" => KMG,
"kmh" => KMH,
"kmi" => KMI,
"kmj" => KMJ,
"kmk" => KMK,
"kml" => KML,
"kmm" => KMM,
"kmn" => KMN,
"kmo" => KMO,
"kmp" => KMP,
"kmq" => KMQ,
"kmr" => KMR,
"kms" => KMS,
"kmt" => KMT,
"kmu" => KMU,
"kmv" => KMV,
"kmw" => KMW,
"kmx" => KMX,
"kmy" => KMY,
"kmz" => KMZ,
"kna" => KNA,
"knb" => KNB,
"knc" => KNC,
"knd" => KND,
"kne" => KNE,
"knf" => KNF,
"kng" => KNG,
"kni" => KNI,
"knj" => KNJ,
"knk" => KNK,
"knl" => KNL,
"knm" => KNM,
"knn" => KNN,
"kno" => KNO,
"knp" => KNP,
"knq" => KNQ,
"knr" => KNR,
"kns" => KNS,
"knt" => KNT,
"knu" => KNU,
"knv" => KNV,
"knw" => KNW,
"knx" => KNX,
"kny" => KNY,
"knz" => KNZ,
"koa" => KOA,
"koc" => KOC,
"kod" => KOD,
"koe" => KOE,
"kof" => KOF,
"kog" => KOG,
"koh" => KOH,
"koi" => KOI,
"kol" => KOL,
"koo" => KOO,
"kop" => KOP,
"koq" => KOQ,
"kot" => KOT,
"kou" => KOU,
"kov" => KOV,
"kow" => KOW,
"koy" => KOY,
"koz" => KOZ,
"kpa" => KPA,
"kpb" => KPB,
"kpc" => KPC,
"kpd" => KPD,
"kpf" => KPF,
"kpg" => KPG,
"kph" => KPH,
"kpi" => KPI,
"kpj" => KPJ,
"kpk" => KPK,
"kpl" => KPL,
"kpm" => KPM,
"kpn" => KPN,
"kpo" => KPO,
"kpq" => KPQ,
"kpr" => KPR,
"kps" => KPS,
"kpt" => KPT,
"kpu" => KPU,
"kpv" => KPV,
"kpw" => KPW,
"kpx" => KPX,
"kpy" => KPY,
"kpz" => KPZ,
"kqa" => KQA,
"kqb" => KQB,
"kqc" => KQC,
"kqd" => KQD,
"kqe" => KQE,
"kqf" => KQF,
"kqg" => KQG,
"kqh" => KQH,
"kqi" => KQI,
"kqj" => KQJ,
"kqk" => KQK,
"kql" => KQL,
"kqm" => KQM,
"kqn" => KQN,
"kqo" => KQO,
"kqp" => KQP,
"kqq" => KQQ,
"kqr" => KQR,
"kqs" => KQS,
"kqt" => KQT,
"kqu" => KQU,
"kqv" => KQV,
"kqw" => KQW,
"kqx" => KQX,
"kqy" => KQY,
"kqz" => KQZ,
"kra" => KRA,
"krb" => KRB,
"krd" => KRD,
"kre" => KRE,
"krf" => KRF,
"krh" => KRH,
"kri" => KRI,
"krj" => KRJ,
"krk" => KRK,
"krn" => KRN,
"krp" => KRP,
"krr" => KRR,
"krs" => KRS,
"krt" => KRT,
"krv" => KRV,
"krw" => KRW,
"krx" => KRX,
"kry" => KRY,
"krz" => KRZ,
"ksa" => KSA,
"ksb" => KSB,
"ksc" => KSC,
"ksd" => KSD,
"kse" => KSE,
"ksf" => KSF,
"ksg" => KSG,
"ksh" => KSH,
"ksi" => KSI,
"ksj" => KSJ,
"ksk" => KSK,
"ksl" => KSL,
"ksm" => KSM,
"ksn" => KSN,
"kso" => KSO,
"ksp" => KSP,
"ksq" => KSQ,
"ksr" => KSR,
"kss" => KSS,
"kst" => KST,
"ksu" => KSU,
"ksv" => KSV,
"ksw" => KSW,
"ksx" => KSX,
"ksy" => KSY,
"ksz" => KSZ,
"kta" => KTA,
"ktb" => KTB,
"ktc" => KTC,
"ktd" => KTD,
"kte" => KTE,
"ktf" => KTF,
"ktg" => KTG,
"kth" => KTH,
"kti" => KTI,
"ktj" => KTJ,
"ktk" => KTK,
"ktl" => KTL,
"ktm" => KTM,
"ktn" => KTN,
"kto" => KTO,
"ktp" => KTP,
"ktq" => KTQ,
"kts" => KTS,
"ktt" => KTT,
"ktu" => KTU,
"ktv" => KTV,
"ktw" => KTW,
"ktx" => KTX,
"kty" => KTY,
"ktz" => KTZ,
"kub" => KUB,
"kuc" => KUC,
"kud" => KUD,
"kue" => KUE,
"kuf" => KUF,
"kug" => KUG,
"kuh" => KUH,
"kui" => KUI,
"kuj" => KUJ,
"kuk" => KUK,
"kul" => KUL,
"kun" => KUN,
"kuo" => KUO,
"kup" => KUP,
"kuq" => KUQ,
"kus" => KUS,
"kuu" => KUU,
"kuv" => KUV,
"kuw" => KUW,
"kux" => KUX,
"kuy" => KUY,
"kuz" => KUZ,
"kva" => KVA,
"kvb" => KVB,
"kvc" => KVC,
"kvd" => KVD,
"kve" => KVE,
"kvf" => KVF,
"kvg" => KVG,
"kvh" => KVH,
"kvi" => KVI,
"kvj" => KVJ,
"kvk" => KVK,
"kvl" => KVL,
"kvm" => KVM,
"kvn" => KVN,
"kvo" => KVO,
"kvp" => KVP,
"kvq" => KVQ,
"kvr" => KVR,
"kvt" => KVT,
"kvu" => KVU,
"kvv" => KVV,
"kvw" => KVW,
"kvx" => KVX,
"kvy" => KVY,
"kvz" => KVZ,
"kwa" => KWA,
"kwb" => KWB,
"kwc" => KWC,
"kwd" => KWD,
"kwe" => KWE,
"kwf" => KWF,
"kwg" => KWG,
"kwh" => KWH,
"kwi" => KWI,
"kwj" => KWJ,
"kwk" => KWK,
"kwl" => KWL,
"kwm" => KWM,
"kwn" => KWN,
"kwo" => KWO,
"kwp" => KWP,
"kwr" => KWR,
"kws" => KWS,
"kwt" => KWT,
"kwu" => KWU,
"kwv" => KWV,
"kww" => KWW,
"kwx" => KWX,
"kwy" => KWY,
"kwz" => KWZ,
"kxa" => KXA,
"kxb" => KXB,
"kxc" => KXC,
"kxd" => KXD,
"kxf" => KXF,
"kxh" => KXH,
"kxi" => KXI,
"kxj" => KXJ,
"kxk" => KXK,
"kxm" => KXM,
"kxn" => KXN,
"kxo" => KXO,
"kxp" => KXP,
"kxq" => KXQ,
"kxr" => KXR,
"kxs" => KXS,
"kxt" => KXT,
"kxv" => KXV,
"kxw" => KXW,
"kxx" => KXX,
"kxy" => KXY,
"kxz" => KXZ,
"kya" => KYA,
"kyb" => KYB,
"kyc" => KYC,
"kyd" => KYD,
"kye" => KYE,
"kyf" => KYF,
"kyg" => KYG,
"kyh" => KYH,
"kyi" => KYI,
"kyj" => KYJ,
"kyk" => KYK,
"kyl" => KYL,
"kym" => KYM,
"kyn" => KYN,
"kyo" => KYO,
"kyp" => KYP,
"kyq" => KYQ,
"kyr" => KYR,
"kys" => KYS,
"kyt" => KYT,
"kyu" => KYU,
"kyv" => KYV,
"kyw" => KYW,
"kyx" => KYX,
"kyy" => KYY,
"kyz" => KYZ,
"kza" => KZA,
"kzb" => KZB,
"kzc" => KZC,
"kzd" => KZD,
"kze" => KZE,
"kzf" => KZF,
"kzg" => KZG,
"kzi" => KZI,
"kzk" => KZK,
"kzl" => KZL,
"kzm" => KZM,
"kzn" => KZN,
"kzo" => KZO,
"kzp" => KZP,
"kzq" => KZQ,
"kzr" => KZR,
"kzs" => KZS,
"kzu" => KZU,
"kzv" => KZV,
"kzw" => KZW,
"kzx" => KZX,
"kzy" => KZY,
"kzz" => KZZ,
"laa" => LAA,
"lab" => LAB,
"lac" => LAC,
"lae" => LAE,
"laf" => LAF,
"lag" => LAG,
"lai" => LAI,
"laj" => LAJ,
"lal" => LAL,
"lan" => LAN,
"lap" => LAP,
"laq" => LAQ,
"lar" => LAR,
"las" => LAS,
"lau" => LAU,
"law" => LAW,
"lax" => LAX,
"lay" => LAY,
"laz" => LAZ,
"lbb" => LBB,
"lbc" => LBC,
"lbe" => LBE,
"lbf" => LBF,
"lbg" => LBG,
"lbi" => LBI,
"lbj" => LBJ,
"lbk" => LBK,
"lbl" => LBL,
"lbm" => LBM,
"lbn" => LBN,
"lbo" => LBO,
"lbq" => LBQ,
"lbr" => LBR,
"lbs" => LBS,
"lbt" => LBT,
"lbu" => LBU,
"lbv" => LBV,
"lbw" => LBW,
"lbx" => LBX,
"lby" => LBY,
"lbz" => LBZ,
"lcc" => LCC,
"lcd" => LCD,
"lce" => LCE,
"lcf" => LCF,
"lch" => LCH,
"lcl" => LCL,
"lcm" => LCM,
"lcp" => LCP,
"lcq" => LCQ,
"lcs" => LCS,
"lda" => LDA,
"ldb" => LDB,
"ldd" => LDD,
"ldg" => LDG,
"ldh" => LDH,
"ldi" => LDI,
"ldj" => LDJ,
"ldk" => LDK,
"ldl" => LDL,
"ldm" => LDM,
"ldn" => LDN,
"ldo" => LDO,
"ldp" => LDP,
"ldq" => LDQ,
"lea" => LEA,
"leb" => LEB,
"lec" => LEC,
"led" => LED,
"lee" => LEE,
"lef" => LEF,
"leh" => LEH,
"lei" => LEI,
"lej" => LEJ,
"lek" => LEK,
"lel" => LEL,
"lem" => LEM,
"len" => LEN,
"leo" => LEO,
"lep" => LEP,
"leq" => LEQ,
"ler" => LER,
"les" => LES,
"let" => LET,
"leu" => LEU,
"lev" => LEV,
"lew" => LEW,
"lex" => LEX,
"ley" => LEY,
"lfa" => LFA,
"lfn" => LFN,
"lga" => LGA,
"lgb" => LGB,
"lgg" => LGG,
"lgh" => LGH,
"lgi" => LGI,
"lgk" => LGK,
"lgl" => LGL,
"lgm" => LGM,
"lgn" => LGN,
"lgo" => LGO,
"lgq" => LGQ,
"lgr" => LGR,
"lgt" => LGT,
"lgu" => LGU,
"lgz" => LGZ,
"lha" => LHA,
"lhh" => LHH,
"lhi" => LHI,
"lhl" => LHL,
"lhm" => LHM,
"lhn" => LHN,
"lhp" => LHP,
"lhs" => LHS,
"lht" => LHT,
"lhu" => LHU,
"lia" => LIA,
"lib" => LIB,
"lic" => LIC,
"lid" => LID,
"lie" => LIE,
"lif" => LIF,
"lig" => LIG,
"lih" => LIH,
"lij" => LIJ,
"lik" => LIK,
"lil" => LIL,
"lio" => LIO,
"lip" => LIP,
"liq" => LIQ,
"lir" => LIR,
"lis" => LIS,
"liu" => LIU,
"liv" => LIV,
"liw" => LIW,
"lix" => LIX,
"liy" => LIY,
"liz" => LIZ,
"lja" => LJA,
"lje" => LJE,
"lji" => LJI,
"ljl" => LJL,
"ljp" => LJP,
"ljw" => LJW,
"ljx" => LJX,
"lka" => LKA,
"lkb" => LKB,
"lkc" => LKC,
"lkd" => LKD,
"lke" => LKE,
"lkh" => LKH,
"lki" => LKI,
"lkj" => LKJ,
"lkl" => LKL,
"lkm" => LKM,
"lkn" => LKN,
"lko" => LKO,
"lkr" => LKR,
"lks" => LKS,
"lkt" => LKT,
"lku" => LKU,
"lky" => LKY,
"lla" => LLA,
"llb" => LLB,
"llc" => LLC,
"lld" => LLD,
"lle" => LLE,
"llf" => LLF,
"llg" => LLG,
"llh" => LLH,
"lli" => LLI,
"llj" => LLJ,
"llk" => LLK,
"lll" => LLL,
"llm" => LLM,
"lln" => LLN,
"llp" => LLP,
"llq" => LLQ,
"lls" => LLS,
"llu" => LLU,
"llx" => LLX,
"lma" => LMA,
"lmb" => LMB,
"lmc" => LMC,
"lmd" => LMD,
"lme" => LME,
"lmf" => LMF,
"lmg" => LMG,
"lmh" => LMH,
"lmi" => LMI,
"lmj" => LMJ,
"lmk" => LMK,
"lml" => LML,
"lmn" => LMN,
"lmo" => LMO,
"lmp" => LMP,
"lmq" => LMQ,
"lmr" => LMR,
"lmu" => LMU,
"lmv" => LMV,
"lmw" => LMW,
"lmx" => LMX,
"lmy" => LMY,
"lna" => LNA,
"lnb" => LNB,
"lnd" => LND,
"lng" => LNG,
"lnh" => LNH,
"lni" => LNI,
"lnj" => LNJ,
"lnl" => LNL,
"lnm" => LNM,
"lnn" => LNN,
"lns" => LNS,
"lnu" => LNU,
"lnw" => LNW,
"lnz" => LNZ,
"loa" => LOA,
"lob" => LOB,
"loc" => LOC,
"loe" => LOE,
"lof" => LOF,
"log" => LOG,
"loh" => LOH,
"loi" => LOI,
"loj" => LOJ,
"lok" => LOK,
"lom" => LOM,
"lon" => LON,
"loo" => LOO,
"lop" => LOP,
"loq" => LOQ,
"lor" => LOR,
"los" => LOS,
"lot" => LOT,
"lou" => LOU,
"lov" => LOV,
"low" => LOW,
"lox" => LOX,
"loy" => LOY,
"lpa" => LPA,
"lpe" => LPE,
"lpn" => LPN,
"lpo" => LPO,
"lpx" => LPX,
"lqr" => LQR,
"lra" => LRA,
"lrc" => LRC,
"lre" => LRE,
"lrg" => LRG,
"lri" => LRI,
"lrk" => LRK,
"lrl" => LRL,
"lrm" => LRM,
"lrn" => LRN,
"lro" => LRO,
"lrr" => LRR,
"lrt" => LRT,
"lrv" => LRV,
"lrz" => LRZ,
"lsa" => LSA,
"lsb" => LSB,
"lsc" => LSC,
"lsd" => LSD,
"lse" => LSE,
"lsh" => LSH,
"lsi" => LSI,
"lsl" => LSL,
"lsm" => LSM,
"lsn" => LSN,
"lso" => LSO,
"lsp" => LSP,
"lsr" => LSR,
"lss" => LSS,
"lst" => LST,
"lsv" => LSV,
"lsw" => LSW,
"lsy" => LSY,
"ltc" => LTC,
"ltg" => LTG,
"lth" => LTH,
"lti" => LTI,
"ltn" => LTN,
"lto" => LTO,
"lts" => LTS,
"ltu" => LTU,
"luc" => LUC,
"lud" => LUD,
"lue" => LUE,
"luf" => LUF,
"luj" => LUJ,
"luk" => LUK,
"lul" => LUL,
"lum" => LUM,
"lup" => LUP,
"luq" => LUQ,
"lur" => LUR,
"lut" => LUT,
"luu" => LUU,
"luv" => LUV,
"luw" => LUW,
"luy" => LUY,
"luz" => LUZ,
"lva" => LVA,
"lvi" => LVI,
"lvk" => LVK,
"lvs" => LVS,
"lvu" => LVU,
"lwa" => LWA,
"lwe" => LWE,
"lwg" => LWG,
"lwh" => LWH,
"lwl" => LWL,
"lwm" => LWM,
"lwo" => LWO,
"lws" => LWS,
"lwt" => LWT,
"lwu" => LWU,
"lww" => LWW,
"lxm" => LXM,
"lya" => LYA,
"lyg" => LYG,
"lyn" => LYN,
"lzh" => LZH,
"lzl" => LZL,
"lzn" => LZN,
"lzz" => LZZ,
"maa" => MAA,
"mab" => MAB,
"mae" => MAE,
"maf" => MAF,
"maj" => MAJ,
"mam" => MAM,
"maq" => MAQ,
"mat" => MAT,
"mau" => MAU,
"mav" => MAV,
"maw" => MAW,
"max" => MAX,
"maz" => MAZ,
"mba" => MBA,
"mbb" => MBB,
"mbc" => MBC,
"mbd" => MBD,
"mbe" => MBE,
"mbf" => MBF,
"mbh" => MBH,
"mbi" => MBI,
"mbj" => MBJ,
"mbk" => MBK,
"mbl" => MBL,
"mbm" => MBM,
"mbn" => MBN,
"mbo" => MBO,
"mbp" => MBP,
"mbq" => MBQ,
"mbr" => MBR,
"mbs" => MBS,
"mbt" => MBT,
"mbu" => MBU,
"mbv" => MBV,
"mbw" => MBW,
"mbx" => MBX,
"mby" => MBY,
"mbz" => MBZ,
"mca" => MCA,
"mcb" => MCB,
"mcc" => MCC,
"mcd" => MCD,
"mce" => MCE,
"mcf" => MCF,
"mcg" => MCG,
"mch" => MCH,
"mci" => MCI,
"mcj" => MCJ,
"mck" => MCK,
"mcl" => MCL,
"mcm" => MCM,
"mcn" => MCN,
"mco" => MCO,
"mcp" => MCP,
"mcq" => MCQ,
"mcr" => MCR,
"mcs" => MCS,
"mct" => MCT,
"mcu" => MCU,
"mcv" => MCV,
"mcw" => MCW,
"mcx" => MCX,
"mcy" => MCY,
"mcz" => MCZ,
"mda" => MDA,
"mdb" => MDB,
"mdc" => MDC,
"mdd" => MDD,
"mde" => MDE,
"mdg" => MDG,
"mdh" => MDH,
"mdi" => MDI,
"mdj" => MDJ,
"mdk" => MDK,
"mdl" => MDL,
"mdm" => MDM,
"mdn" => MDN,
"mdp" => MDP,
"mdq" => MDQ,
"mds" => MDS,
"mdt" => MDT,
"mdu" => MDU,
"mdv" => MDV,
"mdw" => MDW,
"mdx" => MDX,
"mdy" => MDY,
"mdz" => MDZ,
"mea" => MEA,
"meb" => MEB,
"mec" => MEC,
"med" => MED,
"mee" => MEE,
"mef" => MEF,
"meh" => MEH,
"mei" => MEI,
"mej" => MEJ,
"mek" => MEK,
"mel" => MEL,
"mem" => MEM,
"meo" => MEO,
"mep" => MEP,
"meq" => MEQ,
"mer" => MER,
"mes" => MES,
"met" => MET,
"meu" => MEU,
"mev" => MEV,
"mew" => MEW,
"mey" => MEY,
"mez" => MEZ,
"mfa" => MFA,
"mfb" => MFB,
"mfc" => MFC,
"mfd" => MFD,
"mfe" => MFE,
"mff" => MFF,
"mfg" => MFG,
"mfh" => MFH,
"mfi" => MFI,
"mfj" => MFJ,
"mfk" => MFK,
"mfl" => MFL,
"mfm" => MFM,
"mfn" => MFN,
"mfo" => MFO,
"mfp" => MFP,
"mfq" => MFQ,
"mfr" => MFR,
"mfs" => MFS,
"mft" => MFT,
"mfu" => MFU,
"mfv" => MFV,
"mfw" => MFW,
"mfx" => MFX,
"mfy" => MFY,
"mfz" => MFZ,
"mgb" => MGB,
"mgc" => MGC,
"mgd" => MGD,
"mge" => MGE,
"mgf" => MGF,
"mgg" => MGG,
"mgh" => MGH,
"mgi" => MGI,
"mgj" => MGJ,
"mgk" => MGK,
"mgl" => MGL,
"mgm" => MGM,
"mgn" => MGN,
"mgo" => MGO,
"mgp" => MGP,
"mgq" => MGQ,
"mgr" => MGR,
"mgs" => MGS,
"mgt" => MGT,
"mgu" => MGU,
"mgv" => MGV,
"mgw" => MGW,
"mgy" => MGY,
"mgz" => MGZ,
"mha" => MHA,
"mhb" => MHB,
"mhc" => MHC,
"mhd" => MHD,
"mhe" => MHE,
"mhf" => MHF,
"mhg" => MHG,
"mhi" => MHI,
"mhj" => MHJ,
"mhk" => MHK,
"mhl" => MHL,
"mhm" => MHM,
"mhn" => MHN,
"mho" => MHO,
"mhp" => MHP,
"mhq" => MHQ,
"mhr" => MHR,
"mhs" => MHS,
"mht" => MHT,
"mhu" => MHU,
"mhw" => MHW,
"mhx" => MHX,
"mhy" => MHY,
"mhz" => MHZ,
"mia" => MIA,
"mib" => MIB,
"mid" => MID,
"mie" => MIE,
"mif" => MIF,
"mig" => MIG,
"mih" => MIH,
"mii" => MII,
"mij" => MIJ,
"mik" => MIK,
"mil" => MIL,
"mim" => MIM,
"mio" => MIO,
"mip" => MIP,
"miq" => MIQ,
"mir" => MIR,
"mit" => MIT,
"miu" => MIU,
"miw" => MIW,
"mix" => MIX,
"miy" => MIY,
"miz" => MIZ,
"mjb" => MJB,
"mjc" => MJC,
"mjd" => MJD,
"mje" => MJE,
"mjg" => MJG,
"mjh" => MJH,
"mji" => MJI,
"mjj" => MJJ,
"mjk" => MJK,
"mjl" => MJL,
"mjm" => MJM,
"mjn" => MJN,
"mjo" => MJO,
"mjp" => MJP,
"mjq" => MJQ,
"mjr" => MJR,
"mjs" => MJS,
"mjt" => MJT,
"mju" => MJU,
"mjv" => MJV,
"mjw" => MJW,
"mjx" => MJX,
"mjy" => MJY,
"mjz" => MJZ,
"mka" => MKA,
"mkb" => MKB,
"mkc" => MKC,
"mke" => MKE,
"mkf" => MKF,
"mkg" => MKG,
"mki" => MKI,
"mkj" => MKJ,
"mkk" => MKK,
"mkl" => MKL,
"mkm" => MKM,
"mkn" => MKN,
"mko" => MKO,
"mkp" => MKP,
"mkq" => MKQ,
"mkr" => MKR,
"mks" => MKS,
"mkt" => MKT,
"mku" => MKU,
"mkv" => MKV,
"mkw" => MKW,
"mkx" => MKX,
"mky" => MKY,
"mkz" => MKZ,
"mla" => MLA,
"mlb" => MLB,
"mlc" => MLC,
"mle" => MLE,
"mlf" => MLF,
"mlh" => MLH,
"mli" => MLI,
"mlj" => MLJ,
"mlk" => MLK,
"mll" => MLL,
"mlm" => MLM,
"mln" => MLN,
"mlo" => MLO,
"mlp" => MLP,
"mlq" => MLQ,
"mlr" => MLR,
"mls" => MLS,
"mlu" => MLU,
"mlv" => MLV,
"mlw" => MLW,
"mlx" => MLX,
"mlz" => MLZ,
"mma" => MMA,
"mmb" => MMB,
"mmc" => MMC,
"mmd" => MMD,
"mme" => MME,
"mmf" => MMF,
"mmg" => MMG,
"mmh" => MMH,
"mmi" => MMI,
"mmj" => MMJ,
"mmk" => MMK,
"mml" => MML,
"mmm" => MMM,
"mmn" => MMN,
"mmo" => MMO,
"mmp" => MMP,
"mmq" => MMQ,
"mmr" => MMR,
"mmt" => MMT,
"mmu" => MMU,
"mmv" => MMV,
"mmw" => MMW,
"mmx" => MMX,
"mmy" => MMY,
"mmz" => MMZ,
"mna" => MNA,
"mnb" => MNB,
"mnd" => MND,
"mne" => MNE,
"mnf" => MNF,
"mng" => MNG,
"mnh" => MNH,
"mnj" => MNJ,
"mnk" => MNK,
"mnl" => MNL,
"mnm" => MNM,
"mnn" => MNN,
"mnp" => MNP,
"mnq" => MNQ,
"mnr" => MNR,
"mns" => MNS,
"mnu" => MNU,
"mnv" => MNV,
"mnw" => MNW,
"mnx" => MNX,
"mny" => MNY,
"mnz" => MNZ,
"moa" => MOA,
"moc" => MOC,
"mod" => MOD,
"moe" => MOE,
"mog" => MOG,
"moi" => MOI,
"moj" => MOJ,
"mok" => MOK,
"mom" => MOM,
"moo" => MOO,
"mop" => MOP,
"moq" => MOQ,
"mor" => MOR,
"mot" => MOT,
"mou" => MOU,
"mov" => MOV,
"mow" => MOW,
"mox" => MOX,
"moy" => MOY,
"moz" => MOZ,
"mpa" => MPA,
"mpb" => MPB,
"mpc" => MPC,
"mpd" => MPD,
"mpe" => MPE,
"mpg" => MPG,
"mph" => MPH,
"mpi" => MPI,
"mpj" => MPJ,
"mpk" => MPK,
"mpl" => MPL,
"mpm" => MPM,
"mpn" => MPN,
"mpo" => MPO,
"mpp" => MPP,
"mpq" => MPQ,
"mpr" => MPR,
"mps" => MPS,
"mpt" => MPT,
"mpu" => MPU,
"mpv" => MPV,
"mpw" => MPW,
"mpx" => MPX,
"mpy" => MPY,
"mpz" => MPZ,
"mqa" => MQA,
"mqb" => MQB,
"mqc" => MQC,
"mqe" => MQE,
"mqf" => MQF,
"mqg" => MQG,
"mqh" => MQH,
"mqi" => MQI,
"mqj" => MQJ,
"mqk" => MQK,
"mql" => MQL,
"mqm" => MQM,
"mqn" => MQN,
"mqo" => MQO,
"mqp" => MQP,
"mqq" => MQQ,
"mqr" => MQR,
"mqs" => MQS,
"mqt" => MQT,
"mqu" => MQU,
"mqv" => MQV,
"mqw" => MQW,
"mqx" => MQX,
"mqy" => MQY,
"mqz" => MQZ,
"mra" => MRA,
"mrb" => MRB,
"mrc" => MRC,
"mrd" => MRD,
"mre" => MRE,
"mrf" => MRF,
"mrg" => MRG,
"mrh" => MRH,
"mrj" => MRJ,
"mrk" => MRK,
"mrl" => MRL,
"mrm" => MRM,
"mrn" => MRN,
"mro" => MRO,
"mrp" => MRP,
"mrq" => MRQ,
"mrr" => MRR,
"mrs" => MRS,
"mrt" => MRT,
"mru" => MRU,
"mrv" => MRV,
"mrw" => MRW,
"mrx" => MRX,
"mry" => MRY,
"mrz" => MRZ,
"msb" => MSB,
"msc" => MSC,
"msd" => MSD,
"mse" => MSE,
"msf" => MSF,
"msg" => MSG,
"msh" => MSH,
"msi" => MSI,
"msj" => MSJ,
"msk" => MSK,
"msl" => MSL,
"msm" => MSM,
"msn" => MSN,
"mso" => MSO,
"msp" => MSP,
"msq" => MSQ,
"msr" => MSR,
"mss" => MSS,
"msu" => MSU,
"msv" => MSV,
"msw" => MSW,
"msx" => MSX,
"msy" => MSY,
"msz" => MSZ,
"mta" => MTA,
"mtb" => MTB,
"mtc" => MTC,
"mtd" => MTD,
"mte" => MTE,
"mtf" => MTF,
"mtg" => MTG,
"mth" => MTH,
"mti" => MTI,
"mtj" => MTJ,
"mtk" => MTK,
"mtl" => MTL,
"mtm" => MTM,
"mtn" => MTN,
"mto" => MTO,
"mtp" => MTP,
"mtq" => MTQ,
"mtr" => MTR,
"mts" => MTS,
"mtt" => MTT,
"mtu" => MTU,
"mtv" => MTV,
"mtw" => MTW,
"mtx" => MTX,
"mty" => MTY,
"mua" => MUA,
"mub" => MUB,
"muc" => MUC,
"mud" => MUD,
"mue" => MUE,
"mug" => MUG,
"muh" => MUH,
"mui" => MUI,
"muj" => MUJ,
"muk" => MUK,
"mum" => MUM,
"muo" => MUO,
"mup" => MUP,
"muq" => MUQ,
"mur" => MUR,
"mut" => MUT,
"muu" => MUU,
"muv" => MUV,
"mux" => MUX,
"muy" => MUY,
"muz" => MUZ,
"mva" => MVA,
"mvb" => MVB,
"mvd" => MVD,
"mve" => MVE,
"mvf" => MVF,
"mvg" => MVG,
"mvh" => MVH,
"mvi" => MVI,
"mvk" => MVK,
"mvl" => MVL,
"mvn" => MVN,
"mvo" => MVO,
"mvp" => MVP,
"mvq" => MVQ,
"mvr" => MVR,
"mvs" => MVS,
"mvt" => MVT,
"mvu" => MVU,
"mvv" => MVV,
"mvw" => MVW,
"mvx" => MVX,
"mvy" => MVY,
"mvz" => MVZ,
"mwa" => MWA,
"mwb" => MWB,
"mwc" => MWC,
"mwe" => MWE,
"mwf" => MWF,
"mwg" => MWG,
"mwh" => MWH,
"mwi" => MWI,
"mwk" => MWK,
"mwm" => MWM,
"mwn" => MWN,
"mwo" => MWO,
"mwp" => MWP,
"mwq" => MWQ,
"mws" => MWS,
"mwt" => MWT,
"mwu" => MWU,
"mwv" => MWV,
"mww" => MWW,
"mwz" => MWZ,
"mxa" => MXA,
"mxb" => MXB,
"mxc" => MXC,
"mxd" => MXD,
"mxe" => MXE,
"mxf" => MXF,
"mxg" => MXG,
"mxh" => MXH,
"mxi" => MXI,
"mxj" => MXJ,
"mxk" => MXK,
"mxl" => MXL,
"mxm" => MXM,
"mxn" => MXN,
"mxo" => MXO,
"mxp" => MXP,
"mxq" => MXQ,
"mxr" => MXR,
"mxs" => MXS,
"mxt" => MXT,
"mxu" => MXU,
"mxv" => MXV,
"mxw" => MXW,
"mxx" => MXX,
"mxy" => MXY,
"mxz" => MXZ,
"myb" => MYB,
"myc" => MYC,
"mye" => MYE,
"myf" => MYF,
"myg" => MYG,
"myh" => MYH,
"myj" => MYJ,
"myk" => MYK,
"myl" => MYL,
"mym" => MYM,
"myo" => MYO,
"myp" => MYP,
"myr" => MYR,
"mys" => MYS,
"myu" => MYU,
"myw" => MYW,
"myx" => MYX,
"myy" => MYY,
"myz" => MYZ,
"mza" => MZA,
"mzb" => MZB,
"mzc" => MZC,
"mzd" => MZD,
"mze" => MZE,
"mzg" => MZG,
"mzh" => MZH,
"mzi" => MZI,
"mzj" => MZJ,
"mzk" => MZK,
"mzl" => MZL,
"mzm" => MZM,
"mzn" => MZN,
"mzo" => MZO,
"mzp" => MZP,
"mzq" => MZQ,
"mzr" => MZR,
"mzs" => MZS,
"mzt" => MZT,
"mzu" => MZU,
"mzv" => MZV,
"mzw" => MZW,
"mzx" => MZX,
"mzy" => MZY,
"mzz" => MZZ,
"naa" => NAA,
"nab" => NAB,
"nac" => NAC,
"nae" => NAE,
"naf" => NAF,
"nag" => NAG,
"naj" => NAJ,
"nak" => NAK,
"nal" => NAL,
"nam" => NAM,
"nan" => NAN,
"nao" => NAO,
"naq" => NAQ,
"nar" => NAR,
"nas" => NAS,
"nat" => NAT,
"naw" => NAW,
"nax" => NAX,
"nay" => NAY,
"naz" => NAZ,
"nba" => NBA,
"nbb" => NBB,
"nbc" => NBC,
"nbd" => NBD,
"nbe" => NBE,
"nbg" => NBG,
"nbh" => NBH,
"nbi" => NBI,
"nbj" => NBJ,
"nbk" => NBK,
"nbm" => NBM,
"nbn" => NBN,
"nbo" => NBO,
"nbp" => NBP,
"nbq" => NBQ,
"nbr" => NBR,
"nbs" => NBS,
"nbt" => NBT,
"nbu" => NBU,
"nbv" => NBV,
"nbw" => NBW,
"nby" => NBY,
"nca" => NCA,
"ncb" => NCB,
"ncc" => NCC,
"ncd" => NCD,
"nce" => NCE,
"ncf" => NCF,
"ncg" => NCG,
"nch" => NCH,
"nci" => NCI,
"ncj" => NCJ,
"nck" => NCK,
"ncl" => NCL,
"ncm" => NCM,
"ncn" => NCN,
"nco" => NCO,
"ncq" => NCQ,
"ncr" => NCR,
"ncs" => NCS,
"nct" => NCT,
"ncu" => NCU,
"ncx" => NCX,
"ncz" => NCZ,
"nda" => NDA,
"ndb" => NDB,
"ndc" => NDC,
"ndd" => NDD,
"ndf" => NDF,
"ndg" => NDG,
"ndh" => NDH,
"ndi" => NDI,
"ndj" => NDJ,
"ndk" => NDK,
"ndl" => NDL,
"ndm" => NDM,
"ndn" => NDN,
"ndp" => NDP,
"ndq" => NDQ,
"ndr" => NDR,
"ndt" => NDT,
"ndu" => NDU,
"ndv" => NDV,
"ndw" => NDW,
"ndx" => NDX,
"ndy" => NDY,
"ndz" => NDZ,
"nea" => NEA,
"neb" => NEB,
"nec" => NEC,
"ned" => NED,
"nee" => NEE,
"nef" => NEF,
"neg" => NEG,
"neh" => NEH,
"nei" => NEI,
"nej" => NEJ,
"nek" => NEK,
"nem" => NEM,
"nen" => NEN,
"neo" => NEO,
"neq" => NEQ,
"ner" => NER,
"nes" => NES,
"net" => NET,
"neu" => NEU,
"nev" => NEV,
"nex" => NEX,
"ney" => NEY,
"nez" => NEZ,
"nfa" => NFA,
"nfd" => NFD,
"nfl" => NFL,
"nfr" => NFR,
"nfu" => NFU,
"nga" => NGA,
"ngb" => NGB,
"ngc" => NGC,
"ngd" => NGD,
"nge" => NGE,
"ngg" => NGG,
"ngh" => NGH,
"ngi" => NGI,
"ngj" => NGJ,
"ngk" => NGK,
"ngl" => NGL,
"ngm" => NGM,
"ngn" => NGN,
"ngp" => NGP,
"ngq" => NGQ,
"ngr" => NGR,
"ngs" => NGS,
"ngt" => NGT,
"ngu" => NGU,
"ngv" => NGV,
"ngw" => NGW,
"ngx" => NGX,
"ngy" => NGY,
"ngz" => NGZ,
"nha" => NHA,
"nhb" => NHB,
"nhc" => NHC,
"nhd" => NHD,
"nhe" => NHE,
"nhf" => NHF,
"nhg" => NHG,
"nhh" => NHH,
"nhi" => NHI,
"nhk" => NHK,
"nhm" => NHM,
"nhn" => NHN,
"nho" => NHO,
"nhp" => NHP,
"nhq" => NHQ,
"nhr" => NHR,
"nht" => NHT,
"nhu" => NHU,
"nhv" => NHV,
"nhw" => NHW,
"nhx" => NHX,
"nhy" => NHY,
"nhz" => NHZ,
"nib" => NIB,
"nid" => NID,
"nie" => NIE,
"nif" => NIF,
"nig" => NIG,
"nih" => NIH,
"nii" => NII,
"nij" => NIJ,
"nik" => NIK,
"nil" => NIL,
"nim" => NIM,
"nin" => NIN,
"nio" => NIO,
"niq" => NIQ,
"nir" => NIR,
"nis" => NIS,
"nit" => NIT,
"niv" => NIV,
"niw" => NIW,
"nix" => NIX,
"niy" => NIY,
"niz" => NIZ,
"nja" => NJA,
"njb" => NJB,
"njd" => NJD,
"njh" => NJH,
"nji" => NJI,
"njj" => NJJ,
"njl" => NJL,
"njm" => NJM,
"njn" => NJN,
"njo" => NJO,
"njr" => NJR,
"njs" => NJS,
"njt" => NJT,
"nju" => NJU,
"njx" => NJX,
"njy" => NJY,
"njz" => NJZ,
"nka" => NKA,
"nkb" => NKB,
"nkc" => NKC,
"nkd" => NKD,
"nke" => NKE,
"nkf" => NKF,
"nkg" => NKG,
"nkh" => NKH,
"nki" => NKI,
"nkj" => NKJ,
"nkk" => NKK,
"nkm" => NKM,
"nkn" => NKN,
"nko" => NKO,
"nkp" => NKP,
"nkq" => NKQ,
"nkr" => NKR,
"nks" => NKS,
"nkt" => NKT,
"nku" => NKU,
"nkv" => NKV,
"nkw" => NKW,
"nkx" => NKX,
"nkz" => NKZ,
"nla" => NLA,
"nlc" => NLC,
"nle" => NLE,
"nlg" => NLG,
"nli" => NLI,
"nlj" => NLJ,
"nlk" => NLK,
"nll" => NLL,
"nlm" => NLM,
"nlo" => NLO,
"nlq" => NLQ,
"nlu" => NLU,
"nlv" => NLV,
"nlw" => NLW,
"nlx" => NLX,
"nly" => NLY,
"nlz" => NLZ,
"nma" => NMA,
"nmb" => NMB,
"nmc" => NMC,
"nmd" => NMD,
"nme" => NME,
"nmf" => NMF,
"nmg" => NMG,
"nmh" => NMH,
"nmi" => NMI,
"nmj" => NMJ,
"nmk" => NMK,
"nml" => NML,
"nmm" => NMM,
"nmn" => NMN,
"nmo" => NMO,
"nmp" => NMP,
"nmq" => NMQ,
"nmr" => NMR,
"nms" => NMS,
"nmt" => NMT,
"nmu" => NMU,
"nmv" => NMV,
"nmw" => NMW,
"nmx" => NMX,
"nmy" => NMY,
"nmz" => NMZ,
"nna" => NNA,
"nnb" => NNB,
"nnc" => NNC,
"nnd" => NND,
"nne" => NNE,
"nnf" => NNF,
"nng" => NNG,
"nnh" => NNH,
"nni" => NNI,
"nnj" => NNJ,
"nnk" => NNK,
"nnl" => NNL,
"nnm" => NNM,
"nnn" => NNN,
"nnp" => NNP,
"nnq" => NNQ,
"nnr" => NNR,
"nnt" => NNT,
"nnu" => NNU,
"nnv" => NNV,
"nnw" => NNW,
"nny" => NNY,
"nnz" => NNZ,
"noa" => NOA,
"noc" => NOC,
"nod" => NOD,
"noe" => NOE,
"nof" => NOF,
"noh" => NOH,
"noi" => NOI,
"noj" => NOJ,
"nok" => NOK,
"nol" => NOL,
"nom" => NOM,
"nop" => NOP,
"noq" => NOQ,
"nos" => NOS,
"not" => NOT,
"nou" => NOU,
"nov" => NOV,
"now" => NOW,
"noy" => NOY,
"noz" => NOZ,
"npa" => NPA,
"npb" => NPB,
"npg" => NPG,
"nph" => NPH,
"npi" => NPI,
"npl" => NPL,
"npn" => NPN,
"npo" => NPO,
"nps" => NPS,
"npu" => NPU,
"npx" => NPX,
"npy" => NPY,
"nqg" => NQG,
"nqk" => NQK,
"nql" => NQL,
"nqm" => NQM,
"nqn" => NQN,
"nqq" => NQQ,
"nqt" => NQT,
"nqy" => NQY,
"nra" => NRA,
"nrb" => NRB,
"nrc" => NRC,
"nre" => NRE,
"nrf" => NRF,
"nrg" => NRG,
"nri" => NRI,
"nrk" => NRK,
"nrl" => NRL,
"nrm" => NRM,
"nrn" => NRN,
"nrp" => NRP,
"nrr" => NRR,
"nrt" => NRT,
"nru" => NRU,
"nrx" => NRX,
"nrz" => NRZ,
"nsa" => NSA,
"nsb" => NSB,
"nsc" => NSC,
"nsd" => NSD,
"nse" => NSE,
"nsf" => NSF,
"nsg" => NSG,
"nsh" => NSH,
"nsi" => NSI,
"nsk" => NSK,
"nsl" => NSL,
"nsm" => NSM,
"nsn" => NSN,
"nsp" => NSP,
"nsq" => NSQ,
"nsr" => NSR,
"nss" => NSS,
"nst" => NST,
"nsu" => NSU,
"nsv" => NSV,
"nsw" => NSW,
"nsx" => NSX,
"nsy" => NSY,
"nsz" => NSZ,
"ntd" => NTD,
"nte" => NTE,
"ntg" => NTG,
"nti" => NTI,
"ntj" => NTJ,
"ntk" => NTK,
"ntm" => NTM,
"nto" => NTO,
"ntp" => NTP,
"ntr" => NTR,
"ntu" => NTU,
"ntw" => NTW,
"ntx" => NTX,
"nty" => NTY,
"ntz" => NTZ,
"nua" => NUA,
"nuc" => NUC,
"nud" => NUD,
"nue" => NUE,
"nuf" => NUF,
"nug" => NUG,
"nuh" => NUH,
"nui" => NUI,
"nuj" => NUJ,
"nuk" => NUK,
"nul" => NUL,
"num" => NUM,
"nun" => NUN,
"nuo" => NUO,
"nup" => NUP,
"nuq" => NUQ,
"nur" => NUR,
"nus" => NUS,
"nut" => NUT,
"nuu" => NUU,
"nuv" => NUV,
"nuw" => NUW,
"nux" => NUX,
"nuy" => NUY,
"nuz" => NUZ,
"nvh" => NVH,
"nvm" => NVM,
"nvo" => NVO,
"nwa" => NWA,
"nwb" => NWB,
"nwe" => NWE,
"nwg" => NWG,
"nwi" => NWI,
"nwm" => NWM,
"nwo" => NWO,
"nwr" => NWR,
"nww" => NWW,
"nwx" => NWX,
"nwy" => NWY,
"nxa" => NXA,
"nxd" => NXD,
"nxe" => NXE,
"nxg" => NXG,
"nxi" => NXI,
"nxk" => NXK,
"nxl" => NXL,
"nxm" => NXM,
"nxn" => NXN,
"nxo" => NXO,
"nxq" => NXQ,
"nxr" => NXR,
"nxx" => NXX,
"nyb" => NYB,
"nyc" => NYC,
"nyd" => NYD,
"nye" => NYE,
"nyf" => NYF,
"nyg" => NYG,
"nyh" => NYH,
"nyi" => NYI,
"nyj" => NYJ,
"nyk" => NYK,
"nyl" => NYL,
"nyp" => NYP,
"nyq" => NYQ,
"nyr" => NYR,
"nys" => NYS,
"nyt" => NYT,
"nyu" => NYU,
"nyv" => NYV,
"nyw" => NYW,
"nyx" => NYX,
"nyy" => NYY,
"nza" => NZA,
"nzb" => NZB,
"nzd" => NZD,
"nzk" => NZK,
"nzm" => NZM,
"nzs" => NZS,
"nzu" => NZU,
"nzy" => NZY,
"nzz" => NZZ,
"oaa" => OAA,
"oac" => OAC,
"oar" => OAR,
"oav" => OAV,
"obi" => OBI,
"obk" => OBK,
"obl" => OBL,
"obm" => OBM,
"obo" => OBO,
"obr" => OBR,
"obt" => OBT,
"obu" => OBU,
"oca" => OCA,
"och" => OCH,
"ocm" => OCM,
"oco" => OCO,
"ocu" => OCU,
"oda" => ODA,
"odk" => ODK,
"odt" => ODT,
"odu" => ODU,
"ofo" => OFO,
"ofs" => OFS,
"ofu" => OFU,
"ogb" => OGB,
"ogc" => OGC,
"oge" => OGE,
"ogg" => OGG,
"ogo" => OGO,
"ogu" => OGU,
"oht" => OHT,
"ohu" => OHU,
"oia" => OIA,
"oie" => OIE,
"oin" => OIN,
"ojb" => OJB,
"ojc" => OJC,
"ojg" => OJG,
"ojp" => OJP,
"ojs" => OJS,
"ojv" => OJV,
"ojw" => OJW,
"oka" => OKA,
"okb" => OKB,
"okc" => OKC,
"okd" => OKD,
"oke" => OKE,
"okg" => OKG,
"okh" => OKH,
"oki" => OKI,
"okj" => OKJ,
"okk" => OKK,
"okl" => OKL,
"okm" => OKM,
"okn" => OKN,
"oko" => OKO,
"okr" => OKR,
"oks" => OKS,
"oku" => OKU,
"okv" => OKV,
"okx" => OKX,
"okz" => OKZ,
"ola" => OLA,
"old" => OLD,
"ole" => OLE,
"olk" => OLK,
"olm" => OLM,
"olo" => OLO,
"olr" => OLR,
"olt" => OLT,
"olu" => OLU,
"oma" => OMA,
"omb" => OMB,
"omc" => OMC,
"omg" => OMG,
"omi" => OMI,
"omk" => OMK,
"oml" => OML,
"omn" => OMN,
"omo" => OMO,
"omp" => OMP,
"omr" => OMR,
"omt" => OMT,
"omu" => OMU,
"omw" => OMW,
"omx" => OMX,
"omy" => OMY,
"ona" => ONA,
"onb" => ONB,
"one" => ONE,
"ong" => ONG,
"oni" => ONI,
"onj" => ONJ,
"onk" => ONK,
"onn" => ONN,
"ono" => ONO,
"onp" => ONP,
"onr" => ONR,
"ons" => ONS,
"ont" => ONT,
"onu" => ONU,
"onw" => ONW,
"onx" => ONX,
"ood" => OOD,
"oog" => OOG,
"oon" => OON,
"oor" => OOR,
"oos" => OOS,
"opa" => OPA,
"opk" => OPK,
"opm" => OPM,
"opo" => OPO,
"opt" => OPT,
"opy" => OPY,
"ora" => ORA,
"orc" => ORC,
"ore" => ORE,
"org" => ORG,
"orh" => ORH,
"orn" => ORN,
"oro" => ORO,
"orr" => ORR,
"ors" => ORS,
"ort" => ORT,
"oru" => ORU,
"orv" => ORV,
"orw" => ORW,
"orx" => ORX,
"ory" => ORY,
"orz" => ORZ,
"osc" => OSC,
"osi" => OSI,
"osn" => OSN,
"oso" => OSO,
"osp" => OSP,
"ost" => OST,
"osu" => OSU,
"osx" => OSX,
"otb" => OTB,
"otd" => OTD,
"ote" => OTE,
"oti" => OTI,
"otk" => OTK,
"otl" => OTL,
"otm" => OTM,
"otn" => OTN,
"otq" => OTQ,
"otr" => OTR,
"ots" => OTS,
"ott" => OTT,
"otu" => OTU,
"otw" => OTW,
"otx" => OTX,
"oty" => OTY,
"otz" => OTZ,
"oua" => OUA,
"oub" => OUB,
"oue" => OUE,
"oui" => OUI,
"oum" => OUM,
"ovd" => OVD,
"owi" => OWI,
"owl" => OWL,
"oyb" => OYB,
"oyd" => OYD,
"oym" => OYM,
"oyy" => OYY,
"ozm" => OZM,
"pab" => PAB,
"pac" => PAC,
"pad" => PAD,
"pae" => PAE,
"paf" => PAF,
"pah" => PAH,
"pai" => PAI,
"pak" => PAK,
"pao" => PAO,
"paq" => PAQ,
"par" => PAR,
"pas" => PAS,
"pav" => PAV,
"paw" => PAW,
"pax" => PAX,
"pay" => PAY,
"paz" => PAZ,
"pbb" => PBB,
"pbc" => PBC,
"pbe" => PBE,
"pbf" => PBF,
"pbg" => PBG,
"pbh" => PBH,
"pbi" => PBI,
"pbl" => PBL,
"pbm" => PBM,
"pbn" => PBN,
"pbo" => PBO,
"pbp" => PBP,
"pbr" => PBR,
"pbs" => PBS,
"pbt" => PBT,
"pbu" => PBU,
"pbv" => PBV,
"pby" => PBY,
"pca" => PCA,
"pcb" => PCB,
"pcc" => PCC,
"pcd" => PCD,
"pce" => PCE,
"pcf" => PCF,
"pcg" => PCG,
"pch" => PCH,
"pci" => PCI,
"pcj" => PCJ,
"pck" => PCK,
"pcl" => PCL,
"pcm" => PCM,
"pcn" => PCN,
"pcp" => PCP,
"pcw" => PCW,
"pda" => PDA,
"pdc" => PDC,
"pdi" => PDI,
"pdn" => PDN,
"pdo" => PDO,
"pdt" => PDT,
"pdu" => PDU,
"pea" => PEA,
"peb" => PEB,
"ped" => PED,
"pee" => PEE,
"pef" => PEF,
"peg" => PEG,
"peh" => PEH,
"pei" => PEI,
"pej" => PEJ,
"pek" => PEK,
"pel" => PEL,
"pem" => PEM,
"pep" => PEP,
"peq" => PEQ,
"pes" => PES,
"pev" => PEV,
"pex" => PEX,
"pey" => PEY,
"pez" => PEZ,
"pfa" => PFA,
"pfe" => PFE,
"pfl" => PFL,
"pga" => PGA,
"pgd" => PGD,
"pgg" => PGG,
"pgi" => PGI,
"pgk" => PGK,
"pgl" => PGL,
"pgn" => PGN,
"pgs" => PGS,
"pgu" => PGU,
"pgz" => PGZ,
"pha" => PHA,
"phd" => PHD,
"phg" => PHG,
"phh" => PHH,
"phj" => PHJ,
"phk" => PHK,
"phl" => PHL,
"phm" => PHM,
"pho" => PHO,
"phq" => PHQ,
"phr" => PHR,
"pht" => PHT,
"phu" => PHU,
"phv" => PHV,
"phw" => PHW,
"pia" => PIA,
"pib" => PIB,
"pic" => PIC,
"pid" => PID,
"pie" => PIE,
"pif" => PIF,
"pig" => PIG,
"pih" => PIH,
"pij" => PIJ,
"pil" => PIL,
"pim" => PIM,
"pin" => PIN,
"pio" => PIO,
"pip" => PIP,
"pir" => PIR,
"pis" => PIS,
"pit" => PIT,
"piu" => PIU,
"piv" => PIV,
"piw" => PIW,
"pix" => PIX,
"piy" => PIY,
"piz" => PIZ,
"pjt" => PJT,
"pka" => PKA,
"pkb" => PKB,
"pkc" => PKC,
"pkg" => PKG,
"pkh" => PKH,
"pkn" => PKN,
"pko" => PKO,
"pkp" => PKP,
"pkr" => PKR,
"pks" => PKS,
"pkt" => PKT,
"pku" => PKU,
"pla" => PLA,
"plb" => PLB,
"plc" => PLC,
"pld" => PLD,
"ple" => PLE,
"plg" => PLG,
"plh" => PLH,
"plj" => PLJ,
"plk" => PLK,
"pll" => PLL,
"pln" => PLN,
"plo" => PLO,
"plq" => PLQ,
"plr" => PLR,
"pls" => PLS,
"plt" => PLT,
"plu" => PLU,
"plv" => PLV,
"plw" => PLW,
"ply" => PLY,
"plz" => PLZ,
"pma" => PMA,
"pmb" => PMB,
"pmd" => PMD,
"pme" => PME,
"pmf" => PMF,
"pmh" => PMH,
"pmi" => PMI,
"pmj" => PMJ,
"pmk" => PMK,
"pml" => PML,
"pmm" => PMM,
"pmn" => PMN,
"pmo" => PMO,
"pmq" => PMQ,
"pmr" => PMR,
"pms" => PMS,
"pmt" => PMT,
"pmw" => PMW,
"pmx" => PMX,
"pmy" => PMY,
"pmz" => PMZ,
"pna" => PNA,
"pnb" => PNB,
"pnc" => PNC,
"pnd" => PND,
"pne" => PNE,
"png" => PNG,
"pnh" => PNH,
"pni" => PNI,
"pnj" => PNJ,
"pnk" => PNK,
"pnl" => PNL,
"pnm" => PNM,
"pnn" => PNN,
"pno" => PNO,
"pnp" => PNP,
"pnq" => PNQ,
"pnr" => PNR,
"pns" => PNS,
"pnt" => PNT,
"pnu" => PNU,
"pnv" => PNV,
"pnw" => PNW,
"pnx" => PNX,
"pny" => PNY,
"pnz" => PNZ,
"poc" => POC,
"poe" => POE,
"pof" => POF,
"pog" => POG,
"poh" => POH,
"poi" => POI,
"pok" => POK,
"pom" => POM,
"poo" => POO,
"pop" => POP,
"poq" => POQ,
"pos" => POS,
"pot" => POT,
"pov" => POV,
"pow" => POW,
"pox" => POX,
"poy" => POY,
"ppe" => PPE,
"ppi" => PPI,
"ppk" => PPK,
"ppl" => PPL,
"ppm" => PPM,
"ppn" => PPN,
"ppo" => PPO,
"ppp" => PPP,
"ppq" => PPQ,
"pps" => PPS,
"ppt" => PPT,
"ppu" => PPU,
"pqa" => PQA,
"pqm" => PQM,
"prc" => PRC,
"prd" => PRD,
"pre" => PRE,
"prf" => PRF,
"prg" => PRG,
"prh" => PRH,
"pri" => PRI,
"prk" => PRK,
"prl" => PRL,
"prm" => PRM,
"prn" => PRN,
"prp" => PRP,
"prq" => PRQ,
"prr" => PRR,
"prs" => PRS,
"prt" => PRT,
"pru" => PRU,
"prw" => PRW,
"prx" => PRX,
"prz" => PRZ,
"psa" => PSA,
"psc" => PSC,
"psd" => PSD,
"pse" => PSE,
"psg" => PSG,
"psh" => PSH,
"psi" => PSI,
"psl" => PSL,
"psm" => PSM,
"psn" => PSN,
"pso" => PSO,
"psp" => PSP,
"psq" => PSQ,
"psr" => PSR,
"pss" => PSS,
"pst" => PST,
"psu" => PSU,
"psw" => PSW,
"psy" => PSY,
"pta" => PTA,
"pth" => PTH,
"pti" => PTI,
"ptn" => PTN,
"pto" => PTO,
"ptp" => PTP,
"ptq" => PTQ,
"ptr" => PTR,
"ptt" => PTT,
"ptu" => PTU,
"ptv" => PTV,
"ptw" => PTW,
"pty" => PTY,
"pua" => PUA,
"pub" => PUB,
"puc" => PUC,
"pud" => PUD,
"pue" => PUE,
"puf" => PUF,
"pug" => PUG,
"pui" => PUI,
"puj" => PUJ,
"pum" => PUM,
"puo" => PUO,
"pup" => PUP,
"puq" => PUQ,
"pur" => PUR,
"put" => PUT,
"puu" => PUU,
"puw" => PUW,
"pux" => PUX,
"puy" => PUY,
"pwa" => PWA,
"pwb" => PWB,
"pwg" => PWG,
"pwi" => PWI,
"pwm" => PWM,
"pwn" => PWN,
"pwo" => PWO,
"pwr" => PWR,
"pww" => PWW,
"pxm" => PXM,
"pye" => PYE,
"pym" => PYM,
"pyn" => PYN,
"pys" => PYS,
"pyu" => PYU,
"pyx" => PYX,
"pyy" => PYY,
"pzh" => PZH,
"pzn" => PZN,
"qua" => QUA,
"qub" => QUB,
"quc" => QUC,
"qud" => QUD,
"quf" => QUF,
"qug" => QUG,
"quh" => QUH,
"qui" => QUI,
"quk" => QUK,
"qul" => QUL,
"qum" => QUM,
"qun" => QUN,
"qup" => QUP,
"quq" => QUQ,
"qur" => QUR,
"qus" => QUS,
"quv" => QUV,
"quw" => QUW,
"qux" => QUX,
"quy" => QUY,
"quz" => QUZ,
"qva" => QVA,
"qvc" => QVC,
"qve" => QVE,
"qvh" => QVH,
"qvi" => QVI,
"qvj" => QVJ,
"qvl" => QVL,
"qvm" => QVM,
"qvn" => QVN,
"qvo" => QVO,
"qvp" => QVP,
"qvs" => QVS,
"qvw" => QVW,
"qvy" => QVY,
"qvz" => QVZ,
"qwa" => QWA,
"qwc" => QWC,
"qwh" => QWH,
"qwm" => QWM,
"qws" => QWS,
"qwt" => QWT,
"qxa" => QXA,
"qxc" => QXC,
"qxh" => QXH,
"qxl" => QXL,
"qxn" => QXN,
"qxo" => QXO,
"qxp" => QXP,
"qxq" => QXQ,
"qxr" => QXR,
"qxs" => QXS,
"qxt" => QXT,
"qxu" => QXU,
"qxw" => QXW,
"qya" => QYA,
"qyp" => QYP,
"raa" => RAA,
"rab" => RAB,
"rac" => RAC,
"rad" => RAD,
"raf" => RAF,
"rag" => RAG,
"rah" => RAH,
"rai" => RAI,
"rak" => RAK,
"ral" => RAL,
"ram" => RAM,
"ran" => RAN,
"rao" => RAO,
"raq" => RAQ,
"ras" => RAS,
"rat" => RAT,
"rau" => RAU,
"rav" => RAV,
"raw" => RAW,
"rax" => RAX,
"ray" => RAY,
"raz" => RAZ,
"rbb" => RBB,
"rbk" => RBK,
"rbl" => RBL,
"rbp" => RBP,
"rcf" => RCF,
"rdb" => RDB,
"rea" => REA,
"reb" => REB,
"ree" => REE,
"reg" => REG,
"rei" => REI,
"rej" => REJ,
"rel" => REL,
"rem" => REM,
"ren" => REN,
"rer" => RER,
"res" => RES,
"ret" => RET,
"rey" => REY,
"rga" => RGA,
"rge" => RGE,
"rgk" => RGK,
"rgn" => RGN,
"rgr" => RGR,
"rgs" => RGS,
"rgu" => RGU,
"rhg" => RHG,
"rhp" => RHP,
"ria" => RIA,
"rib" => RIB,
"rif" => RIF,
"ril" => RIL,
"rim" => RIM,
"rin" => RIN,
"rir" => RIR,
"rit" => RIT,
"riu" => RIU,
"rjg" => RJG,
"rji" => RJI,
"rjs" => RJS,
"rka" => RKA,
"rkb" => RKB,
"rkh" => RKH,
"rki" => RKI,
"rkm" => RKM,
"rkt" => RKT,
"rkw" => RKW,
"rma" => RMA,
"rmb" => RMB,
"rmc" => RMC,
"rmd" => RMD,
"rme" => RME,
"rmf" => RMF,
"rmg" => RMG,
"rmh" => RMH,
"rmi" => RMI,
"rmk" => RMK,
"rml" => RML,
"rmm" => RMM,
"rmn" => RMN,
"rmo" => RMO,
"rmp" => RMP,
"rmq" => RMQ,
"rms" => RMS,
"rmt" => RMT,
"rmu" => RMU,
"rmv" => RMV,
"rmw" => RMW,
"rmx" => RMX,
"rmy" => RMY,
"rmz" => RMZ,
"rnb" => RNB,
"rnd" => RND,
"rng" => RNG,
"rnl" => RNL,
"rnn" => RNN,
"rnp" => RNP,
"rnr" => RNR,
"rnw" => RNW,
"rob" => ROB,
"roc" => ROC,
"rod" => ROD,
"roe" => ROE,
"rof" => ROF,
"rog" => ROG,
"rol" => ROL,
"roo" => ROO,
"rop" => ROP,
"ror" => ROR,
"rou" => ROU,
"row" => ROW,
"rpn" => RPN,
"rpt" => RPT,
"rri" => RRI,
"rro" => RRO,
"rrt" => RRT,
"rsb" => RSB,
"rsk" => RSK,
"rsl" => RSL,
"rsm" => RSM,
"rsn" => RSN,
"rtc" => RTC,
"rth" => RTH,
"rtm" => RTM,
"rts" => RTS,
"rtw" => RTW,
"rub" => RUB,
"ruc" => RUC,
"rue" => RUE,
"ruf" => RUF,
"rug" => RUG,
"ruh" => RUH,
"rui" => RUI,
"ruk" => RUK,
"ruo" => RUO,
"ruq" => RUQ,
"rut" => RUT,
"ruu" => RUU,
"ruy" => RUY,
"ruz" => RUZ,
"rwa" => RWA,
"rwk" => RWK,
"rwl" => RWL,
"rwm" => RWM,
"rwo" => RWO,
"rwr" => RWR,
"rxd" => RXD,
"rxw" => RXW,
"ryn" => RYN,
"rys" => RYS,
"ryu" => RYU,
"rzh" => RZH,
"saa" => SAA,
"sab" => SAB,
"sac" => SAC,
"sae" => SAE,
"saf" => SAF,
"saj" => SAJ,
"sak" => SAK,
"sao" => SAO,
"saq" => SAQ,
"sar" => SAR,
"sau" => SAU,
"sav" => SAV,
"saw" => SAW,
"sax" => SAX,
"say" => SAY,
"saz" => SAZ,
"sba" => SBA,
"sbb" => SBB,
"sbc" => SBC,
"sbd" => SBD,
"sbe" => SBE,
"sbf" => SBF,
"sbg" => SBG,
"sbh" => SBH,
"sbi" => SBI,
"sbj" => SBJ,
"sbk" => SBK,
"sbl" => SBL,
"sbm" => SBM,
"sbn" => SBN,
"sbo" => SBO,
"sbp" => SBP,
"sbq" => SBQ,
"sbr" => SBR,
"sbs" => SBS,
"sbt" => SBT,
"sbu" => SBU,
"sbv" => SBV,
"sbw" => SBW,
"sbx" => SBX,
"sby" => SBY,
"sbz" => SBZ,
"scb" => SCB,
"sce" => SCE,
"scf" => SCF,
"scg" => SCG,
"sch" => SCH,
"sci" => SCI,
"sck" => SCK,
"scl" => SCL,
"scp" => SCP,
"scq" => SCQ,
"scs" => SCS,
"sct" => SCT,
"scu" => SCU,
"scv" => SCV,
"scw" => SCW,
"scx" => SCX,
"sda" => SDA,
"sdb" => SDB,
"sdc" => SDC,
"sde" => SDE,
"sdf" => SDF,
"sdg" => SDG,
"sdh" => SDH,
"sdj" => SDJ,
"sdk" => SDK,
"sdl" => SDL,
"sdn" => SDN,
"sdo" => SDO,
"sdp" => SDP,
"sdq" => SDQ,
"sdr" => SDR,
"sds" => SDS,
"sdt" => SDT,
"sdu" => SDU,
"sdx" => SDX,
"sdz" => SDZ,
"sea" => SEA,
"seb" => SEB,
"sec" => SEC,
"sed" => SED,
"see" => SEE,
"sef" => SEF,
"seg" => SEG,
"seh" => SEH,
"sei" => SEI,
"sej" => SEJ,
"sek" => SEK,
"sen" => SEN,
"seo" => SEO,
"sep" => SEP,
"seq" => SEQ,
"ser" => SER,
"ses" => SES,
"set" => SET,
"seu" => SEU,
"sev" => SEV,
"sew" => SEW,
"sey" => SEY,
"sez" => SEZ,
"sfb" => SFB,
"sfe" => SFE,
"sfm" => SFM,
"sfs" => SFS,
"sfw" => SFW,
"sgb" => SGB,
"sgc" => SGC,
"sgd" => SGD,
"sge" => SGE,
"sgg" => SGG,
"sgh" => SGH,
"sgi" => SGI,
"sgj" => SGJ,
"sgk" => SGK,
"sgm" => SGM,
"sgp" => SGP,
"sgr" => SGR,
"sgs" => SGS,
"sgt" => SGT,
"sgu" => SGU,
"sgw" => SGW,
"sgx" => SGX,
"sgy" => SGY,
"sgz" => SGZ,
"sha" => SHA,
"shb" => SHB,
"shc" => SHC,
"shd" => SHD,
"she" => SHE,
"shg" => SHG,
"shh" => SHH,
"shi" => SHI,
"shj" => SHJ,
"shk" => SHK,
"shl" => SHL,
"shm" => SHM,
"sho" => SHO,
"shp" => SHP,
"shq" => SHQ,
"shr" => SHR,
"shs" => SHS,
"sht" => SHT,
"shu" => SHU,
"shv" => SHV,
"shw" => SHW,
"shx" => SHX,
"shy" => SHY,
"shz" => SHZ,
"sia" => SIA,
"sib" => SIB,
"sie" => SIE,
"sif" => SIF,
"sig" => SIG,
"sih" => SIH,
"sii" => SII,
"sij" => SIJ,
"sik" => SIK,
"sil" => SIL,
"sim" => SIM,
"sip" => SIP,
"siq" => SIQ,
"sir" => SIR,
"sis" => SIS,
"siu" => SIU,
"siv" => SIV,
"siw" => SIW,
"six" => SIX,
"siy" => SIY,
"siz" => SIZ,
"sja" => SJA,
"sjb" => SJB,
"sjd" => SJD,
"sje" => SJE,
"sjg" => SJG,
"sjk" => SJK,
"sjl" => SJL,
"sjm" => SJM,
"sjn" => SJN,
"sjo" => SJO,
"sjp" => SJP,
"sjr" => SJR,
"sjs" => SJS,
"sjt" => SJT,
"sju" => SJU,
"sjw" => SJW,
"ska" => SKA,
"skb" => SKB,
"skc" => SKC,
"skd" => SKD,
"ske" => SKE,
"skf" => SKF,
"skg" => SKG,
"skh" => SKH,
"ski" => SKI,
"skj" => SKJ,
"skm" => SKM,
"skn" => SKN,
"sko" => SKO,
"skp" => SKP,
"skq" => SKQ,
"skr" => SKR,
"sks" => SKS,
"skt" => SKT,
"sku" => SKU,
"skv" => SKV,
"skw" => SKW,
"skx" => SKX,
"sky" => SKY,
"skz" => SKZ,
"slc" => SLC,
"sld" => SLD,
"sle" => SLE,
"slf" => SLF,
"slg" => SLG,
"slh" => SLH,
"sli" => SLI,
"slj" => SLJ,
"sll" => SLL,
"slm" => SLM,
"sln" => SLN,
"slp" => SLP,
"slq" => SLQ,
"slr" => SLR,
"sls" => SLS,
"slt" => SLT,
"slu" => SLU,
"slw" => SLW,
"slx" => SLX,
"sly" => SLY,
"slz" => SLZ,
"smb" => SMB,
"smc" => SMC,
"smf" => SMF,
"smg" => SMG,
"smh" => SMH,
"smk" => SMK,
"sml" => SML,
"smm" => SMM,
"smp" => SMP,
"smq" => SMQ,
"smr" => SMR,
"smt" => SMT,
"smu" => SMU,
"smv" => SMV,
"smw" => SMW,
"smx" => SMX,
"smy" => SMY,
"smz" => SMZ,
"snc" => SNC,
"sne" => SNE,
"snf" => SNF,
"sng" => SNG,
"sni" => SNI,
"snj" => SNJ,
"snl" => SNL,
"snm" => SNM,
"snn" => SNN,
"sno" => SNO,
"snp" => SNP,
"snq" => SNQ,
"snr" => SNR,
"sns" => SNS,
"snu" => SNU,
"snv" => SNV,
"snw" => SNW,
"snx" => SNX,
"sny" => SNY,
"snz" => SNZ,
"soa" => SOA,
"sob" => SOB,
"soc" => SOC,
"sod" => SOD,
"soe" => SOE,
"soh" => SOH,
"soi" => SOI,
"soj" => SOJ,
"sok" => SOK,
"sol" => SOL,
"soo" => SOO,
"sop" => SOP,
"soq" => SOQ,
"sor" => SOR,
"sos" => SOS,
"sou" => SOU,
"sov" => SOV,
"sow" => SOW,
"sox" => SOX,
"soy" => SOY,
"soz" => SOZ,
"spb" => SPB,
"spc" => SPC,
"spd" => SPD,
"spe" => SPE,
"spg" => SPG,
"spi" => SPI,
"spk" => SPK,
"spl" => SPL,
"spm" => SPM,
"spn" => SPN,
"spo" => SPO,
"spp" => SPP,
"spq" => SPQ,
"spr" => SPR,
"sps" => SPS,
"spt" => SPT,
"spu" => SPU,
"spv" => SPV,
"spx" => SPX,
"spy" => SPY,
"sqa" => SQA,
"sqh" => SQH,
"sqk" => SQK,
"sqm" => SQM,
"sqn" => SQN,
"sqo" => SQO,
"sqq" => SQQ,
"sqr" => SQR,
"sqs" => SQS,
"sqt" => SQT,
"squ" => SQU,
"sqx" => SQX,
"sra" => SRA,
"srb" => SRB,
"src" => SRC,
"sre" => SRE,
"srf" => SRF,
"srg" => SRG,
"srh" => SRH,
"sri" => SRI,
"srk" => SRK,
"srl" => SRL,
"srm" => SRM,
"sro" => SRO,
"srq" => SRQ,
"srs" => SRS,
"srt" => SRT,
"sru" => SRU,
"srv" => SRV,
"srw" => SRW,
"srx" => SRX,
"sry" => SRY,
"srz" => SRZ,
"ssb" => SSB,
"ssc" => SSC,
"ssd" => SSD,
"sse" => SSE,
"ssf" => SSF,
"ssg" => SSG,
"ssh" => SSH,
"ssi" => SSI,
"ssj" => SSJ,
"ssk" => SSK,
"ssl" => SSL,
"ssm" => SSM,
"ssn" => SSN,
"sso" => SSO,
"ssp" => SSP,
"ssq" => SSQ,
"ssr" => SSR,
"sss" => SSS,
"sst" => SST,
"ssu" => SSU,
"ssv" => SSV,
"ssx" => SSX,
"ssy" => SSY,
"ssz" => SSZ,
"sta" => STA,
"stb" => STB,
"std" => STD,
"ste" => STE,
"stf" => STF,
"stg" => STG,
"sth" => STH,
"sti" => STI,
"stj" => STJ,
"stk" => STK,
"stl" => STL,
"stm" => STM,
"stn" => STN,
"sto" => STO,
"stp" => STP,
"stq" => STQ,
"str" => STR,
"sts" => STS,
"stt" => STT,
"stu" => STU,
"stv" => STV,
"stw" => STW,
"sty" => STY,
"sua" => SUA,
"sub" => SUB,
"suc" => SUC,
"sue" => SUE,
"sug" => SUG,
"sui" => SUI,
"suj" => SUJ,
"suo" => SUO,
"suq" => SUQ,
"sur" => SUR,
"sut" => SUT,
"suv" => SUV,
"suw" => SUW,
"suy" => SUY,
"suz" => SUZ,
"sva" => SVA,
"svb" => SVB,
"svc" => SVC,
"sve" => SVE,
"svk" => SVK,
"svm" => SVM,
"svs" => SVS,
"svx" => SVX,
"swb" => SWB,
"swc" => SWC,
"swf" => SWF,
"swg" => SWG,
"swh" => SWH,
"swi" => SWI,
"swj" => SWJ,
"swk" => SWK,
"swl" => SWL,
"swm" => SWM,
"swn" => SWN,
"swo" => SWO,
"swp" => SWP,
"swq" => SWQ,
"swr" => SWR,
"sws" => SWS,
"swt" => SWT,
"swu" => SWU,
"swv" => SWV,
"sww" => SWW,
"swx" => SWX,
"swy" => SWY,
"sxb" => SXB,
"sxc" => SXC,
"sxe" => SXE,
"sxg" => SXG,
"sxk" => SXK,
"sxl" => SXL,
"sxm" => SXM,
"sxn" => SXN,
"sxo" => SXO,
"sxr" => SXR,
"sxs" => SXS,
"sxu" => SXU,
"sxw" => SXW,
"sya" => SYA,
"syb" => SYB,
"syi" => SYI,
"syk" => SYK,
"syl" => SYL,
"sym" => SYM,
"syn" => SYN,
"syo" => SYO,
"sys" => SYS,
"syw" => SYW,
"syx" => SYX,
"syy" => SYY,
"sza" => SZA,
"szb" => SZB,
"szc" => SZC,
"szd" => SZD,
"sze" => SZE,
"szg" => SZG,
"szl" => SZL,
"szn" => SZN,
"szp" => SZP,
"szs" => SZS,
"szv" => SZV,
"szw" => SZW,
"szy" => SZY,
"taa" => TAA,
"tab" => TAB,
"tac" => TAC,
"tad" => TAD,
"tae" => TAE,
"taf" => TAF,
"tag" => TAG,
"taj" => TAJ,
"tak" => TAK,
"tal" => TAL,
"tan" => TAN,
"tao" => TAO,
"tap" => TAP,
"taq" => TAQ,
"tar" => TAR,
"tas" => TAS,
"tau" => TAU,
"tav" => TAV,
"taw" => TAW,
"tax" => TAX,
"tay" => TAY,
"taz" => TAZ,
"tba" => TBA,
"tbc" => TBC,
"tbd" => TBD,
"tbe" => TBE,
"tbf" => TBF,
"tbg" => TBG,
"tbh" => TBH,
"tbi" => TBI,
"tbj" => TBJ,
"tbk" => TBK,
"tbl" => TBL,
"tbm" => TBM,
"tbn" => TBN,
"tbo" => TBO,
"tbp" => TBP,
"tbr" => TBR,
"tbs" => TBS,
"tbt" => TBT,
"tbu" => TBU,
"tbv" => TBV,
"tbw" => TBW,
"tbx" => TBX,
"tby" => TBY,
"tbz" => TBZ,
"tca" => TCA,
"tcb" => TCB,
"tcc" => TCC,
"tcd" => TCD,
"tce" => TCE,
"tcf" => TCF,
"tcg" => TCG,
"tch" => TCH,
"tci" => TCI,
"tck" => TCK,
"tcl" => TCL,
"tcm" => TCM,
"tcn" => TCN,
"tco" => TCO,
"tcp" => TCP,
"tcq" => TCQ,
"tcs" => TCS,
"tct" => TCT,
"tcu" => TCU,
"tcw" => TCW,
"tcx" => TCX,
"tcy" => TCY,
"tcz" => TCZ,
"tda" => TDA,
"tdb" => TDB,
"tdc" => TDC,
"tdd" => TDD,
"tde" => TDE,
"tdf" => TDF,
"tdg" => TDG,
"tdh" => TDH,
"tdi" => TDI,
"tdj" => TDJ,
"tdk" => TDK,
"tdl" => TDL,
"tdm" => TDM,
"tdn" => TDN,
"tdo" => TDO,
"tdq" => TDQ,
"tdr" => TDR,
"tds" => TDS,
"tdt" => TDT,
"tdv" => TDV,
"tdx" => TDX,
"tdy" => TDY,
"tea" => TEA,
"teb" => TEB,
"tec" => TEC,
"ted" => TED,
"tee" => TEE,
"tef" => TEF,
"teg" => TEG,
"teh" => TEH,
"tei" => TEI,
"tek" => TEK,
"ten" => TEN,
"teo" => TEO,
"tep" => TEP,
"teq" => TEQ,
"tes" => TES,
"teu" => TEU,
"tev" => TEV,
"tew" => TEW,
"tex" => TEX,
"tey" => TEY,
"tez" => TEZ,
"tfi" => TFI,
"tfn" => TFN,
"tfo" => TFO,
"tfr" => TFR,
"tft" => TFT,
"tga" => TGA,
"tgb" => TGB,
"tgc" => TGC,
"tgd" => TGD,
"tge" => TGE,
"tgf" => TGF,
"tgh" => TGH,
"tgi" => TGI,
"tgj" => TGJ,
"tgn" => TGN,
"tgo" => TGO,
"tgp" => TGP,
"tgq" => TGQ,
"tgr" => TGR,
"tgs" => TGS,
"tgt" => TGT,
"tgu" => TGU,
"tgv" => TGV,
"tgw" => TGW,
"tgx" => TGX,
"tgy" => TGY,
"tgz" => TGZ,
"thd" => THD,
"the" => THE,
"thf" => THF,
"thh" => THH,
"thi" => THI,
"thk" => THK,
"thl" => THL,
"thm" => THM,
"thn" => THN,
"thp" => THP,
"thq" => THQ,
"thr" => THR,
"ths" => THS,
"tht" => THT,
"thu" => THU,
"thv" => THV,
"thy" => THY,
"thz" => THZ,
"tia" => TIA,
"tic" => TIC,
"tif" => TIF,
"tih" => TIH,
"tii" => TII,
"tij" => TIJ,
"tik" => TIK,
"til" => TIL,
"tim" => TIM,
"tin" => TIN,
"tio" => TIO,
"tip" => TIP,
"tiq" => TIQ,
"tis" => TIS,
"tit" => TIT,
"tiu" => TIU,
"tiw" => TIW,
"tix" => TIX,
"tiy" => TIY,
"tiz" => TIZ,
"tja" => TJA,
"tjg" => TJG,
"tji" => TJI,
"tjj" => TJJ,
"tjl" => TJL,
"tjm" => TJM,
"tjn" => TJN,
"tjo" => TJO,
"tjp" => TJP,
"tjs" => TJS,
"tju" => TJU,
"tjw" => TJW,
"tka" => TKA,
"tkb" => TKB,
"tkd" => TKD,
"tke" => TKE,
"tkf" => TKF,
"tkg" => TKG,
"tkm" => TKM,
"tkn" => TKN,
"tkp" => TKP,
"tkq" => TKQ,
"tkr" => TKR,
"tks" => TKS,
"tkt" => TKT,
"tku" => TKU,
"tkv" => TKV,
"tkw" => TKW,
"tkx" => TKX,
"tkz" => TKZ,
"tla" => TLA,
"tlb" => TLB,
"tlc" => TLC,
"tld" => TLD,
"tlf" => TLF,
"tlg" => TLG,
"tlj" => TLJ,
"tlk" => TLK,
"tll" => TLL,
"tlm" => TLM,
"tln" => TLN,
"tlo" => TLO,
"tlp" => TLP,
"tlq" => TLQ,
"tlr" => TLR,
"tls" => TLS,
"tlt" => TLT,
"tlu" => TLU,
"tlv" => TLV,
"tlx" => TLX,
"tly" => TLY,
"tma" => TMA,
"tmb" => TMB,
"tmc" => TMC,
"tmd" => TMD,
"tme" => TME,
"tmf" => TMF,
"tmg" => TMG,
"tmi" => TMI,
"tmj" => TMJ,
"tmk" => TMK,
"tml" => TML,
"tmm" => TMM,
"tmn" => TMN,
"tmo" => TMO,
"tmq" => TMQ,
"tmr" => TMR,
"tms" => TMS,
"tmt" => TMT,
"tmu" => TMU,
"tmv" => TMV,
"tmw" => TMW,
"tmy" => TMY,
"tmz" => TMZ,
"tna" => TNA,
"tnb" => TNB,
"tnc" => TNC,
"tnd" => TND,
"tng" => TNG,
"tnh" => TNH,
"tni" => TNI,
"tnk" => TNK,
"tnl" => TNL,
"tnm" => TNM,
"tnn" => TNN,
"tno" => TNO,
"tnp" => TNP,
"tnq" => TNQ,
"tnr" => TNR,
"tns" => TNS,
"tnt" => TNT,
"tnu" => TNU,
"tnv" => TNV,
"tnw" => TNW,
"tnx" => TNX,
"tny" => TNY,
"tnz" => TNZ,
"tob" => TOB,
"toc" => TOC,
"tod" => TOD,
"tof" => TOF,
"toh" => TOH,
"toi" => TOI,
"toj" => TOJ,
"tok" => TOK,
"tol" => TOL,
"tom" => TOM,
"too" => TOO,
"top" => TOP,
"toq" => TOQ,
"tor" => TOR,
"tos" => TOS,
"tou" => TOU,
"tov" => TOV,
"tow" => TOW,
"tox" => TOX,
"toy" => TOY,
"toz" => TOZ,
"tpa" => TPA,
"tpc" => TPC,
"tpe" => TPE,
"tpf" => TPF,
"tpg" => TPG,
"tpj" => TPJ,
"tpk" => TPK,
"tpl" => TPL,
"tpm" => TPM,
"tpn" => TPN,
"tpo" => TPO,
"tpp" => TPP,
"tpq" => TPQ,
"tpr" => TPR,
"tpt" => TPT,
"tpu" => TPU,
"tpv" => TPV,
"tpw" => TPW,
"tpx" => TPX,
"tpy" => TPY,
"tpz" => TPZ,
"tqb" => TQB,
"tql" => TQL,
"tqm" => TQM,
"tqn" => TQN,
"tqo" => TQO,
"tqp" => TQP,
"tqq" => TQQ,
"tqr" => TQR,
"tqt" => TQT,
"tqu" => TQU,
"tqw" => TQW,
"tra" => TRA,
"trb" => TRB,
"trc" => TRC,
"trd" => TRD,
"tre" => TRE,
"trf" => TRF,
"trg" => TRG,
"trh" => TRH,
"tri" => TRI,
"trj" => TRJ,
"trl" => TRL,
"trm" => TRM,
"trn" => TRN,
"tro" => TRO,
"trp" => TRP,
"trq" => TRQ,
"trr" => TRR,
"trs" => TRS,
"trt" => TRT,
"tru" => TRU,
"trv" => TRV,
"trw" => TRW,
"trx" => TRX,
"try" => TRY,
"trz" => TRZ,
"tsa" => TSA,
"tsb" => TSB,
"tsc" => TSC,
"tsd" => TSD,
"tse" => TSE,
"tsg" => TSG,
"tsh" => TSH,
"tsj" => TSJ,
"tsk" => TSK,
"tsl" => TSL,
"tsm" => TSM,
"tsp" => TSP,
"tsq" => TSQ,
"tsr" => TSR,
"tss" => TSS,
"tst" => TST,
"tsu" => TSU,
"tsv" => TSV,
"tsw" => TSW,
"tsx" => TSX,
"tsy" => TSY,
"tsz" => TSZ,
"tta" => TTA,
"ttb" => TTB,
"ttc" => TTC,
"ttd" => TTD,
"tte" => TTE,
"ttf" => TTF,
"ttg" => TTG,
"tth" => TTH,
"tti" => TTI,
"ttj" => TTJ,
"ttk" => TTK,
"ttl" => TTL,
"ttm" => TTM,
"ttn" => TTN,
"tto" => TTO,
"ttp" => TTP,
"ttq" => TTQ,
"ttr" => TTR,
"tts" => TTS,
"ttt" => TTT,
"ttu" => TTU,
"ttv" => TTV,
"ttw" => TTW,
"tty" => TTY,
"ttz" => TTZ,
"tua" => TUA,
"tub" => TUB,
"tuc" => TUC,
"tud" => TUD,
"tue" => TUE,
"tuf" => TUF,
"tug" => TUG,
"tuh" => TUH,
"tui" => TUI,
"tuj" => TUJ,
"tul" => TUL,
"tun" => TUN,
"tuo" => TUO,
"tuq" => TUQ,
"tus" => TUS,
"tuu" => TUU,
"tuv" => TUV,
"tux" => TUX,
"tuy" => TUY,
"tuz" => TUZ,
"tva" => TVA,
"tvd" => TVD,
"tve" => TVE,
"tvk" => TVK,
"tvm" => TVM,
"tvn" => TVN,
"tvo" => TVO,
"tvs" => TVS,
"tvt" => TVT,
"tvu" => TVU,
"tvw" => TVW,
"tvx" => TVX,
"tvy" => TVY,
"twa" => TWA,
"twb" => TWB,
"twc" => TWC,
"twd" => TWD,
"twe" => TWE,
"twf" => TWF,
"twg" => TWG,
"twh" => TWH,
"twl" => TWL,
"twm" => TWM,
"twn" => TWN,
"two" => TWO,
"twp" => TWP,
"twq" => TWQ,
"twr" => TWR,
"twt" => TWT,
"twu" => TWU,
"tww" => TWW,
"twx" => TWX,
"twy" => TWY,
"txa" => TXA,
"txb" => TXB,
"txc" => TXC,
"txe" => TXE,
"txg" => TXG,
"txh" => TXH,
"txi" => TXI,
"txj" => TXJ,
"txm" => TXM,
"txn" => TXN,
"txo" => TXO,
"txq" => TXQ,
"txr" => TXR,
"txs" => TXS,
"txt" => TXT,
"txu" => TXU,
"txx" => TXX,
"txy" => TXY,
"tya" => TYA,
"tye" => TYE,
"tyh" => TYH,
"tyi" => TYI,
"tyj" => TYJ,
"tyl" => TYL,
"tyn" => TYN,
"typ" => TYP,
"tyr" => TYR,
"tys" => TYS,
"tyt" => TYT,
"tyu" => TYU,
"tyx" => TYX,
"tyy" => TYY,
"tyz" => TYZ,
"tza" => TZA,
"tzh" => TZH,
"tzj" => TZJ,
"tzl" => TZL,
"tzm" => TZM,
"tzn" => TZN,
"tzo" => TZO,
"tzx" => TZX,
"uam" => UAM,
"uan" => UAN,
"uar" => UAR,
"uba" => UBA,
"ubi" => UBI,
"ubl" => UBL,
"ubr" => UBR,
"ubu" => UBU,
"uby" => UBY,
"uda" => UDA,
"ude" => UDE,
"udg" => UDG,
"udi" => UDI,
"udj" => UDJ,
"udl" => UDL,
"udu" => UDU,
"ues" => UES,
"ufi" => UFI,
"ugb" => UGB,
"uge" => UGE,
"ugh" => UGH,
"ugn" => UGN,
"ugo" => UGO,
"ugy" => UGY,
"uha" => UHA,
"uhn" => UHN,
"uis" => UIS,
"uiv" => UIV,
"uji" => UJI,
"uka" => UKA,
"ukg" => UKG,
"ukh" => UKH,
"uki" => UKI,
"ukk" => UKK,
"ukl" => UKL,
"ukp" => UKP,
"ukq" => UKQ,
"uks" => UKS,
"uku" => UKU,
"ukv" => UKV,
"ukw" => UKW,
"uky" => UKY,
"ula" => ULA,
"ulb" => ULB,
"ulc" => ULC,
"ule" => ULE,
"ulf" => ULF,
"uli" => ULI,
"ulk" => ULK,
"ull" => ULL,
"ulm" => ULM,
"uln" => ULN,
"ulu" => ULU,
"ulw" => ULW,
"uma" => UMA,
"umc" => UMC,
"umd" => UMD,
"umg" => UMG,
"umi" => UMI,
"umm" => UMM,
"umn" => UMN,
"umo" => UMO,
"ump" => UMP,
"umr" => UMR,
"ums" => UMS,
"umu" => UMU,
"una" => UNA,
"une" => UNE,
"ung" => UNG,
"uni" => UNI,
"unk" => UNK,
"unm" => UNM,
"unn" => UNN,
"unr" => UNR,
"unu" => UNU,
"unx" => UNX,
"unz" => UNZ,
"uon" => UON,
"upi" => UPI,
"upv" => UPV,
"ura" => URA,
"urb" => URB,
"urc" => URC,
"ure" => URE,
"urf" => URF,
"urg" => URG,
"urh" => URH,
"uri" => URI,
"urk" => URK,
"url" => URL,
"urm" => URM,
"urn" => URN,
"uro" => URO,
"urp" => URP,
"urr" => URR,
"urt" => URT,
"uru" => URU,
"urv" => URV,
"urw" => URW,
"urx" => URX,
"ury" => URY,
"urz" => URZ,
"usa" => USA,
"ush" => USH,
"usi" => USI,
"usk" => USK,
"usp" => USP,
"uss" => USS,
"usu" => USU,
"uta" => UTA,
"ute" => UTE,
"uth" => UTH,
"utp" => UTP,
"utr" => UTR,
"utu" => UTU,
"uum" => UUM,
"uur" => UUR,
"uuu" => UUU,
"uve" => UVE,
"uvh" => UVH,
"uvl" => UVL,
"uwa" => UWA,
"uya" => UYA,
"uzn" => UZN,
"uzs" => UZS,
"vaa" => VAA,
"vae" => VAE,
"vaf" => VAF,
"vag" => VAG,
"vah" => VAH,
"vaj" => VAJ,
"val" => VAL,
"vam" => VAM,
"van" => VAN,
"vao" => VAO,
"vap" => VAP,
"var" => VAR,
"vas" => VAS,
"vau" => VAU,
"vav" => VAV,
"vay" => VAY,
"vbb" => VBB,
"vbk" => VBK,
"vec" => VEC,
"ved" => VED,
"vel" => VEL,
"vem" => VEM,
"veo" => VEO,
"vep" => VEP,
"ver" => VER,
"vgr" => VGR,
"vgt" => VGT,
"vic" => VIC,
"vid" => VID,
"vif" => VIF,
"vig" => VIG,
"vil" => VIL,
"vin" => VIN,
"vis" => VIS,
"vit" => VIT,
"viv" => VIV,
"vka" => VKA,
"vkj" => VKJ,
"vkk" => VKK,
"vkl" => VKL,
"vkm" => VKM,
"vkn" => VKN,
"vko" => VKO,
"vkp" => VKP,
"vkt" => VKT,
"vku" => VKU,
"vkz" => VKZ,
"vlp" => VLP,
"vls" => VLS,
"vma" => VMA,
"vmb" => VMB,
"vmc" => VMC,
"vmd" => VMD,
"vme" => VME,
"vmf" => VMF,
"vmg" => VMG,
"vmh" => VMH,
"vmi" => VMI,
"vmj" => VMJ,
"vmk" => VMK,
"vml" => VML,
"vmm" => VMM,
"vmp" => VMP,
"vmq" => VMQ,
"vmr" => VMR,
"vms" => VMS,
"vmu" => VMU,
"vmv" => VMV,
"vmw" => VMW,
"vmx" => VMX,
"vmy" => VMY,
"vmz" => VMZ,
"vnk" => VNK,
"vnm" => VNM,
"vnp" => VNP,
"vor" => VOR,
"vra" => VRA,
"vro" => VRO,
"vrs" => VRS,
"vrt" => VRT,
"vsi" => VSI,
"vsl" => VSL,
"vsv" => VSV,
"vto" => VTO,
"vum" => VUM,
"vun" => VUN,
"vut" => VUT,
"vwa" => VWA,
"waa" => WAA,
"wab" => WAB,
"wac" => WAC,
"wad" => WAD,
"wae" => WAE,
"waf" => WAF,
"wag" => WAG,
"wah" => WAH,
"wai" => WAI,
"waj" => WAJ,
"wam" => WAM,
"wan" => WAN,
"wao" => WAO,
"wap" => WAP,
"waq" => WAQ,
"wat" => WAT,
"wau" => WAU,
"wav" => WAV,
"waw" => WAW,
"wax" => WAX,
"way" => WAY,
"waz" => WAZ,
"wba" => WBA,
"wbb" => WBB,
"wbe" => WBE,
"wbf" => WBF,
"wbh" => WBH,
"wbi" => WBI,
"wbj" => WBJ,
"wbk" => WBK,
"wbl" => WBL,
"wbm" => WBM,
"wbp" => WBP,
"wbq" => WBQ,
"wbr" => WBR,
"wbs" => WBS,
"wbt" => WBT,
"wbv" => WBV,
"wbw" => WBW,
"wca" => WCA,
"wci" => WCI,
"wdd" => WDD,
"wdg" => WDG,
"wdj" => WDJ,
"wdk" => WDK,
"wdt" => WDT,
"wdu" => WDU,
"wdy" => WDY,
"wea" => WEA,
"wec" => WEC,
"wed" => WED,
"weg" => WEG,
"weh" => WEH,
"wei" => WEI,
"wem" => WEM,
"weo" => WEO,
"wep" => WEP,
"wer" => WER,
"wes" => WES,
"wet" => WET,
"weu" => WEU,
"wew" => WEW,
"wfg" => WFG,
"wga" => WGA,
"wgb" => WGB,
"wgg" => WGG,
"wgi" => WGI,
"wgo" => WGO,
"wgu" => WGU,
"wgy" => WGY,
"wha" => WHA,
"whg" => WHG,
"whk" => WHK,
"whu" => WHU,
"wib" => WIB,
"wic" => WIC,
"wie" => WIE,
"wif" => WIF,
"wig" => WIG,
"wih" => WIH,
"wii" => WII,
"wij" => WIJ,
"wik" => WIK,
"wil" => WIL,
"wim" => WIM,
"win" => WIN,
"wir" => WIR,
"wiu" => WIU,
"wiv" => WIV,
"wiy" => WIY,
"wja" => WJA,
"wji" => WJI,
"wka" => WKA,
"wkb" => WKB,
"wkd" => WKD,
"wkl" => WKL,
"wkr" => WKR,
"wku" => WKU,
"wkw" => WKW,
"wky" => WKY,
"wla" => WLA,
"wlc" => WLC,
"wle" => WLE,
"wlg" => WLG,
"wlh" => WLH,
"wli" => WLI,
"wlk" => WLK,
"wll" => WLL,
"wlm" => WLM,
"wlo" => WLO,
"wlr" => WLR,
"wls" => WLS,
"wlu" => WLU,
"wlv" => WLV,
"wlw" => WLW,
"wlx" => WLX,
"wly" => WLY,
"wma" => WMA,
"wmb" => WMB,
"wmc" => WMC,
"wmd" => WMD,
"wme" => WME,
"wmg" => WMG,
"wmh" => WMH,
"wmi" => WMI,
"wmm" => WMM,
"wmn" => WMN,
"wmo" => WMO,
"wms" => WMS,
"wmt" => WMT,
"wmw" => WMW,
"wmx" => WMX,
"wnb" => WNB,
"wnc" => WNC,
"wnd" => WND,
"wne" => WNE,
"wng" => WNG,
"wni" => WNI,
"wnk" => WNK,
"wnm" => WNM,
"wnn" => WNN,
"wno" => WNO,
"wnp" => WNP,
"wnu" => WNU,
"wnw" => WNW,
"wny" => WNY,
"woa" => WOA,
"wob" => WOB,
"woc" => WOC,
"wod" => WOD,
"woe" => WOE,
"wof" => WOF,
"wog" => WOG,
"woi" => WOI,
"wok" => WOK,
"wom" => WOM,
"won" => WON,
"woo" => WOO,
"wor" => WOR,
"wos" => WOS,
"wow" => WOW,
"woy" => WOY,
"wpc" => WPC,
"wrb" => WRB,
"wrg" => WRG,
"wrh" => WRH,
"wri" => WRI,
"wrk" => WRK,
"wrl" => WRL,
"wrm" => WRM,
"wrn" => WRN,
"wro" => WRO,
"wrp" => WRP,
"wrr" => WRR,
"wrs" => WRS,
"wru" => WRU,
"wrv" => WRV,
"wrw" => WRW,
"wrx" => WRX,
"wry" => WRY,
"wrz" => WRZ,
"wsa" => WSA,
"wsg" => WSG,
"wsi" => WSI,
"wsk" => WSK,
"wsr" => WSR,
"wss" => WSS,
"wsu" => WSU,
"wsv" => WSV,
"wtf" => WTF,
"wth" => WTH,
"wti" => WTI,
"wtk" => WTK,
"wtm" => WTM,
"wtw" => WTW,
"wua" => WUA,
"wub" => WUB,
"wud" => WUD,
"wuh" => WUH,
"wul" => WUL,
"wum" => WUM,
"wun" => WUN,
"wur" => WUR,
"wut" => WUT,
"wuu" => WUU,
"wuv" => WUV,
"wux" => WUX,
"wuy" => WUY,
"wwa" => WWA,
"wwb" => WWB,
"wwo" => WWO,
"wwr" => WWR,
"www" => WWW,
"wxa" => WXA,
"wxw" => WXW,
"wyb" => WYB,
"wyi" => WYI,
"wym" => WYM,
"wyn" => WYN,
"wyr" => WYR,
"wyy" => WYY,
"xaa" => XAA,
"xab" => XAB,
"xac" => XAC,
"xad" => XAD,
"xae" => XAE,
"xag" => XAG,
"xai" => XAI,
"xaj" => XAJ,
"xak" => XAK,
"xam" => XAM,
"xan" => XAN,
"xao" => XAO,
"xap" => XAP,
"xaq" => XAQ,
"xar" => XAR,
"xas" => XAS,
"xat" => XAT,
"xau" => XAU,
"xav" => XAV,
"xaw" => XAW,
"xay" => XAY,
"xbb" => XBB,
"xbc" => XBC,
"xbd" => XBD,
"xbe" => XBE,
"xbg" => XBG,
"xbi" => XBI,
"xbj" => XBJ,
"xbm" => XBM,
"xbn" => XBN,
"xbo" => XBO,
"xbp" => XBP,
"xbr" => XBR,
"xbw" => XBW,
"xby" => XBY,
"xcb" => XCB,
"xcc" => XCC,
"xce" => XCE,
"xcg" => XCG,
"xch" => XCH,
"xcl" => XCL,
"xcm" => XCM,
"xcn" => XCN,
"xco" => XCO,
"xcr" => XCR,
"xct" => XCT,
"xcu" => XCU,
"xcv" => XCV,
"xcw" => XCW,
"xcy" => XCY,
"xda" => XDA,
"xdc" => XDC,
"xdk" => XDK,
"xdm" => XDM,
"xdo" => XDO,
"xdq" => XDQ,
"xdy" => XDY,
"xeb" => XEB,
"xed" => XED,
"xeg" => XEG,
"xel" => XEL,
"xem" => XEM,
"xep" => XEP,
"xer" => XER,
"xes" => XES,
"xet" => XET,
"xeu" => XEU,
"xfa" => XFA,
"xga" => XGA,
"xgb" => XGB,
"xgd" => XGD,
"xgf" => XGF,
"xgg" => XGG,
"xgi" => XGI,
"xgl" => XGL,
"xgm" => XGM,
"xgr" => XGR,
"xgu" => XGU,
"xgw" => XGW,
"xha" => XHA,
"xhc" => XHC,
"xhd" => XHD,
"xhe" => XHE,
"xhm" => XHM,
"xhr" => XHR,
"xht" => XHT,
"xhu" => XHU,
"xhv" => XHV,
"xib" => XIB,
"xii" => XII,
"xil" => XIL,
"xin" => XIN,
"xir" => XIR,
"xis" => XIS,
"xiv" => XIV,
"xiy" => XIY,
"xjb" => XJB,
"xjt" => XJT,
"xka" => XKA,
"xkb" => XKB,
"xkc" => XKC,
"xkd" => XKD,
"xke" => XKE,
"xkf" => XKF,
"xkg" => XKG,
"xki" => XKI,
"xkj" => XKJ,
"xkk" => XKK,
"xkl" => XKL,
"xkn" => XKN,
"xko" => XKO,
"xkp" => XKP,
"xkq" => XKQ,
"xkr" => XKR,
"xks" => XKS,
"xkt" => XKT,
"xku" => XKU,
"xkv" => XKV,
"xkw" => XKW,
"xkx" => XKX,
"xky" => XKY,
"xkz" => XKZ,
"xla" => XLA,
"xlb" => XLB,
"xlc" => XLC,
"xld" => XLD,
"xle" => XLE,
"xlg" => XLG,
"xli" => XLI,
"xln" => XLN,
"xlo" => XLO,
"xlp" => XLP,
"xls" => XLS,
"xlu" => XLU,
"xly" => XLY,
"xma" => XMA,
"xmb" => XMB,
"xmc" => XMC,
"xmd" => XMD,
"xme" => XME,
"xmf" => XMF,
"xmg" => XMG,
"xmh" => XMH,
"xmj" => XMJ,
"xmk" => XMK,
"xml" => XML,
"xmm" => XMM,
"xmn" => XMN,
"xmo" => XMO,
"xmp" => XMP,
"xmq" => XMQ,
"xmr" => XMR,
"xms" => XMS,
"xmt" => XMT,
"xmu" => XMU,
"xmv" => XMV,
"xmw" => XMW,
"xmx" => XMX,
"xmy" => XMY,
"xmz" => XMZ,
"xna" => XNA,
"xnb" => XNB,
"xng" => XNG,
"xnh" => XNH,
"xni" => XNI,
"xnj" => XNJ,
"xnk" => XNK,
"xnm" => XNM,
"xnn" => XNN,
"xno" => XNO,
"xnq" => XNQ,
"xnr" => XNR,
"xns" => XNS,
"xnt" => XNT,
"xnu" => XNU,
"xny" => XNY,
"xnz" => XNZ,
"xoc" => XOC,
"xod" => XOD,
"xog" => XOG,
"xoi" => XOI,
"xok" => XOK,
"xom" => XOM,
"xon" => XON,
"xoo" => XOO,
"xop" => XOP,
"xor" => XOR,
"xow" => XOW,
"xpa" => XPA,
"xpb" => XPB,
"xpc" => XPC,
"xpd" => XPD,
"xpe" => XPE,
"xpf" => XPF,
"xpg" => XPG,
"xph" => XPH,
"xpi" => XPI,
"xpj" => XPJ,
"xpk" => XPK,
"xpl" => XPL,
"xpm" => XPM,
"xpn" => XPN,
"xpo" => XPO,
"xpp" => XPP,
"xpq" => XPQ,
"xpr" => XPR,
"xps" => XPS,
"xpt" => XPT,
"xpu" => XPU,
"xpv" => XPV,
"xpw" => XPW,
"xpx" => XPX,
"xpy" => XPY,
"xpz" => XPZ,
"xqa" => XQA,
"xqt" => XQT,
"xra" => XRA,
"xrb" => XRB,
"xrd" => XRD,
"xre" => XRE,
"xrg" => XRG,
"xri" => XRI,
"xrm" => XRM,
"xrn" => XRN,
"xrr" => XRR,
"xrt" => XRT,
"xru" => XRU,
"xrw" => XRW,
"xsa" => XSA,
"xsb" => XSB,
"xsc" => XSC,
"xsd" => XSD,
"xse" => XSE,
"xsh" => XSH,
"xsi" => XSI,
"xsj" => XSJ,
"xsl" => XSL,
"xsm" => XSM,
"xsn" => XSN,
"xso" => XSO,
"xsp" => XSP,
"xsq" => XSQ,
"xsr" => XSR,
"xss" => XSS,
"xsu" => XSU,
"xsv" => XSV,
"xsy" => XSY,
"xta" => XTA,
"xtb" => XTB,
"xtc" => XTC,
"xtd" => XTD,
"xte" => XTE,
"xtg" => XTG,
"xth" => XTH,
"xti" => XTI,
"xtj" => XTJ,
"xtl" => XTL,
"xtm" => XTM,
"xtn" => XTN,
"xto" => XTO,
"xtp" => XTP,
"xtq" => XTQ,
"xtr" => XTR,
"xts" => XTS,
"xtt" => XTT,
"xtu" => XTU,
"xtv" => XTV,
"xtw" => XTW,
"xty" => XTY,
"xua" => XUA,
"xub" => XUB,
"xud" => XUD,
"xug" => XUG,
"xuj" => XUJ,
"xul" => XUL,
"xum" => XUM,
"xun" => XUN,
"xuo" => XUO,
"xup" => XUP,
"xur" => XUR,
"xut" => XUT,
"xuu" => XUU,
"xve" => XVE,
"xvi" => XVI,
"xvn" => XVN,
"xvo" => XVO,
"xvs" => XVS,
"xwa" => XWA,
"xwc" => XWC,
"xwd" => XWD,
"xwe" => XWE,
"xwg" => XWG,
"xwj" => XWJ,
"xwk" => XWK,
"xwl" => XWL,
"xwo" => XWO,
"xwr" => XWR,
"xwt" => XWT,
"xww" => XWW,
"xxb" => XXB,
"xxk" => XXK,
"xxm" => XXM,
"xxr" => XXR,
"xxt" => XXT,
"xya" => XYA,
"xyb" => XYB,
"xyj" => XYJ,
"xyk" => XYK,
"xyl" => XYL,
"xyt" => XYT,
"xyy" => XYY,
"xzh" => XZH,
"xzm" => XZM,
"xzp" => XZP,
"yaa" => YAA,
"yab" => YAB,
"yac" => YAC,
"yad" => YAD,
"yae" => YAE,
"yaf" => YAF,
"yag" => YAG,
"yah" => YAH,
"yai" => YAI,
"yaj" => YAJ,
"yak" => YAK,
"yal" => YAL,
"yam" => YAM,
"yan" => YAN,
"yaq" => YAQ,
"yar" => YAR,
"yas" => YAS,
"yat" => YAT,
"yau" => YAU,
"yav" => YAV,
"yaw" => YAW,
"yax" => YAX,
"yay" => YAY,
"yaz" => YAZ,
"yba" => YBA,
"ybb" => YBB,
"ybe" => YBE,
"ybh" => YBH,
"ybi" => YBI,
"ybj" => YBJ,
"ybk" => YBK,
"ybl" => YBL,
"ybm" => YBM,
"ybn" => YBN,
"ybo" => YBO,
"ybx" => YBX,
"yby" => YBY,
"ych" => YCH,
"ycl" => YCL,
"ycn" => YCN,
"ycp" => YCP,
"yda" => YDA,
"ydd" => YDD,
"yde" => YDE,
"ydg" => YDG,
"ydk" => YDK,
"yea" => YEA,
"yec" => YEC,
"yee" => YEE,
"yei" => YEI,
"yej" => YEJ,
"yel" => YEL,
"yer" => YER,
"yes" => YES,
"yet" => YET,
"yeu" => YEU,
"yev" => YEV,
"yey" => YEY,
"yga" => YGA,
"ygi" => YGI,
"ygl" => YGL,
"ygm" => YGM,
"ygp" => YGP,
"ygr" => YGR,
"ygs" => YGS,
"ygu" => YGU,
"ygw" => YGW,
"yha" => YHA,
"yhd" => YHD,
"yhl" => YHL,
"yhs" => YHS,
"yia" => YIA,
"yif" => YIF,
"yig" => YIG,
"yih" => YIH,
"yii" => YII,
"yij" => YIJ,
"yik" => YIK,
"yil" => YIL,
"yim" => YIM,
"yin" => YIN,
"yip" => YIP,
"yiq" => YIQ,
"yir" => YIR,
"yis" => YIS,
"yit" => YIT,
"yiu" => YIU,
"yiv" => YIV,
"yix" => YIX,
"yiz" => YIZ,
"yka" => YKA,
"ykg" => YKG,
"yki" => YKI,
"ykk" => YKK,
"ykl" => YKL,
"ykm" => YKM,
"ykn" => YKN,
"yko" => YKO,
"ykr" => YKR,
"ykt" => YKT,
"yku" => YKU,
"yky" => YKY,
"yla" => YLA,
"ylb" => YLB,
"yle" => YLE,
"ylg" => YLG,
"yli" => YLI,
"yll" => YLL,
"ylm" => YLM,
"yln" => YLN,
"ylo" => YLO,
"ylr" => YLR,
"ylu" => YLU,
"yly" => YLY,
"ymb" => YMB,
"ymc" => YMC,
"ymd" => YMD,
"yme" => YME,
"ymg" => YMG,
"ymh" => YMH,
"ymi" => YMI,
"ymk" => YMK,
"yml" => YML,
"ymm" => YMM,
"ymn" => YMN,
"ymo" => YMO,
"ymp" => YMP,
"ymq" => YMQ,
"ymr" => YMR,
"yms" => YMS,
"ymx" => YMX,
"ymz" => YMZ,
"yna" => YNA,
"ynd" => YND,
"yne" => YNE,
"yng" => YNG,
"ynk" => YNK,
"ynl" => YNL,
"ynn" => YNN,
"yno" => YNO,
"ynq" => YNQ,
"yns" => YNS,
"ynu" => YNU,
"yob" => YOB,
"yog" => YOG,
"yoi" => YOI,
"yok" => YOK,
"yol" => YOL,
"yom" => YOM,
"yon" => YON,
"yot" => YOT,
"yox" => YOX,
"yoy" => YOY,
"ypa" => YPA,
"ypb" => YPB,
"ypg" => YPG,
"yph" => YPH,
"ypm" => YPM,
"ypn" => YPN,
"ypo" => YPO,
"ypp" => YPP,
"ypz" => YPZ,
"yra" => YRA,
"yrb" => YRB,
"yre" => YRE,
"yrk" => YRK,
"yrl" => YRL,
"yrm" => YRM,
"yrn" => YRN,
"yro" => YRO,
"yrs" => YRS,
"yrw" => YRW,
"yry" => YRY,
"ysc" => YSC,
"ysd" => YSD,
"ysg" => YSG,
"ysl" => YSL,
"ysm" => YSM,
"ysn" => YSN,
"yso" => YSO,
"ysp" => YSP,
"ysr" => YSR,
"yss" => YSS,
"ysy" => YSY,
"yta" => YTA,
"ytl" => YTL,
"ytp" => YTP,
"ytw" => YTW,
"yty" => YTY,
"yua" => YUA,
"yub" => YUB,
"yuc" => YUC,
"yud" => YUD,
"yue" => YUE,
"yuf" => YUF,
"yug" => YUG,
"yui" => YUI,
"yuj" => YUJ,
"yuk" => YUK,
"yul" => YUL,
"yum" => YUM,
"yun" => YUN,
"yup" => YUP,
"yuq" => YUQ,
"yur" => YUR,
"yut" => YUT,
"yuw" => YUW,
"yux" => YUX,
"yuy" => YUY,
"yuz" => YUZ,
"yva" => YVA,
"yvt" => YVT,
"ywa" => YWA,
"ywg" => YWG,
"ywl" => YWL,
"ywn" => YWN,
"ywq" => YWQ,
"ywr" => YWR,
"ywt" => YWT,
"ywu" => YWU,
"yww" => YWW,
"yxa" => YXA,
"yxg" => YXG,
"yxl" => YXL,
"yxm" => YXM,
"yxu" => YXU,
"yxy" => YXY,
"yyr" => YYR,
"yyu" => YYU,
"yyz" => YYZ,
"yzg" => YZG,
"yzk" => YZK,
"zaa" => ZAA,
"zab" => ZAB,
"zac" => ZAC,
"zad" => ZAD,
"zae" => ZAE,
"zaf" => ZAF,
"zag" => ZAG,
"zah" => ZAH,
"zai" => ZAI,
"zaj" => ZAJ,
"zak" => ZAK,
"zal" => ZAL,
"zam" => ZAM,
"zao" => ZAO,
"zaq" => ZAQ,
"zar" => ZAR,
"zas" => ZAS,
"zat" => ZAT,
"zau" => ZAU,
"zav" => ZAV,
"zaw" => ZAW,
"zax" => ZAX,
"zay" => ZAY,
"zaz" => ZAZ,
"zba" => ZBA,
"zbc" => ZBC,
"zbe" => ZBE,
"zbt" => ZBT,
"zbu" => ZBU,
"zbw" => ZBW,
"zca" => ZCA,
"zcd" => ZCD,
"zch" => ZCH,
"zdj" => ZDJ,
"zea" => ZEA,
"zeg" => ZEG,
"zeh" => ZEH,
"zga" => ZGA,
"zgb" => ZGB,
"zgm" => ZGM,
"zgn" => ZGN,
"zgr" => ZGR,
"zhb" => ZHB,
"zhd" => ZHD,
"zhi" => ZHI,
"zhn" => ZHN,
"zhw" => ZHW,
"zia" => ZIA,
"zib" => ZIB,
"zik" => ZIK,
"zil" => ZIL,
"zim" => ZIM,
"zin" => ZIN,
"ziw" => ZIW,
"ziz" => ZIZ,
"zka" => ZKA,
"zkb" => ZKB,
"zkd" => ZKD,
"zkg" => ZKG,
"zkh" => ZKH,
"zkk" => ZKK,
"zkn" => ZKN,
"zko" => ZKO,
"zkp" => ZKP,
"zkr" => ZKR,
"zkt" => ZKT,
"zku" => ZKU,
"zkv" => ZKV,
"zkz" => ZKZ,
"zla" => ZLA,
"zlj" => ZLJ,
"zlm" => ZLM,
"zln" => ZLN,
"zlq" => ZLQ,
"zma" => ZMA,
"zmb" => ZMB,
"zmc" => ZMC,
"zmd" => ZMD,
"zme" => ZME,
"zmf" => ZMF,
"zmg" => ZMG,
"zmh" => ZMH,
"zmi" => ZMI,
"zmj" => ZMJ,
"zmk" => ZMK,
"zml" => ZML,
"zmm" => ZMM,
"zmn" => ZMN,
"zmo" => ZMO,
"zmp" => ZMP,
"zmq" => ZMQ,
"zmr" => ZMR,
"zms" => ZMS,
"zmt" => ZMT,
"zmu" => ZMU,
"zmv" => ZMV,
"zmw" => ZMW,
"zmx" => ZMX,
"zmy" => ZMY,
"zmz" => ZMZ,
"zna" => ZNA,
"zne" => ZNE,
"zng" => ZNG,
"znk" => ZNK,
"zns" => ZNS,
"zoc" => ZOC,
"zoh" => ZOH,
"zom" => ZOM,
"zoo" => ZOO,
"zoq" => ZOQ,
"zor" => ZOR,
"zos" => ZOS,
"zpa" => ZPA,
"zpb" => ZPB,
"zpc" => ZPC,
"zpd" => ZPD,
"zpe" => ZPE,
"zpf" => ZPF,
"zpg" => ZPG,
"zph" => ZPH,
"zpi" => ZPI,
"zpj" => ZPJ,
"zpk" => ZPK,
"zpl" => ZPL,
"zpm" => ZPM,
"zpn" => ZPN,
"zpo" => ZPO,
"zpp" => ZPP,
"zpq" => ZPQ,
"zpr" => ZPR,
"zps" => ZPS,
"zpt" => ZPT,
"zpu" => ZPU,
"zpv" => ZPV,
"zpw" => ZPW,
"zpx" => ZPX,
"zpy" => ZPY,
"zpz" => ZPZ,
"zqe" => ZQE,
"zra" => ZRA,
"zrg" => ZRG,
"zrn" => ZRN,
"zro" => ZRO,
"zrp" => ZRP,
"zrs" => ZRS,
"zsa" => ZSA,
"zsk" => ZSK,
"zsl" => ZSL,
"zsm" => ZSM,
"zsr" => ZSR,
"zsu" => ZSU,
"zte" => ZTE,
"ztg" => ZTG,
"ztl" => ZTL,
"ztm" => ZTM,
"ztn" => ZTN,
"ztp" => ZTP,
"ztq" => ZTQ,
"zts" => ZTS,
"ztt" => ZTT,
"ztu" => ZTU,
"ztx" => ZTX,
"zty" => ZTY,
"zua" => ZUA,
"zuh" => ZUH,
"zum" => ZUM,
"zuy" => ZUY,
"zwa" => ZWA,
"zyb" => ZYB,
"zyg" => ZYG,
"zyj" => ZYJ,
"zyn" => ZYN,
"zyp" => ZYP,
"zzj" => ZZJ,

};

